(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[   2614406,      82363]*)
(*NotebookOutlinePosition[   2691977,      85068]*)
(*  CellTagsIndexPosition[   2684387,      84844]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["The FeynCalc Book", "Title",
  TextAlignment->Center],

Cell[TextData[StyleBox["FeynCalc 4.1.0",
  FontWeight->"Bold"]], "Subtitle",
  PageBreakBelow->True,
  TextAlignment->Center],

Cell["Tools for Feynman diagram calculations", "Subtitle",
  TextAlignment->Center],

Cell["Rolf Mertig", "Subsubtitle",
  TextAlignment->Center],

Cell["July 21, 2000 ", "Subsubtitle",
  TextAlignment->Center],

Cell[TextData[{
  "  ",
  ButtonBox["Mertig Research & Consulting",
    ButtonData:>{
      URL[ "http://www.mertig.com"], None},
    ButtonStyle->"Hyperlink",
    ButtonNote->"http://www.mertig.com"]
}], "Subsubtitle",
  TextAlignment->Center],

Cell[TextData[{
  "HTML version: ",
  ButtonBox["http://www.feyncalc.org",
    ButtonData:>{
      URL[ "http://www.feyncalc.org"], None},
    ButtonStyle->"Hyperlink",
    ButtonNote->"http://www.feyncalc.org"]
}], "Subsubtitle",
  TextAlignment->Center],

Cell[CellGroupData[{

Cell["Table of Contents", "Section",
  CellTags->"Table of Contents"],

Cell[TextData[ButtonBox["Introduction",
  ButtonData:>"Short Overview",
  ButtonStyle->"Hyperlink"]], "Contents",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Start FeynCalc",
  ButtonData:>"Load FeynCalc",
  ButtonStyle->"Hyperlink"]], "Contents",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Example Calculations",
  ButtonData:>"Example Calculations",
  ButtonStyle->"Hyperlink"]], "Contents",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Tree level pair production",
  ButtonData:>"Tree level",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["A moment integral",
  ButtonData:>"A moment integral",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Reference Guide",
  ButtonData:>"Reference Guide",
  ButtonStyle->"Hyperlink"]], "ContentsSection",
  GeneratedCell->True],

Cell[TextData[ButtonBox["A0",
  ButtonData:>"A0",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Amplitudes",
  ButtonData:>"Amplitudes",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["AnomalousDimension",
  ButtonData:>"AnomalousDimension",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Anti5",
  ButtonData:>"Anti5",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["AntiCommutator",
  ButtonData:>"AntiCommutator",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["AntiQuarkField",
  ButtonData:>"AntiQuarkField",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["AntiSymmetrize",
  ButtonData:>"AntiSymmetrize",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Apart1",
  ButtonData:>"Apart1",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Apart2",
  ButtonData:>"Apart2",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Apart3",
  ButtonData:>"Apart3",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["B00",
  ButtonData:>"B00",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["B0",
  ButtonData:>"B0",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["B11",
  ButtonData:>"B11",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["B1",
  ButtonData:>"B1",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["BackgroundGluonVertex",
  ButtonData:>"BackgroundGluonVertex",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["C0",
  ButtonData:>"C0",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Calc",
  ButtonData:>"Calc",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["CA",
  ButtonData:>"CA",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Cases2",
  ButtonData:>"Cases2",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["CF",
  ButtonData:>"CF",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["ChangeDimension",
  ButtonData:>"ChangeDimension",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["ChargeConjugationMatrixInv",
  ButtonData:>"ChargeConjugationMatrixInv",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["ChargeConjugationMatrix",
  ButtonData:>"ChargeConjugationMatrix",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["ChiralityProjector",
  ButtonData:>"ChiralityProjector",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Chisholm",
  ButtonData:>"Chisholm",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["ClearScalarProducts",
  ButtonData:>"ClearScalarProducts",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Collect2",
  ButtonData:>"Collect2",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Combine",
  ButtonData:>"Combine",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["CommutatorExplicit",
  ButtonData:>"CommutatorExplicit",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Commutator",
  ButtonData:>"Commutator",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["ComplexConjugate",
  ButtonData:>"ComplexConjugate",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["ComplexIndex",
  ButtonData:>"ComplexIndex",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Contract",
  ButtonData:>"Contract",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Convolute",
  ButtonData:>"Convolute",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["CovariantD",
  ButtonData:>"CovariantD",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["D0",
  ButtonData:>"D0",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["DataType",
  ButtonData:>"DataType",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["DeclareNonCommutative",
  ButtonData:>"DeclareNonCommutative",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["DeltaFunction",
  ButtonData:>"DeltaFunction",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["DeltaFunctionPrime",
  ButtonData:>"DeltaFunctionPrime",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Dimension",
  ButtonData:>"Dimension",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["DiracGammaCombine",
  ButtonData:>"DiracGammaCombine",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["DiracGammaExpand",
  ButtonData:>"DiracGammaExpand",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["DiracGamma",
  ButtonData:>"DiracGamma",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["DiracGammaT",
  ButtonData:>"DiracGammaT",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["DiracMatrix",
  ButtonData:>"DiracMatrix",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["DiracOrder",
  ButtonData:>"DiracOrder",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["DiracReduce",
  ButtonData:>"DiracReduce",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["DiracSigmaExplicit",
  ButtonData:>"DiracSigmaExplicit",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["DiracSigma",
  ButtonData:>"DiracSigma",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["DiracSimplify",
  ButtonData:>"DiracSimplify",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["DiracSimplify2",
  ButtonData:>"DiracSimplify2",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["DiracSlash",
  ButtonData:>"DiracSlash",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["DiracTrace",
  ButtonData:>"DiracTrace",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["DiracTrick",
  ButtonData:>"DiracTrick",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["DotSimplify",
  ButtonData:>"DotSimplify",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["EpsChisholm",
  ButtonData:>"EpsChisholm",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["EpsEvaluate",
  ButtonData:>"EpsEvaluate",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Epsilon",
  ButtonData:>"Epsilon",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Eps",
  ButtonData:>"Eps",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["ExpandPartialD",
  ButtonData:>"ExpandPartialD",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["ExpandScalarProduct",
  ButtonData:>"ExpandScalarProduct",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Explicit",
  ButtonData:>"Explicit",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["ExplicitPartialD",
  ButtonData:>"ExplicitPartialD",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["ExplicitSUNIndex",
  ButtonData:>"ExplicitSUNIndex",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Factor2",
  ButtonData:>"Factor2",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["FAD",
  ButtonData:>"FAD",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["FCE",
  ButtonData:>"FCE",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["FCI",
  ButtonData:>"FCI",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["FermionSpinSum",
  ButtonData:>"FermionSpinSum",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["FeynAmpDenominatorCombine",
  ButtonData:>"FeynAmpDenominatorCombine",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["FeynAmpDenominator",
  ButtonData:>"FeynAmpDenominator",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["FeynAmpDenominatorSimplify",
  ButtonData:>"FeynAmpDenominatorSimplify",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["FeynAmpDenominatorSplit",
  ButtonData:>"FeynAmpDenominatorSplit",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["FeynAmp",
  ButtonData:>"FeynAmp",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["FeynCalc2FORM",
  ButtonData:>"FeynCalc2FORM",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["FeynCalcExternal",
  ButtonData:>"FeynCalcExternal",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["FeynCalcInternal",
  ButtonData:>"FeynCalcInternal",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["FeynRule",
  ButtonData:>"FeynRule",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["FieldStrength",
  ButtonData:>"FieldStrength",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["FORM2FeynCalc",
  ButtonData:>"FORM2FeynCalc",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["FourDivergence",
  ButtonData:>"FourDivergence",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["FourLaplacian",
  ButtonData:>"FourLaplacian",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["FourVector",
  ButtonData:>"FourVector",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["FreeQ2",
  ButtonData:>"FreeQ2",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["FromTFi",
  ButtonData:>"FromTFi",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["FRH",
  ButtonData:>"FRH",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["FunctionalD",
  ButtonData:>"FunctionalD",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["FV",
  ButtonData:>"FV",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["FVD",
  ButtonData:>"FVD",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["GA",
  ButtonData:>"GA",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["GAD",
  ButtonData:>"GAD",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["GammaEpsilon",
  ButtonData:>"GammaEpsilon",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["GammaExpand",
  ButtonData:>"GammaExpand",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["GhostPropagator",
  ButtonData:>"GhostPropagator",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["GHP",
  ButtonData:>"GHP",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["GaugeField",
  ButtonData:>"GluonField",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["GluonGhostVertex",
  ButtonData:>"GluonGhostVertex",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["GluonPropagator",
  ButtonData:>"GluonPropagator",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["GluonVertex",
  ButtonData:>"GluonVertex",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["GSD",
  ButtonData:>"GSD",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["GS",
  ButtonData:>"GS",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Gstrong",
  ButtonData:>"Gstrong",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Hill",
  ButtonData:>"Hill",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["HypergeometricAC",
  ButtonData:>"HypergeometricAC",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["HypergeometricIR",
  ButtonData:>"HypergeometricIR",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["HypergeometricSE",
  ButtonData:>"HypergeometricSE",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["HypExplicit",
  ButtonData:>"HypExplicit",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Integrate2",
  ButtonData:>"Integrate2",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Integrate3",
  ButtonData:>"Integrate3",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["IntegrateByParts",
  ButtonData:>"IntegrateByParts",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["InverseMellin",
  ButtonData:>"InverseMellin",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Isolate",
  ButtonData:>"Isolate",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Kummer",
  ButtonData:>"Kummer",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Lagrangian",
  ButtonData:>"Lagrangian",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["LC",
  ButtonData:>"LC",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["LCD",
  ButtonData:>"LCD",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["LeftPartialD",
  ButtonData:>"LeftPartialD",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["LeftRightPartialD",
  ButtonData:>"LeftRightPartialD",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["LeftRightPartialD2",
  ButtonData:>"LeftRightPartialD2",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["LeviCivita",
  ButtonData:>"LeviCivita",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Li2",
  ButtonData:>"Li2",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Li3",
  ButtonData:>"Li3",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["LorentzIndex",
  ButtonData:>"LorentzIndex",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Map2",
  ButtonData:>"Map2",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["MetricTensor",
  ButtonData:>"MetricTensor",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["MomentumCombine",
  ButtonData:>"MomentumCombine",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["MomentumExpand",
  ButtonData:>"MomentumExpand",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Momentum",
  ButtonData:>"Momentum",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["MT",
  ButtonData:>"MT",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["MTD",
  ButtonData:>"MTD",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Nielsen",
  ButtonData:>"Nielsen",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["NTerms",
  ButtonData:>"NTerms",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["OneLoop",
  ButtonData:>"OneLoop",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["OneLoopSimplify",
  ButtonData:>"OneLoopSimplify",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["OPEDelta",
  ButtonData:>"OPEDelta",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["OPEi, OPEj, OPEk, OPEl, OPEm, OPEn, OPEo",
  ButtonData:>"OPEi",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["OPESumExplicit",
  ButtonData:>"OPESumExplicit",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["OPESum",
  ButtonData:>"OPESum",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["OPESumSimplify",
  ButtonData:>"OPESumSimplify",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Pair",
  ButtonData:>"Pair",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["PartialD",
  ButtonData:>"PartialD",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["PaVe",
  ButtonData:>"PaVe",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["PaVeOrder",
  ButtonData:>"PaVeOrder",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["PaVeReduce",
  ButtonData:>"PaVeReduce",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["PlusDistribution",
  ButtonData:>"PlusDistribution",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Polarization",
  ButtonData:>"Polarization",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["PolarizationSum",
  ButtonData:>"PolarizationSum",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["PolarizationVector",
  ButtonData:>"PolarizationVector",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["PowerSimplify",
  ButtonData:>"PowerSimplify",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["PropagatorDenominatorExplicit",
  ButtonData:>"PropagatorDenominatorExplicit",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["PropagatorDenominator",
  ButtonData:>"PropagatorDenominator",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["QuantumField",
  ButtonData:>"QuantumField",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["QuarkField",
  ButtonData:>"QuarkField",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["QuarkGluonVertex",
  ButtonData:>"QuarkGluonVertex",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["QuarkPropagator",
  ButtonData:>"QuarkPropagator",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["RightPartialD",
  ButtonData:>"RightPartialD",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["RussianTrick",
  ButtonData:>"RussianTrick",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["ScalarProduct",
  ButtonData:>"ScalarProduct",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["ScalarProductCancel",
  ButtonData:>"ScalarProductCancel",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["SD",
  ButtonData:>"SD",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["SelectFree",
  ButtonData:>"SelectFree",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["SelectNotFree",
  ButtonData:>"SelectNotFree",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Series2",
  ButtonData:>"Series2",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["SimplifyDeltaFunction",
  ButtonData:>"SimplifyDeltaFunction",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["SimplifyPolyLog",
  ButtonData:>"SimplifyPolyLog",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["SOD",
  ButtonData:>"SOD",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Solve2",
  ButtonData:>"Solve2",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["SO",
  ButtonData:>"SO",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["SP",
  ButtonData:>"SP",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["SPD",
  ButtonData:>"SPD",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Spinor",
  ButtonData:>"Spinor",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["SpinorUBar",
  ButtonData:>"SpinorUBar",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["SpinorU",
  ButtonData:>"SpinorU",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["SpinorVBar",
  ButtonData:>"SpinorVBar",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["SpinorV",
  ButtonData:>"SpinorV",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["SplittingFunction",
  ButtonData:>"SplittingFunction",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["SumP",
  ButtonData:>"SumP",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["SumS",
  ButtonData:>"SumS",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["SumT",
  ButtonData:>"SumT",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["SUNDelta",
  ButtonData:>"SUNDelta",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["SUND",
  ButtonData:>"SUND",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["SUNF",
  ButtonData:>"SUNF",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["SUNIndex",
  ButtonData:>"SUNIndex",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["SUNSimplify",
  ButtonData:>"SUNSimplify",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["SUNT",
  ButtonData:>"SUNT",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["SUNTrace",
  ButtonData:>"SUNTrace",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Symmetrize",
  ButtonData:>"Symmetrize",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Tdec",
  ButtonData:>"Tdec",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Tensorfunction",
  ButtonData:>"Tensorfunction",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["TFi",
  ButtonData:>"TFi",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["TIDL",
  ButtonData:>"TIDL",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["TID",
  ButtonData:>"TID",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["ToDistribution",
  ButtonData:>"ToDistribution",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["ToHypergeometric",
  ButtonData:>"ToHypergeometric",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["ToLarin",
  ButtonData:>"ToLarin",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["ToTFi",
  ButtonData:>"ToTFi",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Tr",
  ButtonData:>"Tr",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["TrickMandelstam",
  ButtonData:>"TrickMandelstam",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Trick",
  ButtonData:>"Trick",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Twist2GluonOperator",
  ButtonData:>"Twist2GluonOperator",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Twist2QuarkOperator",
  ButtonData:>"Twist2QuarkOperator",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Uncontract",
  ButtonData:>"Uncontract",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["UnDeclareNonCommutative",
  ButtonData:>"UnDeclareNonCommutative",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Write2",
  ButtonData:>"Write2",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Zeta2",
  ButtonData:>"Zeta2",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["$FCT",
  ButtonData:>"$FCT",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["$LorentzIndices",
  ButtonData:>"$LorentzIndices",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["$PairBrackets",
  ButtonData:>"$PairBrackets",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["$VeryVerbose",
  ButtonData:>"$VeryVerbose",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["$West",
  ButtonData:>"$West",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True],

Cell[TextData[ButtonBox["Evaluation time and memory usage",
  ButtonData:>"Evaluation time and memory usage",
  ButtonStyle->"Hyperlink"]], "Index",
  GeneratedCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Introduction", "Section",
  PageBreakAbove->True,
  CellTags->"Short Overview"],

Cell[TextData[{
  "FeynCalc 4.1 is a ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 3.0 or 4.0 package for algebraic calculations in high energy physics. \
Tools are provided for Lorentz index contraction, Dirac algebra manipulation, \
color factor calculation, automatic Feynman rule derivation, general \
noncommutative algebra as well as various look-up tables for Feynman \
parameter integrals, Mellin transforms (e.g. all integrals - except 57,58 and \
59, and correcting a minor misprint in 14) of Appendix 7 from ",
  ButtonBox["hep-ph/9810241",
    ButtonData:>{
      URL[ "http://xxx.lanl.gov/abs/hep-ph/9810241"], None},
    ButtonStyle->"Hyperlink"],
  " are tabulated in ",
  ButtonBox["Integrate2",
    ButtonData:>"Integrate2",
    ButtonStyle->"Hyperlink"],
  " , convolutions and Feynman rules. Furthermore special translation \
facilities are provided to change the FeynCalc syntax to and from FORM \
syntax. Optimized FORTRAN generation can be done with ",
  ButtonBox["Isolate",
    ButtonData:>"Isolate",
    ButtonStyle->"Hyperlink"],
  " and ",
  ButtonBox["Write2",
    ButtonData:>"Write2",
    ButtonStyle->"Hyperlink"],
  "."
}], "Text"],

Cell[TextData[{
  "The more important functions for input of objects like ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox["p", "\[Mu]"], ",", 
          SuperscriptBox["\[Gamma]", "\[Nu]"], ",", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", " ", "\[Nu]"}]], ",", " ", 
          SuperscriptBox["\[CurlyEpsilon]", 
            RowBox[{"\[Mu]", " ", "\[Nu]", " ", "\[Rho]", " ", "\[Sigma]"}]], 
          ",", " ", 
          RowBox[{"etc", "."}]}], TraditionalForm]]],
  " (abbreviations in parentheses) are: ",
  ButtonBox["DiracMatrix",
    ButtonData:>"DiracMatrix",
    ButtonStyle->"Hyperlink"],
  " (",
  ButtonBox["GA",
    ButtonData:>"GA",
    ButtonStyle->"Hyperlink"],
  "), ",
  ButtonBox["DiracSlash",
    ButtonData:>"DiracSlash",
    ButtonStyle->"Hyperlink"],
  " (",
  ButtonBox["GS",
    ButtonData:>"GS",
    ButtonStyle->"Hyperlink"],
  "), ",
  ButtonBox["FourVector",
    ButtonData:>"FourVector",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FourVector"],
  " (",
  ButtonBox["FV",
    ButtonData:>"FV",
    ButtonStyle->"Hyperlink"],
  "), ",
  ButtonBox["LeviCivita",
    ButtonData:>"LeviCivita",
    ButtonStyle->"Hyperlink"],
  " (",
  ButtonBox["LC",
    ButtonData:>"LC",
    ButtonStyle->"Hyperlink"],
  "), ",
  ButtonBox["MetricTensor",
    ButtonData:>"MetricTensor",
    ButtonStyle->"Hyperlink"],
  " (",
  ButtonBox["MT",
    ButtonData:>"MT",
    ButtonStyle->"Hyperlink"],
  "), ",
  ButtonBox["Spinor",
    ButtonData:>"Spinor",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["SUNDelta",
    ButtonData:>"SUNDelta",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["SUNF",
    ButtonData:>"SUNF",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["ScalarProduct",
    ButtonData:>"ScalarProduct",
    ButtonStyle->"Hyperlink"],
  " (",
  ButtonBox["SP",
    ButtonData:>"SP",
    ButtonStyle->"Hyperlink"],
  "). "
}], "Text"],

Cell["The more important functions for manipulations are:", "Text"],

Cell[TextData[{
  ButtonBox["Contract",
    ButtonData:>"Contract",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["Tr",
    ButtonData:>"Tr",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracSimplify"],
  ", ",
  ButtonBox["DiracTrick",
    ButtonData:>"DiracTrick",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracTrick"],
  ", ",
  ButtonBox["Calc",
    ButtonData:>"Calc",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Calc"],
  ", ",
  ButtonBox["Convolute",
    ButtonData:>"Convolute",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["FeynRule",
    ButtonData:>"FeynRule",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["Integrate2",
    ButtonData:>"Integrate2",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["FeynCalc2FORM",
    ButtonData:>"FeynCalc2FORM",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["PaVeReduce",
    ButtonData:>"PaVeReduce",
    ButtonStyle->"Hyperlink"],
  " ,",
  ButtonBox["SimplifyPolyLog",
    ButtonData:>"SimplifyPolyLog",
    ButtonStyle->"Hyperlink"],
  " , ",
  ButtonBox["SUNSimplify",
    ButtonData:>"SUNSimplify",
    ButtonStyle->"Hyperlink"],
  " , ",
  ButtonBox["Isolate",
    ButtonData:>"Isolate",
    ButtonStyle->"Hyperlink"],
  " , ",
  ButtonBox["Collect2",
    ButtonData:>"Collect2",
    ButtonStyle->"Hyperlink"],
  " , ",
  ButtonBox["OneLoopSimplify",
    ButtonData:>"OneLoopSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"OneLoopSimplify"],
  ", ",
  ButtonBox["Series2",
    ButtonData:>"Series2",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Series2"],
  " , ",
  ButtonBox["ScalarProductCancel",
    ButtonData:>"ScalarProductCancel",
    ButtonStyle->"Hyperlink"],
  "."
}], "Text"],

Cell[TextData[{
  "FeynCalc 4.1 is still compatible with FeynArts 2.2 if the FeynArts \
function ToFA1Convention is used on the generated amplitudes. The functions \
OneLoop and OneLoopSum for calculation of Standard Model (-like) 1-loop \
diagrams still work in FeynCalc 3.1 but they have not been developed much \
further compared to the FeynCalc 1.0 version. While ",
  ButtonBox["OneLoop",
    ButtonData:>"OneLoop",
    ButtonStyle->"Hyperlink"],
  " and OneLoopSum still work fine for self-energy, triangle and box diagrams \
not too complicated the reader should be aware of the package FormCalc which \
is more effective for the adventurous researcher calculating thousands of \
diagrams involving four external gauge bosons. Notice however that FormCalc \
does not reduce the Passarino-Veltman integrals to scalar integrals. Also \
FormCalc is limited to the 't Hooft Feynman gauge. FeynCalc provides the \
possibility to reduce Passarino-Veltman integrals to scalar integrals \
algebraically by the function ",
  ButtonBox["PaVeReduce",
    ButtonData:>"PaVeReduce",
    ButtonStyle->"Hyperlink",
    ButtonNote->"PaVeReduce"],
  " ."
}], "Text"],

Cell[TextData[{
  "On a Pentium II 300 MHz computer with 128 MB of RAM running ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 3.0 for Linux this noteboook needs less than 15 minutes to evaluate and \
less than 16 MB of memory."
}], "Text"],

Cell[TextData[{
  "FeynCalc automatically loads any tarcer*.mx file from the tarcer \
directory. For a description of TARCER see ",
  ButtonBox["hep-ph/9801383",
    ButtonData:>{
      URL[ "http://xxx.lanl.gov/abs/hep-ph/9801383"], None},
    ButtonStyle->"Hyperlink"],
  " or the published version in Computer Physics Communications 111 (1998) \
265-273."
}], "Text"],

Cell["\<\
For more detailed information concerning FeynCalc functions you can \
investigate all of the source code in the FeynCalc.m file in the \
HighEnergyPhysics directory and the further packages in the directories \
fctools, fctables, general, qcd, tarcer and others. A complete list of \
FeynCalc objects is stored in the variable $FeynCalcStuff. Not all functions \
are documented in this notebook. Some are either very special ones (e.g. for \
tools for 2-loop QCD diagrams originating in twist-2 OPE) or still \
experimental.\
\>", "Text"],

Cell[TextData[{
  "The ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  "scheme used in OneLoop is the naive one, i.e., ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              SuperscriptBox["\[Gamma]", "5"], ",", 
              SuperscriptBox["\[Gamma]", "\[Mu]"]}], ")"}], "=", "0"}], 
        TraditionalForm]]],
  " in 4 and D dimensions. See also ",
  ButtonBox["ToLarin",
    ButtonData:>"ToLarin",
    ButtonStyle->"Hyperlink"]
}], "Text"],

Cell["The metric used is the one from Bjorken and Drell (+---).", "Text"],

Cell[TextData[{
  "For suggestions and bug reports please contact ",
  ButtonBox["http://www.feyncalc.org",
    ButtonData:>{
      URL[ "http://www.feyncalc.org"], None},
    ButtonStyle->"Hyperlink"]
}], "Text",
  GeneratedCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Start FeynCalc", "Section",
  CellTags->"Load FeynCalc"],

Cell["\<\
This is only for timing purposes of this notebook and can be \
omitted in general.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"absolutestarttime", "=", 
      RowBox[{"AbsoluteTime", "[", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox["3.173197715`15.5221*^9", TraditionalForm]], "Output"]
}, Open  ]],

Cell["This defines typesetting rules for q1 and q2:", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"q1", "/:", 
        RowBox[{"MakeBoxes", "[", 
          RowBox[{"q1", ",", "TraditionalForm"}], "]"}], ":=", 
        RowBox[{"InterpretationBox", "[", 
          RowBox[{
            RowBox[{"SubscriptBox", "[", 
              RowBox[{"q", ",", "1"}], "]"}], ",", "q1"}], "]"}]}], ";"}]], 
  "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"q2", "/:", 
        RowBox[{"MakeBoxes", "[", 
          RowBox[{"q2", ",", "TraditionalForm"}], "]"}], ":=", 
        RowBox[{"InterpretationBox", "[", 
          RowBox[{
            RowBox[{"SubscriptBox", "[", 
              RowBox[{"q", ",", "2"}], "]"}], ",", "q2"}], "]"}]}], ";"}]], 
  "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{"q1", ",", "q2"}], "}"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          InterpretationBox[
            SubscriptBox["q", "1"],
            q1], ",", 
          InterpretationBox[
            SubscriptBox["q", "2"],
            q2]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{"q1", ",", "q2"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
  "By loading FeynCalc the setting of the default output format type is \
automatically set to TraditionalForm. You might want to reverse this after \
quitting FeynCalc by clicking on the ",
  StyleBox["Cell", "TextNormal",
    FontWeight->"Bold"],
  " menu, then go to the ",
  StyleBox["Default Output Format Type ", "TextNormal",
    FontWeight->"Bold"],
  "item and select ",
  StyleBox["StandardForm",
    FontWeight->"Bold"],
  ", or by evaluating ",
  StyleBox[
  "SetOptions[$FrontEnd,\"CommonDefaultFormatTypes\"\[Rule]{\"Output\"\[Rule]\
StandardForm}]", "Input"],
  " ."
}], "Text"],

Cell["This loads FeynCalc:", "Text"],

Cell[BoxData[
    RowBox[{"<<", "HighEnergyPhysics`FeynCalc`"}]], "Input"],

Cell[TextData[{
  StyleBox["FeynCalc",
    FontWeight->"Bold"],
  " ",
  "4.1.0.1",
  "    ",
  " Evaluate ?FeynCalc for help or visit ",
  ButtonBox["www.feyncalc.org",
    ButtonData:>{
      URL[ "http://www.feyncalc.org"], None},
    ButtonStyle->"Hyperlink",
    ButtonNote->"http://www.feyncalc.org"]
}], "Text",
  GeneratedCell->True,
  CellAutoOverwrite->True],

Cell[TextData[{
  "You can also put the last input line into your init.m file, or use the \
button FCStartButton.nb from ",
  ButtonBox["http://www.feyncalc.org/documentation/FCStartButton.nb",
    ButtonData:>{
      URL[ "http://www.feyncalc.org/documentation/FCStartButton.nb"], None},
    ButtonStyle->"Hyperlink"]
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example Calculations", "Section",
  PageBreakAbove->True,
  CellTags->"Example Calculations"],

Cell["\<\
This section is still under development ... Please send suggestions \
to me ...\
\>", "Commentary"],

Cell[TextData[{
  "The following subsections are examples from various areas of research or \
education. See also ",
  ButtonBox["www.feyncalc.com/examples",
    ButtonData:>{
      URL[ "http://www.feyncalc.org/examples"], None},
    ButtonStyle->"Hyperlink",
    ButtonNote->"http://www.feyncalc.org/examples"],
  "."
}], "Text"],

Cell[CellGroupData[{

Cell["Standard Model", "Subsection"],

Cell[CellGroupData[{

Cell[TextData[{
  "Tree level ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"\[Gamma]", " ", "\[Gamma]"}], " ", "\[Rule]", " ", 
          RowBox[{
            SuperscriptBox["e", "+"], 
            SuperscriptBox["e", "-"]}]}], TraditionalForm]]]
}], "Subsubsection",
  CellTags->"Tree level"],

Cell[TextData[{
  "Tree level pair production ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"\[Gamma]", " ", "\[Gamma]"}], " ", "\[Rule]", " ", 
          RowBox[{
            SuperscriptBox["e", "+"], 
            SuperscriptBox["e", "-"]}]}], TraditionalForm]]]
}], "Text"],

Cell["Define typesetting rules for p1, p2, k1, k2", "Text"],

Cell["\<\
p1/: MakeBoxes[p1, fmt_] := InterpretationBox[SubscriptBox[p, 1], \
p1]\
\>", "Input"],

Cell["\<\
k1/: MakeBoxes[k1, fmt_] := InterpretationBox[SubscriptBox[k, 1], \
k1]\
\>", "Input"],

Cell["\<\
p2/: MakeBoxes[p2, fmt_] := InterpretationBox[SubscriptBox[p, 2], \
p2]\
\>", "Input"],

Cell["\<\
k2/: MakeBoxes[k2, fmt_] := InterpretationBox[SubscriptBox[k, 2], \
k2]\
\>", "Input"],

Cell["Define slashes of external momenta to save future typing.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"{", 
        RowBox[{"p1sl", ",", "p2sl", ",", "k1sl", ",", "k2sl"}], "}"}], "=", 
      RowBox[{"DiracSlash", "/@", 
        RowBox[{"{", 
          RowBox[{"p1", ",", "p2", ",", "k1", ",", "k2"}], "}"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox[
              InterpretationBox[
                SubscriptBox["p", "1"],
                p1],
              "TraditionalForm"]}], ",", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox[
              InterpretationBox[
                SubscriptBox["p", "2"],
                p2],
              "TraditionalForm"]}], ",", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox[
              InterpretationBox[
                SubscriptBox["k", "1"],
                k1],
              "TraditionalForm"]}], ",", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox[
              InterpretationBox[
                SubscriptBox["k", "2"],
                k2],
              "TraditionalForm"]}]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "Kinematic definitions so that only ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["p", "1"], ".", 
          SubscriptBox["k", "1"]}], TraditionalForm]]],
  ", ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["p", "1"], ".", 
          SubscriptBox["k", "2"]}], TraditionalForm]]],
  " appear in final results. I.e., define ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubsuperscriptBox["p", "1", "2"], "=", 
          RowBox[{
            SubsuperscriptBox["p", "2", "2"], "=", " ", 
            SuperscriptBox["m", "2"]}]}], TraditionalForm]]],
  " ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubsuperscriptBox["k", "1", "2"], "=", 
          RowBox[{
            SubsuperscriptBox["k", "2", "2"], "=", " ", "0"}]}], 
        TraditionalForm]]],
  " ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            SubscriptBox["k", "1"], "\[CenterDot]", 
            SubscriptBox["k", "2"]}], " ", "=", " ", 
          RowBox[{
            RowBox[{
              SubscriptBox["p", "1"], "\[CenterDot]", 
              SubscriptBox["p", "2"]}], "+", 
            SuperscriptBox["m", "2"]}]}], TraditionalForm]]],
  " ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            SubscriptBox["k", "1"], "\[CenterDot]", 
            SubscriptBox["p", "2"]}], " ", "=", " ", 
          RowBox[{
            SubscriptBox["k", "2"], "\[CenterDot]", 
            SubscriptBox["p", "1"]}]}], TraditionalForm]]],
  " ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            SubscriptBox["k", "2"], "\[CenterDot]", 
            SubscriptBox["p", "2"]}], " ", "=", " ", 
          RowBox[{
            SubscriptBox["k", "1"], "\[CenterDot]", 
            SubscriptBox["p", "1"]}]}], TraditionalForm]]],
  " ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            SubscriptBox["p", "1"], "\[CenterDot]", 
            SubscriptBox["p", "2"]}], "=", " ", 
          RowBox[{
            RowBox[{
              SubscriptBox["k", "2"], "\[CenterDot]", 
              SubscriptBox["p", "1"]}], "-", 
            SuperscriptBox["m", "2"], "+", 
            RowBox[{
              SubscriptBox["k", "1"], "\[CenterDot]", 
              SubscriptBox["p", "1"]}]}]}], TraditionalForm]]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"ScalarProduct", "[", 
          RowBox[{"p1", ",", "p1"}], "]"}], "=", 
        RowBox[{
          RowBox[{"ScalarProduct", "[", 
            RowBox[{"p2", ",", "p2"}], "]"}], "=", 
          SuperscriptBox["m", "2"]}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"ScalarProduct", "[", 
          RowBox[{"k1", ",", "k1"}], "]"}], "=", 
        RowBox[{
          RowBox[{"ScalarProduct", "[", 
            RowBox[{"k2", ",", "k2"}], "]"}], "=", "0"}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"ScalarProduct", "[", 
          RowBox[{"k1", ",", "k2"}], "]"}], "=", 
        RowBox[{
          RowBox[{"ScalarProduct", "[", 
            RowBox[{"p1", ",", "p2"}], "]"}], "+", 
          SuperscriptBox["m", "2"]}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"ScalarProduct", "[", 
          RowBox[{"k1", ",", "p2"}], "]"}], "=", 
        RowBox[{"ScalarProduct", "[", 
          RowBox[{"k2", ",", "p1"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"ScalarProduct", "[", 
          RowBox[{"k2", ",", "p2"}], "]"}], "=", 
        RowBox[{"ScalarProduct", "[", 
          RowBox[{"k1", ",", "p1"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"ScalarProduct", "[", 
          RowBox[{"p1", ",", "p2"}], "]"}], "=", 
        RowBox[{
          RowBox[{"ScalarProduct", "[", 
            RowBox[{"k2", ",", "p1"}], "]"}], "-", 
          SuperscriptBox["m", "2"], "+", 
          RowBox[{"ScalarProduct", "[", 
            RowBox[{"k1", ",", "p1"}], "]"}]}]}], ";"}]], "Input"],

Cell["Define the amplitudes.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"f1", "=", 
      RowBox[{
        RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
        RowBox[{"(", 
          RowBox[{"p1sl", "-", "k1sl", "+", "m"}], ")"}], ".", 
        RowBox[{"DiracMatrix", "[", "\[Nu]", "]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        RowBox[{"(", 
          RowBox[{"m", "-", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                InterpretationBox[
                  SubscriptBox["k", "1"],
                  k1],
                "TraditionalForm"]}], "+", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                InterpretationBox[
                  SubscriptBox["p", "1"],
                  p1],
                "TraditionalForm"]}]}], ")"}], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"f2", "=", 
      RowBox[{
        RowBox[{"DiracMatrix", "[", "\[Nu]", "]"}], ".", 
        RowBox[{"(", 
          RowBox[{"p1sl", "-", "k2sl", "+", "m"}], ")"}], ".", 
        RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        RowBox[{"(", 
          RowBox[{"m", "-", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                InterpretationBox[
                  SubscriptBox["k", "2"],
                  k2],
                "TraditionalForm"]}], "+", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                InterpretationBox[
                  SubscriptBox["p", "1"],
                  p1],
                "TraditionalForm"]}]}], ")"}], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"f3", "=", 
      RowBox[{
        RowBox[{"DiracMatrix", "[", "\[Nu]", "]"}], ".", 
        RowBox[{"(", 
          RowBox[{"p1sl", "-", "k1sl", "+", "m"}], ")"}], ".", 
        RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        RowBox[{"(", 
          RowBox[{"m", "-", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                InterpretationBox[
                  SubscriptBox["k", "1"],
                  k1],
                "TraditionalForm"]}], "+", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                InterpretationBox[
                  SubscriptBox["p", "1"],
                  p1],
                "TraditionalForm"]}]}], ")"}], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"f4", "=", 
      RowBox[{
        RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
        RowBox[{"(", 
          RowBox[{"p1sl", "-", "k2sl", "+", "m"}], ")"}], ".", 
        RowBox[{"DiracMatrix", "[", "\[Nu]", "]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        RowBox[{"(", 
          RowBox[{"m", "-", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                InterpretationBox[
                  SubscriptBox["k", "2"],
                  k2],
                "TraditionalForm"]}], "+", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                InterpretationBox[
                  SubscriptBox["p", "1"],
                  p1],
                "TraditionalForm"]}]}], ")"}], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[" Calculation of the four traces.", "Text"],

Cell[CellGroupData[{

Cell["tr1 = Tr[(p1sl + m) . f1 . (p2sl - m) . f3]", "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", "8"}], " ", 
              SuperscriptBox["m", "4"]}], "+", 
            RowBox[{"8", " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "1"],
                    k1],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  InterpretationBox[
                    SubscriptBox["p", "1"],
                    p1],
                  "TraditionalForm"]}], " ", 
              SuperscriptBox["m", "2"]}], "+", 
            RowBox[{"8", " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "1"],
                    k1],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  InterpretationBox[
                    SubscriptBox["p", "1"],
                    p1],
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "2"],
                    k2],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  InterpretationBox[
                    SubscriptBox["p", "1"],
                    p1],
                  "TraditionalForm"]}]}]}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["tr2 = Tr[(p1sl + m) . f1 . (p2sl - m) . f4]", "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", "8"}], " ", 
              SuperscriptBox["m", "4"]}], "+", 
            RowBox[{"4", " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "1"],
                    k1],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  InterpretationBox[
                    SubscriptBox["p", "1"],
                    p1],
                  "TraditionalForm"]}], " ", 
              SuperscriptBox["m", "2"]}], "+", 
            RowBox[{"4", " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "2"],
                    k2],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  InterpretationBox[
                    SubscriptBox["p", "1"],
                    p1],
                  "TraditionalForm"]}], " ", 
              SuperscriptBox["m", "2"]}]}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["tr3 = Tr[(p1sl + m) . f2 . (p2sl - m) . f3]", "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", "8"}], " ", 
              SuperscriptBox["m", "4"]}], "+", 
            RowBox[{"4", " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "1"],
                    k1],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  InterpretationBox[
                    SubscriptBox["p", "1"],
                    p1],
                  "TraditionalForm"]}], " ", 
              SuperscriptBox["m", "2"]}], "+", 
            RowBox[{"4", " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "2"],
                    k2],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  InterpretationBox[
                    SubscriptBox["p", "1"],
                    p1],
                  "TraditionalForm"]}], " ", 
              SuperscriptBox["m", "2"]}]}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["tr4 = Tr[(p1sl + m) . f2 . (p2sl - m) . f4]", "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", "8"}], " ", 
              SuperscriptBox["m", "4"]}], "+", 
            RowBox[{"8", " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "2"],
                    k2],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  InterpretationBox[
                    SubscriptBox["p", "1"],
                    p1],
                  "TraditionalForm"]}], " ", 
              SuperscriptBox["m", "2"]}], "+", 
            RowBox[{"8", " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "1"],
                    k1],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  InterpretationBox[
                    SubscriptBox["p", "1"],
                    p1],
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "2"],
                    k2],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  InterpretationBox[
                    SubscriptBox["p", "1"],
                    p1],
                  "TraditionalForm"]}]}]}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell["Divide by the denominators and add everything up.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"matrixsq", "=", 
      RowBox[{
        FractionBox["tr1", 
          SuperscriptBox[
            RowBox[{"ScalarProduct", "[", 
              RowBox[{
                RowBox[{"2", " ", "p1"}], ",", "k1"}], "]"}], "2"]], "+", 
        FractionBox["tr4", 
          SuperscriptBox[
            RowBox[{"ScalarProduct", "[", 
              RowBox[{
                RowBox[{"2", " ", "p1"}], ",", "k2"}], "]"}], "2"]], "+", 
        FractionBox[
          RowBox[{"tr2", "+", "tr3"}], 
          RowBox[{
            RowBox[{"ScalarProduct", "[", 
              RowBox[{
                RowBox[{"2", " ", "p1"}], ",", "k1"}], "]"}], " ", 
            RowBox[{"ScalarProduct", "[", 
              RowBox[{
                RowBox[{"2", " ", "p1"}], ",", "k2"}], "]"}]}]]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{"2", " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"-", "8"}], " ", 
                  SuperscriptBox["m", "4"]}], "+", 
                RowBox[{"4", " ", 
                  RowBox[{
                    FormBox[
                      InterpretationBox[
                        SubscriptBox["k", "1"],
                        k1],
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox[
                      InterpretationBox[
                        SubscriptBox["p", "1"],
                        p1],
                      "TraditionalForm"]}], " ", 
                  SuperscriptBox["m", "2"]}], "+", 
                RowBox[{"4", " ", 
                  RowBox[{
                    FormBox[
                      InterpretationBox[
                        SubscriptBox["k", "2"],
                        k2],
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox[
                      InterpretationBox[
                        SubscriptBox["p", "1"],
                        p1],
                      "TraditionalForm"]}], " ", 
                  SuperscriptBox["m", "2"]}]}], ")"}]}], 
          RowBox[{
            RowBox[{
              FormBox[
                InterpretationBox[
                  SubscriptBox["k", "1"],
                  k1],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                InterpretationBox[
                  SubscriptBox["p", "1"],
                  p1],
                "TraditionalForm"]}], " ", 
            RowBox[{
              FormBox[
                InterpretationBox[
                  SubscriptBox["k", "2"],
                  k2],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                InterpretationBox[
                  SubscriptBox["p", "1"],
                  p1],
                "TraditionalForm"]}]}]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{
              RowBox[{"-", "8"}], " ", 
              SuperscriptBox["m", "4"]}], "+", 
            RowBox[{"8", " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "1"],
                    k1],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  InterpretationBox[
                    SubscriptBox["p", "1"],
                    p1],
                  "TraditionalForm"]}], " ", 
              SuperscriptBox["m", "2"]}], "+", 
            RowBox[{"8", " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "1"],
                    k1],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  InterpretationBox[
                    SubscriptBox["p", "1"],
                    p1],
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "2"],
                    k2],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  InterpretationBox[
                    SubscriptBox["p", "1"],
                    p1],
                  "TraditionalForm"]}]}]}], 
          SuperscriptBox[
            RowBox[{"(", 
              FormBox[
                RowBox[{
                  FormBox[
                    InterpretationBox[
                      SubscriptBox["k", "1"],
                      k1],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    InterpretationBox[
                      SubscriptBox["p", "1"],
                      p1],
                    "TraditionalForm"]}],
                "TraditionalForm"], ")"}], "2"]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{
              RowBox[{"-", "8"}], " ", 
              SuperscriptBox["m", "4"]}], "+", 
            RowBox[{"8", " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "2"],
                    k2],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  InterpretationBox[
                    SubscriptBox["p", "1"],
                    p1],
                  "TraditionalForm"]}], " ", 
              SuperscriptBox["m", "2"]}], "+", 
            RowBox[{"8", " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "1"],
                    k1],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  InterpretationBox[
                    SubscriptBox["p", "1"],
                    p1],
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox[
                  InterpretationBox[
                    SubscriptBox["k", "2"],
                    k2],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  InterpretationBox[
                    SubscriptBox["p", "1"],
                    p1],
                  "TraditionalForm"]}]}]}], 
          SuperscriptBox[
            RowBox[{"(", 
              FormBox[
                RowBox[{
                  FormBox[
                    InterpretationBox[
                      SubscriptBox["k", "2"],
                      k2],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    InterpretationBox[
                      SubscriptBox["p", "1"],
                      p1],
                    "TraditionalForm"]}],
                "TraditionalForm"], ")"}], "2"]]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell["Collect with respect to powers of m.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"r", "=", 
      RowBox[{"Collect2", "[", 
        RowBox[{"matrixsq", ",", "m"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox[
            RowBox[{"8", " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      FormBox[
                        InterpretationBox[
                          SubscriptBox["k", "1"],
                          k1],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        InterpretationBox[
                          SubscriptBox["p", "1"],
                          p1],
                        "TraditionalForm"]}], "+", 
                    RowBox[{
                      FormBox[
                        InterpretationBox[
                          SubscriptBox["k", "2"],
                          k2],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        InterpretationBox[
                          SubscriptBox["p", "1"],
                          p1],
                        "TraditionalForm"]}]}], ")"}], "2"], " ", 
              SuperscriptBox["m", "4"]}], 
            RowBox[{
              SuperscriptBox[
                RowBox[{"(", 
                  FormBox[
                    RowBox[{
                      FormBox[
                        InterpretationBox[
                          SubscriptBox["k", "1"],
                          k1],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        InterpretationBox[
                          SubscriptBox["p", "1"],
                          p1],
                        "TraditionalForm"]}],
                    "TraditionalForm"], ")"}], "2"], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  FormBox[
                    RowBox[{
                      FormBox[
                        InterpretationBox[
                          SubscriptBox["k", "2"],
                          k2],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        InterpretationBox[
                          SubscriptBox["p", "1"],
                          p1],
                        "TraditionalForm"]}],
                    "TraditionalForm"], ")"}], "2"]}]]}], "+", 
        FractionBox[
          RowBox[{"16", " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FormBox[
                    InterpretationBox[
                      SubscriptBox["k", "1"],
                      k1],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    InterpretationBox[
                      SubscriptBox["p", "1"],
                      p1],
                    "TraditionalForm"]}], "+", 
                RowBox[{
                  FormBox[
                    InterpretationBox[
                      SubscriptBox["k", "2"],
                      k2],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    InterpretationBox[
                      SubscriptBox["p", "1"],
                      p1],
                    "TraditionalForm"]}]}], ")"}], " ", 
            SuperscriptBox["m", "2"]}], 
          RowBox[{
            RowBox[{
              FormBox[
                InterpretationBox[
                  SubscriptBox["k", "1"],
                  k1],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                InterpretationBox[
                  SubscriptBox["p", "1"],
                  p1],
                "TraditionalForm"]}], " ", 
            RowBox[{
              FormBox[
                InterpretationBox[
                  SubscriptBox["k", "2"],
                  k2],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                InterpretationBox[
                  SubscriptBox["p", "1"],
                  p1],
                "TraditionalForm"]}]}]], "+", 
        FractionBox[
          RowBox[{"8", " ", 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["k", "1"],
                            k1],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["p", "1"],
                            p1],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"], "+", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["k", "2"],
                            k2],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["p", "1"],
                            p1],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"]}], ")"}]}], 
          RowBox[{
            RowBox[{
              FormBox[
                InterpretationBox[
                  SubscriptBox["k", "1"],
                  k1],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                InterpretationBox[
                  SubscriptBox["p", "1"],
                  p1],
                "TraditionalForm"]}], " ", 
            RowBox[{
              FormBox[
                InterpretationBox[
                  SubscriptBox["k", "2"],
                  k2],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                InterpretationBox[
                  SubscriptBox["p", "1"],
                  p1],
                "TraditionalForm"]}]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
This is a possibility to check the symmetry in k1 and k2 \
explicitly.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"(", 
      RowBox[{"r", " ", "-", " ", 
        RowBox[{"(", 
          RowBox[{"r", " ", "/.", " ", 
            RowBox[{"{", " ", 
              RowBox[{
                RowBox[{"k1", "\[RuleDelayed]", "k2"}], ",", " ", 
                RowBox[{"k2", "\[RuleDelayed]", "k1"}]}], "}"}]}], ")"}]}], 
      ")"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
Remove the definitions for the scalar products and for the \
typesetting definitions.\
\>", "Text"],

Cell[BoxData[
    RowBox[{"ClearScalarProducts", ";", " ", 
      RowBox[{"Clear", "[", 
        RowBox[{"p1", ",", "p2", ",", "k1", ",", "k2", ",", "r"}], "]"}], 
      ";"}]], "Input"],

Cell[TextData[{
  "See also: ",
  ButtonBox["ClearScalarProducts",
    ButtonData:>"ClearScalarProducts",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["Collect2",
    ButtonData:>"Collect2",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["DiracMatrix",
    ButtonData:>"DiracMatrix",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["DiracSlash",
    ButtonData:>"DiracSlash",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["ScalarProduct",
    ButtonData:>"ScalarProduct",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["Tr",
    ButtonData:>"Tr",
    ButtonStyle->"Hyperlink"],
  "."
}], "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Miscellaneous calculations", "Subsection"],

Cell[CellGroupData[{

Cell["A moment integral", "Subsubsection",
  CellTags->"A moment integral"],

Cell[TextData[{
  "Moments of ",
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox[
          StyleBox[
            OverscriptBox[
              StyleBox["A",
                FontSlant->"Plain"], "^"],
            FontSlant->"Plain"], "qq", 
          RowBox[{"NS", ",", "PHYS"}]], TraditionalForm]]],
  " from ",
  ButtonBox["hep-ph/9801224",
    ButtonData:>{
      URL[ "http://xxx.lanl.gov/abs/hep-ph/9801224"], None},
    ButtonStyle->"Hyperlink"]
}], "Text"],

Cell["\<\
This is an example of integration and of translation from FORM \
syntax.\
\>", "Text"],

Cell[TextData[{
  "Due to current conservation the first moment of ",
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox[
          StyleBox[
            OverscriptBox[
              StyleBox["A",
                FontSlant->"Plain"], "^"],
            FontSlant->"Plain"], "qq", 
          RowBox[{"NS", ",", "PHYS"}]], TraditionalForm]]],
  "should vanish."
}], "Text"],

Cell["Translate FORM output (jqq12.log) to FeynCalc", "Text"],

Cell[TextData[{
  "The FORM file ",
  "jqq12.log ",
  "was generated by one of the authors of ",
  ButtonBox["hep-ph/9801224",
    ButtonData:>{
      URL[ "http://xxx.lanl.gov/abs/hep-ph/9801224"], None},
    ButtonStyle->"Hyperlink"],
  ". "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"e", " ", "=", " ", "Epsilon"}]], "Input"],

Cell[BoxData[
    FormBox[
      TagBox["\[CurlyEpsilon]",
        TraditionalForm], TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
Special FORM convenctions like ln(1-x) can be translated \
automatically by FORM2FeynCalc, but they have to be listed once in the option \
Replace of FORM2FeynCalc :\
\>", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"FORM2FeynCalc", ",", 
          RowBox[{"Replace", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"\"\<ln(x)\>\"", "\[Rule]", "\"\<Log[x]\>\""}], ",", 
                RowBox[{"\"\<ln(1+x)\>\"", "\[Rule]", "\"\<Log[1+x]\>\""}], 
                ",", 
                RowBox[{"\"\<ln(1-x)\>\"", "\[Rule]", "\"\<Log[1-x]\>\""}], 
                ",", 
                RowBox[{"\"\<Li2(-x)\>\"", "\[Rule]", "\"\<Li2[-x]\>\""}], 
                ",", 
                RowBox[{"\"\<Li2(1-x)\>\"", "\[Rule]", "\"\<Li2[1-x]\>\""}], 
                ",", 
                RowBox[{"\"\<Li3(1-x)\>\"", "\[Rule]", "\"\<Li3[1-x]\>\""}], 
                ",", 
                RowBox[{"\"\<Li3(-x)\>\"", "\[Rule]", "\"\<Li3[-x]\>\""}], 
                ",", 
                RowBox[{
                "\"\<S12(1-x)\>\"", "\[Rule]", "\"\<Nielsen[1,2,1-x]\>\""}], 
                ",", 
                RowBox[{
                "\"\<S12(-x)\>\"", "\[Rule]", "\"\<Nielsen[1,2,-x]\>\""}], 
                ",", 
                RowBox[{
                "\"\<S12(x^2)\>\"", "\[Rule]", "\"\<Nielsen[1,2,x^2]\>\""}], 
                ",", 
                RowBox[{"\"\<Z2\>\"", "\[Rule]", "\"\<Zeta2\>\""}], ",", 
                RowBox[{"\"\<Z3\>\"", "\[Rule]", "\"\<Zeta[3]\>\""}], ",", 
                RowBox[{"\"\<[1+x]^-1\>\"", "\[Rule]", "\"\<(1+x)^-1\>\""}], 
                ",", 
                RowBox[{
                "\"\<[(1-x)+]^-1\>\"", "\[Rule]", "\"\<(1-x)^-1\>\""}], ",", 
                RowBox[{"\"\<[1-x]^-1\>\"", "\[Rule]", "\"\<(1-x)^-1\>\""}], 
                ",", 
                RowBox[{
                "\"\<delta\>\"", "\[Rule]", "\"\<DeltaFunction[1-x]\>\""}], 
                ",", 
                RowBox[{"\"\<Ca\>\"", "\[Rule]", "\"\<CA\>\""}], ",", 
                RowBox[{"\"\<Cf\>\"", "\[Rule]", "\"\<CF\>\""}]}], "}"}]}], 
          ",", 
          RowBox[{"Dot", "\[Rule]", "Times"}], ",", 
          RowBox[{"HoldForm", "\[Rule]", "False"}]}], "]"}], ";"}]], "Input"],

Cell["\<\
Copy the FORM output from jqq12.log into the next cell and put \
FORM2CFeynCalc[\" \"]//ReleaseHold around it.\
\>", "Text"],

Cell["\<\
aqq=FORM2FeynCalc[\"

          + [(-1)^m]*e^-1*O1*Ca*Cf * (
          - 8
          + 4*x*Z2
          + 8*x
          + 8*Z2*[1+x]^-1
          - 4*Z2
          - 4*ln(x)
          - 4*ln(x)*x
          - 8*ln(x)*ln(1 + x)
          + 8*ln(x)*ln(1 + x)*x
          + 16*ln(x)*ln(1 + x)*[1+x]^-1
          + 2*ln(x)*ln(x)
          - 2*ln(x)*ln(x)*x
          - 4*ln(x)*ln(x)*[1+x]^-1
          - 8*Li2( - x)
          + 8*Li2( - x)*x
          + 16*Li2( - x)*[1+x]^-1
          )

       + [(-1)^m]*e^-1*O1*Cf^2 * (
          + 16
          - 8*x*Z2
          - 16*x
          - 16*Z2*[1+x]^-1
          + 8*Z2
          + 8*ln(x)
          + 8*ln(x)*x
          + 16*ln(x)*ln(1 + x)
          - 16*ln(x)*ln(1 + x)*x
          - 32*ln(x)*ln(1 + x)*[1+x]^-1
          - 4*ln(x)*ln(x)
          + 4*ln(x)*ln(x)*x
          + 8*ln(x)*ln(x)*[1+x]^-1
          + 16*Li2( - x)
          - 16*Li2( - x)*x
          - 32*Li2( - x)*[1+x]^-1
          )

       + [(-1)^m]*O1*Ca*Cf * (
          - 41/3
          - 6*x*Z2
          - 2*x*Z3
          + 41/3*x
          + 16/3*x^2*Z2
          + 6*Z2
          + 4*Z3*[1+x]^-1
          - 6*Z3
          - 8*ln(1 - x)
          + 8*ln(1 - x)*x
          - 8*ln(1 - x)*Li2(1 - x)
          + 8*ln(1 - x)*Li2(1 - x)*x
          + 16*ln(1 - x)*Li2(1 - x)*[1+x]^-1
          + 8*ln(1 + x)*x*Z2
          + 8*ln(1 + x)*Z2*[1+x]^-1
          - 8*ln(1 + x)*Li2(1 - x)
          + 8*ln(1 + x)*Li2(1 - x)*x
          + 16*ln(1 + x)*Li2(1 - x)*[1+x]^-1
          + 16*ln(1 + x)*Li2( - x)*x
          + 16*ln(1 + x)*Li2( - x)*[1+x]^-1
          - 37/3*ln(x)
          - 25/3*ln(x)*x
          - 4*ln(x)*ln(1 - x)
          - 4*ln(x)*ln(1 - x)*x
          - 4*ln(x)*ln(1 - x)*ln(1 - x)
          + 4*ln(x)*ln(1 - x)*ln(1 - x)*x
          + 8*ln(x)*ln(1 - x)*ln(1 - x)*[1+x]^-1
          - 8*ln(x)*ln(1 - x)*ln(1 + x)
          + 8*ln(x)*ln(1 - x)*ln(1 + x)*x
          + 16*ln(x)*ln(1 - x)*ln(1 + x)*[1+x]^-1
          + 4/3*ln(x)*ln(1 + x)*x^-1
          + 4*ln(x)*ln(1 + x)*x
          + 16/3*ln(x)*ln(1 + x)*x^2
          + 8*ln(x)*ln(1 + x)*ln(1 + x)*x
          + 8*ln(x)*ln(1 + x)*ln(1 + x)*[1+x]^-1
          - 6*ln(x)*ln(x)
          - 8/3*ln(x)*ln(x)*x^2
          + 2*ln(x)*ln(x)*ln(1 - x)
          - 2*ln(x)*ln(x)*ln(1 - x)*x
          - 4*ln(x)*ln(x)*ln(1 - x)*[1+x]^-1
          - 6*ln(x)*ln(x)*ln(1 + x)
          + 2*ln(x)*ln(x)*ln(1 + x)*x
          + 8*ln(x)*ln(x)*ln(1 + x)*[1+x]^-1
          + ln(x)*ln(x)*ln(x)
          - ln(x)*ln(x)*ln(x)*x
          - 2*ln(x)*ln(x)*ln(x)*[1+x]^-1
          + 4*ln(x)*Li2(1 - x)
          - 4*ln(x)*Li2(1 - x)*x
          - 8*ln(x)*Li2(1 - x)*[1+x]^-1
          - 4*ln(x)*Li2( - x)
          + 4*ln(x)*Li2( - x)*x
          + 8*ln(x)*Li2( - x)*[1+x]^-1
          - 4*Li2(1 - x)
          - 4*Li2(1 - x)*x
          + 4/3*Li2( - x)*x^-1
          + 4*Li2( - x)*x
          + 16/3*Li2( - x)*x^2
          + 8*Li3(1 - x)
          - 8*Li3(1 - x)*x
          - 16*Li3(1 - x)*[1+x]^-1
          - 4*Li3( - x)
          - 4*Li3( - x)*x
          + 4*S12(1 - x)
          - 4*S12(1 - x)*x
          - 8*S12(1 - x)*[1+x]^-1
          - 8*S12( - x)
          + 24*S12( - x)*x
          + 32*S12( - x)*[1+x]^-1
          + 4*S12(x^2)
          - 4*S12(x^2)*x
          - 8*S12(x^2)*[1+x]^-1
          )

       + [(-1)^m]*O1*Cf^2 * (
          + 82/3
          + 12*x*Z2
          + 4*x*Z3
          - 82/3*x
          - 32/3*x^2*Z2
          - 12*Z2
          - 8*Z3*[1+x]^-1
          + 12*Z3
          + 16*ln(1 - x)
          - 16*ln(1 - x)*x
          + 16*ln(1 - x)*Li2(1 - x)
          - 16*ln(1 - x)*Li2(1 - x)*x
          - 32*ln(1 - x)*Li2(1 - x)*[1+x]^-1
          - 16*ln(1 + x)*x*Z2
          - 16*ln(1 + x)*Z2*[1+x]^-1
          + 16*ln(1 + x)*Li2(1 - x)
          - 16*ln(1 + x)*Li2(1 - x)*x
          - 32*ln(1 + x)*Li2(1 - x)*[1+x]^-1
          - 32*ln(1 + x)*Li2( - x)*x
          - 32*ln(1 + x)*Li2( - x)*[1+x]^-1
          + 74/3*ln(x)
          + 50/3*ln(x)*x
          + 8*ln(x)*ln(1 - x)
          + 8*ln(x)*ln(1 - x)*x
          + 8*ln(x)*ln(1 - x)*ln(1 - x)
          - 8*ln(x)*ln(1 - x)*ln(1 - x)*x
          - 16*ln(x)*ln(1 - x)*ln(1 - x)*[1+x]^-1
          + 16*ln(x)*ln(1 - x)*ln(1 + x)
          - 16*ln(x)*ln(1 - x)*ln(1 + x)*x
          - 32*ln(x)*ln(1 - x)*ln(1 + x)*[1+x]^-1
          - 8/3*ln(x)*ln(1 + x)*x^-1
          - 8*ln(x)*ln(1 + x)*x
          - 32/3*ln(x)*ln(1 + x)*x^2
          - 16*ln(x)*ln(1 + x)*ln(1 + x)*x
          - 16*ln(x)*ln(1 + x)*ln(1 + x)*[1+x]^-1
          + 12*ln(x)*ln(x)
          + 16/3*ln(x)*ln(x)*x^2
          - 4*ln(x)*ln(x)*ln(1 - x)
          + 4*ln(x)*ln(x)*ln(1 - x)*x
          + 8*ln(x)*ln(x)*ln(1 - x)*[1+x]^-1
          + 12*ln(x)*ln(x)*ln(1 + x)
          - 4*ln(x)*ln(x)*ln(1 + x)*x
          - 16*ln(x)*ln(x)*ln(1 + x)*[1+x]^-1
          - 2*ln(x)*ln(x)*ln(x)
          + 2*ln(x)*ln(x)*ln(x)*x
          + 4*ln(x)*ln(x)*ln(x)*[1+x]^-1
          - 8*ln(x)*Li2(1 - x)
          + 8*ln(x)*Li2(1 - x)*x
          + 16*ln(x)*Li2(1 - x)*[1+x]^-1
          + 8*ln(x)*Li2( - x)
          - 8*ln(x)*Li2( - x)*x
          - 16*ln(x)*Li2( - x)*[1+x]^-1
          + 8*Li2(1 - x)
          + 8*Li2(1 - x)*x
          - 8/3*Li2( - x)*x^-1
          - 8*Li2( - x)*x
          - 32/3*Li2( - x)*x^2
          - 16*Li3(1 - x)
          + 16*Li3(1 - x)*x
          + 32*Li3(1 - x)*[1+x]^-1
          + 8*Li3( - x)
          + 8*Li3( - x)*x
          - 8*S12(1 - x)
          + 8*S12(1 - x)*x
          + 16*S12(1 - x)*[1+x]^-1
          + 16*S12( - x)
          - 48*S12( - x)*x
          - 64*S12( - x)*[1+x]^-1
          - 8*S12(x^2)
          + 8*S12(x^2)*x
          + 16*S12(x^2)*[1+x]^-1
          )

       + [(-1)^m]*O2*Ca*Cf * (
          + 8/3
          - 8/3*x
          - 16/3*x^2*Z2
          - 8/3*ln(x)
          + 16/3*ln(x)*x
          + 8/3*ln(x)*ln(1 + x)*x^-1
          - 8*ln(x)*ln(1 + x)*x
          - 16/3*ln(x)*ln(1 + x)*x^2
          + 8/3*ln(x)*ln(x)*x^2
          + 8/3*Li2( - x)*x^-1
          - 8*Li2( - x)*x
          - 16/3*Li2( - x)*x^2
          )

       + [(-1)^m]*O2*Cf^2 * (
          - 16/3
          + 16/3*x
          + 32/3*x^2*Z2
          + 16/3*ln(x)
          - 32/3*ln(x)*x
          - 16/3*ln(x)*ln(1 + x)*x^-1
          + 16*ln(x)*ln(1 + x)*x
          + 32/3*ln(x)*ln(1 + x)*x^2
          - 16/3*ln(x)*ln(x)*x^2
          - 16/3*Li2( - x)*x^-1
          + 16*Li2( - x)*x
          + 32/3*Li2( - x)*x^2
          )

       + e^-2*delta*O1*Ca*Cf * (
          - 22
          )

       + e^-2*delta*O1*Cf*Tf * (
          + 8
          )

       + e^-2*delta*O1*Cf^2 * (
          - 18
          + 32*Z2
          )

       + e^-2*O1*Ca*Cf * (
          + 44/3
          + 44/3*x
          - 88/3*[(1-x)+]^-1
          )

       + e^-2*O1*Cf*Tf * (
          - 16/3
          - 16/3*x
          + 32/3*[(1-x)+]^-1
          )

       + e^-2*O1*Cf^2 * (
          + 40
          + 8*x
          - 48*[(1-x)+]^-1
          + 32*ln(1 - x)
          + 32*ln(1 - x)*x
          - 64*ln(1 - x)*[(1-x)+]^-1
          - 24*ln(x)
          - 24*ln(x)*x
          + 32*ln(x)*[1-x]^-1
          )

       + e^-1*delta*O1*Ca*Cf * (
          + 325/6
          - 44/3*Z2
          - 12*Z3
          )

       + e^-1*delta*O1*Cf*Tf * (
          - 58/3
          + 16/3*Z2
          )

       + e^-1*delta*O1*Cf^2 * (
          + 87/2
          - 36*Z2
          - 8*Z3
          )

       + e^-1*O1*Ca*Cf * (
          - 158/9
          + 4*x*Z2
          + 22/9*x
          - 8*Z2*[(1-x)+]^-1
          + 4*Z2
          + 238/9*[(1-x)+]^-1
          + 44/3*ln(1 - x)
          + 44/3*ln(1 - x)*x
          - 88/3*ln(1 - x)*[(1-x)+]^-1
          + 34/3*ln(x)
          + 34/3*ln(x)*x
          - 44/3*ln(x)*[1-x]^-1
          - 2*ln(x)*ln(x)
          - 2*ln(x)*ln(x)*x
          + 4*ln(x)*ln(x)*[1-x]^-1
          )

       + e^-1*O1*Cf*Tf * (
          + 88/9
          - 56/9*x
          - 56/9*[(1-x)+]^-1
          - 16/3*ln(1 - x)
          - 16/3*ln(1 - x)*x
          + 32/3*ln(1 - x)*[(1-x)+]^-1
          - 8/3*ln(x)
          - 8/3*ln(x)*x
          + 16/3*ln(x)*[1-x]^-1
          )

       + e^-1*O1*Cf^2 * (
          - 40
          - 4*x
          + 56*[(1-x)+]^-1
          - 4*ln(1 - x)
          + 44*ln(1 - x)*x
          - 24*ln(1 - x)*[(1-x)+]^-1
          + 24*ln(1 - x)*ln(1 - x)
          + 24*ln(1 - x)*ln(1 - x)*x
          - 48*ln(1 - x)*ln(1 - x)*[(1-x)+]^-1
          + 36*ln(x)
          - 20*ln(x)*x
          - 36*ln(x)*[1-x]^-1
          - 16*ln(x)*ln(1 - x)*[1-x]^-1
          - 14*ln(x)*ln(x)
          - 14*ln(x)*ln(x)*x
          + 16*ln(x)*ln(x)*[1-x]^-1
          - 8*Li2(1 - x)
          - 8*Li2(1 - x)*x
          )

       + e^-1*O2*Ca*Cf * (
          + 20/3
          - 88/3*x
          )

       + e^-1*O2*Cf*Tf * (
          - 16/3
          + 32/3*x
          )

       + e^-1*O2*Cf^2 * (
          - 16
          - 8*x
          - 32*ln(1 - x)*x
          + 16*ln(x)*x
          )

       + delta*O1*Ca*Cf * (
          - 7081/72
          + 301/18*Z2
          + 49/5*Z2^2
          + 28*Z3
          )

       + delta*O1*Cf*Tf * (
          + 569/18
          - 46/9*Z2
          - 8*Z3
          )

       + delta*O1*Cf^2 * (
          - 541/8
          + 97/2*Z2
          - 74/5*Z2^2
          + 54*Z3
          )

       + O1*Ca*Cf * (
          + 941/27
          + 17/3*x*Z2
          - 8*x*Z3
          - 580/27*x
          - 16/3*x^2*Z2
          - 24*x^2*Z3
          - 28/3*Z2*[(1-x)+]^-1
          - 1/3*Z2
          + 12*Z3*[1-x]^-1
          + 22*Z3*[(1-x)+]^-1
          - 20*Z3
          - 670/27*[(1-x)+]^-1
          - 122/9*ln(1 - x)
          + 6*ln(1 - x)*x*Z2
          - 32/9*ln(1 - x)*x
          + 8*ln(1 - x)*x^2*Z2
          - 10*ln(1 - x)*Z2*[(1-x)+]^-1
          + 2*ln(1 - x)*Z2
          + 238/9*ln(1 - x)*[(1-x)+]^-1
          + 22/3*ln(1 - x)*ln(1 - x)
          + 22/3*ln(1 - x)*ln(1 - x)*x
          - 44/3*ln(1 - x)*ln(1 - x)*[(1-x)+]^-1
          + 2*ln(1 - x)*Li2(1 - x)
          - 2*ln(1 - x)*Li2(1 - x)*x
          + 8*ln(1 - x)*Li2(1 - x)*x^2
          - 2*ln(1 - x)*Li2(1 - x)*[1-x]^-1
          + 14/9*ln(x)
          - 2*ln(x)*x*Z2
          + 176/9*ln(x)*x
          - 8*ln(x)*x^2*Z2
          + 10*ln(x)*Z2*[1-x]^-1
          - 6*ln(x)*Z2
          + 101/9*ln(x)*[1-x]^-1
          + 28/3*ln(x)*ln(1 - x)
          + 10/3*ln(x)*ln(1 - x)*x
          - 38/3*ln(x)*ln(1 - x)*[1-x]^-1
          - 12*ln(x)*ln(1 + x)
          - 4/3*ln(x)*ln(1 + x)*x^-1
          - 16*ln(x)*ln(1 + x)*x
          - 16/3*ln(x)*ln(1 + x)*x^2
          + 47/6*ln(x)*ln(x)
          + 47/6*ln(x)*ln(x)*x
          + 8/3*ln(x)*ln(x)*x^2
          - 11/3*ln(x)*ln(x)*[1-x]^-1
          + ln(x)*ln(x)*ln(1 - x)
          - ln(x)*ln(x)*ln(1 - x)*x
          + 4*ln(x)*ln(x)*ln(1 - x)*x^2
          - ln(x)*ln(x)*ln(1 - x)*[1-x]^-1
          - ln(x)*ln(x)*ln(x)
          - ln(x)*ln(x)*ln(x)*x
          + 2*ln(x)*ln(x)*ln(x)*[1-x]^-1
          + 8*ln(x)*Li2(1 - x)
          + 16*ln(x)*Li2(1 - x)*x^2
          - 12*ln(x)*Li2(1 - x)*[1-x]^-1
          + 8*ln(x)*Li2( - x)
          - 8*ln(x)*Li2( - x)*[1-x]^-1
          - 12*Li2(1 - x)*x
          + 4*Li2(1 - x)*[1-x]^-1
          - 12*Li2( - x)
          - 4/3*Li2( - x)*x^-1
          - 16*Li2( - x)*x
          - 16/3*Li2( - x)*x^2
          - 6*Li3(1 - x)
          + 6*Li3(1 - x)*x
          - 24*Li3(1 - x)*x^2
          + 6*Li3(1 - x)*[1-x]^-1
          - 16*Li3( - x)
          + 16*Li3( - x)*[1-x]^-1
          + 14*S12(1 - x)
          - 6*S12(1 - x)*x
          + 24*S12(1 - x)*x^2
          - 14*S12(1 - x)*[1-x]^-1
          )

       + O1*Cf*Tf * (
          - 88/27
          - 4/3*x*Z2
          + 20/27*x
          + 8/3*Z2*[(1-x)+]^-1
          - 4/3*Z2
          + 128/27*[(1-x)+]^-1
          + 88/9*ln(1 - x)
          - 56/9*ln(1 - x)*x
          - 56/9*ln(1 - x)*[(1-x)+]^-1
          - 8/3*ln(1 - x)*ln(1 - x)
          - 8/3*ln(1 - x)*ln(1 - x)*x
          + 16/3*ln(1 - x)*ln(1 - x)*[(1-x)+]^-1
          + 44/9*ln(x)
          - 28/9*ln(x)*x
          - 28/9*ln(x)*[1-x]^-1
          - 8/3*ln(x)*ln(1 - x)
          - 8/3*ln(x)*ln(1 - x)*x
          + 16/3*ln(x)*ln(1 - x)*[1-x]^-1
          - 2/3*ln(x)*ln(x)
          - 2/3*ln(x)*ln(x)*x
          + 4/3*ln(x)*ln(x)*[1-x]^-1
          )

       + O1*Cf^2 * (
          + 188/3
          - 4*x*Z2
          - 24*x*Z3
          + 4/3*x
          + 32/3*x^2*Z2
          + 48*x^2*Z3
          - 4*Z2*[(1-x)+]^-1
          + 20*Z2
          - 24*Z3*[1-x]^-1
          + 24*Z3
          - 56*[(1-x)+]^-1
          - 38*ln(1 - x)
          + 8*ln(1 - x)*x*Z2
          + 38*ln(1 - x)*x
          - 16*ln(1 - x)*x^2*Z2
          - 8*ln(1 - x)*Z2*[(1-x)+]^-1
          + 8*ln(1 - x)*Z2
          + 28*ln(1 - x)*[(1-x)+]^-1
          - 13*ln(1 - x)*ln(1 - x)
          + 31*ln(1 - x)*ln(1 - x)*x
          - 6*ln(1 - x)*ln(1 - x)*[(1-x)+]^-1
          + 28/3*ln(1 - x)*ln(1 - x)*ln(1 - x)
          + 28/3*ln(1 - x)*ln(1 - x)*ln(1 - x)*x
          - 56/3*ln(1 - x)*ln(1 - x)*ln(1 - x)*[(1-x)+]^-1
          - 12*ln(1 - x)*Li2(1 - x)
          + 4*ln(1 - x)*Li2(1 - x)*x
          - 16*ln(1 - x)*Li2(1 - x)*x^2
          - 68/3*ln(x)
          - 14*ln(x)*x*Z2
          - 68/3*ln(x)*x
          + 16*ln(x)*x^2*Z2
          + 2*ln(x)*Z2
          + 44*ln(x)*[1-x]^-1
          - 6*ln(x)*ln(1 - x)
          + 34*ln(x)*ln(1 - x)*x
          - 20*ln(x)*ln(1 - x)*[1-x]^-1
          + 6*ln(x)*ln(1 - x)*ln(1 - x)
          + 6*ln(x)*ln(1 - x)*ln(1 - x)*x
          - 20*ln(x)*ln(1 - x)*ln(1 - x)*[1-x]^-1
          + 24*ln(x)*ln(1 + x)
          + 8/3*ln(x)*ln(1 + x)*x^-1
          + 32*ln(x)*ln(1 + x)*x
          + 32/3*ln(x)*ln(1 + x)*x^2
          + 11*ln(x)*ln(x)
          - 23*ln(x)*ln(x)*x
          - 16/3*ln(x)*ln(x)*x^2
          - 15*ln(x)*ln(x)*[1-x]^-1
          - 6*ln(x)*ln(x)*ln(1 - x)
          + 2*ln(x)*ln(x)*ln(1 - x)*x
          - 8*ln(x)*ln(x)*ln(1 - x)*x^2
          - 4*ln(x)*ln(x)*ln(1 - x)*[1-x]^-1
          - 5*ln(x)*ln(x)*ln(x)
          - 5*ln(x)*ln(x)*ln(x)*x
          + 16/3*ln(x)*ln(x)*ln(x)*[1-x]^-1
          - 8*ln(x)*Li2(1 - x)
          + 24*ln(x)*Li2(1 - x)*x
          - 32*ln(x)*Li2(1 - x)*x^2
          - 16*ln(x)*Li2(1 - x)*[1-x]^-1
          - 16*ln(x)*Li2( - x)
          + 16*ln(x)*Li2( - x)*[1-x]^-1
          - 24*Li2(1 - x)
          + 48*Li2(1 - x)*x
          - 4*Li2(1 - x)*[1-x]^-1
          + 24*Li2( - x)
          + 8/3*Li2( - x)*x^-1
          + 32*Li2( - x)*x
          + 32/3*Li2( - x)*x^2
          + 20*Li3(1 - x)
          - 28*Li3(1 - x)*x
          + 48*Li3(1 - x)*x^2
          + 32*Li3( - x)
          - 32*Li3( - x)*[1-x]^-1
          - 8*S12(1 - x)
          + 56*S12(1 - x)*x
          - 48*S12(1 - x)*x^2
          - 32*S12(1 - x)*[1-x]^-1
          )

       + O2*Ca*Cf * (
          - 26/9
          - 8*x*Z2
          - 24*x*Z3
          + 232/9*x
          + 16/3*x^2*Z2
          + 24*x^2*Z3
          + 4*Z2
          - 16/3*ln(1 - x)
          + 8*ln(1 - x)*x*Z2
          - 64/3*ln(1 - x)*x
          - 8*ln(1 - x)*x^2*Z2
          + 8*ln(1 - x)*Li2(1 - x)*x
          - 8*ln(1 - x)*Li2(1 - x)*x^2
          + 6*ln(x)
          - 8*ln(x)*x*Z2
          - 28*ln(x)*x
          + 8*ln(x)*x^2*Z2
          - 4*ln(x)*ln(1 - x)
          + 8*ln(x)*ln(1 - x)*x
          - 8/3*ln(x)*ln(1 + x)*x^-1
          + 8*ln(x)*ln(1 + x)*x
          + 16/3*ln(x)*ln(1 + x)*x^2
          - 8/3*ln(x)*ln(x)*x^2
          + 4*ln(x)*ln(x)*ln(1 - x)*x
          - 4*ln(x)*ln(x)*ln(1 - x)*x^2
          + 16*ln(x)*Li2(1 - x)*x
          - 16*ln(x)*Li2(1 - x)*x^2
          - 8*Li2(1 - x)
          + 16*Li2(1 - x)*x
          - 8/3*Li2( - x)*x^-1
          + 8*Li2( - x)*x
          + 16/3*Li2( - x)*x^2
          - 24*Li3(1 - x)*x
          + 24*Li3(1 - x)*x^2
          + 24*S12(1 - x)*x
          - 24*S12(1 - x)*x^2
          )

       + O2*Cf*Tf * (
          - 8/9
          - 32/9*x
          - 16/3*ln(1 - x)
          + 32/3*ln(1 - x)*x
          - 8/3*ln(x)
          + 16/3*ln(x)*x
          )

       + O2*Cf^2 * (
          - 32/3
          + 16*x*Z2
          + 48*x*Z3
          - 16/3*x
          - 32/3*x^2*Z2
          - 48*x^2*Z3
          - 8*Z2
          - 16*ln(1 - x)*x*Z2
          - 44*ln(1 - x)*x
          + 16*ln(1 - x)*x^2*Z2
          - 24*ln(1 - x)*ln(1 - x)*x
          - 16*ln(1 - x)*Li2(1 - x)*x
          + 16*ln(1 - x)*Li2(1 - x)*x^2
          - 40/3*ln(x)
          + 16*ln(x)*x*Z2
          + 68/3*ln(x)*x
          - 16*ln(x)*x^2*Z2
          + 8*ln(x)*ln(1 - x)
          - 32*ln(x)*ln(1 - x)*x
          + 16/3*ln(x)*ln(1 + x)*x^-1
          - 16*ln(x)*ln(1 + x)*x
          - 32/3*ln(x)*ln(1 + x)*x^2
          + 12*ln(x)*ln(x)*x
          + 16/3*ln(x)*ln(x)*x^2
          - 8*ln(x)*ln(x)*ln(1 - x)*x
          + 8*ln(x)*ln(x)*ln(1 - x)*x^2
          - 32*ln(x)*Li2(1 - x)*x
          + 32*ln(x)*Li2(1 - x)*x^2
          + 16*Li2(1 - x)
          - 48*Li2(1 - x)*x
          + 16/3*Li2( - x)*x^-1
          - 16*Li2( - x)*x
          - 32/3*Li2( - x)*x^2
          + 48*Li3(1 - x)*x
          - 48*Li3(1 - x)*x^2
          - 48*S12(1 - x)*x
          + 48*S12(1 - x)*x^2
          )
          
          \"]//ReleaseHold;\
\>", "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Short", "[", "aqq", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      TagBox[
        RowBox[{
          FractionBox[
            RowBox[{"O1", " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"32", " ", 
                    RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", "18"}], ")"}], 
              " ", 
              RowBox[{"\[Delta]", "(", 
                FormBox[
                  RowBox[{"1", "-", "x"}],
                  "TraditionalForm"], ")"}], " ", 
              SubsuperscriptBox["C", "F", "2"]}], 
            SuperscriptBox[
              TagBox["\[CurlyEpsilon]",
                TraditionalForm], "2"]], "+", 
          RowBox[{"\[LeftSkeleton]", "29", "\[RightSkeleton]"}], "+", 
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            SubscriptBox["C", "A"], " ", "O1", " ", 
            RowBox[{"(", 
              RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], ")"}], 
            " ", 
            SubscriptBox["C", "F"]}]}],
        Short], TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
Comment: By default all bracketed FORM expressions are translated \
to a Hold[] expression.\
\>", "Commentary"],

Cell[CellGroupData[{

Cell["FORM2FeynCalc[\"-[(-1)^m]\"]", "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        RowBox[{"Hold", "(", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "ReleaseHold"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "m"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["Calculate moments", "Text"],

Cell["\<\
The first moment should be zero, which is verified by Integrate2 in \
a reasonable amount of time. The variable O2 can be set to zero since it is \
the coefficient of unphysical matrix elements.\
\>", "Text"],

Cell[TextData[{
  "This calculates for m=1 the integral ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            SubsuperscriptBox["\[Integral]", "0", 
              RowBox[{"1", " "}]], " ", 
            RowBox[{
              SubsuperscriptBox[
                StyleBox[
                  OverscriptBox[
                    StyleBox["A",
                      FontSlant->"Plain"], "^"],
                  FontSlant->"Plain"], "qq", 
                RowBox[{"NS", ",", "PHYS"}]], " ", 
              RowBox[{"\[DifferentialD]", "x"}]}]}], " "}], 
        TraditionalForm]]]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Timing", "[", 
      RowBox[{"Integrate2", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"aqq", "/.", 
              RowBox[{"m", "\[Rule]", "1"}]}], "/.", 
            RowBox[{"O2", "\[Rule]", "0"}]}], ",", 
          RowBox[{"{", 
            RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"16.8900000000000005`", " ", "Second"}], ",", "0"}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
If you know that no partial fraction decomposition has to be done \
then Integrate3 will be much faster on an expanded or collected \
integral:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Timing", "[", 
      RowBox[{"Expand", "[", 
        RowBox[{"Integrate3", "[", 
          RowBox[{
            RowBox[{"Expand", "[", 
              RowBox[{
                RowBox[{"aqq", "/.", 
                  RowBox[{"m", "\[Rule]", "1"}]}], "/.", 
                RowBox[{"O2", "\[Rule]", "0"}]}], "]"}], ",", 
            RowBox[{"{", 
              RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "]"}], 
      "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"4.39000000000000056`", " ", "Second"}], ",", "0"}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell["Define an auxiliary typesetting function.", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"MakeBoxes", "[", 
          RowBox[{
            RowBox[{"big", "[", "a_", "]"}], ",", " ", "f_"}], "]"}], " ", 
        ":=", " ", 
        RowBox[{"StyleBox", "[", 
          RowBox[{
            RowBox[{"MakeBoxes", "[", 
              RowBox[{"a", ",", " ", "f"}], "]"}], ",", " ", 
            RowBox[{"FontColor", " ", "->", " ", 
              RowBox[{"RGBColor", "[", 
                RowBox[{"0", ",", " ", "0", ",", " ", "1"}], "]"}]}], ",", 
            " ", " ", " ", 
            RowBox[{"FontSize", " ", "->", " ", "18"}]}], "]"}]}], ";"}]], 
  "Input"],

Cell["The second moment takes a little bit longer.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Timing", "[", 
        RowBox[{"Collect2", "[", 
          RowBox[{
            RowBox[{"Integrate2", "[", 
              RowBox[{
                RowBox[{
                  RowBox[{
                    RowBox[{
                      RowBox[{"x", " ", "aqq"}], "/.", 
                      RowBox[{"m", "\[Rule]", "2"}]}], "/.", 
                    RowBox[{"O2", "\[Rule]", "0"}]}], "/.", 
                  RowBox[{"O1", "\[Rule]", "1"}]}], ",", 
                RowBox[{"{", 
                  RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], ",", 
            RowBox[{"{", 
              RowBox[{"CA", ",", "CF", ",", "Tf"}], "}"}], ",", 
            RowBox[{"Factoring", "\[Rule]", "False"}]}], "]"}], "]"}], "/.", 
      " ", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            RowBox[{"CA", " ", "CF"}], " ", "\[RuleDelayed]", " ", 
            RowBox[{"big", "[", " ", 
              RowBox[{"CA", " ", "CF"}], "]"}]}], ",", 
          RowBox[{
            RowBox[{"CF", "^", "2"}], "\[RuleDelayed]", 
            RowBox[{"big", "[", 
              RowBox[{"CF", "^", "2"}], "]"}]}], ",", 
          RowBox[{
            RowBox[{"a_", " ", "CF", " ", "Tf"}], " ", "\[RuleDelayed]", 
            RowBox[{"HoldForm", "[", " ", 
              RowBox[{
                RowBox[{"big", "[", 
                  RowBox[{"CF", " ", "Tf"}], "]"}], " ", "a"}], "]"}]}]}], 
        "}"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"20.3200000000000002`", " ", "Second"}], ",", 
          RowBox[{
            TagBox[
              RowBox[{
                RowBox[{"(", 
                  StyleBox[
                    RowBox[{
                      SubscriptBox["C", "F"], " ", 
                      SubscriptBox["T", "f"]}],
                    FontSize->18,
                    FontColor->RGBColor[0, 0, 1]], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    FractionBox[
                      RowBox[{"16", " ", 
                        RowBox[{"\[Zeta]", "(", "2", ")"}]}], "9"], "+", 
                    FractionBox["376", 
                      RowBox[{"27", " ", 
                        TagBox["\[CurlyEpsilon]",
                          TraditionalForm]}]], "-", 
                    FractionBox["64", 
                      RowBox[{"9", " ", 
                        SuperscriptBox[
                          TagBox["\[CurlyEpsilon]",
                            TraditionalForm], "2"]}]], "-", 
                    FractionBox["2086", "81"]}], ")"}]}],
              HoldForm], "+", 
            RowBox[{
              RowBox[{"(", 
                StyleBox[
                  RowBox[{
                    SubscriptBox["C", "A"], " ", 
                    SubscriptBox["C", "F"]}],
                  FontSize->18,
                  FontColor->RGBColor[0, 0, 1]], ")"}], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"-", 
                    FractionBox[
                      RowBox[{"44", " ", 
                        RowBox[{"\[Zeta]", "(", "2", ")"}]}], "9"]}], "-", 
                  FractionBox["1166", 
                    RowBox[{"27", " ", 
                      TagBox["\[CurlyEpsilon]",
                        TraditionalForm]}]], "+", 
                  FractionBox["176", 
                    RowBox[{"9", " ", 
                      SuperscriptBox[
                        TagBox["\[CurlyEpsilon]",
                          TraditionalForm], "2"]}]], "-", 
                  RowBox[{"12", " ", 
                    TagBox[
                      RowBox[{"\[Zeta]", "(", 
                        TagBox["3",
                          (Editable -> True)], ")"}],
                      InterpretTemplate[ Zeta[ #]&]]}], "+", 
                  FractionBox["12133", "162"]}], ")"}]}], "+", 
            RowBox[{
              StyleBox[
                SubsuperscriptBox["C", "F", "2"],
                FontSize->18,
                FontColor->RGBColor[0, 0, 1]], " ", 
              RowBox[{"(", 
                RowBox[{
                  FractionBox[
                    RowBox[{"32", " ", 
                      RowBox[{"\[Zeta]", "(", "2", ")"}]}], "9"], "+", 
                  FractionBox["752", 
                    RowBox[{"27", " ", 
                      TagBox["\[CurlyEpsilon]",
                        TraditionalForm]}]], "-", 
                  FractionBox["128", 
                    RowBox[{"9", " ", 
                      SuperscriptBox[
                        TagBox["\[CurlyEpsilon]",
                          TraditionalForm], "2"]}]], "+", 
                  RowBox[{"8", " ", 
                    TagBox[
                      RowBox[{"\[Zeta]", "(", 
                        TagBox["3",
                          (Editable -> True)], ")"}],
                      InterpretTemplate[ Zeta[ #]&]]}], "-", 
                  FractionBox["3452", "81"]}], ")"}]}]}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
Try to do the general Mellin transform (replace m by OPEm, which \
has special properties).\
\>", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"gqq", " ", "=", " ", 
        RowBox[{"Collect2", "[", 
          RowBox[{
            RowBox[{
              RowBox[{
                RowBox[{
                  SuperscriptBox["x", 
                    RowBox[{"OPEm", "-", "1"}]], 
                  RowBox[{"(", 
                    RowBox[{"aqq", "/.", 
                      RowBox[{"m", "\[Rule]", "OPEm"}]}], ")"}]}], "/.", 
                RowBox[{"O2", "\[Rule]", "0"}]}], "/.", 
              RowBox[{"O1", "\[Rule]", "1"}]}], ",", "x"}], "]"}]}], ";"}]], 
  "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"gqq", "//", "Length"}]], "Input"],

Cell[BoxData[
    FormBox["102", TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
There are 102 integrals. However, some of the Mellin transforms \
cannot be done at the moment.\
\>", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"ww", "=", 
        RowBox[{"Integrate3", "[", 
          RowBox[{"gqq", ",", 
            RowBox[{"{", 
              RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]}], ";"}]], 
  "Input"],

Cell["\<\
If an integral cannot be performed the head Hold[Integrate3] is \
substituted for Integrate3. Thus, in order to find the list of integrals not \
done you can do:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Cases2", "[", 
      RowBox[{"ww", ",", 
        RowBox[{"Hold", "[", "Integrate3", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{"Hold", "(", "Integrate3", ")"}], ")"}], "(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["x", 
                  RowBox[{"m", "-", "1"}]], " ", 
                RowBox[{
                  SubscriptBox["S", 
                    RowBox[{
                      FormBox["1",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["2",
                        "TraditionalForm"]}]], "(", 
                  FormBox[
                    SuperscriptBox["x", "2"],
                    "TraditionalForm"], ")"}]}], ",", 
              RowBox[{"{", 
                RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], ")"}], ",", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"Hold", "(", "Integrate3", ")"}], ")"}], "(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["x", "m"], " ", 
                RowBox[{
                  SubscriptBox["S", 
                    RowBox[{
                      FormBox["1",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["2",
                        "TraditionalForm"]}]], "(", 
                  FormBox[
                    SuperscriptBox["x", "2"],
                    "TraditionalForm"], ")"}]}], ",", 
              RowBox[{"{", 
                RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], ")"}], ",", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"Hold", "(", "Integrate3", ")"}], ")"}], "(", 
            RowBox[{
              FractionBox[
                RowBox[{
                  SuperscriptBox["x", 
                    RowBox[{"m", "-", "1"}]], " ", 
                  RowBox[{
                    SubscriptBox["S", 
                      RowBox[{
                        FormBox["1",
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox["2",
                          "TraditionalForm"]}]], "(", 
                    FormBox[
                      SuperscriptBox["x", "2"],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"x", "+", "1"}]], ",", 
              RowBox[{"{", 
                RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], ")"}], ",", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"Hold", "(", "Integrate3", ")"}], ")"}], "(", 
            RowBox[{
              FractionBox[
                RowBox[{
                  SuperscriptBox["x", 
                    RowBox[{"m", "-", "1"}]], " ", 
                  RowBox[{"log", "(", 
                    RowBox[{"1", "-", "x"}], ")"}], " ", 
                  RowBox[{
                    SubscriptBox["Li", "2"], "(", 
                    RowBox[{"1", "-", "x"}], ")"}]}], 
                RowBox[{"x", "+", "1"}]], ",", 
              RowBox[{"{", 
                RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], ")"}], ",", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"Hold", "(", "Integrate3", ")"}], ")"}], "(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["x", 
                  RowBox[{"m", "-", "1"}]], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"x", "+", "1"}], ")"}], " ", 
                RowBox[{
                  SubscriptBox["Li", "2"], "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], ",", 
              RowBox[{"{", 
                RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], ")"}], ",", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"Hold", "(", "Integrate3", ")"}], ")"}], "(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["x", "m"], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"x", "+", "1"}], ")"}], " ", 
                RowBox[{
                  SubscriptBox["Li", "2"], "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], ",", 
              RowBox[{"{", 
                RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], ")"}], ",", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"Hold", "(", "Integrate3", ")"}], ")"}], "(", 
            RowBox[{
              FractionBox[
                RowBox[{
                  SuperscriptBox["x", 
                    RowBox[{"m", "-", "1"}]], " ", 
                  RowBox[{"log", "(", 
                    RowBox[{"x", "+", "1"}], ")"}], " ", 
                  RowBox[{
                    SubscriptBox["Li", "2"], "(", 
                    RowBox[{"1", "-", "x"}], ")"}]}], 
                RowBox[{"x", "+", "1"}]], ",", 
              RowBox[{"{", 
                RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], ")"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "Change the appearance and retain only those integrals with ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox["x", 
            RowBox[{"m", "-", "1"}]], ":"}], TraditionalForm]]]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Select2", "[", 
        RowBox[{"%", ",", 
          SuperscriptBox["x", 
            RowBox[{"OPEm", "-", "1"}]]}], "]"}], "/.", " ", 
      RowBox[{
        RowBox[{
          RowBox[{"Hold", "[", "Integrate3", "]"}], "[", "z__", "]"}], 
        "\[RuleDelayed]", " ", 
        RowBox[{"HoldForm", "[", 
          RowBox[{"Integrate", "[", "z", "]"}], "]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          TagBox[
            RowBox[{
              SubsuperscriptBox["\[Integral]", "0", "1"], 
              RowBox[{
                RowBox[{
                  SuperscriptBox["x", 
                    RowBox[{"m", "-", "1"}]], " ", 
                  RowBox[{
                    SubscriptBox["S", 
                      RowBox[{
                        FormBox["1",
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox["2",
                          "TraditionalForm"]}]], "(", 
                    FormBox[
                      SuperscriptBox["x", "2"],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"\[DifferentialD]", "x"}]}]}],
            HoldForm], ",", 
          TagBox[
            RowBox[{
              SubsuperscriptBox["\[Integral]", "0", "1"], 
              RowBox[{
                FractionBox[
                  RowBox[{
                    SuperscriptBox["x", 
                      RowBox[{"m", "-", "1"}]], " ", 
                    RowBox[{
                      SubscriptBox["S", 
                        RowBox[{
                          FormBox["1",
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox["2",
                            "TraditionalForm"]}]], "(", 
                      FormBox[
                        SuperscriptBox["x", "2"],
                        "TraditionalForm"], ")"}]}], 
                  RowBox[{"x", "+", "1"}]], 
                RowBox[{"\[DifferentialD]", "x"}]}]}],
            HoldForm], ",", 
          TagBox[
            RowBox[{
              SubsuperscriptBox["\[Integral]", "0", "1"], 
              RowBox[{
                FractionBox[
                  RowBox[{
                    SuperscriptBox["x", 
                      RowBox[{"m", "-", "1"}]], " ", 
                    RowBox[{"log", "(", 
                      RowBox[{"1", "-", "x"}], ")"}], " ", 
                    RowBox[{
                      SubscriptBox["Li", "2"], "(", 
                      RowBox[{"1", "-", "x"}], ")"}]}], 
                  RowBox[{"x", "+", "1"}]], 
                RowBox[{"\[DifferentialD]", "x"}]}]}],
            HoldForm], ",", 
          TagBox[
            RowBox[{
              SubsuperscriptBox["\[Integral]", "0", "1"], 
              RowBox[{
                RowBox[{
                  SuperscriptBox["x", 
                    RowBox[{"m", "-", "1"}]], " ", 
                  RowBox[{"log", "(", 
                    RowBox[{"x", "+", "1"}], ")"}], " ", 
                  RowBox[{
                    SubscriptBox["Li", "2"], "(", 
                    RowBox[{"1", "-", "x"}], ")"}]}], 
                RowBox[{"\[DifferentialD]", "x"}]}]}],
            HoldForm], ",", 
          TagBox[
            RowBox[{
              SubsuperscriptBox["\[Integral]", "0", "1"], 
              RowBox[{
                FractionBox[
                  RowBox[{
                    SuperscriptBox["x", 
                      RowBox[{"m", "-", "1"}]], " ", 
                    RowBox[{"log", "(", 
                      RowBox[{"x", "+", "1"}], ")"}], " ", 
                    RowBox[{
                      SubscriptBox["Li", "2"], "(", 
                      RowBox[{"1", "-", "x"}], ")"}]}], 
                  RowBox[{"x", "+", "1"}]], 
                RowBox[{"\[DifferentialD]", "x"}]}]}],
            HoldForm]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
If you do know the answers for these integrals please share the \
knowledge.\
\>", "Text"],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"e", ",", "ww", ",", "gqq", ",", "big", ",", "aqq"}], "]"}]], 
  "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"FORM2FeynCalc", ",", 
          RowBox[{"Replace", "\[Rule]", 
            RowBox[{"{", "}"}]}]}], "]"}], ";"}]], "Input"],

Cell[TextData[{
  "See also: ",
  ButtonBox["CA",
    ButtonData:>"CA",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["CF",
    ButtonData:>"CF",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["Cases2",
    ButtonData:>"Cases2",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["Collect2",
    ButtonData:>"Collect2",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["DeltaFunction",
    ButtonData:>"DeltaFunction",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["Epsilon",
    ButtonData:>"Epsilon",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["FORM2FeynCalc",
    ButtonData:>"FORM2FeynCalc",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FORM2FeynCalc"],
  ", ",
  ButtonBox["Integrate2",
    ButtonData:>"Integrate3",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Integrate3"],
  ", ",
  ButtonBox["Integrate3",
    ButtonData:>"Integrate3",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Integrate3"],
  ", ",
  ButtonBox["Li2",
    ButtonData:>"Li2",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["Li3",
    ButtonData:>"Li3",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["Nielsen",
    ButtonData:>"Nielsen",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["Zeta2",
    ButtonData:>"Zeta2",
    ButtonStyle->"Hyperlink"],
  "."
}], "Text"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Reference Guide", "Section",
  CellTags->"Reference Guide"],

Cell[CellGroupData[{

Cell["A0", "Subsection",
  PageBreakAbove->True,
  CellTags->"A0"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"A0", "[", 
            SuperscriptBox["m", "2"], "]"}], " ", "is", "the", "Passarino"}], 
        "-", 
        RowBox[{"Veltman", "one"}], "-", 
        RowBox[{"point", " ", " ", "integral", " ", " ", 
          SubscriptBox["A", "0."]}]}], TraditionalForm]]]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", " ", "A0", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"A0ToB0", "\[Rule]", "True"}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["B0",
    ButtonData:>"B0",
    ButtonStyle->"Hyperlink",
    ButtonNote->"B0"],
  ", ",
  ButtonBox["C0",
    ButtonData:>"C0",
    ButtonStyle->"Hyperlink",
    ButtonNote->"C0"],
  ", ",
  ButtonBox["D0",
    ButtonData:>"D0",
    ButtonStyle->"Hyperlink",
    ButtonNote->"D0"],
  ", ",
  ButtonBox["PaVe",
    ButtonData:>"PaVe",
    ButtonStyle->"Hyperlink",
    ButtonNote->"PaVe"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[TextData[{
  "By default any ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["A", "0"], TraditionalForm]]],
  " is expressed in terms of ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["B", "0"], TraditionalForm]]],
  "'s."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"A0", "[", 
      SuperscriptBox["m", "2"], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{
            SubscriptBox["B", "0"], "(", "0", ",", 
            SuperscriptBox["m", "2"], ",", 
            SuperscriptBox["m", "2"], ")"}], " ", 
          SuperscriptBox["m", "2"]}], "+", 
        SuperscriptBox["m", "2"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"A0", "[", 
      RowBox[{
        SuperscriptBox["m", "2"], ",", 
        RowBox[{"A0ToB0", "\[Rule]", "False"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{
            SubscriptBox["B", "0"], "(", "0", ",", 
            SuperscriptBox["m", "2"], ",", 
            SuperscriptBox["m", "2"], ")"}], " ", 
          SuperscriptBox["m", "2"]}], "+", 
        SuperscriptBox["m", "2"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "According to the rules of dimensional regularization ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["A", "0"], "(", "0", ")"}], TraditionalForm]]],
  " is set to 0."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"A0", "[", "0", "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"A0", "[", 
      RowBox[{"SmallVariable", "[", 
        RowBox[{"M", "^", "2"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Amplitudes", "Subsection",
  CellTags->"Amplitudes"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[
"Amplitudes[name] is a database of Feynman amplitudes. Gauge \[Rule] 1 means \
`t Hooft Feynman gauge; Polarization \[Rule] 0 gives unpolarized OPE-type \
amplitudes, Polarization \[Rule] 1 the polarized ones."], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Amplitudes", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Gauge", "\[Rule]", "1"}], ",", 
          RowBox[{"QuarkMass", "\[Rule]", "0"}], ",", 
          RowBox[{"Polarization", "\[Rule]", "1"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynAmp",
    ButtonData:>"FeynAmp",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FeynAmp"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell["This is the amplitude of a gluon self-energy diagram.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Amplitudes", "[", "\"\<se1g1\>\"", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          RowBox[{
            SubsuperscriptBox["g", "s", "2"], " ", 
            RowBox[{
              SuperscriptBox["V", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], "(", 
              RowBox[{
                FormBox[
                  RowBox[{"-", "p"}],
                  "TraditionalForm"], "\[NoBreak]", 
                RowBox[{",", " "}], "\[NoBreak]", 
                FormBox[
                  RowBox[{"p", "-", "q"}],
                  "TraditionalForm"], "\[NoBreak]", 
                RowBox[{",", " "}], "\[NoBreak]", 
                FormBox["q",
                  "TraditionalForm"]}], ")"}], " ", 
            RowBox[{
              SuperscriptBox["V", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Alpha]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Beta]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], "(", 
              RowBox[{
                FormBox["p",
                  "TraditionalForm"], "\[NoBreak]", 
                RowBox[{",", " "}], "\[NoBreak]", 
                FormBox[
                  RowBox[{"q", "-", "p"}],
                  "TraditionalForm"], "\[NoBreak]", 
                RowBox[{",", " "}], "\[NoBreak]", 
                FormBox[
                  RowBox[{"-", "q"}],
                  "TraditionalForm"]}], ")"}], " ", 
            SuperscriptBox["g", 
              RowBox[{
                FormBox[
                  FormBox["\[Alpha]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            SuperscriptBox["g", 
              RowBox[{
                FormBox[
                  FormBox["\[Beta]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["c",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["d",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["e",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["f",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            SubscriptBox["f", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["c",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["e",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            SubscriptBox["f", 
              RowBox[{
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["d",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["f",
                    "TraditionalForm"],
                  "TraditionalForm"]}]]}], 
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"p", "-", "q"}], ")"}], "2"], " ", 
            SuperscriptBox["q", "2"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Explicit", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        RowBox[{
          FractionBox["1", 
            RowBox[{
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"p", "-", "q"}], ")"}], "2"], " ", 
              SuperscriptBox["q", "2"]}]], 
          RowBox[{"(", 
            RowBox[{
              SubsuperscriptBox["g", "s", "2"], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Alpha]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Beta]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    RowBox[{"-", 
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Alpha]",
                            "TraditionalForm"],
                          "TraditionalForm"]]}], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Beta]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["q",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Alpha]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Beta]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "+", 
                  RowBox[{"2", " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Alpha]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Beta]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Alpha]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox[
                      FormBox["q",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Beta]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Alpha]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Beta]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "+", 
                  RowBox[{"2", " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Alpha]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Beta]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox[
                      FormBox["q",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}]}], ")"}], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{"2", " ", 
                    SuperscriptBox[
                      FormBox["q",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "+", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "+", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox[
                      FormBox["q",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "-", 
                  RowBox[{"2", " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "+", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox[
                      FormBox["q",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}]}], ")"}], " ", 
              SubscriptBox["\[Delta]", 
                RowBox[{
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["\[Delta]", 
                RowBox[{
                  FormBox[
                    FormBox["e",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["f",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["e",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["f",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], ")"}]}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "This is the amplitude for graph 6.2 from the paper Z.Phys C ",
  StyleBox["70:",
    FontWeight->"Bold"],
  "637-654, 1996",
  StyleBox[".",
    FontWeight->"Bold"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FeynAmp", "[", 
      RowBox[{"q1", ",", "q2", ",", 
        RowBox[{"EpsEvaluate", "[", 
          RowBox[{"Trick", "[", 
            RowBox[{"Explicit", "[", 
              RowBox[{"Amplitudes", "[", "\"\<gg2\>\"", "]"}], "]"}], "]"}], 
          "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[Integral]", 
        RowBox[{
          StyleBox[
            RowBox[{
              SuperscriptBox["\[DifferentialD]", "D"], 
              FormBox[
                InterpretationBox[
                  SubscriptBox["q", "1"],
                  q1],
                "TraditionalForm"]}],
            ZeroWidthTimes->True], "\[Integral]", 
          RowBox[{
            SuperscriptBox["\[DifferentialD]", "D"], 
            FormBox[
              InterpretationBox[
                SubscriptBox["q", "2"],
                q2],
              "TraditionalForm"]}]}], "(", 
        FormBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{"2", " ", "\[ImaginaryI]", " ", 
                SubsuperscriptBox["g", "s", "4"], " ", 
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Lambda]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Lambda]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      InterpretationBox[
                        SubscriptBox["q", "2"],
                        q2],
                      "TraditionalForm"]}]], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      RowBox[{"-", 
                        SuperscriptBox[
                          FormBox["p",
                            "TraditionalForm"], 
                          FormBox[
                            FormBox["\[Nu]",
                              "TraditionalForm"],
                            "TraditionalForm"]]}], " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "7"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "10"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]]}], "+", 
                    RowBox[{"2", " ", 
                      SuperscriptBox[
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["q", "1"],
                            q1],
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "7"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "10"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]]}], "+", 
                    RowBox[{"2", " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Nu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "10"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "7"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "-", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Nu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "10"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["q", "1"],
                            q1],
                          "TraditionalForm"], 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "7"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "-", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Nu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "7"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "10"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "-", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Nu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "7"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["q", "1"],
                            q1],
                          "TraditionalForm"], 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "10"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}]}], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "1"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "11"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]]}], "-", 
                    RowBox[{"2", " ", 
                      SuperscriptBox[
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["q", "2"],
                            q2],
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "1"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "11"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]]}], "-", 
                    RowBox[{"2", " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Mu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "11"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "1"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "+", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Mu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "11"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["q", "2"],
                            q2],
                          "TraditionalForm"], 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "1"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "+", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Mu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "1"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "11"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "+", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Mu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "1"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["q", "2"],
                            q2],
                          "TraditionalForm"], 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "11"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}]}], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      RowBox[{"-", "2"}], " ", 
                      SuperscriptBox[
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["q", "1"],
                            q1],
                          "TraditionalForm"], 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "5"],
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "7"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "12"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]]}], "+", 
                    RowBox[{
                      SuperscriptBox[
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["q", "2"],
                            q2],
                          "TraditionalForm"], 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "5"],
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "7"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "12"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]]}], "+", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "5"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "12"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["q", "1"],
                            q1],
                          "TraditionalForm"], 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "7"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "-", 
                    RowBox[{"2", " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "5"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "12"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["q", "2"],
                            q2],
                          "TraditionalForm"], 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "7"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "+", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "5"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "7"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["q", "1"],
                            q1],
                          "TraditionalForm"], 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "12"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "+", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "5"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "7"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["q", "2"],
                            q2],
                          "TraditionalForm"], 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "12"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}]}], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      RowBox[{"-", 
                        SuperscriptBox[
                          FormBox["p",
                            "TraditionalForm"], 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "10"],
                              "TraditionalForm"],
                            "TraditionalForm"]]}], " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "11"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "12"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]]}], "-", 
                    RowBox[{
                      SuperscriptBox[
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["q", "1"],
                            q1],
                          "TraditionalForm"], 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "10"],
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "11"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "12"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]]}], "+", 
                    RowBox[{"2", " ", 
                      SuperscriptBox[
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["q", "2"],
                            q2],
                          "TraditionalForm"], 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "10"],
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "11"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "12"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]]}], "-", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "10"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "12"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "11"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "+", 
                    RowBox[{"2", " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "10"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "12"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["q", "1"],
                            q1],
                          "TraditionalForm"], 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "11"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "-", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "10"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "12"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["q", "2"],
                            q2],
                          "TraditionalForm"], 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "11"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "+", 
                    RowBox[{"2", " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "10"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "11"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "12"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "-", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "10"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "11"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["q", "1"],
                            q1],
                          "TraditionalForm"], 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "12"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "-", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "10"],
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox[
                              SubscriptBox["\[Lambda]", "11"],
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["q", "2"],
                            q2],
                          "TraditionalForm"], 
                        FormBox[
                          FormBox[
                            SubscriptBox["\[Lambda]", "12"],
                            "TraditionalForm"],
                          "TraditionalForm"]]}]}], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{"1", "-", 
                    FormBox[
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{"-", "1"}], ")"}], "m"],
                      "TraditionalForm"]}], ")"}], " ", 
                FormBox[
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["q", "2"],
                            q2],
                          "TraditionalForm"]}], ")"}], 
                    RowBox[{"m", "-", "1"}]],
                  "TraditionalForm"], " ", 
                SubscriptBox["f", 
                  RowBox[{
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\<\"c\"\>", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\<\"c\"\>", "11"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["f", 
                  RowBox[{
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\<\"c\"\>", "7"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\<\"c\"\>", "10"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["f", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\<\"c\"\>", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\<\"c\"\>", "7"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\<\"c\"\>", "12"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["f", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\<\"c\"\>", "10"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\<\"c\"\>", "11"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\<\"c\"\>", "12"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], ")"}], "/", 
            RowBox[{
              SuperscriptBox[
                InterpretationBox[
                  SubscriptBox["q", "1"],
                  q1], "2"], ".", 
              SuperscriptBox[
                InterpretationBox[
                  SubscriptBox["q", "2"],
                  q2], "2"], ".", 
              SuperscriptBox[
                InterpretationBox[
                  SubscriptBox["q", "2"],
                  q2], "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    InterpretationBox[
                      SubscriptBox["q", "2"],
                      q2], "-", "p"}], ")"}], "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    InterpretationBox[
                      SubscriptBox["q", "1"],
                      q1], "-", 
                    InterpretationBox[
                      SubscriptBox["q", "2"],
                      q2]}], ")"}], "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    InterpretationBox[
                      SubscriptBox["q", "1"],
                      q1], "-", "p"}], ")"}], "2"]}]}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["AnomalousDimension", "Subsection",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
AnomalousDimension[name] is a incomplete database of anomalous \
dimensions of twist 2 operators.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "AnomalousDimension", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
          RowBox[{"Simplify", "\[Rule]", "FullSimplify"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  ButtonBox["SplittingFunction",
    ButtonData:>"SplittingFunction",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SplittingFunction"],
  ", ",
  ButtonBox["SumS",
    ButtonData:>"SumS",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SumS"],
  ", ",
  ButtonBox["SumT",
    ButtonData:>"SumT",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SumT"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell["Polarized case", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SetOptions", "[", " ", 
      RowBox[{"AnomalousDimension", ",", 
        RowBox[{"Polarization", "\[Rule]", "1"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
          RowBox[{"Simplify", "\[Rule]", "FullSimplify"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["\[Gamma]", 
          RowBox[{"NS", ",", "qq", " "}], 
          RowBox[{"(", "0", ")", " "}]], TraditionalForm]]],
  "polarized"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"AnomalousDimension", "[", "gnsqq0", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "F"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"8", " ", 
              RowBox[{
                SubscriptBox["S", "1"], "(", 
                FormBox[
                  RowBox[{"m", "-", "1"}],
                  "TraditionalForm"], ")"}]}], "+", 
            FractionBox["4", "m"], "+", 
            FractionBox["4", 
              RowBox[{"m", "+", "1"}]], "-", "6"}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["\[Gamma]", 
          RowBox[{"S", ",", "qg", " "}], 
          RowBox[{"(", "0", ")", " "}]], TraditionalForm]]],
  "polarized"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"AnomalousDimension", "[", "gsqg0", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            FractionBox["8", "m"], "-", 
            FractionBox["16", 
              RowBox[{"m", "+", "1"}]]}], ")"}], " ", 
        SubscriptBox["T", "f"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["\[Gamma]", 
          RowBox[{"S", ",", "gq", " "}], 
          RowBox[{"(", "0", ")", " "}]], TraditionalForm]]],
  "polarized"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"AnomalousDimension", "[", "gsgq0", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "F"], " ", 
        RowBox[{"(", 
          RowBox[{
            FractionBox["4", 
              RowBox[{"m", "+", "1"}]], "-", 
            FractionBox["8", "m"]}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["\[Gamma]", 
          RowBox[{"S", ",", "gg", " "}], 
          RowBox[{"(", "0", ")", " "}]], TraditionalForm]]],
  "polarized"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"AnomalousDimension", "[", "gsgg0", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{"8", " ", 
            SubscriptBox["T", "f"]}], "3"], "+", 
        RowBox[{
          SubscriptBox["C", "A"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"8", " ", 
                RowBox[{
                  SubscriptBox["S", "1"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "-", 
              FractionBox["8", "m"], "+", 
              FractionBox["16", 
                RowBox[{"m", "+", "1"}]], "-", 
              FractionBox["22", "3"]}], ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["\[Gamma]", 
          RowBox[{"PS", ",", "qq", " "}], 
          RowBox[{"(", "0", ")", " "}]], TraditionalForm]]],
  "polarized"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"AnomalousDimension", "[", "gpsqq1", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"16", " ", 
        SubscriptBox["C", "F"], " ", 
        RowBox[{"(", 
          RowBox[{
            FractionBox["1", 
              RowBox[{"m", "+", "1"}]], "+", 
            FractionBox["3", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
            FractionBox["2", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"m", "+", "1"}], ")"}], "3"]], "-", 
            FractionBox["1", "m"], "-", 
            FractionBox["1", 
              SuperscriptBox["m", "2"]], "+", 
            FractionBox["2", 
              SuperscriptBox["m", "3"]]}], ")"}], " ", 
        SubscriptBox["T", "f"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["\[Gamma]", 
          RowBox[{"NS", ",", "qq", " "}], 
          RowBox[{"(", "1", ")", " "}]], TraditionalForm]]],
  "polarized"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"AnomalousDimension", "[", "gnsqq1", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            RowBox[{"(", 
              RowBox[{
                FractionBox[
                  RowBox[{"16", " ", 
                    RowBox[{
                      SubscriptBox["S", "1"], "(", 
                      FormBox[
                        RowBox[{"m", "-", "1"}],
                        "TraditionalForm"], ")"}]}], 
                  SuperscriptBox["m", "2"]], "+", 
                FractionBox[
                  RowBox[{"16", " ", 
                    RowBox[{
                      SubscriptBox["S", "1"], "(", 
                      FormBox[
                        RowBox[{"m", "-", "1"}],
                        "TraditionalForm"], ")"}]}], 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
                FractionBox[
                  RowBox[{"16", " ", 
                    RowBox[{
                      SubscriptBox["S", "2"], "(", 
                      FormBox[
                        RowBox[{"m", "-", "1"}],
                        "TraditionalForm"], ")"}]}], "m"], "+", 
                FractionBox[
                  RowBox[{"16", " ", 
                    RowBox[{
                      SubscriptBox["S", "2"], "(", 
                      FormBox[
                        RowBox[{"m", "-", "1"}],
                        "TraditionalForm"], ")"}]}], 
                  RowBox[{"m", "+", "1"}]], "-", 
                RowBox[{"24", " ", 
                  RowBox[{
                    SubscriptBox["S", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "+", 
                RowBox[{"32", " ", 
                  RowBox[{
                    SubscriptBox["S", "12"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "+", 
                RowBox[{"32", " ", 
                  RowBox[{
                    SubscriptBox["S", "21"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "+", 
                FractionBox[
                  RowBox[{"32", " ", 
                    RowBox[{
                      SubscriptBox[
                        OverscriptBox["S", "~"], "2"], "(", 
                      FormBox[
                        RowBox[{"m", "-", "1"}],
                        "TraditionalForm"], ")"}]}], "m"], "+", 
                FractionBox[
                  RowBox[{"32", " ", 
                    RowBox[{
                      SubscriptBox[
                        OverscriptBox["S", "~"], "2"], "(", 
                      FormBox[
                        RowBox[{"m", "-", "1"}],
                        "TraditionalForm"], ")"}]}], 
                  RowBox[{"m", "+", "1"}]], "-", 
                RowBox[{"32", " ", 
                  RowBox[{
                    SubscriptBox[
                      OverscriptBox["S", "~"], "3"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "+", 
                RowBox[{"64", " ", 
                  RowBox[{
                    SubscriptBox[
                      OverscriptBox["S", "~"], "12"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "-", 
                FractionBox["40", "m"], "+", 
                FractionBox["40", 
                  RowBox[{"m", "+", "1"}]], "+", 
                FractionBox["16", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
                FractionBox["8", 
                  SuperscriptBox["m", "3"]], "+", 
                FractionBox["40", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"m", "+", "1"}], ")"}], "3"]], "+", "3"}], 
              ")"}]}], " ", 
          SubsuperscriptBox["C", "F", "2"]}], "-", 
        RowBox[{
          SubscriptBox["N", "f"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                FractionBox["80", "9"], " ", 
                RowBox[{
                  SubscriptBox["S", "1"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "-", 
              RowBox[{
                FractionBox["16", "3"], " ", 
                RowBox[{
                  SubscriptBox["S", "2"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "-", 
              FractionBox["8", 
                RowBox[{"9", " ", "m"}]], "+", 
              FractionBox["88", 
                RowBox[{"9", " ", 
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}]}]], "-", 
              FractionBox["8", 
                RowBox[{"3", " ", 
                  SuperscriptBox["m", "2"]}]], "-", 
              FractionBox["8", 
                RowBox[{"3", " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "-", 
              FractionBox["2", "3"]}], ")"}], " ", 
          SubscriptBox["C", "F"]}], "-", 
        RowBox[{
          SubscriptBox["C", "A"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  FractionBox["536", "9"]}], " ", 
                RowBox[{
                  SubscriptBox["S", "1"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "+", 
              RowBox[{
                FractionBox["88", "3"], " ", 
                RowBox[{
                  SubscriptBox["S", "2"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "-", 
              RowBox[{"16", " ", 
                RowBox[{
                  SubscriptBox["S", "3"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "-", 
              FractionBox[
                RowBox[{"16", " ", 
                  RowBox[{
                    SubscriptBox[
                      OverscriptBox["S", "~"], "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "m"], "-", 
              FractionBox[
                RowBox[{"16", " ", 
                  RowBox[{
                    SubscriptBox[
                      OverscriptBox["S", "~"], "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"m", "+", "1"}]], "+", 
              RowBox[{"16", " ", 
                RowBox[{
                  SubscriptBox[
                    OverscriptBox["S", "~"], "3"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "-", 
              RowBox[{"32", " ", 
                RowBox[{
                  SubscriptBox[
                    OverscriptBox["S", "~"], "12"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "+", 
              FractionBox["212", 
                RowBox[{"9", " ", "m"}]], "-", 
              FractionBox["748", 
                RowBox[{"9", " ", 
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}]}]], "+", 
              FractionBox["44", 
                RowBox[{"3", " ", 
                  SuperscriptBox["m", "2"]}]], "-", 
              FractionBox["4", 
                RowBox[{"3", " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "-", 
              FractionBox["16", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}], "3"]], "+", 
              FractionBox["17", "3"]}], ")"}], " ", 
          SubscriptBox["C", "F"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["\[Gamma]", 
          RowBox[{"S", ",", "qg", " "}], 
          RowBox[{"(", "1", ")", " "}]], TraditionalForm]]],
  "polarized"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"AnomalousDimension", "[", "gsqg1", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"8", " ", 
          SubscriptBox["C", "F"], " ", 
          SubscriptBox["T", "f"], " ", 
          RowBox[{"(", 
            RowBox[{
              FractionBox[
                RowBox[{"2", " ", 
                  RowBox[{
                    SubsuperscriptBox["S", "1", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "m"], "-", 
              FractionBox[
                RowBox[{"4", " ", 
                  RowBox[{
                    SubsuperscriptBox["S", "1", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"m", "+", "1"}]], "-", 
              FractionBox[
                RowBox[{"2", " ", 
                  RowBox[{
                    SubscriptBox["S", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "m"], "+", 
              FractionBox[
                RowBox[{"4", " ", 
                  RowBox[{
                    SubscriptBox["S", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"m", "+", "1"}]], "+", 
              FractionBox["14", "m"], "-", 
              FractionBox["19", 
                RowBox[{"m", "+", "1"}]], "-", 
              FractionBox["1", 
                SuperscriptBox["m", "2"]], "-", 
              FractionBox["8", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}], "2"]], "-", 
              FractionBox["2", 
                SuperscriptBox["m", "3"]], "+", 
              FractionBox["4", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}], "3"]]}], ")"}]}], "+", 
        RowBox[{"16", " ", 
          SubscriptBox["C", "A"], " ", 
          SubscriptBox["T", "f"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"-", 
                FractionBox[
                  RowBox[{
                    SubsuperscriptBox["S", "1", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}], "m"]}], "+", 
              FractionBox[
                RowBox[{"2", " ", 
                  RowBox[{
                    SubsuperscriptBox["S", "1", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"m", "+", "1"}]], "-", 
              FractionBox[
                RowBox[{"2", " ", 
                  RowBox[{
                    SubscriptBox["S", "1"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                SuperscriptBox["m", "2"]], "+", 
              FractionBox[
                RowBox[{"4", " ", 
                  RowBox[{
                    SubscriptBox["S", "1"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}], "2"]], "-", 
              FractionBox[
                RowBox[{
                  SubscriptBox["S", "2"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}], "m"], "+", 
              FractionBox[
                RowBox[{"2", " ", 
                  RowBox[{
                    SubscriptBox["S", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"m", "+", "1"}]], "-", 
              FractionBox[
                RowBox[{"2", " ", 
                  RowBox[{
                    SubscriptBox[
                      OverscriptBox["S", "~"], "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "m"], "+", 
              FractionBox[
                RowBox[{"4", " ", 
                  RowBox[{
                    SubscriptBox[
                      OverscriptBox["S", "~"], "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"m", "+", "1"}]], "-", 
              FractionBox["4", "m"], "+", 
              FractionBox["3", 
                RowBox[{"m", "+", "1"}]], "-", 
              FractionBox["3", 
                SuperscriptBox["m", "2"]], "+", 
              FractionBox["8", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
              FractionBox["2", 
                SuperscriptBox["m", "3"]], "+", 
              FractionBox["12", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}], "3"]]}], ")"}]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["\[Gamma]", 
          RowBox[{"S", ",", "gq", " "}], 
          RowBox[{"(", "1", ")", " "}]], TraditionalForm]]],
  "polarized"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"AnomalousDimension", "[", "gsgq1", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"4", " ", 
          RowBox[{"(", 
            RowBox[{
              FractionBox[
                RowBox[{"4", " ", 
                  RowBox[{
                    SubsuperscriptBox["S", "1", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "m"], "-", 
              FractionBox[
                RowBox[{"2", " ", 
                  RowBox[{
                    SubsuperscriptBox["S", "1", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"m", "+", "1"}]], "-", 
              FractionBox[
                RowBox[{"8", " ", 
                  RowBox[{
                    SubscriptBox["S", "1"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "m"], "+", 
              FractionBox[
                RowBox[{"2", " ", 
                  RowBox[{
                    SubscriptBox["S", "1"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"m", "+", "1"}]], "+", 
              FractionBox[
                RowBox[{"8", " ", 
                  RowBox[{
                    SubscriptBox["S", "1"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                SuperscriptBox["m", "2"]], "-", 
              FractionBox[
                RowBox[{"4", " ", 
                  RowBox[{
                    SubscriptBox["S", "1"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
              FractionBox[
                RowBox[{"4", " ", 
                  RowBox[{
                    SubscriptBox["S", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "m"], "-", 
              FractionBox[
                RowBox[{"2", " ", 
                  RowBox[{
                    SubscriptBox["S", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"m", "+", "1"}]], "+", 
              FractionBox["15", "m"], "-", 
              FractionBox["6", 
                RowBox[{"m", "+", "1"}]], "-", 
              FractionBox["12", 
                SuperscriptBox["m", "2"]], "+", 
              FractionBox["3", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
              FractionBox["4", 
                SuperscriptBox["m", "3"]], "-", 
              FractionBox["2", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}], "3"]]}], ")"}], " ", 
          SubsuperscriptBox["C", "F", "2"]}], "+", 
        RowBox[{"32", " ", 
          SubscriptBox["T", "f"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"-", 
                FractionBox[
                  RowBox[{"2", " ", 
                    RowBox[{
                      SubscriptBox["S", "1"], "(", 
                      FormBox[
                        RowBox[{"m", "-", "1"}],
                        "TraditionalForm"], ")"}]}], 
                  RowBox[{"3", " ", "m"}]]}], "+", 
              FractionBox[
                RowBox[{
                  SubscriptBox["S", "1"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}], 
                RowBox[{"3", " ", 
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}]}]], "+", 
              FractionBox["7", 
                RowBox[{"9", " ", "m"}]], "-", 
              FractionBox["2", 
                RowBox[{"9", " ", 
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}]}]], "-", 
              FractionBox["2", 
                RowBox[{"3", " ", 
                  SuperscriptBox["m", "2"]}]], "+", 
              FractionBox["1", 
                RowBox[{"3", " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"m", "+", "1"}], ")"}], "2"]}]]}], ")"}], " ", 
          SubscriptBox["C", "F"]}], "+", 
        RowBox[{"8", " ", 
          SubscriptBox["C", "A"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"-", 
                FractionBox[
                  RowBox[{"2", " ", 
                    RowBox[{
                      SubsuperscriptBox["S", "1", "2"], "(", 
                      FormBox[
                        RowBox[{"m", "-", "1"}],
                        "TraditionalForm"], ")"}]}], "m"]}], "+", 
              FractionBox[
                RowBox[{
                  SubsuperscriptBox["S", "1", "2"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}], 
                RowBox[{"m", "+", "1"}]], "+", 
              FractionBox[
                RowBox[{"16", " ", 
                  RowBox[{
                    SubscriptBox["S", "1"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"3", " ", "m"}]], "-", 
              FractionBox[
                RowBox[{"5", " ", 
                  RowBox[{
                    SubscriptBox["S", "1"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"3", " ", 
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}]}]], "+", 
              FractionBox[
                RowBox[{"2", " ", 
                  RowBox[{
                    SubscriptBox["S", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "m"], "-", 
              FractionBox[
                RowBox[{
                  SubscriptBox["S", "2"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}], 
                RowBox[{"m", "+", "1"}]], "+", 
              FractionBox[
                RowBox[{"4", " ", 
                  RowBox[{
                    SubscriptBox[
                      OverscriptBox["S", "~"], "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "m"], "-", 
              FractionBox[
                RowBox[{"2", " ", 
                  RowBox[{
                    SubscriptBox[
                      OverscriptBox["S", "~"], "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"m", "+", "1"}]], "-", 
              FractionBox["56", 
                RowBox[{"9", " ", "m"}]], "-", 
              FractionBox["20", 
                RowBox[{"9", " ", 
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}]}]], "+", 
              FractionBox["28", 
                RowBox[{"3", " ", 
                  SuperscriptBox["m", "2"]}]], "-", 
              FractionBox["38", 
                RowBox[{"3", " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "-", 
              FractionBox["4", 
                SuperscriptBox["m", "3"]], "-", 
              FractionBox["6", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}], "3"]]}], ")"}], " ", 
          SubscriptBox["C", "F"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["\[Gamma]", 
          RowBox[{"S", ",", "gg", " "}], 
          RowBox[{"(", "1", ")", " "}]], TraditionalForm]]],
  "polarized"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"AnomalousDimension", "[", "gsgg1", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"4", " ", 
          RowBox[{"(", 
            RowBox[{
              FractionBox[
                RowBox[{"8", " ", 
                  RowBox[{
                    SubscriptBox["S", "1"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                SuperscriptBox["m", "2"]], "-", 
              FractionBox[
                RowBox[{"16", " ", 
                  RowBox[{
                    SubscriptBox["S", "1"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
              RowBox[{
                FractionBox["134", "9"], " ", 
                RowBox[{
                  SubscriptBox["S", "1"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "+", 
              FractionBox[
                RowBox[{"8", " ", 
                  RowBox[{
                    SubscriptBox["S", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "m"], "-", 
              FractionBox[
                RowBox[{"16", " ", 
                  RowBox[{
                    SubscriptBox["S", "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"m", "+", "1"}]], "+", 
              RowBox[{"4", " ", 
                RowBox[{
                  SubscriptBox["S", "3"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "-", 
              RowBox[{"8", " ", 
                RowBox[{
                  SubscriptBox["S", "12"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "-", 
              RowBox[{"8", " ", 
                RowBox[{
                  SubscriptBox["S", "21"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "+", 
              FractionBox[
                RowBox[{"8", " ", 
                  RowBox[{
                    SubscriptBox[
                      OverscriptBox["S", "~"], "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], "m"], "-", 
              FractionBox[
                RowBox[{"16", " ", 
                  RowBox[{
                    SubscriptBox[
                      OverscriptBox["S", "~"], "2"], "(", 
                    FormBox[
                      RowBox[{"m", "-", "1"}],
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"m", "+", "1"}]], "+", 
              RowBox[{"4", " ", 
                RowBox[{
                  SubscriptBox[
                    OverscriptBox["S", "~"], "3"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "-", 
              RowBox[{"8", " ", 
                RowBox[{
                  SubscriptBox[
                    OverscriptBox["S", "~"], "12"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "-", 
              FractionBox["107", 
                RowBox[{"9", " ", "m"}]], "+", 
              FractionBox["241", 
                RowBox[{"9", " ", 
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}]}]], "+", 
              FractionBox["58", 
                RowBox[{"3", " ", 
                  SuperscriptBox["m", "2"]}]], "-", 
              FractionBox["86", 
                RowBox[{"3", " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "-", 
              FractionBox["8", 
                SuperscriptBox["m", "3"]], "-", 
              FractionBox["48", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}], "3"]], "-", 
              FractionBox["16", "3"]}], ")"}], " ", 
          SubsuperscriptBox["C", "A", "2"]}], "+", 
        RowBox[{"32", " ", 
          SubscriptBox["T", "f"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  FractionBox["5", "9"]}], " ", 
                RowBox[{
                  SubscriptBox["S", "1"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "1"}],
                    "TraditionalForm"], ")"}]}], "+", 
              FractionBox["14", 
                RowBox[{"9", " ", "m"}]], "-", 
              FractionBox["19", 
                RowBox[{"9", " ", 
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}]}]], "-", 
              FractionBox["1", 
                RowBox[{"3", " ", 
                  SuperscriptBox["m", "2"]}]], "-", 
              FractionBox["1", 
                RowBox[{"3", " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "+", 
              FractionBox["1", "3"]}], ")"}], " ", 
          SubscriptBox["C", "A"]}], "+", 
        RowBox[{"8", " ", 
          SubscriptBox["C", "F"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"-", 
                FractionBox["10", 
                  RowBox[{"m", "+", "1"}]]}], "+", 
              FractionBox["2", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
              FractionBox["4", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}], "3"]], "+", "1", "+", 
              FractionBox["10", "m"], "-", 
              FractionBox["10", 
                SuperscriptBox["m", "2"]], "+", 
              FractionBox["4", 
                SuperscriptBox["m", "3"]]}], ")"}], " ", 
          SubscriptBox["T", "f"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["\[Gamma]", 
          RowBox[{"S", ",", "gg", " "}], 
          RowBox[{"(", "1", ")", " "}]], TraditionalForm]]],
  "polarized (different representation)"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"AnomalousDimension", "[", "GSGG1", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"4", " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"-", 
                FractionBox[
                  RowBox[{
                    RowBox[{"m", " ", 
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{"m", " ", 
                            RowBox[{"(", 
                              RowBox[{
                                RowBox[{"m", " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{"m", " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{"48", " ", "m", " ", 
                                        RowBox[{"(", 
                                        RowBox[{"m", "+", "3"}], ")"}]}], 
                                        "+", "469"}], ")"}]}], "+", "698"}], 
                                    ")"}]}], "+", "7"}], ")"}]}], "+", 
                          "258"}], ")"}]}], "+", "144"}], 
                  RowBox[{"9", " ", 
                    SuperscriptBox["m", "3"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"m", "+", "1"}], ")"}], "3"]}]]}], "+", 
              FractionBox[
                RowBox[{"8", " ", 
                  RowBox[{
                    SubsuperscriptBox["S", "2", "'"], "\[NoBreak]", "(", 
                    "\[NoBreak]", 
                    FormBox[
                      FractionBox["m", "2"],
                      "TraditionalForm"], "\[NoBreak]", ")"}]}], 
                RowBox[{"m", " ", 
                  RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}]}]], "-", 
              RowBox[{
                SubsuperscriptBox["S", "3", "'"], "\[NoBreak]", "(", 
                "\[NoBreak]", 
                FormBox[
                  FractionBox["m", "2"],
                  "TraditionalForm"], "\[NoBreak]", ")"}], "+", 
              FractionBox[
                RowBox[{"2", " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"m", " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{"67", " ", "m", " ", 
                              SuperscriptBox[
                                RowBox[{"(", 
                                  RowBox[{"m", "+", "1"}], ")"}], "2"]}], 
                            "+", "144"}], ")"}]}], "+", "72"}], ")"}], " ", 
                  RowBox[{
                    SubscriptBox["S", "1"], "(", 
                    FormBox["m",
                      "TraditionalForm"], ")"}]}], 
                RowBox[{"9", " ", 
                  SuperscriptBox["m", "2"], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "-", 
              RowBox[{"4", " ", 
                RowBox[{
                  SubsuperscriptBox["S", "2", "'"], "\[NoBreak]", "(", 
                  "\[NoBreak]", 
                  FormBox[
                    FractionBox["m", "2"],
                    "TraditionalForm"], "\[NoBreak]", ")"}], " ", 
                RowBox[{
                  SubscriptBox["S", "1"], "(", 
                  FormBox["m",
                    "TraditionalForm"], ")"}]}], "+", 
              RowBox[{"8", " ", 
                RowBox[{
                  OverscriptBox["S", "~"], "(", 
                  FormBox["m",
                    "TraditionalForm"], ")"}]}]}], ")"}], " ", 
          SubsuperscriptBox["C", "A", "2"]}], "+", 
        RowBox[{"32", " ", 
          SubscriptBox["T", "f"], " ", 
          RowBox[{"(", 
            RowBox[{
              FractionBox[
                RowBox[{
                  RowBox[{"m", " ", 
                    RowBox[{"(", 
                      RowBox[{"m", "+", "1"}], ")"}], " ", 
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"3", " ", "m", " ", 
                          RowBox[{"(", 
                            RowBox[{"m", "+", "1"}], ")"}]}], "+", "13"}], 
                      ")"}]}], "-", "3"}], 
                RowBox[{"9", " ", 
                  SuperscriptBox["m", "2"], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "-", 
              FractionBox[
                RowBox[{"5", " ", 
                  RowBox[{
                    SubscriptBox["S", "1"], "(", 
                    FormBox["m",
                      "TraditionalForm"], ")"}]}], "9"]}], ")"}], " ", 
          SubscriptBox["C", "A"]}], "+", 
        FractionBox[
          RowBox[{"8", " ", 
            SubscriptBox["C", "F"], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"m", " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"m", " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{"m", " ", 
                              RowBox[{"(", 
                                RowBox[{
                                  RowBox[{"m", " ", 
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{"m", " ", 
                                        RowBox[{"(", 
                                        RowBox[{"m", "+", "3"}], ")"}]}], 
                                        "+", "5"}], ")"}]}], "+", "1"}], 
                                ")"}]}], "-", "8"}], ")"}]}], "+", "2"}], 
                    ")"}]}], "+", "4"}], ")"}], " ", 
            SubscriptBox["T", "f"]}], 
          RowBox[{
            SuperscriptBox["m", "3"], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"m", "+", "1"}], ")"}], "3"]}]]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "Check that all odd moments give the same for the two representations of ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubsuperscriptBox["\[Gamma]", 
            RowBox[{"S", ",", "gg", " "}], 
            RowBox[{"(", "1", ")", " "}]], "."}], TraditionalForm]]]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"%", "-", "%%"}], "/.", 
          RowBox[{"OPEm", "\[Rule]", "ij"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"ij", ",", "1", ",", "17", ",", "2"}], "}"}]}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
        "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
          "0", ",", "0"}], "}"}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Anti5", "Subsection",
  CellTags->"Anti5"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "Anti5[exp] anticommutes all ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  "in exp to the right. Anti5[exp, n] anticommutes all ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Gamma]", 
          RowBox[{"5", " "}]], TraditionalForm]]],
  "n times to the right. Anti5[exp, -n] anticommutes all ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  " n times to the left."
}], "Text"],

Cell[TextData[{
  "The naive ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  "scheme is used."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracOrder",
    ButtonData:>"DiracOrder",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracOrder"],
  ", ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracSimplify"],
  ", ",
  ButtonBox["DiracTrick",
    ButtonData:>"DiracTrick",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracTrick"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracMatrix", "[", 
        RowBox[{"5", ",", "\[Mu]"}], "]"}], " "}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "5"], 
        SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Anti5", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", "5"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Anti5", "[", 
      RowBox[{"%", ",", 
        RowBox[{"-", "1"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "5"], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracMatrix", "[", 
      RowBox[{
      "5", ",", "\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", 
        "\[Delta]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "5"], 
        SuperscriptBox["\[Gamma]", "\[Alpha]"], 
        SuperscriptBox["\[Gamma]", "\[Beta]"], 
        SuperscriptBox["\[Gamma]", "\[Gamma]"], 
        SuperscriptBox["\[Gamma]", "\[Delta]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Anti5", "[", 
      RowBox[{"%", ",", "2"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Alpha]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Beta]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", "5"], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Gamma]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Delta]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Anti5", "[", 
      RowBox[{"%%", ",", "Infinity"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Alpha]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Beta]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Gamma]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Delta]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Anti5", "[", 
      RowBox[{"%", ",", 
        RowBox[{"-", "Infinity"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "5"], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Alpha]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Beta]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Gamma]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Delta]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "In the naive ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  "- scheme D-dimensional ",
  Cell[BoxData[
      FormBox["\[Gamma]", TraditionalForm]]],
  "-matrices anticommute with ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GAD", "[", 
      RowBox[{"5", ",", "\[Mu]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "5"], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Anti5", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", "5"]}]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["AntiCommutator", "Subsection",
  CellTags->"AntiCommutator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
AntiCommutator[x, y] = c defines the anti-commutator of the non \
commuting objects x and y.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Commutator",
    ButtonData:>"Commutator",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Commutator"],
  ", ",
  ButtonBox["CommutatorExplicit",
    ButtonData:>"CommutatorExplicit",
    ButtonStyle->"Hyperlink",
    ButtonNote->"CommutatorExplicit"],
  ", ",
  ButtonBox["DeclareNonCommutative",
    ButtonData:>"DeclareNonCommutative",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DeclareNonCommutative"],
  ", ",
  ButtonBox["DotSimplify",
    ButtonData:>"DotSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DotSimplify"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[TextData[{
  "This declares ",
  StyleBox["a",
    FontFamily->"Courier"],
  " and ",
  StyleBox["b",
    FontFamily->"Courier"],
  " as noncommutative variables."
}], "Text"],

Cell[BoxData[
    RowBox[{"DeclareNonCommutative", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"AntiCommutator", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
      "{", "\[NoBreak]", "a", "\[NoBreak]", ",", "\[NoBreak]", 
        "\[MediumSpace]", "\[NoBreak]", "b", "\[NoBreak]", "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CommutatorExplicit", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", ".", "b"}], "+", 
        RowBox[{"b", ".", "a"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CommutatorExplicit", "[", 
      RowBox[{"AntiCommutator", "[", 
        RowBox[{
          RowBox[{"a", "+", "b"}], ",", 
          RowBox[{
            RowBox[{"a", "-", 
              RowBox[{"2", "b"}]}], " "}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{"a", "-", 
              RowBox[{"2", " ", "b"}]}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{"a", "+", "b"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{"a", "+", "b"}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{"a", "-", 
              RowBox[{"2", " ", "b"}]}], ")"}]}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotSimplify", "[", 
      RowBox[{"AntiCommutator", "[", 
        RowBox[{
          RowBox[{"a", "+", "b"}], ",", 
          RowBox[{
            RowBox[{"a", "-", 
              RowBox[{"2", "b"}]}], " "}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"2", " ", 
          SuperscriptBox["a", "2"]}], "-", 
        RowBox[{"4", " ", 
          SuperscriptBox["b", "2"]}], "-", 
        RowBox[{"a", ".", "b"}], "-", 
        RowBox[{"b", ".", "a"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"DeclareNonCommutative", "[", 
      RowBox[{"c", ",", "d", ",", 
        OverscriptBox["c", "~"], ",", 
        OverscriptBox["d", "~"]}], "]"}]], "Input"],

Cell["Defining {c,d} = z results in replacements of c.d by z-d.c.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"AntiCommutator", "[", 
        RowBox[{"c", ",", "d"}], "]"}], " ", "=", " ", "z"}]], "Input"],

Cell[BoxData[
    FormBox["z", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotSimplify", "[", " ", 
      RowBox[{"d", " ", ".", " ", "c", " ", ".", " ", "d"}], " ", "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"d", " ", "z"}], "-", 
        RowBox[{
          SuperscriptBox["d", "2"], ".", "c"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"AntiCommutator", "[", 
        RowBox[{
          OverscriptBox["d", "~"], ",", 
          OverscriptBox["c", "~"]}], "]"}], " ", "=", " ", 
      OverscriptBox["z", "~"]}]], "Input"],

Cell[BoxData[
    FormBox[
      OverscriptBox["z", "~"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotSimplify", "[", " ", 
      RowBox[{
        OverscriptBox["d", "~"], " ", ".", " ", 
        OverscriptBox["c", "~"], " ", ".", " ", 
        OverscriptBox["d", "~"]}], " ", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          OverscriptBox["d", "~"], " ", 
          OverscriptBox["z", "~"]}], "-", 
        RowBox[{
          OverscriptBox["c", "~"], ".", 
          SuperscriptBox[
            OverscriptBox["d", "~"], "2"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"UnDeclareNonCommutative", "[", 
      RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", 
        OverscriptBox["c", "~"], ",", 
        OverscriptBox["d", "~"]}], "]"}]], "Input"],

Cell[BoxData[
    RowBox[{"Unset", "[", 
      RowBox[{"AntiCommutator", "[", 
        RowBox[{"c", ",", "d"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    RowBox[{"Unset", "[", 
      RowBox[{"AntiCommutator", "[", 
        RowBox[{
          OverscriptBox["d", "~"], ",", 
          OverscriptBox["c", "~"]}], "]"}], "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["AntiQuarkField", "Subsection",
  CellTags->"AntiQuarkField"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
AntiQuarkField is the name of a fermionic field. AntiQuarkField is \
just a name with no functional properties. Only typeset rules are attached.\
\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["QuantumField",
    ButtonData:>"QuantumField",
    ButtonStyle->"Hyperlink",
    ButtonNote->"QuantumField"],
  ", ",
  ButtonBox["QuarkField",
    ButtonData:>"QuarkField",
    ButtonStyle->"Hyperlink",
    ButtonNote->"QuarkField"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData["AntiQuarkField"], "Input"],

Cell[BoxData[
    FormBox[
      OverscriptBox["\[Psi]", "_"], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["AntiSymmetrize", "Subsection",
  CellTags->"AntiSymmetrize"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
AntiSymmetrize[expr, {a1, a2, ...}] antisymmetrizes expr with \
respect to the variables a1,a2, ... \
\>", "Text"],

Cell[TextData[{
  "See also: ",
  ButtonBox["Symmetrize",
    ButtonData:>"Symmetrize",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Symmetrize"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"AntiSymmetrize", "[", 
      RowBox[{
        RowBox[{"f", "[", 
          RowBox[{"a", ",", "b"}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"a", ",", "b"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"f", "(", 
              RowBox[{"a", ",", "b"}], ")"}], "-", 
            RowBox[{"f", "(", 
              RowBox[{"b", ",", "a"}], ")"}]}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"AntiSymmetrize", "[", 
      RowBox[{
        RowBox[{"f", "[", 
          RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "6"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"f", "(", 
              RowBox[{"x", ",", "y", ",", "z"}], ")"}], "-", 
            RowBox[{"f", "(", 
              RowBox[{"x", ",", "z", ",", "y"}], ")"}], "-", 
            RowBox[{"f", "(", 
              RowBox[{"y", ",", "x", ",", "z"}], ")"}], "+", 
            RowBox[{"f", "(", 
              RowBox[{"y", ",", "z", ",", "x"}], ")"}], "+", 
            RowBox[{"f", "(", 
              RowBox[{"z", ",", "x", ",", "y"}], ")"}], "-", 
            RowBox[{"f", "(", 
              RowBox[{"z", ",", "y", ",", "x"}], ")"}]}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Apart1", "Subsection",
  CellTags->"Apart1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Apart1[expr, x] is equivalent to Apart[expr, x], but it is faster \
than Apart if explicit complex numbers (I's) are in exp.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Apart2",
    ButtonData:>"Apart2",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Apart2"],
  ", ",
  ButtonBox["Apart3",
    ButtonData:>"Apart3",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Apart3"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{"t", "=", 
        RowBox[{
          RowBox[{
            RowBox[{"1", "/", "4"}], "*", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", "x", "*", 
            RowBox[{"(", 
              RowBox[{"21", "+", 
                RowBox[{"13", "*", "x"}], "+", 
                RowBox[{"3", "*", 
                  RowBox[{"x", "^", "2"}]}]}], ")"}]}], "-", 
          RowBox[{
            RowBox[{"1", "/", "24"}], "*", "11", "*", "I", "*", "Pi", "*", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", "x", "*", 
            RowBox[{"(", 
              RowBox[{"21", "+", 
                RowBox[{"13", "*", "x"}], "+", 
                RowBox[{"3", "*", 
                  RowBox[{"x", "^", "2"}]}]}], ")"}]}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", 
                RowBox[{"x", "^", "2"}], "*", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", "*", "x"}], "+", 
                    RowBox[{"3", "*", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"36", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "-", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"I", "*", "Pi", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", 
                RowBox[{"x", "^", "2"}], "*", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", "*", "x"}], "+", 
                    RowBox[{"3", "*", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"12", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "-", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"7", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", 
                RowBox[{"x", "^", "3"}], "*", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", "*", "x"}], "+", 
                    RowBox[{"3", "*", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"48", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"3", "*", "I", "*", "Pi", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", 
                RowBox[{"x", "^", "3"}], "*", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", "*", "x"}], "+", 
                    RowBox[{"3", "*", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"8", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"11", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", 
                RowBox[{"x", "^", "4"}], "*", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", "*", "x"}], "+", 
                    RowBox[{"3", "*", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"24", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "-", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"3", "*", "I", "*", "Pi", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", 
                RowBox[{"x", "^", "4"}], "*", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", "*", "x"}], "+", 
                    RowBox[{"3", "*", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"4", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "-", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"49", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", 
                RowBox[{"x", "^", "5"}], "*", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", "*", "x"}], "+", 
                    RowBox[{"3", "*", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"144", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"11", "*", "I", "*", "Pi", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", 
                RowBox[{"x", "^", "5"}], "*", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", "*", "x"}], "+", 
                    RowBox[{"3", "*", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"24", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "+", 
          RowBox[{
            RowBox[{"1", "/", "4"}], "*", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", "x", "*", 
            RowBox[{"(", 
              RowBox[{"21", "+", 
                RowBox[{"13", "*", "x"}], "+", 
                RowBox[{"3", "*", 
                  RowBox[{"x", "^", "2"}]}]}], ")"}], "*", "Zeta2"}], "+", 
          RowBox[{
            RowBox[{"11", "/", "24"}], "*", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", "x", "*", 
            RowBox[{"(", 
              RowBox[{"21", "+", 
                RowBox[{"13", "*", "x"}], "+", 
                RowBox[{"3", "*", 
                  RowBox[{"x", "^", "2"}]}]}], ")"}], "*", 
            RowBox[{"Log", "[", "x", "]"}]}], "+", 
          RowBox[{
            RowBox[{"1", "/", "4"}], "*", "I", "*", "Pi", "*", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", "x", "*", 
            RowBox[{"(", 
              RowBox[{"21", "+", 
                RowBox[{"13", "*", "x"}], "+", 
                RowBox[{"3", "*", 
                  RowBox[{"x", "^", "2"}]}]}], ")"}], "*", 
            RowBox[{"Log", "[", "x", "]"}]}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", 
                RowBox[{"x", "^", "2"}], "*", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", "*", "x"}], "+", 
                    RowBox[{"3", "*", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}], "*", 
                RowBox[{"Log", "[", "x", "]"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"12", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "-", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"3", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", 
                RowBox[{"x", "^", "3"}], "*", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", "*", "x"}], "+", 
                    RowBox[{"3", "*", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}], "*", 
                RowBox[{"Log", "[", "x", "]"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"8", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"3", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], "*", 
                RowBox[{"x", "^", "4"}], "*", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", "*", "x"}], "+", 
                    RowBox[{"3", "*", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}], "*", 
                RowBox[{"Log", "[", "x", "]"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"4", "*", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}]}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Timing", "[", 
      RowBox[{
        RowBox[{"a1", "=", 
          RowBox[{"Apart1", "[", 
            RowBox[{"t", ",", "x"}], "]"}]}], ";"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"0.530000000000001136`", " ", "Second"}], ",", "Null"}], 
        "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Timing", "[", 
      RowBox[{
        RowBox[{"a2", "=", 
          RowBox[{"Apart", "[", 
            RowBox[{"t", ",", "x"}], "]"}]}], ";"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"2.45999999999999374`", " ", "Second"}], ",", "Null"}], 
        "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"a1", "-", "a2"}], "//", "Expand"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"t", ",", "a1", ",", "a2"}], "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Apart2", "Subsection",
  CellTags->"Apart2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Apart2[expr] partial fractions FeynAmpDenominators (and \
FAD's).\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FAD",
    ButtonData:>"FAD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FAD"],
  ", ",
  ButtonBox["FeynAmpDenominator",
    ButtonData:>"FeynAmpDenominator",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FeynAmpDenominator"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FAD", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"q", ",", "m"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "M"}], "}"}], ",", 
        RowBox[{"q", "-", "p"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          RowBox[{"(", 
            InterpretationBox[
              RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"q", "-", "p"}], ")"}], "2"], 
                "\[InvisibleSpace]", "\<\"]\"\>"}],
              SequenceForm[ "[", 
                Power[ 
                  Plus[ 
                    Times[ -1, p], q], 2], "]"],
              Editable->False], ")"}], " ", 
          RowBox[{"(", 
            InterpretationBox[
              RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                SuperscriptBox["q", "2"], "\[InvisibleSpace]", "\<\"-\"\>", 
                "\[InvisibleSpace]", 
                SuperscriptBox["m", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
              SequenceForm[ "[", 
                Power[ q, 2], "-", 
                Power[ m, 2], "]"],
              Editable->False], ")"}], " ", 
          RowBox[{"(", 
            InterpretationBox[
              RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                SuperscriptBox["q", "2"], "\[InvisibleSpace]", "\<\"-\"\>", 
                "\[InvisibleSpace]", 
                SuperscriptBox["M", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
              SequenceForm[ "[", 
                Power[ q, 2], "-", 
                Power[ M, 2], "]"],
              Editable->False], ")"}]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Apart2", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          FractionBox["1", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["q", "2"], "-", 
                  SuperscriptBox["m", "2"]}], ")"}], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
          FractionBox["1", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["q", "2"], "-", 
                  SuperscriptBox["M", "2"]}], ")"}], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"q", "-", "p"}], ")"}], "2"]}]]}], 
        RowBox[{
          SuperscriptBox["m", "2"], "-", 
          SuperscriptBox["M", "2"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"FCE", "[", "%", "]"}], "]"}]], "Input"],

Cell[BoxData[
    FractionBox[
      RowBox[{
        RowBox[{"FAD", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"q", ",", "m"}], "}"}], ",", 
            RowBox[{
              RowBox[{"-", "p"}], "+", "q"}]}], "]"}], "-", 
        RowBox[{"FAD", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"q", ",", "M"}], "}"}], ",", 
            RowBox[{
              RowBox[{"-", "p"}], "+", "q"}]}], "]"}]}], 
      RowBox[{
        SuperscriptBox["m", "2"], "-", 
        SuperscriptBox["M", "2"]}]]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Apart3", "Subsection",
  CellTags->"Apart3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Apart3[expr, x] is equivalent to Map2[Factor2, \
Collect2[Apart1[expr,x],x]].\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Apart1",
    ButtonData:>"Apart1",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Apart1"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{"t", "=", 
        RowBox[{
          RowBox[{
            RowBox[{"1", "/", "4"}], " ", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", "x", " ", 
            RowBox[{"(", 
              RowBox[{"21", "+", 
                RowBox[{"13", " ", "x"}], "+", 
                RowBox[{"3", " ", 
                  RowBox[{"x", "^", "2"}]}]}], ")"}]}], "-", 
          RowBox[{
            RowBox[{"11", "/", "24"}], " ", "I", " ", "\[Pi]", " ", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", "x", " ", 
            RowBox[{"(", 
              RowBox[{"21", "+", 
                RowBox[{"13", " ", "x"}], "+", 
                RowBox[{"3", " ", 
                  RowBox[{"x", "^", "2"}]}]}], ")"}]}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", 
                RowBox[{"x", "^", "2"}], " ", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", " ", "x"}], "+", 
                    RowBox[{"3", " ", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"36", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "-", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"I", " ", "\[Pi]", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", 
                RowBox[{"x", "^", "2"}], " ", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", " ", "x"}], "+", 
                    RowBox[{"3", " ", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"12", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "-", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"7", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", 
                RowBox[{"x", "^", "3"}], " ", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", " ", "x"}], "+", 
                    RowBox[{"3", " ", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"48", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"3", " ", "I", " ", "\[Pi]", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", 
                RowBox[{"x", "^", "3"}], " ", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", " ", "x"}], "+", 
                    RowBox[{"3", " ", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"8", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"11", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", 
                RowBox[{"x", "^", "4"}], " ", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", " ", "x"}], "+", 
                    RowBox[{"3", " ", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"24", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "-", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"3", " ", "I", " ", "\[Pi]", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", 
                RowBox[{"x", "^", "4"}], " ", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", " ", "x"}], "+", 
                    RowBox[{"3", " ", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"4", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "-", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"49", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", 
                RowBox[{"x", "^", "5"}], " ", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", " ", "x"}], "+", 
                    RowBox[{"3", " ", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"144", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"11", " ", "I", " ", "\[Pi]", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", 
                RowBox[{"x", "^", "5"}], " ", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", " ", "x"}], "+", 
                    RowBox[{"3", " ", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"24", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "+", 
          RowBox[{
            RowBox[{"1", "/", "4"}], " ", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", "x", " ", 
            RowBox[{"(", 
              RowBox[{"21", "+", 
                RowBox[{"13", " ", "x"}], "+", 
                RowBox[{"3", " ", 
                  RowBox[{"x", "^", "2"}]}]}], ")"}], " ", "Zeta2"}], "+", 
          RowBox[{
            RowBox[{"11", "/", "24"}], " ", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", "x", " ", 
            RowBox[{"(", 
              RowBox[{"21", "+", 
                RowBox[{"13", " ", "x"}], "+", 
                RowBox[{"3", " ", 
                  RowBox[{"x", "^", "2"}]}]}], ")"}], " ", 
            RowBox[{"Log", "[", "x", "]"}]}], "+", 
          RowBox[{
            RowBox[{"1", "/", "4"}], " ", "I", " ", "\[Pi]", " ", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", "x", " ", 
            RowBox[{"(", 
              RowBox[{"21", "+", 
                RowBox[{"13", " ", "x"}], "+", 
                RowBox[{"3", " ", 
                  RowBox[{"x", "^", "2"}]}]}], ")"}], " ", 
            RowBox[{"Log", "[", "x", "]"}]}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", 
                RowBox[{"x", "^", "2"}], " ", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", " ", "x"}], "+", 
                    RowBox[{"3", " ", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}], " ", 
                RowBox[{"Log", "[", "x", "]"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"12", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "-", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"3", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", 
                RowBox[{"x", "^", "3"}], " ", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", " ", "x"}], "+", 
                    RowBox[{"3", " ", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}], " ", 
                RowBox[{"Log", "[", "x", "]"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"8", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"3", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], "^", "4"}], " ", 
                RowBox[{"x", "^", "4"}], " ", 
                RowBox[{"(", 
                  RowBox[{"21", "+", 
                    RowBox[{"13", " ", "x"}], "+", 
                    RowBox[{"3", " ", 
                      RowBox[{"x", "^", "2"}]}]}], ")"}], " ", 
                RowBox[{"Log", "[", "x", "]"}]}], ")"}], "/", 
            RowBox[{"(", 
              RowBox[{"4", " ", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "x"}], ")"}], "^", "4"}]}], 
              ")"}]}]}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Timing", "[", 
      RowBox[{
        RowBox[{"a1", "=", 
          RowBox[{"Apart3", "[", 
            RowBox[{"t", ",", "x"}], "]"}]}], ";"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"0.760000000000005115`", " ", "Second"}], ",", "Null"}], 
        "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Timing", "[", 
      RowBox[{
        RowBox[{"a2", "=", 
          RowBox[{"Apart", "[", 
            RowBox[{"t", ",", "x"}], "]"}]}], ";"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"2.46000000000000795`", " ", "Second"}], ",", "Null"}], 
        "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"a1", "-", "a2"}], "//", "Expand"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"t", ",", "a1", ",", "a2"}], "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["B00", "Subsection",
  CellTags->"B00"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
B00[pp, ma^2,mb^2] is the Passarino-Veltman B00-function, i.e., the \
coefficient function of the metric tensor. All arguments are scalars and have \
dimension mass^2.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "B00", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"BReduce", "\[Rule]", "True"}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["B0",
    ButtonData:>"B0",
    ButtonStyle->"Hyperlink",
    ButtonNote->"B0"],
  ", ",
  ButtonBox["B1",
    ButtonData:>"B1",
    ButtonStyle->"Hyperlink",
    ButtonNote->"B1"],
  ", ",
  ButtonBox["PaVe",
    ButtonData:>"PaVe",
    ButtonStyle->"Hyperlink",
    ButtonNote->"PaVe"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[TextData[{
  "Remember that SP[p] is a short hand input for ScalarProduct[p,p], i.e., ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["p", "2"], TraditionalForm]]],
  "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SP", "[", "p", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        FormBox["p",
          "TraditionalForm"], "2"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B00", "[", 
      RowBox[{
        RowBox[{"SP", "[", "p", "]"}], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "3"], " ", 
          RowBox[{
            SubscriptBox["B", "0"], "(", 
            FormBox[
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"],
              "TraditionalForm"], ",", 
            SuperscriptBox["m", "2"], ",", 
            SuperscriptBox["M", "2"], ")"}], " ", 
          SuperscriptBox["m", "2"]}], "+", 
        RowBox[{
          FractionBox["1", "18"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"3", " ", 
                SuperscriptBox["m", "2"]}], "+", 
              RowBox[{"3", " ", 
                SuperscriptBox["M", "2"]}], "-", 
              FormBox[
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"],
                "TraditionalForm"]}], ")"}]}], "+", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{
                  SubscriptBox["B", "0"], "(", "0", ",", 
                  SuperscriptBox["M", "2"], ",", 
                  SuperscriptBox["M", "2"], ")"}], " ", 
                SuperscriptBox["M", "2"]}], "+", 
              SuperscriptBox["M", "2"], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    FractionBox[
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            SuperscriptBox["M", "2"], "-", 
                            SuperscriptBox["m", "2"]}], ")"}], " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              SubscriptBox["B", "0"], "(", 
                              FormBox[
                                SuperscriptBox[
                                  FormBox["p",
                                    "TraditionalForm"], "2"],
                                "TraditionalForm"], ",", 
                              SuperscriptBox["m", "2"], ",", 
                              SuperscriptBox["M", "2"], ")"}], "-", 
                            RowBox[{
                              SubscriptBox["B", "0"], "(", "0", ",", 
                              SuperscriptBox["m", "2"], ",", 
                              SuperscriptBox["M", "2"], ")"}]}], ")"}]}], 
                      RowBox[{"2", " ", 
                        FormBox[
                          SuperscriptBox[
                            FormBox["p",
                              "TraditionalForm"], "2"],
                          "TraditionalForm"]}]], "-", 
                    RowBox[{
                      FractionBox["1", "2"], " ", 
                      RowBox[{
                        SubscriptBox["B", "0"], "(", 
                        FormBox[
                          SuperscriptBox[
                            FormBox["p",
                              "TraditionalForm"], "2"],
                          "TraditionalForm"], ",", 
                        SuperscriptBox["m", "2"], ",", 
                        SuperscriptBox["M", "2"], ")"}]}]}], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox["m", "2"], "-", 
                    SuperscriptBox["M", "2"], "+", 
                    FormBox[
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], "2"],
                      "TraditionalForm"]}], ")"}]}]}], ")"}]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B00", "[", 
      RowBox[{
        RowBox[{"SP", "[", "p", "]"}], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "3"], " ", 
          RowBox[{
            SubscriptBox["B", "0"], "(", 
            FormBox[
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"],
              "TraditionalForm"], ",", 
            SuperscriptBox["m", "2"], ",", 
            SuperscriptBox["m", "2"], ")"}], " ", 
          SuperscriptBox["m", "2"]}], "+", 
        RowBox[{
          FractionBox["1", "18"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"6", " ", 
                SuperscriptBox["m", "2"]}], "-", 
              FormBox[
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"],
                "TraditionalForm"]}], ")"}]}], "+", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{
                  SubscriptBox["B", "0"], "(", "0", ",", 
                  SuperscriptBox["m", "2"], ",", 
                  SuperscriptBox["m", "2"], ")"}], " ", 
                SuperscriptBox["m", "2"]}], "+", 
              SuperscriptBox["m", "2"], "-", 
              RowBox[{
                FractionBox["1", "2"], " ", 
                RowBox[{
                  SubscriptBox["B", "0"], "(", 
                  FormBox[
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"],
                    "TraditionalForm"], ",", 
                  SuperscriptBox["m", "2"], ",", 
                  SuperscriptBox["m", "2"], ")"}], " ", 
                FormBox[
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"],
                  "TraditionalForm"]}]}], ")"}]}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B00", "[", 
      RowBox[{
        RowBox[{"SP", "[", "p", "]"}], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ",", 
        RowBox[{"BReduce", "\[Rule]", "False"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          SubscriptBox["\<\"B\"\>", "\<\"00\"\>"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          FormBox[
            SuperscriptBox[
              FormBox["p",
                "TraditionalForm"], "2"],
            "TraditionalForm"],
          "TraditionalForm"], "\[NoBreak]", 
        RowBox[{",", " "}], "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m", "2"],
          "TraditionalForm"], "\[NoBreak]", 
        RowBox[{",", " "}], "\[NoBreak]", 
        FormBox[
          SuperscriptBox["M", "2"],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B00", "[", 
      RowBox[{"0", ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "3"], " ", 
          RowBox[{
            SubscriptBox["B", "0"], "(", "0", ",", 
            SuperscriptBox["m", "2"], ",", 
            SuperscriptBox["m", "2"], ")"}], " ", 
          SuperscriptBox["m", "2"]}], "+", 
        FractionBox[
          SuperscriptBox["m", "2"], "3"], "+", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{
                  SubscriptBox["B", "0"], "(", "0", ",", 
                  SuperscriptBox["m", "2"], ",", 
                  SuperscriptBox["m", "2"], ")"}], " ", 
                SuperscriptBox["m", "2"]}], "+", 
              SuperscriptBox["m", "2"]}], ")"}]}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B00", "[", 
      RowBox[{
        RowBox[{"SmallVariable", "[", 
          SuperscriptBox["M", "2"], "]"}], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "3"], " ", 
          RowBox[{
            SubscriptBox["B", "0"], "(", 
            SuperscriptBox["M", "2"], ",", 
            SuperscriptBox["m", "2"], ",", 
            SuperscriptBox["m", "2"], ")"}], " ", 
          SuperscriptBox["m", "2"]}], "+", 
        FractionBox[
          SuperscriptBox["m", "2"], "3"], "+", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{
                  SubscriptBox["B", "0"], "(", "0", ",", 
                  SuperscriptBox["m", "2"], ",", 
                  SuperscriptBox["m", "2"], ")"}], " ", 
                SuperscriptBox["m", "2"]}], "+", 
              SuperscriptBox["m", "2"]}], ")"}]}]}], TraditionalForm]], 
  "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["B0", "Subsection",
  CellTags->"B0"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
B0[pp, ma^2, mb^2] is the Passarino - Veltman two-point integral \
B0. All arguments are scalars and have dimension mass^2. If the option \
BReduce is set to True certain B0's are reduced to A0's. Setting the option \
B0Unique to True simplifies B0[a,0,a] and B0[0,0,a].\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "B0", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"BReduce", "\[Rule]", "False"}], ",", 
          RowBox[{"B0Unique", "\[Rule]", "False"}], ",", 
          RowBox[{"B0Real", "\[Rule]", "False"}]}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["B1",
    ButtonData:>"B1",
    ButtonStyle->"Hyperlink",
    ButtonNote->"B1"],
  ", ",
  ButtonBox["B00",
    ButtonData:>"B00",
    ButtonStyle->"Hyperlink",
    ButtonNote->"B00"],
  ", ",
  ButtonBox["B11",
    ButtonData:>"B11",
    ButtonStyle->"Hyperlink",
    ButtonNote->"B11"],
  ", ",
  ButtonBox["PaVe",
    ButtonData:>"PaVe",
    ButtonStyle->"Hyperlink",
    ButtonNote->"PaVe"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B0", "[", 
      RowBox[{
        RowBox[{"SP", "[", 
          RowBox[{"p", ",", "p"}], "]"}], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["B", "0"], "(", 
        FormBox[
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"],
          "TraditionalForm"], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"B0", ",", 
          RowBox[{"B0Unique", "\[Rule]", "True"}], ",", 
          RowBox[{"B0Real", "\[Rule]", "True"}]}], "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B0", "[", 
      RowBox[{"0", ",", "0", ",", 
        SuperscriptBox["m", "2"]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["B", "0"], "(", "0", ",", 
          SuperscriptBox["m", "2"], ",", 
          SuperscriptBox["m", "2"], ")"}], "+", "1"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B0", "[", 
      RowBox[{
        SuperscriptBox["m", "2"], ",", "0", ",", 
        SuperscriptBox["m", "2"]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["B", "0"], "(", "0", ",", 
          SuperscriptBox["m", "2"], ",", 
          SuperscriptBox["m", "2"], ")"}], "+", "2"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B0", "[", 
      RowBox[{"0", ",", 
        RowBox[{"m", "^", "2"}], ",", 
        RowBox[{"m", "^", "2"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"], ")"}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["B11", "Subsection",
  CellTags->"B11"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "B11[pp, ma^2, mb^2] is the Passarino - Veltman ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["B", "11"], TraditionalForm]]],
  "-function, i.e., the coefficient function of ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox["p", "\[Mu]"], " ", 
          SuperscriptBox["p", "\[Nu]"]}], TraditionalForm]]],
  ". All arguments are scalars and have dimension mass",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Null]", "2"], TraditionalForm]]],
  "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "B11", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"BReduce", "\[Rule]", "True"}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["B0",
    ButtonData:>"B0",
    ButtonStyle->"Hyperlink",
    ButtonNote->"B0"],
  ", ",
  ButtonBox["B00",
    ButtonData:>"B00",
    ButtonStyle->"Hyperlink",
    ButtonNote->"B00"],
  ", ",
  ButtonBox["B1",
    ButtonData:>"B1",
    ButtonStyle->"Hyperlink",
    ButtonNote->"B1"],
  ", ",
  ButtonBox["PaVe",
    ButtonData:>"PaVe",
    ButtonStyle->"Hyperlink",
    ButtonNote->"PaVe"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[TextData[{
  "Remember that SP[p] is a short hand input for ScalarProduct[p,p], i.e. ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox["p", "2"], "."}], TraditionalForm]]]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SP", "[", "p", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        FormBox["p",
          "TraditionalForm"], "2"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B11", "[", 
      RowBox[{
        RowBox[{"SP", "[", "p", "]"}], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", 
          RowBox[{"3", " ", 
            FormBox[
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"],
              "TraditionalForm"]}]], 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", 
                RowBox[{
                  SubscriptBox["B", "0"], "(", 
                  FormBox[
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"],
                    "TraditionalForm"], ",", 
                  SuperscriptBox["m", "2"], ",", 
                  SuperscriptBox["M", "2"], ")"}]}], " ", 
              SuperscriptBox["m", "2"]}], "+", 
            SuperscriptBox["M", "2"], "+", 
            RowBox[{
              SuperscriptBox["M", "2"], " ", 
              RowBox[{
                SubscriptBox["B", "0"], "(", "0", ",", 
                SuperscriptBox["M", "2"], ",", 
                SuperscriptBox["M", "2"], ")"}]}], "+", 
            RowBox[{
              FractionBox["1", "6"], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    RowBox[{"-", "3"}], " ", 
                    SuperscriptBox["m", "2"]}], "-", 
                  RowBox[{"3", " ", 
                    SuperscriptBox["M", "2"]}], "+", 
                  FormBox[
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"],
                    "TraditionalForm"]}], ")"}]}], "-", 
            RowBox[{"2", " ", 
              RowBox[{"(", 
                RowBox[{
                  FractionBox[
                    RowBox[{
                      RowBox[{"(", 
                        RowBox[{
                          SuperscriptBox["M", "2"], "-", 
                          SuperscriptBox["m", "2"]}], ")"}], " ", 
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{
                            SubscriptBox["B", "0"], "(", 
                            FormBox[
                              SuperscriptBox[
                                FormBox["p",
                                  "TraditionalForm"], "2"],
                              "TraditionalForm"], ",", 
                            SuperscriptBox["m", "2"], ",", 
                            SuperscriptBox["M", "2"], ")"}], "-", 
                          RowBox[{
                            SubscriptBox["B", "0"], "(", "0", ",", 
                            SuperscriptBox["m", "2"], ",", 
                            SuperscriptBox["M", "2"], ")"}]}], ")"}]}], 
                    RowBox[{"2", " ", 
                      FormBox[
                        SuperscriptBox[
                          FormBox["p",
                            "TraditionalForm"], "2"],
                        "TraditionalForm"]}]], "-", 
                  RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{
                      SubscriptBox["B", "0"], "(", 
                      FormBox[
                        SuperscriptBox[
                          FormBox["p",
                            "TraditionalForm"], "2"],
                        "TraditionalForm"], ",", 
                      SuperscriptBox["m", "2"], ",", 
                      SuperscriptBox["M", "2"], ")"}]}]}], ")"}], " ", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["m", "2"], "-", 
                  SuperscriptBox["M", "2"], "+", 
                  FormBox[
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"],
                    "TraditionalForm"]}], ")"}]}]}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B11", "[", 
      RowBox[{
        RowBox[{"SP", "[", "p", "]"}], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          RowBox[{
            RowBox[{
              SubscriptBox["B", "0"], "(", "0", ",", 
              SuperscriptBox["m", "2"], ",", 
              SuperscriptBox["m", "2"], ")"}], " ", 
            SuperscriptBox["m", "2"]}], "+", 
          SuperscriptBox["m", "2"], "+", 
          RowBox[{
            FractionBox["1", "6"], " ", 
            RowBox[{"(", 
              RowBox[{
                FormBox[
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"],
                  "TraditionalForm"], "-", 
                RowBox[{"6", " ", 
                  SuperscriptBox["m", "2"]}]}], ")"}]}], "+", 
          RowBox[{
            RowBox[{
              SubscriptBox["B", "0"], "(", 
              FormBox[
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"],
                "TraditionalForm"], ",", 
              SuperscriptBox["m", "2"], ",", 
              SuperscriptBox["m", "2"], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{
                FormBox[
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"],
                  "TraditionalForm"], "-", 
                SuperscriptBox["m", "2"]}], ")"}]}]}], 
        RowBox[{"3", " ", 
          FormBox[
            SuperscriptBox[
              FormBox["p",
                "TraditionalForm"], "2"],
            "TraditionalForm"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SetOptions", "[", 
      RowBox[{"B11", ",", 
        RowBox[{"BReduce", "\[Rule]", "False"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"BReduce", "\[Rule]", "False"}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B11", "[", 
      RowBox[{
        RowBox[{"SP", "[", "p", "]"}], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          SubscriptBox["\<\"B\"\>", "\<\"11\"\>"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          FormBox[
            SuperscriptBox[
              FormBox["p",
                "TraditionalForm"], "2"],
            "TraditionalForm"],
          "TraditionalForm"], "\[NoBreak]", 
        RowBox[{",", " "}], "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m", "2"],
          "TraditionalForm"], "\[NoBreak]", 
        RowBox[{",", " "}], "\[NoBreak]", 
        FormBox[
          SuperscriptBox["M", "2"],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SetOptions", "[", 
      RowBox[{"B11", ",", 
        RowBox[{"BReduce", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"BReduce", "\[Rule]", "True"}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B11", "[", 
      RowBox[{"0", ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "3"], " ", 
        RowBox[{
          SubscriptBox["B", "0"], "(", "0", ",", 
          SuperscriptBox["m", "2"], ",", 
          SuperscriptBox["m", "2"], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B11", "[", 
      RowBox[{
        RowBox[{"SmallVariable", "[", 
          SuperscriptBox["M", "2"], "]"}], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "3"], " ", 
        RowBox[{
          SubscriptBox["B", "0"], "(", 
          SuperscriptBox["M", "2"], ",", 
          SuperscriptBox["m", "2"], ",", 
          SuperscriptBox["m", "2"], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["B1", "Subsection",
  CellTags->"B1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "B1[pp, ma^2, mb^2] the Passarino - Veltman ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["B", "1"], TraditionalForm]]],
  "-function. All arguments are scalars and have dimension mass^2."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "B1", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"BReduce", "\[Rule]", "True"}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["B0",
    ButtonData:>"B0",
    ButtonStyle->"Hyperlink",
    ButtonNote->"B0"],
  ", ",
  ButtonBox["B00",
    ButtonData:>"B00",
    ButtonStyle->"Hyperlink",
    ButtonNote->"B00"],
  ", ",
  ButtonBox["B11",
    ButtonData:>"B11",
    ButtonStyle->"Hyperlink",
    ButtonNote->"B11"],
  ", ",
  ButtonBox["PaVe",
    ButtonData:>"PaVe",
    ButtonStyle->"Hyperlink",
    ButtonNote->"PaVe"],
  ", ",
  ButtonBox["PaVeReduce",
    ButtonData:>"PaVeReduce",
    ButtonStyle->"Hyperlink",
    ButtonNote->"PaVeReduce"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B1", "[", 
      RowBox[{
        RowBox[{"SP", "[", "p", "]"}], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox["M", "2"], "-", 
                SuperscriptBox["m", "2"]}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  SubscriptBox["B", "0"], "(", 
                  FormBox[
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"],
                    "TraditionalForm"], ",", 
                  SuperscriptBox["m", "2"], ",", 
                  SuperscriptBox["M", "2"], ")"}], "-", 
                RowBox[{
                  SubscriptBox["B", "0"], "(", "0", ",", 
                  SuperscriptBox["m", "2"], ",", 
                  SuperscriptBox["M", "2"], ")"}]}], ")"}]}], 
          RowBox[{"2", " ", 
            FormBox[
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"],
              "TraditionalForm"]}]], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{
            SubscriptBox["B", "0"], "(", 
            FormBox[
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"],
              "TraditionalForm"], ",", 
            SuperscriptBox["m", "2"], ",", 
            SuperscriptBox["M", "2"], ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B1", "[", 
      RowBox[{
        RowBox[{"SP", "[", "p", "]"}], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ",", 
        RowBox[{"BReduce", "\[Rule]", "False"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["B", "1"], "(", 
        FormBox[
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"],
          "TraditionalForm"], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"B1", ",", 
          RowBox[{"BReduce", "\[Rule]", "True"}]}], "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B1", "[", 
      RowBox[{
        RowBox[{"SP", "[", "p", "]"}], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "2"]}], " ", 
        RowBox[{
          SubscriptBox["B", "0"], "(", 
          FormBox[
            SuperscriptBox[
              FormBox["p",
                "TraditionalForm"], "2"],
            "TraditionalForm"], ",", 
          SuperscriptBox["m", "2"], ",", 
          SuperscriptBox["m", "2"], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B1", "[", 
      RowBox[{
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"], ",", "0"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"-", 
                RowBox[{
                  SubscriptBox["B", "0"], "(", "0", ",", 
                  SuperscriptBox["m", "2"], ",", 
                  SuperscriptBox["m", "2"], ")"}]}], "-", "2"}], ")"}]}], 
        "-", 
        FractionBox["1", "2"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B1", "[", 
      RowBox[{"0", ",", "0", ",", 
        SuperscriptBox["m", "2"]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"-", 
                RowBox[{
                  SubscriptBox["B", "0"], "(", "0", ",", 
                  SuperscriptBox["m", "2"], ",", 
                  SuperscriptBox["m", "2"], ")"}]}], "-", "1"}], ")"}]}], 
        "+", 
        FractionBox["1", "4"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B1", "[", 
      RowBox[{"pp", ",", 
        RowBox[{"SmallVariable", "[", 
          SubsuperscriptBox["m", "e", "2"], "]"}], ",", 
        SubsuperscriptBox["m", "2", "2"]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  SubscriptBox["B", "0"], "(", "pp", ",", 
                  SubsuperscriptBox["m", "e", "2"], ",", 
                  SubsuperscriptBox["m", "2", "2"], ")"}], "-", 
                RowBox[{
                  SubscriptBox["B", "0"], "(", "0", ",", 
                  SubsuperscriptBox["m", "e", "2"], ",", 
                  SubsuperscriptBox["m", "2", "2"], ")"}]}], ")"}], " ", 
            SubsuperscriptBox["m", "2", "2"]}], 
          RowBox[{"2", " ", "pp"}]], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{
            SubscriptBox["B", "0"], "(", "pp", ",", 
            SubsuperscriptBox["m", "e", "2"], ",", 
            SubsuperscriptBox["m", "2", "2"], ")"}]}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"B1", "[", 
      RowBox[{
        RowBox[{"SmallVariable", "[", 
          SubsuperscriptBox["m", "e", "2"], "]"}], ",", 
        RowBox[{"SmallVariable", "[", 
          SubsuperscriptBox["m", "e", "2"], "]"}], ",", "0"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"-", 
                RowBox[{
                  SubscriptBox["B", "0"], "(", "0", ",", 
                  SubsuperscriptBox["m", "e", "2"], ",", 
                  SubsuperscriptBox["m", "e", "2"], ")"}]}], "-", "2"}], 
            ")"}]}], "-", 
        FractionBox["1", "2"]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["BackgroundGluonVertex", "Subsection",
  CellTags->"BackgroundGluonVertex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "BackgroundGluonVertex[{p,mu,a}, {q,nu,b}, {k,la,c}] or \
BackgroundGluonVertex[ p,mu,a , q,nu,b , k,la,c ] yields the 3-gluon vertex \
in the background field gauge, where the first set of arguments corresponds \
to the external background field. BackgroundGluonVertex[{p,mu,a}, {q,nu,b}, \
{k,la,c}, {s,si,d}] or BackgroundGluonVertex[{mu,a}, {nu,b}, {la,c}, {",
  "si",
  ",d}] or BackgroundGluonVertex[p,mu,a , q,nu,b , k,la,c , s,si,d] or \
BackgroundGluonVertex[ mu,a , nu,b , la,c , si,d ] yields the 4-gluon vertex, \
with {p,mu,a} and {k,la,c} denoting the external background fields. The \
gauge, dimension and the name of the coupling constant are determined by the \
options Gauge, Dimension and CouplingConstant. The Feynman rules are taken \
from L.Abbot ",
  StyleBox["NPB ",
    FontWeight->"Bold"],
  "185 (1981), 189-203; except that all momenta are incoming. Note that \
Abbots coupling constant convention is consistent with the default setting of \
GluonVertex."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "BackgroundGluonVertex", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Gauge", "\[Rule]", "1"}]}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["GluonVertex",
    ButtonData:>"GluonVertex",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GluonVertex"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"BackgroundGluonVertex", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"k", ",", "\[Lambda]", ",", "c"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        RowBox[{"-", "k"}], "+", "p", "-", "q"}],
                      "TraditionalForm"], ")"}], 
                  FormBox[
                    FormBox["\[Lambda]",
                      "TraditionalForm"],
                    "TraditionalForm"]], ")"}], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Lambda]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{"q", "-", "k"}],
                      "TraditionalForm"], ")"}], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], ")"}]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Lambda]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{"k", "-", "p", "+", "q"}],
                      "TraditionalForm"], ")"}], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], ")"}]}]}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"BackgroundGluonVertex", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"k", ",", "\[Lambda]", ",", "c"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Lambda]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Lambda]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Lambda]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u6",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u6",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Lambda]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Lambda]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u6",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u6",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Lambda]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Lambda]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "+", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Lambda]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u6",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u6",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}]}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"BackgroundGluonVertex", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"k", ",", "\[Lambda]", ",", "c"}], "}"}], ",", 
        RowBox[{"Gauge", "\[Rule]", "\[Alpha]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{"p", "-", "q"}], ")"}], "-", 
                        FractionBox["k", "\[Alpha]"]}],
                      "TraditionalForm"], ")"}], 
                  FormBox[
                    FormBox["\[Lambda]",
                      "TraditionalForm"],
                    "TraditionalForm"]], ")"}], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Lambda]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{"q", "-", "k"}],
                      "TraditionalForm"], ")"}], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], ")"}]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Lambda]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{"k", "-", "p"}], ")"}], "+", 
                        FractionBox["q", "\[Alpha]"]}],
                      "TraditionalForm"], ")"}], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], ")"}]}]}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"BackgroundGluonVertex", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"k", ",", "\[Lambda]", ",", "c"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}], ",", 
        RowBox[{"Gauge", "\[Rule]", "\[Alpha]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Lambda]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  FractionBox[
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Lambda]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox["\[Nu]",
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Mu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox["\[Sigma]",
                              "TraditionalForm"],
                            "TraditionalForm"]}]]}], "\[Alpha]"], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Lambda]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u7",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u7",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Lambda]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Lambda]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u7",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u7",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  FractionBox[
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Lambda]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox["\[Sigma]",
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Mu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox["\[Nu]",
                              "TraditionalForm"],
                            "TraditionalForm"]}]]}], "\[Alpha]"], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Lambda]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "+", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Lambda]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u7",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u7",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}]}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["C0", "Subsection",
  CellTags->"C0"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "C0[p10, p12, p20, m1^2, m2^2, m3^2] is the scalar Passarino - Veltman ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["C", "0"], TraditionalForm]]],
  "function. The convention for the arguments is that if the denominator of \
the integrand has the form ([q^2-m1^2] [(q+p1)^2-m2^2] [(q+p2)^2-m3^2]), the \
first three arguments of C0 are the scalar products p10 = p1^2, p12 = \
(p1-p2).(p1-p2), p20 = p2^2."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["B0",
    ButtonData:>"B0",
    ButtonStyle->"Hyperlink",
    ButtonNote->"B0"],
  ", ",
  ButtonBox["D0",
    ButtonData:>"D0",
    ButtonStyle->"Hyperlink",
    ButtonNote->"D0"],
  ", ",
  ButtonBox["PaVe",
    ButtonData:>"PaVe",
    ButtonStyle->"Hyperlink",
    ButtonNote->"PaVe"],
  ", ",
  ButtonBox["PaVeOrder",
    ButtonData:>"PaVeOrder",
    ButtonStyle->"Hyperlink",
    ButtonNote->"PaVeOrder"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"C0", "[", 
      RowBox[{
      "a", ",", "b", ",", "c", ",", " ", "m12", ",", "m22", ",", "m32"}], 
      "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "a", 
        "\[NoBreak]", ",", "\[NoBreak]", "b", "\[NoBreak]", ",", 
        "\[NoBreak]", "c", "\[NoBreak]", ",", "\[NoBreak]", "m12", 
        "\[NoBreak]", ",", "\[NoBreak]", "m22", "\[NoBreak]", ",", 
        "\[NoBreak]", "m32", "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"C0", "[", 
        RowBox[{
        "b", ",", "a", ",", "c", ",", "m32", ",", "m22", ",", "m12"}], "]"}], 
      "//", "PaVeOrder"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "a", 
        "\[NoBreak]", ",", "\[NoBreak]", "b", "\[NoBreak]", ",", 
        "\[NoBreak]", "c", "\[NoBreak]", ",", "\[NoBreak]", "m12", 
        "\[NoBreak]", ",", "\[NoBreak]", "m22", "\[NoBreak]", ",", 
        "\[NoBreak]", "m32", "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PaVeOrder", "[", 
      RowBox[{
        RowBox[{"C0", "[", 
          RowBox[{
          "b", ",", "a", ",", "c", ",", "m32", ",", "m22", ",", "m12"}], 
          "]"}], ",", 
        RowBox[{"PaVeOrderList", "->", 
          RowBox[{"{", 
            RowBox[{"c", ",", "a"}], "}"}]}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "b", 
        "\[NoBreak]", ",", "\[NoBreak]", "c", "\[NoBreak]", ",", 
        "\[NoBreak]", "a", "\[NoBreak]", ",", "\[NoBreak]", "m22", 
        "\[NoBreak]", ",", "\[NoBreak]", "m32", "\[NoBreak]", ",", 
        "\[NoBreak]", "m12", "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Calc", "Subsection",
  CellTags->"Calc"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Calc[exp] performs several basic simplifications. Calc[exp] is the \
same as DotSimplify[DiracSimplify[Contract[DiracSimplify[Explicit[ \
SUNSimplify[Trick[exp], Explicit->False] ]]]]]\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracSimplify"],
  ", ",
  ButtonBox["DiracTrick",
    ButtonData:>"DiracTrick",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracTrick"],
  ", ",
  ButtonBox["Trick",
    ButtonData:>"Trick",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Trick"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection"],

Cell[TextData[{
  "This calculates ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Mu]"], 
          SubscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]]],
  " in 4 dimensions and ",
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["g", "\[Nu]", "\[Nu]"], TraditionalForm]]],
  " in D dimensions."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{"GA", "[", 
            RowBox[{"\[Mu]", ",", "\[Mu]"}], "]"}], ",", " ", 
          RowBox[{"MTD", "[", 
            RowBox[{"\[Nu]", ",", "\[Nu]"}], "]"}]}], "}"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"4", ",", "D"}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "This simplifies ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["f", "abc"], " ", 
          RowBox[{
            SubscriptBox["f", "abe"], "."}]}], TraditionalForm]]]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", 
      RowBox[{
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "e"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "A"], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FV", "[", 
        RowBox[{
          RowBox[{"p", "+", "r"}], ",", "\[Mu]"}], "]"}], " ", 
      RowBox[{"MT", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
      RowBox[{"FV", "[", 
        RowBox[{
          RowBox[{"q", "-", "p"}], ",", "\[Nu]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          SuperscriptBox[
            RowBox[{"(", 
              FormBox[
                RowBox[{"q", "-", "p"}],
                "TraditionalForm"], ")"}], "\[Nu]"], ")"}], " ", 
        RowBox[{"(", 
          SuperscriptBox[
            RowBox[{"(", 
              FormBox[
                RowBox[{"p", "+", "r"}],
                "TraditionalForm"], ")"}], "\[Mu]"], ")"}], " ", 
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"]}], "+", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}], "-", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["r",
            "TraditionalForm"]}], "+", 
        RowBox[{
          FormBox["q",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["r",
            "TraditionalForm"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonVertex", "[", 
      RowBox[{"p", ",", "1", ",", "q", ",", "2", ",", 
        RowBox[{
          RowBox[{"-", "p"}], "-", "q"}], ",", "3"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        RowBox[{
          SuperscriptBox["V", 
            RowBox[{
              FormBox[
                FormBox["li1",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["li2",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["li3",
                  "TraditionalForm"],
                "TraditionalForm"]}]], "(", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", 
            RowBox[{",", " "}], "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"], "\[NoBreak]", 
            RowBox[{",", " "}], "\[NoBreak]", 
            FormBox[
              RowBox[{
                RowBox[{"-", "p"}], "-", "q"}],
              "TraditionalForm"]}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["ci1",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["ci2",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["ci3",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", 
      RowBox[{
        RowBox[{"%", " ", 
          RowBox[{"FVD", "[", 
            RowBox[{"p", ",", "li1"}], "]"}]}], " ", 
        RowBox[{"FVD", "[", 
          RowBox[{"q", ",", "li2"}], "]"}], " ", 
        RowBox[{"FVD", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"-", "p"}], "-", "q"}], ",", "li3"}], "]"}]}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["CA", "Subsection",
  CellTags->"CA"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["CA is one of the Casimir operators of SU(N) (CA = N).", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["CF",
    ButtonData:>"CF",
    ButtonStyle->"Hyperlink",
    ButtonNote->"CF"],
  ", ",
  ButtonBox["SUNSimplify",
    ButtonData:>"SUNSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SUNSimplify"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData["CA"], "Input"],

Cell[BoxData[
    FormBox[
      SubscriptBox["C", "A"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{"CA", ",", 
        RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["N", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["SUNN"], "Input"],

Cell[BoxData[
    FormBox["N", TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Cases2", "Subsection",
  CellTags->"Cases2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Cases2[expr, f] returns a list of all objects in expr with head f. \
Cases2[expr,f] is equivalent to Cases2[{expr},f[___],Infinity]//Union. \
Cases2[expr, f, g, ...] or Cases2[expr, {f,g, ...}] is equivalent to \
Cases[{expr},f[___] | g[___] ...] .\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Cases2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"f", "[", "a", "]"}], "+", 
          SuperscriptBox[
            RowBox[{"f", "[", "b", "]"}], "2"], "+", 
          RowBox[{"f", "[", 
            RowBox[{"c", ",", "d"}], "]"}]}], ",", "f"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"f", "(", "a", ")"}], ",", 
          RowBox[{"f", "(", "b", ")"}], ",", 
          RowBox[{"f", "(", 
            RowBox[{"c", ",", "d"}], ")"}]}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Cases2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"Sin", "[", "x", "]"}], " ", 
            RowBox[{"Sin", "[", 
              RowBox[{"y", "-", "z"}], "]"}]}], "+", 
          RowBox[{"g", "[", "y", "]"}]}], ",", "Sin", ",", "g"}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"g", "(", "y", ")"}], ",", 
          RowBox[{"sin", "(", "x", ")"}], ",", 
          RowBox[{"sin", "(", 
            RowBox[{"y", "-", "z"}], ")"}]}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Cases2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"Sin", "[", "x", "]"}], " ", 
            RowBox[{"Sin", "[", 
              RowBox[{"y", "-", "z"}], "]"}]}], "+", 
          RowBox[{"g", "[", "x", "]"}], "+", 
          RowBox[{"g", "[", 
            RowBox[{"a", ",", "b", ",", "c"}], "]"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"Sin", ",", "g"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"g", "(", "x", ")"}], ",", 
          RowBox[{"g", "(", 
            RowBox[{"a", ",", "b", ",", "c"}], ")"}], ",", 
          RowBox[{"sin", "(", "x", ")"}], ",", 
          RowBox[{"sin", "(", 
            RowBox[{"y", "-", "z"}], ")"}]}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Cases2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"DiracSlash", "[", "p", "]"}], ".", 
            RowBox[{"DiracSlash", "[", "q", "]"}]}], "+", 
          RowBox[{"ScalarProduct", "[", 
            RowBox[{"p", ",", "p"}], "]"}]}], ",", "Dot"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["p",
                "TraditionalForm"]}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["q",
                "TraditionalForm"]}], ")"}]}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["CF", "Subsection",
  CellTags->"CF"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
CF is one of the Casimir operators of SU(N) (CF = (N^2-1)/(2 N)).\
\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["CA",
    ButtonData:>"CA",
    ButtonStyle->"Hyperlink",
    ButtonNote->"CA"],
  ", ",
  ButtonBox["SUNSimplify",
    ButtonData:>"SUNSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SUNSimplify"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData["CF"], "Input"],

Cell[BoxData[
    FormBox[
      SubscriptBox["C", "F"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{"CF", ",", " ", 
        RowBox[{"SUNNToCACF", " ", "->", " ", "False"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          SuperscriptBox["N", "2"], "-", "1"}], 
        RowBox[{"2", " ", "N"}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["SUNN"], "Input"],

Cell[BoxData[
    FormBox["N", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{"CF", ",", " ", 
        RowBox[{"SUNNToCACF", " ", "->", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubscriptBox["C", "F"], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["ChangeDimension", "Subsection",
  CellTags->"ChangeDimension"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
ChangeDimension[exp, dim] changes all LorentzIndex and Momenta in \
exp to dimension dim (and also Levi-Civita-tensors, Dirac slashes and Dirac \
matrices).\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["LorentzIndex",
    ButtonData:>"LorentzIndex",
    ButtonStyle->"Hyperlink",
    ButtonNote->"LorentzIndex"],
  ", ",
  ButtonBox["Momentum",
    ButtonData:>"Momentum",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Momentum"],
  ", ",
  ButtonBox["DiracGamma",
    ButtonData:>"DiracGamma",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracGamma"],
  ", ",
  ButtonBox["Eps",
    ButtonData:>"Eps",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Eps"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell["\<\
Remember that LorentzIndex[mu, 4] is simplified to LorentzIndex[mu] \
and Momentum[p, 4] to Momentum[p]. Thus the fullowing objects are defined in \
four dimensions.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", " ", 
        RowBox[{"Momentum", "[", "p", "]"}]}], "}"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          FormBox["\[Mu]",
            "TraditionalForm"], ",", "p"}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ChangeDimension", "[", 
      RowBox[{"%", ",", " ", "D"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          FormBox["\[Mu]",
            "TraditionalForm"], ",", "p"}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
        RowBox[{"Momentum", "[", 
          RowBox[{"p", ",", "D"}], "]"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
This changes all non-4-dimensional objects to 4-dimensional ones.\
\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"ChangeDimension", "[", 
        RowBox[{"%%", ",", " ", "4"}], "]"}], " ", "//", " ", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"Momentum", "[", "p", "]"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["Consider the following list of 4- and D-dimensional object.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{
          RowBox[{"GA", "[", 
            RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
          RowBox[{"MT", "[", 
            RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}], ",", " ", 
        RowBox[{
          RowBox[{"GAD", "[", 
            RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
          RowBox[{"MTD", "[", 
            RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
          RowBox[{"f", "[", "D", "]"}]}]}], "}"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            RowBox[{
              SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
              SuperscriptBox["\[Gamma]", "\[Nu]"]}], " ", 
            SuperscriptBox["g", 
              RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], ",", 
          RowBox[{
            RowBox[{
              SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
              SuperscriptBox["\[Gamma]", "\[Nu]"]}], " ", 
            RowBox[{"f", "(", "D", ")"}], " ", 
            SuperscriptBox["g", 
              RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrick", "[", 
      RowBox[{"Contract", "[", "%", "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"4", ",", 
          RowBox[{"D", " ", 
            RowBox[{"f", "(", "D", ")"}]}]}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrick", "[", 
      RowBox[{"Contract", "[", 
        RowBox[{"ChangeDimension", "[", 
          RowBox[{"%%", ",", "n"}], "]"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"n", ",", 
          RowBox[{"n", " ", 
            RowBox[{"f", "(", "D", ")"}]}]}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell["\<\
Any explicit occurence of D (like in f(D)) is not replaced by \
ChangeDimension.\
\>", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"LeviCivita", ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";"}]], "Input"],

Cell["\<\
The option Dimension of Eps must be changed too, since with the \
default setting Dimension->4 the arguments of Eps are automatically changed \
to 4 dimensions.\
\>", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"Eps", ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a1", " ", "=", " ", 
      RowBox[{"LeviCivita", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
        "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Rho]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Sigma]",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a2", " ", "=", " ", 
      RowBox[{"ChangeDimension", "[", 
        RowBox[{"a1", ",", "4"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Rho]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Sigma]",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Factor2", "[", 
      RowBox[{"Contract", "[", 
        SuperscriptBox["a1", "2"], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"1", "-", "D"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"2", "-", "D"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"3", "-", "D"}], ")"}], " ", "D"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", 
      SuperscriptBox["a2", "2"], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", "24"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"Eps", ",", 
          RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", 
        RowBox[{"a1", ",", "a2"}], "]"}], ";"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["ChargeConjugationMatrixInv", "Subsection",
  CellTags->"ChargeConjugationMatrixInv"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
ChargeConjugationMatrixInv is the inverse of \
ChargeConjugationMatrix. It is substituted immediately by \
-ChargeConjugationMatrix.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["ChargeConjugationMatrix",
    ButtonData:>"ChargeConjugationMatrix",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ChargeConjugationMatrix"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ChargeConjugationMatrix", ".", 
      RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
      "ChargeConjugationMatrixInv"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"C", ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        RowBox[{"(", 
          RowBox[{"-", "C"}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        SubsuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "T"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ChargeConjugationMatrix", ".", 
      RowBox[{"DiracGamma", "[", "5", "]"}], ".", 
      "ChargeConjugationMatrixInv"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"C", ".", 
        SuperscriptBox["\[Gamma]", "5"], ".", 
        RowBox[{"(", 
          RowBox[{"-", "C"}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox["\[Gamma]", 
        FormBox["5",
          "TraditionalForm"], "T"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ChargeConjugationMatrix", ".", "ChargeConjugationMatrixInv"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"C", ".", 
        RowBox[{"(", 
          RowBox[{"-", "C"}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "Calc"}]], "Input"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ChargeConjugationMatrixInv", "^", "2"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", "1"}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["ChargeConjugationMatrix", "Subsection",
  CellTags->"ChargeConjugationMatrix"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "ChargeConjugationMatrix denotes the charge conjugation matrix ",
  StyleBox["C",
    FontSlant->"Italic"],
  "."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["ChargeConjugationMatrixInv",
    ButtonData:>"ChargeConjugationMatrixInv",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ChargeConjugationMatrixInv"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ChargeConjugationMatrix", ".", 
      RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
      "ChargeConjugationMatrixInv"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"C", ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        RowBox[{"(", 
          RowBox[{"-", "C"}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        SubsuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "T"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ChargeConjugationMatrix", ".", 
      RowBox[{"DiracGamma", "[", "5", "]"}], ".", 
      "ChargeConjugationMatrixInv"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"C", ".", 
        SuperscriptBox["\[Gamma]", "5"], ".", 
        RowBox[{"(", 
          RowBox[{"-", "C"}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox["\[Gamma]", 
        FormBox["5",
          "TraditionalForm"], "T"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ChargeConjugationMatrix", "^", "2"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", "1"}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["ChiralityProjector", "Subsection",
  CellTags->"ChiralityProjector"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "ChiralityProjector[+1] denotes ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"1", "/", "2"}], 
          RowBox[{"(", 
            RowBox[{"1", "+", 
              SuperscriptBox["\[Gamma]", "5"]}], ")"}]}], TraditionalForm]]],
  ". ChiralityProjector[-1] denotes ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"1", "/", "2"}], 
          RowBox[{"(", 
            RowBox[{"1", "+", 
              SuperscriptBox["\[Gamma]", "5"]}], ")"}]}], TraditionalForm]]],
  "."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracGamma",
    ButtonData:>"DiracGamma",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracGamma"],
  ", ",
  ButtonBox["DiracMatrix",
    ButtonData:>"DiracMatrix",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracMatrix"],
  ", ",
  ButtonBox["FCI",
    ButtonData:>"FCI",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FCI"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"ChiralityProjector", "[", 
          RowBox[{"+", "1"}], "]"}], ",", 
        RowBox[{"ChiralityProjector", "[", 
          RowBox[{"-", "1"}], "]"}]}], "}"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "6"], ",", 
          SuperscriptBox["\[Gamma]", "7"]}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FCI", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "6"], ",", 
          SuperscriptBox["\[Gamma]", "7"]}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSimplify", "[", 
      RowBox[{"%", ",", 
        RowBox[{"DiracSubstitute67", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            FractionBox[
              SuperscriptBox["\[Gamma]", "5"], "2"], "+", 
            FractionBox["1", "2"]}], ",", 
          RowBox[{
            FractionBox["1", "2"], "-", 
            FractionBox[
              SuperscriptBox["\[Gamma]", "5"], "2"]}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Chisholm", "Subsection",
  CellTags->"Chisholm"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Chisholm[x] substitutes products of three Dirac matrices or slashes \
by the Chisholm identity.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["EpsChisholm",
    ButtonData:>"EpsChisholm",
    ButtonStyle->"Hyperlink",
    ButtonNote->"EpsChisholm"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Chisholm", "[", 
      RowBox[{"GA", "[", 
        RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Mu]", ",", "\[Nu]"}], 
        "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", "\[ImaginaryI]"}], " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["$MU$26",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Alpha]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Beta]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["$MU$26",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Beta]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Mu]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Alpha]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Beta]", "\[NoBreak]", "\[Mu]"}]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{"DiracMatrix", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], 
        SuperscriptBox["\[Gamma]", "\[Nu]"], 
        SuperscriptBox["\[Gamma]", "\[Rho]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", " ", "=", 
      RowBox[{"Chisholm", "[", "t1", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["$MU$27",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["$MU$27",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], "-", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]]}], "+", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell["The $MU$ variables are unique indices.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", 
      RowBox[{"t1", ".", "t1"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["16", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t3", " ", "=", 
      RowBox[{"Chisholm", "[", "t1", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["$MU$28",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["$MU$28",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], "-", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]]}], "+", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", 
      RowBox[{"t2", ".", "t3"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["16", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t4", "=", 
      RowBox[{"DiracSlash", "[", 
        RowBox[{"a", ",", "b", ",", "c"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["a",
              "TraditionalForm"]}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["b",
              "TraditionalForm"]}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["c",
              "TraditionalForm"]}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Chisholm", "[", "t4", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", "\[ImaginaryI]"}], " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["$MU$29",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["$MU$29",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox["a",
                "TraditionalForm"], "\[NoBreak]", 
              FormBox["b",
                "TraditionalForm"], "\[NoBreak]", 
              FormBox["c",
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["c",
              "TraditionalForm"]}], " ", 
          RowBox[{
            FormBox["a",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["b",
              "TraditionalForm"]}]}], "-", 
        RowBox[{
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["b",
              "TraditionalForm"]}], " ", 
          RowBox[{
            FormBox["a",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["c",
              "TraditionalForm"]}]}], "+", 
        RowBox[{
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["a",
              "TraditionalForm"]}], " ", 
          RowBox[{
            FormBox["b",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["c",
              "TraditionalForm"]}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a1", "=", 
      RowBox[{"GA", "[", 
        RowBox[{
        "\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]", ",", "\[Tau]", 
          ",", "\[Kappa]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Rho]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Sigma]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Tau]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Kappa]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a2", " ", "=", 
      RowBox[{"Chisholm", "[", "a1", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", "\[ImaginaryI]"}], " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["$MU$32",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["$MU$32",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], "+", 
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["$MU$33",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["$MU$33",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["$MU$34",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["$MU$34",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Tau]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Tau]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Tau]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Tau]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Tau]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Tau]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Tau]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Tau]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Tau]"}]]}], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["$MU$30",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["$MU$30",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Sigma]", "\[NoBreak]", "\[Tau]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Sigma]", "\[NoBreak]", "\[Tau]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Sigma]", "\[NoBreak]", "\[Tau]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Sigma]", "\[NoBreak]", "\[Tau]"}]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a3", " ", "=", 
      RowBox[{"Chisholm", "[", "a1", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", "\[ImaginaryI]"}], " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["$MU$37",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["$MU$37",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], "+", 
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["$MU$38",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["$MU$38",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["$MU$39",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["$MU$39",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Tau]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Tau]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Tau]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Tau]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Tau]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Tau]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Tau]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Tau]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Tau]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Tau]"}]]}], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["$MU$35",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["$MU$35",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Sigma]", "\[NoBreak]", "\[Tau]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Sigma]", "\[NoBreak]", "\[Tau]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Sigma]", "\[NoBreak]", "\[Tau]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Kappa]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Sigma]", "\[NoBreak]", "\[Tau]"}]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell["Check that both a1.a1 and a2.a3 give the same.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", 
      RowBox[{"a1", ".", "a1"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", "2048"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", 
      RowBox[{"a2", ".", "a3"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", "2048"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{
      "t1", ",", "t2", ",", "t3", ",", "t4", ",", "a1", ",", "a2", ",", 
        "a3"}], "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["ClearScalarProducts", "Subsection",
  CellTags->"ClearScalarProducts"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
ClearScalarProducts removes all user-performed specific settings \
for ScalarProduct's.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  ButtonBox["ScalarProduct",
    ButtonData:>"ScalarProduct",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ScalarProduct"],
  ", ",
  ButtonBox["Pair",
    ButtonData:>"Pair",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Pair"],
  ", ",
  ButtonBox["SP",
    ButtonData:>"SP",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SP"],
  ", ",
  ButtonBox["SPD",
    ButtonData:>"SPD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SPD"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"ScalarProduct", "[", 
        RowBox[{"p", ",", "p"}], "]"}], "=", 
      RowBox[{"m", "^", "2"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["m", "2"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", 
        RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["m", "2"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData["ClearScalarProducts"], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", 
        RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        FormBox["p",
          "TraditionalForm"], "2"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ScalarProduct", "[", 
      RowBox[{"p", ",", "p"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        FormBox["p",
          "TraditionalForm"], "2"], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Collect2", "Subsection",
  CellTags->"Collect2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Collect2[expr, x] collects together terms which are not free of any \
occurrence of x. Collect2[expr, {x1, x2, ...}] (or also Collect2[expr, x1, \
x2, ...]) collects together terms which are not free of any occurrence of x1, \
x2, .... The coefficients are put over a common denominator. If expr is \
expanded before collecting depends on the option Factoring, which may be set \
to Factor, Factor2, or any other function, which is applied to the \
coefficients. If expr is already expanded with respect to x (x1,x2, ...), the \
option Expanding can be set to False.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Collect2", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Denominator", "\[Rule]", "False"}], ",", 
          RowBox[{"Dot", "\[Rule]", "False"}], ",", 
          RowBox[{"Expanding", "\[Rule]", "True"}], ",", 
          RowBox[{"Factoring", "\[Rule]", "Factor2"}], ",", 
          RowBox[{"IsolateNames", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Isolate",
    ButtonData:>"Isolate",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Isolate"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Collect2", "[", 
      RowBox[{
        RowBox[{"t1", "=", 
          RowBox[{"a", "+", 
            RowBox[{"r", " ", "a"}], "+", 
            RowBox[{
              SuperscriptBox["k", "2"], " ", 
              RowBox[{"f", "[", "a", "]"}]}], "-", 
            RowBox[{"k", " ", 
              RowBox[{"f", "[", "a", "]"}]}], "+", 
            FractionBox["x", "2"], "-", 
            FractionBox["y", "w"]}]}], ",", "a"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", " ", 
          RowBox[{"(", 
            RowBox[{"r", "+", "1"}], ")"}]}], "+", 
        FractionBox[
          RowBox[{
            RowBox[{"w", " ", "x"}], "-", 
            RowBox[{"2", " ", "y"}]}], 
          RowBox[{"2", " ", "w"}]], "-", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{"1", "-", "k"}], ")"}], " ", "k", " ", 
          RowBox[{"f", "(", "a", ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Collect2", "[", 
      RowBox[{"t1", ",", "a", ",", 
        RowBox[{"Factoring", "->", "False"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", " ", 
          RowBox[{"(", 
            RowBox[{"r", "+", "1"}], ")"}]}], "+", 
        FractionBox["x", "2"], "-", 
        FractionBox["y", "w"], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox["k", "2"], "-", "k"}], ")"}], " ", 
          RowBox[{"f", "(", "a", ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Collect2", "[", 
      RowBox[{"t1", ",", "a", ",", 
        RowBox[{"Factoring", "->", "Factor"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", " ", 
          RowBox[{"(", 
            RowBox[{"r", "+", "1"}], ")"}]}], "+", 
        FractionBox[
          RowBox[{
            RowBox[{"w", " ", "x"}], "-", 
            RowBox[{"2", " ", "y"}]}], 
          RowBox[{"2", " ", "w"}]], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{"k", "-", "1"}], ")"}], " ", "k", " ", 
          RowBox[{"f", "(", "a", ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Collect2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"2", " ", "a", " ", 
            RowBox[{"(", 
              RowBox[{"b", "-", "a"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"h", "-", "1"}], ")"}]}], "-", 
          RowBox[{
            SuperscriptBox["b", "2"], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"e", " ", "a"}], "-", "c"}], ")"}]}], "+", 
          SuperscriptBox["b", "2"]}], ",", 
        RowBox[{"{", 
          RowBox[{"a", ",", "b"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"2", " ", 
          RowBox[{"(", 
            RowBox[{"1", "-", "h"}], ")"}], " ", 
          SuperscriptBox["a", "2"]}], "-", 
        RowBox[{
          SuperscriptBox["b", "2"], " ", "e", " ", "a"}], "-", 
        RowBox[{"2", " ", "b", " ", 
          RowBox[{"(", 
            RowBox[{"1", "-", "h"}], ")"}], " ", "a"}], "+", 
        RowBox[{
          SuperscriptBox["b", "2"], " ", 
          RowBox[{"(", 
            RowBox[{"c", "+", "1"}], ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Collect2", "[", 
      RowBox[{
        RowBox[{"Expand", "[", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"a", "-", "b", "-", "c", "-", "d"}], ")"}], "5"], 
          "]"}], ",", "a", ",", 
        RowBox[{"IsolateNames", "\[Rule]", "L"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["a", "5"], "-", 
        RowBox[{"5", " ", 
          TagBox[
            RowBox[{"L", "(", "1", ")"}],
            HoldForm], " ", 
          SuperscriptBox["a", "4"]}], "+", 
        RowBox[{"10", " ", 
          SuperscriptBox[
            RowBox[{"(", 
              TagBox[
                RowBox[{"L", "(", "1", ")"}],
                HoldForm], ")"}], "2"], " ", 
          SuperscriptBox["a", "3"]}], "-", 
        RowBox[{"10", " ", 
          SuperscriptBox[
            RowBox[{"(", 
              TagBox[
                RowBox[{"L", "(", "1", ")"}],
                HoldForm], ")"}], "3"], " ", 
          SuperscriptBox["a", "2"]}], "+", 
        RowBox[{"5", " ", 
          SuperscriptBox[
            RowBox[{"(", 
              TagBox[
                RowBox[{"L", "(", "1", ")"}],
                HoldForm], ")"}], "4"], " ", "a"}], "-", 
        SuperscriptBox[
          RowBox[{"(", 
            TagBox[
              RowBox[{"L", "(", "1", ")"}],
              HoldForm], ")"}], "5"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ReleaseHold", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["a", "5"], "-", 
        RowBox[{"5", " ", 
          RowBox[{"(", 
            RowBox[{"b", "+", "c", "+", "d"}], ")"}], " ", 
          SuperscriptBox["a", "4"]}], "+", 
        RowBox[{"10", " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"b", "+", "c", "+", "d"}], ")"}], "2"], " ", 
          SuperscriptBox["a", "3"]}], "-", 
        RowBox[{"10", " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"b", "+", "c", "+", "d"}], ")"}], "3"], " ", 
          SuperscriptBox["a", "2"]}], "+", 
        RowBox[{"5", " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"b", "+", "c", "+", "d"}], ")"}], "4"], " ", "a"}], 
        "-", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"b", "+", "c", "+", "d"}], ")"}], "5"]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"t1", ",", "L"}], "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Collect2", "[", 
      RowBox[{
        RowBox[{"Expand", "[", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"a", "-", "b", "-", "c"}], ")"}], "3"], "]"}], ",", 
        "a", ",", 
        RowBox[{"Factoring", "\[Rule]", "fun"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"fun", "(", "1", ")"}], " ", 
          SuperscriptBox["a", "3"]}], "+", 
        RowBox[{
          RowBox[{"fun", "(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", "3"}], " ", "b"}], "-", 
              RowBox[{"3", " ", "c"}]}], ")"}], " ", 
          SuperscriptBox["a", "2"]}], "+", 
        RowBox[{
          RowBox[{"fun", "(", 
            RowBox[{
              RowBox[{"3", " ", 
                SuperscriptBox["b", "2"]}], "+", 
              RowBox[{"6", " ", "c", " ", "b"}], "+", 
              RowBox[{"3", " ", 
                SuperscriptBox["c", "2"]}]}], ")"}], " ", "a"}], "+", 
        RowBox[{"fun", "(", 
          RowBox[{
            RowBox[{"-", 
              SuperscriptBox["b", "3"]}], "-", 
            RowBox[{"3", " ", "c", " ", 
              SuperscriptBox["b", "2"]}], "-", 
            RowBox[{"3", " ", 
              SuperscriptBox["c", "2"], " ", "b"}], "-", 
            SuperscriptBox["c", "3"]}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", " ", "/.", " ", 
      RowBox[{"fun", "->", "FactorTerms"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["a", "3"], "-", 
        RowBox[{"3", " ", 
          RowBox[{"(", 
            RowBox[{"b", "+", "c"}], ")"}], " ", 
          SuperscriptBox["a", "2"]}], "+", 
        RowBox[{"3", " ", 
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox["b", "2"], "+", 
              RowBox[{"2", " ", "c", " ", "b"}], "+", 
              SuperscriptBox["c", "2"]}], ")"}], " ", "a"}], "-", 
        SuperscriptBox["b", "3"], "-", 
        SuperscriptBox["c", "3"], "-", 
        RowBox[{"3", " ", "b", " ", 
          SuperscriptBox["c", "2"]}], "-", 
        RowBox[{"3", " ", 
          SuperscriptBox["b", "2"], " ", "c"}]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Combine", "Subsection",
  CellTags->"Combine"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Combine[expr]puts terms in a sum over a common denominator,and \
cancels factors in the result. Combine is similar to Together, but accepts \
the option Expanding and works usually better than Together for polynomials \
involving rationals with sums in the denominator.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Combine", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Expanding", "\[Rule]", "False"}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Factor2",
    ButtonData:>"Factor2",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Factor2"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Combine", "[", 
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "b"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"c", "-", "d"}], ")"}]}], "e"], "+", "g"}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "b"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"c", "-", "d"}], ")"}]}], "+", 
          RowBox[{"e", " ", "g"}]}], "e"], TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
Here the result from Together where the numerator is automatically \
expanded.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Together", "[", 
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "b"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"c", "-", "d"}], ")"}]}], "e"], "+", "g"}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          RowBox[{"a", " ", "c"}], "-", 
          RowBox[{"b", " ", "c"}], "-", 
          RowBox[{"a", " ", "d"}], "+", 
          RowBox[{"b", " ", "d"}], "+", 
          RowBox[{"e", " ", "g"}]}], "e"], TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
If the option Expanding is set to True, the result of Combine is \
the same as Together, but uses a slightly different algorithm.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Combine", "[", 
      RowBox[{
        RowBox[{
          FractionBox[
            RowBox[{
              RowBox[{"(", 
                RowBox[{"a", "-", "b"}], ")"}], " ", 
              RowBox[{"(", 
                RowBox[{"c", "-", "d"}], ")"}]}], "e"], "+", "g"}], ",", 
        RowBox[{"Expanding", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          RowBox[{"a", " ", "c"}], "-", 
          RowBox[{"b", " ", "c"}], "-", 
          RowBox[{"a", " ", "d"}], "+", 
          RowBox[{"b", " ", "d"}], "+", 
          RowBox[{"e", " ", "g"}]}], "e"], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["CommutatorExplicit", "Subsection",
  CellTags->"CommutatorExplicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
CommutatorExplicit[exp] substitutes any Commutator and \
AntiCommutator in exp by their definitions.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Calc",
    ButtonData:>"Calc",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Calc"],
  ", ",
  ButtonBox["DotSimplify",
    ButtonData:>"DotSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DotSimplify"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[BoxData[
    RowBox[{"DeclareNonCommutative", "[", 
      RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Commutator", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
      "[", "\[NoBreak]", "a", "\[NoBreak]", ",", "b", "\[NoBreak]", "]"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CommutatorExplicit", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", ".", "b"}], "-", 
        RowBox[{"b", ".", "a"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"AntiCommutator", "[", 
      RowBox[{
        RowBox[{"a", "-", "c"}], ",", 
        RowBox[{"b", "-", "d"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", "\[NoBreak]", 
        FormBox[
          RowBox[{"a", "-", "c"}],
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
        "\[MediumSpace]", "\[NoBreak]", 
        FormBox[
          RowBox[{"b", "-", "d"}],
          "TraditionalForm"], "\[NoBreak]", "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CommutatorExplicit", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{"a", "-", "c"}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{"b", "-", "d"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{"b", "-", "d"}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{"a", "-", "c"}], ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"CommutatorExplicit", "[", "%%", "]"}], "//", "DotSimplify"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", ".", "b"}], "-", 
        RowBox[{"a", ".", "d"}], "+", 
        RowBox[{"b", ".", "a"}], "-", 
        RowBox[{"b", ".", "c"}], "-", 
        RowBox[{"c", ".", "b"}], "+", 
        RowBox[{"c", ".", "d"}], "-", 
        RowBox[{"d", ".", "a"}], "+", 
        RowBox[{"d", ".", "c"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"UnDeclareNonCommutative", "[", 
      RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Commutator", "Subsection",
  CellTags->"Commutator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Commutator[x, y] = c defines the commutator between the \
non-commuting objects x and y. \
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["AntiCommutator",
    ButtonData:>"AntiCommutator",
    ButtonStyle->"Hyperlink",
    ButtonNote->"AntiCommutator"],
  ", ",
  ButtonBox["CommutatorExplicit",
    ButtonData:>"CommutatorExplicit",
    ButtonStyle->"Hyperlink",
    ButtonNote->"CommutatorExplicit"],
  ", ",
  ButtonBox["DeclareNonCommutative",
    ButtonData:>"DeclareNonCommutative",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DeclareNonCommutative"],
  ", ",
  ButtonBox["DotSimplify",
    ButtonData:>"DotSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DotSimplify"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[BoxData[
    RowBox[{"DeclareNonCommutative", "[", 
      RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Commutator", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
      "[", "\[NoBreak]", "a", "\[NoBreak]", ",", "b", "\[NoBreak]", "]"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CommutatorExplicit", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", ".", "b"}], "-", 
        RowBox[{"b", ".", "a"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DotSimplify", "[", 
        RowBox[{"Commutator", "[", 
          RowBox[{
            RowBox[{"a", "+", "b"}], ",", 
            RowBox[{"c", "+", "d"}]}], "]"}], "]"}], " "}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", ".", "c"}], "+", 
        RowBox[{"a", ".", "d"}], "+", 
        RowBox[{"b", ".", "c"}], "+", 
        RowBox[{"b", ".", "d"}], "-", 
        RowBox[{"c", ".", "a"}], "-", 
        RowBox[{"c", ".", "b"}], "-", 
        RowBox[{"d", ".", "a"}], "-", 
        RowBox[{"d", ".", "b"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"UnDeclareNonCommutative", "[", 
      RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]], "Input"],

Cell["Verify the Jacobi identity.", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"\[Chi]", "=", "Commutator"}], ";", " ", 
      RowBox[{"DeclareNonCommutative", "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"\[Chi]", "[", 
        RowBox[{"x", ",", 
          RowBox[{"\[Chi]", "[", 
            RowBox[{"y", ",", "z"}], "]"}]}], "]"}], "+", 
      RowBox[{"\[Chi]", "[", 
        RowBox[{"y", ",", 
          RowBox[{"\[Chi]", "[", 
            RowBox[{"z", ",", "x"}], "]"}]}], "]"}], "+", 
      RowBox[{"\[Chi]", "[", 
        RowBox[{"z", ",", 
          RowBox[{"\[Chi]", "[", 
            RowBox[{"x", ",", "y"}], "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"[", "\[NoBreak]", "x", "\[NoBreak]", ",", 
          FormBox[
            RowBox[{
            "[", "\[NoBreak]", "y", "\[NoBreak]", ",", "z", "\[NoBreak]", 
              "]"}],
            "TraditionalForm"], "\[NoBreak]", "]"}], "+", 
        RowBox[{"[", "\[NoBreak]", "y", "\[NoBreak]", ",", 
          FormBox[
            RowBox[{
            "[", "\[NoBreak]", "z", "\[NoBreak]", ",", "x", "\[NoBreak]", 
              "]"}],
            "TraditionalForm"], "\[NoBreak]", "]"}], "+", 
        RowBox[{"[", "\[NoBreak]", "z", "\[NoBreak]", ",", 
          FormBox[
            RowBox[{
            "[", "\[NoBreak]", "x", "\[NoBreak]", ",", "y", "\[NoBreak]", 
              "]"}],
            "TraditionalForm"], "\[NoBreak]", "]"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotSimplify", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", "\[Chi]", "]"}]], "Input"],

Cell[BoxData[
    RowBox[{"UnDeclareNonCommutative", "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["ComplexConjugate", "Subsection",
  CellTags->"ComplexConjugate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
ComplexConjugate[expr] complex conjugates expr. It operates on \
fermion lines, i.e., products of Spinor[..] .DiracMatrix[..] . Spinor[..], \
and changes all occuring LorentzIndex[mu] into \
LorentzIndex[ComplexIndex[mu]]. For taking the spin sum (i.e. constructing \
the traces) use FermionSpinSum. WARNING: In expr should be NO explicit I's in \
denominators!\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["ComplexIndex",
    ButtonData:>"ComplexIndex",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ComplexIndex"],
  ", ",
  ButtonBox["FermionSpinSum",
    ButtonData:>"FermionSpinSum",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FermionSpinSum"],
  ", ",
  ButtonBox["LorentzIndex",
    ButtonData:>"LorentzIndex",
    ButtonStyle->"Hyperlink",
    ButtonNote->"LorentzIndex"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ComplexConjugate", "[", 
      RowBox[{"MetricTensor", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["g", 
        RowBox[{
          FormBox[
            SuperscriptBox["\[Mu]", "*"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            SuperscriptBox["\[Nu]", "*"],
            "TraditionalForm"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"ComplexIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"ComplexIndex", "[", "\[Nu]", "]"}], "]"}]}], "]"}]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GA", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "5"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
        SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ComplexConjugate", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "5"], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox[
                SuperscriptBox["\[Nu]", "*"],
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox[
                SuperscriptBox["\[Mu]", "*"],
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNTrace", "[", 
      RowBox[{"SUNT", "[", 
        RowBox[{"a", ",", "b", ",", "c"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]]}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ComplexConjugate", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]]}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ComplexConjugate", "[", 
      RowBox[{"SUNF", "[", 
        RowBox[{"a", ",", "b", ",", "c"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubscriptBox["f", 
        RowBox[{
          FormBox[
            FormBox["a",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["b",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["c",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"FCE", "[", 
        RowBox[{"ComplexConjugate", "[", 
          RowBox[{"MetricTensor", "[", 
            RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "]"}], "]"}], "]"}]], 
  "Input"],

Cell[BoxData[
    RowBox[{"MT", "[", 
      RowBox[{
        RowBox[{"ComplexIndex", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"ComplexIndex", "[", "\[Nu]", "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SpinorUBar", "[", 
        RowBox[{"k1", ",", "m"}], "]"}], ".", 
      RowBox[{"GA", "[", "\[Lambda]", "]"}], ".", 
      RowBox[{"SpinorU", "[", 
        RowBox[{"p1", ",", "m"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FormBox[
            OverscriptBox["\<\"u\"\>", "_"],
            "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "k1", 
          "\[NoBreak]", ",", "\[NoBreak]", "m", "\[NoBreak]", ")"}], ".", 
        SuperscriptBox["\[Gamma]", "\[Lambda]"], ".", 
        RowBox[{
        "u", "\[NoBreak]", "(", "\[NoBreak]", "p1", "\[NoBreak]", ",", 
          "\[NoBreak]", "m", "\[NoBreak]", ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ComplexConjugate", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"\[CurlyPhi]", "\[NoBreak]", "(", "\[NoBreak]", 
          FormBox["p1",
            "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m", 
          "\[NoBreak]", ")"}], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox[
              SuperscriptBox["\[Lambda]", "*"],
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        RowBox[{"\[CurlyPhi]", "\[NoBreak]", "(", "\[NoBreak]", 
          FormBox["k1",
            "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m", 
          "\[NoBreak]", ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
Notice that SpinorUBar and SpinorU are only input functions. \
Internally they are converted to Spinor objects.\
\>", "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["ComplexIndex", "Subsection",
  CellTags->"ComplexIndex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
ComplexIndex is the head of a \"complex conjugate index\", which is \
understood as just another index.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["LorentzIndex",
    ButtonData:>"LorentzIndex",
    ButtonStyle->"Hyperlink",
    ButtonNote->"LorentzIndex"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]], "Input"],

Cell[BoxData[
    FormBox["\[Mu]", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ComplexConjugate", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Mu]", "*"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"LorentzIndex", "[", 
      RowBox[{"ComplexIndex", "[", "\[Mu]", "]"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FourVector", "[", 
      RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubscriptBox["p", "\[Mu]"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ComplexConjugate", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        FormBox["p",
          "TraditionalForm"], 
        FormBox[
          FormBox[
            SuperscriptBox["\[Mu]", "*"],
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"ComplexIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
        RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ComplexConjugate", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        FormBox["p",
          "TraditionalForm"], 
        FormBox[
          FormBox["\[Mu]",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Contract", "Subsection",
  CellTags->"Contract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Contract[expr] contracts pairs of Lorentz indices of metric \
tensors, four-vectors and (depending on the optionEpsContract) of Levi-Civita \
tensors in expr. For the contraction of Dirac matrices with each other use \
DiracSimplify. Contract[exp1, exp2] contracts (exp1*exp2), where exp1 and \
exp2 may be larger products of sums of metric tensors and 4-vectors. \
\>", 
  "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Contract", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Collecting", "\[Rule]", "True"}], ",", 
          RowBox[{"EpsContract", "\[Rule]", "True"}], ",", 
          RowBox[{"Expanding", "\[Rule]", "True"}], ",", 
          RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
          RowBox[{"FeynCalcInternal", "\[Rule]", "False"}], ",", 
          RowBox[{"MomentumCombine", "\[Rule]", "False"}], ",", 
          RowBox[{"Rename", "\[Rule]", "False"}], ",", 
          RowBox[{"Schouten", "\[Rule]", "0"}]}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Pair",
    ButtonData:>"Pair",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Pair"],
  ", ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracSimplify"],
  ", ",
  ButtonBox["MomentumCombine",
    ButtonData:>"MomentumCombine",
    ButtonStyle->"Hyperlink",
    ButtonNote->"MomentumCombine"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"MetricTensor", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
      RowBox[{"FourVector", "[", 
        RowBox[{"p", ",", "\[Mu]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["p", "\[Mu]"], " ", 
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        FormBox["p",
          "TraditionalForm"], 
        FormBox[
          FormBox["\[Nu]",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FourVector", "[", 
        RowBox[{"p", ",", "\[Mu]"}], "]"}], 
      RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SubscriptBox["p", "\[Mu]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        FormBox["p",
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"MetricTensor", "[", 
      RowBox[{"\[Mu]", ",", "\[Mu]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[NoBreak]", "\[Mu]"}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell["The default dimension for MetricTensor is 4.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox["4", TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
A short way to enter D-dimensional metric tensors is given by MTD. \
The \".\" as multiplication operator is not necessary but just convenient for \
typesetting.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"MTD", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", ".", " ", 
      RowBox[{"MTD", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], ".", 
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox["D", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"MTD", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", ".", " ", 
      RowBox[{"MTD", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], ".", 
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FourVector", "[", 
        RowBox[{"p", ",", "\[Mu]"}], "]"}], " ", 
      RowBox[{"FourVector", "[", 
        RowBox[{"q", ",", "\[Mu]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["p", "\[Mu]"], " ", 
        SubscriptBox["q", "\[Mu]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", "%", " ", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["p",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        FormBox["q",
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FourVector", "[", 
        RowBox[{
          RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}], " ", 
      RowBox[{"FourVector", "[", 
        RowBox[{
          RowBox[{"a", "-", "b"}], ",", "\[Mu]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          RowBox[{"(", "\[NoBreak]", 
            FormBox[
              TagBox[
                RowBox[{"a", "-", "b"}],
                HoldForm],
              "TraditionalForm"], "\[NoBreak]", ")"}], "\[Mu]"], " ", 
        SubscriptBox[
          RowBox[{"(", "\[NoBreak]", 
            FormBox[
              TagBox[
                RowBox[{"p", "-", "q"}],
                HoldForm],
              "TraditionalForm"], "\[NoBreak]", ")"}], "\[Mu]"]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FormBox["a",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["p",
            "TraditionalForm"]}], "-", 
        RowBox[{
          FormBox["a",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}], "-", 
        RowBox[{
          FormBox["b",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["p",
            "TraditionalForm"]}], "+", 
        RowBox[{
          FormBox["b",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"LeviCivita", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Alpha]", ",", "\[Sigma]"}], 
        "]"}], " ", 
      RowBox[{"FourVector", "[", 
        RowBox[{"p", ",", "\[Sigma]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Alpha]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"]}]], " ", 
        SubscriptBox["p", "\[Sigma]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox[
            FormBox["\[Alpha]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox["p",
            "TraditionalForm"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"LeviCivita", "[", 
          RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Alpha]", ",", "\[Beta]"}], 
          "]"}], " ", 
        RowBox[{"LeviCivita", "[", 
          RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Alpha]", ",", "\[Sigma]"}], 
          "]"}]}], " "}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Alpha]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Beta]",
                "TraditionalForm"],
              "TraditionalForm"]}]], " ", 
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Alpha]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "6"}], " ", 
        SuperscriptBox["g", 
          RowBox[{"\[Beta]", "\[NoBreak]", "\[Sigma]"}]]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"Eps", ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";", 
      RowBox[{
        RowBox[{"LCD", "[", 
          RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Alpha]", ",", "\[Beta]"}], 
          "]"}], " ", 
        RowBox[{"LCD", "[", 
          RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Alpha]", ",", "\[Sigma]"}], 
          "]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
          "\[Mu]", "\[NoBreak]", "\[Nu]", "\[NoBreak]", "\[Alpha]", 
            "\[NoBreak]", "\[Beta]"}]], " ", 
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
          "\[Mu]", "\[NoBreak]", "\[Nu]", "\[NoBreak]", "\[Alpha]", 
            "\[NoBreak]", "\[Sigma]"}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Contract", "[", "%", "]"}], "//", "Factor2"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"1", "-", "D"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"2", "-", "D"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"3", "-", "D"}], ")"}], " ", 
        SuperscriptBox["g", 
          RowBox[{
            FormBox[
              FormBox["\[Beta]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"Eps", ",", 
          RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}], ";"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Convolute", "Subsection",
  CellTags->"Convolute"],

Cell["Description", "Text"],

Cell[TextData[{
  "Convolute[f, g, x] convolutes f(x) and g(x), i.e., ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubsuperscriptBox["\[Integral]", "0", "1"], 
          RowBox[{
            SubscriptBox["dx", "1"], " ", 
            RowBox[{
              SubsuperscriptBox["\[Integral]", "0", "1"], 
              RowBox[{
                SubscriptBox["dx", "2"], " ", 
                RowBox[{"\[Delta]", "(", 
                  RowBox[{"x", " ", "-", " ", 
                    RowBox[{
                      SubscriptBox["x", "1"], " ", 
                      SubscriptBox["x", "2"]}]}], ")"}], " ", 
                RowBox[{"f", "(", 
                  SubscriptBox["x", "1"], ")"}], " ", 
                RowBox[{
                  RowBox[{"g", "(", 
                    SubscriptBox["x", "2"], ")"}], " ", "."}]}]}]}]}], 
        TraditionalForm]]],
  " Convolute[f, g] is equivalent to Convolute[f, g, x]. Convolute[exp, {x1, \
x2}] assumes that exp is polynomial in x1 and x2. Convolute uses \
table-look-up and does not do any integral calculations, only linear \
algebra."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Convolute", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Bracket", "\[Rule]", 
            RowBox[{"{", 
              TagBox["\[CurlyEpsilon]",
                TraditionalForm], "}"}]}], ",", 
          RowBox[{"FinalSubstitutions", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{"PlusDistribution", "\[Rule]", "Identity"}], "}"}]}]}], 
        "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["PlusDistribution",
    ButtonData:>"PlusDistribution",
    ButtonStyle->"Hyperlink",
    ButtonNote->"PlusDistribution"],
  "."
}], "Text"],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{"1", ",", "1"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{"x", ",", "x"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "x"}], " ", 
        RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{"1", ",", "x"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"1", "-", "x"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{"1", ",", 
        FractionBox["1", 
          RowBox[{"1", "-", "x"}]]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"log", "(", 
          RowBox[{"1", "-", "x"}], ")"}], "-", 
        RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{"1", ",", 
        RowBox[{"PlusDistribution", "[", 
          FractionBox["1", 
            RowBox[{"1", "-", "x"}]], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"log", "(", 
          RowBox[{"1", "-", "x"}], ")"}], "-", 
        RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{
        FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ",", "x"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], "-", 
        RowBox[{
          RowBox[{"log", "(", "x", ")"}], " ", "x"}], "-", "x", "+", "1"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{
        FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ",", 
        FractionBox["1", 
          RowBox[{"1", "-", "x"}]]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], " ", 
          RowBox[{"\[Delta]", "(", 
            FormBox[
              RowBox[{"1", "-", "x"}],
              "TraditionalForm"], ")"}]}], "+", 
        FractionBox[
          RowBox[{"2", " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], 
          RowBox[{"1", "-", "x"}]], "-", 
        FractionBox[
          RowBox[{"log", "(", "x", ")"}], 
          RowBox[{"1", "-", "x"}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{"1", ",", 
        RowBox[{"Log", "[", 
          RowBox[{"1", "-", "x"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], " ", 
          RowBox[{"log", "(", "x", ")"}]}], "-", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          RowBox[{"1", "-", "x"}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{"1", ",", 
        RowBox[{"x", " ", 
          RowBox[{"Log", "[", 
            RowBox[{"1", "-", "x"}], "]"}]}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"x", "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{"1", "-", "x"}], ")"}], " ", 
          RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "-", "1"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{
        FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ",", 
        RowBox[{"Log", "[", 
          RowBox[{"1", "-", "x"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["log", "2"], "(", 
          RowBox[{"1", "-", "x"}], ")"}], "-", 
        RowBox[{
          RowBox[{"log", "(", "x", ")"}], " ", 
          RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "-", 
        RowBox[{"\[Zeta]", "(", "2", ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{
        FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ",", 
        RowBox[{"x", " ", 
          RowBox[{"Log", "[", 
            RowBox[{"1", "-", "x"}], "]"}]}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"x", " ", 
          RowBox[{
            SuperscriptBox["log", "2"], "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{"1", "-", "x"}], ")"}], " ", 
          RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "-", 
        RowBox[{"x", " ", 
          RowBox[{"log", "(", "x", ")"}], " ", 
          RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "+", "x", "-", 
        RowBox[{"x", " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", "1"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{
        FractionBox[
          RowBox[{"Log", "[", 
            RowBox[{"1", "-", "x"}], "]"}], 
          RowBox[{"1", "-", "x"}]], ",", "x"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "2"], " ", "x", " ", 
          RowBox[{
            SuperscriptBox["log", "2"], "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{"1", "-", "x"}], ")"}], " ", 
          RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "-", 
        RowBox[{"x", " ", 
          RowBox[{"log", "(", "x", ")"}], " ", 
          RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "+", 
        RowBox[{"x", " ", 
          RowBox[{"log", "(", "x", ")"}]}], "-", 
        RowBox[{"x", " ", 
          RowBox[{
            SubscriptBox["Li", "2"], "(", 
            RowBox[{"1", "-", "x"}], ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{"1", ",", 
        RowBox[{"x", " ", 
          RowBox[{"Log", "[", "x", "]"}]}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            RowBox[{"log", "(", "x", ")"}]}], " ", "x"}], "+", "x", "-", 
        "1"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{
        RowBox[{"Log", "[", 
          RowBox[{"1", "-", "x"}], "]"}], ",", "x"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{"1", "-", "x"}], ")"}], " ", 
          RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "+", 
        RowBox[{"x", " ", 
          RowBox[{"log", "(", "x", ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{
        FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ",", 
        FractionBox[
          RowBox[{"Log", "[", "x", "]"}], 
          RowBox[{"1", "-", "x"}]]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}], " ", 
            RowBox[{"log", "(", "x", ")"}]}], 
          RowBox[{"1", "-", "x"}]], "-", 
        FractionBox[
          RowBox[{
            SuperscriptBox["log", "2"], "(", "x", ")"}], 
          RowBox[{"2", " ", 
            RowBox[{"(", 
              RowBox[{"1", "-", "x"}], ")"}]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{"1", ",", 
        RowBox[{"Log", "[", "x", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "2"]}], " ", 
        RowBox[{
          SuperscriptBox["log", "2"], "(", "x", ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{"x", ",", " ", 
        RowBox[{"x", " ", 
          RowBox[{"Log", "[", "x", "]"}]}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "2"]}], " ", "x", " ", 
        RowBox[{
          SuperscriptBox["log", "2"], "(", "x", ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{
        FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ",", 
        RowBox[{"Log", "[", "x", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            FractionBox["1", "2"]}], " ", 
          RowBox[{
            SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
        RowBox[{
          RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}], " ", 
          RowBox[{"log", "(", "x", ")"}]}], "+", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          RowBox[{"1", "-", "x"}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{"1", ",", 
        FractionBox[
          RowBox[{"Log", "[", "x", "]"}], 
          RowBox[{"1", "-", "x"}]]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            FractionBox["1", "2"]}], " ", 
          RowBox[{
            SuperscriptBox["log", "2"], "(", "x", ")"}]}], "-", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          RowBox[{"1", "-", "x"}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{
        FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ",", 
        RowBox[{"x", " ", 
          RowBox[{"Log", "[", "x", "]"}]}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            FractionBox["1", "2"]}], " ", "x", " ", 
          RowBox[{
            SuperscriptBox["log", "2"], "(", "x", ")"}]}], "-", 
        RowBox[{"x", " ", 
          RowBox[{"log", "(", "x", ")"}]}], "+", 
        RowBox[{"x", " ", 
          RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}], " ", 
          RowBox[{"log", "(", "x", ")"}]}], "+", "x", "+", 
        RowBox[{"x", " ", 
          RowBox[{
            SubscriptBox["Li", "2"], "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "-", "1"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{
        FractionBox[
          RowBox[{"Log", "[", "x", "]"}], 
          RowBox[{"1", "-", "x"}]], ",", "x"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            FractionBox["1", "2"]}], " ", "x", " ", 
          RowBox[{
            SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
        RowBox[{"log", "(", "x", ")"}], "-", "x", "-", 
        RowBox[{"x", " ", 
          RowBox[{
            SubscriptBox["Li", "2"], "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "+", "1"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{"1", ",", 
        RowBox[{"x", " ", 
          RowBox[{"Log", "[", "x", "]"}]}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            RowBox[{"log", "(", "x", ")"}]}], " ", "x"}], "+", "x", "-", 
        "1"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{
        RowBox[{"Log", "[", "x", "]"}], ",", "x"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "x"}], "+", 
        RowBox[{"log", "(", "x", ")"}], "+", "1"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{
        FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ",", 
        FractionBox[
          RowBox[{"Log", "[", 
            RowBox[{"1", "-", "x"}], "]"}], 
          RowBox[{"1", "-", "x"}]]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{"3", " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], 
          RowBox[{"2", " ", 
            RowBox[{"(", 
              RowBox[{"1", "-", "x"}], ")"}]}]], "-", 
        FractionBox[
          RowBox[{
            RowBox[{"log", "(", "x", ")"}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], 
          RowBox[{"1", "-", "x"}]], "-", 
        FractionBox[
          RowBox[{"\[Zeta]", "(", "2", ")"}], 
          RowBox[{"1", "-", "x"}]], "+", 
        RowBox[{
          RowBox[{"\[Delta]", "(", 
            FormBox[
              RowBox[{"1", "-", "x"}],
              "TraditionalForm"], ")"}], " ", 
          TagBox[
            RowBox[{"\[Zeta]", "(", 
              TagBox["3",
                (Editable -> True)], ")"}],
            InterpretTemplate[ Zeta[ #]&]]}]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["CovariantD", "Subsection",
  CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
CovariantD[mu, a, b] is the covariant derivative for a bosonic \
field. CovariantD[mu] is the covariant derivative for a fermionic field. \
CovariantD[OPEDelta, a, b] is a short form for \
CovariantD[mu,a,b]*FourVector[OPEDelta, mu]. CovariantD[{OPEDelta, a, b}, \
{n}] yields the product of m operators., where n is an integer. \
CovariantD[OPEDelta, a, b, {m, n}] gives the expanded form of \
CovariantD[OPEDelta, a, b]^m up to order g^n for the gluon, where n is an \
integer and g the couplingconstant indicated by the setting of the option \
CouplingConstant. CovariantD[OPEDelta, {m, n}] gives the expanded form of \
CovariantD[OPEDelta]^m up to order g^n of the fermionic field.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "CovariantD", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"DummyIndex", "\[Rule]", "Automatic"}], ",", 
          RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
          RowBox[{"PartialD", "\[Rule]", "RightPartialD"}], ",", 
          RowBox[{"QuantumField", "\[Rule]", "A"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
 Possible settings of PartialD are: LeftPartialD, \
LeftRigthPartialD, RightPartialD. The default setting of QuantumField is \
GaugeField.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["LeftPartialD",
    ButtonData:>"LeftPartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"LeftPartialD"],
  ", ",
  ButtonBox["LeftRightPartialD",
    ButtonData:>"LeftRightPartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"LeftRightPartialD"],
  ", ",
  ButtonBox["RightPartialD",
    ButtonData:>"RightPartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"RightPartialD"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CovariantD", "[", "\[Mu]", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubscriptBox["D", "\[Mu]"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CovariantD", "[", 
      RowBox[{"\[Mu]", ",", "a", ",", "b"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox["D", "\[Mu]", 
        RowBox[{"a", "\[NoBreak]", "b"}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CovariantD", "[", 
      RowBox[{"\[Mu]", ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[RightArrow]"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
          SubscriptBox["g", "s"], " ", 
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox[
                  InterpretationBox[
                    SubscriptBox["c", "1"],
                    Subscripted[ 
                      c[ 1]],
                    Editable->False],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox[
                  InterpretationBox[
                    SubscriptBox["c", "1"],
                    Subscripted[ 
                      c[ 1]],
                    Editable->False],
                  "TraditionalForm"],
                "TraditionalForm"]]}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
The first argument of CovariantD is intepreted as type \
LorentzIndex, except for OPEDelta, which is type Momentum.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CovariantD", "[", "OPEDelta", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubscriptBox["D", 
        FormBox["\[CapitalDelta]",
          "TraditionalForm"]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CovariantD", "[", 
      RowBox[{"OPEDelta", ",", "a", ",", "b"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox["D", 
        FormBox["\[CapitalDelta]",
          "TraditionalForm"], 
        RowBox[{"a", "\[NoBreak]", "b"}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CovariantD", "[", 
      RowBox[{"OPEDelta", ",", "a", ",", "b", ",", 
        RowBox[{"Explicit", "->", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox["\[CapitalDelta]",
              "TraditionalForm"]], " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{"a", "\[NoBreak]", "b"}]]}], "-", 
        RowBox[{
          SubscriptBox["g", "s"], " ", 
          SubsuperscriptBox[
            FormBox["A",
              "TraditionalForm"], 
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox[
                InterpretationBox[
                  SubscriptBox["c", "2"],
                  Subscripted[ 
                    c[ 2]],
                  Editable->False],
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{"a", "\[NoBreak]", "b", "\[NoBreak]", 
              FormBox[
                InterpretationBox[
                  SubscriptBox["c", "2"],
                  Subscripted[ 
                    c[ 2]],
                  Editable->False],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CovariantD", "[", 
      RowBox[{"OPEDelta", ",", 
        RowBox[{"Explicit", "->", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[RightArrow]"], 
          FormBox["\[CapitalDelta]",
            "TraditionalForm"]], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
          SubscriptBox["g", "s"], " ", 
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox[
                  InterpretationBox[
                    SubscriptBox["c", "3"],
                    Subscripted[ 
                      c[ 3]],
                    Editable->False],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"], 
              FormBox[
                FormBox[
                  InterpretationBox[
                    SubscriptBox["c", "3"],
                    Subscripted[ 
                      c[ 3]],
                    Editable->False],
                  "TraditionalForm"],
                "TraditionalForm"]]}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CovariantD", "[", 
      RowBox[{"OPEDelta", ",", "a", ",", "b", ",", 
        RowBox[{"{", "2", "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SubscriptBox[
                OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"]], " ", 
              SubscriptBox["\[Delta]", 
                RowBox[{"a", "\[NoBreak]", "c8"}]]}], "-", 
            RowBox[{
              SubscriptBox["g", "s"], " ", 
              SubsuperscriptBox[
                FormBox["A",
                  "TraditionalForm"], 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["e1",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SubscriptBox["f", 
                RowBox[{"a", "\[NoBreak]", "c8", "\[NoBreak]", "e1"}]]}]}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SubscriptBox[
                OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"]], " ", 
              SubscriptBox["\[Delta]", 
                RowBox[{"b", "\[NoBreak]", "c8"}]]}], "-", 
            RowBox[{
              SubscriptBox["g", "s"], " ", 
              SubsuperscriptBox[
                FormBox["A",
                  "TraditionalForm"], 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["e2",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SubscriptBox["f", 
                RowBox[{"c8", "\[NoBreak]", "b", "\[NoBreak]", "e2"}]]}]}], 
          ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "This gives",
  Cell[BoxData[
      FormBox[
        RowBox[{" ", 
          RowBox[{"m", " ", "times", " ", 
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
              "\[CapitalDelta]"]}], ",", " "}], TraditionalForm]]],
  "the partial derivative ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[RightArrow]"], 
          RowBox[{"\[Mu]", " "}]], TraditionalForm]]],
  "contracted with ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox["\[CapitalDelta]", "\[Mu]"], "."}], 
        TraditionalForm]]]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CovariantD", "[", 
      RowBox[{"OPEDelta", ",", "a", ",", "b", ",", 
        RowBox[{"{", 
          RowBox[{"OPEm", ",", "0"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          RowBox[{"(", 
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], ")"}], "m"], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "The expansion up to first order in the coupling constant ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["g", "s"], " ", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"the", "sum", "is", "the", "FeynCalc", "OPESum"}], 
              ")"}], "."}]}], TraditionalForm]]]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CovariantD", "[", 
      RowBox[{"OPEDelta", ",", "a", ",", "b", ",", 
        RowBox[{"{", 
          RowBox[{"OPEm", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              SubscriptBox[
                OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"]], ")"}], "m"], " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          SubscriptBox["g", "s"], " ", 
          FormBox[
            TagBox[
              RowBox[{
                UnderoverscriptBox["\[Sum]", 
                  RowBox[{"i", "=", "0"}], 
                  RowBox[{"m", "-", "1"}]], 
                RowBox[{
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        SubscriptBox[
                          OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"]], ")"}], "i"], ".", 
                    SubsuperscriptBox[
                      FormBox["A",
                        "TraditionalForm"], 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["c", "1"],
                            Subscripted[ 
                              c[ 1]],
                            Editable->False],
                          "TraditionalForm"],
                        "TraditionalForm"]], ".", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        SubscriptBox[
                          OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"]], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "1"}]]}], " ", 
                  SubscriptBox["f", 
                    RowBox[{
                      FormBox[
                        FormBox["a",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["b",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["c", "1"],
                            Subscripted[ 
                              c[ 1]],
                            Editable->False],
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}]}],
              HoldForm],
            "TraditionalForm"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "The expansion up to second order in the ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["g", "s"], "."}], TraditionalForm]]]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CovariantD", "[", 
      RowBox[{"OPEDelta", ",", "a", ",", "b", ",", 
        RowBox[{"{", 
          RowBox[{"OPEm", ",", "2"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              SubscriptBox[
                OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"]], ")"}], "m"]}], "-", 
        RowBox[{
          SubsuperscriptBox["g", "s", "2"], " ", 
          FormBox[
            TagBox[
              RowBox[{
                UnderoverscriptBox["\[Sum]", 
                  RowBox[{"j", "=", "0"}], 
                  RowBox[{"m", "-", "2"}]], 
                FormBox[
                  TagBox[
                    RowBox[{
                      UnderoverscriptBox["\[Sum]", 
                        RowBox[{"i", "=", "0"}], "j"], 
                      RowBox[{
                        RowBox[{
                          SuperscriptBox[
                            RowBox[{"(", 
                              SubscriptBox[
                                
                                OverscriptBox["\[PartialD]", 
                                  "\[RightArrow]"], 
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"]], ")"}], "i"], ".", 
                          SubsuperscriptBox[
                            FormBox["A",
                              "TraditionalForm"], 
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], 
                            FormBox[
                              FormBox[
                                InterpretationBox[
                                  SubscriptBox["c", "1"],
                                  Subscripted[ 
                                    c[ 1]],
                                  Editable->False],
                                "TraditionalForm"],
                              "TraditionalForm"]], ".", 
                          SuperscriptBox[
                            RowBox[{"(", 
                              SubscriptBox[
                                
                                OverscriptBox["\[PartialD]", 
                                  "\[RightArrow]"], 
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"]], ")"}], 
                            RowBox[{"j", "-", "i"}]], ".", 
                          SubsuperscriptBox[
                            FormBox["A",
                              "TraditionalForm"], 
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], 
                            FormBox[
                              FormBox[
                                InterpretationBox[
                                  SubscriptBox["c", "2"],
                                  Subscripted[ 
                                    c[ 2]],
                                  Editable->False],
                                "TraditionalForm"],
                              "TraditionalForm"]], ".", 
                          SuperscriptBox[
                            RowBox[{"(", 
                              SubscriptBox[
                                
                                OverscriptBox["\[PartialD]", 
                                  "\[RightArrow]"], 
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"]], ")"}], 
                            RowBox[{
                              RowBox[{"-", "j"}], "+", "m", "-", "2"}]]}], 
                        " ", 
                        SubscriptBox["f", 
                          RowBox[{
                            FormBox[
                              FormBox["a",
                                "TraditionalForm"],
                              "TraditionalForm"], "\[NoBreak]", 
                            FormBox[
                              FormBox[
                                InterpretationBox[
                                  SubscriptBox["c", "1"],
                                  Subscripted[ 
                                    c[ 1]],
                                  Editable->False],
                                "TraditionalForm"],
                              "TraditionalForm"], "\[NoBreak]", 
                            FormBox[
                              FormBox[
                                InterpretationBox[
                                  SubscriptBox["e", "1"],
                                  Subscripted[ 
                                    e[ 1]],
                                  Editable->False],
                                "TraditionalForm"],
                              "TraditionalForm"]}]], " ", 
                        SubscriptBox["f", 
                          RowBox[{
                            FormBox[
                              FormBox["b",
                                "TraditionalForm"],
                              "TraditionalForm"], "\[NoBreak]", 
                            FormBox[
                              FormBox[
                                InterpretationBox[
                                  SubscriptBox["c", "2"],
                                  Subscripted[ 
                                    c[ 2]],
                                  Editable->False],
                                "TraditionalForm"],
                              "TraditionalForm"], "\[NoBreak]", 
                            FormBox[
                              FormBox[
                                InterpretationBox[
                                  SubscriptBox["e", "1"],
                                  Subscripted[ 
                                    e[ 1]],
                                  Editable->False],
                                "TraditionalForm"],
                              "TraditionalForm"]}]]}]}],
                    HoldForm],
                  "TraditionalForm"]}],
              HoldForm],
            "TraditionalForm"]}], "-", 
        RowBox[{
          SubscriptBox["g", "s"], " ", 
          FormBox[
            TagBox[
              RowBox[{
                UnderoverscriptBox["\[Sum]", 
                  RowBox[{"i", "=", "0"}], 
                  RowBox[{"m", "-", "1"}]], 
                RowBox[{
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        SubscriptBox[
                          OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"]], ")"}], "i"], ".", 
                    SubsuperscriptBox[
                      FormBox["A",
                        "TraditionalForm"], 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["c", "1"],
                            Subscripted[ 
                              c[ 1]],
                            Editable->False],
                          "TraditionalForm"],
                        "TraditionalForm"]], ".", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        SubscriptBox[
                          OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"]], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "1"}]]}], " ", 
                  SubscriptBox["f", 
                    RowBox[{
                      FormBox[
                        FormBox["a",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["b",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox[
                          InterpretationBox[
                            SubscriptBox["c", "1"],
                            Subscripted[ 
                              c[ 1]],
                            Editable->False],
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}]}],
              HoldForm],
            "TraditionalForm"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    SuperscriptBox[
      RowBox[{"CovariantD", "[", 
        RowBox[{"OPEDelta", ",", "a", ",", "b"}], "]"}], "OPEm"]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        RowBox[{"(", 
          SubsuperscriptBox["D", 
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            RowBox[{"a", "\[NoBreak]", "b"}]], ")"}], "m"], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CovariantD", "[", 
      RowBox[{"OPEDelta", ",", 
        RowBox[{"{", 
          RowBox[{"OPEm", ",", "2"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          RowBox[{"(", 
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], ")"}], "m"], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
          SubscriptBox["g", "s"], " ", 
          FormBox[
            TagBox[
              RowBox[{
                UnderoverscriptBox["\[Sum]", 
                  RowBox[{"i", "=", "0"}], 
                  RowBox[{"m", "-", "1"}]], 
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox[
                        InterpretationBox[
                          SubscriptBox["c", "1"],
                          Subscripted[ 
                            c[ 1]],
                          Editable->False],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      SubscriptBox[
                        OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"]], ")"}], "i"], ".", 
                  SubsuperscriptBox[
                    FormBox["A",
                      "TraditionalForm"], 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox[
                        InterpretationBox[
                          SubscriptBox["c", "1"],
                          Subscripted[ 
                            c[ 1]],
                          Editable->False],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      SubscriptBox[
                        OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"]], ")"}], 
                    RowBox[{
                      RowBox[{"-", "i"}], "+", "m", "-", "1"}]]}]}],
              HoldForm],
            "TraditionalForm"]}], "-", 
        RowBox[{
          SubsuperscriptBox["g", "s", "2"], " ", 
          FormBox[
            TagBox[
              RowBox[{
                UnderoverscriptBox["\[Sum]", 
                  RowBox[{"j", "=", "0"}], 
                  RowBox[{"m", "-", "2"}]], 
                FormBox[
                  TagBox[
                    RowBox[{
                      UnderoverscriptBox["\[Sum]", 
                        RowBox[{"i", "=", "0"}], "j"], 
                      RowBox[{
                        SubscriptBox["T", 
                          FormBox[
                            FormBox[
                              InterpretationBox[
                                SubscriptBox["c", "1"],
                                Subscripted[ 
                                  c[ 1]],
                                Editable->False],
                              "TraditionalForm"],
                            "TraditionalForm"]], ".", 
                        SubscriptBox["T", 
                          FormBox[
                            FormBox[
                              InterpretationBox[
                                SubscriptBox["c", "2"],
                                Subscripted[ 
                                  c[ 2]],
                                Editable->False],
                              "TraditionalForm"],
                            "TraditionalForm"]], ".", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            SubscriptBox[
                              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"]], ")"}], "i"], ".", 
                        SubsuperscriptBox[
                          FormBox["A",
                            "TraditionalForm"], 
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], 
                          FormBox[
                            FormBox[
                              InterpretationBox[
                                SubscriptBox["c", "1"],
                                Subscripted[ 
                                  c[ 1]],
                                Editable->False],
                              "TraditionalForm"],
                            "TraditionalForm"]], ".", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            SubscriptBox[
                              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"]], ")"}], 
                          RowBox[{"j", "-", "i"}]], ".", 
                        SubsuperscriptBox[
                          FormBox["A",
                            "TraditionalForm"], 
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], 
                          FormBox[
                            FormBox[
                              InterpretationBox[
                                SubscriptBox["c", "2"],
                                Subscripted[ 
                                  c[ 2]],
                                Editable->False],
                              "TraditionalForm"],
                            "TraditionalForm"]], ".", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            SubscriptBox[
                              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"]], ")"}], 
                          RowBox[{
                            RowBox[{"-", "j"}], "+", "m", "-", "2"}]]}]}],
                    HoldForm],
                  "TraditionalForm"]}],
              HoldForm],
            "TraditionalForm"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"CovariantD", "[", 
        RowBox[{"OPEDelta", ",", 
          RowBox[{"Explicit", "->", "True"}]}], "]"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"-", "I"}], " ", "Gstrong", " ", 
        RowBox[{
          RowBox[{"SUNT", "[", 
            RowBox[{"SUNIndex", "[", 
              InterpretationBox[
                SubscriptBox["c", "4"],
                Subscripted[ 
                  c[ 4]],
                Editable->False], "]"}], "]"}], ".", 
          RowBox[{"QuantumField", "[", 
            RowBox[{"GaugeField", ",", 
              RowBox[{"Momentum", "[", "OPEDelta", "]"}], ",", 
              RowBox[{"SUNIndex", "[", 
                InterpretationBox[
                  SubscriptBox["c", "4"],
                  Subscripted[ 
                    c[ 4]],
                  Editable->False], "]"}]}], "]"}]}]}], "+", 
      RowBox[{"RightPartialD", "[", 
        RowBox[{"Momentum", "[", "OPEDelta", "]"}], "]"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"CovariantD", "[", 
        RowBox[{"\[Mu]", ",", "a", ",", "b", ",", 
          RowBox[{"Explicit", "->", "True"}]}], "]"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"RightPartialD", "[", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], " ", 
        RowBox[{"SUNDelta", "[", 
          RowBox[{"a", ",", "b"}], "]"}]}], "-", 
      RowBox[{"Gstrong", " ", 
        RowBox[{"QuantumField", "[", 
          RowBox[{"GaugeField", ",", 
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
            RowBox[{"SUNIndex", "[", 
              InterpretationBox[
                SubscriptBox["c", "5"],
                Subscripted[ 
                  c[ 5]],
                Editable->False], "]"}]}], "]"}], " ", 
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", 
            InterpretationBox[
              SubscriptBox["c", "5"],
              Subscripted[ 
                c[ 5]],
              Editable->False]}], "]"}]}]}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["D0", "Subsection",
  CellTags->"D0"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "D0[ p10, p12, p23, p30, p20, p13, m1^2, m2^2, m3^2, m4^2 ] is the \
Passarino-Veltman ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["D", "0"], TraditionalForm]]],
  " function. The convention for the arguments is that if the denominator of \
the integrand has the form ([q^2-m1^2] [(q+p1)^2-m2^2] [(q+p2)^2-m3^2] \
[(q+p3)^2-m4^2] ), the first six arguments of D0 are the scalar products p10 \
= p1^2, p12 = (p1-p2)^2, p23 = (p2-p3)^2, p30 = p3^2, p20 = p2^2, p13 = \
(p1-p3)^2."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["B0",
    ButtonData:>"B0",
    ButtonStyle->"Hyperlink",
    ButtonNote->"B0"],
  ", ",
  ButtonBox["C0",
    ButtonData:>"C0",
    ButtonStyle->"Hyperlink",
    ButtonNote->"C0"],
  ", ",
  ButtonBox["PaVe",
    ButtonData:>"PaVe",
    ButtonStyle->"Hyperlink",
    ButtonNote->"PaVe"],
  ", ",
  ButtonBox["PaVeOrder",
    ButtonData:>"PaVeOrder",
    ButtonStyle->"Hyperlink",
    ButtonNote->"PaVeOrder"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"D0", "[", 
      RowBox[{
      "p10", ",", "p12", ",", "p23", ",", "p30", ",", "p20", ",", "p13", 
        ",", 
        RowBox[{"m1", "^", "2"}], ",", 
        RowBox[{"m2", "^", "2"}], ",", 
        RowBox[{"m3", "^", "2"}], ",", 
        RowBox[{"m4", "^", "2"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "p10", 
        "\[NoBreak]", ",", "\[NoBreak]", "p12", "\[NoBreak]", ",", 
        "\[NoBreak]", "p23", "\[NoBreak]", ",", "\[NoBreak]", "p30", 
        "\[NoBreak]", ",", "\[NoBreak]", "p20", "\[NoBreak]", ",", 
        "\[NoBreak]", "p13", "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m1", "2"],
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m2", "2"],
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m3", "2"],
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m4", "2"],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PaVeOrder", "[", 
      RowBox[{
        RowBox[{"D0", "[", 
          RowBox[{
          "p10", ",", "p12", ",", "p23", ",", "p30", ",", "p20", ",", "p13", 
            ",", 
            RowBox[{"m1", "^", "2"}], ",", 
            RowBox[{"m2", "^", "2"}], ",", 
            RowBox[{"m3", "^", "2"}], ",", 
            RowBox[{"m4", "^", "2"}]}], "]"}], ",", 
        RowBox[{"PaVeOrderList", "->", 
          RowBox[{"{", 
            RowBox[{"p13", ",", "p20"}], "}"}]}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "p10", 
        "\[NoBreak]", ",", "\[NoBreak]", "p30", "\[NoBreak]", ",", 
        "\[NoBreak]", "p23", "\[NoBreak]", ",", "\[NoBreak]", "p12", 
        "\[NoBreak]", ",", "\[NoBreak]", "p13", "\[NoBreak]", ",", 
        "\[NoBreak]", "p20", "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m2", "2"],
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m1", "2"],
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m4", "2"],
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m3", "2"],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PaVeOrder", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "p10", 
        "\[NoBreak]", ",", "\[NoBreak]", "p12", "\[NoBreak]", ",", 
        "\[NoBreak]", "p23", "\[NoBreak]", ",", "\[NoBreak]", "p30", 
        "\[NoBreak]", ",", "\[NoBreak]", "p20", "\[NoBreak]", ",", 
        "\[NoBreak]", "p13", "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m1", "2"],
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m2", "2"],
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m3", "2"],
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox[
          SuperscriptBox["m4", "2"],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["DataType", "Subsection",
  CellTags->"DataType"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
DataType[exp, type] = True defines the object exp to have data-type \
type. DataType[exp1, exp2, ..., type] defines the objects exp1, exp2, ...to \
have data-type type. The default setting is DataType[__, _] := False. To \
assign a certain data-type, do, e.g., DataType[x, PositiveInteger] = \
True.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DeclareNonCommutative",
    ButtonData:>"DeclareNonCommutative",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DeclareNonCommutative"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell["NonCommutative is just a data-type.", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"DataType", "[", 
          RowBox[{"f", ",", "g", ",", " ", "NonCommutative"}], "]"}], " ", 
        "=", " ", "True"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{
        RowBox[{"f", ".", "g"}], "-", 
        RowBox[{"g", ".", 
          RowBox[{"(", 
            RowBox[{"2", "a"}], ")"}], ".", "f"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"f", ".", "g"}], "-", 
        RowBox[{"g", ".", 
          RowBox[{"(", 
            RowBox[{"2", " ", "a"}], ")"}], ".", "f"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell["\<\
Since \"f \"and \"g\" have DataType NonCommutative the function \
DotSimplify extracts only \"a\" out of the noncommutative product.\
\>", 
  "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotSimplify", "[", "t", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"f", ".", "g"}], "-", 
        RowBox[{"2", " ", "a", " ", 
          RowBox[{"g", ".", "f"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"DataType", "[", 
          RowBox[{"m", ",", "odd"}], "]"}], "=", 
        RowBox[{
          RowBox[{"DataType", "[", 
            RowBox[{"a", ",", "even"}], "]"}], "=", "True"}]}], ";"}]], 
  "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"ptest1", "[", "x_", "]"}], ":=", 
        RowBox[{"x", "/.", 
          RowBox[{
            RowBox[{
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "n_"], "/;", 
              RowBox[{"DataType", "[", 
                RowBox[{"n", ",", "odd"}], "]"}]}], "\[RuleDelayed]", 
            RowBox[{"-", "1"}]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"ptest2", "[", "x_", "]"}], ":=", 
        RowBox[{"x", "/.", 
          RowBox[{
            RowBox[{
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "n_"], "/;", 
              RowBox[{"DataType", "[", 
                RowBox[{"n", ",", "even"}], "]"}]}], "\[RuleDelayed]", 
            "1"}]}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "^", "m"}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "^", "a"}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "^", "z"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "a"], "+", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "m"], "+", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "z"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ptest1", "[", "t", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "1"}], "+", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "a"], "+", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "z"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ptest2", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "z"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", 
        RowBox[{"ptest1", ",", "ptest2", ",", "t", ",", "a", ",", "m"}], 
        "]"}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"DataType", "[", 
          RowBox[{"m", ",", "ganzeZahl"}], "]"}], "=", "True"}], ";"}]], 
  "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"f", "[", "x_", "]"}], ":=", 
        RowBox[{"x", "/.", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"-", "1"}], ")"}], "^", "p_"}], "/;", 
                RowBox[{"DataType", "[", 
                  RowBox[{"p", ",", "ganzeZahl"}], "]"}]}], ":>", "1"}], 
            "}"}]}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"test", "=", 
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "^", "m"}], "+", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "^", "n"}], " ", "x"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "n"], " ", "x"}], "+", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "m"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"f", "[", "test", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "n"], " ", "x"}], "+", "1"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", 
        RowBox[{"f", ",", "test"}], "]"}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"DataType", "[", 
          RowBox[{"f", ",", "g", ",", " ", "NonCommutative"}], "]"}], " ", 
        "=", " ", "False"}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"DataType", "[", 
          RowBox[{"m", ",", "odd"}], "]"}], "=", 
        RowBox[{
          RowBox[{"DataType", "[", 
            RowBox[{"a", ",", "even"}], "]"}], "=", "False"}]}], ";"}]], 
  "Input"],

Cell["\<\
Certain FeynCalc objects have DataType PositiveInteger set to True.\
\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DataType", "[", 
      RowBox[{"OPEm", ",", "PositiveInteger"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["True", TraditionalForm]], "Output"]
}, Open  ]],

Cell["PowerSimplify uses the DataType information.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PowerSimplify", "[", " ", 
      RowBox[{
        RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "^", 
        RowBox[{"(", 
          RowBox[{"2", "OPEm"}], ")"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PowerSimplify", "[", " ", 
      RowBox[{
        RowBox[{"(", 
          RowBox[{"-", " ", 
            RowBox[{"SO", "[", "q", "]"}]}], ")"}], "^", "OPEm"}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "m"], " ", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[CapitalDelta]\"\>",
                "TraditionalForm"], 
              FormBox["\<\"\[CenterDot]\"\>",
                "TraditionalForm"], 
              FormBox["q",
                "TraditionalForm"]}], ")"}], "m"]}], TraditionalForm]], 
  "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["DeclareNonCommutative", "Subsection",
  CellTags->"DeclareNonCommutative"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
DeclareNonCommutative[a, b, ...] declares a,b, ... to be \
non-commutative, i.e., DataType[a,b, ..., NonCommutative] is set to \
True.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DataType",
    ButtonData:>"DataType",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DataType"],
  ", ",
  ButtonBox["UnDeclareNonCommutative",
    ButtonData:>"UnDeclareNonCommutative",
    ButtonStyle->"Hyperlink",
    ButtonNote->"UnDeclareNonCommutative"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[BoxData[
    RowBox[{"DeclareNonCommutative", "[", "x", "]"}]], "Input"],

Cell["\<\
As a side effect of DeclareNonCommutative x is declared to be of \
data type NonCommutative.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DataType", "[", 
      RowBox[{"x", ",", "NonCommutative"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["True", TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"DeclareNonCommutative", "[", 
      RowBox[{"y", ",", "z"}], "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DataType", "[", 
      RowBox[{"a", ",", "x", ",", "y", ",", "z", ",", "NonCommutative"}], 
      "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"False", ",", "True", ",", "True", ",", "True"}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"UnDeclareNonCommutative", "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DataType", "[", 
      RowBox[{"a", ",", "x", ",", "y", ",", "z", ",", "NonCommutative"}], 
      "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"False", ",", "False", ",", "False", ",", "False"}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["DeltaFunction", "Subsection",
  CellTags->"DeltaFunction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  " DeltaFunction[x] is the Dirac delta-function ",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[Delta]", "(", "x", ")"}], TraditionalForm]]],
  "."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Convolute",
    ButtonData:>"Convolute",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Convolute"],
  ", ",
  ButtonBox["DeltaFunctionPrime",
    ButtonData:>"DeltaFunctionPrime",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DeltaFunctionPrime"],
  ", ",
  ButtonBox["Integrate2",
    ButtonData:>"Integrate2",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Integrate2"],
  ", ",
  ButtonBox["SimplifyDeltaFunction",
    ButtonData:>"SimplifyDeltaFunction",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SimplifyDeltaFunction"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DeltaFunction", "[", 
      RowBox[{"1", "-", "x"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[Delta]", "(", 
        FormBox[
          RowBox[{"1", "-", "x"}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"DeltaFunction", "[", 
            RowBox[{"1", "-", "x"}], "]"}], " ", 
          RowBox[{"f", "[", "x", "]"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"f", "(", "1", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"DeltaFunction", "[", "x", "]"}], " ", 
          RowBox[{"f", "[", "x", "]"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"f", "(", "0", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"DeltaFunction", "[", 
            RowBox[{"1", "-", "x"}], "]"}], " ", 
          RowBox[{"f", "[", "x", "]"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"f", "(", "1", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Convolute", "[", 
      RowBox[{
        RowBox[{"DeltaFunction", "[", 
          RowBox[{"1", "-", "x"}], "]"}], ",", "x"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["x", TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["DeltaFunctionPrime", "Subsection",
  CellTags->"DeltaFunctionPrime"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  " DeltaFunctionPrime[1-x] is the derivative of the Dirac ",
  Cell[BoxData[
      FormBox["\[Delta]", TraditionalForm]]],
  " function."
}], "Text"],

Cell[TextData[{
  "See also: ",
  ButtonBox["Convolute",
    ButtonData:>"Convolute",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Convolute"],
  ", ",
  ButtonBox["DeltaFunction",
    ButtonData:>"DeltaFunction",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DeltaFunction"],
  ", ",
  ButtonBox["Integrate2",
    ButtonData:>"Integrate2",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Integrate2"],
  ", ",
  ButtonBox["SimplifyDeltaFunction",
    ButtonData:>"SimplifyDeltaFunction",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SimplifyDeltaFunction"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DeltaFunctionPrime", "[", 
      RowBox[{"1", "-", "x"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
        FormBox[
          RowBox[{"1", "-", "x"}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"DeltaFunctionPrime", "[", 
            RowBox[{"1", "-", "x"}], "]"}], " ", 
          RowBox[{"f", "[", "x", "]"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["f", "\[Prime]",
          MultilineFunction->None], "(", "1", ")"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"DeltaFunctionPrime", "[", 
            RowBox[{"1", "-", "x"}], "]"}], " ", 
          SuperscriptBox["x", "2"]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["2", TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Dimension", "Subsection",
  CellTags->"Dimension"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Dimension is an option of several functions and denotes the \
space-time dimension. Possible settings are: 4, n, d, D, ... ,the variable \
does not matter, but it should have Head Symbol.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "MetricTensor", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Dimension", "\[Rule]", "4"}], ",", 
          RowBox[{"FeynCalcInternal", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"MetricTensor", "[", 
          RowBox[{"m", ",", "n", ",", 
            RowBox[{"Dimension", "\[Rule]", "d"}]}], "]"}], 
        RowBox[{"DiracMatrix", "[", 
          RowBox[{"\[Alpha]", ",", 
            RowBox[{"Dimension", "\[Rule]", "d"}]}], "]"}]}], "//", "FCI"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Alpha]",
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SuperscriptBox["g", 
          RowBox[{
            FormBox[
              FormBox["m",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["n",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
The dimension of the indices is not shown by default but can be \
inspected easily.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"MetricTensor", "[", 
          RowBox[{"m", ",", "n", ",", 
            RowBox[{"Dimension", "\[Rule]", "d"}]}], "]"}], 
        RowBox[{"DiracMatrix", "[", 
          RowBox[{"\[Alpha]", ",", 
            RowBox[{"Dimension", "\[Rule]", "d"}]}], "]"}]}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracGamma", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Alpha]", ",", "d"}], "]"}], ",", "d"}], "]"}], " ", 
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"m", ",", "d"}], "]"}], ",", 
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"n", ",", "d"}], "]"}]}], "]"}]}]], "Output"]
}, Open  ]],

Cell["\<\
Setting the global variable $LorentzIndices to True will display \
the dimension (if different from 4) as a subscript.\
\>", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"$LorentzIndices", "=", "True"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"MetricTensor", "[", 
        RowBox[{"\[Alpha]", ",", "\[Beta]", ",", 
          RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}], 
      RowBox[{"DiracMatrix", "[", 
        RowBox[{"\[Alpha]", ",", 
          RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            SubscriptBox[
              FormBox["\[Alpha]",
                "TraditionalForm"], 
              FormBox["n",
                "TraditionalForm"]],
            "TraditionalForm"]], " ", 
        SuperscriptBox["g", 
          RowBox[{
            FormBox[
              SubscriptBox[
                FormBox["\[Alpha]",
                  "TraditionalForm"], 
                FormBox["n",
                  "TraditionalForm"]],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              SubscriptBox[
                FormBox["\[Beta]",
                  "TraditionalForm"], 
                FormBox["n",
                  "TraditionalForm"]],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracGamma", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Alpha]", ",", "n"}], "]"}], ",", "n"}], "]"}], " ", 
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Alpha]", ",", "n"}], "]"}], ",", 
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Beta]", ",", "n"}], "]"}]}], "]"}]}]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"$LorentzIndices", "=", "False"}], ";"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["DiracGammaCombine", "Subsection",
  CellTags->"DiracGammaCombine"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
DiracGammaCombine[exp] is (nearly) the inverse operation to \
DiracGammaExpand.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracGamma",
    ButtonData:>"DiracGamma",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracGamma"],
  ", ",
  ButtonBox["DiracGammaExpand",
    ButtonData:>"DiracGammaExpand",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracGammaExpand"],
  ", ",
  ButtonBox["DiracMatrix",
    ButtonData:>"DiracMatrix",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracMatrix"],
  ", ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracSimplify"],
  ", ",
  ButtonBox["DiracSlash",
    ButtonData:>"DiracSlash",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracSlash"],
  ", ",
  ButtonBox["DiracTrick",
    ButtonData:>"DiracTrick",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracTrick"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracGammaCombine", "[", 
      RowBox[{
        RowBox[{"GS", "[", "p", "]"}], " ", "+", " ", 
        RowBox[{"GS", "[", "q", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        "(", "\[NoBreak]", 
        FormBox[
          RowBox[{"p", "+", "q"}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    RowBox[{"DiracGamma", "[", 
      RowBox[{"Momentum", "[", 
        RowBox[{"p", "+", "q"}], "]"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracGammaCombine", "[", 
      RowBox[{
        RowBox[{"2", " ", 
          RowBox[{"GS", "[", "p", "]"}]}], " ", "-", " ", 
        RowBox[{"2", " ", 
          RowBox[{"GS", "[", "q", "]"}]}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        "(", "\[NoBreak]", 
        FormBox[
          RowBox[{
            RowBox[{"2", " ", "p"}], "-", 
            RowBox[{"2", " ", "q"}]}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    RowBox[{"DiracGamma", "[", 
      RowBox[{"Momentum", "[", 
        RowBox[{
          RowBox[{"2", " ", "p"}], "-", 
          RowBox[{"2", " ", "q"}]}], "]"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracGammaExpand", "[", "%%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"2", " ", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["p",
              "TraditionalForm"]}]}], "-", 
        RowBox[{"2", " ", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"]}]}]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["DiracGammaExpand", "Subsection",
  CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
DiracGammaExpand[exp] expands all DiracGamma[Momentum[a+b+..]] in \
exp into (DiracGamma[Momentum[a]] + DiracGamma[Momentum[b]] + ...).\
\>", 
  "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracGamma",
    ButtonData:>"DiracGamma",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracGamma"],
  ", ",
  ButtonBox["DiracGammaCombine",
    ButtonData:>"DiracGammaCombine",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracGammaCombine"],
  ", ",
  ButtonBox["DiracMatrix",
    ButtonData:>"DiracMatrix",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracMatrix"],
  ", ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracSimplify"],
  ", ",
  ButtonBox["DiracSlash",
    ButtonData:>"DiracSlash",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracSlash"],
  ", ",
  ButtonBox["DiracTrick",
    ButtonData:>"DiracTrick",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracTrick"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{
        RowBox[{"DiracGamma", "[", 
          RowBox[{"Momentum", "[", "q", "]"}], "]"}], " ", ".", " ", 
        RowBox[{"DiracGamma", "[", 
          RowBox[{"Momentum", "[", 
            RowBox[{"p", "-", "q"}], "]"}], "]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"]}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            "(", "\[NoBreak]", 
            FormBox[
              RowBox[{"p", "-", "q"}],
              "TraditionalForm"], "\[NoBreak]", ")"}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
Momentum is the head of p-q, i.e., it is treated as one \
four-momentum.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", "t", "]"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracGamma", "[", 
        RowBox[{"Momentum", "[", "q", "]"}], "]"}], ".", 
      RowBox[{"DiracGamma", "[", 
        RowBox[{"Momentum", "[", 
          RowBox[{"p", "-", "q"}], "]"}], "]"}]}]], "Output"]
}, Open  ]],

Cell["With DiracGammaExpand the Momentum[p-q] gets expanded.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracGammaExpand", "[", "t", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"]}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["p",
                "TraditionalForm"]}], "-", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["q",
                "TraditionalForm"]}]}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["The inverse operation is DiracGammaCombine.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"DiracGammaCombine", "[", 
        RowBox[{"DiracGammaExpand", "[", "t", "]"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracGamma", "[", 
        RowBox[{"Momentum", "[", "q", "]"}], "]"}], ".", 
      RowBox[{"DiracGamma", "[", 
        RowBox[{"Momentum", "[", 
          RowBox[{"p", "-", "q"}], "]"}], "]"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"DiracGammaExpand", "[", "t", "]"}], "]"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracGamma", "[", 
        RowBox[{"Momentum", "[", "q", "]"}], "]"}], ".", 
      RowBox[{"(", 
        RowBox[{
          RowBox[{"DiracGamma", "[", 
            RowBox[{"Momentum", "[", "p", "]"}], "]"}], "-", 
          RowBox[{"DiracGamma", "[", 
            RowBox[{"Momentum", "[", "q", "]"}], "]"}]}], ")"}]}]], "Output"]
}, Open  ]],

Cell["In order to do non-commutative expansion use DiracSimplify.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSimplify", "[", "t", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["q",
                "TraditionalForm"]}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["p",
                "TraditionalForm"]}], ")"}]}], "-", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], "2"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", "t", "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["DiracGamma", "Subsection",
  CellTags->"DiracGamma"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "DiracGamma[x, dim] is the head of all Dirac matrices and slashes (in the \
internal representation). Use DiracMatrix (or GA, GAD) and DiracSlash (or GS, \
GSD) for manual (short) input. DiracGamma[x, 4] simplifies to DiracGamma[x]. \
DiracGamma[5] is ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  ". DiracGamma[6] is ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"(", 
            RowBox[{"1", "+", 
              SuperscriptBox["\[Gamma]", "5"]}], ")"}], "/", "2."}], 
        TraditionalForm]]],
  " DiracGamma[7] is ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"(", 
            RowBox[{"1", "-", 
              SuperscriptBox["\[Gamma]", "5"]}], ")"}], "/", "2."}], 
        TraditionalForm]]]
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracGammaExpand",
    ButtonData:>"DiracGammaExpand",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracGammaExpand"],
  ", ",
  ButtonBox["DiracMatrix",
    ButtonData:>"DiracMatrix",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracMatrix"],
  ", ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracSimplify"],
  ", ",
  ButtonBox["DiracSlash",
    ButtonData:>"DiracSlash",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracSlash"],
  ", ",
  ButtonBox["DiracTrick",
    ButtonData:>"DiracTrick",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracTrick"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracGamma", "[", "5", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Gamma]", "5"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracGamma", "[", 
      RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Gamma]", 
        FormBox[
          FormBox["\[Alpha]",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "A Dirac-slash, i.e., ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Mu]"], 
          SubscriptBox["q", "\[Mu]"]}], TraditionalForm]]],
  ", is displayed as ",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], TraditionalForm]]],
  "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracGamma", "[", 
        RowBox[{"Momentum", "[", "q", "]"}], "]"}], " "}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        FormBox["q",
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracGamma", "[", 
        RowBox[{"Momentum", "[", "q", "]"}], "]"}], " ", ".", " ", 
      RowBox[{"DiracGamma", "[", 
        RowBox[{"Momentum", "[", 
          RowBox[{"p", "-", "q"}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"]}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            "(", "\[NoBreak]", 
            FormBox[
              RowBox[{"p", "-", "q"}],
              "TraditionalForm"], "\[NoBreak]", ")"}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracGamma", "[", 
        RowBox[{
          RowBox[{"Momentum", "[", 
            RowBox[{"q", ",", "D"}], "]"}], ",", "D"}], "]"}], " "}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        FormBox["q",
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a1", "=", 
      RowBox[{
        RowBox[{"GS", "[", 
          RowBox[{"p", "-", "q"}], "]"}], ".", 
        RowBox[{"GS", "[", "p", "]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "(", 
            "\[NoBreak]", 
            FormBox[
              RowBox[{"p", "-", "q"}],
              "TraditionalForm"], "\[NoBreak]", ")"}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
          ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a2", "=", 
      RowBox[{"DiracGammaExpand", "[", "a1", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["p",
                "TraditionalForm"]}], "-", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["q",
                "TraditionalForm"]}]}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["p",
              "TraditionalForm"]}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a3", "=", 
      RowBox[{
        RowBox[{"GAD", "[", "\[Mu]", "]"}], ".", 
        RowBox[{"GSD", "[", 
          RowBox[{"p", "-", "q"}], "]"}], ".", 
        RowBox[{"GSD", "[", "q", "]"}], ".", 
        RowBox[{"GAD", "[", "\[Mu]", "]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "(", 
            "\[NoBreak]", 
            FormBox[
              RowBox[{"p", "-", "q"}],
              "TraditionalForm"], "\[NoBreak]", ")"}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "q"}], 
          ")"}], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a4", "=", 
      RowBox[{"DiracTrick", "[", "a3", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{"D", "-", "4"}], ")"}], " ", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", "(", "\[NoBreak]", 
                FormBox[
                  RowBox[{"p", "-", "q"}],
                  "TraditionalForm"], "\[NoBreak]", ")"}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["q",
                  "TraditionalForm"]}], ")"}]}]}], "+", 
        RowBox[{"4", " ", 
          RowBox[{"(", "\[NoBreak]", 
            FormBox[
              RowBox[{"p", "-", "q"}],
              "TraditionalForm"], "\[NoBreak]", ")", "\[NoBreak]", 
            "\[CenterDot]", "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"]}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a5", "=", 
      RowBox[{"DiracSimplify", "[", "a4", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"D", " ", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["q",
                  "TraditionalForm"]}], ")"}]}]}], "-", 
        RowBox[{"4", " ", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["q",
                  "TraditionalForm"]}], ")"}]}]}], "+", 
        RowBox[{"4", " ", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"]}]}], "-", 
        RowBox[{"D", " ", 
          SuperscriptBox[
            FormBox["q",
              "TraditionalForm"], "2"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"a1", ",", "a2", ",", "a3", ",", "a4", ",", "a5"}], "]"}]], 
  "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["DiracGammaT", "Subsection",
  CellTags->"DiracGammaT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
DiracGammaT[x] denotes the transpose of DiracGamma[x]. \
Transpose[DiracGammaT[x]] gives DiracGamma[x]. Note that x must have Head \
LorentzIndex or Momentum.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracGamma",
    ButtonData:>"DiracGamma",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracGamma"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracGammaT", "[", 
      RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox["\[Gamma]", 
        FormBox[
          FormBox["\[Mu]",
            "TraditionalForm"],
          "TraditionalForm"], "T"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Transpose", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Gamma]", 
        FormBox[
          FormBox["\[Mu]",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"GS", "[", "p", "]"}], "//", "FCI"}], "//", "Transpose"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        RowBox[{
        "(", "\[NoBreak]", "\[Gamma]", "\[NoBreak]", "\[CenterDot]", 
          "\[NoBreak]", 
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", ")"}], "T"], TraditionalForm]], 
  "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["DiracMatrix", "Subsection",
  CellTags->"DiracMatrix"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "DiracMatrix[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "] denotes a Dirac gamma matrix with Lorentz index ",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  ". DiracMatrix[",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"\[Mu]", ",", "\[Nu]", ","}], " "}], TraditionalForm]]],
  "...] is a product of ",
  Cell[BoxData[
      FormBox["\[Gamma]", TraditionalForm]]],
  " matrices with Lorentz indices ",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[Mu]", ",", " ", "\[Nu]", ",", " ", "..."}], TraditionalForm
        ]]],
  " DiracMatrix[5] is ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  ". DiracMatrix[6] is ",
  Cell[BoxData[
      FormBox[
        RowBox[{"1", "/", "2"}], TraditionalForm]]],
  Cell[BoxData[
      FormBox[
        RowBox[{" ", 
          RowBox[{
            RowBox[{"+", " ", 
              SuperscriptBox["\[Gamma]", "5"]}], "/", "2"}]}], TraditionalForm
        ]]],
  ". DiracMatrix[7] is",
  Cell[BoxData[
      FormBox[
        RowBox[{" ", "1", "/", "2"}], TraditionalForm]]],
  Cell[BoxData[
      FormBox[
        RowBox[{" ", 
          RowBox[{
            RowBox[{"-", " ", 
              SuperscriptBox["\[Gamma]", "5"]}], "/", "2"}]}], TraditionalForm
        ]]],
  "."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracGammaExpand",
    ButtonData:>"DiracGammaExpand",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracGammaExpand"],
  ", ",
  ButtonBox["DiracGamma",
    ButtonData:>"DiracGamma",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracGamma"],
  ", ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracSimplify"],
  ", ",
  ButtonBox["DiracSlash",
    ButtonData:>"DiracSlash",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracSlash"],
  ", ",
  ButtonBox["DiracTrick",
    ButtonData:>"DiracTrick",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracTrick"],
  ", ",
  ButtonBox["GA",
    ButtonData:>"GA",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GA"],
  ", ",
  ButtonBox["GAD",
    ButtonData:>"GAD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GAD"],
  ", ",
  ButtonBox["GS",
    ButtonData:>"GS",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GS"],
  ", ",
  ButtonBox["GSD",
    ButtonData:>"GSD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GSD"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Gamma]", 
        FormBox[
          FormBox["\[Mu]",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "This is how to enter the non-commutative product of two ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]]}], TraditionalForm]]],
  ". The ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Dot \".\" is used as non-commutative multiplication operator."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
      RowBox[{"DiracMatrix", "[", "\[Nu]", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracMatrix", "[", "\[Alpha]", "]"}], "//", "StandardForm"}]], 
  "Input"],

Cell[BoxData[
    RowBox[{"DiracGamma", "[", 
      RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], "//", "FCE"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Gamma]", "\[Mu]"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"GA", "[", "\[Mu]", "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GAD", "[", "\[Mu]", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Gamma]", "\[Mu]"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"%", "//", "FCI"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"DiracGamma", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", "D"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GA", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Rho]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"GA", "[", 
        RowBox[{"a", " ", ".", " ", "b"}], "]"}], "//", "FCI"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["a",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["b",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracGamma", "[", 
        RowBox[{"LorentzIndex", "[", "a", "]"}], "]"}], ".", 
      RowBox[{"DiracGamma", "[", 
        RowBox[{"LorentzIndex", "[", "b", "]"}], "]"}]}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["DiracOrder", "Subsection",
  CellTags->"DiracOrder"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
DiracOrder[expr] orders the Dirac matrices in expr alphabetically. \
DiracOrder[expr,orderlist] orders the Dirac matrices in expr according to \
orderlist.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracSimplify"],
  ", ",
  ButtonBox["DiracTrick",
    ButtonData:>"DiracTrick",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracTrick"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", " ", "=", " ", 
      RowBox[{"GA", "[", 
        RowBox[{"\[Beta]", ",", "\[Alpha]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Beta]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Alpha]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracOrder", "[", "t1", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"2", " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]]}], "-", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Alpha]",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Beta]",
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["This is a string of Dirac matrices in D dimensions. ", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{"GAD", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracOrder", "[", "t1", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"2", " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]]}], "-", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Alpha]",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Beta]",
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t3", "=", 
      RowBox[{"GA", "[", 
        RowBox[{"5", ",", "\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "5"], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "By default ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox["\[Gamma]", "5"], " "}], TraditionalForm]]],
  "is moved to the right."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracOrder", "[", "t3", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t4", "=", 
      RowBox[{"GA", "[", 
        RowBox[{"6", ",", "\[Mu]", ",", "7"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "6"], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracOrder", "[", "t4", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t5", "=", 
      RowBox[{"GA", "[", 
        RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Delta]"}], "]"}]}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Alpha]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Beta]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Delta]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "This orders the ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Alpha]"], 
          SuperscriptBox["\[Gamma]", "\[Beta]"], 
          SuperscriptBox["\[Gamma]", "\[Delta]"]}], TraditionalForm]]],
  " in reverse order."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracOrder", "[", 
      RowBox[{"t5", ",", 
        RowBox[{"{", 
          RowBox[{"\[Delta]", ",", "\[Beta]", ",", "\[Alpha]"}], "}"}]}], 
      "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Delta]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Beta]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Alpha]",
                  "TraditionalForm"],
                "TraditionalForm"]]}]}], "+", 
        RowBox[{"2", " ", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Delta]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]]}], "-", 
        RowBox[{"2", " ", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Beta]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Delta]"}]]}], "+", 
        RowBox[{"2", " ", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Alpha]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Beta]", "\[NoBreak]", "\[Delta]"}]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracOrder", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Alpha]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Beta]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Delta]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", 
        RowBox[{"t1", ",", "t2", ",", "t3", ",", "t4", ",", "t5"}], "]"}], 
      ";"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["DiracReduce", "Subsection",
  CellTags->"DiracReduce"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
DiracReduce[exp] reduces all four-dimensional Dirac matrices in exp \
to the standard basis (S,P,V,A,T) using the Chisholm identity (see Chisholm). \
In the result the basic Dirac structures are wrapped with a head DiracBasis. \
I.e., S corresponds to DiracBasis[1], P : DiracBasis[DiracMatrix[5]], V: \
DiracBasis[DiracMatrix[mu]], A: DiracBasis[DiracMatrix[mu, 5]], T: \
DiracBasis[DiracSigma[DiracMatrix[mu, nu]]]. By default DiracBasis is \
substituted to Identity. Notice that the result of DiracReduce is given in \
the FeynCalcExternal - way, i.e.,evtl. you may have to use FeynCalcInternal \
on the result.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "DiracReduce", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
          RowBox[{"FinalSubstitutions", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{"DiracBasis", "\[Rule]", "Identity"}], "}"}]}]}], 
        "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracSimplify"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", " ", "=", " ", 
      RowBox[{"GA", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracReduce", "[", "t1", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
          SuperscriptBox["\[Sigma]", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{"DiracMatrix", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], 
        SuperscriptBox["\[Gamma]", "\[Nu]"], 
        SuperscriptBox["\[Gamma]", "\[Rho]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracReduce", "[", "t2", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              RowBox[{"$MU", "(", "1", ")"}]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
            "\[Mu]", "\[NoBreak]", "\[Nu]", "\[NoBreak]", "\[Rho]", 
              "\[NoBreak]", 
              FormBox[
                RowBox[{"$MU", "(", "1", ")"}],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Rho]"], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], "-", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Nu]"], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]]}], "+", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Mu]"], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t3", "=", 
      RowBox[{"DiracMatrix", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
        "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], 
        SuperscriptBox["\[Gamma]", "\[Nu]"], 
        SuperscriptBox["\[Gamma]", "\[Rho]"], 
        SuperscriptBox["\[Gamma]", "\[Sigma]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t4", "=", 
      RowBox[{"DiracReduce", "[", "t3", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", "\[ImaginaryI]"}], " ", 
          SuperscriptBox["\[Gamma]", "5"], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
            "\[Mu]", "\[NoBreak]", "\[Nu]", "\[NoBreak]", "\[Rho]", 
              "\[NoBreak]", "\[Sigma]"}]]}], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
          SuperscriptBox["\[Sigma]", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], "+", 
        RowBox[{"\[ImaginaryI]", " ", 
          SuperscriptBox["\[Sigma]", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
          SuperscriptBox["\[Sigma]", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]]}], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
          SuperscriptBox["\[Sigma]", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "+", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "+", 
        RowBox[{"\[ImaginaryI]", " ", 
          SuperscriptBox["\[Sigma]", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]]}], "-", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]]}], "-", 
        RowBox[{"\[ImaginaryI]", " ", 
          SuperscriptBox["\[Sigma]", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}], "+", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t5", " ", "=", 
      RowBox[{"Calc", "[", 
        RowBox[{"DiracSimplify", "[", 
          RowBox[{"DiracSigmaExplicit", "[", 
            RowBox[{"t4", ".", "t4"}], "]"}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", "128"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", 
      RowBox[{"t4", ".", "t4"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", "128"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"t1", ",", "t2", ",", "t3", ",", "t4"}], "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["DiracSigmaExplicit", "Subsection",
  CellTags->"DiracSigmaExplicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
DiracSigmaExplicit[exp] inserts in exp for all DiracSigma its \
definition. DiracSigmaExplict is also an option of DiracSimplify.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracSigma",
    ButtonData:>"DiracSigma",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracSigma"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSigma", "[", 
      RowBox[{
        RowBox[{"GA", "[", "\[Alpha]", "]"}], ",", 
        RowBox[{"GA", "[", "\[Beta]", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Sigma]", 
        RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSigmaExplicit", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SuperscriptBox["\[Gamma]", 
                FormBox[
                  FormBox["\[Alpha]",
                    "TraditionalForm"],
                  "TraditionalForm"]], ".", 
              SuperscriptBox["\[Gamma]", 
                FormBox[
                  FormBox["\[Beta]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "-", 
            RowBox[{
              SuperscriptBox["\[Gamma]", 
                FormBox[
                  FormBox["\[Beta]",
                    "TraditionalForm"],
                  "TraditionalForm"]], ".", 
              SuperscriptBox["\[Gamma]", 
                FormBox[
                  FormBox["\[Alpha]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}]}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["DiracSigma", "Subsection",
  CellTags->"DiracSigma"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
DiracSigma[a, b] stands for i/2*(a . b - b . a) in 4 dimensions. a \
and b must have Head DiracGamma, DiracMatrix or DiracSlash. Only antisymmetry \
is implemented.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracSigmaExplicit",
    ButtonData:>"DiracSigmaExplicit",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracSigmaExplicit"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{"DiracSigma", "[", 
        RowBox[{
          RowBox[{"GA", "[", "\[Alpha]", "]"}], ",", 
          RowBox[{"GA", "[", "\[Beta]", "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Sigma]", 
        RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSigmaExplicit", "[", "t1", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SuperscriptBox["\[Gamma]", 
                FormBox[
                  FormBox["\[Alpha]",
                    "TraditionalForm"],
                  "TraditionalForm"]], ".", 
              SuperscriptBox["\[Gamma]", 
                FormBox[
                  FormBox["\[Beta]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "-", 
            RowBox[{
              SuperscriptBox["\[Gamma]", 
                FormBox[
                  FormBox["\[Beta]",
                    "TraditionalForm"],
                  "TraditionalForm"]], ".", 
              SuperscriptBox["\[Gamma]", 
                FormBox[
                  FormBox["\[Alpha]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}]}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{"DiracSigma", "[", 
        RowBox[{
          RowBox[{"GA", "[", "\[Beta]", "]"}], ",", 
          RowBox[{"GA", "[", "\[Alpha]", "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        SuperscriptBox["\[Sigma]", 
          RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t3", "=", 
      RowBox[{"DiracSigma", "[", 
        RowBox[{
          RowBox[{"GS", "[", "p", "]"}], ",", 
          RowBox[{"GS", "[", "q", "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Sigma]", 
        RowBox[{"p", "\[NoBreak]", "q"}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSigmaExplicit", "[", "t3", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  FormBox["\<\"\[Gamma]\"\>",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  FormBox["\<\"\[Gamma]\"\>",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], ")"}]}], "-", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  FormBox["\<\"\[Gamma]\"\>",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  FormBox["\<\"\[Gamma]\"\>",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}], ")"}]}]}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"t1", ",", "t2", ",", "t3"}], "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["DiracSimplify", "Subsection",
  CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
DiracSimplify[expr] simplifies products of Dirac matrices in expr \
and expands non-commutative products. Double Lorentz indices and four vectors \
are contracted. The Dirac equation is applied. All DiracMatrix[5], \
DiracMatrix[6] and DiracMatrix[7] are moved to the right. The order of the \
other Dirac matrices is not changed.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "DiracSimplify", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"DiracCanonical", "\[Rule]", "False"}], ",", 
          RowBox[{"DiracSigmaExplicit", "\[Rule]", "True"}], ",", 
          RowBox[{"DiracSimpCombine", "\[Rule]", "False"}], ",", 
          RowBox[{"DiracSubstitute67", "\[Rule]", "False"}], ",", 
          RowBox[{"Expanding", "\[Rule]", "True"}], ",", 
          RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
          RowBox[{"FeynCalcInternal", "\[Rule]", "False"}], ",", 
          RowBox[{"InsideDiracTrace", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Calc",
    ButtonData:>"Calc",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Calc"],
  ", ",
  ButtonBox["DiracGammaExpand",
    ButtonData:>"DiracGammaExpand",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracGammaExpand"],
  ", ",
  ButtonBox["DiracTrick",
    ButtonData:>"DiracTrick",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracTrick"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell["This is a string of Dirac matrices in four dimensions. ", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{"GA", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSimplify", "[", "t1", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "2"}], " ", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["This is a string of Dirac matrices in D dimensions. ", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{"GAD", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSimplify", "[", "t1", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "2"}], " ", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t3", "=", 
      RowBox[{"GA", "[", 
        RowBox[{"5", ",", "\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "5"], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "By default ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  "is moved to the right."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSimplify", "[", "t3", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t4", "=", 
      RowBox[{"GA", "[", 
        RowBox[{"6", ",", "\[Mu]", ",", "7"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "6"], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSimplify", "[", "t4", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t5", "=", 
      RowBox[{
        RowBox[{"GS", "[", 
          RowBox[{"a", "+", "b"}], "]"}], " ", ".", " ", 
        RowBox[{"GS", "[", "p", "]"}], ".", 
        RowBox[{"GS", "[", "p", "]"}], ".", 
        RowBox[{"GS", "[", 
          RowBox[{"c", "+", "d"}], "]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "(", 
            "\[NoBreak]", 
            FormBox[
              RowBox[{"a", "+", "b"}],
              "TraditionalForm"], "\[NoBreak]", ")"}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "(", 
            "\[NoBreak]", 
            FormBox[
              RowBox[{"c", "+", "d"}],
              "TraditionalForm"], "\[NoBreak]", ")"}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
Contrary to DiracTrick DiracSimplify does non-commutative \
expansion.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSimplify", "[", "t5", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["a",
                  "TraditionalForm"]}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["c",
                  "TraditionalForm"]}], ")"}]}], " ", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"]}], "+", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["a",
                  "TraditionalForm"]}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["d",
                  "TraditionalForm"]}], ")"}]}], " ", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"]}], "+", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["b",
                  "TraditionalForm"]}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["c",
                  "TraditionalForm"]}], ")"}]}], " ", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"]}], "+", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["b",
                  "TraditionalForm"]}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["d",
                  "TraditionalForm"]}], ")"}]}], " ", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrick", "[", "t5", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", "(", "\[NoBreak]", 
              FormBox[
                RowBox[{"a", "+", "b"}],
                "TraditionalForm"], "\[NoBreak]", ")"}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", "(", "\[NoBreak]", 
              FormBox[
                RowBox[{"c", "+", "d"}],
                "TraditionalForm"], "\[NoBreak]", ")"}], ")"}]}], " ", 
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], "2"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t6", " ", "=", " ", 
      RowBox[{
        RowBox[{"SpinorVBar", "[", "p", "]"}], " ", ".", " ", 
        RowBox[{"GS", "[", "p", "]"}], " ", ".", " ", 
        RowBox[{"SpinorUBar", "[", "q", "]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FormBox[
            OverscriptBox["\<\"v\"\>", "_"],
            "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "p", 
          "\[NoBreak]", ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
          ")"}], ".", 
        RowBox[{
          FormBox[
            OverscriptBox["\<\"u\"\>", "_"],
            "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "q", 
          "\[NoBreak]", ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSimplify", "[", "t6", "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GAD", "@@", 
      RowBox[{"Join", "[", 
        RowBox[{
          RowBox[{"{", "\[Mu]", "}"}], ",", 
          RowBox[{"Table", "[", 
            RowBox[{
              SubscriptBox["\[Nu]", "i"], ",", 
              RowBox[{"{", 
                RowBox[{"i", ",", "6"}], "}"}]}], "]"}], ",", 
          RowBox[{"{", "\[Mu]", "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", 
          SubscriptBox["\[Nu]", "1"]], ".", 
        SuperscriptBox["\[Gamma]", 
          SubscriptBox["\[Nu]", "2"]], ".", 
        SuperscriptBox["\[Gamma]", 
          SubscriptBox["\[Nu]", "3"]], ".", 
        SuperscriptBox["\[Gamma]", 
          SubscriptBox["\[Nu]", "4"]], ".", 
        SuperscriptBox["\[Gamma]", 
          SubscriptBox["\[Nu]", "5"]], ".", 
        SuperscriptBox["\[Gamma]", 
          SubscriptBox["\[Nu]", "6"]], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSimplify", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"D", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]]}]}], "-", 
        RowBox[{"12", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]]}]}], "+", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{"4", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
 With the option DiracCanonical an alphabetic ordering is \
done.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSimplify", "[", 
      RowBox[{
        RowBox[{"GA", "[", 
          RowBox[{"\[Nu]", ",", "\[Mu]"}], "]"}], ",", 
        RowBox[{"DiracCanonical", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"2", " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], "-", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[
"Setting InsideDiracTrace\[Rule]True assumes that a trace is still to be \
taken later on."], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSimplify", "[", 
      RowBox[{
        RowBox[{"GA", "[", 
          RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
          "]"}], ",", 
        RowBox[{"InsideDiracTrace", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]]}], "+", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSimplify", "[", 
      RowBox[{
        RowBox[{"GA", "[", 
          RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}], ",", 
        RowBox[{"InsideDiracTrace", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"t1", ",", "t2", ",", "t3", ",", "t4", ",", "t5", ",", "t6"}], 
      "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["DiracSimplify2", "Subsection",
  CellTags->"DiracSimplify2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "DiracSimplify2[exp] simplifies the Dirac structure but leaves any ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  " untouched."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracSimplify"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GAD", "[", 
      RowBox[{
      "\[Mu]", ",", "\[Nu]", ",", "\[Mu]", ",", "5", ",", "\[Alpha]", ",", 
        "\[Beta]", ",", "\[Alpha]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "5"], ".", 
        SuperscriptBox["\[Gamma]", "\[Alpha]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Beta]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Alpha]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSimplify2", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", "5"], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Alpha]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Beta]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Alpha]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["DiracSlash", "Subsection",
  CellTags->"DiracSlash"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "DiracSlash[p] is the contraction ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            SuperscriptBox["p", "\[Mu]"], 
            SubscriptBox["\[Gamma]", "\[Mu]"]}], " "}], TraditionalForm]]],
  "(FourVector[p, ",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "] DiracMatrix[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "]). Products of those can be entered in the form DiracSlash[p1, p2, ...]."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "DiracSlash", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Dimension", "\[Rule]", "4"}], ",", 
          RowBox[{"FeynCalcInternal", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracGammaExpand",
    ButtonData:>"DiracGammaExpand",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracGammaExpand"],
  ", ",
  ButtonBox["DiracGamma",
    ButtonData:>"DiracGamma",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracGamma"],
  ", ",
  ButtonBox["DiracMatrix",
    ButtonData:>"DiracMatrix",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracMatrix"],
  ", ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracSimplify"],
  ", ",
  ButtonBox["DiracTrick",
    ButtonData:>"DiracTrick",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracTrick"],
  ", ",
  ButtonBox["GS",
    ButtonData:>"GS",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GS"],
  ", ",
  ButtonBox["GSD",
    ButtonData:>"GSD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GSD"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[TextData[{
  "This is q-slash, i.e., ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Mu]"], 
          RowBox[{
            SubscriptBox["q", "\[Mu]"], "."}]}], TraditionalForm]]]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSlash", "[", "q", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        FormBox["q",
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracSlash", "[", "p", "]"}], ".", 
      RowBox[{"DiracSlash", "[", "q", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["p",
              "TraditionalForm"]}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"]}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSlash", "[", 
      RowBox[{"p", ",", "q"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["p",
              "TraditionalForm"]}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"]}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GS", "[", "p", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracSlash", "[", "q", "]"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"DiracGamma", "[", 
      RowBox[{"Momentum", "[", "q", "]"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracSlash", "[", 
        RowBox[{"q", ",", 
          RowBox[{"Dimension", "->", "n"}]}], "]"}], "//", "StandardForm"}]], 
  "Input"],

Cell[BoxData[
    RowBox[{"DiracGamma", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", 
          RowBox[{"q", ",", "n"}], "]"}], ",", "n"}], "]"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["DiracTrace", "Subsection",
  CellTags->"DiracTrace"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
DiracTrace[expr] is the head of Dirac Traces. Whether the trace is \
evaluated depends on the option DiracTraceEvaluate. Direct trace evaluation \
is performed by Tr. The argument expr may be a product of Dirac matrices or \
slashes separated by the Mathematica Dot \".\".\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "DiracTrace", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"EpsContract", "\[Rule]", "False"}], ",", 
          RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
          RowBox[{"FeynCalcExternal", "\[Rule]", "False"}], ",", 
          RowBox[{"Mandelstam", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"PairCollect", "\[Rule]", "True"}], ",", 
          RowBox[{"DiracTraceEvaluate", "\[Rule]", "False"}], ",", 
          RowBox[{"Schouten", "\[Rule]", "0"}], ",", 
          RowBox[{"LeviCivitaSign", "\[Rule]", 
            RowBox[{"-", "1"}]}]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "For comments regarding ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  "schemes see the notes for ",
  ButtonBox["Tr",
    ButtonData:>"Tr",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Tr"],
  "."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Tr",
    ButtonData:>"Tr",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Tr"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrace", "[", 
      RowBox[{"GA", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"tr", "(", 
        FormBox[
          RowBox[{
            SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
            SuperscriptBox["\[Gamma]", "\[Nu]"]}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrace", "[", 
      RowBox[{"GA", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
        "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"tr", "(", 
        FormBox[
          RowBox[{
            SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
            SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
            SuperscriptBox["\[Gamma]", "\[Rho]"], ".", 
            SuperscriptBox["\[Gamma]", "\[Sigma]"]}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", " ", "/.", " ", 
      RowBox[{"DiracTrace", "\[Rule]", "Tr"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}]}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrace", "[", 
      RowBox[{
        RowBox[{"GA", "[", 
          RowBox[{
          "\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]", ",", "5"}], 
          "]"}], ",", 
        RowBox[{"DiracTraceEvaluate", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "4"}], " ", "\[ImaginaryI]", " ", 
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrace", "[", 
      RowBox[{
        RowBox[{"GA", "[", 
          RowBox[{
          "\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]", ",", 
            "\[Delta]", ",", "\[Tau]", ",", "5"}], "]"}], ",", 
        RowBox[{"DiracTraceEvaluate", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Delta]", "\[NoBreak]", "\[Mu]"}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Delta]", "\[NoBreak]", "\[Nu]"}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Delta]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Delta]", "\[NoBreak]", "\[Sigma]"}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Delta]", "\[NoBreak]", "\[Tau]"}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Delta]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Delta]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Delta]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Delta]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Tau]"}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Delta]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Delta]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Delta]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Tau]"}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Delta]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Delta]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Rho]", "\[NoBreak]", "\[Tau]"}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Delta]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Sigma]", "\[NoBreak]", "\[Tau]"}]]}]}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrace", "[", 
      RowBox[{"GS", "[", 
        RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"tr", "(", 
        FormBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{
              "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
              ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
              "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "q"}], 
              ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
              "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "r"}], 
              ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
              "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "s"}], 
              ")"}]}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrace", "[", 
      RowBox[{
        RowBox[{"GA", "[", 
          RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], ",", 
        RowBox[{"DiracTraceEvaluate", "\[Rule]", "True"}], ",", 
        RowBox[{"FCE", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"4", " ", 
      RowBox[{"MT", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["DiracTrick", "Subsection",
  CellTags->"DiracTrick"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
DiracTrick[exp] contracts gamma matrices with each other and \
performs several simplifications, but no expansion, use Calc or DiracSimplify \
for non-commutative expansion.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "DiracTrick", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Expanding", "\[Rule]", "False"}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Calc",
    ButtonData:>"Calc",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Calc"],
  ", ",
  ButtonBox["DiracGammaExpand",
    ButtonData:>"DiracGammaExpand",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracGammaExpand"],
  ", ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracSimplify"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell["This is a string of Dirac matrices in four dimensions. ", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{"GA", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrick", "[", "t1", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "2"}], " ", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["This is a string of Dirac matrices in D dimensions. ", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{"GAD", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrick", "[", "t2", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"2", "-", "D"}], ")"}], " ", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t3", "=", 
      RowBox[{"GA", "[", 
        RowBox[{"5", ",", "\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "5"], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "By default ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  "is moved to the right."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrick", "[", "t3", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t4", "=", 
      RowBox[{"GA", "[", 
        RowBox[{"6", ",", "\[Mu]", ",", "7"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "6"], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrick", "[", "t4", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t5", "=", 
      RowBox[{
        RowBox[{"GS", "[", 
          RowBox[{"a", "+", "b"}], "]"}], " ", ".", " ", 
        RowBox[{"GS", "[", "p", "]"}], ".", 
        RowBox[{"GS", "[", "p", "]"}], ".", 
        RowBox[{"GS", "[", 
          RowBox[{"c", "+", "d"}], "]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "(", 
            "\[NoBreak]", 
            FormBox[
              RowBox[{"a", "+", "b"}],
              "TraditionalForm"], "\[NoBreak]", ")"}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "(", 
            "\[NoBreak]", 
            FormBox[
              RowBox[{"c", "+", "d"}],
              "TraditionalForm"], "\[NoBreak]", ")"}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrick", "[", "t5", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", "(", "\[NoBreak]", 
              FormBox[
                RowBox[{"a", "+", "b"}],
                "TraditionalForm"], "\[NoBreak]", ")"}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", "(", "\[NoBreak]", 
              FormBox[
                RowBox[{"c", "+", "d"}],
                "TraditionalForm"], "\[NoBreak]", ")"}], ")"}]}], " ", 
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], "2"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", "t5", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["a",
                  "TraditionalForm"]}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["c",
                  "TraditionalForm"]}], ")"}]}], " ", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"]}], "+", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["a",
                  "TraditionalForm"]}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["d",
                  "TraditionalForm"]}], ")"}]}], " ", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"]}], "+", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["b",
                  "TraditionalForm"]}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["c",
                  "TraditionalForm"]}], ")"}]}], " ", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"]}], "+", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["b",
                  "TraditionalForm"]}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["d",
                  "TraditionalForm"]}], ")"}]}], " ", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GAD", "@@", 
      RowBox[{"Join", "[", 
        RowBox[{
          RowBox[{"{", "\[Mu]", "}"}], ",", 
          RowBox[{"Table", "[", 
            RowBox[{
              SubscriptBox["\[Nu]", "i"], ",", 
              RowBox[{"{", 
                RowBox[{"i", ",", "6"}], "}"}]}], "]"}], ",", 
          RowBox[{"{", "\[Mu]", "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", 
          SubscriptBox["\[Nu]", "1"]], ".", 
        SuperscriptBox["\[Gamma]", 
          SubscriptBox["\[Nu]", "2"]], ".", 
        SuperscriptBox["\[Gamma]", 
          SubscriptBox["\[Nu]", "3"]], ".", 
        SuperscriptBox["\[Gamma]", 
          SubscriptBox["\[Nu]", "4"]], ".", 
        SuperscriptBox["\[Gamma]", 
          SubscriptBox["\[Nu]", "5"]], ".", 
        SuperscriptBox["\[Gamma]", 
          SubscriptBox["\[Nu]", "6"]], ".", 
        SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrick", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{"D", "-", "12"}], ")"}], " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "1"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "2"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "3"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "4"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "5"],
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox[
                  SubscriptBox["\[Nu]", "6"],
                  "TraditionalForm"],
                "TraditionalForm"]]}]}], "-", 
        RowBox[{"4", " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  RowBox[{
                    SuperscriptBox["\[Gamma]", 
                      FormBox[
                        FormBox[
                          SubscriptBox["\[Nu]", "3"],
                          "TraditionalForm"],
                        "TraditionalForm"]], ".", 
                    SuperscriptBox["\[Gamma]", 
                      FormBox[
                        FormBox[
                          SubscriptBox["\[Nu]", "4"],
                          "TraditionalForm"],
                        "TraditionalForm"]], ".", 
                    SuperscriptBox["\[Gamma]", 
                      FormBox[
                        FormBox[
                          SubscriptBox["\[Nu]", "5"],
                          "TraditionalForm"],
                        "TraditionalForm"]], ".", 
                    SuperscriptBox["\[Gamma]", 
                      FormBox[
                        FormBox[
                          SubscriptBox["\[Nu]", "6"],
                          "TraditionalForm"],
                        "TraditionalForm"]]}]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "+", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "4"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "6"],
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "3"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "-", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "3"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "6"],
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "4"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "+", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "3"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "4"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "6"],
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "-", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "3"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "4"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "6"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "-", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "4"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "6"],
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "3"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "+", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "3"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "6"],
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "4"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "-", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "3"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "4"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "6"],
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "+", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "3"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "4"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "6"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "-", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "6"],
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "3"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "4"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "+", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "4"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "6"],
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "3"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "-", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "4"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "3"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "6"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "-", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "3"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "6"],
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "4"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "+", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "3"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "4"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "6"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "-", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "1"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "2"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "3"],
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SuperscriptBox["\[Gamma]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "4"],
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "5"],
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox[
                        SubscriptBox["\[Nu]", "6"],
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}]}], ")"}]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["DotSimplify", "Subsection",
  CellTags->"DotSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
DotSimplify[expr] expands and reorders noncommutative terms in \
expr. Simplifying relations may be specified by the option \
DotSimplifyRelations or by Commutator and AntiCommutator definitions. Whether \
expr is expanded noncommutatively depends on the option Expanding.\
\>", 
  "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "DotSimplify", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Expanding", "\[Rule]", "True"}], ",", 
          RowBox[{"DotSimplifyRelations", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"DotPower", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["AntiCommutator",
    ButtonData:>"AntiCommutator",
    ButtonStyle->"Hyperlink",
    ButtonNote->"AntiCommutator"],
  ", ",
  ButtonBox["Commutator",
    ButtonData:>"Commutator",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Commutator"],
  ", ",
  ButtonBox["Calc",
    ButtonData:>"Calc",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Calc"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{
        RowBox[{"GA", "[", "\[Mu]", "]"}], ".", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"2", " ", 
              RowBox[{"GS", "[", "p", "]"}]}], "-", 
            RowBox[{"GS", "[", "q", "]"}]}], ")"}], ".", 
        RowBox[{"GA", "[", "\[Nu]", "]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"2", " ", 
              RowBox[{
              "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
                "p"}]}], "-", 
            RowBox[{
            "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "q"}]}], 
          ")"}], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotSimplify", "[", "t1", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"2", " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
            RowBox[{"(", 
              RowBox[{
              "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
              ")"}], ".", 
            SuperscriptBox["\[Gamma]", "\[Nu]"]}]}], "-", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
          RowBox[{"(", 
            RowBox[{
            "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "q"}], 
            ")"}], ".", 
          SuperscriptBox["\[Gamma]", "\[Nu]"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"DeclareNonCommutative", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{"a", ".", 
        RowBox[{"(", 
          RowBox[{"b", "-", 
            RowBox[{"z", " ", "c"}]}], ")"}], ".", "a"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"a", ".", 
        RowBox[{"(", 
          RowBox[{"b", "-", 
            RowBox[{"c", " ", "z"}]}], ")"}], ".", "a"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotSimplify", "[", "t2", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", ".", "b", ".", "a"}], "-", 
        RowBox[{"z", " ", 
          RowBox[{"a", ".", "c", ".", "a"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Commutator", "[", 
        RowBox[{"a", ",", "c"}], "]"}], "=", "1"}]], "Input"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotSimplify", "[", "t2", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", ".", "b", ".", "a"}], "-", 
        RowBox[{"z", " ", 
          RowBox[{"(", 
            RowBox[{"a", "+", 
              RowBox[{"c", ".", 
                SuperscriptBox["a", "2"]}]}], ")"}]}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"Commutator", "[", 
        RowBox[{"a", ",", "c"}], "]"}], "=", "."}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotSimplify", "[", "t2", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", ".", "b", ".", "a"}], "-", 
        RowBox[{"z", " ", 
          RowBox[{"a", ".", "c", ".", "a"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"AntiCommutator", "[", 
        RowBox[{"b", ",", "a"}], "]"}], "=", "c"}]], "Input"],

Cell[BoxData[
    FormBox["c", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotSimplify", "[", "t2", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", ".", "c"}], "-", 
        RowBox[{
          SuperscriptBox["a", "2"], ".", "b"}], "-", 
        RowBox[{"z", " ", 
          RowBox[{"a", ".", "c", ".", "a"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"AntiCommutator", "[", 
        RowBox[{"b", ",", "a"}], "]"}], "=", "."}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotSimplify", "[", 
      RowBox[{"t2", ",", 
        RowBox[{"DotSimplifyRelations", "\[Rule]", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"a", ".", "c"}], "\[Rule]", 
              RowBox[{"1", "/", "z"}]}], "}"}]}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", ".", "b", ".", "a"}], "-", "a"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"DeclareNonCommutative", "[", "x", "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotSimplify", "[", 
      RowBox[{"x", ".", "x", ".", "x"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["x", "3"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DotSimplify", "[", 
      RowBox[{
        RowBox[{"x", ".", "x", ".", "x"}], ",", 
        RowBox[{"DotPower", "\[Rule]", "False"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"x", ".", "x", ".", "x"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"UnDeclareNonCommutative", "[", 
      RowBox[{"a", ",", "b", ",", "c", ",", "x"}], "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["EpsChisholm", "Subsection",
  CellTags->"EpsChisholm"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
EpsChisholm[expr] substitutes for a gamma matrix contracted with a \
Levi-Civita tensor (Eps) the Chisholm identity.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Chisholm",
    ButtonData:>"Chisholm",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Chisholm"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Chisholm", "[", 
      RowBox[{"GA", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
        "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", "\[ImaginaryI]"}], " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["$MU$184",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", "5"]}], " ", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["$MU$184",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], "-", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]]}], "+", 
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"EpsChisholm", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Rho]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Sigma]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["EpsEvaluate", "Subsection",
  CellTags->"EpsEvaluate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
EpsEvaluate[expr] applies total antisymmetry and linearity (w.r.t. \
Momentum's) to all Levi-Civita tensors (Eps') in expr.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Contract",
    ButtonData:>"Contract",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Contract"],
  ", ",
  ButtonBox["Eps",
    ButtonData:>"Eps",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Eps"],
  ", ",
  ButtonBox["LeviCivita",
    ButtonData:>"LeviCivita",
    ButtonStyle->"Hyperlink",
    ButtonNote->"LeviCivita"],
  ", ",
  ButtonBox["Trick",
    ButtonData:>"Trick",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Trick"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Trick", "[", 
      RowBox[{
        RowBox[{"LeviCivita", "[", 
          RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
          "]"}], " ", 
        RowBox[{"FourVector", "[", 
          RowBox[{
            RowBox[{"p", "+", "q"}], ",", "\[Sigma]"}], "]"}]}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Rho]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            RowBox[{"p", "+", "q"}],
            "TraditionalForm"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"EpsEvaluate", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox["p",
              "TraditionalForm"]}]], "+", 
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Eps", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
          RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
          RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], ",", 
          RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], "+", 
      RowBox[{"Eps", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
          RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
          RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], ",", 
          RowBox[{"Momentum", "[", "q", "]"}]}], "]"}]}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Epsilon", "Subsection",
  CellTags->"Epsilon"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "Epsilon is (",
  StyleBox["n",
    FontSlant->"Italic"],
  "-4), where ",
  StyleBox["n",
    FontSlant->"Italic"],
  " is the space-time dimension. Epsilon stands for a small positive number."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Series2",
    ButtonData:>"Series2",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Series2"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData["Epsilon"], "Input"],

Cell[BoxData[
    FormBox[
      TagBox["\[CurlyEpsilon]",
        TraditionalForm], TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
Epsilon has no functional properties, but some upvalues are \
changed:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{
          RowBox[{"Re", "[", "Epsilon", "]"}], " ", ">", " ", 
          RowBox[{"-", "4"}]}], ",", 
        RowBox[{
          RowBox[{"Re", "[", "Epsilon", "]"}], " ", ">", " ", 
          RowBox[{"-", "3"}]}], ",", 
        RowBox[{
          RowBox[{"Re", "[", "Epsilon", "]"}], " ", ">", " ", 
          RowBox[{"-", "2"}]}], ",", 
        RowBox[{
          RowBox[{"Re", "[", "Epsilon", "]"}], " ", ">", " ", 
          RowBox[{"-", "1"}]}], ",", 
        RowBox[{
          RowBox[{"Re", "[", "Epsilon", "]"}], " ", ">", " ", "0"}]}], 
      "}"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"True", ",", "True", ",", "True", ",", "True", ",", "True"}], 
        "}"}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Eps", "Subsection",
  CellTags->"Eps"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "Eps[a, b, c, d] is the head of the totally antisymmetric ",
  Cell[BoxData[
      FormBox[
        RowBox[{" ", "\[Epsilon]"}], TraditionalForm]]],
  " (Levi-Civita) tensor. The a,b, ... may have head LorentzIndex, Momentum \
or Integer. In case of integers the Levi-Civita tensor is evaluated \
immediately. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Eps", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Dimension", "\[Rule]", "4"}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["EpsEvaluate",
    ButtonData:>"EpsEvaluate",
    ButtonStyle->"Hyperlink",
    ButtonNote->"EpsEvaluate"],
  ", ",
  ButtonBox["LC",
    ButtonData:>"LC",
    ButtonStyle->"Hyperlink",
    ButtonNote->"LC"],
  ", ",
  ButtonBox["LCD",
    ButtonData:>"LCD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"LCD"],
  ", ",
  ButtonBox["LeviCivita",
    ButtonData:>"LeviCivita",
    ButtonStyle->"Hyperlink",
    ButtonNote->"LeviCivita"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Eps", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Sigma]", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Rho]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Sigma]",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Eps", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Sigma]", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Rho]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Sigma]",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Eps", "[", 
        RowBox[{"b", ",", "a", ",", "c", ",", "d"}], "]"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"-", 
      RowBox[{"Eps", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Eps", "[", 
      RowBox[{"0", ",", "1", ",", "2", ",", "3"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Eps", "[", 
      RowBox[{"1", ",", "0", ",", "2", ",", "3"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", "1"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"Eps", ",", 
          RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a1", " ", "=", 
      RowBox[{"Eps", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Nu]", ",", "D"}], "]"}], ",", 
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", 
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Sigma]", ",", "D"}], "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Rho]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Sigma]",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", 
      RowBox[{"a1", "a1"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", "24"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"Eps", ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a2", " ", "=", 
      RowBox[{"Eps", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Nu]", ",", "D"}], "]"}], ",", 
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", 
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Sigma]", ",", "D"}], "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Rho]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Sigma]",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Contract", "[", 
        RowBox[{"a2", "a2"}], "]"}], "//", "Factor2"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"1", "-", "D"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"2", "-", "D"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"3", "-", "D"}], ")"}], " ", "D"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"g5", "=", 
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            FractionBox["\[ImaginaryI]", "24"]}], " ", 
          RowBox[{
            RowBox[{"LCD", "[", 
              RowBox[{
              "\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Alpha]"}], "]"}], 
            ".", 
            RowBox[{"GAD", "[", 
              RowBox[{
              "\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Alpha]"}], 
              "]"}]}]}], "//", "FCI"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "24"]}], " ", "\[ImaginaryI]", " ", 
        RowBox[{
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Alpha]",
                  "TraditionalForm"],
                "TraditionalForm"]}]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Alpha]",
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"g5p", "=", 
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            FractionBox["\[ImaginaryI]", "24"]}], " ", 
          RowBox[{
            RowBox[{"LCD", "[", 
              RowBox[{
                SuperscriptBox["\[Mu]", "\[Prime]",
                  MultilineFunction->None], ",", 
                SuperscriptBox["\[Nu]", "\[Prime]",
                  MultilineFunction->None], ",", 
                SuperscriptBox["\[Rho]", "\[Prime]",
                  MultilineFunction->None], ",", 
                SuperscriptBox["\[Alpha]", "\[Prime]",
                  MultilineFunction->None]}], "]"}], ".", 
            RowBox[{"GAD", "[", 
              RowBox[{
                SuperscriptBox["\[Mu]", "\[Prime]",
                  MultilineFunction->None], ",", 
                SuperscriptBox["\[Nu]", "\[Prime]",
                  MultilineFunction->None], ",", 
                SuperscriptBox["\[Rho]", "\[Prime]",
                  MultilineFunction->None], ",", 
                SuperscriptBox["\[Alpha]", "\[Prime]",
                  MultilineFunction->None]}], "]"}]}]}], "//", "FCI"}]}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "24"]}], " ", "\[ImaginaryI]", " ", 
        RowBox[{
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox[
                  SuperscriptBox["\[Mu]", "\[Prime]",
                    MultilineFunction->None],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SuperscriptBox["\[Nu]", "\[Prime]",
                    MultilineFunction->None],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SuperscriptBox["\[Rho]", "\[Prime]",
                    MultilineFunction->None],
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox[
                  SuperscriptBox["\[Alpha]", "\[Prime]",
                    MultilineFunction->None],
                  "TraditionalForm"],
                "TraditionalForm"]}]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox[
                SuperscriptBox["\[Mu]", "\[Prime]",
                  MultilineFunction->None],
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox[
                SuperscriptBox["\[Nu]", "\[Prime]",
                  MultilineFunction->None],
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox[
                SuperscriptBox["\[Rho]", "\[Prime]",
                  MultilineFunction->None],
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox[
                SuperscriptBox["\[Alpha]", "\[Prime]",
                  MultilineFunction->None],
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"g52", "=", 
      RowBox[{"Factor2", "[", 
        RowBox[{"Calc", "[", 
          RowBox[{"g5", ".", "g5p"}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "24"]}], " ", 
        RowBox[{"(", 
          RowBox[{"1", "-", "D"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"2", "-", "D"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"3", "-", "D"}], ")"}], " ", "D"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"g52", "/.", 
      RowBox[{"D", "\[Rule]", "4"}]}]], "Input"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"a1", ",", "a2", ",", "g5", ",", "g5p", ",", "g52"}], "]"}]], 
  "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["ExpandPartialD", "Subsection",
  CellTags->"ExpandPartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
ExpandPartialD[exp] expands all products of QuantumField's and \
partial differentiation operators in exp and applies Leibniz rule.\
\>", 
  "Text"],

Cell[TextData[{
  "See also: ",
  ButtonBox["ExplicitPartialD",
    ButtonData:>"ExplicitPartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ExplicitPartialD"],
  ", ",
  ButtonBox["LeftPartialD",
    ButtonData:>"LeftPartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"LeftPartialD"],
  ", ",
  ButtonBox["LeftRightPartialD",
    ButtonData:>"LeftRightPartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"LeftRightPartialD"],
  ", ",
  ButtonBox["RightPartialD",
    ButtonData:>"RightPartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"RightPartialD"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"RightPartialD", "[", "\[Mu]", "]"}], ".", 
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[RightArrow]"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SubscriptBox["A", "\[Mu]"], ".", 
        SubscriptBox["A", "\[Nu]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["A", "\[Mu]"], ".", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[Null]"]}]}], "+", 
        RowBox[{
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[Null]"]}], ".", 
          SubscriptBox["A", "\[Nu]"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"QuantumField", "[", 
          RowBox[{"A", ",", 
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
        RowBox[{"QuantumField", "[", 
          RowBox[{
            RowBox[{"PartialD", "[", 
              RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", "A", 
            ",", 
            RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}], "+", 
      RowBox[{
        RowBox[{"QuantumField", "[", 
          RowBox[{
            RowBox[{"PartialD", "[", 
              RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", "A", 
            ",", 
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
        RowBox[{"QuantumField", "[", 
          RowBox[{"A", ",", 
            RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}]}]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"LeftRightPartialD", "[", "\[Mu]", "]"}], ".", 
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SubscriptBox["A", "\[Nu]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[Null]"]}], "2"], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["A", "\[Nu]"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
      RowBox[{"(", 
        RowBox[{
          RowBox[{"LeftRightPartialD", "[", "OPEDelta", "]"}], "^", "2"}], 
        ")"}], ".", 
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["A", "\[Mu]"], ".", 
        SubsuperscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], "2"], ".", 
        SubscriptBox["A", "\[Rho]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            SubscriptBox["A", "\[Mu]"], ".", 
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  SuperscriptBox["\[CapitalDelta]", "2"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[Null]"]}]}]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{
            SubscriptBox["A", "\[Mu]"], ".", 
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[Null]"]}]}], " ", 
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
            FormBox["\[CapitalDelta]",
              "TraditionalForm"]]}], "+", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            SubscriptBox["A", "\[Mu]"], ".", 
            SubscriptBox["A", "\[Rho]"]}], " ", 
          SubsuperscriptBox[
            OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
            RowBox[{" ", "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], 
            FormBox["2",
              "TraditionalForm"]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"8", " ", 
      RowBox[{
        RowBox[{"LeftRightPartialD", "[", "OPEDelta", "]"}], "^", "3"}]}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"8", " ", 
        SubsuperscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], "3"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExplicitPartialD", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], "-", 
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]]}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], "-", 
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]]}], ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], "-", 
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]]}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SubsuperscriptBox[
            OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
            RowBox[{" ", "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], 
            FormBox["3",
              "TraditionalForm"]]}], "+", 
        RowBox[{"3", " ", 
          SubsuperscriptBox[
            OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
            RowBox[{" ", "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], 
            FormBox["2",
              "TraditionalForm"]], " ", 
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox["\[CapitalDelta]",
              "TraditionalForm"]]}], "-", 
        RowBox[{"3", " ", 
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
            FormBox["\[CapitalDelta]",
              "TraditionalForm"]], " ", 
          SubsuperscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            RowBox[{" ", "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], 
            FormBox["2",
              "TraditionalForm"]]}], "+", 
        SubsuperscriptBox[
          OverscriptBox["\[PartialD]", "\[RightArrow]"], 
          RowBox[{" ", "\[NoBreak]", 
            FormBox["\[CapitalDelta]",
              "TraditionalForm"]}], 
          FormBox["3",
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"LeviCivita", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Tau]"}], "]"}], 
      " ", 
      RowBox[{"RightPartialD", "[", 
        RowBox[{"\[Alpha]", ",", "\[Mu]", ",", "\[Beta]", ",", "\[Nu]"}], 
        "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox[
              FormBox["\[Alpha]",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox[
              FormBox["\[Beta]",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]]}], " ", 
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Tau]",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["ExpandScalarProduct", "Subsection",
  CellTags->"ExpandScalarProduct"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
ExpandScalarProduct[expr] expands scalar products of sums of \
momenta in expr. ExpandScalarProduct does not use Expand on expr.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Calc",
    ButtonData:>"Calc",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Calc"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SP", "[", 
      RowBox[{"p", ",", 
        RowBox[{"p", "-", "q"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["p",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        "(", "\[NoBreak]", 
        FormBox[
          RowBox[{"p", "-", "q"}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandScalarProduct", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], "2"], "-", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FV", "[", 
      RowBox[{
        RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        RowBox[{"(", 
          FormBox[
            RowBox[{"p", "-", "q"}],
            "TraditionalForm"], ")"}], "\[Mu]"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandScalarProduct", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], "-", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SP", "[", 
        RowBox[{
          RowBox[{"p", "-", "q"}], ",", 
          RowBox[{"q", "-", "r"}]}], "]"}], "//", "FCI"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"(", "\[NoBreak]", 
        FormBox[
          RowBox[{"p", "-", "q"}],
          "TraditionalForm"], "\[NoBreak]", ")", "\[NoBreak]", 
        "\[CenterDot]", "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          RowBox[{"q", "-", "r"}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "/.", 
      RowBox[{"Pair", "\[Rule]", "ExpandScalarProduct"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}], "-", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["r",
            "TraditionalForm"]}], "-", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], "2"], "+", 
        RowBox[{
          FormBox["q",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["r",
            "TraditionalForm"]}]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Explicit", "Subsection",
  CellTags->"Explicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "Explicit",
  " is an option for FieldStrength, GluonVertex, SUNF, and \
Twist2GluonOperator. If set to True the full form of the operator is \
inserted. ",
  StyleBox["Explicit[exp]",
    FontWeight->"Bold"],
  " inserts explicit expressions of GluonVertex, Twist2GluonOperator, etc., \
in exp. SUNF's are replaced by SUNTrace objects."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["GluonVertex",
    ButtonData:>"GluonVertex",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GluonVertex"],
  ", ",
  ButtonBox["Twist2GluonOperator",
    ButtonData:>"Twist2GluonOperator",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Twist2GluonOperator"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonVertex", "[", 
      RowBox[{
      "p", ",", "\[Mu]", ",", "a", ",", " ", "q", ",", "\[Nu]", ",", "b", 
        ",", "r", ",", "\[Rho]", ",", "c"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        RowBox[{
          SuperscriptBox["V", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]}]], "(", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", 
            RowBox[{",", " "}], "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"], "\[NoBreak]", 
            RowBox[{",", " "}], "\[NoBreak]", 
            FormBox["r",
              "TraditionalForm"]}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Explicit", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "-", 
            RowBox[{
              SuperscriptBox[
                FormBox["r",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                FormBox["r",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}]}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b"}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "m"],
              "TraditionalForm"], "+", "1"}], ")"}], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]], " ", 
        RowBox[{"(", 
          RowBox[{
            SubsuperscriptBox["O", 
              RowBox[{
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[VeryThinSpace]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]}], "G2"], "(", 
            FormBox["p",
              "TraditionalForm"], ")"}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Explicit", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  FormBox[
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}],
                    "TraditionalForm"], ")"}], "2"]}], "-", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox[
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "+", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}]}], ")"}], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}]}], "+", 
            RowBox[{
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"]}]}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "m"],
              "TraditionalForm"], "+", "1"}], ")"}], " ", 
        FormBox[
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], ")"}], 
            RowBox[{"m", "-", "2"}]],
          "TraditionalForm"], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FieldStrength", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox["F", 
        RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}], "a"], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Explicit", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox[
            FormBox["A",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox[
            FormBox["A",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]]}], "+", 
        RowBox[{
          SubscriptBox["g", "s"], " ", 
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b1",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["c10",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{"a", "\[NoBreak]", "b1", "\[NoBreak]", "c10"}]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["ExplicitPartialD", "Subsection",
  CellTags->"ExplicitPartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
ExplicitPartialD[exp] inserts in exp the definition for \
LeftRightPartialD[z] (and LeftRightPartialD2[z]).\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  ButtonBox["ExpandPartialD",
    ButtonData:>"ExpandPartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ExpandPartialD"],
  ", ",
  ButtonBox["LeftRightPartialD",
    ButtonData:>"LeftRightPartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"LeftRightPartialD"],
  ", ",
  ButtonBox["LeftRightPartialD2",
    ButtonData:>"LeftRightPartialD2",
    ButtonStyle->"Hyperlink",
    ButtonNote->"LeftRightPartialD2"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExplicitPartialD", "[", " ", 
      RowBox[{"LeftRightPartialD", "[", "\[Mu]", "]"}], " ", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], "-", 
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExplicitPartialD", "[", " ", 
      RowBox[{"LeftRightPartialD2", "[", "\[Mu]", "]"}], " ", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], "+", 
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[RightArrow]"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExplicitPartialD", "[", " ", 
      RowBox[{"LeftRightPartialD", "[", "OPEDelta", "]"}], " ", "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], "-", 
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]]}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"16", " ", 
      RowBox[{
        RowBox[{"LeftRightPartialD", "[", "OPEDelta", "]"}], "^", "4"}]}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"16", " ", 
        SubsuperscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], "4"]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["ExplicitSUNIndex", "Subsection",
  CellTags->"ExplicitSUNIndex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
ExplicitSUNIndex[ind] is a specific SU(N) index, i.e., ind is an \
integer.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  ButtonBox["FCI",
    ButtonData:>"FCI",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FCI"],
  ", ",
  ButtonBox["SUNDelta",
    ButtonData:>"SUNDelta",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SUNDelta"],
  ", ",
  ButtonBox["SUNF",
    ButtonData:>"SUNF",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SUNF"],
  ", ",
  ButtonBox["SUNIndex",
    ButtonData:>"SUNIndex",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SUNIndex"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExplicitSUNIndex", "[", "1", "]"}]], "Input"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SUNDelta", "[", 
          RowBox[{"1", ",", "a"}], "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"SUNDelta", "[", 
      RowBox[{
        RowBox[{"ExplicitSUNIndex", "[", "1", "]"}], ",", 
        RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Factor2", "Subsection",
  CellTags->"Factor2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Factor2[poly] factors a polynomial in a standard way. Factor2 works \
sometimes better than Factor on polynomials involving rationals with sums in \
the denominator. Factor2 uses Factor internally and is in general slower than \
Factor. There are four possible settings of the option Method (0,1,2,3). In \
general Factor will work faster than Factor2.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Factor2", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"FactorFull", "\[Rule]", "False"}], ",", 
          RowBox[{"Method", "\[Rule]", "3"}]}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Collect2",
    ButtonData:>"Collect2",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Collect2"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{
        RowBox[{"(", 
          RowBox[{"a", "-", "x"}], ")"}], 
        RowBox[{"(", 
          RowBox[{"b", "-", "x"}], ")"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"a", "-", "x"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"b", "-", "x"}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Factor2", "[", "t1", "]"}], ",", " ", 
          RowBox[{"Factor", "[", "t1", "]"}]}], "}"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "x"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"b", "-", "x"}], ")"}]}], ",", 
          RowBox[{
            RowBox[{"-", 
              RowBox[{"(", 
                RowBox[{"a", "-", "x"}], ")"}]}], " ", 
            RowBox[{"(", 
              RowBox[{"x", "-", "b"}], ")"}]}]}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t3", "=", 
      RowBox[{"Expand", "[", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{"a", "-", "b"}], ")"}], 
          RowBox[{"(", 
            RowBox[{"a", "+", "b"}], ")"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["a", "2"], "-", 
        SuperscriptBox["b", "2"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Factor", "[", "t3", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"a", "-", "b"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"a", "+", "b"}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Factor2", "[", "t3", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["a", "2"], "-", 
        SuperscriptBox["b", "2"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Factor2", "[", 
      RowBox[{"t3", ",", 
        RowBox[{"FactorFull", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"a", "-", "b"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"a", "+", "b"}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"t1", ",", "t2", ",", "t3"}], "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["FAD", "Subsection",
  CellTags->"FAD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
FAD is the FeynCalc external form of FeynAmpDenominator and denotes \
an inverse propagator. FAD[q, q-p, ...] is 1/(q^2 (q-p)^2 ...). FAD[{q1,m}, \
{q1-p,m}, q2, ...] is 1/( (q1^2 - m^2) ( (q1-p)^2 - m^2 ) q2^2 ... ). \
Translation into FeynCalc internal form is performed by \
FeynCalcInternal.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FAD",
    ButtonData:>"FAD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FAD"],
  ", ",
  ButtonBox["FCE",
    ButtonData:>"FCE",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FCE"],
  ", ",
  ButtonBox["FCI",
    ButtonData:>"FCI",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FCI"],
  ", ",
  ButtonBox["FeynAmpDenominator",
    ButtonData:>"FeynAmpDenominator",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FeynAmpDenominator"],
  ", ",
  ButtonBox["FeynAmpDenominatorSimplify",
    ButtonData:>"FeynAmpDenominatorSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FeynAmpDenominatorSimplify"],
  ", ",
  ButtonBox["PropagatorDenominator",
    ButtonData:>"PropagatorDenominator",
    ButtonStyle->"Hyperlink",
    ButtonNote->"PropagatorDenominator"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FAD", "[", 
      RowBox[{"q", ",", 
        RowBox[{"p", "-", "q"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          RowBox[{"(", 
            InterpretationBox[
              RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"p", "-", "q"}], ")"}], "2"], 
                "\[InvisibleSpace]", "\<\"]\"\>"}],
              SequenceForm[ "[", 
                Power[ 
                  Plus[ p, 
                    Times[ -1, q]], 2], "]"],
              Editable->False], ")"}], " ", 
          RowBox[{"(", 
            InterpretationBox[
              RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                SuperscriptBox["q", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
              SequenceForm[ "[", 
                Power[ q, 2], "]"],
              Editable->False], ")"}]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FAD", "[", 
      RowBox[{"p", ",", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"p", "-", "q"}], ",", "m"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          RowBox[{"(", 
            InterpretationBox[
              RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                SuperscriptBox["p", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
              SequenceForm[ "[", 
                Power[ p, 2], "]"],
              Editable->False], ")"}], " ", 
          RowBox[{"(", 
            InterpretationBox[
              RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"p", "-", "q"}], ")"}], "2"], 
                "\[InvisibleSpace]", "\<\"-\"\>", "\[InvisibleSpace]", 
                SuperscriptBox["m", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
              SequenceForm[ "[", 
                Power[ 
                  Plus[ p, 
                    Times[ -1, q]], 2], "-", 
                Power[ m, 2], "]"],
              Editable->False], ")"}]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"FAD", "[", 
            RowBox[{"q", ",", 
              RowBox[{"p", "-", "q"}]}], "]"}], "//", "FCI"}], "//", "FCE"}], 
      "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"FAD", "[", 
      RowBox[{"q", ",", 
        RowBox[{"p", "-", "q"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"FAD", "[", 
          RowBox[{"q", ",", 
            RowBox[{"p", "-", "q"}]}], "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"FeynAmpDenominator", "[", 
      RowBox[{
        RowBox[{"PropagatorDenominator", "[", 
          RowBox[{
            RowBox[{"Momentum", "[", 
              RowBox[{"q", ",", "D"}], "]"}], ",", "0"}], "]"}], ",", 
        RowBox[{"PropagatorDenominator", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"Momentum", "[", 
                RowBox[{"p", ",", "D"}], "]"}], "-", 
              RowBox[{"Momentum", "[", 
                RowBox[{"q", ",", "D"}], "]"}]}], ",", "0"}], "]"}]}], 
      "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"FAD", "[", "p", "]"}], " ", 
          RowBox[{"FAD", "[", 
            RowBox[{"p", "-", "q"}], "]"}]}], " ", "//", " ", 
        "FeynAmpDenominatorCombine"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"FeynAmpDenominator", "[", 
      RowBox[{
        RowBox[{"PropagatorDenominator", "[", 
          RowBox[{
            RowBox[{"Momentum", "[", 
              RowBox[{"p", ",", "D"}], "]"}], ",", "0"}], "]"}], ",", 
        RowBox[{"PropagatorDenominator", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"Momentum", "[", 
                RowBox[{"p", ",", "D"}], "]"}], "-", 
              RowBox[{"Momentum", "[", 
                RowBox[{"q", ",", "D"}], "]"}]}], ",", "0"}], "]"}]}], 
      "]"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["FCE", "Subsection",
  CellTags->"FCE"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
FCE[exp] translates exp from the internal FeynCalc representation \
to a short form.\
\>", "Text"],

Cell["FCE is equivalent to FeynCalcExternal.", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynCalcExternal",
    ButtonData:>"FeynCalcExternal",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FeynCalcExternal"],
  ", ",
  ButtonBox["FCI",
    ButtonData:>"FCI",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FCI"],
  ", ",
  ButtonBox["FeynCalcInternal",
    ButtonData:>"FeynCalcInternal",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FeynCalcInternal"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FCE", "[", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{"DiracGamma", "[", "5", "]"}], ",", 
          RowBox[{"DiracGamma", "[", 
            RowBox[{"Momentum", "[", "p", "]"}], "]"}]}], "}"}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "5"], ",", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}]}], 
        "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"GA", "[", "5", "]"}], ",", 
        RowBox[{"GS", "[", "p", "]"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"GS", "[", "p", "]"}], ",", 
        RowBox[{"SP", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", 
        RowBox[{"MT", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Mu]"], ",", 
          SuperscriptBox["\[Gamma]", "\[Rho]"], ",", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
          ",", 
          FormBox[
            RowBox[{
              FormBox["p",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["q",
                "TraditionalForm"]}],
            "TraditionalForm"], ",", 
          SuperscriptBox["g", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]], ",", 
          SuperscriptBox["p", "\[Mu]"]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"GS", "[", "p", "]"}], ",", 
        RowBox[{"SP", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", 
        RowBox[{"MT", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "FCI"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], ",", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]], ",", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["p",
              "TraditionalForm"]}], ",", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"]}], ",", 
          SuperscriptBox["g", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]], ",", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"DiracGamma", "[", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
        RowBox[{"DiracGamma", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", 
              RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", "D"}], "]"}], ",", 
        RowBox[{"DiracGamma", "[", 
          RowBox[{"Momentum", "[", "p", "]"}], "]"}], ",", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"Momentum", "[", "p", "]"}], ",", 
            RowBox[{"Momentum", "[", "q", "]"}]}], "]"}], ",", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
            RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}], ",", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
            RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FCE", "[", "%", "]"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"GS", "[", "p", "]"}], ",", 
        RowBox[{"SP", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", 
        RowBox[{"MT", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["FCI", "Subsection",
  CellTags->"FCI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
FCI[exp] translates exp into the internal FeynCalc \
(datatype-)representation.FCI is equivalent to FeynCalcInternal.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynCalcExternal",
    ButtonData:>"FeynCalcExternal",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FeynCalcExternal"],
  ", ",
  ButtonBox["FeynCalcInternal",
    ButtonData:>"FeynCalcInternal",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FeynCalcInternal"],
  ", ",
  ButtonBox["FCE",
    ButtonData:>"FCE",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FCE"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"GS", "[", "p", "]"}], ",", 
        RowBox[{"SP", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", 
        RowBox[{"MT", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Mu]"], ",", 
          SuperscriptBox["\[Gamma]", "\[Rho]"], ",", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
          ",", 
          FormBox[
            RowBox[{
              FormBox["p",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["q",
                "TraditionalForm"]}],
            "TraditionalForm"], ",", 
          SuperscriptBox["g", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]], ",", 
          SuperscriptBox["p", "\[Mu]"]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"GS", "[", "p", "]"}], ",", 
        RowBox[{"SP", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", 
        RowBox[{"MT", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "FCI"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], ",", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]], ",", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["p",
              "TraditionalForm"]}], ",", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"]}], ",", 
          SuperscriptBox["g", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]], ",", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"DiracGamma", "[", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
        RowBox[{"DiracGamma", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", 
              RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", "D"}], "]"}], ",", 
        RowBox[{"DiracGamma", "[", 
          RowBox[{"Momentum", "[", "p", "]"}], "]"}], ",", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"Momentum", "[", "p", "]"}], ",", 
            RowBox[{"Momentum", "[", "q", "]"}]}], "]"}], ",", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
            RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}], ",", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
            RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FCE", "[", "%", "]"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"GS", "[", "p", "]"}], ",", 
        RowBox[{"SP", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", 
        RowBox[{"MT", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["FermionSpinSum", "Subsection",
  CellTags->"FermionSpinSum"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
FermionSpinSum[x] constructs the Traces out of squared ampliudes.\
\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "FermionSpinSum", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"SpinPolarizationSum", "\[Rule]", "Identity"}], ",", 
          RowBox[{"SpinorCollect", "\[Rule]", "False"}], ",", 
          RowBox[{"ExtraFactor", "\[Rule]", "1"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["ComplexConjugate",
    ButtonData:>"ComplexConjugate",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ComplexConjugate"],
  ", ",
  ButtonBox["DiracTrace",
    ButtonData:>"DiracTrace",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracTrace"],
  ", ",
  ButtonBox["Tr",
    ButtonData:>"Tr",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Tr"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{
        RowBox[{"Spinor", "[", 
          RowBox[{"k1", ",", "m"}], "]"}], ".", 
        RowBox[{"DiracSlash", "[", "p", "]"}], ".", 
        RowBox[{"GA", "[", "5", "]"}], ".", 
        RowBox[{"Spinor", "[", 
          RowBox[{"p1", ",", "m"}], "]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"\[CurlyPhi]", "\[NoBreak]", "(", "\[NoBreak]", 
          FormBox["k1",
            "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m", 
          "\[NoBreak]", ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["p",
              "TraditionalForm"]}], ")"}], ".", 
        SuperscriptBox["\[Gamma]", "5"], ".", 
        RowBox[{"\[CurlyPhi]", "\[NoBreak]", "(", "\[NoBreak]", 
          FormBox["p1",
            "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m", 
          "\[NoBreak]", ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
Notice that SpinorUBar and SpinorU are only input functions. \
Internally they are converted to Spinor objects.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ct", "=", 
      RowBox[{"ComplexConjugate", "[", "t", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        RowBox[{
          RowBox[{"\[CurlyPhi]", "\[NoBreak]", "(", "\[NoBreak]", 
            FormBox["p1",
              "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m", 
            "\[NoBreak]", ")"}], ".", 
          SuperscriptBox["\[Gamma]", "5"], ".", 
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["p",
                "TraditionalForm"]}], ")"}], ".", 
          RowBox[{"\[CurlyPhi]", "\[NoBreak]", "(", "\[NoBreak]", 
            FormBox["k1",
              "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m", 
            "\[NoBreak]", ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FermionSpinSum", "[", 
      RowBox[{"t", " ", "ct"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        RowBox[{"tr", "(", 
          FormBox[
            RowBox[{
              RowBox[{"(", 
                RowBox[{"m", "+", 
                  RowBox[{
                    FormBox["\<\"\[Gamma]\"\>",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["k1",
                      "TraditionalForm"]}]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  FormBox["\<\"\[Gamma]\"\>",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}], ")"}], ".", 
              SuperscriptBox["\[Gamma]", "5"], ".", 
              RowBox[{"(", 
                RowBox[{"m", "+", 
                  RowBox[{
                    FormBox["\<\"\[Gamma]\"\>",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["p1",
                      "TraditionalForm"]}]}], ")"}], ".", 
              SuperscriptBox["\[Gamma]", "5"], ".", 
              RowBox[{"(", 
                RowBox[{
                  FormBox["\<\"\[Gamma]\"\>",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}], ")"}]}],
            "TraditionalForm"], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", " ", "/.", 
      RowBox[{"DiracTrace", "->", "Tr"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "4"}], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"], " ", 
              SuperscriptBox["m", "2"]}], "+", 
            RowBox[{
              RowBox[{
                FormBox["k1",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p1",
                  "TraditionalForm"]}], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"]}], "-", 
            RowBox[{"2", " ", 
              RowBox[{
                FormBox["k1",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox["p",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p1",
                  "TraditionalForm"]}]}]}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"t", ",", " ", "ct"}], "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["FeynAmpDenominatorCombine", "Subsection",
  CellTags->"FeynAmpDenominatorCombine"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
FeynAmpDenominatorCombine[expr] expands expr with respect to \
FeynAmpDenominator and combines products of FeynAmpDenominator in expr into \
one FeynAmpDenominator.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynAmpDenominatorSplit",
    ButtonData:>"FeynAmpDenominatorSplit",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FeynAmpDenominatorSplit"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", " ", "=", " ", 
      RowBox[{
        RowBox[{"FAD", "[", "q", "]"}], " ", 
        RowBox[{"FAD", "[", 
          RowBox[{"q", "-", "p"}], "]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          FractionBox["1", 
            InterpretationBox[
              RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                SuperscriptBox["q", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
              SequenceForm[ "[", 
                Power[ q, 2], "]"],
              Editable->False]],
          "TraditionalForm"], " ", 
        FormBox[
          FractionBox["1", 
            InterpretationBox[
              RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"q", "-", "p"}], ")"}], "2"], 
                "\[InvisibleSpace]", "\<\"]\"\>"}],
              SequenceForm[ "[", 
                Power[ 
                  Plus[ 
                    Times[ -1, p], q], 2], "]"],
              Editable->False]],
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"FeynAmpDenominatorCombine", "[", "%", "]"}], "//", "FCE"}], 
      "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"FAD", "[", 
      RowBox[{"q", ",", 
        RowBox[{
          RowBox[{"-", "p"}], "+", "q"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"FeynAmpDenominatorSplit", "[", "%", "]"}], "//", "FCE"}], 
      "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"FAD", "[", "q", "]"}], " ", 
      RowBox[{"FAD", "[", 
        RowBox[{
          RowBox[{"-", "p"}], "+", "q"}], "]"}]}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["FeynAmpDenominator", "Subsection",
  CellTags->"FeynAmpDenominator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
FeynAmpDenominator[ PropagatorDenominator[ ... ], \
PropagatorDenominator[ ... ], ...] is the head of the denominators of the \
propagators, i.e., FeynAmpDenominator[x] is the representation of 1/x .\
\>", 
  "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FAD",
    ButtonData:>"FAD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FAD"],
  ", ",
  ButtonBox["FeynAmpDenominatorSimplify",
    ButtonData:>"FeynAmpDenominatorSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FeynAmpDenominatorSimplify"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FeynAmpDenominator", "[", 
      RowBox[{"PropagatorDenominator", "[", 
        RowBox[{"p", ",", "m"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          SuperscriptBox["p", "2"], "-", 
          SuperscriptBox["m", "2"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FeynAmpDenominator", "[", 
      RowBox[{
        RowBox[{"PropagatorDenominator", "[", 
          RowBox[{"p", ",", "m"}], "]"}], ",", 
        RowBox[{"PropagatorDenominator", "[", 
          RowBox[{
            RowBox[{"p", "-", "q"}], ",", "m"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox["p", "2"], "-", 
              SuperscriptBox["m", "2"]}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"p", "-", "q"}], ")"}], "2"], "-", 
              SuperscriptBox["m", "2"]}], ")"}]}]], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"t", "=", 
        RowBox[{"FeynAmpDenominator", "[", 
          RowBox[{"PropagatorDenominator", "[", 
            RowBox[{"p", ",", "m"}], "]"}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"t", "//", "FCI"}], "]"}]], "Input"],

Cell[BoxData[
    RowBox[{"FeynAmpDenominator", "[", 
      RowBox[{"PropagatorDenominator", "[", 
        RowBox[{
          RowBox[{"Momentum", "[", 
            RowBox[{"p", ",", "D"}], "]"}], ",", "m"}], "]"}], "]"}]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"t", "//", "FCE"}], "]"}]], "Input"],

Cell[BoxData[
    RowBox[{"FAD", "[", 
      RowBox[{"{", 
        RowBox[{"p", ",", "m"}], "}"}], "]"}]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", "t", "]"}], ";"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["FeynAmpDenominatorSimplify", "Subsection",
  CellTags->"FeynAmpDenominatorSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
FeynAmpDenominatorSimplify[exp] tries to simplify each \
PropagatorDenominator in a canonical way. FeynAmpDenominatorSimplify[exp, q1] \
simplifies all FeynAmpDenominator's in exp in a canonical way, including some \
translation of momenta. FeynAmpDenominatorSimplify[exp, q1, q2] additionally \
removes integrals with no mass scale.\
\>", "Text"],

Cell["FDS can be used as an alias.", "Text"],

Cell[CellGroupData[{

Cell[BoxData["FDS"], "Input"],

Cell[BoxData[
    FormBox["FeynAmpDenominatorSimplify", TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OneLoopSimplify",
    ButtonData:>"OneLoopSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"OneLoopSimplify"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[TextData[{
  "The cornerstone of dimensional regularization is that ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"\[Integral]", 
            RowBox[{
              SuperscriptBox["d", "n"], "k", " ", 
              RowBox[{
                RowBox[{"f", "(", "k", ")"}], "/", 
                SuperscriptBox["k", 
                  RowBox[{"2", "m"}]]}]}]}], "=", " ", 
          RowBox[{"0", " ", "."}]}], TraditionalForm]]]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FeynAmpDenominatorSimplify", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"f", "[", "k", "]"}], " ", 
          RowBox[{"FAD", "[", 
            RowBox[{"k", ",", "k"}], "]"}]}], ",", "k"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "This brings ",
  Cell[BoxData[
      FormBox[
        RowBox[{"1", "/", 
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"k", "-", 
                    SubscriptBox["p", "1"]}], ")"}], "2"], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"k", "-", 
                    SubscriptBox["p", "2"]}], ")"}], "2"]}], ")"}]}], 
        TraditionalForm]]],
  " into a standard form."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FeynAmpDenominatorSimplify", "[", 
      RowBox[{
        RowBox[{"FAD", "[", 
          RowBox[{
            RowBox[{"k", "-", 
              SubscriptBox["p", "1"]}], ",", 
            RowBox[{"k", "-", 
              SubscriptBox["p", "2"]}]}], "]"}], ",", "k"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          SuperscriptBox["k", "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"k", "-", 
                SubscriptBox["p", "1"], "+", 
                SubscriptBox["p", "2"]}], ")"}], "2"]}]], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{"FeynAmpDenominatorSimplify", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"FAD", "[", 
              RowBox[{
                RowBox[{"k", "-", 
                  SubscriptBox["p", "1"]}], ",", 
                RowBox[{"k", "-", 
                  SubscriptBox["p", "2"]}]}], "]"}], " ", 
            RowBox[{"SPD", "[", 
              RowBox[{"k", ",", "k"}], "]"}]}], ",", "k"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          SuperscriptBox[
            FormBox["k",
              "TraditionalForm"], "2"], 
          RowBox[{
            SuperscriptBox["k", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"k", "-", 
                  SubscriptBox["p", "1"], "+", 
                  SubscriptBox["p", "2"]}], ")"}], "2"]}]], "+", 
        FractionBox[
          RowBox[{"2", " ", 
            RowBox[{
              FormBox["k",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                SubscriptBox["p", "2"],
                "TraditionalForm"]}]}], 
          RowBox[{
            SuperscriptBox["k", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"k", "-", 
                  SubscriptBox["p", "1"], "+", 
                  SubscriptBox["p", "2"]}], ")"}], "2"]}]], "+", 
        FractionBox[
          SubsuperscriptBox["p", "2", "2"], 
          RowBox[{
            SuperscriptBox["k", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"k", "-", 
                  SubscriptBox["p", "1"], "+", 
                  SubscriptBox["p", "2"]}], ")"}], "2"]}]]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"r", "=", 
      RowBox[{"SPC", "[", 
        RowBox[{"t", ",", "k", ",", 
          RowBox[{"FDS", "\[Rule]", "True"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          SubsuperscriptBox["p", "2", "2"], 
          RowBox[{
            SuperscriptBox["k", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"k", "-", 
                  SubscriptBox["p", "1"], "+", 
                  SubscriptBox["p", "2"]}], ")"}], "2"]}]], "-", 
        FractionBox[
          RowBox[{"2", " ", 
            RowBox[{
              FormBox["k",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                SubscriptBox["p", "2"],
                "TraditionalForm"]}]}], 
          RowBox[{
            SuperscriptBox["k", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"k", "+", 
                  SubscriptBox["p", "1"], "-", 
                  SubscriptBox["p", "2"]}], ")"}], "2"]}]]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OneLoopSimplify", "[", 
      RowBox[{"r", ",", "k"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"HighEnergyPhysics`fctools`Contract3", "(", 
        RowBox[{
          FractionBox[
            RowBox[{"2", " ", 
              RowBox[{
                FormBox["k",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["p", "2"],
                  "TraditionalForm"]}]}], 
            RowBox[{
              SuperscriptBox["k", "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"k", "-", 
                    SubscriptBox["p", "1"], "+", 
                    SubscriptBox["p", "2"]}], ")"}], "2"]}]], "+", 
          FractionBox[
            SubsuperscriptBox["p", "2", "2"], 
            RowBox[{
              SuperscriptBox["k", "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"k", "-", 
                    SubscriptBox["p", "1"], "+", 
                    SubscriptBox["p", "2"]}], ")"}], "2"]}]]}], ")"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FDS", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"FAD", "[", 
            RowBox[{
              RowBox[{"k", "-", "p1"}], ",", 
              RowBox[{"k", "-", "p2"}]}], "]"}], 
          RowBox[{
            RowBox[{"SPD", "[", 
              RowBox[{"k", ",", "OPEDelta"}], "]"}], "^", "2"}]}], ",", 
        "k"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          SuperscriptBox[
            RowBox[{"(", 
              FormBox[
                RowBox[{
                  FormBox["k",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"]}],
                "TraditionalForm"], ")"}], "2"], 
          RowBox[{
            SuperscriptBox["k", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"k", "-", "p1", "+", "p2"}], ")"}], "2"]}]], "+", 
        FractionBox[
          RowBox[{"2", " ", 
            RowBox[{
              FormBox["k",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], " ", 
            RowBox[{
              FormBox["\[CapitalDelta]",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["p2",
                "TraditionalForm"]}]}], 
          RowBox[{
            SuperscriptBox["k", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"k", "-", "p1", "+", "p2"}], ")"}], "2"]}]], "+", 
        FractionBox[
          SuperscriptBox[
            RowBox[{"(", 
              FormBox[
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p2",
                    "TraditionalForm"]}],
                "TraditionalForm"], ")"}], "2"], 
          RowBox[{
            SuperscriptBox["k", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"k", "-", "p1", "+", "p2"}], ")"}], "2"]}]]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"t", ",", "r"}], "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["FeynAmpDenominatorSplit", "Subsection",
  CellTags->"FeynAmpDenominatorSplit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
FeynAmpDenominatorSplit[expr] splits all FeynAmpDenominator[a,b, \
...] in expr into FeynAmpDenominator[a]*FeynAmpDenominator[b] ... . \
FeynAmpDenominatorSplit[expr, q1] splits all FeynAmpDenominator in expr into \
a product of two, one containing q1 and other momenta, the second without q1.\
\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynAmpDenominatorCombine",
    ButtonData:>"FeynAmpDenominatorCombine",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FeynAmpDenominatorCombine"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{
        RowBox[{"FAD", "[", 
          RowBox[{"q1", ",", 
            RowBox[{"q1", "-", "p"}], ",", 
            RowBox[{"q1", "-", "q2"}], ",", "q2", ",", 
            RowBox[{"q2", "-", "p"}]}], "]"}], "//", "FCI"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          SuperscriptBox[
            InterpretationBox[
              SubscriptBox["q", "1"],
              q1], "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                InterpretationBox[
                  SubscriptBox["q", "1"],
                  q1], "-", "p"}], ")"}], "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                InterpretationBox[
                  SubscriptBox["q", "1"],
                  q1], "-", 
                InterpretationBox[
                  SubscriptBox["q", "2"],
                  q2]}], ")"}], "2"], ".", 
          SuperscriptBox[
            InterpretationBox[
              SubscriptBox["q", "2"],
              q2], "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                InterpretationBox[
                  SubscriptBox["q", "2"],
                  q2], "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "//", "Head"}]], "Input"],

Cell[BoxData[
    FormBox["FeynAmpDenominator", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FeynAmpDenominatorSplit", "[", "t", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          SuperscriptBox[
            InterpretationBox[
              SubscriptBox["q", "1"],
              q1], "2"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                InterpretationBox[
                  SubscriptBox["q", "1"],
                  q1], "-", "p"}], ")"}], "2"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                InterpretationBox[
                  SubscriptBox["q", "1"],
                  q1], "-", 
                InterpretationBox[
                  SubscriptBox["q", "2"],
                  q2]}], ")"}], "2"], " ", 
          SuperscriptBox[
            InterpretationBox[
              SubscriptBox["q", "2"],
              q2], "2"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                InterpretationBox[
                  SubscriptBox["q", "2"],
                  q2], "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"%", "//", "FCE"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"FAD", "[", "q1", "]"}], " ", 
      RowBox[{"FAD", "[", 
        RowBox[{
          RowBox[{"-", "p"}], "+", "q1"}], "]"}], " ", 
      RowBox[{"FAD", "[", 
        RowBox[{"q1", "-", "q2"}], "]"}], " ", 
      RowBox[{"FAD", "[", "q2", "]"}], " ", 
      RowBox[{"FAD", "[", 
        RowBox[{
          RowBox[{"-", "p"}], "+", "q2"}], "]"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"FeynAmpDenominatorSplit", "[", 
          RowBox[{"t", ",", "q1"}], "]"}], "//", "FCE"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"FAD", "[", 
        RowBox[{"q2", ",", 
          RowBox[{
            RowBox[{"-", "p"}], "+", "q2"}]}], "]"}], " ", 
      RowBox[{"FAD", "[", 
        RowBox[{"q1", ",", 
          RowBox[{
            RowBox[{"-", "p"}], "+", "q1"}], ",", 
          RowBox[{"q1", "-", "q2"}]}], "]"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"FeynAmpDenominatorCombine", "[", "%", "]"}], "//", "FCE"}], 
      "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"FAD", "[", 
      RowBox[{"q1", ",", "q2", ",", 
        RowBox[{"q1", "-", "q2"}], ",", 
        RowBox[{
          RowBox[{"-", "p"}], "+", "q1"}], ",", 
        RowBox[{
          RowBox[{"-", "p"}], "+", "q2"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", "t", "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["FeynAmp", "Subsection",
  CellTags->"FeynAmp"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
FeynAmp[q, amp] is the head of a Feynman amplitude. amp denotes the \
analytical expression for the amplitude and q is the integration variable. \
FeynAmp[q1, q2, amp] denotes a two-loop amplitude.\
\>", "Text"],

Cell["\<\
FeynAmp has no functional properties and serves just as a head. \
There are however special typesetting rules attached.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Amplitudes",
    ButtonData:>"Amplitudes",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Amplitudes"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell["\<\
This is a 1-loop gluon self-energy amplitude (ignoring factors of \
(2 \[Pi])).\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FeynAmp", "[", 
      RowBox[{"q", ",", 
        RowBox[{
          RowBox[{"GV", "[", 
            RowBox[{"p", ",", "\[Mu]", ",", "a", ",", " ", 
              RowBox[{"q", "-", "p"}], ",", "\[Alpha]", ",", "c", ",", " ", 
              RowBox[{"-", "q"}], ",", "\[Beta]", ",", "e"}], "]"}], " ", 
          RowBox[{"GP", "[", 
            RowBox[{
              RowBox[{"p", "-", "q"}], ",", " ", "\[Alpha]", ",", "c", ",", 
              " ", "\[Rho]", ",", "d"}], "]"}], 
          RowBox[{"GV", "[", 
            RowBox[{
              RowBox[{"-", "p"}], ",", "\[Nu]", ",", "b", ",", " ", 
              RowBox[{"p", "-", "q"}], ",", "\[Rho]", ",", "d", ",", " ", 
              "q", ",", "\[Sigma]", ",", "f"}], "]"}], " ", 
          RowBox[{"GP", "[", 
            RowBox[{
            "q", ",", " ", "\[Beta]", ",", "e", ",", " ", "\[Sigma]", ",", 
              "f"}], "]"}]}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[Integral]", 
        StyleBox[
          RowBox[{
            SuperscriptBox["\[DifferentialD]", "D"], "q"}],
          ZeroWidthTimes->True], "(", 
        FormBox[
          RowBox[{"-", 
            FractionBox[
              RowBox[{
                SubsuperscriptBox["g", "s", "2"], " ", 
                RowBox[{
                  SuperscriptBox["V", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], "(", 
                  RowBox[{
                    FormBox[
                      RowBox[{"-", "p"}],
                      "TraditionalForm"], "\[NoBreak]", 
                    RowBox[{",", " "}], "\[NoBreak]", 
                    FormBox[
                      RowBox[{"p", "-", "q"}],
                      "TraditionalForm"], "\[NoBreak]", 
                    RowBox[{",", " "}], "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}], ")"}], " ", 
                RowBox[{
                  SuperscriptBox["V", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Alpha]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Beta]",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], "(", 
                  RowBox[{
                    FormBox["p",
                      "TraditionalForm"], "\[NoBreak]", 
                    RowBox[{",", " "}], "\[NoBreak]", 
                    FormBox[
                      RowBox[{"q", "-", "p"}],
                      "TraditionalForm"], "\[NoBreak]", 
                    RowBox[{",", " "}], "\[NoBreak]", 
                    FormBox[
                      RowBox[{"-", "q"}],
                      "TraditionalForm"]}], ")"}], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Alpha]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Beta]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["\[Delta]", 
                  RowBox[{
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["\[Delta]", 
                  RowBox[{
                    FormBox[
                      FormBox["e",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["f",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["f", 
                  RowBox[{
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["e",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["f", 
                  RowBox[{
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["f",
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"p", "-", "q"}], ")"}], "2"], " ", 
                SuperscriptBox["q", "2"]}]]}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["This is a generic 2-loop amplitude.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FeynAmp", "[", 
      RowBox[{
        SubscriptBox["q", "1"], ",", 
        SubscriptBox["q", "2"], ",", "anyexpression"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[Integral]", 
        RowBox[{
          StyleBox[
            RowBox[{
              SuperscriptBox["\[DifferentialD]", "D"], 
              SubscriptBox["q", "1"]}],
            ZeroWidthTimes->True], "\[Integral]", 
          RowBox[{
            SuperscriptBox["\[DifferentialD]", "D"], 
            SubscriptBox["q", "2"]}]}], "(", "anyexpression", ")"}], 
      TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["FeynCalc2FORM", "Subsection",
  CellTags->"FeynCalc2FORM"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
FeynCalc2FORM[expr] displays expr in FORM syntax. \
FeynCalc2FORM[file, x] writes x in FORM syntax to a file. FeynCalc2FORM[file, \
x==y] writes x=y to a file in FORM syntax.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FORM2FeynCalc",
    ButtonData:>"FORM2FeynCalc",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FORM2FeynCalc"],
  "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "FeynCalc2FORM", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"EpsDiscard", "\[Rule]", "False"}], ",", 
          RowBox[{"FORMEpilog", "\[Rule]", "\<\"\"\>"}], ",", 
          RowBox[{"FORMProlog", "\[Rule]", "\<\"write statistics;\"\>"}], 
          ",", 
          RowBox[{"Replace", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"\<\"\[Alpha]\"\>", "\[Rule]", "\<\"al\"\>"}], ",", 
                RowBox[{"\<\"\[Beta]\"\>", "\[Rule]", "\<\"be\"\>"}], ",", 
                RowBox[{"\<\"\[Gamma]\"\>", "\[Rule]", "\<\"ga\"\>"}], ",", 
                RowBox[{"\<\"\[Delta]\"\>", "\[Rule]", "\<\"de\"\>"}], ",", 
                RowBox[{"\<\"\[Mu]\"\>", "\[Rule]", "\<\"mu\"\>"}], ",", 
                RowBox[{"\<\"\[Nu]\"\>", "\[Rule]", "\<\"nu\"\>"}], ",", 
                RowBox[{"\<\"\[Rho]\"\>", "\[Rule]", "\<\"ro\"\>"}], ",", 
                RowBox[{"\<\"\[Sigma]\"\>", "\[Rule]", "\<\"si\"\>"}]}], 
              "}"}]}], ",", 
          RowBox[{"TraceDimension", "\[Rule]", "4"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"MT", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], 
      RowBox[{"FV", "[", 
        RowBox[{"p", ",", "\[Rho]"}], "]"}], " ", 
      RowBox[{
        RowBox[{"y", "^", "2"}], "/", "d"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          SuperscriptBox["y", "2"], " ", 
          SuperscriptBox["p", "\[Rho]"], " ", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], "d"], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FeynCalc2FORM", "[", "%", "]"}], ";"}]], "Input"],

Cell[BoxData[
    FormBox["\<\"(y^2*d_(mu,nu)*p(ro))/d\"\>", TraditionalForm]], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{" ", 
      RowBox[{"LC", "[", 
        RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Delta]", ",", "\[Rho]"}], 
        "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
        "\[Alpha]", "\[NoBreak]", "\[Beta]", "\[NoBreak]", "\[Delta]", 
          "\[NoBreak]", "\[Rho]"}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FeynCalc2FORM", "[", "%", "]"}], ";"}]], "Input"],

Cell[BoxData[
    FormBox["\<\"-i_*e_(al,be,de,ro)\"\>", TraditionalForm]], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracTrace", "[", 
      RowBox[{"GA", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
        "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"tr", "(", 
        FormBox[
          RowBox[{
            SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
            SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
            SuperscriptBox["\[Gamma]", "\[Rho]"], ".", 
            SuperscriptBox["\[Gamma]", "\[Sigma]"]}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FeynCalc2FORM", "[", "%", "]"}], ";"}]], "Input"],

Cell[BoxData[
    FormBox["\<\"g_(1,mu)*g_(1,nu)*g_(1,ro)*g_(1,si)\"\>", TraditionalForm]], 
  "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracTrace", "[", 
        RowBox[{"GA", "[", 
          RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "]"}], 
      RowBox[{"DiracTrace", "[", 
        RowBox[{"GA", "[", 
          RowBox[{"\[Mu]", ",", "\[Rho]"}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"tr", "(", 
          FormBox[
            RowBox[{
              SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
              SuperscriptBox["\[Gamma]", "\[Nu]"]}],
            "TraditionalForm"], ")"}], " ", 
        RowBox[{"tr", "(", 
          FormBox[
            RowBox[{
              SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
              SuperscriptBox["\[Gamma]", "\[Rho]"]}],
            "TraditionalForm"], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FeynCalc2FORM", "[", "%", "]"}], ";"}]], "Input"],

Cell[BoxData[
    FormBox["\<\"g_(1,mu)*g_(1,nu)*g_(2,mu)*g_(2,ro)\"\>", TraditionalForm]], 
  "Print"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"FeynCalc2FORM", "[", 
        RowBox[{"\"\<fc2ftest.f\>\"", ",", 
          RowBox[{
            RowBox[{"MT", "[", 
              RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], 
            RowBox[{"FV", "[", 
              RowBox[{"p", ",", "\[Mu]"}], "]"}]}]}], "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ReadList", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"If", "[", 
            RowBox[{
              RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", 
              "\"\<:\>\"", ",", "\"\<\>\""}], "]"}], "<>", 
          "\"\<fc2ftest.f\>\""}], ",", "String"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", "\<\"d_(mu,nu)*p(mu)\"\>", "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{"Tr", "[", 
        RowBox[{
          RowBox[{"GA", "[", 
            RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
            "]"}], ".", 
          RowBox[{"GS", "[", 
            RowBox[{"p", ",", "q"}], "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}], "-", 
            RowBox[{
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
              RowBox[{
                FormBox["p",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["q",
                  "TraditionalForm"]}], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}], "-", 
            RowBox[{
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "-", 
            RowBox[{
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]], " ", 
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]], " ", 
              RowBox[{
                FormBox["p",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["q",
                  "TraditionalForm"]}]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
              RowBox[{
                FormBox["p",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["q",
                  "TraditionalForm"]}]}]}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"FeynCalc2FORM", "[", 
        RowBox[{"\"\<fc2ftest.f\>\"", ",", 
          RowBox[{"L", " ", "==", "t"}]}], "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TableForm", "[", 
      RowBox[{"ReadList", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"If", "[", 
              RowBox[{
                RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", 
                "\"\<:\>\"", ",", "\"\<\>\""}], "]"}], "<>", 
            "\"\<fc2ftest.f\>\""}], ",", "String"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      InterpretationBox[GridBox[{
            {"\<\"Indices mu,nu,ro,si;\"\>"},
            {"\<\"Vectors p,q;\"\>"},
            {"\<\"\"\>"},
            {"\<\"write statistics;\"\>"},
            {"\<\"\"\>"},
            {"\<\"Local L = ( \"\>"},
            {
              "\<\"4*(d_(mu,si)*d_(nu,ro)*q.p-d_(mu,ro)*d_(nu,si)*q.p+d_(mu,\
nu)*d_(ro,si)*q.p+\"\>"},
            {
              "\<\"d_(ro,si)*p(nu)*q(mu)-d_(nu,si)*p(ro)*q(mu)+d_(nu,ro)*p(si)\
*q(mu)-\"\>"},
            {
              "\<\"d_(ro,si)*p(mu)*q(nu)+d_(mu,si)*p(ro)*q(nu)-d_(mu,ro)*p(si)\
*q(nu)+\"\>"},
            {
              "\<\"d_(nu,si)*p(mu)*q(ro)-d_(mu,si)*p(nu)*q(ro)+d_(mu,nu)*p(si)\
*q(ro)-\"\>"},
            {
              "\<\"d_(nu,ro)*p(mu)*q(si)+d_(mu,ro)*p(nu)*q(si)-d_(mu,nu)*p(ro)\
*q(si)) ); \"\>"},
            {"\<\"   \"\>"},
            {"\<\"Print; \"\>"},
            {"\<\".end\"\>"}
            },
          RowSpacings->1,
          ColumnSpacings->3,
          RowAlignments->Baseline,
          ColumnAlignments->{Left}],
        TableForm[ {"Indices mu,nu,ro,si;", "Vectors p,q;", "", 
          "write statistics;", "", "Local L = ( ", 
          "4*(d_(mu,si)*d_(nu,ro)*q.p-d_(mu,ro)*d_(nu,si)*q.p+d_(mu,nu)*d_(ro,\
si)*q.p+", 
          "d_(ro,si)*p(nu)*q(mu)-d_(nu,si)*p(ro)*q(mu)+d_(nu,ro)*p(si)*q(mu)-\
", "d_(ro,si)*p(mu)*q(nu)+d_(mu,si)*p(ro)*q(nu)-d_(mu,ro)*p(si)*q(nu)+", 
          "d_(nu,si)*p(mu)*q(ro)-d_(mu,si)*p(nu)*q(ro)+d_(mu,nu)*p(si)*q(ro)-\
", "d_(nu,ro)*p(mu)*q(si)+d_(mu,ro)*p(nu)*q(si)-d_(mu,nu)*p(ro)*q(si)) ); ", 
          "   ", "Print; ", ".end"}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"If", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"FileNames", "[", "\"\<fc2ftest.f\>\"", "]"}], "=!=", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"DeleteFile", "[", "\"\<fc2ftest.f\>\"", "]"}]}], "]"}], 
      ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", "t", "]"}], ";"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["FeynCalcExternal", "Subsection",
  CellTags->"FeynCalcExternal"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
FeynCalcExternal[exp] translates exp from the internal FeynCalc \
representation to a shorthand form.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynCalcInternal",
    ButtonData:>"FeynCalcInternal",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FeynCalcInternal"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FeynCalcExternal", "[", 
      RowBox[{"DiracGamma", "[", "5", "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Gamma]", "5"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"GA", "[", "5", "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"GS", "[", "p", "]"}], ",", 
        RowBox[{"SP", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", 
        RowBox[{"MT", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Mu]"], ",", 
          SuperscriptBox["\[Gamma]", "\[Rho]"], ",", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
          ",", 
          FormBox[
            RowBox[{
              FormBox["p",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["q",
                "TraditionalForm"]}],
            "TraditionalForm"], ",", 
          SuperscriptBox["g", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]], ",", 
          SuperscriptBox["p", "\[Mu]"]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"GS", "[", "p", "]"}], ",", 
        RowBox[{"SP", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", 
        RowBox[{"MT", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "FeynCalcInternal"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], ",", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]], ",", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["p",
              "TraditionalForm"]}], ",", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"]}], ",", 
          SuperscriptBox["g", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]], ",", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"DiracGamma", "[", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
        RowBox[{"DiracGamma", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", 
              RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", "D"}], "]"}], ",", 
        RowBox[{"DiracGamma", "[", 
          RowBox[{"Momentum", "[", "p", "]"}], "]"}], ",", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"Momentum", "[", "p", "]"}], ",", 
            RowBox[{"Momentum", "[", "q", "]"}]}], "]"}], ",", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
            RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}], ",", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
            RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FeynCalcExternal", "[", "%", "]"}], "//", "StandardForm"}]], 
  "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"GS", "[", "p", "]"}], ",", 
        RowBox[{"SP", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", 
        RowBox[{"MT", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["FeynCalcInternal", "Subsection",
  CellTags->"FeynCalcInternal"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
FeynCalcInternal[exp] translates exp into the internal FeynCalc \
(abstract data-type) representation.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynCalcExternal",
    ButtonData:>"FeynCalcExternal",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FeynCalcExternal"],
  ", ",
  ButtonBox["FCI",
    ButtonData:>"FCI",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FCI"],
  ", ",
  ButtonBox["FCE",
    ButtonData:>"FCE",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FCE"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"GS", "[", "p", "]"}], ",", 
        RowBox[{"SP", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", 
        RowBox[{"MT", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Mu]"], ",", 
          SuperscriptBox["\[Gamma]", "\[Rho]"], ",", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
          ",", 
          FormBox[
            RowBox[{
              FormBox["p",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["q",
                "TraditionalForm"]}],
            "TraditionalForm"], ",", 
          SuperscriptBox["g", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]], ",", 
          SuperscriptBox["p", "\[Mu]"]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"GS", "[", "p", "]"}], ",", 
        RowBox[{"SP", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", 
        RowBox[{"MT", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "FeynCalcInternal"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], ",", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]], ",", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["p",
              "TraditionalForm"]}], ",", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"]}], ",", 
          SuperscriptBox["g", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]], ",", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"DiracGamma", "[", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
        RowBox[{"DiracGamma", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", 
              RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", "D"}], "]"}], ",", 
        RowBox[{"DiracGamma", "[", 
          RowBox[{"Momentum", "[", "p", "]"}], "]"}], ",", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"Momentum", "[", "p", "]"}], ",", 
            RowBox[{"Momentum", "[", "q", "]"}]}], "]"}], ",", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
            RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}], ",", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
            RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FeynCalcExternal", "[", "%", "]"}], "//", "StandardForm"}]], 
  "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"GS", "[", "p", "]"}], ",", 
        RowBox[{"SP", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", 
        RowBox[{"MT", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FCI", "[", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{"SD", "[", 
            RowBox[{"a", ",", "b"}], "]"}], ",", 
          RowBox[{"SUND", "[", 
            RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
          RowBox[{"SUNF", "[", 
            RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
          RowBox[{"FAD", "[", "q", "]"}], ",", 
          RowBox[{"LC", "[", 
            RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
            "]"}]}], "}"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]}]], ",", 
          SubscriptBox["d", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]}]], ",", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]}]], ",", 
          FractionBox["1", 
            SuperscriptBox["q", "2"]], ",", 
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"SUNDelta", "[", 
          RowBox[{
            RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
            RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}], ",", 
        RowBox[{"SUND", "[", 
          RowBox[{
            RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
            RowBox[{"SUNIndex", "[", "b", "]"}], ",", 
            RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}], ",", 
        RowBox[{"SUNF", "[", 
          RowBox[{
            RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
            RowBox[{"SUNIndex", "[", "b", "]"}], ",", 
            RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}], ",", 
        RowBox[{"FeynAmpDenominator", "[", 
          RowBox[{"PropagatorDenominator", "[", 
            RowBox[{
              RowBox[{"Momentum", "[", 
                RowBox[{"q", ",", "D"}], "]"}], ",", "0"}], "]"}], "]"}], 
        ",", 
        RowBox[{"Eps", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
            RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
            RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], ",", 
            RowBox[{"LorentzIndex", "[", "\[Sigma]", "]"}]}], "]"}]}], 
      "}"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["FeynRule", "Subsection",
  CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
FeynRule[lag, {fields}] derives the Feynman rule corresponding to \
the field configuration fields of the lagrangian lag.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "FeynRule", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Anti5", "\[Rule]", 
            InterpretationBox[
              RowBox[{"-", "\[Infinity]"}],
              DirectedInfinity[ -1]]}], ",", 
          RowBox[{"Contract", "\[Rule]", "False"}], ",", 
          RowBox[{"Factor1", "\[Rule]", "False"}], ",", 
          RowBox[{"FinalSubstitutions", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"PartialD", "\[Rule]", "RightPartialD"}], ",", 
          RowBox[{"Schouten", "\[Rule]", "False"}], ",", 
          RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell["FeynRule does not calculate propagator Feynman rules.", "Text"],

Cell["\<\
The option ZeroMomentumInsertion can be used for twist-2 and higher \
twist operators.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Lagrangian",
    ButtonData:>"Lagrangian",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Lagrangian"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"gou", "=", 
      RowBox[{"Lagrangian", "[", "\"\<ogu\>\"", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        SuperscriptBox["\[ImaginaryI]", 
          RowBox[{"m", "-", "1"}]], " ", 
        RowBox[{
          SubsuperscriptBox["F", 
            RowBox[{"\[Alpha]", "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], "a"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              SubsuperscriptBox["D", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                RowBox[{"a", "\[NoBreak]", "b"}]], ")"}], 
            RowBox[{"m", "-", "2"}]], ".", 
          SubsuperscriptBox["F", 
            RowBox[{"\[Alpha]", "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], "b"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"gop", "=", 
      RowBox[{"Lagrangian", "[", "\"\<ogp\>\"", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        SuperscriptBox["\[ImaginaryI]", "m"], " ", 
        RowBox[{
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Alpha]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Beta]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Gamma]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}]], ".", 
          SubsuperscriptBox["F", 
            RowBox[{"\[Beta]", "\[NoBreak]", "\[Gamma]"}], "a"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              SubsuperscriptBox["D", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                RowBox[{"a", "\[NoBreak]", "b"}]], ")"}], 
            RowBox[{"m", "-", "2"}]], ".", 
          SubsuperscriptBox["F", 
            RowBox[{"\[Alpha]", "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], "b"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Explicit", "[", "gop", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        SuperscriptBox["\[ImaginaryI]", "m"], " ", 
        RowBox[{
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Alpha]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Beta]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Gamma]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}]], ".", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Beta]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox[
                  FormBox["A",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Gamma]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Gamma]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox[
                  FormBox["A",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Beta]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                SubscriptBox["g", "s"], " ", 
                RowBox[{
                  SubsuperscriptBox[
                    FormBox["A",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Beta]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["b2",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubsuperscriptBox[
                    FormBox["A",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Gamma]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["c11",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SubscriptBox["f", 
                  RowBox[{
                  "a", "\[NoBreak]", "b2", "\[NoBreak]", "c11"}]]}]}], ")"}], 
          ".", 
          SuperscriptBox[
            RowBox[{"(", 
              SubsuperscriptBox["D", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                RowBox[{"a", "\[NoBreak]", "b"}]], ")"}], 
            RowBox[{"m", "-", "2"}]], ".", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Alpha]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"], 
                SubsuperscriptBox[
                  FormBox["A",
                    "TraditionalForm"], 
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"]], 
                SubsuperscriptBox[
                  FormBox["A",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Alpha]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                SubscriptBox["g", "s"], " ", 
                RowBox[{
                  SubsuperscriptBox[
                    FormBox["A",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Alpha]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["b3",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubsuperscriptBox[
                    FormBox["A",
                      "TraditionalForm"], 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["c12",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SubscriptBox["f", 
                  RowBox[{
                  "b", "\[NoBreak]", "b3", "\[NoBreak]", "c12"}]]}]}], 
            ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Cases2", "[", 
      RowBox[{"%", ",", "QuantumField"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SubsuperscriptBox[
            FormBox["A",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Alpha]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b3",
                "TraditionalForm"],
              "TraditionalForm"]], ",", 
          SubsuperscriptBox[
            FormBox["A",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Beta]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b2",
                "TraditionalForm"],
              "TraditionalForm"]], ",", 
          SubsuperscriptBox[
            FormBox["A",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Gamma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c11",
                "TraditionalForm"],
              "TraditionalForm"]], ",", 
          SubsuperscriptBox[
            FormBox["A",
              "TraditionalForm"], 
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["c12",
                "TraditionalForm"],
              "TraditionalForm"]], ",", 
          RowBox[{
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Alpha]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"], 
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], ",", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Beta]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Gamma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]]}], ",", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Gamma]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Beta]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]]}], ",", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], 
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Alpha]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["2-gluon Feynman rules (unpolarized)", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"fi", "=", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            RowBox[{"QuantumField", "[", 
              RowBox[{"GaugeField", ",", 
                RowBox[{"{", "\[Mu]", "}"}], ",", 
                RowBox[{"{", "a", "}"}]}], "]"}], "[", "p", "]"}], ",", 
          RowBox[{
            RowBox[{"QuantumField", "[", 
              RowBox[{"GaugeField", ",", 
                RowBox[{"{", "\[Nu]", "}"}], ",", 
                RowBox[{"{", "b", "}"}]}], "]"}], "[", "q", "]"}]}], 
        "}"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", "p", ")"}], ",", 
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", "q", ")"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"f2u", "=", 
      RowBox[{"FullSimplify", "/@", 
        RowBox[{"Factor2", "[", 
          RowBox[{"FeynRule", "[", 
            RowBox[{"gou", ",", "fi", ",", 
              RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "]"}], 
          "]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"], " ", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}], ")"}], "m"]}], "+", 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["q",
                        "TraditionalForm"]}], ")"}], "m"], " ", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"]}]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["q",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}]}], "-", 
                    RowBox[{
                      SuperscriptBox[
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}]}]}], ")"}]}]}], ")"}], " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], 
        RowBox[{"2", " ", 
          SuperscriptBox[
            RowBox[{"(", 
              FormBox[
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}],
                "TraditionalForm"], ")"}], "2"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              FormBox[
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}],
                "TraditionalForm"], ")"}], "2"]}]], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell["2-gluon Feynman rules (polarized)", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"fi", "=", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            RowBox[{"QuantumField", "[", 
              RowBox[{"GaugeField", ",", 
                RowBox[{"{", "\[Mu]", "}"}], ",", 
                RowBox[{"{", "a", "}"}]}], "]"}], "[", "p", "]"}], ",", 
          RowBox[{
            RowBox[{"QuantumField", "[", 
              RowBox[{"GaugeField", ",", 
                RowBox[{"{", "\[Nu]", "}"}], ",", 
                RowBox[{"{", "b", "}"}]}], "]"}], "[", "q", "]"}]}], 
        "}"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", "p", ")"}], ",", 
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", "q", ")"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"f2p", "=", 
      RowBox[{"FullSimplify", "/@", 
        RowBox[{"Factor2", "[", 
          RowBox[{"FeynRule", "[", 
            RowBox[{"gop", ",", "fi", ",", 
              RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "]"}], 
          "]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          RowBox[{"\[ImaginaryI]", " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}], ")"}], "m"], " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        SuperscriptBox["\[Epsilon]", 
                          RowBox[{
                            FormBox[
                              FormBox["\[Nu]",
                                "TraditionalForm"],
                              "TraditionalForm"], "\[NoBreak]", 
                            FormBox["p",
                              "TraditionalForm"], "\[NoBreak]", 
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"]}]], " ", 
                        SuperscriptBox[
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], 
                          FormBox[
                            FormBox["\[Mu]",
                              "TraditionalForm"],
                            "TraditionalForm"]]}], "+", 
                      RowBox[{
                        SuperscriptBox["\[Epsilon]", 
                          RowBox[{
                            FormBox[
                              FormBox["\[Mu]",
                                "TraditionalForm"],
                              "TraditionalForm"], "\[NoBreak]", 
                            FormBox[
                              FormBox["\[Nu]",
                                "TraditionalForm"],
                              "TraditionalForm"], "\[NoBreak]", 
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"]}]], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}]}]}], ")"}], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      FormBox[
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}],
                        "TraditionalForm"], ")"}], "2"]}], "-", 
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      FormBox[
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}],
                        "TraditionalForm"], ")"}], "2"], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}], ")"}], "m"], " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        SuperscriptBox["\[Epsilon]", 
                          RowBox[{
                            FormBox[
                              FormBox["\[Mu]",
                                "TraditionalForm"],
                              "TraditionalForm"], "\[NoBreak]", 
                            FormBox["p",
                              "TraditionalForm"], "\[NoBreak]", 
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"]}]], " ", 
                        SuperscriptBox[
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], 
                          FormBox[
                            FormBox["\[Nu]",
                              "TraditionalForm"],
                            "TraditionalForm"]]}], "+", 
                      RowBox[{
                        SuperscriptBox["\[Epsilon]", 
                          RowBox[{
                            FormBox[
                              FormBox["\[Mu]",
                                "TraditionalForm"],
                              "TraditionalForm"], "\[NoBreak]", 
                            FormBox[
                              FormBox["\[Nu]",
                                "TraditionalForm"],
                              "TraditionalForm"], "\[NoBreak]", 
                            FormBox["p",
                              "TraditionalForm"], "\[NoBreak]", 
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"]}]], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}]}]}], ")"}]}]}], ")"}], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"]}]]}], 
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                FormBox[
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["p",
                      "TraditionalForm"]}],
                  "TraditionalForm"], ")"}], "2"], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                FormBox[
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}],
                  "TraditionalForm"], ")"}], "2"]}]]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Factor2", "[", 
      RowBox[{"Calc", "[", 
        RowBox[{"f2p", "/.", 
          RowBox[{"p", "\[Rule]", 
            RowBox[{"-", "q"}]}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        RowBox[{"(", 
          RowBox[{"1", "-", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"], "\[NoBreak]", 
            FormBox["\[CapitalDelta]",
              "TraditionalForm"]}]], " ", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{
              FormBox["\[CapitalDelta]",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["q",
                "TraditionalForm"]}], ")"}], 
          RowBox[{"m", "-", "1"}]], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
Compare with the Feynman rule tabulated in \
Twist2GluonOperator.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{"q", ",", 
        RowBox[{"{", 
          RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[ImaginaryI]", " ", 
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"]}]], " ", 
        RowBox[{"(", 
          RowBox[{"1", "-", 
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "m"],
              "TraditionalForm"]}], ")"}], " ", 
        FormBox[
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["q",
                  "TraditionalForm"]}], ")"}], 
            RowBox[{"m", "-", "1"}]],
          "TraditionalForm"], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["quark-quark Feynman rule (unpolarized)", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"qo", "=", 
      RowBox[{"Lagrangian", "[", "\"\<oqu\>\"", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[ImaginaryI]", "m"], " ", 
        RowBox[{
          FormBox[
            OverscriptBox["\[Psi]", "_"],
            "TraditionalForm"], ".", 
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}], ".", 
          SuperscriptBox[
            SubscriptBox["D", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], 
            RowBox[{"m", "-", "1"}]], ".", 
          FormBox["\[Psi]",
            "TraditionalForm"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["quark-quark -gluon-gluon Feynman rule (unpolarized)", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"qo", "=", 
      RowBox[{"Lagrangian", "[", "\"\<oqu\>\"", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[ImaginaryI]", "m"], " ", 
        RowBox[{
          FormBox[
            OverscriptBox["\[Psi]", "_"],
            "TraditionalForm"], ".", 
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}], ".", 
          SuperscriptBox[
            SubscriptBox["D", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], 
            RowBox[{"m", "-", "1"}]], ".", 
          FormBox["\[Psi]",
            "TraditionalForm"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"qggf", "=", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            RowBox[{"QuantumField", "[", "QuarkField", "]"}], "[", "p", 
            "]"}], ",", 
          RowBox[{
            RowBox[{"QuantumField", "[", "AntiQuarkField", "]"}], "[", "q", 
            "]"}], ",", 
          RowBox[{
            RowBox[{"QuantumField", "[", 
              RowBox[{"GaugeField", ",", 
                RowBox[{"{", "\[Mu]", "}"}], ",", 
                RowBox[{"{", "a", "}"}]}], "]"}], "[", "r", "]"}], ",", 
          RowBox[{
            RowBox[{"QuantumField", "[", 
              RowBox[{"GaugeField", ",", 
                RowBox[{"{", "\[Nu]", "}"}], ",", 
                RowBox[{"{", "b", "}"}]}], "]"}], "[", "s", "]"}]}], 
        "}"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            FormBox["\[Psi]",
              "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "p", 
            "\[NoBreak]", ")"}], ",", 
          RowBox[{
            FormBox[
              OverscriptBox["\[Psi]", "_"],
              "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "q", 
            "\[NoBreak]", ")"}], ",", 
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", "r", ")"}], ",", 
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", "s", ")"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"n4", "=", 
      RowBox[{"FeynRule", "[", 
        RowBox[{"qo", ",", "qggf", ",", 
          RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "]"}]}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubsuperscriptBox["g", "s", "2"], " ", 
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"]}], ")"}], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          RowBox[{"(", 
            RowBox[{
              FormBox[
                TagBox[
                  RowBox[{
                    UnderoverscriptBox["\[Sum]", 
                      RowBox[{"j", "=", "0"}], 
                      RowBox[{"m", "-", "3"}]], "\<\"\"\>"}],
                  HoldForm],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", "1"}], ")"}], "j"], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "j"}], "+", "m", "-", "3"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}], ")"}], "i"], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}], "+", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["s",
                            "TraditionalForm"]}]}], ")"}], 
                    RowBox[{"j", "-", "i"}]]}],
                "TraditionalForm"]}], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          SubsuperscriptBox["g", "s", "2"], " ", 
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"]}], ")"}], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          RowBox[{"(", 
            RowBox[{
              FormBox[
                TagBox[
                  RowBox[{
                    UnderoverscriptBox["\[Sum]", 
                      RowBox[{"j", "=", "0"}], 
                      RowBox[{"m", "-", "3"}]], "\<\"\"\>"}],
                  HoldForm],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", "1"}], ")"}], "j"], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}], ")"}], "i"], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "j"}], "+", "m", "-", "3"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}], "+", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["s",
                            "TraditionalForm"]}]}], ")"}], 
                    RowBox[{"j", "-", "i"}]]}],
                "TraditionalForm"]}], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t4", "=", 
      RowBox[{"Twist2QuarkOperator", "[", 
        RowBox[{
          RowBox[{"{", "p", "}"}], ",", 
          RowBox[{"{", "q", "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"r", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"s", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
          RowBox[{"Polarization", "\[Rule]", "0"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"]}], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    FormBox[
                      TagBox[
                        RowBox[{
                          UnderoverscriptBox["\[Sum]", 
                            RowBox[{"i", "=", "0"}], 
                            RowBox[{"m", "-", "3"}]], "\<\"\"\>"}],
                        HoldForm],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      RowBox[{
                        UnderoverscriptBox["\[Sum]", 
                          RowBox[{"j", "=", "0"}], "i"], "\<\"\"\>"}],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      RowBox[{
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{"-", 
                              RowBox[{"(", 
                                RowBox[{
                                  FormBox["\[CapitalDelta]",
                                    "TraditionalForm"], "\[NoBreak]", 
                                  "\[CenterDot]", "\[NoBreak]", 
                                  FormBox["p",
                                    "TraditionalForm"]}], ")"}]}], ")"}], 
                          RowBox[{
                            RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], ")"}], "j"], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}]}], ")"}], 
                          RowBox[{"i", "-", "j"}]]}],
                      "TraditionalForm"]}], ")"}]}], "+", 
              RowBox[{
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    FormBox[
                      TagBox[
                        RowBox[{
                          UnderoverscriptBox["\[Sum]", 
                            RowBox[{"i", "=", "0"}], 
                            RowBox[{"m", "-", "3"}]], "\<\"\"\>"}],
                        HoldForm],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      RowBox[{
                        UnderoverscriptBox["\[Sum]", 
                          RowBox[{"j", "=", "0"}], "i"], "\<\"\"\>"}],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      RowBox[{
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{"-", 
                              RowBox[{"(", 
                                RowBox[{
                                  FormBox["\[CapitalDelta]",
                                    "TraditionalForm"], "\[NoBreak]", 
                                  "\[CenterDot]", "\[NoBreak]", 
                                  FormBox["p",
                                    "TraditionalForm"]}], ")"}]}], ")"}], 
                          RowBox[{
                            RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], ")"}], "j"], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["s",
                                  "TraditionalForm"]}]}], ")"}], 
                          RowBox[{"i", "-", "j"}]]}],
                      "TraditionalForm"]}], ")"}]}]}], ")"}]}], " ", 
        SuperscriptBox[
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SuperscriptBox[
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"n4", "-", "t4"}], "/.", 
            RowBox[{"OPEm", "\[Rule]", "5"}]}], "/.", 
          RowBox[{"s", "\[Rule]", 
            RowBox[{
              RowBox[{"-", "p"}], "-", "q", "-", "r"}]}]}], "/.", 
        RowBox[{"D", "\[Rule]", "4"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
 In general equality can be shown by Timing[Factor2[ \
FCE[Calc[ChangeDimension[FCE[OPESumExplicit[n4-t4]],4]/.s\[Rule]-p-q-r]]]] \
but it is a little bit slow ...\
\>", "Text"],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"qggf", ",", "n2", ",", "n4"}], "]"}]], "Input"],

Cell["QCD vertices", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"fii", "=", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            RowBox[{"QuantumField", "[", 
              RowBox[{"GaugeField", ",", 
                RowBox[{"{", "\[Mu]", "}"}], ",", 
                RowBox[{"{", "a", "}"}]}], "]"}], "[", "p", "]"}], ",", 
          RowBox[{
            RowBox[{"QuantumField", "[", 
              RowBox[{"GaugeField", ",", 
                RowBox[{"{", "\[Nu]", "}"}], ",", 
                RowBox[{"{", "b", "}"}]}], "]"}], "[", "q", "]"}], ",", 
          RowBox[{
            RowBox[{"QuantumField", "[", 
              RowBox[{"GaugeField", ",", 
                RowBox[{"{", "\[Rho]", "}"}], ",", 
                RowBox[{"{", "c", "}"}]}], "]"}], "[", "r", "]"}]}], 
        "}"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", "p", ")"}], ",", 
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", "q", ")"}], ",", 
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", "r", ")"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"g3", "=", 
      RowBox[{"FeynRule", "[", 
        RowBox[{
          RowBox[{"Lagrangian", "[", "\"\<QCD\>\"", "]"}], ",", "fii"}], 
        "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox[
                    FormBox["q",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], "-", 
                  SuperscriptBox[
                    FormBox["r",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], ")"}], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], "-", 
                  SuperscriptBox[
                    FormBox["r",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], ")"}]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]], "-", 
                  SuperscriptBox[
                    FormBox["q",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], ")"}]}]}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonVertex", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{"q", "-", "r"}],
                      "TraditionalForm"], ")"}], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], ")"}], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{"r", "-", "p"}],
                      "TraditionalForm"], ")"}], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], ")"}]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{"p", "-", "q"}],
                      "TraditionalForm"], ")"}], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], ")"}]}]}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", 
      RowBox[{"g3", "-", 
        RowBox[{"ChangeDimension", "[", 
          RowBox[{"%", ",", "4"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"fi4", "=", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            RowBox[{"QuantumField", "[", 
              RowBox[{"GaugeField", ",", 
                RowBox[{"{", "\[Mu]", "}"}], ",", 
                RowBox[{"{", "a", "}"}]}], "]"}], "[", "p", "]"}], ",", 
          RowBox[{
            RowBox[{"QuantumField", "[", 
              RowBox[{"GaugeField", ",", 
                RowBox[{"{", "\[Nu]", "}"}], ",", 
                RowBox[{"{", "b", "}"}]}], "]"}], "[", "q", "]"}], ",", 
          RowBox[{
            RowBox[{"QuantumField", "[", 
              RowBox[{"GaugeField", ",", 
                RowBox[{"{", "\[Rho]", "}"}], ",", 
                RowBox[{"{", "c", "}"}]}], "]"}], "[", "r", "]"}], ",", 
          RowBox[{
            RowBox[{"QuantumField", "[", 
              RowBox[{"GaugeField", ",", 
                RowBox[{"{", "\[Sigma]", "}"}], ",", 
                RowBox[{"{", "d", "}"}]}], "]"}], "[", "s", "]"}]}], 
        "}"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", "p", ")"}], ",", 
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", "q", ")"}], ",", 
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", "r", ")"}], ",", 
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", "s", ")"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"g4", "=", 
      RowBox[{"FeynRule", "[", 
        RowBox[{
          RowBox[{"Lagrangian", "[", "\"\<QCD\>\"", "]"}], ",", "fi4"}], 
        "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
                SuperscriptBox["g", 
                  RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
                SuperscriptBox["g", 
                  RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}]}], ")"}], 
          " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["si1",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["si1",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubsuperscriptBox["g", "s", "2"]}], "+", 
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
                SuperscriptBox["g", 
                  RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
                SuperscriptBox["g", 
                  RowBox[{"\[Rho]", "\[NoBreak]", "\[Sigma]"}]]}]}], ")"}], 
          " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["si1",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["si1",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubsuperscriptBox["g", "s", "2"]}], "+", 
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
                SuperscriptBox["g", 
                  RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
                SuperscriptBox["g", 
                  RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]]}]}], ")"}], 
          " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["si1",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["si1",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubsuperscriptBox["g", "s", "2"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonVertex", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u8",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u8",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u8",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u8",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u8",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u8",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}]}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", 
      RowBox[{"g4", "-", 
        RowBox[{"ChangeDimension", "[", 
          RowBox[{"%", ",", "4"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{
      "f2p", ",", "f2u", ",", "f3", ",", "f32", ",", "fi", ",", "fi4", ",", 
        "fii", ",", "g3", ",", "g4", ",", "gop", ",", "gou", ",", "n3", ",", 
        "nf3", ",", "n4", ",", "np2", ",", "npf3", ",", "p33", ",", "pf3", 
        ",", "pn3", ",", "pqo", ",", "qf", ",", "qp", ",", "qgf", ",", "qo", 
        ",", "t4"}], "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["FieldStrength", "Subsection",
  CellTags->"FieldStrength"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "FieldStrength[\[Mu], \[Nu], a] is the field strength tensor ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            SubscriptBox["\[PartialD]", "\[Mu]"], 
            SubsuperscriptBox["A", "\[Nu]", "a"]}], "-", 
          RowBox[{
            SubscriptBox["\[PartialD]", "\[Nu]"], 
            SubsuperscriptBox["A", "\[Mu]", "a"]}], "+", 
          RowBox[{
            SubscriptBox["g", "s"], " ", 
            SubsuperscriptBox["A", "\[Mu]", "b1"], 
            SubsuperscriptBox["A", "\[Nu]", "c1"]}]}], TraditionalForm]]],
  Cell[BoxData[
      FormBox[
        RowBox[{" ", 
          SuperscriptBox["f", 
            RowBox[{"a", " ", "b1", " ", "c1"}]]}], TraditionalForm]]],
  ". FieldStrength[\[Mu], \[Nu]] is the field strength tensor ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            SubscriptBox["\[PartialD]", "\[Mu]"], 
            SubsuperscriptBox["A", "\[Nu]", "\[Null]"]}], "-", 
          RowBox[{
            RowBox[{
              SubscriptBox["\[PartialD]", "\[Nu]"], 
              SubsuperscriptBox["A", "\[Mu]", "\[Null]"]}], "."}]}], 
        TraditionalForm]]],
  " The name of the field (A) and the coupling constant (g) can be set \
through the options or by additional arguments. The first two indices are \
interpreted as type LorentzIndex, except OPEDelta, which is converted to \
Momentum[OPEDelta]."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "FieldStrength", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
          RowBox[{"IndexPosition", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{"0", ",", "0"}], "}"}]}], ",", 
          RowBox[{"Symbol", "\[Rule]", "\<\"F\"\>"}], ",", 
          RowBox[{"QuantumField", "\[Rule]", "A"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["QuantumField",
    ButtonData:>"QuantumField",
    ButtonStyle->"Hyperlink",
    ButtonNote->"QuantumField"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FieldStrength", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox["F", 
        RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}], "\[Null]"], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FieldStrength", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox["F", 
        RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}], "a"], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FieldStrength", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox[
            FormBox["A",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[Null]"]}], "-", 
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox[
            FormBox["A",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[Null]"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FieldStrength", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a", ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox[
            FormBox["A",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox[
            FormBox["A",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]]}], "+", 
        RowBox[{
          SubscriptBox["g", "s"], " ", 
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b12",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["c47",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{"a", "\[NoBreak]", "b12", "\[NoBreak]", "c47"}]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"FieldStrength", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", 
          RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"QuantumField", "[", 
        RowBox[{
          RowBox[{"PartialD", "[", 
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
          "GaugeField", ",", 
          RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}], "-", 
      RowBox[{"QuantumField", "[", 
        RowBox[{
          RowBox[{"PartialD", "[", 
            RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], "]"}], ",", 
          "GaugeField", ",", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"FieldStrength", "[", 
        RowBox[{"\[Mu]", ",", "OPEDelta", ",", 
          RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"QuantumField", "[", 
        RowBox[{
          RowBox[{"PartialD", "[", 
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
          "GaugeField", ",", 
          RowBox[{"Momentum", "[", "OPEDelta", "]"}]}], "]"}], "-", 
      RowBox[{"QuantumField", "[", 
        RowBox[{
          RowBox[{"PartialD", "[", 
            RowBox[{"Momentum", "[", "OPEDelta", "]"}], "]"}], ",", 
          "GaugeField", ",", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FieldStrength", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a", ",", 
        RowBox[{"CouplingConstant", "\[Rule]", 
          RowBox[{"-", "Gstrong"}]}], ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox[
            FormBox["A",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox[
            FormBox["A",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          SubscriptBox["g", "s"], " ", 
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b13",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["c48",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{"a", "\[NoBreak]", "b13", "\[NoBreak]", "c48"}]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["FORM2FeynCalc", "Subsection",
  CellTags->"FORM2FeynCalc"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
FORM2FeynCalc[expr] translates the FORM expr into FeynCalc \
notation. FORM2FeynCalc[file] translates the FORM expresssions in file into \
FeynCalcnotation. FORM2FeynCalc[file, x1, x2, ...] reads in a file in \
FORM-format and translates the assignments for the variables a, b, ... into \
FeynCalc syntax. If the option Set is True, the variables x1, x2 are assigned \
to the right hand sides defined in the FORM - file.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynCalc2FORM",
    ButtonData:>"FeynCalc2FORM",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FeynCalc2FORM"],
  "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "FORM2FeynCalc", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Dimension", "\[Rule]", "4"}], ",", 
          RowBox[{"FinalSubstitutions", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"Dot", "\[Rule]", "Times"}], ",", 
          RowBox[{"HoldForm", "\[Rule]", "False"}], ",", 
          RowBox[{"LorentzIndex", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{"mu", ",", "nu", ",", "al", ",", "be"}], "}"}]}], ",", 
          RowBox[{"Set", "\[Rule]", "False"}], ",", 
          RowBox[{"Replace", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"Vectors", "\[Rule]", "Automatic"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FORM2FeynCalc", "[", "\"\<p.q + 2*x m^2\>\"", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"2", " ", "x", " ", 
          SuperscriptBox["m", "2"]}], "+", 
        FormBox[
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"]}],
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"2", " ", 
        SuperscriptBox["m", "2"], " ", "x"}], "+", 
      RowBox[{"SP", "[", 
        RowBox[{"p", ",", "q"}], "]"}]}]], "Output"]
}, Open  ]],

Cell["\<\
Functions are automatically converted right, but bracketed \
expressions need to be substituted explicitly.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FORM2FeynCalc", "[", 
      RowBox[{"\"\<x +f(z)+ log(x)^2+[li2(1-x)]\>\"", ",", 
        RowBox[{"Replace", "\[Rule]", 
          RowBox[{"{", 
            RowBox[{"\"\<[li2(1-x)]\>\"", "->", "\"\<PolyLog[2,1-x]\>\""}], 
            "}"}]}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["log", "2"], "(", "x", ")"}], "+", "x", "+", 
        RowBox[{"f", "(", "z", ")"}], "+", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          RowBox[{"1", "-", "x"}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"x", "+", 
      RowBox[{"f", "[", "z", "]"}], "+", 
      SuperscriptBox[
        RowBox[{"Log", "[", "x", "]"}], "2"], "+", 
      RowBox[{"PolyLog", "[", 
        RowBox[{"2", ",", 
          RowBox[{"1", "-", "x"}]}], "]"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FORM2FeynCalc", "[", "\"\<x + [(1)]*y -[(-1)^m]\>\"", "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"x", "+", 
        RowBox[{"y", " ", 
          RowBox[{"Hold", "(", "1", ")"}]}], "-", 
        RowBox[{"Hold", "(", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ReleaseHold", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"x", "-", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "m"], "+", "y"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FORM2FeynCalc", "[", "\"\<p(mu)*q(nu)+d_(mu,nu)\>\"", "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"mu", " ", "p", " ", 
          SuperscriptBox["q", "nu"]}], "+", 
        SuperscriptBox["g", 
          RowBox[{"mu", "\[NoBreak]", "nu"}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"mu", " ", "p", " ", 
        RowBox[{"FV", "[", 
          RowBox[{"q", ",", "nu"}], "]"}]}], "+", 
      RowBox[{"MT", "[", 
        RowBox[{"mu", ",", "nu"}], "]"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FORM2FeynCalc", "[", 
      RowBox[{"\"\<p(mu)*q(nu)+d_(mu,nu)\>\"", ",", 
        RowBox[{"Replace", "->", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"mu", "->", "\[Mu]"}], ",", 
              RowBox[{"nu", "->", "\[Nu]"}]}], "}"}]}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"p", " ", "\[Mu]", " ", 
          SuperscriptBox["q", "\[Nu]"]}], "+", 
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
    "FORM2FeynCalc", "[", 
      "\"\<i_*az*bz*aM^2*D1*[(1)]*b_G1 * ( 4*eperp(mu,nu)*avec.bvec*blam \
)\>\"", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", "\[ImaginaryI]", " ", 
        SuperscriptBox["aM", "2"], " ", "az", " ", "blam", " ", "bz", " ", 
        "b$G1", " ", "D1", " ", 
        RowBox[{"eperp", "(", 
          RowBox[{"mu", ",", "nu"}], ")"}], " ", 
        RowBox[{"Hold", "(", "1", ")"}], " ", 
        FormBox[
          RowBox[{
            FormBox["avec",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["bvec",
              "TraditionalForm"]}],
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["FourDivergence", "Subsection",
  CellTags->"FourDivergence"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
FourDivergence[exp, FourVector[p, mu]] calculates the partial \
derivative of exp w.r.t. p(mu). FourDivergence[exp, FourVector[p, mu], \
FourVector[p,nu], ...]gives the multiple derivative.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["RussianTrick",
    ButtonData:>"RussianTrick",
    ButtonStyle->"Hyperlink",
    ButtonNote->"RussianTrick"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{"ScalarProduct", "[", 
        RowBox[{"p", ",", "q"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["p",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        FormBox["q",
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FourDivergence", "[", 
      RowBox[{"t", ",", 
        RowBox[{"FourVector", "[", 
          RowBox[{"q", ",", "\[Mu]"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        FormBox["p",
          "TraditionalForm"], 
        FormBox[
          FormBox["\[Mu]",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{"ScalarProduct", "[", 
        RowBox[{
          RowBox[{"p", "-", "k"}], ",", "q"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"(", "\[NoBreak]", 
        FormBox[
          RowBox[{"p", "-", "k"}],
          "TraditionalForm"], "\[NoBreak]", ")", "\[NoBreak]", 
        "\[CenterDot]", "\[NoBreak]", 
        FormBox["q",
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FourDivergence", "[", 
      RowBox[{"t", ",", 
        RowBox[{"FourVector", "[", 
          RowBox[{
            RowBox[{"k", "-", "p"}], ",", "\[Mu]"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", "t", "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["FourLaplacian", "Subsection",
  CellTags->"FourLaplacian"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "FourLaplacian[exp, p, q] is ",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[PartialD]", 
          RowBox[{"/", 
            RowBox[{"\[PartialD]", 
              SubscriptBox["p", "\[Mu]"]}]}], " "}], TraditionalForm]]],
  Cell[BoxData[
      FormBox[
        RowBox[{"\[PartialD]", 
          RowBox[{"/", 
            RowBox[{"\[PartialD]", 
              SubscriptBox["q", "\[Mu]"]}]}], " "}], TraditionalForm]]],
  "exp."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "FourLaplacian", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Dimension", "\[Rule]", "D"}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FourDivergence",
    ButtonData:>"FourDivergence",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FourDivergence"],
  ", ",
  ButtonBox["RussianTrick",
    ButtonData:>"RussianTrick",
    ButtonStyle->"Hyperlink",
    ButtonNote->"RussianTrick"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SP", "[", 
      RowBox[{"q", ",", "q"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        FormBox["q",
          "TraditionalForm"], "2"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FourLaplacian", "[", 
      RowBox[{"%", ",", "q", ",", "q"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"2", " ", "D"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"SOD", "[", "q", "]"}], "^", "OPEm"}], 
        RowBox[{"FAD", "[", 
          RowBox[{"q", ",", 
            RowBox[{"q", "-", "p"}]}], "]"}]}], "//", "FCI"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{
              FormBox["\[CapitalDelta]",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["q",
                "TraditionalForm"]}], ")"}], "m"], 
        RowBox[{
          SuperscriptBox["q", "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"q", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FourLaplacian", "[", 
      RowBox[{"%", ",", "q", ",", " ", "q"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{"4", " ", "m", " ", 
            RowBox[{
              FormBox["\[CapitalDelta]",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["p",
                "TraditionalForm"]}], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], ")"}], 
              RowBox[{"m", "-", "1"}]]}], 
          RowBox[{
            SuperscriptBox["q", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
        FractionBox[
          RowBox[{"2", " ", "D", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], ")"}], "m"]}], 
          RowBox[{
            SuperscriptBox["q", "2"], ".", 
            SuperscriptBox["q", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
        FractionBox[
          RowBox[{"4", " ", "m", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], ")"}], "m"]}], 
          RowBox[{
            SuperscriptBox["q", "2"], ".", 
            SuperscriptBox["q", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "+", 
        FractionBox[
          RowBox[{"12", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], ")"}], "m"]}], 
          RowBox[{
            SuperscriptBox["q", "2"], ".", 
            SuperscriptBox["q", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
        FractionBox[
          RowBox[{"2", " ", "D", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], ")"}], "m"]}], 
          RowBox[{
            SuperscriptBox["q", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
        FractionBox[
          RowBox[{"4", " ", "m", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], ")"}], "m"]}], 
          RowBox[{
            SuperscriptBox["q", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "+", 
        FractionBox[
          RowBox[{"12", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], ")"}], "m"]}], 
          RowBox[{
            SuperscriptBox["q", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
        FractionBox[
          RowBox[{"4", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], ")"}], "m"], " ", 
            SuperscriptBox[
              FormBox["p",
                "TraditionalForm"], "2"]}], 
          RowBox[{
            SuperscriptBox["q", "2"], ".", 
            SuperscriptBox["q", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"]}]]}], TraditionalForm]], 
  "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["FourVector", "Subsection",
  CellTags->"FourVector"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "FourVector[p, ",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "] is the four-dimensional vector p with Lorentz index ",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  ". A vector with space-time Dimension D is obtained by supplying the option \
Dimension \[Rule] D."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FV",
    ButtonData:>"FV",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FV"],
  ", ",
  ButtonBox["FVD",
    ButtonData:>"FVD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FVD"],
  ", ",
  ButtonBox["Pair",
    ButtonData:>"Pair",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Pair"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FourVector", "[", 
      RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubscriptBox["p", "\[Mu]"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FourVector", "[", 
      RowBox[{
        RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubscriptBox[
        RowBox[{"(", "\[NoBreak]", 
          FormBox[
            TagBox[
              RowBox[{"p", "-", "q"}],
              HoldForm],
            "TraditionalForm"], "\[NoBreak]", ")"}], "\[Mu]"], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"FourVector", "[", 
        RowBox[{"p", ",", "\[Mu]"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    RowBox[{"FourVector", "[", 
      RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"FourVector", "[", 
        RowBox[{"p", ",", "\[Mu]", ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    RowBox[{"FourVector", "[", 
      RowBox[{"p", ",", "\[Mu]", ",", 
        RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"FCE", "[", 
        RowBox[{"FourVector", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    RowBox[{"FourVector", "[", 
      RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Output"]
}, Open  ]],

Cell["\<\
There is no special function to expand momenta in FourVector. Since \
FourVector is turned into Pair internally ExpandScalarProduct may be \
used.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandScalarProduct", "[", 
      RowBox[{"FourVector", "[", 
        RowBox[{
          RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], "-", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["FreeQ2", "Subsection",
  CellTags->"FreeQ2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
FreeQ2[expr, {form1, form2, ...}] yields True if expr does not \
contain any occurence of form1, form2, ... and False otherwise. FreeQ2[expr, \
form] is the same as FreeQ[expr, form].\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["SelectFree",
    ButtonData:>"SelectFree",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SelectFree"],
  ", ",
  ButtonBox["SelectNotFree",
    ButtonData:>"SelectNotFree",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SelectNotFree"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FreeQ2", "[", 
      RowBox[{
        RowBox[{"x", "+", 
          RowBox[{"f", "[", "x", "]"}], "+", "y"}], ",", " ", 
        RowBox[{"{", 
          RowBox[{"a", ",", "x"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["False", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FreeQ2", "[", 
      RowBox[{
        RowBox[{"x", "+", 
          RowBox[{"f", "[", "x", "]"}], "+", "y"}], ",", 
        RowBox[{"{", 
          RowBox[{"a", ",", "b"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["True", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FreeQ2", "[", 
      RowBox[{"x", ",", " ", "y"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["True", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FreeQ2", "[", 
      RowBox[{
        RowBox[{"f", "[", "x", "]"}], ",", " ", "f"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["False", TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["FromTFi", "Subsection",
  CellTags->"FromTFi"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
FromTFi[expr, q1, q2, p] translates the TFi notatation from the \
TARCER package to the usual FeynCalc notation. See TFi for details on the \
conventions.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  ButtonBox["TFi",
    ButtonData:>"TFi",
    ButtonStyle->"Hyperlink",
    ButtonNote->"TFi"],
  ", ",
  ButtonBox["ToTFi",
    ButtonData:>"ToTFi",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ToTFi"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FAD", "[", 
        RowBox[{"q1", ",", 
          RowBox[{"q1", "-", "p"}], ",", 
          RowBox[{"{", 
            RowBox[{"q2", ",", "M"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"q2", "-", "p"}], ",", "m"}], "}"}], ",", 
          RowBox[{"q1", "-", "q2"}]}], "]"}], "//", "ToTFi"}]], "Input"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        SubsuperscriptBox[
          StyleBox["F",
            SingleLetterItalics->False,
            FontWeight->"Bold"], 
          RowBox[{
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "M"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "m"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}],
              "TraditionalForm"]}], 
          RowBox[{"(", 
            FormBox["D",
              "TraditionalForm"], ")"}]],
        TFi[ D, 
          SPD[ p, p], {{1, 0}, {1, M}, {1, 0}, {1, m}, {1, 0}}],
        Editable->True], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FromTFi", "[", 
      RowBox[{
        RowBox[{"TFi", "[", 
          RowBox[{"D", ",", 
            RowBox[{"SPD", "[", 
              RowBox[{"p", ",", "p"}], "]"}], ",", 
            RowBox[{"SOD", "[", "p", "]"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{"1", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"1", ",", "M"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"1", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"1", ",", "m"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"1", ",", "0"}], "}"}]}], "}"}]}], "]"}], ",", 
        "q1", ",", "q2", ",", "p"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          SuperscriptBox[
            InterpretationBox[
              SubscriptBox["q", "2"],
              q2], "2"], ".", 
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    InterpretationBox[
                      SubscriptBox["q", "1"],
                      q1], "-", "p"}], ")"}], "2"], "-", 
              SuperscriptBox["M", "2"]}], ")"}], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                InterpretationBox[
                  SubscriptBox["q", "2"],
                  q2], "-", "p"}], ")"}], "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                InterpretationBox[
                  SubscriptBox["q", "1"],
                  q1], "-", 
                InterpretationBox[
                  SubscriptBox["q", "2"],
                  q2]}], ")"}], "2"], ".", 
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox[
                InterpretationBox[
                  SubscriptBox["q", "1"],
                  q1], "2"], "-", 
              SuperscriptBox["m", "2"]}], ")"}]}]], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FromTFi", "[", 
      RowBox[{
        RowBox[{"TFi", "[", 
          RowBox[{"D", ",", 
            RowBox[{"SPD", "[", 
              RowBox[{"p", ",", "p"}], "]"}], ",", 
            RowBox[{"SOD", "[", "p", "]"}], ",", 
            RowBox[{"{", 
              RowBox[{"0", ",", "1"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{"1", ",", "m"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"1", ",", "M"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"1", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"1", ",", "m"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"1", ",", "0"}], "}"}]}], "}"}]}], "]"}], ",", 
        "q1", ",", "q2", ",", "p"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox[
            InterpretationBox[
              SubscriptBox["q", "2"],
              q2],
            "TraditionalForm"]}], 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                InterpretationBox[
                  SubscriptBox["q", "1"],
                  q1], "-", 
                InterpretationBox[
                  SubscriptBox["q", "2"],
                  q2]}], ")"}], "2"], ".", 
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox[
                InterpretationBox[
                  SubscriptBox["q", "2"],
                  q2], "2"], "-", 
              SuperscriptBox["M", "2"]}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    InterpretationBox[
                      SubscriptBox["q", "2"],
                      q2], "-", "p"}], ")"}], "2"], "-", 
              SuperscriptBox["m", "2"]}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox[
                InterpretationBox[
                  SubscriptBox["q", "1"],
                  q1], "2"], "-", 
              SuperscriptBox["m", "2"]}], ")"}], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                InterpretationBox[
                  SubscriptBox["q", "1"],
                  q1], "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["FRH", "Subsection",
  CellTags->"FRH"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
FRH[exp_] := FixedPoint[ReleaseHold, exp], i.e., FRH removes all \
HoldForm and Hold in exp.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Isolate",
    ButtonData:>"Isolate",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Isolate"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Hold", "[", 
      RowBox[{"1", "-", "1", " ", "-", " ", 
        RowBox[{"Hold", "[", 
          RowBox[{"2", "-", "2"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"Hold", "(", 
        RowBox[{
          RowBox[{"-", 
            RowBox[{"Hold", "(", 
              RowBox[{"2", "-", "2"}], ")"}]}], "+", "1", "-", "1"}], ")"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FRH", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Isolate", "[", 
      RowBox[{
        RowBox[{"Solve", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"x", "^", "3"}], "-", "x", "-", "1"}], "==", "0"}], 
          "]"}], ",", "x", ",", 
        RowBox[{"IsolateNames", "->", "H"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"x", "\[Rule]", 
              TagBox[
                RowBox[{"H", "(", "3", ")"}],
                HoldForm]}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"x", "\[Rule]", 
              TagBox[
                RowBox[{"H", "(", "6", ")"}],
                HoldForm]}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"x", "\[Rule]", 
              TagBox[
                RowBox[{"H", "(", "7", ")"}],
                HoldForm]}], "}"}]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FRH", "[", 
      RowBox[{"H", "[", "3", "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "3"], " ", 
          RadicalBox[
            RowBox[{
              FractionBox["27", "2"], "-", 
              FractionBox[
                RowBox[{"3", " ", 
                  SqrtBox["69"]}], "2"]}], "3"]}], "+", 
        FractionBox[
          RadicalBox[
            RowBox[{
              FractionBox["1", "2"], " ", 
              RowBox[{"(", 
                RowBox[{"9", "+", 
                  SqrtBox["69"]}], ")"}]}], "3"], 
          SuperscriptBox["3", 
            RowBox[{"2", "/", "3"}]]]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["FunctionalD", "Subsection",
  CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
FunctionalD[expr, {QuantumField[name, {mu}, {a}][p], ...}] \
calculates the functional derivative of expr with respect to the field list \
(with incoming momenta p, etc.) and does the fourier transform. \
FunctionalD[expr, {QuantumField[name, {mu},{a}], ...}] calculates the \
functional derivate and does partial integration but omits the x-space delta \
functions.\
\>", "Text"],

Cell["FunctionalD is a low level function used in FeynRule.", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynRule",
    ButtonData:>"FeynRule",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FeynRule"],
  ", ",
  ButtonBox["QuantumField",
    ButtonData:>"QuantumField",
    ButtonStyle->"Hyperlink",
    ButtonNote->"QuantumField"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[TextData[{
  "Instead of the usual ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            RowBox[{"\[Delta]\[Phi]", "(", "x", ")"}], "/", 
            RowBox[{"\[Delta]\[Phi]", "(", "y", ")"}]}], "=", 
          RowBox[{
            SuperscriptBox["\[Delta]", 
              RowBox[{"(", "D", ")"}]], "(", 
            RowBox[{"x", "-", "y"}], ")"}]}], TraditionalForm]]],
  " the arguments and the \[Delta] function are omitted, i.e., for the \
program for simplicity: ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"\[Delta]\[Phi]", "/", "\[Delta]\[Phi]"}], "=", "1"}], 
        TraditionalForm]]]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FunctionalD", "[", 
      RowBox[{
        RowBox[{"QuantumField", "[", "\[Phi]", "]"}], ",", 
        RowBox[{"QuantumField", "[", "\[Phi]", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FunctionalD", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"QuantumField", "[", "\[Phi]", "]"}], "^", "2"}], ",", 
        RowBox[{"QuantumField", "[", "\[Phi]", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"2", " ", "\[Phi]"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "Instead of the usual ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{"\[Delta]", " ", 
                RowBox[{
                  SubscriptBox["\[PartialD]", "\[Mu]"], 
                  RowBox[{"\[Phi]", "(", "x", ")"}]}]}], ")"}], "/", 
            RowBox[{"\[Delta]\[Phi]", "(", "y", ")"}]}], "=", 
          RowBox[{
            SubscriptBox["\[PartialD]", "\[Mu]"], 
            RowBox[{
              SuperscriptBox["\[Delta]", 
                RowBox[{"(", "D", ")"}]], "(", 
              RowBox[{"x", "-", "y"}], ")"}]}]}], TraditionalForm]]],
  " the arguments are omitted, and the ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["\[PartialD]", "\[Mu]"], TraditionalForm]]],
  "operator is specified by default to be an integration by parts operator, \
i.e., the right hand side will be just ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"-", 
            FormBox[
              SubscriptBox["\[PartialD]", "\[Mu]"],
              "TraditionalForm"]}], ","}], TraditionalForm]]],
  " or, more precisely (by default) ",
  Cell[BoxData[
      FormBox[
        RowBox[{"-", 
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]]}], TraditionalForm]]],
  "."
}], "Text",
  ZeroWidthTimes->True],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FunctionalD", "[", 
      RowBox[{
        RowBox[{"QuantumField", "[", 
          RowBox[{
            RowBox[{"PartialD", "[", "\[Mu]", "]"}], ",", "\[Phi]"}], "]"}], 
        ",", 
        RowBox[{"QuantumField", "[", "\[Phi]", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[RightArrow]"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"S", "[", "\[Phi]", "]"}], " ", "=", 
        RowBox[{
          RowBox[{"1", "/", "2"}], 
          RowBox[{"\[Integral]", " ", 
            RowBox[{
              SuperscriptBox["d", "D"], 
              RowBox[{"x", " ", "[", " ", 
                RowBox[{
                  RowBox[{
                    RowBox[{
                      SubscriptBox["\[PartialD]", "\[Mu]"], 
                      RowBox[{"\[Phi]", "(", "x", ")"}]}], " ", 
                    RowBox[{
                      SuperscriptBox["\[PartialD]", "\[Mu]"], 
                      RowBox[{"\[Phi]", "(", "x", ")"}]}]}], "-", 
                  RowBox[{
                    SuperscriptBox["m", "2"], 
                    RowBox[{"\[Phi]", "(", "x", ")"}], " ", 
                    RowBox[{"\[Phi]", "(", "y", ")"}]}]}], "]"}]}]}]}]}], 
      TraditionalForm]]]], "Text",
  ZeroWidthTimes->True],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"s", "[", "\[Phi]", "]"}], "=", 
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"QuantumField", "[", 
                RowBox[{
                  RowBox[{"PartialD", "[", "\[Mu]", "]"}], ",", "\[Phi]"}], 
                "]"}], ".", 
              RowBox[{"QuantumField", "[", 
                RowBox[{
                  RowBox[{"PartialD", "[", "\[Mu]", "]"}], ",", "\[Phi]"}], 
                "]"}]}], "-", 
            RowBox[{
              RowBox[{"m", "^", "2"}], " ", 
              RowBox[{
                RowBox[{"QuantumField", "[", "\[Phi]", "]"}], ".", 
                RowBox[{"QuantumField", "[", "\[Phi]", "]"}]}]}]}], ")"}], 
        "/", "2"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["\[Phi]", "\[Null]", "\[Null]"]}], ".", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["\[Phi]", "\[Null]", "\[Null]"]}]}], "-", 
            RowBox[{
              SuperscriptBox["m", "2"], " ", 
              RowBox[{"\[Phi]", ".", "\[Phi]"}]}]}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FunctionalD", "[", 
      RowBox[{
        RowBox[{"s", "[", "\[Phi]", "]"}], ",", 
        RowBox[{"QuantumField", "[", "\[Phi]", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", "\[Phi]"}], " ", 
          SuperscriptBox["m", "2"]}], "-", 
        RowBox[{
          RowBox[{
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"], 
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"]}], 
          SubsuperscriptBox["\[Phi]", "\[Null]", "\[Null]"]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"S", "[", "A", "]"}], " ", "=", " ", 
        RowBox[{"-", 
          RowBox[{"\[Integral]", " ", 
            RowBox[{
              SuperscriptBox["d", "D"], "x", " ", 
              FractionBox["1", "4"], " ", 
              RowBox[{
                SubsuperscriptBox["F", "a", "\[Mu]\[Nu]"], "(", "x", ")"}], 
              " ", 
              RowBox[{
                SubscriptBox["F", "\[Mu]\[Nu]a"], "(", "x", ")"}]}]}]}]}], 
      TraditionalForm]]]], "Text",
  ZeroWidthTimes->True],

Cell[TextData[StyleBox["First approach",
  FontWeight->"Bold"]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"F1", "=", 
      RowBox[{"FieldStrength", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a", ",", 
          RowBox[{"{", 
            RowBox[{"A", ",", "b", ",", "c"}], "}"}], ",", "1", ",", 
          RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]]}], "+", 
        RowBox[{
          RowBox[{
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{"a", "\[NoBreak]", "b", "\[NoBreak]", "c"}]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"F2", "=", 
      RowBox[{"FieldStrength", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a", ",", 
          RowBox[{"{", 
            RowBox[{"A", ",", "d", ",", "e"}], "}"}], ",", "1", ",", 
          RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]]}], "+", 
        RowBox[{
          RowBox[{
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{"a", "\[NoBreak]", "d", "\[NoBreak]", "e"}]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"S", "[", "A", "]"}], "=", 
      RowBox[{
        RowBox[{"-", 
          RowBox[{"1", "/", "4"}]}], 
        RowBox[{"F1", ".", "F2"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "4"]}], " ", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                RowBox[{
                  SubsuperscriptBox["A", 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubsuperscriptBox["A", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SubscriptBox["f", 
                  RowBox[{"a", "\[NoBreak]", "b", "\[NoBreak]", "c"}]]}]}], 
            ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                RowBox[{
                  SubsuperscriptBox["A", 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubsuperscriptBox["A", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["e",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SubscriptBox["f", 
                  RowBox[{"a", "\[NoBreak]", "d", "\[NoBreak]", "e"}]]}]}], 
            ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "In order to derive the equation of motion the functional derivative of ",
  Cell[BoxData[
      FormBox["S", TraditionalForm]]],
  " with respect to ",
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["A", "\[Sigma]", "g"], TraditionalForm]]],
  "has to be set to zero. Bearing in mind that for FeynCalc we have to be \
precise as to where which operators (coming from the substitution of the \
derivative of the \[Delta] function) act:, act with the functional derivative \
operator on the first field strength:"
}], "Text"],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            FormBox[
              RowBox[{
                FormBox[
                  RowBox[{
                    FormBox[
                      RowBox[{"0", " ", "=", " ", 
                        RowBox[{
                          FormBox[
                            RowBox[{
                              RowBox[{"(", 
                                RowBox[{"\[Delta]", "S"}], ")"}], "/", 
                              RowBox[{"(", 
                                RowBox[{"\[Delta]", 
                                  RowBox[{
                                    SubsuperscriptBox["A", "\[Sigma]", "g"], 
                                    "(", "y", ")"}]}], ")"}]}],
                            "TraditionalForm"], "=", 
                          RowBox[{"-", 
                            RowBox[{"2", "/", "4"}]}]}]}],
                      "TraditionalForm"], 
                    RowBox[{"\[Integral]", 
                      RowBox[{
                        SuperscriptBox["d", "D"], "x"}]}]}],
                  "TraditionalForm"], " ", 
                RowBox[{"(", 
                  RowBox[{"\[Delta]", "/", 
                    RowBox[{"(", 
                      RowBox[{"\[Delta]", 
                        RowBox[{
                          SubsuperscriptBox["A", "\[Sigma]", "g"], "(", "y", 
                          ")"}]}], ")"}]}]}]}],
              "TraditionalForm"], " ", 
            RowBox[{
              SubscriptBox["F", "\[Mu]\[Nu]a"], "(", "x", ")"}]}], ")"}], 
        TraditionalForm]]],
  Cell[BoxData[
      FormBox[
        RowBox[{
          FormBox[
            RowBox[{" ", 
              SubsuperscriptBox["F", "a", 
                RowBox[{",", "\[Mu]\[Nu]"}]]}],
            "TraditionalForm"], "(", "x", ")"}], TraditionalForm]]]
}], "Text",
  ZeroWidthTimes->True],

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{"See", " ", "what", " ", "happens", " ", "with", " ", "just", 
        RowBox[{
          FormBox[
            RowBox[{
              RowBox[{"(", 
                RowBox[{"\[Delta]", 
                  RowBox[{"S", "[", "A", "]"}]}], ")"}], "/", 
              RowBox[{"(", 
                RowBox[{"\[Delta]", 
                  SubsuperscriptBox["A", "\[Sigma]", "g"]}], ")"}]}],
            "TraditionalForm"], "."}]}], TraditionalForm]]]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Ag", "=", 
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"{", "\[Sigma]", "}"}], ",", 
          RowBox[{"{", "g", "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox["A", 
        FormBox[
          FormBox["\[Sigma]",
            "TraditionalForm"],
          "TraditionalForm"], 
        FormBox[
          FormBox["g",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{"FunctionalD", "[", 
        RowBox[{"F1", ",", "Ag"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            SuperscriptBox["g", 
              RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]]}], " ", 
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[
"In order to minimize the number of dummy indices, replace b \[Rule] c."], 
  "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", " ", "=", " ", 
      RowBox[{"t1", " ", "/.", " ", 
        RowBox[{"b", "\[Rule]", "c"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            SuperscriptBox["g", 
              RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]]}], " ", 
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{"\[Nu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "Instead of inserting the definition for the second ",
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["F", "a", "\[Mu]\[Nu]"], TraditionalForm]]],
  ", introduce a QuantumField object with antisymmetry built into the Lorentz \
indices:"
}], "Text"],

Cell[BoxData[
    RowBox[{"F", "/:", " ", 
      RowBox[{"QuantumField", "[", 
        RowBox[{
        "pard___", ",", "F", ",", "\[Beta]_", ",", "\[Alpha]_", ",", "s_"}], 
        "]"}], ":=", 
      RowBox[{
        RowBox[{"-", 
          RowBox[{"QuantumField", "[", 
            RowBox[{
            "pard", ",", "F", ",", "\[Alpha]", ",", "\[Beta]", ",", "s"}], 
            "]"}]}], "/;", 
        RowBox[{"!", 
          RowBox[{"OrderedQ", "[", 
            RowBox[{"{", 
              RowBox[{"\[Beta]", ",", "\[Alpha]"}], "}"}], "]"}]}]}]}]], 
  "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuantumField", "[", 
      RowBox[{"F", ",", 
        RowBox[{"{", 
          RowBox[{"\[Mu]", ",", "\[Nu]"}], "}"}], ",", 
        RowBox[{"{", "a", "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox["F", 
        RowBox[{
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]}], 
        FormBox[
          FormBox["a",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "/.", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{"\[Mu]", "\[RuleDelayed]", "\[Nu]"}], ",", 
          RowBox[{"\[Nu]", "\[RuleDelayed]", "\[Mu]"}]}], "}"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        SubsuperscriptBox["F", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]}], 
          FormBox[
            FormBox["a",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{
        RowBox[{"Contract", "[", 
          RowBox[{"ExpandPartialD", "[", 
            RowBox[{
              RowBox[{"-", 
                RowBox[{"1", "/", "2"}]}], " ", 
              RowBox[{"t1", ".", 
                RowBox[{"QuantumField", "[", 
                  RowBox[{"F", ",", 
                    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
                    RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
                    RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]}]}], "]"}], 
          "]"}], "/.", 
        RowBox[{"Dot", "\[Rule]", "Times"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["F", 
              RowBox[{
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]}], 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]]}]}], "+", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["F", 
              RowBox[{
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]}], 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]]}]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["F", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]}], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["F", 
            RowBox[{
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]}], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t3", " ", "=", " ", 
      RowBox[{"t2", " ", "/.", " ", 
        RowBox[{"\[Nu]", "\[Rule]", "\[Mu]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox["F", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]}], 
            FormBox[
              FormBox["g",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["F", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]}], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t4", "=", 
      RowBox[{
        RowBox[{"FCE", "[", "t3", "]"}], "/.", " ", 
        RowBox[{
          RowBox[{"SUNF", "[", 
            RowBox[{"a", ",", "c", ",", "g"}], "]"}], "\[Rule]", 
          RowBox[{"-", 
            RowBox[{"SUNF", "[", 
              RowBox[{"g", ",", "c", ",", "a"}], "]"}]}]}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox["F", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]}], 
            FormBox[
              FormBox["g",
                "TraditionalForm"],
              "TraditionalForm"]]}], "+", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["F", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]}], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{"g", "\[NoBreak]", "c", "\[NoBreak]", "a"}]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "Since the variational derivative vanishes t4 implies that 0 = ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["D", "\[Mu]"], 
          SubsuperscriptBox["F", "g", "\[Mu]\[Sigma]"]}], TraditionalForm]]],
  " ."
}], "Text"],

Cell[TextData[StyleBox["Second approach",
  FontWeight->"Bold"]], "Text"],

Cell["\<\
It is of course also possible to do the functional deriviate on the \
S[A] with both field strength tensors inserted.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"S", "[", "A", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "4"]}], " ", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                RowBox[{
                  SubsuperscriptBox["A", 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubsuperscriptBox["A", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SubscriptBox["f", 
                  RowBox[{"a", "\[NoBreak]", "b", "\[NoBreak]", "c"}]]}]}], 
            ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                RowBox[{
                  SubsuperscriptBox["A", 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubsuperscriptBox["A", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["e",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SubscriptBox["f", 
                  RowBox[{"a", "\[NoBreak]", "d", "\[NoBreak]", "e"}]]}]}], 
            ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"r1", "=", 
      RowBox[{"FunctionalD", "[", 
        RowBox[{
          RowBox[{"S", "[", "A", "]"}], ",", "Ag"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"], 
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"]}], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["g",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ")"}]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"], 
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"]}], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["g",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ")"}]}], "+", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"], 
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"]}], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["g",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ")"}]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"], 
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"]}], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["g",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ")"}]}], "-", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"]]}]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"]]}]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"]]}]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"]]}]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["c",
                    "TraditionalForm"],
                  "TraditionalForm"]]}]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["c",
                    "TraditionalForm"],
                  "TraditionalForm"]]}]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["e",
                    "TraditionalForm"],
                  "TraditionalForm"]]}]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["e",
                    "TraditionalForm"],
                  "TraditionalForm"]]}]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["d",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["d",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ".", 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
This is just funcional derivatves and partial integration and \
simple contraction of indices. No attempt is made to rename dummy indices \
(since this is difficult in general ...).\
\>", "Text"],

Cell["\<\
With a general replacement rule only valid for commuting fields the \
color indices can be canonicalized a bit more. The idea is to use the \
commutative properties of the vector fields, and canonicalize the color \
indices by a trick. This function will work on any commuting product of \
fields.\
\>", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", "symfun", "]"}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"symfun", "[", 
        RowBox[{"z_", ",", "fieldname_Symbol"}], "]"}], ":=", 
      RowBox[{"Expand", "[", 
        RowBox[{"SUNSimplify", "[", 
          RowBox[{"FixedPoint", "[", 
            RowBox[{
              RowBox[{
                RowBox[{"Collect2", "[", 
                  RowBox[{
                    RowBox[{
                      RowBox[{
                        RowBox[{"DotSimplify", "[", 
                          RowBox[{"#1", "/.", 
                            RowBox[{"Times", "\[Rule]", "Dot"}]}], "]"}], 
                        "/.", 
                        RowBox[{
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{"(", 
                                RowBox[{
                                  RowBox[{"(", "qi___", ")"}], ".", 
                                  RowBox[{"QuantumField", "[", 
                                    RowBox[{
                                    "par1___", ",", "fieldname", ",", "li1_", 
                                      ",", "sui1_"}], "]"}], ".", 
                                  RowBox[{"QuantumField", "[", 
                                    RowBox[{
                                    "par2___", ",", "fieldname", ",", "li2_", 
                                      ",", "sui2_"}], "]"}], ".", "qf___"}], 
                                " ", ")"}], "any_"}], ")"}], 
                          "\[RuleDelayed]", 
                          RowBox[{
                            RowBox[{"(", 
                              RowBox[{
                                RowBox[{"(", 
                                  RowBox[{"qi", ".", 
                                    RowBox[{"QuantumField", "[", 
                                      RowBox[{
                                      "par1", ",", "A", ",", "li1", ",", 
                                        "sui2"}], "]"}], ".", 
                                    RowBox[{"QuantumField", "[", 
                                      RowBox[{
                                      "par2", ",", "fieldname", ",", "li2", 
                                        ",", "sui1"}], "]"}], ".", "qf"}], 
                                  " ", ")"}], 
                                RowBox[{"(", 
                                  RowBox[{"any", "/.", 
                                    RowBox[{"{", 
                                      RowBox[{
                                        RowBox[{
                                        "sui1", "\[RuleDelayed]", "sui2"}], 
                                        ",", 
                                        RowBox[{
                                        "sui2", "\[RuleDelayed]", "sui1"}]}], 
                                      "}"}]}], ")"}]}], ")"}], "/;", 
                            RowBox[{
                              RowBox[{"(", 
                                RowBox[{"!", 
                                  RowBox[{"(", 
                                    RowBox[{"FreeQ2", "[", 
                                      RowBox[{"any", ",", 
                                        RowBox[{"{", 
                                        RowBox[{"sui1", ",", "su2"}], 
                                        "}"}]}], "]"}], ")"}]}], ")"}], 
                              "&&", 
                              RowBox[{"!", 
                                RowBox[{"(", 
                                  RowBox[{"OrderedQ", "[", 
                                    RowBox[{"{", 
                                      RowBox[{"sui1", ",", "sui2"}], "}"}], 
                                    "]"}], ")"}]}]}]}]}]}], "/.", 
                      RowBox[{"Dot", "\[Rule]", "Times"}]}], ",", 
                    "QuantumField"}], "]"}], "&"}], ",", "z", ",", "42"}], 
            "]"}], "]"}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"r2", " ", "=", " ", 
      RowBox[{"symfun", "[", 
        RowBox[{"r1", ",", "A"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"], 
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"]}], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["g",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ")"}]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"], 
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"]}], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["g",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ")"}]}], "+", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"], 
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"]}], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["g",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ")"}]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"], 
                FormBox[
                  SubscriptBox["\[PartialD]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]],
                  "TraditionalForm"]}], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"], 
                FormBox[
                  FormBox["g",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], ")"}]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"r3", "=", 
      RowBox[{"r2", "/.", 
        RowBox[{"\[Nu]", "\[Rule]", "\[Mu]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"], 
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"]}], 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["g",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          RowBox[{
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"], 
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"]}], 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["g",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{"2", " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["3", "4"], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "Inspection reveals that still terms are the same. Gather the terms with \
two ",
  Cell[BoxData[
      FormBox[
        RowBox[{"f", " "}], TraditionalForm]]],
  "'s:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"twof", "=", 
      RowBox[{"Select", "[", 
        RowBox[{"r3", ",", 
          RowBox[{
            RowBox[{
              RowBox[{"Count", "[", 
                RowBox[{"#", ",", 
                  RowBox[{"SUNF", "[", "__", "]"}]}], "]"}], "===", "2"}], 
            "&"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            FractionBox["3", "4"]}], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "4"], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"twofnew", "=", 
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            RowBox[{
            "twof", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
            "+", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"twof", "[", 
                  RowBox[{"[", "2", "]"}], "]"}], "/.", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"b", "\[RuleDelayed]", "c"}], ",", 
                    RowBox[{"c", "\[RuleDelayed]", "b"}]}], "}"}]}], ")"}]}], 
          ")"}], "/.", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"a", "\[RuleDelayed]", "c"}], ",", 
            RowBox[{"c", "\[RuleDelayed]", "a"}]}], "}"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]]}], " ", 
        SubsuperscriptBox["A", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], 
          FormBox[
            FormBox["b",
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SubsuperscriptBox["A", 
          FormBox[
            FormBox["\[Sigma]",
              "TraditionalForm"],
            "TraditionalForm"], 
          FormBox[
            FormBox["e",
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"]}]], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["g",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"r4", " ", "=", " ", 
      RowBox[{"r3", "-", "twof", "+", "twofnew"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"], 
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"]}], 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["g",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          RowBox[{
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"], 
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"]}], 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["g",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{"2", " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
Check that this is now indeed the same as the t4 result from the \
first attempt.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData["t4"], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox["F", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]}], 
            FormBox[
              FormBox["g",
                "TraditionalForm"],
              "TraditionalForm"]]}], "+", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["F", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]}], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{"g", "\[NoBreak]", "c", "\[NoBreak]", "a"}]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"w0", "=", 
      RowBox[{
        RowBox[{
          RowBox[{"RightPartialD", "[", "\[Mu]", "]"}], ".", 
          RowBox[{"FieldStrength", "[", 
            RowBox[{"\[Mu]", ",", "\[Sigma]", ",", "g", ",", 
              RowBox[{"{", 
                RowBox[{"A", ",", "a", ",", "b"}], "}"}], ",", "1"}], 
            "]"}]}], "+", 
        RowBox[{
          RowBox[{"QuantumField", "[", 
            RowBox[{"A", ",", 
              RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
              RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}], 
          RowBox[{"FieldStrength", "[", 
            RowBox[{"\[Mu]", ",", "\[Sigma]", ",", "a", ",", 
              RowBox[{"{", 
                RowBox[{"A", ",", "b", ",", "d"}], "}"}], ",", "1"}], "]"}], 
          RowBox[{"SUNF", "[", 
            RowBox[{"g", ",", "c", ",", "a"}], "]"}]}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SubsuperscriptBox["F", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}], 
            RowBox[{"g", "\[NoBreak]", 
              FormBox[
                RowBox[{"{", 
                  RowBox[{"A", ",", "a", ",", "b"}], "}"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox["1",
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SubsuperscriptBox["F", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Sigma]"}], 
            RowBox[{"a", "\[NoBreak]", 
              FormBox[
                RowBox[{"{", 
                  RowBox[{"A", ",", "b", ",", "d"}], "}"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox["1",
                "TraditionalForm"]}]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{"g", "\[NoBreak]", "c", "\[NoBreak]", "a"}]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"w1", "=", 
      RowBox[{"Explicit", "[", "w0", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["g",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["g",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                RowBox[{
                  SubsuperscriptBox["A", 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubsuperscriptBox["A", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SubscriptBox["f", 
                  RowBox[{"g", "\[NoBreak]", "a", "\[NoBreak]", "b"}]]}]}], 
            ")"}]}], "+", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox["A", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                RowBox[{
                  SubsuperscriptBox["A", 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubsuperscriptBox["A", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SubscriptBox["f", 
                  RowBox[{"a", "\[NoBreak]", "b", "\[NoBreak]", "d"}]]}]}], 
            ")"}], " ", 
          SubscriptBox["f", 
            RowBox[{"g", "\[NoBreak]", "c", "\[NoBreak]", "a"}]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"w2", "=", 
      RowBox[{
        RowBox[{"ExpandPartialD", "[", "w1", "]"}], "/.", 
        RowBox[{"Dot", "\[Rule]", "Times"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"], 
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"]}], 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["g",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          RowBox[{
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"], 
            FormBox[
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Sigma]",
                    "TraditionalForm"],
                  "TraditionalForm"]],
              "TraditionalForm"]}], 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["g",
                "TraditionalForm"],
              "TraditionalForm"]]}], "+", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["d",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"dif1", " ", "=", " ", 
      RowBox[{"w2", "-", "r4"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Sigma]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["d",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"], 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["g",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"dif2", "=", 
      RowBox[{"symfun", "[", 
        RowBox[{"dif1", ",", "A"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell["quod erat demonstrandum.", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Unset", "[", 
        RowBox[{"s", "[", "\[Phi]", "]"}], "]"}], ";", 
      RowBox[{"Unset", "[", 
        RowBox[{"S", "[", "A", "]"}], "]"}], ";", 
      RowBox[{"Clear", "[", 
        RowBox[{
        "Ag", ",", "F1", ",", "F2", ",", "t1", ",", "t2", ",", "t3", ",", 
          "t4", ",", "F", ",", "r1", ",", "r2", ",", "r3", ",", "r4", ",", 
          "symfun", ",", "twof", ",", "twofnew", ",", "w0", ",", "w1", ",", 
          "w2", ",", "dif1", ",", "dif2", ",", "dif3"}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples of funcional differentiation as used in FeynRule", 
  "Subsubsection"],

Cell["This is a part of the QCD Lagrangian.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"(", 
        RowBox[{"Gstrong", "*", 
          RowBox[{
            RowBox[{
              RowBox[{"QuantumField", "[", 
                RowBox[{"GaugeField", ",", " ", 
                  RowBox[{"LorentzIndex", "[", "li1", "]"}], ",", " ", " ", 
                  RowBox[{"SUNIndex", "[", "si2", "]"}]}], "]"}], "[", "]"}], 
            " ", ".", " ", " ", 
            RowBox[{
              RowBox[{"QuantumField", "[", 
                RowBox[{"GaugeField", ",", " ", 
                  RowBox[{"LorentzIndex", "[", "li2", "]"}], ",", " ", " ", 
                  " ", 
                  RowBox[{"SUNIndex", "[", "si4", "]"}]}], "]"}], "[", "]"}], 
            " ", ".", " ", " ", 
            RowBox[{
              RowBox[{"QuantumField", "[", 
                RowBox[{
                  RowBox[{"PartialD", "[", 
                    RowBox[{"LorentzIndex", "[", "li1", "]"}], "]"}], ",", 
                  " ", "GaugeField", ",", " ", " ", " ", 
                  RowBox[{"LorentzIndex", "[", "li2", "]"}], ",", " ", 
                  RowBox[{"SUNIndex", "[", "si1", "]"}]}], "]"}], "[", 
              "]"}]}], "*", " ", " ", 
          RowBox[{"SUNF", "[", 
            RowBox[{
              RowBox[{"SUNIndex", "[", "si1", "]"}], ",", " ", 
              RowBox[{"SUNIndex", "[", "si2", "]"}], ",", " ", 
              RowBox[{"SUNIndex", "[", "si4", "]"}]}], "]"}]}], ")"}], "/", 
      "4"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "4"], " ", 
        SubscriptBox["g", "s"], " ", 
        RowBox[{
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["li1",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["si2",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", ")"}], ".", 
          RowBox[{
            SubsuperscriptBox[
              FormBox["A",
                "TraditionalForm"], 
              FormBox[
                FormBox["li2",
                  "TraditionalForm"],
                "TraditionalForm"], 
              FormBox[
                FormBox["si4",
                  "TraditionalForm"],
                "TraditionalForm"]], "(", ")"}], ".", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SubscriptBox["\[PartialD]", 
                  FormBox[
                    FormBox["li1",
                      "TraditionalForm"],
                    "TraditionalForm"]], 
                SubsuperscriptBox[
                  FormBox["A",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["li2",
                      "TraditionalForm"],
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["si1",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], ")"}], "(", ")"}]}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["si1",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["si2",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["si4",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FunctionalD", "[", 
      RowBox[{"%", ",", " ", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{
              RowBox[{"QuantumField", "[", 
                RowBox[{"GaugeField", ",", 
                  RowBox[{"{", "\[Mu]", "}"}], ",", 
                  RowBox[{"{", "a", "}"}]}], "]"}], "[", "p", "]"}], ",", 
            " ", 
            RowBox[{
              RowBox[{"QuantumField", "[", 
                RowBox[{"GaugeField", ",", 
                  RowBox[{"{", "\[Nu]", "}"}], ",", 
                  RowBox[{"{", "b", "}"}]}], "]"}], "[", "q", "]"}], ",", 
            " ", 
            RowBox[{
              RowBox[{"QuantumField", "[", 
                RowBox[{"GaugeField", ",", 
                  RowBox[{"{", "\[Rho]", "}"}], ",", 
                  RowBox[{"{", "c", "}"}]}], "]"}], "[", "r", "]"}]}], 
          "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "4"], " ", 
        SubscriptBox["g", "s"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{"li1", "\[NoBreak]", "\[Mu]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["a",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si2",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{"li2", "\[NoBreak]", "\[Nu]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["b",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si4",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"-", "\[ImaginaryI]"}], " ", 
                  SuperscriptBox[
                    FormBox["r",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["li1",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox["g", 
                    RowBox[{"li2", "\[NoBreak]", "\[Rho]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si1",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{"li1", "\[NoBreak]", "\[Mu]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["a",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si2",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{"li2", "\[NoBreak]", "\[Rho]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si4",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"-", "\[ImaginaryI]"}], " ", 
                  SuperscriptBox[
                    FormBox["q",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["li1",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox["g", 
                    RowBox[{"li2", "\[NoBreak]", "\[Nu]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["b",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si1",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{"li1", "\[NoBreak]", "\[Nu]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["b",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si2",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{"li2", "\[NoBreak]", "\[Mu]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["a",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si4",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"-", "\[ImaginaryI]"}], " ", 
                  SuperscriptBox[
                    FormBox["r",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["li1",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox["g", 
                    RowBox[{"li2", "\[NoBreak]", "\[Rho]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si1",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{"li1", "\[NoBreak]", "\[Nu]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["b",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si2",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{"li2", "\[NoBreak]", "\[Rho]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si4",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"-", "\[ImaginaryI]"}], " ", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["li1",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox["g", 
                    RowBox[{"li2", "\[NoBreak]", "\[Mu]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["a",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si1",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{"li1", "\[NoBreak]", "\[Rho]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si2",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{"li2", "\[NoBreak]", "\[Mu]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["a",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si4",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"-", "\[ImaginaryI]"}], " ", 
                  SuperscriptBox[
                    FormBox["q",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["li1",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox["g", 
                    RowBox[{"li2", "\[NoBreak]", "\[Nu]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["b",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si1",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{"li1", "\[NoBreak]", "\[Rho]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si2",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{"li2", "\[NoBreak]", "\[Nu]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["b",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si4",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"-", "\[ImaginaryI]"}], " ", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["li1",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox["g", 
                    RowBox[{"li2", "\[NoBreak]", "\[Mu]"}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["a",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["si1",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], ")"}]}]}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["si1",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["si2",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["si4",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Calc", "[", 
        RowBox[{"%", "//", "Calc"}], "]"}], "//", "Factor"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
        SubscriptBox["g", "s"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
            RowBox[{
              SuperscriptBox[
                FormBox["r",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox["g", 
                RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
              SuperscriptBox[
                FormBox["r",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}]}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["FV", "Subsection",
  CellTags->"FV"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "FV[",
  StyleBox["p",
    FontSlant->"Italic"],
  ", ",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "] is the four-dimensional vector ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["p", "\[Mu]"], TraditionalForm]]],
  ". "
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FCE",
    ButtonData:>"FCE",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FCE"],
  ", ",
  ButtonBox["FCI",
    ButtonData:>"FCI",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FCI"],
  ", ",
  ButtonBox["FVD",
    ButtonData:>"FVD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FVD"],
  ", ",
  ButtonBox["FourVector",
    ButtonData:>"FourVector",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FourVector"],
  ", ",
  ButtonBox["Pair",
    ButtonData:>"Pair",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Pair"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FV", "[", 
      RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["p", "\[Mu]"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FV", "[", 
      RowBox[{
        RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        RowBox[{"(", 
          FormBox[
            RowBox[{"p", "-", "q"}],
            "TraditionalForm"], ")"}], "\[Mu]"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FV", "[", 
        RowBox[{"p", ",", "\[Mu]"}], "]"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"FV", "[", 
      RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FCI", "[", 
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}], "]"}], "//", "StandardForm"}]], 
  "Input"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell["\<\
There is no special function to expand momenta in FV; \
ExpandScalarProduct does the job.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandScalarProduct", "[", 
      RowBox[{"FV", "[", 
        RowBox[{
          RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], "-", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
          RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], "-", 
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
          RowBox[{"Momentum", "[", "q", "]"}]}], "]"}]}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["FVD", "Subsection",
  CellTags->"FVD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "FVD[",
  StyleBox["p",
    FontSlant->"Italic"],
  ", ",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "] is the D-dimensional vector ",
  StyleBox["p",
    FontSlant->"Italic"],
  " with Lorentz index ",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FCE",
    ButtonData:>"FCE",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FCE"],
  ", ",
  ButtonBox["FCI",
    ButtonData:>"FCI",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FCI"],
  ", ",
  ButtonBox["FV",
    ButtonData:>"FV",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FV"],
  ", ",
  ButtonBox["FourVector",
    ButtonData:>"FourVector",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FourVector"],
  ", ",
  ButtonBox["Pair",
    ButtonData:>"Pair",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Pair"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FVD", "[", 
      RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["p", "\[Mu]"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FVD", "[", 
      RowBox[{
        RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        RowBox[{"(", 
          FormBox[
            RowBox[{"p", "-", "q"}],
            "TraditionalForm"], ")"}], "\[Mu]"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FVD", "[", 
        RowBox[{"p", ",", "\[Mu]"}], "]"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"FVD", "[", 
      RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FCI", "[", 
        RowBox[{"FVD", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}], "]"}], "//", "StandardForm"}]], 
  "Input"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
        RowBox[{"Momentum", "[", 
          RowBox[{"p", ",", "D"}], "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell["There is no special function to expand momenta in FVD.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandScalarProduct", "[", 
      RowBox[{"FVD", "[", 
        RowBox[{
          RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], "-", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
          RowBox[{"Momentum", "[", 
            RowBox[{"p", ",", "D"}], "]"}]}], "]"}], "-", 
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
          RowBox[{"Momentum", "[", 
            RowBox[{"q", ",", "D"}], "]"}]}], "]"}]}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["GA", "Subsection",
  CellTags->"GA"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "GA[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "] can be used as input for a 4-dimensional ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["\[Gamma]", "\[Mu]"], " "}], TraditionalForm]]],
  "and is transformed into DiracGamma[LorentzIndex[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "]] by FeynCalcInternal (=FCI).GA[",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[Mu]", ",", "\[Nu]", ","}], TraditionalForm]]],
  " ...] is a short form for GA[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "].GA[",
  Cell[BoxData[
      FormBox["\[Nu]", TraditionalForm]]],
  "]. ... ."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracMatrix",
    ButtonData:>"DiracMatrix",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracMatrix"],
  ", ",
  ButtonBox["GAD",
    ButtonData:>"GAD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GAD"],
  ", ",
  ButtonBox["GS",
    ButtonData:>"GS",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GS"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GA", "[", "\[Mu]", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Gamma]", "\[Mu]"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"GA", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "-", 
      RowBox[{"GA", "[", 
        RowBox[{"\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
          SuperscriptBox["\[Gamma]", "\[Nu]"]}], "-", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
          SuperscriptBox["\[Gamma]", "\[Mu]"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"FCI", "[", 
        RowBox[{"GA", "[", "\[Mu]", "]"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    RowBox[{"DiracGamma", "[", 
      RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GA", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
      "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Rho]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Sigma]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"GA", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
        "]"}], "]"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"GA", "[", "\[Mu]", "]"}], ".", 
      RowBox[{"GA", "[", "\[Nu]", "]"}], ".", 
      RowBox[{"GA", "[", "\[Rho]", "]"}], ".", 
      RowBox[{"GA", "[", "\[Sigma]", "]"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"GA", "[", "\[Alpha]", "]"}], ".", 
      RowBox[{"(", 
        RowBox[{
          RowBox[{"GS", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
      RowBox[{"GA", "[", "\[Beta]", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Alpha]"], ".", 
        RowBox[{"(", 
          RowBox[{"m", "+", 
            RowBox[{
            "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}]}], 
          ")"}], ".", 
        SuperscriptBox["\[Gamma]", "\[Beta]"]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["GAD", "Subsection",
  CellTags->"GAD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "GAD[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "] can be used as input for a D-dimensional ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["\[Gamma]", "\[Mu]"], " "}], TraditionalForm]]],
  "and is transformed into DiracGamma[LorentzIndex[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  ",D],D] by FeynCalcInternal (=FCI).GAD[",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", " ", "..."}], TraditionalForm]]],
  "] is a short form for GAD[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "].GAD[",
  Cell[BoxData[
      FormBox["\[Nu]", TraditionalForm]]],
  "]. ... ."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracMatrix",
    ButtonData:>"DiracMatrix",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracMatrix"],
  ", ",
  ButtonBox["GA",
    ButtonData:>"GA",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GA"],
  ", ",
  ButtonBox["GS",
    ButtonData:>"GS",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GS"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GAD", "[", "\[Mu]", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Gamma]", "\[Mu]"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"GAD", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "-", 
      RowBox[{"GAD", "[", 
        RowBox[{"\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
          SuperscriptBox["\[Gamma]", "\[Nu]"]}], "-", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
          SuperscriptBox["\[Gamma]", "\[Mu]"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"FCI", "[", 
        RowBox[{"GAD", "[", "\[Mu]", "]"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    RowBox[{"DiracGamma", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", "D"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GAD", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
      "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Rho]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Sigma]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"GAD", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
        "]"}], "]"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"GAD", "[", "\[Mu]", "]"}], ".", 
      RowBox[{"GAD", "[", "\[Nu]", "]"}], ".", 
      RowBox[{"GAD", "[", "\[Rho]", "]"}], ".", 
      RowBox[{"GAD", "[", "\[Sigma]", "]"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"GAD", "[", "\[Alpha]", "]"}], ".", 
      RowBox[{"(", 
        RowBox[{
          RowBox[{"GSD", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
      RowBox[{"GAD", "[", "\[Beta]", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Alpha]"], ".", 
        RowBox[{"(", 
          RowBox[{"m", "+", 
            RowBox[{
            "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}]}], 
          ")"}], ".", 
        SuperscriptBox["\[Gamma]", "\[Beta]"]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["GammaEpsilon", "Subsection",
  CellTags->"GammaEpsilon"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
GammaEpsilon[exp] gives a series expansion of Gamma[exp] in Epsilon \
up to order 6 (where EulerGamma is neglected).\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["GammaExpand",
    ButtonData:>"GammaExpand",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GammaExpand"],
  ", ",
  ButtonBox["Series2",
    ButtonData:>"Series2",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Series2"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell["\<\
If the argument is of the form (1+a Epsilon) the result is not \
calculated but tabulated.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GammaEpsilon", "[", 
      RowBox[{"1", "+", 
        RowBox[{"a", " ", "Epsilon"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"C$592", " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "6"]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  FractionBox["1", "5"]}], " ", 
                TagBox[
                  RowBox[{"\[Zeta]", "(", 
                    TagBox["5",
                      (Editable -> True)], ")"}],
                  InterpretTemplate[ Zeta[ #]&]], " ", 
                SuperscriptBox["a", "5"]}], "-", 
              RowBox[{
                FractionBox["1", "36"], " ", 
                SuperscriptBox["\[Pi]", "2"], " ", 
                TagBox[
                  RowBox[{"\[Zeta]", "(", 
                    TagBox["3",
                      (Editable -> True)], ")"}],
                  InterpretTemplate[ Zeta[ #]&]], " ", 
                SuperscriptBox["a", "5"]}]}], ")"}], " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "5"]}], "+", 
        RowBox[{
          FractionBox["1", "160"], " ", 
          SuperscriptBox["a", "4"], " ", 
          SuperscriptBox["\[Pi]", "4"], " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "4"]}], "-", 
        RowBox[{
          FractionBox["1", "3"], " ", 
          SuperscriptBox["a", "3"], " ", 
          TagBox[
            RowBox[{"\[Zeta]", "(", 
              TagBox["3",
                (Editable -> True)], ")"}],
            InterpretTemplate[ Zeta[ #]&]], " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "3"]}], "+", 
        RowBox[{
          FractionBox["1", "12"], " ", 
          SuperscriptBox["a", "2"], " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "2"]}], "+", "1"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GammaEpsilon", "[", 
      RowBox[{"1", "-", 
        RowBox[{"Epsilon", "/", "2"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"C$593", " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "6"]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FractionBox[
                RowBox[{
                  SuperscriptBox["\[Pi]", "2"], " ", 
                  TagBox[
                    RowBox[{"\[Zeta]", "(", 
                      TagBox["3",
                        (Editable -> True)], ")"}],
                    InterpretTemplate[ Zeta[ #]&]]}], "1152"], "+", 
              FractionBox[
                TagBox[
                  RowBox[{"\[Zeta]", "(", 
                    TagBox["5",
                      (Editable -> True)], ")"}],
                  InterpretTemplate[ Zeta[ #]&]], "160"]}], ")"}], " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "5"]}], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox["\[Pi]", "4"], " ", 
            SuperscriptBox[
              TagBox["\[CurlyEpsilon]",
                TraditionalForm], "4"]}], "2560"], "+", 
        RowBox[{
          FractionBox["1", "24"], " ", 
          TagBox[
            RowBox[{"\[Zeta]", "(", 
              TagBox["3",
                (Editable -> True)], ")"}],
            InterpretTemplate[ Zeta[ #]&]], " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "3"]}], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox["\[Pi]", "2"], " ", 
            SuperscriptBox[
              TagBox["\[CurlyEpsilon]",
                TraditionalForm], "2"]}], "48"], "+", "1"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell["For other arguments the expansion is calculated.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GammaEpsilon", "[", "Epsilon", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"C$597", " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "6"]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                FractionBox["1", "36"], " ", 
                SuperscriptBox["\[Pi]", "2"], " ", 
                RowBox[{
                  SuperscriptBox[
                    TagBox["\[Psi]",
                      PolyGamma], 
                    RowBox[{"(", "2", ")"}]], "(", "1", ")"}]}], "-", 
              FractionBox[
                RowBox[{
                  RowBox[{
                    FractionBox["1", "36"], " ", 
                    SuperscriptBox["\[Pi]", "3"], " ", 
                    RowBox[{
                      SuperscriptBox[
                        TagBox["\[Psi]",
                          PolyGamma], 
                        RowBox[{"(", "2", ")"}]], "(", "1", ")"}]}], "+", 
                  RowBox[{
                    FractionBox["1", "120"], " ", "\[Pi]", " ", 
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{
                          RowBox[{"-", 
                            FractionBox["5", "3"]}], " ", 
                          SuperscriptBox["\[Pi]", "2"], " ", 
                          RowBox[{
                            SuperscriptBox[
                              TagBox["\[Psi]",
                                PolyGamma], 
                              RowBox[{"(", "2", ")"}]], "(", "1", ")"}]}], 
                        "+", 
                        RowBox[{"24", " ", 
                          TagBox[
                            RowBox[{"\[Zeta]", "(", 
                              TagBox["5",
                                (Editable -> True)], ")"}],
                            InterpretTemplate[ Zeta[ #]&]]}]}], ")"}]}]}], 
                "\[Pi]"]}], ")"}], " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "4"]}], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox["\[Pi]", "4"], " ", 
            SuperscriptBox[
              TagBox["\[CurlyEpsilon]",
                TraditionalForm], "3"]}], "160"], "+", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          RowBox[{
            SuperscriptBox[
              TagBox["\[Psi]",
                PolyGamma], 
              RowBox[{"(", "2", ")"}]], "(", "1", ")"}], " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "2"]}], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox["\[Pi]", "2"], " ", 
            TagBox["\[CurlyEpsilon]",
              TraditionalForm]}], "12"], "+", 
        FractionBox["1", 
          FormBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GammaEpsilon", "[", "x", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[CapitalGamma]", "(", "x", ")"}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["GammaExpand", "Subsection",
  CellTags->"GammaExpand"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
GammaExpand[exp] rewrites Gamma[n + m] in exp (where n has Head \
Integer).\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["GammaEpsilon",
    ButtonData:>"GammaEpsilon",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GammaEpsilon"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GammaExpand", "[", 
      RowBox[{"Gamma", "[", 
        RowBox[{"2", " ", "+", " ", "Epsilon"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "+", "1"}], ")"}], " ", 
        RowBox[{"\[CapitalGamma]", "(", 
          RowBox[{
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "+", "1"}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GammaExpand", "[", 
      RowBox[{"Gamma", "[", 
        RowBox[{
          RowBox[{"-", "3"}], "+", "Epsilon"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"\[CapitalGamma]", "(", 
          RowBox[{
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "+", "1"}], ")"}], 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              TagBox["\[CurlyEpsilon]",
                TraditionalForm], "-", "3"}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              TagBox["\[CurlyEpsilon]",
                TraditionalForm], "-", "2"}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              TagBox["\[CurlyEpsilon]",
                TraditionalForm], "-", "1"}], ")"}], " ", 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GammaExpand", "[", 
      RowBox[{"Gamma", "[", 
        RowBox[{"1", " ", "+", " ", "Epsilon"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[CapitalGamma]", "(", 
        RowBox[{
          TagBox["\[CurlyEpsilon]",
            TraditionalForm], "+", "1"}], ")"}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["GhostPropagator", "Subsection",
  CellTags->"GhostPropagator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "GhostPropagator[p, a, b] gives the ghost propagator where \"a\" and \"b\" \
are the color indices. GhostPropagator[p] omits the ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["\[Delta]", 
            RowBox[{"a", "\[NoBreak]", "b"}]], "."}], TraditionalForm]]]
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["GHP",
    ButtonData:>"GHP",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GHP"],
  ", ",
  ButtonBox["GluonPropagator",
    ButtonData:>"GluonPropagator",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GluonPropagator"],
  ", ",
  ButtonBox["GluonGhostVertex",
    ButtonData:>"GluonGhostVertex",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GluonGhostVertex"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GhostPropagator", "[", 
      RowBox[{"p", ",", "a", ",", "b"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"\[ImaginaryI]", " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], 
        SuperscriptBox["p", "2"]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GhostPropagator", "[", "p", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox["\[ImaginaryI]", 
        SuperscriptBox["p", "2"]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GhostPropagator", "[", 
      RowBox[{"p", ",", "1", ",", "2"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"\[ImaginaryI]", " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["ci1",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["ci2",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], 
        SuperscriptBox["p", "2"]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"GhostPropagator", "[", 
          RowBox[{
            RowBox[{"-", "k"}], ",", "3", ",", "4"}], "]"}], "//", " ", 
        "FCE"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"I", " ", 
      RowBox[{"FAD", "[", 
        RowBox[{"-", "k"}], "]"}], " ", 
      RowBox[{"SD", "[", 
        RowBox[{"ci3", ",", "ci4"}], "]"}]}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["GHP", "Subsection",
  CellTags->"GHP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "GHP[p, a, b] gives the ghost propagator where \"a\" and \"b\" are the \
color indices. GHP[p] omits the ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["\[Delta]", 
            RowBox[{"a", "\[NoBreak]", "b"}]], "."}], TraditionalForm]]]
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["GhostPropagator",
    ButtonData:>"GhostPropagator",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GhostPropagator"],
  ", ",
  ButtonBox["GluonPropagator",
    ButtonData:>"GluonPropagator",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GluonPropagator"],
  ", ",
  ButtonBox["GluonGhostVertex",
    ButtonData:>"GluonGhostVertex",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GluonGhostVertex"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GHP", "[", 
      RowBox[{"p", ",", "a", ",", "b"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"\[ImaginaryI]", " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], 
        SuperscriptBox["p", "2"]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GHP", "[", "p", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox["\[ImaginaryI]", 
        SuperscriptBox["p", "2"]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GHP", "[", 
      RowBox[{"p", ",", "1", ",", "2"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"\[ImaginaryI]", " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["ci1",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["ci2",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], 
        SuperscriptBox["p", "2"]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"FCE", "[", 
        RowBox[{"GHP", "[", 
          RowBox[{
            RowBox[{"-", "k"}], ",", "3", ",", "4"}], "]"}], "]"}], "]"}]], 
  "Input"],

Cell[BoxData[
    RowBox[{"I", " ", 
      RowBox[{"FAD", "[", 
        RowBox[{"-", "k"}], "]"}], " ", 
      RowBox[{"SD", "[", 
        RowBox[{"ci3", ",", "ci4"}], "]"}]}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["GaugeField", "Subsection",
  CellTags->"GluonField"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
GaugeField is just a name. No functional properties are associated \
with it. GaugeField is used as default setting for the option QuantumField of \
FieldStrength.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FieldStrength",
    ButtonData:>"FieldStrength",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FieldStrength"],
  ", ",
  ButtonBox["QuantumField",
    ButtonData:>"QuantumField",
    ButtonStyle->"Hyperlink",
    ButtonNote->"QuantumField"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData["GaugeField"], "Input"],

Cell[BoxData[
    FormBox["A", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox[
        FormBox["A",
          "TraditionalForm"], 
        FormBox[
          FormBox["\[Mu]",
            "TraditionalForm"],
          "TraditionalForm"], 
        FormBox[
          FormBox["a",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["GluonGhostVertex", "Subsection",
  CellTags->"GluonGhostVertex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
GluonGhostVertex[{p,mu,a}, {q,nu,b}, {k,rho,c}] or \
GluonGhostVertex[ p,mu,a , q,nu,b , k,rho,c ] yields the Gluon-Ghost vertex. \
The first argument represents the gluon and the third argument the outgoing \
ghost field (but incoming four-momentum). The dimension and the name of the \
coupling constant are determined by the options Dimension and \
CouplingConstant.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  ButtonBox["GhostPropagator",
    ButtonData:>"GhostPropagator",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GhostPropagator"],
  ", ",
  ButtonBox["GluonVertex",
    ButtonData:>"GluonVertex",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GluonVertex"],
  "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "GluonGhostVertex", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonGhostVertex", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"k", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        SuperscriptBox[
          FormBox["k",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Gstrong", " ", 
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
          RowBox[{"Momentum", "[", 
            RowBox[{"k", ",", "D"}], "]"}]}], "]"}], " ", 
      RowBox[{"SUNF", "[", 
        RowBox[{
          RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
          RowBox[{"SUNIndex", "[", "b", "]"}], ",", 
          RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonGhostVertex", "[", 
      RowBox[{"p", ",", "1", ",", "q", ",", "2", ",", "k", ",", "3"}], 
      "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        SuperscriptBox[
          FormBox["k",
            "TraditionalForm"], 
          FormBox[
            FormBox["li1",
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["ci1",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["ci2",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["ci3",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["GluonPropagator", "Subsection",
  CellTags->"GluonPropagator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "GluonPropagator[p, {mu, a}, {nu, b}] or GluonPropagator[p, mu, a , nu, b ] \
yields the gluon propagator. GluonPropagator[p, {mu}, {nu}] or \
GluonPropagator[p, mu, nu] omits the SUNDelta. The gauge and the dimension is \
determined by the option Gauge and Dimension. The following settings of Gauge \
are possible: 1 for the Feynman gauge; ",
  Cell[BoxData[
      FormBox["\[Alpha]", TraditionalForm]]],
  " for the general covariant gauge; {Momentum[n] ,1} for the axial gauge."
}], "Text"],

Cell["GP can be used as an abbreviation of GluonPropagator.", "Text"],

Cell[CellGroupData[{

Cell[BoxData["GP"], "Input"],

Cell[BoxData[
    FormBox["GluonPropagator", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "GluonPropagator", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CounterTerm", "\[Rule]", "False"}], ",", 
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Gauge", "\[Rule]", "1"}], ",", 
          RowBox[{"\[CapitalKoppa]", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["GluonVertex",
    ButtonData:>"GluonVertex",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GluonVertex"],
  ", ",
  ButtonBox["GhostPropagator",
    ButtonData:>"GhostPropagator",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GhostPropagator"],
  ", ",
  ButtonBox["GluonGhostVertex",
    ButtonData:>"GluonGhostVertex",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GluonGhostVertex"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonPropagator", "[", 
      RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b"}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          RowBox[{"\[ImaginaryI]", " ", 
            SuperscriptBox["g", 
              RowBox[{
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"]}]]}], 
          SuperscriptBox["p", "2"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonPropagator", "[", 
      RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
        RowBox[{"Gauge", "\[Rule]", "\[Alpha]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"(", 
            RowBox[{
              FractionBox[
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "\[Alpha]"}], ")"}], " ", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], 
                SuperscriptBox["p", "2"]], "-", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], ")"}], " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], 
        SuperscriptBox["p", "2"]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonPropagator", "[", 
      RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
        RowBox[{"Gauge", "\[Rule]", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"Momentum", "[", "n", "]"}], ",", "1"}], "}"}]}]}], 
      "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"-", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["n",
                    "TraditionalForm"], "2"]}], "+", 
              FractionBox[
                RowBox[{
                  SuperscriptBox[
                    FormBox["n",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox[
                    FormBox["n",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"]}], 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox["n",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"]], "+", 
              FractionBox[
                RowBox[{
                  RowBox[{
                    SuperscriptBox[
                      FormBox["n",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "+", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["n",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}]}], 
                RowBox[{
                  FormBox["n",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]]}], ")"}], " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], 
        SuperscriptBox["p", "2"]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonPropagator", "[", 
      RowBox[{"p", ",", "\[Mu]", ",", "\[Nu]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          RowBox[{"\[ImaginaryI]", " ", 
            SuperscriptBox["g", 
              RowBox[{
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]}]]}], 
          SuperscriptBox["p", "2"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
This is a convenient way to enter amplitudes by hand (GP is an \
abbreviation GluonPropagator).\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GP", "[", 
      RowBox[{"p", ",", "1", ",", "2"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          RowBox[{"\[ImaginaryI]", " ", 
            SuperscriptBox["g", 
              RowBox[{
                FormBox[
                  FormBox["li1",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["li2",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["ci1",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["ci2",
                    "TraditionalForm"],
                  "TraditionalForm"]}]]}], 
          SuperscriptBox["p", "2"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"GP", "[", 
          RowBox[{
            RowBox[{"-", "k"}], ",", "3", ",", "4"}], "]"}], "//", "FCE"}], 
      "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"-", "I"}], " ", 
      RowBox[{"FAD", "[", 
        RowBox[{"-", "k"}], "]"}], " ", 
      RowBox[{"MTD", "[", 
        RowBox[{"li3", ",", "li4"}], "]"}], " ", 
      RowBox[{"SD", "[", 
        RowBox[{"ci3", ",", "ci4"}], "]"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonPropagator", "[", 
      RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
        RowBox[{"CounterTerm", "\[Rule]", " ", "1"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          RowBox[{"\[ImaginaryI]", " ", 
            SubscriptBox["C", "A"], " ", 
            SubsuperscriptBox["g", "s", "2"], " ", 
            SubscriptBox["S", "n"], " ", 
            RowBox[{"(", 
              RowBox[{
                FractionBox[
                  RowBox[{"11", " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "3"], "-", 
                RowBox[{
                  FractionBox["19", "6"], " ", 
                  SuperscriptBox["g", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"]}]}], ")"}], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"]}]]}], 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonPropagator", "[", 
      RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
        RowBox[{"CounterTerm", "\[Rule]", " ", "2"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          RowBox[{"\[ImaginaryI]", " ", 
            SubscriptBox["C", "A"], " ", 
            SubsuperscriptBox["g", "s", "2"], " ", 
            SubscriptBox["S", "n"], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"-", 
                    FractionBox["1", "3"]}], " ", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], "-", 
                RowBox[{
                  FractionBox["1", "6"], " ", 
                  SuperscriptBox["g", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"]}]}], ")"}], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"]}]]}], 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonPropagator", "[", 
      RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
        RowBox[{"CounterTerm", "\[Rule]", " ", "3"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          RowBox[{"2", " ", "\[ImaginaryI]", " ", 
            SubsuperscriptBox["g", "s", "2"], " ", 
            SubscriptBox["S", "n"], " ", 
            SubscriptBox["T", "f"], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FractionBox["4", "3"], " ", 
                  SuperscriptBox["g", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"]}], "-", 
                FractionBox[
                  RowBox[{"4", " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "3"]}], ")"}], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"]}]]}], 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonPropagator", "[", 
      RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
        RowBox[{"CounterTerm", "\[Rule]", " ", "4"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          RowBox[{"\[ImaginaryI]", " ", 
            SubscriptBox["C", "A"], " ", 
            SubsuperscriptBox["g", "s", "2"], " ", 
            SubscriptBox["S", "n"], " ", 
            RowBox[{"(", 
              RowBox[{
                FractionBox[
                  RowBox[{"10", " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "3"], "-", 
                RowBox[{
                  FractionBox["10", "3"], " ", 
                  SuperscriptBox["g", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"]}]}], ")"}], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"]}]]}], 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonPropagator", "[", 
      RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
        RowBox[{"CounterTerm", "\[Rule]", " ", "5"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{"\[ImaginaryI]", " ", 
            SubscriptBox["C", "A"], " ", 
            SubscriptBox["S", "n"], " ", 
            RowBox[{"(", 
              RowBox[{
                FractionBox[
                  RowBox[{"10", " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "3"], "-", 
                RowBox[{
                  FractionBox["10", "3"], " ", 
                  SuperscriptBox["g", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"]}]}], ")"}], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            SubsuperscriptBox["g", "s", "2"]}], 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm]], "+", 
        FractionBox[
          RowBox[{"\[ImaginaryI]", " ", 
            SubscriptBox["S", "n"], " ", 
            SubscriptBox["T", "f"], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FractionBox["4", "3"], " ", 
                  SuperscriptBox["g", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"]}], "-", 
                FractionBox[
                  RowBox[{"4", " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "3"]}], ")"}], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            SubsuperscriptBox["g", "s", "2"]}], 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm]]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["GluonVertex", "Subsection",
  CellTags->"GluonVertex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["Note: All momenta are flowing into the vertex.", "Text"],

Cell["\<\
GluonVertex[{p,mu,a}, {q,nu,b}, {k,la,c}] or GluonVertex[p,mu,a , \
q,nu,b , k,la,c ] yields the 3-gluon vertex. GluonVertex[{p,mu}, {q,nu}, \
{k,la}] yields the 3-gluon vertex without color structure and the coupling \
constant. GluonVertex[{p,mu,a}, {q,nu,b}, {k,la,c}, {s,si,d}] or \
GluonVertex[{mu,a}, {nu,b}, {la,c}, {si,d}] or GluonVertex[p,mu,a , q,nu,b , \
k,la,c , s,si,d] or GluonVertex[ mu,a , nu,b , la,c , si,d ] yields the \
4-gluon vertex. The dimension and the name of the coupling constant are \
determined by the options Dimension and CouplingConstant.\
\>", "Text"],

Cell["GV can be used as an abbreviation of GluonVertex.", "Text"],

Cell[CellGroupData[{

Cell[BoxData["GV"], "Input"],

Cell[BoxData[
    FormBox["GluonVertex", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "GluonVertex", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
          RowBox[{"\[CapitalKoppa]", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["GluonGhostVertex",
    ButtonData:>"GluonGhostVertex",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GluonGhostVertex"],
  ", ",
  ButtonBox["GluonPropagator",
    ButtonData:>"GluonPropagator",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GluonPropagator"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonVertex", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        RowBox[{
          SuperscriptBox["V", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]}]], "(", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", 
            RowBox[{",", " "}], "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"], "\[NoBreak]", 
            RowBox[{",", " "}], "\[NoBreak]", 
            FormBox["r",
              "TraditionalForm"]}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"GluonVertex", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}], "//", 
      "Explicit"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "-", 
            RowBox[{
              SuperscriptBox[
                FormBox["r",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                FormBox["r",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}]}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonVertex", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["V", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]", "\[NoBreak]", "\[Rho]"}]], 
        "(", 
        RowBox[{"p", "\[NoBreak]", 
          RowBox[{",", " "}], "\[NoBreak]", "q", "\[NoBreak]", 
          RowBox[{",", " "}], "\[NoBreak]", "r"}], ")"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"GluonVertex", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"r", ",", "\[Rho]"}], "}"}]}], "]"}], "//", 
      "Explicit"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox[
            FormBox["q",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          SuperscriptBox[
            FormBox["r",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]]}], "+", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SuperscriptBox[
            FormBox["r",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]]}], "+", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SuperscriptBox[
            FormBox["q",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonVertex", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u9",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u9",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u9",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u9",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u9",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u9",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}]}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonVertex", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"\[Rho]", ",", "c"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"\[Sigma]", ",", "d"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u10",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u10",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u10",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u10",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u10",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u10",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}]}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GluonVertex", "[", 
      RowBox[{
      "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", "\[Rho]", ",", "c", 
        ",", "\[Sigma]", ",", "d"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u11",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u11",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u11",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u11",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Sigma]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u11",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u11",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}]}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell["\<\
A very convenient way to enter diagrams by hand is to label each \
line hitting a vertex by a number and put this number after the inflowing \
momentum.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GV", "[", 
      RowBox[{"p", ",", "1", ",", "q", ",", "2", ",", "r", ",", "3"}], 
      "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        RowBox[{
          SuperscriptBox["V", 
            RowBox[{
              FormBox[
                FormBox["li1",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["li2",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["li3",
                  "TraditionalForm"],
                "TraditionalForm"]}]], "(", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", 
            RowBox[{",", " "}], "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"], "\[NoBreak]", 
            RowBox[{",", " "}], "\[NoBreak]", 
            FormBox["r",
              "TraditionalForm"]}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["ci1",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["ci2",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["ci3",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GV", "[", 
      RowBox[{
      "p", ",", "1", ",", "q", ",", "2", ",", "r", ",", "3", ",", "s", ",", 
        "4"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["li1",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["li2",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["li3",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["li4",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["li1",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["li3",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["li2",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["li4",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["ci1",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["ci4",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u12",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["ci2",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["ci3",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u12",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["li1",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["li2",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["li3",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["li4",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["li1",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["li4",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["li2",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["li3",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["ci1",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["ci3",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u12",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["ci2",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["ci4",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u12",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["li1",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["li3",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["li2",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["li4",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}], "-", 
                  RowBox[{
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["li1",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["li4",
                            "TraditionalForm"],
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox["g", 
                      RowBox[{
                        FormBox[
                          FormBox["li2",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["li3",
                            "TraditionalForm"],
                          "TraditionalForm"]}]]}]}], ")"}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["ci1",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["ci2",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u12",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["ci3",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["ci4",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["u12",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}]}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["GSD", "Subsection",
  CellTags->"GSD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "GSD[p] can be used as input for a D-dimensional ",
  Cell[BoxData[
      FormBox[
        RowBox[{
        "p", " ", "\[NegativeMediumSpace]", "\[NegativeMediumSpace]", 
          "\[NegativeMediumSpace]", "\[NegativeMediumSpace]", "/", " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"=", 
                RowBox[{"\[Gamma]", ".", "p"}]}], " ", "=", " ", 
              RowBox[{
                SubscriptBox["\[Gamma]", "\[Mu]"], 
                SuperscriptBox["p", "\[Mu]"]}]}], ")"}]}], 
        TraditionalForm]]],
  " and is transformed into DiracGamma[Momentum[p,D],D] by FeynCalcInternal \
(=FCI). GSD[p,q, ...] is a short form for GSD[p].GSD[q]. ... ."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracGamma",
    ButtonData:>"DiracGamma",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracGamma"],
  ", ",
  ButtonBox["DiracSlash",
    ButtonData:>"DiracSlash",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracSlash"],
  ", ",
  ButtonBox["GA",
    ButtonData:>"GA",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GA"],
  ", ",
  ButtonBox["GAD",
    ButtonData:>"GAD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GAD"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GSD", "[", "p", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"GSD", "[", "p", "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"DiracGamma", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", 
          RowBox[{"p", ",", "D"}], "]"}], ",", "D"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GSD", "[", 
      RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "q"}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "r"}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "s"}], 
          ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"GSD", "[", 
        RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"GSD", "[", "p", "]"}], ".", 
      RowBox[{"GSD", "[", "q", "]"}], ".", 
      RowBox[{"GSD", "[", "r", "]"}], ".", 
      RowBox[{"GSD", "[", "s", "]"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"GSD", "[", "q", "]"}], ".", 
      RowBox[{"(", 
        RowBox[{
          RowBox[{"GSD", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
      RowBox[{"GSD", "[", "q", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "q"}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{"m", "+", 
            RowBox[{
            "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}]}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "q"}], 
          ")"}]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["GS", "Subsection",
  CellTags->"GS"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "GS[p] can be used as input for a 4-dimensional ",
  Cell[BoxData[
      FormBox[
        RowBox[{
        "p", " ", "\[NegativeMediumSpace]", "\[NegativeMediumSpace]", 
          "\[NegativeMediumSpace]", "\[NegativeMediumSpace]", "/", " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"=", 
                RowBox[{"\[Gamma]", ".", "p"}]}], " ", "=", " ", 
              RowBox[{
                SubscriptBox["\[Gamma]", "\[Mu]"], 
                SuperscriptBox["p", "\[Mu]"]}]}], ")"}]}], 
        TraditionalForm]]],
  " and is transformed into DiracGamma[Momentum[p]] by FeynCalcInternal \
(=FCI). GS[p,q, ...] is a short form for GS[p].GS[q]. ... ."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracGamma",
    ButtonData:>"DiracGamma",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracGamma"],
  ", ",
  ButtonBox["DiracSlash",
    ButtonData:>"DiracSlash",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracSlash"],
  ", ",
  ButtonBox["GA",
    ButtonData:>"GA",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GA"],
  ", ",
  ButtonBox["GAD",
    ButtonData:>"GAD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GAD"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GS", "[", "p", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"GS", "[", "p", "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"DiracGamma", "[", 
      RowBox[{"Momentum", "[", "p", "]"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GS", "[", 
      RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "q"}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "r"}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "s"}], 
          ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"GS", "[", 
        RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"GS", "[", "p", "]"}], ".", 
      RowBox[{"GS", "[", "q", "]"}], ".", 
      RowBox[{"GS", "[", "r", "]"}], ".", 
      RowBox[{"GS", "[", "s", "]"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"GS", "[", "q", "]"}], ".", 
      RowBox[{"(", 
        RowBox[{
          RowBox[{"GS", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
      RowBox[{"GS", "[", "q", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "q"}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{"m", "+", 
            RowBox[{
            "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "p"}]}], 
          ")"}], ".", 
        RowBox[{"(", 
          RowBox[{
          "\[Gamma]", "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", "q"}], 
          ")"}]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Gstrong", "Subsection",
  CellTags->"Gstrong"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["Gstrong denotes the strong coupling constant.", "Text"],

Cell[TextData[{
  "See also: ",
  ButtonBox["CovariantD",
    ButtonData:>"CovariantD",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["FieldStrength",
    ButtonData:>"FieldStrength",
    ButtonStyle->"Hyperlink"],
  ", ",
  ButtonBox["GluonVertex",
    ButtonData:>"GluonVertex",
    ButtonStyle->"Hyperlink"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell["\<\
Gstrong has no functional properties. Only a typesetting rule is \
defined.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData["Gstrong"], "Input"],

Cell[BoxData[
    FormBox[
      SubscriptBox["g", "s"], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Hill", "Subsection",
  CellTags->"Hill"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Hill[x, y] gives the Hill identity with arguments x and y. The \
returned object is 0.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["SimplifyPolyLog",
    ButtonData:>"SimplifyPolyLog",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SimplifyPolyLog"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Hill", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"log", "(", "a", ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"log", "(", 
                RowBox[{"1", "-", "a"}], ")"}], "-", 
              RowBox[{"log", "(", 
                RowBox[{"1", "-", "b"}], ")"}]}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"log", "(", 
            FractionBox[
              RowBox[{"1", "-", "a"}], 
              RowBox[{"1", "-", "b"}]], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"-", 
                RowBox[{"log", "(", "a", ")"}]}], "+", 
              RowBox[{"log", "(", 
                RowBox[{"1", "-", "b"}], ")"}], "-", 
              RowBox[{"log", "(", 
                FractionBox[
                  RowBox[{"a", "-", "b"}], "a"], ")"}], "+", 
              RowBox[{"log", "(", 
                FractionBox[
                  RowBox[{"a", "-", "b"}], 
                  RowBox[{"1", "-", "b"}]], ")"}]}], ")"}]}], "-", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{"log", "(", 
                RowBox[{"1", "-", "b"}], ")"}], "-", 
              RowBox[{"log", "(", 
                FractionBox[
                  RowBox[{"a", "-", "b"}], "a"], ")"}], "+", 
              RowBox[{"log", "(", 
                FractionBox[
                  RowBox[{"a", "-", "b"}], 
                  RowBox[{"a", " ", 
                    RowBox[{"(", 
                      RowBox[{"1", "-", "b"}], ")"}]}]], ")"}]}], ")"}], 
          " ", 
          RowBox[{"log", "(", 
            FractionBox[
              RowBox[{
                RowBox[{"(", 
                  RowBox[{"1", "-", "a"}], ")"}], " ", "b"}], 
              RowBox[{"a", " ", 
                RowBox[{"(", 
                  RowBox[{"1", "-", "b"}], ")"}]}]], ")"}]}], "+", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", "a", ")"}], "+", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          FractionBox[
            RowBox[{"1", "-", "a"}], 
            RowBox[{"1", "-", "b"}]], ")"}], "-", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", "b", ")"}], "+", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          FractionBox["b", "a"], ")"}], "-", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          FractionBox[
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "a"}], ")"}], " ", "b"}], 
            RowBox[{"a", " ", 
              RowBox[{"(", 
                RowBox[{"1", "-", "b"}], ")"}]}]], ")"}], "-", 
        FractionBox[
          SuperscriptBox["\[Pi]", "2"], "6"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"%", " ", "/.", " ", 
          RowBox[{"a", ":>", " ", ".123"}]}], " ", "/.", " ", 
        RowBox[{"b", ":>", " ", ".656"}]}], " ", "//", " ", "Chop"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"Hill", "[", 
            RowBox[{"x", ",", 
              RowBox[{"x", " ", "y"}]}], "]"}], "//", "PowerExpand"}], "//", 
        "SimplifyPolyLog"}], "//", "Expand"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"\[Zeta]", "(", "2", ")"}], "-", 
        RowBox[{
          RowBox[{"log", "(", 
            RowBox[{"1", "-", "y"}], ")"}], " ", 
          RowBox[{"log", "(", "y", ")"}]}], "-", 
        RowBox[{
          RowBox[{"log", "(", "x", ")"}], " ", 
          RowBox[{"log", "(", 
            RowBox[{"1", "-", 
              RowBox[{"x", " ", "y"}]}], ")"}]}], "-", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          RowBox[{"1", "-", "x"}], ")"}], "-", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          RowBox[{"1", "-", "y"}], ")"}], "-", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          RowBox[{"x", " ", "y"}], ")"}], "+", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          FractionBox[
            RowBox[{"1", "-", "x"}], 
            RowBox[{"1", "-", 
              RowBox[{"x", " ", "y"}]}]], ")"}], "-", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          FractionBox[
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], " ", "y"}], 
            RowBox[{"1", "-", 
              RowBox[{"x", " ", "y"}]}]], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"%", " ", "/.", " ", 
            RowBox[{"x", ":>", " ", ".34"}]}], "/.", " ", 
          RowBox[{"y", "->", " ", ".6"}]}], "//", "N"}], "//", "Chop"}]], 
  "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["HypergeometricAC", "Subsection",
  CellTags->"HypergeometricAC"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
HypergeometricAC[n][exp] analytically continues Hypergeometric2F1 \
functions in exp. The second argument n refers to the equation number (n) in \
chapter 2.10 of \"Higher Transcendental Functions\" by Ergelyi, Magnus, \
Oberhettinger, Tricomi. In case of eq. (6) (p.109) the last line is returned \
for HypergeometricAC[6][exp], while the first equality is given by \
HypergeometricAC[61][exp]. ((2.10.1) is identical to eq. (9.5.7) of \"Special \
Functions & their Applications\" by N.N.Lebedev).\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "HypergeometricAC", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Collect2", "\[Rule]", "True"}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["HypExplicit",
    ButtonData:>"HypExplicit",
    ButtonStyle->"Hyperlink",
    ButtonNote->"HypExplicit"],
  ", ",
  ButtonBox["HypergeometricIR",
    ButtonData:>"HypergeometricIR",
    ButtonStyle->"Hyperlink",
    ButtonNote->"HypergeometricIR"],
  ", ",
  ButtonBox["HypergeometricSE",
    ButtonData:>"HypergeometricSE",
    ButtonStyle->"Hyperlink",
    ButtonNote->"HypergeometricSE"],
  ", ",
  ButtonBox["ToHypergeometric",
    ButtonData:>"ToHypergeometric",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ToHypergeometric"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell["These are all transformation rules currently built in.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"HypergeometricAC", "[", "1", "]"}], "[", 
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], 
        "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"\[Alpha]", "+", "\[Beta]", "-", "\[Gamma]"}], ")"}], 
            " ", 
            RowBox[{"\[CapitalGamma]", "(", "\[Gamma]", ")"}], " ", 
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[ThinSpace]", "2"], 
                  SubscriptBox["F", "1"]}], "(", 
                RowBox[{
                  RowBox[{
                    TagBox[
                      RowBox[{"\[Gamma]", "-", "\[Alpha]"}],
                      (Editable -> True)], ",", 
                    TagBox[
                      RowBox[{"\[Gamma]", "-", "\[Beta]"}],
                      (Editable -> True)]}], ";", 
                  TagBox[
                    RowBox[{
                      RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", 
                      "\[Gamma]", "+", "1"}],
                    (Editable -> True)], ";", 
                  TagBox[
                    RowBox[{"1", "-", "z"}],
                    (Editable -> True)]}], ")"}],
              InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"1", "-", "z"}], ")"}], 
              RowBox[{
                RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", 
                "\[Gamma]"}]]}], 
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", "\[Alpha]", ")"}], " ", 
            RowBox[{"\[CapitalGamma]", "(", "\[Beta]", ")"}]}]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", "\[Gamma]", ")"}], " ", 
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{
                RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", "\[Gamma]"}], 
              ")"}], " ", 
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[ThinSpace]", "2"], 
                  SubscriptBox["F", "1"]}], "(", 
                RowBox[{
                  RowBox[{
                    TagBox["\[Alpha]",
                      (Editable -> True)], ",", 
                    TagBox["\[Beta]",
                      (Editable -> True)]}], ";", 
                  TagBox[
                    RowBox[{
                    "\[Alpha]", "+", "\[Beta]", "-", "\[Gamma]", "+", "1"}],
                    (Editable -> True)], ";", 
                  TagBox[
                    RowBox[{"1", "-", "z"}],
                    (Editable -> True)]}], ")"}],
              InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}], 
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"\[Gamma]", "-", "\[Alpha]"}], ")"}], " ", 
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"\[Gamma]", "-", "\[Beta]"}], ")"}]}]]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"HypergeometricAC", "[", "2", "]"}], "[", 
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], 
        "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"\[Beta]", "-", "\[Alpha]"}], ")"}], " ", 
            RowBox[{"\[CapitalGamma]", "(", "\[Gamma]", ")"}], " ", 
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[ThinSpace]", "2"], 
                  SubscriptBox["F", "1"]}], "(", 
                RowBox[{
                  RowBox[{
                    TagBox["\[Alpha]",
                      (Editable -> True)], ",", 
                    TagBox[
                      RowBox[{"\[Alpha]", "-", "\[Gamma]", "+", "1"}],
                      (Editable -> True)]}], ";", 
                  TagBox[
                    RowBox[{"\[Alpha]", "-", "\[Beta]", "+", "1"}],
                    (Editable -> True)], ";", 
                  TagBox[
                    FractionBox["1", "z"],
                    (Editable -> True)]}], ")"}],
              InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "z"}], ")"}], 
              RowBox[{"-", "\[Alpha]"}]]}], 
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", "\[Beta]", ")"}], " ", 
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"\[Gamma]", "-", "\[Alpha]"}], ")"}]}]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"\[Alpha]", "-", "\[Beta]"}], ")"}], " ", 
            RowBox[{"\[CapitalGamma]", "(", "\[Gamma]", ")"}], " ", 
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[ThinSpace]", "2"], 
                  SubscriptBox["F", "1"]}], "(", 
                RowBox[{
                  RowBox[{
                    TagBox["\[Beta]",
                      (Editable -> True)], ",", 
                    TagBox[
                      RowBox[{"\[Beta]", "-", "\[Gamma]", "+", "1"}],
                      (Editable -> True)]}], ";", 
                  TagBox[
                    RowBox[{
                      RowBox[{"-", "\[Alpha]"}], "+", "\[Beta]", "+", "1"}],
                    (Editable -> True)], ";", 
                  TagBox[
                    FractionBox["1", "z"],
                    (Editable -> True)]}], ")"}],
              InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "z"}], ")"}], 
              RowBox[{"-", "\[Beta]"}]]}], 
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", "\[Alpha]", ")"}], " ", 
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"\[Gamma]", "-", "\[Beta]"}], ")"}]}]]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"HypergeometricAC", "[", "3", "]"}], "[", 
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], 
        "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"\[Beta]", "-", "\[Alpha]"}], ")"}], " ", 
            RowBox[{"\[CapitalGamma]", "(", "\[Gamma]", ")"}], " ", 
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[ThinSpace]", "2"], 
                  SubscriptBox["F", "1"]}], "(", 
                RowBox[{
                  RowBox[{
                    TagBox["\[Alpha]",
                      (Editable -> True)], ",", 
                    TagBox[
                      RowBox[{"\[Gamma]", "-", "\[Beta]"}],
                      (Editable -> True)]}], ";", 
                  TagBox[
                    RowBox[{"\[Alpha]", "-", "\[Beta]", "+", "1"}],
                    (Editable -> True)], ";", 
                  TagBox[
                    FractionBox["1", 
                      RowBox[{"1", "-", "z"}]],
                    (Editable -> True)]}], ")"}],
              InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"1", "-", "z"}], ")"}], 
              RowBox[{"-", "\[Alpha]"}]]}], 
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", "\[Beta]", ")"}], " ", 
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"\[Gamma]", "-", "\[Alpha]"}], ")"}]}]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"\[Alpha]", "-", "\[Beta]"}], ")"}], " ", 
            RowBox[{"\[CapitalGamma]", "(", "\[Gamma]", ")"}], " ", 
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[ThinSpace]", "2"], 
                  SubscriptBox["F", "1"]}], "(", 
                RowBox[{
                  RowBox[{
                    TagBox["\[Beta]",
                      (Editable -> True)], ",", 
                    TagBox[
                      RowBox[{"\[Gamma]", "-", "\[Alpha]"}],
                      (Editable -> True)]}], ";", 
                  TagBox[
                    RowBox[{
                      RowBox[{"-", "\[Alpha]"}], "+", "\[Beta]", "+", "1"}],
                    (Editable -> True)], ";", 
                  TagBox[
                    FractionBox["1", 
                      RowBox[{"1", "-", "z"}]],
                    (Editable -> True)]}], ")"}],
              InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"1", "-", "z"}], ")"}], 
              RowBox[{"-", "\[Beta]"}]]}], 
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", "\[Alpha]", ")"}], " ", 
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"\[Gamma]", "-", "\[Beta]"}], ")"}]}]]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"HypergeometricAC", "[", "4", "]"}], "[", 
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], 
        "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", "\[Gamma]", ")"}], " ", 
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{
                RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", "\[Gamma]"}], 
              ")"}], " ", 
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[ThinSpace]", "2"], 
                  SubscriptBox["F", "1"]}], "(", 
                RowBox[{
                  RowBox[{
                    TagBox["\[Alpha]",
                      (Editable -> True)], ",", 
                    TagBox[
                      RowBox[{"\[Alpha]", "-", "\[Gamma]", "+", "1"}],
                      (Editable -> True)]}], ";", 
                  TagBox[
                    RowBox[{
                    "\[Alpha]", "+", "\[Beta]", "-", "\[Gamma]", "+", "1"}],
                    (Editable -> True)], ";", 
                  TagBox[
                    RowBox[{"-", 
                      FractionBox[
                        RowBox[{"1", "-", "z"}], "z"]}],
                    (Editable -> True)]}], ")"}],
              InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], " ", 
            SuperscriptBox["z", 
              RowBox[{"-", "\[Alpha]"}]]}], 
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"\[Gamma]", "-", "\[Alpha]"}], ")"}], " ", 
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"\[Gamma]", "-", "\[Beta]"}], ")"}]}]], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"1", "-", "z"}], ")"}], 
              RowBox[{
                RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", 
                "\[Gamma]"}]], " ", 
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"\[Alpha]", "+", "\[Beta]", "-", "\[Gamma]"}], ")"}], 
            " ", 
            RowBox[{"\[CapitalGamma]", "(", "\[Gamma]", ")"}], " ", 
            TagBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["\[ThinSpace]", "2"], 
                  SubscriptBox["F", "1"]}], "(", 
                RowBox[{
                  RowBox[{
                    TagBox[
                      RowBox[{"\[Gamma]", "-", "\[Alpha]"}],
                      (Editable -> True)], ",", 
                    TagBox[
                      RowBox[{"1", "-", "\[Alpha]"}],
                      (Editable -> True)]}], ";", 
                  TagBox[
                    RowBox[{
                      RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", 
                      "\[Gamma]", "+", "1"}],
                    (Editable -> True)], ";", 
                  TagBox[
                    RowBox[{"-", 
                      FractionBox[
                        RowBox[{"1", "-", "z"}], "z"]}],
                    (Editable -> True)]}], ")"}],
              InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], " ", 
            SuperscriptBox["z", 
              RowBox[{"\[Alpha]", "-", "\[Gamma]"}]]}], 
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", "\[Alpha]", ")"}], " ", 
            RowBox[{"\[CapitalGamma]", "(", "\[Beta]", ")"}]}]]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"HypergeometricAC", "[", "6", "]"}], "[", 
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], 
        "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"1", "-", "z"}], ")"}], 
          RowBox[{"-", "\[Beta]"}]], " ", 
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["\[Beta]",
                  (Editable -> True)], ",", 
                TagBox[
                  RowBox[{"\[Gamma]", "-", "\[Alpha]"}],
                  (Editable -> True)]}], ";", 
              TagBox["\[Gamma]",
                (Editable -> True)], ";", 
              TagBox[
                RowBox[{"-", 
                  FractionBox["z", 
                    RowBox[{"1", "-", "z"}]]}],
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"HypergeometricAC", "[", "61", "]"}], "[", 
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], 
        "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"1", "-", "z"}], ")"}], 
          RowBox[{"-", "\[Alpha]"}]], " ", 
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["\[Alpha]",
                  (Editable -> True)], ",", 
                TagBox[
                  RowBox[{"\[Gamma]", "-", "\[Beta]"}],
                  (Editable -> True)]}], ";", 
              TagBox["\[Gamma]",
                (Editable -> True)], ";", 
              TagBox[
                RowBox[{"-", 
                  FractionBox["z", 
                    RowBox[{"1", "-", "z"}]]}],
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}], 
      TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["HypergeometricIR", "Subsection",
  CellTags->"HypergeometricIR"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[
"HypergeometricIR[exp, t] substitutes for all Hypergeometric2F1[a,b,c,z] in \
exp by its Euler integral reprentation. The factor Integratedx[t, 0, 1] can \
be omitted by setting the option Integratedx \[Rule] False."], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["HypergeometricAC",
    ButtonData:>"HypergeometricAC",
    ButtonStyle->"Hyperlink",
    ButtonNote->"HypergeometricAC"],
  ", ",
  ButtonBox["HypergeometricSE",
    ButtonData:>"HypergeometricSE",
    ButtonStyle->"Hyperlink",
    ButtonNote->"HypergeometricSE"],
  ", ",
  ButtonBox["ToHypergeometric",
    ButtonData:>"ToHypergeometric",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ToHypergeometric"],
  "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "HypergeometricIR", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Integratedx", "\[Rule]", "False"}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"HypergeometricIR", "[", 
      RowBox[{
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], ",", "t"}], 
      "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "t"}], ")"}], 
            RowBox[{
              RowBox[{"-", "b"}], "+", "c", "-", "1"}]], " ", 
          SuperscriptBox["t", 
            RowBox[{"b", "-", "1"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", 
                RowBox[{"t", " ", "z"}]}], ")"}], 
            RowBox[{"-", "a"}]], " ", 
          RowBox[{"\[CapitalGamma]", "(", "c", ")"}]}], 
        RowBox[{
          RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"c", "-", "b"}], ")"}]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ToHypergeometric", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"t", "^", "b"}], " ", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"1", "-", "t"}], ")"}], "^", "c"}], " ", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"1", "+", 
                RowBox[{"t", " ", "z"}]}], ")"}], "^", "a"}]}], ",", "t"}], 
      "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"b", "+", "1"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"c", "+", "1"}], ")"}], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox[
                    RowBox[{"-", "a"}],
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"b", "+", "1"}],
                    (Editable -> True)]}], ";", 
                TagBox[
                  RowBox[{"b", "+", "c", "+", "2"}],
                  (Editable -> True)], ";", 
                TagBox[
                  RowBox[{"-", "z"}],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}], 
        RowBox[{"\[CapitalGamma]", "(", 
          RowBox[{"b", "+", "c", "+", "2"}], ")"}]], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"HypergeometricIR", "[", 
      RowBox[{"%", ",", "t"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"1", "-", "t"}], ")"}], "c"], " ", 
        SuperscriptBox["t", "b"], " ", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{
              RowBox[{"t", " ", "z"}], "+", "1"}], ")"}], "a"]}], 
      TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["HypergeometricSE", "Subsection",
  CellTags->"HypergeometricSE"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "HypergeometricSE[exp, ",
  Cell[BoxData[
      FormBox["\[Nu]", TraditionalForm]]],
  "] expresses Hypergeometric functions by their series expansion in terms of \
a sum (the Sum is omitted and ",
  Cell[BoxData[
      FormBox["\[Nu]", TraditionalForm]]],
  ", running from 0 to ",
  Cell[BoxData[
      FormBox["\[Infinity]", TraditionalForm]]],
  ", is the summation index)."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "HypergeometricSE", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Simplify", "\[Rule]", "FullSimplify"}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["HypergeometricIR",
    ButtonData:>"HypergeometricIR",
    ButtonStyle->"Hyperlink",
    ButtonNote->"HypergeometricIR"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"HypergeometricSE", "[", 
      RowBox[{
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], ",", "\[Nu]"}], 
      "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          SuperscriptBox["z", "\[Nu]"], " ", 
          RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"a", "+", "\[Nu]"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"b", "+", "\[Nu]"}], ")"}]}], 
        RowBox[{
          RowBox[{"\[CapitalGamma]", "(", "a", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"\[Nu]", "+", "1"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"c", "+", "\[Nu]"}], ")"}]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"HypergeometricSE", "[", 
      RowBox[{
        RowBox[{"HypergeometricPFQ", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"a", ",", "b", ",", "c"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"d", ",", "e"}], "}"}], ",", "z"}], "]"}], ",", 
        "\[Nu]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          SuperscriptBox["z", "\[Nu]"], " ", 
          RowBox[{"\[CapitalGamma]", "(", "d", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", "e", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"a", "+", "\[Nu]"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"b", "+", "\[Nu]"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"c", "+", "\[Nu]"}], ")"}]}], 
        RowBox[{
          RowBox[{"\[CapitalGamma]", "(", "a", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"\[Nu]", "+", "1"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"d", "+", "\[Nu]"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"e", "+", "\[Nu]"}], ")"}]}]], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["HypExplicit", "Subsection",
  CellTags->"HypExplicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "HypExplicit[exp, ",
  Cell[BoxData[
      FormBox["\[Nu]", TraditionalForm]]],
  "] expresses Hypergeometric functions in exp by their definition in terms \
of a sum (the Sum is omitted and ",
  Cell[BoxData[
      FormBox["\[Nu]", TraditionalForm]]],
  " is the summation index)."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["HypergeometricIR",
    ButtonData:>"HypergeometricIR",
    ButtonStyle->"Hyperlink",
    ButtonNote->"HypergeometricIR"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Hypergeometric2F1", "[", 
      RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      TagBox[
        RowBox[{
          RowBox[{
            SubscriptBox["\[ThinSpace]", "2"], 
            SubscriptBox["F", "1"]}], "(", 
          RowBox[{
            RowBox[{
              TagBox["a",
                (Editable -> True)], ",", 
              TagBox["b",
                (Editable -> True)]}], ";", 
            TagBox["c",
              (Editable -> True)], ";", 
            TagBox["z",
              (Editable -> True)]}], ")"}],
        InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"HypExplicit", "[", 
      RowBox[{"%", ",", "\[Nu]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          SuperscriptBox["z", "\[Nu]"], " ", 
          RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"a", "+", "\[Nu]"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"b", "+", "\[Nu]"}], ")"}]}], 
        RowBox[{
          RowBox[{"\[CapitalGamma]", "(", "a", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"\[Nu]", "+", "1"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"c", "+", "\[Nu]"}], ")"}]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"HypergeometricPFQ", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"a", ",", "b", ",", "c"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"d", ",", "e"}], "}"}], ",", "z"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      TagBox[
        RowBox[{
          RowBox[{
            SubscriptBox["\[ThinSpace]", "3"], 
            SubscriptBox["F", "2"]}], "(", 
          RowBox[{
            TagBox[
              TagBox[
                RowBox[{
                  TagBox["a",
                    (Editable -> True)], ",", 
                  TagBox["b",
                    (Editable -> True)], ",", 
                  TagBox["c",
                    (Editable -> True)]}],
                InterpretTemplate[ {
                  SlotSequence[ 1]}&]],
              (Editable -> False)], ";", 
            TagBox[
              TagBox[
                RowBox[{
                  TagBox["d",
                    (Editable -> True)], ",", 
                  TagBox["e",
                    (Editable -> True)]}],
                InterpretTemplate[ {
                  SlotSequence[ 1]}&]],
              (Editable -> False)], ";", 
            TagBox["z",
              (Editable -> True)]}], ")"}],
        InterpretTemplate[ HypergeometricPFQ[ #, #2, #3]&]], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"HypExplicit", "[", 
      RowBox[{"%", ",", "\[Nu]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          SuperscriptBox["z", "\[Nu]"], " ", 
          RowBox[{"\[CapitalGamma]", "(", "d", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", "e", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"a", "+", "\[Nu]"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"b", "+", "\[Nu]"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"c", "+", "\[Nu]"}], ")"}]}], 
        RowBox[{
          RowBox[{"\[CapitalGamma]", "(", "a", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"\[Nu]", "+", "1"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"d", "+", "\[Nu]"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"e", "+", "\[Nu]"}], ")"}]}]], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Integrate2", "Subsection",
  CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "Integrate2 is like Integrate, but : Integrate2[a_Plus, b__] := \
Map[Integrate2[#, b]&, a] ( more linear algebra and partial fraction \
decomposition is done) Integrate2[f[x] DeltaFunction[x], x] \[Rule] f[0] \
Integrate2[f[x] DeltaFunction[x0-x], x] \[Rule] f[x0]. Integrate2[f[x] \
DeltaFunction[a + b x], x] \[Rule] Integrate[f[x] (1/Abs[b]) \
DeltaFunction[a/b + x], x], where abs[b] \[Rule] b, if b is a Symbol, and if \
b = -c, then abs[-c] \[Rule] c, i.e., the variable contained in b is supposed \
to be positive. ",
  Cell[BoxData[
      FormBox[
        RowBox[{" ", 
          SuperscriptBox["\[Pi]", "2"]}], TraditionalForm]]],
  " is replaced by 6 Zeta2. Integrate2[1/(1-y),{y,x,1}] is intepreted as \
distribution, i.e. as Integrate2[-1/(1-y)],{y, 0, x}] \[Rule] Log[1-y]. \
Integrate2[1/(1-x),{x,0,1}] \[Rule] 0."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DeltaFunction",
    ButtonData:>"DeltaFunction",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DeltaFunction"],
  ", ",
  ButtonBox["Integrate3",
    ButtonData:>"Integrate3",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Integrate3"],
  ", ",
  ButtonBox["SumS",
    ButtonData:>"SumS",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SumS"],
  ", ",
  ButtonBox["SumT",
    ButtonData:>"SumT",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SumT"],
  "."
}], "Text"],

Cell["\<\
NOTE: Since Integrate2 does do a reordering and partial fraction \
decomposition before calling the integral table of Integrate3 it will in \
general be slower compared to Integrate3 for sums of integrals. I.e., if the \
integrand has already an expanded form and if partial fraction decomposition \
is not necessary it is more effective to use Integrate3.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Integrate2", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"Log", "[", 
              RowBox[{"1", "+", "x"}], "]"}], 
            RowBox[{
              RowBox[{"Log", "[", "x", "]"}], "/", 
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}]}]}], ",", 
          RowBox[{"{", 
            RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", 
      "Timing"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"0.200000000000017053`", " ", "Second"}], ",", 
          RowBox[{
            TagBox[
              RowBox[{"\[Zeta]", "(", 
                TagBox["3",
                  (Editable -> True)], ")"}],
              InterpretTemplate[ Zeta[ #]&]], "-", 
            RowBox[{
              FractionBox["3", "2"], " ", 
              RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
              RowBox[{"log", "(", "2", ")"}]}]}]}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "Since Integrate2 uses table-look-up methods it is much faster than ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s Integrate."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{"PolyLog", "[", 
          RowBox[{"2", ",", 
            RowBox[{"x", "^", "2"}]}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"\[Zeta]", "(", "2", ")"}], "+", 
        RowBox[{"4", " ", 
          RowBox[{"log", "(", "2", ")"}]}], "-", "4"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{"PolyLog", "[", 
          RowBox[{"3", ",", 
            RowBox[{"-", "x"}]}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{"\[Zeta]", "(", "2", ")"}], "2"], "-", 
        FractionBox[
          RowBox[{"3", " ", 
            TagBox[
              RowBox[{"\[Zeta]", "(", 
                TagBox["3",
                  (Editable -> True)], ")"}],
              InterpretTemplate[ Zeta[ #]&]]}], "4"], "-", 
        RowBox[{"2", " ", 
          RowBox[{"log", "(", "2", ")"}]}], "+", "1"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{"PolyLog", "[", 
          RowBox[{"3", ",", 
            RowBox[{"1", "/", 
              RowBox[{"(", 
                RowBox[{"1", "+", "x"}], ")"}]}]}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            RowBox[{"log", "(", "2", ")"}]}], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
        FractionBox[
          RowBox[{"3", " ", 
            TagBox[
              RowBox[{"\[Zeta]", "(", 
                TagBox["3",
                  (Editable -> True)], ")"}],
              InterpretTemplate[ Zeta[ #]&]]}], "4"], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox["log", "3"], "(", "2", ")"}], "3"], "-", 
        RowBox[{
          SuperscriptBox["log", "2"], "(", "2", ")"}], "+", 
        RowBox[{"2", " ", 
          RowBox[{"log", "(", "2", ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"DeltaFunction", "[", 
            RowBox[{"1", "-", "x"}], "]"}], " ", 
          RowBox[{"f", "[", "x", "]"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"f", "(", "1", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "Integrate2 does integration in a Hadamard sense, i.e., ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubsuperscriptBox["\[Integral]", "0", "1"], 
          RowBox[{
            RowBox[{"f", "(", "x", ")"}], 
            RowBox[{"\[DifferentialD]", "x"}]}], " "}], TraditionalForm]]],
  " means acutally expanding the result of ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubsuperscriptBox["\[Integral]", "\[Delta]", 
            RowBox[{"1", "-", "\[Delta]"}]], 
          RowBox[{
            RowBox[{"f", "(", "x", ")"}], 
            RowBox[{"\[DifferentialD]", "x"}]}], " "}], TraditionalForm]]],
  "up do ",
  Cell[BoxData[
      FormBox[
        RowBox[{"O", "(", "\[Delta]", ")"}], TraditionalForm]]],
  " and neglecting all ",
  Cell[BoxData[
      FormBox["\[Delta]", TraditionalForm]]],
  "-dependent terms. E.g. ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            RowBox[{
              SubsuperscriptBox["\[Integral]", "\[Delta]", 
                RowBox[{"1", "-", "\[Delta]"}]], 
              RowBox[{
                RowBox[{"1", "/", 
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}]}], 
                RowBox[{"\[DifferentialD]", "x"}]}]}], " ", "=", " ", 
            RowBox[{
              RowBox[{
                RowBox[{
                  RowBox[{"-", "log"}], 
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}]}], 
                SubsuperscriptBox["\[VerticalSeparator]", "\[Delta]", 
                  RowBox[{"1", "-", "\[Delta]"}]]}], "=", 
              RowBox[{
                RowBox[{
                  RowBox[{"-", 
                    RowBox[{"log", "(", "\[Delta]", ")"}]}], "+", 
                  RowBox[{"log", "(", "1", ")"}]}], "\[Implies]", "0."}]}]}], 
          " "}], TraditionalForm]]]
}], "Text",
  ZeroWidthTimes->False],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{"1", "/", 
          RowBox[{"(", 
            RowBox[{"1", "-", "x"}], ")"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "In the physics literature sometimes the \"+\" notation is used. In \
FeynCalc the ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          RowBox[{"(", 
            FractionBox["1", 
              RowBox[{"1", "-", "x"}]], ")"}], "+"], TraditionalForm]]],
  " is represented by ",
  Cell[BoxData[
      FormBox[
        RowBox[{"PlusDistribution", "[", 
          RowBox[{"1", "/", 
            RowBox[{"(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "]"}], TraditionalForm]]],
  " or just ",
  Cell[BoxData[
      FormBox[
        RowBox[{"1", "/", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"1", "-", "x"}], ")"}], " ", "."}]}], 
        TraditionalForm]]]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{"PlusDistribution", "[", 
          RowBox[{"1", "/", 
            RowBox[{"(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"PolyLog", "[", 
            RowBox[{"2", ",", 
              RowBox[{"1", "-", "x"}]}], "]"}], "/", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"1", "-", "x"}], ")"}], "^", "2"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"2", "-", 
        RowBox[{"\[Zeta]", "(", "2", ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{"Log", "[", "x", "]"}], " ", 
              RowBox[{"Log", "[", 
                RowBox[{"1", "+", "x"}], "]"}]}], ")"}], "/", 
          RowBox[{"(", 
            RowBox[{"1", "+", "x"}], ")"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          TagBox[
            RowBox[{"\[Zeta]", "(", 
              TagBox["3",
                (Editable -> True)], ")"}],
            InterpretTemplate[ Zeta[ #]&]], "8"]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"Log", "[", "x", "]"}], "^", "2"}], "/", 
          RowBox[{"(", 
            RowBox[{"1", "-", "x"}], ")"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"2", " ", 
        TagBox[
          RowBox[{"\[Zeta]", "(", 
            TagBox["3",
              (Editable -> True)], ")"}],
          InterpretTemplate[ Zeta[ #]&]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"PolyLog", "[", 
            RowBox[{"2", ",", 
              RowBox[{"-", "x"}]}], "]"}], "/", 
          RowBox[{"(", 
            RowBox[{"1", "+", "x"}], ")"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          TagBox[
            RowBox[{"\[Zeta]", "(", 
              TagBox["3",
                (Editable -> True)], ")"}],
            InterpretTemplate[ Zeta[ #]&]], "4"], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
          RowBox[{"log", "(", "2", ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"Log", "[", "x", "]"}], " ", 
          RowBox[{"PolyLog", "[", 
            RowBox[{"2", ",", "x"}], "]"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"3", "-", 
        RowBox[{"2", " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{"x", " ", 
          RowBox[{"PolyLog", "[", 
            RowBox[{"3", ",", "x"}], "]"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox[
            RowBox[{"\[Zeta]", "(", "2", ")"}], "4"]}], "+", 
        FractionBox[
          TagBox[
            RowBox[{"\[Zeta]", "(", 
              TagBox["3",
                (Editable -> True)], ")"}],
            InterpretTemplate[ Zeta[ #]&]], "2"], "+", 
        FractionBox["3", "16"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"Log", "[", "x", "]"}], "^", "2"}], " ", 
              RowBox[{"Log", "[", 
                RowBox[{"1", "-", "x"}], "]"}]}], ")"}], "/", 
          RowBox[{"(", 
            RowBox[{"1", "+", "x"}], ")"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{
            SuperscriptBox["log", "2"], "(", "2", ")"}], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
        RowBox[{"4", " ", 
          RowBox[{
            SubscriptBox["Li", "4"], "(", 
            FractionBox["1", "2"], ")"}]}], "-", 
        FractionBox[
          RowBox[{
            SuperscriptBox["log", "4"], "(", "2", ")"}], "6"], "+", 
        FractionBox[
          SuperscriptBox["\[Pi]", "4"], "90"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"PolyLog", "[", 
            RowBox[{"2", ",", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"x", " ", 
                          RowBox[{"(", 
                            RowBox[{"1", "-", "z"}], ")"}]}], "+", "z"}], 
                      ")"}], " ", 
                    RowBox[{"(", 
                      RowBox[{"1", "-", "x", "+", 
                        RowBox[{"x", " ", "z"}]}], ")"}]}], ")"}], "/", 
                "z"}]}], "]"}], "/", 
          RowBox[{"(", 
            RowBox[{"1", "-", "x", "+", 
              RowBox[{"x", " ", "z"}]}], ")"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SuperscriptBox["log", "3"], "(", "z", ")"}], 
          RowBox[{"6", " ", 
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}]}]], "-", 
        FractionBox[
          RowBox[{
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "z"}], ")"}], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", "z", ")"}]}], 
          RowBox[{"1", "-", "z"}]], "-", 
        FractionBox[
          RowBox[{
            RowBox[{"log", "(", 
              RowBox[{"z", "+", "1"}], ")"}], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", "z", ")"}]}], 
          RowBox[{"1", "-", "z"}]], "-", 
        FractionBox[
          RowBox[{"\[ImaginaryI]", " ", "\[Pi]", " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", "z", ")"}]}], 
          RowBox[{"2", " ", 
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}]}]], "-", 
        FractionBox[
          RowBox[{
            RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
            RowBox[{"log", "(", "z", ")"}]}], 
          RowBox[{"1", "-", "z"}]], "+", 
        FractionBox[
          RowBox[{"4", " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "z"}], ")"}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"z", "+", "1"}], ")"}], " ", 
            RowBox[{"log", "(", "z", ")"}]}], 
          RowBox[{"1", "-", "z"}]], "+", 
        FractionBox[
          RowBox[{"2", " ", "\[ImaginaryI]", " ", "\[Pi]", " ", 
            RowBox[{"log", "(", 
              RowBox[{"z", "+", "1"}], ")"}], " ", 
            RowBox[{"log", "(", "z", ")"}]}], 
          RowBox[{"1", "-", "z"}]], "-", 
        FractionBox[
          RowBox[{"2", " ", 
            RowBox[{
              SubscriptBox["Li", "2"], "(", 
              RowBox[{"1", "-", "z"}], ")"}], " ", 
            RowBox[{"log", "(", "z", ")"}]}], 
          RowBox[{"1", "-", "z"}]], "-", 
        FractionBox[
          RowBox[{"2", " ", 
            RowBox[{
              SubscriptBox["Li", "2"], "(", 
              RowBox[{"-", "z"}], ")"}], " ", 
            RowBox[{"log", "(", "z", ")"}]}], 
          RowBox[{"1", "-", "z"}]], "-", 
        FractionBox[
          RowBox[{"2", " ", 
            RowBox[{
              SuperscriptBox["log", "3"], "(", 
              RowBox[{"z", "+", "1"}], ")"}]}], 
          RowBox[{"3", " ", 
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}]}]], "+", 
        FractionBox[
          RowBox[{"\[ImaginaryI]", " ", "\[Pi]", " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], 
          RowBox[{"1", "-", "z"}]], "+", 
        FractionBox[
          RowBox[{"2", " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "z"}], ")"}]}], 
          RowBox[{"1", "-", "z"}]], "+", 
        FractionBox[
          RowBox[{"2", " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"z", "+", "1"}], ")"}]}], 
          RowBox[{"1", "-", "z"}]], "-", 
        FractionBox[
          RowBox[{"2", " ", 
            RowBox[{
              SubscriptBox["S", 
                RowBox[{
                  FormBox["1",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["2",
                    "TraditionalForm"]}]], "(", 
              FormBox[
                RowBox[{"1", "-", "z"}],
                "TraditionalForm"], ")"}]}], 
          RowBox[{"1", "-", "z"}]], "+", 
        FractionBox[
          RowBox[{"4", " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "z"}], ")"}], " ", 
            RowBox[{
              SubscriptBox["Li", "2"], "(", 
              RowBox[{"-", "z"}], ")"}]}], 
          RowBox[{"1", "-", "z"}]], "+", 
        FractionBox[
          RowBox[{"2", " ", "\[ImaginaryI]", " ", "\[Pi]", " ", 
            RowBox[{
              SubscriptBox["Li", "2"], "(", 
              RowBox[{"-", "z"}], ")"}]}], 
          RowBox[{"1", "-", "z"}]], "+", 
        FractionBox[
          RowBox[{"4", " ", 
            RowBox[{
              SubscriptBox["Li", "3"], "(", 
              RowBox[{"1", "-", "z"}], ")"}]}], 
          RowBox[{"1", "-", "z"}]], "+", 
        FractionBox[
          RowBox[{"2", " ", 
            RowBox[{
              SubscriptBox["Li", "3"], "(", 
              RowBox[{"-", "z"}], ")"}]}], 
          RowBox[{"1", "-", "z"}]], "+", 
        FractionBox[
          RowBox[{"4", " ", 
            RowBox[{
              SubscriptBox["Li", "3"], "(", 
              FractionBox["1", 
                RowBox[{"z", "+", "1"}]], ")"}]}], 
          RowBox[{"1", "-", "z"}]], "+", 
        FractionBox[
          RowBox[{"4", " ", 
            RowBox[{
              SubscriptBox["Li", "3"], "(", 
              RowBox[{"-", 
                FractionBox[
                  RowBox[{"1", "-", "z"}], 
                  RowBox[{"z", "+", "1"}]]}], ")"}]}], 
          RowBox[{"1", "-", "z"}]], "-", 
        FractionBox[
          RowBox[{"4", " ", 
            RowBox[{
              SubscriptBox["Li", "3"], "(", 
              FractionBox[
                RowBox[{"1", "-", "z"}], 
                RowBox[{"z", "+", "1"}]], ")"}]}], 
          RowBox[{"1", "-", "z"}]], "-", 
        FractionBox[
          RowBox[{"2", " ", 
            TagBox[
              RowBox[{"\[Zeta]", "(", 
                TagBox["3",
                  (Editable -> True)], ")"}],
              InterpretTemplate[ Zeta[ #]&]]}], 
          RowBox[{"1", "-", "z"}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Apart", "[", 
      RowBox[{
        RowBox[{"Integrate2", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"x", "^", 
                RowBox[{"(", 
                  RowBox[{"OPEm", "-", "1"}], ")"}]}], " ", 
              RowBox[{"PolyLog", "[", 
                RowBox[{"3", ",", 
                  RowBox[{"1", "-", "x"}]}], "]"}]}], ",", 
            RowBox[{"{", 
              RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], ",", 
        "OPEm"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox[
            RowBox[{"\[Zeta]", "(", "2", ")"}], 
            RowBox[{"m", "-", "1"}]]}], "-", 
        FractionBox[
          RowBox[{"\[Zeta]", "(", "2", ")"}], 
          SuperscriptBox["m", "2"]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{
              RowBox[{"-", 
                RowBox[{
                  SubscriptBox["S", "1"], "(", 
                  FormBox[
                    RowBox[{"m", "-", "2"}],
                    "TraditionalForm"], ")"}]}], " ", 
              RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
            RowBox[{"\[Zeta]", "(", "2", ")"}], "+", 
            RowBox[{
              SubscriptBox["S", "12"], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}], "+", 
            TagBox[
              RowBox[{"\[Zeta]", "(", 
                TagBox["3",
                  (Editable -> True)], ")"}],
              InterpretTemplate[ Zeta[ #]&]]}], "m"]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"x", "^", 
            RowBox[{"(", 
              RowBox[{"OPEm", "-", "1"}], ")"}]}], " ", 
          RowBox[{"Log", "[", 
            RowBox[{"1", "-", "x"}], "]"}], " ", 
          RowBox[{"Log", "[", "x", "]"}], " ", 
          FractionBox[
            RowBox[{"Log", "[", 
              RowBox[{"1", "+", "x"}], "]"}], 
            RowBox[{"1", "+", "x"}]]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "4"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
          RowBox[{
            SubsuperscriptBox["S", 
              RowBox[{"-", "1"}], "2"], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "+", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{"log", "(", "2", ")"}], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{"-", "2"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{"-", "1"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "-", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          TagBox[
            RowBox[{"\[Zeta]", "(", 
              TagBox["3",
                (Editable -> True)], ")"}],
            InterpretTemplate[ Zeta[ #]&]], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{"-", "1"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "+", 
        RowBox[{
          FractionBox["3", "2"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
          RowBox[{"log", "(", "2", ")"}], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{"-", "1"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{"-", "2"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "+", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{
            SuperscriptBox["log", "2"], "(", "2", ")"}], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{"-", "2"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "-", 
        RowBox[{
          FractionBox["3", "2"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
          RowBox[{"log", "(", "2", ")"}], " ", 
          RowBox[{
            SubscriptBox["S", "1"], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "+", 
        RowBox[{
          FractionBox["3", "4"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
          RowBox[{
            SubscriptBox["S", "2"], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{
            SuperscriptBox["log", "2"], "(", "2", ")"}], " ", 
          RowBox[{
            SubscriptBox["S", "2"], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "+", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{"log", "(", "2", ")"}], " ", 
          RowBox[{
            SubscriptBox["S", "3"], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "-", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{"log", "(", "2", ")"}], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{"-", "21"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "-", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{"log", "(", "2", ")"}], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{"-", "12"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "-", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{"1", "-", "1"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "+", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{
                RowBox[{"-", "2"}], "-", "1", "-", "1"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "+", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{
                RowBox[{"-", "1"}], "-", "2", "-", "1"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "+", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{
                RowBox[{"-", "1"}], "-", "1", "-", "2"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "+", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{"1", "-", "21"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "+", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{"1", "-", "12"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "+", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{"2", "-", "11"}]], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "+", 
        RowBox[{
          FractionBox["13", "8"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{
            SubscriptBox["S", "1"], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}], " ", 
          TagBox[
            RowBox[{"\[Zeta]", "(", 
              TagBox["3",
                (Editable -> True)], ")"}],
            InterpretTemplate[ Zeta[ #]&]]}], "-", 
        RowBox[{
          FractionBox["21", "8"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{"log", "(", "2", ")"}], " ", 
          TagBox[
            RowBox[{"\[Zeta]", "(", 
              TagBox["3",
                (Editable -> True)], ")"}],
            InterpretTemplate[ Zeta[ #]&]]}], "-", 
        RowBox[{"2", " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{
            SubscriptBox["Li", "4"], "(", 
            FractionBox["1", "2"], ")"}]}], "-", 
        RowBox[{
          FractionBox["1", "12"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{
            SuperscriptBox["log", "4"], "(", "2", ")"}]}], "+", 
        RowBox[{
          FractionBox["5", "4"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
          RowBox[{
            SuperscriptBox["log", "2"], "(", "2", ")"}]}], "+", 
        RowBox[{
          FractionBox["1", "45"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          SuperscriptBox["\[Pi]", "4"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", " ", "/.", " ", 
      RowBox[{"OPEm", "\[Rule]", "2"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["5", "4"], " ", 
          RowBox[{
            SuperscriptBox["log", "2"], "(", "2", ")"}], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
        RowBox[{"3", " ", 
          RowBox[{"log", "(", "2", ")"}], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
        FractionBox[
          RowBox[{"5", " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], "2"], "-", 
        RowBox[{
          FractionBox["21", "8"], " ", 
          RowBox[{"log", "(", "2", ")"}], " ", 
          TagBox[
            RowBox[{"\[Zeta]", "(", 
              TagBox["3",
                (Editable -> True)], ")"}],
            InterpretTemplate[ Zeta[ #]&]]}], "+", 
        FractionBox[
          RowBox[{"21", " ", 
            TagBox[
              RowBox[{"\[Zeta]", "(", 
                TagBox["3",
                  (Editable -> True)], ")"}],
              InterpretTemplate[ Zeta[ #]&]]}], "8"], "-", 
        RowBox[{"2", " ", 
          RowBox[{
            SubscriptBox["Li", "4"], "(", 
            FractionBox["1", "2"], ")"}]}], "-", 
        FractionBox[
          RowBox[{
            SuperscriptBox["log", "4"], "(", "2", ")"}], "12"], "-", 
        RowBox[{
          SuperscriptBox["log", "2"], "(", "2", ")"}], "+", 
        RowBox[{"4", " ", 
          RowBox[{"log", "(", "2", ")"}]}], "+", 
        FractionBox[
          SuperscriptBox["\[Pi]", "4"], "45"], "-", "6"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"N", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox["0.0505138354027443448`", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"NIntegrate", "[", 
      RowBox[{
        RowBox[{"x", " ", 
          RowBox[{"Log", "[", 
            RowBox[{"1", "-", "x"}], "]"}], " ", 
          RowBox[{"Log", "[", "x", "]"}], " ", 
          FractionBox[
            RowBox[{"Log", "[", 
              RowBox[{"1", "+", "x"}], "]"}], 
            RowBox[{"1", "+", "x"}]]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["0.0505138353982719667`", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"x", "^", 
            RowBox[{"(", 
              RowBox[{"OPEm", "-", "1"}], ")"}]}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"PolyLog", "[", 
                RowBox[{"3", ",", 
                  FractionBox[
                    RowBox[{"1", "-", "x"}], 
                    RowBox[{"1", "+", "x"}]]}], "]"}], "-", 
              RowBox[{"PolyLog", "[", 
                RowBox[{"3", ",", 
                  RowBox[{"-", 
                    FractionBox[
                      RowBox[{"1", "-", "x"}], 
                      RowBox[{"1", "+", "x"}]]}]}], "]"}]}], ")"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{
              SubscriptBox["S", 
                RowBox[{"-", "1"}]], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}], " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], "m"], "-", 
        FractionBox[
          RowBox[{
            RowBox[{
              SubscriptBox["S", 
                RowBox[{"-", "1"}]], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}], " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], 
          RowBox[{"2", " ", "m"}]], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}], " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], 
          RowBox[{"2", " ", "m"}]], "-", 
        FractionBox[
          RowBox[{
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}], " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], "m"], "+", 
        FractionBox[
          RowBox[{"3", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{"log", "(", "2", ")"}], " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], 
          RowBox[{"2", " ", "m"}]], "-", 
        FractionBox[
          RowBox[{"3", " ", 
            RowBox[{"log", "(", "2", ")"}], " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], 
          RowBox[{"2", " ", "m"}]], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{
              SubscriptBox["S", 
                RowBox[{"-", "3"}]], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}]}], "m"], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{
              SubscriptBox["S", 
                RowBox[{"-", "2"}]], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}], " ", 
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}]}], "m"], "+", 
        FractionBox[
          RowBox[{
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}], " ", 
            RowBox[{
              SubscriptBox["S", "2"], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}]}], "m"], "+", 
        FractionBox[
          RowBox[{
            SubscriptBox["S", "3"], "(", 
            FormBox["m",
              "TraditionalForm"], ")"}], "m"], "-", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{
              SubscriptBox["S", 
                RowBox[{"-", "21"}]], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}]}], "m"], "-", 
        FractionBox[
          RowBox[{
            SubscriptBox["S", 
              RowBox[{
                RowBox[{"-", "1"}], "-", "2"}]], "(", 
            FormBox["m",
              "TraditionalForm"], ")"}], "m"], "-", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{
              SubscriptBox["S", 
                RowBox[{"-", "12"}]], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}]}], "m"], "-", 
        FractionBox[
          RowBox[{
            SubscriptBox["S", "21"], "(", 
            FormBox["m",
              "TraditionalForm"], ")"}], "m"], "-", 
        FractionBox[
          RowBox[{"7", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            TagBox[
              RowBox[{"\[Zeta]", "(", 
                TagBox["3",
                  (Editable -> True)], ")"}],
              InterpretTemplate[ Zeta[ #]&]]}], 
          RowBox[{"8", " ", "m"}]], "+", 
        FractionBox[
          RowBox[{"21", " ", 
            TagBox[
              RowBox[{"\[Zeta]", "(", 
                TagBox["3",
                  (Editable -> True)], ")"}],
              InterpretTemplate[ Zeta[ #]&]]}], 
          RowBox[{"8", " ", "m"}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DataType", "[", 
      RowBox[{"OPEm", ",", "PositiveInteger"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["True", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"x", "^", 
            RowBox[{"(", 
              RowBox[{"OPEm", "-", "1"}], ")"}]}], " ", 
          RowBox[{"DeltaFunction", "[", 
            RowBox[{"1", "-", "x"}], "]"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
This is the polarized non-singlet spin splitting function whose \
first moment vanishes.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{"SplittingFunction", "[", "PQQNS", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", "4"}], " ", 
                RowBox[{"(", 
                  RowBox[{"x", "+", "1"}], ")"}], " ", 
                RowBox[{
                  SuperscriptBox["log", "2"], "(", "x", ")"}]}], "-", 
              RowBox[{"8", " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"2", " ", "x"}], "+", 
                    FractionBox["3", 
                      RowBox[{"1", "-", "x"}]]}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "-", 
              FractionBox[
                RowBox[{"16", " ", 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
                  RowBox[{"log", "(", 
                    RowBox[{"1", "-", "x"}], ")"}], " ", 
                  RowBox[{"log", "(", "x", ")"}]}], 
                RowBox[{"1", "-", "x"}]], "-", 
              RowBox[{"40", " ", 
                RowBox[{"(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"\[Delta]", "(", 
                  FormBox[
                    RowBox[{"1", "-", "x"}],
                    "TraditionalForm"], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      RowBox[{"-", "24"}], " ", 
                      RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
                    RowBox[{"48", " ", 
                      TagBox[
                        RowBox[{"\[Zeta]", "(", 
                          TagBox["3",
                            (Editable -> True)], ")"}],
                        InterpretTemplate[ Zeta[ #]&]]}], "+", "3"}], 
                  ")"}]}]}], ")"}], " ", 
          SubsuperscriptBox["C", "F", "2"]}], "+", 
        RowBox[{
          SubscriptBox["N", "f"], " ", 
          RowBox[{"(", 
            RowBox[{
              FractionBox[
                RowBox[{"88", " ", "x"}], "9"], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"-", 
                      FractionBox[
                        RowBox[{"16", " ", 
                          RowBox[{"\[Zeta]", "(", "2", ")"}]}], "3"]}], "-", 
                    FractionBox["2", "3"]}], ")"}], " ", 
                RowBox[{"\[Delta]", "(", 
                  FormBox[
                    RowBox[{"1", "-", "x"}],
                    "TraditionalForm"], ")"}]}], "-", 
              FractionBox[
                RowBox[{"8", " ", 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
                  RowBox[{"log", "(", "x", ")"}]}], 
                RowBox[{"3", " ", 
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}]}]], "-", 
              RowBox[{
                FractionBox["80", "9"], " ", 
                SubscriptBox[
                  RowBox[{"(", 
                    FractionBox["1", 
                      RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", 
              FractionBox["8", "9"]}], ")"}], " ", 
          SubscriptBox["C", "F"]}], "-", 
        RowBox[{"8", " ", 
          RowBox[{"(", 
            RowBox[{
              SubscriptBox["C", "F"], "-", 
              FractionBox[
                SubscriptBox["C", "A"], "2"]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"4", " ", 
                RowBox[{"(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{"2", " ", 
                RowBox[{"(", 
                  RowBox[{"x", "+", "1"}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "+", 
              FractionBox[
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        SuperscriptBox["log", "2"], "(", "x", ")"}], "-", 
                      RowBox[{"4", " ", 
                        RowBox[{"log", "(", 
                          RowBox[{"x", "+", "1"}], ")"}], " ", 
                        RowBox[{"log", "(", "x", ")"}]}], "-", 
                      RowBox[{"2", " ", 
                        RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
                      RowBox[{"4", " ", 
                        RowBox[{
                          SubscriptBox["Li", "2"], "(", 
                          RowBox[{"-", "x"}], ")"}]}]}], ")"}]}], 
                RowBox[{"x", "+", "1"}]]}], ")"}], " ", 
          SubscriptBox["C", "F"]}], "+", 
        RowBox[{
          SubscriptBox["C", "A"], " ", 
          RowBox[{"(", 
            RowBox[{
              FractionBox[
                RowBox[{"4", " ", 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
                  RowBox[{
                    SuperscriptBox["log", "2"], "(", "x", ")"}]}], 
                RowBox[{"1", "-", "x"}]], "-", 
              RowBox[{
                FractionBox["4", "3"], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"5", " ", "x"}], "-", 
                    FractionBox["22", 
                      RowBox[{"1", "-", "x"}]], "+", "5"}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "+", 
              RowBox[{
                FractionBox["4", "9"], " ", 
                RowBox[{"(", 
                  RowBox[{"53", "-", 
                    RowBox[{"187", " ", "x"}]}], ")"}]}], "+", 
              RowBox[{"8", " ", 
                RowBox[{"(", 
                  RowBox[{"x", "+", "1"}], ")"}], " ", 
                RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    FractionBox["536", "9"], "-", 
                    RowBox[{"16", " ", 
                      RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
                SubscriptBox[
                  RowBox[{"(", 
                    FractionBox["1", 
                      RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", 
              RowBox[{
                RowBox[{"\[Delta]", "(", 
                  FormBox[
                    RowBox[{"1", "-", "x"}],
                    "TraditionalForm"], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    FractionBox[
                      RowBox[{"88", " ", 
                        RowBox[{"\[Zeta]", "(", "2", ")"}]}], "3"], "-", 
                    RowBox[{"24", " ", 
                      TagBox[
                        RowBox[{"\[Zeta]", "(", 
                          TagBox["3",
                            (Editable -> True)], ")"}],
                        InterpretTemplate[ Zeta[ #]&]]}], "+", 
                    FractionBox["17", "3"]}], ")"}]}]}], ")"}], " ", 
          SubscriptBox["C", "F"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Integrate2", "[", 
        RowBox[{"t", ",", 
          RowBox[{"{", 
            RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", 
      "Timing"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"1.62999999999999545`", " ", "Second"}], ",", "0"}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
Expanding t with respect to x yields a form already suitable for \
Integrate3 and therefore the following is faster:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"Integrate3", "[", 
          RowBox[{
            RowBox[{"Expand", "[", 
              RowBox[{"t", ",", "x"}], "]"}], ",", 
            RowBox[{"{", 
              RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", 
        "Expand"}], "//", "Timing"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"0.489999999999980673`", " ", "Second"}], ",", "0"}], 
        "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", "t", "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"DeltaFunction", "[", 
            RowBox[{"1", "-", "x"}], "]"}], " ", 
          RowBox[{"f", "[", "x", "]"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"f", "(", "1", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          SuperscriptBox["x", "5"], 
          RowBox[{
            RowBox[{"Log", "[", 
              RowBox[{"1", "+", "x"}], "]"}], "^", "2"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["6959", "10800"]}], "+", 
        FractionBox[
          RowBox[{"46", " ", 
            RowBox[{"log", "(", "2", ")"}]}], "45"]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"N", "@", "%"}]], "Input"],

Cell[BoxData[
    FormBox["0.0641985993872032878`", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"NIntegrate", "[", 
      RowBox[{
        RowBox[{
          SuperscriptBox["x", "5"], 
          RowBox[{
            RowBox[{"Log", "[", 
              RowBox[{"1", "+", "x"}], "]"}], "^", "2"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["0.0641985993872023463`", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"x", "^", 
            RowBox[{"(", 
              RowBox[{"OPEm", "-", "1"}], ")"}]}], 
          SuperscriptBox[
            RowBox[{"Log", "[", 
              RowBox[{"1", "+", "x"}], "]"}], "2"]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox[
            RowBox[{"2", " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "m"], " ", 
              RowBox[{
                SubsuperscriptBox["S", "1", "2"], "(", 
                FormBox["m",
                  "TraditionalForm"], ")"}]}], "m"]}], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox[
                FractionBox[
                  RowBox[{"m", "-", "1"}], "2"],
                "TraditionalForm"], ")"}], " ", 
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}]}], "m"], "-", 
        FractionBox[
          RowBox[{
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox[
                FractionBox[
                  RowBox[{"m", "-", "1"}], "2"],
                "TraditionalForm"], ")"}], " ", 
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}]}], "m"], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox[
                FractionBox["m", "2"],
                "TraditionalForm"], ")"}], " ", 
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}]}], "m"], "+", 
        FractionBox[
          RowBox[{
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox[
                FractionBox["m", "2"],
                "TraditionalForm"], ")"}], " ", 
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}]}], "m"], "+", 
        FractionBox[
          RowBox[{"4", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{"log", "(", "2", ")"}], " ", 
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}]}], "m"], "-", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{"log", "(", "2", ")"}], " ", 
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox[
                FractionBox[
                  RowBox[{"m", "-", "1"}], "2"],
                "TraditionalForm"], ")"}]}], "m"], "+", 
        FractionBox[
          RowBox[{
            RowBox[{"log", "(", "2", ")"}], " ", 
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox[
                FractionBox[
                  RowBox[{"m", "-", "1"}], "2"],
                "TraditionalForm"], ")"}]}], "m"], "-", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{"log", "(", "2", ")"}], " ", 
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox[
                FractionBox["m", "2"],
                "TraditionalForm"], ")"}]}], "m"], "-", 
        FractionBox[
          RowBox[{
            RowBox[{"log", "(", "2", ")"}], " ", 
            RowBox[{
              SubscriptBox["S", "1"], "(", 
              FormBox[
                FractionBox["m", "2"],
                "TraditionalForm"], ")"}]}], "m"], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{
              SubscriptBox["S", "2"], "(", 
              FormBox[
                FractionBox[
                  RowBox[{"m", "-", "1"}], "2"],
                "TraditionalForm"], ")"}]}], 
          RowBox[{"2", " ", "m"}]], "-", 
        FractionBox[
          RowBox[{
            SubscriptBox["S", "2"], "(", 
            FormBox[
              FractionBox[
                RowBox[{"m", "-", "1"}], "2"],
              "TraditionalForm"], ")"}], 
          RowBox[{"2", " ", "m"}]], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{
              SubscriptBox["S", "2"], "(", 
              FormBox[
                FractionBox["m", "2"],
                "TraditionalForm"], ")"}]}], 
          RowBox[{"2", " ", "m"}]], "+", 
        FractionBox[
          RowBox[{
            SubscriptBox["S", "2"], "(", 
            FormBox[
              FractionBox["m", "2"],
              "TraditionalForm"], ")"}], 
          RowBox[{"2", " ", "m"}]], "-", 
        FractionBox[
          RowBox[{"2", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{
              SubscriptBox["S", "2"], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}]}], "m"], "-", 
        FractionBox[
          RowBox[{"2", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{
              SubscriptBox["S", 
                RowBox[{"-", "11"}]], "(", 
              FormBox["m",
                "TraditionalForm"], ")"}]}], "m"], "-", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", "2", ")"}]}], "m"], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox["log", "2"], "(", "2", ")"}], "m"]}], 
      TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Integrate3", "Subsection",
  CellTags->"Integrate3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Integrate3 contains the integral table used by Integrate2. \
Integration is performed in a distributional sense. Integrate3 works more \
effectively on a sum of expressions if they are expanded or collected with \
respect to the integration variable. See the examples in Integrate2.\
\>", 
  "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Integrate2",
    ButtonData:>"Integrate2",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Integrate2"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate3", "[", 
      RowBox[{
        RowBox[{
          SuperscriptBox["x", "OPEm"], " ", 
          RowBox[{"Log", "[", "x", "]"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox["1", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"m", "+", "1"}], ")"}], "2"]]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate3", "[", 
      RowBox[{
        FractionBox[
          RowBox[{
            SuperscriptBox["x", "OPEm"], " ", 
            RowBox[{"Log", "[", "x", "]"}], " ", 
            RowBox[{"Log", "[", 
              RowBox[{"1", "-", "x"}], "]"}]}], 
          RowBox[{"1", "-", "x"}]], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
          RowBox[{
            SubscriptBox["S", "1"], "(", 
            FormBox["m",
              "TraditionalForm"], ")"}]}], "-", 
        RowBox[{
          SubscriptBox["S", "12"], "(", 
          FormBox["m",
            "TraditionalForm"], ")"}], "-", 
        RowBox[{
          SubscriptBox["S", "21"], "(", 
          FormBox["m",
            "TraditionalForm"], ")"}], "+", 
        TagBox[
          RowBox[{"\[Zeta]", "(", 
            TagBox["3",
              (Editable -> True)], ")"}],
          InterpretTemplate[ Zeta[ #]&]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate3", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"a", 
            FractionBox[
              RowBox[{
                SuperscriptBox["x", "OPEm"], " ", 
                RowBox[{"Log", "[", "x", "]"}], " ", 
                RowBox[{"Log", "[", 
                  RowBox[{"1", "-", "x"}], "]"}]}], 
              RowBox[{"1", "-", "x"}]]}], "+", 
          RowBox[{"b", 
            FractionBox[
              RowBox[{
                SuperscriptBox["x", "OPEm"], 
                RowBox[{"PolyLog", "[", 
                  RowBox[{"3", ",", 
                    RowBox[{"-", "x"}]}], "]"}]}], 
              RowBox[{"1", "+", "x"}]]}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"a", " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
                RowBox[{
                  SubscriptBox["S", "1"], "(", 
                  FormBox["m",
                    "TraditionalForm"], ")"}]}], "-", 
              RowBox[{
                SubscriptBox["S", "12"], "(", 
                FormBox["m",
                  "TraditionalForm"], ")"}], "-", 
              RowBox[{
                SubscriptBox["S", "21"], "(", 
                FormBox["m",
                  "TraditionalForm"], ")"}], "+", 
              TagBox[
                RowBox[{"\[Zeta]", "(", 
                  TagBox["3",
                    (Editable -> True)], ")"}],
                InterpretTemplate[ Zeta[ #]&]]}], ")"}]}], "+", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", "b", " ", 
          RowBox[{"(", 
            RowBox[{
              FractionBox[
                SuperscriptBox[
                  RowBox[{"\[Zeta]", "(", "2", ")"}], "2"], "8"], "+", 
              RowBox[{
                FractionBox["1", "2"], " ", 
                RowBox[{
                  SubscriptBox["S", 
                    RowBox[{"-", "2"}]], "(", 
                  FormBox["m",
                    "TraditionalForm"], ")"}], " ", 
                RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
              RowBox[{
                RowBox[{"log", "(", "2", ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      SubscriptBox["S", "3"], "(", 
                      FormBox["m",
                        "TraditionalForm"], ")"}], "-", 
                    RowBox[{
                      SubscriptBox["S", 
                        RowBox[{"-", "3"}]], "(", 
                      FormBox["m",
                        "TraditionalForm"], ")"}]}], ")"}]}], "+", 
              RowBox[{
                SubscriptBox["S", 
                  RowBox[{"3", "-", "1"}]], "(", 
                FormBox["m",
                  "TraditionalForm"], ")"}], "-", 
              RowBox[{
                FractionBox["3", "4"], " ", 
                RowBox[{
                  SubscriptBox["S", 
                    RowBox[{"-", "1"}]], "(", 
                  FormBox["m",
                    "TraditionalForm"], ")"}], " ", 
                TagBox[
                  RowBox[{"\[Zeta]", "(", 
                    TagBox["3",
                      (Editable -> True)], ")"}],
                  InterpretTemplate[ Zeta[ #]&]]}], "-", 
              RowBox[{
                FractionBox["3", "4"], " ", 
                RowBox[{"log", "(", "2", ")"}], " ", 
                TagBox[
                  RowBox[{"\[Zeta]", "(", 
                    TagBox["3",
                      (Editable -> True)], ")"}],
                  InterpretTemplate[ Zeta[ #]&]]}]}], ")"}]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate3", "[", 
      RowBox[{
        RowBox[{"DeltaFunctionPrime", "[", 
          RowBox[{"1", "-", "x"}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate3", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"f", "[", "x", "]"}], " ", 
          RowBox[{"DeltaFunctionPrime", "[", 
            RowBox[{"1", "-", "x"}], "]"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["f", "\[Prime]",
          MultilineFunction->None], "(", "1", ")"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate3", "[", 
      RowBox[{
        RowBox[{"1", "/", 
          RowBox[{"(", 
            RowBox[{"1", "-", "x"}], ")"}]}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["IntegrateByParts", "Subsection",
  CellTags->"IntegrateByParts"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
IntegrateByParts[(1-t)^(a Epsilon -1) g[t], deriv, t] does an \
integration by parts of the definite integral over t from 0 to 1.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "IntegrateByParts", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Hold", "\[Rule]", "False"}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"IntegrateByParts", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{"1", "-", "t"}], ")"}], "^", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"a", " ", "Epsilon"}], "-", "1"}], ")"}]}], " ", 
          RowBox[{"g", "[", "t", "]"}]}], ",", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{"1", "-", "t"}], ")"}], "^", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"a", " ", "Epsilon"}], "-", "1"}], ")"}]}], ",", "t"}], 
      "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{
              SuperscriptBox["g", "\[Prime]",
                MultilineFunction->None], "(", "t", ")"}], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"1", "-", "t"}], ")"}], 
              RowBox[{"a", " ", 
                TagBox["\[CurlyEpsilon]",
                  TraditionalForm]}]]}], 
          RowBox[{"a", " ", 
            TagBox["\[CurlyEpsilon]",
              TraditionalForm]}]], "+", 
        FractionBox[
          RowBox[{"g", "(", "0", ")"}], 
          RowBox[{"a", " ", 
            TagBox["\[CurlyEpsilon]",
              TraditionalForm]}]]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["InverseMellin", "Subsection",
  CellTags->"InverseMellin"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "InverseMellin[exp, y] performs the inverse Mellin transform of polynomials \
in OPEm. The inverse transforms are not calculated but a table-lookup is \
done. WARNING: do not \"trust\" the results for the inverse Mellin transform \
involving SumT's; there is an unresolved inconsistency here (related to ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "^", "m"}], ")"}], "."}], 
        TraditionalForm]]]
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DeltaFunction",
    ButtonData:>"DeltaFunction",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DeltaFunction"],
  ", ",
  ButtonBox["Integrate2",
    ButtonData:>"Integrate2",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Integrate2"],
  ", ",
  ButtonBox["OPEm",
    ButtonData:>"OPEi",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SumS"],
  ", ",
  ButtonBox["SumS",
    ButtonData:>"SumS",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SumS"],
  ", ",
  ButtonBox["SumT",
    ButtonData:>"SumT",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SumT"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"InverseMellin", "[", 
      RowBox[{
        RowBox[{"1", "/", "OPEm"}], ",", "y"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["y", 
        RowBox[{"m", "-", "1"}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"InverseMellin", "[", 
      RowBox[{
        RowBox[{"1", "/", 
          RowBox[{"(", 
            RowBox[{"OPEm", "+", "3"}], ")"}]}], ",", "y"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["y", 
        RowBox[{"m", "+", "2"}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"InverseMellin", "[", 
      RowBox[{"1", ",", "y"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["y", 
          RowBox[{"m", "-", "1"}]], " ", 
        RowBox[{"\[Delta]", "(", 
          FormBox[
            RowBox[{"1", "-", "y"}],
            "TraditionalForm"], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"InverseMellin", "[", 
      RowBox[{
        RowBox[{"1", "/", 
          RowBox[{"OPEm", "^", "4"}]}], ",", "y"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "6"]}], " ", 
        SuperscriptBox["y", 
          RowBox[{"m", "-", "1"}]], " ", 
        RowBox[{
          SuperscriptBox["log", "3"], "(", "y", ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"InverseMellin", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"1", "/", "OPEm"}], "+", "1"}], ",", "y"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"\[Delta]", "(", 
            FormBox[
              RowBox[{"1", "-", "y"}],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox["y", 
            RowBox[{"m", "-", "1"}]]}], "+", 
        SuperscriptBox["y", 
          RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"InverseMellin", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"1", "/", "i"}], "+", "1"}], ",", "y", ",", "i"}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"\[Delta]", "(", 
            FormBox[
              RowBox[{"1", "-", "y"}],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox["y", 
            RowBox[{"i", "-", "1"}]]}], "+", 
        SuperscriptBox["y", 
          RowBox[{"i", "-", "1"}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
The inverse operation to InverseMellin is done by Integrate2.\
\>", 
  "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{"InverseMellin", "[", 
          RowBox[{
            RowBox[{"1", "/", "OPEm"}], ",", "y"}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"y", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox["1", "m"], TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
Below is a list of all built-in basic inverse Mellin transforms .\
\
\>", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"list", "=", 
        RowBox[{"{", 
          RowBox[{"1", ",", 
            FractionBox["1", 
              RowBox[{"OPEm", "+", "n"}]], ",", 
            FractionBox["1", 
              RowBox[{
                RowBox[{"-", "OPEm"}], "+", "n"}]], ",", 
            RowBox[{"PolyGamma", "[", 
              RowBox[{"0", ",", "OPEm"}], "]"}], ",", 
            RowBox[{"SumS", "[", 
              RowBox[{"1", ",", 
                RowBox[{
                  RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
            FractionBox[
              RowBox[{"SumS", "[", 
                RowBox[{"1", ",", 
                  RowBox[{
                    RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], 
              RowBox[{"OPEm", "-", "1"}]], ",", 
            FractionBox[
              RowBox[{"SumS", "[", 
                RowBox[{"1", ",", 
                  RowBox[{
                    RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], 
              RowBox[{"1", "-", "OPEm"}]], ",", 
            FractionBox[
              RowBox[{"SumS", "[", 
                RowBox[{"1", ",", 
                  RowBox[{
                    RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], 
              RowBox[{"OPEm", "+", "1"}]], ",", 
            FractionBox[
              RowBox[{"SumS", "[", 
                RowBox[{"1", ",", 
                  RowBox[{
                    RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], 
              SuperscriptBox["OPEm", "2"]], ",", 
            FractionBox[
              RowBox[{"SumS", "[", 
                RowBox[{"1", ",", 
                  RowBox[{
                    RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "OPEm"], 
            ",", 
            FractionBox[
              SuperscriptBox[
                RowBox[{"SumS", "[", 
                  RowBox[{"1", ",", 
                    RowBox[{
                      RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "2"], 
              "OPEm"], ",", 
            RowBox[{"SumS", "[", 
              RowBox[{"2", ",", 
                RowBox[{
                  RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
            FractionBox[
              RowBox[{"SumS", "[", 
                RowBox[{"2", ",", 
                  RowBox[{
                    RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "OPEm"], 
            ",", 
            RowBox[{"SumS", "[", 
              RowBox[{"3", ",", 
                RowBox[{
                  RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
            RowBox[{"SumS", "[", 
              RowBox[{"1", ",", "1", ",", 
                RowBox[{
                  RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
            SuperscriptBox[
              RowBox[{"SumS", "[", 
                RowBox[{"1", ",", 
                  RowBox[{"OPEm", "-", "1"}]}], "]"}], "2"], ",", 
            RowBox[{"SumS", "[", 
              RowBox[{"1", ",", "2", ",", 
                RowBox[{
                  RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
            RowBox[{"SumS", "[", 
              RowBox[{"2", ",", "1", ",", 
                RowBox[{
                  RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
            SuperscriptBox[
              RowBox[{"SumS", "[", 
                RowBox[{"1", ",", 
                  RowBox[{
                    RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "3"], ",", 
            RowBox[{
              RowBox[{"SumS", "[", 
                RowBox[{"1", ",", 
                  RowBox[{
                    RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], " ", 
              RowBox[{"SumS", "[", 
                RowBox[{"2", ",", 
                  RowBox[{
                    RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}]}], ",", 
            RowBox[{"SumS", "[", 
              RowBox[{"1", ",", "1", ",", "1", ",", 
                RowBox[{
                  RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}]}], "}"}]}], 
      ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"im", "[", "z_", "]"}], ":=", 
      RowBox[{"z", " ", "\[LongRightArrow]", 
        RowBox[{"InverseMellin", "[", 
          RowBox[{"z", ",", "y"}], "]"}]}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"im", "[", 
      RowBox[{"OPEm", "^", 
        RowBox[{"(", 
          RowBox[{"-", "3"}], ")"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", 
          SuperscriptBox["m", "3"]], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            FractionBox["1", "2"], " ", 
            SuperscriptBox["y", 
              RowBox[{"m", "-", "1"}]], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", "y", ")"}]}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"im", "[", 
      RowBox[{"OPEm", "^", 
        RowBox[{"(", 
          RowBox[{"-", "2"}], ")"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", 
          SuperscriptBox["m", "2"]], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"-", 
              SuperscriptBox["y", 
                RowBox[{"m", "-", "1"}]]}], " ", 
            RowBox[{"log", "(", "y", ")"}]}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"im", "[", 
      RowBox[{"PolyGamma", "[", 
        RowBox[{"0", ",", "OPEm"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox[
            TagBox["\[Psi]",
              PolyGamma], 
            RowBox[{"(", "0", ")"}]], "(", "m", ")"}], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", 
                TagBox["\[Gamma]",
                  EulerGamma]}], " ", 
              RowBox[{"\[Delta]", "(", 
                FormBox[
                  RowBox[{"1", "-", "y"}],
                  "TraditionalForm"], ")"}], " ", 
              SuperscriptBox["y", 
                RowBox[{"m", "-", "1"}]]}], "-", 
            RowBox[{
              SubscriptBox[
                RowBox[{"(", 
                  FractionBox["1", 
                    RowBox[{"1", "-", "y"}]], ")"}], "+"], " ", 
              SuperscriptBox["y", 
                RowBox[{"m", "-", "1"}]]}]}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[BoxData[" "], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"im", "[", 
      RowBox[{"SumS", "[", 
        RowBox[{"1", ",", 
          RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["S", "1"], "(", 
          FormBox[
            RowBox[{"m", "-", "1"}],
            "TraditionalForm"], ")"}], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"-", 
              SuperscriptBox["y", 
                RowBox[{"m", "-", "1"}]]}], " ", 
            SubscriptBox[
              RowBox[{"(", 
                FractionBox["1", 
                  RowBox[{"1", "-", "y"}]], ")"}], "+"]}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"im", "[", 
      RowBox[{
        RowBox[{"SumS", "[", 
          RowBox[{"1", ",", 
            RowBox[{"OPEm", "-", "1"}]}], "]"}], "/", 
        RowBox[{"(", 
          RowBox[{"OPEm", "-", "1"}], ")"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SubscriptBox["S", "1"], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}], 
          RowBox[{"m", "-", "1"}]], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"-", 
              SuperscriptBox["y", 
                RowBox[{"m", "-", "2"}]]}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "y"}], ")"}]}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"im", "[", 
      RowBox[{
        RowBox[{"SumS", "[", 
          RowBox[{"1", ",", 
            RowBox[{"OPEm", "-", "1"}]}], "]"}], "/", 
        RowBox[{"(", 
          RowBox[{"OPEm", "+", "1"}], ")"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SubscriptBox["S", "1"], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}], 
          RowBox[{"m", "+", "1"}]], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"-", 
              SuperscriptBox["y", 
                RowBox[{"m", "-", "1"}]]}], "-", 
            RowBox[{
              RowBox[{"log", "(", 
                RowBox[{"1", "-", "y"}], ")"}], " ", 
              SuperscriptBox["y", "m"]}], "+", 
            RowBox[{
              RowBox[{"log", "(", "y", ")"}], " ", 
              SuperscriptBox["y", "m"]}], "+", 
            SuperscriptBox["y", "m"]}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"im", "[", 
      RowBox[{
        RowBox[{"SumS", "[", 
          RowBox[{"1", ",", 
            RowBox[{
              RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "/", 
        RowBox[{"OPEm", "^", "2"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SubscriptBox["S", "1"], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}], 
          SuperscriptBox["m", "2"]], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            SuperscriptBox["y", 
              RowBox[{"m", "-", "1"}]], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"-", 
                    FractionBox["1", "2"]}], " ", 
                  RowBox[{
                    SuperscriptBox["log", "2"], "(", "y", ")"}]}], "+", 
                RowBox[{"\[Zeta]", "(", "2", ")"}], "-", 
                RowBox[{
                  SubscriptBox["Li", "2"], "(", "y", ")"}]}], ")"}]}], 
          ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"im", "[", 
      RowBox[{
        RowBox[{"SumS", "[", 
          RowBox[{"1", ",", 
            RowBox[{
              RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "/", "OPEm"}], 
      "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SubscriptBox["S", "1"], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}], "m"], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            SuperscriptBox["y", 
              RowBox[{"m", "-", "1"}]], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"log", "(", "y", ")"}], "-", 
                RowBox[{"log", "(", 
                  RowBox[{"1", "-", "y"}], ")"}]}], ")"}]}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"im", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"SumS", "[", 
            RowBox[{"1", ",", 
              RowBox[{
                RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "^", "2"}], "/", 
        "OPEm"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SubsuperscriptBox["S", "1", "2"], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}], "m"], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            SuperscriptBox["y", 
              RowBox[{"m", "-", "1"}]], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["log", "2"], "(", 
                  RowBox[{"1", "-", "y"}], ")"}], "+", 
                FractionBox[
                  RowBox[{
                    SuperscriptBox["log", "2"], "(", "y", ")"}], "2"], "-", 
                RowBox[{"3", " ", 
                  RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
                RowBox[{
                  SubscriptBox["Li", "2"], "(", 
                  RowBox[{"1", "-", "y"}], ")"}], "+", 
                RowBox[{"2", " ", 
                  RowBox[{
                    SubscriptBox["Li", "2"], "(", "y", ")"}]}]}], ")"}]}], 
          ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"im", "[", 
      RowBox[{"SumS", "[", 
        RowBox[{"2", ",", 
          RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["S", "2"], "(", 
          FormBox[
            RowBox[{"m", "-", "1"}],
            "TraditionalForm"], ")"}], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            SuperscriptBox["y", 
              RowBox[{"m", "-", "1"}]], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
                  RowBox[{"\[Delta]", "(", 
                    FormBox[
                      RowBox[{"1", "-", "y"}],
                      "TraditionalForm"], ")"}]}], "+", 
                FractionBox[
                  RowBox[{"log", "(", "y", ")"}], 
                  RowBox[{"1", "-", "y"}]]}], ")"}]}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"im", "[", 
      RowBox[{
        RowBox[{"SumS", "[", 
          RowBox[{"2", ",", 
            RowBox[{"OPEm", "-", "1"}]}], "]"}], "/", "OPEm"}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SubscriptBox["S", "2"], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}], "m"], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            SuperscriptBox["y", 
              RowBox[{"m", "-", "1"}]], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"-", 
                    FractionBox["1", "2"]}], " ", 
                  RowBox[{
                    SuperscriptBox["log", "2"], "(", "y", ")"}]}], "+", 
                RowBox[{"\[Zeta]", "(", "2", ")"}], "-", 
                RowBox[{
                  SubscriptBox["Li", "2"], "(", 
                  RowBox[{"1", "-", "y"}], ")"}]}], ")"}]}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"im", "[", 
      RowBox[{"SumS", "[", 
        RowBox[{"3", ",", 
          RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["S", "3"], "(", 
          FormBox[
            RowBox[{"m", "-", "1"}],
            "TraditionalForm"], ")"}], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            SuperscriptBox["y", 
              RowBox[{"m", "-", "1"}]], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"\[Delta]", "(", 
                    FormBox[
                      RowBox[{"1", "-", "y"}],
                      "TraditionalForm"], ")"}], " ", 
                  TagBox[
                    RowBox[{"\[Zeta]", "(", 
                      TagBox["3",
                        (Editable -> True)], ")"}],
                    InterpretTemplate[ Zeta[ #]&]]}], "-", 
                FractionBox[
                  RowBox[{
                    SuperscriptBox["log", "2"], "(", "y", ")"}], 
                  RowBox[{"2", " ", 
                    RowBox[{"(", 
                      RowBox[{"1", "-", "y"}], ")"}]}]]}], ")"}]}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"im", "[", 
      RowBox[{"SumS", "[", 
        RowBox[{"1", ",", "1", ",", 
          RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["S", "11"], "(", 
          FormBox[
            RowBox[{"m", "-", "1"}],
            "TraditionalForm"], ")"}], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            SuperscriptBox["y", 
              RowBox[{"m", "-", "1"}]], " ", 
            SubscriptBox[
              RowBox[{"(", 
                FractionBox[
                  RowBox[{"log", "(", 
                    RowBox[{"1", "-", "y"}], ")"}], 
                  RowBox[{"1", "-", "y"}]], ")"}], "+"]}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{" ", 
      RowBox[{"im", "[", 
        RowBox[{"SumS", "[", 
          RowBox[{"1", ",", "2", ",", 
            RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["S", "12"], "(", 
          FormBox[
            RowBox[{"m", "-", "1"}],
            "TraditionalForm"], ")"}], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            SuperscriptBox["y", 
              RowBox[{"m", "-", "1"}]], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"-", 
                    TagBox[
                      RowBox[{"\[Zeta]", "(", 
                        TagBox["3",
                          (Editable -> True)], ")"}],
                      InterpretTemplate[ Zeta[ #]&]]}], " ", 
                  RowBox[{"\[Delta]", "(", 
                    FormBox[
                      RowBox[{"1", "-", "y"}],
                      "TraditionalForm"], ")"}]}], "-", 
                RowBox[{
                  RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
                  SubscriptBox[
                    RowBox[{"(", 
                      FractionBox["1", 
                        RowBox[{"1", "-", "y"}]], ")"}], "+"]}], "+", 
                FractionBox[
                  RowBox[{
                    SubscriptBox["Li", "2"], "(", 
                    RowBox[{"1", "-", "y"}], ")"}], 
                  RowBox[{"1", "-", "y"}]]}], ")"}]}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{" ", 
      RowBox[{"im", "[", 
        RowBox[{"SumS", "[", 
          RowBox[{"2", ",", "1", ",", 
            RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["S", "21"], "(", 
          FormBox[
            RowBox[{"m", "-", "1"}],
            "TraditionalForm"], ")"}], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            SuperscriptBox["y", 
              RowBox[{"m", "-", "1"}]], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"2", " ", 
                  TagBox[
                    RowBox[{"\[Zeta]", "(", 
                      TagBox["3",
                        (Editable -> True)], ")"}],
                    InterpretTemplate[ Zeta[ #]&]], " ", 
                  RowBox[{"\[Delta]", "(", 
                    FormBox[
                      RowBox[{"1", "-", "y"}],
                      "TraditionalForm"], ")"}]}], "-", 
                RowBox[{
                  RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
                  SubscriptBox[
                    RowBox[{"(", 
                      FractionBox["1", 
                        RowBox[{"1", "-", "y"}]], ")"}], "+"]}], "+", 
                FractionBox[
                  RowBox[{
                    SubscriptBox["Li", "2"], "(", "y", ")"}], 
                  RowBox[{"1", "-", "y"}]]}], ")"}]}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"im", "[", 
      RowBox[{"SumS", "[", 
        RowBox[{"1", ",", "1", ",", "1", ",", 
          RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["S", "111"], "(", 
          FormBox[
            RowBox[{"m", "-", "1"}],
            "TraditionalForm"], ")"}], "\[LongRightArrow]", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"-", 
              FractionBox["1", "2"]}], " ", 
            SuperscriptBox["y", 
              RowBox[{"m", "-", "1"}]], " ", 
            SubscriptBox[
              RowBox[{"(", 
                FractionBox[
                  RowBox[{
                    SuperscriptBox["log", "2"], "(", 
                    RowBox[{"1", "-", "y"}], ")"}], 
                  RowBox[{"1", "-", "y"}]], ")"}], "+"]}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", 
        RowBox[{"im", ",", "list"}], "]"}], ";"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Isolate", "Subsection",
  CellTags->"Isolate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Isolate[expr] substitutes abbreviations KK[i] for all Plus[...] \
(sub-sums) in expr. The inserted KK[i] have head HoldForm. Isolate[expr, \
varlist] substitutes KK[i] for all subsums in expr which are free of any \
occurence of a member of the list varlist. Instead of KK any other head or a \
list of names of the abbreviations may be specified with the option \
IsolateNames.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Isolate", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"IsolateNames", "\[Rule]", "KK"}], ",", 
          RowBox[{"IsolatePrint", "\[Rule]", "False"}], ",", 
          RowBox[{"IsolateSplit", "\[Rule]", 
            InterpretationBox["\[Infinity]",
              DirectedInfinity[ 1]]}]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t0", "=", 
      RowBox[{"Isolate", "[", 
        RowBox[{"a", "+", "b"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      TagBox[
        RowBox[{"KK", "(", "1", ")"}],
        HoldForm], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{"Isolate", "[", 
        RowBox[{
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{"a", "+", "b"}], ")"}], " ", "f"}], " ", "+", " ", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"c", "+", "d"}], ")"}], " ", "f"}], " ", "+", " ", 
            "e"}], ",", "f"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"e", "+", 
        RowBox[{"f", " ", 
          TagBox[
            RowBox[{"KK", "(", "1", ")"}],
            HoldForm]}], "+", 
        RowBox[{"f", " ", 
          TagBox[
            RowBox[{"KK", "(", "2", ")"}],
            HoldForm]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", "t1", "]"}]], "Input"],

Cell[BoxData[
    RowBox[{"e", "+", 
      RowBox[{"f", " ", 
        TagBox[
          RowBox[{"KK", "[", "1", "]"}],
          HoldForm]}], "+", 
      RowBox[{"f", " ", 
        TagBox[
          RowBox[{"KK", "[", "2", "]"}],
          HoldForm]}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{"t0", ",", " ", "t1", ",", " ", 
        RowBox[{"ReleaseHold", "[", "t1", "]"}]}], "}"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          TagBox[
            RowBox[{"KK", "(", "1", ")"}],
            HoldForm], ",", 
          RowBox[{"e", "+", 
            RowBox[{"f", " ", 
              TagBox[
                RowBox[{"KK", "(", "1", ")"}],
                HoldForm]}], "+", 
            RowBox[{"f", " ", 
              TagBox[
                RowBox[{"KK", "(", "2", ")"}],
                HoldForm]}]}], ",", 
          RowBox[{"e", "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"a", "+", "b"}], ")"}], " ", "f"}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"c", "+", "d"}], ")"}], " ", "f"}]}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Isolate", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"a", "[", "z", "]"}], " ", 
            RowBox[{"(", 
              RowBox[{"b", "+", 
                RowBox[{"c", " ", 
                  RowBox[{"(", 
                    RowBox[{"y", "+", "z"}], ")"}]}]}], ")"}]}], "+", 
          RowBox[{
            RowBox[{"d", "[", "z", "]"}], " ", 
            RowBox[{"(", 
              RowBox[{"y", "+", "z"}], ")"}]}]}], ",", 
        RowBox[{"{", 
          RowBox[{"a", ",", "d"}], "}"}], ",", 
        RowBox[{"IsolateNames", "\[Rule]", "F"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"d", "(", "z", ")"}], " ", 
          TagBox[
            RowBox[{"F", "(", "3", ")"}],
            HoldForm]}], "+", 
        RowBox[{
          RowBox[{"a", "(", "z", ")"}], " ", 
          TagBox[
            RowBox[{"F", "(", "4", ")"}],
            HoldForm]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"??", "F"}]], "Input"],

Cell[BoxData[
    FormBox["\<\"Global`F\"\>", TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        StyleBox[
          RowBox[{
            RowBox[{
              RowBox[{"F", "[", "3", "]"}], " ", "=", " ", 
              RowBox[{"y", " ", "+", " ", "z"}]}], "\n", " ", "\n", 
            RowBox[{
              RowBox[{"F", "[", "4", "]"}], " ", "=", " ", 
              RowBox[{"b", " ", "+", " ", 
                RowBox[{"c", "*", 
                  RowBox[{"HoldForm", "[", 
                    RowBox[{"F", "[", "3", "]"}], "]"}]}]}]}]}],
          ShowStringCharacters->True,
          NumberMarks->True],
        InputForm[ 
          Definition[ F]],
        Editable->True,
        AutoDelete->True], TraditionalForm]], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Isolate", "[", 
      RowBox[{
        RowBox[{"a", "-", "b", "-", "c", "-", "d", "-", "e"}], ",", 
        RowBox[{"IsolateNames", "\[Rule]", "L"}], ",", 
        RowBox[{"IsolateSplit", "\[Rule]", "15"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      TagBox[
        RowBox[{"L", "(", "3", ")"}],
        HoldForm], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"L", "[", "2", "]"}], ",", 
        RowBox[{"L", "[", "1", "]"}]}], "}"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"a", "-", "b"}], ",", 
          RowBox[{"L", "(", "1", ")"}]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"t0", ",", "t1", ",", "L"}], "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Kummer", "Subsection",
  CellTags->"Kummer"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Kummer[i][exp] applies Kummer relation number i (i =1, ... 24, \
94,95,96) to all Hypergeometric2F1 in exp. i = 94 corresponds to eq. 9.131.2, \
i = 95 to eq. 9.132.1 and i = 96 to eq. 9.132.2 in Gradsteyn & Ryzhik.\
\>", 
  "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["HypergeometricAC",
    ButtonData:>"HypergeometricAC",
    ButtonStyle->"Hyperlink",
    ButtonNote->"HypergeometricAC"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "2", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["a",
                  (Editable -> True)], ",", 
                TagBox["b",
                  (Editable -> True)]}], ";", 
              TagBox["c",
                (Editable -> True)], ";", 
              TagBox["z",
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{
              RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox[
                    RowBox[{"c", "-", "a"}],
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"c", "-", "b"}],
                    (Editable -> True)]}], ";", 
                TagBox["c",
                  (Editable -> True)], ";", 
                TagBox["z",
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "3", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["a",
                  (Editable -> True)], ",", 
                TagBox["b",
                  (Editable -> True)]}], ";", 
              TagBox["c",
                (Editable -> True)], ";", 
              TagBox["z",
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{"-", "a"}]], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox["a",
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"c", "-", "b"}],
                    (Editable -> True)]}], ";", 
                TagBox["c",
                  (Editable -> True)], ";", 
                TagBox[
                  RowBox[{"-", 
                    FractionBox["z", 
                      RowBox[{"1", "-", "z"}]]}],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "4", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["a",
                  (Editable -> True)], ",", 
                TagBox["b",
                  (Editable -> True)]}], ";", 
              TagBox["c",
                (Editable -> True)], ";", 
              TagBox["z",
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{"-", "b"}]], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox[
                    RowBox[{"c", "-", "a"}],
                    (Editable -> True)], ",", 
                  TagBox["b",
                    (Editable -> True)]}], ";", 
                TagBox["c",
                  (Editable -> True)], ";", 
                TagBox[
                  RowBox[{"-", 
                    FractionBox["z", 
                      RowBox[{"1", "-", "z"}]]}],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", 
          RowBox[{"1", "-", "z"}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "6", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", 
            RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["a",
                  (Editable -> True)], ",", 
                TagBox["b",
                  (Editable -> True)]}], ";", 
              TagBox["c",
                (Editable -> True)], ";", 
              TagBox[
                RowBox[{"1", "-", "z"}],
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox["z", 
            RowBox[{
              RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox[
                    RowBox[{"c", "-", "b"}],
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"c", "-", "a"}],
                    (Editable -> True)]}], ";", 
                TagBox["c",
                  (Editable -> True)], ";", 
                TagBox[
                  RowBox[{"1", "-", "z"}],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", 
          RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
          RowBox[{"1", "-", "z"}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "6", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", 
            RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
            RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["a",
                  (Editable -> True)], ",", 
                TagBox["b",
                  (Editable -> True)]}], ";", 
              TagBox[
                RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
                (Editable -> True)], ";", 
              TagBox[
                RowBox[{"1", "-", "z"}],
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox["z", 
            RowBox[{"1", "-", "c"}]], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox[
                    RowBox[{"a", "-", "c", "+", "1"}],
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"b", "-", "c", "+", "1"}],
                    (Editable -> True)]}], ";", 
                TagBox[
                  RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
                  (Editable -> True)], ";", 
                TagBox[
                  RowBox[{"1", "-", "z"}],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", 
          RowBox[{"1", "-", "z"}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "7", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", 
            RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["a",
                  (Editable -> True)], ",", 
                TagBox["b",
                  (Editable -> True)]}], ";", 
              TagBox["c",
                (Editable -> True)], ";", 
              TagBox[
                RowBox[{"1", "-", "z"}],
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox["z", 
            RowBox[{"-", "a"}]], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox["a",
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"c", "-", "b"}],
                    (Editable -> True)]}], ";", 
                TagBox["c",
                  (Editable -> True)], ";", 
                TagBox[
                  RowBox[{"-", 
                    FractionBox[
                      RowBox[{"1", "-", "z"}], "z"]}],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", 
          RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
          RowBox[{"1", "-", "z"}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "7", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", 
            RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
            RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["a",
                  (Editable -> True)], ",", 
                TagBox["b",
                  (Editable -> True)]}], ";", 
              TagBox[
                RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
                (Editable -> True)], ";", 
              TagBox[
                RowBox[{"1", "-", "z"}],
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox["z", 
            RowBox[{"-", "a"}]], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox["a",
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"a", "-", "c", "+", "1"}],
                    (Editable -> True)]}], ";", 
                TagBox[
                  RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
                  (Editable -> True)], ";", 
                TagBox[
                  RowBox[{"-", 
                    FractionBox[
                      RowBox[{"1", "-", "z"}], "z"]}],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", 
          RowBox[{"1", "-", "z"}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "8", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", 
            RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["a",
                  (Editable -> True)], ",", 
                TagBox["b",
                  (Editable -> True)]}], ";", 
              TagBox["c",
                (Editable -> True)], ";", 
              TagBox[
                RowBox[{"1", "-", "z"}],
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox["z", 
            RowBox[{"-", "b"}]], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox[
                    RowBox[{"c", "-", "a"}],
                    (Editable -> True)], ",", 
                  TagBox["b",
                    (Editable -> True)]}], ";", 
                TagBox["c",
                  (Editable -> True)], ";", 
                TagBox[
                  RowBox[{"-", 
                    FractionBox[
                      RowBox[{"1", "-", "z"}], "z"]}],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", 
          RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
          RowBox[{"1", "-", "z"}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "8", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", 
            RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
            RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["a",
                  (Editable -> True)], ",", 
                TagBox["b",
                  (Editable -> True)]}], ";", 
              TagBox[
                RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
                (Editable -> True)], ";", 
              TagBox[
                RowBox[{"1", "-", "z"}],
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox["z", 
            RowBox[{"-", "b"}]], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox[
                    RowBox[{"b", "-", "c", "+", "1"}],
                    (Editable -> True)], ",", 
                  TagBox["b",
                    (Editable -> True)]}], ";", 
                TagBox[
                  RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
                  (Editable -> True)], ";", 
                TagBox[
                  RowBox[{"-", 
                    FractionBox[
                      RowBox[{"1", "-", "z"}], "z"]}],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", 
          RowBox[{"z", "^", 
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "10", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", 
            RowBox[{"z", "^", 
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["a",
                  (Editable -> True)], ",", 
                TagBox["b",
                  (Editable -> True)]}], ";", 
              TagBox["c",
                (Editable -> True)], ";", 
              TagBox[
                FractionBox["1", "z"],
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{
              RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "z"}], ")"}], 
            RowBox[{"a", "+", "b", "-", "c"}]], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox[
                    RowBox[{"c", "-", "a"}],
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"c", "-", "b"}],
                    (Editable -> True)]}], ";", 
                TagBox["c",
                  (Editable -> True)], ";", 
                TagBox[
                  FractionBox["1", "z"],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", 
          RowBox[{"a", "+", "1", "-", "c"}], ",", 
          RowBox[{"a", "+", "1", "-", "b"}], ",", 
          RowBox[{"z", "^", 
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "10", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", 
            RowBox[{"a", "+", "1", "-", "c"}], ",", 
            RowBox[{"a", "+", "1", "-", "b"}], ",", 
            RowBox[{"z", "^", 
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["a",
                  (Editable -> True)], ",", 
                TagBox[
                  RowBox[{"a", "-", "c", "+", "1"}],
                  (Editable -> True)]}], ";", 
              TagBox[
                RowBox[{"a", "-", "b", "+", "1"}],
                (Editable -> True)], ";", 
              TagBox[
                FractionBox["1", "z"],
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{
              RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "z"}], ")"}], 
            RowBox[{"a", "+", "b", "-", "c"}]], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox[
                    RowBox[{"1", "-", "b"}],
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"c", "-", "b"}],
                    (Editable -> True)]}], ";", 
                TagBox[
                  RowBox[{"a", "-", "b", "+", "1"}],
                  (Editable -> True)], ";", 
                TagBox[
                  FractionBox["1", "z"],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", 
          RowBox[{"z", "^", 
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "11", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", 
            RowBox[{"z", "^", 
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["a",
                  (Editable -> True)], ",", 
                TagBox["b",
                  (Editable -> True)]}], ";", 
              TagBox["c",
                (Editable -> True)], ";", 
              TagBox[
                FractionBox["1", "z"],
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", 
                FractionBox[
                  RowBox[{"1", "-", "z"}], "z"]}], ")"}], 
            RowBox[{"-", "a"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "z"}], ")"}], "a"], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox["a",
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"c", "-", "b"}],
                    (Editable -> True)]}], ";", 
                TagBox["c",
                  (Editable -> True)], ";", 
                TagBox[
                  FractionBox["1", 
                    RowBox[{"1", "-", "z"}]],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", 
          RowBox[{"a", "+", "1", "-", "c"}], ",", 
          RowBox[{"a", "+", "1", "-", "b"}], ",", 
          RowBox[{"z", "^", 
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "11", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", 
            RowBox[{"a", "+", "1", "-", "c"}], ",", 
            RowBox[{"a", "+", "1", "-", "b"}], ",", 
            RowBox[{"z", "^", 
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["a",
                  (Editable -> True)], ",", 
                TagBox[
                  RowBox[{"a", "-", "c", "+", "1"}],
                  (Editable -> True)]}], ";", 
              TagBox[
                RowBox[{"a", "-", "b", "+", "1"}],
                (Editable -> True)], ";", 
              TagBox[
                FractionBox["1", "z"],
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", 
                FractionBox[
                  RowBox[{"1", "-", "z"}], "z"]}], ")"}], 
            RowBox[{"-", "a"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "z"}], ")"}], "a"], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox["a",
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"c", "-", "b"}],
                    (Editable -> True)]}], ";", 
                TagBox[
                  RowBox[{"a", "-", "b", "+", "1"}],
                  (Editable -> True)], ";", 
                TagBox[
                  FractionBox["1", 
                    RowBox[{"1", "-", "z"}]],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", 
          RowBox[{"z", "^", 
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "12", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", 
            RowBox[{"z", "^", 
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["a",
                  (Editable -> True)], ",", 
                TagBox["b",
                  (Editable -> True)]}], ";", 
              TagBox["c",
                (Editable -> True)], ";", 
              TagBox[
                FractionBox["1", "z"],
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{"-", "b"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "z"}], ")"}], 
            RowBox[{"b", "-", "a"}]], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox["b",
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"c", "-", "a"}],
                    (Editable -> True)]}], ";", 
                TagBox["c",
                  (Editable -> True)], ";", 
                TagBox[
                  FractionBox["1", 
                    RowBox[{"1", "-", "z"}]],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", 
          RowBox[{"a", "+", "1", "-", "c"}], ",", 
          RowBox[{"a", "+", "1", "-", "b"}], ",", 
          RowBox[{"z", "^", 
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "12", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", 
            RowBox[{"a", "+", "1", "-", "c"}], ",", 
            RowBox[{"a", "+", "1", "-", "b"}], ",", 
            RowBox[{"z", "^", 
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["a",
                  (Editable -> True)], ",", 
                TagBox[
                  RowBox[{"a", "-", "c", "+", "1"}],
                  (Editable -> True)]}], ";", 
              TagBox[
                RowBox[{"a", "-", "b", "+", "1"}],
                (Editable -> True)], ";", 
              TagBox[
                FractionBox["1", "z"],
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{
              RowBox[{"-", "a"}], "+", "c", "-", "1"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "z"}], ")"}], 
            RowBox[{"1", "-", "c"}]], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox[
                    RowBox[{"a", "-", "c", "+", "1"}],
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"1", "-", "b"}],
                    (Editable -> True)]}], ";", 
                TagBox[
                  RowBox[{"a", "-", "b", "+", "1"}],
                  (Editable -> True)], ";", 
                TagBox[
                  FractionBox["1", 
                    RowBox[{"1", "-", "z"}]],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", 
          RowBox[{"z", "^", 
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "14", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", 
            RowBox[{"z", "^", 
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["a",
                  (Editable -> True)], ",", 
                TagBox["b",
                  (Editable -> True)]}], ";", 
              TagBox["c",
                (Editable -> True)], ";", 
              TagBox[
                FractionBox["1", "z"],
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{
              RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "z"}], ")"}], 
            RowBox[{"a", "+", "b", "-", "c"}]], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox[
                    RowBox[{"c", "-", "b"}],
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"c", "-", "a"}],
                    (Editable -> True)]}], ";", 
                TagBox["c",
                  (Editable -> True)], ";", 
                TagBox[
                  FractionBox["1", "z"],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{
          RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
          RowBox[{"b", "+", "1", "-", "a"}], ",", 
          RowBox[{"z", "^", 
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "14", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{
            RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
            RowBox[{"b", "+", "1", "-", "a"}], ",", 
            RowBox[{"z", "^", 
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox[
                  RowBox[{"b", "-", "c", "+", "1"}],
                  (Editable -> True)], ",", 
                TagBox["b",
                  (Editable -> True)]}], ";", 
              TagBox[
                RowBox[{
                  RowBox[{"-", "a"}], "+", "b", "+", "1"}],
                (Editable -> True)], ";", 
              TagBox[
                FractionBox["1", "z"],
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{
              RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "z"}], ")"}], 
            RowBox[{"a", "+", "b", "-", "c"}]], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox[
                    RowBox[{"1", "-", "a"}],
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"c", "-", "a"}],
                    (Editable -> True)]}], ";", 
                TagBox[
                  RowBox[{
                    RowBox[{"-", "a"}], "+", "b", "+", "1"}],
                  (Editable -> True)], ";", 
                TagBox[
                  FractionBox["1", "z"],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", 
          RowBox[{"z", "^", 
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "15", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", 
            RowBox[{"z", "^", 
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["a",
                  (Editable -> True)], ",", 
                TagBox["b",
                  (Editable -> True)]}], ";", 
              TagBox["c",
                (Editable -> True)], ";", 
              TagBox[
                FractionBox["1", "z"],
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{"-", "b"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "z"}], ")"}], "b"], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox["b",
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"c", "-", "a"}],
                    (Editable -> True)]}], ";", 
                TagBox["c",
                  (Editable -> True)], ";", 
                TagBox[
                  FractionBox["1", 
                    RowBox[{"1", "-", "z"}]],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{
          RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
          RowBox[{"b", "+", "1", "-", "a"}], ",", 
          RowBox[{"z", "^", 
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "15", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{
            RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
            RowBox[{"b", "+", "1", "-", "a"}], ",", 
            RowBox[{"z", "^", 
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox[
                  RowBox[{"b", "-", "c", "+", "1"}],
                  (Editable -> True)], ",", 
                TagBox["b",
                  (Editable -> True)]}], ";", 
              TagBox[
                RowBox[{
                  RowBox[{"-", "a"}], "+", "b", "+", "1"}],
                (Editable -> True)], ";", 
              TagBox[
                FractionBox["1", "z"],
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{"-", "b"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "z"}], ")"}], "b"], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox["b",
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"c", "-", "a"}],
                    (Editable -> True)]}], ";", 
                TagBox[
                  RowBox[{
                    RowBox[{"-", "a"}], "+", "b", "+", "1"}],
                  (Editable -> True)], ";", 
                TagBox[
                  FractionBox["1", 
                    RowBox[{"1", "-", "z"}]],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", 
          RowBox[{"z", "^", 
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "16", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", 
            RowBox[{"z", "^", 
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["a",
                  (Editable -> True)], ",", 
                TagBox["b",
                  (Editable -> True)]}], ";", 
              TagBox["c",
                (Editable -> True)], ";", 
              TagBox[
                FractionBox["1", "z"],
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{"-", "a"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "z"}], ")"}], "a"], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox["a",
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"c", "-", "b"}],
                    (Editable -> True)]}], ";", 
                TagBox["c",
                  (Editable -> True)], ";", 
                TagBox[
                  FractionBox["1", 
                    RowBox[{"1", "-", "z"}]],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{
          RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
          RowBox[{"b", "+", "1", "-", "a"}], ",", 
          RowBox[{"z", "^", 
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "16", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{
            RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
            RowBox[{"b", "+", "1", "-", "a"}], ",", 
            RowBox[{"z", "^", 
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox[
                  RowBox[{"b", "-", "c", "+", "1"}],
                  (Editable -> True)], ",", 
                TagBox["b",
                  (Editable -> True)]}], ";", 
              TagBox[
                RowBox[{
                  RowBox[{"-", "a"}], "+", "b", "+", "1"}],
                (Editable -> True)], ";", 
              TagBox[
                FractionBox["1", "z"],
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{
              RowBox[{"-", "b"}], "+", "c", "-", "1"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "z"}], ")"}], 
            RowBox[{"b", "-", "c", "+", "1"}]], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox[
                    RowBox[{"b", "-", "c", "+", "1"}],
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"1", "-", "a"}],
                    (Editable -> True)]}], ";", 
                TagBox[
                  RowBox[{
                    RowBox[{"-", "a"}], "+", "b", "+", "1"}],
                  (Editable -> True)], ";", 
                TagBox[
                  FractionBox["1", 
                    RowBox[{"1", "-", "z"}]],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{
          RowBox[{"a", "+", "1", "-", "c"}], ",", 
          RowBox[{"b", "+", "1", "-", "c"}], ",", 
          RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "18", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{
            RowBox[{"a", "+", "1", "-", "c"}], ",", 
            RowBox[{"b", "+", "1", "-", "c"}], ",", 
            RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox[
                  RowBox[{"a", "-", "c", "+", "1"}],
                  (Editable -> True)], ",", 
                TagBox[
                  RowBox[{"b", "-", "c", "+", "1"}],
                  (Editable -> True)]}], ";", 
              TagBox[
                RowBox[{"2", "-", "c"}],
                (Editable -> True)], ";", 
              TagBox["z",
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{
              RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox[
                    RowBox[{"1", "-", "a"}],
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"1", "-", "b"}],
                    (Editable -> True)]}], ";", 
                TagBox[
                  RowBox[{"2", "-", "c"}],
                  (Editable -> True)], ";", 
                TagBox["z",
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "18", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["a",
                  (Editable -> True)], ",", 
                TagBox["b",
                  (Editable -> True)]}], ";", 
              TagBox["c",
                (Editable -> True)], ";", 
              TagBox["z",
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{
              RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox[
                    RowBox[{"c", "-", "a"}],
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"c", "-", "b"}],
                    (Editable -> True)]}], ";", 
                TagBox["c",
                  (Editable -> True)], ";", 
                TagBox["z",
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{
          RowBox[{"a", "+", "1", "-", "c"}], ",", 
          RowBox[{"b", "+", "1", "-", "c"}], ",", 
          RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "19", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{
            RowBox[{"a", "+", "1", "-", "c"}], ",", 
            RowBox[{"b", "+", "1", "-", "c"}], ",", 
            RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox[
                  RowBox[{"a", "-", "c", "+", "1"}],
                  (Editable -> True)], ",", 
                TagBox[
                  RowBox[{"b", "-", "c", "+", "1"}],
                  (Editable -> True)]}], ";", 
              TagBox[
                RowBox[{"2", "-", "c"}],
                (Editable -> True)], ";", 
              TagBox["z",
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{
              RowBox[{"-", "a"}], "+", "c", "-", "1"}]], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox[
                    RowBox[{"a", "-", "c", "+", "1"}],
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"1", "-", "b"}],
                    (Editable -> True)]}], ";", 
                TagBox[
                  RowBox[{"2", "-", "c"}],
                  (Editable -> True)], ";", 
                TagBox[
                  FractionBox["z", 
                    RowBox[{"z", "-", "1"}]],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "19", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["a",
                  (Editable -> True)], ",", 
                TagBox["b",
                  (Editable -> True)]}], ";", 
              TagBox["c",
                (Editable -> True)], ";", 
              TagBox["z",
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{"-", "a"}]], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox["a",
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"c", "-", "b"}],
                    (Editable -> True)]}], ";", 
                TagBox["c",
                  (Editable -> True)], ";", 
                TagBox[
                  FractionBox["z", 
                    RowBox[{"z", "-", "1"}]],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{
          RowBox[{"a", "+", "1", "-", "c"}], ",", 
          RowBox[{"b", "+", "1", "-", "c"}], ",", 
          RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "20", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{
            RowBox[{"a", "+", "1", "-", "c"}], ",", 
            RowBox[{"b", "+", "1", "-", "c"}], ",", 
            RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox[
                  RowBox[{"a", "-", "c", "+", "1"}],
                  (Editable -> True)], ",", 
                TagBox[
                  RowBox[{"b", "-", "c", "+", "1"}],
                  (Editable -> True)]}], ";", 
              TagBox[
                RowBox[{"2", "-", "c"}],
                (Editable -> True)], ";", 
              TagBox["z",
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{
              RowBox[{"-", "b"}], "+", "c", "-", "1"}]], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox[
                    RowBox[{"b", "-", "c", "+", "1"}],
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"1", "-", "a"}],
                    (Editable -> True)]}], ";", 
                TagBox[
                  RowBox[{"2", "-", "c"}],
                  (Editable -> True)], ";", 
                TagBox[
                  FractionBox["z", 
                    RowBox[{"z", "-", "1"}]],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "20", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["a",
                  (Editable -> True)], ",", 
                TagBox["b",
                  (Editable -> True)]}], ";", 
              TagBox["c",
                (Editable -> True)], ";", 
              TagBox["z",
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{"-", "b"}]], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox["b",
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"c", "-", "a"}],
                    (Editable -> True)]}], ";", 
                TagBox["c",
                  (Editable -> True)], ";", 
                TagBox[
                  FractionBox["z", 
                    RowBox[{"z", "-", "1"}]],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{
          RowBox[{"c", "-", "a"}], ",", 
          RowBox[{"c", "-", "b"}], ",", 
          RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
          RowBox[{"1", "-", "z"}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "22", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{
            RowBox[{"c", "-", "a"}], ",", 
            RowBox[{"c", "-", "b"}], ",", 
            RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
            RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox[
                  RowBox[{"c", "-", "a"}],
                  (Editable -> True)], ",", 
                TagBox[
                  RowBox[{"c", "-", "b"}],
                  (Editable -> True)]}], ";", 
              TagBox[
                RowBox[{
                  RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],
                (Editable -> True)], ";", 
              TagBox[
                RowBox[{"1", "-", "z"}],
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox["z", 
            RowBox[{"1", "-", "c"}]], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox[
                    RowBox[{"1", "-", "a"}],
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"1", "-", "b"}],
                    (Editable -> True)]}], ";", 
                TagBox[
                  RowBox[{
                    RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],
                  (Editable -> True)], ";", 
                TagBox[
                  RowBox[{"1", "-", "z"}],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", 
          RowBox[{"1", "-", "z"}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "22", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", 
            RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["a",
                  (Editable -> True)], ",", 
                TagBox["b",
                  (Editable -> True)]}], ";", 
              TagBox["c",
                (Editable -> True)], ";", 
              TagBox[
                RowBox[{"1", "-", "z"}],
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox["z", 
            RowBox[{
              RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox[
                    RowBox[{"c", "-", "b"}],
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"c", "-", "a"}],
                    (Editable -> True)]}], ";", 
                TagBox["c",
                  (Editable -> True)], ";", 
                TagBox[
                  RowBox[{"1", "-", "z"}],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{
          RowBox[{"c", "-", "a"}], ",", 
          RowBox[{"c", "-", "b"}], ",", 
          RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
          RowBox[{"1", "-", "z"}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "23", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{
            RowBox[{"c", "-", "a"}], ",", 
            RowBox[{"c", "-", "b"}], ",", 
            RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
            RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox[
                  RowBox[{"c", "-", "a"}],
                  (Editable -> True)], ",", 
                TagBox[
                  RowBox[{"c", "-", "b"}],
                  (Editable -> True)]}], ";", 
              TagBox[
                RowBox[{
                  RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],
                (Editable -> True)], ";", 
              TagBox[
                RowBox[{"1", "-", "z"}],
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{"a", "-", "c"}]], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox[
                    RowBox[{"c", "-", "a"}],
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"1", "-", "a"}],
                    (Editable -> True)]}], ";", 
                TagBox[
                  RowBox[{
                    RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],
                  (Editable -> True)], ";", 
                TagBox[
                  RowBox[{"1", "-", 
                    FractionBox["1", 
                      RowBox[{"1", "-", "z"}]]}],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", 
          RowBox[{"1", "-", "z"}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "23", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", 
            RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["a",
                  (Editable -> True)], ",", 
                TagBox["b",
                  (Editable -> True)]}], ";", 
              TagBox["c",
                (Editable -> True)], ";", 
              TagBox[
                RowBox[{"1", "-", "z"}],
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "z"}], ")"}], 
            RowBox[{"-", "a"}]], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox["a",
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"c", "-", "b"}],
                    (Editable -> True)]}], ";", 
                TagBox["c",
                  (Editable -> True)], ";", 
                TagBox[
                  RowBox[{"1", "-", 
                    FractionBox["1", 
                      RowBox[{"1", "-", "z"}]]}],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{
          RowBox[{"c", "-", "a"}], ",", 
          RowBox[{"c", "-", "b"}], ",", 
          RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
          RowBox[{"1", "-", "z"}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "24", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{
            RowBox[{"c", "-", "a"}], ",", 
            RowBox[{"c", "-", "b"}], ",", 
            RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
            RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox[
                  RowBox[{"c", "-", "a"}],
                  (Editable -> True)], ",", 
                TagBox[
                  RowBox[{"c", "-", "b"}],
                  (Editable -> True)]}], ";", 
              TagBox[
                RowBox[{
                  RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],
                (Editable -> True)], ";", 
              TagBox[
                RowBox[{"1", "-", "z"}],
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox["z", 
            RowBox[{"b", "-", "c"}]], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox[
                    RowBox[{"c", "-", "b"}],
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"1", "-", "b"}],
                    (Editable -> True)]}], ";", 
                TagBox[
                  RowBox[{
                    RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],
                  (Editable -> True)], ";", 
                TagBox[
                  RowBox[{"-", 
                    FractionBox[
                      RowBox[{"1", "-", "z"}], "z"]}],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", 
          RowBox[{"1", "-", "z"}]}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "24", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", 
            RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["a",
                  (Editable -> True)], ",", 
                TagBox["b",
                  (Editable -> True)]}], ";", 
              TagBox["c",
                (Editable -> True)], ";", 
              TagBox[
                RowBox[{"1", "-", "z"}],
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          SuperscriptBox["z", 
            RowBox[{"-", "b"}]], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox["b",
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"c", "-", "a"}],
                    (Editable -> True)]}], ";", 
                TagBox["c",
                  (Editable -> True)], ";", 
                TagBox[
                  RowBox[{"-", 
                    FractionBox[
                      RowBox[{"1", "-", "z"}], "z"]}],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "94", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["a",
                  (Editable -> True)], ",", 
                TagBox["b",
                  (Editable -> True)]}], ";", 
              TagBox["c",
                (Editable -> True)], ";", 
              TagBox["z",
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          FractionBox[
            RowBox[{
              RowBox[{"\[CapitalGamma]", "(", 
                RowBox[{"a", "+", "b", "-", "c"}], ")"}], " ", 
              RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
              TagBox[
                RowBox[{
                  RowBox[{
                    SubscriptBox["\[ThinSpace]", "2"], 
                    SubscriptBox["F", "1"]}], "(", 
                  RowBox[{
                    RowBox[{
                      TagBox[
                        RowBox[{"c", "-", "a"}],
                        (Editable -> True)], ",", 
                      TagBox[
                        RowBox[{"c", "-", "b"}],
                        (Editable -> True)]}], ";", 
                    TagBox[
                      RowBox[{
                        RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],
                      (Editable -> True)], ";", 
                    TagBox[
                      RowBox[{"1", "-", "z"}],
                      (Editable -> True)]}], ")"}],
                InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], 
              " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"1", "-", "z"}], ")"}], 
                RowBox[{
                  RowBox[{"-", "a"}], "-", "b", "+", "c"}]]}], 
            RowBox[{
              RowBox[{"\[CapitalGamma]", "(", "a", ")"}], " ", 
              RowBox[{"\[CapitalGamma]", "(", "b", ")"}]}]], "+", 
          FractionBox[
            RowBox[{
              RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
              RowBox[{"\[CapitalGamma]", "(", 
                RowBox[{
                  RowBox[{"-", "a"}], "-", "b", "+", "c"}], ")"}], " ", 
              TagBox[
                RowBox[{
                  RowBox[{
                    SubscriptBox["\[ThinSpace]", "2"], 
                    SubscriptBox["F", "1"]}], "(", 
                  RowBox[{
                    RowBox[{
                      TagBox["a",
                        (Editable -> True)], ",", 
                      TagBox["b",
                        (Editable -> True)]}], ";", 
                    TagBox[
                      RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
                      (Editable -> True)], ";", 
                    TagBox[
                      RowBox[{"1", "-", "z"}],
                      (Editable -> True)]}], ")"}],
                InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}], 
            RowBox[{
              RowBox[{"\[CapitalGamma]", "(", 
                RowBox[{"c", "-", "a"}], ")"}], " ", 
              RowBox[{"\[CapitalGamma]", "(", 
                RowBox[{"c", "-", "b"}], ")"}]}]]}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "95", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["a",
                  (Editable -> True)], ",", 
                TagBox["b",
                  (Editable -> True)]}], ";", 
              TagBox["c",
                (Editable -> True)], ";", 
              TagBox["z",
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          FractionBox[
            RowBox[{
              RowBox[{"\[CapitalGamma]", "(", 
                RowBox[{"b", "-", "a"}], ")"}], " ", 
              RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
              TagBox[
                RowBox[{
                  RowBox[{
                    SubscriptBox["\[ThinSpace]", "2"], 
                    SubscriptBox["F", "1"]}], "(", 
                  RowBox[{
                    RowBox[{
                      TagBox["a",
                        (Editable -> True)], ",", 
                      TagBox[
                        RowBox[{"c", "-", "b"}],
                        (Editable -> True)]}], ";", 
                    TagBox[
                      RowBox[{"a", "-", "b", "+", "1"}],
                      (Editable -> True)], ";", 
                    TagBox[
                      FractionBox["1", 
                        RowBox[{"1", "-", "z"}]],
                      (Editable -> True)]}], ")"}],
                InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], 
              " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"1", "-", "z"}], ")"}], 
                RowBox[{"-", "a"}]]}], 
            RowBox[{
              RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
              RowBox[{"\[CapitalGamma]", "(", 
                RowBox[{"c", "-", "a"}], ")"}]}]], "+", 
          FractionBox[
            RowBox[{
              RowBox[{"\[CapitalGamma]", "(", 
                RowBox[{"a", "-", "b"}], ")"}], " ", 
              RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
              TagBox[
                RowBox[{
                  RowBox[{
                    SubscriptBox["\[ThinSpace]", "2"], 
                    SubscriptBox["F", "1"]}], "(", 
                  RowBox[{
                    RowBox[{
                      TagBox["b",
                        (Editable -> True)], ",", 
                      TagBox[
                        RowBox[{"c", "-", "a"}],
                        (Editable -> True)]}], ";", 
                    TagBox[
                      RowBox[{
                        RowBox[{"-", "a"}], "+", "b", "+", "1"}],
                      (Editable -> True)], ";", 
                    TagBox[
                      FractionBox["1", 
                        RowBox[{"1", "-", "z"}]],
                      (Editable -> True)]}], ")"}],
                InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], 
              " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"1", "-", "z"}], ")"}], 
                RowBox[{"-", "b"}]]}], 
            RowBox[{
              RowBox[{"\[CapitalGamma]", "(", "a", ")"}], " ", 
              RowBox[{"\[CapitalGamma]", "(", 
                RowBox[{"c", "-", "b"}], ")"}]}]]}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Hypergeometric2F1", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
      RowBox[{
        RowBox[{"Kummer", "[", "96", "]"}], "[", 
        RowBox[{"Hypergeometric2F1", "[", 
          RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        TagBox[
          RowBox[{
            RowBox[{
              SubscriptBox["\[ThinSpace]", "2"], 
              SubscriptBox["F", "1"]}], "(", 
            RowBox[{
              RowBox[{
                TagBox["a",
                  (Editable -> True)], ",", 
                TagBox["b",
                  (Editable -> True)]}], ";", 
              TagBox["c",
                (Editable -> True)], ";", 
              TagBox["z",
                (Editable -> True)]}], ")"}],
          InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], "==", 
        RowBox[{
          FractionBox[
            RowBox[{
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "a"], " ", 
              RowBox[{"\[CapitalGamma]", "(", 
                RowBox[{"b", "-", "a"}], ")"}], " ", 
              RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
              TagBox[
                RowBox[{
                  RowBox[{
                    SubscriptBox["\[ThinSpace]", "2"], 
                    SubscriptBox["F", "1"]}], "(", 
                  RowBox[{
                    RowBox[{
                      TagBox["a",
                        (Editable -> True)], ",", 
                      TagBox[
                        RowBox[{"a", "-", "c", "+", "1"}],
                        (Editable -> True)]}], ";", 
                    TagBox[
                      RowBox[{"a", "-", "b", "+", "1"}],
                      (Editable -> True)], ";", 
                    TagBox[
                      FractionBox["1", "z"],
                      (Editable -> True)]}], ")"}],
                InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], 
              " ", 
              SuperscriptBox["z", 
                RowBox[{"-", "a"}]]}], 
            RowBox[{
              RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
              RowBox[{"\[CapitalGamma]", "(", 
                RowBox[{"c", "-", "a"}], ")"}]}]], "+", 
          FractionBox[
            RowBox[{
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "b"], " ", 
              RowBox[{"\[CapitalGamma]", "(", 
                RowBox[{"a", "-", "b"}], ")"}], " ", 
              RowBox[{"\[CapitalGamma]", "(", "c", ")"}], " ", 
              TagBox[
                RowBox[{
                  RowBox[{
                    SubscriptBox["\[ThinSpace]", "2"], 
                    SubscriptBox["F", "1"]}], "(", 
                  RowBox[{
                    RowBox[{
                      TagBox["b",
                        (Editable -> True)], ",", 
                      TagBox[
                        RowBox[{"b", "-", "c", "+", "1"}],
                        (Editable -> True)]}], ";", 
                    TagBox[
                      RowBox[{
                        RowBox[{"-", "a"}], "+", "b", "+", "1"}],
                      (Editable -> True)], ";", 
                    TagBox[
                      FractionBox["1", "z"],
                      (Editable -> True)]}], ")"}],
                InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], 
              " ", 
              SuperscriptBox["z", 
                RowBox[{"-", "b"}]]}], 
            RowBox[{
              RowBox[{"\[CapitalGamma]", "(", "a", ")"}], " ", 
              RowBox[{"\[CapitalGamma]", "(", 
                RowBox[{"c", "-", "b"}], ")"}]}]]}]}], TraditionalForm]], 
  "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Lagrangian", "Subsection",
  CellTags->"Lagrangian"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Lagrangian[oqu] gives the unpolarized quark operator. \
Lagrangian[oqp] gives the polarized quark operator. Lagrangian[\"QCD\"] gives \
part of the QCD Lagrangian.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynRule",
    ButtonData:>"FeynRule",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FeynRule"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Lagrangian", "[", "\"\<QCD\>\"", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "4"]}], " ", 
        RowBox[{
          SubsuperscriptBox["F", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}], "a"], ".", 
          SubsuperscriptBox["F", 
            RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}], "a"]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell["Twist-2 operator product expansion operators", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Lagrangian", "[", "\"\<ogu\>\"", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        SuperscriptBox["\[ImaginaryI]", 
          RowBox[{"m", "-", "1"}]], " ", 
        RowBox[{
          SubsuperscriptBox["F", 
            RowBox[{"\[Alpha]", "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], "a"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              SubsuperscriptBox["D", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                RowBox[{"a", "\[NoBreak]", "b"}]], ")"}], 
            RowBox[{"m", "-", "2"}]], ".", 
          SubsuperscriptBox["F", 
            RowBox[{"\[Alpha]", "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], "b"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Lagrangian", "[", "\"\<ogp\>\"", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        SuperscriptBox["\[ImaginaryI]", "m"], " ", 
        RowBox[{
          SuperscriptBox["\[Epsilon]", 
            RowBox[{
              FormBox[
                FormBox["\[Alpha]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Beta]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["\[Gamma]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}]], ".", 
          SubsuperscriptBox["F", 
            RowBox[{"\[Beta]", "\[NoBreak]", "\[Gamma]"}], "a"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              SubsuperscriptBox["D", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                RowBox[{"a", "\[NoBreak]", "b"}]], ")"}], 
            RowBox[{"m", "-", "2"}]], ".", 
          SubsuperscriptBox["F", 
            RowBox[{"\[Alpha]", "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], "b"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Lagrangian", "[", "\"\<oqu\>\"", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[ImaginaryI]", "m"], " ", 
        RowBox[{
          FormBox[
            OverscriptBox["\[Psi]", "_"],
            "TraditionalForm"], ".", 
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}], ".", 
          SuperscriptBox[
            SubscriptBox["D", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], 
            RowBox[{"m", "-", "1"}]], ".", 
          FormBox["\[Psi]",
            "TraditionalForm"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Lagrangian", "[", "\"\<oqp\>\"", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[ImaginaryI]", "m"], " ", 
        RowBox[{
          FormBox[
            OverscriptBox["\[Psi]", "_"],
            "TraditionalForm"], ".", 
          SuperscriptBox["\[Gamma]", "5"], ".", 
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}], ".", 
          SuperscriptBox[
            SubscriptBox["D", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]], 
            RowBox[{"m", "-", "1"}]], ".", 
          FormBox["\[Psi]",
            "TraditionalForm"]}]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["LC", "Subsection",
  CellTags->"LC"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
LC[m,n,r,s] evaluates to 4-dimensional LeviCivita[m,n,r,s] by \
virtue of applying FeynCalcInternal. LC[m,...][p, ...] evaluates to \
4-dimensional LeviCivita[m,...][p,...] applying FeynCalcInternal.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["LeviCivita",
    ButtonData:>"LeviCivita",
    ButtonStyle->"Hyperlink",
    ButtonNote->"LeviCivita"],
  ", ",
  ButtonBox["LCD",
    ButtonData:>"LCD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"LCD"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"LC", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
      "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
        "\[Mu]", "\[NoBreak]", "\[Nu]", "\[NoBreak]", "\[Rho]", "\[NoBreak]", 
          "\[Sigma]"}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "FCI"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Rho]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Sigma]",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Eps", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Sigma]", "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"LC", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "[", 
      RowBox[{"p", ",", "q"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
        "\[Mu]", "\[NoBreak]", "\[Nu]", "\[NoBreak]", "p", "\[NoBreak]", 
          "q"}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"%", "//", "FCI"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Eps", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
        RowBox[{"Momentum", "[", 
          RowBox[{"p", ",", "D"}], "]"}], ",", 
        RowBox[{"Momentum", "[", 
          RowBox[{"q", ",", "D"}], "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Contract", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"LC", "[", 
              RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}], "[", 
            "p", "]"}], " ", 
          RowBox[{
            RowBox[{"LC", "[", 
              RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}], "[", 
            "q", "]"}]}], "]"}], " "}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"18", " ", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"]}]}], "-", 
        RowBox[{"24", " ", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"]}]}]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["LCD", "Subsection",
  CellTags->"LCD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
LCD[m,n,r,s] evaluates to D-dimensional LeviCivita[m,n,r,s] by \
virtue of FeynCalcInternal. LCD[m,...][p, ...] evaluates to D-dimensional \
LeviCivita[m,...][p,...] applying FeynCalcInternal.\
\>", "Text"],

Cell["\<\
You need to do SetOptions[Eps, Dimension->D] before LCD can be used \
as D-dimensional Levi-Civita input function.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["LeviCivita",
    ButtonData:>"LeviCivita",
    ButtonStyle->"Hyperlink",
    ButtonNote->"LeviCivita"],
  ", ",
  ButtonBox["LC",
    ButtonData:>"LC",
    ButtonStyle->"Hyperlink",
    ButtonNote->"LC"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"Eps", ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"LCD", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
      "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
        "\[Mu]", "\[NoBreak]", "\[Nu]", "\[NoBreak]", "\[Rho]", "\[NoBreak]", 
          "\[Sigma]"}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "FCI"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Rho]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Sigma]",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Eps", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Nu]", ",", "D"}], "]"}], ",", 
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", 
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Sigma]", ",", "D"}], "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"LCD", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "[", 
      RowBox[{"p", ",", "q"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
        "\[Mu]", "\[NoBreak]", "\[Nu]", "\[NoBreak]", "p", "\[NoBreak]", 
          "q"}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"%", "//", "FCI"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Eps", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Nu]", ",", "D"}], "]"}], ",", 
        RowBox[{"Momentum", "[", 
          RowBox[{"p", ",", "D"}], "]"}], ",", 
        RowBox[{"Momentum", "[", 
          RowBox[{"q", ",", "D"}], "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Factor2", "[", 
      RowBox[{"Contract", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"LCD", "[", 
              RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}], "[", 
            "p", "]"}], " ", 
          RowBox[{
            RowBox[{"LCD", "[", 
              RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}], "[", 
            "q", "]"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"1", "-", "D"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"2", "-", "D"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"3", "-", "D"}], ")"}], " ", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SetOptions", "[", 
      RowBox[{"Eps", ",", 
        RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Dimension", "\[Rule]", "4"}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["LeftPartialD", "Subsection",
  CellTags->"LeftPartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "LeftPartialD[\[Mu]] denotes ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], TraditionalForm]]],
  "acting to the left."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["ExpandPartialD",
    ButtonData:>"ExpandPartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ExpandPartialD"],
  ", ",
  ButtonBox["PartialD",
    ButtonData:>"PartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"PartialD"],
  ", ",
  ButtonBox["LeftRightPartialD",
    ButtonData:>"LeftRightPartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"LeftRightPartialD"],
  ", ",
  ButtonBox["RightPartialD",
    ButtonData:>"RightPartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"RightPartialD"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
      RowBox[{"LeftPartialD", "[", "\[Nu]", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["A", "\[Mu]"], ".", 
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SubscriptBox["A", "\[Mu]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"LeftPartialD", "[", 
        RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], "]"}], " ", 
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"LeftPartialD", "[", "\[Mu]", "]"}], "]"}]], "Input"],

Cell[BoxData[
    RowBox[{"LeftPartialD", "[", 
      RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}], ".", 
      RowBox[{"LeftPartialD", "[", "\[Rho]", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["A", "\[Mu]"], ".", 
        SubscriptBox["A", "\[Nu]"], ".", 
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
          FormBox[
            FormBox["\[Rho]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["A", "\[Mu]"], ".", 
          SubscriptBox["A", "\[Nu]"]}], " ", 
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
          FormBox[
            FormBox["\[Rho]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"QuantumField", "[", 
          RowBox[{"A", ",", 
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
        RowBox[{"QuantumField", "[", 
          RowBox[{"A", ",", 
            RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}], " ", 
      RowBox[{"LeftPartialD", "[", 
        RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], "]"}]}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["LeftRightPartialD", "Subsection",
  CellTags->"LeftRightPartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "LeftRightPartialD[mu] denotes ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], TraditionalForm]]],
  ", acting to the left and right. ExplitPartialD[LeftRightPartialD[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "]] gives 1/2 (RightPartialD[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "] - LeftPartialD[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "])."
}], "Text"],

Cell[TextData[{
  "See also: ",
  ButtonBox["ExplicitPartialD",
    ButtonData:>"ExplicitPartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ExplicitPartialD"],
  ", ",
  ButtonBox["ExpandPartialD",
    ButtonData:>"ExpandPartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ExpandPartialD"],
  ", ",
  ButtonBox["PartialD",
    ButtonData:>"PartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"PartialD"],
  ", ",
  ButtonBox["LeftPartialD",
    ButtonData:>"LeftPartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"LeftPartialD"],
  ", ",
  ButtonBox["LeftRightPartialD2",
    ButtonData:>"LeftRightPartialD2",
    ButtonStyle->"Hyperlink",
    ButtonNote->"LeftRightPartialD2"],
  ", ",
  ButtonBox["RightPartialD",
    ButtonData:>"RightPartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"RightPartialD"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"LeftRightPartialD", "[", "\[Mu]", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubscriptBox[
        OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
        FormBox[
          FormBox["\[Mu]",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExplicitPartialD", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[RightArrow]"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], "-", 
            SubscriptBox[
              OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"LeftRightPartialD", "[", "\[Mu]", "]"}], ".", 
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SubscriptBox["A", "\[Nu]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[Null]"]}], "2"], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["A", "\[Nu]"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
      RowBox[{"LeftRightPartialD", "[", "\[Nu]", "]"}], ".", 
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["A", "\[Mu]"], ".", 
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SubscriptBox["A", "\[Rho]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{
            SubscriptBox["A", "\[Mu]"], ".", 
            RowBox[{
              SubscriptBox["\[PartialD]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], 
              SubsuperscriptBox["A", 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[Null]"]}]}]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{
            SubscriptBox["A", "\[Mu]"], ".", 
            SubscriptBox["A", "\[Rho]"]}], " ", 
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["LeftRightPartialD2", "Subsection",
  CellTags->"LeftRightPartialD2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "LeftRightPartialD2[mu] denotes ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], TraditionalForm]]],
  ", acting to the left and right. ExplitPartialD[LeftRightPartialD2[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "]] gives (RightPartialD[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "] + LeftPartialD[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "])."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["ExplicitPartialD",
    ButtonData:>"ExplicitPartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ExplicitPartialD"],
  ", ",
  ButtonBox["ExpandPartialD",
    ButtonData:>"ExpandPartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ExpandPartialD"],
  ", ",
  ButtonBox["PartialD",
    ButtonData:>"PartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"PartialD"],
  ", ",
  ButtonBox["LeftPartialD",
    ButtonData:>"LeftPartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"LeftPartialD"],
  ", ",
  ButtonBox["RightPartialD",
    ButtonData:>"RightPartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"RightPartialD"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"LeftRightPartialD2", "[", "\[Mu]", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubscriptBox[
        OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
        FormBox[
          FormBox["\[Mu]",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExplicitPartialD", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], "+", 
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[RightArrow]"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"LeftRightPartialD2", "[", "\[Mu]", "]"}], ".", 
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SubscriptBox["A", "\[Nu]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SubscriptBox["A", "\[Nu]"]}], "+", 
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          SubsuperscriptBox["A", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[Null]"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
      RowBox[{"LeftRightPartialD2", "[", "\[Nu]", "]"}], ".", 
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["A", "\[Mu]"], ".", 
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SubscriptBox["A", "\[Rho]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["A", "\[Mu]"], ".", 
          RowBox[{
            SubscriptBox["\[PartialD]", 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], 
            SubsuperscriptBox["A", 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"], "\[Null]"]}]}], "+", 
        RowBox[{
          RowBox[{
            SubscriptBox["A", "\[Mu]"], ".", 
            SubscriptBox["A", "\[Rho]"]}], " ", 
          SubscriptBox[
            OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["LeviCivita ", "Subsection",
  CellTags->"LeviCivita"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "LeviCivita[",
  Cell[BoxData[
      FormBox[
        RowBox[{
        "\[Mu]", ",", " ", "\[Nu]", ",", " ", "\[Rho]", ",", " ", 
          "\[Sigma]"}], TraditionalForm]]],
  "] is an input function for the totally antisymmetric Levi-Civita tensor. \
It evaluates automatically to the internal representation Eps[ LorentzIndex[",
  
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "], LorentzIndex[",
  Cell[BoxData[
      FormBox["\[Nu]", TraditionalForm]]],
  "], LorentzIndex[",
  Cell[BoxData[
      FormBox[
        FormBox["\[Rho]",
          "TraditionalForm"], TraditionalForm]]],
  "], LorentzIndex[",
  Cell[BoxData[
      FormBox[
        ErrorBox[
          RowBox[{"Cell", "[", "\[Sigma]", "]"}]], TraditionalForm]]],
  "] ] (or with a second argument in LorentzIndex for the Dimension, if the \
option Dimension of LeviCivita is changed). LeviCivita[",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[Mu]", ",", " ", 
          RowBox[{"\[Nu]", " ", "..."}]}], TraditionalForm]]],
  "][ ",
  Cell[BoxData[
      FormBox[
        RowBox[{"p", ",", " ", ".."}], TraditionalForm]]],
  ".] evaluates to Eps[LorentzIndex[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "], LorentzIndex[",
  Cell[BoxData[
      FormBox["\[Nu]", TraditionalForm]]],
  "], ..., Momentum[",
  Cell[BoxData[
      FormBox["p", TraditionalForm]]],
  "], ...]."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "LeviCivita", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Dimension", "\[Rule]", "D"}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["LC",
    ButtonData:>"LC",
    ButtonStyle->"Hyperlink",
    ButtonNote->"LC"],
  ", ",
  ButtonBox["LCD",
    ButtonData:>"LCD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"LCD"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"LeviCivita", "[", 
      RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Delta]"}], 
      "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox[
            FormBox["\[Alpha]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Beta]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Gamma]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Delta]",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"LeviCivita", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "[", 
      RowBox[{"OPEDelta", ",", "p"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], "\[NoBreak]", 
          FormBox["p",
            "TraditionalForm"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"LeviCivita", "[", "]"}], "[", 
      RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"], "\[NoBreak]", 
          FormBox["r",
            "TraditionalForm"], "\[NoBreak]", 
          FormBox["s",
            "TraditionalForm"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"LeviCivita", "[", 
      RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Beta]", ",", "\[Delta]"}], 
      "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"LeviCivita", "[", 
          RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "[", 
        RowBox[{"OPEDelta", ",", "p"}], "]"}], "//", "StandardForm"}]], 
  "Input"],

Cell[BoxData[
    RowBox[{"Eps", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
        RowBox[{"Momentum", "[", "OPEDelta", "]"}], ",", 
        RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"t1", " ", "=", " ", 
        RowBox[{
          RowBox[{"LeviCivita", "[", 
            RowBox[{
            "\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Rho]"}], 
            "]"}], ".", 
          RowBox[{"LeviCivita", "[", 
            RowBox[{
            "\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Rho]"}], 
            "]"}]}]}], " "}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Alpha]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Beta]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Gamma]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]}]], ".", 
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Alpha]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Beta]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Gamma]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", "t1", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", "24"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"LeviCivita", ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";", 
      RowBox[{"SetOptions", "[", 
        RowBox[{"Eps", ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Contract", "[", " ", 
        RowBox[{
          RowBox[{"LeviCivita", "[", 
            RowBox[{
            "\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Rho]"}], 
            "]"}], ".", 
          RowBox[{"LeviCivita", "[", 
            RowBox[{
            "\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Rho]"}], 
            "]"}]}], "]"}], "//", "Factor2"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"1", "-", "D"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"2", "-", "D"}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{"3", "-", "D"}], ")"}], " ", "D"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"LeviCivita", ",", 
          RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}], ";", 
      RowBox[{"SetOptions", "[", 
        RowBox[{"Eps", ",", 
          RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"LC", "[", 
      RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Delta]"}], 
      "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
        "\[Alpha]", "\[NoBreak]", "\[Beta]", "\[NoBreak]", "\[Gamma]", 
          "\[NoBreak]", "\[Delta]"}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", "t1", "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Li2 ", "Subsection",
  CellTags->"Li2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
 Li2 is an abbreviation for the dilog function, i.e., Li2 = \
PolyLog[2, #]&.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  ButtonBox["Li3",
    ButtonData:>"Li3",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Li3"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Li2", "[", "x", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["Li", "2"], "(", "x", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Li2", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"PolyLog", "[", 
        RowBox[{"2", ",", "#1"}], "]"}], "&"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"-", 
      RowBox[{"\[Integral]", 
        RowBox[{
          FractionBox[
            RowBox[{"Log", "[", 
              RowBox[{"1", "-", "x"}], "]"}], "x"], 
          RowBox[{"\[DifferentialD]", "x"}]}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["Li", "2"], "(", "x", ")"}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Li3 ", "Subsection",
  CellTags->"Li3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
 Li3 is an abbreviation for the trilog function, i.e., Li3 = \
PolyLog[3, #]&.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  ButtonBox["Li2",
    ButtonData:>"Li2",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Li2"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Li3", "[", "x", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["Li", "3"], "(", "x", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Li3", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"PolyLog", "[", 
        RowBox[{"3", ",", "#1"}], "]"}], "&"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"D", "[", 
      RowBox[{
        RowBox[{"Li3", "[", "x", "]"}], ",", "x"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          SubscriptBox["Li", "2"], "(", "x", ")"}], "x"], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"\[Integral]", 
      RowBox[{
        FractionBox[
          RowBox[{"Li2", "[", "x", "]"}], "x"], 
        RowBox[{"\[DifferentialD]", "x"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["Li", "3"], "(", "x", ")"}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["LorentzIndex", "Subsection",
  CellTags->"LorentzIndex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
LorentzIndex[mu] denotes a four dimensional Lorentz index. For \
other than four dimensions: LorentzIndex[mu, D] or LorentzIndex[mu], etc. \
LorentzIndex[mu, 4] simplifies to LorentzIndex[mu].\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["ChangeDimension",
    ButtonData:>"ChangeDimension",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ChangeDimension"],
  ", ",
  ButtonBox["Momentum",
    ButtonData:>"Momentum",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Momentum"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell["This denotes a four-dimensional Lorentz index.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}]], "Input"],

Cell[BoxData[
    FormBox["\[Alpha]", TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
An optional second argument can be given for a dimension different \
from 4.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"LorentzIndex", "[", 
      RowBox[{"\[Alpha]", ",", "n"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["\[Alpha]", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData["LorentzIndex[ComplexIndex[\[Alpha]]]"], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Alpha]", "*"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ComplexConjugate", "[", 
      RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Mu]", "*"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ComplexConjugate", "[", 
      RowBox[{"LorentzIndex", "[", 
        RowBox[{"ComplexIndex", "[", "\[Beta]", "]"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["\[Beta]", TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
Setting $LorentzIndices=True displays the dimension (if different \
from 4) as a subindex.\
\>", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"$LorentzIndices", "=", "True"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Alpha]", ",", "D"}], "]"}], ",", 
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Nu]", ",", "n"}], "]"}], ",", 
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Beta]", ",", "4"}], "]"}]}], "}"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SubscriptBox[
            FormBox["\[Alpha]",
              "TraditionalForm"], 
            FormBox["D",
              "TraditionalForm"]], ",", 
          SubscriptBox[
            FormBox["\[Nu]",
              "TraditionalForm"], 
            FormBox["n",
              "TraditionalForm"]], ",", "\[Beta]"}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"$LorentzIndices", " ", "=", " ", "False"}], ";"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Map2", "Subsection",
  CellTags->"Map2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Map2[f, exp] is equivalent to Map if Nterms[exp] > 0, otherwise \
Map2[f, exp] gives f[exp].\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["NTerms",
    ButtonData:>"NTerms",
    ButtonStyle->"Hyperlink",
    ButtonNote->"NTerms"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Map2", "[", 
      RowBox[{"f", ",", 
        RowBox[{"a", "-", "b"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"f", "(", "a", ")"}], "+", 
        RowBox[{"f", "(", 
          RowBox[{"-", "b"}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Map2", "[", 
      RowBox[{"f", ",", "x"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"f", "(", "x", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Map2", "[", 
      RowBox[{"f", ",", 
        RowBox[{"{", 
          RowBox[{"a", ",", "b", ",", "c"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"f", "(", 
        RowBox[{"{", 
          RowBox[{"a", ",", "b", ",", "c"}], "}"}], ")"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Map2", "[", 
      RowBox[{"f", ",", "1"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"f", "(", "1", ")"}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MetricTensor", "Subsection",
  CellTags->"MetricTensor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "MetricTensor[",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[Mu]", ",", " ", "\[Nu]"}], TraditionalForm]]],
  "] is the metric tensor. The default dimension is 4."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynCalcExternal",
    ButtonData:>"FeynCalcExternal",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FeynCalcExternal"],
  ", ",
  ButtonBox["FCE",
    ButtonData:>"FCE",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FCE"],
  ", ",
  ButtonBox["FCI",
    ButtonData:>"FCI",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FCI"],
  ", ",
  ButtonBox["MT",
    ButtonData:>"MT",
    ButtonStyle->"Hyperlink",
    ButtonNote->"MT"],
  ", ",
  ButtonBox["MTD",
    ButtonData:>"MTD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"MTD"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"MetricTensor", "[", 
      RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["g", 
        RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", 
      RowBox[{"%", " ", "%"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["4", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"MetricTensor", "[", 
      RowBox[{"\[Alpha]", ",", "\[Beta]", ",", 
        RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["g", 
        RowBox[{
          FormBox[
            FormBox["\[Alpha]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Beta]",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", 
      RowBox[{"%", " ", "%"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["n", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"MetricTensor", "[", 
        RowBox[{"a", ",", "b"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "a", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "b", "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"MetricTensor", "[", 
        RowBox[{"a", ",", "b", ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"a", ",", "D"}], "]"}], ",", 
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"b", ",", "D"}], "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"FCE", "[", 
        RowBox[{"MetricTensor", "[", 
          RowBox[{"a", ",", "b"}], "]"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    RowBox[{"MT", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"FCE", "[", 
        RowBox[{"MetricTensor", "[", 
          RowBox[{"a", ",", "b", ",", 
            RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], "]"}], "]"}]], 
  "Input"],

Cell[BoxData[
    RowBox[{"MTD", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MomentumCombine", "Subsection",
  CellTags->"MomentumCombine"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
MomentumCombine[expr] is the inverse operation to MomentumExpand \
and ExpandScalarProduct. MomentumCombine combines also Pair`s.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  ButtonBox["ExpandScalarProduct",
    ButtonData:>"ExpandScalarProduct",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ExpandScalarProduct"],
  ", ",
  ButtonBox["Momentum",
    ButtonData:>"Momentum",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Momentum"],
  ", ",
  ButtonBox["MomentumExpand",
    ButtonData:>"MomentumExpand",
    ButtonStyle->"Hyperlink",
    ButtonNote->"MomentumExpand"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"Momentum", "[", "p", "]"}], "-", 
            RowBox[{"2", " ", 
              RowBox[{"Momentum", "[", "q", "]"}]}]}], " "}], "//", " ", 
        "MomentumCombine"}], " ", "//", " ", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Momentum", "[", 
      RowBox[{"p", "-", 
        RowBox[{"2", " ", "q"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{
        RowBox[{
          RowBox[{"FV", "[", 
            RowBox[{"p", ",", "\[Mu]"}], "]"}], " ", "+", " ", 
          RowBox[{"2", " ", 
            RowBox[{"FV", "[", 
              RowBox[{"q", ",", "\[Mu]"}], "]"}]}]}], " "}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["p", "\[Mu]"], "+", 
        RowBox[{"2", " ", 
          SuperscriptBox["q", "\[Mu]"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"MomentumCombine", "[", "t1", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        RowBox[{"(", 
          FormBox[
            RowBox[{"p", "+", 
              RowBox[{"2", " ", "q"}]}],
            "TraditionalForm"], ")"}], 
        FormBox[
          FormBox["\[Mu]",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"MomentumCombine", "[", "t1", "]"}], "//", "StandardForm"}]], 
  "Input"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"Momentum", "[", 
          RowBox[{"p", "+", 
            RowBox[{"2", " ", "q"}]}], "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"MomentumCombine", "[", "t1", "]"}], "//", 
      "ExpandScalarProduct"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], "+", 
        RowBox[{"2", " ", 
          SuperscriptBox[
            FormBox["q",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
          RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], "+", 
      RowBox[{"2", " ", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
            RowBox[{"Momentum", "[", "q", "]"}]}], "]"}]}]}]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", "t1", "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MomentumExpand", "Subsection",
  CellTags->"MomentumExpand"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
MomentumExpand[expr] expands Momentum[a+b+ ...] in expr into \
Momentum[a] + Momentum[b] + ... .\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["ExpandScalarProduct",
    ButtonData:>"ExpandScalarProduct",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ExpandScalarProduct"],
  ", ",
  ButtonBox["MomentumCombine",
    ButtonData:>"MomentumCombine",
    ButtonStyle->"Hyperlink",
    ButtonNote->"MomentumCombine"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"MomentumExpand", "[", 
        RowBox[{"Momentum", "[", 
          RowBox[{"p", "+", "q"}], "]"}], "]"}], "//", "StandardForm"}]], 
  "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Momentum", "[", "p", "]"}], "+", 
      RowBox[{"Momentum", "[", "q", "]"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ScalarProduct", "[", 
      RowBox[{
        RowBox[{"p", "+", "q"}], ",", "r"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"(", "\[NoBreak]", 
        FormBox[
          RowBox[{"p", "+", "q"}],
          "TraditionalForm"], "\[NoBreak]", ")", "\[NoBreak]", 
        "\[CenterDot]", "\[NoBreak]", 
        FormBox["r",
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", 
          RowBox[{"p", "+", "q"}], "]"}], ",", 
        RowBox[{"Momentum", "[", "r", "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"MomentumExpand", "[", 
      RowBox[{"ScalarProduct", "[", 
        RowBox[{
          RowBox[{"p", "+", "q"}], ",", "r"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"(", "\[NoBreak]", 
        FormBox[
          RowBox[{"p", "+", "q"}],
          "TraditionalForm"], "\[NoBreak]", ")", "\[NoBreak]", 
        "\[CenterDot]", "\[NoBreak]", 
        FormBox["r",
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"Momentum", "[", "p", "]"}], "+", 
          RowBox[{"Momentum", "[", "q", "]"}]}], ",", 
        RowBox[{"Momentum", "[", "r", "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"MomentumExpand", "[", 
      RowBox[{"ScalarProduct", "[", 
        RowBox[{
          RowBox[{"p", "+", "q"}], ",", 
          RowBox[{"r", "-", "p"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"(", "\[NoBreak]", 
        FormBox[
          RowBox[{"p", "+", "q"}],
          "TraditionalForm"], "\[NoBreak]", ")", "\[NoBreak]", 
        "\[CenterDot]", "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          RowBox[{"r", "-", "p"}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"Momentum", "[", "p", "]"}], "+", 
          RowBox[{"Momentum", "[", "q", "]"}]}], ",", 
        RowBox[{
          RowBox[{"-", 
            RowBox[{"Momentum", "[", "p", "]"}]}], "+", 
          RowBox[{"Momentum", "[", "r", "]"}]}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"]}], "-", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}], "+", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["r",
            "TraditionalForm"]}], "+", 
        RowBox[{
          FormBox["q",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["r",
            "TraditionalForm"]}]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Momentum", "Subsection",
  CellTags->"Momentum"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Momentum[p] is the head of a four momentum (p). The internal \
representation of a four-dimensional p is Momentum[p]. For other than four \
dimensions: Momentum[p, dim]. Momentum[p, 4] simplifies to Momentum[p].\
\>", 
  "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracGamma",
    ButtonData:>"DiracGamma",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracGamma"],
  ", ",
  ButtonBox["Eps",
    ButtonData:>"Eps",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Eps"],
  ", ",
  ButtonBox["LorentzIndex",
    ButtonData:>"LorentzIndex",
    ButtonStyle->"Hyperlink",
    ButtonNote->"LorentzIndex"],
  ", ",
  ButtonBox["MomentumExpand",
    ButtonData:>"MomentumExpand",
    ButtonStyle->"Hyperlink",
    ButtonNote->"MomentumExpand"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell["This is a four dimensional momentum.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Momentum", "[", "p", "]"}]], "Input"],

Cell[BoxData[
    FormBox["p", TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
As an optional second argument the dimension must be specified if \
it is different from 4.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Momentum", "[", 
      RowBox[{"p", ",", "D"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["p", TraditionalForm]], "Output"]
}, Open  ]],

Cell["The dimension index is supressed in the output.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Momentum", "[", 
        RowBox[{"p", ",", "d"}], "]"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Momentum", "[", 
      RowBox[{"p", ",", "d"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a1", "=", 
      RowBox[{"Momentum", "[", 
        RowBox[{"-", "q"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", "q"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a1", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"-", 
      RowBox[{"Momentum", "[", "q", "]"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a2", " ", "=", " ", 
      RowBox[{
        RowBox[{"Momentum", "[", 
          RowBox[{"p", "-", "q"}], "]"}], " ", "+", " ", 
        RowBox[{"Momentum", "[", 
          RowBox[{"2", "q"}], "]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"p", "-", "q"}], ")"}], "+", 
        RowBox[{"2", " ", "q"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a2", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Momentum", "[", 
        RowBox[{"p", "-", "q"}], "]"}], "+", 
      RowBox[{"2", " ", 
        RowBox[{"Momentum", "[", "q", "]"}]}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"a2", "//", "MomentumExpand"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Momentum", "[", "p", "]"}], "+", 
      RowBox[{"Momentum", "[", "q", "]"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"a2", "//", "MomentumCombine"}], "//", "StandardForm"}]], 
  "Input"],

Cell[BoxData[
    RowBox[{"Momentum", "[", 
      RowBox[{"p", "+", "q"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"ChangeDimension", "[", 
        RowBox[{
          RowBox[{"Momentum", "[", "p", "]"}], ",", "d"}], "]"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Momentum", "[", 
      RowBox[{"p", ",", "d"}], "]"}]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"a1", ",", "a2"}], "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MT", "Subsection",
  CellTags->"MT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "MT[",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[Mu]", ",", " ", "\[Nu]"}], TraditionalForm]]],
  "] is the metric tensor in 4 dimensions."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynCalcExternal",
    ButtonData:>"FeynCalcExternal",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FeynCalcExternal"],
  ", ",
  ButtonBox["FCE",
    ButtonData:>"FCE",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FCE"],
  ", ",
  ButtonBox["FCI",
    ButtonData:>"FCI",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FCI"],
  ", ",
  ButtonBox["MetricTensor",
    ButtonData:>"MetricTensor",
    ButtonStyle->"Hyperlink",
    ButtonNote->"MetricTensor"],
  ", ",
  ButtonBox["MTD",
    ButtonData:>"MTD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"MTD"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"MT", "[", 
      RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["g", 
        RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", 
      RowBox[{
        RowBox[{"MT", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], " ", 
        RowBox[{"MT", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["4", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"MT", "[", 
        RowBox[{"a", ",", "b"}], "]"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"MT", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FCI", "[", 
        RowBox[{"MT", "[", 
          RowBox[{"a", ",", "b"}], "]"}], "]"}], "//", "StandardForm"}]], 
  "Input"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "a", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "b", "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FCE", "[", 
        RowBox[{"FCI", "[", 
          RowBox[{"MT", "[", 
            RowBox[{"a", ",", "b"}], "]"}], "]"}], "]"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"MT", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MTD", "Subsection",
  CellTags->"MTD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "MTD[",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[Mu]", ",", " ", "\[Nu]"}], TraditionalForm]]],
  "] is the metric tensor in D dimensions."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynCalcExternal",
    ButtonData:>"FeynCalcExternal",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FeynCalcExternal"],
  ", ",
  ButtonBox["FCE",
    ButtonData:>"FCE",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FCE"],
  ", ",
  ButtonBox["FCI",
    ButtonData:>"FCI",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FCI"],
  ", ",
  ButtonBox["MetricTensor",
    ButtonData:>"MetricTensor",
    ButtonStyle->"Hyperlink",
    ButtonNote->"MetricTensor"],
  ", ",
  ButtonBox["MT",
    ButtonData:>"MT",
    ButtonStyle->"Hyperlink",
    ButtonNote->"MT"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"MTD", "[", 
      RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["g", 
        RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", 
      RowBox[{
        RowBox[{"MTD", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], " ", 
        RowBox[{"MTD", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["D", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"MTD", "[", 
        RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], "//", "StandardForm"}]], 
  "Input"],

Cell[BoxData[
    RowBox[{"MTD", "[", 
      RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FCI", "[", 
        RowBox[{"MTD", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], "]"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Alpha]", ",", "D"}], "]"}], ",", 
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Beta]", ",", "D"}], "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FCE", "[", 
        RowBox[{"FCI", "[", 
          RowBox[{"MTD", "[", 
            RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "]"}], "]"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"MTD", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Nielsen", "Subsection",
  CellTags->"Nielsen"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["Nielsen[i,j, x] denotes Nielsen's polylogarithm.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Nielsen", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"PolyLog", "\[Rule]", "False"}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["SimplifyPolyLog",
    ButtonData:>"SimplifyPolyLog",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SimplifyPolyLog"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Nielsen", "[", 
      RowBox[{"1", ",", "2", ",", "x"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["S", 
          RowBox[{
            FormBox["1",
              "TraditionalForm"], "\[NoBreak]", 
            FormBox["2",
              "TraditionalForm"]}]], "(", "x", ")"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell["\<\
Numerical evaluation is done via N[Nielsen[n_,p_,x_]] := \
(-1)^(n+p-1)/(n-1)!/p! NIntegrate[Log[1-x t]^p Log[t]^(n-1)/t,{t,0,1}];\
\>", 
  "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"N", "[", 
      RowBox[{"Nielsen", "[", 
        RowBox[{"1", ",", "2", ",", ".45"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["0.0728716261210464733`", TraditionalForm]], "Output"]
}, Open  ]],

Cell["Some special values are built in.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"Nielsen", "[", 
          RowBox[{"1", ",", "2", ",", "0"}], "]"}], ",", 
        RowBox[{"Nielsen", "[", 
          RowBox[{"1", ",", "2", ",", 
            RowBox[{"-", "1"}]}], "]"}], ",", 
        RowBox[{"Nielsen", "[", 
          RowBox[{"1", ",", "2", ",", 
            RowBox[{"1", "/", "2"}]}], "]"}], ",", 
        RowBox[{"Nielsen", "[", 
          RowBox[{"1", ",", "2", ",", "1"}], "]"}]}], "}"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"0", ",", 
          FractionBox[
            TagBox[
              RowBox[{"\[Zeta]", "(", 
                TagBox["3",
                  (Editable -> True)], ")"}],
              InterpretTemplate[ Zeta[ #]&]], "8"], ",", 
          FractionBox[
            TagBox[
              RowBox[{"\[Zeta]", "(", 
                TagBox["3",
                  (Editable -> True)], ")"}],
              InterpretTemplate[ Zeta[ #]&]], "8"], ",", 
          TagBox[
            RowBox[{"\[Zeta]", "(", 
              TagBox["3",
                (Editable -> True)], ")"}],
            InterpretTemplate[ Zeta[ #]&]]}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Nielsen", "[", 
      RowBox[{"1", ",", "2", ",", "x", ",", 
        RowBox[{"PolyLog", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"log", "(", "x", ")"}], " ", 
          RowBox[{
            SuperscriptBox["log", "2"], "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{
            SubscriptBox["Li", "2"], "(", 
            RowBox[{"1", "-", "x"}], ")"}], " ", 
          RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "-", 
        RowBox[{
          SubscriptBox["Li", "3"], "(", 
          RowBox[{"1", "-", "x"}], ")"}], "+", 
        TagBox[
          RowBox[{"\[Zeta]", "(", 
            TagBox["3",
              (Editable -> True)], ")"}],
          InterpretTemplate[ Zeta[ #]&]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Nielsen", "[", 
      RowBox[{"1", ",", "3", ",", "x", ",", 
        RowBox[{"PolyLog", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            FractionBox["1", "6"]}], " ", 
          RowBox[{"log", "(", "x", ")"}], " ", 
          RowBox[{
            SuperscriptBox["log", "3"], "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{
            SubscriptBox["Li", "2"], "(", 
            RowBox[{"1", "-", "x"}], ")"}], " ", 
          RowBox[{
            SuperscriptBox["log", "2"], "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{
            SubscriptBox["Li", "3"], "(", 
            RowBox[{"1", "-", "x"}], ")"}], " ", 
          RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}]}], "-", 
        RowBox[{
          SubscriptBox["Li", "4"], "(", 
          RowBox[{"1", "-", "x"}], ")"}], "+", 
        FractionBox[
          SuperscriptBox["\[Pi]", "4"], "90"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Nielsen", "[", 
      RowBox[{"3", ",", "1", ",", "x", ",", 
        RowBox[{"PolyLog", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["Li", "4"], "(", "x", ")"}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["NTerms", "Subsection",
  CellTags->"NTerms"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
NTerms[x] is equivalent to Length if x is a sum; otherwise \
NTerms[x] returns 1, except NTerms[0] -> 0.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"NTerms", "[", 
      RowBox[{"a", "-", "b"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["2", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"NTerms", "[", 
      RowBox[{"a", " ", "b", " ", "c"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"NTerms", "[", "9", "]"}]], "Input"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"NTerms", "[", "0", "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["OneLoop", "Subsection",
  CellTags->"OneLoop"],

Cell[TextData[{
  "Documentation unfinished ....; see also the old FeynCalc documentation at \
",
  ButtonBox["http://www.mertig.com/oldfc",
    ButtonData:>{
      URL[ "http://www.mertig.com/oldfc"], None},
    ButtonStyle->"Hyperlink"],
  ". NOTICE: While OneLoop is restricted to 't Hooft Feynman gauge the \
function ",
  ButtonBox["OneLoopSimplify",
    ButtonData:>"OneLoopSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"OneLoopSimplify"],
  " does not have this restriction (but is usually slower). OneLoop handles \
selfenergies, vertex and box-graphs (those only up to 3rd rank tensor in the \
integration variable)."
}], "Text"],

Cell["WARNING: If you encounter anomalies:", "Text"],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
          "The", " ", "default", " ", "setting", " ", "of", " ", "$West", 
            " ", "is", " ", "True"}], ",", " ", 
          RowBox[{"i", ".", "e", "."}], ",", " ", 
          RowBox[{"the", " ", "way", " ", 
            RowBox[{"tr", "(", 
              RowBox[{
                SuperscriptBox["\[Gamma]", "\[Mu]"], 
                SuperscriptBox["\[Gamma]", "\[Nu]"], 
                SuperscriptBox["\[Gamma]", "\[Rho]"], 
                SuperscriptBox["\[Gamma]", "\[Sigma]"], 
                SuperscriptBox["\[Gamma]", "\[Tau]"], 
                SuperscriptBox["\[Gamma]", "\[Lambda]"], 
                SuperscriptBox["\[Gamma]", "5"]}], ")"}]}]}], 
        TraditionalForm]]],
  " ",
  "is calculated in D dimensions has changed compared to the old FeynCalc \
version. Please keep in mind that the issue of ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
  "schemes is inherintly tricky."
}], "Text"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
OneLoop[q, amplitude] calculates the one-loop Feynman diagram \
amplitude (n-point, where n<=4 and the highest tensor rank of the integration \
momenta (after cancellation of scalar products) may be 3; unless \
OneLoopSimplify is used).\
\>", "Text"],

Cell["\<\
The argument q denotes the integration variable, i.e., the loop \
momentum. OneLoop[name, q, amplitude] has as first argument a name of the \
amplitude. If the second argument has head FeynAmp then OneLoop[q, \
FeynAmp[name, k, expr]] and OneLoop[FeynAmp[name, k, expr]] tranform to \
OneLoop[name, k, expr].\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "OneLoop", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Apart2", "\[Rule]", "True"}], ",", 
          RowBox[{"CancelQP", "\[Rule]", "True"}], ",", 
          RowBox[{"DenominatorOrder", "\[Rule]", "False"}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"FinalSubstitutions", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
          RowBox[{"FormatType", "\[Rule]", "InputForm"}], ",", 
          RowBox[{"InitialSubstitutions", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"IntermediateSubstitutions", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"IsolateHead", "\[Rule]", "False"}], ",", 
          RowBox[{"Mandelstam", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"OneLoopSimplify", "\[Rule]", "False"}], ",", 
          RowBox[{"Prefactor", "\[Rule]", "1"}], ",", 
          RowBox[{"ReduceGamma", "\[Rule]", "False"}], ",", 
          RowBox[{"ReduceToScalars", "\[Rule]", "False"}], ",", 
          RowBox[{"SmallVariables", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"WriteOut", "\[Rule]", "False"}], ",", 
          RowBox[{"WriteOutPaVe", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["B0",
    ButtonData:>"B0",
    ButtonStyle->"Hyperlink",
    ButtonNote->"B0"],
  ", ",
  ButtonBox["C0",
    ButtonData:>"C0",
    ButtonStyle->"Hyperlink",
    ButtonNote->"C0"],
  ", ",
  ButtonBox["D0",
    ButtonData:>"D0",
    ButtonStyle->"Hyperlink",
    ButtonNote->"D0"],
  ", ",
  ButtonBox["OneLoopSimplify",
    ButtonData:>"OneLoopSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"OneLoopSimplify"],
  ", ",
  ButtonBox["TID",
    ButtonData:>"TID",
    ButtonStyle->"Hyperlink",
    ButtonNote->"TID"],
  ", ",
  ButtonBox["TIDL",
    ButtonData:>"TIDL",
    ButtonStyle->"Hyperlink",
    ButtonNote->"TIDL"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"-", "I"}], "/", 
          RowBox[{"Pi", "^", "2"}]}], " ", 
        RowBox[{"FAD", "[", 
          RowBox[{"{", 
            RowBox[{"q", ",", "m"}], "}"}], "]"}]}], "//", "FCI"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox["\[ImaginaryI]", 
          RowBox[{
            SuperscriptBox["\[Pi]", "2"], " ", 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox["q", "2"], "-", 
                SuperscriptBox["m", "2"]}], ")"}]}]]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OneLoop", "[", 
      RowBox[{"q", ",", "%"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{
            SubscriptBox["B", "0"], "(", "0", ",", 
            SuperscriptBox["m", "2"], ",", 
            SuperscriptBox["m", "2"], ")"}], " ", 
          SuperscriptBox["m", "2"]}], "+", 
        SuperscriptBox["m", "2"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"mf", "/:", 
        RowBox[{"MakeBoxes", "[", 
          RowBox[{"mf", ",", "TraditionalForm"}], "]"}], "=", 
        RowBox[{"InterpretationBox", "[", 
          RowBox[{
            RowBox[{"SubscriptBox", "[", 
              RowBox[{"\"\<m\>\"", ",", "\"\<f\>\""}], "]"}], ",", "mf"}], 
          "]"}]}], ";"}]], "Input"],

Cell[TextData[{
  "Remember that FAD[{q,mf},{q-k,mf}] is a fast possibility to enter ",
  Cell[BoxData[
      FormBox[
        RowBox[{"1", "/", 
          RowBox[{
            RowBox[{"(", " ", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox["q", "2"], "-", 
                    SuperscriptBox["mf", "2"]}], ")"}], " ", 
                RowBox[{"(", " ", 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"q", "-", "k"}], ")"}], "2"], "-", 
                    SuperscriptBox["mf", "2"]}], ")"}]}], " ", ")"}], 
            "."}]}], TraditionalForm]]]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{
        RowBox[{"I", " ", 
          FractionBox[
            SuperscriptBox["el", "2"], 
            RowBox[{"16", " ", 
              SuperscriptBox["Pi", "4"], 
              RowBox[{"(", 
                RowBox[{"1", "-", "D"}], ")"}]}]], 
          RowBox[{"FAD", "[", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"q", ",", "mf"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{
                  RowBox[{"q", "-", "k"}], ",", "mf"}], "}"}]}], "]"}], 
          RowBox[{"DiracTrace", "[", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"mf", "+", 
                  RowBox[{"GS", "[", 
                    RowBox[{"q", "-", "k"}], "]"}]}], ")"}], ".", 
              RowBox[{"GA", "[", "\[Mu]", "]"}], ".", 
              RowBox[{"(", 
                RowBox[{"mf", "+", 
                  RowBox[{"GS", "[", "q", "]"}]}], ")"}], ".", 
              RowBox[{"GA", "[", "\[Mu]", "]"}]}], "]"}]}], " ", "//", " ", 
        "FCI"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"\[ImaginaryI]", " ", 
          SuperscriptBox["el", "2"], " ", 
          RowBox[{"tr", "(", 
            FormBox[
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    InterpretationBox[
                      SubscriptBox["m", "f"],
                      mf], "+", 
                    RowBox[{
                      FormBox["\<\"\[Gamma]\"\>",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", "(", "\[NoBreak]", 
                      FormBox[
                        RowBox[{"q", "-", "k"}],
                        "TraditionalForm"], "\[NoBreak]", ")"}]}], ")"}], 
                ".", 
                SuperscriptBox["\[Gamma]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], ".", 
                RowBox[{"(", 
                  RowBox[{
                    InterpretationBox[
                      SubscriptBox["m", "f"],
                      mf], "+", 
                    RowBox[{
                      FormBox["\<\"\[Gamma]\"\>",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["q",
                        "TraditionalForm"]}]}], ")"}], ".", 
                SuperscriptBox["\[Gamma]", 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}],
              "TraditionalForm"], ")"}]}], 
        RowBox[{"16", " ", 
          RowBox[{"(", 
            RowBox[{"1", "-", "D"}], ")"}], " ", 
          SuperscriptBox["\[Pi]", "4"], " ", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox["q", "2"], "-", 
                SuperscriptBox[
                  InterpretationBox[
                    SubscriptBox["m", "f"],
                    mf], "2"]}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"q", "-", "k"}], ")"}], "2"], "-", 
                SuperscriptBox[
                  InterpretationBox[
                    SubscriptBox["m", "f"],
                    mf], "2"]}], ")"}]}]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
The input to OneLoop may be in 4 dimensions, since the function \
changes the dimension of the objects automatically to the setting of the \
Dimension option (D by default).\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OneLoop", "[", 
      RowBox[{"q", ",", "t"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          RowBox[{
            SuperscriptBox["el", "2"], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FractionBox["8", "3"], " ", 
                  RowBox[{
                    SubscriptBox["B", "0"], "(", "0", ",", 
                    SuperscriptBox[
                      InterpretationBox[
                        SubscriptBox["m", "f"],
                        mf], "2"], ",", 
                    SuperscriptBox[
                      InterpretationBox[
                        SubscriptBox["m", "f"],
                        mf], "2"], ")"}], " ", 
                  SuperscriptBox[
                    InterpretationBox[
                      SubscriptBox["m", "f"],
                      mf], "2"]}], "-", 
                RowBox[{
                  FractionBox["8", "3"], " ", 
                  RowBox[{
                    SubscriptBox["B", "0"], "(", 
                    SuperscriptBox[
                      FormBox["k",
                        "TraditionalForm"], "2"], ",", 
                    SuperscriptBox[
                      InterpretationBox[
                        SubscriptBox["m", "f"],
                        mf], "2"], ",", 
                    SuperscriptBox[
                      InterpretationBox[
                        SubscriptBox["m", "f"],
                        mf], "2"], ")"}], " ", 
                  SuperscriptBox[
                    InterpretationBox[
                      SubscriptBox["m", "f"],
                      mf], "2"]}], "-", 
                RowBox[{
                  FractionBox["4", "3"], " ", 
                  RowBox[{
                    SubscriptBox["B", "0"], "(", 
                    SuperscriptBox[
                      FormBox["k",
                        "TraditionalForm"], "2"], ",", 
                    SuperscriptBox[
                      InterpretationBox[
                        SubscriptBox["m", "f"],
                        mf], "2"], ",", 
                    SuperscriptBox[
                      InterpretationBox[
                        SubscriptBox["m", "f"],
                        mf], "2"], ")"}], " ", 
                  SuperscriptBox[
                    FormBox["k",
                      "TraditionalForm"], "2"]}], "+", 
                FractionBox[
                  RowBox[{"4", " ", 
                    SuperscriptBox[
                      FormBox["k",
                        "TraditionalForm"], "2"]}], "9"]}], ")"}]}], 
          RowBox[{"16", " ", 
            SuperscriptBox["\[Pi]", "2"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FullSimplify", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          RowBox[{
            SuperscriptBox["el", "2"], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"6", " ", 
                  RowBox[{
                    SubscriptBox["B", "0"], "(", "0", ",", 
                    SuperscriptBox[
                      InterpretationBox[
                        SubscriptBox["m", "f"],
                        mf], "2"], ",", 
                    SuperscriptBox[
                      InterpretationBox[
                        SubscriptBox["m", "f"],
                        mf], "2"], ")"}], " ", 
                  SuperscriptBox[
                    InterpretationBox[
                      SubscriptBox["m", "f"],
                      mf], "2"]}], "+", 
                SuperscriptBox[
                  FormBox["k",
                    "TraditionalForm"], "2"], "-", 
                RowBox[{"3", " ", 
                  RowBox[{
                    SubscriptBox["B", "0"], "(", 
                    SuperscriptBox[
                      FormBox["k",
                        "TraditionalForm"], "2"], ",", 
                    SuperscriptBox[
                      InterpretationBox[
                        SubscriptBox["m", "f"],
                        mf], "2"], ",", 
                    SuperscriptBox[
                      InterpretationBox[
                        SubscriptBox["m", "f"],
                        mf], "2"], ")"}], " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"2", " ", 
                        SuperscriptBox[
                          InterpretationBox[
                            SubscriptBox["m", "f"],
                            mf], "2"]}], "+", 
                      SuperscriptBox[
                        FormBox["k",
                          "TraditionalForm"], "2"]}], ")"}]}]}], ")"}]}], 
          RowBox[{"36", " ", 
            SuperscriptBox["\[Pi]", "2"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SP", "[", 
          RowBox[{"k", ",", "r"}], "]"}], " ", 
        RowBox[{"FAD", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"k", ",", "m"}], "}"}], " ", ",", " ", 
            RowBox[{"k", " ", "-", " ", "p"}]}], "]"}]}], "//", "FCI"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          FormBox["k",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["r",
            "TraditionalForm"]}], 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox["k", "2"], "-", 
              SuperscriptBox["m", "2"]}], ")"}], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"k", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OneLoop", "[", 
      RowBox[{"k", ",", "%"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[ImaginaryI]", " ", 
        SuperscriptBox["\[Pi]", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"-", 
              FractionBox[
                RowBox[{
                  RowBox[{
                    SubscriptBox["B", "0"], "(", "0", ",", 
                    SuperscriptBox["m", "2"], ",", 
                    SuperscriptBox["m", "2"], ")"}], " ", 
                  RowBox[{
                    FormBox["p",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["r",
                      "TraditionalForm"]}], " ", 
                  SuperscriptBox["m", "2"]}], 
                RowBox[{"2", " ", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"]}]]}], "+", 
            FractionBox[
              RowBox[{
                RowBox[{
                  SubscriptBox["B", "0"], "(", 
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"], ",", "0", ",", 
                  SuperscriptBox["m", "2"], ")"}], " ", 
                RowBox[{
                  FormBox["p",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}], " ", 
                SuperscriptBox["m", "2"]}], 
              RowBox[{"2", " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"]}]], "-", 
            FractionBox[
              RowBox[{
                RowBox[{
                  FormBox["p",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}], " ", 
                SuperscriptBox["m", "2"]}], 
              RowBox[{"2", " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"]}]], "+", 
            RowBox[{
              FractionBox["1", "2"], " ", 
              RowBox[{
                SubscriptBox["B", "0"], "(", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"], ",", "0", ",", 
                SuperscriptBox["m", "2"], ")"}], " ", 
              RowBox[{
                FormBox["p",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["r",
                  "TraditionalForm"]}]}]}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", "t", "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["OneLoopSimplify", "Subsection",
  CellTags->"OneLoopSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
OneLoopSimplify[amp, q] simplifies the one-loop amplitude amp. The \
second argument denotes the integration momentum.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "OneLoopSimplify", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Collecting", "\[Rule]", "False"}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"DimensionalReduction", "\[Rule]", "False"}], ",", 
          RowBox[{"DiracSimplify", "\[Rule]", "True"}], ",", 
          RowBox[{"FinalSubstitutions", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"IntegralTable", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"OPE1Loop", "\[Rule]", "False"}], ",", 
          RowBox[{"ScalarProductCancel", "\[Rule]", "True"}], ",", 
          RowBox[{"SUNNToCACF", "\[Rule]", "True"}], ",", 
          RowBox[{"SUNTrace", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OneLoop",
    ButtonData:>"OneLoop",
    ButtonStyle->"Hyperlink",
    ButtonNote->"OneLoop"],
  ", ",
  ButtonBox["TID",
    ButtonData:>"TID",
    ButtonStyle->"Hyperlink",
    ButtonNote->"TID"],
  ", ",
  ButtonBox["TIDL",
    ButtonData:>"TIDL",
    ButtonStyle->"Hyperlink",
    ButtonNote->"TIDL"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SP", "[", 
          RowBox[{"k", ",", "r"}], "]"}], " ", 
        RowBox[{"FAD", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"k", ",", "m"}], "}"}], " ", ",", " ", 
            RowBox[{"k", " ", "-", " ", "p"}]}], "]"}]}], "//", "FCI"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          FormBox["k",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["r",
            "TraditionalForm"]}], 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox["k", "2"], "-", 
              SuperscriptBox["m", "2"]}], ")"}], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"k", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OneLoopSimplify", "[", 
      RowBox[{"%", ",", "k"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"HighEnergyPhysics`fctools`Contract3", "(", 
        FractionBox[
          RowBox[{
            FormBox["k",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["r",
              "TraditionalForm"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox["k", "2"], "-", 
                SuperscriptBox["m", "2"]}], ")"}], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"k", "-", "p"}], ")"}], "2"]}]], ")"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OneLoopSimplify", "[", 
      RowBox[{
        RowBox[{"%", "/.", 
          RowBox[{"m", "\[Rule]", "0"}]}], ",", "k"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"HighEnergyPhysics`fctools`Contract3", "(", 
        RowBox[{"HighEnergyPhysics`fctools`Contract3", "(", 
          FractionBox[
            RowBox[{
              FormBox["k",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["r",
                "TraditionalForm"]}], 
            RowBox[{
              SuperscriptBox["k", "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"k", "-", "p"}], ")"}], "2"]}]], ")"}], ")"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"FAD", "[", 
          RowBox[{"k", ",", "k", ",", " ", 
            RowBox[{"k", " ", "-", " ", 
              SubscriptBox["p", "1"]}], ",", " ", 
            RowBox[{"k", " ", "-", " ", 
              SubscriptBox["p", "2"]}]}], "]"}], " ", 
        RowBox[{"FVD", "[", 
          RowBox[{"k", ",", "\[Mu]"}], "]"}]}], "//", "FCI"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        SuperscriptBox[
          FormBox["k",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], 
        RowBox[{
          SuperscriptBox["k", "2"], ".", 
          SuperscriptBox["k", "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"k", "-", 
                SubscriptBox["p", "1"]}], ")"}], "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"k", "-", 
                SubscriptBox["p", "2"]}], ")"}], "2"]}]], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OneLoopSimplify", "[", " ", 
      RowBox[{"%", ",", "k"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"HighEnergyPhysics`fctools`Contract3", "(", 
        FractionBox[
          SuperscriptBox[
            FormBox["k",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          RowBox[{
            SuperscriptBox["k", "2"], ".", 
            SuperscriptBox["k", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"k", "-", 
                  SubscriptBox["p", "1"]}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"k", "-", 
                  SubscriptBox["p", "2"]}], ")"}], "2"]}]], ")"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"FCE", "[", "%", "]"}], "/.", 
        RowBox[{
          RowBox[{"SPD", "[", 
            SubscriptBox["p", "1"], "]"}], "\[Rule]", "0"}]}], "//", 
      "FCI"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"HighEnergyPhysics`fctools`Contract3", "(", 
        FractionBox[
          SuperscriptBox[
            FormBox["k",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          RowBox[{
            SuperscriptBox["k", "2"], ".", 
            SuperscriptBox["k", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"k", "-", 
                  SubscriptBox["p", "1"]}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"k", "-", 
                  SubscriptBox["p", "2"]}], ")"}], "2"]}]], ")"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OneLoopSimplify", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"FAD", "[", 
            RowBox[{
              RowBox[{"k", "-", 
                SubscriptBox["p", "1"]}], ",", 
              RowBox[{"k", "-", 
                SubscriptBox["p", "2"]}]}], "]"}], " ", 
          RowBox[{
            RowBox[{"SP", "[", 
              RowBox[{"k", ",", "l"}], "]"}], "^", "2"}]}], ",", "k"}], 
      "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"HighEnergyPhysics`fctools`Contract3", "(", 
        RowBox[{
          FractionBox[
            SuperscriptBox[
              RowBox[{"(", 
                FormBox[
                  RowBox[{
                    FormBox["k",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["l",
                      "TraditionalForm"]}],
                  "TraditionalForm"], ")"}], "2"], 
            RowBox[{
              SuperscriptBox["k", "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"k", "-", 
                    SubscriptBox["p", "1"], "+", 
                    SubscriptBox["p", "2"]}], ")"}], "2"]}]], "+", 
          FractionBox[
            RowBox[{"2", " ", 
              RowBox[{
                FormBox["k",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["l",
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox["l",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["p", "2"],
                  "TraditionalForm"]}]}], 
            RowBox[{
              SuperscriptBox["k", "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"k", "-", 
                    SubscriptBox["p", "1"], "+", 
                    SubscriptBox["p", "2"]}], ")"}], "2"]}]], "+", 
          FractionBox[
            SuperscriptBox[
              RowBox[{"(", 
                FormBox[
                  RowBox[{
                    FormBox["l",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox[
                      SubscriptBox["p", "2"],
                      "TraditionalForm"]}],
                  "TraditionalForm"], ")"}], "2"], 
            RowBox[{
              SuperscriptBox["k", "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"k", "-", 
                    SubscriptBox["p", "1"], "+", 
                    SubscriptBox["p", "2"]}], ")"}], "2"]}]]}], ")"}], 
      TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["OPEDelta", "Subsection",
  CellTags->"OPEDelta"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
OPEDelta is a lightlike axial vector as used e.g. in the operator \
product expansion in QCD.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Twist2QuarkOperator",
    ButtonData:>"Twist2QuarkOperator",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Twist2QuarkOperator"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FourVector", "[", 
      RowBox[{"OPEDelta", ",", "\[Mu]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubscriptBox[
        FormBox["\[CapitalDelta]",
          "TraditionalForm"], "\[Mu]"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", 
      RowBox[{"%", " ", "%"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SP", "[", 
      RowBox[{"OPEDelta", ",", "OPEDelta"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["OPEi, OPEj, OPEk, OPEl, OPEm, OPEn, OPEo", "Subsection",
  CellTags->"OPEi"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
OPEi, etc. are variables with DataType PositiveInteger which are \
used in functions relating to the operator product expansion.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData["OPEi"], "Input"],

Cell[BoxData[
    FormBox["i", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DataType", "[", 
      RowBox[{
      "OPEi", ",", " ", "OPEj", ",", "OPEk", ",", "OPEl", ",", " ", "OPEm", 
        ",", " ", "OPEn", ",", " ", "OPEo", ",", " ", "PositiveInteger"}], 
      "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
        "True", ",", "True", ",", "True", ",", "True", ",", "True", ",", 
          "True", ",", "True"}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PowerSimplify", "[", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "^", 
            RowBox[{"(", 
              RowBox[{"2", "OPEi"}], ")"}]}], ",", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "^", 
            RowBox[{"(", 
              RowBox[{"2", "OPEj"}], ")"}]}], ",", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "^", 
            RowBox[{"(", 
              RowBox[{"2", "OPEk"}], ")"}]}], ",", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "^", 
            RowBox[{"(", 
              RowBox[{"2", "OPEl"}], ")"}]}], ",", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "^", 
            RowBox[{"(", 
              RowBox[{"2", "OPEm"}], ")"}]}], ",", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "^", 
            RowBox[{"(", 
              RowBox[{"2", "OPEn"}], ")"}]}], ",", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "^", 
            RowBox[{"(", 
              RowBox[{"2", "OPEo"}], ")"}]}]}], "}"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
        "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1"}], 
        "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["Re has been changed:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{
          RowBox[{"Re", "[", "OPEi", "]"}], ">", 
          RowBox[{"-", "3"}]}], ",", " ", 
        RowBox[{
          RowBox[{"Re", "[", "OPEi", "]"}], ">", 
          RowBox[{"-", "2"}]}], ",", " ", 
        RowBox[{
          RowBox[{"Re", "[", "OPEi", "]"}], ">", 
          RowBox[{"-", "1"}]}], ",", " ", " ", " ", 
        RowBox[{
          RowBox[{"Re", "[", "OPEi", "]"}], ">", "0"}], ",", " ", 
        RowBox[{
          RowBox[{"Re", "[", "OPEi", "]"}], ">", "1"}]}], "}"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"True", ",", "True", ",", "True", ",", "True", ",", "True"}], 
        "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{
          RowBox[{"Re", "[", 
            RowBox[{
              RowBox[{"-", "OPEi"}], " ", "+", " ", "OPEm"}], "]"}], " ", 
          ">", " ", "0"}], ",", " ", 
        RowBox[{
          RowBox[{"Re", "[", 
            RowBox[{
              RowBox[{"-", "OPEi"}], " ", "+", " ", "OPEm"}], "]"}], " ", 
          ">", " ", "1"}], ",", 
        RowBox[{
          RowBox[{"Re", "[", 
            RowBox[{
              RowBox[{"-", "OPEi"}], " ", "+", " ", "OPEm"}], "]"}], " ", 
          ">", " ", "2"}]}], "}"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"True", ",", "True", ",", "True"}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{
          RowBox[{"Re", "[", "OPEm", "]"}], ">", 
          RowBox[{"-", "3"}]}], ",", " ", 
        RowBox[{
          RowBox[{"Re", "[", "OPEm", "]"}], ">", 
          RowBox[{"-", "2"}]}], ",", " ", 
        RowBox[{
          RowBox[{"Re", "[", "OPEm", "]"}], ">", 
          RowBox[{"-", "1"}]}], ",", " ", " ", " ", 
        RowBox[{
          RowBox[{"Re", "[", "OPEm", "]"}], ">", "0"}], ",", " ", 
        RowBox[{
          RowBox[{"Re", "[", "OPEm", "]"}], ">", "1"}]}], "}"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"True", ",", "True", ",", "True", ",", "True", ",", "True"}], 
        "}"}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["OPESumExplicit", "Subsection",
  CellTags->"OPESumExplicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["OPESumExplicit[exp] calculates OPESum's.", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OPESum",
    ButtonData:>"OPESum",
    ButtonStyle->"Hyperlink",
    ButtonNote->"OPESum"],
  ", ",
  ButtonBox["OPESumSimplify",
    ButtonData:>"OPESumSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"OPESumSimplify"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{"OPESum", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"A", "^", "i"}], 
            RowBox[{"B", "^", 
              RowBox[{"(", 
                RowBox[{"m", "-", "i", "-", "3"}], ")"}]}]}], ",", 
          RowBox[{"{", 
            RowBox[{"i", ",", "0", ",", 
              RowBox[{"m", "-", "3"}]}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      TagBox[
        RowBox[{
          UnderoverscriptBox["\[Sum]", 
            RowBox[{"i", "=", "0"}], 
            RowBox[{"m", "-", "3"}]], 
          RowBox[{
            SuperscriptBox["A", "i"], " ", 
            SuperscriptBox["B", 
              RowBox[{
                RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
        HoldForm], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OPESumExplicit", "[", "t1", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          FormBox[
            SuperscriptBox["A", 
              RowBox[{"m", "-", "2"}]],
            "TraditionalForm"], 
          RowBox[{"A", "-", "B"}]], "-", 
        FractionBox[
          FormBox[
            SuperscriptBox["B", 
              RowBox[{"m", "-", "2"}]],
            "TraditionalForm"], 
          RowBox[{"A", "-", "B"}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", " ", "=", " ", 
      RowBox[{"OPESum", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"a", "^", "i"}], 
            RowBox[{"b", "^", 
              RowBox[{"(", 
                RowBox[{"j", "-", "i"}], ")"}]}], 
            RowBox[{"c", "^", 
              RowBox[{"(", 
                RowBox[{"m", "-", "j", "-", "4"}], ")"}]}]}], ",", 
          RowBox[{"{", 
            RowBox[{"i", ",", "0", ",", "j"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"j", ",", "0", ",", 
              RowBox[{"m", "-", "4"}]}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          TagBox[
            RowBox[{
              UnderoverscriptBox["\[Sum]", 
                RowBox[{"j", "=", "0"}], 
                RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
            HoldForm],
          "TraditionalForm"], "\[NoBreak]", 
        FormBox[
          RowBox[{
            UnderoverscriptBox["\[Sum]", 
              RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
          "TraditionalForm"], "\[NoBreak]", 
        FormBox[
          RowBox[{
            SuperscriptBox["a", "i"], " ", 
            SuperscriptBox["b", 
              RowBox[{"j", "-", "i"}]], " ", 
            SuperscriptBox["c", 
              RowBox[{
                RowBox[{"-", "j"}], "+", "m", "-", "4"}]]}],
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OPESumExplicit", "[", "t2", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox[
            RowBox[{"c", " ", 
              FormBox[
                SuperscriptBox["a", 
                  RowBox[{"m", "-", "2"}]],
                "TraditionalForm"]}], 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"a", "-", "b"}], ")"}], " ", 
              RowBox[{"(", 
                RowBox[{"a", "-", "c"}], ")"}], " ", 
              RowBox[{"(", 
                RowBox[{"b", "-", "c"}], ")"}]}]]}], "+", 
        FractionBox[
          RowBox[{"b", " ", 
            FormBox[
              SuperscriptBox["a", 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "b"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"a", "-", "c"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"b", "-", "c"}], ")"}]}]], "+", 
        FractionBox[
          RowBox[{"c", " ", 
            FormBox[
              SuperscriptBox["b", 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "b"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"a", "-", "c"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"b", "-", "c"}], ")"}]}]], "-", 
        FractionBox[
          RowBox[{"a", " ", 
            FormBox[
              SuperscriptBox["b", 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "b"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"a", "-", "c"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"b", "-", "c"}], ")"}]}]], "-", 
        FractionBox[
          RowBox[{"b", " ", 
            FormBox[
              SuperscriptBox["c", 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "b"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"a", "-", "c"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"b", "-", "c"}], ")"}]}]], "+", 
        FractionBox[
          RowBox[{"a", " ", 
            FormBox[
              SuperscriptBox["c", 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "b"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"a", "-", "c"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"b", "-", "c"}], ")"}]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", 
        RowBox[{"t1", ",", "t2"}], "]"}], ";"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["OPESum", "Subsection",
  CellTags->"OPESum"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
OPESum[exp, {i, 0, m}] denotes a symbolic sum.The syntax is the \
same as for Sum.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OPESumExplicit",
    ButtonData:>"OPESumExplicit",
    ButtonStyle->"Hyperlink",
    ButtonNote->"OPESumExplicit"],
  ", ",
  ButtonBox["OPESumSimplify",
    ButtonData:>"OPESumSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"OPESumSimplify"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{"OPESum", "[", 
        RowBox[{
          RowBox[{
            RowBox[{
              RowBox[{"SO", "[", "p", "]"}], "^", "OPEi"}], 
            RowBox[{
              RowBox[{"SO", "[", "k", "]"}], "^", 
              RowBox[{"(", 
                RowBox[{"OPEm", "-", "OPEi", "-", "3"}], ")"}]}]}], ",", 
          RowBox[{"{", 
            RowBox[{"OPEi", ",", "0", ",", 
              RowBox[{"OPEm", "-", "3"}]}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      TagBox[
        RowBox[{
          UnderoverscriptBox["\[Sum]", 
            RowBox[{"i", "=", "0"}], 
            RowBox[{"m", "-", "3"}]], 
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\<\"\[CapitalDelta]\"\>",
                    "TraditionalForm"], 
                  FormBox["\<\"\[CenterDot]\"\>",
                    "TraditionalForm"], 
                  FormBox["k",
                    "TraditionalForm"]}], ")"}], 
              RowBox[{
                RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\<\"\[CapitalDelta]\"\>",
                    "TraditionalForm"], 
                  FormBox["\<\"\[CenterDot]\"\>",
                    "TraditionalForm"], 
                  FormBox["p",
                    "TraditionalForm"]}], ")"}], "i"]}]}],
        HoldForm], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OPESumExplicit", "[", "t1", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          FormBox[
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\<\"\[CapitalDelta]\"\>",
                    "TraditionalForm"], 
                  FormBox["\<\"\[CenterDot]\"\>",
                    "TraditionalForm"], 
                  FormBox["k",
                    "TraditionalForm"]}], ")"}], 
              RowBox[{"m", "-", "2"}]],
            "TraditionalForm"], 
          RowBox[{
            RowBox[{
              FormBox["\<\"\[CapitalDelta]\"\>",
                "TraditionalForm"], 
              FormBox["\<\"\[CenterDot]\"\>",
                "TraditionalForm"], 
              FormBox["k",
                "TraditionalForm"]}], "-", 
            RowBox[{
              FormBox["\<\"\[CapitalDelta]\"\>",
                "TraditionalForm"], 
              FormBox["\<\"\[CenterDot]\"\>",
                "TraditionalForm"], 
              FormBox["p",
                "TraditionalForm"]}]}]], "-", 
        FractionBox[
          FormBox[
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\<\"\[CapitalDelta]\"\>",
                    "TraditionalForm"], 
                  FormBox["\<\"\[CenterDot]\"\>",
                    "TraditionalForm"], 
                  FormBox["p",
                    "TraditionalForm"]}], ")"}], 
              RowBox[{"m", "-", "2"}]],
            "TraditionalForm"], 
          RowBox[{
            RowBox[{
              FormBox["\<\"\[CapitalDelta]\"\>",
                "TraditionalForm"], 
              FormBox["\<\"\[CenterDot]\"\>",
                "TraditionalForm"], 
              FormBox["k",
                "TraditionalForm"]}], "-", 
            RowBox[{
              FormBox["\<\"\[CapitalDelta]\"\>",
                "TraditionalForm"], 
              FormBox["\<\"\[CenterDot]\"\>",
                "TraditionalForm"], 
              FormBox["p",
                "TraditionalForm"]}]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", " ", "=", " ", 
      RowBox[{"OPESum", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"a", "^", "i"}], 
            RowBox[{"b", "^", 
              RowBox[{"(", 
                RowBox[{"j", "-", "i"}], ")"}]}], 
            RowBox[{"c", "^", 
              RowBox[{"(", 
                RowBox[{"m", "-", "j", "-", "4"}], ")"}]}]}], ",", 
          RowBox[{"{", 
            RowBox[{"i", ",", "0", ",", "j"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"j", ",", "0", ",", 
              RowBox[{"m", "-", "4"}]}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          TagBox[
            RowBox[{
              UnderoverscriptBox["\[Sum]", 
                RowBox[{"j", "=", "0"}], 
                RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
            HoldForm],
          "TraditionalForm"], "\[NoBreak]", 
        FormBox[
          RowBox[{
            UnderoverscriptBox["\[Sum]", 
              RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
          "TraditionalForm"], "\[NoBreak]", 
        FormBox[
          RowBox[{
            SuperscriptBox["a", "i"], " ", 
            SuperscriptBox["b", 
              RowBox[{"j", "-", "i"}]], " ", 
            SuperscriptBox["c", 
              RowBox[{
                RowBox[{"-", "j"}], "+", "m", "-", "4"}]]}],
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OPESumExplicit", "[", "t2", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox[
            RowBox[{"c", " ", 
              FormBox[
                SuperscriptBox["a", 
                  RowBox[{"m", "-", "2"}]],
                "TraditionalForm"]}], 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"a", "-", "b"}], ")"}], " ", 
              RowBox[{"(", 
                RowBox[{"a", "-", "c"}], ")"}], " ", 
              RowBox[{"(", 
                RowBox[{"b", "-", "c"}], ")"}]}]]}], "+", 
        FractionBox[
          RowBox[{"b", " ", 
            FormBox[
              SuperscriptBox["a", 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "b"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"a", "-", "c"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"b", "-", "c"}], ")"}]}]], "+", 
        FractionBox[
          RowBox[{"c", " ", 
            FormBox[
              SuperscriptBox["b", 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "b"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"a", "-", "c"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"b", "-", "c"}], ")"}]}]], "-", 
        FractionBox[
          RowBox[{"a", " ", 
            FormBox[
              SuperscriptBox["b", 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "b"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"a", "-", "c"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"b", "-", "c"}], ")"}]}]], "-", 
        FractionBox[
          RowBox[{"b", " ", 
            FormBox[
              SuperscriptBox["c", 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "b"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"a", "-", "c"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"b", "-", "c"}], ")"}]}]], "+", 
        FractionBox[
          RowBox[{"a", " ", 
            FormBox[
              SuperscriptBox["c", 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"a", "-", "b"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"a", "-", "c"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{"b", "-", "c"}], ")"}]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{" ", 
      RowBox[{"Clear", "[", 
        RowBox[{"t1", ",", "t2"}], "]"}]}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["OPESumSimplify", "Subsection",
  CellTags->"OPESumSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["OPESumSimplify[exp] simplifies OPESum's in exp.", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OPESum",
    ButtonData:>"OPESum",
    ButtonStyle->"Hyperlink",
    ButtonNote->"OPESum"],
  ", ",
  ButtonBox["OPESumExplicit",
    ButtonData:>"OPESumExplicit",
    ButtonStyle->"Hyperlink",
    ButtonNote->"OPESumExplicit"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell["\<\
OPESum[(-SOD[p])^(OPEi+1) \
SOD[p-q]^(OPEm-OPEi-2),{OPEi,0,OPEm}]\
\>", "Input",
  FormatType->InputForm],

Cell[BoxData[
    FormBox[
      TagBox[
        RowBox[{
          UnderoverscriptBox["\[Sum]", 
            RowBox[{"i", "=", "0"}], "m"], 
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", 
                  RowBox[{"(", 
                    RowBox[{
                      FormBox["\<\"\[CapitalDelta]\"\>",
                        "TraditionalForm"], 
                      FormBox["\<\"\[CenterDot]\"\>",
                        "TraditionalForm"], 
                      FormBox["p",
                        "TraditionalForm"]}], ")"}]}], ")"}], 
              RowBox[{"i", "+", "1"}]], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\<\"\[CapitalDelta]\"\>",
                    "TraditionalForm"], 
                  FormBox["\<\"\[CenterDot]\"\>",
                    "TraditionalForm"], 
                  FormBox["\<\"(\"\>",
                    "TraditionalForm"], 
                  FormBox[
                    RowBox[{"p", "-", "q"}],
                    "TraditionalForm"], 
                  FormBox["\<\")\"\>",
                    "TraditionalForm"]}], ")"}], 
              RowBox[{
                RowBox[{"-", "i"}], "+", "m", "-", "2"}]]}]}],
        HoldForm], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["OPESumSimplify[%]", "Input",
  FormatType->InputForm],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FormBox[
            TagBox[
              RowBox[{
                UnderoverscriptBox["\[Sum]", 
                  RowBox[{"i", "=", "0"}], "m"], 
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", "1"}], ")"}], "i"], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}], ")"}], "i"], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", "(", "\[NoBreak]", 
                        FormBox[
                          RowBox[{"p", "-", "q"}],
                          "TraditionalForm"], "\[NoBreak]", ")"}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "i"}], "+", "m", "-", "2"}]]}]}],
              HoldForm],
            "TraditionalForm"]}], " ", 
        RowBox[{
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["p",
            "TraditionalForm"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OPESumSimplify", "[", 
      RowBox[{
        RowBox[{"OPESum", "[", 
          RowBox[{"{", 
            RowBox[{"OPEi", ",", "0", ",", "OPEm"}], "}"}], "]"}], " ", 
        RowBox[{"a", "^", "OPEi"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      TagBox[
        RowBox[{
          UnderoverscriptBox["\[Sum]", 
            RowBox[{"i", "=", "0"}], "m"], 
          FormBox[
            SuperscriptBox["a", "i"],
            "TraditionalForm"]}],
        HoldForm], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"OPESumSimplify", "[", 
      RowBox[{
        RowBox[{"OPESum", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"j", ",", "0", ",", "i"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"i", ",", "0", ",", "m"}], "}"}]}], "]"}], " ", 
        RowBox[{"a", "^", 
          RowBox[{"(", 
            RowBox[{"j", "-", "i"}], ")"}]}], " ", 
        RowBox[{"b", "^", "i"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          TagBox[
            RowBox[{
              UnderoverscriptBox["\[Sum]", 
                RowBox[{"i", "=", "0"}], "m"], "\<\"\"\>"}],
            HoldForm],
          "TraditionalForm"], "\[NoBreak]", 
        FormBox[
          RowBox[{
            UnderoverscriptBox["\[Sum]", 
              RowBox[{"j", "=", "0"}], "i"], "\<\"\"\>"}],
          "TraditionalForm"], "\[NoBreak]", 
        FormBox[
          RowBox[{
            SuperscriptBox["a", 
              RowBox[{"j", "-", "i"}]], " ", 
            SuperscriptBox["b", "i"]}],
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"OPESum", "[", 
      RowBox[{
        RowBox[{
          SuperscriptBox["a", 
            RowBox[{
              RowBox[{"-", "i"}], "+", "j"}]], " ", 
          SuperscriptBox["b", "i"]}], ",", 
        RowBox[{"{", 
          RowBox[{"i", ",", "0", ",", "m"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"j", ",", "0", ",", "i"}], "}"}]}], "]"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Pair", "Subsection",
  CellTags->"Pair"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "Pair[x, y] is the head of a special pairing used in the internal \
representation: x and y may have heads LorentzIndex or Momentum. If both x \
and y have head LorentzIndex, the metric tensor is understood. If x and y \
have head Momentum, a scalar product is meant. If one of x and y has head \
LorentzIndex and the other Momentum, a Lorentz vector ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["p", "\[Mu]"], TraditionalForm]]],
  " is understood."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FourVector",
    ButtonData:>"FourVector",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FourVector"],
  ", ",
  ButtonBox["FV",
    ButtonData:>"FV",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FV"],
  ", ",
  ButtonBox["FVD",
    ButtonData:>"FVD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FVD"],
  ", ",
  ButtonBox["MetricTensor",
    ButtonData:>"MetricTensor",
    ButtonStyle->"Hyperlink",
    ButtonNote->"MetricTensor"],
  ", ",
  ButtonBox["MT",
    ButtonData:>"MT",
    ButtonStyle->"Hyperlink",
    ButtonNote->"MT"],
  ", ",
  ButtonBox["MTD",
    ButtonData:>"MTD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"MTD"],
  ", ",
  ButtonBox["ScalarProduct",
    ButtonData:>"ScalarProduct",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ScalarProduct"],
  ", ",
  ButtonBox["SP",
    ButtonData:>"SP",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SP"],
  ", ",
  ButtonBox["SPD",
    ButtonData:>"SPD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SPD"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell["This represents a four-dimensional metric tensor.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["g", 
        RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell["This is a D-dimensional metric tensor.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Alpha]", ",", "D"}], "]"}], ",", 
        RowBox[{"LorentzIndex", "[", 
          RowBox[{"\[Beta]", ",", "D"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["g", 
        RowBox[{
          FormBox[
            FormBox["\[Alpha]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Beta]",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
        RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        FormBox["p",
          "TraditionalForm"], 
        FormBox[
          FormBox["\[Alpha]",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "q", "]"}], ",", 
        RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["p",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        FormBox["q",
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", 
        RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        FormBox["p",
          "TraditionalForm"], "2"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", 
          RowBox[{"p", "-", "q"}], "]"}], ",", 
        RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["p",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        "(", "\[NoBreak]", 
        FormBox[
          RowBox[{"p", "-", "q"}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"Momentum", "[", "p", "]"}], ",", 
          RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], "^", "2"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        FormBox["p",
          "TraditionalForm"], "4"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"Momentum", "[", "p", "]"}], ",", 
          RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], "^", "3"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        FormBox["p",
          "TraditionalForm"], "6"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Alpha]", ",", 
              RowBox[{"n", "-", "4"}]}], "]"}], ",", 
          RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}], "//", 
      "Contract"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandScalarProduct", "[", 
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"Momentum", "[", 
            RowBox[{"p", "-", "q"}], "]"}], ",", 
          RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], "2"], "-", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"Momentum", "[", 
            RowBox[{"-", "q"}], "]"}], ",", 
          RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], " ", "+", " ", 
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"Momentum", "[", "q", "]"}], ",", 
          RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["PartialD", "Subsection",
  CellTags->"PartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "PartialD[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "] denotes the four-dimensional ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], "."}], TraditionalForm]]],
  " PartialD is used to denote derivative fields. PartialD[LorentzIndex[",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[Mu]", ",", "D"}], TraditionalForm]]],
  "]] denotes the ",
  Cell[BoxData[
      FormBox["D", TraditionalForm]]],
  "-dimensional ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["\[PartialD]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], "."}], TraditionalForm]]]
}], "Text",
  ZeroWidthTimes->True],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["ExpandPartialD",
    ButtonData:>"ExpandPartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ExpandPartialD"],
  ", ",
  ButtonBox["LeftRightPartialD",
    ButtonData:>"LeftRightPartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"LeftRightPartialD"],
  ", ",
  ButtonBox["RightPartialD",
    ButtonData:>"RightPartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"RightPartialD"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"{", "\[Mu]", "}"}]}], "]"}], ".", 
      RowBox[{"LeftPartialD", "[", "\[Nu]", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["A", "\[Mu]"], ".", 
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SubscriptBox["A", "\[Mu]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"LeftPartialD", "[", 
        RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], "]"}], " ", 
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["PaVe", "Subsection",
  CellTags->"PaVe"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
 PaVe[ i,j,... {p10,p12,...},{m1^2, mw^2, ...} ] denotes the \
invariant (and scalar) Passarino-Veltman integrals, i.e. the coefficient \
functions of the tensor integral decomposition. Joining plist and mlist gives \
the same conventions as for A0, B0, C0, D0. Automatic simlifications are \
performed for the coefficient functions of two-point integrals and for the \
scalar integrals.\
\>", "Text"],

Cell[TextData[{
  "For a more detailed description see the manual (no changes from there \
basically): ",
  ButtonBox["http://www.mertig.com/oldfc",
    ButtonData:>{
      URL[ "http://www.mertig.com/oldfc"], None},
    ButtonStyle->"Hyperlink"]
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["PaVeReduce",
    ButtonData:>"PaVeReduce",
    ButtonStyle->"Hyperlink",
    ButtonNote->"PaVeReduce"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell["Some of the PaVe's reduce to special cases.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PaVe", "[", 
      RowBox[{"0", ",", 
        RowBox[{"{", 
          RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", "e", ",", "f"}], 
          "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"m1", ",", "m2", ",", "m3", ",", "m4"}], "}"}]}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "a", 
        "\[NoBreak]", ",", "\[NoBreak]", "b", "\[NoBreak]", ",", 
        "\[NoBreak]", "c", "\[NoBreak]", ",", "\[NoBreak]", "d", 
        "\[NoBreak]", ",", "\[NoBreak]", "e", "\[NoBreak]", ",", 
        "\[NoBreak]", "f", "\[NoBreak]", ",", "\[NoBreak]", "m1", 
        "\[NoBreak]", ",", "\[NoBreak]", "m2", "\[NoBreak]", ",", 
        "\[NoBreak]", "m3", "\[NoBreak]", ",", "\[NoBreak]", "m4", 
        "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PaVe", "[", 
      RowBox[{"0", ",", "0", ",", 
        RowBox[{"{", "pp", "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"m", "^", "2"}], ",", 
            RowBox[{"M", "^", "2"}]}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "3"], " ", 
          RowBox[{
            SubscriptBox["B", "0"], "(", "pp", ",", 
            SuperscriptBox["m", "2"], ",", 
            SuperscriptBox["M", "2"], ")"}], " ", 
          SuperscriptBox["m", "2"]}], "+", 
        RowBox[{
          FractionBox["1", "18"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"3", " ", 
                SuperscriptBox["m", "2"]}], "+", 
              RowBox[{"3", " ", 
                SuperscriptBox["M", "2"]}], "-", "pp"}], ")"}]}], "+", 
        RowBox[{
          FractionBox["1", "6"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{
                  SubscriptBox["B", "0"], "(", "0", ",", 
                  SuperscriptBox["M", "2"], ",", 
                  SuperscriptBox["M", "2"], ")"}], " ", 
                SuperscriptBox["M", "2"]}], "+", 
              SuperscriptBox["M", "2"], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox["m", "2"], "-", 
                    SuperscriptBox["M", "2"], "+", "pp"}], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    FractionBox[
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            SuperscriptBox["M", "2"], "-", 
                            SuperscriptBox["m", "2"]}], ")"}], " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              SubscriptBox["B", "0"], "(", "pp", ",", 
                              SuperscriptBox["m", "2"], ",", 
                              SuperscriptBox["M", "2"], ")"}], "-", 
                            RowBox[{
                              SubscriptBox["B", "0"], "(", "0", ",", 
                              SuperscriptBox["m", "2"], ",", 
                              SuperscriptBox["M", "2"], ")"}]}], ")"}]}], 
                      RowBox[{"2", " ", "pp"}]], "-", 
                    RowBox[{
                      FractionBox["1", "2"], " ", 
                      RowBox[{
                        SubscriptBox["B", "0"], "(", "pp", ",", 
                        SuperscriptBox["m", "2"], ",", 
                        SuperscriptBox["M", "2"], ")"}]}]}], ")"}]}]}], 
            ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["PaVeOrder", "Subsection",
  CellTags->"PaVeOrder"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
PaVeOrder[expr] orders the arguments of all D0 in expr in a \
standard way. PaVeOrder[expr, PaVeOrderList -> { {..., s, u, ...}, {... m1^2, \
m2^2, ...}, ...}] orders the arguments of all D0 in expr according to the \
specified ordering lists. The lists may contain only a subsequence of the \
D0-variables.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["PaVeReduce",
    ButtonData:>"PaVeReduce",
    ButtonStyle->"Hyperlink",
    ButtonNote->"PaVeReduce"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PaVeOrder", "[", 
      RowBox[{
        RowBox[{"D0", "[", 
          RowBox[{
          "me2", ",", "me2", ",", "mw2", ",", "mw2", ",", "t", ",", "s", ",", 
            "me2", ",", "0", ",", "me2", ",", "0"}], "]"}], ",", 
        RowBox[{"PaVeOrderList", "\[Rule]", 
          RowBox[{"{", 
            RowBox[{"me2", ",", "me2", ",", "0", ",", "0"}], "}"}]}]}], 
      "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "me2", 
        "\[NoBreak]", ",", "\[NoBreak]", "s", "\[NoBreak]", ",", 
        "\[NoBreak]", "mw2", "\[NoBreak]", ",", "\[NoBreak]", "t", 
        "\[NoBreak]", ",", "\[NoBreak]", "mw2", "\[NoBreak]", ",", 
        "\[NoBreak]", "me2", "\[NoBreak]", ",", "\[NoBreak]", "me2", 
        "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox["0",
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox["0",
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "me2", 
        "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PaVeOrder", "[", 
      RowBox[{
        RowBox[{"D0", "[", 
          RowBox[{
          "me2", ",", "me2", ",", "mw2", ",", "mw2", ",", "t", ",", "s", ",", 
            "me2", ",", "0", ",", "me2", ",", "0"}], "]"}], ",", 
        RowBox[{"PaVeOrderList", "\[Rule]", 
          RowBox[{"{", 
            RowBox[{"me2", ",", "me2", ",", "0", ",", "0"}], "}"}]}]}], 
      "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "me2", 
        "\[NoBreak]", ",", "\[NoBreak]", "s", "\[NoBreak]", ",", 
        "\[NoBreak]", "mw2", "\[NoBreak]", ",", "\[NoBreak]", "t", 
        "\[NoBreak]", ",", "\[NoBreak]", "mw2", "\[NoBreak]", ",", 
        "\[NoBreak]", "me2", "\[NoBreak]", ",", "\[NoBreak]", "me2", 
        "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox["0",
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
        FormBox["0",
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "me2", 
        "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PaVeOrder", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"D0", "[", 
            RowBox[{
            "a", ",", "b", ",", "c", ",", "d", ",", "e", ",", "f", ",", 
              "m12", ",", "m22", ",", "m32", ",", "m42"}], "]"}], "+", 
          RowBox[{"D0", "[", 
            RowBox[{
            "me2", ",", "me2", ",", "mw2", ",", "mw2", ",", "t", ",", "s", 
              ",", "me2", ",", "0", ",", "me2", ",", "0"}], "]"}]}], ",", 
        RowBox[{"PaVeOrderList", "\[Rule]", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"me2", ",", "me2", ",", "0", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"f", ",", "e"}], "}"}]}], "}"}]}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FormBox[
            SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
            "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "a", 
          "\[NoBreak]", ",", "\[NoBreak]", "d", "\[NoBreak]", ",", 
          "\[NoBreak]", "c", "\[NoBreak]", ",", "\[NoBreak]", "b", 
          "\[NoBreak]", ",", "\[NoBreak]", "f", "\[NoBreak]", ",", 
          "\[NoBreak]", "e", "\[NoBreak]", ",", "\[NoBreak]", "m22", 
          "\[NoBreak]", ",", "\[NoBreak]", "m12", "\[NoBreak]", ",", 
          "\[NoBreak]", "m42", "\[NoBreak]", ",", "\[NoBreak]", "m32", 
          "\[NoBreak]", ")"}], "+", 
        RowBox[{
          FormBox[
            SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
            "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "me2", 
          "\[NoBreak]", ",", "\[NoBreak]", "s", "\[NoBreak]", ",", 
          "\[NoBreak]", "mw2", "\[NoBreak]", ",", "\[NoBreak]", "t", 
          "\[NoBreak]", ",", "\[NoBreak]", "mw2", "\[NoBreak]", ",", 
          "\[NoBreak]", "me2", "\[NoBreak]", ",", "\[NoBreak]", "me2", 
          "\[NoBreak]", ",", "\[NoBreak]", 
          FormBox["0",
            "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
          FormBox["0",
            "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "me2", 
          "\[NoBreak]", ")"}]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["PaVeReduce", "Subsection",
  CellTags->"PaVeReduce"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
PaVeReduce[expr] reduces all Passarino-Veltman integrals (i.e. all \
PaVe's) in expr down to scalar A0, B0, C0 and D0.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "PaVeReduce", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Dimension", "\[Rule]", "True"}], ",", 
          RowBox[{"IsolateNames", "\[Rule]", "False"}], ",", 
          RowBox[{"Mandelstam", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"PaVeOrderList", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"WriteOutPaVe", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"?", "WriteOutPaVe"}]], "Input"],

Cell[BoxData[
    FormBox[
    "\<\"WriteOutPaVe is an option for PaVeReduce and OneLoopSum.  If set to \
a string, the results of all Passarino-Veltman PaVe's are written out.\"\>", 
      TraditionalForm]], "Print"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FRH",
    ButtonData:>"FRH",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FRH"],
  ", ",
  ButtonBox["PaVeOrder",
    ButtonData:>"PaVeOrder",
    ButtonStyle->"Hyperlink",
    ButtonNote->"PaVeOrder"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PaVeReduce", "[", 
      RowBox[{
        RowBox[{"PaVe", "[", 
          RowBox[{"1", ",", "2", ",", 
            RowBox[{"{", 
              RowBox[{"s", ",", 
                RowBox[{"m", "^", "2"}], ",", 
                RowBox[{"m", "^", "2"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"m", "^", "2"}], ",", 
                RowBox[{"m", "^", "2"}], ",", 
                RowBox[{"M", "^", "2"}]}], "}"}]}], "]"}], ",", 
        RowBox[{"IsolateHead", "\[Rule]", "L"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{"8", " ", 
                  SuperscriptBox["m", "2"]}], "-", 
                RowBox[{"3", " ", 
                  SuperscriptBox["M", "2"]}], "-", 
                RowBox[{"2", " ", "s"}]}], ")"}], " ", 
            RowBox[{
              FormBox[
                SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
                "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", 
              FormBox[
                SuperscriptBox["m", "2"],
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox[
                SuperscriptBox["m", "2"],
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "s", 
              "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox[
                SuperscriptBox["m", "2"],
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox[
                SuperscriptBox["M", "2"],
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox[
                SuperscriptBox["m", "2"],
                "TraditionalForm"], "\[NoBreak]", ")"}], " ", 
            SuperscriptBox["M", "2"]}], 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                RowBox[{"4", " ", 
                  SuperscriptBox["m", "2"]}], "-", "s"}], ")"}], "2"]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox["m", "2"], "-", 
                SuperscriptBox["M", "2"]}], ")"}], " ", 
            RowBox[{
              SubscriptBox["B", "0"], "(", "0", ",", 
              SuperscriptBox["m", "2"], ",", 
              SuperscriptBox["M", "2"], ")"}]}], 
          RowBox[{"2", " ", 
            SuperscriptBox["m", "2"], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"4", " ", 
                  SuperscriptBox["m", "2"]}], "-", "s"}], ")"}]}]], "-", 
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{"8", " ", 
                  SuperscriptBox["m", "4"]}], "-", 
                RowBox[{"10", " ", 
                  SuperscriptBox["M", "2"], " ", 
                  SuperscriptBox["m", "2"]}], "-", 
                RowBox[{"2", " ", "s", " ", 
                  SuperscriptBox["m", "2"]}], "+", 
                RowBox[{
                  SuperscriptBox["M", "2"], " ", "s"}]}], ")"}], " ", 
            RowBox[{
              SubscriptBox["B", "0"], "(", 
              SuperscriptBox["m", "2"], ",", 
              SuperscriptBox["m", "2"], ",", 
              SuperscriptBox["M", "2"], ")"}]}], 
          RowBox[{"2", " ", 
            SuperscriptBox["m", "2"], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"4", " ", 
                    SuperscriptBox["m", "2"]}], "-", "s"}], ")"}], "2"]}]], 
        "+", 
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{"4", " ", 
                  SuperscriptBox["m", "2"]}], "-", 
                RowBox[{"6", " ", 
                  SuperscriptBox["M", "2"]}], "-", "s"}], ")"}], " ", 
            RowBox[{
              SubscriptBox["B", "0"], "(", "s", ",", 
              SuperscriptBox["m", "2"], ",", 
              SuperscriptBox["m", "2"], ")"}]}], 
          RowBox[{"2", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"4", " ", 
                    SuperscriptBox["m", "2"]}], "-", "s"}], ")"}], "2"]}]], 
        "+", 
        FractionBox["1", 
          RowBox[{"2", " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"4", " ", 
                  SuperscriptBox["m", "2"]}], "-", "s"}], ")"}]}]]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"L", "[", "10", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"L", "(", "10", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FRH", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"L", "(", "10", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PaVeReduce", "[", 
      RowBox[{
        RowBox[{"PaVe", "[", 
          RowBox[{"2", ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"SmallVariable", "[", "me2", "]"}], ",", "mw2", ",", 
                "t"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"SmallVariable", "[", "me2", "]"}], ",", "0", ",", 
                "mw2"}], "}"}]}], "]"}], ",", 
        RowBox[{"WriteOutPaVe", "\[Rule]", "\"\<p\>\""}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SubscriptBox["B", "0"], "(", "0", ",", "mw2", ",", "mw2", ")"}], 
          RowBox[{"mw2", "-", "t"}]], "-", 
        FractionBox[
          RowBox[{
            SubscriptBox["B", "0"], "(", "t", ",", "mw2", ",", "me2", ")"}], 
          RowBox[{"mw2", "-", "t"}]], "+", 
        FractionBox["2", 
          RowBox[{"mw2", "-", "t"}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TableForm", "[", 
      RowBox[{"ReadList", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"If", "[", 
              RowBox[{
                RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", 
                "\"\<:\>\"", ",", "\"\<\>\""}], "]"}], "<>", 
            "\"\<pPaVe2Csmame2mw2tCsmame20mw2.s\>\""}], ",", "String"}], 
        "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      InterpretationBox[GridBox[{
            {"\<\"( 2/(mw2 - t) + B0[0, mw2, mw2]/(mw2 - t) - \"\>"},
            {"\<\"  B0[t, mw2, SmallVariable[me2]]/(mw2 - t)\"\>"},
            {"\<\"  ) \"\>"}
            },
          RowSpacings->1,
          ColumnSpacings->3,
          RowAlignments->Baseline,
          ColumnAlignments->{Left}],
        TableForm[ {"( 2/(mw2 - t) + B0[0, mw2, mw2]/(mw2 - t) - ", 
          "  B0[t, mw2, SmallVariable[me2]]/(mw2 - t)", "  ) "}]], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"DeleteFile", "/@", 
        RowBox[{
        "FileNames", "[", "\"\<pPaVe2Csmame2mw2tCsmame20mw2.s\>\"", "]"}]}], 
      ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"se", "=", 
        RowBox[{"SmallVariable", "[", "ME2", "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"d122", "=", 
      RowBox[{"PaVeReduce", "[", 
        RowBox[{
          RowBox[{"PaVe", "[", 
            RowBox[{"1", ",", "2", ",", "2", ",", 
              RowBox[{"{", 
                RowBox[{
                "se", ",", "MW2", ",", "MW2", ",", "se", ",", "S", ",", 
                  "T"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"0", ",", "se", ",", "0", ",", "se"}], "}"}]}], 
            "]"}], ",", 
          RowBox[{"Mandelstam", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{"S", ",", "T", ",", "U", ",", 
                RowBox[{"2", " ", "MW2"}]}], "}"}]}], ",", 
          RowBox[{"IsolateHead", "\[Rule]", "F"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{"MW2", "-", "S"}], ")"}], " ", 
            SuperscriptBox["T", "2"], " ", 
            RowBox[{
              FormBox[
                SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
                "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "MW2", 
              "\[NoBreak]", ",", "\[NoBreak]", "MW2", "\[NoBreak]", ",", 
              "\[NoBreak]", 
              FormBox["ME2",
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox["ME2",
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "T", 
              "\[NoBreak]", ",", "\[NoBreak]", "S", "\[NoBreak]", ",", 
              "\[NoBreak]", 
              FormBox["ME2",
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox["0",
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox["ME2",
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox["0",
                "TraditionalForm"], "\[NoBreak]", ")"}], " ", 
            SuperscriptBox["S", "3"]}], 
          RowBox[{"2", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["MW2", "2"], "-", 
                  RowBox[{"S", " ", "U"}]}], ")"}], "3"]}]], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"MW2", "-", "S"}], ")"}], "2"], " ", "T", " ", 
            RowBox[{
              FormBox[
                SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
                "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "MW2", 
              "\[NoBreak]", ",", "\[NoBreak]", "S", "\[NoBreak]", ",", 
              "\[NoBreak]", 
              FormBox["ME2",
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox["ME2",
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox["0",
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox["0",
                "TraditionalForm"], "\[NoBreak]", ")"}], " ", 
            SuperscriptBox["S", "2"]}], 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox["MW2", "2"], "-", 
                RowBox[{"S", " ", "U"}]}], ")"}], "3"]], "-", 
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{"MW2", "-", "S"}], ")"}], " ", 
            SuperscriptBox["T", "2"], " ", 
            RowBox[{
              FormBox[
                SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
                "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "T", 
              "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox["ME2",
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox["ME2",
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox["ME2",
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox["ME2",
                "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
              FormBox["0",
                "TraditionalForm"], "\[NoBreak]", ")"}], " ", 
            SuperscriptBox["S", "2"]}], 
          RowBox[{"2", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["MW2", "2"], "-", 
                  RowBox[{"S", " ", "U"}]}], ")"}], "3"]}]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox["MW2", "2"], "-", 
                RowBox[{"4", " ", "S", " ", "MW2"}], "+", 
                RowBox[{"2", " ", 
                  SuperscriptBox["S", "2"]}], "+", 
                RowBox[{"S", " ", "U"}]}], ")"}], " ", 
            RowBox[{
              SubscriptBox["B", "0"], "(", "S", ",", "0", ",", "0", ")"}], 
            " ", "S"}], 
          RowBox[{"2", " ", 
            RowBox[{"(", 
              RowBox[{"MW2", "-", "S"}], ")"}], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["MW2", "2"], "-", 
                  RowBox[{"S", " ", "U"}]}], ")"}], "2"]}]], "-", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"4", " ", 
                    SuperscriptBox["MW2", "5"]}], "-", 
                  RowBox[{"5", " ", "S", " ", 
                    SuperscriptBox["MW2", "4"]}], "-", 
                  RowBox[{"U", " ", 
                    SuperscriptBox["MW2", "4"]}], "-", 
                  RowBox[{"16", " ", 
                    SuperscriptBox["S", "2"], " ", 
                    SuperscriptBox["MW2", "3"]}], "+", 
                  RowBox[{"4", " ", 
                    SuperscriptBox["S", "3"], " ", 
                    SuperscriptBox["MW2", "2"]}], "+", 
                  RowBox[{"4", " ", "S", " ", 
                    SuperscriptBox["U", "2"], " ", 
                    SuperscriptBox["MW2", "2"]}], "-", 
                  RowBox[{"4", " ", 
                    SuperscriptBox["S", "2"], " ", "U", " ", 
                    SuperscriptBox["MW2", "2"]}], "+", 
                  RowBox[{"4", " ", 
                    SuperscriptBox["S", "4"], " ", "MW2"}], "+", 
                  RowBox[{"8", " ", 
                    SuperscriptBox["S", "3"], " ", "U", " ", "MW2"}], "+", 
                  RowBox[{
                    SuperscriptBox["S", "2"], " ", 
                    SuperscriptBox["U", "3"]}], "+", 
                  RowBox[{
                    SuperscriptBox["S", "3"], " ", 
                    SuperscriptBox["U", "2"]}]}], ")"}], " ", 
              RowBox[{
                SubscriptBox["B", "0"], "(", "MW2", ",", "0", ",", "ME2", 
                ")"}]}], ")"}], "/", 
          RowBox[{"(", 
            RowBox[{"2", " ", 
              RowBox[{"(", 
                RowBox[{"MW2", "-", "S"}], ")"}], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"4", " ", "MW2"}], "-", "T"}], ")"}], "2"], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox["MW2", "2"], "-", 
                    RowBox[{"S", " ", "U"}]}], ")"}], "2"]}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"MW2", "+", "S"}], ")"}], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"4", " ", 
                    SuperscriptBox["MW2", "3"]}], "-", 
                  RowBox[{"9", " ", "S", " ", 
                    SuperscriptBox["MW2", "2"]}], "-", 
                  RowBox[{"U", " ", 
                    SuperscriptBox["MW2", "2"]}], "-", 
                  RowBox[{"4", " ", "S", " ", "U", " ", "MW2"}], "+", 
                  RowBox[{"2", " ", 
                    SuperscriptBox["S", "3"]}], "+", 
                  RowBox[{"3", " ", "S", " ", 
                    SuperscriptBox["U", "2"]}], "+", 
                  RowBox[{"5", " ", 
                    SuperscriptBox["S", "2"], " ", "U"}]}], ")"}], " ", 
              RowBox[{
                SubscriptBox["B", "0"], "(", "T", ",", "ME2", ",", "ME2", 
                ")"}]}], ")"}], "/", 
          RowBox[{"(", 
            RowBox[{"2", " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"4", " ", "MW2"}], "-", "T"}], ")"}], "2"], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox["MW2", "2"], "-", 
                    RowBox[{"S", " ", "U"}]}], ")"}], "2"]}], ")"}]}], "-", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"MW2", "+", "S"}], ")"}], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"2", " ", 
                    SuperscriptBox["MW2", "6"]}], "-", 
                  RowBox[{"8", " ", "S", " ", 
                    SuperscriptBox["MW2", "5"]}], "-", 
                  RowBox[{"2", " ", "T", " ", 
                    SuperscriptBox["MW2", "5"]}], "+", 
                  RowBox[{"12", " ", 
                    SuperscriptBox["S", "2"], " ", 
                    SuperscriptBox["MW2", "4"]}], "+", 
                  RowBox[{"20", " ", "S", " ", "T", " ", 
                    SuperscriptBox["MW2", "4"]}], "-", 
                  RowBox[{"8", " ", 
                    SuperscriptBox["S", "3"], " ", 
                    SuperscriptBox["MW2", "3"]}], "-", 
                  RowBox[{"6", " ", "S", " ", 
                    SuperscriptBox["T", "2"], " ", 
                    SuperscriptBox["MW2", "3"]}], "-", 
                  RowBox[{"36", " ", 
                    SuperscriptBox["S", "2"], " ", "T", " ", 
                    SuperscriptBox["MW2", "3"]}], "+", 
                  RowBox[{"2", " ", 
                    SuperscriptBox["S", "4"], " ", 
                    SuperscriptBox["MW2", "2"]}], "+", 
                  RowBox[{"6", " ", 
                    SuperscriptBox["S", "2"], " ", 
                    SuperscriptBox["T", "2"], " ", 
                    SuperscriptBox["MW2", "2"]}], "+", 
                  RowBox[{"20", " ", 
                    SuperscriptBox["S", "3"], " ", "T", " ", 
                    SuperscriptBox["MW2", "2"]}], "+", 
                  RowBox[{"4", " ", 
                    SuperscriptBox["S", "2"], " ", 
                    SuperscriptBox["T", "3"], " ", "MW2"}], "-", 
                  RowBox[{"6", " ", 
                    SuperscriptBox["S", "3"], " ", 
                    SuperscriptBox["T", "2"], " ", "MW2"}], "-", 
                  RowBox[{"2", " ", 
                    SuperscriptBox["S", "4"], " ", "T", " ", "MW2"}], "-", 
                  RowBox[{
                    SuperscriptBox["S", "2"], " ", 
                    SuperscriptBox["T", "4"]}]}], ")"}], " ", 
              RowBox[{
                FormBox[
                  SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
                  "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "MW2", 
                "\[NoBreak]", ",", "\[NoBreak]", "MW2", "\[NoBreak]", ",", 
                "\[NoBreak]", "T", "\[NoBreak]", ",", "\[NoBreak]", 
                FormBox["ME2",
                  "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
                FormBox["0",
                  "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", 
                FormBox["ME2",
                  "TraditionalForm"], "\[NoBreak]", ")"}]}], ")"}], "/", 
          RowBox[{"(", 
            RowBox[{"2", " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"4", " ", "MW2"}], "-", "T"}], ")"}], "2"], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox["MW2", "2"], "-", 
                    RowBox[{"S", " ", "U"}]}], ")"}], "3"]}], ")"}]}], "-", 
        FractionBox[
          RowBox[{"MW2", "+", "S"}], 
          RowBox[{"2", " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"4", " ", "MW2"}], "-", "T"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox["MW2", "2"], "-", 
                RowBox[{"S", " ", "U"}]}], ")"}]}]]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"Write2", "[", 
        RowBox[{"\"\<fctd122.for\>\"", ",", 
          RowBox[{"d122res", "==", "d122"}], ",", 
          RowBox[{"FormatType", "\[Rule]", "FortranForm"}]}], "]"}], ";"}]], 
  "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TableForm", "[", 
      RowBox[{"ReadList", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"If", "[", 
              RowBox[{
                RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", 
                "\"\<:\>\"", ",", "\"\<\>\""}], "]"}], "<>", 
            "\"\<fctd122.for\>\""}], ",", "String"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      InterpretationBox[GridBox[{
            {
              "\<\"        d122res = -(MW2 + S)/(2D0*(4*MW2 - T)*(MW2**2 - \
S*U)) - \"\>"},
            {"\<\"     &  ((4*MW2**5 - 5*MW2**4*S - 16*MW2**3*S**2 + \"\>"},
            {
              "\<\"     &       4*MW2**2*S**3 + 4*MW2*S**4 - MW2**4*U - \
\"\>"},
            {"\<\"     &       4*MW2**2*S**2*U + 8*MW2*S**3*U + \"\>"},
            {
              "\<\"     &       4*MW2**2*S*U**2 + S**3*U**2 + \
S**2*U**3)*\"\>"},
            {"\<\"     &     B0(MW2,ME2,Null))/\"\>"},
            {
              "\<\"     &   (2D0*(MW2 - S)*(4*MW2 - T)**2*(MW2**2 - S*U)**2) \
+ \"\>"},
            {
              "\<\"     &  (S*(MW2**2 - 4*MW2*S + 2*S**2 + \
S*U)*B0(S,Null,Null))/\"\>"},
            {"\<\"     &   (2D0*(MW2 - S)*(MW2**2 - S*U)**2) + \"\>"},
            {
              "\<\"     &  ((MW2 + S)*(4*MW2**3 - 9*MW2**2*S + 2*S**3 - \
\"\>"},
            {
              "\<\"     &       MW2**2*U - 4*MW2*S*U + 5*S**2*U + \
3*S*U**2)*\"\>"},
            {"\<\"     &     B0(T,ME2,ME2))/\"\>"},
            {"\<\"     &   (2D0*(4*MW2 - T)**2*(MW2**2 - S*U)**2) - \"\>"},
            {
              "\<\"     &  ((MW2 + S)*(2*MW2**6 - 8*MW2**5*S + 12*MW2**4*S**2 \
- \"\>"},
            {
              "\<\"     &       8*MW2**3*S**3 + 2*MW2**2*S**4 - 2*MW2**5*T + \
\"\>"},
            {"\<\"     &       20*MW2**4*S*T - 36*MW2**3*S**2*T + \"\>"},
            {"\<\"     &       20*MW2**2*S**3*T - 2*MW2*S**4*T - \"\>"},
            {"\<\"     &       6*MW2**3*S*T**2 + 6*MW2**2*S**2*T**2 - \"\>"},
            {
              "\<\"     &       6*MW2*S**3*T**2 + 4*MW2*S**2*T**3 - \
S**2*T**4)*\"\>"},
            {"\<\"     &     C0(MW2,MW2,T,ME2,Null,ME2))/\"\>"},
            {"\<\"     &   (2D0*(4*MW2 - T)**2*(MW2**2 - S*U)**3) + \"\>"},
            {
              "\<\"     &  ((MW2 - S)**2*S**2*T*C0(MW2,S,ME2,ME2,Null,Null))/\
\"\>"},
            {"\<\"     &   (MW2**2 - S*U)**3 - \"\>"},
            {
              "\<\"     &  ((MW2 - \
S)*S**2*T**2*C0(T,ME2,ME2,ME2,ME2,Null))/\"\>"},
            {"\<\"     &   (2D0*(MW2**2 - S*U)**3) + \"\>"},
            {"\<\"     &  ((MW2 - S)*S**3*T**2*\"\>"},
            {
              "\<\"     &     \
D0(MW2,MW2,ME2,ME2,T,S,ME2,Null,ME2,Null))/\"\>"},
            {"\<\"     &   (2D0*(MW2**2 - S*U)**3)\"\>"}
            },
          RowSpacings->1,
          ColumnSpacings->3,
          RowAlignments->Baseline,
          ColumnAlignments->{Left}],
        TableForm[ {
          "        d122res = -(MW2 + S)/(2D0*(4*MW2 - T)*(MW2**2 - S*U)) - ", 
          "     &  ((4*MW2**5 - 5*MW2**4*S - 16*MW2**3*S**2 + ", 
          "     &       4*MW2**2*S**3 + 4*MW2*S**4 - MW2**4*U - ", 
          "     &       4*MW2**2*S**2*U + 8*MW2*S**3*U + ", 
          "     &       4*MW2**2*S*U**2 + S**3*U**2 + S**2*U**3)*", 
          "     &     B0(MW2,ME2,Null))/", 
          "     &   (2D0*(MW2 - S)*(4*MW2 - T)**2*(MW2**2 - S*U)**2) + ", 
          "     &  (S*(MW2**2 - 4*MW2*S + 2*S**2 + S*U)*B0(S,Null,Null))/", 
          "     &   (2D0*(MW2 - S)*(MW2**2 - S*U)**2) + ", 
          "     &  ((MW2 + S)*(4*MW2**3 - 9*MW2**2*S + 2*S**3 - ", 
          "     &       MW2**2*U - 4*MW2*S*U + 5*S**2*U + 3*S*U**2)*", 
          "     &     B0(T,ME2,ME2))/", 
          "     &   (2D0*(4*MW2 - T)**2*(MW2**2 - S*U)**2) - ", 
          "     &  ((MW2 + S)*(2*MW2**6 - 8*MW2**5*S + 12*MW2**4*S**2 - ", 
          "     &       8*MW2**3*S**3 + 2*MW2**2*S**4 - 2*MW2**5*T + ", 
          "     &       20*MW2**4*S*T - 36*MW2**3*S**2*T + ", 
          "     &       20*MW2**2*S**3*T - 2*MW2*S**4*T - ", 
          "     &       6*MW2**3*S*T**2 + 6*MW2**2*S**2*T**2 - ", 
          "     &       6*MW2*S**3*T**2 + 4*MW2*S**2*T**3 - S**2*T**4)*", 
          "     &     C0(MW2,MW2,T,ME2,Null,ME2))/", 
          "     &   (2D0*(4*MW2 - T)**2*(MW2**2 - S*U)**3) + ", 
          "     &  ((MW2 - S)**2*S**2*T*C0(MW2,S,ME2,ME2,Null,Null))/", 
          "     &   (MW2**2 - S*U)**3 - ", 
          "     &  ((MW2 - S)*S**2*T**2*C0(T,ME2,ME2,ME2,ME2,Null))/", 
          "     &   (2D0*(MW2**2 - S*U)**3) + ", 
          "     &  ((MW2 - S)*S**3*T**2*", 
          "     &     D0(MW2,MW2,ME2,ME2,T,S,ME2,Null,ME2,Null))/", 
          "     &   (2D0*(MW2**2 - S*U)**3)"}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"DeleteFile", "/@", 
        RowBox[{"FileNames", "[", "\"\<fctd122.for\>\"", "]"}]}], ";", 
      RowBox[{"Clear", "[", 
        RowBox[{"d122", ",", "se"}], "]"}], ";"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["PlusDistribution", "Subsection",
  CellTags->"PlusDistribution"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
PlusDistribution[1/(1-x)] denotes a distribution (in the sense of \
the \"+\" prescription).\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Integrate2",
    ButtonData:>"Integrate2",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Integrate2"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PlusDistribution", "[", 
      RowBox[{"1", "/", 
        RowBox[{"(", 
          RowBox[{"1", "-", "x"}], ")"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubscriptBox[
        RowBox[{"(", 
          FractionBox["1", 
            RowBox[{"1", "-", "x"}]], ")"}], "+"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PlusDistribution", "[", 
      RowBox[{
        RowBox[{"Log", "[", 
          RowBox[{"1", "-", "x"}], "]"}], "/", 
        RowBox[{"(", 
          RowBox[{"1", "-", "x"}], ")"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubscriptBox[
        RowBox[{"(", 
          FractionBox[
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}], 
            RowBox[{"1", "-", "x"}]], ")"}], "+"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{"PlusDistribution", "[", 
          RowBox[{"1", "/", 
            RowBox[{"(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "]"}], ",", " ", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{"PlusDistribution", "[", 
          RowBox[{
            RowBox[{"Log", "[", 
              RowBox[{"1", "-", "x"}], "]"}], "/", 
            RowBox[{"(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "]"}], ",", " ", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{"PlusDistribution", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"Log", "[", 
                RowBox[{"1", "-", "x"}], "]"}], "^", "2"}], "/", 
            RowBox[{"(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "]"}], ",", " ", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PlusDistribution", "[", 
      RowBox[{
        RowBox[{"Log", "[", 
          RowBox[{"x", " ", 
            RowBox[{"(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "]"}], "/", 
        RowBox[{"(", 
          RowBox[{"1", "-", "x"}], ")"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{"log", "(", "x", ")"}], 
          RowBox[{"1", "-", "x"}]], "+", 
        SubscriptBox[
          RowBox[{"(", 
            FractionBox[
              RowBox[{"log", "(", 
                RowBox[{"1", "-", "x"}], ")"}], 
              RowBox[{"1", "-", "x"}]], ")"}], "+"]}], TraditionalForm]], 
  "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Polarization", "Subsection",
  CellTags->"Polarization"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Polarization[k] = Polarization[k, I] is the head of a polarization \
momentum with (incoming) momentum k. A slashed polarization vector (e1(k) \
slash) has to be entered as DiracSlash[Polarization[k]]. The internal \
representation for a polarization vector e1 corresponding to a boson with \
four momentum k is: Momentum[ Polarization[ k, I ] ]. With this notation \
transversality of polarization vectors is provided, i.e. , Pair[ Momentum[k], \
Momentum[ Polarization[k, I] ] ] yields 0. Polarization[k,-I] denotes the \
complex conjugate polarization. Polarization is also an option of various \
functions related to the operator product expansion. The setting 0 denotes \
the unpolarized and 1 the polarized case.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["PolarizationVector",
    ButtonData:>"PolarizationVector",
    ButtonStyle->"Hyperlink",
    ButtonNote->"PolarizationVector"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Polarization", "[", "k", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
      "\[CurlyEpsilon]", "\[NoBreak]", "(", "\[NoBreak]", "k", "\[NoBreak]", 
        ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Polarization", "[", "k", "]"}], "//", "StandardForm"}]], 
  "Input"],

Cell[BoxData[
    RowBox[{"Polarization", "[", 
      RowBox[{"k", ",", "I"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Polarization", "[", "k", "]"}], "//", "ComplexConjugate"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          InterpretationBox[
            SuperscriptBox["\<\"\[CurlyEpsilon]\"\>", "\<\"*\"\>"],
            Superscript[ "\[CurlyEpsilon]", "*"],
            Editable->False],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "k", 
        "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"Polarization", "[", "k", "]"}], "//", "ComplexConjugate"}], 
      "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Polarization", "[", 
      RowBox[{"k", ",", 
        RowBox[{"-", "I"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSlash", "[", 
      RowBox[{"Polarization", "[", "k", "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        FormBox[
          RowBox[{
          "\[CurlyEpsilon]", "\[NoBreak]", "(", "\[NoBreak]", "k", 
            "\[NoBreak]", ")"}],
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracSlash", "[", 
        RowBox[{"Polarization", "[", "k", "]"}], "]"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"DiracGamma", "[", 
      RowBox[{"Momentum", "[", 
        RowBox[{"Polarization", "[", 
          RowBox[{"k", ",", "I"}], "]"}], "]"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Pair", "[", " ", 
      RowBox[{
        RowBox[{"Momentum", "[", "k", "]"}], ",", " ", 
        RowBox[{"Momentum", "[", " ", 
          RowBox[{"Polarization", "[", 
            RowBox[{"k", ",", " ", "I"}], "]"}], " ", "]"}]}], " ", "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["PolarizationSum", "Subsection",
  CellTags->"PolarizationSum"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
PolarizationSum[ mu,nu, ... ] defines (as abbreviations) several \
polarization sums. The first two arguments are the interpreted as Lorentz \
indices, all further ones are momenta. PolarizationSum performs no \
calculations.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "PolarizationSum", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Dimension", "\[Rule]", "4"}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Polarization",
    ButtonData:>"Polarization",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Polarization"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PolarizationSum", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PolarizationSum", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "k"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SuperscriptBox[
              FormBox["k",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            SuperscriptBox[
              FormBox["k",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]]}], 
          SuperscriptBox[
            FormBox["k",
              "TraditionalForm"], "2"]], "-", 
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PolarizationSum", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "k", ",", "n"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], "-", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              FormBox["k",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            SuperscriptBox[
              FormBox["k",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            SuperscriptBox[
              FormBox["n",
                "TraditionalForm"], "2"]}], 
          SuperscriptBox[
            RowBox[{"(", 
              FormBox[
                RowBox[{
                  FormBox["k",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["n",
                    "TraditionalForm"]}],
                "TraditionalForm"], ")"}], "2"]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{
              SuperscriptBox[
                FormBox["n",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["k",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox[
                FormBox["k",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["n",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}]}], 
          RowBox[{
            FormBox["k",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["n",
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PolarizationSum", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "k", ",", 
        RowBox[{
          SubscriptBox["p", "1"], "-", 
          SubscriptBox["p", "2"]}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], "-", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              FormBox["k",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            SuperscriptBox[
              FormBox["k",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            SubsuperscriptBox["p", "1", "2"]}], 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FormBox["k",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "1"],
                    "TraditionalForm"]}], "-", 
                RowBox[{
                  FormBox["k",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "2"],
                    "TraditionalForm"]}]}], ")"}], "2"]], "+", 
        FractionBox[
          RowBox[{"2", " ", 
            SuperscriptBox[
              FormBox["k",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            SuperscriptBox[
              FormBox["k",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            RowBox[{
              FormBox[
                SubscriptBox["p", "1"],
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                SubscriptBox["p", "2"],
                "TraditionalForm"]}]}], 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FormBox["k",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "1"],
                    "TraditionalForm"]}], "-", 
                RowBox[{
                  FormBox["k",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "2"],
                    "TraditionalForm"]}]}], ")"}], "2"]], "-", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              FormBox["k",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            SuperscriptBox[
              FormBox["k",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            SubsuperscriptBox["p", "2", "2"]}], 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FormBox["k",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "1"],
                    "TraditionalForm"]}], "-", 
                RowBox[{
                  FormBox["k",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "2"],
                    "TraditionalForm"]}]}], ")"}], "2"]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        SubscriptBox["p", "1"], "-", 
                        SubscriptBox["p", "2"]}],
                      "TraditionalForm"], ")"}], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], ")"}], " ", 
              SuperscriptBox[
                FormBox["k",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox[
                FormBox["k",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{"(", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        SubscriptBox["p", "1"], "-", 
                        SubscriptBox["p", "2"]}],
                      "TraditionalForm"], ")"}], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], ")"}]}]}], 
          RowBox[{
            RowBox[{
              FormBox["k",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                SubscriptBox["p", "1"],
                "TraditionalForm"]}], "-", 
            RowBox[{
              FormBox["k",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                SubscriptBox["p", "2"],
                "TraditionalForm"]}]}]]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["PolarizationVector", "Subsection",
  CellTags->"PolarizationVector"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["PolarizationVector[p, mu] gives a polarization vector.", "Text"],

Cell[TextData[{
  "See also: ",
  ButtonBox["FourVector",
    ButtonData:>"FourVector",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FourVector"],
  ", ",
  ButtonBox["Pair",
    ButtonData:>"Pair",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Pair"],
  ", ",
  ButtonBox["Polarization",
    ButtonData:>"Polarization",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Polarization"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PolarizationVector", "[", 
      RowBox[{"k", ",", "\[Mu]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", "k", ")"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Conjugate", "[", 
      RowBox[{"PolarizationVector", "[", 
        RowBox[{"k", ",", "\[Mu]"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubsuperscriptBox["\[CurlyEpsilon]", "\[Mu]", "*"], "(", "k", ")"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "A polarization vector ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", "k", ")"}], 
        TraditionalForm]]],
  "is a special four-vector."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"PolarizationVector", "[", 
        RowBox[{"k", ",", "mu"}], "]"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "mu", "]"}], ",", 
        RowBox[{"Momentum", "[", 
          RowBox[{"Polarization", "[", 
            RowBox[{"k", ",", "I"}], "]"}], "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[TextData[{
  "The transverality property ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            SuperscriptBox[
              FormBox["k",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            RowBox[{
              SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", "k", ")"}]}], 
          "=", "0"}], TraditionalForm]]],
  " is built in."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a1", " ", "=", " ", 
      RowBox[{
        RowBox[{"PolarizationVector", "[", 
          RowBox[{"k", ",", "\[Mu]"}], "]"}], " ", 
        RowBox[{"FourVector", "[", 
          RowBox[{"k", ",", "\[Mu]"}], "]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["k", "\[Mu]"], " ", 
        RowBox[{
          SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", "k", ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", "a1", "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a2", " ", "=", " ", 
      RowBox[{
        RowBox[{"PolarizationVector", "[", 
          RowBox[{
            RowBox[{"k", "-", "p"}], ",", "\[Mu]"}], "]"}], " ", 
        RowBox[{"FourVector", "[", 
          RowBox[{"k", ",", "\[Mu]"}], "]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["k", "\[Mu]"], " ", 
        RowBox[{
          SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", 
          FormBox[
            RowBox[{"k", "-", "p"}],
            "TraditionalForm"], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", "a2", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["k",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        "(", "\[NoBreak]", 
        FormBox[
          RowBox[{"\[CurlyEpsilon]", "\[NoBreak]", "(", "\[NoBreak]", 
            FormBox[
              RowBox[{"k", "-", "p"}],
              "TraditionalForm"], "\[NoBreak]", ")"}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "Depending on the alphabetical ordering of the momenta simplifcations are \
done, e.g., ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            SuperscriptBox[
              FormBox["p",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            RowBox[{
              SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", 
              FormBox[
                RowBox[{"k", "-", "p"}],
                "TraditionalForm"], ")"}]}], "=", 
          RowBox[{
            RowBox[{
              RowBox[{"-", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"k", "-", "p", "-", "k"}], ")"}], "\[Mu]"]}], 
              RowBox[{
                SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", 
                FormBox[
                  RowBox[{"k", "-", "p"}],
                  "TraditionalForm"], ")"}]}], "=", 
            RowBox[{
              RowBox[{
                SuperscriptBox["k", "\[Mu]"], 
                RowBox[{
                  SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", 
                  FormBox[
                    RowBox[{"k", "-", "p"}],
                    "TraditionalForm"], ")"}]}], "=", 
              RowBox[{
                FormBox["k",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                RowBox[{
                  RowBox[{"(", "\[NoBreak]", 
                    FormBox[
                      RowBox[{
                      "\[CurlyEpsilon]", "\[NoBreak]", "(", "\[NoBreak]", 
                        FormBox[
                          RowBox[{"k", "-", "p"}],
                          "TraditionalForm"], "\[NoBreak]", ")"}],
                      "TraditionalForm"], "\[NoBreak]", ")"}], "."}]}]}]}]}], 
        TraditionalForm]]]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a3", " ", "=", " ", 
      RowBox[{
        RowBox[{"PolarizationVector", "[", 
          RowBox[{
            RowBox[{"k", "-", "p"}], ",", "\[Mu]"}], "]"}], " ", 
        RowBox[{"FourVector", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["p", "\[Mu]"], " ", 
        RowBox[{
          SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", 
          FormBox[
            RowBox[{"k", "-", "p"}],
            "TraditionalForm"], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", "a3", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["k",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        "(", "\[NoBreak]", 
        FormBox[
          RowBox[{"\[CurlyEpsilon]", "\[NoBreak]", "(", "\[NoBreak]", 
            FormBox[
              RowBox[{"k", "-", "p"}],
              "TraditionalForm"], "\[NoBreak]", ")"}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"a1", ",", "a2", ",", "a3"}], "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["PowerSimplify", "Subsection",
  CellTags->"PowerSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
PowerSimplify[exp] simplifies (-x)^a to (-1)^a x^a and (y-x)^n to \
(-1)^n (x-y)^n; thus assuming that the exponent is an integer (even if it is \
symbolic). Furthermore (-1)^(a+n) and I^(a+n) are expanded and (I)^(2 m) -> \
(-1)^m and (-1)^(n_Integer?EvenQ m) -> 1 and (-1)^(n_Integer?OddQ m) -> \
(-1)^m and (-1)^(-n) -> (-1)^n and Exp[I m Pi] -> (-1)^m.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DataType",
    ButtonData:>"DataType",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DataType"],
  ", ",
  "OPEm",
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PowerSimplify", "[", 
      RowBox[{
        RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "^", 
        RowBox[{"(", 
          RowBox[{"2", "OPEm"}], ")"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PowerSimplify", "[", 
      RowBox[{
        RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "^", 
        RowBox[{"(", 
          RowBox[{"OPEm", "+", "2"}], ")"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "m"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PowerSimplify", "[", 
      RowBox[{
        RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "^", 
        RowBox[{"(", 
          RowBox[{"OPEm", "-", "2"}], ")"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "m"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PowerSimplify", "[", 
      RowBox[{"I", "^", 
        RowBox[{"(", 
          RowBox[{"2", "OPEm"}], ")"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "m"], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["PropagatorDenominatorExplicit", "Subsection",
  CellTags->"PropagatorDenominatorExplicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
PropagatorDenominatorExplicit[exp] changes each occurence of \
PropagatorDenominator[a,b] in exp into 1/(ScalarProduct[a,a]-b^2) and \
replaces FeynAmpDenominator by Identity.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynAmpDenominator",
    ButtonData:>"FeynAmpDenominator",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FeynAmpDenominator"],
  ", ",
  ButtonBox["PropagatorDenominator",
    ButtonData:>"PropagatorDenominator",
    ButtonStyle->"Hyperlink",
    ButtonNote->"PropagatorDenominator"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FAD", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"q", ",", "m"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"q", "-", "p"}], ",", "0"}], "}"}]}], "]"}], "//", 
      "FCI"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox["q", "2"], "-", 
              SuperscriptBox["m", "2"]}], ")"}], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"q", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"PropagatorDenominatorExplicit", "[", "%", "]"}], "//", 
      "FCE"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox[
                SuperscriptBox[
                  FormBox["q",
                    "TraditionalForm"], "2"],
                "TraditionalForm"], "-", 
              SuperscriptBox["m", "2"]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              FormBox[
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"],
                "TraditionalForm"], "-", 
              RowBox[{"2", " ", 
                FormBox[
                  RowBox[{
                    FormBox["p",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}],
                  "TraditionalForm"]}], "+", 
              FormBox[
                SuperscriptBox[
                  FormBox["q",
                    "TraditionalForm"], "2"],
                "TraditionalForm"]}], ")"}]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    FractionBox["1", 
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            RowBox[{"-", 
              SuperscriptBox["m", "2"]}], "+", 
            RowBox[{"SPD", "[", 
              RowBox[{"q", ",", "q"}], "]"}]}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"SPD", "[", 
              RowBox[{"p", ",", "p"}], "]"}], "-", 
            RowBox[{"2", " ", 
              RowBox[{"SPD", "[", 
                RowBox[{"p", ",", "q"}], "]"}]}], "+", 
            RowBox[{"SPD", "[", 
              RowBox[{"q", ",", "q"}], "]"}]}], ")"}]}]]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["PropagatorDenominator", "Subsection",
  CellTags->"PropagatorDenominator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "PropagatorDenominator[q, m] is a factor of the denominator of a \
propagator. If p is supposed to be D-dimensional enter: \
PropagatorDenominator[Momentum[q, D], m]. What is meant is ",
  Cell[BoxData[
      FormBox[
        RowBox[{" ", "1", "/", 
          RowBox[{"(", 
            RowBox[{"q", "^", "2"}]}]}], TraditionalForm]]],
  " - ",
  Cell[BoxData[
      FormBox[
        RowBox[{"m", "^", "2"}], TraditionalForm]]],
  "). PropagatorDenominator[p] evaluates to PropagatorDenominator[p, 0]. "
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynAmpDenominator",
    ButtonData:>"FeynAmpDenominator",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FeynAmpDenominator"],
  ", ",
  ButtonBox["PropagatorDenominatorExplicit",
    ButtonData:>"PropagatorDenominatorExplicit",
    ButtonStyle->"Hyperlink",
    ButtonNote->"PropagatorDenominatorExplicit"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{" ", 
      RowBox[{"PropagatorDenominator", "[", 
        RowBox[{"p", ",", "m"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          SuperscriptBox["p", "2"], "-", 
          SuperscriptBox["m", "2"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PropagatorDenominator", "[", "p", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        SuperscriptBox["p", "2"]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"t1", "=", 
        RowBox[{"PropagatorDenominator", "[", 
          RowBox[{"q", ",", "m"}], "]"}]}], " "}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          SuperscriptBox["q", "2"], "-", 
          SuperscriptBox["m", "2"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"FCI", "[", "t1", "]"}], "]"}]], "Input"],

Cell[BoxData[
    RowBox[{"PropagatorDenominator", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", 
          RowBox[{"q", ",", "D"}], "]"}], ",", "m"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"ChangeDimension", "[", 
        RowBox[{"t1", ",", "D"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    RowBox[{"PropagatorDenominator", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", 
          RowBox[{"q", ",", "D"}], "]"}], ",", "m"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"PropagatorDenominatorExplicit", "[", "t1", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          SuperscriptBox[
            FormBox["q",
              "TraditionalForm"], "2"], "-", 
          SuperscriptBox["m", "2"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FractionBox["1", 
      RowBox[{
        RowBox[{"-", 
          SuperscriptBox["m", "2"]}], "+", 
        RowBox[{"Pair", "[", 
          RowBox[{
            RowBox[{"Momentum", "[", "q", "]"}], ",", 
            RowBox[{"Momentum", "[", "q", "]"}]}], "]"}]}]]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", "t1", "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["QuantumField", "Subsection",
  CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
QuantumField is the head of quantized fields and their derivatives. \
QuantumField[par, ftype, {lorind}, {sunind}] denotes a quantum field of type \
ftype with (possible) Lorentz-indices lorind and SU(N)-indices sunind. The \
optional first argument par denotes a partial derivative acting on the field.\
\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynRule",
    ButtonData:>"FeynRule",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FeynRule"],
  ", ",
  ButtonBox["PartialD",
    ButtonData:>"PartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"PartialD"],
  ", ",
  ButtonBox["ExpandPartialD",
    ButtonData:>"ExpandPartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ExpandPartialD"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell["This denotes a scalar field.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuantumField", "[", "S", "]"}]], "Input"],

Cell[BoxData[
    FormBox["S", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuantumField", "[", "AntiQuarkField", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      OverscriptBox["\[Psi]", "_"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuantumField", "[", "QuarkField", "]"}]], "Input"],

Cell[BoxData[
    FormBox["\[Psi]", TraditionalForm]], "Output"]
}, Open  ]],

Cell["This is a field with a Lorentz index.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuantumField", "[", 
      RowBox[{"B", ",", 
        RowBox[{"{", "\[Mu]", "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubscriptBox["B", "\[Mu]"], TraditionalForm]], "Output"]
}, Open  ]],

Cell["Color indices should be put after the Lorentz ones.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
        RowBox[{"{", "\[Mu]", "}"}], ",", 
        RowBox[{"{", "a", "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox[
        FormBox["A",
          "TraditionalForm"], 
        FormBox[
          FormBox["\[Mu]",
            "TraditionalForm"],
          "TraditionalForm"], 
        FormBox[
          FormBox["a",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["A", "\[CapitalDelta]", "a"], TraditionalForm]]],
  " is a short form for ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"\[CapitalDelta]", "^", "\[Mu]"}], " ", 
          RowBox[{
            SubsuperscriptBox["A", "\[Mu]", "a"], "."}]}], 
        TraditionalForm]]],
  " "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuantumField", "[", 
      RowBox[{"A", ",", 
        RowBox[{"{", "OPEDelta", "}"}], ",", 
        RowBox[{"{", "a", "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox["A", 
        FormBox["\[CapitalDelta]",
          "TraditionalForm"], 
        FormBox[
          FormBox["a",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
 The first list of indices is usually interpreted as type \
LorentzIndex, except for OPEDelta, which gets converted to type Momentum. \
\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"{", "OPEDelta", "}"}], ",", 
          RowBox[{"{", "a", "}"}]}], "]"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"QuantumField", "[", 
      RowBox[{"A", ",", 
        RowBox[{"Momentum", "[", "OPEDelta", "]"}], ",", 
        RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell["Derivatives of fields are denoted as follows.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuantumField", "[", 
      RowBox[{
        RowBox[{"PartialD", "[", "\[Mu]", "]"}], ",", "A", ",", 
        RowBox[{"{", "\[Mu]", "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["\[PartialD]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], 
        SubsuperscriptBox["A", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[Null]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuantumField", "[", 
      RowBox[{
        RowBox[{"PartialD", "[", "OPEDelta", "]"}], ",", "S"}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["\[PartialD]", 
          FormBox["\[CapitalDelta]",
            "TraditionalForm"]], 
        SubsuperscriptBox["S", "\[Null]", "\[Null]"]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuantumField", "[", 
      RowBox[{
        RowBox[{"PartialD", "[", "OPEDelta", "]"}], ",", "A", ",", 
        RowBox[{"{", "OPEDelta", "}"}], ",", 
        RowBox[{"{", "a", "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          SubscriptBox["\[PartialD]", 
            FormBox["\[CapitalDelta]",
              "TraditionalForm"]],
          "TraditionalForm"], 
        SubsuperscriptBox["A", 
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["a",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuantumField", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"PartialD", "[", "OPEDelta", "]"}], "^", "OPEm"}], ",", 
        "A", ",", 
        RowBox[{"{", "OPEDelta", "}"}], ",", 
        RowBox[{"{", "a", "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubsuperscriptBox["\[PartialD]", 
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox["m",
            "TraditionalForm"]], 
        SubsuperscriptBox["A", 
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["a",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"QuantumField", "[", 
        RowBox[{"QuantumField", "[", "A", "]"}], "]"}], " ", "===", " ", 
      RowBox[{"QuantumField", "[", "A", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox["True", TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["QuarkField", "Subsection",
  CellTags->"QuarkField"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
QuarkField is the name of a fermionic field. QuarkField is just a \
name with no functional properties. Only typesetting rules are attached.\
\>",
   "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["AntiQuarkField",
    ButtonData:>"AntiQuarkField",
    ButtonStyle->"Hyperlink",
    ButtonNote->"AntiQuarkField"],
  ", ",
  ButtonBox["QuantumField",
    ButtonData:>"QuantumField",
    ButtonStyle->"Hyperlink",
    ButtonNote->"QuantumField"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData["QuarkField"], "Input"],

Cell[BoxData[
    FormBox["\[Psi]", TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["QuarkGluonVertex", "Subsection",
  CellTags->"QuarkGluonVertex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "QuarkGluonVertex[",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[Mu]", ",", " ", "a"}], TraditionalForm]]],
  "] gives the Feynman rule for the quark-gluon vertex. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "QuarkGluonVertex", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CounterTerm", "\[Rule]", "False"}], ",", 
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"\[CapitalKoppa]", "\[Rule]", "False"}], ",", 
          RowBox[{"Polarization", "\[Rule]", "0"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["GluonVertex",
    ButtonData:>"GluonVertex",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GluonVertex"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuarkGluonVertex", "[", 
      RowBox[{"\[Mu]", ",", "a"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[ImaginaryI]", " ", 
        SubscriptBox["g", "s"], " ", 
        RowBox[{
          SubscriptBox["T", 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuarkGluonVertex", "[", 
      RowBox[{"\[Mu]", ",", "a", ",", 
        RowBox[{"CounterTerm", " ", "\[Rule]", "1"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"2", " ", "\[ImaginaryI]", " ", 
          RowBox[{"(", 
            RowBox[{
              SubscriptBox["C", "F"], "-", 
              FractionBox[
                SubscriptBox["C", "A"], "2"]}], ")"}], " ", 
          SubsuperscriptBox["g", "s", "3"], " ", 
          SubscriptBox["S", "n"], " ", 
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]]}]}], 
        TagBox["\[CurlyEpsilon]",
          TraditionalForm]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuarkGluonVertex", "[", 
      RowBox[{"\[Mu]", ",", "a", ",", 
        RowBox[{"CounterTerm", " ", "\[Rule]", "2"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"3", " ", "\[ImaginaryI]", " ", 
          SubscriptBox["C", "A"], " ", 
          SubsuperscriptBox["g", "s", "3"], " ", 
          SubscriptBox["S", "n"], " ", 
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]]}]}], 
        TagBox["\[CurlyEpsilon]",
          TraditionalForm]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuarkGluonVertex", "[", 
      RowBox[{"\[Mu]", ",", "a", ",", 
        RowBox[{"CounterTerm", " ", "\[Rule]", "3"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"2", " ", "\[ImaginaryI]", " ", 
          RowBox[{"(", 
            RowBox[{
              SubscriptBox["C", "A"], "+", 
              SubscriptBox["C", "F"]}], ")"}], " ", 
          SubsuperscriptBox["g", "s", "3"], " ", 
          SubscriptBox["S", "n"], " ", 
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]]}]}], 
        TagBox["\[CurlyEpsilon]",
          TraditionalForm]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuarkGluonVertex", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", "q", "}"}], ",", 
        RowBox[{"{", "k", "}"}], ",", 
        RowBox[{"OPE", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"\[ImaginaryI]", " ", 
          SubscriptBox["g", "s"], " ", 
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SuperscriptBox["\[Gamma]", 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]]}]}], "+", 
        RowBox[{
          SubscriptBox["g", "s"], " ", "\[CapitalKoppa]", " ", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"]}], ")"}], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]]}], " ", 
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "2"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", "1"}], ")"}], "i"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["k",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"]}], ")"}], "i"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "2"}]]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuarkGluonVertex", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", "q", "}"}], ",", 
        RowBox[{"{", "k", "}"}], ",", 
        RowBox[{"OPE", "\[Rule]", "False"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[ImaginaryI]", " ", 
        SubscriptBox["g", "s"], " ", 
        RowBox[{
          SubscriptBox["T", 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["QuarkPropagator", "Subsection",
  CellTags->"QuarkPropagator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
QuarkPropagator[p] is the massless quark propagator. \
QuarkPropagator[{p,m}] or gives the quark propagator with mass m.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "QuarkPropagator", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CounterTerm", "\[Rule]", "False"}], ",", 
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Loop", "\[Rule]", "0"}], ",", 
          RowBox[{"\[CapitalKoppa]", "\[Rule]", "False"}], ",", 
          RowBox[{"Polarization", "\[Rule]", "0"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["GluonPropagator",
    ButtonData:>"GluonPropagator",
    ButtonStyle->"Hyperlink",
    ButtonNote->"GluonPropagator"],
  ", ",
  ButtonBox["QuarkGluonVertex",
    ButtonData:>"QuarkGluonVertex",
    ButtonStyle->"Hyperlink",
    ButtonNote->"QuarkGluonVertex"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuarkPropagator", "[", "p", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["p",
              "TraditionalForm"]}]}], 
        SuperscriptBox["p", "2"]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuarkPropagator", "[", 
      RowBox[{"{", 
        RowBox[{"p", ",", "m"}], "}"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"(", 
            RowBox[{"m", "+", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}]}], ")"}]}], 
        RowBox[{
          SuperscriptBox["p", "2"], "-", 
          SuperscriptBox["m", "2"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"QuarkPropagator", "[", 
      RowBox[{"p", ",", "1", ",", "2"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{
            FormBox["\<\"\[Gamma]\"\>",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["p",
              "TraditionalForm"]}]}], 
        SuperscriptBox["p", "2"]], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["RightPartialD", "Subsection",
  CellTags->"RightPartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "RightPartialD[",
  Cell[BoxData[
      FormBox["\[Mu]", TraditionalForm]]],
  "] denotes ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["\[PartialD]", "\[Mu]"], TraditionalForm]]],
  ", acting to the right."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["ExpandPartialD",
    ButtonData:>"ExpandPartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ExpandPartialD"],
  ", ",
  ButtonBox["PartialD",
    ButtonData:>"PartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"PartialD"],
  ", ",
  ButtonBox["LeftPartialD",
    ButtonData:>"LeftPartialD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"LeftPartialD"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"RightPartialD", "[", "\[Mu]", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubscriptBox[
        OverscriptBox["\[PartialD]", "\[RightArrow]"], 
        FormBox[
          FormBox["\[Mu]",
            "TraditionalForm"],
          "TraditionalForm"]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"RightPartialD", "[", "\[Mu]", "]"}], ".", 
      RowBox[{"QuantumField", "[", 
        RowBox[{"A", ",", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["\[PartialD]", "\[RightArrow]"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SubscriptBox["A", "\[Mu]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["\[PartialD]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], 
        SubsuperscriptBox["A", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[Null]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"QuantumField", "[", 
      RowBox[{
        RowBox[{"PartialD", "[", 
          RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", "A", ",", 
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"RightPartialD", "[", "\[Mu]", "]"}], "//", "StandardForm"}]], 
  "Input"],

Cell[BoxData[
    RowBox[{"RightPartialD", "[", 
      RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["RussianTrick", "Subsection",
  CellTags->"RussianTrick"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
RussianTrick[exp, k, {q1,q2,p}] (=RussianTrick[exp,p,p,{q1,q2,p}]) \
does the integration by parts where p is the external momentum. \
RussianTrick[exp, k,l, {q1,q2,p}] (=RussianTrick[exp,k,l]) does integration \
by parts where l is the momentum to be differentiated.\
\>", "Text"],

Cell["The result is an expression which is vanishing.", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FourDivergence",
    ButtonData:>"FourDivergence",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FourDivergence"],
  ", ",
  ButtonBox["FourLaplacian",
    ButtonData:>"FourLaplacian",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FourLaplacian"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{"RHI", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{
            "OPEm", ",", "0", ",", " ", "0", ",", "0", ",", " ", "0"}], 
            "}"}], ",", " ", 
          RowBox[{"{", 
            RowBox[{
            "1", ",", " ", "1", ",", " ", "1", ",", " ", "1", ",", " ", 
              "1"}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox[
        StyleBox["T",
          FontFamily->"Times",
          FontWeight->"Bold"], 
        StyleBox[
          RowBox[{"1", "1", "1", "1", "1"}],
          FontColor->RGBColor[0, 0, 1]], 
        RowBox[{
          FormBox["m",
            "TraditionalForm"], 
          FormBox["0",
            "TraditionalForm"], 
          FormBox["0",
            "TraditionalForm"], 
          FormBox["0",
            "TraditionalForm"], 
          FormBox["0",
            "TraditionalForm"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "//", "RHI2FC"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{
              FormBox["\[CapitalDelta]",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox[
                InterpretationBox[
                  SubscriptBox["q", "1"],
                  q1],
                "TraditionalForm"]}], ")"}], "m"], 
        RowBox[{
          SuperscriptBox[
            InterpretationBox[
              SubscriptBox["q", "1"],
              q1], "2"], ".", 
          SuperscriptBox[
            InterpretationBox[
              SubscriptBox["q", "2"],
              q2], "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                InterpretationBox[
                  SubscriptBox["q", "1"],
                  q1], "-", "p"}], ")"}], "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                InterpretationBox[
                  SubscriptBox["q", "2"],
                  q2], "-", "p"}], ")"}], "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                InterpretationBox[
                  SubscriptBox["q", "1"],
                  q1], "-", 
                InterpretationBox[
                  SubscriptBox["q", "2"],
                  q2]}], ")"}], "2"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"RussianTrick", "[", 
      RowBox[{
        RowBox[{"%", "//", "RHI2FC"}], ",", "q2"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox[
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    InterpretationBox[
                      SubscriptBox["q", "1"],
                      q1],
                    "TraditionalForm"]}], ")"}], "m"], 
            RowBox[{
              SuperscriptBox[
                InterpretationBox[
                  SubscriptBox["q", "1"],
                  q1], "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    InterpretationBox[
                      SubscriptBox["q", "1"],
                      q1], "-", "p"}], ")"}], "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    InterpretationBox[
                      SubscriptBox["q", "2"],
                      q2], "-", "p"}], ")"}], "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    InterpretationBox[
                      SubscriptBox["q", "2"],
                      q2], "-", 
                    InterpretationBox[
                      SubscriptBox["q", "1"],
                      q1]}], ")"}], "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    InterpretationBox[
                      SubscriptBox["q", "2"],
                      q2], "-", 
                    InterpretationBox[
                      SubscriptBox["q", "1"],
                      q1]}], ")"}], "2"]}]]}], "-", 
        FractionBox[
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  InterpretationBox[
                    SubscriptBox["q", "1"],
                    q1],
                  "TraditionalForm"]}], ")"}], "m"], 
          RowBox[{
            SuperscriptBox[
              InterpretationBox[
                SubscriptBox["q", "1"],
                q1], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  InterpretationBox[
                    SubscriptBox["q", "1"],
                    q1], "-", "p"}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  InterpretationBox[
                    SubscriptBox["q", "2"],
                    q2], "-", 
                  InterpretationBox[
                    SubscriptBox["q", "1"],
                    q1]}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  InterpretationBox[
                    SubscriptBox["q", "2"],
                    q2], "-", "p"}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  InterpretationBox[
                    SubscriptBox["q", "2"],
                    q2], "-", "p"}], ")"}], "2"]}]], "-", 
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{"4", "-", "D"}], ")"}], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    InterpretationBox[
                      SubscriptBox["q", "1"],
                      q1],
                    "TraditionalForm"]}], ")"}], "m"]}], 
          RowBox[{
            SuperscriptBox[
              InterpretationBox[
                SubscriptBox["q", "1"],
                q1], "2"], ".", 
            SuperscriptBox[
              InterpretationBox[
                SubscriptBox["q", "2"],
                q2], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  InterpretationBox[
                    SubscriptBox["q", "1"],
                    q1], "-", "p"}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  InterpretationBox[
                    SubscriptBox["q", "2"],
                    q2], "-", 
                  InterpretationBox[
                    SubscriptBox["q", "1"],
                    q1]}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  InterpretationBox[
                    SubscriptBox["q", "2"],
                    q2], "-", "p"}], ")"}], "2"]}]], "+", 
        FractionBox[
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  InterpretationBox[
                    SubscriptBox["q", "1"],
                    q1],
                  "TraditionalForm"]}], ")"}], "m"], 
          RowBox[{
            SuperscriptBox[
              InterpretationBox[
                SubscriptBox["q", "2"],
                q2], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  InterpretationBox[
                    SubscriptBox["q", "1"],
                    q1], "-", "p"}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  InterpretationBox[
                    SubscriptBox["q", "2"],
                    q2], "-", "p"}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  InterpretationBox[
                    SubscriptBox["q", "2"],
                    q2], "-", 
                  InterpretationBox[
                    SubscriptBox["q", "1"],
                    q1]}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  InterpretationBox[
                    SubscriptBox["q", "2"],
                    q2], "-", 
                  InterpretationBox[
                    SubscriptBox["q", "1"],
                    q1]}], ")"}], "2"]}]], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    InterpretationBox[
                      SubscriptBox["q", "1"],
                      q1],
                    "TraditionalForm"]}], ")"}], "m"], " ", 
            SuperscriptBox[
              FormBox["p",
                "TraditionalForm"], "2"]}], 
          RowBox[{
            SuperscriptBox[
              InterpretationBox[
                SubscriptBox["q", "2"],
                q2], "2"], ".", 
            SuperscriptBox[
              InterpretationBox[
                SubscriptBox["q", "1"],
                q1], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  InterpretationBox[
                    SubscriptBox["q", "2"],
                    q2], "-", 
                  InterpretationBox[
                    SubscriptBox["q", "1"],
                    q1]}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  InterpretationBox[
                    SubscriptBox["q", "1"],
                    q1], "-", "p"}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  InterpretationBox[
                    SubscriptBox["q", "2"],
                    q2], "-", "p"}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  InterpretationBox[
                    SubscriptBox["q", "2"],
                    q2], "-", "p"}], ")"}], "2"]}]]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FC2RHI", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          SubsuperscriptBox[
            StyleBox["T",
              FontFamily->"Times",
              FontWeight->"Bold"], 
            StyleBox[
              RowBox[{"0", "1", "1", "1", "2"}],
              FontColor->RGBColor[0, 0, 1]], 
            RowBox[{
              FormBox["m",
                "TraditionalForm"], 
              FormBox["0",
                "TraditionalForm"], 
              FormBox["0",
                "TraditionalForm"], 
              FormBox["0",
                "TraditionalForm"], 
              FormBox["0",
                "TraditionalForm"]}]],
          "TraditionalForm"], "-", 
        FormBox[
          SubsuperscriptBox[
            StyleBox["T",
              FontFamily->"Times",
              FontWeight->"Bold"], 
            StyleBox[
              RowBox[{"1", "0", "1", "1", "2"}],
              FontColor->RGBColor[0, 0, 1]], 
            RowBox[{
              FormBox["m",
                "TraditionalForm"], 
              FormBox["0",
                "TraditionalForm"], 
              FormBox["0",
                "TraditionalForm"], 
              FormBox["0",
                "TraditionalForm"], 
              FormBox["0",
                "TraditionalForm"]}]],
          "TraditionalForm"], "-", 
        FormBox[
          SubsuperscriptBox[
            StyleBox["T",
              FontFamily->"Times",
              FontWeight->"Bold"], 
            StyleBox[
              RowBox[{"1", "0", "1", "2", "1"}],
              FontColor->RGBColor[0, 0, 1]], 
            RowBox[{
              FormBox["m",
                "TraditionalForm"], 
              FormBox["0",
                "TraditionalForm"], 
              FormBox["0",
                "TraditionalForm"], 
              FormBox["0",
                "TraditionalForm"], 
              FormBox["0",
                "TraditionalForm"]}]],
          "TraditionalForm"], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{"D", "-", "4"}], ")"}], " ", 
          FormBox[
            SubsuperscriptBox[
              StyleBox["T",
                FontFamily->"Times",
                FontWeight->"Bold"], 
              StyleBox[
                RowBox[{"1", "1", "1", "1", "1"}],
                FontColor->RGBColor[0, 0, 1]], 
              RowBox[{
                FormBox["m",
                  "TraditionalForm"], 
                FormBox["0",
                  "TraditionalForm"], 
                FormBox["0",
                  "TraditionalForm"], 
                FormBox["0",
                  "TraditionalForm"], 
                FormBox["0",
                  "TraditionalForm"]}]],
            "TraditionalForm"]}], "+", 
        RowBox[{
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"], " ", 
          FormBox[
            SubsuperscriptBox[
              StyleBox["T",
                FontFamily->"Times",
                FontWeight->"Bold"], 
              StyleBox[
                RowBox[{"1", "1", "1", "2", "1"}],
                FontColor->RGBColor[0, 0, 1]], 
              RowBox[{
                FormBox["m",
                  "TraditionalForm"], 
                FormBox["0",
                  "TraditionalForm"], 
                FormBox["0",
                  "TraditionalForm"], 
                FormBox["0",
                  "TraditionalForm"], 
                FormBox["0",
                  "TraditionalForm"]}]],
            "TraditionalForm"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Solve2", "[", 
      RowBox[{"%", ",", "t"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          FormBox[
            SubsuperscriptBox[
              StyleBox["T",
                FontFamily->"Times",
                FontWeight->"Bold"], 
              StyleBox[
                RowBox[{"1", "1", "1", "1", "1"}],
                FontColor->RGBColor[0, 0, 1]], 
              RowBox[{
                FormBox["m",
                  "TraditionalForm"], 
                FormBox["0",
                  "TraditionalForm"], 
                FormBox["0",
                  "TraditionalForm"], 
                FormBox["0",
                  "TraditionalForm"], 
                FormBox["0",
                  "TraditionalForm"]}]],
            "TraditionalForm"], "\[Rule]", 
          FractionBox[
            RowBox[{
              FormBox[
                SubsuperscriptBox[
                  StyleBox["T",
                    FontFamily->"Times",
                    FontWeight->"Bold"], 
                  StyleBox[
                    RowBox[{"0", "1", "1", "1", "2"}],
                    FontColor->RGBColor[0, 0, 1]], 
                  RowBox[{
                    FormBox["m",
                      "TraditionalForm"], 
                    FormBox["0",
                      "TraditionalForm"], 
                    FormBox["0",
                      "TraditionalForm"], 
                    FormBox["0",
                      "TraditionalForm"], 
                    FormBox["0",
                      "TraditionalForm"]}]],
                "TraditionalForm"], "-", 
              FormBox[
                SubsuperscriptBox[
                  StyleBox["T",
                    FontFamily->"Times",
                    FontWeight->"Bold"], 
                  StyleBox[
                    RowBox[{"1", "0", "1", "1", "2"}],
                    FontColor->RGBColor[0, 0, 1]], 
                  RowBox[{
                    FormBox["m",
                      "TraditionalForm"], 
                    FormBox["0",
                      "TraditionalForm"], 
                    FormBox["0",
                      "TraditionalForm"], 
                    FormBox["0",
                      "TraditionalForm"], 
                    FormBox["0",
                      "TraditionalForm"]}]],
                "TraditionalForm"], "-", 
              FormBox[
                SubsuperscriptBox[
                  StyleBox["T",
                    FontFamily->"Times",
                    FontWeight->"Bold"], 
                  StyleBox[
                    RowBox[{"1", "0", "1", "2", "1"}],
                    FontColor->RGBColor[0, 0, 1]], 
                  RowBox[{
                    FormBox["m",
                      "TraditionalForm"], 
                    FormBox["0",
                      "TraditionalForm"], 
                    FormBox["0",
                      "TraditionalForm"], 
                    FormBox["0",
                      "TraditionalForm"], 
                    FormBox["0",
                      "TraditionalForm"]}]],
                "TraditionalForm"], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"], " ", 
                FormBox[
                  SubsuperscriptBox[
                    StyleBox["T",
                      FontFamily->"Times",
                      FontWeight->"Bold"], 
                    StyleBox[
                      RowBox[{"1", "1", "1", "2", "1"}],
                      FontColor->RGBColor[0, 0, 1]], 
                    RowBox[{
                      FormBox["m",
                        "TraditionalForm"], 
                      FormBox["0",
                        "TraditionalForm"], 
                      FormBox["0",
                        "TraditionalForm"], 
                      FormBox["0",
                        "TraditionalForm"], 
                      FormBox["0",
                        "TraditionalForm"]}]],
                  "TraditionalForm"]}]}], 
            RowBox[{"4", "-", "D"}]]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", "t", "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["ScalarProduct", "Subsection",
  CellTags->"ScalarProduct"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "ScalarProduct[p, q] is the input for scalar product. ScalarProduct[p] is \
equivalent to ScalarProduct[p, p]. Expansion of sums of momenta in \
ScalarProduct is done with ExpandScalarProduct. Scalar product may be set, \
e.g., ScalarProduct[a, b] = ",
  Cell[BoxData[
      FormBox[
        RowBox[{"m", "^", "2"}], TraditionalForm]]],
  "; but a and b must not contain sums. Note that ScalarProduct[a, b] = ",
  Cell[BoxData[
      FormBox[
        RowBox[{"m", "^", "2"}], TraditionalForm]]],
  " actually sets also: Pair[Momentum[a, ___], Momentum[b, ___]] = ",
  Cell[BoxData[
      FormBox[
        RowBox[{"m", "^", "2"}], TraditionalForm]]],
  ". It is encouraged to always set ScalarProduct's ",
  StyleBox["before",
    FontWeight->"Bold"],
  " any calculation. This improves the performance of FeynCalc ."
}], "Text"],

Cell[TextData[{
  "See also: ",
  ButtonBox["Calc",
    ButtonData:>"Calc",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Calc"],
  ", ",
  ButtonBox["ClearScalarProducts",
    ButtonData:>"ClearScalarProducts",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ClearScalarProducts"],
  ", ",
  ButtonBox["ExpandScalarProduct",
    ButtonData:>"ExpandScalarProduct",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ExpandScalarProduct"],
  ", ",
  ButtonBox["Pair",
    ButtonData:>"Pair",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Pair"],
  ", ",
  ButtonBox["SP",
    ButtonData:>"SP",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SP"],
  ", ",
  ButtonBox["SPD",
    ButtonData:>"SPD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SPD"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ScalarProduct", "[", 
      RowBox[{"p", ",", "q"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["p",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        FormBox["q",
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ScalarProduct", "[", 
      RowBox[{
        RowBox[{"p", "+", "q"}], ",", 
        RowBox[{"-", "q"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        RowBox[{"(", 
          RowBox[{
            FormBox["q",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
            "(", "\[NoBreak]", 
            FormBox[
              RowBox[{"p", "+", "q"}],
              "TraditionalForm"], "\[NoBreak]", ")"}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ScalarProduct", "[", 
      RowBox[{"p", ",", "p"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        FormBox["p",
          "TraditionalForm"], "2"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ScalarProduct", "[", "q", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox[
        FormBox["q",
          "TraditionalForm"], "2"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"ScalarProduct", "[", 
        RowBox[{"p", ",", "q"}], "]"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", 
        RowBox[{"Momentum", "[", "q", "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"ScalarProduct", "[", 
        RowBox[{"p", ",", "q", ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", 
          RowBox[{"p", ",", "D"}], "]"}], ",", 
        RowBox[{"Momentum", "[", 
          RowBox[{"q", ",", "D"}], "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"ScalarProduct", "[", 
        RowBox[{
          SubscriptBox["p", "1"], ",", 
          SubscriptBox["p", "2"]}], "]"}], " ", "=", " ", 
      RowBox[{"s", "/", "2"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox["s", "2"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandScalarProduct", "[", " ", 
      RowBox[{"ScalarProduct", "[", 
        RowBox[{
          RowBox[{
            SubscriptBox["p", "1"], "-", "q"}], ",", 
          RowBox[{
            SubscriptBox["p", "2"], "-", "k"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["s", "2"], "+", 
        RowBox[{
          FormBox["k",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}], "-", 
        RowBox[{
          FormBox["k",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox[
            SubscriptBox["p", "1"],
            "TraditionalForm"]}], "-", 
        RowBox[{
          FormBox["q",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox[
            SubscriptBox["p", "2"],
            "TraditionalForm"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", " ", 
      RowBox[{"ScalarProduct", "[", 
        RowBox[{
          RowBox[{
            SubscriptBox["p", "1"], "-", "q"}], ",", 
          RowBox[{
            SubscriptBox["p", "2"], "-", "k"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["s", "2"], "+", 
        RowBox[{
          FormBox["k",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}], "-", 
        RowBox[{
          FormBox["k",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox[
            SubscriptBox["p", "1"],
            "TraditionalForm"]}], "-", 
        RowBox[{
          FormBox["q",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox[
            SubscriptBox["p", "2"],
            "TraditionalForm"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData["ClearScalarProducts"], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["ScalarProductCancel", "Subsection",
  CellTags->"ScalarProductCancel"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
ScalarProductCancel[exp, q1, q2, ...] cancels scalar products with \
propagators. ScalarProductCancel[exp] cancels simple cases.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData["SPC"], "Input"],

Cell[BoxData[
    FormBox["ScalarProductCancel", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ScalarProductCancel", "//", "Options"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"ChangeDimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Collecting", "\[Rule]", "True"}], ",", 
          RowBox[{"FeynAmpDenominatorSimplify", "\[Rule]", "False"}], ",", 
          RowBox[{"FeynAmpDenominatorCombine", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"?", "SPC"}]], "Input"],

Cell[BoxData[
    FormBox[
    "\<\"SPC is an abbreviation for ScalarProductCancel.\"\>", 
      TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FeynAmpDenominatorSimplify",
    ButtonData:>"FeynAmpDenominatorSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FeynAmpDenominatorSimplify"],
  "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", " ", "=", " ", 
      RowBox[{
        RowBox[{
          RowBox[{"SPD", "[", 
            RowBox[{"q", ",", "p"}], "]"}], " ", 
          RowBox[{"FAD", "[", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"q", ",", "m"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{
                  RowBox[{"q", "-", "p"}], ",", "0"}], "}"}]}], "]"}]}], 
        "//", "FCI"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}], 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox["q", "2"], "-", 
              SuperscriptBox["m", "2"]}], ")"}], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"q", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ScalarProductCancel", "[", 
      RowBox[{"t1", ",", "q"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", 
            RowBox[{"2", " ", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["q", "2"], "-", 
                  SuperscriptBox["m", "2"]}], ")"}]}]]}], "+", 
        FractionBox["1", 
          RowBox[{"2", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "+", 
        FractionBox[
          RowBox[{
            FractionBox[
              SuperscriptBox["m", "2"], "2"], "+", 
            FractionBox[
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"], "2"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox["q", "2"], "-", 
                SuperscriptBox["m", "2"]}], ")"}], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"]}]]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FeynAmpDenominatorSimplify", "[", 
      RowBox[{"%", ",", "q"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            FractionBox[
              SuperscriptBox["m", "2"], "2"], "+", 
            FractionBox[
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"], "2"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox["q", "2"], "-", 
                SuperscriptBox["m", "2"]}], ")"}], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
        FractionBox["1", 
          RowBox[{"2", " ", 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox["q", "2"], "-", 
                SuperscriptBox["m", "2"]}], ")"}]}]]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SPC", "[", 
      RowBox[{"t1", ",", "q", ",", 
        RowBox[{"FDS", "->", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            FractionBox[
              SuperscriptBox["m", "2"], "2"], "+", 
            FractionBox[
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"], "2"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox["q", "2"], "-", 
                SuperscriptBox["m", "2"]}], ")"}], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
        FractionBox["1", 
          RowBox[{"2", " ", 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox["q", "2"], "-", 
                SuperscriptBox["m", "2"]}], ")"}]}]]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", " ", "=", " ", 
      RowBox[{
        RowBox[{
          RowBox[{"SPD", "[", 
            RowBox[{"q2", ",", "p"}], "]"}], 
          RowBox[{"SPD", "[", 
            RowBox[{"q1", ",", "p"}], "]"}], " ", 
          RowBox[{"FAD", "[", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"q1", ",", "m"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"q2", ",", "m"}], "}"}], ",", 
              RowBox[{"q1", "-", "p"}], ",", 
              RowBox[{"q2", "-", "p"}], ",", 
              RowBox[{"q2", "-", "q1"}]}], "]"}]}], "//", "FCI"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox[
              InterpretationBox[
                SubscriptBox["q", "1"],
                q1],
              "TraditionalForm"]}], " ", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox[
              InterpretationBox[
                SubscriptBox["q", "2"],
                q2],
              "TraditionalForm"]}]}], 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox[
                InterpretationBox[
                  SubscriptBox["q", "1"],
                  q1], "2"], "-", 
              SuperscriptBox["m", "2"]}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox[
                InterpretationBox[
                  SubscriptBox["q", "2"],
                  q2], "2"], "-", 
              SuperscriptBox["m", "2"]}], ")"}], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                InterpretationBox[
                  SubscriptBox["q", "1"],
                  q1], "-", "p"}], ")"}], "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                InterpretationBox[
                  SubscriptBox["q", "2"],
                  q2], "-", "p"}], ")"}], "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                InterpretationBox[
                  SubscriptBox["q", "2"],
                  q2], "-", 
                InterpretationBox[
                  SubscriptBox["q", "1"],
                  q1]}], ")"}], "2"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SPC", "[", 
      RowBox[{"t2", ",", "q1", ",", "q2", ",", 
        RowBox[{"FDS", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", 
          RowBox[{"4", " ", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox[
                    InterpretationBox[
                      SubscriptBox["q", "1"],
                      q1], "2"], "-", 
                  SuperscriptBox["m", "2"]}], ")"}], ".", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox[
                    InterpretationBox[
                      SubscriptBox["q", "2"],
                      q2], "2"], "-", 
                  SuperscriptBox["m", "2"]}], ")"}], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    InterpretationBox[
                      SubscriptBox["q", "2"],
                      q2], "-", 
                    InterpretationBox[
                      SubscriptBox["q", "1"],
                      q1]}], ")"}], "2"]}]}]], "-", 
        FractionBox["1", 
          RowBox[{"2", " ", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox[
                    InterpretationBox[
                      SubscriptBox["q", "1"],
                      q1], "2"], "-", 
                  SuperscriptBox["m", "2"]}], ")"}], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    InterpretationBox[
                      SubscriptBox["q", "2"],
                      q2], "-", "p"}], ")"}], "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    InterpretationBox[
                      SubscriptBox["q", "2"],
                      q2], "-", 
                    InterpretationBox[
                      SubscriptBox["q", "1"],
                      q1]}], ")"}], "2"]}]}]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{"-", 
              FractionBox[
                SuperscriptBox["m", "2"], "2"]}], "-", 
            FractionBox[
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"], "2"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox[
                  InterpretationBox[
                    SubscriptBox["q", "1"],
                    q1], "2"], "-", 
                SuperscriptBox["m", "2"]}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox[
                  InterpretationBox[
                    SubscriptBox["q", "2"],
                    q2], "2"], "-", 
                SuperscriptBox["m", "2"]}], ")"}], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  InterpretationBox[
                    SubscriptBox["q", "1"],
                    q1], "-", "p"}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  InterpretationBox[
                    SubscriptBox["q", "2"],
                    q2], "-", 
                  InterpretationBox[
                    SubscriptBox["q", "1"],
                    q1]}], ")"}], "2"]}]], "+", 
        FractionBox[
          RowBox[{
            FractionBox[
              SuperscriptBox["m", "2"], "2"], "+", 
            FractionBox[
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"], "2"]}], 
          RowBox[{
            SuperscriptBox[
              InterpretationBox[
                SubscriptBox["q", "1"],
                q1], "2"], ".", 
            SuperscriptBox[
              InterpretationBox[
                SubscriptBox["q", "2"],
                q2], "2"], ".", 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        SubscriptBox["q", "1"],
                        q1], "-", "p"}], ")"}], "2"], "-", 
                SuperscriptBox["m", "2"]}], ")"}], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  InterpretationBox[
                    SubscriptBox["q", "2"],
                    q2], "-", 
                  InterpretationBox[
                    SubscriptBox["q", "1"],
                    q1]}], ")"}], "2"]}]], "+", 
        FractionBox[
          RowBox[{
            FractionBox[
              SuperscriptBox["m", "4"], "4"], "+", 
            FractionBox[
              RowBox[{
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"], " ", 
                SuperscriptBox["m", "2"]}], "2"], "+", 
            FractionBox[
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "4"], "4"]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox[
                  InterpretationBox[
                    SubscriptBox["q", "1"],
                    q1], "2"], "-", 
                SuperscriptBox["m", "2"]}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox[
                  InterpretationBox[
                    SubscriptBox["q", "2"],
                    q2], "2"], "-", 
                SuperscriptBox["m", "2"]}], ")"}], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  InterpretationBox[
                    SubscriptBox["q", "1"],
                    q1], "-", "p"}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  InterpretationBox[
                    SubscriptBox["q", "2"],
                    q2], "-", "p"}], ")"}], "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  InterpretationBox[
                    SubscriptBox["q", "2"],
                    q2], "-", 
                  InterpretationBox[
                    SubscriptBox["q", "1"],
                    q1]}], ")"}], "2"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", 
        RowBox[{"t1", ",", "t2"}], "]"}], ";"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["SD", "Subsection",
  CellTags->"SD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
SD[i, j] is the (FeynCalc-external) Kronecker-delta for SU(N) with \
color indices i and j. SD[i,j] is transformed into \
SUNDelta[SUNIndex[i],SUNIndex[j]] by FeynCalcInternal.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["SUNDelta",
    ButtonData:>"SUNDelta",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SUNDelta"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SD", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubscriptBox["\[Delta]", 
        RowBox[{"a", "\[NoBreak]", "b"}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"%", "//", "FCI"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"SUNDelta", "[", 
      RowBox[{
        RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
        RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"%", "//", "FCE"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"SD", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["SelectFree", "Subsection",
  CellTags->"SelectFree"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
SelectFree[expr, a, b, ...] is equivalent to Select[expr, FreeQ2[#, \
{a,b, ...}]&], except the special cases: SelectFree[a, b] returns a and \
SelectFree[a,a] returns 1 (where a is not a product or a sum).\
\>", "Text"],

Cell["SelectFree is equivalent to Select1.", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FreeQ2",
    ButtonData:>"FreeQ2",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FreeQ2"],
  ", ",
  ButtonBox["SelectNotFree",
    ButtonData:>"SelectNotFree",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SelectNotFree"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectFree", "[", 
      RowBox[{
        RowBox[{"a", "+", "b", "+", 
          RowBox[{"f", "[", "a", "]"}], "+", "d"}], ",", "a"}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"b", "+", "d"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectFree", "[", 
      RowBox[{
        RowBox[{"x", " ", "y"}], ",", " ", "x"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["y", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectFree", "[", 
      RowBox[{
        RowBox[{"2", " ", "x", " ", "y", " ", "z", " ", 
          RowBox[{"f", "[", "x", "]"}]}], ",", " ", 
        RowBox[{"{", 
          RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"2", " ", "z"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectFree", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["a", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectFree", "[", 
      RowBox[{"a", ",", "a"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectFree", "[", 
      RowBox[{"1", ",", "c"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectFree", "[", 
      RowBox[{
        RowBox[{"f", "[", "x", "]"}], ",", "x"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["SelectNotFree", "Subsection",
  CellTags->"SelectNotFree"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
SelectNotFree[expr, x] returns that part of expr which is not free \
of any occurance of x.\
\>", "Text"],

Cell["\<\
SelectNotFree[expr, a, b, ...] is equivalent to Select[expr, \
!FreeQ2[#, {a, b, ...}]&], except the special cases: SelectNotFree[a, b] \
returns 1 and SelectNotFree[a, a] returns a (where a is not a product or a \
sum).\
\>", "Text"],

Cell["SelectNotFree is equivalent to Select2.", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["FreeQ2",
    ButtonData:>"FreeQ2",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FreeQ2"],
  ", ",
  ButtonBox["SelectFree",
    ButtonData:>"SelectFree",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SelectFree"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectNotFree", "[", 
      RowBox[{
        RowBox[{"a", "+", "b", "+", 
          RowBox[{"f", "[", "a", "]"}]}], ",", "a"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"a", "+", 
        RowBox[{"f", "(", "a", ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectNotFree", "[", 
      RowBox[{
        RowBox[{"2", " ", "x", " ", "y", " ", 
          RowBox[{"f", "[", "x", "]"}], " ", "z"}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"x", " ", "y", " ", 
        RowBox[{"f", "(", "x", ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectNotFree", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectNotFree", "[", 
      RowBox[{
        RowBox[{"a", "+", "x"}], ",", "b"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectNotFree", "[", 
      RowBox[{"a", ",", "a"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["a", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectNotFree", "[", 
      RowBox[{"1", ",", "c"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["1", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SelectNotFree", "[", 
      RowBox[{
        RowBox[{"f", "[", "x", "]"}], ",", "x"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"f", "(", "x", ")"}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Series2", "Subsection",
  CellTags->"Series2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
 Series2 performs a series expansion around 0. Series2 is (up to \
the Gamma - bug) equivalent to Series, except that it applies Normal on the \
result and has an option FinalSubstitutions. Series2[f, e, n] is equivalent \
to Series2[f, {e, 0, n}].\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Series2", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Collecting", "\[Rule]", "False"}], ",", 
          RowBox[{"Factoring", "\[Rule]", "True"}], ",", 
          RowBox[{"FinalSubstitutions", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{
                TagBox["\[Gamma]",
                  EulerGamma], "\[Rule]", "0"}], "}"}]}], ",", 
          RowBox[{"Print", "\[Rule]", "False"}]}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Series2", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{"x", " ", 
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}]}], ")"}], "^", 
          RowBox[{"(", 
            RowBox[{"\[Delta]", "/", "2"}], ")"}]}], ",", "\[Delta]", ",", 
        "1"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "2"], " ", "\[Delta]", " ", 
          RowBox[{"log", "(", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "+", 
        "1"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Series2", "[", 
      RowBox[{
        RowBox[{"Gamma", "[", "x", "]"}], ",", "x", ",", "1"}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SuperscriptBox["\[Pi]", "2"], " ", "x"}], "12"], "+", 
        FractionBox["1", "x"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Series", "[", 
      RowBox[{
        RowBox[{"Gamma", "[", "x", "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        RowBox[{
          FractionBox["1", "x"], "+", 
          InterpretationBox[
            RowBox[{"O", "(", 
              SuperscriptBox["x", "0"], ")"}],
            SeriesData[ x, 0, {}, -1, 0, 1]]}],
        SeriesData[ x, 0, {1}, -1, 0, 1]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Series2", "[", 
      RowBox[{
        RowBox[{"Gamma", "[", "x", "]"}], ",", "x", ",", "2"}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            FractionBox["1", "3"]}], " ", 
          TagBox[
            RowBox[{"\[Zeta]", "(", 
              TagBox["3",
                (Editable -> True)], ")"}],
            InterpretTemplate[ Zeta[ #]&]], " ", 
          SuperscriptBox["x", "2"]}], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox["\[Pi]", "2"], " ", "x"}], "12"], "+", 
        FractionBox["1", "x"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Series2", "[", 
        RowBox[{
          RowBox[{"Gamma", "[", "x", "]"}], ",", "x", ",", "2", ",", 
          RowBox[{"FinalSubstitutions", "\[Rule]", 
            RowBox[{"{", "}"}]}]}], "]"}], "//", "FullSimplify"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          RowBox[{
            RowBox[{"-", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"2", " ", 
                    SuperscriptBox[
                      TagBox["\[Gamma]",
                        EulerGamma], "3"]}], "+", 
                  RowBox[{
                    TagBox["\[Gamma]",
                      EulerGamma], " ", 
                    SuperscriptBox["\[Pi]", "2"]}], "+", 
                  RowBox[{"4", " ", 
                    TagBox[
                      RowBox[{"\[Zeta]", "(", 
                        TagBox["3",
                          (Editable -> True)], ")"}],
                      InterpretTemplate[ Zeta[ #]&]]}]}], ")"}]}], " ", 
            SuperscriptBox["x", "3"]}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{"6", " ", 
                  SuperscriptBox[
                    TagBox["\[Gamma]",
                      EulerGamma], "2"]}], "+", 
                SuperscriptBox["\[Pi]", "2"]}], ")"}], " ", 
            SuperscriptBox["x", "2"]}], "-", 
          RowBox[{"12", " ", 
            TagBox["\[Gamma]",
              EulerGamma], " ", "x"}], "+", "12"}], 
        RowBox[{"12", " ", "x"}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"Series", "[", 
          RowBox[{
            RowBox[{"Gamma", "[", "x", "]"}], ",", 
            RowBox[{"{", 
              RowBox[{"x", ",", "0", ",", "4"}], "}"}]}], "]"}], "//", 
        "Normal"}], "//", "FullSimplify"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          RowBox[{
            RowBox[{"-", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"2", " ", 
                    SuperscriptBox[
                      TagBox["\[Gamma]",
                        EulerGamma], "3"]}], "+", 
                  RowBox[{
                    TagBox["\[Gamma]",
                      EulerGamma], " ", 
                    SuperscriptBox["\[Pi]", "2"]}], "+", 
                  RowBox[{"4", " ", 
                    TagBox[
                      RowBox[{"\[Zeta]", "(", 
                        TagBox["3",
                          (Editable -> True)], ")"}],
                      InterpretTemplate[ Zeta[ #]&]]}]}], ")"}]}], " ", 
            SuperscriptBox["x", "3"]}], "+", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{"6", " ", 
                  SuperscriptBox[
                    TagBox["\[Gamma]",
                      EulerGamma], "2"]}], "+", 
                SuperscriptBox["\[Pi]", "2"]}], ")"}], " ", 
            SuperscriptBox["x", "2"]}], "-", 
          RowBox[{"12", " ", 
            TagBox["\[Gamma]",
              EulerGamma], " ", "x"}], "+", "12"}], 
        RowBox[{"12", " ", "x"}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
There is a table of expansions of special hypergeometric functions.\
\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Series2", "[", 
      RowBox[{
        RowBox[{"HypergeometricPFQ", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"1", ",", 
                RowBox[{"OPEm", "-", "1"}], ",", 
                RowBox[{
                  RowBox[{"Epsilon", "/", "2"}], "+", "OPEm"}]}], "}"}], 
            ",", 
            RowBox[{"{", 
              RowBox[{"OPEm", ",", 
                RowBox[{"OPEm", "+", "Epsilon"}]}], "}"}], ",", "1"}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"Epsilon", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "2"], " ", 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm], " ", 
          RowBox[{
            SuperscriptBox[
              TagBox["\[Psi]",
                PolyGamma], 
              RowBox[{"(", "1", ")"}]], "(", "m", ")"}], " ", 
          RowBox[{"(", 
            RowBox[{"m", "-", "1"}], ")"}]}], "+", 
        FractionBox[
          RowBox[{"2", " ", 
            RowBox[{"(", 
              RowBox[{"m", "-", "1"}], ")"}]}], 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm]], "+", "1"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{" ", 
      RowBox[{"Series2", "[", 
        RowBox[{
          RowBox[{"HypergeometricPFQ", "[", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"1", ",", " ", "OPEm", ",", " ", 
                  RowBox[{
                    RowBox[{"Epsilon", "/", "2"}], " ", "+", " ", "OPEm"}]}], 
                "}"}], ",", " ", 
              RowBox[{"{", 
                RowBox[{
                  RowBox[{"1", " ", "+", " ", "OPEm"}], ",", " ", 
                  RowBox[{"Epsilon", " ", "+", " ", "OPEm"}]}], "}"}], ",", 
              " ", " ", "1"}], "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"Epsilon", ",", "0", ",", "1"}], "}"}]}], "]"}]}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "4"], " ", 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm], " ", "m", " ", 
          SuperscriptBox[
            RowBox[{
              SuperscriptBox[
                TagBox["\[Psi]",
                  PolyGamma], 
                RowBox[{"(", "0", ")"}]], "(", "m", ")"}], "2"]}], "+", 
        FractionBox[
          RowBox[{"2", " ", "m"}], 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm]], "+", 
        RowBox[{
          FractionBox["3", "4"], " ", 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm], " ", "m", " ", 
          RowBox[{
            SuperscriptBox[
              TagBox["\[Psi]",
                PolyGamma], 
              RowBox[{"(", "1", ")"}]], "(", "m", ")"}]}], "-", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm], " ", "m", " ", 
          RowBox[{
            SubscriptBox["S", "11"], "(", 
            FormBox[
              RowBox[{"m", "-", "1"}],
              "TraditionalForm"], ")"}]}], "+", 
        RowBox[{
          FractionBox["1", "24"], " ", 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm], " ", "m", " ", 
          SuperscriptBox["\[Pi]", "2"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Hypergeometric2F1", "[", 
      RowBox[{"1", ",", " ", "Epsilon", ",", " ", 
        RowBox[{"1", " ", "+", " ", 
          RowBox[{"2", " ", "Epsilon"}]}], ",", "x"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      TagBox[
        RowBox[{
          RowBox[{
            SubscriptBox["\[ThinSpace]", "2"], 
            SubscriptBox["F", "1"]}], "(", 
          RowBox[{
            RowBox[{
              TagBox["1",
                (Editable -> True)], ",", 
              TagBox[
                TagBox["\[CurlyEpsilon]",
                  TraditionalForm],
                (Editable -> True)]}], ";", 
            TagBox[
              RowBox[{
                RowBox[{"2", " ", 
                  TagBox["\[CurlyEpsilon]",
                    TraditionalForm]}], "+", "1"}],
              (Editable -> True)], ";", 
            TagBox["x",
              (Editable -> True)]}], ")"}],
        InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Series2", "[", 
      RowBox[{"%", ",", 
        RowBox[{"{", 
          RowBox[{"Epsilon", ",", "0", ",", "3"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  FractionBox["1", "6"]}], " ", 
                RowBox[{
                  SuperscriptBox["log", "3"], "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"log", "(", "x", ")"}], " ", 
                RowBox[{
                  SuperscriptBox["log", "2"], "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "-", 
              RowBox[{
                RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "-", 
              RowBox[{
                FractionBox["1", "6"], " ", 
                SuperscriptBox["\[Pi]", "2"], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{"2", " ", 
                RowBox[{
                  SubscriptBox["Li", "3"], "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{"4", " ", 
                RowBox[{
                  SubscriptBox["Li", "3"], "(", "x", ")"}]}], "-", 
              RowBox[{"2", " ", 
                TagBox[
                  RowBox[{"\[Zeta]", "(", 
                    TagBox["3",
                      (Editable -> True)], ")"}],
                  InterpretTemplate[ Zeta[ #]&]]}]}], ")"}], " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "3"]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  FractionBox["1", "2"]}], " ", 
                RowBox[{
                  SuperscriptBox["log", "2"], "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{"2", " ", 
                RowBox[{"log", "(", "x", ")"}], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "-", 
              RowBox[{"3", " ", 
                RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
              RowBox[{"2", " ", 
                RowBox[{
                  SubscriptBox["Li", "2"], "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              FractionBox[
                SuperscriptBox["\[Pi]", "2"], "6"]}], ")"}], " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "2"]}], "-", 
        RowBox[{
          RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}], " ", 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm]}], "+", "1"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "There are over 100 more special expansions of ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["\[ThinSpace]", "2"], 
          SubscriptBox["F", "1"]}], TraditionalForm]]],
  " tabulated in Series2.m. The interested user can consult the source code \
(search for HYPERLIST)."
}], "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["SimplifyDeltaFunction", "Subsection",
  CellTags->"SimplifyDeltaFunction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
SimplifyDeltaFunction[exp, x] simplifies f[x]*DeltaFunction[1-x] \
toLimit[f[x],x->1] DeltaFunction[1-x] and applies a list of transformation \
rules for DeltaFunctionPrime[1-x]*x^(OPEm-1)*f[x] where x^(OPEm-1) is \
suppressed in exp.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DeltaFunction",
    ButtonData:>"DeltaFunction",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DeltaFunction"],
  ", ",
  ButtonBox["DeltaFunctionPrime",
    ButtonData:>"DeltaFunctionPrime",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DeltaFunctionPrime"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"g", "[", "x", "]"}], " ", 
      RowBox[{"DeltaFunction", "[", 
        RowBox[{"1", "-", "x"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"\[Delta]", "(", 
          FormBox[
            RowBox[{"1", "-", "x"}],
            "TraditionalForm"], ")"}], " ", 
        RowBox[{"g", "(", "x", ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SimplifyDeltaFunction", "[", " ", 
      RowBox[{"%", ",", "x"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"\[Delta]", "(", 
          FormBox[
            RowBox[{"1", "-", "x"}],
            "TraditionalForm"], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{
            UnderscriptBox["lim", 
              RowBox[{"x", "\[Rule]", "1"}]], "\[ThinSpace]", 
            RowBox[{"g", "(", "x", ")"}]}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"g", "[", "x", "]"}], 
      RowBox[{"DeltaFunctionPrime", "[", 
        RowBox[{"1", "-", "x"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
          FormBox[
            RowBox[{"1", "-", "x"}],
            "TraditionalForm"], ")"}], " ", 
        RowBox[{"g", "(", "x", ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SimplifyDeltaFunction", "[", " ", 
      RowBox[{"%", ",", "x"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{
            SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
            FormBox[
              RowBox[{"1", "-", "x"}],
              "TraditionalForm"], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              UnderscriptBox["lim", 
                RowBox[{"x", "\[Rule]", "1"}]], "\[ThinSpace]", 
              RowBox[{"g", "(", "x", ")"}]}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"\[Delta]", "(", 
            FormBox[
              RowBox[{"1", "-", "x"}],
              "TraditionalForm"], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              UnderscriptBox["lim", 
                RowBox[{"x", "\[Rule]", "1"}]], "\[ThinSpace]", 
              RowBox[{
                SuperscriptBox["g", "\[Prime]",
                  MultilineFunction->None], "(", "x", ")"}]}], ")"}]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"x", " ", 
      RowBox[{"Log", "[", "x", "]"}], " ", 
      RowBox[{"DeltaFunctionPrime", "[", 
        RowBox[{"1", "-", "x"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"x", " ", 
        RowBox[{
          SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
          FormBox[
            RowBox[{"1", "-", "x"}],
            "TraditionalForm"], ")"}], " ", 
        RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SimplifyDeltaFunction", "[", " ", 
      RowBox[{"%", ",", "x"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[Delta]", "(", 
        FormBox[
          RowBox[{"1", "-", "x"}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"PolyLog", "[", 
        RowBox[{"2", ",", 
          RowBox[{"1", "-", "x"}]}], "]"}], " ", 
      RowBox[{"DeltaFunctionPrime", "[", 
        RowBox[{"1", "-", "x"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
          FormBox[
            RowBox[{"1", "-", "x"}],
            "TraditionalForm"], ")"}], " ", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          RowBox[{"1", "-", "x"}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SimplifyDeltaFunction", "[", " ", 
      RowBox[{"%", ",", "x"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        RowBox[{"\[Delta]", "(", 
          FormBox[
            RowBox[{"1", "-", "x"}],
            "TraditionalForm"], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Log", "[", "x", "]"}], 
      RowBox[{"PolyLog", "[", 
        RowBox[{"2", ",", 
          RowBox[{"1", "-", "x"}]}], "]"}], " ", 
      RowBox[{"DeltaFunctionPrime", "[", 
        RowBox[{"1", "-", "x"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
          FormBox[
            RowBox[{"1", "-", "x"}],
            "TraditionalForm"], ")"}], " ", 
        RowBox[{"log", "(", "x", ")"}], " ", 
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          RowBox[{"1", "-", "x"}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SimplifyDeltaFunction", "[", " ", 
      RowBox[{"%", ",", "x"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"PolyLog", "[", 
        RowBox[{"3", ",", 
          RowBox[{"1", "-", "x"}]}], "]"}], " ", 
      RowBox[{"DeltaFunctionPrime", "[", 
        RowBox[{"1", "-", "x"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
          FormBox[
            RowBox[{"1", "-", "x"}],
            "TraditionalForm"], ")"}], " ", 
        RowBox[{
          SubscriptBox["Li", "3"], "(", 
          RowBox[{"1", "-", "x"}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SimplifyDeltaFunction", "[", " ", 
      RowBox[{"%", ",", "x"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        RowBox[{"\[Delta]", "(", 
          FormBox[
            RowBox[{"1", "-", "x"}],
            "TraditionalForm"], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["SimplifyPolyLog", "Subsection",
  CellTags->"SimplifyPolyLog"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
SimplifyPolyLog[y] performs several simplifications assuming that \
the variables occuring in the Log and PolyLog functions are between 0 and 1. \
\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Nielsen",
    ButtonData:>"Nielsen",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Nielsen"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{"sip", "[", "y_", "]"}], " ", ":=", " ", 
      RowBox[{"y", " ", "==", " ", 
        RowBox[{"SimplifyPolyLog", "[", "y", "]"}]}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"PolyLog", "[", 
        RowBox[{"2", ",", 
          RowBox[{"1", "/", "x"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          FractionBox["1", "x"], ")"}], "==", 
        RowBox[{
          RowBox[{
            RowBox[{"-", 
              FractionBox["1", "2"]}], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
          RowBox[{
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}], " ", 
            RowBox[{"log", "(", "x", ")"}]}], "+", 
          RowBox[{"\[ImaginaryI]", " ", "\[Pi]", " ", 
            RowBox[{"log", "(", "x", ")"}]}], "+", 
          RowBox[{"\[Zeta]", "(", "2", ")"}], "+", 
          RowBox[{
            SubscriptBox["Li", "2"], "(", 
            RowBox[{"1", "-", "x"}], ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"PolyLog", "[", 
        RowBox[{"2", ",", "x"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["Li", "2"], "(", "x", ")"}], "==", 
        RowBox[{
          RowBox[{"\[Zeta]", "(", "2", ")"}], "-", 
          RowBox[{
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}], " ", 
            RowBox[{"log", "(", "x", ")"}]}], "-", 
          RowBox[{
            SubscriptBox["Li", "2"], "(", 
            RowBox[{"1", "-", "x"}], ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"PolyLog", "[", 
        RowBox[{"2", ",", 
          RowBox[{"1", "-", 
            RowBox[{"x", "^", "2"}]}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          RowBox[{"1", "-", 
            SuperscriptBox["x", "2"]}], ")"}], "==", 
        RowBox[{
          RowBox[{"-", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
          RowBox[{"2", " ", 
            RowBox[{"log", "(", "x", ")"}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"x", "+", "1"}], ")"}]}], "+", 
          RowBox[{"2", " ", 
            RowBox[{
              SubscriptBox["Li", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "-", 
          RowBox[{"2", " ", 
            RowBox[{
              SubscriptBox["Li", "2"], "(", 
              RowBox[{"-", "x"}], ")"}]}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"PolyLog", "[", 
        RowBox[{"2", ",", 
          RowBox[{"x", "^", "2"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          SuperscriptBox["x", "2"], ")"}], "==", 
        RowBox[{
          RowBox[{"2", " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
          RowBox[{"2", " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}], " ", 
            RowBox[{"log", "(", "x", ")"}]}], "-", 
          RowBox[{"2", " ", 
            RowBox[{
              SubscriptBox["Li", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "+", 
          RowBox[{"2", " ", 
            RowBox[{
              SubscriptBox["Li", "2"], "(", 
              RowBox[{"-", "x"}], ")"}]}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"PolyLog", "[", 
        RowBox[{"2", ",", 
          RowBox[{"-", 
            RowBox[{"x", "/", 
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}]}]}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          RowBox[{"-", 
            FractionBox["x", 
              RowBox[{"1", "-", "x"}]]}], ")"}], "==", 
        RowBox[{
          RowBox[{
            RowBox[{"-", 
              FractionBox["1", "2"]}], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "+", 
          RowBox[{
            RowBox[{"log", "(", "x", ")"}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "-", 
          RowBox[{"\[Zeta]", "(", "2", ")"}], "+", 
          RowBox[{
            SubscriptBox["Li", "2"], "(", 
            RowBox[{"1", "-", "x"}], ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"PolyLog", "[", 
        RowBox[{"2", ",", 
          RowBox[{"x", "/", 
            RowBox[{"(", 
              RowBox[{"x", "-", "1"}], ")"}]}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["Li", "2"], "(", 
          FractionBox["x", 
            RowBox[{"x", "-", "1"}]], ")"}], "==", 
        RowBox[{
          RowBox[{
            RowBox[{"-", 
              FractionBox["1", "2"]}], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "+", 
          RowBox[{
            RowBox[{"log", "(", "x", ")"}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "-", 
          RowBox[{"\[Zeta]", "(", "2", ")"}], "+", 
          RowBox[{
            SubscriptBox["Li", "2"], "(", 
            RowBox[{"1", "-", "x"}], ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"Nielsen", "[", 
        RowBox[{"1", ",", "2", ",", 
          RowBox[{"-", 
            RowBox[{"x", "/", 
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}]}]}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["S", 
            RowBox[{
              FormBox["1",
                "TraditionalForm"], "\[NoBreak]", 
              FormBox["2",
                "TraditionalForm"]}]], "(", 
          FormBox[
            RowBox[{"-", 
              FractionBox["x", 
                RowBox[{"1", "-", "x"}]]}],
            "TraditionalForm"], ")"}], "==", 
        RowBox[{
          RowBox[{
            SubscriptBox["S", 
              RowBox[{
                FormBox["1",
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox["2",
                  "TraditionalForm"]}]], "(", "x", ")"}], "-", 
          RowBox[{
            FractionBox["1", "6"], " ", 
            RowBox[{
              SuperscriptBox["log", "3"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}]}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"PolyLog", "[", 
        RowBox[{"3", ",", 
          RowBox[{"-", 
            RowBox[{"1", "/", "x"}]}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["Li", "3"], "(", 
          RowBox[{"-", 
            FractionBox["1", "x"]}], ")"}], "==", 
        RowBox[{
          FractionBox[
            RowBox[{
              SuperscriptBox["log", "3"], "(", "x", ")"}], "6"], "+", 
          RowBox[{
            RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
            RowBox[{"log", "(", "x", ")"}]}], "+", 
          RowBox[{
            SubscriptBox["Li", "3"], "(", 
            RowBox[{"-", "x"}], ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"PolyLog", "[", 
        RowBox[{"3", ",", 
          RowBox[{"1", "-", "x"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["True", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"PolyLog", "[", 
        RowBox[{"3", ",", 
          RowBox[{"x", "^", "2"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["Li", "3"], "(", 
          SuperscriptBox["x", "2"], ")"}], "==", 
        RowBox[{
          RowBox[{
            RowBox[{"-", "2"}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
          RowBox[{"4", " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
            RowBox[{"log", "(", "x", ")"}]}], "-", 
          RowBox[{"4", " ", 
            RowBox[{
              SubscriptBox["Li", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}], " ", 
            RowBox[{"log", "(", "x", ")"}]}], "-", 
          RowBox[{"4", " ", 
            RowBox[{
              SubscriptBox["S", 
                RowBox[{
                  FormBox["1",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["2",
                    "TraditionalForm"]}]], "(", 
              FormBox[
                RowBox[{"1", "-", "x"}],
                "TraditionalForm"], ")"}]}], "+", 
          RowBox[{"4", " ", 
            RowBox[{
              SubscriptBox["Li", "3"], "(", 
              RowBox[{"-", "x"}], ")"}]}], "+", 
          RowBox[{"4", " ", 
            TagBox[
              RowBox[{"\[Zeta]", "(", 
                TagBox["3",
                  (Editable -> True)], ")"}],
              InterpretTemplate[ Zeta[ #]&]]}]}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"PolyLog", "[", 
        RowBox[{"3", ",", 
          RowBox[{"-", 
            RowBox[{"x", "/", 
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}]}]}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["Li", "3"], "(", 
          RowBox[{"-", 
            FractionBox["x", 
              RowBox[{"1", "-", "x"}]]}], ")"}], "==", 
        RowBox[{
          RowBox[{
            FractionBox["1", "6"], " ", 
            RowBox[{
              SuperscriptBox["log", "3"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "-", 
          RowBox[{
            FractionBox["1", "2"], " ", 
            RowBox[{"log", "(", "x", ")"}], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "+", 
          RowBox[{
            FractionBox["1", "2"], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", "x", ")"}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "+", 
          RowBox[{
            RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "-", 
          RowBox[{
            RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
            RowBox[{"log", "(", "x", ")"}]}], "+", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{
                FormBox["1",
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox["2",
                  "TraditionalForm"]}]], "(", 
            FormBox[
              RowBox[{"1", "-", "x"}],
              "TraditionalForm"], ")"}], "+", 
          RowBox[{
            RowBox[{"log", "(", "x", ")"}], " ", 
            RowBox[{
              SubscriptBox["Li", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "-", 
          RowBox[{
            SubscriptBox["Li", "3"], "(", 
            RowBox[{"1", "-", "x"}], ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"PolyLog", "[", 
        RowBox[{"3", ",", 
          RowBox[{"1", "-", 
            RowBox[{"1", "/", "x"}]}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["Li", "3"], "(", 
          RowBox[{"1", "-", 
            FractionBox["1", "x"]}], ")"}], "==", 
        RowBox[{
          FractionBox[
            RowBox[{
              SuperscriptBox["log", "3"], "(", "x", ")"}], "6"], "-", 
          RowBox[{
            FractionBox["1", "2"], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}], " ", 
            RowBox[{"log", "(", "x", ")"}]}], "+", 
          RowBox[{
            RowBox[{
              SubscriptBox["Li", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}], " ", 
            RowBox[{"log", "(", "x", ")"}]}], "+", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{
                FormBox["1",
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox["2",
                  "TraditionalForm"]}]], "(", 
            FormBox[
              RowBox[{"1", "-", "x"}],
              "TraditionalForm"], ")"}], "+", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{
                FormBox["1",
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox["2",
                  "TraditionalForm"]}]], "(", "x", ")"}], "-", 
          RowBox[{
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}], " ", 
            RowBox[{
              SubscriptBox["Li", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "-", 
          TagBox[
            RowBox[{"\[Zeta]", "(", 
              TagBox["3",
                (Editable -> True)], ")"}],
            InterpretTemplate[ Zeta[ #]&]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"PolyLog", "[", 
        RowBox[{"4", ",", 
          RowBox[{"-", 
            RowBox[{"x", "/", 
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}]}]}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["Li", "4"], "(", 
          RowBox[{"-", 
            FractionBox["x", 
              RowBox[{"1", "-", "x"}]]}], ")"}], "==", 
        RowBox[{
          RowBox[{
            RowBox[{"-", 
              FractionBox["1", "24"]}], " ", 
            RowBox[{
              SuperscriptBox["log", "4"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "+", 
          RowBox[{
            FractionBox["1", "2"], " ", 
            RowBox[{"log", "(", "x", ")"}], " ", 
            RowBox[{
              SuperscriptBox["log", "3"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "-", 
          RowBox[{
            FractionBox["1", "2"], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", "x", ")"}], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "-", 
          RowBox[{
            FractionBox["1", "2"], " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "+", 
          RowBox[{
            FractionBox["1", "2"], " ", 
            RowBox[{
              SubscriptBox["Li", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "+", 
          RowBox[{
            RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
            RowBox[{"log", "(", "x", ")"}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "-", 
          RowBox[{
            RowBox[{
              SubscriptBox["S", 
                RowBox[{
                  FormBox["1",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["2",
                    "TraditionalForm"]}]], "(", 
              FormBox[
                RowBox[{"1", "-", "x"}],
                "TraditionalForm"], ")"}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "-", 
          RowBox[{
            RowBox[{
              SubscriptBox["S", 
                RowBox[{
                  FormBox["1",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["2",
                    "TraditionalForm"]}]], "(", "x", ")"}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "-", 
          RowBox[{
            RowBox[{"log", "(", "x", ")"}], " ", 
            RowBox[{
              SubscriptBox["Li", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}], " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "+", 
          RowBox[{
            TagBox[
              RowBox[{"\[Zeta]", "(", 
                TagBox["3",
                  (Editable -> True)], ")"}],
              InterpretTemplate[ Zeta[ #]&]], " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], "-", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{
                FormBox["1",
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox["3",
                  "TraditionalForm"]}]], "(", "x", ")"}], "+", 
          RowBox[{
            SubscriptBox["S", 
              RowBox[{
                FormBox["2",
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox["2",
                  "TraditionalForm"]}]], "(", "x", ")"}], "-", 
          RowBox[{
            SubscriptBox["Li", "4"], "(", "x", ")"}]}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"Log", "[", 
        RowBox[{"a", "+", 
          RowBox[{"b", "/", "c"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"log", "(", 
          RowBox[{"a", "+", 
            FractionBox["b", "c"]}], ")"}], "==", 
        RowBox[{"log", "(", 
          FractionBox[
            RowBox[{"b", "+", 
              RowBox[{"a", " ", "c"}]}], "c"], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"Log", "[", 
        RowBox[{"1", "/", "x"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"log", "(", 
          FractionBox["1", "x"], ")"}], "==", 
        RowBox[{"-", 
          RowBox[{"log", "(", "x", ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"ArcTanh", "[", "x", "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["tanh", 
            RowBox[{"-", "1"}]], "(", "x", ")"}], "==", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"log", "(", 
            RowBox[{"-", 
              FractionBox[
                RowBox[{"x", "+", "1"}], 
                RowBox[{"1", "-", "x"}]]}], ")"}]}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"ArcSinh", "[", "x", "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["sinh", 
            RowBox[{"-", "1"}]], "(", "x", ")"}], "==", 
        RowBox[{"log", "(", 
          RowBox[{"x", "+", 
            SqrtBox[
              RowBox[{
                SuperscriptBox["x", "2"], "+", "1"}]]}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"sip", "[", 
      RowBox[{"ArcCosh", "[", "x", "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["cosh", 
            RowBox[{"-", "1"}]], "(", "x", ")"}], "==", 
        RowBox[{"log", "(", 
          RowBox[{"x", "+", 
            SqrtBox[
              RowBox[{
                SuperscriptBox["x", "2"], "-", "1"}]]}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", "sip", "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["SOD", "Subsection",
  CellTags->"SOD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
SOD[q] is a D-dimensional scalar product of OPEDelta with q. It is \
transformed into Pair[Momentum[q,D], Momentum[OPEDelta,D] by \
FeynCalcInternal.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OPEDelta",
    ButtonData:>"OPEDelta",
    ButtonStyle->"Hyperlink",
    ButtonNote->"OPEDelta"],
  ", ",
  ButtonBox["Pair",
    ButtonData:>"Pair",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Pair"],
  ", ",
  ButtonBox["ScalarProduct",
    ButtonData:>"ScalarProduct",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ScalarProduct"],
  ", ",
  ButtonBox["SOD",
    ButtonData:>"SOD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SOD"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SOD", "[", "p", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["\<\"\[CapitalDelta]\"\>",
          "TraditionalForm"], 
        FormBox["\<\"\[CenterDot]\"\>",
          "TraditionalForm"], 
        FormBox["p",
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SOD", "[", 
      RowBox[{"p", "-", "q"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["\<\"\[CapitalDelta]\"\>",
          "TraditionalForm"], 
        FormBox["\<\"\[CenterDot]\"\>",
          "TraditionalForm"], 
        FormBox["\<\"(\"\>",
          "TraditionalForm"], 
        FormBox[
          RowBox[{"p", "-", "q"}],
          "TraditionalForm"], 
        FormBox["\<\")\"\>",
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SOD", "[", "p", "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", 
          RowBox[{"OPEDelta", ",", "D"}], "]"}], ",", 
        RowBox[{"Momentum", "[", 
          RowBox[{"p", ",", "D"}], "]"}]}], "]"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solve2", "Subsection",
  CellTags->"Solve2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Solve2 is equivalent to Solve, except that it works only for linear \
equations (and returns just a list) and accepts the options Factoring and \
FinalSubstitutions. Solve2 uses the \"high school algorithm\" and factors \
intermediate results.Therefore it can be drastically more useful than \
Solve.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Solve2", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Factoring", "\[Rule]", "Factor2"}], ",", 
          RowBox[{"FinalSubstitutions", "\[Rule]", 
            RowBox[{"{", "}"}]}]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Solve2", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            RowBox[{
              RowBox[{"2", " ", "x"}], "==", 
              RowBox[{"b", "-", 
                RowBox[{"w", "/", "2"}]}]}], ",", 
            RowBox[{
              RowBox[{"y", "-", "d"}], "==", "p"}]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"x", "\[Rule]", 
            RowBox[{
              FractionBox["1", "4"], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"2", " ", "b"}], "-", "w"}], ")"}]}]}], ",", 
          RowBox[{"y", "\[Rule]", 
            RowBox[{"d", "+", "p"}]}]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
If no equation sign is given the polynomials are supposed to be \
0.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Solve2", "[", 
      RowBox[{
        RowBox[{"x", "+", "y"}], ",", "x"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"x", "\[Rule]", 
          RowBox[{"-", "y"}]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Solve2", "[", 
      RowBox[{
        RowBox[{"x", "+", "y"}], ",", "x", ",", 
        RowBox[{"FinalSubstitutions", "\[Rule]", 
          RowBox[{"{", 
            RowBox[{"y", "\[Rule]", "h"}], "}"}]}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"x", "\[Rule]", 
          RowBox[{"-", "h"}]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Solve2", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            RowBox[{
              RowBox[{"2", " ", "x"}], "==", 
              RowBox[{"b", "-", 
                RowBox[{"w", "/", "2"}]}]}], ",", 
            RowBox[{
              RowBox[{"y", "-", "d"}], "==", "p"}]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "y"}], "}"}], ",", 
        RowBox[{"Factoring", "\[Rule]", "Expand"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"x", "\[Rule]", 
            RowBox[{
              FractionBox["b", "2"], "-", 
              FractionBox["w", "4"]}]}], ",", 
          RowBox[{"y", "\[Rule]", 
            RowBox[{"d", "+", "p"}]}]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Solve", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            RowBox[{
              RowBox[{"2", " ", "x"}], "==", 
              RowBox[{"b", "-", 
                RowBox[{"w", "/", "2"}]}]}], ",", 
            RowBox[{
              RowBox[{"y", "-", "d"}], "==", "p"}]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"x", "\[Rule]", 
              RowBox[{
                FractionBox["1", "4"], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"2", " ", "b"}], "-", "w"}], ")"}]}]}], ",", 
            RowBox[{"y", "\[Rule]", 
              RowBox[{"d", "+", "p"}]}]}], "}"}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["SO", "Subsection",
  CellTags->"SO"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
SO[q] is a four-dimensional scalar product of OPEDelta with q. It \
is transformed into Pair[Momentum[q], Momentum[OPEDelta] by FCI.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  ButtonBox["FCI",
    ButtonData:>"FCI",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FCI"],
  ", ",
  ButtonBox["OPEDelta",
    ButtonData:>"OPEDelta",
    ButtonStyle->"Hyperlink",
    ButtonNote->"OPEDelta"],
  ", ",
  ButtonBox["Pair",
    ButtonData:>"Pair",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Pair"],
  ", ",
  ButtonBox["ScalarProduct",
    ButtonData:>"ScalarProduct",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ScalarProduct"],
  ", ",
  ButtonBox["SOD",
    ButtonData:>"SOD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SOD"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SO", "[", "p", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["\<\"\[CapitalDelta]\"\>",
          "TraditionalForm"], 
        FormBox["\<\"\[CenterDot]\"\>",
          "TraditionalForm"], 
        FormBox["p",
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SO", "[", 
      RowBox[{"p", "-", "q"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["\<\"\[CapitalDelta]\"\>",
          "TraditionalForm"], 
        FormBox["\<\"\[CenterDot]\"\>",
          "TraditionalForm"], 
        FormBox["\<\"(\"\>",
          "TraditionalForm"], 
        FormBox[
          RowBox[{"p", "-", "q"}],
          "TraditionalForm"], 
        FormBox["\<\")\"\>",
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SO", "[", "p", "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "OPEDelta", "]"}], ",", 
        RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["SP", "Subsection",
  CellTags->"SP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "SP[a, b] denotes a four-dimensional scalar product. SP[a, b] is \
transformed into ScalarProduct[a, b] by FeynCalcInternal. SP[p] is the same \
as SP[p, p] ",
  Cell[BoxData[
      FormBox[
        RowBox[{"(", 
          RowBox[{"=", 
            SuperscriptBox["p", "2"]}]}], TraditionalForm]]],
  ")."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Calc",
    ButtonData:>"Calc",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Calc"],
  ", ",
  ButtonBox["ExpandScalarProduct",
    ButtonData:>"ExpandScalarProduct",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ExpandScalarProduct"],
  ", ",
  ButtonBox["ScalarProduct",
    ButtonData:>"ScalarProduct",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ScalarProduct"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SP", "[", 
        RowBox[{"p", ",", "q"}], "]"}], " ", "+", " ", 
      RowBox[{"SP", "[", "q", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"]}],
          "TraditionalForm"], "+", 
        FormBox[
          SuperscriptBox[
            FormBox["q",
              "TraditionalForm"], "2"],
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SP", "[", 
      RowBox[{
        RowBox[{"p", "-", "q"}], ",", 
        RowBox[{"q", "+", 
          RowBox[{"2", "p"}]}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"(", "\[NoBreak]", 
        FormBox[
          RowBox[{"p", "-", "q"}],
          "TraditionalForm"], "\[NoBreak]", ")", "\[NoBreak]", 
        "\[CenterDot]", "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          RowBox[{
            RowBox[{"2", " ", "p"}], "+", "q"}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", " ", 
      RowBox[{"SP", "[", 
        RowBox[{
          RowBox[{"p", "-", "q"}], ",", 
          RowBox[{"q", "+", 
            RowBox[{"2", "p"}]}]}], "]"}], " ", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"2", " ", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"]}], "-", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}], "-", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], "2"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandScalarProduct", "[", 
      RowBox[{"SP", "[", 
        RowBox[{"p", "-", "q"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], "2"], "-", 
        RowBox[{"2", " ", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"]}]}], "+", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], "2"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SP", "[", 
        RowBox[{"a", ",", "b"}], "]"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"SP", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SP", "[", 
          RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "a", "]"}], ",", 
        RowBox[{"Momentum", "[", "b", "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"SP", "[", 
            RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", "FCE"}], 
      "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"SP", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["SPD", "Subsection",
  CellTags->"SPD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "SPD[a, b] denotes a D-dimensional scalar product. SPD[a, b] is transformed \
into ScalarProduct[a, b,Dimension\[Rule]D] by FeynCalcInternal. SPD[p] is the \
same as SPD[p,p] ",
  Cell[BoxData[
      FormBox[
        RowBox[{"(", 
          RowBox[{"=", 
            SuperscriptBox["p", "2"]}]}], TraditionalForm]]],
  ")."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Calc",
    ButtonData:>"Calc",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Calc"],
  ", ",
  ButtonBox["ExpandScalarProduct",
    ButtonData:>"ExpandScalarProduct",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ExpandScalarProduct"],
  ", ",
  ButtonBox["ScalarProduct",
    ButtonData:>"ScalarProduct",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ScalarProduct"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SPD", "[", 
        RowBox[{"p", ",", "q"}], "]"}], " ", "+", " ", 
      RowBox[{"SPD", "[", "q", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"]}],
          "TraditionalForm"], "+", 
        FormBox[
          SuperscriptBox[
            FormBox["q",
              "TraditionalForm"], "2"],
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SPD", "[", 
      RowBox[{
        RowBox[{"p", "-", "q"}], ",", 
        RowBox[{"q", "+", 
          RowBox[{"2", "p"}]}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"(", "\[NoBreak]", 
        FormBox[
          RowBox[{"p", "-", "q"}],
          "TraditionalForm"], "\[NoBreak]", ")", "\[NoBreak]", 
        "\[CenterDot]", "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          RowBox[{
            RowBox[{"2", " ", "p"}], "+", "q"}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", " ", 
      RowBox[{"SPD", "[", 
        RowBox[{
          RowBox[{"p", "-", "q"}], ",", 
          RowBox[{"q", "+", 
            RowBox[{"2", "p"}]}]}], "]"}], " ", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"2", " ", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"]}], "-", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}], "-", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], "2"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ExpandScalarProduct", "[", 
      RowBox[{"SPD", "[", 
        RowBox[{"p", "-", "q"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], "2"], "-", 
        RowBox[{"2", " ", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"]}]}], "+", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], "2"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SPD", "[", 
        RowBox[{"a", ",", "b"}], "]"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"SPD", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SPD", "[", 
          RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Pair", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", 
          RowBox[{"a", ",", "D"}], "]"}], ",", 
        RowBox[{"Momentum", "[", 
          RowBox[{"b", ",", "D"}], "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"SPD", "[", 
            RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", "FCE"}], 
      "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"SPD", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FCE", "[", 
        RowBox[{"ChangeDimension", "[", 
          RowBox[{
            RowBox[{"SP", "[", 
              RowBox[{"p", ",", "q"}], "]"}], ",", " ", "D"}], "]"}], "]"}], 
      "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"SPD", "[", 
      RowBox[{"p", ",", "q"}], "]"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Spinor", "Subsection",
  CellTags->"Spinor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "Spinor[p, m, optarg] is the head of Dirac spinors. Which of the spinors u, \
v, ",
  Cell[BoxData[
      FormBox[
        OverscriptBox["u", "_"], TraditionalForm]]],
  " ",
  Cell[BoxData[
      FormBox[
        RowBox[{",", " "}], TraditionalForm]]],
  "or ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          OverscriptBox["v", "_"], " "}], TraditionalForm]]],
  "is understood, depends on the sign of the momentum (p) argument and the \
relative position of DiracSlash[p]: Spinor[sign p, mass] is that spinor which \
yields: sign*mass*Spinor[p, mass] if the Dirac equation is applied (by \
DiracEquation or DiracSimplify)."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracSimplify"],
  ", ",
  ButtonBox["SpinorU",
    ButtonData:>"SpinorU",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SpinorU"],
  ", ",
  ButtonBox["SpinorV",
    ButtonData:>"SpinorV",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SpinorV"],
  ", ",
  ButtonBox["SpinorUBar",
    ButtonData:>"SpinorUBar",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SpinorUBar"],
  ", ",
  ButtonBox["SpinorVBar",
    ButtonData:>"SpinorVBar",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SpinorVBar"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Spinor", "[", "p", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[CurlyPhi]", "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox["p",
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Spinor", "[", 
      RowBox[{"p", ",", "m"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[CurlyPhi]", "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox["p",
          "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m", 
        "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DiracSimplify", "[", 
      RowBox[{
        RowBox[{"Spinor", "[", 
          RowBox[{
            RowBox[{"-", "p"}], ",", "m"}], "]"}], ".", 
        RowBox[{"DiracSlash", "[", "p", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "m"}], " ", 
        RowBox[{"\[CurlyPhi]", "\[NoBreak]", "(", "\[NoBreak]", 
          FormBox[
            RowBox[{"-", "p"}],
            "TraditionalForm"], "\[NoBreak]", ",", "\[NoBreak]", "m", 
          "\[NoBreak]", ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Spinor", "[", "p", "]"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Spinor", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", "0", ",", "1"}], "]"}]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"ChangeDimension", "[", 
        RowBox[{
          RowBox[{"Spinor", "[", "p", "]"}], ",", "D"}], "]"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Spinor", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", "0", ",", "1"}], "]"}]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Spinor", "[", 
        RowBox[{"p", ",", "m"}], "]"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Spinor", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], "]"}]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Spinor", "[", 
        RowBox[{"p", ",", 
          RowBox[{"SmallVariable", "[", "m", "]"}]}], "]"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Spinor", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", "0", ",", "1"}], "]"}]], 
  "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["SpinorUBar", "Subsection",
  CellTags->"SpinorUBar"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "SpinorUBar[p, m] denotes a ",
  Cell[BoxData[
      FormBox[
        OverscriptBox["u", "_"], TraditionalForm]]],
  " - spinor."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Spinor",
    ButtonData:>"Spinor",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Spinor"],
  ", ",
  ButtonBox["SpinorU",
    ButtonData:>"SpinorU",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SpinorU"],
  ", ",
  ButtonBox["SpinorV",
    ButtonData:>"SpinorV",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SpinorV"],
  ", ",
  ButtonBox["SpinorVBar",
    ButtonData:>"SpinorVBar",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SpinorVBar"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell["One argument only assumes a massless spinor.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SpinorUBar", "[", "p", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          OverscriptBox["\<\"u\"\>", "_"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "p", 
        "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SpinorUBar", "[", 
      RowBox[{"p", ",", "m"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          OverscriptBox["\<\"u\"\>", "_"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "p", 
        "\[NoBreak]", ",", "\[NoBreak]", "m", "\[NoBreak]", ")"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SpinorUBar", "[", 
        RowBox[{"p", ",", "m"}], "]"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"SpinorUBar", "[", 
      RowBox[{"p", ",", "m"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SpinorUBar", "[", 
          RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Spinor", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], "]"}]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"SpinorUBar", "[", 
            RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", "FCE"}], 
      "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Spinor", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], "]"}]], 
  "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["SpinorU", "Subsection",
  CellTags->"SpinorU"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["SpinorU[p, m, optarg] denotes a u-spinor.", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Spinor",
    ButtonData:>"Spinor",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Spinor"],
  ", ",
  ButtonBox["SpinorUBar",
    ButtonData:>"SpinorUBar",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SpinorUBar"],
  ", ",
  ButtonBox["SpinorV",
    ButtonData:>"SpinorV",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SpinorV"],
  ", ",
  ButtonBox["SpinorVBar",
    ButtonData:>"SpinorVBar",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SpinorVBar"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SpinorU", "[", "p", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"u", "\[NoBreak]", "(", "\[NoBreak]", "p", "\[NoBreak]", ")"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SpinorU", "[", 
      RowBox[{"p", ",", "m"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
      "u", "\[NoBreak]", "(", "\[NoBreak]", "p", "\[NoBreak]", ",", 
        "\[NoBreak]", "m", "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SpinorU", "[", 
        RowBox[{"p", ",", "m"}], "]"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"SpinorU", "[", 
      RowBox[{"p", ",", "m"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SpinorU", "[", 
          RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Spinor", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], "]"}]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"SpinorU", "[", 
            RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", "FCE"}], 
      "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Spinor", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], "]"}]], 
  "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["SpinorVBar", "Subsection",
  CellTags->"SpinorVBar"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "SpinorVBar[p, m] denotes a ",
  Cell[BoxData[
      FormBox[
        OverscriptBox["v", "_"], TraditionalForm]]],
  "(p,m) - spinor."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Spinor",
    ButtonData:>"Spinor",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Spinor"],
  ", ",
  ButtonBox["SpinorU",
    ButtonData:>"SpinorU",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SpinorU"],
  ", ",
  ButtonBox["SpinorV",
    ButtonData:>"SpinorV",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SpinorV"],
  ", ",
  ButtonBox["SpinorUBar",
    ButtonData:>"SpinorUBar",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SpinorUBar"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SpinorVBar", "[", "p", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          OverscriptBox["\<\"v\"\>", "_"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "p", 
        "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SpinorVBar", "[", 
      RowBox[{"p", ",", "m"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          OverscriptBox["\<\"v\"\>", "_"],
          "TraditionalForm"], "\[NoBreak]", "(", "\[NoBreak]", "p", 
        "\[NoBreak]", ",", "\[NoBreak]", "m", "\[NoBreak]", ")"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SpinorVBar", "[", 
        RowBox[{"p", ",", "m"}], "]"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"SpinorVBar", "[", 
      RowBox[{"p", ",", "m"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SpinorVBar", "[", 
          RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Spinor", "[", 
      RowBox[{
        RowBox[{"-", 
          RowBox[{"Momentum", "[", "p", "]"}]}], ",", "m", ",", "1"}], 
      "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"SpinorVBar", "[", 
            RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", "FCE"}], 
      "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Spinor", "[", 
      RowBox[{
        RowBox[{"-", 
          RowBox[{"Momentum", "[", "p", "]"}]}], ",", "m", ",", "1"}], 
      "]"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["SpinorV", "Subsection",
  CellTags->"SpinorV"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["SpinorV[p, m, optarg] denotes a v-spinor.", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Spinor",
    ButtonData:>"Spinor",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Spinor"],
  ", ",
  ButtonBox["SpinorUBar",
    ButtonData:>"SpinorUBar",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SpinorUBar"],
  ", ",
  ButtonBox["SpinorU",
    ButtonData:>"SpinorU",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SpinorU"],
  ", ",
  ButtonBox["SpinorVBar",
    ButtonData:>"SpinorVBar",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SpinorVBar"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SpinorV", "[", "p", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"v", "\[NoBreak]", "(", "\[NoBreak]", "p", "\[NoBreak]", ")"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SpinorV", "[", 
      RowBox[{"p", ",", "m"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
      "v", "\[NoBreak]", "(", "\[NoBreak]", "p", "\[NoBreak]", ",", 
        "\[NoBreak]", "m", "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SpinorV", "[", 
        RowBox[{"p", ",", "m"}], "]"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"SpinorV", "[", 
      RowBox[{"p", ",", "m"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SpinorV", "[", 
          RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Spinor", "[", 
      RowBox[{
        RowBox[{"-", 
          RowBox[{"Momentum", "[", "p", "]"}]}], ",", "m", ",", "1"}], 
      "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"SpinorV", "[", 
            RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", "FCE"}], 
      "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"Spinor", "[", 
      RowBox[{
        RowBox[{"-", 
          RowBox[{"Momentum", "[", "p", "]"}]}], ",", "m", ",", "1"}], 
      "]"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["SplittingFunction", "Subsection",
  CellTags->"SplittingFunction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "SplittingFunction[pxy] is a database of splitting functions in the ",
  Cell[BoxData[
      FormBox[
        OverscriptBox["MS", "_"], TraditionalForm]]],
  " scheme."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "SplittingFunction", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Polarization", "\[Rule]", "1"}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["AnomalousDimension",
    ButtonData:>"AnomalousDimension",
    ButtonStyle->"Hyperlink",
    ButtonNote->"AnomalousDimension"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell["Unpolarized case:", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"SplittingFunction", ",", 
          RowBox[{"Polarization", "\[Rule]", "0"}]}], "]"}], ";"}]], "Input"],

Cell["\<\
In general the argument should be a string, but if the variables \
Pqq, etc. have no value, you can omit the \"\".\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "Pqq", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "F"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", "4"}], " ", "x"}], "+", 
            RowBox[{"6", " ", 
              RowBox[{"\[Delta]", "(", 
                FormBox[
                  RowBox[{"1", "-", "x"}],
                  "TraditionalForm"], ")"}]}], "+", 
            RowBox[{"8", " ", 
              SubscriptBox[
                RowBox[{"(", 
                  FractionBox["1", 
                    RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", "4"}], 
          ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "Pqg", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["T", "f"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"16", " ", 
              SuperscriptBox["x", "2"]}], "-", 
            RowBox[{"16", " ", "x"}], "+", "8"}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "Pgq", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "F"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"4", " ", "x"}], "-", "8", "+", 
            FractionBox["8", "x"]}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "Pgg", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"8", " ", 
          SubscriptBox["C", "A"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"-", 
                SuperscriptBox["x", "2"]}], "+", "x", "+", 
              RowBox[{
                FractionBox["11", "12"], " ", 
                RowBox[{"\[Delta]", "(", 
                  FormBox[
                    RowBox[{"1", "-", "x"}],
                    "TraditionalForm"], ")"}]}], "+", 
              SubscriptBox[
                RowBox[{"(", 
                  FractionBox["1", 
                    RowBox[{"1", "-", "x"}]], ")"}], "+"], "-", "2", "+", 
              FractionBox["1", "x"]}], ")"}]}], "-", 
        RowBox[{
          FractionBox["8", "3"], " ", 
          SubscriptBox["N", "f"], " ", 
          SubscriptBox["T", "f"], " ", 
          RowBox[{"\[Delta]", "(", 
            FormBox[
              RowBox[{"1", "-", "x"}],
              "TraditionalForm"], ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "aqq", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "F"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"2", " ", "x"}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"7", "-", 
                  RowBox[{"4", " ", 
                    RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
              RowBox[{"\[Delta]", "(", 
                FormBox[
                  RowBox[{"1", "-", "x"}],
                  "TraditionalForm"], ")"}]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"2", " ", "x"}], "+", "2"}], ")"}], " ", 
              RowBox[{"log", "(", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], 
            "-", 
            RowBox[{"4", " ", 
              RowBox[{"(", 
                RowBox[{
                  FractionBox[
                    RowBox[{"log", "(", "x", ")"}], 
                    RowBox[{"1", "-", "x"}]], "+", 
                  SubscriptBox[
                    RowBox[{"(", 
                      FractionBox[
                        RowBox[{"log", "(", 
                          RowBox[{"1", "-", "x"}], ")"}], 
                        RowBox[{"1", "-", "x"}]], ")"}], "+"]}], ")"}]}], 
            "-", "4"}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "agq", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "F"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", "2"}], " ", "x"}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    RowBox[{"-", "2"}], " ", "x"}], "+", "4", "-", 
                  FractionBox["4", "x"]}], ")"}], " ", 
              RowBox[{"log", "(", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "+", 
            "2", "-", 
            FractionBox["4", "x"]}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "aqg", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["T", "f"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    RowBox[{"-", "8"}], " ", 
                    SuperscriptBox["x", "2"]}], "+", 
                  RowBox[{"8", " ", "x"}], "-", "4"}], ")"}], " ", 
              RowBox[{"log", "(", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", 
            "4"}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "agg", "]"}]], "Input"],

Cell["Polarized case:", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"SplittingFunction", ",", 
          RowBox[{"Polarization", "\[Rule]", "1"}]}], "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "Pqq", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "F"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", "4"}], " ", "x"}], "+", 
            RowBox[{"6", " ", 
              RowBox[{"\[Delta]", "(", 
                FormBox[
                  RowBox[{"1", "-", "x"}],
                  "TraditionalForm"], ")"}]}], "+", 
            RowBox[{"8", " ", 
              SubscriptBox[
                RowBox[{"(", 
                  FractionBox["1", 
                    RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", "4"}], 
          ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "Pqg", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["T", "f"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"16", " ", "x"}], "-", "8"}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "Pgq", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "F"], " ", 
        RowBox[{"(", 
          RowBox[{"8", "-", 
            RowBox[{"4", " ", "x"}]}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "Pgg", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["C", "A"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", "16"}], " ", "x"}], "+", 
              RowBox[{
                FractionBox["22", "3"], " ", 
                RowBox[{"\[Delta]", "(", 
                  FormBox[
                    RowBox[{"1", "-", "x"}],
                    "TraditionalForm"], ")"}]}], "+", 
              RowBox[{"8", " ", 
                SubscriptBox[
                  RowBox[{"(", 
                    FractionBox["1", 
                      RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", "8"}], 
            ")"}]}], "-", 
        RowBox[{
          FractionBox["8", "3"], " ", 
          SubscriptBox["N", "f"], " ", 
          SubscriptBox["T", "f"], " ", 
          RowBox[{"\[Delta]", "(", 
            FormBox[
              RowBox[{"1", "-", "x"}],
              "TraditionalForm"], ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "aqq", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "F"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"8", " ", 
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}]}], "+", 
            RowBox[{"2", " ", "x"}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"7", "-", 
                  RowBox[{"4", " ", 
                    RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
              RowBox[{"\[Delta]", "(", 
                FormBox[
                  RowBox[{"1", "-", "x"}],
                  "TraditionalForm"], ")"}]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"2", " ", "x"}], "+", "2"}], ")"}], " ", 
              RowBox[{"log", "(", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], 
            "-", 
            RowBox[{"4", " ", 
              RowBox[{"log", "(", "x", ")"}], " ", 
              SubscriptBox[
                RowBox[{"(", 
                  FractionBox["1", 
                    RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", 
            RowBox[{"4", " ", 
              SubscriptBox[
                RowBox[{"(", 
                  FractionBox[
                    RowBox[{"log", "(", 
                      RowBox[{"1", "-", "x"}], ")"}], 
                    RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", "4"}], 
          ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "agq", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "F"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", "4"}], " ", "x"}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"2", " ", "x"}], "-", "4"}], ")"}], " ", 
              RowBox[{"log", "(", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "+", 
            "2"}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "agqd", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "F"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"2", " ", "x"}], "-", "4"}], ")"}], " ", 
              RowBox[{"log", "(", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", 
            "2"}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "aqg", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["T", "f"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{"4", "-", 
                  RowBox[{"8", " ", "x"}]}], ")"}], " ", 
              RowBox[{"log", "(", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", 
            "4"}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "aqgd", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["T", "f"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{"4", "-", 
                  RowBox[{"8", " ", "x"}]}], ")"}], " ", 
              RowBox[{"log", "(", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", 
            "4"}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "agg", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["C", "A"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    FractionBox["67", "9"], "-", 
                    RowBox[{"4", " ", 
                      RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
                RowBox[{"\[Delta]", "(", 
                  FormBox[
                    RowBox[{"1", "-", "x"}],
                    "TraditionalForm"], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"8", " ", "x"}], "-", "4"}], ")"}], " ", 
                RowBox[{"log", "(", 
                  RowBox[{
                    RowBox[{"(", 
                      RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], 
              "-", 
              RowBox[{"4", " ", 
                RowBox[{"(", 
                  RowBox[{
                    FractionBox[
                      RowBox[{"log", "(", "x", ")"}], 
                      RowBox[{"1", "-", "x"}]], "+", 
                    SubscriptBox[
                      RowBox[{"(", 
                        FractionBox[
                          RowBox[{"log", "(", 
                            RowBox[{"1", "-", "x"}], ")"}], 
                          RowBox[{"1", "-", "x"}]], ")"}], "+"]}], ")"}]}], 
              "+", "2"}], ")"}]}], "-", 
        RowBox[{
          FractionBox["20", "9"], " ", 
          SubscriptBox["T", "f"], " ", 
          RowBox[{"\[Delta]", "(", 
            FormBox[
              RowBox[{"1", "-", "x"}],
              "TraditionalForm"], ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "aggd", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["C", "A"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    FractionBox["67", "9"], "-", 
                    RowBox[{"4", " ", 
                      RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
                RowBox[{"\[Delta]", "(", 
                  FormBox[
                    RowBox[{"1", "-", "x"}],
                    "TraditionalForm"], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"8", " ", "x"}], "-", "4"}], ")"}], " ", 
                RowBox[{"log", "(", 
                  RowBox[{
                    RowBox[{"(", 
                      RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], 
              "-", 
              RowBox[{"4", " ", 
                RowBox[{"(", 
                  RowBox[{
                    FractionBox[
                      RowBox[{"log", "(", "x", ")"}], 
                      RowBox[{"1", "-", "x"}]], "+", 
                    SubscriptBox[
                      RowBox[{"(", 
                        FractionBox[
                          RowBox[{"log", "(", 
                            RowBox[{"1", "-", "x"}], ")"}], 
                          RowBox[{"1", "-", "x"}]], ")"}], "+"]}], ")"}]}], 
              "+", "2"}], ")"}]}], "-", 
        RowBox[{
          FractionBox["20", "9"], " ", 
          SubscriptBox["T", "f"], " ", 
          RowBox[{"\[Delta]", "(", 
            FormBox[
              RowBox[{"1", "-", "x"}],
              "TraditionalForm"], ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "PQQS", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "F"], " ", 
        SubscriptBox["T", "f"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", "16"}], " ", 
              RowBox[{"(", 
                RowBox[{"x", "+", "1"}], ")"}], " ", 
              RowBox[{
                SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"48", " ", "x"}], "-", "16"}], ")"}], " ", 
              RowBox[{"log", "(", "x", ")"}]}], "+", 
            RowBox[{"16", " ", 
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}]}]}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "PQQNS", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", "4"}], " ", 
                RowBox[{"(", 
                  RowBox[{"x", "+", "1"}], ")"}], " ", 
                RowBox[{
                  SuperscriptBox["log", "2"], "(", "x", ")"}]}], "-", 
              RowBox[{"8", " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"2", " ", "x"}], "+", 
                    FractionBox["3", 
                      RowBox[{"1", "-", "x"}]]}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "-", 
              FractionBox[
                RowBox[{"16", " ", 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
                  RowBox[{"log", "(", 
                    RowBox[{"1", "-", "x"}], ")"}], " ", 
                  RowBox[{"log", "(", "x", ")"}]}], 
                RowBox[{"1", "-", "x"}]], "-", 
              RowBox[{"40", " ", 
                RowBox[{"(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"\[Delta]", "(", 
                  FormBox[
                    RowBox[{"1", "-", "x"}],
                    "TraditionalForm"], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      RowBox[{"-", "24"}], " ", 
                      RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
                    RowBox[{"48", " ", 
                      TagBox[
                        RowBox[{"\[Zeta]", "(", 
                          TagBox["3",
                            (Editable -> True)], ")"}],
                        InterpretTemplate[ Zeta[ #]&]]}], "+", "3"}], 
                  ")"}]}]}], ")"}], " ", 
          SubsuperscriptBox["C", "F", "2"]}], "+", 
        RowBox[{
          SubscriptBox["N", "f"], " ", 
          RowBox[{"(", 
            RowBox[{
              FractionBox[
                RowBox[{"88", " ", "x"}], "9"], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"-", 
                      FractionBox[
                        RowBox[{"16", " ", 
                          RowBox[{"\[Zeta]", "(", "2", ")"}]}], "3"]}], "-", 
                    FractionBox["2", "3"]}], ")"}], " ", 
                RowBox[{"\[Delta]", "(", 
                  FormBox[
                    RowBox[{"1", "-", "x"}],
                    "TraditionalForm"], ")"}]}], "-", 
              FractionBox[
                RowBox[{"8", " ", 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
                  RowBox[{"log", "(", "x", ")"}]}], 
                RowBox[{"3", " ", 
                  RowBox[{"(", 
                    RowBox[{"1", "-", "x"}], ")"}]}]], "-", 
              RowBox[{
                FractionBox["80", "9"], " ", 
                SubscriptBox[
                  RowBox[{"(", 
                    FractionBox["1", 
                      RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", 
              FractionBox["8", "9"]}], ")"}], " ", 
          SubscriptBox["C", "F"]}], "-", 
        RowBox[{"8", " ", 
          RowBox[{"(", 
            RowBox[{
              SubscriptBox["C", "F"], "-", 
              FractionBox[
                SubscriptBox["C", "A"], "2"]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"4", " ", 
                RowBox[{"(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{"2", " ", 
                RowBox[{"(", 
                  RowBox[{"x", "+", "1"}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "+", 
              FractionBox[
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        SuperscriptBox["log", "2"], "(", "x", ")"}], "-", 
                      RowBox[{"4", " ", 
                        RowBox[{"log", "(", 
                          RowBox[{"x", "+", "1"}], ")"}], " ", 
                        RowBox[{"log", "(", "x", ")"}]}], "-", 
                      RowBox[{"2", " ", 
                        RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
                      RowBox[{"4", " ", 
                        RowBox[{
                          SubscriptBox["Li", "2"], "(", 
                          RowBox[{"-", "x"}], ")"}]}]}], ")"}]}], 
                RowBox[{"x", "+", "1"}]]}], ")"}], " ", 
          SubscriptBox["C", "F"]}], "+", 
        RowBox[{
          SubscriptBox["C", "A"], " ", 
          RowBox[{"(", 
            RowBox[{
              FractionBox[
                RowBox[{"4", " ", 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
                  RowBox[{
                    SuperscriptBox["log", "2"], "(", "x", ")"}]}], 
                RowBox[{"1", "-", "x"}]], "-", 
              RowBox[{
                FractionBox["4", "3"], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"5", " ", "x"}], "-", 
                    FractionBox["22", 
                      RowBox[{"1", "-", "x"}]], "+", "5"}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "+", 
              RowBox[{
                FractionBox["4", "9"], " ", 
                RowBox[{"(", 
                  RowBox[{"53", "-", 
                    RowBox[{"187", " ", "x"}]}], ")"}]}], "+", 
              RowBox[{"8", " ", 
                RowBox[{"(", 
                  RowBox[{"x", "+", "1"}], ")"}], " ", 
                RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    FractionBox["536", "9"], "-", 
                    RowBox[{"16", " ", 
                      RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
                SubscriptBox[
                  RowBox[{"(", 
                    FractionBox["1", 
                      RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", 
              RowBox[{
                RowBox[{"\[Delta]", "(", 
                  FormBox[
                    RowBox[{"1", "-", "x"}],
                    "TraditionalForm"], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    FractionBox[
                      RowBox[{"88", " ", 
                        RowBox[{"\[Zeta]", "(", "2", ")"}]}], "3"], "-", 
                    RowBox[{"24", " ", 
                      TagBox[
                        RowBox[{"\[Zeta]", "(", 
                          TagBox["3",
                            (Editable -> True)], ")"}],
                        InterpretTemplate[ Zeta[ #]&]]}], "+", 
                    FractionBox["17", "3"]}], ")"}]}]}], ")"}], " ", 
          SubscriptBox["C", "F"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "PQG", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"4", " ", 
          SubscriptBox["C", "F"], " ", 
          SubscriptBox["T", "f"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"8", " ", "x"}], "-", "4"}], ")"}], " ", 
                RowBox[{
                  SuperscriptBox["log", "2"], "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{"16", "-", 
                    RowBox[{"16", " ", "x"}]}], ")"}], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{"8", "-", 
                    RowBox[{"16", " ", "x"}]}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"4", " ", "x"}], "-", "2"}], ")"}], " ", 
                RowBox[{
                  SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
              RowBox[{"54", " ", "x"}], "-", 
              RowBox[{"16", " ", "x", " ", 
                RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
              RowBox[{"8", " ", 
                RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
              RowBox[{"18", " ", 
                RowBox[{"log", "(", "x", ")"}]}], "-", "44"}], ")"}]}], "+", 
        RowBox[{"4", " ", 
          SubscriptBox["C", "A"], " ", 
          SubscriptBox["T", "f"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"(", 
                  RowBox[{"4", "-", 
                    RowBox[{"8", " ", "x"}]}], ")"}], " ", 
                RowBox[{
                  SuperscriptBox["log", "2"], "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"16", " ", "x"}], "-", "16"}], ")"}], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      RowBox[{"-", "8"}], " ", "x"}], "-", "4"}], ")"}], 
                " ", 
                RowBox[{
                  SuperscriptBox["log", "2"], "(", "x", ")"}]}], "-", 
              RowBox[{"44", " ", "x"}], "-", 
              RowBox[{"8", " ", 
                RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"32", " ", "x"}], "+", "4"}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      RowBox[{"-", "16"}], " ", "x"}], "-", "8"}], ")"}], 
                " ", 
                RowBox[{"log", "(", "x", ")"}], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"x", "+", "1"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      RowBox[{"-", "16"}], " ", "x"}], "-", "8"}], ")"}], 
                " ", 
                RowBox[{
                  SubscriptBox["Li", "2"], "(", 
                  RowBox[{"-", "x"}], ")"}]}], "+", "48"}], ")"}]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "PGQ", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SubscriptBox["C", "A"], " ", 
              SubscriptBox["C", "F"]}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"(", 
                  RowBox[{"16", "-", 
                    RowBox[{"8", " ", "x"}]}], ")"}], " ", 
                RowBox[{
                  SuperscriptBox["log", "2"], "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    FractionBox[
                      RowBox[{"8", " ", "x"}], "3"], "+", 
                    FractionBox["80", "3"]}], ")"}], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"16", " ", "x"}], "-", "32"}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"8", " ", "x"}], "+", "16"}], ")"}], " ", 
                RowBox[{
                  SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
              FractionBox[
                RowBox[{"280", " ", "x"}], "9"], "+", 
              RowBox[{"16", " ", "x", " ", 
                RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{"32", "-", 
                    RowBox[{"104", " ", "x"}]}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"16", " ", "x"}], "+", "32"}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"x", "+", "1"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"16", " ", "x"}], "+", "32"}], ")"}], " ", 
                RowBox[{
                  SubscriptBox["Li", "2"], "(", 
                  RowBox[{"-", "x"}], ")"}]}], "+", 
              FractionBox["328", "9"]}], ")"}]}], "+", 
        RowBox[{
          SubsuperscriptBox["C", "F", "2"], ".", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"8", " ", "x"}], "-", "16"}], ")"}], " ", 
                RowBox[{
                  SuperscriptBox["log", "2"], "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      RowBox[{"-", "8"}], " ", "x"}], "-", "16"}], ")"}], 
                " ", 
                RowBox[{"log", "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{"8", "-", 
                    RowBox[{"4", " ", "x"}]}], ")"}], " ", 
                RowBox[{
                  SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
              RowBox[{"32", " ", "x"}], "-", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"32", " ", "x"}], "+", "64"}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"36", " ", "x"}], "+", "48"}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "-", "68"}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SubscriptBox["C", "F"], " ", 
              SubscriptBox["T", "f"]}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"-", 
                FractionBox[
                  RowBox[{"32", " ", "x"}], "9"]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    FractionBox[
                      RowBox[{"32", " ", "x"}], "3"], "-", 
                    FractionBox["64", "3"]}], ")"}], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"1", "-", "x"}], ")"}]}], "-", 
              FractionBox["128", "9"]}], ")"}]}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SplittingFunction", "[", "PGG", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"-", 
                      FractionBox["8", 
                        RowBox[{"x", "+", "1"}]]}], "+", "32", "+", 
                    FractionBox["8", 
                      RowBox[{"1", "-", "x"}]]}], ")"}], " ", 
                RowBox[{
                  SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    FractionBox["232", "3"], "-", 
                    FractionBox[
                      RowBox[{"536", " ", "x"}], "3"]}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"64", " ", "x"}], "-", 
                    FractionBox["32", 
                      RowBox[{"1", "-", "x"}]], "-", "32"}], ")"}], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"1", "-", "x"}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"64", " ", "x"}], "+", 
                    FractionBox["32", 
                      RowBox[{"x", "+", "1"}]], "+", "32"}], ")"}], " ", 
                RowBox[{"log", "(", 
                  RowBox[{"x", "+", "1"}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "-", 
              FractionBox[
                RowBox[{"388", " ", "x"}], "9"], "+", 
              RowBox[{
                FractionBox["64", "3"], " ", 
                RowBox[{"\[Delta]", "(", 
                  FormBox[
                    RowBox[{"1", "-", "x"}],
                    "TraditionalForm"], ")"}]}], "+", 
              RowBox[{
                RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"64", " ", "x"}], "-", 
                    RowBox[{"16", " ", 
                      SubscriptBox[
                        RowBox[{"(", 
                          FractionBox["1", 
                            RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", 
                    FractionBox["16", 
                      RowBox[{"x", "+", "1"}]]}], ")"}]}], "+", 
              RowBox[{
                FractionBox["536", "9"], " ", 
                SubscriptBox[
                  RowBox[{"(", 
                    FractionBox["1", 
                      RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"64", " ", "x"}], "+", 
                    FractionBox["32", 
                      RowBox[{"x", "+", "1"}]], "+", "32"}], ")"}], " ", 
                RowBox[{
                  SubscriptBox["Li", "2"], "(", 
                  RowBox[{"-", "x"}], ")"}]}], "+", 
              RowBox[{"24", " ", 
                RowBox[{"\[Delta]", "(", 
                  FormBox[
                    RowBox[{"1", "-", "x"}],
                    "TraditionalForm"], ")"}], " ", 
                TagBox[
                  RowBox[{"\[Zeta]", "(", 
                    TagBox["3",
                      (Editable -> True)], ")"}],
                  InterpretTemplate[ Zeta[ #]&]]}], "-", 
              FractionBox["148", "9"]}], ")"}], " ", 
          SubsuperscriptBox["C", "A", "2"]}], "+", 
        RowBox[{
          SubscriptBox["T", "f"], " ", 
          RowBox[{"(", 
            RowBox[{
              FractionBox[
                RowBox[{"608", " ", "x"}], "9"], "-", 
              RowBox[{
                FractionBox["32", "3"], " ", 
                RowBox[{"\[Delta]", "(", 
                  FormBox[
                    RowBox[{"1", "-", "x"}],
                    "TraditionalForm"], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"-", 
                      FractionBox[
                        RowBox[{"32", " ", "x"}], "3"]}], "-", 
                    FractionBox["32", "3"]}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "-", 
              RowBox[{
                FractionBox["160", "9"], " ", 
                SubscriptBox[
                  RowBox[{"(", 
                    FractionBox["1", 
                      RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", 
              FractionBox["448", "9"]}], ")"}], " ", 
          SubscriptBox["C", "A"]}], "+", 
        RowBox[{
          SubscriptBox["C", "F"], " ", 
          SubscriptBox["T", "f"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      RowBox[{"-", "16"}], " ", "x"}], "-", "16"}], ")"}], 
                " ", 
                RowBox[{
                  SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"16", " ", "x"}], "-", "80"}], ")"}], " ", 
                RowBox[{"log", "(", "x", ")"}]}], "+", 
              RowBox[{"80", " ", "x"}], "-", 
              RowBox[{"8", " ", 
                RowBox[{"\[Delta]", "(", 
                  FormBox[
                    RowBox[{"1", "-", "x"}],
                    "TraditionalForm"], ")"}]}], "-", "80"}], ")"}]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["SumP", "Subsection",
  CellTags->"SumP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "SumP[k, m] is ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"2", "^", 
            RowBox[{"(", 
              RowBox[{"k", "-", "1"}], ")"}]}], 
          RowBox[{
            SubsuperscriptBox["\[Sum]", 
              RowBox[{"i", "=", "1"}], 
              RowBox[{"2", "m"}]], 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "+", 
                  RowBox[{
                    RowBox[{"(", 
                      RowBox[{"-", "1"}], ")"}], "^", "i"}]}], ")"}], "/", 
              SuperscriptBox["i", "k"]}]}]}], TraditionalForm]]]
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["SumS",
    ButtonData:>"SumS",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SumS"],
  ", ",
  ButtonBox["SumT",
    ButtonData:>"SumT",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SumT"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumP", "[", 
      RowBox[{"1", ",", 
        RowBox[{"m", "-", "1"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubsuperscriptBox["S", "1", "'"], "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          RowBox[{"m", "-", "1"}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumP", "[", 
      RowBox[{"2", ",", 
        RowBox[{"m", "-", "1"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubsuperscriptBox["S", "2", "'"], "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          RowBox[{"m", "-", "1"}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumP", "[", 
      RowBox[{"1", ",", "m"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubsuperscriptBox["S", "1", "'"], "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox["m",
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumP", "[", 
      RowBox[{"1", ",", "4"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox["25", "12"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "1"}], "8"], 
      RowBox[{
        RowBox[{"(", 
          RowBox[{"1", "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "^", "i"}]}], ")"}], "/", 
        "i"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox["25", "12"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Explicit", "[", 
      RowBox[{"SumP", "[", 
        RowBox[{"1", ",", 
          RowBox[{"n", "/", "2"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
            RowBox[{"1", "-", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "n"]}], ")"}], " ", 
          RowBox[{
            SubscriptBox["S", "1"], "(", 
            FormBox[
              FractionBox[
                RowBox[{"n", "-", "1"}], "2"],
              "TraditionalForm"], ")"}]}], "+", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"(", 
            RowBox[{"1", "+", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "n"]}], ")"}], " ", 
          RowBox[{
            SubscriptBox["S", "1"], "(", 
            FormBox[
              FractionBox["n", "2"],
              "TraditionalForm"], ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "/.", 
      RowBox[{"n", "\[Rule]", "8"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox["25", "12"], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["SumS", "Subsection",
  CellTags->"SumS"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "SumS[1, m] is the harmonic number ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubsuperscriptBox["\[Sum]", 
            RowBox[{"i", "=", "1"}], "m"], " ", 
          RowBox[{"i", "^", 
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}]}]}], TraditionalForm]]],
  " ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"(", 
            RowBox[{"=", 
              RowBox[{
                SubscriptBox["S", "1"], "(", "m", ")"}]}], " ", ")"}], "."}], 
        TraditionalForm]]],
  " SumS[1,1,m] is ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubsuperscriptBox["\[Sum]", 
            RowBox[{"i", "=", "1"}], "m"], 
          SubscriptBox["S", "1"]}], TraditionalForm]]],
  "(i)/i. SumS[k,l,m] is ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubsuperscriptBox["\[Sum]", 
            RowBox[{"i", "=", "1"}], "m"], " ", 
          RowBox[{
            RowBox[{
              SubscriptBox["S", "l"], "(", "i", ")"}], "/", 
            SuperscriptBox["i", "k"]}]}], TraditionalForm]]],
  ". SumS[r, n] represents Sum[Sign[r]^i/i^Abs[r], {i, 1, n}]. SumS[r,s, n] \
is Sum[Sign[r]^k/k^Abs[r] Sign[s]^j/j^Abs[s], {k, 1, n}, {j, 1, k}], etc."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "SumS", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Reduce", "\[Rule]", "False"}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["SumP",
    ButtonData:>"SumP",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SumP"],
  ", ",
  ButtonBox["SumT",
    ButtonData:>"SumT",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SumT"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumS", "[", 
      RowBox[{"1", ",", 
        RowBox[{"m", "-", "1"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["S", "1"], "(", 
        FormBox[
          RowBox[{"m", "-", "1"}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumS", "[", 
      RowBox[{"2", ",", 
        RowBox[{"m", "-", "1"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["S", "2"], "(", 
        FormBox[
          RowBox[{"m", "-", "1"}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumS", "[", 
      RowBox[{
        RowBox[{"-", "1"}], ",", "m"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["S", 
          RowBox[{"-", "1"}]], "(", "m", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumS", "[", 
      RowBox[{"1", ",", "m", ",", 
        RowBox[{"Reduce", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["S", "1"], "(", 
          FormBox[
            RowBox[{"m", "-", "1"}],
            "TraditionalForm"], ")"}], "+", 
        FractionBox["1", "m"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumS", "[", 
      RowBox[{"3", ",", 
        RowBox[{"m", "+", "2"}], ",", 
        RowBox[{"Reduce", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["S", "3"], "(", 
          FormBox[
            RowBox[{"m", "+", "1"}],
            "TraditionalForm"], ")"}], "+", 
        FractionBox["1", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"m", "+", "2"}], ")"}], "3"]]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"SumS", ",", 
          RowBox[{"Reduce", "\[Rule]", "True"}]}], "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumS", "[", 
      RowBox[{"3", ",", 
        RowBox[{"m", "+", "2"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["S", "3"], "(", 
          FormBox[
            RowBox[{"m", "-", "1"}],
            "TraditionalForm"], ")"}], "+", 
        FractionBox["1", 
          SuperscriptBox["m", "3"]], "+", 
        FractionBox["1", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"m", "+", "1"}], ")"}], "3"]], "+", 
        FractionBox["1", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"m", "+", "2"}], ")"}], "3"]]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"SumS", ",", 
          RowBox[{"Reduce", "\[Rule]", "False"}]}], "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumS", "[", 
      RowBox[{"1", ",", "4"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox["25", "12"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "1"}], "4"], 
      RowBox[{"1", "/", "i"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox["25", "12"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumS", "[", 
      RowBox[{"1", ",", "2", ",", 
        RowBox[{"m", "-", "1"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["S", "12"], "(", 
        FormBox[
          RowBox[{"m", "-", "1"}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumS", "[", 
      RowBox[{"1", ",", "1", ",", "1", ",", "11"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox["31276937512951", "4260000729600"], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumS", "[", 
      RowBox[{
        RowBox[{"-", "1"}], ",", "4"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox["7", "12"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumT", "[", 
      RowBox[{"1", ",", "4"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox["7", "12"]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["SumT", "Subsection",
  CellTags->"SumT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "SumT[1, m] is the alternative harmonic number ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubsuperscriptBox["\[Sum]", 
            RowBox[{"i", "=", "1"}], "m"], " ", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "^", "i"}], "/", 
            RowBox[{"i", " "}]}]}], TraditionalForm]]],
  " SumT[r, n] represents Sum[(-1)^i/i^r, {i,1,n}], SumT[r,s, n] is Sum[1/k^r \
(-1)^j/j^s, {k, 1, n}, {j, 1, k}]."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["SumP",
    ButtonData:>"SumP",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SumP"],
  ", ",
  ButtonBox["SumS",
    ButtonData:>"SumS",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SumS"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumT", "[", 
      RowBox[{"1", ",", 
        RowBox[{"m", "-", "1"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["S", "~"], "1"], "(", 
        FormBox[
          RowBox[{"m", "-", "1"}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumT", "[", 
      RowBox[{"2", ",", 
        RowBox[{"m", "-", "1"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["S", "~"], "2"], "(", 
        FormBox[
          RowBox[{"m", "-", "1"}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumT", "[", 
      RowBox[{"1", ",", "m"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["S", "~"], "1"], "(", "m", ")"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumT", "[", 
      RowBox[{"1", ",", "m", ",", 
        RowBox[{"Reduce", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox[
            OverscriptBox["S", "~"], "1"], "(", 
          FormBox[
            RowBox[{"m", "-", "1"}],
            "TraditionalForm"], ")"}], "+", 
        FractionBox[
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], "m"]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumT", "[", 
      RowBox[{"1", ",", "4"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox["7", "12"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "1"}], "4"], 
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "^", "i"}], "/", "i"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox["7", "12"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumT", "[", 
      RowBox[{"1", ",", "2", ",", 
        RowBox[{"m", "-", "1"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox[
          OverscriptBox["S", "~"], "12"], "(", 
        FormBox[
          RowBox[{"m", "-", "1"}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumT", "[", 
      RowBox[{"1", ",", "2", ",", "42"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox["38987958697055013360489864298703621429610152138683927", 
          "10512121660702378405316004964483761080879190528000000"]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumT", "[", 
      RowBox[{"1", ",", "4"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox["7", "12"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumS", "[", 
      RowBox[{
        RowBox[{"-", "1"}], ",", "4"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox["7", "12"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "1"}], 
        RowBox[{"m", "-", "1"}]], 
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "^", "i"}], "/", "i"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "2"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  TagBox["\[Psi]",
                    PolyGamma], 
                  RowBox[{"(", "0", ")"}]], "(", 
                FractionBox["m", "2"], ")"}], "-", 
              RowBox[{
                SuperscriptBox[
                  TagBox["\[Psi]",
                    PolyGamma], 
                  RowBox[{"(", "0", ")"}]], "(", 
                FractionBox[
                  RowBox[{"m", "+", "1"}], "2"], ")"}]}], ")"}]}], "-", 
        FractionBox[
          RowBox[{"log", "(", "4", ")"}], "2"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumT", "[", 
      RowBox[{"1", ",", "2", ",", "12"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox["57561743656913", "21300003648000"]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SumS", "[", 
      RowBox[{"1", ",", 
        RowBox[{"-", "2"}], ",", "42"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox["38987958697055013360489864298703621429610152138683927", 
          "10512121660702378405316004964483761080879190528000000"]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Array", "[", 
      RowBox[{"SumT", ",", "6"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"-", "1"}], ",", 
          RowBox[{"-", 
            FractionBox["5", "8"]}], ",", 
          RowBox[{"-", 
            FractionBox["179", "216"]}], ",", 
          RowBox[{"-", 
            FractionBox["1207", "1728"]}], ",", 
          RowBox[{"-", 
            FractionBox["170603", "216000"]}], ",", 
          RowBox[{"-", 
            FractionBox["155903", "216000"]}]}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Array", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"SumS", "[", 
            RowBox[{
              RowBox[{"-", "2"}], ",", "1", ",", "#1"}], "]"}], "&"}], ",", 
        "6"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"-", "1"}], ",", 
          RowBox[{"-", 
            FractionBox["5", "8"]}], ",", 
          RowBox[{"-", 
            FractionBox["179", "216"]}], ",", 
          RowBox[{"-", 
            FractionBox["1207", "1728"]}], ",", 
          RowBox[{"-", 
            FractionBox["170603", "216000"]}], ",", 
          RowBox[{"-", 
            FractionBox["155903", "216000"]}]}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["SUNDelta", "Subsection",
  CellTags->"SUNDelta"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
SUNDelta[a, b] is the Kronecker-delta for SU(N) with color indices \
a and b.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  ButtonBox["ExplicitSUNIndex",
    ButtonData:>"ExplicitSUNIndex",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ExplicitSUNIndex"],
  ", ",
  ButtonBox["SD",
    ButtonData:>"SD",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SD"],
  ", ",
  ButtonBox["SUNF",
    ButtonData:>"SUNF",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SUNF"],
  ", ",
  ButtonBox["SUNIndex",
    ButtonData:>"SUNIndex",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SUNIndex"],
  ", ",
  ButtonBox["SUNSimplify",
    ButtonData:>"SUNSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SUNSimplify"],
  ", ",
  ButtonBox["Trick",
    ButtonData:>"Trick",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Trick"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNDelta", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubscriptBox["\[Delta]", 
        RowBox[{"a", "\[NoBreak]", "b"}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Trick", "[", " ", 
      RowBox[{
        RowBox[{"SUNDelta", "[", 
          RowBox[{"a", ",", "b"}], "]"}], " ", 
        RowBox[{"SUNDelta", "[", 
          RowBox[{"b", ",", "c"}], "]"}]}], " ", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubscriptBox["\[Delta]", 
        RowBox[{
          FormBox[
            FormBox["a",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["c",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNDelta", "[", 
      RowBox[{
        RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
        RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubscriptBox["\[Delta]", 
        RowBox[{
          FormBox[
            FormBox["a",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["b",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SUNDelta", "[", 
        RowBox[{"a", ",", "b"}], "]"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"SUNDelta", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SUNDelta", "[", 
          RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"SUNDelta", "[", 
      RowBox[{
        RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
        RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"SUNDelta", "[", 
            RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", "FCE"}], 
      "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"SD", "[", 
      RowBox[{"a", ",", "b"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SD", "[", 
          RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"SUNDelta", "[", 
      RowBox[{
        RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
        RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell["\<\
Symbolic arguments to SUNDelta are transformed into the data type \
SUNIndex and integer arguments are transformed to ExplicitSUNIndex. The \
difference is that SUNSimplify will only sum over symbolic indices.\
\>", 
  "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SUNDelta", "[", 
          RowBox[{"a", ",", "2"}], "]"}], 
        RowBox[{"SUNDelta", "[", 
          RowBox[{"a", ",", "b"}], "]"}], 
        RowBox[{"SUNDelta", "[", 
          RowBox[{"c", ",", "2"}], "]"}]}], "//", "SUNSimplify"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox["2",
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox["2",
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"SUNDelta", "[", 
        RowBox[{
          RowBox[{"ExplicitSUNIndex", "[", "2", "]"}], ",", 
          RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}], " ", 
      RowBox[{"SUNDelta", "[", 
        RowBox[{
          RowBox[{"ExplicitSUNIndex", "[", "2", "]"}], ",", 
          RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SUNDelta", "[", 
          RowBox[{"1", ",", "2"}], "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"SUNDelta", "[", 
      RowBox[{
        RowBox[{"ExplicitSUNIndex", "[", "1", "]"}], ",", 
        RowBox[{"ExplicitSUNIndex", "[", "2", "]"}]}], "]"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["SUND", "Subsection",
  CellTags->"SUND"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "SUND[a,b,c] are the symmetric SU(N) ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["d", "abc"], "."}], TraditionalForm]]]
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["SUNDelta",
    ButtonData:>"SUNDelta",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SUNDelta"],
  ", ",
  ButtonBox["SUNF",
    ButtonData:>"SUNF",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SUNF"],
  ", ",
  ButtonBox["SUNSimplify",
    ButtonData:>"SUNSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SUNSimplify"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUND", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubscriptBox["d", 
        RowBox[{"a", "\[NoBreak]", "b", "\[NoBreak]", "c"}]], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"tt", "=", 
      RowBox[{"SUND", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", 
          RowBox[{"Explicit", "->", "True"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"2", " ", 
          RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
            FormBox[
              RowBox[{
                SubscriptBox["T", 
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"]], ".", 
                SubscriptBox["T", 
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"]], ".", 
                SubscriptBox["T", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"]]}],
              "TraditionalForm"], "\[NoBreak]", ")"}]}], "+", 
        RowBox[{"2", " ", 
          RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
            FormBox[
              RowBox[{
                SubscriptBox["T", 
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"]], ".", 
                SubscriptBox["T", 
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"]], ".", 
                SubscriptBox["T", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"]]}],
              "TraditionalForm"], "\[NoBreak]", ")"}]}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUND", "[", 
      RowBox[{"c", ",", "a", ",", "b"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubscriptBox["d", 
        RowBox[{"a", "\[NoBreak]", "b", "\[NoBreak]", "c"}]], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUND", "[", 
      RowBox[{"a", ",", "b", ",", "b"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{
        RowBox[{"SUND", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
        RowBox[{"SUND", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "2"}], " ", 
        RowBox[{"(", 
          RowBox[{"4", "-", 
            SubsuperscriptBox["C", "A", "2"]}], ")"}], " ", 
        SubscriptBox["C", "F"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SUNSimplify", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"SUND", "[", 
              RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
            RowBox[{"SUND", "[", 
              RowBox[{"a", ",", "b", ",", "c"}], "]"}]}], ",", 
          RowBox[{"SUNNToCACF", "->", "False"}]}], "]"}], "//", "Factor2"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          RowBox[{"(", 
            RowBox[{"1", "-", 
              SuperscriptBox["N", "2"]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{"4", "-", 
              SuperscriptBox["N", "2"]}], ")"}]}], "N"], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SUNSimplify", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"SUND", "[", 
              RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
            RowBox[{"SUND", "[", 
              RowBox[{"e", ",", "b", ",", "c"}], "]"}]}], ",", 
          RowBox[{"SUNNToCACF", "->", "False"}]}], "]"}], "//", "Factor2"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{"4", "-", 
                SuperscriptBox["N", "2"]}], ")"}], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["e",
                    "TraditionalForm"],
                  "TraditionalForm"]}]]}], "N"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SUND", "[", 
        RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", "StandardForm"}]], 
  "Input"],

Cell[BoxData[
    RowBox[{"SUND", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SUND", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"SUND", "[", 
      RowBox[{
        RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
        RowBox[{"SUNIndex", "[", "b", "]"}], ",", 
        RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"SUND", "[", 
            RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", "FCI"}], "//", 
        "FCE"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"SUND", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", "tt", "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["SUNF", "Subsection",
  CellTags->"SUNF"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
SUNF[a,b,c] are the structure constants of SU(N). The arguments \
a,b,c should be of symbolic type.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "SUNF", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Explicit", "\[Rule]", "False"}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["SUND",
    ButtonData:>"SUND",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SUND"],
  ", ",
  ButtonBox["SUNDelta",
    ButtonData:>"SUNDelta",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SUNDelta"],
  ", ",
  ButtonBox["SUNIndex",
    ButtonData:>"SUNIndex",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SUNIndex"],
  ", ",
  ButtonBox["SUNSimplify",
    ButtonData:>"SUNSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SUNSimplify"],
  ", ",
  ButtonBox["SUNT",
    ButtonData:>"SUNT",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SUNT"],
  ", ",
  ButtonBox["Trick",
    ButtonData:>"Trick",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Trick"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{
        RowBox[{
          RowBox[{"SUNF", "[", 
            RowBox[{"a", ",", "b", ",", "c"}], "]"}], "x"}], "+", 
        RowBox[{
          RowBox[{"SUNF", "[", 
            RowBox[{"b", ",", "a", ",", "c"}], "]"}], "y"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"x", " ", 
          SubscriptBox["f", 
            RowBox[{"a", "\[NoBreak]", "b", "\[NoBreak]", "c"}]]}], "+", 
        RowBox[{"y", " ", 
          SubscriptBox["f", 
            RowBox[{"b", "\[NoBreak]", "a", "\[NoBreak]", "c"}]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Calc", "[", "t1", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"x", " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{"y", " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", "t1", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"x", "-", "y"}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNF", "[", 
      RowBox[{"a", ",", "a", ",", "b"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubscriptBox["f", 
        RowBox[{"a", "\[NoBreak]", "a", "\[NoBreak]", "b"}]], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SUNF", "[", 
        RowBox[{"a", ",", "a", ",", "b"}], "]"}], "//", "Calc"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "This is a consequence of the usual choice for the normalization of the ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["T", "a"], " ", "generators", "in", " ", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"see", " ", " ", "SUNT"}], ")"}], "."}]}], 
        TraditionalForm]]]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNF", "[", 
      RowBox[{"a", ",", "b", ",", "c", ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"2", " ", "\[ImaginaryI]", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]]}],
                "TraditionalForm"], "\[NoBreak]", ")"}], "-", 
            RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"]]}],
                "TraditionalForm"], "\[NoBreak]", ")"}]}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "d"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "A"], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["d",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"]]}],
                "TraditionalForm"], "\[NoBreak]", ")"}], "-", 
            RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"]]}],
                "TraditionalForm"], "\[NoBreak]", ")"}]}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SUNF", "[", 
        RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", "StandardForm"}]], 
  "Input"],

Cell[BoxData[
    RowBox[{"SUNF", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"SUNF", "[", 
      RowBox[{
        RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
        RowBox[{"SUNIndex", "[", "b", "]"}], ",", 
        RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"SUNF", "[", 
            RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", "FCI"}], "//", 
        "FCE"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"SUNF", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNF", "[", 
      RowBox[{"b", ",", "a", ",", "c"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubscriptBox["f", 
        RowBox[{"b", "\[NoBreak]", "a", "\[NoBreak]", "c"}]], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SUNF", "[", 
        RowBox[{"b", ",", "a", ",", "c"}], "]"}], "//", "FCI"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["SUNIndex", "Subsection",
  CellTags->"SUNIndex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["SUNIndex is the head of SU(N)indices.", "Text"],

Cell[TextData[{
  "See also: ",
  ButtonBox["ExplicitSUNIndex",
    ButtonData:>"ExplicitSUNIndex",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ExplicitSUNIndex"],
  ", ",
  ButtonBox["SUNDelta",
    ButtonData:>"SUNDelta",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SUNDelta"],
  ", ",
  ButtonBox["SUNF",
    ButtonData:>"SUNF",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SUNF"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNIndex", "[", "i", "]"}]], "Input"],

Cell[BoxData[
    FormBox["i", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"SUNIndex", "[", "i", "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SUNDelta", "[", 
          RowBox[{"i", ",", "j"}], "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"SUNDelta", "[", 
      RowBox[{
        RowBox[{"SUNIndex", "[", "i", "]"}], ",", 
        RowBox[{"SUNIndex", "[", "j", "]"}]}], "]"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["SUNSimplify", "Subsection",
  CellTags->"SUNSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
SUNSimplify simplifies products of SUNT (and complex conjugated) \
matrices. Basic renaming of dummy indices is done. If the option SUNTrace is \
set to False, then any SUNT-matrices are taken out of DiracTrace[...]; \
otherwise a color-trace is taken (by SUNTrace) before taking the SUN-objects \
in front of DiracTrace[...].\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "SUNSimplify", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Expanding", "\[Rule]", "False"}], ",", 
          RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
          RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
          RowBox[{"SUNIndexRename", "\[Rule]", "True"}], ",", 
          RowBox[{"SUNFJacobi", "\[Rule]", "False"}], ",", 
          RowBox[{"SUNNToCACF", "\[Rule]", "True"}], ",", 
          RowBox[{"SUNTrace", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Trick",
    ButtonData:>"Trick",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Trick"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{
        RowBox[{"SUNDelta", "[", 
          RowBox[{"a", ",", "b"}], "]"}], " ", 
        RowBox[{"SUNDelta", "[", 
          RowBox[{"b", ",", "c"}], "]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["\[Delta]", 
          RowBox[{"a", "\[NoBreak]", "b"}]], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{"b", "\[NoBreak]", "c"}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", "t1", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubscriptBox["\[Delta]", 
        RowBox[{
          FormBox[
            FormBox["a",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["c",
              "TraditionalForm"],
            "TraditionalForm"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{
        RowBox[{"SUNT", "[", "a", "]"}], ".", 
        RowBox[{"SUNT", "[", "a", "]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["T", 
          FormBox["a",
            "TraditionalForm"]], ".", 
        SubscriptBox["T", 
          FormBox["a",
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", "t2", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubscriptBox["C", "F"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{"t2", ",", 
        RowBox[{"SUNNToCACF", "->", "False"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          SuperscriptBox["N", "2"], "-", "1"}], 
        RowBox[{"2", " ", "N"}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t3", "=", 
      RowBox[{
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "r", ",", "s"}], "]"}], 
        RowBox[{"SUNF", "[", 
          RowBox[{"b", ",", "r", ",", "s"}], "]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["f", 
          RowBox[{"a", "\[NoBreak]", "r", "\[NoBreak]", "s"}]], " ", 
        SubscriptBox["f", 
          RowBox[{"b", "\[NoBreak]", "r", "\[NoBreak]", "s"}]]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", "t3", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "A"], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t4", "=", 
      RowBox[{
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", ".", " ", 
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["f", 
          RowBox[{"a", "\[NoBreak]", "b", "\[NoBreak]", "c"}]], ".", 
        SubscriptBox["f", 
          RowBox[{"a", "\[NoBreak]", "b", "\[NoBreak]", "c"}]]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", "t4", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"2", " ", 
        SubsuperscriptBox["C", "A", "2"], " ", 
        SubscriptBox["C", "F"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t5", "=", 
      RowBox[{
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
        RowBox[{"SUNF", "[", 
          RowBox[{"d", ",", "b", ",", "c"}], "]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["f", 
          RowBox[{"a", "\[NoBreak]", "b", "\[NoBreak]", "c"}]], " ", 
        SubscriptBox["f", 
          RowBox[{"d", "\[NoBreak]", "b", "\[NoBreak]", "c"}]]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", "t5", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "A"], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["d",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t6", "=", 
      RowBox[{
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
        RowBox[{"SUND", "[", 
          RowBox[{"d", ",", "b", ",", "c"}], "]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["d", 
          RowBox[{"b", "\[NoBreak]", "c", "\[NoBreak]", "d"}]], " ", 
        SubscriptBox["f", 
          RowBox[{"a", "\[NoBreak]", "b", "\[NoBreak]", "c"}]]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{"t6", ",", 
        RowBox[{"Explicit", "->", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SUNSimplify", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"SUND", "[", 
              RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
            RowBox[{"SUND", "[", 
              RowBox[{"a", ",", "b", ",", "c"}], "]"}]}], ",", 
          RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}], "//", 
      "Factor2"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          RowBox[{"(", 
            RowBox[{"1", "-", 
              SuperscriptBox["N", "2"]}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{"4", "-", 
              SuperscriptBox["N", "2"]}], ")"}]}], "N"], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SUNSimplify", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"SUND", "[", 
              RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
            RowBox[{"SUND", "[", 
              RowBox[{"e", ",", "b", ",", "c"}], "]"}]}], ",", 
          RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}], "//", 
      "FullSimplify"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox["N", "2"], "-", "4"}], ")"}], " ", 
          SubscriptBox["\[Delta]", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "N"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"]]}],
                "TraditionalForm"], "\[NoBreak]", ")"}], "-", 
            RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"]]}],
                "TraditionalForm"], "\[NoBreak]", ")"}]}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{
        RowBox[{"SUND", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"2", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"]]}],
                "TraditionalForm"], "\[NoBreak]", ")"}], "+", 
            RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"]]}],
                "TraditionalForm"], "\[NoBreak]", ")"}]}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
        RowBox[{"SUNT", "[", 
          RowBox[{"c", ",", "b", ",", "a"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "2"]}], " ", "\[ImaginaryI]", " ", 
        SubscriptBox["C", "A"], " ", 
        SubscriptBox["C", "F"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a1", "=", 
      RowBox[{
        RowBox[{
          RowBox[{"SUNF", "[", 
            RowBox[{"a", ",", "b", ",", "e"}], "]"}], 
          RowBox[{"SUNF", "[", 
            RowBox[{"c", ",", "d", ",", "e"}], "]"}]}], "+", 
        RowBox[{
          RowBox[{"SUNF", "[", 
            RowBox[{"a", ",", "b", ",", "z"}], "]"}], 
          RowBox[{"SUNF", "[", 
            RowBox[{"c", ",", "d", ",", "z"}], "]"}]}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["f", 
            RowBox[{"a", "\[NoBreak]", "b", "\[NoBreak]", "e"}]], " ", 
          SubscriptBox["f", 
            RowBox[{"c", "\[NoBreak]", "d", "\[NoBreak]", "e"}]]}], "+", 
        RowBox[{
          SubscriptBox["f", 
            RowBox[{"a", "\[NoBreak]", "b", "\[NoBreak]", "z"}]], " ", 
          SubscriptBox["f", 
            RowBox[{"c", "\[NoBreak]", "d", "\[NoBreak]", "z"}]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{"a1", ",", 
        RowBox[{"Explicit", "\[Rule]", "False"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"2", " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"]}]], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["d",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{"a1", ",", 
        RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
        RowBox[{"SUNIndexRename", "\[Rule]", "False"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["z",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["z",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"a2", "=", 
      RowBox[{"DiracTrace", "[", 
        RowBox[{
          RowBox[{"SUNT", "[", 
            RowBox[{"a", ",", "b"}], "]"}], ".", 
          RowBox[{"DiracMatrix", "[", 
            RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"tr", "(", 
        FormBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                SubscriptBox["T", 
                  FormBox["a",
                    "TraditionalForm"]], 
                SubscriptBox["T", 
                  FormBox["b",
                    "TraditionalForm"]]}], ")"}], ".", 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox["\[Gamma]", "\[Mu]"], 
                SuperscriptBox["\[Gamma]", "\[Nu]"]}], ")"}]}],
          "TraditionalForm"], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", "a2", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"tr", "(", 
          FormBox[
            RowBox[{
              SuperscriptBox["\[Gamma]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], ".", 
              SuperscriptBox["\[Gamma]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}],
            "TraditionalForm"], ")"}], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{"a2", ",", 
        RowBox[{"SUNTrace", "\[Rule]", "False"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"tr", "(", 
          FormBox[
            RowBox[{
              SuperscriptBox["\[Gamma]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], ".", 
              SuperscriptBox["\[Gamma]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}],
            "TraditionalForm"], ")"}], ".", 
        SubscriptBox["T", 
          FormBox[
            FormBox["a",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SubscriptBox["T", 
          FormBox[
            FormBox["b",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SUNSimplify", "[", 
        RowBox[{"a2", ",", 
          RowBox[{"SUNTrace", "\[Rule]", "False"}]}], "]"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracTrace", "[", 
        RowBox[{
          RowBox[{"DiracGamma", "[", 
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ".", 
          RowBox[{"DiracGamma", "[", 
            RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], "]"}]}], "]"}], 
      ".", 
      RowBox[{"SUNT", "[", 
        RowBox[{"SUNIndex", "[", "a", "]"}], "]"}], ".", 
      RowBox[{"SUNT", "[", 
        RowBox[{"SUNIndex", "[", "b", "]"}], "]"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{"1", "-", 
        RowBox[{"SUNDelta", "[", 
          RowBox[{"i1", ",", "i2"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"1", "-", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["i1",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["i2",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"1", "-", 
      RowBox[{"SUNDelta", "[", 
        RowBox[{
          RowBox[{"SUNIndex", "[", "i1", "]"}], ",", 
          RowBox[{"SUNIndex", "[", "i2", "]"}]}], "]"}]}]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{
      "t1", ",", "t2", ",", "t3", ",", "t4", ",", "t5", ",", "t6", ",", "a1", 
        ",", "a2"}], "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["SUNT", "Subsection",
  CellTags->"SUNT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "SUNT[a] is the SU(N) ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["T", "a"], TraditionalForm]]],
  " generator in the fundamental representation."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["CA",
    ButtonData:>"CA",
    ButtonStyle->"Hyperlink",
    ButtonNote->"CA"],
  ", ",
  ButtonBox["CF",
    ButtonData:>"CF",
    ButtonStyle->"Hyperlink",
    ButtonNote->"CF"],
  ", ",
  ButtonBox["SUND",
    ButtonData:>"SUND",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SUND"],
  ", ",
  ButtonBox["SUNDelta",
    ButtonData:>"SUNDelta",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SUNDelta"],
  ", ",
  ButtonBox["SUNF",
    ButtonData:>"SUNF",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SUNF"],
  ", ",
  ButtonBox["SUNSimplify",
    ButtonData:>"SUNSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SUNSimplify"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNT", "[", "a", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubscriptBox["T", 
        FormBox["a",
          "TraditionalForm"]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "Since ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["T", "a"], TraditionalForm]]],
  " is a noncommutative object, products have to separated by a dot (.)."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SUNT", "[", "a", "]"}], ".", " ", 
      RowBox[{"SUNT", "[", "b", "]"}], ".", " ", 
      RowBox[{"SUNT", "[", "c", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["T", 
          FormBox["a",
            "TraditionalForm"]], ".", 
        SubscriptBox["T", 
          FormBox["b",
            "TraditionalForm"]], ".", 
        SubscriptBox["T", 
          FormBox["c",
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNT", "[", 
      RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["T", 
          FormBox["a",
            "TraditionalForm"]], 
        SubscriptBox["T", 
          FormBox["b",
            "TraditionalForm"]], 
        SubscriptBox["T", 
          FormBox["c",
            "TraditionalForm"]], 
        SubscriptBox["T", 
          FormBox["d",
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{
        RowBox[{"SUNT", "[", 
          RowBox[{"a", ",", "b", ",", "a"}], "]"}], ",", 
        RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          SubscriptBox["T", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], 
          RowBox[{"2", " ", "N"}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{"SUNT", "[", 
        RowBox[{"a", ",", "b", ",", "b", ",", "a"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      SubsuperscriptBox["C", "F", "2"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{"SUNT", "[", 
        RowBox[{"a", ",", "b", ",", "a"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "2"]}], " ", 
        RowBox[{"(", 
          RowBox[{
            SubscriptBox["C", "A"], "-", 
            RowBox[{"2", " ", 
              SubscriptBox["C", "F"]}]}], ")"}], " ", 
        SubscriptBox["T", 
          FormBox[
            FormBox["b",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{
        RowBox[{"SUNT", "[", 
          RowBox[{"a", ",", "b", ",", "a"}], "]"}], ",", 
        RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"-", 
        FractionBox[
          SubscriptBox["T", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]], 
          RowBox[{"2", " ", "N"}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "The normalizaton of the generators is chosen in the standard way, \
therefore ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"tr", "(", 
            RowBox[{
              SubscriptBox["T", "a"], 
              SubscriptBox["T", "b"]}], ")"}], " ", "=", " ", 
          RowBox[{
            RowBox[{"1", "/", "2"}], " ", 
            RowBox[{
              SubscriptBox["\[Delta]", "ab"], "."}]}]}], TraditionalForm]]]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNTrace", "[", 
      RowBox[{"SUNT", "[", 
        RowBox[{"a", ",", "b"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]], "2"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "In case you want ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["T", "f"], TraditionalForm]]],
  ", you need to include a factor 2Tf inside the trace."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNTrace", "[", 
      RowBox[{"2", " ", "Tf", " ", 
        RowBox[{"SUNT", "[", 
          RowBox[{"a", ",", "b"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["T", "f"], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SUNTrace", "[", 
        RowBox[{"SUNT", "[", 
          RowBox[{"a", ",", "b"}], "]"}], "]"}], "//", "StandardForm"}]], 
  "Input"],

Cell[BoxData[
    RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{"SUNDelta", "[", 
        RowBox[{
          RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
          RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SUNT", "[", "a", "]"}], "//", "FCI"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"SUNT", "[", 
      RowBox[{"SUNIndex", "[", "a", "]"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"SUNT", "[", "a", "]"}], "//", "FCI"}], "//", "FCE"}], 
      "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"SUNT", "[", "a", "]"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["SUNTrace", "Subsection",
  CellTags->"SUNTrace"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["SUNTrace[expr] calculates the color-trace.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "SUNTrace", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"Explicit", "\[Rule]", "False"}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["SUNSimplify",
    ButtonData:>"SUNSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SUNSimplify"],
  ", ",
  ButtonBox["Tr",
    ButtonData:>"Tr",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Tr"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNTrace", "[", 
      RowBox[{"SUNT", "[", 
        RowBox[{"a", ",", "b"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]], "2"], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNTrace", "[", 
      RowBox[{"SUNT", "[", 
        RowBox[{"a", ",", "b", ",", "c"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]]}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNTrace", "[", 
      RowBox[{
        RowBox[{"SUNT", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "4"], " ", 
          SubscriptBox["d", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNTrace", "[", 
      RowBox[{"SUNT", "[", 
        RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]]}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{"SUNTrace", "[", 
        RowBox[{
          RowBox[{"SUNT", "[", 
            RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}], ",", 
          RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "8"], " ", 
          SubscriptBox["d", 
            RowBox[{"e", "\[NoBreak]", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["d", 
            RowBox[{"e", "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "e"}]], " ", 
          SubscriptBox["d", 
            RowBox[{"e", "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "-", 
        RowBox[{
          FractionBox["1", "8"], " ", 
          SubscriptBox["d", 
            RowBox[{"e", "\[NoBreak]", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["d", 
            RowBox[{"e", "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        RowBox[{
          FractionBox["1", "8"], " ", 
          SubscriptBox["d", 
            RowBox[{"e", "\[NoBreak]", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["d", 
            RowBox[{"e", "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}], "+", 
        FractionBox[
          RowBox[{
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["d",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["c",
                    "TraditionalForm"],
                  "TraditionalForm"]}]]}], 
          RowBox[{"4", " ", "N"}]], "-", 
        FractionBox[
          RowBox[{
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["c",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["d",
                    "TraditionalForm"],
                  "TraditionalForm"]}]]}], 
          RowBox[{"4", " ", "N"}]], "+", 
        FractionBox[
          RowBox[{
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["c",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["d",
                    "TraditionalForm"],
                  "TraditionalForm"]}]]}], 
          RowBox[{"4", " ", "N"}]], "+", 
        RowBox[{
          FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
          SubscriptBox["d", 
            RowBox[{"e", "\[NoBreak]", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "e"}]]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{"t1", ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]]}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{"SUNTrace", "[", 
        RowBox[{
          RowBox[{"SUNT", "[", 
            RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", "e"}], "]"}], 
          ",", 
          RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SubscriptBox["\[Delta]", 
              RowBox[{
                FormBox[
                  FormBox["a",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["b",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FractionBox["1", "4"], " ", 
                  SubscriptBox["d", 
                    RowBox[{
                      FormBox[
                        FormBox["c",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["d",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["e",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], "+", 
                RowBox[{
                  FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
                  SubscriptBox["f", 
                    RowBox[{
                      FormBox[
                        FormBox["c",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["d",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["e",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}]}], ")"}]}], 
          RowBox[{"2", " ", "N"}]], "+", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          SubscriptBox["d", 
            RowBox[{"c101", "\[NoBreak]", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]}]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  FractionBox["1", "8"]}], " ", 
                SubscriptBox["d", 
                  RowBox[{"c102", "\[NoBreak]", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["e",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["d", 
                  RowBox[{"c102", "\[NoBreak]", 
                    FormBox[
                      FormBox["c101",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "+", 
              RowBox[{
                FractionBox["1", "8"], " ", 
                SubscriptBox["d", 
                  RowBox[{"c102", "\[NoBreak]", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["d", 
                  RowBox[{"c102", "\[NoBreak]", 
                    FormBox[
                      FormBox["c101",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["e",
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "+", 
              RowBox[{
                FractionBox["1", "8"], " ", 
                SubscriptBox["d", 
                  RowBox[{"c102", "\[NoBreak]", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["c101",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["d", 
                  RowBox[{"c102", "\[NoBreak]", 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["e",
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "-", 
              FractionBox[
                RowBox[{
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["e",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c101",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["d",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], 
                RowBox[{"4", " ", "N"}]], "+", 
              FractionBox[
                RowBox[{
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["d",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c101",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["e",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], 
                RowBox[{"4", " ", "N"}]], "+", 
              FractionBox[
                RowBox[{
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["c101",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["d",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["e",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], 
                RowBox[{"4", " ", "N"}]], "+", 
              RowBox[{
                FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
                SubscriptBox["d", 
                  RowBox[{"c102", "\[NoBreak]", 
                    FormBox[
                      FormBox["c101",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["e",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["f", 
                  RowBox[{
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", "c102"}]]}], "-", 
              RowBox[{
                FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
                SubscriptBox["d", 
                  RowBox[{"c102", "\[NoBreak]", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["f", 
                  RowBox[{
                    FormBox[
                      FormBox["c101",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["e",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", "c102"}]]}]}], 
            ")"}]}], "+", 
        RowBox[{
          FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", "c101"}]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  FractionBox["1", "8"]}], " ", 
                SubscriptBox["d", 
                  RowBox[{"c103", "\[NoBreak]", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["e",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["d", 
                  RowBox[{"c103", "\[NoBreak]", 
                    FormBox[
                      FormBox["c101",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "+", 
              RowBox[{
                FractionBox["1", "8"], " ", 
                SubscriptBox["d", 
                  RowBox[{"c103", "\[NoBreak]", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["d", 
                  RowBox[{"c103", "\[NoBreak]", 
                    FormBox[
                      FormBox["c101",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["e",
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "+", 
              RowBox[{
                FractionBox["1", "8"], " ", 
                SubscriptBox["d", 
                  RowBox[{"c103", "\[NoBreak]", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["c101",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["d", 
                  RowBox[{"c103", "\[NoBreak]", 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["e",
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], "-", 
              FractionBox[
                RowBox[{
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["e",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c101",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["d",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], 
                RowBox[{"4", " ", "N"}]], "+", 
              FractionBox[
                RowBox[{
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["d",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c101",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["e",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], 
                RowBox[{"4", " ", "N"}]], "+", 
              FractionBox[
                RowBox[{
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["c",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["c101",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SubscriptBox["\[Delta]", 
                    RowBox[{
                      FormBox[
                        FormBox["d",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["e",
                          "TraditionalForm"],
                        "TraditionalForm"]}]]}], 
                RowBox[{"4", " ", "N"}]], "+", 
              RowBox[{
                FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
                SubscriptBox["d", 
                  RowBox[{"c103", "\[NoBreak]", 
                    FormBox[
                      FormBox["c101",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["e",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["f", 
                  RowBox[{
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", "c103"}]]}], "-", 
              RowBox[{
                FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
                SubscriptBox["d", 
                  RowBox[{"c103", "\[NoBreak]", 
                    FormBox[
                      FormBox["c",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["d",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SubscriptBox["f", 
                  RowBox[{
                    FormBox[
                      FormBox["c101",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["e",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", "c103"}]]}]}], 
            ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{"t2", ",", 
        RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"tr", "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
          RowBox[{
            SubscriptBox["T", 
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["d",
                  "TraditionalForm"],
                "TraditionalForm"]], ".", 
            SubscriptBox["T", 
              FormBox[
                FormBox["e",
                  "TraditionalForm"],
                "TraditionalForm"]]}],
          "TraditionalForm"], "\[NoBreak]", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
        RowBox[{"SUND", "[", 
          RowBox[{"d", ",", "b", ",", "c"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
        RowBox[{"SUND", "[", 
          RowBox[{"a", ",", "b", ",", "d"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{
        RowBox[{"SUNF", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
        RowBox[{"SUND", "[", 
          RowBox[{"a", ",", "d", ",", "c"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SUNSimplify", "[", 
      RowBox[{
        RowBox[{"SUND", "[", 
          RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
        RowBox[{"SUND", "[", 
          RowBox[{"d", ",", "b", ",", "c"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          RowBox[{"(", 
            RowBox[{"4", "-", 
              SubsuperscriptBox["C", "A", "2"]}], ")"}]}], " ", 
        RowBox[{"(", 
          RowBox[{
            SubscriptBox["C", "A"], "-", 
            RowBox[{"2", " ", 
              SubscriptBox["C", "F"]}]}], ")"}], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["d",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Symmetrize", "Subsection",
  CellTags->"Symmetrize"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Symmetrize[expr, {a1, a2, ...}] antisymmetrizes expr with respect \
to the variables a1,a2, ... \
\>", "Text"],

Cell[TextData[{
  "See also: ",
  ButtonBox["AntiSymmetrize",
    ButtonData:>"AntiSymmetrize",
    ButtonStyle->"Hyperlink",
    ButtonNote->"AntiSymmetrize"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Symmetrize", "[", 
      RowBox[{
        RowBox[{"f", "[", 
          RowBox[{"a", ",", "b"}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"a", ",", "b"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"f", "(", 
              RowBox[{"a", ",", "b"}], ")"}], "+", 
            RowBox[{"f", "(", 
              RowBox[{"b", ",", "a"}], ")"}]}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Symmetrize", "[", 
      RowBox[{
        RowBox[{"f", "[", 
          RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "6"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"f", "(", 
              RowBox[{"x", ",", "y", ",", "z"}], ")"}], "+", 
            RowBox[{"f", "(", 
              RowBox[{"x", ",", "z", ",", "y"}], ")"}], "+", 
            RowBox[{"f", "(", 
              RowBox[{"y", ",", "x", ",", "z"}], ")"}], "+", 
            RowBox[{"f", "(", 
              RowBox[{"y", ",", "z", ",", "x"}], ")"}], "+", 
            RowBox[{"f", "(", 
              RowBox[{"z", ",", "x", ",", "y"}], ")"}], "+", 
            RowBox[{"f", "(", 
              RowBox[{"z", ",", "y", ",", "x"}], ")"}]}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Tdec", "Subsection",
  CellTags->"Tdec"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Tdec[{q,mu}, {p}]; Tdec[{{qi, mu}, {qj, nu}, ...}, {p1, p2, ...}] \
or Tdec[exp, {{qi, mu}, {qj, nu}, ...}, {p1, p2, ...}] calculates the \
tensorial decomposition formulas. The more common ones are saved in \
TIDL.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Tdec", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Factoring", "\[Rule]", "Factor2"}], ",", 
          RowBox[{"FeynCalcExternal", "\[Rule]", "True"}], ",", 
          RowBox[{"List", "\[Rule]", "True"}], ",", 
          RowBox[{"NumberOfMetricTensors", "\[Rule]", 
            InterpretationBox["\[Infinity]",
              DirectedInfinity[ 1]]}]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["TID",
    ButtonData:>"TID",
    ButtonStyle->"Hyperlink",
    ButtonNote->"TID"],
  ", ",
  ButtonBox["TIDL",
    ButtonData:>"TIDL",
    ButtonStyle->"Hyperlink",
    ButtonNote->"TIDL"],
  ", ",
  ButtonBox["OneLoopSimplify",
    ButtonData:>"OneLoopSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"OneLoopSimplify"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[TextData[{
  "Check that ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"\[Integral]", 
            RowBox[{
              SuperscriptBox["d", "D"], 
              RowBox[{"f", "(", 
                RowBox[{"p", ",", "q"}], ")"}], 
              SuperscriptBox["q", "\[Mu]"]}]}], "=", " ", 
          RowBox[{
            FractionBox[
              SuperscriptBox["p", "\[Mu]"], 
              SuperscriptBox["p", "2"]], 
            RowBox[{"\[Integral]", 
              RowBox[{
                SuperscriptBox["d", "D"], 
                RowBox[{"f", "(", 
                  RowBox[{"p", ",", "q"}], ")"}], 
                RowBox[{"p", "\[CenterDot]", "q"}]}]}]}]}], 
        TraditionalForm]]]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tdec", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", "p", "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              RowBox[{"X1", "\[Rule]", 
                FormBox[
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"],
                  "TraditionalForm"]}], ",", 
              RowBox[{"X2", "\[Rule]", 
                FormBox[
                  RowBox[{
                    FormBox["p",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}],
                  "TraditionalForm"]}]}], "}"}], ",", 
          FractionBox[
            RowBox[{"X2", " ", 
              SuperscriptBox["p", "\[Mu]"]}], "X1"]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"%", "[", 
        RowBox[{"[", "2", "]"}], "]"}], "/.", 
      RowBox[{"%", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          SuperscriptBox["p", "\[Mu]"], " ", 
          FormBox[
            RowBox[{
              FormBox["p",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["q",
                "TraditionalForm"]}],
            "TraditionalForm"]}], 
        FormBox[
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"],
          "TraditionalForm"]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "This calculates integral transformation for any ",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[Integral]", 
          RowBox[{
            SuperscriptBox["d", "D"], 
            SubscriptBox["q", "1"], 
            SuperscriptBox["d", "D"], 
            SubscriptBox["q", "2"], 
            SuperscriptBox["d", "D"], 
            SubscriptBox["q", "3"]}]}], TraditionalForm]]],
  " ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"f", "(", 
            RowBox[{"p", ",", 
              RowBox[{
                SubscriptBox["q", 
                  RowBox[{"1", ","}]], 
                SubscriptBox["q", "2"]}], ",", 
              SubscriptBox["q", "3"]}], ")"}], " ", 
          SubsuperscriptBox["q", "1", "\[Mu]"], 
          SubsuperscriptBox["q", "2", "\[Nu]"], 
          SubsuperscriptBox["q", "3", "\[Rho]"]}], TraditionalForm]]],
  "."
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"t", "=", 
        RowBox[{"Tdec", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SubscriptBox["q", "1"], ",", "\[Mu]"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    SubscriptBox["q", "2"], ",", "\[Nu]"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    SubscriptBox["q", "3"], ",", "\[Rho]"}], "}"}]}], "}"}], 
            ",", 
            RowBox[{"{", "p", "}"}]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"t", "[", 
        RowBox[{"[", "2", "]"}], "]"}], "/.", 
      RowBox[{"t", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{"1", "-", 
                SuperscriptBox["D", "2"]}], ")"}], " ", 
            SuperscriptBox["p", "\[Rho]"], " ", 
            SuperscriptBox["g", 
              RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]], " ", 
            FormBox[
              RowBox[{
                FormBox["p",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["q", "3"],
                  "TraditionalForm"]}],
              "TraditionalForm"], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FormBox[
                    RowBox[{
                      FormBox["p",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["q", "1"],
                        "TraditionalForm"]}],
                    "TraditionalForm"], " ", 
                  FormBox[
                    RowBox[{
                      FormBox["p",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["q", "2"],
                        "TraditionalForm"]}],
                    "TraditionalForm"]}], "-", 
                RowBox[{
                  FormBox[
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"],
                    "TraditionalForm"], " ", 
                  FormBox[
                    RowBox[{
                      FormBox[
                        SubscriptBox["q", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["q", "2"],
                        "TraditionalForm"]}],
                    "TraditionalForm"]}]}], ")"}]}], 
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"1", "-", "D"}], ")"}], "2"], " ", 
            RowBox[{"(", 
              RowBox[{"D", "+", "1"}], ")"}], " ", 
            SuperscriptBox[
              FormBox[
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"],
                "TraditionalForm"], "2"]}]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{"1", "-", 
                SuperscriptBox["D", "2"]}], ")"}], " ", 
            SuperscriptBox["p", "\[Nu]"], " ", 
            SuperscriptBox["g", 
              RowBox[{"\[Mu]", "\[NoBreak]", "\[Rho]"}]], " ", 
            FormBox[
              RowBox[{
                FormBox["p",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["q", "2"],
                  "TraditionalForm"]}],
              "TraditionalForm"], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FormBox[
                    RowBox[{
                      FormBox["p",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["q", "1"],
                        "TraditionalForm"]}],
                    "TraditionalForm"], " ", 
                  FormBox[
                    RowBox[{
                      FormBox["p",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["q", "3"],
                        "TraditionalForm"]}],
                    "TraditionalForm"]}], "-", 
                RowBox[{
                  FormBox[
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"],
                    "TraditionalForm"], " ", 
                  FormBox[
                    RowBox[{
                      FormBox[
                        SubscriptBox["q", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["q", "3"],
                        "TraditionalForm"]}],
                    "TraditionalForm"]}]}], ")"}]}], 
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"1", "-", "D"}], ")"}], "2"], " ", 
            RowBox[{"(", 
              RowBox[{"D", "+", "1"}], ")"}], " ", 
            SuperscriptBox[
              FormBox[
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"],
                "TraditionalForm"], "2"]}]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{"(", 
              RowBox[{"1", "-", 
                SuperscriptBox["D", "2"]}], ")"}], " ", 
            SuperscriptBox["p", "\[Mu]"], " ", 
            SuperscriptBox["g", 
              RowBox[{"\[Nu]", "\[NoBreak]", "\[Rho]"}]], " ", 
            FormBox[
              RowBox[{
                FormBox["p",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["q", "1"],
                  "TraditionalForm"]}],
              "TraditionalForm"], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FormBox[
                    RowBox[{
                      FormBox["p",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["q", "2"],
                        "TraditionalForm"]}],
                    "TraditionalForm"], " ", 
                  FormBox[
                    RowBox[{
                      FormBox["p",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["q", "3"],
                        "TraditionalForm"]}],
                    "TraditionalForm"]}], "-", 
                RowBox[{
                  FormBox[
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"],
                    "TraditionalForm"], " ", 
                  FormBox[
                    RowBox[{
                      FormBox[
                        SubscriptBox["q", "2"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["q", "3"],
                        "TraditionalForm"]}],
                    "TraditionalForm"]}]}], ")"}]}], 
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"1", "-", "D"}], ")"}], "2"], " ", 
            RowBox[{"(", 
              RowBox[{"D", "+", "1"}], ")"}], " ", 
            SuperscriptBox[
              FormBox[
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "2"],
                "TraditionalForm"], "2"]}]], "-", 
        RowBox[{
          FractionBox["1", 
            RowBox[{
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"1", "-", "D"}], ")"}], "2"], " ", 
              RowBox[{"(", 
                RowBox[{"D", "+", "1"}], ")"}], " ", 
              SuperscriptBox[
                FormBox[
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"],
                  "TraditionalForm"], "3"]}]], 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", 
                  SuperscriptBox["D", "2"]}], ")"}], " ", 
              SuperscriptBox["p", "\[Mu]"], " ", 
              SuperscriptBox["p", "\[Nu]"], " ", 
              SuperscriptBox["p", "\[Rho]"], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"D", " ", 
                    FormBox[
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["q", "1"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], " ", 
                    FormBox[
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["q", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], " ", 
                    FormBox[
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["q", "3"],
                          "TraditionalForm"]}],
                      "TraditionalForm"]}], "+", 
                  RowBox[{"2", " ", 
                    FormBox[
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["q", "1"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], " ", 
                    FormBox[
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["q", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], " ", 
                    FormBox[
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["q", "3"],
                          "TraditionalForm"]}],
                      "TraditionalForm"]}], "-", 
                  RowBox[{
                    FormBox[
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], "2"],
                      "TraditionalForm"], " ", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["q", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["q", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], " ", 
                    FormBox[
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["q", "3"],
                          "TraditionalForm"]}],
                      "TraditionalForm"]}], "-", 
                  RowBox[{
                    FormBox[
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], "2"],
                      "TraditionalForm"], " ", 
                    FormBox[
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["q", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], " ", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["q", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["q", "3"],
                          "TraditionalForm"]}],
                      "TraditionalForm"]}], "-", 
                  RowBox[{
                    FormBox[
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], "2"],
                      "TraditionalForm"], " ", 
                    FormBox[
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["q", "1"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], " ", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["q", "2"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["q", "3"],
                          "TraditionalForm"]}],
                      "TraditionalForm"]}]}], ")"}]}], ")"}]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Contract", "[", 
        RowBox[{
          RowBox[{"%", " ", 
            RowBox[{"FVD", "[", 
              RowBox[{"p", ",", "\[Mu]"}], "]"}]}], 
          RowBox[{"FVD", "[", 
            RowBox[{"p", ",", "\[Nu]"}], "]"}], " ", 
          RowBox[{"FVD", "[", 
            RowBox[{"p", ",", "\[Rho]"}], "]"}]}], "]"}], "//", "Factor"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox[
            SubscriptBox["q", "1"],
            "TraditionalForm"]}], " ", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox[
            SubscriptBox["q", "2"],
            "TraditionalForm"]}], " ", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox[
            SubscriptBox["q", "3"],
            "TraditionalForm"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", "t", "]"}], ";"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Tensorfunction", "Subsection",
  CellTags->"Tensorfunction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Tensorfunction[t, mu, nu, ...] transform into t[LorentzIndex[mu], \
LorentzIndex[nu], ...], i.e., it can be used as an unspecified tensoriell \
function t. A symmetric tensor can be obtained by Tensorfunction[{t, \"S\"}, \
mu, nu, ...], and an antisymmteric one by Tensorfunction[{t, \"A\"}, mu, nu, \
...].\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Symmetrize",
    ButtonData:>"Symmetrize",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Symmetrize"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tensorfunction", "[", 
      RowBox[{"t", ",", "\[Mu]", ",", "\[Nu]", ",", "\[Tau]"}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"t", "(", 
        RowBox[{
          FormBox["\[Mu]",
            "TraditionalForm"], ",", 
          FormBox["\[Nu]",
            "TraditionalForm"], ",", 
          FormBox["\[Tau]",
            "TraditionalForm"]}], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"t", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Tau]", "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Contract", "[", 
      RowBox[{
        RowBox[{"FV", "[", 
          RowBox[{"p", ",", "\[Mu]"}], "]"}], " ", "%"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"t", "(", 
        RowBox[{"p", ",", 
          FormBox["\[Nu]",
            "TraditionalForm"], ",", 
          FormBox["\[Tau]",
            "TraditionalForm"]}], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"t", "[", 
      RowBox[{
        RowBox[{"Momentum", "[", "p", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Tau]", "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tensorfunction", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"f", ",", "\"\<S\>\""}], "}"}], ",", "\[Alpha]", ",", 
        "\[Beta]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"f", "(", 
        RowBox[{
          FormBox["\[Alpha]",
            "TraditionalForm"], ",", 
          FormBox["\[Beta]",
            "TraditionalForm"]}], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tensorfunction", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"f", ",", "\"\<S\>\""}], "}"}], ",", "\[Beta]", ",", 
        "\[Alpha]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"f", "(", 
        RowBox[{
          FormBox["\[Alpha]",
            "TraditionalForm"], ",", 
          FormBox["\[Beta]",
            "TraditionalForm"]}], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"f", "[", 
      RowBox[{
        RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
        RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Attributes", "[", "f", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", "Orderless", "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"ClearAttributes", "[", 
      RowBox[{"f", ",", "Orderless"}], "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["TFi", "Subsection",
  CellTags->"TFi"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
TFi[d, pp, {{n1,m1},{n2,m2},{n3,m3},{n4,m4},{n5,m5}}] is the \
\.02loop d-dimensional integral 1/( (q1^2 - m1^2)^n1 (q2^2 - m2^2)^n2 \
((q1-p)^2 - m3^2)^n3 *((q2-p)^2 - m4^2)^n4 * ((q1-q2)^2 - m5^2)^n5 ) . TFi[d, \
pp, {x,y,z,v,w}, {{n1,m1},{n2,m2},{n3,m3},{n4,m4},{n5,m5}}] has as additional \
factors in the numerator (q1^2)^x*(q2^2)^y*(q1.p)^z* (q2.p)^v*(q1.q2)^w. \
TFi[d, pp, dp, {a,b}, {{n1,m1},{n2,m2},{n3,m3},{n4,m4},{n5,m5}}] has as \
additional factors in the numerator (OPEDelta.q1)^a * (OPEDelta.q2)^b; dp is \
(OPEDelta.p).\
\>", "Text"],

Cell[TextData[{
  "TFi is similar to TFI from the TARCER package, see ",
  " ",
  ButtonBox["hep-ph/9801383.",
    ButtonData:>{
      URL[ "http://xxx.lanl.gov/abs/hep-ph/9801383"], None},
    ButtonStyle->"Hyperlink"]
}], "Text"],

Cell["\<\
The function TarcerRecurse from the TARCER package recognize TFi \
(as well as TFI, which is defined in the HighEnergyPhysics`Tarcer` \
context).\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  ButtonBox["ToTFi",
    ButtonData:>"ToTFi",
    ButtonStyle->"Hyperlink",
    ButtonNote->"ToTFi"],
  ", ",
  ButtonBox["FromTFi",
    ButtonData:>"FromTFi",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FromTFi"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TFi", "[", 
      RowBox[{"D", ",", 
        RowBox[{"M", "^", "2"}], ",", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"2", ",", "m1"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"1", ",", "m2"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"3", ",", "m3"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"1", ",", "m4"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"1", ",", "m5"}], "}"}]}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        SubsuperscriptBox[
          StyleBox["F",
            SingleLetterItalics->False,
            FontWeight->"Bold"], 
          RowBox[{
            FormBox[
              RowBox[{"{", 
                RowBox[{"2", ",", "m1"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "m2"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"3", ",", "m3"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "m4"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "m5"}], "}"}],
              "TraditionalForm"]}], 
          RowBox[{"(", 
            FormBox["D",
              "TraditionalForm"], ")"}]],
        TFi[ D, 
          Power[ M, 2], {{2, m1}, {1, m2}, {3, m3}, {1, m4}, {1, m5}}],
        Editable->True], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TFi", "[", 
      RowBox[{"D", ",", 
        RowBox[{"M", "^", "2"}], ",", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"2", ",", "M"}], "}"}], ",", "1", ",", "1", ",", "1", 
            ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        SubsuperscriptBox[
          StyleBox["F",
            SingleLetterItalics->False,
            FontWeight->"Bold"], 
          RowBox[{
            FormBox[
              RowBox[{"{", 
                RowBox[{"2", ",", "M"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}],
              "TraditionalForm"]}], 
          RowBox[{"(", 
            FormBox["D",
              "TraditionalForm"], ")"}]],
        TFi[ D, 
          Power[ M, 2], {{2, M}, {1, 0}, {1, 0}, {1, 0}, {1, 0}}],
        Editable->True], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["TIDL", "Subsection",
  CellTags->"TIDL"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[" TIDL is a database of tensorial reduction formalae.", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["TID",
    ButtonData:>"TID",
    ButtonStyle->"Hyperlink",
    ButtonNote->"TID"]
}], "Text"]
}, Open  ]],

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["B", "\[Mu]"], "-", "type"}], TraditionalForm]]]], 
  "Subsubsection"],

Cell[TextData[{
  "In any n-dimensional integral ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"\[Integral]", 
            RowBox[{
              SuperscriptBox["d", "n"], "\[NegativeVeryThinSpace]", " ", 
              SuperscriptBox["q", "\[Mu]"], " ", 
              RowBox[{"f", "(", 
                RowBox[{"q", ",", "p"}], ")"}], " ", "the", " ", 
              SuperscriptBox["q", "\[Mu]"]}]}], " "}], TraditionalForm]]],
  "can be replaced by"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TIDL", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", "p", "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"]}]}], 
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], "2"]], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["B", "\[Mu]\[Nu]"], "-", "type"}], TraditionalForm]]]], 
  "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TIDL", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"q", ",", "\[Nu]"}], "}"}]}], "}"}], ",", 
        RowBox[{"{", "p", "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SuperscriptBox["g", 
              RowBox[{
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"], "-", 
                RowBox[{
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"], " ", 
                  SuperscriptBox[
                    FormBox["q",
                      "TraditionalForm"], "2"]}]}], ")"}]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"1", "-", "D"}], ")"}], " ", 
            SuperscriptBox[
              FormBox["p",
                "TraditionalForm"], "2"]}]], "-", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              FormBox["p",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            SuperscriptBox[
              FormBox["p",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Nu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"D", " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      FormBox[
                        RowBox[{
                          FormBox["p",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}],
                        "TraditionalForm"], ")"}], "2"]}], "-", 
                RowBox[{
                  SuperscriptBox[
                    FormBox["p",
                      "TraditionalForm"], "2"], " ", 
                  SuperscriptBox[
                    FormBox["q",
                      "TraditionalForm"], "2"]}]}], ")"}]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"1", "-", "D"}], ")"}], " ", 
            SuperscriptBox[
              FormBox["p",
                "TraditionalForm"], "4"]}]]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "\[Mu]"], "-", "type"}], TraditionalForm]]]], 
  "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TIDL", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
            SubscriptBox["p", "1"], ",", 
            SubscriptBox["p", "2"]}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["2",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{
                FormBox["q",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["p", "1"],
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox[
                  SubscriptBox["p", "1"],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["p", "2"],
                  "TraditionalForm"]}]}], "-", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["2",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{
                FormBox["q",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["p", "2"],
                  "TraditionalForm"]}], " ", 
              SubsuperscriptBox["p", "1", "2"]}]}], 
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                FormBox[
                  RowBox[{
                    FormBox[
                      SubscriptBox["p", "1"],
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox[
                      SubscriptBox["p", "2"],
                      "TraditionalForm"]}],
                  "TraditionalForm"], ")"}], "2"], "-", 
            RowBox[{
              SubsuperscriptBox["p", "1", "2"], " ", 
              SubsuperscriptBox["p", "2", "2"]}]}]], "+", 
        FractionBox[
          RowBox[{
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["1",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{
                FormBox["q",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["p", "2"],
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox[
                  SubscriptBox["p", "1"],
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["p", "2"],
                  "TraditionalForm"]}]}], "-", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["1",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{
                FormBox["q",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox[
                  SubscriptBox["p", "1"],
                  "TraditionalForm"]}], " ", 
              SubsuperscriptBox["p", "2", "2"]}]}], 
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                FormBox[
                  RowBox[{
                    FormBox[
                      SubscriptBox["p", "1"],
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox[
                      SubscriptBox["p", "2"],
                      "TraditionalForm"]}],
                  "TraditionalForm"], ")"}], "2"], "-", 
            RowBox[{
              SubsuperscriptBox["p", "1", "2"], " ", 
              SubsuperscriptBox["p", "2", "2"]}]}]]}], TraditionalForm]], 
  "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "\[Mu]\[Nu]"], "-", "type"}], TraditionalForm]]]], 
  "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TIDL", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"q", ",", "\[Nu]"}], "}"}]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
            SubscriptBox["p", "1"], ",", 
            SubscriptBox["p", "2"]}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SuperscriptBox["g", 
              RowBox[{
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]}]], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  SubsuperscriptBox["p", "2", "2"], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      FormBox[
                        RowBox[{
                          FormBox["q",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox[
                            SubscriptBox["p", "1"],
                            "TraditionalForm"]}],
                        "TraditionalForm"], ")"}], "2"]}], "-", 
                RowBox[{"2", " ", 
                  RowBox[{
                    FormBox["q",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox[
                      SubscriptBox["p", "2"],
                      "TraditionalForm"]}], " ", 
                  RowBox[{
                    FormBox[
                      SubscriptBox["p", "1"],
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox[
                      SubscriptBox["p", "2"],
                      "TraditionalForm"]}], " ", 
                  RowBox[{
                    FormBox["q",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox[
                      SubscriptBox["p", "1"],
                      "TraditionalForm"]}]}], "+", 
                RowBox[{
                  SuperscriptBox[
                    FormBox["q",
                      "TraditionalForm"], "2"], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      FormBox[
                        RowBox[{
                          FormBox[
                            SubscriptBox["p", "1"],
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox[
                            SubscriptBox["p", "2"],
                            "TraditionalForm"]}],
                        "TraditionalForm"], ")"}], "2"]}], "+", 
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      FormBox[
                        RowBox[{
                          FormBox["q",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox[
                            SubscriptBox["p", "2"],
                            "TraditionalForm"]}],
                        "TraditionalForm"], ")"}], "2"], " ", 
                  SubsuperscriptBox["p", "1", "2"]}], "-", 
                RowBox[{
                  SuperscriptBox[
                    FormBox["q",
                      "TraditionalForm"], "2"], " ", 
                  SubsuperscriptBox["p", "1", "2"], " ", 
                  SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"D", "-", "2"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"], "-", 
                RowBox[{
                  SubsuperscriptBox["p", "1", "2"], " ", 
                  SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}]], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["2",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SubsuperscriptBox["p", 
                FormBox["2",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"D", " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"]}], "-", 
                  RowBox[{"2", " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"]}], "+", 
                  RowBox[{
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    SubsuperscriptBox["p", "2", "2"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"]}], "-", 
                  RowBox[{"2", " ", "D", " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{"2", " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{"D", " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SubsuperscriptBox["p", "1", "4"]}], "-", 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SubsuperscriptBox["p", "1", "4"]}], "+", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["q",
                        "TraditionalForm"], "2"], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"]}], "-", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["q",
                        "TraditionalForm"], "2"], " ", 
                    SubsuperscriptBox["p", "1", "4"], " ", 
                    SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}], ")"}], 
          "/", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"D", "-", "2"}], ")"}], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], "-", 
                    RowBox[{
                      SubsuperscriptBox["p", "1", "2"], " ", 
                      SubsuperscriptBox["p", "2", "2"]}]}], ")"}], "2"]}], 
            ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["2",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SubsuperscriptBox["p", 
                FormBox["1",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    RowBox[{"-", 
                      SuperscriptBox[
                        FormBox["q",
                          "TraditionalForm"], "2"]}], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "3"]}], "+", 
                  RowBox[{"D", " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"]}], "-", 
                  RowBox[{"D", " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}]}], "-", 
                  RowBox[{"D", " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SubsuperscriptBox["p", "2", "2"], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SubsuperscriptBox["p", "2", "2"], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["q",
                        "TraditionalForm"], "2"], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    SubsuperscriptBox["p", "2", "2"], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{"D", " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    SubsuperscriptBox["p", "2", "2"]}], "-", 
                  RowBox[{"2", " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}], ")"}], 
          "/", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"D", "-", "2"}], ")"}], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], "-", 
                    RowBox[{
                      SubsuperscriptBox["p", "1", "2"], " ", 
                      SubsuperscriptBox["p", "2", "2"]}]}], ")"}], "2"]}], 
            ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["1",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SubsuperscriptBox["p", 
                FormBox["2",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    RowBox[{"-", 
                      SuperscriptBox[
                        FormBox["q",
                          "TraditionalForm"], "2"]}], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "3"]}], "+", 
                  RowBox[{"D", " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"]}], "-", 
                  RowBox[{"D", " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}]}], "-", 
                  RowBox[{"D", " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SubsuperscriptBox["p", "2", "2"], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SubsuperscriptBox["p", "2", "2"], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["q",
                        "TraditionalForm"], "2"], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    SubsuperscriptBox["p", "2", "2"], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{"D", " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    SubsuperscriptBox["p", "2", "2"]}], "-", 
                  RowBox[{"2", " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}], ")"}], 
          "/", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"D", "-", "2"}], ")"}], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], "-", 
                    RowBox[{
                      SubsuperscriptBox["p", "1", "2"], " ", 
                      SubsuperscriptBox["p", "2", "2"]}]}], ")"}], "2"]}], 
            ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["1",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SubsuperscriptBox["p", 
                FormBox["1",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"D", " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"]}], "-", 
                  RowBox[{"2", " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"]}], "+", 
                  RowBox[{
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    SubsuperscriptBox["p", "2", "2"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"]}], "-", 
                  RowBox[{"2", " ", "D", " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "2", "2"], " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{"2", " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "2", "2"], " ", 
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{"D", " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SubsuperscriptBox["p", "2", "4"]}], "-", 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["q",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SubsuperscriptBox["p", "2", "4"]}], "-", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["q",
                        "TraditionalForm"], "2"], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    SubsuperscriptBox["p", "2", "4"]}], "+", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["q",
                        "TraditionalForm"], "2"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], " ", 
                    SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}], ")"}], 
          "/", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"D", "-", "2"}], ")"}], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"], "-", 
                    RowBox[{
                      SubsuperscriptBox["p", "1", "2"], " ", 
                      SubsuperscriptBox["p", "2", "2"]}]}], ")"}], "2"]}], 
            ")"}]}]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "\[Mu]\[Nu]\[Rho]"], "-", "type"}], 
      TraditionalForm]]]], "Subsubsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{"TIDL", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Rho]"}], "}"}]}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"p", ",", "k"}], "}"}]}], "]"}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["C", "\[Mu]\[Nu]\[Rho]\[Sigma]"], "-", "type"}], 
      TraditionalForm]]]], "Subsubsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{"TIDL", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Rho]"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Sigma]"}], "}"}]}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"p", ",", "k"}], "}"}]}], "]"}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["D", "\[Mu]"], "-", "type"}], TraditionalForm]]]], 
  "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TIDL", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
            SubscriptBox["p", "1"], ",", 
            SubscriptBox["p", "2"], ",", 
            SubscriptBox["p", "3"]}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["3",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"]}], "-", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}]}], "-", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}]}], "-", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    SubsuperscriptBox["p", "2", "2"]}], "+", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "2", "2"]}], "+", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}]}]}], ")"}]}], ")"}], "/", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SubsuperscriptBox["p", "3", "2"], " ", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"]}], "-", 
              RowBox[{"2", " ", 
                RowBox[{
                  FormBox[
                    SubscriptBox["p", "1"],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "3"],
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox[
                    SubscriptBox["p", "2"],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "3"],
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox[
                    SubscriptBox["p", "1"],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "2"],
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SubsuperscriptBox["p", "1", "2"], " ", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "2"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "3"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"]}], "+", 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "3"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"], " ", 
                SubsuperscriptBox["p", "2", "2"]}], "-", 
              RowBox[{
                SubsuperscriptBox["p", "1", "2"], " ", 
                SubsuperscriptBox["p", "2", "2"], " ", 
                SubsuperscriptBox["p", "3", "2"]}]}], ")"}]}], "-", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["2",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    RowBox[{"-", 
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["q",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox[
                            SubscriptBox["p", "2"],
                            "TraditionalForm"]}], ")"}]}], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "1"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "3"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"]}], "+", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}]}], "-", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "1", "2"], " ", 
                    SubsuperscriptBox["p", "3", "2"]}], "-", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "3", "2"]}]}], ")"}]}], ")"}], 
          "/", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SubsuperscriptBox["p", "3", "2"], " ", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"]}], "-", 
              RowBox[{"2", " ", 
                RowBox[{
                  FormBox[
                    SubscriptBox["p", "1"],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "3"],
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox[
                    SubscriptBox["p", "2"],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "3"],
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox[
                    SubscriptBox["p", "1"],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "2"],
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SubsuperscriptBox["p", "1", "2"], " ", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "2"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "3"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"]}], "+", 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "3"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"], " ", 
                SubsuperscriptBox["p", "2", "2"]}], "-", 
              RowBox[{
                SubsuperscriptBox["p", "1", "2"], " ", 
                SubsuperscriptBox["p", "2", "2"], " ", 
                SubsuperscriptBox["p", "3", "2"]}]}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              SubsuperscriptBox["p", 
                FormBox["1",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox[
                              SubscriptBox["p", "2"],
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox[
                              SubscriptBox["p", "3"],
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"]}], "-", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}]}], "-", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}]}], "+", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "3"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "2", "2"]}], "+", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "3", "2"]}], "-", 
                  RowBox[{
                    RowBox[{
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"]}], " ", 
                    SubsuperscriptBox["p", "2", "2"], " ", 
                    SubsuperscriptBox["p", "3", "2"]}]}], ")"}]}], ")"}], 
          "/", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SubsuperscriptBox["p", "3", "2"], " ", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"]}], "-", 
              RowBox[{"2", " ", 
                RowBox[{
                  FormBox[
                    SubscriptBox["p", "1"],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "3"],
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox[
                    SubscriptBox["p", "2"],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "3"],
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox[
                    SubscriptBox["p", "1"],
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox[
                    SubscriptBox["p", "2"],
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SubsuperscriptBox["p", "1", "2"], " ", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "2"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "3"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"]}], "+", 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "3"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"], " ", 
                SubsuperscriptBox["p", "2", "2"]}], "-", 
              RowBox[{
                SubsuperscriptBox["p", "1", "2"], " ", 
                SubsuperscriptBox["p", "2", "2"], " ", 
                SubsuperscriptBox["p", "3", "2"]}]}], ")"}]}]}], 
      TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["D", "\[Mu]\[Nu]"], "-", "type"}], TraditionalForm]]]], 
  "Subsubsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{"TIDL", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Nu]"}], "}"}]}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"p", ",", "k", ",", "r"}], "}"}]}], "]"}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["D", "\[Mu]\[Nu]\[Rho]"], "-", "type"}], 
      TraditionalForm]]]], "Subsubsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{"TIDL", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Rho]"}], "}"}]}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"p", ",", "k", ",", "r"}], "}"}]}], "]"}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["D", "\[Mu]\[Nu]\[Rho]\[Sigma]"], "-", "type"}], 
      TraditionalForm]]]], "Subsubsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{"TIDL", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Rho]"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"q", ",", "\[Sigma]"}], "}"}]}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"p", ",", "k", ",", "r", ",", "s"}], "}"}]}], "]"}], 
      ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"Length", "[", "%", "]"}], ",", " ", 
        RowBox[{"LeafCount", "[", "%", "]"}], ",", 
        RowBox[{"ByteCount", "[", "%", "]"}]}], "}"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"4", ",", "29", ",", "320"}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["TID", "Subsection",
  CellTags->"TID"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["TID[amp,q] performs a tensorial decomposition.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "TID", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Collecting", "\[Rule]", "True"}], ",", 
          RowBox[{"Contract", "\[Rule]", "False"}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"ChangeDimension", "\[Rule]", "D"}], ",", 
          RowBox[{"DimensionalReduction", "\[Rule]", "False"}], ",", 
          RowBox[{"FeynAmpDenominatorCombine", "\[Rule]", "True"}], ",", 
          RowBox[{"FeynAmpDenominatorSimplify", "\[Rule]", "False"}], ",", 
          RowBox[{"Isolate", "\[Rule]", "False"}], ",", 
          RowBox[{"ScalarProductCancel", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["OneLoopSimplify",
    ButtonData:>"OneLoopSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"OneLoopSimplify"],
  ", ",
  ButtonBox["TIDL",
    ButtonData:>"TIDL",
    ButtonStyle->"Hyperlink",
    ButtonNote->"TIDL"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"FAD", "[", 
          RowBox[{"k", ",", " ", 
            RowBox[{"k", " ", "-", " ", 
              SubscriptBox["p", "1"]}], ",", " ", 
            RowBox[{"k", " ", "-", " ", 
              SubscriptBox["p", "2"]}]}], "]"}], 
        RowBox[{"FVD", "[", 
          RowBox[{"k", ",", "\[Mu]"}], "]"}]}], "//", "FCI"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        SuperscriptBox[
          FormBox["k",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], 
        RowBox[{
          SuperscriptBox["k", "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"k", "-", 
                SubscriptBox["p", "1"]}], ")"}], "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"k", "-", 
                SubscriptBox["p", "2"]}], ")"}], "2"]}]], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Factor", "/@", 
        RowBox[{"TID", "[", 
          RowBox[{"%", ",", "k"}], "]"}]}], "//", "FCE"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox[
            RowBox[{
              FormBox[
                FractionBox["1", 
                  RowBox[{
                    RowBox[{"(", 
                      InterpretationBox[
                        RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                          SuperscriptBox["k", "2"], "\[InvisibleSpace]", 
                          "\<\"]\"\>"}],
                        SequenceForm[ "[", 
                          Power[ k, 2], "]"],
                        Editable->False], ")"}], " ", 
                    RowBox[{"(", 
                      InterpretationBox[
                        RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                          SuperscriptBox[
                            RowBox[{"(", 
                              RowBox[{"k", "-", 
                                SubscriptBox["p", "1"]}], ")"}], "2"], 
                          "\[InvisibleSpace]", "\<\"]\"\>"}],
                        SequenceForm[ "[", 
                          Power[ 
                            Plus[ k, 
                              Times[ -1, 
                                Subscript[ p, 1]]], 2], "]"],
                        Editable->False], ")"}]}]],
                "TraditionalForm"], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SubsuperscriptBox["p", 
                      FormBox["2",
                        "TraditionalForm"], "\[Mu]"], " ", 
                    FormBox[
                      SubsuperscriptBox["p", "1", "2"],
                      "TraditionalForm"]}], "-", 
                  RowBox[{
                    SubsuperscriptBox["p", 
                      FormBox["1",
                        "TraditionalForm"], "\[Mu]"], " ", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"]}]}], ")"}]}], 
            RowBox[{"2", " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    FormBox[
                      SubsuperscriptBox["p", "1", "2"],
                      "TraditionalForm"], " ", 
                    FormBox[
                      SubsuperscriptBox["p", "2", "2"],
                      "TraditionalForm"]}], "-", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      FormBox[
                        RowBox[{
                          FormBox[
                            SubscriptBox["p", "1"],
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox[
                            SubscriptBox["p", "2"],
                            "TraditionalForm"]}],
                        "TraditionalForm"], ")"}], "2"]}], ")"}]}]]}], "-", 
        FractionBox[
          RowBox[{
            FormBox[
              FractionBox["1", 
                RowBox[{
                  RowBox[{"(", 
                    InterpretationBox[
                      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                        SuperscriptBox["k", "2"], "\[InvisibleSpace]", 
                        "\<\"]\"\>"}],
                      SequenceForm[ "[", 
                        Power[ k, 2], "]"],
                      Editable->False], ")"}], " ", 
                  RowBox[{"(", 
                    InterpretationBox[
                      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{"k", "-", 
                              SubscriptBox["p", "1"]}], ")"}], "2"], 
                        "\[InvisibleSpace]", "\<\"]\"\>"}],
                      SequenceForm[ "[", 
                        Power[ 
                          Plus[ k, 
                            Times[ -1, 
                              Subscript[ p, 1]]], 2], "]"],
                      Editable->False], ")"}], " ", 
                  RowBox[{"(", 
                    InterpretationBox[
                      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{"k", "-", 
                              SubscriptBox["p", "2"]}], ")"}], "2"], 
                        "\[InvisibleSpace]", "\<\"]\"\>"}],
                      SequenceForm[ "[", 
                        Power[ 
                          Plus[ k, 
                            Times[ -1, 
                              Subscript[ p, 2]]], 2], "]"],
                      Editable->False], ")"}]}]],
              "TraditionalForm"], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  SubsuperscriptBox["p", 
                    FormBox["2",
                      "TraditionalForm"], "\[Mu]"], " ", 
                  FormBox[
                    SubsuperscriptBox["p", "1", "2"],
                    "TraditionalForm"], " ", 
                  FormBox[
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}],
                    "TraditionalForm"]}], "+", 
                RowBox[{
                  SubsuperscriptBox["p", 
                    FormBox["1",
                      "TraditionalForm"], "\[Mu]"], " ", 
                  FormBox[
                    SubsuperscriptBox["p", "2", "2"],
                    "TraditionalForm"], " ", 
                  FormBox[
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}],
                    "TraditionalForm"]}], "-", 
                RowBox[{
                  SubsuperscriptBox["p", 
                    FormBox["1",
                      "TraditionalForm"], "\[Mu]"], " ", 
                  FormBox[
                    SubsuperscriptBox["p", "1", "2"],
                    "TraditionalForm"], " ", 
                  FormBox[
                    SubsuperscriptBox["p", "2", "2"],
                    "TraditionalForm"]}], "-", 
                RowBox[{
                  SubsuperscriptBox["p", 
                    FormBox["2",
                      "TraditionalForm"], "\[Mu]"], " ", 
                  FormBox[
                    SubsuperscriptBox["p", "1", "2"],
                    "TraditionalForm"], " ", 
                  FormBox[
                    SubsuperscriptBox["p", "2", "2"],
                    "TraditionalForm"]}]}], ")"}]}], 
          RowBox[{"2", " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FormBox[
                    SubsuperscriptBox["p", "1", "2"],
                    "TraditionalForm"], " ", 
                  FormBox[
                    SubsuperscriptBox["p", "2", "2"],
                    "TraditionalForm"]}], "-", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"]}], ")"}]}]], "-", 
        FractionBox[
          RowBox[{
            FormBox[
              FractionBox["1", 
                RowBox[{
                  RowBox[{"(", 
                    InterpretationBox[
                      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                        SuperscriptBox["k", "2"], "\[InvisibleSpace]", 
                        "\<\"]\"\>"}],
                      SequenceForm[ "[", 
                        Power[ k, 2], "]"],
                      Editable->False], ")"}], " ", 
                  RowBox[{"(", 
                    InterpretationBox[
                      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{"k", "-", 
                              SubscriptBox["p", "2"]}], ")"}], "2"], 
                        "\[InvisibleSpace]", "\<\"]\"\>"}],
                      SequenceForm[ "[", 
                        Power[ 
                          Plus[ k, 
                            Times[ -1, 
                              Subscript[ p, 2]]], 2], "]"],
                      Editable->False], ")"}]}]],
              "TraditionalForm"], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  SubsuperscriptBox["p", 
                    FormBox["2",
                      "TraditionalForm"], "\[Mu]"], " ", 
                  FormBox[
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}],
                    "TraditionalForm"]}], "-", 
                RowBox[{
                  SubsuperscriptBox["p", 
                    FormBox["1",
                      "TraditionalForm"], "\[Mu]"], " ", 
                  FormBox[
                    SubsuperscriptBox["p", "2", "2"],
                    "TraditionalForm"]}]}], ")"}]}], 
          RowBox[{"2", " ", 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"], "-", 
                RowBox[{
                  FormBox[
                    SubsuperscriptBox["p", "1", "2"],
                    "TraditionalForm"], " ", 
                  FormBox[
                    SubsuperscriptBox["p", "2", "2"],
                    "TraditionalForm"]}]}], ")"}]}]], "+", 
        FractionBox[
          RowBox[{
            FormBox[
              FractionBox["1", 
                RowBox[{
                  RowBox[{"(", 
                    InterpretationBox[
                      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{"k", "-", 
                              SubscriptBox["p", "1"]}], ")"}], "2"], 
                        "\[InvisibleSpace]", "\<\"]\"\>"}],
                      SequenceForm[ "[", 
                        Power[ 
                          Plus[ k, 
                            Times[ -1, 
                              Subscript[ p, 1]]], 2], "]"],
                      Editable->False], ")"}], " ", 
                  RowBox[{"(", 
                    InterpretationBox[
                      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{"k", "-", 
                              SubscriptBox["p", "2"]}], ")"}], "2"], 
                        "\[InvisibleSpace]", "\<\"]\"\>"}],
                      SequenceForm[ "[", 
                        Power[ 
                          Plus[ k, 
                            Times[ -1, 
                              Subscript[ p, 2]]], 2], "]"],
                      Editable->False], ")"}]}]],
              "TraditionalForm"], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  SubsuperscriptBox["p", 
                    FormBox["2",
                      "TraditionalForm"], "\[Mu]"], " ", 
                  FormBox[
                    SubsuperscriptBox["p", "1", "2"],
                    "TraditionalForm"]}], "-", 
                RowBox[{
                  SubsuperscriptBox["p", 
                    FormBox["1",
                      "TraditionalForm"], "\[Mu]"], " ", 
                  FormBox[
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}],
                    "TraditionalForm"]}], "-", 
                RowBox[{
                  SubsuperscriptBox["p", 
                    FormBox["2",
                      "TraditionalForm"], "\[Mu]"], " ", 
                  FormBox[
                    RowBox[{
                      FormBox[
                        SubscriptBox["p", "1"],
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox[
                        SubscriptBox["p", "2"],
                        "TraditionalForm"]}],
                    "TraditionalForm"]}], "+", 
                RowBox[{
                  SubsuperscriptBox["p", 
                    FormBox["1",
                      "TraditionalForm"], "\[Mu]"], " ", 
                  FormBox[
                    SubsuperscriptBox["p", "2", "2"],
                    "TraditionalForm"]}]}], ")"}]}], 
          RowBox[{"2", " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FormBox[
                    SubsuperscriptBox["p", "1", "2"],
                    "TraditionalForm"], " ", 
                  FormBox[
                    SubsuperscriptBox["p", "2", "2"],
                    "TraditionalForm"]}], "-", 
                SuperscriptBox[
                  RowBox[{"(", 
                    FormBox[
                      RowBox[{
                        FormBox[
                          SubscriptBox["p", "1"],
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox[
                          SubscriptBox["p", "2"],
                          "TraditionalForm"]}],
                      "TraditionalForm"], ")"}], "2"]}], ")"}]}]]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"FAD", "[", 
          RowBox[{"k", ",", " ", 
            RowBox[{"k", " ", "-", " ", "p"}]}], "]"}], 
        RowBox[{"FVD", "[", 
          RowBox[{"k", ",", "\[Mu]"}], "]"}]}], "//", "FCI"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        SuperscriptBox[
          FormBox["k",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], 
        RowBox[{
          SuperscriptBox["k", "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"k", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TID", "[", 
      RowBox[{"%", ",", "k"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          RowBox[{"2", " ", 
            RowBox[{
              SuperscriptBox["k", "2"], ".", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"k", "-", "p"}], ")"}], "2"]}]}]], "-", 
        FractionBox[
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          RowBox[{"2", " ", 
            SuperscriptBox["k", "2"], " ", 
            SuperscriptBox[
              FormBox["p",
                "TraditionalForm"], "2"]}]], "+", 
        FractionBox[
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], 
          RowBox[{"2", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"k", "-", "p"}], ")"}], "2"], " ", 
            SuperscriptBox[
              FormBox["p",
                "TraditionalForm"], "2"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TID", "[", 
      RowBox[{"%%", ",", "k", ",", 
        RowBox[{"FeynAmpDenominatorSimplify", "\[Rule]", "True"}]}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], 
        RowBox[{"2", " ", 
          RowBox[{
            SuperscriptBox["k", "2"], ".", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"k", "-", "p"}], ")"}], "2"]}]}]], TraditionalForm]], 
  "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["ToDistribution", "Subsection",
  CellTags->"ToDistribution"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
ToDistribution[exp,x] replaces (1-x)^(a Epsilon - 1) in exp by 1/(a \
Epsilon) DeltaFunction[1-x] + 1/(1-x) + a Epsilon Log[1-x]/(1-x) + 1/2 a^2 \
Epsilon^2 Log[1-x]^2/(1-x)] and (1-x)^(a Epsilon - 2) in exp by -1/(a \
Epsilon) DeltaFunctionPrime[1-x] + 1/(1-x)^2 + (a Epsilon) Log[1-x]/(1-x)^2 + \
a^2 Epsilon^2/2 Log[1-x]^2/(1-x)^2 + a^3 Epsilon^3/6 Log[1-x]^3/(1-x)^2.\
\>",
   "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "ToDistribution", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"PlusDistribution", "\[Rule]", "Identity"}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["PlusDistribution",
    ButtonData:>"PlusDistribution",
    ButtonStyle->"Hyperlink",
    ButtonNote->"PlusDistribution"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SetOptions", "[", 
      RowBox[{"ToDistribution", ",", 
        RowBox[{"PlusDistribution", "\[Rule]", "PlusDistribution"}]}], 
      "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"PlusDistribution", "\[Rule]", "PlusDistribution"}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ToDistribution", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{"1", "-", "x"}], ")"}], "^", 
          RowBox[{"(", 
            RowBox[{"Epsilon", "-", "1"}], ")"}]}], ",", "x"}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", "6"], " ", 
          SubscriptBox[
            RowBox[{"(", 
              FractionBox[
                RowBox[{
                  SuperscriptBox["log", "3"], "(", 
                  RowBox[{"1", "-", "x"}], ")"}], 
                RowBox[{"1", "-", "x"}]], ")"}], "+"], " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "3"]}], "+", 
        RowBox[{
          FractionBox["1", "2"], " ", 
          SubscriptBox[
            RowBox[{"(", 
              FractionBox[
                RowBox[{
                  SuperscriptBox["log", "2"], "(", 
                  RowBox[{"1", "-", "x"}], ")"}], 
                RowBox[{"1", "-", "x"}]], ")"}], "+"], " ", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "2"]}], "+", 
        RowBox[{
          SubscriptBox[
            RowBox[{"(", 
              FractionBox[
                RowBox[{"log", "(", 
                  RowBox[{"1", "-", "x"}], ")"}], 
                RowBox[{"1", "-", "x"}]], ")"}], "+"], " ", 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm]}], "+", 
        SubscriptBox[
          RowBox[{"(", 
            FractionBox["1", 
              RowBox[{"1", "-", "x"}]], ")"}], "+"], "+", 
        FractionBox[
          RowBox[{"\[Delta]", "(", 
            FormBox[
              RowBox[{"1", "-", "x"}],
              "TraditionalForm"], ")"}], 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SetOptions", "[", 
      RowBox[{"ToDistribution", ",", 
        RowBox[{"PlusDistribution", "\[Rule]", "Identity"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"PlusDistribution", "\[Rule]", "Identity"}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ToDistribution", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{"1", "-", "x"}], ")"}], "^", 
          RowBox[{"(", 
            RowBox[{"Epsilon", "-", "2"}], ")"}]}], ",", "x"}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[
          RowBox[{
            SuperscriptBox[
              TagBox["\[CurlyEpsilon]",
                TraditionalForm], "3"], " ", 
            RowBox[{
              SuperscriptBox["log", "3"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], 
          RowBox[{"6", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], "2"]}]], "+", 
        FractionBox[
          RowBox[{
            SuperscriptBox[
              TagBox["\[CurlyEpsilon]",
                TraditionalForm], "2"], " ", 
            RowBox[{
              SuperscriptBox["log", "2"], "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], 
          RowBox[{"2", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], "2"]}]], "+", 
        FractionBox[
          RowBox[{
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], " ", 
            RowBox[{"log", "(", 
              RowBox[{"1", "-", "x"}], ")"}]}], 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "x"}], ")"}], "2"]], "-", 
        FractionBox[
          RowBox[{
            SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
            FormBox[
              RowBox[{"1", "-", "x"}],
              "TraditionalForm"], ")"}], 
          TagBox["\[CurlyEpsilon]",
            TraditionalForm]], "+", 
        FractionBox["1", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "x"}], ")"}], "2"]]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Series2", "[", 
      RowBox[{
        RowBox[{"Integrate", "[", 
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], 
              RowBox[{"Epsilon", "-", "2"}]], ",", 
            RowBox[{"{", 
              RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
            RowBox[{"GenerateConditions", "\[Rule]", "False"}]}], "]"}], ",", 
        "Epsilon", ",", "3"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "3"]}], "-", 
        SuperscriptBox[
          TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"], "-", 
        TagBox["\[CurlyEpsilon]",
          TraditionalForm], "-", "1"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "@", "Integrate"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Assumptions", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"GenerateConditions", "\[Rule]", "Automatic"}], ",", 
          RowBox[{"PrincipalValue", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate2", "[", 
      RowBox[{
        RowBox[{"ToDistribution", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{"1", "-", "x"}], ")"}], "^", 
              RowBox[{"(", 
                RowBox[{"Epsilon", "-", "2"}], ")"}]}], ",", "x"}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
              TraditionalForm], "3"]}], "-", 
        SuperscriptBox[
          TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"], "-", 
        TagBox["\[CurlyEpsilon]",
          TraditionalForm], "-", "1"}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["ToHypergeometric", "Subsection",
  CellTags->"ToHypergeometric"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "ToHypergeometric[",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox["t", "b"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "t"}], ")"}], "c"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "+", 
                RowBox[{"t", " ", "z"}]}], ")"}], "a"]}], TraditionalForm]]],
  ", t] returns ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"b", "+", "1"}], ")"}], " ", 
          RowBox[{
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"c", "+", "1"}], ")"}], "/", 
            RowBox[{"\[CapitalGamma]", "(", 
              RowBox[{"b", "+", "c", "+", "2"}], ")"}]}], " ", 
          FormBox[
            RowBox[{
              SubscriptBox["\[Null]", "2"], 
              RowBox[{
                SubscriptBox["F", "1"], "(", 
                RowBox[{
                  RowBox[{
                    RowBox[{"-", "a"}], ",", 
                    RowBox[{"b", "+", "1"}]}], ";", 
                  RowBox[{"b", "+", "c", "+", "2"}], ";", 
                  RowBox[{"-", "z"}]}], ")"}]}],
            "TraditionalForm"]}], TraditionalForm]]],
  ". Remember that Re b >0 and Re (c-b) > 0 should hold (need not be set in ",
  
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ")."
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["HypergeometricAC",
    ButtonData:>"HypergeometricAC",
    ButtonStyle->"Hyperlink",
    ButtonNote->"HypergeometricAC"],
  ", ",
  ButtonBox["HypergeometricIR",
    ButtonData:>"HypergeometricIR",
    ButtonStyle->"Hyperlink",
    ButtonNote->"HypergeometricIR"],
  ", ",
  ButtonBox["HypergeometricSE",
    ButtonData:>"HypergeometricSE",
    ButtonStyle->"Hyperlink",
    ButtonNote->"HypergeometricSE"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ToHypergeometric", "[", 
      RowBox[{
        RowBox[{
          SuperscriptBox["t", "b"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "t"}], ")"}], "c"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "+", 
                RowBox[{"t", " ", "z"}]}], ")"}], "a"]}], ",", "t"}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"b", "+", "1"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"c", "+", "1"}], ")"}], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox[
                    RowBox[{"-", "a"}],
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"b", "+", "1"}],
                    (Editable -> True)]}], ";", 
                TagBox[
                  RowBox[{"b", "+", "c", "+", "2"}],
                  (Editable -> True)], ";", 
                TagBox[
                  RowBox[{"-", "z"}],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}], 
        RowBox[{"\[CapitalGamma]", "(", 
          RowBox[{"b", "+", "c", "+", "2"}], ")"}]], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ToHypergeometric", "[", 
      RowBox[{
        RowBox[{"w", " ", 
          SuperscriptBox["t", 
            RowBox[{"b", "-", "1"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "t"}], ")"}], 
            RowBox[{"c", "-", "b", "-", "1"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", 
                RowBox[{"t", " ", "z"}]}], ")"}], 
            RowBox[{"-", "a"}]]}], ",", "t"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{"w", " ", 
          RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"c", "-", "b"}], ")"}], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox["a",
                    (Editable -> True)], ",", 
                  TagBox["b",
                    (Editable -> True)]}], ";", 
                TagBox["c",
                  (Editable -> True)], ";", 
                TagBox["z",
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}], 
        RowBox[{"\[CapitalGamma]", "(", "c", ")"}]], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ToHypergeometric", "[", 
      RowBox[{
        RowBox[{
          SuperscriptBox["t", "b"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "t"}], ")"}], "c"], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"u", "+", 
                RowBox[{"t", " ", "z"}]}], ")"}], "a"]}], ",", "t"}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          SuperscriptBox["u", "a"], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"b", "+", "1"}], ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"c", "+", "1"}], ")"}], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox[
                    RowBox[{"-", "a"}],
                    (Editable -> True)], ",", 
                  TagBox[
                    RowBox[{"b", "+", "1"}],
                    (Editable -> True)]}], ";", 
                TagBox[
                  RowBox[{"b", "+", "c", "+", "2"}],
                  (Editable -> True)], ";", 
                TagBox[
                  RowBox[{"-", 
                    FractionBox["z", "u"]}],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}], 
        RowBox[{"\[CapitalGamma]", "(", 
          RowBox[{"b", "+", "c", "+", "2"}], ")"}]], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ToHypergeometric", "[", 
      RowBox[{
        RowBox[{"w", " ", 
          SuperscriptBox["t", 
            RowBox[{"b", "-", "1"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"1", "-", "t"}], ")"}], 
            RowBox[{"c", "-", "b", "-", "1"}]], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"u", "-", 
                RowBox[{"t", " ", "z"}]}], ")"}], 
            RowBox[{"-", "a"}]]}], ",", "t"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          SuperscriptBox["u", 
            RowBox[{"-", "a"}]], " ", "w", " ", 
          RowBox[{"\[CapitalGamma]", "(", "b", ")"}], " ", 
          RowBox[{"\[CapitalGamma]", "(", 
            RowBox[{"c", "-", "b"}], ")"}], " ", 
          TagBox[
            RowBox[{
              RowBox[{
                SubscriptBox["\[ThinSpace]", "2"], 
                SubscriptBox["F", "1"]}], "(", 
              RowBox[{
                RowBox[{
                  TagBox["a",
                    (Editable -> True)], ",", 
                  TagBox["b",
                    (Editable -> True)]}], ";", 
                TagBox["c",
                  (Editable -> True)], ";", 
                TagBox[
                  FractionBox["z", "u"],
                  (Editable -> True)]}], ")"}],
            InterpretTemplate[ Hypergeometric2F1[ #, #2, #3, #4]&]]}], 
        RowBox[{"\[CapitalGamma]", "(", "c", ")"}]], TraditionalForm]], 
  "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["ToLarin", "Subsection",
  CellTags->"ToLarin"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "ToLarin[exp] translates all ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox["\[Gamma]", "\[Mu]"], " ", 
          SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]]],
  " in exp into ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            RowBox[{"-", "\[ImaginaryI]"}], "/", "6"}], " ", 
          SuperscriptBox["\[CurlyEpsilon]", "\[Mu]\[Nu]\[Lambda]\[Sigma]"], 
          SuperscriptBox["\[Gamma]", "\[Nu]"], " ", 
          SuperscriptBox["\[Gamma]", "\[Lambda]"], " ", 
          RowBox[{
            SuperscriptBox["\[Gamma]", "\[Sigma]"], "."}]}], 
        TraditionalForm]]]
}], "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Eps",
    ButtonData:>"Eps",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Eps"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracMatrix", "[", "\[Nu]", "]"}], ".", 
      RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
      RowBox[{"DiracMatrix", "[", "5", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], ".", 
        SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ToLarin", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "6"]}], " ", "\[ImaginaryI]", " ", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["du1",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["du2",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["du3",
                "TraditionalForm"],
              "TraditionalForm"]]}], " ", 
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["du1",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["du2",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["du3",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["ToTFi", "Subsection",
  CellTags->"ToTFi"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
ToTFi[expr, q1, q2, p] translates FeynCalc 2-loop self energy type \
integrals into the TFi notatation, which can be used as input for the \
function TarcerRecurse from the TARCER package. See TFi for details on the \
conventions.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  ButtonBox["TFi",
    ButtonData:>"TFi",
    ButtonStyle->"Hyperlink",
    ButtonNote->"TFi"],
  ", ",
  ButtonBox["FromTFi",
    ButtonData:>"FromTFi",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FromTFi"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FAD", "[", 
      RowBox[{"q1", ",", 
        RowBox[{"q1", "-", "p"}], ",", 
        RowBox[{"{", 
          RowBox[{"q2", ",", "M"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"q2", "-", "p"}], ",", "m"}], "}"}], ",", 
        RowBox[{"q1", "-", "q2"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox["1", 
        RowBox[{
          RowBox[{"(", 
            InterpretationBox[
              RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                SuperscriptBox[
                  InterpretationBox[
                    SubscriptBox["q", "1"],
                    q1], "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
              SequenceForm[ "[", 
                Power[ q1, 2], "]"],
              Editable->False], ")"}], " ", 
          RowBox[{"(", 
            InterpretationBox[
              RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        SubscriptBox["q", "1"],
                        q1], "-", "p"}], ")"}], "2"], "\[InvisibleSpace]", 
                "\<\"]\"\>"}],
              SequenceForm[ "[", 
                Power[ 
                  Plus[ 
                    Times[ -1, p], q1], 2], "]"],
              Editable->False], ")"}], " ", 
          RowBox[{"(", 
            InterpretationBox[
              RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        SubscriptBox["q", "1"],
                        q1], "-", 
                      InterpretationBox[
                        SubscriptBox["q", "2"],
                        q2]}], ")"}], "2"], "\[InvisibleSpace]", 
                "\<\"]\"\>"}],
              SequenceForm[ "[", 
                Power[ 
                  Plus[ q1, 
                    Times[ -1, q2]], 2], "]"],
              Editable->False], ")"}], " ", 
          RowBox[{"(", 
            InterpretationBox[
              RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                SuperscriptBox[
                  InterpretationBox[
                    SubscriptBox["q", "2"],
                    q2], "2"], "\[InvisibleSpace]", "\<\"-\"\>", 
                "\[InvisibleSpace]", 
                SuperscriptBox["M", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
              SequenceForm[ "[", 
                Power[ q2, 2], "-", 
                Power[ M, 2], "]"],
              Editable->False], ")"}], " ", 
          RowBox[{"(", 
            InterpretationBox[
              RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        SubscriptBox["q", "2"],
                        q2], "-", "p"}], ")"}], "2"], "\[InvisibleSpace]", 
                "\<\"-\"\>", "\[InvisibleSpace]", 
                SuperscriptBox["m", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
              SequenceForm[ "[", 
                Power[ 
                  Plus[ 
                    Times[ -1, p], q2], 2], "-", 
                Power[ m, 2], "]"],
              Editable->False], ")"}]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ToTFi", "[", 
      RowBox[{"%", ",", "q1", ",", "q2", ",", "p"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        SubsuperscriptBox[
          StyleBox["F",
            SingleLetterItalics->False,
            FontWeight->"Bold"], 
          RowBox[{
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "M"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "m"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}],
              "TraditionalForm"]}], 
          RowBox[{"(", 
            FormBox["D",
              "TraditionalForm"], ")"}]],
        TFi[ D, 
          SPD[ p, p], {{1, 0}, {1, M}, {1, 0}, {1, m}, {1, 0}}],
        Editable->True], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"TFi", "[", 
      RowBox[{"D", ",", 
        RowBox[{"SPD", "[", 
          RowBox[{"p", ",", "p"}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"1", ",", "0"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"1", ",", "M"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"1", ",", "0"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"1", ",", "m"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"1", ",", "0"}], "}"}]}], "}"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"SOD", "[", "q1", "]"}], " ", 
        RowBox[{"SOD", "[", "q2", "]"}], 
        RowBox[{"FAD", "[", 
          RowBox[{"q1", ",", 
            RowBox[{"q1", "-", "p"}], ",", 
            RowBox[{"{", 
              RowBox[{"q2", ",", "M"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"q2", "-", "p"}], ",", "m"}], "}"}], ",", 
            RowBox[{"q1", "-", "q2"}]}], "]"}]}], "//", "FCI"}]], "Input"],

Cell[BoxData[
    FormBox[
      FractionBox[
        RowBox[{
          RowBox[{
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox[
              InterpretationBox[
                SubscriptBox["q", "1"],
                q1],
              "TraditionalForm"]}], " ", 
          RowBox[{
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox[
              InterpretationBox[
                SubscriptBox["q", "2"],
                q2],
              "TraditionalForm"]}]}], 
        RowBox[{
          SuperscriptBox[
            InterpretationBox[
              SubscriptBox["q", "1"],
              q1], "2"], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                InterpretationBox[
                  SubscriptBox["q", "1"],
                  q1], "-", "p"}], ")"}], "2"], ".", 
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox[
                InterpretationBox[
                  SubscriptBox["q", "2"],
                  q2], "2"], "-", 
              SuperscriptBox["M", "2"]}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    InterpretationBox[
                      SubscriptBox["q", "2"],
                      q2], "-", "p"}], ")"}], "2"], "-", 
              SuperscriptBox["m", "2"]}], ")"}], ".", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                InterpretationBox[
                  SubscriptBox["q", "1"],
                  q1], "-", 
                InterpretationBox[
                  SubscriptBox["q", "2"],
                  q2]}], ")"}], "2"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"ToTFi", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        SubsuperscriptBox[
          StyleBox["F",
            SingleLetterItalics->False,
            FontWeight->"Bold"], 
          RowBox[{
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "M"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "m"}], "}"}],
              "TraditionalForm"], 
            FormBox[
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}],
              "TraditionalForm"]}], 
          RowBox[{"(", 
            FormBox["D",
              "TraditionalForm"], ")", " ", "1", "1"}]],
        TFi[ D, 
          SPD[ p, p], 
          SOD[ p], {1, 1}, {{1, 0}, {1, M}, {1, 0}, {1, m}, {1, 0}}],
        Editable->True], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"TFi", "[", 
      RowBox[{"D", ",", 
        RowBox[{"SPD", "[", 
          RowBox[{"p", ",", "p"}], "]"}], ",", 
        RowBox[{"SOD", "[", "p", "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"1", ",", "1"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"1", ",", "0"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"1", ",", "M"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"1", ",", "0"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"1", ",", "m"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"1", ",", "0"}], "}"}]}], "}"}]}], "]"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Tr", "Subsection",
  CellTags->"Tr"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Tr[exp] calculates the Dirac trace of exp. Depending on the setting \
of the option SUNTrace also a trace over SU(N) objects is performed. Tr is \
identical toDiracTrace, up to the default setting of DiracTraceEvaluate.\
\>",
   "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Tr", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"DiracTraceEvaluate", "\[Rule]", "True"}], ",", 
          RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
          RowBox[{"FeynCalcExternal", "\[Rule]", "False"}], ",", 
          RowBox[{"LeviCivitaSign", "\[Rule]", 
            RowBox[{"-", "1"}]}], ",", 
          RowBox[{"Mandelstam", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"PairCollect", "\[Rule]", "False"}], ",", 
          RowBox[{"Schouten", "\[Rule]", "442"}], ",", 
          RowBox[{"SUNTrace", "\[Rule]", "True"}], ",", 
          RowBox[{"TraceOfOne", "\[Rule]", "4"}]}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  ButtonBox["DiracSimplify",
    ButtonData:>"DiracSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracSimplify"],
  ", ",
  ButtonBox["DiracTrace",
    ButtonData:>"DiracTrace",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracTrace"],
  ", ",
  ButtonBox["FermionSpinSum",
    ButtonData:>"FermionSpinSum",
    ButtonStyle->"Hyperlink",
    ButtonNote->"FermionSpinSum"],
  ", ",
  ButtonBox["SUNTrace",
    ButtonData:>"SUNTrace",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SUNTrace"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"GA", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tr", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tr", "[", 
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            RowBox[{"GSD", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
        RowBox[{"GAD", "[", "\[Mu]", "]"}], ".", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"GSD", "[", "q", "]"}], "-", "m"}], ")"}], ".", 
        RowBox[{"GAD", "[", "\[Nu]", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], " ", 
              SuperscriptBox["m", "2"]}], "+", 
            RowBox[{
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{
                FormBox["p",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["q",
                  "TraditionalForm"]}]}]}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tr", "[", 
      RowBox[{"GA", "[", 
        RowBox[{
        "\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]", ",", "5"}], 
        "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "4"}], " ", "\[ImaginaryI]", " ", 
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tr", "[", 
      RowBox[{"GAD", "[", 
        RowBox[{
        "\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]", ",", "\[Tau]", 
          ",", "\[Xi]", ",", "5"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Xi]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Xi]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Xi]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Xi]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Xi]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Xi]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Xi]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Xi]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Xi]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Xi]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Xi]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Xi]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Xi]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Xi]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Xi]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Tau]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}]}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["$West"], "Input"],

Cell[BoxData[
    FormBox["True", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"?", "$West"}]], "Input"],

Cell[BoxData[
    FormBox[
    "\<\"If $West is set to True (which is the default),  traces involving \
more than 4 Dirac matrices and gamma5 are calculated recursively according to \
formula (A.5) from Comp. Phys. Comm 77 (1993) 286-298, which is based on the \
Breitenlohner Maison gamma5 - scheme.\"\>", TraditionalForm]], "Print"]
}, Open  ]],

Cell["\<\
$West=False should be set before any calculation is done (because \
intermediate results from trace calculations are cached).\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"$West", "=", "False"}]], "Input"],

Cell[BoxData[
    FormBox["False", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tr", "[", 
      RowBox[{
        RowBox[{"GAD", "[", 
          RowBox[{
            SubscriptBox["\[Nu]", "1"], ",", 
            SubscriptBox["\[Nu]", "2"], ",", 
            SubscriptBox["\[Nu]", "3"], ",", 
            SubscriptBox["\[Nu]", "4"], ",", 
            SubscriptBox["\[Nu]", "5"], ",", 
            SubscriptBox["\[Nu]", "6"]}], "]"}], ".", 
        RowBox[{"GA", "[", "5", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", "\[ImaginaryI]"}], " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "3"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "4"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "5"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "6"],
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "1"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "2"],
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "2"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "4"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "5"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "6"],
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "1"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "3"],
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "1"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "4"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "5"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "6"],
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "2"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "3"],
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "1"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "2"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "3"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "6"],
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "4"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "5"],
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "1"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "2"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "3"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "5"],
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "4"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "6"],
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "1"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "2"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "3"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "4"],
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "5"],
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox[
                      SubscriptBox["\[Nu]", "6"],
                      "TraditionalForm"],
                    "TraditionalForm"]}]]}]}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tr", "[", 
      RowBox[{"GS", "[", 
        RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{
                FormBox["p",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["s",
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox["q",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["r",
                  "TraditionalForm"]}]}], "-", 
            RowBox[{
              RowBox[{
                FormBox["p",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["r",
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox["q",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["s",
                  "TraditionalForm"]}]}], "+", 
            RowBox[{
              RowBox[{
                FormBox["p",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["q",
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox["r",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["s",
                  "TraditionalForm"]}]}]}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tr", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{"GS", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
          RowBox[{"GA", "[", "\[Mu]", "]"}], ".", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"GS", "[", "q", "]"}], "+", "m"}], ")"}], ".", 
          RowBox[{"GA", "[", "\[Mu]", "]"}]}], ",", 
        RowBox[{"Factoring", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"8", " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"2", " ", 
              SuperscriptBox["m", "2"]}], "-", 
            RowBox[{
              FormBox["p",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["q",
                "TraditionalForm"]}]}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tr", "[", 
      RowBox[{
        RowBox[{"GA", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
        RowBox[{"FCE", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", 
        SuperscriptBox["g", 
          RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"4", " ", 
      RowBox[{"MT", "[", 
        RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Tr", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"GAD", "[", "\[Mu]", "]"}], ".", 
          RowBox[{"GA", "[", "\[Alpha]", "]"}], ".", 
          RowBox[{"GA", "[", "\[Beta]", "]"}], ".", 
          RowBox[{"GAD", "[", "\[Mu]", "]"}]}], ",", 
        RowBox[{"FCE", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"4", " ", "D", " ", 
        SuperscriptBox["g", 
          RowBox[{"\[Alpha]", "\[NoBreak]", "\[Beta]"}]]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"4", " ", "D", " ", 
      RowBox[{"MT", "[", 
        RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["TrickMandelstam", "Subsection",
  CellTags->"TrickMandelstam"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[TextData[{
  "TrickMandelstam[expr, {s, t, u, ",
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["m", "1", "2"], TraditionalForm]]],
  " + ",
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["m", "2", "2"], TraditionalForm]]],
  "+ ",
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["m", "3", "2"], TraditionalForm]]],
  "+ ",
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["m", "4", "2"], TraditionalForm]]],
  "}] simplifies all sums in expr so that one of the Mandelstam variables s, \
t or u is eliminated by the relation s + t + u = ",
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["m", "1", "2"], TraditionalForm]]],
  " + ",
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["m", "2", "2"], TraditionalForm]]],
  "+ ",
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["m", "3", "2"], TraditionalForm]]],
  "+ ",
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["m", "4", "2"], TraditionalForm]]],
  " . The trick is that the resulting sum has the most short number of \
terms."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"TrickMandelstam", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{"s", "+", "t", "-", "u"}], ")"}], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"2", 
                  SubsuperscriptBox["M", "W", "2"]}], "-", "t", "-", "u"}], 
              ")"}]}], ",", 
          RowBox[{"{", 
            RowBox[{"s", ",", "t", ",", "u", ",", 
              RowBox[{"2", 
                SubsuperscriptBox["M", "W", "2"]}]}], "}"}]}], "]"}], "//", 
      "Factor2"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", "2"}], " ", "s", " ", 
        RowBox[{"(", 
          RowBox[{"u", "-", 
            SubsuperscriptBox["M", "W", "2"]}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TrickMandelstam", "[", 
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"M", "^", "2"}], " ", "s"}], " ", "-", " ", 
          RowBox[{"s", "^", "2"}], " ", "+", " ", 
          RowBox[{
            RowBox[{"M", "^", "2"}], " ", "t"}], " ", "-", " ", 
          RowBox[{"s", "t"}], " ", "+", " ", 
          RowBox[{
            RowBox[{"M", "^", "2"}], " ", "u"}], " ", "-", " ", 
          RowBox[{"s", "u"}]}], ",", " ", 
        RowBox[{"{", 
          RowBox[{"s", ",", "t", ",", "u", ",", 
            RowBox[{"2", 
              RowBox[{"M", "^", "2"}]}]}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"2", " ", 
        SuperscriptBox["M", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            SuperscriptBox["M", "2"], "-", "s"}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Trick", "Subsection",
  CellTags->"Trick"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Trick[exp] performs several basic simplifications without \
expansion. Trick[exp] uses Contract, DotSimplify and SUNDeltaContract.\
\>", 
  "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Calc",
    ButtonData:>"Calc",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Calc"],
  ", ",
  ButtonBox["Contract",
    ButtonData:>"Contract",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Contract"],
  ", ",
  ButtonBox["DiracTrick",
    ButtonData:>"DiracTrick",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracTrick"],
  ", ",
  ButtonBox["DotSimplify",
    ButtonData:>"DotSimplify",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DotSimplify"],
  ", ",
  ButtonBox["DiracTrick",
    ButtonData:>"DiracTrick",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DiracTrick"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection"],

Cell[TextData[{
  "This calculates ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SuperscriptBox["g", 
            RowBox[{"\[Mu]", " ", "\[Nu]"}]], 
          SubscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]]],
  "and ",
  Cell[BoxData[
      FormBox[
        SubsuperscriptBox["g", "\[Nu]", "\[Nu]"], TraditionalForm]]],
  " in D dimensions."
}], "Text",
  ZeroWidthTimes->True],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Trick", "[", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            RowBox[{"GA", "[", "\[Mu]", "]"}], " ", 
            RowBox[{"MT", "[", 
              RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}], ",", " ", 
          RowBox[{"MTD", "[", 
            RowBox[{"\[Nu]", ",", "\[Nu]"}], "]"}]}], "}"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], ",", "D"}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"FV", "[", 
        RowBox[{
          RowBox[{"p", "+", "r"}], ",", "\[Mu]"}], "]"}], " ", 
      RowBox[{"MT", "[", 
        RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
      RowBox[{"FV", "[", 
        RowBox[{
          RowBox[{"q", "-", "p"}], ",", "\[Nu]"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          SuperscriptBox[
            RowBox[{"(", 
              FormBox[
                RowBox[{"q", "-", "p"}],
                "TraditionalForm"], ")"}], "\[Nu]"], ")"}], " ", 
        RowBox[{"(", 
          SuperscriptBox[
            RowBox[{"(", 
              FormBox[
                RowBox[{"p", "+", "r"}],
                "TraditionalForm"], ")"}], "\[Mu]"], ")"}], " ", 
        SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[NoBreak]", "\[Nu]"}]]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Trick", "[", "%", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SuperscriptBox[
            FormBox["p",
              "TraditionalForm"], "2"]}], "+", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}], "-", 
        RowBox[{
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["r",
            "TraditionalForm"]}], "+", 
        RowBox[{
          FormBox["q",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["r",
            "TraditionalForm"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Trick", "[", 
      RowBox[{"c", ".", "b", ".", "a", " ", ".", " ", 
        RowBox[{"GA", "[", "d", "]"}], ".", 
        RowBox[{"GA", "[", "e", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"a", " ", "b", " ", "c", " ", 
        RowBox[{
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["d",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", 
            FormBox[
              FormBox["e",
                "TraditionalForm"],
              "TraditionalForm"]]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"%", "//", "FCE"}], "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{"a", " ", "b", " ", "c", " ", 
      RowBox[{
        RowBox[{"GA", "[", "d", "]"}], ".", 
        RowBox[{"GA", "[", "e", "]"}]}]}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Twist2GluonOperator", "Subsection",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Twist2GluonOperator[{p, mu, a}, {nu, b}] or Twist2GluonOperator[p, \
{mu, a}, {nu, b}] or Twist2GluonOperator[p, mu,a, nu,b] yields the 2-gluon \
operator (p is ingoing momentum corresponding to Lorentz index mu). \
Twist2GluonOperator[{p,mu,a}, {q,nu,b}, {k,la,c}] or Twist2GluonOperator[ \
p,mu,a , q,nu,b , k,la,c ] gives the 3-gluon operator. \
Twist2GluonOperator[{p,mu,a}, {q,nu,b}, {k,la,c}, {s,si,d}] or \
Twist2GluonOperator[p,mu,a , q,nu,b , k,la,c , s,si,d] yields the 4-Gluon \
operator. The dimension is determined by the option and Dimension. The \
setting of the option Polarization (unpolarized: 0; polarized: 1) determines \
whether the unpolarized or polarized feynman rule is returned. With the \
setting Explicit to False the color-structure and the (1+(-1)^OPEm) (for \
polarized: (1-(-1)^OPEm)) is extracted and the color indices are omitted in \
the arguments of Twist2GluonOperator.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Twist2QuarkOperator",
    ButtonData:>"Twist2QuarkOperator",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Twist2QuarkOperator"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Polarized case, zero-momentum insertion ", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SetOptions", "[", 
      RowBox[{"Twist2GluonOperator", ",", " ", 
        RowBox[{"Polarization", " ", "->", " ", "1"}], ",", 
        RowBox[{"Explicit", "->", "False"}], ",", " ", 
        RowBox[{"ZeroMomentumInsertion", " ", "->", " ", "True"}]}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
          RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
          RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell["2-gluon legs", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{"p", ",", 
        RowBox[{"{", 
          RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"\[Nu]", ",", "b"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[ImaginaryI]", " ", 
        RowBox[{"(", 
          RowBox[{"1", "-", 
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "m"],
              "TraditionalForm"]}], ")"}], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]], " ", 
        RowBox[{"(", 
          RowBox[{
            SubsuperscriptBox["O", 
              RowBox[{
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[VeryThinSpace]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]}], "G2"], "(", 
            FormBox["p",
              "TraditionalForm"], ")"}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{"p", ",", 
        RowBox[{"{", 
          RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"Explicit", "->", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[ImaginaryI]", " ", 
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], "\[NoBreak]", 
            FormBox["p",
              "TraditionalForm"]}]], " ", 
        RowBox[{"(", 
          RowBox[{"1", "-", 
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "m"],
              "TraditionalForm"]}], ")"}], " ", 
        FormBox[
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], ")"}], 
            RowBox[{"m", "-", "1"}]],
          "TraditionalForm"], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{"p", ",", 
        RowBox[{"{", "\[Mu]", "}"}], ",", 
        RowBox[{"{", "\[Nu]", "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubsuperscriptBox["O", 
          RowBox[{
            FormBox["\[Mu]",
              "TraditionalForm"], "\[VeryThinSpace]", 
            FormBox["\[Nu]",
              "TraditionalForm"]}], "G2"], "(", 
        FormBox["p",
          "TraditionalForm"], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{"p", ",", 
        RowBox[{"{", "\[Mu]", "}"}], ",", 
        RowBox[{"{", "\[Nu]", "}"}], ",", 
        RowBox[{"Explicit", "->", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], "\[NoBreak]", 
            FormBox["p",
              "TraditionalForm"]}]], " ", 
        FormBox[
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], ")"}], 
            RowBox[{"m", "-", "1"}]],
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["3-gluon legs", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        RowBox[{"(", 
          RowBox[{"1", "-", 
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "m"],
              "TraditionalForm"]}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]], " ", 
        RowBox[{"(", 
          RowBox[{
            SubsuperscriptBox["O", 
              RowBox[{
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[VeryThinSpace]", 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[VeryThinSpace]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]}], "G3"], "(", 
            FormBox["q",
              "TraditionalForm"], ",", 
            FormBox["r",
              "TraditionalForm"], ",", 
            FormBox["p",
              "TraditionalForm"], ")"}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubsuperscriptBox["O", 
          RowBox[{
            FormBox["\[Nu]",
              "TraditionalForm"], "\[VeryThinSpace]", 
            FormBox["\[Rho]",
              "TraditionalForm"], "\[VeryThinSpace]", 
            FormBox["\[Mu]",
              "TraditionalForm"]}], "G3"], "(", 
        FormBox["q",
          "TraditionalForm"], ",", 
        FormBox["r",
          "TraditionalForm"], ",", 
        FormBox["p",
          "TraditionalForm"], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
        RowBox[{"Explicit", "->", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], "-", 
                RowBox[{
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}]}], ")"}]}], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], ")"}], 
            RowBox[{"m", "-", "2"}]]}], "-", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}]}], ")"}], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["q",
                  "TraditionalForm"]}], ")"}], 
            RowBox[{"m", "-", "2"}]]}], "-", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["r",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["r",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}]}], ")"}], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["r",
                  "TraditionalForm"]}], ")"}], 
            RowBox[{"m", "-", "2"}]]}], "+", 
        RowBox[{
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "3"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", "1"}], ")"}], "i"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}], ")"}], "i"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["p",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}]}], ")"}]}], "-", 
        RowBox[{
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "3"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", "1"}], ")"}], "i"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}], ")"}], "i"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["r",
                            "TraditionalForm"]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["p",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["r",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["r",
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}]}], ")"}]}], "-", 
        RowBox[{
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "3"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", "1"}], ")"}], "i"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}], ")"}], "i"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["r",
                            "TraditionalForm"]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["r",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["r",
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]}]}], ")"}]}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell["4-gluon legs", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[ImaginaryI]", " ", 
        RowBox[{"(", 
          RowBox[{"1", "-", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", 
                SubscriptBox["f", 
                  RowBox[{
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["c112",
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c112",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SubsuperscriptBox["O", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}], "G4"], "(", 
                  FormBox["p",
                    "TraditionalForm"], ",", 
                  FormBox["q",
                    "TraditionalForm"], ",", 
                  FormBox["r",
                    "TraditionalForm"], ",", 
                  FormBox["s",
                    "TraditionalForm"], ")"}], ")"}]}], "-", 
            RowBox[{
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c112",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c112",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SubsuperscriptBox["O", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}], "G4"], "(", 
                  FormBox["p",
                    "TraditionalForm"], ",", 
                  FormBox["r",
                    "TraditionalForm"], ",", 
                  FormBox["q",
                    "TraditionalForm"], ",", 
                  FormBox["s",
                    "TraditionalForm"], ")"}], ")"}]}], "+", 
            RowBox[{
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c112",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c112",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SubsuperscriptBox["O", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}], "G4"], "(", 
                  FormBox["r",
                    "TraditionalForm"], ",", 
                  FormBox["q",
                    "TraditionalForm"], ",", 
                  FormBox["p",
                    "TraditionalForm"], ",", 
                  FormBox["s",
                    "TraditionalForm"], ")"}], ")"}]}]}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"s", ",", "\[Sigma]"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubsuperscriptBox["O", 
          RowBox[{
            FormBox["\[Mu]",
              "TraditionalForm"], "\[VeryThinSpace]", 
            FormBox["\[Nu]",
              "TraditionalForm"], "\[VeryThinSpace]", 
            FormBox["\[Rho]",
              "TraditionalForm"], "\[VeryThinSpace]", 
            FormBox["\[Sigma]",
              "TraditionalForm"]}], "G4"], "(", 
        FormBox["p",
          "TraditionalForm"], ",", 
        FormBox["q",
          "TraditionalForm"], ",", 
        FormBox["r",
          "TraditionalForm"], ",", 
        FormBox["s",
          "TraditionalForm"], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"s", ",", "\[Sigma]"}], "}"}], ",", 
        RowBox[{"Explicit", "->", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}]}], ")"}], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}], "+", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["s",
                    "TraditionalForm"]}]}], ")"}], 
            RowBox[{"m", "-", "2"}]]}], "-", 
        RowBox[{
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "3"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{"-", 
                            RowBox[{"(", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}], ")"}]}], "-", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["q",
                              "TraditionalForm"]}]}], ")"}], "i"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["s",
                            "TraditionalForm"]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["s",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["s",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}]}], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]]}], "+", 
        RowBox[{
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "3"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["p",
                              "TraditionalForm"]}], "+", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["q",
                              "TraditionalForm"]}]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}], ")"}]}], ")"}], 
                      "i"]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["r",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["r",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}]}], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"]]}], "-", 
        RowBox[{
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "3"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], ")"}]}], ")"}], "i"], 
                    " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["r",
                              "TraditionalForm"]}], "+", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["s",
                              "TraditionalForm"]}]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["p",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["p",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}]}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "3"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], ")"}]}], ")"}], "i"], 
                    " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["r",
                              "TraditionalForm"]}], "+", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["s",
                              "TraditionalForm"]}]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}]}], ")"}]}], "-", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox[
                TagBox[
                  RowBox[{
                    UnderoverscriptBox["\[Sum]", 
                      RowBox[{"j", "=", "0"}], 
                      RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
                  HoldForm],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}], ")"}]}], "-", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["s",
                            "TraditionalForm"]}]}], ")"}], 
                    RowBox[{"j", "-", "i"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", 
                        RowBox[{"(", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["s",
                              "TraditionalForm"]}], ")"}]}], ")"}], "i"]}],
                "TraditionalForm"]}], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["s",
                        "TraditionalForm"]}]]}], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["s",
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}]}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox[
                TagBox[
                  RowBox[{
                    UnderoverscriptBox["\[Sum]", 
                      RowBox[{"j", "=", "0"}], 
                      RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
                  HoldForm],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}], ")"}]}], "-", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["s",
                            "TraditionalForm"]}]}], ")"}], 
                    RowBox[{"j", "-", "i"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", 
                        RowBox[{"(", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["s",
                              "TraditionalForm"]}], ")"}]}], ")"}], "i"]}],
                "TraditionalForm"]}], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["s",
                        "TraditionalForm"]}]]}], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["s",
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]}]}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox[
                TagBox[
                  RowBox[{
                    UnderoverscriptBox["\[Sum]", 
                      RowBox[{"j", "=", "0"}], 
                      RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
                  HoldForm],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", 
                        RowBox[{"(", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["r",
                              "TraditionalForm"]}], ")"}]}], ")"}], "i"], 
                  " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}], ")"}]}], "-", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["s",
                            "TraditionalForm"]}]}], ")"}], 
                    RowBox[{"j", "-", "i"}]]}],
                "TraditionalForm"]}], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["r",
                        "TraditionalForm"]}]]}], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["p",
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]}]}], ")"}]}], "-", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox[
                TagBox[
                  RowBox[{
                    UnderoverscriptBox["\[Sum]", 
                      RowBox[{"j", "=", "0"}], 
                      RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
                  HoldForm],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", 
                        RowBox[{"(", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["r",
                              "TraditionalForm"]}], ")"}]}], ")"}], "i"], 
                  " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}], ")"}]}], "-", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["s",
                            "TraditionalForm"]}]}], ")"}], 
                    RowBox[{"j", "-", "i"}]]}],
                "TraditionalForm"]}], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["q",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["r",
                        "TraditionalForm"]}]]}], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]}]}], ")"}]}]}], TraditionalForm]], 
  "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Unpolarized case, zero-momentum insertion", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SetOptions", "[", 
      RowBox[{"Twist2GluonOperator", ",", 
        RowBox[{"Polarization", "\[Rule]", "0"}], ",", 
        RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Polarization", "\[Rule]", "0"}], ",", 
          RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
          RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell["2-gluon legs", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{"p", ",", 
        RowBox[{"{", 
          RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"\[Nu]", ",", "b"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "m"],
              "TraditionalForm"], "+", "1"}], ")"}], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]], " ", 
        RowBox[{"(", 
          RowBox[{
            SubsuperscriptBox["O", 
              RowBox[{
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[VeryThinSpace]", 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]}], "G2"], "(", 
            FormBox["p",
              "TraditionalForm"], ")"}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{"p", ",", 
        RowBox[{"{", 
          RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"Explicit", "->", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  FormBox[
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}],
                    "TraditionalForm"], ")"}], "2"]}], "-", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox[
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "+", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}]}], ")"}], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}]}], "+", 
            RowBox[{
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"]}]}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "m"],
              "TraditionalForm"], "+", "1"}], ")"}], " ", 
        FormBox[
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], ")"}], 
            RowBox[{"m", "-", "2"}]],
          "TraditionalForm"], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{"p", ",", 
        RowBox[{"{", "\[Mu]", "}"}], ",", 
        RowBox[{"{", "\[Nu]", "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubsuperscriptBox["O", 
          RowBox[{
            FormBox["\[Mu]",
              "TraditionalForm"], "\[VeryThinSpace]", 
            FormBox["\[Nu]",
              "TraditionalForm"]}], "G2"], "(", 
        FormBox["p",
          "TraditionalForm"], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{"p", ",", 
        RowBox[{"{", "\[Mu]", "}"}], ",", 
        RowBox[{"{", "\[Nu]", "}"}], ",", 
        RowBox[{"Explicit", "->", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  FormBox[
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}],
                    "TraditionalForm"], ")"}], "2"]}], "-", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox[
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "+", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}]}], ")"}], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}]}], "+", 
            RowBox[{
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"]}]}], ")"}], " ", 
        FormBox[
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], ")"}], 
            RowBox[{"m", "-", "2"}]],
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["3-gluon legs", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "2"]}], " ", "\[ImaginaryI]", " ", 
        SubscriptBox["g", "s"], " ", 
        RowBox[{"(", 
          RowBox[{
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "m"],
              "TraditionalForm"], "+", "1"}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]], " ", 
        RowBox[{"(", 
          RowBox[{
            SubsuperscriptBox["O", 
              RowBox[{
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[VeryThinSpace]", 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"], "\[VeryThinSpace]", 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]}], "G3"], "(", 
            FormBox["q",
              "TraditionalForm"], ",", 
            FormBox["r",
              "TraditionalForm"], ",", 
            FormBox["p",
              "TraditionalForm"], ")"}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubsuperscriptBox["O", 
          RowBox[{
            FormBox["\[Nu]",
              "TraditionalForm"], "\[VeryThinSpace]", 
            FormBox["\[Rho]",
              "TraditionalForm"], "\[VeryThinSpace]", 
            FormBox["\[Mu]",
              "TraditionalForm"]}], "G3"], "(", 
        FormBox["q",
          "TraditionalForm"], ",", 
        FormBox["r",
          "TraditionalForm"], ",", 
        FormBox["p",
          "TraditionalForm"], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
        RowBox[{"Explicit", "->", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "3"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], ")"}]}], ")"}], "i"], 
                    " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["q",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]}]}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "3"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}], ")"}]}], ")"}], 
                      "i"]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["r",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]}]}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "3"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], ")"}]}], ")"}], "i"], 
                    " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["r",
                            "TraditionalForm"]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["r",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["q",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["q",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]}]}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox[
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "-", 
                    RowBox[{
                      SuperscriptBox[
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]]}]}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Mu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox["\[Rho]",
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "-", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Mu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox["\[Nu]",
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]]}]}], ")"}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          FormBox[
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}], ")"}], 
              RowBox[{"m", "-", "2"}]],
            "TraditionalForm"]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      SuperscriptBox[
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox[
                        FormBox["q",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "-", 
                    RowBox[{
                      SuperscriptBox[
                        FormBox["q",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox[
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]]}]}], ")"}]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Mu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox["\[Nu]",
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Rho]",
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "-", 
                    RowBox[{
                      SuperscriptBox[
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Nu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox["\[Rho]",
                              "TraditionalForm"],
                            "TraditionalForm"]}]]}]}], ")"}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          FormBox[
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], ")"}], 
              RowBox[{"m", "-", "2"}]],
            "TraditionalForm"]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      SuperscriptBox[
                        FormBox["r",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox[
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]]}], "-", 
                    RowBox[{
                      SuperscriptBox[
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox[
                        FormBox["r",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]]}]}], ")"}], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      SuperscriptBox[
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"]], " ", 
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Nu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox["\[Rho]",
                              "TraditionalForm"],
                            "TraditionalForm"]}]]}], "-", 
                    RowBox[{
                      SuperscriptBox["g", 
                        RowBox[{
                          FormBox[
                            FormBox["\[Mu]",
                              "TraditionalForm"],
                            "TraditionalForm"], "\[NoBreak]", 
                          FormBox[
                            FormBox["\[Rho]",
                              "TraditionalForm"],
                            "TraditionalForm"]}]], " ", 
                      SuperscriptBox[
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"]]}]}], ")"}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]}]}], ")"}], " ", 
          FormBox[
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}], ")"}], 
              RowBox[{"m", "-", "2"}]],
            "TraditionalForm"]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["4-gluon legs", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
          RowBox[{"1", "+", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", 
                SubscriptBox["f", 
                  RowBox[{
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["c113",
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c113",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SubsuperscriptBox["O", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}], "G4"], "(", 
                  FormBox["p",
                    "TraditionalForm"], ",", 
                  FormBox["q",
                    "TraditionalForm"], ",", 
                  FormBox["r",
                    "TraditionalForm"], ",", 
                  FormBox["s",
                    "TraditionalForm"], ")"}], ")"}]}], "-", 
            RowBox[{
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c113",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c113",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SubsuperscriptBox["O", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}], "G4"], "(", 
                  FormBox["p",
                    "TraditionalForm"], ",", 
                  FormBox["r",
                    "TraditionalForm"], ",", 
                  FormBox["q",
                    "TraditionalForm"], ",", 
                  FormBox["s",
                    "TraditionalForm"], ")"}], ")"}]}], "+", 
            RowBox[{
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c113",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c113",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SubsuperscriptBox["O", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}], "G4"], "(", 
                  FormBox["r",
                    "TraditionalForm"], ",", 
                  FormBox["q",
                    "TraditionalForm"], ",", 
                  FormBox["p",
                    "TraditionalForm"], ",", 
                  FormBox["s",
                    "TraditionalForm"], ")"}], ")"}]}]}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"s", ",", "\[Sigma]"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubsuperscriptBox["O", 
          RowBox[{
            FormBox["\[Mu]",
              "TraditionalForm"], "\[VeryThinSpace]", 
            FormBox["\[Nu]",
              "TraditionalForm"], "\[VeryThinSpace]", 
            FormBox["\[Rho]",
              "TraditionalForm"], "\[VeryThinSpace]", 
            FormBox["\[Sigma]",
              "TraditionalForm"]}], "G4"], "(", 
        FormBox["p",
          "TraditionalForm"], ",", 
        FormBox["q",
          "TraditionalForm"], ",", 
        FormBox["r",
          "TraditionalForm"], ",", 
        FormBox["s",
          "TraditionalForm"], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"s", ",", "\[Sigma]"}], "}"}], ",", 
        RowBox[{"Explicit", "->", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}]}], ")"}], " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}], "+", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["s",
                    "TraditionalForm"]}]}], ")"}], 
            RowBox[{"m", "-", "2"}]]}], "+", 
        RowBox[{
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "3"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], ")"}]}], ")"}], "i"], 
                    " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["r",
                              "TraditionalForm"]}], "+", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["s",
                              "TraditionalForm"]}]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}]}], ")"}]}], "-", 
        RowBox[{
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "3"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], ")"}]}], ")"}], "i"], 
                    " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["r",
                              "TraditionalForm"]}], "+", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["s",
                              "TraditionalForm"]}]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["q",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["q",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]}]}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "3"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["p",
                              "TraditionalForm"]}], "+", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["q",
                              "TraditionalForm"]}]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}], ")"}]}], ")"}], 
                      "i"]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["r",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["r",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]}]}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            FormBox[
              TagBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], 
                    RowBox[{"m", "-", "3"}]], 
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{"-", 
                            RowBox[{"(", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}], ")"}]}], "-", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["q",
                              "TraditionalForm"]}]}], ")"}], "i"], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["s",
                            "TraditionalForm"]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "i"}], "+", "m", "-", "3"}]]}]}],
                HoldForm],
              "TraditionalForm"], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  FormBox["s",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["s",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["s",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["s",
                    "TraditionalForm"]}]}]}], ")"}]}], "-", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox[
                TagBox[
                  RowBox[{
                    UnderoverscriptBox["\[Sum]", 
                      RowBox[{"j", "=", "0"}], 
                      RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
                  HoldForm],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", 
                        RowBox[{"(", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["r",
                              "TraditionalForm"]}], ")"}]}], ")"}], "i"], 
                  " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}], ")"}]}], "-", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["s",
                            "TraditionalForm"]}]}], ")"}], 
                    RowBox[{"j", "-", "i"}]]}],
                "TraditionalForm"]}], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  SuperscriptBox[
                    FormBox["r",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]}]}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox[
                TagBox[
                  RowBox[{
                    UnderoverscriptBox["\[Sum]", 
                      RowBox[{"j", "=", "0"}], 
                      RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
                  HoldForm],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}], ")"}]}], "-", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["s",
                            "TraditionalForm"]}]}], ")"}], 
                    RowBox[{"j", "-", "i"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", 
                        RowBox[{"(", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["s",
                              "TraditionalForm"]}], ")"}]}], ")"}], "i"]}],
                "TraditionalForm"]}], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  SuperscriptBox[
                    FormBox["s",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["s",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["s",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["p",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["s",
                    "TraditionalForm"]}]}]}], ")"}]}], "+", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox[
                TagBox[
                  RowBox[{
                    UnderoverscriptBox["\[Sum]", 
                      RowBox[{"j", "=", "0"}], 
                      RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
                  HoldForm],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", 
                        RowBox[{"(", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["r",
                              "TraditionalForm"]}], ")"}]}], ")"}], "i"], 
                  " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}], ")"}]}], "-", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["s",
                            "TraditionalForm"]}]}], ")"}], 
                    RowBox[{"j", "-", "i"}]]}],
                "TraditionalForm"]}], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Sigma]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  SuperscriptBox[
                    FormBox["r",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["q",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["q",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]}]}], ")"}]}], "-", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox[
                TagBox[
                  RowBox[{
                    UnderoverscriptBox["\[Sum]", 
                      RowBox[{"j", "=", "0"}], 
                      RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
                  HoldForm],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}], ")"}]}], "-", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["s",
                            "TraditionalForm"]}]}], ")"}], 
                    RowBox[{"j", "-", "i"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", 
                        RowBox[{"(", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["s",
                              "TraditionalForm"]}], ")"}]}], ")"}], "i"]}],
                "TraditionalForm"]}], ")"}], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          SuperscriptBox[
            FormBox["\[CapitalDelta]",
              "TraditionalForm"], 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  SuperscriptBox[
                    FormBox["s",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox["g", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]}]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["s",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["q",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["s",
                    "TraditionalForm"]}]}], "+", 
              RowBox[{
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Sigma]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["q",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["s",
                    "TraditionalForm"]}]}]}], ")"}]}]}], TraditionalForm]], 
  "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Advanced Examples", "Subsubsection"],

Cell["The setting All for Explicit performs the sums.", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"Twist2GluonOperator", ",", " ", 
          RowBox[{"Polarization", " ", "->", " ", "1"}], ",", " ", 
          RowBox[{"Explicit", "->", "All"}]}], "]"}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Twist2GluonOperator", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}], ";"}]], 
  "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Polarized case, non-zero-momentum insertion", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SetOptions", "[", 
      RowBox[{"Twist2GluonOperator", ",", " ", 
        RowBox[{"Polarization", " ", "->", " ", "1"}], ",", 
        RowBox[{"ZeroMomentumInsertion", "->", "False"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
          RowBox[{"Explicit", "\[Rule]", "All"}], ",", 
          RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell["2-gluon legs", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "q"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"\[Nu]", ",", "b"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
        RowBox[{"(", 
          RowBox[{"1", "-", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["\[Epsilon]", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox[
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], "-", 
                  RowBox[{
                    SuperscriptBox["\[Epsilon]", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}]], " ", 
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}]}]}], ")"}], " ", 
              FormBox[
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}], ")"}], 
                  RowBox[{"m", "-", "2"}]],
                "TraditionalForm"]}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox["\[Epsilon]", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox[
                          FormBox["\[Nu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}]], " ", 
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["q",
                        "TraditionalForm"]}]}], "-", 
                  RowBox[{
                    SuperscriptBox["\[Epsilon]", 
                      RowBox[{
                        FormBox[
                          FormBox["\[Mu]",
                            "TraditionalForm"],
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"], "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}]], " ", 
                    SuperscriptBox[
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}]}], ")"}], " ", 
              FormBox[
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["q",
                        "TraditionalForm"]}], ")"}], 
                  RowBox[{"m", "-", "2"}]],
                "TraditionalForm"]}]}], ")"}], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["3-gluon legs", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", 
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["p",
                      "TraditionalForm"]}]]}], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["p",
                      "TraditionalForm"]}], ")"}], 
                RowBox[{"m", "-", "2"}]]}], "+", 
            RowBox[{
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}], ")"}], 
                RowBox[{"m", "-", "2"}]]}], "-", 
            RowBox[{
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["r",
                      "TraditionalForm"]}], ")"}], 
                RowBox[{"m", "-", "2"}]]}], "-", 
            RowBox[{
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                FormBox[
                  TagBox[
                    RowBox[{
                      UnderoverscriptBox["\[Sum]", 
                        RowBox[{"i", "=", "0"}], 
                        RowBox[{"m", "-", "3"}]], 
                      RowBox[{
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], ")"}], 
                          RowBox[{
                            RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}]}], ")"}], "i"]}]}],
                    HoldForm],
                  "TraditionalForm"], ")"}], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                FormBox[
                  TagBox[
                    RowBox[{
                      UnderoverscriptBox["\[Sum]", 
                        RowBox[{"i", "=", "0"}], 
                        RowBox[{"m", "-", "3"}]], 
                      RowBox[{
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], ")"}], 
                          RowBox[{
                            RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}]}], ")"}], "i"]}]}],
                    HoldForm],
                  "TraditionalForm"], ")"}], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                FormBox[
                  TagBox[
                    RowBox[{
                      UnderoverscriptBox["\[Sum]", 
                        RowBox[{"i", "=", "0"}], 
                        RowBox[{"m", "-", "3"}]], 
                      RowBox[{
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], ")"}], 
                          RowBox[{
                            RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}]}], ")"}], "i"]}]}],
                    HoldForm],
                  "TraditionalForm"], ")"}], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                FormBox[
                  TagBox[
                    RowBox[{
                      UnderoverscriptBox["\[Sum]", 
                        RowBox[{"i", "=", "0"}], 
                        RowBox[{"m", "-", "3"}]], 
                      RowBox[{
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}], ")"}], 
                          RowBox[{
                            RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}]}], ")"}], "i"]}]}],
                    HoldForm],
                  "TraditionalForm"], ")"}], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                FormBox[
                  TagBox[
                    RowBox[{
                      UnderoverscriptBox["\[Sum]", 
                        RowBox[{"i", "=", "0"}], 
                        RowBox[{"m", "-", "3"}]], 
                      RowBox[{
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], ")"}], 
                          RowBox[{
                            RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}]}], ")"}], "i"]}]}],
                    HoldForm],
                  "TraditionalForm"], ")"}], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "-", 
            RowBox[{
              SuperscriptBox["\[Epsilon]", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                FormBox[
                  TagBox[
                    RowBox[{
                      UnderoverscriptBox["\[Sum]", 
                        RowBox[{"i", "=", "0"}], 
                        RowBox[{"m", "-", "3"}]], 
                      RowBox[{
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}], ")"}], 
                          RowBox[{
                            RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}]}], ")"}], "i"]}]}],
                    HoldForm],
                  "TraditionalForm"], ")"}], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]]}], "+", 
            RowBox[{
              FractionBox["1", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}], "+", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}]}], ")"}], "2"]], 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["r",
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}]}], ")"}], "m"], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "3"]}], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "2"]}], "+", 
                      RowBox[{"2", " ", 
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "2"], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}]}]}], ")"}]}], ")"}]}], "-", 
            RowBox[{
              FractionBox["1", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}], "+", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}]}], ")"}], "2"]], 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["r",
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}]}], ")"}], "m"], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "3"]}], "+", 
                      RowBox[{"2", " ", 
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "2"]}], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "2"], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}]}]}], ")"}]}], ")"}]}], "-", 
            RowBox[{
              FractionBox["1", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}], "+", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["r",
                          "TraditionalForm"]}]}], ")"}], "2"]], 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["q",
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}]}], ")"}], "m"], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "3"]}], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "2"]}], "+", 
                      RowBox[{"2", " ", 
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "2"], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["r",
                            "TraditionalForm"]}]}]}], ")"}]}], ")"}]}], "-", 
            RowBox[{
              FractionBox["1", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}], "+", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["r",
                          "TraditionalForm"]}]}], ")"}], "2"]], 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["q",
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}]}], ")"}], "m"], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "3"]}], "+", 
                      RowBox[{"2", " ", 
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "2"]}], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["p",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "2"], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["r",
                            "TraditionalForm"]}]}]}], ")"}]}], ")"}]}], "-", 
            RowBox[{
              FractionBox["1", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}], "+", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["r",
                          "TraditionalForm"]}]}], ")"}], "2"]], 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}]}], ")"}], "m"], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "3"]}], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "2"]}], "+", 
                      RowBox[{"2", " ", 
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "2"], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["r",
                            "TraditionalForm"]}]}]}], ")"}]}], ")"}]}], "+", 
            RowBox[{
              FractionBox["1", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}], "+", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["r",
                          "TraditionalForm"]}]}], ")"}], "2"]], 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}]}], ")"}], "m"], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "3"]}], "+", 
                      RowBox[{"2", " ", 
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "2"]}], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          FormBox[
                            TagBox[
                              RowBox[{
                                UnderoverscriptBox["\[Sum]", 
                                  RowBox[{"i", "=", "0"}], 
                                  RowBox[{"m", "-", "3"}]], 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}], ")"}], 
                                    RowBox[{
                                      RowBox[{"-", "i"}], "+", "m", "-", 
                                      "3"}]], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["q",
                                        "TraditionalForm"]}], "+", 
                                        RowBox[{
                                        FormBox["\[CapitalDelta]",
                                        "TraditionalForm"], "\[NoBreak]", 
                                        "\[CenterDot]", "\[NoBreak]", 
                                        FormBox["r",
                                        "TraditionalForm"]}]}], ")"}], 
                                    "i"]}]}],
                              HoldForm],
                            "TraditionalForm"], ")"}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            FormBox[
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}],
                              "TraditionalForm"], ")"}], "2"], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["r",
                            "TraditionalForm"]}]}]}], ")"}]}], ")"}]}], "-", 
            FractionBox[
              RowBox[{
                SuperscriptBox["\[Epsilon]", 
                  RowBox[{
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"]}]], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}], " ", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["r",
                          "TraditionalForm"]}], " ", 
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["p",
                              "TraditionalForm"]}], ")"}], "m"]}], "+", 
                    RowBox[{
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}], " ", 
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["r",
                              "TraditionalForm"]}], ")"}], "m"], " ", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}]}], "+", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["q",
                              "TraditionalForm"]}], ")"}], "m"], " ", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["r",
                          "TraditionalForm"]}], " ", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}]}]}], ")"}]}], 
              RowBox[{
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["r",
                    "TraditionalForm"]}], " ", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}]}]]}], ")"}], " ", 
        SubscriptBox["f", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["c",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["4-gluon legs", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "False"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[ImaginaryI]", " ", 
        RowBox[{"(", 
          RowBox[{"1", "-", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", 
                SubscriptBox["f", 
                  RowBox[{
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["c114",
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c114",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SubsuperscriptBox["O", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}], "G4"], "(", 
                  FormBox["p",
                    "TraditionalForm"], ",", 
                  FormBox["q",
                    "TraditionalForm"], ",", 
                  FormBox["r",
                    "TraditionalForm"], ",", 
                  FormBox["s",
                    "TraditionalForm"], ")"}], ")"}]}], "-", 
            RowBox[{
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c114",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c114",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SubsuperscriptBox["O", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}], "G4"], "(", 
                  FormBox["p",
                    "TraditionalForm"], ",", 
                  FormBox["r",
                    "TraditionalForm"], ",", 
                  FormBox["q",
                    "TraditionalForm"], ",", 
                  FormBox["s",
                    "TraditionalForm"], ")"}], ")"}]}], "+", 
            RowBox[{
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c114",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c114",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SubsuperscriptBox["O", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}], "G4"], "(", 
                  FormBox["r",
                    "TraditionalForm"], ",", 
                  FormBox["q",
                    "TraditionalForm"], ",", 
                  FormBox["p",
                    "TraditionalForm"], ",", 
                  FormBox["s",
                    "TraditionalForm"], ")"}], ")"}]}]}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Unpolarized case, non-zero-momentum insertion", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SetOptions", "[", 
      RowBox[{"Twist2GluonOperator", ",", " ", 
        RowBox[{"Polarization", " ", "->", " ", "0"}], ",", 
        RowBox[{"ZeroMomentumInsertion", "->", "False"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Polarization", "\[Rule]", "0"}], ",", 
          RowBox[{"Explicit", "\[Rule]", "All"}], ",", 
          RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell["2-gluon legs", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "q"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"\[Nu]", ",", "b"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "2"]}], " ", 
        RowBox[{"(", 
          RowBox[{"1", "+", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SuperscriptBox[
                FormBox["q",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["q",
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}]}], "+", 
            RowBox[{
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["q",
                        "TraditionalForm"]}]}], "-", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    RowBox[{
                      FormBox["p",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["q",
                        "TraditionalForm"]}]}]}], ")"}]}]}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["p",
                      "TraditionalForm"]}], ")"}], 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"], "+", 
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}], ")"}], 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"]}], ")"}], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["b",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["3-gluon legs", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Twist2GluonOperator", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}], ";"}]], 
  "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "2"]}], " ", 
        RowBox[{"(", 
          RowBox[{"1", "+", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SuperscriptBox[
                FormBox["\[Mu]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["q",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["r",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["q",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["r",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["\[Mu]",
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}]}], "+", 
            RowBox[{
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["q",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["r",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["\[Mu]",
                        "TraditionalForm"]}]}], "-", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["r",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    RowBox[{
                      FormBox["p",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["\[Mu]",
                        "TraditionalForm"]}]}]}], ")"}]}]}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["p",
                      "TraditionalForm"]}], ")"}], 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"], "+", 
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["\[Mu]",
                      "TraditionalForm"]}], ")"}], 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"]}], ")"}], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
        RowBox[{"Explicit", "->", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          FractionBox["1", "2"]}], " ", 
        RowBox[{"(", 
          RowBox[{"1", "+", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              SuperscriptBox[
                FormBox["\[Mu]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["q",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["r",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}]}], "-", 
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["q",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["r",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["\[Mu]",
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}]}], "+", 
            RowBox[{
              SuperscriptBox[
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["q",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["r",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["\[Mu]",
                        "TraditionalForm"]}]}], "-", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["r",
                          "TraditionalForm"],
                        "TraditionalForm"]], " ", 
                    RowBox[{
                      FormBox["p",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["\[Mu]",
                        "TraditionalForm"]}]}]}], ")"}]}]}], ")"}], " ", 
        RowBox[{"(", 
          RowBox[{
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["p",
                      "TraditionalForm"]}], ")"}], 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"], "+", 
            FormBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["\[Mu]",
                      "TraditionalForm"]}], ")"}], 
                RowBox[{"m", "-", "2"}]],
              "TraditionalForm"]}], ")"}], " ", 
        SubscriptBox["\[Delta]", 
          RowBox[{
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Rho]",
                "TraditionalForm"],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["4-gluon legs", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2GluonOperator", "[", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "False"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
          RowBox[{"1", "+", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"-", 
                SubscriptBox["f", 
                  RowBox[{
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      FormBox["c115",
                        "TraditionalForm"],
                      "TraditionalForm"]}]]}], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c115",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SubsuperscriptBox["O", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}], "G4"], "(", 
                  FormBox["p",
                    "TraditionalForm"], ",", 
                  FormBox["q",
                    "TraditionalForm"], ",", 
                  FormBox["r",
                    "TraditionalForm"], ",", 
                  FormBox["s",
                    "TraditionalForm"], ")"}], ")"}]}], "-", 
            RowBox[{
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c115",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c115",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SubsuperscriptBox["O", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}], "G4"], "(", 
                  FormBox["p",
                    "TraditionalForm"], ",", 
                  FormBox["r",
                    "TraditionalForm"], ",", 
                  FormBox["q",
                    "TraditionalForm"], ",", 
                  FormBox["s",
                    "TraditionalForm"], ")"}], ")"}]}], "+", 
            RowBox[{
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["a",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c115",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["d",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SubscriptBox["f", 
                RowBox[{
                  FormBox[
                    FormBox["b",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["c115",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SubsuperscriptBox["O", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[VeryThinSpace]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}], "G4"], "(", 
                  FormBox["r",
                    "TraditionalForm"], ",", 
                  FormBox["q",
                    "TraditionalForm"], ",", 
                  FormBox["p",
                    "TraditionalForm"], ",", 
                  FormBox["s",
                    "TraditionalForm"], ")"}], ")"}]}]}], ")"}]}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell["Suppress the lengthy output.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Short", "[", 
      RowBox[{
        RowBox[{"Twist2GluonOperator", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"s", ",", "\[Sigma]"}], "}"}], ",", 
            RowBox[{"Explicit", "->", "True"}]}], "]"}], ",", "6"}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      TagBox[
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox["g", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], "-", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Rho]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], "-", 
                RowBox[{
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Mu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox["g", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], "+", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]]}]}], ")"}], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["r",
                      "TraditionalForm"]}], "+", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["s",
                      "TraditionalForm"]}]}], ")"}], 
              RowBox[{"m", "-", "2"}]]}], "+", 
          RowBox[{"\[LeftSkeleton]", "8", "\[RightSkeleton]"}], "+", 
          RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "-", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FormBox[
                  RowBox[{
                    UnderoverscriptBox["\[Sum]", 
                      RowBox[{"j", "=", "0"}], 
                      RowBox[{"m", "-", "4"}]], "\<\"\"\>"}],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  RowBox[{
                    UnderoverscriptBox["\[Sum]", 
                      RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
                  "TraditionalForm"], "\[NoBreak]", 
                FormBox[
                  RowBox[{
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}], ")"}], 
                      RowBox[{
                        RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{"-", 
                            RowBox[{"(", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}], ")"}]}], "-", 
                          RowBox[{
                            FormBox["\[CapitalDelta]",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["s",
                              "TraditionalForm"]}]}], ")"}], 
                      RowBox[{"j", "-", "i"}]], " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["s",
                                "TraditionalForm"]}], ")"}]}], ")"}], "i"]}],
                  "TraditionalForm"]}], ")"}], " ", 
            SuperscriptBox[
              FormBox["\[CapitalDelta]",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Mu]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            SuperscriptBox[
              FormBox["\[CapitalDelta]",
                "TraditionalForm"], 
              FormBox[
                FormBox["\[Rho]",
                  "TraditionalForm"],
                "TraditionalForm"]], " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"-", 
                    SuperscriptBox[
                      FormBox["s",
                        "TraditionalForm"], 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"]]}], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}]}], "+", 
                RowBox[{
                  SuperscriptBox["g", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Sigma]",
                          "TraditionalForm"],
                        "TraditionalForm"]}]], " ", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["s",
                      "TraditionalForm"]}], " ", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}]}], "-", 
                RowBox[{
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox[
                    FormBox["q",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["s",
                      "TraditionalForm"]}]}], "+", 
                RowBox[{
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Sigma]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  RowBox[{
                    FormBox["q",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["s",
                      "TraditionalForm"]}]}]}], ")"}]}]}],
        (Short[ #, 6]&)], TraditionalForm]], "Output"]
}, Open  ]],

Cell["The setting Explicit -> All performs the sums.", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetOptions", "[", 
        RowBox[{"Twist2GluonOperator", ",", " ", 
          RowBox[{"Polarization", " ", "->", " ", "1"}], ",", " ", 
          RowBox[{"Explicit", "->", "All"}]}], "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Short", "[", 
      RowBox[{
        RowBox[{"Twist2GluonOperator", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}], ",", 
        "5"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      TagBox[
        RowBox[{
          SubscriptBox["g", "s"], " ", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"-", 
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}]]}], " ", 
                SuperscriptBox[
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}], ")"}], 
                  RowBox[{"m", "-", "2"}]]}], "+", 
              RowBox[{"\[LeftSkeleton]", "20", "\[RightSkeleton]"}], "+", 
              FractionBox[
                RowBox[{
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}]], " ", 
                  SuperscriptBox[
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], 
                    FormBox[
                      FormBox["\[Nu]",
                        "TraditionalForm"],
                      "TraditionalForm"]], " ", 
                  RowBox[{"(", 
                    RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
                    ")"}]}], 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}], "+", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["r",
                          "TraditionalForm"]}]}], ")"}], "2"]], "-", 
              FractionBox[
                RowBox[{
                  SuperscriptBox["\[Epsilon]", 
                    RowBox[{
                      FormBox[
                        FormBox["\[Mu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Nu]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox[
                        FormBox["\[Rho]",
                          "TraditionalForm"],
                        "TraditionalForm"], "\[NoBreak]", 
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"]}]], " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["r",
                            "TraditionalForm"]}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], ")"}], "m"]}], "+", 
                      RowBox[{
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}], ")"}], "m"], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}]}], "+", 
                      RowBox[{
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], ")"}], "m"], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["r",
                            "TraditionalForm"]}], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}]}]}], ")"}]}], 
                RowBox[{
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["q",
                      "TraditionalForm"]}], " ", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["r",
                      "TraditionalForm"]}], " ", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["p",
                      "TraditionalForm"]}]}]]}], ")"}], " ", 
          SubscriptBox["f", 
            RowBox[{
              FormBox[
                FormBox["a",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["b",
                  "TraditionalForm"],
                "TraditionalForm"], "\[NoBreak]", 
              FormBox[
                FormBox["c",
                  "TraditionalForm"],
                "TraditionalForm"]}]]}],
        (Short[ #, 5]&)], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Twist2QuarkOperator", "Subsection",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Twist2QuarkOperator[p] or Twist2QuarkOperator[p,_,_] yields the \
quark-antiquark operator (p is momentum in the direction of the incoming \
quark). Twist2QuarkOperator[{p,q}] yields the 2-quark operator for non-zero \
momentum insertion (p is momentum in the direction of the incoming quark). \
Twist2QuarkOperator[{p1,___}, {p2,___}, {p3, mu, a}] or \
Twist2QuarkOperator[p1,_,_, p2,_,_, p3,mu,a] is the quark-antiquark-gluon \
operator, where p1 is the incoming quark, p2 the incoming antiquark and p3 \
denotes the incoming gluon momentum. Twist2QuarkOperator[{p1}, {p2}, {p3, mu, \
a}, {p4, nu, b}] or Twist2QuarkOperator[{p1,___}, {p2,___}, {p3, mu, a}, {p4, \
nu, b}] or Twist2QuarkOperator[p1,_,_, p2,_,_, p3,mu,a, p4, nu, b] gives the \
Quark-Quark-Gluon-Gluon-operator. The setting of the option Polarization \
(unpolarized: 0; polarized: 1) determines whether the unpolarized or \
polarized operator is returned\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Twist2GluonOperator",
    ButtonData:>"Twist2GluonOperator",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Twist2GluonOperator"],
  "."
}], "Text"]
}, Open  ]],

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell["Polarized case, zero-momentum insertion", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SetOptions", "[", 
      RowBox[{"Twist2QuarkOperator", ",", 
        RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
        RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Explicit", "\[Rule]", "True"}], ",", 
          RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
          RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell["Quark-antiquark operator.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{"Twist2QuarkOperator", "[", "p", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FormBox["\<\"\[Gamma]\"\>",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"]}], ")"}], ".", 
            SuperscriptBox["\[Gamma]", "5"]}]}], " ", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{
              FormBox["\[CapitalDelta]",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["p",
                "TraditionalForm"]}], ")"}], 
          RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["Quark-antiquark-gluon operator.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{"Twist2QuarkOperator", "[", 
        RowBox[{
          RowBox[{"{", "p", "}"}], ",", 
          RowBox[{"{", "q", "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SubscriptBox["g", "s"]}], " ", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}], ".", 
          SuperscriptBox["\[Gamma]", "5"], ".", 
          SubscriptBox["T", 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]]}], " ", 
        RowBox[{"(", 
          FormBox[
            TagBox[
              RowBox[{
                UnderoverscriptBox["\[Sum]", 
                  RowBox[{"i", "=", "0"}], 
                  RowBox[{"m", "-", "2"}]], 
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", "1"}], ")"}], "i"], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "i"}], "+", "m", "-", "2"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}], ")"}], "i"]}]}],
              HoldForm],
            "TraditionalForm"], ")"}], " ", 
        SuperscriptBox[
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["Quark-antiquark-gluon-gluon operator.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t3", "=", 
      RowBox[{"Twist2QuarkOperator", "[", 
        RowBox[{
          RowBox[{"{", "p", "}"}], ",", 
          RowBox[{"{", "q", "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"r", ",", "\[Nu]", ",", "b"}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SubsuperscriptBox["g", "s", "2"]}], " ", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}], ".", 
          SuperscriptBox["\[Gamma]", "5"], ".", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    FormBox[
                      TagBox[
                        RowBox[{
                          UnderoverscriptBox["\[Sum]", 
                            RowBox[{"j", "=", "0"}], 
                            RowBox[{"m", "-", "3"}]], "\<\"\"\>"}],
                        HoldForm],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      RowBox[{
                        UnderoverscriptBox["\[Sum]", 
                          RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      RowBox[{
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{"-", "1"}], ")"}], "j"], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], ")"}], "i"], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], ")"}], 
                          RowBox[{
                            RowBox[{"-", "j"}], "+", "m", "-", "3"}]], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}]}], ")"}], 
                          RowBox[{"j", "-", "i"}]]}],
                      "TraditionalForm"]}], ")"}]}], "-", 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"-", "1"}], ")"}], "m"], " ", 
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    FormBox[
                      TagBox[
                        RowBox[{
                          UnderoverscriptBox["\[Sum]", 
                            RowBox[{"j", "=", "0"}], 
                            RowBox[{"m", "-", "3"}]], "\<\"\"\>"}],
                        HoldForm],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      RowBox[{
                        UnderoverscriptBox["\[Sum]", 
                          RowBox[{"i", "=", "0"}], "j"], "\<\"\"\>"}],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      RowBox[{
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{"-", "1"}], ")"}], "j"], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], ")"}], 
                          RowBox[{
                            RowBox[{"-", "j"}], "+", "m", "-", "3"}]], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], ")"}], "i"], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}]}], ")"}], 
                          RowBox[{"j", "-", "i"}]]}],
                      "TraditionalForm"]}], ")"}]}]}], ")"}]}], " ", 
        SuperscriptBox[
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SuperscriptBox[
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Unpolarized case, zero-momentum insertion", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SetOptions", "[", 
      RowBox[{"Twist2QuarkOperator", ",", " ", 
        RowBox[{"Polarization", "->", "0"}], ",", 
        RowBox[{"ZeroMomentumInsertion", "->", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Explicit", "\[Rule]", "True"}], ",", 
          RowBox[{"Polarization", "\[Rule]", "0"}], ",", 
          RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell["Quark-antiquark operator.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t4", "=", 
      RowBox[{"Twist2QuarkOperator", "[", "p", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FormBox["\<\"\[Gamma]\"\>",
            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
          FormBox["\[CapitalDelta]",
            "TraditionalForm"]}], " ", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{
              FormBox["\[CapitalDelta]",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["p",
                "TraditionalForm"]}], ")"}], 
          RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["Quark-antiquark-gluon operator.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t5", "=", 
      RowBox[{"Twist2QuarkOperator", "[", 
        RowBox[{
          RowBox[{"{", "p", "}"}], ",", 
          RowBox[{"{", "q", "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SubscriptBox["g", "s"], " ", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}], ".", 
          SubscriptBox["T", 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]]}], " ", 
        RowBox[{"(", 
          FormBox[
            TagBox[
              RowBox[{
                UnderoverscriptBox["\[Sum]", 
                  RowBox[{"i", "=", "0"}], 
                  RowBox[{"m", "-", "2"}]], 
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{"-", "1"}], ")"}], "i"], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["p",
                          "TraditionalForm"]}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "i"}], "+", "m", "-", "2"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        FormBox["\[CapitalDelta]",
                          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                        "\[NoBreak]", 
                        FormBox["q",
                          "TraditionalForm"]}], ")"}], "i"]}]}],
              HoldForm],
            "TraditionalForm"], ")"}], " ", 
        SuperscriptBox[
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["Quark-antiquark-gluon-gluon operator.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t6", "=", 
      RowBox[{"Twist2QuarkOperator", "[", 
        RowBox[{
          RowBox[{"{", "p", "}"}], ",", 
          RowBox[{"{", "q", "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"r", ",", "\[Nu]", ",", "b"}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"]}], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    FormBox[
                      TagBox[
                        RowBox[{
                          UnderoverscriptBox["\[Sum]", 
                            RowBox[{"i", "=", "0"}], 
                            RowBox[{"m", "-", "3"}]], "\<\"\"\>"}],
                        HoldForm],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      RowBox[{
                        UnderoverscriptBox["\[Sum]", 
                          RowBox[{"j", "=", "0"}], "i"], "\<\"\"\>"}],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      RowBox[{
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{"-", 
                              RowBox[{"(", 
                                RowBox[{
                                  FormBox["\[CapitalDelta]",
                                    "TraditionalForm"], "\[NoBreak]", 
                                  "\[CenterDot]", "\[NoBreak]", 
                                  FormBox["p",
                                    "TraditionalForm"]}], ")"}]}], ")"}], 
                          RowBox[{
                            RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], ")"}], "j"], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["k",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}]}], ")"}], 
                          RowBox[{"i", "-", "j"}]]}],
                      "TraditionalForm"]}], ")"}]}], "+", 
              RowBox[{
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    FormBox[
                      TagBox[
                        RowBox[{
                          UnderoverscriptBox["\[Sum]", 
                            RowBox[{"i", "=", "0"}], 
                            RowBox[{"m", "-", "3"}]], "\<\"\"\>"}],
                        HoldForm],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      RowBox[{
                        UnderoverscriptBox["\[Sum]", 
                          RowBox[{"j", "=", "0"}], "i"], "\<\"\"\>"}],
                      "TraditionalForm"], "\[NoBreak]", 
                    FormBox[
                      RowBox[{
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{"-", 
                              RowBox[{"(", 
                                RowBox[{
                                  FormBox["\[CapitalDelta]",
                                    "TraditionalForm"], "\[NoBreak]", 
                                  "\[CenterDot]", "\[NoBreak]", 
                                  FormBox["p",
                                    "TraditionalForm"]}], ")"}]}], ")"}], 
                          RowBox[{
                            RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], ")"}], "j"], " ", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}]}], ")"}], 
                          RowBox[{"i", "-", "j"}]]}],
                      "TraditionalForm"]}], ")"}]}]}], ")"}]}], " ", 
        SuperscriptBox[
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SuperscriptBox[
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["This shows the FeynCalcExternal (FCE) form.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"Twist2QuarkOperator", "[", "p", "]"}], "//", "FCE"}], "//", 
      "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"GSD", "[", "OPEDelta", "]"}], " ", 
      SuperscriptBox[
        RowBox[{"SOD", "[", "p", "]"}], 
        RowBox[{
          RowBox[{"-", "1"}], "+", "OPEm"}]]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2QuarkOperator", "[", 
      RowBox[{
        RowBox[{"{", "p", "}"}], ",", 
        RowBox[{"{", "q", "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "All"}], ",", 
        RowBox[{"Polarization", "\[Rule]", "0"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SubscriptBox["g", "s"]}], " ", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}], ".", 
          SubscriptBox["T", 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]]}], " ", 
        SuperscriptBox[
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        RowBox[{"(", 
          RowBox[{
            FractionBox[
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    FormBox["\[CapitalDelta]",
                      "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                    "\[NoBreak]", 
                    FormBox["p",
                      "TraditionalForm"]}], ")"}], 
                RowBox[{"m", "-", "1"}]], 
              RowBox[{
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}], "+", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]}]], "+", 
            FractionBox[
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"-", "1"}], ")"}], "m"], " ", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["q",
                        "TraditionalForm"]}], ")"}], 
                  RowBox[{"m", "-", "1"}]]}], 
              RowBox[{
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}], "+", 
                RowBox[{
                  FormBox["\[CapitalDelta]",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["q",
                    "TraditionalForm"]}]}]]}], ")"}]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Twist2QuarkOperator", "[", 
      RowBox[{
        RowBox[{"{", "p", "}"}], ",", 
        RowBox[{"{", "q", "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"r", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
        RowBox[{"Explicit", "\[Rule]", "All"}], ",", 
        RowBox[{"Polarization", "\[Rule]", "0"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"-", 
          SubsuperscriptBox["g", "s", "2"]}], " ", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}], ".", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"-", 
                      FractionBox[
                        RowBox[{
                          FormBox[
                            SuperscriptBox[
                              RowBox[{"(", 
                                RowBox[{"-", "1"}], ")"}], "m"],
                            "TraditionalForm"], " ", 
                          FormBox[
                            SuperscriptBox[
                              RowBox[{"(", 
                                RowBox[{
                                  FormBox["\[CapitalDelta]",
                                    "TraditionalForm"], "\[NoBreak]", 
                                  "\[CenterDot]", "\[NoBreak]", 
                                  FormBox["p",
                                    "TraditionalForm"]}], ")"}], 
                              RowBox[{"m", "-", "1"}]],
                            "TraditionalForm"]}], 
                        RowBox[{
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}]}], ")"}], " ", 
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["k",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}]}], ")"}]}]]}], "-", 
                    FractionBox[
                      FormBox[
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], ")"}], 
                          RowBox[{"m", "-", "1"}]],
                        "TraditionalForm"], 
                      RowBox[{
                        RowBox[{
                          FormBox["k",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"]}], " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}]}], ")"}]}]], "+", 
                    FractionBox[
                      FormBox[
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["k",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}]}], ")"}], 
                          RowBox[{"m", "-", "1"}]],
                        "TraditionalForm"], 
                      RowBox[{
                        RowBox[{
                          FormBox["k",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"]}], " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              FormBox["k",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}]}], ")"}]}]]}], ")"}]}], 
              "+", 
              RowBox[{
                RowBox[{
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["b",
                        "TraditionalForm"],
                      "TraditionalForm"]], ".", 
                  SubscriptBox["T", 
                    FormBox[
                      FormBox["a",
                        "TraditionalForm"],
                      "TraditionalForm"]]}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"-", 
                      FractionBox[
                        RowBox[{
                          FormBox[
                            SuperscriptBox[
                              RowBox[{"(", 
                                RowBox[{"-", "1"}], ")"}], "m"],
                            "TraditionalForm"], " ", 
                          FormBox[
                            SuperscriptBox[
                              RowBox[{"(", 
                                RowBox[{
                                  FormBox["\[CapitalDelta]",
                                    "TraditionalForm"], "\[NoBreak]", 
                                  "\[CenterDot]", "\[NoBreak]", 
                                  FormBox["p",
                                    "TraditionalForm"]}], ")"}], 
                              RowBox[{"m", "-", "1"}]],
                            "TraditionalForm"]}], 
                        RowBox[{
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}]}], ")"}], " ", 
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["p",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}]}], ")"}]}]]}], "-", 
                    FractionBox[
                      FormBox[
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], ")"}], 
                          RowBox[{"m", "-", "1"}]],
                        "TraditionalForm"], 
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}]}], ")"}], " ", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["r",
                            "TraditionalForm"]}]}]], "+", 
                    FractionBox[
                      FormBox[
                        SuperscriptBox[
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["q",
                                  "TraditionalForm"]}], "+", 
                              RowBox[{
                                FormBox["\[CapitalDelta]",
                                  "TraditionalForm"], "\[NoBreak]", 
                                "\[CenterDot]", "\[NoBreak]", 
                                FormBox["r",
                                  "TraditionalForm"]}]}], ")"}], 
                          RowBox[{"m", "-", "1"}]],
                        "TraditionalForm"], 
                      RowBox[{
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["r",
                            "TraditionalForm"]}], " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["q",
                                "TraditionalForm"]}], "+", 
                            RowBox[{
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["r",
                                "TraditionalForm"]}]}], ")"}]}]]}], 
                  ")"}]}]}], ")"}]}], " ", 
        SuperscriptBox[
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SuperscriptBox[
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        FormBox[
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "m"],
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Polarized case, non-zero momentum insertion", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SetOptions", "[", 
      RowBox[{"Twist2QuarkOperator", ",", 
        RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
        RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Explicit", "\[Rule]", "True"}], ",", 
          RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
          RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[
"With the setting ZeroMomentumInsertion \[Rule] False a non-zero momentum is \
assumed to flow into the operator vertex. "], "Text"],

Cell["\<\
This is the Feynman rule associated with the quark-antiquark \
operator, where p is the momentum of the incoming quark and q the momentum of \
the antiquark. The momentum flowing into the operator vertex is thus \
-p-q.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", "=", 
      RowBox[{"Twist2QuarkOperator", "[", 
        RowBox[{"{", 
          RowBox[{"p", ",", "q"}], "}"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["2", 
          RowBox[{"1", "-", "m"}]], " ", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "5"], ".", 
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}]}], " ", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], "-", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["q",
                  "TraditionalForm"]}]}], ")"}], 
          RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["This is the quark-antiquark-gluon operator vertex.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{"Twist2QuarkOperator", "[", 
        RowBox[{
          RowBox[{"{", "p", "}"}], ",", 
          RowBox[{"{", "q", "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["2", 
          RowBox[{"2", "-", "m"}]], " ", 
        SubscriptBox["g", "s"], " ", 
        RowBox[{
          SubscriptBox["T", 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", "5"], ".", 
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}]}], " ", 
        RowBox[{"(", 
          FormBox[
            TagBox[
              RowBox[{
                UnderoverscriptBox["\[Sum]", 
                  RowBox[{"i", "=", "0"}], 
                  RowBox[{"m", "-", "2"}]], 
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["k",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"]}], ")"}]}], "+", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}], "-", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}]}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "i"}], "+", "m", "-", "2"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{
                          FormBox["k",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"]}], "+", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}], "-", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}]}], ")"}], "i"]}]}],
              HoldForm],
            "TraditionalForm"], ")"}], " ", 
        SuperscriptBox[
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["This shows the FeynCalcExternal form.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"StandardForm", "[", 
      RowBox[{"FCE", "[", "t2", "]"}], "]"}]], "Input"],

Cell[BoxData[
    RowBox[{
      SuperscriptBox["2", 
        RowBox[{"2", "-", "OPEm"}]], " ", "Gstrong", " ", 
      RowBox[{
        RowBox[{"SUNT", "[", "a", "]"}], ".", 
        RowBox[{"GA", "[", "5", "]"}], ".", 
        RowBox[{"GSD", "[", "OPEDelta", "]"}]}], " ", 
      RowBox[{"FVD", "[", 
        RowBox[{"OPEDelta", ",", "\[Mu]"}], "]"}], " ", 
      RowBox[{"OPESum", "[", 
        RowBox[{
          RowBox[{
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"-", 
                    RowBox[{"SOD", "[", "k", "]"}]}], "+", 
                  RowBox[{"SOD", "[", "p", "]"}], "-", 
                  RowBox[{"SOD", "[", "q", "]"}]}], ")"}], 
              RowBox[{
                RowBox[{"-", "2"}], "-", "OPEi", "+", "OPEm"}]], " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"SOD", "[", "k", "]"}], "+", 
                  RowBox[{"SOD", "[", "p", "]"}], "-", 
                  RowBox[{"SOD", "[", "q", "]"}]}], ")"}], "OPEi"]}], ",", 
          RowBox[{"{", 
            RowBox[{"OPEi", ",", "0", ",", 
              RowBox[{
                RowBox[{"-", "2"}], "+", "OPEm"}]}], "}"}]}], "]"}]}]], 
  "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Unpolarized case, non-zero momentum insertion", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SetOptions", "[", 
      RowBox[{"Twist2QuarkOperator", ",", 
        RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
        RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "]"}]], 
  "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"CouplingConstant", "\[Rule]", 
            SubscriptBox["g", "s"]}], ",", 
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"Explicit", "\[Rule]", "True"}], ",", 
          RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
          RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "}"}], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell["Quark-antiquark operator.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t4", "=", 
      RowBox[{"Twist2QuarkOperator", "[", 
        RowBox[{"{", 
          RowBox[{"p", ",", "q"}], "}"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["2", 
          RowBox[{"1", "-", "m"}]], " ", 
        RowBox[{
          SuperscriptBox["\[Gamma]", "5"], ".", 
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}]}], " ", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], "-", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["q",
                  "TraditionalForm"]}]}], ")"}], 
          RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["Quark-antiquark-gluon operator.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t5", "=", 
      RowBox[{"Twist2QuarkOperator", "[", 
        RowBox[{
          RowBox[{"{", "p", "}"}], ",", 
          RowBox[{"{", "q", "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["2", 
          RowBox[{"2", "-", "m"}]], " ", 
        SubscriptBox["g", "s"], " ", 
        RowBox[{
          SubscriptBox["T", 
            FormBox[
              FormBox["a",
                "TraditionalForm"],
              "TraditionalForm"]], ".", 
          SuperscriptBox["\[Gamma]", "5"], ".", 
          RowBox[{"(", 
            RowBox[{
              FormBox["\<\"\[Gamma]\"\>",
                "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
              "\[NoBreak]", 
              FormBox["\[CapitalDelta]",
                "TraditionalForm"]}], ")"}]}], " ", 
        RowBox[{"(", 
          FormBox[
            TagBox[
              RowBox[{
                UnderoverscriptBox["\[Sum]", 
                  RowBox[{"i", "=", "0"}], 
                  RowBox[{"m", "-", "2"}]], 
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"-", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox["k",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["\[CapitalDelta]",
                                "TraditionalForm"]}], ")"}]}], "+", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}], "-", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}]}], ")"}], 
                    RowBox[{
                      RowBox[{"-", "i"}], "+", "m", "-", "2"}]], " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{
                          FormBox["k",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"]}], "+", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}], "-", 
                        RowBox[{
                          FormBox["\[CapitalDelta]",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["q",
                            "TraditionalForm"]}]}], ")"}], "i"]}]}],
              HoldForm],
            "TraditionalForm"], ")"}], " ", 
        SuperscriptBox[
          FormBox["\[CapitalDelta]",
            "TraditionalForm"], 
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", 
        RowBox[{
        "t1", ",", "t2", ",", "t3", ",", "t4", ",", "t5", ",", "t6"}], "]"}], 
      ";"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Uncontract", "Subsection",
  CellTags->"Uncontract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Uncontract[exp, q1, q2, ...] uncontracts Eps and DiracGamma. \
Uncontract[exp, q1, q2, Pair -> {p}] uncontracts also p.q1 and p.q2; the \
option Pair -> All uncontracts all momenta except OPEDelta.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Uncontract", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
          RowBox[{"DimensionalReduction", "\[Rule]", "False"}], ",", 
          RowBox[{"Pair", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"Unique", "\[Rule]", "True"}]}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Contract",
    ButtonData:>"Contract",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Contract"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t1", " ", "=", " ", 
      RowBox[{
        RowBox[{"LeviCivita", "[", 
          RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "[", 
        RowBox[{"p", ",", "q"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      SuperscriptBox["\[Epsilon]", 
        RowBox[{
          FormBox[
            FormBox["\[Mu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox[
            FormBox["\[Nu]",
              "TraditionalForm"],
            "TraditionalForm"], "\[NoBreak]", 
          FormBox["p",
            "TraditionalForm"], "\[NoBreak]", 
          FormBox["q",
            "TraditionalForm"]}]], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Uncontract", "[", 
      RowBox[{"t1", ",", " ", "p"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox[
                RowBox[{"$AL$2974", "(", "1", ")"}],
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"]}]], " ", 
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox[
              RowBox[{"$AL$2974", "(", "1", ")"}],
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{"DiracSlash", "[", "p", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        FormBox["p",
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Uncontract", "[", 
      RowBox[{"t2", ",", " ", "p"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            FormBox[
              RowBox[{"$AL$2975", "(", "1", ")"}],
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox[
              RowBox[{"$AL$2975", "(", "1", ")"}],
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Uncontract", "[", 
      RowBox[{"t1", ",", " ", "p", ",", "q"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Epsilon]", 
          RowBox[{
            FormBox[
              FormBox["\[Mu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox["\[Nu]",
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox[
                RowBox[{"$AL$2977", "(", "1", ")"}],
                "TraditionalForm"],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              FormBox[
                RowBox[{"$AL$2976", "(", "1", ")"}],
                "TraditionalForm"],
              "TraditionalForm"]}]], " ", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], 
          FormBox[
            FormBox[
              RowBox[{"$AL$2976", "(", "1", ")"}],
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox[
              RowBox[{"$AL$2977", "(", "1", ")"}],
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["By default scalar products are not uncontracted.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Uncontract", "[", 
      RowBox[{
        RowBox[{"ScalarProduct", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", " ", "q"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox["p",
          "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
        FormBox["q",
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[
"With the option Pair\[Rule]All they are \[OpenCurlyDoubleQuote]uncontracted \
\[CloseCurlyDoubleQuote]."], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Uncontract", "[", 
      RowBox[{
        RowBox[{"ScalarProduct", "[", 
          RowBox[{"p", ",", "q"}], "]"}], ",", "q", ",", 
        RowBox[{"Pair", "\[Rule]", "All"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox[
              RowBox[{"$AL$2979", "(", "1", ")"}],
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], 
          FormBox[
            FormBox[
              RowBox[{"$AL$2979", "(", "1", ")"}],
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Uncontract", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"ScalarProduct", "[", 
            RowBox[{"p", ",", "q"}], "]"}], "^", "2"}], ",", "q", ",", 
        RowBox[{"Pair", "\[Rule]", "All"}]}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox[
              RowBox[{"$AL$2980", "(", "1", ")"}],
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], 
          FormBox[
            FormBox[
              RowBox[{"$AL$2980", "(", "1", ")"}],
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            FormBox[
              RowBox[{"$AL$2980", "(", "2", ")"}],
              "TraditionalForm"],
            "TraditionalForm"]], " ", 
        SuperscriptBox[
          FormBox["q",
            "TraditionalForm"], 
          FormBox[
            FormBox[
              RowBox[{"$AL$2980", "(", "2", ")"}],
              "TraditionalForm"],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Clear", "[", 
      RowBox[{"t1", ",", "t2"}], "]"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["UnDeclareNonCommutative", "Subsection",
  CellTags->"UnDeclareNonCommutative"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
UnDeclareNonCommutative[a, b, ...] undeclares a,b, ... to be \
noncommutative, i.e., DataType[a,b, ..., NonCommutative] is set to \
False.\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["DataType",
    ButtonData:>"DataType",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DataType"],
  ", ",
  ButtonBox["DeclareNonCommutative",
    ButtonData:>"DeclareNonCommutative",
    ButtonStyle->"Hyperlink",
    ButtonNote->"DeclareNonCommutative"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[BoxData[
    RowBox[{"DeclareNonCommutative", "[", "x", "]"}]], "Input"],

Cell["\<\
As a side-effect of DeclareNonCommutative x is declared to be of \
DataType NonCommutative.\
\>", "Text"],

Cell[CellGroupData[{

Cell["DataType[x,NonCommutative]", "Input"],

Cell[BoxData[
    FormBox["True", TraditionalForm]], "Output"]
}, Open  ]],

Cell["The inverse operation is UnDeclareNonCommutative.", "Text"],

Cell[BoxData[
    RowBox[{"UnDeclareNonCommutative", "[", "x", "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DataType", "[", 
      RowBox[{"x", ",", "NonCommutative"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox["False", TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"DeclareNonCommutative", "[", 
      RowBox[{"y", ",", "z"}], "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DataType", "[", 
      RowBox[{"y", ",", "z", ",", "NonCommutative"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"True", ",", "True"}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"UnDeclareNonCommutative", "[", 
      RowBox[{"y", ",", "z"}], "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"DataType", "[", 
      RowBox[{"y", ",", "z", ",", "NonCommutative"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"False", ",", "False"}], "}"}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Write2", "Subsection",
  CellTags->"Write2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Write2[file, val1 = expr1, val2 = expr2, ...] writes the settings \
val1 = expr1, val2 = expr2 in sequence followed by a newline, to the \
specified output file. Setting the option FormatType of Write2 to FortranForm \
results in FORTRAN syntax output.\
\>", "Text"],

Cell["\<\
The continuation character for FORTRAN output can be controlled by \
changing $FortranContinuationCharacter.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FullForm", "[", "$FortranContinuationCharacter", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      TagBox[
        StyleBox["\"\<&\>\"",
          ShowSpecialCharacters->False,
          ShowStringCharacters->True],
        FullForm], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Options", "[", "Write2", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"FinalSubstitutions", "\[Rule]", 
            RowBox[{"{", "}"}]}], ",", 
          RowBox[{"FormatType", "\[Rule]", "InputForm"}], ",", 
          RowBox[{"D0Convention", "\[Rule]", "0"}], ",", 
          RowBox[{"PageWidth", "\[Rule]", "62"}], ",", 
          RowBox[{"PreFortranFile", "\[Rule]", "\<\"\"\>"}], ",", 
          RowBox[{"PostFortranFile", "\[Rule]", "\<\"\"\>"}], ",", 
          RowBox[{"StringReplace", "\[Rule]", 
            RowBox[{"{", "}"}]}]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Attributes", "[", "Write2", "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{"HoldRest", ",", "ReadProtected"}], "}"}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "See also: ",
  " ",
  ButtonBox["Isolate",
    ButtonData:>"Isolate",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Isolate"],
  ", ",
  ButtonBox["PaVeReduce",
    ButtonData:>"PaVeReduce",
    ButtonStyle->"Hyperlink",
    ButtonNote->"PaVeReduce"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t", "=", 
      RowBox[{"Collect", "[", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"a", "-", "c"}], ")"}], "2"], "+", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{"a", "-", "b"}], ")"}], "2"]}], ")"}], "2"], ",", 
          "a", ",", "Factor"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"4", " ", 
          SuperscriptBox["a", "4"]}], "-", 
        RowBox[{"8", " ", 
          RowBox[{"(", 
            RowBox[{"b", "+", "c"}], ")"}], " ", 
          SuperscriptBox["a", "3"]}], "+", 
        RowBox[{"8", " ", 
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox["b", "2"], "+", 
              RowBox[{"c", " ", "b"}], "+", 
              SuperscriptBox["c", "2"]}], ")"}], " ", 
          SuperscriptBox["a", "2"]}], "-", 
        RowBox[{"4", " ", 
          RowBox[{"(", 
            RowBox[{"b", "+", "c"}], ")"}], " ", 
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox["b", "2"], "+", 
              SuperscriptBox["c", "2"]}], ")"}], " ", "a"}], "+", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{
              SuperscriptBox["b", "2"], "+", 
              SuperscriptBox["c", "2"]}], ")"}], "2"]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"tempfilename", "=", 
        RowBox[{
          RowBox[{"ToString", "[", "$SessionID", "]"}], "<>", 
          "\"\<.s\>\""}]}], ";"}]], "Input"],

Cell["This writes the assignment r=t to a file.", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Write2", "[", 
        RowBox[{"tempfilename", ",", 
          RowBox[{"r", "=", "t"}]}], "]"}], ";"}]], "Input"],

Cell["This shows the contents of the file.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TableForm", "[", 
      RowBox[{"ReadList", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"If", "[", 
              RowBox[{
                RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", 
                "\"\<:\>\"", ",", "\"\<\>\""}], "]"}], "<>", 
            "tempfilename"}], ",", "String"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      InterpretationBox[GridBox[{
            {
              "\<\"r = ( 4*a^4 - 8*a^3*(b + c) - 4*a*(b + c)*(b^2 + c^2) + \"\
\>"},
            {"\<\"  (b^2 + c^2)^2 + 8*a^2*(b^2 + b*c + c^2)\"\>"},
            {"\<\"       );\"\>"}
            },
          RowSpacings->1,
          ColumnSpacings->3,
          RowAlignments->Baseline,
          ColumnAlignments->{Left}],
        TableForm[ {
          "r = ( 4*a^4 - 8*a^3*(b + c) - 4*a*(b + c)*(b^2 + c^2) + ", 
          "  (b^2 + c^2)^2 + 8*a^2*(b^2 + b*c + c^2)", "       );"}]], 
      TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"DeleteFile", "[", 
      RowBox[{
        RowBox[{"If", "[", 
          RowBox[{
            RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", 
            "\"\<:\>\"", ",", "\"\<\>\""}], "]"}], "<>", "tempfilename"}], 
      "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"t2", "=", 
      RowBox[{"x", "+", 
        RowBox[{"Isolate", "[", 
          RowBox[{"t", ",", "a", ",", 
            RowBox[{"IsolateNames", "\[Rule]", "W"}]}], "]"}]}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"4", " ", 
          SuperscriptBox["a", "4"]}], "-", 
        RowBox[{"8", " ", 
          TagBox[
            RowBox[{"W", "(", "1", ")"}],
            HoldForm], " ", 
          SuperscriptBox["a", "3"]}], "+", 
        RowBox[{"8", " ", 
          TagBox[
            RowBox[{"W", "(", "3", ")"}],
            HoldForm], " ", 
          SuperscriptBox["a", "2"]}], "-", 
        RowBox[{"4", " ", 
          TagBox[
            RowBox[{"W", "(", "1", ")"}],
            HoldForm], " ", 
          TagBox[
            RowBox[{"W", "(", "2", ")"}],
            HoldForm], " ", "a"}], "+", 
        SuperscriptBox[
          RowBox[{"(", 
            TagBox[
              RowBox[{"W", "(", "2", ")"}],
              HoldForm], ")"}], "2"], "+", "x"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"Write2", "[", 
        RowBox[{"tempfilename", ",", 
          RowBox[{"r", "=", "t2"}]}], "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TableForm", "[", 
      RowBox[{"ReadList", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"If", "[", 
              RowBox[{
                RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", 
                "\"\<:\>\"", ",", "\"\<\>\""}], "]"}], "<>", 
            "tempfilename"}], ",", "String"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      InterpretationBox[GridBox[{
            {"\<\"W[1] = (b + c\"\>"},
            {"\<\"       );\"\>"},
            {"\<\"W[2] = (b^2 + c^2\"\>"},
            {"\<\"       );\"\>"},
            {"\<\"W[3] = (b^2 + b*c + c^2\"\>"},
            {"\<\"       );\"\>"},
            {"\<\"r = ( 4*a^4 + x - 8*a^3*HoldForm[W[1]] - \"\>"},
            {
              "\<\"  4*a*HoldForm[W[1]]*HoldForm[W[2]] + HoldForm[W[2]]^2 + \
\"\>"},
            {"\<\"  8*a^2*HoldForm[W[3]]\"\>"},
            {"\<\"       );\"\>"}
            },
          RowSpacings->1,
          ColumnSpacings->3,
          RowAlignments->Baseline,
          ColumnAlignments->{Left}],
        TableForm[ {"W[1] = (b + c", "       );", "W[2] = (b^2 + c^2", 
          "       );", "W[3] = (b^2 + b*c + c^2", "       );", 
          "r = ( 4*a^4 + x - 8*a^3*HoldForm[W[1]] - ", 
          "  4*a*HoldForm[W[1]]*HoldForm[W[2]] + HoldForm[W[2]]^2 + ", 
          "  8*a^2*HoldForm[W[3]]", "       );"}]], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"DeleteFile", "[", 
      RowBox[{
        RowBox[{"If", "[", 
          RowBox[{
            RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", 
            "\"\<:\>\"", ",", "\"\<\>\""}], "]"}], "<>", "tempfilename"}], 
      "]"}]], "Input"],

Cell["\<\
This is how to write out the expression t2 in Fortran format.\
\>", 
  "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Write2", "[", 
        RowBox[{"tempfilename", ",", 
          RowBox[{"r", "=", "t2"}], ",", 
          RowBox[{"FormatType", "->", "FortranForm"}]}], "]"}], ";"}]], 
  "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TableForm", "[", 
      RowBox[{"ReadList", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"If", "[", 
              RowBox[{
                RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", 
                "\"\<:\>\"", ",", "\"\<\>\""}], "]"}], "<>", 
            "tempfilename"}], ",", "String"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      InterpretationBox[GridBox[{
            {"\<\"        W(1)= b + c\"\>"},
            {"\<\"        W(2)= b**2 + c**2\"\>"},
            {"\<\"        W(3)= b**2 + b*c + c**2\"\>"},
            {
              "\<\"        r = 4*a**4 + x - 8*a**3*W(1) - 4*a*W(1)*W(2) + \
W(2)**2 + \"\>"},
            {"\<\"     &  8*a**2*W(3)\"\>"},
            {"\<\"                  \"\>"}
            },
          RowSpacings->1,
          ColumnSpacings->3,
          RowAlignments->Baseline,
          ColumnAlignments->{Left}],
        TableForm[ {"        W(1)= b + c", "        W(2)= b**2 + c**2", 
          "        W(3)= b**2 + b*c + c**2", 
          "        r = 4*a**4 + x - 8*a**3*W(1) - 4*a*W(1)*W(2) + W(2)**2 + ",
           "     &  8*a**2*W(3)", "                  "}]], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"DeleteFile", "[", 
        RowBox[{
          RowBox[{"If", "[", 
            RowBox[{
              RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", 
              "\"\<:\>\"", ",", "\"\<\>\""}], "]"}], "<>", "tempfilename"}], 
        "]"}], ";", 
      RowBox[{"Clear", "[", 
        RowBox[{"t", ",", "t2", ",", "tempfilename"}], "]"}], ";"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Zeta2", "Subsection",
  CellTags->"Zeta2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["Zeta2 denotes Zeta[2]. N[Zeta[2]] evaluates to N[Zeta[2]].", "Text"],

Cell[TextData[{
  "See also: ",
  ButtonBox["SimplifyPolyLog",
    ButtonData:>"SimplifyPolyLog",
    ButtonStyle->"Hyperlink",
    ButtonNote->"SimplifyPolyLog"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData["Zeta2"], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"\[Zeta]", "(", "2", ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"N", "[", "Zeta2", "]"}]], "Input"],

Cell[BoxData[
    FormBox["1.64493406684822618`", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"SimplifyPolyLog", "[", 
      RowBox[{"Pi", "^", "2"}], "]"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"6", " ", 
        RowBox[{"\[Zeta]", "(", "2", ")"}]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["$FCT", "Subsection",
  CellTags->"$FCT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
Setting the global variable $FCT to True changes the default \
typesetting rules for the noncommutative multiplication operator Dot[].\
\>", 
  "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"$FCT", " ", "=", " ", "True"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"GA", "[", "\[Mu]", "]"}], " ", ".", " ", 
      RowBox[{"GA", "[", "\[Nu]", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], " ", 
        SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"$FCT", " ", "=", " ", "False"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"GA", "[", "\[Mu]", "]"}], " ", ".", " ", 
      RowBox[{"GA", "[", "\[Nu]", "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
        SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["$LorentzIndices", "Subsection",
  CellTags->"$LorentzIndices"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
$LorentzIndices is a global variable. If set to True the dimension \
of LorentzIndex is displayed as an index.\
\>", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"$LorentzIndices", "=", "True"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"MetricTensor", "[", 
        RowBox[{"\[Alpha]", ",", "\[Beta]", ",", 
          RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}], 
      RowBox[{"DiracMatrix", "[", 
        RowBox[{"\[Alpha]", ",", 
          RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            SubscriptBox[
              FormBox["\[Alpha]",
                "TraditionalForm"], 
              FormBox["n",
                "TraditionalForm"]],
            "TraditionalForm"]], " ", 
        SuperscriptBox["g", 
          RowBox[{
            FormBox[
              SubscriptBox[
                FormBox["\[Alpha]",
                  "TraditionalForm"], 
                FormBox["n",
                  "TraditionalForm"]],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              SubscriptBox[
                FormBox["\[Beta]",
                  "TraditionalForm"], 
                FormBox["n",
                  "TraditionalForm"]],
              "TraditionalForm"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracGamma", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Alpha]", ",", "n"}], "]"}], ",", "n"}], "]"}], " ", 
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Alpha]", ",", "n"}], "]"}], ",", 
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Beta]", ",", "n"}], "]"}]}], "]"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"MTD", "[", 
          RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], " ", 
        RowBox[{"FVD", "[", 
          RowBox[{"p", ",", "\[Beta]"}], "]"}], 
        RowBox[{"GAD", "[", "\[Mu]", "]"}]}], "//", "FCI"}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["\[Gamma]", 
          FormBox[
            SubscriptBox[
              FormBox["\[Mu]",
                "TraditionalForm"], 
              FormBox["D",
                "TraditionalForm"]],
            "TraditionalForm"]], " ", 
        SuperscriptBox["g", 
          RowBox[{
            FormBox[
              SubscriptBox[
                FormBox["\[Alpha]",
                  "TraditionalForm"], 
                FormBox["D",
                  "TraditionalForm"]],
              "TraditionalForm"], "\[NoBreak]", 
            FormBox[
              SubscriptBox[
                FormBox["\[Beta]",
                  "TraditionalForm"], 
                FormBox["D",
                  "TraditionalForm"]],
              "TraditionalForm"]}]], " ", 
        SuperscriptBox[
          FormBox["p",
            "TraditionalForm"], 
          FormBox[
            SubscriptBox[
              FormBox["\[Beta]",
                "TraditionalForm"], 
              FormBox["D",
                "TraditionalForm"]],
            "TraditionalForm"]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"%", "//", "StandardForm"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"DiracGamma", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", "D"}], "]"}], " ", 
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Alpha]", ",", "D"}], "]"}], ",", 
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Beta]", ",", "D"}], "]"}]}], "]"}], " ", 
      RowBox[{"Pair", "[", 
        RowBox[{
          RowBox[{"LorentzIndex", "[", 
            RowBox[{"\[Beta]", ",", "D"}], "]"}], ",", 
          RowBox[{"Momentum", "[", 
            RowBox[{"p", ",", "D"}], "]"}]}], "]"}]}]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"$LorentzIndices", "=", "False"}], ";"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["$PairBrackets", "Subsection",
  CellTags->"$PairBrackets"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell["\<\
$PairBrackets determines whether brackets are drawn around scalar \
products in the notebook interface.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData["$PairBrackets"], "Input"],

Cell[BoxData[
    FormBox["False", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SP", "[", 
        RowBox[{"p", ",", "q"}], "]"}], 
      RowBox[{"SP", "[", 
        RowBox[{"r", ",", "s"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          RowBox[{
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"]}],
          "TraditionalForm"], " ", 
        FormBox[
          RowBox[{
            FormBox["r",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["s",
              "TraditionalForm"]}],
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"$PairBrackets", "=", "True"}]], "Input"],

Cell[BoxData[
    FormBox["True", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"SP", "[", 
        RowBox[{"p", ",", "q"}], "]"}], 
      RowBox[{"SP", "[", 
        RowBox[{"r", ",", "s"}], "]"}]}]], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        FormBox[
          RowBox[{"(", "\[NoBreak]", 
            FormBox["p",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["q",
              "TraditionalForm"], "\[NoBreak]", ")"}],
          "TraditionalForm"], " ", 
        FormBox[
          RowBox[{"(", "\[NoBreak]", 
            FormBox["r",
              "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
            "\[NoBreak]", 
            FormBox["s",
              "TraditionalForm"], "\[NoBreak]", ")"}],
          "TraditionalForm"]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"$PairBrackets", "=", "False"}]], "Input"],

Cell[BoxData[
    FormBox["False", TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["$VeryVerbose", "Subsection",
  CellTags->"$VeryVerbose"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"?", "$VeryVerbose"}]], "Input"],

Cell[BoxData[
    FormBox[
    "\<\"$VeryVerbose is a global variable with default setting 0.  If set to \
1, 2, ..., less and more intermediate comments and informations are displayed \
during calculations.\"\>", TraditionalForm]], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["$VeryVerbose"], "Input"],

Cell[BoxData[
    FormBox["0", TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"$VeryVerbose", " ", "=", " ", "3"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Collect2", "[", 
      RowBox[{
        RowBox[{"Expand", "[", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"x", "-", "y", "-", "z"}], ")"}], "^", "6"}], "]"}], 
        ",", "x"}], "]"}]], "Input"],

Cell["\<\
expanding. inhomogeneous part; LeafCount = 43; factored. \
#1#2#3#4#5#6\
\>", "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        RowBox[{
        "\<\"length \"\>", "\[InvisibleSpace]", "21", "\[InvisibleSpace]", 
          "\<\".\"\>"}],
        SequenceForm[ "length ", 21, "."],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        RowBox[{
        "\<\"collecting \"\>", "\[InvisibleSpace]", "6", "\[InvisibleSpace]", 
          "\<\" terms.\"\>"}],
        SequenceForm[ "collecting ", 6, " terms."],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["x", "6"], "-", 
        RowBox[{"6", " ", 
          RowBox[{"(", 
            RowBox[{"y", "+", "z"}], ")"}], " ", 
          SuperscriptBox["x", "5"]}], "+", 
        RowBox[{"15", " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"y", "+", "z"}], ")"}], "2"], " ", 
          SuperscriptBox["x", "4"]}], "-", 
        RowBox[{"20", " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"y", "+", "z"}], ")"}], "3"], " ", 
          SuperscriptBox["x", "3"]}], "+", 
        RowBox[{"15", " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"y", "+", "z"}], ")"}], "4"], " ", 
          SuperscriptBox["x", "2"]}], "-", 
        RowBox[{"6", " ", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"y", "+", "z"}], ")"}], "5"], " ", "x"}], "+", 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"y", "+", "z"}], ")"}], "6"]}], TraditionalForm]], 
  "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"$VeryVerbose", "=", "0"}], ";"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["$West", "Subsection",
  CellTags->"$West"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"?", "$West"}]], "Input"],

Cell[BoxData[
    FormBox[
    "\<\"If $West is set to True (which is the default),  traces involving \
more than 4 Dirac matrices and gamma5 are calculated recursively according to \
formula (A.5) from Comp. Phys. Comm 77 (1993) 286-298, which is based on the \
Breitenlohner Maison gamma5 - scheme.\"\>", TraditionalForm]], "Print"]
}, Open  ]],

Cell[TextData[{
  "See examples in ",
  ButtonBox["Tr",
    ButtonData:>"Tr",
    ButtonStyle->"Hyperlink",
    ButtonNote->"Tr"],
  "."
}], "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Evaluation time and memory usage", "Subsection",
  CellTags->"Evaluation time and memory usage"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"AbsoluteTime", "[", "]"}], "-", "absolutestarttime"}]], "Input"],

Cell[BoxData[
    FormBox["584.`8.486", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"MemoryInUse", "[", "]"}]], "Input"],

Cell[BoxData[
    FormBox["12350084", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"MaxMemoryUsed", "[", "]"}]], "Input"],

Cell[BoxData[
    FormBox["14427748", TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"TimeUsed", "[", "]"}]], "Input"],

Cell[BoxData[
    FormBox["288.750000000000017`", TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"X 3.0",
ScreenRectangle->{{0, 1024}, {0, 768}},
ScreenStyleEnvironment->"AllBrackets",
ShowPageBreaks->False,
WindowToolbars->"EditBar",
WindowSize->{569, 597},
WindowMargins->{{15, Automatic}, {9, Automatic}},
PrintingStartingPageNumber->1,
PrintingPageRange->{Automatic, Automatic},
PageHeaders->{{Inherited, Cell[ 
        TextData[ 
          ValueBox[ "FileName"]], "Header"], Inherited}, {Inherited, Cell[ 
        TextData[ 
          ValueBox[ "FileName"]], "Header"], Inherited}},
PageFooters->{{Inherited, Cell[ 
        TextData[ 
          ValueBox[ "DateLong"]], "Footer"], Inherited}, {Inherited, Cell[ 
        TextData[ 
          ValueBox[ "DateLong"]], "Footer"], Inherited}},
PrintingOptions->{"PrintingMargins"->{{54, 54}, {72, 72}},
"PaperSize"->{597.562, 842.375},
"PaperOrientation"->"Portrait",
"PrintCellBrackets"->False,
"PrintSelectionHighlighting"->False,
"PrintRegistrationMarks"->True,
"PrintMultipleHorizontalPages"->False,
"FirstPageHeader"->False,
"FirstPageFooter"->True,
"FacingPages"->True,
"FirstPageFace"->Left,
"GraphicsPrintingFormat"->"Automatic",
"Magnification"->1},
PrivateNotebookOptions->{"ShortBoxForm"->False},
TaggingRules:>{"MakeContentsNonDefaultOptions" -> {AutomaticCellTagging -> 
  FromContent, PageNumbers -> False, SaveOptions -> True, 
  SelectCellStyles -> {"Section", "Subsection"}, TOCCellStyles -> {
  "Contents", "Index"}}},
CellLabelAutoDelete->True,
StyleDefinitions -> Notebook[{
  
  Cell[CellGroupData[{
  Cell["Style Definitions", "Subtitle"],
  
  Cell["\<\
Modify the definitions below to change the default appearance of \
all cells in a given style.  Make modifications to any definition using \
commands in the Format menu.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell["Notebook Options", "Section"],
  
  Cell["\<\
The options defined for the style below will be used at the \
Notebook level.\
\>", "Text"],
  
  Cell[StyleData["Notebook"],
    PrintingStyleEnvironment->"Printout",
    PageHeaders->{{Cell[ 
            TextData[ {
              CounterBox[ "Page"]}], "PageNumber"], None, Cell[ 
            TextData[ {
              ValueBox[ "FileName"]}], "Header"]}, {Cell[ 
            TextData[ {
              ValueBox[ "FileName"]}], "Header"], None, Cell[ 
            TextData[ {
              CounterBox[ "Page"]}], "PageNumber"]}},
    PrintingOptions->{"PrintingMargins"->{{97, 97}, {180, 64}},
    "FirstPageHeader"->False,
    "FacingPages"->True},
    CellLabelAutoDelete->False,
    CellFrameLabelMargins->6,
    StyleMenuListing->None]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Style Environment Names", "Section"],
  
  Cell[StyleData[All, "Working"],
    PageWidth->WindowWidth,
    ShowCellBracket->False,
    ScriptMinSize->9],
  
  Cell[StyleData[All, "AllBrackets"],
    PageWidth->WindowWidth,
    ShowCellBracket->True,
    ScriptMinSize->9],
  
  Cell[StyleData[All, "Printout"],
    PageWidth->PaperWidth,
    ShowCellBracket->False,
    ScriptMinSize->5,
    FontSize->10,
    PrivateFontOptions->{"FontType"->"Outline"}],
  
  Cell[StyleData[All, "EnhancedPrintout"],
    PageWidth->PaperWidth,
    ShowCellBracket->False,
    ScriptMinSize->5,
    FontFamily->"Palatino",
    FontSize->10,
    PrivateFontOptions->{"FontType"->"Outline"}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Headings", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Title"],
    CellMargins->{{12, Inherited}, {20, 40}},
    CellGroupingRules->{"TitleGrouping", 0},
    PageBreakAbove->True,
    PageBreakBelow->False,
    CounterIncrements->"Title",
    CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}, {
        "Subtitle", 0}, {"Subsubtitle", 0}},
    FontFamily->"Helvetica",
    FontSize->36,
    FontWeight->"Bold"],
  
  Cell[StyleData["Title", "Printout"],
    CellMargins->{{2, 10}, {15, 30}},
    FontSize->30],
  
  Cell[StyleData["Title", "EnhancedPrintout"],
    CellMargins->{{2, 10}, {15, 30}},
    FontFamily->"Futura",
    FontSize->30]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Subtitle"],
    CellMargins->{{12, Inherited}, {10, 2}},
    CellGroupingRules->{"TitleGrouping", 10},
    PageBreakBelow->False,
    CounterIncrements->"Subtitle",
    CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}, {
        "Subsubtitle", 0}},
    FontFamily->"Helvetica",
    FontSize->28],
  
  Cell[StyleData["Subtitle", "Printout"],
    CellMargins->{{2, 10}, {10, 15}},
    FontSize->24],
  
  Cell[StyleData["Subtitle", "EnhancedPrintout"],
    CellMargins->{{2, 10}, {10, 15}},
    FontFamily->"Futura",
    FontSize->24]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Subsubtitle"],
    CellMargins->{{12, Inherited}, {10, 20}},
    CellGroupingRules->{"TitleGrouping", 20},
    PageBreakBelow->False,
    CounterIncrements->"Subsubtitle",
    CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}},
    FontFamily->"Helvetica",
    FontSize->14,
    FontSlant->"Italic"],
  
  Cell[StyleData["Subsubtitle", "Printout"],
    CellMargins->{{2, 10}, {8, 10}},
    FontSize->12],
  
  Cell[StyleData["Subsubtitle", "EnhancedPrintout"],
    CellMargins->{{2, 10}, {8, 10}},
    FontFamily->"Futura",
    FontSize->12]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Section"],
    CellFrame->{{0, 0}, {0.5, 0}},
    CellMargins->{{10, Inherited}, {8, 30}},
    CellGroupingRules->{"SectionGrouping", 30},
    PageBreakBelow->False,
    CellFrameMargins->8,
    CounterIncrements->"Section",
    CounterAssignments->{{"Subsection", 0}, {"Subsubsection", 0}},
    FontFamily->"Helvetica",
    FontSize->20,
    FontWeight->"Bold"],
  
  Cell[StyleData["Section", "Printout"],
    CellMargins->{{2, 0}, {7, 24}},
    FontSize->16],
  
  Cell[StyleData["Section", "EnhancedPrintout"],
    CellMargins->{{2, 0}, {7, 24}},
    FontFamily->"Futura",
    FontSize->16]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Subsection"],
    CellDingbat->"\[FilledSquare]",
    CellMargins->{{24, Inherited}, {8, 30}},
    CellGroupingRules->{"SectionGrouping", 40},
    PageBreakBelow->False,
    CounterIncrements->"Subsection",
    CounterAssignments->{{"Subsubsection", 0}},
    FontFamily->"Helvetica",
    FontSize->14,
    FontWeight->"Bold"],
  
  Cell[StyleData["Subsection", "Printout"],
    CellMargins->{{9, 0}, {7, 24}},
    FontSize->12],
  
  Cell[StyleData["Subsection", "EnhancedPrintout"],
    CellMargins->{{9, 0}, {7, 24}},
    FontFamily->"Futura",
    FontSize->14]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Subsubsection"],
    CellMargins->{{12, Inherited}, {8, 24}},
    CellGroupingRules->{"SectionGrouping", 50},
    PageBreakBelow->False,
    CounterIncrements->"Subsubsection",
    FontFamily->"Helvetica",
    FontWeight->"Bold"],
  
  Cell[StyleData["Subsubsection", "Printout"],
    CellMargins->{{2, 0}, {7, 18}},
    FontSize->10],
  
  Cell[StyleData["Subsubsection", "EnhancedPrintout"],
    CellMargins->{{2, 0}, {7, 18}},
    FontFamily->"Futura",
    FontSize->10]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Body Text", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Text"],
    CellMargins->{{12, 10}, {7, 7}},
    LineSpacing->{1, 3},
    ParagraphSpacing->{0.7, 0},
    CounterIncrements->"Text"],
  
  Cell[StyleData["Text", "Printout"],
    CellMargins->{{2, 2}, {6, 6}},
    TextJustification->1,
    FontSize->11],
  
  Cell[StyleData["Text", "EnhancedPrintout"],
    CellMargins->{{2, 2}, {6, 6}},
    TextJustification->1,
    FontFamily->"Palatino",
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["SmallText"],
    CellMargins->{{12, 10}, {6, 6}},
    LineSpacing->{1, 3},
    ParagraphSpacing->{0.7, 0},
    CounterIncrements->"SmallText",
    FontSize->9],
  
  Cell[StyleData["SmallText", "Printout"],
    CellMargins->{{2, 2}, {5, 5}},
    TextJustification->1,
    FontSize->7],
  
  Cell[StyleData["SmallText", "EnhancedPrintout"],
    CellMargins->{{2, 2}, {5, 5}},
    TextJustification->1,
    FontFamily->"Palatino",
    FontSize->7]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Input/Output", "Section"],
  
  Cell["\<\
The cells in this section define styles used for input and output \
to the kernel.  Be careful when modifying, renaming, or removing these \
styles, because the front end associates special meanings with these style \
names.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Input"],
    ShowCellBracket->True,
    CellMargins->{{45, 10}, {5, 7}},
    Evaluatable->True,
    CellGroupingRules->"InputGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    CellLabelMargins->{{11, Inherited}, {Inherited, Inherited}},
    DefaultFormatType->DefaultInputFormatType,
    AutoItalicWords->{},
    FormatType->InputForm,
    ShowStringCharacters->True,
    NumberMarks->True,
    CounterIncrements->"Input",
    FontWeight->"Bold"],
  
  Cell[StyleData["Input", "Printout"],
    ShowCellBracket->False,
    CellMargins->{{39, 0}, {4, 6}},
    FontSize->9],
  
  Cell[StyleData["Input", "EnhancedPrintout"],
    ShowCellBracket->False,
    CellMargins->{{39, 0}, {4, 6}},
    FontSize->9]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Output"],
    ShowCellBracket->True,
    CellMargins->{{46, 10}, {7, 5}},
    CellEditDuplicate->True,
    CellGroupingRules->"OutputGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    GeneratedCell->True,
    CellAutoOverwrite->True,
    CellLabelMargins->{{11, Inherited}, {Inherited, Inherited}},
    DefaultFormatType->DefaultOutputFormatType,
    AutoItalicWords->{},
    FormatType->InputForm,
    CounterIncrements->"Output"],
  
  Cell[StyleData["Output", "Printout"],
    ShowCellBracket->False,
    CellMargins->{{39, 0}, {6, 4}},
    FontSize->9],
  
  Cell[StyleData["Output", "EnhancedPrintout"],
    ShowCellBracket->False,
    CellMargins->{{39, 0}, {6, 4}},
    FontSize->9]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Message"],
    ShowCellBracket->True,
    CellMargins->{{45, Inherited}, {Inherited, Inherited}},
    CellGroupingRules->"OutputGrouping",
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    GeneratedCell->True,
    CellAutoOverwrite->True,
    ShowCellLabel->False,
    CellLabelMargins->{{11, Inherited}, {Inherited, Inherited}},
    DefaultFormatType->DefaultOutputFormatType,
    AutoItalicWords->{},
    FormatType->InputForm,
    CounterIncrements->"Message",
    StyleMenuListing->None,
    FontSlant->"Italic",
    FontColor->RGBColor[1, 0, 0]],
  
  Cell[StyleData["Message", "Printout"],
    ShowCellBracket->False,
    CellMargins->{{39, Inherited}, {Inherited, Inherited}},
    FontSize->9,
    FontColor->GrayLevel[0]],
  
  Cell[StyleData["Message", "EnhancedPrintout"],
    ShowCellBracket->False,
    CellMargins->{{39, Inherited}, {Inherited, Inherited}},
    FontFamily->"Courier",
    FontSize->9,
    FontColor->GrayLevel[0]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Print"],
    ShowCellBracket->True,
    CellMargins->{{75, Inherited}, {Inherited, Inherited}},
    CellGroupingRules->"OutputGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    GeneratedCell->True,
    CellAutoOverwrite->True,
    ShowCellLabel->False,
    CellLabelMargins->{{11, Inherited}, {Inherited, Inherited}},
    DefaultFormatType->DefaultOutputFormatType,
    AutoItalicWords->{},
    FormatType->InputForm,
    CounterIncrements->"Print",
    StyleMenuListing->None],
  
  Cell[StyleData["Print", "Printout"],
    ShowCellBracket->False,
    CellMargins->{{57, Inherited}, {Inherited, Inherited}},
    FontSize->9],
  
  Cell[StyleData["Print", "EnhancedPrintout"],
    ShowCellBracket->False,
    CellMargins->{{57, Inherited}, {Inherited, Inherited}},
    FontFamily->"Courier",
    FontSize->9]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Graphics"],
    ShowCellBracket->True,
    CellMargins->{{47, Inherited}, {Inherited, Inherited}},
    CellGroupingRules->"GraphicsGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GeneratedCell->True,
    CellAutoOverwrite->True,
    ShowCellLabel->False,
    DefaultFormatType->DefaultOutputFormatType,
    FormatType->InputForm,
    CounterIncrements->"Graphics",
    StyleMenuListing->None],
  
  Cell[StyleData["Graphics", "Printout"],
    ShowCellBracket->False,
    CellMargins->{{39, Inherited}, {Inherited, Inherited}},
    ImageSize->{250, 250},
    FontSize->9],
  
  Cell[StyleData["Graphics", "EnhancedPrintout"],
    ShowCellBracket->False,
    CellMargins->{{39, Inherited}, {Inherited, Inherited}},
    ImageSize->{250, 250},
    FontFamily->"Courier",
    FontSize->9]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["CellLabel"],
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->9,
    FontColor->RGBColor[0, 0, 1]],
  
  Cell[StyleData["CellLabel", "Printout"],
    FontFamily->"Courier",
    FontSize->8,
    FontSlant->"Italic",
    FontColor->GrayLevel[0]],
  
  Cell[StyleData["CellLabel", "EnhancedPrintout"],
    FontFamily->"Courier",
    FontSize->8,
    FontSlant->"Italic",
    FontColor->GrayLevel[0]]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Inline Formatting", "Section"],
  
  Cell["\<\
These styles are for modifying individual words or letters in a \
cell exclusive of the cell tag.\
\>", "Text"],
  
  Cell[StyleData["RM"],
    StyleMenuListing->None,
    FontWeight->"Plain",
    FontSlant->"Plain"],
  
  Cell[StyleData["BF"],
    StyleMenuListing->None,
    FontWeight->"Bold"],
  
  Cell[StyleData["IT"],
    StyleMenuListing->None,
    FontSlant->"Italic"],
  
  Cell[StyleData["TR"],
    StyleMenuListing->None,
    FontFamily->"Times",
    FontWeight->"Plain",
    FontSlant->"Plain"],
  
  Cell[StyleData["TI"],
    StyleMenuListing->None,
    FontFamily->"Times",
    FontWeight->"Plain",
    FontSlant->"Italic"],
  
  Cell[StyleData["TB"],
    StyleMenuListing->None,
    FontFamily->"Times",
    FontWeight->"Bold",
    FontSlant->"Plain"],
  
  Cell[StyleData["TBI"],
    StyleMenuListing->None,
    FontFamily->"Times",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  
  Cell[StyleData["MR"],
    StyleMenuListing->None,
    FontFamily->"Courier",
    FontWeight->"Plain",
    FontSlant->"Plain"],
  
  Cell[StyleData["MO"],
    StyleMenuListing->None,
    FontFamily->"Courier",
    FontWeight->"Plain",
    FontSlant->"Italic"],
  
  Cell[StyleData["MB"],
    StyleMenuListing->None,
    FontFamily->"Courier",
    FontWeight->"Bold",
    FontSlant->"Plain"],
  
  Cell[StyleData["MBO"],
    StyleMenuListing->None,
    FontFamily->"Courier",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  
  Cell[StyleData["SR"],
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontWeight->"Plain",
    FontSlant->"Plain"],
  
  Cell[StyleData["SO"],
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontWeight->"Plain",
    FontSlant->"Italic"],
  
  Cell[StyleData["SB"],
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontWeight->"Bold",
    FontSlant->"Plain"],
  
  Cell[StyleData["SBO"],
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["SO10"],
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->10,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  
  Cell[StyleData["SO10", "Printout"],
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->7,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  
  Cell[StyleData["SO10", "EnhancedPrintout"],
    StyleMenuListing->None,
    FontFamily->"Futura",
    FontSize->7,
    FontWeight->"Plain",
    FontSlant->"Italic"]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Miscellaneous Styles", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["MathCaption"],
    CellDingbat->"\[FilledSmallSquare]",
    CellMargins->{{46, 42}, {0, 14}},
    PageBreakBelow->False,
    LineSpacing->{1, 1},
    ParagraphSpacing->{0, 8},
    FontColor->RGBColor[0, 0, 0.6]],
  
  Cell[StyleData["MathCaption", "Printout"],
    CellMargins->{{42, 42}, {0, 14}},
    FontSize->10,
    FontColor->GrayLevel[0]],
  
  Cell[StyleData["MathCaption", "EnhancedPrintout"],
    CellMargins->{{42, 42}, {0, 14}},
    FontFamily->"Palatino",
    FontSize->10,
    FontColor->GrayLevel[0]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Commentary"],
    CellFrame->{{2, 0}, {0, 0}},
    CellMargins->{{36, 10}, {7, 7}},
    PageBreakBelow->False,
    CellFrameMargins->8,
    CellFrameColor->RGBColor[0, 0.2, 1],
    LineSpacing->{1, 3},
    ParagraphSpacing->{0, 8},
    FontSlant->"Italic"],
  
  Cell[StyleData["Commentary", "Printout"],
    CellMargins->{{36, 0}, {6, 6}},
    CellFrameColor->GrayLevel[0.8],
    FontSize->10],
  
  Cell[StyleData["Commentary", "EnhancedPrintout"],
    CellMargins->{{36, 0}, {6, 6}},
    CellFrameColor->GrayLevel[0.8],
    FontFamily->"Palatino",
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["ItemizedText"],
    CellMargins->{{20, 4}, {0, 4}},
    LineSpacing->{1, 3},
    ParagraphSpacing->{0, 4},
    ParagraphIndent->-21,
    CounterIncrements->"ItemizedText"],
  
  Cell[StyleData["ItemizedText", "Printout"],
    ParagraphIndent->-18,
    FontSize->11],
  
  Cell[StyleData["ItemizedText", "EnhancedPrintout"],
    ParagraphIndent->-18,
    FontFamily->"Palatino",
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["ItemizedTextNote"],
    CellMargins->{{41, 4}, {0, 4}},
    LineSpacing->{1, 3},
    ParagraphSpacing->{0, 4},
    CounterIncrements->"Text"],
  
  Cell[StyleData["ItemizedTextNote", "Printout"],
    CellMargins->{{38, 4}, {0, 4}},
    FontSize->11],
  
  Cell[StyleData["ItemizedTextNote", "EnhancedPrintout"],
    CellMargins->{{38, 4}, {0, 4}},
    FontFamily->"Palatino",
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["IndentedText"],
    CellMargins->{{20, 4}, {0, 6}},
    LineSpacing->{1, 3},
    ParagraphSpacing->{0, 8},
    CounterIncrements->"Text"],
  
  Cell[StyleData["IndentedText", "Printout"],
    FontSize->11],
  
  Cell[StyleData["IndentedText", "EnhancedPrintout"],
    FontFamily->"Palatino",
    FontSize->10]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Emphasis Boxes and Pictures", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["DefinitionBox"],
    CellFrame->0.5,
    CellMargins->{{10, 4}, {0, 8}},
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    AutoIndent->False,
    AutoSpacing->False,
    LineIndent->0,
    StyleMenuListing->None,
    FontWeight->"Plain",
    Background->RGBColor[1, 0.6, 0.6],
    GridBoxOptions->{RowSpacings->1.5,
    ColumnSpacings->1,
    ColumnWidths->{0.3, 0.7},
    ColumnAlignments->{Left}}],
  
  Cell[StyleData["DefinitionBox", "Printout"],
    CellMargins->{{2, 4}, {0, 8}},
    FontSize->10,
    Background->GrayLevel[1]],
  
  Cell[StyleData["DefinitionBox", "EnhancedPrintout"],
    CellMargins->{{2, 4}, {0, 8}},
    FontFamily->"Palatino",
    FontSize->10,
    Background->GrayLevel[1]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["DefinitionBox3Col"],
    CellFrame->0.5,
    CellMargins->{{10, 4}, {0, 8}},
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    AutoIndent->False,
    AutoSpacing->False,
    LineIndent->0,
    StyleMenuListing->None,
    FontWeight->"Plain",
    Background->RGBColor[1, 0.6, 0.6],
    GridBoxOptions->{RowSpacings->1.5,
    ColumnSpacings->1,
    ColumnWidths->{0.2, 0.3, 0.5},
    ColumnAlignments->{Left}}],
  
  Cell[StyleData["DefinitionBox3Col", "Printout"],
    CellMargins->{{2, 4}, {0, 8}},
    FontSize->10,
    Background->GrayLevel[1]],
  
  Cell[StyleData["DefinitionBox3Col", "EnhancedPrintout"],
    CellMargins->{{2, 4}, {0, 8}},
    FontFamily->"Palatino",
    FontSize->10,
    Background->GrayLevel[1]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["DefinitionBox4Col"],
    CellFrame->0.5,
    CellMargins->{{10, 4}, {0, 8}},
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    AutoIndent->False,
    AutoSpacing->False,
    LineIndent->0,
    StyleMenuListing->None,
    FontWeight->"Plain",
    Background->RGBColor[1, 0.6, 0.6],
    GridBoxOptions->{RowSpacings->1.5,
    ColumnSpacings->1,
    ColumnWidths->{0.15, 0.35, 0.15, 0.35},
    ColumnAlignments->{Left}}],
  
  Cell[StyleData["DefinitionBox4Col", "Printout"],
    CellMargins->{{2, 4}, {0, 8}},
    FontSize->10,
    Background->GrayLevel[1]],
  
  Cell[StyleData["DefinitionBox4Col", "EnhancedPrintout"],
    CellMargins->{{2, 4}, {0, 8}},
    FontFamily->"Palatino",
    FontSize->10,
    Background->GrayLevel[1]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["DefinitionBox5Col"],
    CellFrame->0.5,
    CellMargins->{{10, 4}, {0, 8}},
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    AutoIndent->False,
    AutoSpacing->False,
    LineIndent->0,
    StyleMenuListing->None,
    FontWeight->"Plain",
    Background->RGBColor[1, 0.6, 0.6],
    GridBoxOptions->{RowSpacings->1.5,
    ColumnSpacings->1,
    ColumnWidths->0.2,
    ColumnAlignments->{Left}}],
  
  Cell[StyleData["DefinitionBox5Col", "Printout"],
    CellMargins->{{2, 4}, {0, 8}},
    FontSize->10,
    Background->GrayLevel[1]],
  
  Cell[StyleData["DefinitionBox5Col", "EnhancedPrintout"],
    CellMargins->{{2, 4}, {0, 8}},
    FontFamily->"Palatino",
    FontSize->10,
    Background->GrayLevel[1]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["DefinitionBox6Col"],
    CellFrame->0.5,
    CellMargins->{{10, 4}, {0, 8}},
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    AutoIndent->False,
    AutoSpacing->False,
    LineIndent->0,
    StyleMenuListing->None,
    FontWeight->"Plain",
    Background->RGBColor[1, 0.6, 0.6],
    GridBoxOptions->{RowSpacings->1.5,
    ColumnSpacings->1,
    ColumnWidths->{0.13, 0.24, 0.13, 0.13, 0.24, 0.13},
    ColumnAlignments->{Left}}],
  
  Cell[StyleData["DefinitionBox6Col", "Printout"],
    CellMargins->{{2, 4}, {0, 8}},
    FontSize->10,
    Background->GrayLevel[1]],
  
  Cell[StyleData["DefinitionBox6Col", "EnhancedPrintout"],
    CellMargins->{{2, 4}, {0, 8}},
    FontFamily->"Palatino",
    FontSize->10,
    Background->GrayLevel[1]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["TopBox"],
    CellFrame->{{0.5, 0.5}, {0, 0.5}},
    CellMargins->{{11, 4}, {0, 8}},
    CellHorizontalScrolling->True,
    PageBreakBelow->False,
    AutoIndent->False,
    AutoSpacing->False,
    LineIndent->0,
    StyleMenuListing->None,
    FontWeight->"Plain",
    Background->RGBColor[1, 0.6, 0.6],
    GridBoxOptions->{RowSpacings->1.5,
    ColumnSpacings->1,
    ColumnWidths->{0.31, 0.62},
    ColumnAlignments->{Left}}],
  
  Cell[StyleData["TopBox", "Printout"],
    CellMargins->{{2, 0}, {0, 8}},
    Background->GrayLevel[1]],
  
  Cell[StyleData["TopBox", "EnhancedPrintout"],
    CellMargins->{{2, 0}, {0, 8}},
    FontFamily->"Palatino",
    Background->GrayLevel[1]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["MiddleBox"],
    CellFrame->{{0.5, 0.5}, {0, 0}},
    CellMargins->{{11, 4}, {0, -7}},
    CellHorizontalScrolling->True,
    PageBreakAbove->False,
    PageBreakBelow->False,
    AutoIndent->False,
    AutoSpacing->False,
    LineIndent->0,
    StyleMenuListing->None,
    FontWeight->"Plain",
    Background->RGBColor[1, 0.6, 0.6],
    GridBoxOptions->{RowSpacings->1.5,
    ColumnSpacings->1,
    ColumnWidths->{0.31, 0.62},
    ColumnAlignments->{Left}}],
  
  Cell[StyleData["MiddleBox", "Printout"],
    CellMargins->{{2, 0}, {0, 2}},
    Background->GrayLevel[1]],
  
  Cell[StyleData["MiddleBox", "EnhancedPrintout"],
    CellMargins->{{2, 0}, {0, 4}},
    FontFamily->"Palatino",
    Background->GrayLevel[1]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["MiddleSpacedBox"],
    CellFrame->{{0.5, 0.5}, {0, 0}},
    CellMargins->{{11, 4}, {0, -7}},
    CellHorizontalScrolling->True,
    PageBreakAbove->False,
    PageBreakBelow->False,
    AutoIndent->False,
    AutoSpacing->False,
    LineIndent->0,
    StyleMenuListing->None,
    FontWeight->"Plain",
    Background->RGBColor[1, 0.6, 0.6],
    GridBoxOptions->{RowSpacings->1.5,
    ColumnSpacings->1,
    ColumnWidths->{0.31, 0.62},
    ColumnAlignments->{Left},
    RowMinHeight->1.2}],
  
  Cell[StyleData["MiddleSpacedBox", "Printout"],
    CellMargins->{{2, 0}, {0, 0}},
    Background->GrayLevel[1],
    GridBoxOptions->{RowMinHeight->1.8}],
  
  Cell[StyleData["MiddleSpacedBox", "EnhancedPrintout"],
    CellMargins->{{2, 0}, {0, 0}},
    FontFamily->"Palatino",
    Background->GrayLevel[1],
    GridBoxOptions->{RowMinHeight->1.8}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["BottomBox"],
    CellFrame->{{0.5, 0.5}, {0.5, 0}},
    CellMargins->{{11, 4}, {0, -7}},
    CellHorizontalScrolling->True,
    PageBreakAbove->False,
    PageBreakBelow->False,
    AutoIndent->False,
    AutoSpacing->False,
    LineIndent->0,
    StyleMenuListing->None,
    FontWeight->"Plain",
    Background->RGBColor[1, 0.6, 0.6],
    GridBoxOptions->{RowSpacings->1.5,
    ColumnSpacings->1,
    ColumnWidths->{0.31, 0.62},
    ColumnAlignments->{Left},
    RowMinHeight->1.2}],
  
  Cell[StyleData["BottomBox", "Printout"],
    CellMargins->{{2, 0}, {0, -5}},
    FontSize->10,
    Background->GrayLevel[1],
    GridBoxOptions->{RowMinHeight->2.2}],
  
  Cell[StyleData["BottomBox", "EnhancedPrintout"],
    CellMargins->{{2, 0}, {0, -5}},
    FontFamily->"Palatino",
    FontSize->10,
    Background->GrayLevel[1],
    GridBoxOptions->{RowMinHeight->2.2}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["TopSpanBox"],
    CellFrame->{{0.5, 0.5}, {0, 0.5}},
    CellMargins->{{11, 4}, {-2, 8}},
    CellHorizontalScrolling->True,
    PageBreakBelow->False,
    AutoIndent->False,
    AutoSpacing->False,
    LineIndent->0,
    StyleMenuListing->None,
    FontWeight->"Plain",
    Background->RGBColor[1, 0.6, 0.6],
    GridBoxOptions->{RowSpacings->1.5,
    ColumnSpacings->1,
    ColumnWidths->{0.9, 0.03},
    ColumnAlignments->{Left}}],
  
  Cell[StyleData["TopSpanBox", "Printout"],
    CellMargins->{{2, 0}, {-2, 8}},
    FontSize->10,
    Background->GrayLevel[1]],
  
  Cell[StyleData["TopSpanBox", "EnhancedPrintout"],
    CellMargins->{{2, 0}, {-4, 8}},
    FontFamily->"Palatino",
    FontSize->10,
    Background->GrayLevel[1]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["MiddleSpanBox"],
    CellFrame->{{0.5, 0.5}, {0, 0}},
    CellMargins->{{11, 4}, {-3, -8}},
    CellHorizontalScrolling->True,
    PageBreakAbove->False,
    PageBreakBelow->False,
    AutoIndent->False,
    AutoSpacing->False,
    LineIndent->0,
    StyleMenuListing->None,
    FontWeight->"Plain",
    Background->RGBColor[1, 0.6, 0.6],
    GridBoxOptions->{RowSpacings->1.5,
    ColumnSpacings->1,
    ColumnWidths->{0.9, 0.03},
    ColumnAlignments->{Left},
    RowMinHeight->1.2}],
  
  Cell[StyleData["MiddleSpanBox", "Printout"],
    CellMargins->{{2, 0}, {-5, 0}},
    FontSize->10,
    Background->GrayLevel[1],
    GridBoxOptions->{RowMinHeight->1.8}],
  
  Cell[StyleData["MiddleSpanBox", "EnhancedPrintout"],
    CellMargins->{{2, 0}, {-7, 0}},
    FontFamily->"Palatino",
    FontSize->10,
    Background->GrayLevel[1],
    GridBoxOptions->{RowMinHeight->1.8}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Picture"],
    CellMargins->{{12, Inherited}, {0, 8}},
    CellHorizontalScrolling->True],
  
  Cell[StyleData["Picture", "Printout"],
    CellMargins->{{2, Inherited}, {0, 8}}],
  
  Cell[StyleData["Picture", "EnhancedPrintout"],
    CellMargins->{{2, Inherited}, {0, 8}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Caption"],
    CellMargins->{{12, 4}, {0, 2}},
    PageBreakAbove->False,
    FontFamily->"Helvetica",
    FontSize->9],
  
  Cell[StyleData["Caption", "Printout"],
    CellMargins->{{2, 4}, {2, 4}},
    FontSize->7],
  
  Cell[StyleData["Caption", "EnhancedPrintout"],
    CellMargins->{{2, 4}, {2, 4}},
    FontFamily->"Futura",
    FontSize->7]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Tables", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["SingleRowTable"],
    CellMargins->{{10, 4}, {0, 8}},
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    AutoIndent->False,
    AutoSpacing->False,
    LineIndent->0,
    StyleMenuListing->None,
    GridBoxOptions->{RowSpacings->1.5,
    ColumnSpacings->2,
    ColumnAlignments->{Left}}],
  
  Cell[StyleData["SingleRowTable", "Printout"],
    CellMargins->{{2, 0}, {0, 8}}],
  
  Cell[StyleData["SingleRowTable", "EnhancedPrintout"],
    CellMargins->{{2, 0}, {0, 8}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["2ColumnTable"],
    CellMargins->{{10, 4}, {0, 8}},
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    AutoIndent->False,
    AutoSpacing->False,
    LineIndent->0,
    StyleMenuListing->None,
    GridBoxOptions->{RowSpacings->1.5,
    ColumnSpacings->2,
    ColumnWidths->{0.34, 0.64},
    ColumnAlignments->{Left, Center}}],
  
  Cell[StyleData["2ColumnTable", "Printout"],
    CellMargins->{{2, 0}, {0, 8}}],
  
  Cell[StyleData["2ColumnTable", "EnhancedPrintout"],
    CellMargins->{{2, 0}, {0, 8}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["2ColumnEvenTable"],
    CellMargins->{{10, 4}, {0, 8}},
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    AutoIndent->False,
    AutoSpacing->False,
    LineIndent->0,
    StyleMenuListing->None,
    GridBoxOptions->{RowSpacings->1.5,
    ColumnSpacings->2,
    ColumnWidths->0.49,
    ColumnAlignments->{Left, Center}}],
  
  Cell[StyleData["2ColumnEvenTable", "Printout"],
    CellMargins->{{2, 0}, {0, 8}}],
  
  Cell[StyleData["2ColumnEvenTable", "EnhancedPrintout"],
    CellMargins->{{2, 0}, {0, 8}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["3ColumnTable"],
    CellMargins->{{10, 4}, {0, 8}},
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    AutoIndent->False,
    AutoSpacing->False,
    LineIndent->0,
    StyleMenuListing->None,
    GridBoxOptions->{RowSpacings->1.5,
    ColumnSpacings->2,
    ColumnWidths->{0.28, 0.28, 0.43},
    ColumnAlignments->{Left, Center}}],
  
  Cell[StyleData["3ColumnTable", "Printout"],
    CellMargins->{{2, 0}, {0, 8}}],
  
  Cell[StyleData["3ColumnTable", "EnhancedPrintout"],
    CellMargins->{{2, 0}, {0, 8}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["4ColumnTable"],
    CellMargins->{{10, 4}, {0, 8}},
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    AutoIndent->False,
    AutoSpacing->False,
    LineIndent->0,
    StyleMenuListing->None,
    GridBoxOptions->{RowSpacings->1.5,
    ColumnSpacings->2,
    ColumnWidths->0.25,
    ColumnAlignments->{Left, Center}}],
  
  Cell[StyleData["4ColumnTable", "Printout"],
    CellMargins->{{2, 0}, {0, 8}}],
  
  Cell[StyleData["4ColumnTable", "EnhancedPrintout"],
    CellMargins->{{2, 0}, {0, 8}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["5ColumnTable"],
    CellMargins->{{10, 4}, {0, 8}},
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    AutoIndent->False,
    AutoSpacing->False,
    LineIndent->0,
    StyleMenuListing->None,
    GridBoxOptions->{RowSpacings->1.5,
    ColumnSpacings->2,
    ColumnWidths->0.2,
    ColumnAlignments->{Left, Center}}],
  
  Cell[StyleData["5ColumnTable", "Printout"],
    CellMargins->{{2, 0}, {0, 8}}],
  
  Cell[StyleData["5ColumnTable", "EnhancedPrintout"],
    CellMargins->{{2, 0}, {0, 8}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["6ColumnTable"],
    CellMargins->{{10, 4}, {0, 8}},
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    AutoIndent->False,
    AutoSpacing->False,
    LineIndent->0,
    StyleMenuListing->None,
    GridBoxOptions->{RowSpacings->1.5,
    ColumnSpacings->2,
    ColumnWidths->0.16,
    ColumnAlignments->{Left, Center}}],
  
  Cell[StyleData["6ColumnTable", "Printout"],
    CellMargins->{{2, 0}, {0, 8}}],
  
  Cell[StyleData["6ColumnTable", "EnhancedPrintout"],
    CellMargins->{{2, 0}, {0, 8}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["7ColumnTable"],
    CellMargins->{{10, 4}, {0, 8}},
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    AutoIndent->False,
    AutoSpacing->False,
    LineIndent->0,
    StyleMenuListing->None,
    GridBoxOptions->{RowSpacings->1.5,
    ColumnSpacings->2,
    ColumnWidths->0.14,
    ColumnAlignments->{Left, Center}}],
  
  Cell[StyleData["7ColumnTable", "Printout"],
    CellMargins->{{2, 0}, {0, 8}}],
  
  Cell[StyleData["7ColumnTable", "EnhancedPrintout"],
    CellMargins->{{2, 0}, {0, 8}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["8ColumnTable"],
    CellMargins->{{10, 4}, {0, 8}},
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    AutoIndent->False,
    AutoSpacing->False,
    LineIndent->0,
    StyleMenuListing->None,
    GridBoxOptions->{RowSpacings->1.5,
    ColumnSpacings->2,
    ColumnWidths->0.12,
    ColumnAlignments->{Left, Center}}],
  
  Cell[StyleData["8ColumnTable", "Printout"],
    CellMargins->{{2, 0}, {0, 8}}],
  
  Cell[StyleData["8ColumnTable", "EnhancedPrintout"],
    CellMargins->{{2, 0}, {0, 8}}]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Automatic Numbering", "Section"],
  
  Cell["\<\
The following styles are useful for numbered equations, figures, \
etc.  They automatically give the cell a FrameLabel containing a reference to \
a particular counter, and also increment that counter.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["NumberedEquation"],
    CellMargins->{{45, 10}, {7, 7}},
    CellFrameLabels->{{None, Cell[ 
            TextData[ {"(", 
              CounterBox[ "NumberedEquation"], ")"}]]}, {None, None}},
    DefaultFormatType->DefaultInputFormatType,
    CounterIncrements->"NumberedEquation",
    FormatTypeAutoConvert->False],
  
  Cell[StyleData["NumberedEquation", "Printout"],
    CellMargins->{{39, 55}, {0, 10}},
    FontSize->10],
  
  Cell[StyleData["NumberedEquation", "EnhancedPrintout"],
    CellMargins->{{39, 55}, {0, 10}},
    FontFamily->"Palatino",
    FontSize->10]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Formulas and Programming", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["DisplayFormula"],
    CellMargins->{{45, Inherited}, {7, 7}},
    CellHorizontalScrolling->True,
    ScriptLevel->0,
    SingleLetterItalics->True,
    UnderoverscriptBoxOptions->{LimitsPositioning->True}],
  
  Cell[StyleData["DisplayFormula", "Printout"]],
  
  Cell[StyleData["DisplayFormula", "EnhancedPrintout"],
    FontFamily->"Palatino"]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["DisplayMath"],
    CellMargins->{{45, 10}, {7, 7}},
    Evaluatable->False,
    CellGroupingRules->"InputGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    DefaultFormatType->DefaultInputFormatType,
    AutoItalicWords->{},
    FormatType->InputForm,
    ShowStringCharacters->True,
    NumberMarks->True,
    CounterIncrements->"Input"],
  
  Cell[StyleData["DisplayMath", "Printout"],
    CellMargins->{{39, 0}, {6, 6}},
    FontSize->9],
  
  Cell[StyleData["DisplayMath", "EnhancedPrintout"],
    CellMargins->{{39, 0}, {4, 6}},
    FontFamily->"WRICourier",
    FontSize->9]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["EquationNumber"],
    LineSpacing->{1.83, 0}],
  
  Cell[StyleData["EquationNumber", "Printout"],
    LineSpacing->{1.7, 0}],
  
  Cell[StyleData["EquationNumber", "EnhancedPrintout"],
    LineSpacing->{1.7, 0},
    FontFamily->"Palatino"]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Program"],
    CellFrame->{{0, 0}, {0.5, 0.5}},
    CellMargins->{{12, Inherited}, {7, 7}},
    CellFrameMargins->{{0, 0}, {7, 5}},
    FontFamily->"Courier"],
  
  Cell[StyleData["Program", "Printout"],
    CellMargins->{{18, 30}, {Inherited, 4}},
    FontSize->9.5],
  
  Cell[StyleData["Program", "EnhancedPrintout"],
    CellMargins->{{18, 30}, {Inherited, 4}},
    FontFamily->"WRICourier",
    FontSize->9.5]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Contents and Index", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["ContentsTitle"],
    CellMargins->{{10, 4}, {0, 18}},
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->26,
    FontWeight->"Bold"],
  
  Cell[StyleData["ContentsTitle", "Printout"],
    CellMargins->{{2, 0}, {0, 18}},
    PageBreakBelow->False,
    FontSize->18],
  
  Cell[StyleData["ContentsTitle", "EnhancedPrintout"],
    CellMargins->{{2, 0}, {0, 18}},
    PageBreakBelow->False,
    FontFamily->"Futura",
    FontSize->18]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["ContentsSection"],
    CellMargins->{{20, 4}, {3, 18}},
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Bold"],
  
  Cell[StyleData["ContentsSection", "Printout"],
    CellMargins->{{12, 0}, {3, 18}},
    PageBreakBelow->False,
    FontSize->11],
  
  Cell[StyleData["ContentsSection", "EnhancedPrintout"],
    CellMargins->{{12, 0}, {3, 18}},
    PageBreakBelow->False,
    FontFamily->"Futura",
    FontSize->11]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Contents"],
    CellMargins->{{21, 4}, {0, 8}},
    StyleMenuListing->None],
  
  Cell[StyleData["Contents", "Printout"],
    CellMargins->{{13, 0}, {0, 8}}],
  
  Cell[StyleData["Contents", "EnhancedPrintout"],
    CellMargins->{{13, 0}, {0, 8}},
    FontFamily->"Palatino",
    FontSize->11]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Index"],
    CellMargins->{{21, 4}, {0, 0}},
    ParagraphIndent->-48,
    StyleMenuListing->None],
  
  Cell[StyleData["Index", "Printout"],
    CellMargins->{{13, 0}, {0, 0}},
    FontSize->10],
  
  Cell[StyleData["Index", "EnhancedPrintout"],
    CellMargins->{{13, 0}, {0, 0}},
    FontFamily->"Palatino",
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["IndexSubentry"],
    CellMargins->{{36, 4}, {0, 0}},
    ParagraphIndent->-48,
    StyleMenuListing->None],
  
  Cell[StyleData["IndexSubentry", "Printout"],
    CellMargins->{{24, 0}, {0, 0}},
    FontSize->10],
  
  Cell[StyleData["IndexSubentry", "EnhancedPrintout"],
    CellMargins->{{24, 0}, {0, 0}},
    FontFamily->"Palatino",
    FontSize->10]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Browser Emulation", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["TextNormal"],
    CellMargins->{{12, 10}, {7, 7}},
    LineSpacing->{1, 3},
    CounterIncrements->"Text",
    FontFamily->"Times",
    FontSize->12],
  
  Cell[StyleData["TextNormal", "Printout"],
    CellMargins->{{2, 2}, {6, 6}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["BrowserEmulation"],
    CellMargins->{{12, 1}, {7, 24}},
    CellGroupingRules->{"SectionGrouping", 70},
    LineSpacing->{1, 3},
    FontFamily->"Helvetica",
    GridBoxOptions->{ColumnWidths->0.25,
    ColumnAlignments->{Left},
    GridFrame->True,
    GridFrameMargins->0.5,
    ColumnLines->True}],
  
  Cell[StyleData["Text", "Printout"],
    CellMargins->{{2, 2}, {6, 6}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["CategoryDefs"],
    CellMargins->{{300, 10}, {7, 7}},
    LineSpacing->{1, 3},
    FontFamily->"Helvetica",
    Background->RGBColor[1, 0.8, 0.6],
    GridBoxOptions->{ColumnAlignments->{Left},
    GridFrame->False,
    RowLines->True,
    ColumnLines->True}],
  
  Cell[StyleData["Text", "Printout"],
    CellMargins->{{2, 2}, {6, 6}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Category1"],
    ShowCellBracket->False,
    ShowGroupOpenCloseIcon->True,
    CellMargins->{{12, 10}, {7, 7}},
    CellGroupingRules->{"SectionGrouping", 50},
    FontSize->16,
    FontWeight->"Bold"],
  
  Cell[StyleData["Category1", "Printout"],
    CellMargins->{{2, 2}, {6, 6}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Category2"],
    ShowCellBracket->False,
    ShowGroupOpenCloseIcon->True,
    CellMargins->{{85, 10}, {7, 7}},
    CellGroupingRules->{"SectionGrouping", 60},
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 0.500008]],
  
  Cell[StyleData["Category2", "Printout"],
    CellMargins->{{2, 2}, {6, 6}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Category3"],
    ShowCellBracket->False,
    ShowGroupOpenCloseIcon->True,
    CellMargins->{{157, 10}, {7, 7}},
    CellGroupingRules->{"SectionGrouping", 70},
    FontSize->12,
    FontWeight->"Bold",
    FontColor->RGBColor[0.2, 0, 0.4]],
  
  Cell[StyleData["Category3", "Printout"],
    CellMargins->{{2, 2}, {6, 6}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Category4"],
    ShowCellBracket->False,
    ShowGroupOpenCloseIcon->True,
    CellMargins->{{230, 10}, {7, 7}},
    CellGroupingRules->{"SectionGrouping", 80},
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0.300008, 0.4]],
  
  Cell[StyleData["Category4", "Printout"],
    CellMargins->{{2, 2}, {6, 6}}]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Headers and Footers", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Header"],
    CellMargins->{{0, 0}, {4, 1}},
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->8,
    FontWeight->"Bold"],
  
  Cell[StyleData["Header", "Printout"]],
  
  Cell[StyleData["Header", "EnhancedPrintout"],
    FontFamily->"Futura"]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Footer"],
    CellMargins->{{0, 0}, {0, 4}},
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->7,
    FontSlant->"Plain"],
  
  Cell[StyleData["Footer", "Printout"]],
  
  Cell[StyleData["Footer", "EnhancedPrintout"],
    FontFamily->"Futura"]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["PageNumber"],
    CellMargins->{{0, 0}, {4, 1}},
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->8,
    FontWeight->"Bold"],
  
  Cell[StyleData["PageNumber", "Printout"]],
  
  Cell[StyleData["PageNumber", "EnhancedPrintout"],
    FontFamily->"Futura"]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Hyperlink Styles", "Section"],
  
  Cell["\<\
The cells below define styles useful for making hypertext \
ButtonBoxes.  The \"Hyperlink\" style is for links within the same Notebook, \
or between Notebooks.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Hyperlink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`NotebookLocate[ #2]}]&),
    Active->True,
    ButtonNote->ButtonData}],
  
  Cell[StyleData["Hyperlink", "Printout"],
    FontSize->11,
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}],
  
  Cell[StyleData["Hyperlink", "EnhancedPrintout"],
    FontFamily->"Palatino",
    FontSize->10,
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}]
  }, Closed]],
  
  Cell["\<\
The following styles are for linking automatically to the on-line \
help system.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["MainBookLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "MainBook", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["MainBookLink", "Printout"],
    FontSize->11,
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}],
  
  Cell[StyleData["MainBookLink", "EnhancedPrintout"],
    FontFamily->"Palatino",
    FontSize->10,
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["AddOnsLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontFamily->"Courier",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "AddOns", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["AddOnsLink", "Printout"],
    FontSize->10,
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}],
  
  Cell[StyleData["AddOnsLink", "EnhancedPrintout"],
    FontFamily->"WRICourier",
    FontSize->10,
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["RefGuideLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontFamily->"Courier",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "RefGuide", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["RefGuideLink", "Printout"],
    FontSize->10,
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}],
  
  Cell[StyleData["RefGuideLink", "EnhancedPrintout"],
    FontFamily->"WRICourier",
    FontSize->10,
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["GettingStartedLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "GettingStarted", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["GettingStartedLink", "Printout"],
    FontSize->11,
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}],
  
  Cell[StyleData["GettingStartedLink", "EnhancedPrintout"],
    FontFamily->"Palatino",
    FontSize->10,
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["OtherInformationLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "OtherInformation", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["OtherInformationLink", "Printout"],
    FontSize->11,
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}],
  
  Cell[StyleData["OtherInformationLink", "EnhancedPrintout"],
    FontFamily->"Palatino",
    FontSize->10,
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}]
  }, Closed]]
  }, Closed]]
  }, Open  ]]
  }]
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "Table of Contents"->{
    Cell[2661, 90, 69, 1, 73, "Section",
      CellTags->"Table of Contents"]},
  "Short Overview"->{
    Cell[31359, 1186, 85, 2, 73, "Section",
      PageBreakAbove->True,
      CellTags->"Short Overview"]},
  "Load FeynCalc"->{
    Cell[39648, 1464, 62, 1, 73, "Section",
      CellTags->"Load FeynCalc"]},
  "Example Calculations"->{
    Cell[42850, 1586, 99, 2, 73, "Section",
      PageBreakAbove->True,
      CellTags->"Example Calculations"]},
  "Tree level"->{
    Cell[43481, 1612, 317, 10, 48, "Subsubsection",
      CellTags->"Tree level"]},
  "A moment integral"->{
    Cell[78280, 2699, 75, 1, 48, "Subsubsection",
      CellTags->"A moment integral"]},
  "Reference Guide"->{
    Cell[121787, 4129, 65, 1, 73, "Section",
      CellTags->"Reference Guide"]},
  "A0"->{
    Cell[121877, 4134, 66, 2, 56, "Subsection",
      PageBreakAbove->True,
      CellTags->"A0"]},
  "Amplitudes"->{
    Cell[125052, 4278, 58, 1, 56, "Subsection",
      CellTags->"Amplitudes"]},
  "AnomalousDimension"->{
    Cell[178215, 5583, 74, 1, 56, "Subsection",
      CellTags->"AnomalousDimension"]},
  "Anti5"->{
    Cell[221189, 6794, 48, 1, 56, "Subsection",
      CellTags->"Anti5"]},
  "AntiCommutator"->{
    Cell[227928, 7064, 66, 1, 56, "Subsection",
      CellTags->"AntiCommutator"]},
  "AntiQuarkField"->{
    Cell[233474, 7289, 66, 1, 56, "Subsection",
      CellTags->"AntiQuarkField"]},
  "AntiSymmetrize"->{
    Cell[234371, 7335, 66, 1, 56, "Subsection",
      CellTags->"AntiSymmetrize"]},
  "Apart1"->{
    Cell[236497, 7419, 50, 1, 56, "Subsection",
      CellTags->"Apart1"]},
  "Apart2"->{
    Cell[248781, 7772, 50, 1, 56, "Subsection",
      CellTags->"Apart2"]},
  "Apart3"->{
    Cell[252824, 7915, 50, 1, 56, "Subsection",
      CellTags->"Apart3"]},
  "B00"->{
    Cell[264956, 8263, 44, 1, 56, "Subsection",
      CellTags->"B00"]},
  "B0"->{
    Cell[275588, 8607, 42, 1, 56, "Subsection",
      CellTags->"B0"]},
  "B11"->{
    Cell[278990, 8751, 44, 1, 56, "Subsection",
      CellTags->"B11"]},
  "B1"->{
    Cell[289389, 9105, 42, 1, 56, "Subsection",
      CellTags->"B1"]},
  "BackgroundGluonVertex"->{
    Cell[297021, 9385, 80, 1, 56, "Subsection",
      CellTags->"BackgroundGluonVertex"]},
  "C0"->{
    Cell[328843, 10222, 42, 1, 56, "Subsection",
      CellTags->"C0"]},
  "Calc"->{
    Cell[332156, 10340, 46, 1, 56, "Subsection",
      CellTags->"Calc"]},
  "CA"->{
    Cell[338679, 10585, 42, 1, 56, "Subsection",
      CellTags->"CA"]},
  "Cases2"->{
    Cell[339777, 10646, 50, 1, 56, "Subsection",
      CellTags->"Cases2"]},
  "CF"->{
    Cell[343370, 10778, 42, 1, 56, "Subsection",
      CellTags->"CF"]},
  "ChangeDimension"->{
    Cell[344887, 10858, 68, 1, 56, "Subsection",
      CellTags->"ChangeDimension"]},
  "ChargeConjugationMatrixInv"->{
    Cell[352790, 11169, 90, 1, 56, "Subsection",
      CellTags->"ChargeConjugationMatrixInv"]},
  "ChargeConjugationMatrix"->{
    Cell[355540, 11294, 84, 1, 56, "Subsection",
      CellTags->"ChargeConjugationMatrix"]},
  "ChiralityProjector"->{
    Cell[357839, 11398, 74, 1, 56, "Subsection",
      CellTags->"ChiralityProjector"]},
  "Chisholm"->{
    Cell[360395, 11509, 54, 1, 56, "Subsection",
      CellTags->"Chisholm"]},
  "ClearScalarProducts"->{
    Cell[405242, 12836, 76, 1, 56, "Subsection",
      CellTags->"ClearScalarProducts"]},
  "Collect2"->{
    Cell[407338, 12937, 54, 1, 56, "Subsection",
      CellTags->"Collect2"]},
  "Combine"->{
    Cell[417166, 13266, 52, 1, 56, "Subsection",
      CellTags->"Combine"]},
  "CommutatorExplicit"->{
    Cell[420340, 13395, 74, 1, 56, "Subsection",
      CellTags->"CommutatorExplicit"]},
  "Commutator"->{
    Cell[423398, 13524, 58, 1, 56, "Subsection",
      CellTags->"Commutator"]},
  "ComplexConjugate"->{
    Cell[427774, 13691, 70, 1, 56, "Subsection",
      CellTags->"ComplexConjugate"]},
  "ComplexIndex"->{
    Cell[435223, 13962, 62, 1, 56, "Subsection",
      CellTags->"ComplexIndex"]},
  "Contract"->{
    Cell[437660, 14079, 54, 1, 56, "Subsection",
      CellTags->"Contract"]},
  "Convolute"->{
    Cell[450698, 14565, 56, 1, 56, "Subsection",
      CellTags->"Convolute"]},
  "CovariantD"->{
    Cell[466671, 15165, 58, 1, 56, "Subsection",
      CellTags->"CovariantD"]},
  "D0"->{
    Cell[499789, 16098, 42, 1, 56, "Subsection",
      CellTags->"D0"]},
  "DataType"->{
    Cell[504841, 16258, 54, 1, 56, "Subsection",
      CellTags->"DataType"]},
  "DeclareNonCommutative"->{
    Cell[513075, 16585, 80, 1, 56, "Subsection",
      CellTags->"DeclareNonCommutative"]},
  "DeltaFunction"->{
    Cell[515136, 16676, 64, 1, 56, "Subsection",
      CellTags->"DeltaFunction"]},
  "DeltaFunctionPrime"->{
    Cell[518036, 16801, 74, 1, 56, "Subsection",
      CellTags->"DeltaFunctionPrime"]},
  "Dimension"->{
    Cell[520325, 16899, 56, 1, 56, "Subsection",
      CellTags->"Dimension"]},
  "DiracGammaCombine"->{
    Cell[524975, 17067, 72, 1, 56, "Subsection",
      CellTags->"DiracGammaCombine"]},
  "DiracGammaExpand"->{
    Cell[528577, 17213, 70, 1, 56, "Subsection",
      CellTags->"DiracGammaExpand"]},
  "DiracGamma"->{
    Cell[534253, 17423, 58, 1, 56, "Subsection",
      CellTags->"DiracGamma"]},
  "DiracGammaT"->{
    Cell[544425, 17782, 60, 1, 56, "Subsection",
      CellTags->"DiracGammaT"]},
  "DiracMatrix"->{
    Cell[546128, 17862, 60, 1, 56, "Subsection",
      CellTags->"DiracMatrix"]},
  "DiracOrder"->{
    Cell[552743, 18138, 58, 1, 56, "Subsection",
      CellTags->"DiracOrder"]},
  "DiracReduce"->{
    Cell[560772, 18444, 60, 1, 56, "Subsection",
      CellTags->"DiracReduce"]},
  "DiracSigmaExplicit"->{
    Cell[568258, 18688, 74, 1, 56, "Subsection",
      CellTags->"DiracSigmaExplicit"]},
  "DiracSigma"->{
    Cell[570304, 18770, 58, 1, 56, "Subsection",
      CellTags->"DiracSigma"]},
  "DiracSimplify"->{
    Cell[574828, 18934, 64, 1, 56, "Subsection",
      CellTags->"DiracSimplify"]},
  "DiracSimplify2"->{
    Cell[612608, 20112, 66, 1, 56, "Subsection",
      CellTags->"DiracSimplify2"]},
  "DiracSlash"->{
    Cell[615073, 20207, 58, 1, 56, "Subsection",
      CellTags->"DiracSlash"]},
  "DiracTrace"->{
    Cell[619928, 20407, 58, 1, 56, "Subsection",
      CellTags->"DiracTrace"]},
  "DiracTrick"->{
    Cell[639737, 20969, 58, 1, 56, "Subsection",
      CellTags->"DiracTrick"]},
  "DotSimplify"->{
    Cell[675437, 21979, 60, 1, 56, "Subsection",
      CellTags->"DotSimplify"]},
  "EpsChisholm"->{
    Cell[682266, 22254, 60, 1, 56, "Subsection",
      CellTags->"EpsChisholm"]},
  "EpsEvaluate"->{
    Cell[686518, 22403, 60, 1, 56, "Subsection",
      CellTags->"EpsEvaluate"]},
  "Epsilon"->{
    Cell[690342, 22546, 52, 1, 56, "Subsection",
      CellTags->"Epsilon"]},
  "Eps"->{
    Cell[692090, 22625, 44, 1, 56, "Subsection",
      CellTags->"Eps"]},
  "ExpandPartialD"->{
    Cell[705579, 23095, 66, 1, 56, "Subsection",
      CellTags->"ExpandPartialD"]},
  "ExpandScalarProduct"->{
    Cell[718890, 23537, 76, 1, 56, "Subsection",
      CellTags->"ExpandScalarProduct"]},
  "Explicit"->{
    Cell[722736, 23694, 54, 1, 56, "Subsection",
      CellTags->"Explicit"]},
  "ExplicitPartialD"->{
    Cell[737783, 24171, 70, 1, 56, "Subsection",
      CellTags->"ExplicitPartialD"]},
  "ExplicitSUNIndex"->{
    Cell[741164, 24300, 70, 1, 56, "Subsection",
      CellTags->"ExplicitSUNIndex"]},
  "Factor2"->{
    Cell[742575, 24370, 52, 1, 56, "Subsection",
      CellTags->"Factor2"]},
  "FAD"->{
    Cell[746314, 24528, 44, 1, 56, "Subsection",
      CellTags->"FAD"]},
  "FCE"->{
    Cell[752004, 24727, 44, 1, 56, "Subsection",
      CellTags->"FCE"]},
  "FCI"->{
    Cell[758314, 24951, 44, 1, 56, "Subsection",
      CellTags->"FCI"]},
  "FermionSpinSum"->{
    Cell[763802, 25140, 66, 1, 56, "Subsection",
      CellTags->"FermionSpinSum"]},
  "FeynAmpDenominatorCombine"->{
    Cell[770267, 25356, 88, 1, 56, "Subsection",
      CellTags->"FeynAmpDenominatorCombine"]},
  "FeynAmpDenominator"->{
    Cell[772812, 25456, 74, 1, 56, "Subsection",
      CellTags->"FeynAmpDenominator"]},
  "FeynAmpDenominatorSimplify"->{
    Cell[775733, 25575, 90, 1, 56, "Subsection",
      CellTags->"FeynAmpDenominatorSimplify"]},
  "FeynAmpDenominatorSplit"->{
    Cell[785508, 25903, 84, 1, 56, "Subsection",
      CellTags->"FeynAmpDenominatorSplit"]},
  "FeynAmp"->{
    Cell[790678, 26095, 52, 1, 56, "Subsection",
      CellTags->"FeynAmp"]},
  "FeynCalc2FORM"->{
    Cell[798746, 26326, 64, 1, 56, "Subsection",
      CellTags->"FeynCalc2FORM"]},
  "FeynCalcExternal"->{
    Cell[816549, 26876, 70, 1, 56, "Subsection",
      CellTags->"FeynCalcExternal"]},
  "FeynCalcInternal"->{
    Cell[822239, 27076, 70, 1, 56, "Subsection",
      CellTags->"FeynCalcInternal"]},
  "FeynRule"->{
    Cell[831727, 27384, 54, 1, 56, "Subsection",
      CellTags->"FeynRule"]},
  "FieldStrength"->{
    Cell[912591, 29648, 64, 1, 56, "Subsection",
      CellTags->"FieldStrength"]},
  "FORM2FeynCalc"->{
    Cell[922882, 29994, 64, 1, 56, "Subsection",
      CellTags->"FORM2FeynCalc"]},
  "FourDivergence"->{
    Cell[929334, 30235, 66, 1, 56, "Subsection",
      CellTags->"FourDivergence"]},
  "FourLaplacian"->{
    Cell[931677, 30337, 64, 1, 56, "Subsection",
      CellTags->"FourLaplacian"]},
  "FourVector"->{
    Cell[940061, 30612, 58, 1, 56, "Subsection",
      CellTags->"FourVector"]},
  "FreeQ2"->{
    Cell[943515, 30761, 50, 1, 56, "Subsection",
      CellTags->"FreeQ2"]},
  "FromTFi"->{
    Cell[945350, 30847, 52, 1, 56, "Subsection",
      CellTags->"FromTFi"]},
  "FRH"->{
    Cell[952271, 31077, 44, 1, 56, "Subsection",
      CellTags->"FRH"]},
  "FunctionalD"->{
    Cell[955100, 31196, 60, 1, 56, "Subsection",
      CellTags->"FunctionalD"]},
  "FV"->{
    Cell[1129076, 36437, 42, 1, 56, "Subsection",
      CellTags->"FV"]},
  "FVD"->{
    Cell[1132680, 36601, 44, 1, 56, "Subsection",
      CellTags->"FVD"]},
  "GA"->{
    Cell[1136475, 36770, 42, 1, 56, "Subsection",
      CellTags->"GA"]},
  "GAD"->{
    Cell[1140327, 36927, 44, 1, 56, "Subsection",
      CellTags->"GAD"]},
  "GammaEpsilon"->{
    Cell[1144267, 37086, 62, 1, 56, "Subsection",
      CellTags->"GammaEpsilon"]},
  "GammaExpand"->{
    Cell[1152531, 37349, 60, 1, 56, "Subsection",
      CellTags->"GammaExpand"]},
  "GhostPropagator"->{
    Cell[1154933, 37448, 68, 1, 56, "Subsection",
      CellTags->"GhostPropagator"]},
  "GHP"->{
    Cell[1157885, 37569, 44, 1, 56, "Subsection",
      CellTags->"GHP"]},
  "GluonField"->{
    Cell[1160774, 37690, 58, 1, 56, "Subsection",
      CellTags->"GluonField"]},
  "GluonGhostVertex"->{
    Cell[1162239, 37758, 70, 1, 56, "Subsection",
      CellTags->"GluonGhostVertex"]},
  "GluonPropagator"->{
    Cell[1166209, 37904, 68, 1, 56, "Subsection",
      CellTags->"GluonPropagator"]},
  "GluonVertex"->{
    Cell[1191230, 38640, 60, 1, 56, "Subsection",
      CellTags->"GluonVertex"]},
  "GSD"->{
    Cell[1243515, 40079, 44, 1, 56, "Subsection",
      CellTags->"GSD"]},
  "GS"->{
    Cell[1247492, 40236, 42, 1, 56, "Subsection",
      CellTags->"GS"]},
  "Gstrong"->{
    Cell[1251388, 40391, 52, 1, 56, "Subsection",
      CellTags->"Gstrong"]},
  "Hill"->{
    Cell[1252281, 40439, 46, 1, 56, "Subsection",
      CellTags->"Hill"]},
  "HypergeometricAC"->{
    Cell[1257855, 40630, 70, 1, 56, "Subsection",
      CellTags->"HypergeometricAC"]},
  "HypergeometricIR"->{
    Cell[1275064, 41110, 70, 1, 56, "Subsection",
      CellTags->"HypergeometricIR"]},
  "HypergeometricSE"->{
    Cell[1279360, 41264, 70, 1, 56, "Subsection",
      CellTags->"HypergeometricSE"]},
  "HypExplicit"->{
    Cell[1282865, 41382, 60, 1, 56, "Subsection",
      CellTags->"HypExplicit"]},
  "Integrate2"->{
    Cell[1287759, 41548, 58, 1, 56, "Subsection",
      CellTags->"Integrate2"]},
  "Integrate3"->{
    Cell[1346499, 43374, 58, 1, 56, "Subsection",
      CellTags->"Integrate3"]},
  "IntegrateByParts"->{
    Cell[1353971, 43625, 70, 1, 56, "Subsection",
      CellTags->"IntegrateByParts"]},
  "InverseMellin"->{
    Cell[1355993, 43703, 64, 1, 56, "Subsection",
      CellTags->"InverseMellin"]},
  "Isolate"->{
    Cell[1381715, 44575, 52, 1, 56, "Subsection",
      CellTags->"Isolate"]},
  "Kummer"->{
    Cell[1387966, 44810, 50, 1, 56, "Subsection",
      CellTags->"Kummer"]},
  "Lagrangian"->{
    Cell[1473139, 47336, 58, 1, 56, "Subsection",
      CellTags->"Lagrangian"]},
  "LC"->{
    Cell[1478448, 47518, 42, 1, 56, "Subsection",
      CellTags->"LC"]},
  "LCD"->{
    Cell[1482538, 47678, 44, 1, 56, "Subsection",
      CellTags->"LCD"]},
  "LeftPartialD"->{
    Cell[1487368, 47866, 62, 1, 56, "Subsection",
      CellTags->"LeftPartialD"]},
  "LeftRightPartialD"->{
    Cell[1491944, 48045, 72, 1, 56, "Subsection",
      CellTags->"LeftRightPartialD"]},
  "LeftRightPartialD2"->{
    Cell[1498073, 48270, 74, 1, 56, "Subsection",
      CellTags->"LeftRightPartialD2"]},
  "LeviCivita"->{
    Cell[1503717, 48483, 59, 1, 56, "Subsection",
      CellTags->"LeviCivita"]},
  "Li2"->{
    Cell[1512492, 48808, 45, 1, 56, "Subsection",
      CellTags->"Li2"]},
  "Li3"->{
    Cell[1513818, 48877, 45, 1, 56, "Subsection",
      CellTags->"Li3"]},
  "LorentzIndex"->{
    Cell[1515392, 48959, 62, 1, 56, "Subsection",
      CellTags->"LorentzIndex"]},
  "Map2"->{
    Cell[1518557, 49095, 46, 1, 56, "Subsection",
      CellTags->"Map2"]},
  "MetricTensor"->{
    Cell[1520199, 49179, 62, 1, 56, "Subsection",
      CellTags->"MetricTensor"]},
  "MomentumCombine"->{
    Cell[1523972, 49346, 68, 1, 56, "Subsection",
      CellTags->"MomentumCombine"]},
  "MomentumExpand"->{
    Cell[1527919, 49507, 66, 1, 56, "Subsection",
      CellTags->"MomentumExpand"]},
  "Momentum"->{
    Cell[1532500, 49689, 54, 1, 56, "Subsection",
      CellTags->"Momentum"]},
  "MT"->{
    Cell[1536489, 49872, 42, 1, 56, "Subsection",
      CellTags->"MT"]},
  "MTD"->{
    Cell[1539129, 49996, 44, 1, 56, "Subsection",
      CellTags->"MTD"]},
  "Nielsen"->{
    Cell[1541914, 50123, 52, 1, 56, "Subsection",
      CellTags->"Nielsen"]},
  "NTerms"->{
    Cell[1547226, 50323, 50, 1, 56, "Subsection",
      CellTags->"NTerms"]},
  "OneLoop"->{
    Cell[1548295, 50382, 52, 1, 56, "Subsection",
      CellTags->"OneLoop"]},
  "OneLoopSimplify"->{
    Cell[1567717, 50960, 68, 1, 56, "Subsection",
      CellTags->"OneLoopSimplify"]},
  "OPEDelta"->{
    Cell[1577797, 51300, 54, 1, 56, "Subsection",
      CellTags->"OPEDelta"]},
  "OPEi"->{
    Cell[1579036, 51364, 82, 1, 56, "Subsection",
      CellTags->"OPEi"]},
  "OPESumExplicit"->{
    Cell[1583930, 51542, 66, 1, 56, "Subsection",
      CellTags->"OPESumExplicit"]},
  "OPESum"->{
    Cell[1590397, 51768, 50, 1, 56, "Subsection",
      CellTags->"OPESum"]},
  "OPESumSimplify"->{
    Cell[1599279, 52059, 66, 1, 56, "Subsection",
      CellTags->"OPESumSimplify"]},
  "Pair"->{
    Cell[1605282, 52264, 46, 1, 56, "Subsection",
      CellTags->"Pair"]},
  "PartialD"->{
    Cell[1612245, 52545, 54, 1, 56, "Subsection",
      CellTags->"PartialD"]},
  "PaVe"->{
    Cell[1615200, 52665, 46, 1, 56, "Subsection",
      CellTags->"PaVe"]},
  "PaVeOrder"->{
    Cell[1620115, 52811, 56, 1, 56, "Subsection",
      CellTags->"PaVeOrder"]},
  "PaVeReduce"->{
    Cell[1625430, 52961, 58, 1, 56, "Subsection",
      CellTags->"PaVeReduce"]},
  "PlusDistribution"->{
    Cell[1652043, 53664, 70, 1, 56, "Subsection",
      CellTags->"PlusDistribution"]},
  "Polarization"->{
    Cell[1655635, 53811, 62, 1, 56, "Subsection",
      CellTags->"Polarization"]},
  "PolarizationSum"->{
    Cell[1659433, 53955, 68, 1, 56, "Subsection",
      CellTags->"PolarizationSum"]},
  "PolarizationVector"->{
    Cell[1670442, 54311, 74, 1, 56, "Subsection",
      CellTags->"PolarizationVector"]},
  "PowerSimplify"->{
    Cell[1678047, 54588, 64, 1, 56, "Subsection",
      CellTags->"PowerSimplify"]},
  "PropagatorDenominatorExplicit"->{
    Cell[1680298, 54687, 96, 1, 56, "Subsection",
      CellTags->"PropagatorDenominatorExplicit"]},
  "PropagatorDenominator"->{
    Cell[1683807, 54818, 80, 1, 56, "Subsection",
      CellTags->"PropagatorDenominator"]},
  "QuantumField"->{
    Cell[1687441, 54968, 62, 1, 56, "Subsection",
      CellTags->"QuantumField"]},
  "QuarkField"->{
    Cell[1694628, 55258, 58, 1, 56, "Subsection",
      CellTags->"QuarkField"]},
  "QuarkGluonVertex"->{
    Cell[1695495, 55303, 70, 1, 56, "Subsection",
      CellTags->"QuarkGluonVertex"]},
  "QuarkPropagator"->{
    Cell[1703919, 55582, 68, 1, 56, "Subsection",
      CellTags->"QuarkPropagator"]},
  "RightPartialD"->{
    Cell[1706916, 55698, 64, 1, 56, "Subsection",
      CellTags->"RightPartialD"]},
  "RussianTrick"->{
    Cell[1709886, 55826, 62, 1, 56, "Subsection",
      CellTags->"RussianTrick"]},
  "ScalarProduct"->{
    Cell[1729922, 56442, 64, 1, 56, "Subsection",
      CellTags->"ScalarProduct"]},
  "ScalarProductCancel"->{
    Cell[1736471, 56693, 76, 1, 56, "Subsection",
      CellTags->"ScalarProductCancel"]},
  "SD"->{
    Cell[1751263, 57176, 42, 1, 56, "Subsection",
      CellTags->"SD"]},
  "SelectFree"->{
    Cell[1752641, 57244, 58, 1, 56, "Subsection",
      CellTags->"SelectFree"]},
  "SelectNotFree"->{
    Cell[1755179, 57364, 64, 1, 56, "Subsection",
      CellTags->"SelectNotFree"]},
  "Series2"->{
    Cell[1757957, 57492, 52, 1, 56, "Subsection",
      CellTags->"Series2"]},
  "SimplifyDeltaFunction"->{
    Cell[1772535, 57961, 80, 1, 56, "Subsection",
      CellTags->"SimplifyDeltaFunction"]},
  "SimplifyPolyLog"->{
    Cell[1779629, 58236, 68, 1, 56, "Subsection",
      CellTags->"SimplifyPolyLog"]},
  "SOD"->{
    Cell[1800620, 58940, 44, 1, 56, "Subsection",
      CellTags->"SOD"]},
  "Solve2"->{
    Cell[1802863, 59042, 50, 1, 56, "Subsection",
      CellTags->"Solve2"]},
  "SO"->{
    Cell[1807216, 59207, 42, 1, 56, "Subsection",
      CellTags->"SO"]},
  "SP"->{
    Cell[1809472, 59310, 42, 1, 56, "Subsection",
      CellTags->"SP"]},
  "SPD"->{
    Cell[1814081, 59501, 44, 1, 56, "Subsection",
      CellTags->"SPD"]},
  "Spinor"->{
    Cell[1819172, 59710, 50, 1, 56, "Subsection",
      CellTags->"Spinor"]},
  "SpinorUBar"->{
    Cell[1823336, 59883, 58, 1, 56, "Subsection",
      CellTags->"SpinorUBar"]},
  "SpinorU"->{
    Cell[1826082, 60009, 52, 1, 56, "Subsection",
      CellTags->"SpinorU"]},
  "SpinorVBar"->{
    Cell[1828474, 60121, 58, 1, 56, "Subsection",
      CellTags->"SpinorVBar"]},
  "SpinorV"->{
    Cell[1831222, 60247, 52, 1, 56, "Subsection",
      CellTags->"SpinorV"]},
  "SplittingFunction"->{
    Cell[1833674, 60361, 72, 1, 56, "Subsection",
      CellTags->"SplittingFunction"]},
  "SumP"->{
    Cell[1873755, 61544, 46, 1, 56, "Subsection",
      CellTags->"SumP"]},
  "SumS"->{
    Cell[1878002, 61721, 46, 1, 56, "Subsection",
      CellTags->"SumS"]},
  "SumT"->{
    Cell[1884665, 62008, 46, 1, 56, "Subsection",
      CellTags->"SumT"]},
  "SUNDelta"->{
    Cell[1892391, 62331, 54, 1, 56, "Subsection",
      CellTags->"SUNDelta"]},
  "SUND"->{
    Cell[1898331, 62576, 46, 1, 56, "Subsection",
      CellTags->"SUND"]},
  "SUNF"->{
    Cell[1905105, 62839, 46, 1, 56, "Subsection",
      CellTags->"SUNF"]},
  "SUNIndex"->{
    Cell[1916344, 63252, 54, 1, 56, "Subsection",
      CellTags->"SUNIndex"]},
  "SUNSimplify"->{
    Cell[1917766, 63323, 60, 1, 56, "Subsection",
      CellTags->"SUNSimplify"]},
  "SUNT"->{
    Cell[1939984, 64104, 46, 1, 56, "Subsection",
      CellTags->"SUNT"]},
  "SUNTrace"->{
    Cell[1947490, 64420, 54, 1, 56, "Subsection",
      CellTags->"SUNTrace"]},
  "Symmetrize"->{
    Cell[1980689, 65397, 58, 1, 56, "Subsection",
      CellTags->"Symmetrize"]},
  "Tdec"->{
    Cell[1982807, 65481, 46, 1, 56, "Subsection",
      CellTags->"Tdec"]},
  "Tensorfunction"->{
    Cell[2003909, 66089, 66, 1, 56, "Subsection",
      CellTags->"Tensorfunction"]},
  "TFi"->{
    Cell[2007807, 66250, 44, 1, 56, "Subsection",
      CellTags->"TFi"]},
  "TIDL"->{
    Cell[2012388, 66409, 46, 1, 56, "Subsection",
      CellTags->"TIDL"]},
  "TID"->{
    Cell[2094260, 68474, 44, 1, 56, "Subsection",
      CellTags->"TID"]},
  "ToDistribution"->{
    Cell[2115257, 69055, 66, 1, 56, "Subsection",
      CellTags->"ToDistribution"]},
  "ToHypergeometric"->{
    Cell[2123122, 69331, 70, 1, 56, "Subsection",
      CellTags->"ToHypergeometric"]},
  "ToLarin"->{
    Cell[2131507, 69603, 52, 1, 56, "Subsection",
      CellTags->"ToLarin"]},
  "ToTFi"->{
    Cell[2134758, 69725, 48, 1, 56, "Subsection",
      CellTags->"ToTFi"]},
  "Tr"->{
    Cell[2145365, 70066, 42, 1, 56, "Subsection",
      CellTags->"Tr"]},
  "TrickMandelstam"->{
    Cell[2182719, 71142, 68, 1, 56, "Subsection",
      CellTags->"TrickMandelstam"]},
  "Trick"->{
    Cell[2185792, 71256, 48, 1, 56, "Subsection",
      CellTags->"Trick"]},
  "Twist2GluonOperator"->{
    Cell[2190660, 71448, 76, 1, 56, "Subsection",
      CellTags->"Twist2GluonOperator"]},
  "Twist2QuarkOperator"->{
    Cell[2520935, 79550, 76, 1, 56, "Subsection",
      CellTags->"Twist2QuarkOperator"]},
  "Uncontract"->{
    Cell[2579951, 81058, 58, 1, 56, "Subsection",
      CellTags->"Uncontract"]},
  "UnDeclareNonCommutative"->{
    Cell[2588106, 81361, 84, 1, 56, "Subsection",
      CellTags->"UnDeclareNonCommutative"]},
  "Write2"->{
    Cell[2590331, 81461, 50, 1, 56, "Subsection",
      CellTags->"Write2"]},
  "Zeta2"->{
    Cell[2600767, 81811, 48, 1, 56, "Subsection",
      CellTags->"Zeta2"]},
  "$FCT"->{
    Cell[2601868, 81868, 46, 1, 56, "Subsection",
      CellTags->"$FCT"]},
  "$LorentzIndices"->{
    Cell[2603063, 81921, 68, 1, 56, "Subsection",
      CellTags->"$LorentzIndices"]},
  "$PairBrackets"->{
    Cell[2607555, 82079, 64, 1, 56, "Subsection",
      CellTags->"$PairBrackets"]},
  "$VeryVerbose"->{
    Cell[2609971, 82181, 62, 1, 56, "Subsection",
      CellTags->"$VeryVerbose"]},
  "$West"->{
    Cell[2612801, 82287, 48, 1, 56, "Subsection",
      CellTags->"$West"]},
  "Evaluation time and memory usage"->{
    Cell[2613535, 82320, 102, 1, 56, "Subsection",
      CellTags->"Evaluation time and memory usage"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"Table of Contents", 2662339, 84180},
  {"Short Overview", 2662441, 84183},
  {"Load FeynCalc", 2662570, 84187},
  {"Example Calculations", 2662677, 84190},
  {"Tree level", 2662809, 84194},
  {"A moment integral", 2662918, 84197},
  {"Reference Guide", 2663030, 84200},
  {"A0", 2663122, 84203},
  {"Amplitudes", 2663240, 84207},
  {"AnomalousDimension", 2663346, 84210},
  {"Anti5", 2663447, 84213},
  {"AntiCommutator", 2663544, 84216},
  {"AntiQuarkField", 2663650, 84219},
  {"AntiSymmetrize", 2663756, 84222},
  {"Apart1", 2663854, 84225},
  {"Apart2", 2663944, 84228},
  {"Apart3", 2664034, 84231},
  {"B00", 2664121, 84234},
  {"B0", 2664204, 84237},
  {"B11", 2664287, 84240},
  {"B1", 2664370, 84243},
  {"BackgroundGluonVertex", 2664471, 84246},
  {"C0", 2664572, 84249},
  {"Calc", 2664657, 84252},
  {"CA", 2664742, 84255},
  {"Cases2", 2664829, 84258},
  {"CF", 2664916, 84261},
  {"ChangeDimension", 2665012, 84264},
  {"ChargeConjugationMatrixInv", 2665132, 84267},
  {"ChargeConjugationMatrix", 2665260, 84270},
  {"ChiralityProjector", 2665380, 84273},
  {"Chisholm", 2665485, 84276},
  {"ClearScalarProducts", 2665591, 84279},
  {"Collect2", 2665697, 84282},
  {"Combine", 2665791, 84285},
  {"CommutatorExplicit", 2665895, 84288},
  {"Commutator", 2666002, 84291},
  {"ComplexConjugate", 2666107, 84294},
  {"ComplexIndex", 2666214, 84297},
  {"Contract", 2666313, 84300},
  {"Convolute", 2666409, 84303},
  {"CovariantD", 2666507, 84306},
  {"D0", 2666598, 84309},
  {"DataType", 2666687, 84312},
  {"DeclareNonCommutative", 2666795, 84315},
  {"DeltaFunction", 2666908, 84318},
  {"DeltaFunctionPrime", 2667018, 84321},
  {"Dimension", 2667124, 84324},
  {"DiracGammaCombine", 2667229, 84327},
  {"DiracGammaExpand", 2667341, 84330},
  {"DiracGamma", 2667446, 84333},
  {"DiracGammaT", 2667546, 84336},
  {"DiracMatrix", 2667647, 84339},
  {"DiracOrder", 2667747, 84342},
  {"DiracReduce", 2667847, 84345},
  {"DiracSigmaExplicit", 2667955, 84348},
  {"DiracSigma", 2668062, 84351},
  {"DiracSimplify", 2668164, 84354},
  {"DiracSimplify2", 2668270, 84357},
  {"DiracSlash", 2668373, 84360},
  {"DiracTrace", 2668472, 84363},
  {"DiracTrick", 2668571, 84366},
  {"DotSimplify", 2668671, 84369},
  {"EpsChisholm", 2668772, 84372},
  {"EpsEvaluate", 2668873, 84375},
  {"Epsilon", 2668970, 84378},
  {"Eps", 2669059, 84381},
  {"ExpandPartialD", 2669155, 84384},
  {"ExpandScalarProduct", 2669267, 84387},
  {"Explicit", 2669373, 84390},
  {"ExplicitPartialD", 2669476, 84393},
  {"ExplicitSUNIndex", 2669587, 84396},
  {"Factor2", 2669689, 84399},
  {"FAD", 2669778, 84402},
  {"FCE", 2669863, 84405},
  {"FCI", 2669948, 84408},
  {"FermionSpinSum", 2670044, 84411},
  {"FeynAmpDenominatorCombine", 2670162, 84414},
  {"FeynAmpDenominator", 2670284, 84417},
  {"FeynAmpDenominatorSimplify", 2670407, 84420},
  {"FeynAmpDenominatorSplit", 2670535, 84423},
  {"FeynAmp", 2670644, 84426},
  {"FeynCalc2FORM", 2670743, 84429},
  {"FeynCalcExternal", 2670851, 84432},
  {"FeynCalcInternal", 2670962, 84435},
  {"FeynRule", 2671065, 84438},
  {"FieldStrength", 2671165, 84441},
  {"FORM2FeynCalc", 2671270, 84444},
  {"FourDivergence", 2671376, 84447},
  {"FourLaplacian", 2671482, 84450},
  {"FourVector", 2671584, 84453},
  {"FreeQ2", 2671679, 84456},
  {"FromTFi", 2671771, 84459},
  {"FRH", 2671860, 84462},
  {"FunctionalD", 2671953, 84465},
  {"FV", 2672045, 84468},
  {"FVD", 2672130, 84471},
  {"GA", 2672215, 84474},
  {"GAD", 2672300, 84477},
  {"GammaEpsilon", 2672395, 84480},
  {"GammaExpand", 2672498, 84483},
  {"GhostPropagator", 2672604, 84486},
  {"GHP", 2672702, 84489},
  {"GluonField", 2672795, 84492},
  {"GluonGhostVertex", 2672901, 84495},
  {"GluonPropagator", 2673012, 84498},
  {"GluonVertex", 2673118, 84501},
  {"GSD", 2673212, 84504},
  {"GS", 2673297, 84507},
  {"Gstrong", 2673386, 84510},
  {"Hill", 2673477, 84513},
  {"HypergeometricAC", 2673577, 84516},
  {"HypergeometricIR", 2673689, 84519},
  {"HypergeometricSE", 2673801, 84522},
  {"HypExplicit", 2673908, 84525},
  {"Integrate2", 2674009, 84528},
  {"Integrate3", 2674109, 84531},
  {"IntegrateByParts", 2674215, 84534},
  {"InverseMellin", 2674324, 84537},
  {"Isolate", 2674424, 84540},
  {"Kummer", 2674517, 84543},
  {"Lagrangian", 2674613, 84546},
  {"LC", 2674705, 84549},
  {"LCD", 2674790, 84552},
  {"LeftPartialD", 2674885, 84555},
  {"LeftRightPartialD", 2674994, 84558},
  {"LeftRightPartialD2", 2675109, 84561},
  {"LeviCivita", 2675217, 84564},
  {"Li2", 2675310, 84567},
  {"Li3", 2675396, 84570},
  {"LorentzIndex", 2675491, 84573},
  {"Map2", 2675587, 84576},
  {"MetricTensor", 2675683, 84579},
  {"MomentumCombine", 2675790, 84582},
  {"MomentumExpand", 2675899, 84585},
  {"Momentum", 2676001, 84588},
  {"MT", 2676091, 84591},
  {"MTD", 2676176, 84594},
  {"Nielsen", 2676266, 84597},
  {"NTerms", 2676359, 84600},
  {"OneLoop", 2676452, 84603},
  {"OneLoopSimplify", 2676554, 84606},
  {"OPEDelta", 2676657, 84609},
  {"OPEi", 2676749, 84612},
  {"OPESumExplicit", 2676847, 84615},
  {"OPESum", 2676947, 84618},
  {"OPESumSimplify", 2677047, 84621},
  {"Pair", 2677145, 84624},
  {"PartialD", 2677237, 84627},
  {"PaVe", 2677329, 84630},
  {"PaVeOrder", 2677422, 84633},
  {"PaVeReduce", 2677521, 84636},
  {"PlusDistribution", 2677627, 84639},
  {"Polarization", 2677735, 84642},
  {"PolarizationSum", 2677842, 84645},
  {"PolarizationVector", 2677955, 84648},
  {"PowerSimplify", 2678066, 84651},
  {"PropagatorDenominatorExplicit", 2678188, 84654},
  {"PropagatorDenominator", 2678318, 84657},
  {"QuantumField", 2678431, 84660},
  {"QuarkField", 2678533, 84663},
  {"QuarkGluonVertex", 2678639, 84666},
  {"QuarkPropagator", 2678750, 84669},
  {"RightPartialD", 2678858, 84672},
  {"RussianTrick", 2678963, 84675},
  {"ScalarProduct", 2679068, 84678},
  {"ScalarProductCancel", 2679180, 84681},
  {"SD", 2679281, 84684},
  {"SelectFree", 2679373, 84687},
  {"SelectNotFree", 2679476, 84690},
  {"Series2", 2679576, 84693},
  {"SimplifyDeltaFunction", 2679684, 84696},
  {"SimplifyPolyLog", 2679800, 84699},
  {"SOD", 2679898, 84702},
  {"Solve2", 2679987, 84705},
  {"SO", 2680075, 84708},
  {"SP", 2680159, 84711},
  {"SPD", 2680244, 84714},
  {"Spinor", 2680333, 84717},
  {"SpinorUBar", 2680429, 84720},
  {"SpinorU", 2680526, 84723},
  {"SpinorVBar", 2680623, 84726},
  {"SpinorV", 2680720, 84729},
  {"SplittingFunction", 2680824, 84732},
  {"SumP", 2680925, 84735},
  {"SumS", 2681013, 84738},
  {"SumT", 2681101, 84741},
  {"SUNDelta", 2681193, 84744},
  {"SUND", 2681285, 84747},
  {"SUNF", 2681373, 84750},
  {"SUNIndex", 2681465, 84753},
  {"SUNSimplify", 2681564, 84756},
  {"SUNT", 2681659, 84759},
  {"SUNTrace", 2681751, 84762},
  {"Symmetrize", 2681849, 84765},
  {"Tdec", 2681943, 84768},
  {"Tensorfunction", 2682041, 84771},
  {"TFi", 2682138, 84774},
  {"TIDL", 2682225, 84777},
  {"TID", 2682312, 84780},
  {"ToDistribution", 2682409, 84783},
  {"ToHypergeometric", 2682519, 84786},
  {"ToLarin", 2682622, 84789},
  {"ToTFi", 2682714, 84792},
  {"Tr", 2682801, 84795},
  {"TrickMandelstam", 2682898, 84798},
  {"Trick", 2682998, 84801},
  {"Twist2GluonOperator", 2683102, 84804},
  {"Twist2QuarkOperator", 2683220, 84807},
  {"Uncontract", 2683329, 84810},
  {"UnDeclareNonCommutative", 2683442, 84813},
  {"Write2", 2683551, 84816},
  {"Zeta2", 2683642, 84819},
  {"$FCT", 2683731, 84822},
  {"$LorentzIndices", 2683830, 84825},
  {"$PairBrackets", 2683938, 84828},
  {"$VeryVerbose", 2684043, 84831},
  {"$West", 2684140, 84834},
  {"Evaluation time and memory usage", 2684257, 84837}
  }
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 59, 1, 107, "Title"],
Cell[1793, 54, 125, 3, 49, "Subtitle",
  PageBreakBelow->True],
Cell[1921, 59, 83, 1, 49, "Subtitle"],
Cell[2007, 62, 59, 1, 48, "Subsubtitle"],
Cell[2069, 65, 62, 1, 48, "Subsubtitle"],
Cell[2134, 68, 244, 8, 48, "Subsubtitle"],
Cell[2381, 78, 255, 8, 48, "Subsubtitle"],

Cell[CellGroupData[{
Cell[2661, 90, 69, 1, 73, "Section",
  CellTags->"Table of Contents"],
Cell[2733, 93, 136, 3, 24, "Contents"],
Cell[2872, 98, 137, 3, 24, "Contents"],
Cell[3012, 103, 150, 3, 24, "Contents"],
Cell[3165, 108, 143, 3, 16, "Index"],
Cell[3311, 113, 141, 3, 16, "Index"],
Cell[3455, 118, 147, 3, 37, "ContentsSection"],
Cell[3605, 123, 111, 3, 16, "Index"],
Cell[3719, 128, 127, 3, 16, "Index"],
Cell[3849, 133, 143, 3, 16, "Index"],
Cell[3995, 138, 117, 3, 16, "Index"],
Cell[4115, 143, 135, 3, 16, "Index"],
Cell[4253, 148, 135, 3, 16, "Index"],
Cell[4391, 153, 135, 3, 16, "Index"],
Cell[4529, 158, 119, 3, 16, "Index"],
Cell[4651, 163, 119, 3, 16, "Index"],
Cell[4773, 168, 119, 3, 16, "Index"],
Cell[4895, 173, 113, 3, 16, "Index"],
Cell[5011, 178, 111, 3, 16, "Index"],
Cell[5125, 183, 113, 3, 16, "Index"],
Cell[5241, 188, 111, 3, 16, "Index"],
Cell[5355, 193, 149, 3, 16, "Index"],
Cell[5507, 198, 111, 3, 16, "Index"],
Cell[5621, 203, 115, 3, 16, "Index"],
Cell[5739, 208, 111, 3, 16, "Index"],
Cell[5853, 213, 119, 3, 16, "Index"],
Cell[5975, 218, 111, 3, 16, "Index"],
Cell[6089, 223, 137, 3, 16, "Index"],
Cell[6229, 228, 159, 3, 16, "Index"],
Cell[6391, 233, 153, 3, 16, "Index"],
Cell[6547, 238, 143, 3, 16, "Index"],
Cell[6693, 243, 123, 3, 16, "Index"],
Cell[6819, 248, 145, 3, 16, "Index"],
Cell[6967, 253, 123, 3, 16, "Index"],
Cell[7093, 258, 121, 3, 16, "Index"],
Cell[7217, 263, 143, 3, 16, "Index"],
Cell[7363, 268, 127, 3, 16, "Index"],
Cell[7493, 273, 139, 3, 16, "Index"],
Cell[7635, 278, 131, 3, 16, "Index"],
Cell[7769, 283, 123, 3, 16, "Index"],
Cell[7895, 288, 125, 3, 16, "Index"],
Cell[8023, 293, 127, 3, 16, "Index"],
Cell[8153, 298, 111, 3, 16, "Index"],
Cell[8267, 303, 123, 3, 16, "Index"],
Cell[8393, 308, 149, 3, 16, "Index"],
Cell[8545, 313, 133, 3, 16, "Index"],
Cell[8681, 318, 143, 3, 16, "Index"],
Cell[8827, 323, 125, 3, 16, "Index"],
Cell[8955, 328, 141, 3, 16, "Index"],
Cell[9099, 333, 139, 3, 16, "Index"],
Cell[9241, 338, 127, 3, 16, "Index"],
Cell[9371, 343, 129, 3, 16, "Index"],
Cell[9503, 348, 129, 3, 16, "Index"],
Cell[9635, 353, 127, 3, 16, "Index"],
Cell[9765, 358, 129, 3, 16, "Index"],
Cell[9897, 363, 143, 3, 16, "Index"],
Cell[10043, 368, 127, 3, 16, "Index"],
Cell[10173, 373, 133, 3, 16, "Index"],
Cell[10309, 378, 135, 3, 16, "Index"],
Cell[10447, 383, 127, 3, 16, "Index"],
Cell[10577, 388, 127, 3, 16, "Index"],
Cell[10707, 393, 127, 3, 16, "Index"],
Cell[10837, 398, 129, 3, 16, "Index"],
Cell[10969, 403, 129, 3, 16, "Index"],
Cell[11101, 408, 129, 3, 16, "Index"],
Cell[11233, 413, 121, 3, 16, "Index"],
Cell[11357, 418, 113, 3, 16, "Index"],
Cell[11473, 423, 135, 3, 16, "Index"],
Cell[11611, 428, 145, 3, 16, "Index"],
Cell[11759, 433, 123, 3, 16, "Index"],
Cell[11885, 438, 139, 3, 16, "Index"],
Cell[12027, 443, 139, 3, 16, "Index"],
Cell[12169, 448, 121, 3, 16, "Index"],
Cell[12293, 453, 113, 3, 16, "Index"],
Cell[12409, 458, 113, 3, 16, "Index"],
Cell[12525, 463, 113, 3, 16, "Index"],
Cell[12641, 468, 135, 3, 16, "Index"],
Cell[12779, 473, 157, 3, 16, "Index"],
Cell[12939, 478, 143, 3, 16, "Index"],
Cell[13085, 483, 159, 3, 16, "Index"],
Cell[13247, 488, 153, 3, 16, "Index"],
Cell[13403, 493, 121, 3, 16, "Index"],
Cell[13527, 498, 133, 3, 16, "Index"],
Cell[13663, 503, 139, 3, 16, "Index"],
Cell[13805, 508, 139, 3, 16, "Index"],
Cell[13947, 513, 123, 3, 16, "Index"],
Cell[14073, 518, 133, 3, 16, "Index"],
Cell[14209, 523, 133, 3, 16, "Index"],
Cell[14345, 528, 135, 3, 16, "Index"],
Cell[14483, 533, 133, 3, 16, "Index"],
Cell[14619, 538, 127, 3, 16, "Index"],
Cell[14749, 543, 119, 3, 16, "Index"],
Cell[14871, 548, 121, 3, 16, "Index"],
Cell[14995, 553, 113, 3, 16, "Index"],
Cell[15111, 558, 129, 3, 16, "Index"],
Cell[15243, 563, 111, 3, 16, "Index"],
Cell[15357, 568, 113, 3, 16, "Index"],
Cell[15473, 573, 111, 3, 16, "Index"],
Cell[15587, 578, 113, 3, 16, "Index"],
Cell[15703, 583, 131, 3, 16, "Index"],
Cell[15837, 588, 129, 3, 16, "Index"],
Cell[15969, 593, 137, 3, 16, "Index"],
Cell[16109, 598, 113, 3, 16, "Index"],
Cell[16225, 603, 127, 3, 16, "Index"],
Cell[16355, 608, 139, 3, 16, "Index"],
Cell[16497, 613, 137, 3, 16, "Index"],
Cell[16637, 618, 129, 3, 16, "Index"],
Cell[16769, 623, 113, 3, 16, "Index"],
Cell[16885, 628, 111, 3, 16, "Index"],
Cell[16999, 633, 121, 3, 16, "Index"],
Cell[17123, 638, 115, 3, 16, "Index"],
Cell[17241, 643, 139, 3, 16, "Index"],
Cell[17383, 648, 139, 3, 16, "Index"],
Cell[17525, 653, 139, 3, 16, "Index"],
Cell[17667, 658, 129, 3, 16, "Index"],
Cell[17799, 663, 127, 3, 16, "Index"],
Cell[17929, 668, 127, 3, 16, "Index"],
Cell[18059, 673, 139, 3, 16, "Index"],
Cell[18201, 678, 133, 3, 16, "Index"],
Cell[18337, 683, 121, 3, 16, "Index"],
Cell[18461, 688, 119, 3, 16, "Index"],
Cell[18583, 693, 127, 3, 16, "Index"],
Cell[18713, 698, 111, 3, 16, "Index"],
Cell[18827, 703, 113, 3, 16, "Index"],
Cell[18943, 708, 131, 3, 16, "Index"],
Cell[19077, 713, 141, 3, 16, "Index"],
Cell[19221, 718, 143, 3, 16, "Index"],
Cell[19367, 723, 127, 3, 16, "Index"],
Cell[19497, 728, 113, 3, 16, "Index"],
Cell[19613, 733, 113, 3, 16, "Index"],
Cell[19729, 738, 131, 3, 16, "Index"],
Cell[19863, 743, 115, 3, 16, "Index"],
Cell[19981, 748, 131, 3, 16, "Index"],
Cell[20115, 753, 137, 3, 16, "Index"],
Cell[20255, 758, 135, 3, 16, "Index"],
Cell[20393, 763, 123, 3, 16, "Index"],
Cell[20519, 768, 111, 3, 16, "Index"],
Cell[20633, 773, 113, 3, 16, "Index"],
Cell[20749, 778, 121, 3, 16, "Index"],
Cell[20873, 783, 119, 3, 16, "Index"],
Cell[20995, 788, 121, 3, 16, "Index"],
Cell[21119, 793, 137, 3, 16, "Index"],
Cell[21259, 798, 123, 3, 16, "Index"],
Cell[21385, 803, 151, 3, 16, "Index"],
Cell[21539, 808, 135, 3, 16, "Index"],
Cell[21677, 813, 119, 3, 16, "Index"],
Cell[21799, 818, 135, 3, 16, "Index"],
Cell[21937, 823, 115, 3, 16, "Index"],
Cell[22055, 828, 123, 3, 16, "Index"],
Cell[22181, 833, 115, 3, 16, "Index"],
Cell[22299, 838, 125, 3, 16, "Index"],
Cell[22427, 843, 127, 3, 16, "Index"],
Cell[22557, 848, 139, 3, 16, "Index"],
Cell[22699, 853, 131, 3, 16, "Index"],
Cell[22833, 858, 137, 3, 16, "Index"],
Cell[22973, 863, 143, 3, 16, "Index"],
Cell[23119, 868, 133, 3, 16, "Index"],
Cell[23255, 873, 165, 3, 16, "Index"],
Cell[23423, 878, 149, 3, 16, "Index"],
Cell[23575, 883, 131, 3, 16, "Index"],
Cell[23709, 888, 127, 3, 16, "Index"],
Cell[23839, 893, 139, 3, 16, "Index"],
Cell[23981, 898, 137, 3, 16, "Index"],
Cell[24121, 903, 133, 3, 16, "Index"],
Cell[24257, 908, 131, 3, 16, "Index"],
Cell[24391, 913, 133, 3, 16, "Index"],
Cell[24527, 918, 145, 3, 16, "Index"],
Cell[24675, 923, 111, 3, 16, "Index"],
Cell[24789, 928, 127, 3, 16, "Index"],
Cell[24919, 933, 133, 3, 16, "Index"],
Cell[25055, 938, 121, 3, 16, "Index"],
Cell[25179, 943, 149, 3, 16, "Index"],
Cell[25331, 948, 137, 3, 16, "Index"],
Cell[25471, 953, 113, 3, 16, "Index"],
Cell[25587, 958, 119, 3, 16, "Index"],
Cell[25709, 963, 111, 3, 16, "Index"],
Cell[25823, 968, 111, 3, 16, "Index"],
Cell[25937, 973, 113, 3, 16, "Index"],
Cell[26053, 978, 119, 3, 16, "Index"],
Cell[26175, 983, 127, 3, 16, "Index"],
Cell[26305, 988, 121, 3, 16, "Index"],
Cell[26429, 993, 127, 3, 16, "Index"],
Cell[26559, 998, 121, 3, 16, "Index"],
Cell[26683, 1003, 141, 3, 16, "Index"],
Cell[26827, 1008, 115, 3, 16, "Index"],
Cell[26945, 1013, 115, 3, 16, "Index"],
Cell[27063, 1018, 115, 3, 16, "Index"],
Cell[27181, 1023, 123, 3, 16, "Index"],
Cell[27307, 1028, 115, 3, 16, "Index"],
Cell[27425, 1033, 115, 3, 16, "Index"],
Cell[27543, 1038, 123, 3, 16, "Index"],
Cell[27669, 1043, 129, 3, 16, "Index"],
Cell[27801, 1048, 115, 3, 16, "Index"],
Cell[27919, 1053, 123, 3, 16, "Index"],
Cell[28045, 1058, 127, 3, 16, "Index"],
Cell[28175, 1063, 115, 3, 16, "Index"],
Cell[28293, 1068, 135, 3, 16, "Index"],
Cell[28431, 1073, 113, 3, 16, "Index"],
Cell[28547, 1078, 115, 3, 16, "Index"],
Cell[28665, 1083, 113, 3, 16, "Index"],
Cell[28781, 1088, 135, 3, 16, "Index"],
Cell[28919, 1093, 139, 3, 16, "Index"],
Cell[29061, 1098, 121, 3, 16, "Index"],
Cell[29185, 1103, 117, 3, 16, "Index"],
Cell[29305, 1108, 111, 3, 16, "Index"],
Cell[29419, 1113, 137, 3, 16, "Index"],
Cell[29559, 1118, 117, 3, 16, "Index"],
Cell[29679, 1123, 145, 3, 16, "Index"],
Cell[29827, 1128, 145, 3, 16, "Index"],
Cell[29975, 1133, 127, 3, 16, "Index"],
Cell[30105, 1138, 153, 3, 16, "Index"],
Cell[30261, 1143, 119, 3, 16, "Index"],
Cell[30383, 1148, 117, 3, 16, "Index"],
Cell[30503, 1153, 115, 3, 16, "Index"],
Cell[30621, 1158, 137, 3, 16, "Index"],
Cell[30761, 1163, 133, 3, 16, "Index"],
Cell[30897, 1168, 131, 3, 16, "Index"],
Cell[31031, 1173, 117, 3, 16, "Index"],
Cell[31151, 1178, 171, 3, 16, "Index"]
}, Open  ]],

Cell[CellGroupData[{
Cell[31359, 1186, 85, 2, 73, "Section",
  PageBreakAbove->True,
  CellTags->"Short Overview"],
Cell[31447, 1190, 1176, 29, 158, "Text"],
Cell[32626, 1221, 1917, 75, 68, "Text"],
Cell[34546, 1298, 67, 0, 32, "Text"],
Cell[34616, 1300, 1792, 74, 68, "Text"],
Cell[36411, 1376, 1156, 22, 194, "Text"],
Cell[37570, 1400, 251, 6, 50, "Text"],
Cell[37824, 1408, 370, 9, 68, "Text"],
Cell[38197, 1419, 548, 9, 104, "Text"],
Cell[38748, 1430, 548, 18, 50, "Text"],
Cell[39299, 1450, 73, 0, 32, "Text"],
Cell[39375, 1452, 236, 7, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[39648, 1464, 62, 1, 73, "Section",
  CellTags->"Load FeynCalc"],
Cell[39713, 1467, 106, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[39844, 1474, 106, 2, 27, "Input"],
Cell[39953, 1478, 80, 1, 29, "Output"]
}, Open  ]],
Cell[40048, 1482, 61, 0, 32, "Text"],
Cell[40112, 1484, 344, 9, 43, "Input"],
Cell[40459, 1495, 344, 9, 43, "Input"],

Cell[CellGroupData[{
Cell[40828, 1508, 81, 2, 27, "Input"],
Cell[40912, 1512, 271, 9, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[41220, 1526, 64, 1, 27, "Input"],
Cell[41287, 1529, 82, 2, 40, "Output"]
}, Open  ]],
Cell[41384, 1534, 609, 17, 86, "Text"],
Cell[41996, 1553, 36, 0, 32, "Text"],
Cell[42035, 1555, 74, 1, 27, "Input"],
Cell[42112, 1558, 368, 14, 32, "Text"],
Cell[42483, 1574, 330, 7, 50, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[42850, 1586, 99, 2, 73, "Section",
  PageBreakAbove->True,
  CellTags->"Example Calculations"],
Cell[42952, 1590, 109, 3, 32, "Commentary"],
Cell[43064, 1595, 331, 9, 50, "Text"],

Cell[CellGroupData[{
Cell[43420, 1608, 36, 0, 56, "Subsection"],

Cell[CellGroupData[{
Cell[43481, 1612, 317, 10, 48, "Subsubsection",
  CellTags->"Tree level"],
Cell[43801, 1624, 298, 9, 32, "Text"],
Cell[44102, 1635, 59, 0, 32, "Text"],
Cell[44164, 1637, 96, 3, 42, "Input"],
Cell[44263, 1642, 96, 3, 42, "Input"],
Cell[44362, 1647, 96, 3, 42, "Input"],
Cell[44461, 1652, 96, 3, 42, "Input"],
Cell[44560, 1657, 73, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[44658, 1661, 261, 6, 27, "Input"],
Cell[44922, 1669, 1327, 39, 28, "Output"]
}, Open  ]],
Cell[46264, 1711, 2373, 76, 50, "Text"],
Cell[48640, 1789, 297, 8, 29, "Input"],
Cell[48940, 1799, 265, 7, 27, "Input"],
Cell[49208, 1808, 297, 8, 29, "Input"],
Cell[49508, 1818, 232, 6, 27, "Input"],
Cell[49743, 1826, 232, 6, 27, "Input"],
Cell[49978, 1834, 390, 10, 45, "Input"],
Cell[50371, 1846, 38, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[50434, 1850, 262, 6, 27, "Input"],
Cell[50699, 1858, 1092, 32, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[51828, 1895, 262, 6, 27, "Input"],
Cell[52093, 1903, 1092, 32, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[53222, 1940, 262, 6, 27, "Input"],
Cell[53487, 1948, 1092, 32, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[54616, 1985, 262, 6, 27, "Input"],
Cell[54881, 1993, 1092, 32, 28, "Output"]
}, Open  ]],
Cell[55988, 2028, 48, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[56061, 2032, 60, 0, 27, "Input"],
Cell[56124, 2034, 1643, 47, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[57804, 2086, 60, 0, 27, "Input"],
Cell[57867, 2088, 1258, 36, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[59162, 2129, 60, 0, 27, "Input"],
Cell[59225, 2131, 1258, 36, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[60520, 2172, 60, 0, 27, "Input"],
Cell[60583, 2174, 1643, 47, 29, "Output"]
}, Open  ]],
Cell[62241, 2224, 65, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[62331, 2228, 807, 21, 87, "Input"],
Cell[63141, 2251, 6525, 183, 92, "Output"]
}, Open  ]],
Cell[69681, 2437, 52, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[69758, 2441, 127, 3, 27, "Input"],
Cell[69888, 2446, 6792, 181, 49, "Output"]
}, Open  ]],
Cell[76695, 2630, 94, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[76814, 2637, 349, 9, 27, "Input"],
Cell[77166, 2648, 59, 1, 28, "Output"]
}, Open  ]],
Cell[77240, 2652, 109, 3, 32, "Text"],
Cell[77352, 2657, 186, 4, 27, "Input"],
Cell[77541, 2663, 617, 26, 32, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[78207, 2695, 48, 0, 56, "Subsection"],

Cell[CellGroupData[{
Cell[78280, 2699, 75, 1, 48, "Subsubsection",
  CellTags->"A moment integral"],
Cell[78358, 2702, 471, 16, 36, "Text"],
Cell[78832, 2720, 96, 3, 32, "Text"],
Cell[78931, 2725, 376, 12, 36, "Text"],
Cell[79310, 2739, 61, 0, 32, "Text"],
Cell[79374, 2741, 255, 9, 32, "Text"],

Cell[CellGroupData[{
Cell[79654, 2754, 68, 1, 27, "Input"],
Cell[79725, 2757, 113, 3, 28, "Output"]
}, Open  ]],
Cell[79853, 2763, 189, 4, 50, "Text"],
Cell[80045, 2769, 2106, 44, 171, "Input"],
Cell[82154, 2815, 134, 3, 50, "Text"],
Cell[82291, 2820, 17103, 628, 9417, "Input"],

Cell[CellGroupData[{
Cell[99419, 3452, 63, 1, 27, "Input"],
Cell[99485, 3455, 1086, 29, 58, "Output"]
}, Open  ]],
Cell[100586, 3487, 121, 3, 32, "Commentary"],

Cell[CellGroupData[{
Cell[100732, 3494, 45, 0, 27, "Input"],
Cell[100780, 3496, 216, 7, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[101033, 3508, 63, 1, 27, "Input"],
Cell[101099, 3511, 170, 5, 28, "Output"]
}, Open  ]],
Cell[101284, 3519, 33, 0, 32, "Text"],
Cell[101320, 3521, 218, 4, 50, "Text"],
Cell[101541, 3527, 611, 18, 44, "Text"],

Cell[CellGroupData[{
Cell[102177, 3549, 355, 9, 27, "Input"],
Cell[102535, 3560, 177, 5, 28, "Output"]
}, Open  ]],
Cell[102727, 3568, 167, 4, 50, "Text"],

Cell[CellGroupData[{
Cell[102919, 3576, 466, 12, 27, "Input"],
Cell[103388, 3590, 177, 5, 28, "Output"]
}, Open  ]],
Cell[103580, 3598, 57, 0, 32, "Text"],
Cell[103640, 3600, 626, 16, 43, "Input"],
Cell[104269, 3618, 60, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[104354, 3622, 1457, 36, 91, "Input"],
Cell[105814, 3660, 3589, 87, 121, "Output"]
}, Open  ]],
Cell[109418, 3750, 115, 3, 32, "Text"],
Cell[109536, 3755, 568, 15, 31, "Input"],

Cell[CellGroupData[{
Cell[110129, 3774, 60, 1, 27, "Input"],
Cell[110192, 3777, 61, 1, 28, "Output"]
}, Open  ]],
Cell[110268, 3781, 119, 3, 32, "Text"],
Cell[110390, 3786, 232, 7, 27, "Input"],
Cell[110625, 3795, 185, 4, 50, "Text"],

Cell[CellGroupData[{
Cell[110835, 3803, 136, 3, 27, "Input"],
Cell[110974, 3808, 4817, 121, 189, "Output"]
}, Open  ]],
Cell[115806, 3932, 235, 7, 32, "Text"],

Cell[CellGroupData[{
Cell[116066, 3943, 416, 11, 48, "Input"],
Cell[116485, 3956, 3502, 89, 87, "Output"]
}, Open  ]],
Cell[120002, 4048, 100, 3, 32, "Text"],
Cell[120105, 4053, 128, 3, 27, "Input"],
Cell[120236, 4058, 200, 5, 27, "Input"],
Cell[120439, 4065, 1287, 57, 50, "Text"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[121787, 4129, 65, 1, 73, "Section",
  CellTags->"Reference Guide"],

Cell[CellGroupData[{
Cell[121877, 4134, 66, 2, 56, "Subsection",
  PageBreakAbove->True,
  CellTags->"A0"],

Cell[CellGroupData[{
Cell[121968, 4140, 36, 0, 48, "Subsubsection"],
Cell[122007, 4142, 363, 9, 28, "Text"],

Cell[CellGroupData[{
Cell[122395, 4155, 69, 1, 27, "Input"],
Cell[122467, 4158, 132, 4, 28, "Output"]
}, Open  ]],
Cell[122614, 4165, 458, 23, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[123109, 4193, 33, 0, 48, "Subsubsection"],
Cell[123145, 4195, 254, 10, 32, "Text"],

Cell[CellGroupData[{
Cell[123424, 4209, 86, 2, 29, "Input"],
Cell[123513, 4213, 333, 9, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[123883, 4227, 159, 4, 31, "Input"],
Cell[124045, 4233, 333, 9, 29, "Output"]
}, Open  ]],
Cell[124393, 4245, 223, 7, 32, "Text"],

Cell[CellGroupData[{
Cell[124641, 4256, 58, 1, 27, "Input"],
Cell[124702, 4259, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[124798, 4265, 131, 3, 27, "Input"],
Cell[124932, 4270, 59, 1, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[125052, 4278, 58, 1, 56, "Subsection",
  CellTags->"Amplitudes"],

Cell[CellGroupData[{
Cell[125135, 4283, 36, 0, 48, "Subsubsection"],
Cell[125174, 4285, 235, 3, 68, "Text"],

Cell[CellGroupData[{
Cell[125434, 4292, 72, 1, 27, "Input"],
Cell[125509, 4295, 321, 8, 28, "Output"]
}, Open  ]],
Cell[125845, 4306, 164, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[126046, 4319, 33, 0, 48, "Subsubsection"],
Cell[126082, 4321, 69, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[126176, 4325, 78, 1, 27, "Input"],
Cell[126257, 4328, 4772, 130, 49, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[131066, 4463, 64, 1, 27, "Input"],
Cell[131133, 4466, 12237, 302, 67, "Output"]
}, Open  ]],
Cell[143385, 4771, 195, 7, 32, "Text"],

Cell[CellGroupData[{
Cell[143605, 4782, 301, 7, 27, "Input"],
Cell[143909, 4791, 34245, 785, 196, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[178215, 5583, 74, 1, 56, "Subsection",
  CellTags->"AnomalousDimension"],

Cell[CellGroupData[{
Cell[178314, 5588, 36, 0, 48, "Subsubsection"],
Cell[178353, 5590, 121, 3, 50, "Text"],

Cell[CellGroupData[{
Cell[178499, 5597, 80, 1, 27, "Input"],
Cell[178582, 5600, 225, 6, 28, "Output"]
}, Open  ]],
Cell[178822, 5609, 401, 17, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[179260, 5631, 33, 0, 48, "Subsubsection"],
Cell[179296, 5633, 30, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[179351, 5637, 161, 3, 27, "Input"],
Cell[179515, 5642, 225, 6, 28, "Output"]
}, Open  ]],
Cell[179755, 5651, 215, 7, 34, "Text"],

Cell[CellGroupData[{
Cell[179995, 5662, 79, 1, 27, "Input"],
Cell[180077, 5665, 509, 15, 45, "Output"]
}, Open  ]],
Cell[180601, 5683, 214, 7, 34, "Text"],

Cell[CellGroupData[{
Cell[180840, 5694, 78, 1, 27, "Input"],
Cell[180921, 5697, 272, 8, 45, "Output"]
}, Open  ]],
Cell[181208, 5708, 214, 7, 34, "Text"],

Cell[CellGroupData[{
Cell[181447, 5719, 78, 1, 27, "Input"],
Cell[181528, 5722, 271, 8, 45, "Output"]
}, Open  ]],
Cell[181814, 5733, 214, 7, 33, "Text"],

Cell[CellGroupData[{
Cell[182053, 5744, 78, 1, 27, "Input"],
Cell[182134, 5747, 680, 19, 46, "Output"]
}, Open  ]],
Cell[182829, 5769, 215, 7, 34, "Text"],

Cell[CellGroupData[{
Cell[183069, 5780, 79, 1, 27, "Input"],
Cell[183151, 5783, 780, 21, 46, "Output"]
}, Open  ]],
Cell[183946, 5807, 215, 7, 34, "Text"],

Cell[CellGroupData[{
Cell[184186, 5818, 79, 1, 27, "Input"],
Cell[184268, 5821, 8475, 213, 336, "Output"]
}, Open  ]],
Cell[192758, 6037, 214, 7, 34, "Text"],

Cell[CellGroupData[{
Cell[192997, 6048, 78, 1, 27, "Input"],
Cell[193078, 6051, 5368, 137, 222, "Output"]
}, Open  ]],
Cell[198461, 6191, 214, 7, 34, "Text"],

Cell[CellGroupData[{
Cell[198700, 6202, 78, 1, 27, "Input"],
Cell[198781, 6205, 8275, 210, 326, "Output"]
}, Open  ]],
Cell[207071, 6418, 214, 7, 33, "Text"],

Cell[CellGroupData[{
Cell[207310, 6429, 78, 1, 27, "Input"],
Cell[207391, 6432, 6455, 165, 257, "Output"]
}, Open  ]],
Cell[213861, 6600, 241, 7, 33, "Text"],

Cell[CellGroupData[{
Cell[214127, 6611, 78, 1, 27, "Input"],
Cell[214208, 6614, 6089, 143, 185, "Output"]
}, Open  ]],
Cell[220312, 6760, 307, 8, 33, "Text"],

Cell[CellGroupData[{
Cell[220644, 6772, 276, 8, 27, "Input"],
Cell[220923, 6782, 205, 5, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[221189, 6794, 48, 1, 56, "Subsection",
  CellTags->"Anti5"],

Cell[CellGroupData[{
Cell[221262, 6799, 36, 0, 48, "Subsubsection"],
Cell[221301, 6801, 507, 15, 50, "Text"],
Cell[221811, 6818, 155, 6, 32, "Text"],
Cell[221969, 6826, 432, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[222438, 6849, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[222496, 6853, 122, 3, 27, "Input"],
Cell[222621, 6858, 159, 4, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[222817, 6867, 61, 1, 27, "Input"],
Cell[222881, 6870, 306, 9, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[223224, 6884, 112, 3, 27, "Input"],
Cell[223339, 6889, 270, 8, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[223646, 6902, 167, 4, 27, "Input"],
Cell[223816, 6908, 308, 7, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[224161, 6920, 88, 2, 27, "Input"],
Cell[224252, 6924, 752, 23, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[225041, 6952, 96, 2, 27, "Input"],
Cell[225140, 6956, 752, 23, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[225929, 6984, 119, 3, 27, "Input"],
Cell[226051, 6989, 752, 23, 29, "Output"]
}, Open  ]],
Cell[226818, 7015, 360, 13, 32, "Text"],

Cell[CellGroupData[{
Cell[227203, 7032, 90, 2, 27, "Input"],
Cell[227296, 7036, 164, 4, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[227497, 7045, 61, 1, 27, "Input"],
Cell[227561, 7048, 306, 9, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[227928, 7064, 66, 1, 56, "Subsection",
  CellTags->"AntiCommutator"],

Cell[CellGroupData[{
Cell[228019, 7069, 36, 0, 48, "Subsubsection"],
Cell[228058, 7071, 116, 3, 32, "Text"],
Cell[228177, 7076, 608, 23, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[228822, 7104, 33, 0, 48, "Subsubsection"],
Cell[228858, 7106, 180, 8, 32, "Text"],
Cell[229041, 7116, 104, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[229170, 7122, 97, 2, 27, "Input"],
Cell[229270, 7126, 207, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[229514, 7136, 74, 1, 27, "Input"],
Cell[229591, 7139, 144, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[229772, 7148, 268, 7, 27, "Input"],
Cell[230043, 7157, 473, 15, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[230553, 7177, 261, 7, 27, "Input"],
Cell[230817, 7186, 286, 8, 29, "Output"]
}, Open  ]],
Cell[231118, 7197, 182, 4, 33, "Input"],
Cell[231303, 7203, 75, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[231403, 7207, 136, 3, 27, "Input"],
Cell[231542, 7212, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[231638, 7218, 137, 3, 27, "Input"],
Cell[231778, 7223, 176, 5, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[231991, 7233, 225, 6, 33, "Input"],
Cell[232219, 7241, 86, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[232342, 7248, 221, 5, 33, "Input"],
Cell[232566, 7255, 302, 9, 32, "Output"]
}, Open  ]],
Cell[232883, 7267, 204, 4, 33, "Input"],
Cell[233090, 7273, 135, 3, 27, "Input"],
Cell[233228, 7278, 197, 5, 33, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[233474, 7289, 66, 1, 56, "Subsection",
  CellTags->"AntiQuarkField"],

Cell[CellGroupData[{
Cell[233565, 7294, 36, 0, 48, "Subsubsection"],
Cell[233604, 7296, 170, 4, 50, "Text"],
Cell[233777, 7302, 304, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[234118, 7320, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[234176, 7324, 40, 0, 27, "Input"],
Cell[234219, 7326, 91, 2, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[234371, 7335, 66, 1, 56, "Subsection",
  CellTags->"AntiSymmetrize"],

Cell[CellGroupData[{
Cell[234462, 7340, 36, 0, 48, "Subsubsection"],
Cell[234501, 7342, 124, 3, 32, "Text"],
Cell[234628, 7347, 166, 7, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[234831, 7359, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[234889, 7363, 222, 6, 27, "Input"],
Cell[235114, 7371, 322, 10, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[235473, 7386, 242, 6, 27, "Input"],
Cell[235718, 7394, 718, 18, 43, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[236497, 7419, 50, 1, 56, "Subsection",
  CellTags->"Apart1"],

Cell[CellGroupData[{
Cell[236572, 7424, 36, 0, 48, "Subsubsection"],
Cell[236611, 7426, 148, 3, 50, "Text"],
Cell[236762, 7431, 274, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[237073, 7449, 33, 0, 48, "Subsubsection"],
Cell[237109, 7451, 10514, 267, 267, "Input"],

Cell[CellGroupData[{
Cell[247648, 7722, 188, 5, 27, "Input"],
Cell[247839, 7729, 183, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[248059, 7739, 187, 5, 27, "Input"],
Cell[248249, 7746, 182, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[248468, 7756, 87, 2, 27, "Input"],
Cell[248558, 7760, 59, 1, 28, "Output"]
}, Open  ]],
Cell[248632, 7764, 100, 2, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[248781, 7772, 50, 1, 56, "Subsection",
  CellTags->"Apart2"],

Cell[CellGroupData[{
Cell[248856, 7777, 36, 0, 48, "Subsubsection"],
Cell[248895, 7779, 89, 3, 32, "Text"],
Cell[248987, 7784, 301, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[249325, 7802, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[249383, 7806, 245, 7, 27, "Input"],
Cell[249631, 7815, 1469, 35, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[251137, 7855, 62, 1, 27, "Input"],
Cell[251202, 7858, 858, 24, 53, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[252097, 7887, 102, 2, 27, "Input"],
Cell[252202, 7891, 561, 17, 55, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[252824, 7915, 50, 1, 56, "Subsection",
  CellTags->"Apart3"],

Cell[CellGroupData[{
Cell[252899, 7920, 36, 0, 48, "Subsubsection"],
Cell[252938, 7922, 101, 3, 32, "Text"],
Cell[253042, 7927, 161, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[253240, 7940, 33, 0, 48, "Subsubsection"],
Cell[253276, 7942, 10522, 267, 267, "Input"],

Cell[CellGroupData[{
Cell[263823, 8213, 188, 5, 27, "Input"],
Cell[264014, 8220, 183, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[264234, 8230, 187, 5, 27, "Input"],
Cell[264424, 8237, 182, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[264643, 8247, 87, 2, 27, "Input"],
Cell[264733, 8251, 59, 1, 28, "Output"]
}, Open  ]],
Cell[264807, 8255, 100, 2, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[264956, 8263, 44, 1, 56, "Subsection",
  CellTags->"B00"],

Cell[CellGroupData[{
Cell[265025, 8268, 36, 0, 48, "Subsubsection"],
Cell[265064, 8270, 191, 4, 50, "Text"],

Cell[CellGroupData[{
Cell[265280, 8278, 65, 1, 27, "Input"],
Cell[265348, 8281, 133, 4, 28, "Output"]
}, Open  ]],
Cell[265496, 8288, 357, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[265890, 8311, 33, 0, 48, "Subsubsection"],
Cell[265926, 8313, 196, 6, 32, "Text"],

Cell[CellGroupData[{
Cell[266147, 8323, 58, 1, 27, "Input"],
Cell[266208, 8326, 134, 4, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[266379, 8335, 191, 5, 31, "Input"],
Cell[266573, 8342, 3675, 89, 125, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[270285, 8436, 191, 5, 31, "Input"],
Cell[270479, 8443, 1908, 53, 81, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[272424, 8501, 246, 6, 31, "Input"],
Cell[272673, 8509, 700, 20, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[273410, 8534, 156, 4, 31, "Input"],
Cell[273569, 8540, 825, 23, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[274431, 8568, 234, 6, 31, "Input"],
Cell[274668, 8576, 859, 24, 45, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[275588, 8607, 42, 1, 56, "Subsection",
  CellTags->"B0"],

Cell[CellGroupData[{
Cell[275655, 8612, 36, 0, 48, "Subsubsection"],
Cell[275694, 8614, 294, 5, 68, "Text"],

Cell[CellGroupData[{
Cell[276013, 8623, 64, 1, 27, "Input"],
Cell[276080, 8626, 269, 7, 28, "Output"]
}, Open  ]],
Cell[276364, 8636, 464, 23, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[276865, 8664, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[276923, 8668, 221, 6, 31, "Input"],
Cell[277147, 8676, 333, 10, 29, "Output"]
}, Open  ]],
Cell[277495, 8689, 220, 5, 27, "Input"],

Cell[CellGroupData[{
Cell[277740, 8698, 125, 3, 31, "Input"],
Cell[277868, 8703, 237, 7, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[278142, 8715, 155, 4, 31, "Input"],
Cell[278300, 8721, 237, 7, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[278574, 8733, 153, 4, 27, "Input"],
Cell[278730, 8739, 199, 5, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[278990, 8751, 44, 1, 56, "Subsection",
  CellTags->"B11"],

Cell[CellGroupData[{
Cell[279059, 8756, 36, 0, 48, "Subsubsection"],
Cell[279098, 8758, 525, 16, 50, "Text"],

Cell[CellGroupData[{
Cell[279648, 8778, 65, 1, 27, "Input"],
Cell[279716, 8781, 133, 4, 28, "Output"]
}, Open  ]],
Cell[279864, 8788, 461, 23, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[280362, 8816, 33, 0, 48, "Subsubsection"],
Cell[280398, 8818, 214, 6, 32, "Text"],

Cell[CellGroupData[{
Cell[280637, 8828, 58, 1, 27, "Input"],
Cell[280698, 8831, 134, 4, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[280869, 8840, 191, 5, 31, "Input"],
Cell[281063, 8847, 3866, 95, 127, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[284966, 8947, 191, 5, 31, "Input"],
Cell[285160, 8954, 1587, 45, 53, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[286784, 9004, 140, 3, 27, "Input"],
Cell[286927, 9009, 134, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[287098, 9018, 191, 5, 31, "Input"],
Cell[287292, 9025, 700, 20, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[288029, 9050, 139, 3, 27, "Input"],
Cell[288171, 9055, 133, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[288341, 9064, 156, 4, 31, "Input"],
Cell[288500, 9070, 261, 7, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[288798, 9082, 234, 6, 31, "Input"],
Cell[289035, 9090, 293, 8, 43, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[289389, 9105, 42, 1, 56, "Subsection",
  CellTags->"B1"],

Cell[CellGroupData[{
Cell[289456, 9110, 36, 0, 48, "Subsubsection"],
Cell[289495, 9112, 227, 6, 50, "Text"],

Cell[CellGroupData[{
Cell[289747, 9122, 64, 1, 27, "Input"],
Cell[289814, 9125, 133, 4, 28, "Output"]
}, Open  ]],
Cell[289962, 9132, 589, 28, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[290588, 9165, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[290646, 9169, 190, 5, 31, "Input"],
Cell[290839, 9176, 1453, 40, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[292329, 9221, 245, 6, 31, "Input"],
Cell[292577, 9229, 333, 10, 29, "Output"]
}, Open  ]],
Cell[292925, 9242, 164, 4, 27, "Input"],

Cell[CellGroupData[{
Cell[293114, 9250, 190, 5, 31, "Input"],
Cell[293307, 9257, 431, 13, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[293775, 9275, 155, 4, 31, "Input"],
Cell[293933, 9281, 454, 13, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[294424, 9299, 125, 3, 31, "Input"],
Cell[294552, 9304, 454, 13, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[295043, 9322, 220, 5, 31, "Input"],
Cell[295266, 9329, 913, 23, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[296216, 9357, 267, 6, 31, "Input"],
Cell[296486, 9365, 474, 13, 43, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[297021, 9385, 80, 1, 56, "Subsection",
  CellTags->"BackgroundGluonVertex"],

Cell[CellGroupData[{
Cell[297126, 9390, 36, 0, 48, "Subsubsection"],
Cell[297165, 9392, 1013, 18, 194, "Text"],

Cell[CellGroupData[{
Cell[298203, 9414, 83, 1, 27, "Input"],
Cell[298289, 9417, 298, 8, 28, "Output"]
}, Open  ]],
Cell[298602, 9428, 176, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[298815, 9441, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[298873, 9445, 340, 8, 27, "Input"],
Cell[299216, 9455, 2996, 83, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[302249, 9543, 427, 10, 27, "Input"],
Cell[302679, 9555, 10876, 268, 47, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[313592, 9828, 396, 9, 27, "Input"],
Cell[313991, 9839, 3220, 88, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[317248, 9932, 483, 11, 43, "Input"],
Cell[317734, 9945, 11048, 270, 87, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[328843, 10222, 42, 1, 56, "Subsection",
  CellTags->"C0"],

Cell[CellGroupData[{
Cell[328910, 10227, 36, 0, 48, "Subsubsection"],
Cell[328949, 10229, 445, 9, 86, "Text"],
Cell[329397, 10240, 479, 23, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[329913, 10268, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[329971, 10272, 150, 4, 27, "Input"],
Cell[330124, 10278, 458, 9, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[330619, 10292, 185, 5, 27, "Input"],
Cell[330807, 10299, 458, 9, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[331302, 10313, 332, 9, 27, "Input"],
Cell[331637, 10324, 458, 9, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[332156, 10340, 46, 1, 56, "Subsection",
  CellTags->"Calc"],

Cell[CellGroupData[{
Cell[332227, 10345, 36, 0, 48, "Subsubsection"],
Cell[332266, 10347, 208, 4, 68, "Text"],
Cell[332477, 10353, 417, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[332931, 10376, 42, 0, 47, "Subsubsection"],
Cell[332976, 10378, 362, 12, 32, "Text"],

Cell[CellGroupData[{
Cell[333363, 10394, 278, 7, 27, "Input"],
Cell[333644, 10403, 115, 3, 28, "Output"]
}, Open  ]],
Cell[333774, 10409, 224, 8, 32, "Text"],

Cell[CellGroupData[{
Cell[334023, 10421, 243, 6, 27, "Input"],
Cell[334269, 10429, 417, 13, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[334723, 10447, 326, 9, 27, "Input"],
Cell[335052, 10458, 565, 17, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[335654, 10480, 60, 1, 27, "Input"],
Cell[335717, 10483, 727, 21, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[336481, 10509, 189, 4, 27, "Input"],
Cell[336673, 10515, 1443, 43, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[338153, 10563, 403, 12, 27, "Input"],
Cell[338559, 10577, 59, 1, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[338679, 10585, 42, 1, 56, "Subsection",
  CellTags->"CA"],

Cell[CellGroupData[{
Cell[338746, 10590, 36, 0, 48, "Subsubsection"],
Cell[338785, 10592, 69, 0, 32, "Text"],
Cell[338857, 10594, 277, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[339171, 10612, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[339229, 10616, 28, 0, 27, "Input"],
Cell[339260, 10618, 85, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[339382, 10625, 143, 3, 27, "Input"],
Cell[339528, 10630, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[339624, 10636, 30, 0, 27, "Input"],
Cell[339657, 10638, 59, 1, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[339777, 10646, 50, 1, 56, "Subsection",
  CellTags->"Cases2"],

Cell[CellGroupData[{
Cell[339852, 10651, 36, 0, 48, "Subsubsection"],
Cell[339891, 10653, 272, 5, 68, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[340200, 10663, 42, 0, 47, "Subsubsection"],

Cell[CellGroupData[{
Cell[340267, 10667, 303, 8, 31, "Input"],
Cell[340573, 10677, 269, 8, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[340879, 10690, 315, 9, 27, "Input"],
Cell[341197, 10701, 273, 8, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[341507, 10714, 451, 12, 27, "Input"],
Cell[341961, 10728, 362, 10, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[342360, 10743, 326, 8, 27, "Input"],
Cell[342689, 10753, 620, 18, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[343370, 10778, 42, 1, 56, "Subsection",
  CellTags->"CF"],

Cell[CellGroupData[{
Cell[343437, 10783, 36, 0, 48, "Subsubsection"],
Cell[343476, 10785, 91, 3, 32, "Text"],
Cell[343570, 10790, 277, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[343884, 10808, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[343942, 10812, 28, 0, 27, "Input"],
Cell[343973, 10814, 85, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[344095, 10821, 153, 3, 27, "Input"],
Cell[344251, 10826, 174, 5, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[344462, 10836, 30, 0, 27, "Input"],
Cell[344495, 10838, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[344591, 10844, 147, 3, 27, "Input"],
Cell[344741, 10849, 85, 2, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[344887, 10858, 68, 1, 56, "Subsection",
  CellTags->"ChangeDimension"],

Cell[CellGroupData[{
Cell[344980, 10863, 36, 0, 48, "Subsubsection"],
Cell[345019, 10865, 180, 4, 50, "Text"],
Cell[345202, 10871, 527, 23, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[345766, 10899, 33, 0, 48, "Subsubsection"],
Cell[345802, 10901, 189, 4, 50, "Text"],

Cell[CellGroupData[{
Cell[346016, 10909, 174, 4, 27, "Input"],
Cell[346193, 10915, 170, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[346400, 10925, 103, 2, 27, "Input"],
Cell[346506, 10929, 170, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[346713, 10939, 64, 1, 27, "Input"],
Cell[346780, 10942, 232, 6, 40, "Output"]
}, Open  ]],
Cell[347027, 10951, 91, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[347143, 10958, 162, 4, 27, "Input"],
Cell[347308, 10964, 170, 4, 40, "Output"]
}, Open  ]],
Cell[347493, 10971, 75, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[347593, 10975, 502, 13, 27, "Input"],
Cell[348098, 10990, 658, 17, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[348793, 11012, 105, 2, 27, "Input"],
Cell[348901, 11016, 187, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[349125, 11027, 185, 4, 27, "Input"],
Cell[349313, 11033, 187, 6, 28, "Output"]
}, Open  ]],
Cell[349515, 11042, 104, 3, 32, "Text"],
Cell[349622, 11047, 171, 4, 27, "Input"],
Cell[349796, 11053, 184, 4, 50, "Text"],
Cell[349983, 11059, 164, 4, 27, "Input"],

Cell[CellGroupData[{
Cell[350172, 11067, 182, 4, 27, "Input"],
Cell[350357, 11073, 616, 19, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[351010, 11097, 139, 3, 27, "Input"],
Cell[351152, 11102, 616, 19, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[351805, 11126, 133, 3, 31, "Input"],
Cell[351941, 11131, 289, 9, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[352267, 11145, 93, 2, 29, "Input"],
Cell[352363, 11149, 82, 2, 28, "Output"]
}, Open  ]],
Cell[352460, 11154, 164, 4, 27, "Input"],
Cell[352627, 11160, 114, 3, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[352790, 11169, 90, 1, 56, "Subsection",
  CellTags->"ChargeConjugationMatrixInv"],

Cell[CellGroupData[{
Cell[352905, 11174, 36, 0, 48, "Subsubsection"],
Cell[352944, 11176, 156, 4, 50, "Text"],
Cell[353103, 11182, 212, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[353352, 11195, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[353410, 11199, 162, 3, 43, "Input"],
Cell[353575, 11204, 298, 9, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[353910, 11218, 60, 1, 27, "Input"],
Cell[353973, 11221, 236, 7, 31, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[354246, 11233, 157, 3, 27, "Input"],
Cell[354406, 11238, 188, 5, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[354631, 11248, 60, 1, 27, "Input"],
Cell[354694, 11251, 149, 4, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[354880, 11260, 102, 2, 27, "Input"],
Cell[354985, 11264, 141, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[355163, 11273, 56, 1, 27, "Input"],
Cell[355222, 11276, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[355318, 11282, 77, 1, 27, "Input"],
Cell[355398, 11285, 81, 2, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[355540, 11294, 84, 1, 56, "Subsection",
  CellTags->"ChargeConjugationMatrix"],

Cell[CellGroupData[{
Cell[355649, 11299, 36, 0, 48, "Subsubsection"],
Cell[355688, 11301, 143, 5, 32, "Text"],
Cell[355834, 11308, 221, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[356092, 11321, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[356150, 11325, 162, 3, 43, "Input"],
Cell[356315, 11330, 298, 9, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[356650, 11344, 60, 1, 27, "Input"],
Cell[356713, 11347, 236, 7, 31, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[356986, 11359, 157, 3, 27, "Input"],
Cell[357146, 11364, 188, 5, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[357371, 11374, 60, 1, 27, "Input"],
Cell[357434, 11377, 149, 4, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[357620, 11386, 74, 1, 27, "Input"],
Cell[357697, 11389, 81, 2, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[357839, 11398, 74, 1, 56, "Subsection",
  CellTags->"ChiralityProjector"],

Cell[CellGroupData[{
Cell[357938, 11403, 36, 0, 48, "Subsubsection"],
Cell[357977, 11405, 543, 18, 32, "Text"],
Cell[358523, 11425, 405, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[358965, 11448, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[359023, 11452, 233, 6, 27, "Input"],
Cell[359259, 11460, 196, 6, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[359492, 11471, 59, 1, 27, "Input"],
Cell[359554, 11474, 196, 6, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[359787, 11485, 150, 3, 27, "Input"],
Cell[359940, 11490, 394, 12, 45, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[360395, 11509, 54, 1, 56, "Subsection",
  CellTags->"Chisholm"],

Cell[CellGroupData[{
Cell[360474, 11514, 36, 0, 48, "Subsubsection"],
Cell[360513, 11516, 119, 3, 32, "Text"],
Cell[360635, 11521, 176, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[360848, 11534, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[360906, 11538, 176, 4, 27, "Input"],
Cell[361085, 11544, 2681, 77, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[363803, 11626, 147, 3, 27, "Input"],
Cell[363953, 11631, 210, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[364200, 11641, 98, 2, 27, "Input"],
Cell[364301, 11645, 1838, 53, 30, "Output"]
}, Open  ]],
Cell[366154, 11701, 54, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[366233, 11705, 89, 2, 27, "Input"],
Cell[366325, 11709, 60, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[366422, 11715, 98, 2, 27, "Input"],
Cell[366523, 11719, 1838, 53, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[368398, 11777, 89, 2, 27, "Input"],
Cell[368490, 11781, 60, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[368587, 11787, 133, 3, 27, "Input"],
Cell[368723, 11792, 807, 23, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[369567, 11820, 65, 1, 27, "Input"],
Cell[369635, 11823, 2138, 62, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[371810, 11890, 207, 5, 27, "Input"],
Cell[372020, 11897, 380, 8, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[372437, 11910, 98, 2, 27, "Input"],
Cell[372538, 11914, 15925, 438, 122, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[388500, 12357, 98, 2, 27, "Input"],
Cell[388601, 12361, 15925, 438, 122, "Output"]
}, Open  ]],
Cell[404541, 12802, 62, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[404628, 12806, 89, 2, 27, "Input"],
Cell[404720, 12810, 84, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[404841, 12817, 89, 2, 27, "Input"],
Cell[404933, 12821, 84, 2, 28, "Output"]
}, Open  ]],
Cell[405032, 12826, 161, 4, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[405242, 12836, 76, 1, 56, "Subsection",
  CellTags->"ClearScalarProducts"],

Cell[CellGroupData[{
Cell[405343, 12841, 36, 0, 48, "Subsubsection"],
Cell[405382, 12843, 111, 3, 32, "Text"],
Cell[405496, 12848, 487, 22, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[406020, 12875, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[406078, 12879, 152, 4, 27, "Input"],
Cell[406233, 12885, 87, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[406357, 12892, 169, 4, 27, "Input"],
Cell[406529, 12898, 87, 2, 29, "Output"]
}, Open  ]],
Cell[406631, 12903, 45, 0, 27, "Input"],

Cell[CellGroupData[{
Cell[406701, 12907, 169, 4, 27, "Input"],
Cell[406873, 12913, 134, 4, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[407044, 12922, 96, 2, 27, "Input"],
Cell[407143, 12926, 134, 4, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[407338, 12937, 54, 1, 56, "Subsection",
  CellTags->"Collect2"],

Cell[CellGroupData[{
Cell[407417, 12942, 36, 0, 48, "Subsubsection"],
Cell[407456, 12944, 590, 9, 122, "Text"],

Cell[CellGroupData[{
Cell[408071, 12957, 70, 1, 27, "Input"],
Cell[408144, 12960, 397, 9, 46, "Output"]
}, Open  ]],
Cell[408556, 12972, 164, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[408757, 12985, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[408815, 12989, 469, 12, 40, "Input"],
Cell[409287, 13003, 483, 14, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[409807, 13022, 144, 3, 27, "Input"],
Cell[409954, 13027, 418, 12, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[410409, 13044, 145, 3, 27, "Input"],
Cell[410557, 13049, 483, 14, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[411077, 13068, 579, 16, 31, "Input"],
Cell[411659, 13086, 550, 15, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[412246, 13106, 317, 8, 31, "Input"],
Cell[412566, 13116, 1103, 33, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[413706, 13154, 67, 1, 27, "Input"],
Cell[413776, 13157, 934, 26, 29, "Output"]
}, Open  ]],
Cell[414725, 13186, 89, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[414839, 13192, 304, 8, 31, "Input"],
Cell[415146, 13202, 1083, 29, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[416266, 13236, 103, 2, 27, "Input"],
Cell[416372, 13240, 733, 19, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[417166, 13266, 52, 1, 56, "Subsection",
  CellTags->"Combine"],

Cell[CellGroupData[{
Cell[417243, 13271, 36, 0, 48, "Subsubsection"],
Cell[417282, 13273, 293, 5, 68, "Text"],

Cell[CellGroupData[{
Cell[417600, 13282, 69, 1, 27, "Input"],
Cell[417672, 13285, 136, 4, 28, "Output"]
}, Open  ]],
Cell[417823, 13292, 164, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[418024, 13305, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[418082, 13309, 287, 9, 43, "Input"],
Cell[418372, 13320, 311, 9, 43, "Output"]
}, Open  ]],
Cell[418698, 13332, 102, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[418825, 13339, 288, 9, 43, "Input"],
Cell[419116, 13350, 298, 8, 43, "Output"]
}, Open  ]],
Cell[419429, 13361, 153, 3, 50, "Text"],

Cell[CellGroupData[{
Cell[419607, 13368, 371, 10, 43, "Input"],
Cell[419981, 13380, 298, 8, 43, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[420340, 13395, 74, 1, 56, "Subsection",
  CellTags->"CommutatorExplicit"],

Cell[CellGroupData[{
Cell[420439, 13400, 36, 0, 48, "Subsubsection"],
Cell[420478, 13402, 124, 3, 50, "Text"],
Cell[420605, 13407, 283, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[420925, 13425, 33, 0, 48, "Subsubsection"],
Cell[420961, 13427, 124, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[421110, 13433, 93, 2, 27, "Input"],
Cell[421206, 13437, 152, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[421395, 13446, 74, 1, 27, "Input"],
Cell[421472, 13449, 144, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[421653, 13458, 155, 4, 27, "Input"],
Cell[421811, 13464, 349, 9, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[422197, 13478, 74, 1, 27, "Input"],
Cell[422274, 13481, 400, 12, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[422711, 13498, 116, 3, 27, "Input"],
Cell[422830, 13503, 378, 10, 28, "Output"]
}, Open  ]],
Cell[423223, 13516, 126, 2, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[423398, 13524, 58, 1, 56, "Subsection",
  CellTags->"Commutator"],

Cell[CellGroupData[{
Cell[423481, 13529, 36, 0, 48, "Subsubsection"],
Cell[423520, 13531, 113, 3, 32, "Text"],
Cell[423636, 13536, 620, 23, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[424293, 13564, 33, 0, 48, "Subsubsection"],
Cell[424329, 13566, 124, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[424478, 13572, 93, 2, 27, "Input"],
Cell[424574, 13576, 152, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[424763, 13585, 74, 1, 27, "Input"],
Cell[424840, 13588, 144, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[425021, 13597, 229, 6, 27, "Input"],
Cell[425253, 13605, 378, 10, 28, "Output"]
}, Open  ]],
Cell[425646, 13618, 126, 2, 27, "Input"],
Cell[425775, 13622, 43, 0, 32, "Text"],
Cell[425821, 13624, 194, 4, 27, "Input"],

Cell[CellGroupData[{
Cell[426040, 13632, 482, 13, 27, "Input"],
Cell[426525, 13647, 834, 21, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[427396, 13673, 67, 1, 27, "Input"],
Cell[427466, 13676, 59, 1, 28, "Output"]
}, Open  ]],
Cell[427540, 13680, 66, 1, 27, "Input"],
Cell[427609, 13683, 116, 2, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[427774, 13691, 70, 1, 56, "Subsection",
  CellTags->"ComplexConjugate"],

Cell[CellGroupData[{
Cell[427869, 13696, 36, 0, 48, "Subsubsection"],
Cell[427908, 13698, 385, 7, 86, "Text"],
Cell[428296, 13707, 447, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[428780, 13730, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[428838, 13734, 152, 3, 27, "Input"],
Cell[428993, 13739, 303, 9, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[429333, 13753, 68, 1, 27, "Input"],
Cell[429404, 13756, 283, 7, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[429724, 13768, 103, 2, 27, "Input"],
Cell[429830, 13772, 215, 5, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[430082, 13782, 72, 1, 27, "Input"],
Cell[430157, 13785, 549, 16, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[430743, 13806, 138, 3, 27, "Input"],
Cell[430884, 13811, 684, 20, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[431605, 13836, 72, 1, 27, "Input"],
Cell[431680, 13839, 684, 20, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[432401, 13864, 146, 3, 27, "Input"],
Cell[432550, 13869, 460, 15, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[433047, 13889, 242, 6, 27, "Input"],
Cell[433292, 13897, 184, 4, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[433513, 13906, 242, 6, 27, "Input"],
Cell[433758, 13914, 497, 12, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[434292, 13931, 72, 1, 27, "Input"],
Cell[434367, 13934, 657, 16, 30, "Output"]
}, Open  ]],
Cell[435039, 13953, 135, 3, 50, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[435223, 13962, 62, 1, 56, "Subsection",
  CellTags->"ComplexIndex"],

Cell[CellGroupData[{
Cell[435310, 13967, 36, 0, 48, "Subsubsection"],
Cell[435349, 13969, 127, 3, 32, "Text"],
Cell[435479, 13974, 179, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[435695, 13987, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[435753, 13991, 72, 1, 27, "Input"],
Cell[435828, 13994, 63, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[435928, 14000, 72, 1, 27, "Input"],
Cell[436003, 14003, 91, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[436131, 14010, 64, 1, 27, "Input"],
Cell[436198, 14013, 116, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[436351, 14020, 97, 2, 27, "Input"],
Cell[436451, 14024, 89, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[436577, 14031, 72, 1, 27, "Input"],
Cell[436652, 14034, 270, 9, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[436959, 14048, 64, 1, 27, "Input"],
Cell[437026, 14051, 225, 5, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[437288, 14061, 72, 1, 27, "Input"],
Cell[437363, 14064, 236, 8, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[437660, 14079, 54, 1, 56, "Subsection",
  CellTags->"Contract"],

Cell[CellGroupData[{
Cell[437739, 14084, 36, 0, 48, "Subsubsection"],
Cell[437778, 14086, 391, 7, 86, "Text"],

Cell[CellGroupData[{
Cell[438194, 14097, 70, 1, 27, "Input"],
Cell[438267, 14100, 575, 12, 64, "Output"]
}, Open  ]],
Cell[438857, 14115, 429, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[439323, 14138, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[439381, 14142, 206, 5, 27, "Input"],
Cell[439590, 14149, 199, 6, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[439826, 14160, 64, 1, 27, "Input"],
Cell[439893, 14163, 236, 8, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[440166, 14176, 167, 4, 27, "Input"],
Cell[440336, 14182, 265, 8, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[440638, 14195, 64, 1, 27, "Input"],
Cell[440705, 14198, 234, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[440976, 14209, 103, 2, 27, "Input"],
Cell[441082, 14213, 133, 3, 28, "Output"]
}, Open  ]],
Cell[441230, 14219, 60, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[441315, 14223, 64, 1, 27, "Input"],
Cell[441382, 14226, 59, 1, 28, "Output"]
}, Open  ]],
Cell[441456, 14230, 185, 4, 50, "Text"],

Cell[CellGroupData[{
Cell[441666, 14238, 204, 5, 27, "Input"],
Cell[441873, 14245, 245, 7, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[442155, 14257, 64, 1, 27, "Input"],
Cell[442222, 14260, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[442318, 14266, 204, 5, 27, "Input"],
Cell[442525, 14273, 245, 7, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[442807, 14285, 200, 5, 27, "Input"],
Cell[443010, 14292, 150, 4, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[443197, 14301, 69, 1, 27, "Input"],
Cell[443269, 14304, 219, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[443525, 14315, 262, 7, 27, "Input"],
Cell[443790, 14324, 557, 17, 32, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[444384, 14346, 64, 1, 27, "Input"],
Cell[444451, 14349, 786, 22, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[445274, 14376, 250, 6, 27, "Input"],
Cell[445527, 14384, 716, 21, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[446280, 14410, 64, 1, 27, "Input"],
Cell[446347, 14413, 556, 17, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[446940, 14435, 327, 8, 27, "Input"],
Cell[447270, 14445, 1275, 38, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[448582, 14488, 64, 1, 27, "Input"],
Cell[448649, 14491, 200, 6, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[448886, 14502, 435, 11, 27, "Input"],
Cell[449324, 14515, 411, 10, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[449772, 14530, 98, 2, 27, "Input"],
Cell[449873, 14534, 597, 18, 29, "Output"]
}, Open  ]],
Cell[450485, 14555, 164, 4, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[450698, 14565, 56, 1, 56, "Subsection",
  CellTags->"Convolute"],
Cell[450757, 14568, 27, 0, 32, "Text"],
Cell[450787, 14570, 1111, 27, 69, "Text"],

Cell[CellGroupData[{
Cell[451923, 14601, 71, 1, 27, "Input"],
Cell[451997, 14604, 420, 11, 28, "Output"]
}, Open  ]],
Cell[452432, 14618, 191, 8, 32, "Text"],

Cell[CellGroupData[{
Cell[452648, 14630, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[452706, 14634, 92, 2, 27, "Input"],
Cell[452801, 14638, 117, 3, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[452955, 14646, 92, 2, 27, "Input"],
Cell[453050, 14650, 146, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[453233, 14659, 92, 2, 27, "Input"],
Cell[453328, 14663, 86, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[453451, 14670, 150, 4, 43, "Input"],
Cell[453604, 14676, 189, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[453830, 14686, 203, 5, 43, "Input"],
Cell[454036, 14693, 189, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[454262, 14703, 150, 4, 43, "Input"],
Cell[454415, 14709, 280, 8, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[454732, 14722, 208, 6, 43, "Input"],
Cell[454943, 14730, 588, 17, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[455568, 14752, 159, 4, 27, "Input"],
Cell[455730, 14758, 350, 10, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[456117, 14773, 192, 5, 27, "Input"],
Cell[456312, 14780, 263, 8, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[456612, 14793, 217, 6, 43, "Input"],
Cell[456832, 14801, 373, 10, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[457242, 14816, 250, 7, 43, "Input"],
Cell[457495, 14825, 650, 18, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[458182, 14848, 221, 6, 43, "Input"],
Cell[458406, 14856, 799, 22, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[459242, 14883, 159, 4, 27, "Input"],
Cell[459404, 14889, 200, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[459641, 14900, 159, 4, 27, "Input"],
Cell[459803, 14906, 317, 9, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[460157, 14920, 246, 7, 43, "Input"],
Cell[460406, 14929, 488, 14, 47, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[460931, 14948, 128, 3, 27, "Input"],
Cell[461062, 14953, 210, 7, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[461309, 14965, 164, 4, 27, "Input"],
Cell[461476, 14971, 220, 7, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[461733, 14983, 186, 5, 43, "Input"],
Cell[461922, 14990, 488, 14, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[462447, 15009, 188, 5, 42, "Input"],
Cell[462638, 15016, 340, 10, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[463015, 15031, 217, 6, 43, "Input"],
Cell[463235, 15039, 643, 18, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[463915, 15062, 188, 5, 42, "Input"],
Cell[464106, 15069, 454, 13, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[464597, 15087, 159, 4, 27, "Input"],
Cell[464759, 15093, 200, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[464996, 15104, 128, 3, 27, "Input"],
Cell[465127, 15109, 159, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[465323, 15119, 279, 8, 43, "Input"],
Cell[465605, 15129, 1005, 29, 47, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[466671, 15165, 58, 1, 56, "Subsection",
  CellTags->"CovariantD"],

Cell[CellGroupData[{
Cell[466754, 15170, 36, 0, 48, "Subsubsection"],
Cell[466793, 15172, 709, 11, 158, "Text"],

Cell[CellGroupData[{
Cell[467527, 15187, 72, 1, 27, "Input"],
Cell[467602, 15190, 442, 10, 46, "Output"]
}, Open  ]],
Cell[468059, 15203, 162, 4, 50, "Text"],
Cell[468224, 15209, 459, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[468720, 15232, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[468778, 15236, 70, 1, 27, "Input"],
Cell[468851, 15239, 89, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[468977, 15246, 107, 2, 27, "Input"],
Cell[469087, 15250, 137, 3, 32, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[469261, 15258, 142, 3, 27, "Input"],
Cell[469406, 15263, 1250, 37, 34, "Output"]
}, Open  ]],
Cell[470671, 15303, 139, 3, 50, "Text"],

Cell[CellGroupData[{
Cell[470835, 15310, 73, 1, 27, "Input"],
Cell[470911, 15313, 146, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[471094, 15322, 110, 2, 27, "Input"],
Cell[471207, 15326, 194, 5, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[471438, 15336, 160, 3, 27, "Input"],
Cell[471601, 15341, 1183, 34, 35, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[472821, 15380, 140, 3, 27, "Input"],
Cell[472964, 15385, 1152, 33, 34, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[474153, 15423, 149, 3, 27, "Input"],
Cell[474305, 15428, 1796, 48, 37, "Output"]
}, Open  ]],
Cell[476116, 15479, 649, 21, 35, "Text"],

Cell[CellGroupData[{
Cell[476790, 15504, 183, 4, 27, "Input"],
Cell[476976, 15510, 612, 18, 33, "Output"]
}, Open  ]],
Cell[477603, 15531, 346, 10, 32, "Text"],

Cell[CellGroupData[{
Cell[477974, 15545, 183, 4, 27, "Input"],
Cell[478160, 15551, 3132, 78, 53, "Output"]
}, Open  ]],
Cell[481307, 15632, 181, 6, 32, "Text"],

Cell[CellGroupData[{
Cell[481513, 15642, 183, 4, 27, "Input"],
Cell[481699, 15648, 8874, 201, 116, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[490610, 15854, 143, 3, 28, "Input"],
Cell[490756, 15859, 274, 8, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[491067, 15872, 163, 4, 27, "Input"],
Cell[491233, 15878, 6294, 144, 116, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[497564, 16027, 190, 5, 27, "Input"],
Cell[497757, 16034, 853, 22, 72, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[498647, 16061, 207, 5, 27, "Input"],
Cell[498857, 16068, 871, 23, 72, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[499789, 16098, 42, 1, 56, "Subsection",
  CellTags->"D0"],

Cell[CellGroupData[{
Cell[499856, 16103, 36, 0, 48, "Subsubsection"],
Cell[499895, 16105, 519, 11, 104, "Text"],
Cell[500417, 16118, 479, 23, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[500933, 16146, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[500991, 16150, 313, 8, 27, "Input"],
Cell[501307, 16160, 936, 21, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[502280, 16186, 520, 13, 43, "Input"],
Cell[502803, 16201, 936, 21, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[503776, 16227, 65, 1, 27, "Input"],
Cell[503844, 16230, 936, 21, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[504841, 16258, 54, 1, 56, "Subsection",
  CellTags->"DataType"],

Cell[CellGroupData[{
Cell[504920, 16263, 36, 0, 48, "Subsubsection"],
Cell[504959, 16265, 323, 6, 68, "Text"],
Cell[505285, 16273, 206, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[505528, 16286, 33, 0, 48, "Subsubsection"],
Cell[505564, 16288, 51, 0, 32, "Text"],
Cell[505618, 16290, 196, 5, 27, "Input"],

Cell[CellGroupData[{
Cell[505839, 16299, 206, 6, 27, "Input"],
Cell[506048, 16307, 221, 7, 28, "Output"]
}, Open  ]],
Cell[506284, 16317, 159, 4, 50, "Text"],

Cell[CellGroupData[{
Cell[506468, 16325, 67, 1, 27, "Input"],
Cell[506538, 16328, 185, 5, 28, "Output"]
}, Open  ]],
Cell[506738, 16336, 262, 8, 27, "Input"],
Cell[507003, 16346, 445, 12, 27, "Input"],
Cell[507451, 16360, 431, 12, 27, "Input"],

Cell[CellGroupData[{
Cell[507907, 16376, 353, 11, 27, "Input"],
Cell[508263, 16389, 363, 11, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[508663, 16405, 62, 1, 27, "Input"],
Cell[508728, 16408, 298, 9, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[509063, 16422, 62, 1, 27, "Input"],
Cell[509128, 16425, 142, 4, 28, "Output"]
}, Open  ]],
Cell[509285, 16432, 161, 4, 27, "Input"],
Cell[509449, 16438, 160, 5, 27, "Input"],
Cell[509612, 16445, 463, 13, 27, "Input"],

Cell[CellGroupData[{
Cell[510100, 16462, 293, 9, 27, "Input"],
Cell[510396, 16473, 299, 9, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[510732, 16487, 60, 1, 27, "Input"],
Cell[510795, 16490, 217, 7, 28, "Output"]
}, Open  ]],
Cell[511027, 16500, 115, 3, 27, "Input"],
Cell[511145, 16505, 197, 5, 27, "Input"],
Cell[511345, 16512, 263, 8, 27, "Input"],
Cell[511611, 16522, 93, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[511729, 16529, 108, 2, 27, "Input"],
Cell[511840, 16533, 62, 1, 28, "Output"]
}, Open  ]],
Cell[511917, 16537, 60, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[512002, 16541, 214, 6, 27, "Input"],
Cell[512219, 16549, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[512315, 16555, 206, 6, 27, "Input"],
Cell[512524, 16563, 490, 15, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[513075, 16585, 80, 1, 56, "Subsection",
  CellTags->"DeclareNonCommutative"],

Cell[CellGroupData[{
Cell[513180, 16590, 36, 0, 48, "Subsubsection"],
Cell[513219, 16592, 158, 4, 50, "Text"],
Cell[513380, 16598, 331, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[513748, 16616, 33, 0, 48, "Subsubsection"],
Cell[513784, 16618, 77, 1, 27, "Input"],
Cell[513864, 16621, 116, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[514005, 16628, 104, 2, 27, "Input"],
Cell[514112, 16632, 62, 1, 28, "Output"]
}, Open  ]],
Cell[514189, 16636, 104, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[514318, 16642, 141, 3, 27, "Input"],
Cell[514462, 16647, 155, 4, 28, "Output"]
}, Open  ]],
Cell[514632, 16654, 116, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[514773, 16660, 141, 3, 27, "Input"],
Cell[514917, 16665, 158, 4, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[515136, 16676, 64, 1, 56, "Subsection",
  CellTags->"DeltaFunction"],

Cell[CellGroupData[{
Cell[515225, 16681, 36, 0, 48, "Subsubsection"],
Cell[515264, 16683, 181, 6, 32, "Text"],
Cell[515448, 16691, 602, 23, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[516087, 16719, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[516145, 16723, 96, 2, 27, "Input"],
Cell[516244, 16727, 176, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[516457, 16737, 309, 8, 27, "Input"],
Cell[516769, 16747, 91, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[516897, 16754, 276, 7, 27, "Input"],
Cell[517176, 16763, 91, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[517304, 16770, 309, 8, 27, "Input"],
Cell[517616, 16780, 91, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[517744, 16787, 169, 4, 27, "Input"],
Cell[517916, 16793, 59, 1, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[518036, 16801, 74, 1, 56, "Subsection",
  CellTags->"DeltaFunctionPrime"],

Cell[CellGroupData[{
Cell[518135, 16806, 36, 0, 48, "Subsubsection"],
Cell[518174, 16808, 166, 5, 32, "Text"],
Cell[518343, 16815, 580, 22, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[518960, 16842, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[519018, 16846, 101, 2, 27, "Input"],
Cell[519122, 16850, 213, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[519372, 16861, 314, 8, 27, "Input"],
Cell[519689, 16871, 166, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[519892, 16881, 310, 8, 31, "Input"],
Cell[520205, 16891, 59, 1, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[520325, 16899, 56, 1, 56, "Subsection",
  CellTags->"Dimension"],

Cell[CellGroupData[{
Cell[520406, 16904, 36, 0, 48, "Subsubsection"],
Cell[520445, 16906, 211, 4, 50, "Text"],

Cell[CellGroupData[{
Cell[520681, 16914, 74, 1, 27, "Input"],
Cell[520758, 16917, 222, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[521017, 16928, 342, 9, 27, "Input"],
Cell[521362, 16939, 534, 17, 28, "Output"]
}, Open  ]],
Cell[521911, 16959, 107, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[522043, 16966, 355, 9, 43, "Input"],
Cell[522401, 16977, 434, 11, 56, "Output"]
}, Open  ]],
Cell[522850, 16991, 142, 3, 50, "Text"],
Cell[522995, 16996, 91, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[523111, 17002, 310, 7, 27, "Input"],
Cell[523424, 17011, 844, 26, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[524305, 17042, 64, 1, 27, "Input"],
Cell[524372, 17045, 447, 11, 56, "Output"]
}, Open  ]],
Cell[524834, 17059, 92, 2, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[524975, 17067, 72, 1, 56, "Subsection",
  CellTags->"DiracGammaCombine"],

Cell[CellGroupData[{
Cell[525072, 17072, 36, 0, 48, "Subsubsection"],
Cell[525111, 17074, 103, 3, 32, "Text"],
Cell[525217, 17079, 828, 33, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[526082, 17117, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[526140, 17121, 180, 4, 27, "Input"],
Cell[526323, 17127, 312, 8, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[526672, 17140, 68, 1, 27, "Input"],
Cell[526743, 17143, 135, 3, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[526915, 17151, 242, 6, 27, "Input"],
Cell[527160, 17159, 378, 10, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[527575, 17174, 68, 1, 27, "Input"],
Cell[527646, 17177, 197, 5, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[527880, 17187, 73, 1, 27, "Input"],
Cell[527956, 17190, 560, 16, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[528577, 17213, 70, 1, 56, "Subsection",
  CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{
Cell[528672, 17218, 36, 0, 48, "Subsubsection"],
Cell[528711, 17220, 162, 4, 50, "Text"],
Cell[528876, 17226, 831, 33, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[529744, 17264, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[529802, 17268, 294, 7, 27, "Input"],
Cell[530099, 17277, 640, 18, 28, "Output"]
}, Open  ]],
Cell[530754, 17298, 96, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[530875, 17305, 68, 1, 27, "Input"],
Cell[530946, 17308, 248, 6, 40, "Output"]
}, Open  ]],
Cell[531209, 17317, 70, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[531304, 17321, 72, 1, 27, "Input"],
Cell[531379, 17324, 823, 23, 28, "Output"]
}, Open  ]],
Cell[532217, 17350, 59, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[532301, 17354, 165, 3, 27, "Input"],
Cell[532469, 17359, 248, 6, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[532754, 17370, 115, 2, 27, "Input"],
Cell[532872, 17374, 371, 9, 56, "Output"]
}, Open  ]],
Cell[533258, 17386, 75, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[533358, 17390, 69, 1, 27, "Input"],
Cell[533430, 17393, 698, 20, 29, "Output"]
}, Open  ]],
Cell[534143, 17416, 61, 1, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[534253, 17423, 58, 1, 56, "Subsection",
  CellTags->"DiracGamma"],

Cell[CellGroupData[{
Cell[534336, 17428, 36, 0, 48, "Subsubsection"],
Cell[534375, 17430, 826, 24, 86, "Text"],
Cell[535204, 17456, 703, 28, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[535944, 17489, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[536002, 17493, 66, 1, 27, "Input"],
Cell[536071, 17496, 94, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[536202, 17503, 116, 2, 27, "Input"],
Cell[536321, 17507, 199, 6, 28, "Output"]
}, Open  ]],
Cell[536535, 17516, 342, 12, 32, "Text"],

Cell[CellGroupData[{
Cell[536902, 17532, 129, 3, 27, "Input"],
Cell[537034, 17537, 234, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[537305, 17548, 257, 6, 27, "Input"],
Cell[537565, 17556, 640, 18, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[538242, 17579, 193, 5, 27, "Input"],
Cell[538438, 17586, 234, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[538709, 17597, 181, 5, 27, "Input"],
Cell[538893, 17604, 472, 13, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[539402, 17622, 101, 2, 27, "Input"],
Cell[539506, 17626, 823, 23, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[540366, 17654, 283, 7, 27, "Input"],
Cell[540652, 17663, 574, 15, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[541263, 17683, 95, 2, 27, "Input"],
Cell[541361, 17687, 1112, 29, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[542510, 17721, 98, 2, 27, "Input"],
Cell[542611, 17725, 1624, 45, 29, "Output"]
}, Open  ]],
Cell[544250, 17773, 126, 3, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[544425, 17782, 60, 1, 56, "Subsection",
  CellTags->"DiracGammaT"],

Cell[CellGroupData[{
Cell[544510, 17787, 36, 0, 48, "Subsubsection"],
Cell[544549, 17789, 182, 4, 50, "Text"],
Cell[544734, 17795, 173, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[544944, 17808, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[545002, 17812, 114, 2, 27, "Input"],
Cell[545119, 17816, 204, 6, 31, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[545360, 17827, 65, 1, 27, "Input"],
Cell[545428, 17830, 196, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[545661, 17841, 129, 4, 27, "Input"],
Cell[545793, 17847, 274, 8, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[546128, 17862, 60, 1, 56, "Subsection",
  CellTags->"DiracMatrix"],

Cell[CellGroupData[{
Cell[546213, 17867, 36, 0, 48, "Subsubsection"],
Cell[546252, 17869, 1348, 47, 68, "Text"],
Cell[547603, 17918, 1110, 48, 50, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[548750, 17971, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[548808, 17975, 71, 1, 27, "Input"],
Cell[548882, 17978, 196, 6, 28, "Output"]
}, Open  ]],
Cell[549093, 17987, 613, 19, 50, "Text"],

Cell[CellGroupData[{
Cell[549731, 18010, 144, 3, 27, "Input"],
Cell[549878, 18015, 380, 12, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[550295, 18032, 116, 3, 27, "Input"],
Cell[550414, 18037, 117, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[550568, 18044, 101, 2, 27, "Input"],
Cell[550672, 18048, 98, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[550807, 18055, 64, 1, 27, "Input"],
Cell[550874, 18058, 63, 1, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[550974, 18064, 63, 1, 27, "Input"],
Cell[551040, 18067, 98, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[551175, 18074, 94, 2, 27, "Input"],
Cell[551272, 18078, 174, 4, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[551483, 18087, 108, 2, 27, "Input"],
Cell[551594, 18091, 220, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[551851, 18101, 127, 3, 27, "Input"],
Cell[551981, 18106, 372, 12, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[552390, 18123, 64, 1, 27, "Input"],
Cell[552457, 18126, 225, 5, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[552743, 18138, 58, 1, 56, "Subsection",
  CellTags->"DiracOrder"],

Cell[CellGroupData[{
Cell[552826, 18143, 36, 0, 48, "Subsubsection"],
Cell[552865, 18145, 179, 4, 50, "Text"],
Cell[553047, 18151, 307, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[553391, 18169, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[553449, 18173, 138, 3, 27, "Input"],
Cell[553590, 18178, 173, 4, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[553800, 18187, 67, 1, 27, "Input"],
Cell[553870, 18190, 550, 16, 29, "Output"]
}, Open  ]],
Cell[554435, 18209, 68, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[554528, 18213, 138, 3, 27, "Input"],
Cell[554669, 18218, 219, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[554925, 18228, 67, 1, 27, "Input"],
Cell[554995, 18231, 550, 16, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[555582, 18252, 133, 3, 27, "Input"],
Cell[555718, 18257, 215, 5, 29, "Output"]
}, Open  ]],
Cell[555948, 18265, 189, 7, 32, "Text"],

Cell[CellGroupData[{
Cell[556162, 18276, 67, 1, 27, "Input"],
Cell[556232, 18279, 427, 13, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[556696, 18297, 129, 3, 27, "Input"],
Cell[556828, 18302, 211, 5, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[557076, 18312, 67, 1, 27, "Input"],
Cell[557146, 18315, 270, 8, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[557453, 18328, 148, 4, 27, "Input"],
Cell[557604, 18334, 227, 5, 29, "Output"]
}, Open  ]],
Cell[557846, 18342, 293, 9, 32, "Text"],

Cell[CellGroupData[{
Cell[558164, 18355, 191, 5, 27, "Input"],
Cell[558358, 18362, 1545, 44, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[559940, 18411, 66, 1, 27, "Input"],
Cell[560009, 18414, 545, 17, 29, "Output"]
}, Open  ]],
Cell[560569, 18434, 154, 4, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[560772, 18444, 60, 1, 56, "Subsection",
  CellTags->"DiracReduce"],

Cell[CellGroupData[{
Cell[560857, 18449, 36, 0, 48, "Subsubsection"],
Cell[560896, 18451, 638, 10, 140, "Text"],

Cell[CellGroupData[{
Cell[561559, 18465, 73, 1, 27, "Input"],
Cell[561635, 18468, 317, 8, 28, "Output"]
}, Open  ]],
Cell[561967, 18479, 182, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[562186, 18492, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[562244, 18496, 133, 3, 27, "Input"],
Cell[562380, 18501, 168, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[562585, 18510, 68, 1, 27, "Input"],
Cell[562656, 18513, 297, 8, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[562990, 18526, 147, 3, 27, "Input"],
Cell[563140, 18531, 210, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[563387, 18541, 68, 1, 27, "Input"],
Cell[563458, 18544, 1049, 27, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[564544, 18576, 173, 4, 27, "Input"],
Cell[564720, 18582, 259, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[565016, 18593, 96, 2, 27, "Input"],
Cell[565115, 18597, 2405, 55, 47, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[567557, 18657, 227, 5, 27, "Input"],
Cell[567787, 18664, 83, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[567907, 18671, 89, 2, 27, "Input"],
Cell[567999, 18675, 83, 2, 28, "Output"]
}, Open  ]],
Cell[568097, 18680, 112, 2, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[568258, 18688, 74, 1, 56, "Subsection",
  CellTags->"DiracSigmaExplicit"],

Cell[CellGroupData[{
Cell[568357, 18693, 36, 0, 48, "Subsubsection"],
Cell[568396, 18695, 153, 3, 50, "Text"],
Cell[568552, 18700, 173, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[568762, 18713, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[568820, 18717, 176, 4, 27, "Input"],
Cell[568999, 18723, 148, 4, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[569184, 18732, 74, 1, 27, "Input"],
Cell[569261, 18735, 982, 28, 43, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[570304, 18770, 58, 1, 56, "Subsection",
  CellTags->"DiracSigma"],

Cell[CellGroupData[{
Cell[570387, 18775, 36, 0, 48, "Subsubsection"],
Cell[570426, 18777, 188, 4, 50, "Text"],
Cell[570617, 18783, 197, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[570851, 18796, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[570909, 18800, 210, 5, 27, "Input"],
Cell[571122, 18807, 148, 4, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[571307, 18816, 75, 1, 27, "Input"],
Cell[571385, 18819, 982, 28, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[572404, 18852, 210, 5, 27, "Input"],
Cell[572617, 18859, 178, 5, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[572832, 18869, 197, 5, 27, "Input"],
Cell[573032, 18876, 132, 3, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[573201, 18884, 75, 1, 27, "Input"],
Cell[573279, 18887, 1384, 36, 43, "Output"]
}, Open  ]],
Cell[574678, 18926, 101, 2, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[574828, 18934, 64, 1, 56, "Subsection",
  CellTags->"DiracSimplify"],

Cell[CellGroupData[{
Cell[574917, 18939, 36, 0, 48, "Subsubsection"],
Cell[574956, 18941, 354, 6, 86, "Text"],

Cell[CellGroupData[{
Cell[575335, 18951, 75, 1, 27, "Input"],
Cell[575413, 18954, 615, 12, 64, "Output"]
}, Open  ]],
Cell[576043, 18969, 423, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[576503, 18992, 33, 0, 48, "Subsubsection"],
Cell[576539, 18994, 71, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[576635, 18998, 137, 3, 27, "Input"],
Cell[576775, 19003, 219, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[577031, 19013, 70, 1, 27, "Input"],
Cell[577104, 19016, 257, 8, 28, "Output"]
}, Open  ]],
Cell[577376, 19027, 68, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[577469, 19031, 138, 3, 27, "Input"],
Cell[577610, 19036, 219, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[577866, 19046, 70, 1, 27, "Input"],
Cell[577939, 19049, 257, 8, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[578233, 19062, 133, 3, 27, "Input"],
Cell[578369, 19067, 215, 5, 29, "Output"]
}, Open  ]],
Cell[578599, 19075, 163, 6, 32, "Text"],

Cell[CellGroupData[{
Cell[578787, 19085, 70, 1, 27, "Input"],
Cell[578860, 19088, 427, 13, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[579324, 19106, 129, 3, 27, "Input"],
Cell[579456, 19111, 211, 5, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[579704, 19121, 70, 1, 27, "Input"],
Cell[579777, 19124, 270, 8, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[580084, 19137, 312, 8, 27, "Input"],
Cell[580399, 19147, 885, 25, 28, "Output"]
}, Open  ]],
Cell[581299, 19175, 94, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[581418, 19182, 70, 1, 27, "Input"],
Cell[581491, 19185, 2814, 78, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[584342, 19268, 67, 1, 27, "Input"],
Cell[584412, 19271, 848, 22, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[585297, 19298, 241, 5, 27, "Input"],
Cell[585541, 19305, 582, 16, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[586160, 19326, 70, 1, 27, "Input"],
Cell[586233, 19329, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[586329, 19335, 382, 10, 27, "Input"],
Cell[586714, 19347, 654, 16, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[587405, 19368, 69, 1, 27, "Input"],
Cell[587477, 19371, 22577, 648, 118, "Output"]
}, Open  ]],
Cell[610069, 20022, 89, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[610183, 20029, 221, 5, 27, "Input"],
Cell[610407, 20036, 540, 16, 28, "Output"]
}, Open  ]],
Cell[610962, 20055, 116, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[611103, 20061, 266, 6, 27, "Input"],
Cell[611372, 20069, 694, 18, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[612103, 20092, 238, 5, 27, "Input"],
Cell[612344, 20099, 59, 1, 28, "Output"]
}, Open  ]],
Cell[612418, 20103, 141, 3, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[612608, 20112, 66, 1, 56, "Subsection",
  CellTags->"DiracSimplify2"],

Cell[CellGroupData[{
Cell[612699, 20117, 36, 0, 48, "Subsubsection"],
Cell[612738, 20119, 207, 6, 32, "Text"],
Cell[612948, 20127, 182, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[613167, 20140, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[613225, 20144, 184, 4, 27, "Input"],
Cell[613412, 20150, 427, 9, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[613876, 20164, 70, 1, 27, "Input"],
Cell[613949, 20167, 1063, 33, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[615073, 20207, 58, 1, 56, "Subsection",
  CellTags->"DiracSlash"],

Cell[CellGroupData[{
Cell[615156, 20212, 36, 0, 48, "Subsubsection"],
Cell[615195, 20214, 489, 15, 50, "Text"],

Cell[CellGroupData[{
Cell[615709, 20233, 72, 1, 27, "Input"],
Cell[615784, 20236, 222, 6, 28, "Output"]
}, Open  ]],
Cell[616021, 20245, 908, 38, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[616966, 20288, 33, 0, 48, "Subsubsection"],
Cell[617002, 20290, 239, 8, 32, "Text"],

Cell[CellGroupData[{
Cell[617266, 20302, 66, 1, 27, "Input"],
Cell[617335, 20305, 234, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[617606, 20316, 134, 3, 27, "Input"],
Cell[617743, 20321, 560, 16, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[618340, 20342, 93, 2, 27, "Input"],
Cell[618436, 20346, 560, 16, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[619033, 20367, 58, 1, 27, "Input"],
Cell[619094, 20370, 139, 3, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[619270, 20378, 105, 2, 27, "Input"],
Cell[619378, 20382, 106, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[619521, 20389, 177, 5, 27, "Input"],
Cell[619701, 20396, 166, 4, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[619928, 20407, 58, 1, 56, "Subsection",
  CellTags->"DiracTrace"],

Cell[CellGroupData[{
Cell[620011, 20412, 36, 0, 48, "Subsubsection"],
Cell[620050, 20414, 296, 5, 68, "Text"],

Cell[CellGroupData[{
Cell[620371, 20423, 72, 1, 27, "Input"],
Cell[620446, 20426, 635, 13, 46, "Output"]
}, Open  ]],
Cell[621096, 20442, 279, 11, 32, "Text"],
Cell[621378, 20455, 149, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[621564, 20468, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[621622, 20472, 136, 3, 27, "Input"],
Cell[621761, 20477, 260, 7, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[622058, 20489, 177, 4, 27, "Input"],
Cell[622238, 20495, 374, 9, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[622649, 20509, 106, 2, 27, "Input"],
Cell[622758, 20513, 814, 20, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[623609, 20538, 286, 7, 27, "Input"],
Cell[623898, 20547, 725, 21, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[624660, 20573, 320, 7, 27, "Input"],
Cell[624983, 20582, 13033, 320, 65, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[638053, 20907, 148, 3, 27, "Input"],
Cell[638204, 20912, 766, 21, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[639007, 20938, 272, 6, 27, "Input"],
Cell[639282, 20946, 167, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[639486, 20956, 64, 1, 27, "Input"],
Cell[639553, 20959, 123, 3, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[639737, 20969, 58, 1, 56, "Subsection",
  CellTags->"DiracTrick"],

Cell[CellGroupData[{
Cell[639820, 20974, 36, 0, 48, "Subsubsection"],
Cell[639859, 20976, 197, 4, 50, "Text"],

Cell[CellGroupData[{
Cell[640081, 20984, 72, 1, 27, "Input"],
Cell[640156, 20987, 136, 4, 28, "Output"]
}, Open  ]],
Cell[640307, 20994, 432, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[640776, 21017, 33, 0, 48, "Subsubsection"],
Cell[640812, 21019, 71, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[640908, 21023, 137, 3, 27, "Input"],
Cell[641048, 21028, 219, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[641304, 21038, 67, 1, 27, "Input"],
Cell[641374, 21041, 257, 8, 28, "Output"]
}, Open  ]],
Cell[641646, 21052, 68, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[641739, 21056, 138, 3, 27, "Input"],
Cell[641880, 21061, 219, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[642136, 21071, 67, 1, 27, "Input"],
Cell[642206, 21074, 293, 9, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[642536, 21088, 133, 3, 27, "Input"],
Cell[642672, 21093, 215, 5, 29, "Output"]
}, Open  ]],
Cell[642902, 21101, 163, 6, 32, "Text"],

Cell[CellGroupData[{
Cell[643090, 21111, 67, 1, 27, "Input"],
Cell[643160, 21114, 427, 13, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[643624, 21132, 129, 3, 27, "Input"],
Cell[643756, 21137, 211, 5, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[644004, 21147, 67, 1, 27, "Input"],
Cell[644074, 21150, 270, 8, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[644381, 21163, 312, 8, 27, "Input"],
Cell[644696, 21173, 885, 25, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[645618, 21203, 67, 1, 27, "Input"],
Cell[645688, 21206, 848, 22, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[646573, 21233, 61, 1, 27, "Input"],
Cell[646637, 21236, 2814, 78, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[649488, 21319, 382, 10, 27, "Input"],
Cell[649873, 21331, 654, 16, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[650564, 21352, 66, 1, 27, "Input"],
Cell[650633, 21355, 24743, 617, 118, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[675437, 21979, 60, 1, 56, "Subsection",
  CellTags->"DotSimplify"],

Cell[CellGroupData[{
Cell[675522, 21984, 36, 0, 48, "Subsubsection"],
Cell[675561, 21986, 299, 6, 68, "Text"],

Cell[CellGroupData[{
Cell[675885, 21996, 73, 1, 27, "Input"],
Cell[675961, 21999, 311, 8, 28, "Output"]
}, Open  ]],
Cell[676287, 22010, 417, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[676741, 22033, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[676799, 22037, 341, 9, 27, "Input"],
Cell[677143, 22048, 487, 13, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[677667, 22066, 68, 1, 27, "Input"],
Cell[677738, 22069, 651, 17, 28, "Output"]
}, Open  ]],
Cell[678404, 22089, 114, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[678543, 22095, 183, 5, 27, "Input"],
Cell[678729, 22102, 192, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[678958, 22113, 68, 1, 27, "Input"],
Cell[679029, 22116, 195, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[679261, 22126, 122, 3, 27, "Input"],
Cell[679386, 22131, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[679482, 22137, 68, 1, 27, "Input"],
Cell[679553, 22140, 294, 9, 29, "Output"]
}, Open  ]],
Cell[679862, 22152, 122, 3, 27, "Input"],

Cell[CellGroupData[{
Cell[680009, 22159, 68, 1, 27, "Input"],
Cell[680080, 22162, 195, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[680312, 22172, 126, 3, 27, "Input"],
Cell[680441, 22177, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[680537, 22183, 68, 1, 27, "Input"],
Cell[680608, 22186, 256, 7, 29, "Output"]
}, Open  ]],
Cell[680879, 22196, 126, 3, 27, "Input"],

Cell[CellGroupData[{
Cell[681030, 22203, 289, 7, 27, "Input"],
Cell[681322, 22212, 128, 4, 28, "Output"]
}, Open  ]],
Cell[681465, 22219, 77, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[681567, 22224, 104, 2, 27, "Input"],
Cell[681674, 22228, 87, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[681798, 22235, 179, 4, 27, "Input"],
Cell[681980, 22241, 96, 2, 28, "Output"]
}, Open  ]],
Cell[682091, 22246, 126, 2, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[682266, 22254, 60, 1, 56, "Subsection",
  CellTags->"EpsChisholm"],

Cell[CellGroupData[{
Cell[682351, 22259, 36, 0, 48, "Subsubsection"],
Cell[682390, 22261, 140, 3, 50, "Text"],
Cell[682533, 22266, 167, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[682737, 22279, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[682795, 22283, 175, 4, 27, "Input"],
Cell[682973, 22289, 2679, 77, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[685689, 22371, 67, 1, 27, "Input"],
Cell[685759, 22374, 698, 22, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[686518, 22403, 60, 1, 56, "Subsection",
  CellTags->"EpsEvaluate"],

Cell[CellGroupData[{
Cell[686603, 22408, 36, 0, 48, "Subsubsection"],
Cell[686642, 22410, 147, 3, 50, "Text"],
Cell[686792, 22415, 506, 23, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[687335, 22443, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[687393, 22447, 330, 9, 27, "Input"],
Cell[687726, 22458, 587, 18, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[688350, 22481, 67, 1, 27, "Input"],
Cell[688420, 22484, 1140, 34, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[689597, 22523, 68, 1, 27, "Input"],
Cell[689668, 22526, 613, 13, 72, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[690342, 22546, 52, 1, 56, "Subsection",
  CellTags->"Epsilon"],

Cell[CellGroupData[{
Cell[690419, 22551, 36, 0, 48, "Subsubsection"],
Cell[690458, 22553, 224, 8, 32, "Text"],
Cell[690685, 22563, 164, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[690886, 22576, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[690944, 22580, 33, 0, 27, "Input"],
Cell[690980, 22582, 113, 3, 28, "Output"]
}, Open  ]],
Cell[691108, 22588, 94, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[691227, 22595, 630, 17, 43, "Input"],
Cell[691860, 22614, 169, 4, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[692090, 22625, 44, 1, 56, "Subsection",
  CellTags->"Eps"],

Cell[CellGroupData[{
Cell[692159, 22630, 36, 0, 48, "Subsubsection"],
Cell[692198, 22632, 341, 8, 68, "Text"],

Cell[CellGroupData[{
Cell[692564, 22644, 65, 1, 27, "Input"],
Cell[692632, 22647, 132, 4, 28, "Output"]
}, Open  ]],
Cell[692779, 22654, 506, 23, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[693322, 22682, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[693380, 22686, 306, 6, 43, "Input"],
Cell[693689, 22694, 616, 19, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[694342, 22718, 298, 6, 27, "Input"],
Cell[694643, 22726, 557, 17, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[695237, 22748, 154, 4, 27, "Input"],
Cell[695394, 22754, 131, 3, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[695562, 22762, 106, 2, 27, "Input"],
Cell[695671, 22766, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[695767, 22772, 106, 2, 27, "Input"],
Cell[695876, 22776, 81, 2, 28, "Output"]
}, Open  ]],
Cell[695972, 22781, 164, 4, 27, "Input"],

Cell[CellGroupData[{
Cell[696161, 22789, 481, 11, 43, "Input"],
Cell[696645, 22802, 616, 19, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[697298, 22826, 88, 2, 27, "Input"],
Cell[697389, 22830, 82, 2, 28, "Output"]
}, Open  ]],
Cell[697486, 22835, 164, 4, 27, "Input"],

Cell[CellGroupData[{
Cell[697675, 22843, 481, 11, 43, "Input"],
Cell[698159, 22856, 616, 19, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[698812, 22880, 124, 3, 27, "Input"],
Cell[698939, 22885, 289, 9, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[699265, 22899, 501, 14, 42, "Input"],
Cell[699769, 22915, 1482, 43, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[701288, 22963, 1151, 27, 42, "Input"],
Cell[702442, 22992, 2202, 59, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[704681, 23056, 163, 4, 27, "Input"],
Cell[704847, 23062, 353, 11, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[705237, 23078, 88, 2, 27, "Input"],
Cell[705328, 23082, 59, 1, 28, "Output"]
}, Open  ]],
Cell[705402, 23086, 128, 3, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[705579, 23095, 66, 1, 56, "Subsection",
  CellTags->"ExpandPartialD"],

Cell[CellGroupData[{
Cell[705670, 23100, 36, 0, 48, "Subsubsection"],
Cell[705709, 23102, 158, 4, 50, "Text"],
Cell[705870, 23108, 595, 22, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[706502, 23135, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[706560, 23139, 356, 8, 43, "Input"],
Cell[706919, 23149, 351, 10, 34, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[707307, 23164, 70, 1, 27, "Input"],
Cell[707380, 23167, 948, 28, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[708365, 23200, 64, 1, 27, "Input"],
Cell[708432, 23203, 872, 22, 88, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[709341, 23230, 227, 5, 27, "Input"],
Cell[709571, 23237, 313, 9, 34, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[709921, 23251, 70, 1, 27, "Input"],
Cell[709994, 23254, 785, 23, 44, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[710816, 23282, 433, 11, 43, "Input"],
Cell[711252, 23295, 321, 8, 35, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[711610, 23308, 70, 1, 27, "Input"],
Cell[711683, 23311, 1653, 45, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[713373, 23361, 139, 4, 27, "Input"],
Cell[713515, 23367, 246, 6, 34, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[713798, 23378, 72, 1, 27, "Input"],
Cell[713873, 23381, 1245, 32, 37, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[715155, 23418, 70, 1, 27, "Input"],
Cell[715228, 23421, 1559, 41, 34, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[716824, 23467, 290, 7, 27, "Input"],
Cell[717117, 23476, 1543, 45, 34, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[718697, 23526, 70, 1, 27, "Input"],
Cell[718770, 23529, 59, 1, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[718890, 23537, 76, 1, 56, "Subsection",
  CellTags->"ExpandScalarProduct"],

Cell[CellGroupData[{
Cell[718991, 23542, 36, 0, 48, "Subsubsection"],
Cell[719030, 23544, 152, 3, 50, "Text"],
Cell[719185, 23549, 155, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[719377, 23562, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[719435, 23566, 114, 3, 27, "Input"],
Cell[719552, 23571, 297, 8, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[719886, 23584, 75, 1, 27, "Input"],
Cell[719964, 23587, 337, 10, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[720338, 23602, 118, 3, 27, "Input"],
Cell[720459, 23607, 204, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[720700, 23618, 75, 1, 27, "Input"],
Cell[720778, 23621, 468, 16, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[721283, 23642, 179, 5, 27, "Input"],
Cell[721465, 23649, 366, 9, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[721868, 23663, 107, 2, 27, "Input"],
Cell[721978, 23667, 697, 20, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[722736, 23694, 54, 1, 56, "Subsection",
  CellTags->"Explicit"],

Cell[CellGroupData[{
Cell[722815, 23699, 36, 0, 48, "Subsubsection"],
Cell[722854, 23701, 367, 9, 68, "Text"],
Cell[723224, 23712, 328, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[723589, 23730, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[723647, 23734, 198, 4, 27, "Input"],
Cell[723848, 23740, 1377, 41, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[725262, 23786, 64, 1, 27, "Input"],
Cell[725329, 23789, 4451, 127, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[729817, 23921, 143, 3, 27, "Input"],
Cell[729963, 23926, 1139, 34, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[731139, 23965, 64, 1, 27, "Input"],
Cell[731206, 23968, 4098, 111, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[735341, 24084, 114, 2, 27, "Input"],
Cell[735458, 24088, 144, 4, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[735639, 24097, 64, 1, 27, "Input"],
Cell[735706, 24100, 2016, 64, 31, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[737783, 24171, 70, 1, 56, "Subsection",
  CellTags->"ExplicitPartialD"],

Cell[CellGroupData[{
Cell[737878, 24176, 36, 0, 48, "Subsubsection"],
Cell[737917, 24178, 131, 3, 50, "Text"],
Cell[738051, 24183, 473, 17, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[738561, 24205, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[738619, 24209, 134, 2, 27, "Input"],
Cell[738756, 24213, 602, 17, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[739395, 24235, 135, 2, 27, "Input"],
Cell[739533, 24239, 467, 14, 34, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[740037, 24258, 140, 3, 27, "Input"],
Cell[740180, 24263, 496, 13, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[740713, 24281, 140, 4, 27, "Input"],
Cell[740856, 24287, 247, 6, 34, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[741164, 24300, 70, 1, 56, "Subsection",
  CellTags->"ExplicitSUNIndex"],

Cell[CellGroupData[{
Cell[741259, 24305, 36, 0, 48, "Subsubsection"],
Cell[741298, 24307, 99, 3, 32, "Text"],
Cell[741400, 24312, 490, 22, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[741927, 24339, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[741985, 24343, 72, 1, 27, "Input"],
Cell[742060, 24346, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[742156, 24352, 173, 5, 27, "Input"],
Cell[742332, 24359, 182, 4, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[742575, 24370, 52, 1, 56, "Subsection",
  CellTags->"Factor2"],

Cell[CellGroupData[{
Cell[742652, 24375, 36, 0, 48, "Subsubsection"],
Cell[742691, 24377, 376, 6, 86, "Text"],

Cell[CellGroupData[{
Cell[743092, 24387, 69, 1, 27, "Input"],
Cell[743164, 24390, 210, 6, 28, "Output"]
}, Open  ]],
Cell[743389, 24399, 167, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[743593, 24412, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[743651, 24416, 195, 6, 27, "Input"],
Cell[743849, 24424, 206, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[744092, 24435, 199, 5, 27, "Input"],
Cell[744294, 24442, 484, 15, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[744815, 24462, 242, 7, 27, "Input"],
Cell[745060, 24471, 146, 4, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[745243, 24480, 63, 1, 27, "Input"],
Cell[745309, 24483, 206, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[745552, 24494, 64, 1, 27, "Input"],
Cell[745619, 24497, 146, 4, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[745802, 24506, 138, 3, 27, "Input"],
Cell[745943, 24511, 206, 6, 28, "Output"]
}, Open  ]],
Cell[746164, 24520, 101, 2, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[746314, 24528, 44, 1, 56, "Subsection",
  CellTags->"FAD"],

Cell[CellGroupData[{
Cell[746383, 24533, 36, 0, 48, "Subsubsection"],
Cell[746422, 24535, 319, 6, 86, "Text"],
Cell[746744, 24543, 840, 33, 50, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[747621, 24581, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[747679, 24585, 115, 3, 27, "Input"],
Cell[747797, 24590, 862, 22, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[748696, 24617, 179, 5, 27, "Input"],
Cell[748878, 24624, 996, 24, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[749911, 24653, 239, 7, 27, "Input"],
Cell[750153, 24662, 116, 3, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[750306, 24670, 201, 6, 27, "Input"],
Cell[750510, 24678, 560, 14, 56, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[751107, 24697, 273, 7, 27, "Input"],
Cell[751383, 24706, 560, 14, 56, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[752004, 24727, 44, 1, 56, "Subsection",
  CellTags->"FCE"],

Cell[CellGroupData[{
Cell[752073, 24732, 36, 0, 48, "Subsubsection"],
Cell[752112, 24734, 108, 3, 32, "Text"],
Cell[752223, 24739, 54, 0, 32, "Text"],
Cell[752280, 24741, 438, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[752755, 24764, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[752813, 24768, 253, 7, 27, "Input"],
Cell[753069, 24777, 251, 7, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[753357, 24789, 64, 1, 27, "Input"],
Cell[753424, 24792, 150, 4, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[753611, 24801, 449, 11, 27, "Input"],
Cell[754063, 24814, 720, 19, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[754820, 24838, 64, 1, 27, "Input"],
Cell[754887, 24841, 450, 11, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[755374, 24857, 55, 1, 27, "Input"],
Cell[755432, 24860, 1163, 34, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[756632, 24899, 64, 1, 27, "Input"],
Cell[756699, 24902, 966, 22, 88, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[757702, 24929, 98, 2, 27, "Input"],
Cell[757803, 24933, 450, 11, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[758314, 24951, 44, 1, 56, "Subsection",
  CellTags->"FCI"],

Cell[CellGroupData[{
Cell[758383, 24956, 36, 0, 48, "Subsubsection"],
Cell[758422, 24958, 141, 3, 50, "Text"],
Cell[758566, 24963, 438, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[759041, 24986, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[759099, 24990, 449, 11, 27, "Input"],
Cell[759551, 25003, 720, 19, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[760308, 25027, 64, 1, 27, "Input"],
Cell[760375, 25030, 450, 11, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[760862, 25046, 55, 1, 27, "Input"],
Cell[760920, 25049, 1163, 34, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[762120, 25088, 64, 1, 27, "Input"],
Cell[762187, 25091, 966, 22, 88, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[763190, 25118, 98, 2, 27, "Input"],
Cell[763291, 25122, 450, 11, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[763802, 25140, 66, 1, 56, "Subsection",
  CellTags->"FermionSpinSum"],

Cell[CellGroupData[{
Cell[763893, 25145, 36, 0, 48, "Subsubsection"],
Cell[763932, 25147, 91, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[764048, 25154, 76, 1, 27, "Input"],
Cell[764127, 25157, 294, 7, 28, "Output"]
}, Open  ]],
Cell[764436, 25167, 417, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[764890, 25190, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[764948, 25194, 319, 8, 27, "Input"],
Cell[765270, 25204, 754, 18, 29, "Output"]
}, Open  ]],
Cell[766039, 25225, 135, 3, 50, "Text"],

Cell[CellGroupData[{
Cell[766199, 25232, 100, 2, 27, "Input"],
Cell[766302, 25236, 810, 19, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[767149, 25260, 98, 2, 27, "Input"],
Cell[767250, 25264, 1542, 38, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[768829, 25307, 96, 2, 27, "Input"],
Cell[768928, 25311, 1181, 34, 29, "Output"]
}, Open  ]],
Cell[770124, 25348, 94, 2, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[770267, 25356, 88, 1, 56, "Subsection",
  CellTags->"FeynAmpDenominatorCombine"],

Cell[CellGroupData[{
Cell[770380, 25361, 36, 0, 48, "Subsubsection"],
Cell[770419, 25363, 188, 4, 50, "Text"],
Cell[770610, 25369, 212, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[770859, 25382, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[770917, 25386, 192, 5, 27, "Input"],
Cell[771112, 25393, 925, 25, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[772074, 25423, 159, 4, 27, "Input"],
Cell[772236, 25429, 142, 4, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[772415, 25438, 157, 4, 27, "Input"],
Cell[772575, 25444, 176, 5, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[772812, 25456, 74, 1, 56, "Subsection",
  CellTags->"FeynAmpDenominator"],

Cell[CellGroupData[{
Cell[772911, 25461, 36, 0, 48, "Subsubsection"],
Cell[772950, 25463, 226, 5, 50, "Text"],
Cell[773179, 25470, 325, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[773541, 25488, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[773599, 25492, 155, 3, 27, "Input"],
Cell[773757, 25497, 177, 5, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[773971, 25507, 304, 7, 43, "Input"],
Cell[774278, 25516, 464, 14, 46, "Output"]
}, Open  ]],
Cell[774757, 25533, 214, 5, 27, "Input"],

Cell[CellGroupData[{
Cell[774996, 25542, 98, 2, 27, "Input"],
Cell[775097, 25546, 235, 6, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[775369, 25557, 98, 2, 27, "Input"],
Cell[775470, 25561, 116, 3, 40, "Output"]
}, Open  ]],
Cell[775601, 25567, 83, 2, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[775733, 25575, 90, 1, 56, "Subsection",
  CellTags->"FeynAmpDenominatorSimplify"],

Cell[CellGroupData[{
Cell[775848, 25580, 36, 0, 48, "Subsubsection"],
Cell[775887, 25582, 357, 6, 86, "Text"],
Cell[776247, 25590, 44, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[776316, 25594, 29, 0, 27, "Input"],
Cell[776348, 25596, 84, 1, 28, "Output"]
}, Open  ]],
Cell[776447, 25600, 188, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[776672, 25613, 33, 0, 48, "Subsubsection"],
Cell[776708, 25615, 465, 13, 32, "Text"],

Cell[CellGroupData[{
Cell[777198, 25632, 245, 6, 27, "Input"],
Cell[777446, 25640, 59, 1, 28, "Output"]
}, Open  ]],
Cell[777520, 25644, 533, 17, 32, "Text"],

Cell[CellGroupData[{
Cell[778078, 25665, 310, 8, 27, "Input"],
Cell[778391, 25675, 330, 10, 47, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[778758, 25690, 469, 12, 27, "Input"],
Cell[779230, 25704, 1354, 39, 49, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[780621, 25748, 167, 4, 27, "Input"],
Cell[780791, 25754, 973, 28, 49, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[781801, 25787, 98, 2, 27, "Input"],
Cell[781902, 25791, 1091, 29, 49, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[783030, 25825, 375, 11, 27, "Input"],
Cell[783408, 25838, 1948, 54, 49, "Output"]
}, Open  ]],
Cell[785371, 25895, 88, 2, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[785508, 25903, 84, 1, 56, "Subsection",
  CellTags->"FeynAmpDenominatorSplit"],

Cell[CellGroupData[{
Cell[785617, 25908, 36, 0, 48, "Subsubsection"],
Cell[785656, 25910, 319, 6, 86, "Text"],
Cell[785978, 25918, 218, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[786233, 25931, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[786291, 25935, 284, 7, 27, "Input"],
Cell[786578, 25944, 1059, 32, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[787674, 25981, 56, 1, 27, "Input"],
Cell[787733, 25984, 76, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[787846, 25990, 79, 1, 27, "Input"],
Cell[787928, 25993, 1059, 32, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[789024, 26030, 94, 2, 27, "Input"],
Cell[789121, 26034, 398, 11, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[789556, 26050, 189, 5, 27, "Input"],
Cell[789748, 26057, 344, 10, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[790129, 26072, 159, 4, 27, "Input"],
Cell[790291, 26078, 262, 7, 40, "Output"]
}, Open  ]],
Cell[790568, 26088, 61, 1, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[790678, 26095, 52, 1, 56, "Subsection",
  CellTags->"FeynAmp"],

Cell[CellGroupData[{
Cell[790755, 26100, 36, 0, 48, "Subsubsection"],
Cell[790794, 26102, 221, 4, 50, "Text"],
Cell[791018, 26108, 143, 3, 50, "Text"],
Cell[791164, 26113, 173, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[791374, 26126, 33, 0, 48, "Subsubsection"],
Cell[791410, 26128, 103, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[791538, 26135, 935, 20, 43, "Input"],
Cell[792476, 26157, 5508, 137, 52, "Output"]
}, Open  ]],
Cell[797999, 26297, 51, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[798075, 26301, 168, 4, 27, "Input"],
Cell[798246, 26307, 439, 12, 41, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[798746, 26326, 64, 1, 56, "Subsection",
  CellTags->"FeynCalc2FORM"],

Cell[CellGroupData[{
Cell[798835, 26331, 36, 0, 48, "Subsubsection"],
Cell[798874, 26333, 198, 4, 50, "Text"],
Cell[799075, 26339, 182, 8, 32, "Text"],

Cell[CellGroupData[{
Cell[799282, 26351, 75, 1, 27, "Input"],
Cell[799360, 26354, 1099, 21, 64, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[800508, 26381, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[800566, 26385, 250, 7, 27, "Input"],
Cell[800819, 26394, 281, 8, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[801137, 26407, 91, 2, 27, "Input"],
Cell[801231, 26411, 88, 1, 24, "Print"]
}, Open  ]],

Cell[CellGroupData[{
Cell[801356, 26417, 163, 4, 27, "Input"],
Cell[801522, 26423, 217, 5, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[801776, 26433, 91, 2, 27, "Input"],
Cell[801870, 26437, 84, 1, 24, "Print"]
}, Open  ]],

Cell[CellGroupData[{
Cell[801991, 26443, 177, 4, 27, "Input"],
Cell[802171, 26449, 374, 9, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[802582, 26463, 91, 2, 27, "Input"],
Cell[802676, 26467, 103, 2, 24, "Print"]
}, Open  ]],

Cell[CellGroupData[{
Cell[802816, 26474, 278, 7, 27, "Input"],
Cell[803097, 26483, 513, 14, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[803647, 26502, 91, 2, 27, "Input"],
Cell[803741, 26506, 103, 2, 24, "Print"]
}, Open  ]],
Cell[803859, 26511, 323, 8, 27, "Input"],

Cell[CellGroupData[{
Cell[804207, 26523, 328, 8, 43, "Input"],
Cell[804538, 26533, 111, 3, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[804686, 26541, 307, 8, 27, "Input"],
Cell[804996, 26551, 8847, 242, 64, "Output"]
}, Open  ]],
Cell[813858, 26796, 174, 4, 27, "Input"],

Cell[CellGroupData[{
Cell[814057, 26804, 382, 9, 59, "Input"],
Cell[814442, 26815, 1655, 40, 273, "Output"]
}, Open  ]],
Cell[816112, 26858, 302, 8, 27, "Input"],
Cell[816417, 26868, 83, 2, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[816549, 26876, 70, 1, 56, "Subsection",
  CellTags->"FeynCalcExternal"],

Cell[CellGroupData[{
Cell[816644, 26881, 36, 0, 48, "Subsubsection"],
Cell[816683, 26883, 125, 3, 50, "Text"],
Cell[816811, 26888, 191, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[817039, 26901, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[817097, 26905, 113, 2, 27, "Input"],
Cell[817213, 26909, 94, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[817344, 26916, 64, 1, 27, "Input"],
Cell[817411, 26919, 59, 1, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[817507, 26925, 449, 11, 27, "Input"],
Cell[817959, 26938, 720, 19, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[818716, 26962, 64, 1, 27, "Input"],
Cell[818783, 26965, 450, 11, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[819270, 26981, 68, 1, 27, "Input"],
Cell[819341, 26984, 1163, 34, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[820541, 27023, 64, 1, 27, "Input"],
Cell[820608, 27026, 966, 22, 88, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[821611, 27053, 114, 3, 27, "Input"],
Cell[821728, 27058, 450, 11, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[822239, 27076, 70, 1, 56, "Subsection",
  CellTags->"FeynCalcInternal"],

Cell[CellGroupData[{
Cell[822334, 27081, 36, 0, 48, "Subsubsection"],
Cell[822373, 27083, 126, 3, 50, "Text"],
Cell[822502, 27088, 399, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[822938, 27111, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[822996, 27115, 449, 11, 27, "Input"],
Cell[823448, 27128, 720, 19, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[824205, 27152, 64, 1, 27, "Input"],
Cell[824272, 27155, 450, 11, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[824759, 27171, 68, 1, 27, "Input"],
Cell[824830, 27174, 1163, 34, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[826030, 27213, 64, 1, 27, "Input"],
Cell[826097, 27216, 966, 22, 88, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[827100, 27243, 114, 3, 27, "Input"],
Cell[827217, 27248, 450, 11, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[827704, 27264, 540, 13, 27, "Input"],
Cell[828247, 27279, 2074, 61, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[830358, 27345, 64, 1, 27, "Input"],
Cell[830425, 27348, 1241, 29, 104, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[831727, 27384, 54, 1, 56, "Subsection",
  CellTags->"FeynRule"],

Cell[CellGroupData[{
Cell[831806, 27389, 36, 0, 48, "Subsubsection"],
Cell[831845, 27391, 145, 3, 50, "Text"],

Cell[CellGroupData[{
Cell[832015, 27398, 70, 1, 27, "Input"],
Cell[832088, 27401, 663, 15, 46, "Output"]
}, Open  ]],
Cell[832766, 27419, 69, 0, 32, "Text"],
Cell[832838, 27421, 110, 3, 32, "Text"],
Cell[832951, 27426, 173, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[833161, 27439, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[833219, 27443, 105, 2, 27, "Input"],
Cell[833327, 27447, 824, 21, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[834188, 27473, 105, 2, 27, "Input"],
Cell[834296, 27477, 1316, 34, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[835649, 27516, 66, 1, 27, "Input"],
Cell[835718, 27519, 5737, 154, 66, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[841492, 27678, 100, 2, 27, "Input"],
Cell[841595, 27682, 3595, 111, 32, "Output"]
}, Open  ]],
Cell[845205, 27796, 51, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[845281, 27800, 552, 14, 43, "Input"],
Cell[845836, 27816, 892, 28, 32, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[846765, 27849, 298, 7, 43, "Input"],
Cell[847066, 27858, 5983, 151, 48, "Output"]
}, Open  ]],
Cell[853064, 28012, 49, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[853138, 28016, 552, 14, 43, "Input"],
Cell[853693, 28032, 892, 28, 32, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[854622, 28065, 298, 7, 43, "Input"],
Cell[854923, 28074, 7250, 162, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[862210, 28241, 195, 5, 27, "Input"],
Cell[862408, 28248, 1404, 41, 29, "Output"]
}, Open  ]],
Cell[863827, 28292, 89, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[863941, 28299, 351, 8, 43, "Input"],
Cell[864295, 28309, 1506, 44, 29, "Output"]
}, Open  ]],
Cell[865816, 28356, 54, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[865895, 28360, 104, 2, 27, "Input"],
Cell[866002, 28364, 757, 21, 29, "Output"]
}, Open  ]],
Cell[866774, 28388, 67, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[866866, 28392, 104, 2, 27, "Input"],
Cell[866973, 28396, 757, 21, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[867767, 28422, 792, 20, 75, "Input"],
Cell[868562, 28444, 1249, 37, 32, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[869848, 28486, 198, 5, 27, "Input"],
Cell[870049, 28493, 7263, 183, 114, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[877349, 28681, 428, 10, 43, "Input"],
Cell[877780, 28693, 7267, 163, 132, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[885084, 28861, 372, 10, 27, "Input"],
Cell[885459, 28873, 59, 1, 28, "Output"]
}, Open  ]],
Cell[885533, 28877, 187, 4, 68, "Text"],
Cell[885723, 28883, 103, 2, 27, "Input"],
Cell[885829, 28887, 28, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[885882, 28891, 781, 19, 59, "Input"],
Cell[886666, 28912, 1288, 40, 32, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[887991, 28957, 187, 5, 27, "Input"],
Cell[888181, 28964, 2778, 76, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[890996, 29045, 382, 9, 27, "Input"],
Cell[891381, 29056, 2927, 82, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[894345, 29143, 167, 4, 27, "Input"],
Cell[894515, 29149, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[894611, 29155, 1011, 24, 75, "Input"],
Cell[895625, 29181, 1686, 52, 32, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[897348, 29238, 187, 5, 27, "Input"],
Cell[897538, 29245, 4770, 131, 47, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[902345, 29381, 469, 11, 43, "Input"],
Cell[902817, 29394, 9042, 226, 47, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[911896, 29625, 167, 4, 27, "Input"],
Cell[912066, 29631, 59, 1, 28, "Output"]
}, Open  ]],
Cell[912140, 29635, 402, 7, 43, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[912591, 29648, 64, 1, 56, "Subsection",
  CellTags->"FieldStrength"],

Cell[CellGroupData[{
Cell[912680, 29653, 36, 0, 48, "Subsubsection"],
Cell[912719, 29655, 1424, 36, 86, "Text"],

Cell[CellGroupData[{
Cell[914168, 29695, 75, 1, 27, "Input"],
Cell[914246, 29698, 499, 12, 46, "Output"]
}, Open  ]],
Cell[914760, 29713, 179, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[914976, 29726, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[915034, 29730, 104, 2, 27, "Input"],
Cell[915141, 29734, 154, 4, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[915332, 29743, 114, 2, 27, "Input"],
Cell[915449, 29747, 144, 4, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[915630, 29756, 159, 3, 27, "Input"],
Cell[915792, 29761, 888, 28, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[916717, 29794, 169, 3, 27, "Input"],
Cell[916889, 29799, 2018, 64, 31, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[918944, 29868, 206, 4, 27, "Input"],
Cell[919153, 29874, 551, 13, 72, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[919741, 29892, 209, 4, 27, "Input"],
Cell[919953, 29898, 549, 13, 72, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[920539, 29916, 261, 5, 27, "Input"],
Cell[920803, 29923, 2018, 64, 31, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[922882, 29994, 64, 1, 56, "Subsection",
  CellTags->"FORM2FeynCalc"],

Cell[CellGroupData[{
Cell[922971, 29999, 36, 0, 48, "Subsubsection"],
Cell[923010, 30001, 444, 7, 104, "Text"],
Cell[923457, 30010, 182, 8, 32, "Text"],

Cell[CellGroupData[{
Cell[923664, 30022, 75, 1, 27, "Input"],
Cell[923742, 30025, 705, 16, 46, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[924496, 30047, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[924554, 30051, 89, 1, 27, "Input"],
Cell[924646, 30054, 397, 12, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[925080, 30071, 64, 1, 27, "Input"],
Cell[925147, 30074, 182, 5, 42, "Output"]
}, Open  ]],
Cell[925344, 30082, 131, 3, 50, "Text"],

Cell[CellGroupData[{
Cell[925500, 30089, 288, 6, 43, "Input"],
Cell[925791, 30097, 304, 8, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[926132, 30110, 64, 1, 27, "Input"],
Cell[926199, 30113, 267, 7, 42, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[926503, 30125, 100, 2, 27, "Input"],
Cell[926606, 30129, 299, 9, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[926942, 30143, 67, 1, 27, "Input"],
Cell[927012, 30146, 188, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[927237, 30157, 100, 2, 27, "Input"],
Cell[927340, 30161, 231, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[927608, 30172, 64, 1, 27, "Input"],
Cell[927675, 30175, 222, 6, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[927934, 30186, 306, 7, 43, "Input"],
Cell[928243, 30195, 246, 7, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[928526, 30207, 153, 4, 43, "Input"],
Cell[928682, 30213, 591, 15, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[929334, 30235, 66, 1, 56, "Subsection",
  CellTags->"FourDivergence"],

Cell[CellGroupData[{
Cell[929425, 30240, 36, 0, 48, "Subsubsection"],
Cell[929464, 30242, 213, 4, 50, "Text"],
Cell[929680, 30248, 179, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[929896, 30261, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[929954, 30265, 125, 3, 27, "Input"],
Cell[930082, 30270, 219, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[930338, 30281, 175, 4, 27, "Input"],
Cell[930516, 30287, 236, 8, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[930789, 30300, 156, 4, 27, "Input"],
Cell[930948, 30306, 297, 8, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[931282, 30319, 208, 5, 27, "Input"],
Cell[931493, 30326, 59, 1, 28, "Output"]
}, Open  ]],
Cell[931567, 30330, 61, 1, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[931677, 30337, 64, 1, 56, "Subsection",
  CellTags->"FourLaplacian"],

Cell[CellGroupData[{
Cell[931766, 30342, 36, 0, 48, "Subsubsection"],
Cell[931805, 30344, 462, 15, 32, "Text"],

Cell[CellGroupData[{
Cell[932292, 30363, 75, 1, 27, "Input"],
Cell[932370, 30366, 132, 4, 28, "Output"]
}, Open  ]],
Cell[932517, 30373, 316, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[932870, 30391, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[932928, 30395, 85, 2, 27, "Input"],
Cell[933016, 30399, 134, 4, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[933187, 30408, 106, 2, 27, "Input"],
Cell[933296, 30412, 86, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[933419, 30419, 247, 7, 27, "Input"],
Cell[933669, 30428, 528, 16, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[934234, 30449, 111, 2, 27, "Input"],
Cell[934348, 30453, 5652, 152, 130, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[940061, 30612, 58, 1, 56, "Subsection",
  CellTags->"FourVector"],

Cell[CellGroupData[{
Cell[940144, 30617, 36, 0, 48, "Subsubsection"],
Cell[940183, 30619, 327, 9, 50, "Text"],
Cell[940513, 30630, 360, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[940910, 30653, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[940968, 30657, 97, 2, 27, "Input"],
Cell[941068, 30661, 89, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[941194, 30668, 126, 3, 27, "Input"],
Cell[941323, 30673, 284, 9, 32, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[941644, 30687, 142, 3, 27, "Input"],
Cell[941789, 30692, 98, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[941924, 30699, 197, 4, 27, "Input"],
Cell[942124, 30705, 151, 3, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[942312, 30713, 180, 4, 27, "Input"],
Cell[942495, 30719, 98, 2, 40, "Output"]
}, Open  ]],
Cell[942608, 30724, 170, 4, 50, "Text"],

Cell[CellGroupData[{
Cell[942803, 30732, 180, 4, 27, "Input"],
Cell[942986, 30738, 468, 16, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[943515, 30761, 50, 1, 56, "Subsection",
  CellTags->"FreeQ2"],

Cell[CellGroupData[{
Cell[943590, 30766, 36, 0, 48, "Subsubsection"],
Cell[943629, 30768, 207, 4, 50, "Text"],
Cell[943839, 30774, 307, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[944183, 30792, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[944241, 30796, 229, 6, 27, "Input"],
Cell[944473, 30804, 63, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[944573, 30810, 224, 6, 27, "Input"],
Cell[944800, 30818, 62, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[944899, 30824, 94, 2, 27, "Input"],
Cell[944996, 30828, 62, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[945095, 30834, 128, 3, 27, "Input"],
Cell[945226, 30839, 63, 1, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[945350, 30847, 52, 1, 56, "Subsection",
  CellTags->"FromTFi"],

Cell[CellGroupData[{
Cell[945427, 30852, 36, 0, 48, "Subsubsection"],
Cell[945466, 30854, 178, 4, 50, "Text"],
Cell[945647, 30860, 255, 12, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[945939, 30877, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[945997, 30881, 381, 10, 27, "Input"],
Cell[946381, 30893, 1084, 33, 31, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[947502, 30931, 801, 20, 59, "Input"],
Cell[948306, 30953, 1302, 39, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[949645, 30997, 879, 22, 59, "Input"],
Cell[950527, 31021, 1683, 49, 45, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[952271, 31077, 44, 1, 56, "Subsection",
  CellTags->"FRH"],

Cell[CellGroupData[{
Cell[952340, 31082, 36, 0, 48, "Subsubsection"],
Cell[952379, 31084, 116, 3, 32, "Text"],
Cell[952498, 31089, 164, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[952699, 31102, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[952757, 31106, 175, 4, 27, "Input"],
Cell[952935, 31112, 242, 7, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[953214, 31124, 59, 1, 27, "Input"],
Cell[953276, 31127, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[953372, 31133, 300, 8, 27, "Input"],
Cell[953675, 31143, 608, 18, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[954320, 31166, 91, 2, 27, "Input"],
Cell[954414, 31170, 625, 19, 71, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[955100, 31196, 60, 1, 56, "Subsection",
  CellTags->"FunctionalD"],

Cell[CellGroupData[{
Cell[955185, 31201, 36, 0, 48, "Subsubsection"],
Cell[955224, 31203, 390, 7, 86, "Text"],
Cell[955617, 31212, 69, 0, 32, "Text"],
Cell[955689, 31214, 298, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[956024, 31232, 33, 0, 48, "Subsubsection"],
Cell[956060, 31234, 665, 19, 50, "Text"],

Cell[CellGroupData[{
Cell[956750, 31257, 194, 4, 27, "Input"],
Cell[956947, 31263, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[957043, 31269, 225, 5, 27, "Input"],
Cell[957271, 31276, 91, 2, 28, "Output"]
}, Open  ]],
Cell[957377, 31281, 1471, 43, 71, "Text"],

Cell[CellGroupData[{
Cell[958873, 31328, 283, 7, 27, "Input"],
Cell[959159, 31337, 272, 8, 34, "Output"]
}, Open  ]],
Cell[959446, 31348, 941, 23, 29, "Text"],

Cell[CellGroupData[{
Cell[960412, 31375, 767, 20, 59, "Input"],
Cell[961182, 31397, 889, 24, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[962108, 31426, 183, 4, 27, "Input"],
Cell[962294, 31432, 758, 23, 30, "Output"]
}, Open  ]],
Cell[963067, 31458, 563, 15, 33, "Text"],
Cell[963633, 31475, 72, 1, 32, "Text"],

Cell[CellGroupData[{
Cell[963730, 31480, 295, 6, 27, "Input"],
Cell[964028, 31488, 1742, 55, 32, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[965807, 31548, 295, 6, 27, "Input"],
Cell[966105, 31556, 1742, 55, 32, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[967884, 31616, 192, 6, 27, "Input"],
Cell[968079, 31624, 4244, 115, 43, "Output"]
}, Open  ]],
Cell[972338, 31742, 552, 12, 86, "Text"],
Cell[972893, 31756, 1899, 49, 30, "Text"],
Cell[974795, 31807, 507, 12, 29, "Text"],

Cell[CellGroupData[{
Cell[975327, 31823, 204, 5, 27, "Input"],
Cell[975534, 31830, 298, 10, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[975869, 31845, 126, 3, 27, "Input"],
Cell[975998, 31850, 3142, 93, 35, "Output"]
}, Open  ]],
Cell[979155, 31946, 100, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[979280, 31952, 137, 3, 27, "Input"],
Cell[979420, 31957, 3142, 93, 35, "Output"]
}, Open  ]],
Cell[982577, 32053, 276, 7, 50, "Text"],
Cell[982856, 32062, 566, 16, 43, "Input"],

Cell[CellGroupData[{
Cell[983447, 32082, 202, 5, 27, "Input"],
Cell[983652, 32089, 451, 15, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[984140, 32109, 213, 5, 27, "Input"],
Cell[984356, 32116, 501, 16, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[984894, 32137, 646, 15, 59, "Input"],
Cell[985543, 32154, 4084, 124, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[989664, 32283, 145, 3, 27, "Input"],
Cell[989812, 32288, 1938, 60, 31, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[991787, 32353, 352, 9, 27, "Input"],
Cell[992142, 32364, 1579, 49, 31, "Output"]
}, Open  ]],
Cell[993736, 32416, 267, 8, 33, "Text"],
Cell[994006, 32426, 73, 1, 32, "Text"],
Cell[994082, 32429, 141, 3, 50, "Text"],

Cell[CellGroupData[{
Cell[994248, 32436, 57, 1, 27, "Input"],
Cell[994308, 32439, 4244, 115, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[998589, 32559, 161, 4, 27, "Input"],
Cell[998753, 32565, 33658, 1002, 259, "Output"]
}, Open  ]],
Cell[1032426, 33570, 205, 4, 50, "Text"],
Cell[1032634, 33576, 321, 6, 68, "Text"],
Cell[1032958, 33584, 88, 2, 27, "Input"],
Cell[1033049, 33588, 3904, 75, 203, "Input"],

Cell[CellGroupData[{
Cell[1036978, 33667, 130, 3, 27, "Input"],
Cell[1037111, 33672, 17295, 523, 115, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1054443, 34200, 125, 3, 27, "Input"],
Cell[1054571, 34205, 9155, 283, 66, "Output"]
}, Open  ]],
Cell[1063741, 34491, 198, 7, 32, "Text"],

Cell[CellGroupData[{
Cell[1063964, 34502, 325, 9, 27, "Input"],
Cell[1064292, 34513, 3776, 117, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1068105, 34635, 752, 20, 27, "Input"],
Cell[1068860, 34657, 1781, 58, 32, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1070678, 34720, 110, 2, 27, "Input"],
Cell[1070791, 34724, 7275, 225, 54, "Output"]
}, Open  ]],
Cell[1078081, 34952, 105, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[1078211, 34959, 28, 0, 27, "Input"],
Cell[1078242, 34961, 1579, 49, 31, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1079858, 35015, 885, 20, 59, "Input"],
Cell[1080746, 35037, 1443, 40, 35, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1082226, 35082, 93, 2, 27, "Input"],
Cell[1082322, 35086, 4821, 132, 35, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1087180, 35223, 170, 4, 27, "Input"],
Cell[1087353, 35229, 8530, 264, 54, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1095920, 35498, 94, 2, 27, "Input"],
Cell[1096017, 35502, 11260, 348, 54, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1107314, 35855, 124, 3, 27, "Input"],
Cell[1107441, 35860, 59, 1, 28, "Output"]
}, Open  ]],
Cell[1107515, 35864, 40, 0, 32, "Text"],
Cell[1107558, 35866, 541, 11, 43, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1108136, 35882, 85, 1, 48, "Subsubsection"],
Cell[1108224, 35885, 53, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[1108302, 35889, 1460, 32, 139, "Input"],
Cell[1109765, 35923, 2068, 62, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1111870, 35990, 898, 22, 59, "Input"],
Cell[1112771, 36014, 13224, 326, 104, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1126032, 36345, 126, 3, 27, "Input"],
Cell[1126161, 36350, 2854, 80, 43, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1129076, 36437, 42, 1, 56, "Subsection",
  CellTags->"FV"],

Cell[CellGroupData[{
Cell[1129143, 36442, 36, 0, 48, "Subsubsection"],
Cell[1129182, 36444, 280, 12, 32, "Text"],
Cell[1129465, 36458, 592, 28, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1130094, 36491, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1130152, 36495, 89, 2, 27, "Input"],
Cell[1130244, 36499, 91, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1130372, 36506, 118, 3, 27, "Input"],
Cell[1130493, 36511, 204, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1130734, 36522, 130, 3, 27, "Input"],
Cell[1130867, 36527, 90, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1130994, 36534, 171, 5, 27, "Input"],
Cell[1131168, 36541, 178, 4, 40, "Output"]
}, Open  ]],
Cell[1131361, 36548, 113, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[1131499, 36555, 172, 4, 27, "Input"],
Cell[1131674, 36561, 468, 16, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1132179, 36582, 68, 1, 27, "Input"],
Cell[1132250, 36585, 369, 9, 56, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1132680, 36601, 44, 1, 56, "Subsection",
  CellTags->"FVD"],

Cell[CellGroupData[{
Cell[1132749, 36606, 36, 0, 48, "Subsubsection"],
Cell[1132788, 36608, 315, 14, 32, "Text"],
Cell[1133106, 36624, 589, 28, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1133732, 36657, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1133790, 36661, 90, 2, 27, "Input"],
Cell[1133883, 36665, 91, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1134011, 36672, 119, 3, 27, "Input"],
Cell[1134133, 36677, 204, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1134374, 36688, 131, 3, 27, "Input"],
Cell[1134508, 36693, 91, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1134636, 36700, 172, 5, 27, "Input"],
Cell[1134811, 36707, 240, 6, 40, "Output"]
}, Open  ]],
Cell[1135066, 36716, 70, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[1135161, 36720, 173, 4, 27, "Input"],
Cell[1135337, 36726, 468, 16, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1135842, 36747, 68, 1, 27, "Input"],
Cell[1135913, 36750, 501, 13, 56, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1136475, 36770, 42, 1, 56, "Subsection",
  CellTags->"GA"],

Cell[CellGroupData[{
Cell[1136542, 36775, 36, 0, 48, "Subsubsection"],
Cell[1136581, 36777, 690, 23, 68, "Text"],
Cell[1137274, 36802, 381, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1137692, 36825, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1137750, 36829, 62, 1, 27, "Input"],
Cell[1137815, 36832, 98, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1137950, 36839, 192, 5, 27, "Input"],
Cell[1138145, 36846, 316, 8, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1138498, 36859, 141, 3, 27, "Input"],
Cell[1138642, 36864, 114, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1138793, 36871, 132, 3, 27, "Input"],
Cell[1138928, 36876, 274, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1139239, 36887, 179, 4, 27, "Input"],
Cell[1139421, 36893, 225, 5, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1139683, 36903, 234, 6, 27, "Input"],
Cell[1139920, 36911, 346, 9, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1140327, 36927, 44, 1, 56, "Subsection",
  CellTags->"GAD"],

Cell[CellGroupData[{
Cell[1140396, 36932, 36, 0, 48, "Subsubsection"],
Cell[1140435, 36934, 706, 23, 68, "Text"],
Cell[1141144, 36959, 378, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1141559, 36982, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1141617, 36986, 63, 1, 27, "Input"],
Cell[1141683, 36989, 98, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1141818, 36996, 194, 5, 27, "Input"],
Cell[1142015, 37003, 316, 8, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1142368, 37016, 142, 3, 27, "Input"],
Cell[1142513, 37021, 174, 4, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1142724, 37030, 133, 3, 27, "Input"],
Cell[1142860, 37035, 274, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1143171, 37046, 180, 4, 27, "Input"],
Cell[1143354, 37052, 229, 5, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1143620, 37062, 237, 6, 27, "Input"],
Cell[1143860, 37070, 346, 9, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1144267, 37086, 62, 1, 56, "Subsection",
  CellTags->"GammaEpsilon"],

Cell[CellGroupData[{
Cell[1144354, 37091, 36, 0, 48, "Subsubsection"],
Cell[1144393, 37093, 140, 3, 50, "Text"],
Cell[1144536, 37098, 292, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1144865, 37116, 33, 0, 48, "Subsubsection"],
Cell[1144901, 37118, 114, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[1145040, 37125, 130, 3, 27, "Input"],
Cell[1145173, 37130, 2028, 56, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1147238, 37191, 130, 3, 27, "Input"],
Cell[1147371, 37196, 1743, 49, 47, "Output"]
}, Open  ]],
Cell[1149129, 37248, 64, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[1149218, 37252, 74, 1, 27, "Input"],
Cell[1149295, 37255, 2962, 77, 97, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1152294, 37337, 68, 1, 27, "Input"],
Cell[1152365, 37340, 105, 2, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1152531, 37349, 60, 1, 56, "Subsection",
  CellTags->"GammaExpand"],

Cell[CellGroupData[{
Cell[1152616, 37354, 36, 0, 48, "Subsubsection"],
Cell[1152655, 37356, 99, 3, 32, "Text"],
Cell[1152757, 37361, 179, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1152973, 37374, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1153031, 37378, 148, 3, 27, "Input"],
Cell[1153182, 37383, 360, 11, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1153579, 37399, 164, 4, 27, "Input"],
Cell[1153746, 37405, 742, 21, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1154525, 37431, 148, 3, 27, "Input"],
Cell[1154676, 37436, 196, 5, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1154933, 37448, 68, 1, 56, "Subsection",
  CellTags->"GhostPropagator"],

Cell[CellGroupData[{
Cell[1155026, 37453, 36, 0, 48, "Subsubsection"],
Cell[1155065, 37455, 318, 8, 50, "Text"],
Cell[1155386, 37465, 435, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1155858, 37488, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1155916, 37492, 108, 2, 27, "Input"],
Cell[1156027, 37496, 478, 14, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1156542, 37515, 71, 1, 27, "Input"],
Cell[1156616, 37518, 126, 3, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1156779, 37526, 108, 2, 27, "Input"],
Cell[1156890, 37530, 482, 14, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1157409, 37549, 225, 6, 27, "Input"],
Cell[1157637, 37557, 187, 5, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1157885, 37569, 44, 1, 56, "Subsection",
  CellTags->"GHP"],

Cell[CellGroupData[{
Cell[1157954, 37574, 36, 0, 48, "Subsubsection"],
Cell[1157993, 37576, 294, 8, 46, "Text"],
Cell[1158290, 37586, 471, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1158798, 37609, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1158856, 37613, 96, 2, 27, "Input"],
Cell[1158955, 37617, 478, 14, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1159470, 37636, 59, 1, 27, "Input"],
Cell[1159532, 37639, 126, 3, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1159695, 37647, 96, 2, 27, "Input"],
Cell[1159794, 37651, 482, 14, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1160313, 37670, 210, 6, 27, "Input"],
Cell[1160526, 37678, 187, 5, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1160774, 37690, 58, 1, 56, "Subsection",
  CellTags->"GluonField"],

Cell[CellGroupData[{
Cell[1160857, 37695, 36, 0, 48, "Subsubsection"],
Cell[1160896, 37697, 187, 4, 50, "Text"],
Cell[1161086, 37703, 313, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1161436, 37721, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1161494, 37725, 36, 0, 27, "Input"],
Cell[1161533, 37727, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1161629, 37733, 204, 4, 27, "Input"],
Cell[1161836, 37739, 342, 12, 30, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1162239, 37758, 70, 1, 56, "Subsection",
  CellTags->"GluonGhostVertex"],

Cell[CellGroupData[{
Cell[1162334, 37763, 36, 0, 48, "Subsubsection"],
Cell[1162373, 37765, 393, 7, 86, "Text"],
Cell[1162769, 37774, 309, 12, 32, "Text"],

Cell[CellGroupData[{
Cell[1163103, 37790, 78, 1, 27, "Input"],
Cell[1163184, 37793, 244, 6, 28, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1163477, 37805, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1163535, 37809, 332, 8, 27, "Input"],
Cell[1163870, 37819, 744, 24, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1164651, 37848, 64, 1, 27, "Input"],
Cell[1164718, 37851, 496, 12, 56, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1165251, 37868, 146, 3, 27, "Input"],
Cell[1165400, 37873, 748, 24, 30, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1166209, 37904, 68, 1, 56, "Subsection",
  CellTags->"GluonPropagator"],

Cell[CellGroupData[{
Cell[1166302, 37909, 36, 0, 48, "Subsubsection"],
Cell[1166341, 37911, 512, 9, 104, "Text"],
Cell[1166856, 37922, 69, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[1166950, 37926, 28, 0, 27, "Input"],
Cell[1166981, 37928, 73, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1167091, 37934, 77, 1, 27, "Input"],
Cell[1167171, 37937, 428, 10, 28, "Output"]
}, Open  ]],
Cell[1167614, 37950, 459, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1168110, 37973, 42, 0, 47, "Subsubsection"],

Cell[CellGroupData[{
Cell[1168177, 37977, 139, 3, 27, "Input"],
Cell[1168319, 37982, 882, 25, 47, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1169238, 38012, 192, 3, 27, "Input"],
Cell[1169433, 38017, 1645, 45, 60, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1171115, 38067, 303, 7, 27, "Input"],
Cell[1171421, 38076, 4235, 112, 61, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1175693, 38193, 116, 2, 27, "Input"],
Cell[1175812, 38197, 529, 15, 47, "Output"]
}, Open  ]],
Cell[1176356, 38215, 119, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[1176500, 38222, 95, 2, 27, "Input"],
Cell[1176598, 38226, 882, 25, 49, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1177517, 38256, 205, 6, 27, "Input"],
Cell[1177725, 38264, 286, 8, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1178048, 38277, 196, 3, 27, "Input"],
Cell[1178247, 38282, 2007, 52, 54, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1180291, 38339, 196, 3, 27, "Input"],
Cell[1180490, 38344, 2013, 53, 50, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1182540, 38402, 196, 3, 27, "Input"],
Cell[1182739, 38407, 2015, 52, 54, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1184791, 38464, 196, 3, 27, "Input"],
Cell[1184990, 38469, 2007, 52, 54, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1187034, 38526, 196, 3, 27, "Input"],
Cell[1187233, 38531, 3936, 102, 54, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1191230, 38640, 60, 1, 56, "Subsection",
  CellTags->"GluonVertex"],

Cell[CellGroupData[{
Cell[1191315, 38645, 36, 0, 48, "Subsubsection"],
Cell[1191354, 38647, 62, 0, 32, "Text"],
Cell[1191419, 38649, 595, 9, 122, "Text"],
Cell[1192017, 38660, 65, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[1192107, 38664, 28, 0, 27, "Input"],
Cell[1192138, 38666, 69, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1192244, 38672, 73, 1, 27, "Input"],
Cell[1192320, 38675, 374, 9, 28, "Output"]
}, Open  ]],
Cell[1192709, 38687, 331, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1193077, 38705, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1193135, 38709, 327, 8, 27, "Input"],
Cell[1193465, 38719, 1377, 41, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1194879, 38765, 383, 10, 27, "Input"],
Cell[1195265, 38777, 4451, 127, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1199753, 38909, 297, 8, 27, "Input"],
Cell[1200053, 38919, 352, 9, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1200442, 38933, 353, 10, 27, "Input"],
Cell[1200798, 38945, 3492, 110, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1204327, 39060, 414, 10, 27, "Input"],
Cell[1204744, 39072, 9042, 226, 47, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1213823, 39303, 374, 10, 27, "Input"],
Cell[1214200, 39315, 9048, 226, 47, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1223285, 39546, 190, 4, 27, "Input"],
Cell[1223478, 39552, 9048, 226, 47, "Output"]
}, Open  ]],
Cell[1232541, 39781, 176, 4, 50, "Text"],

Cell[CellGroupData[{
Cell[1232742, 39789, 132, 3, 27, "Input"],
Cell[1232877, 39794, 1376, 41, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1234290, 39840, 161, 4, 27, "Input"],
Cell[1234454, 39846, 9000, 226, 51, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1243515, 40079, 44, 1, 56, "Subsection",
  CellTags->"GSD"],

Cell[CellGroupData[{
Cell[1243584, 40084, 36, 0, 48, "Subsubsection"],
Cell[1243623, 40086, 717, 17, 68, "Text"],
Cell[1244343, 40105, 503, 23, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1244883, 40133, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1244941, 40137, 59, 1, 27, "Input"],
Cell[1245003, 40140, 139, 3, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1245179, 40148, 137, 4, 27, "Input"],
Cell[1245319, 40154, 166, 4, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1245522, 40163, 106, 2, 27, "Input"],
Cell[1245631, 40167, 618, 18, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1246286, 40190, 154, 4, 27, "Input"],
Cell[1246443, 40196, 209, 5, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1246689, 40206, 224, 6, 27, "Input"],
Cell[1246916, 40214, 515, 15, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1247492, 40236, 42, 1, 56, "Subsection",
  CellTags->"GS"],

Cell[CellGroupData[{
Cell[1247559, 40241, 36, 0, 48, "Subsubsection"],
Cell[1247598, 40243, 709, 17, 68, "Text"],
Cell[1248310, 40262, 503, 23, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1248850, 40290, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1248908, 40294, 58, 1, 27, "Input"],
Cell[1248969, 40297, 139, 3, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1249145, 40305, 136, 4, 27, "Input"],
Cell[1249284, 40311, 106, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1249427, 40318, 105, 2, 27, "Input"],
Cell[1249535, 40322, 618, 18, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1250190, 40345, 153, 4, 27, "Input"],
Cell[1250346, 40351, 205, 5, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1250588, 40361, 221, 6, 27, "Input"],
Cell[1250812, 40369, 515, 15, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1251388, 40391, 52, 1, 56, "Subsection",
  CellTags->"Gstrong"],

Cell[CellGroupData[{
Cell[1251465, 40396, 36, 0, 48, "Subsubsection"],
Cell[1251504, 40398, 61, 0, 32, "Text"],
Cell[1251568, 40400, 334, 14, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1251939, 40419, 33, 0, 48, "Subsubsection"],
Cell[1251975, 40421, 99, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[1252099, 40428, 33, 0, 27, "Input"],
Cell[1252135, 40430, 85, 2, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1252281, 40439, 46, 1, 56, "Subsection",
  CellTags->"Hill"],

Cell[CellGroupData[{
Cell[1252352, 40444, 36, 0, 48, "Subsubsection"],
Cell[1252391, 40446, 110, 3, 32, "Text"],
Cell[1252504, 40451, 188, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1252729, 40464, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1252787, 40468, 87, 2, 27, "Input"],
Cell[1252877, 40472, 2711, 74, 144, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1255625, 40551, 222, 5, 27, "Input"],
Cell[1255850, 40558, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1255946, 40564, 256, 7, 27, "Input"],
Cell[1256205, 40573, 1247, 35, 66, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1257489, 40613, 243, 7, 27, "Input"],
Cell[1257735, 40622, 59, 1, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1257855, 40630, 70, 1, 56, "Subsection",
  CellTags->"HypergeometricAC"],

Cell[CellGroupData[{
Cell[1257950, 40635, 36, 0, 48, "Subsubsection"],
Cell[1257989, 40637, 522, 8, 104, "Text"],

Cell[CellGroupData[{
Cell[1258536, 40649, 78, 1, 27, "Input"],
Cell[1258617, 40652, 134, 4, 28, "Output"]
}, Open  ]],
Cell[1258766, 40659, 605, 23, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1259408, 40687, 33, 0, 48, "Subsubsection"],
Cell[1259444, 40689, 70, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[1259539, 40693, 230, 5, 27, "Input"],
Cell[1259772, 40700, 2927, 71, 86, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1262736, 40776, 230, 5, 27, "Input"],
Cell[1262969, 40783, 2836, 68, 98, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1265842, 40856, 230, 5, 27, "Input"],
Cell[1266075, 40863, 2912, 70, 98, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1269024, 40938, 230, 5, 27, "Input"],
Cell[1269257, 40945, 3318, 80, 98, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1272612, 41030, 230, 5, 27, "Input"],
Cell[1272845, 41037, 943, 27, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1273825, 41069, 231, 5, 27, "Input"],
Cell[1274059, 41076, 944, 27, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1275064, 41110, 70, 1, 56, "Subsection",
  CellTags->"HypergeometricIR"],

Cell[CellGroupData[{
Cell[1275159, 41115, 36, 0, 48, "Subsubsection"],
Cell[1275198, 41117, 241, 3, 50, "Text"],
Cell[1275442, 41122, 477, 18, 32, "Text"],

Cell[CellGroupData[{
Cell[1275944, 41144, 78, 1, 27, "Input"],
Cell[1276025, 41147, 142, 4, 28, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1276216, 41157, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1276274, 41161, 207, 5, 27, "Input"],
Cell[1276484, 41168, 748, 20, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1277269, 41193, 411, 12, 27, "Input"],
Cell[1277683, 41207, 1121, 30, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1278841, 41242, 99, 2, 27, "Input"],
Cell[1278943, 41246, 356, 11, 30, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1279360, 41264, 70, 1, 56, "Subsection",
  CellTags->"HypergeometricSE"],

Cell[CellGroupData[{
Cell[1279455, 41269, 36, 0, 48, "Subsubsection"],
Cell[1279494, 41271, 408, 12, 50, "Text"],

Cell[CellGroupData[{
Cell[1279927, 41287, 78, 1, 27, "Input"],
Cell[1280008, 41290, 146, 4, 28, "Output"]
}, Open  ]],
Cell[1280169, 41297, 191, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1280397, 41310, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1280455, 41314, 211, 5, 27, "Input"],
Cell[1280669, 41321, 716, 16, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1281422, 41342, 349, 9, 27, "Input"],
Cell[1281774, 41353, 1030, 22, 46, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1282865, 41382, 60, 1, 56, "Subsection",
  CellTags->"HypExplicit"],

Cell[CellGroupData[{
Cell[1282950, 41387, 36, 0, 48, "Subsubsection"],
Cell[1282989, 41389, 312, 9, 50, "Text"],
Cell[1283304, 41400, 191, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1283532, 41413, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1283590, 41417, 120, 2, 27, "Input"],
Cell[1283713, 41421, 582, 18, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1284332, 41444, 98, 2, 27, "Input"],
Cell[1284433, 41448, 716, 16, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1285186, 41469, 240, 6, 27, "Input"],
Cell[1285429, 41477, 1101, 33, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1286567, 41515, 98, 2, 27, "Input"],
Cell[1286668, 41519, 1030, 22, 46, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1287759, 41548, 58, 1, 56, "Subsection",
  CellTags->"Integrate2"],

Cell[CellGroupData[{
Cell[1287842, 41553, 36, 0, 48, "Subsubsection"],
Cell[1287881, 41555, 859, 16, 140, "Text"],
Cell[1288743, 41573, 521, 23, 32, "Text"],
Cell[1289267, 41598, 380, 6, 86, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1289684, 41609, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1289742, 41613, 457, 13, 27, "Input"],
Cell[1290202, 41628, 544, 15, 43, "Output"]
}, Open  ]],
Cell[1290761, 41646, 170, 5, 32, "Text"],

Cell[CellGroupData[{
Cell[1290956, 41655, 267, 7, 27, "Input"],
Cell[1291226, 41664, 206, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1291469, 41675, 262, 7, 27, "Input"],
Cell[1291734, 41684, 478, 14, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1292249, 41703, 339, 9, 27, "Input"],
Cell[1292591, 41714, 720, 20, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1293348, 41739, 309, 8, 27, "Input"],
Cell[1293660, 41749, 91, 2, 28, "Output"]
}, Open  ]],
Cell[1293766, 41754, 1916, 53, 70, "Text"],

Cell[CellGroupData[{
Cell[1295707, 41811, 256, 7, 27, "Input"],
Cell[1295966, 41820, 59, 1, 28, "Output"]
}, Open  ]],
Cell[1296040, 41824, 729, 24, 53, "Text"],

Cell[CellGroupData[{
Cell[1296794, 41852, 311, 8, 27, "Input"],
Cell[1297108, 41862, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1297204, 41868, 401, 11, 27, "Input"],
Cell[1297608, 41881, 126, 3, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1297771, 41889, 441, 12, 27, "Input"],
Cell[1298215, 41903, 276, 9, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1298528, 41917, 327, 9, 27, "Input"],
Cell[1298858, 41928, 241, 7, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1299136, 41940, 361, 10, 27, "Input"],
Cell[1299500, 41952, 426, 12, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1299963, 41969, 305, 8, 27, "Input"],
Cell[1300271, 41979, 157, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1300465, 41988, 267, 7, 27, "Input"],
Cell[1300735, 41997, 413, 12, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1301185, 42014, 478, 13, 27, "Input"],
Cell[1301666, 42029, 541, 15, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1302244, 42049, 891, 23, 43, "Input"],
Cell[1303138, 42074, 5698, 157, 204, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1308873, 42236, 521, 14, 27, "Input"],
Cell[1309397, 42252, 1056, 30, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1310490, 42287, 560, 15, 43, "Input"],
Cell[1311053, 42304, 9541, 279, 243, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1320631, 42588, 99, 2, 27, "Input"],
Cell[1320733, 42592, 1516, 42, 83, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1322286, 42639, 57, 1, 27, "Input"],
Cell[1322346, 42642, 80, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1322463, 42648, 458, 12, 43, "Input"],
Cell[1322924, 42662, 80, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1323041, 42668, 802, 21, 65, "Input"],
Cell[1323846, 42691, 4892, 143, 151, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1328775, 42839, 108, 2, 27, "Input"],
Cell[1328886, 42843, 62, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1328985, 42849, 375, 10, 27, "Input"],
Cell[1329363, 42861, 59, 1, 28, "Output"]
}, Open  ]],
Cell[1329437, 42865, 112, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[1329574, 42872, 104, 2, 27, "Input"],
Cell[1329681, 42876, 7231, 176, 305, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1336949, 43057, 209, 6, 27, "Input"],
Cell[1337161, 43065, 177, 5, 28, "Output"]
}, Open  ]],
Cell[1337353, 43073, 140, 3, 50, "Text"],

Cell[CellGroupData[{
Cell[1337518, 43080, 330, 9, 27, "Input"],
Cell[1337851, 43091, 180, 5, 28, "Output"]
}, Open  ]],
Cell[1338046, 43099, 83, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[1338154, 43105, 309, 8, 27, "Input"],
Cell[1338466, 43115, 91, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1338594, 43122, 325, 9, 31, "Input"],
Cell[1338922, 43133, 249, 8, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1339208, 43146, 52, 1, 27, "Input"],
Cell[1339263, 43149, 80, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1339380, 43155, 325, 9, 31, "Input"],
Cell[1339708, 43166, 80, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1339825, 43172, 396, 11, 31, "Input"],
Cell[1340224, 43185, 6214, 182, 181, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1346499, 43374, 58, 1, 56, "Subsection",
  CellTags->"Integrate3"],

Cell[CellGroupData[{
Cell[1346582, 43379, 36, 0, 48, "Subsubsection"],
Cell[1346621, 43381, 309, 6, 68, "Text"],
Cell[1346933, 43389, 173, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1347143, 43402, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1347201, 43406, 265, 7, 28, "Input"],
Cell[1347469, 43415, 211, 7, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1347717, 43427, 414, 11, 44, "Input"],
Cell[1348134, 43440, 687, 21, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1348858, 43466, 783, 21, 65, "Input"],
Cell[1349644, 43489, 3047, 80, 85, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1352728, 43574, 245, 6, 27, "Input"],
Cell[1352976, 43582, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1353072, 43588, 314, 8, 27, "Input"],
Cell[1353389, 43598, 166, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1353592, 43608, 256, 7, 27, "Input"],
Cell[1353851, 43617, 59, 1, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1353971, 43625, 70, 1, 56, "Subsection",
  CellTags->"IntegrateByParts"],

Cell[CellGroupData[{
Cell[1354066, 43630, 36, 0, 48, "Subsubsection"],
Cell[1354105, 43632, 153, 3, 50, "Text"],

Cell[CellGroupData[{
Cell[1354283, 43639, 78, 1, 27, "Input"],
Cell[1354364, 43642, 131, 4, 28, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1354544, 43652, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1354602, 43656, 589, 17, 43, "Input"],
Cell[1355194, 43675, 738, 21, 44, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1355993, 43703, 64, 1, 56, "Subsection",
  CellTags->"InverseMellin"],

Cell[CellGroupData[{
Cell[1356082, 43708, 36, 0, 48, "Subsubsection"],
Cell[1356121, 43710, 549, 13, 82, "Text"],
Cell[1356673, 43725, 628, 28, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1357338, 43758, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1357396, 43762, 128, 3, 27, "Input"],
Cell[1357527, 43767, 116, 3, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1357680, 43775, 192, 5, 27, "Input"],
Cell[1357875, 43782, 116, 3, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1358028, 43790, 96, 2, 27, "Input"],
Cell[1358127, 43794, 272, 8, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1358436, 43807, 159, 4, 27, "Input"],
Cell[1358598, 43813, 281, 9, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1358916, 43827, 159, 4, 27, "Input"],
Cell[1359078, 43833, 374, 11, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1359489, 43849, 169, 5, 27, "Input"],
Cell[1359661, 43856, 374, 11, 30, "Output"]
}, Open  ]],
Cell[1360050, 43870, 88, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[1360163, 43877, 276, 7, 27, "Input"],
Cell[1360442, 43886, 84, 2, 43, "Output"]
}, Open  ]],
Cell[1360541, 43891, 91, 3, 32, "Text"],
Cell[1360635, 43896, 3955, 101, 211, "Input"],
Cell[1364593, 43999, 212, 5, 27, "Input"],

Cell[CellGroupData[{
Cell[1364830, 44008, 143, 4, 27, "Input"],
Cell[1364976, 44014, 414, 12, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1365427, 44031, 143, 4, 27, "Input"],
Cell[1365573, 44037, 365, 11, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1365975, 44053, 130, 3, 27, "Input"],
Cell[1366108, 44058, 927, 27, 47, "Output"]
}, Open  ]],
Cell[1367050, 44088, 27, 0, 27, "Input"],

Cell[CellGroupData[{
Cell[1367102, 44092, 156, 4, 27, "Input"],
Cell[1367261, 44098, 559, 17, 47, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1367857, 44120, 252, 7, 27, "Input"],
Cell[1368112, 44129, 557, 17, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1368706, 44151, 252, 7, 27, "Input"],
Cell[1368961, 44160, 792, 22, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1369790, 44187, 251, 7, 27, "Input"],
Cell[1370044, 44196, 864, 24, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1370945, 44225, 229, 7, 27, "Input"],
Cell[1371177, 44234, 618, 18, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1371832, 44257, 268, 8, 27, "Input"],
Cell[1372103, 44267, 1110, 29, 47, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1373250, 44301, 156, 4, 27, "Input"],
Cell[1373409, 44307, 826, 23, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1374272, 44335, 195, 6, 27, "Input"],
Cell[1374470, 44343, 867, 24, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1375374, 44372, 156, 4, 27, "Input"],
Cell[1375533, 44378, 1107, 30, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1376677, 44413, 166, 4, 27, "Input"],
Cell[1376846, 44419, 615, 18, 47, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1377498, 44442, 194, 5, 27, "Input"],
Cell[1377695, 44449, 1368, 36, 47, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1379100, 44490, 194, 5, 27, "Input"],
Cell[1379297, 44497, 1293, 34, 47, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1380627, 44536, 176, 4, 27, "Input"],
Cell[1380806, 44542, 729, 21, 49, "Output"]
}, Open  ]],
Cell[1381550, 44566, 116, 3, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1381715, 44575, 52, 1, 56, "Subsection",
  CellTags->"Isolate"],

Cell[CellGroupData[{
Cell[1381792, 44580, 36, 0, 48, "Subsubsection"],
Cell[1381831, 44582, 402, 7, 86, "Text"],

Cell[CellGroupData[{
Cell[1382258, 44593, 69, 1, 27, "Input"],
Cell[1382330, 44596, 352, 8, 28, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1382731, 44610, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1382789, 44614, 120, 3, 27, "Input"],
Cell[1382912, 44619, 127, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1383076, 44628, 404, 11, 27, "Input"],
Cell[1383483, 44641, 314, 10, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1383834, 44656, 69, 1, 27, "Input"],
Cell[1383906, 44659, 265, 9, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1384208, 44673, 146, 3, 27, "Input"],
Cell[1384357, 44678, 754, 23, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1385148, 44706, 626, 17, 27, "Input"],
Cell[1385777, 44725, 374, 12, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1386188, 44742, 48, 1, 27, "Input"],
Cell[1386239, 44745, 73, 1, 24, "Print"],
Cell[1386315, 44748, 700, 19, 60, "Print"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1387052, 44772, 252, 5, 27, "Input"],
Cell[1387307, 44779, 126, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1387470, 44788, 147, 4, 27, "Input"],
Cell[1387620, 44794, 182, 5, 28, "Output"]
}, Open  ]],
Cell[1387817, 44802, 100, 2, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1387966, 44810, 50, 1, 56, "Subsection",
  CellTags->"Kummer"],

Cell[CellGroupData[{
Cell[1388041, 44815, 36, 0, 48, "Subsubsection"],
Cell[1388080, 44817, 242, 5, 68, "Text"],
Cell[1388325, 44824, 191, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1388553, 44837, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1388611, 44841, 325, 8, 43, "Input"],
Cell[1388939, 44851, 1506, 43, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1390482, 44899, 325, 8, 43, "Input"],
Cell[1390810, 44909, 1536, 44, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1392383, 44958, 325, 8, 43, "Input"],
Cell[1392711, 44968, 1536, 44, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1394284, 45017, 386, 9, 43, "Input"],
Cell[1394673, 45028, 1514, 43, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1396224, 45076, 490, 11, 43, "Input"],
Cell[1396717, 45089, 1610, 44, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1398364, 45138, 386, 9, 43, "Input"],
Cell[1398753, 45149, 1505, 43, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1400295, 45197, 490, 11, 43, "Input"],
Cell[1400788, 45210, 1631, 45, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1402456, 45260, 386, 9, 43, "Input"],
Cell[1402845, 45271, 1505, 43, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1404387, 45319, 490, 11, 43, "Input"],
Cell[1404880, 45332, 1631, 45, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1406548, 45382, 517, 13, 43, "Input"],
Cell[1407068, 45397, 1726, 49, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1408831, 45451, 685, 17, 43, "Input"],
Cell[1409519, 45470, 1871, 52, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1411427, 45527, 517, 13, 43, "Input"],
Cell[1411947, 45542, 1707, 49, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1413691, 45596, 685, 17, 43, "Input"],
Cell[1414379, 45615, 1852, 52, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1416268, 45672, 517, 13, 43, "Input"],
Cell[1416788, 45687, 1671, 48, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1418496, 45740, 685, 17, 43, "Input"],
Cell[1419184, 45759, 1912, 53, 83, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1421133, 45817, 517, 13, 43, "Input"],
Cell[1421653, 45832, 1726, 49, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1423416, 45886, 685, 17, 43, "Input"],
Cell[1424104, 45905, 1941, 54, 83, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1426082, 45964, 517, 13, 43, "Input"],
Cell[1426602, 45979, 1638, 47, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1428277, 46031, 685, 17, 43, "Input"],
Cell[1428965, 46050, 1853, 52, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1430855, 46107, 517, 13, 43, "Input"],
Cell[1431375, 46122, 1638, 47, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1433050, 46174, 685, 17, 43, "Input"],
Cell[1433738, 46193, 1992, 55, 83, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1435767, 46253, 555, 13, 43, "Input"],
Cell[1436325, 46268, 1680, 47, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1438042, 46320, 326, 8, 43, "Input"],
Cell[1438371, 46330, 1506, 43, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1439914, 46378, 555, 13, 43, "Input"],
Cell[1440472, 46393, 1768, 49, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1442277, 46447, 326, 8, 43, "Input"],
Cell[1442606, 46457, 1498, 43, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1444141, 46505, 555, 13, 43, "Input"],
Cell[1444699, 46520, 1768, 49, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1446504, 46574, 326, 8, 43, "Input"],
Cell[1446833, 46584, 1498, 43, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1448368, 46632, 619, 15, 43, "Input"],
Cell[1448990, 46649, 1738, 48, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1450765, 46702, 387, 9, 43, "Input"],
Cell[1451155, 46713, 1514, 43, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1452706, 46761, 619, 15, 43, "Input"],
Cell[1453328, 46778, 1888, 52, 64, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1455253, 46835, 387, 9, 43, "Input"],
Cell[1455643, 46846, 1578, 45, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1457258, 46896, 619, 15, 43, "Input"],
Cell[1457880, 46913, 1815, 50, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1459732, 46968, 387, 9, 43, "Input"],
Cell[1460122, 46979, 1505, 43, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1461664, 47027, 326, 8, 43, "Input"],
Cell[1461993, 47037, 3359, 84, 87, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1465389, 47126, 326, 8, 43, "Input"],
Cell[1465718, 47136, 3491, 89, 98, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1469246, 47230, 326, 8, 43, "Input"],
Cell[1469575, 47240, 3503, 89, 98, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1473139, 47336, 58, 1, 56, "Subsection",
  CellTags->"Lagrangian"],

Cell[CellGroupData[{
Cell[1473222, 47341, 36, 0, 48, "Subsubsection"],
Cell[1473261, 47343, 187, 4, 50, "Text"],
Cell[1473451, 47349, 167, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1473655, 47362, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1473713, 47366, 76, 1, 27, "Input"],
Cell[1473792, 47369, 365, 10, 43, "Output"]
}, Open  ]],
Cell[1474172, 47382, 60, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[1474257, 47386, 76, 1, 27, "Input"],
Cell[1474336, 47389, 824, 21, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1475197, 47415, 76, 1, 27, "Input"],
Cell[1475276, 47418, 1316, 34, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1476629, 47457, 76, 1, 27, "Input"],
Cell[1476708, 47460, 757, 21, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1477502, 47486, 76, 1, 27, "Input"],
Cell[1477581, 47489, 806, 22, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1478448, 47518, 42, 1, 56, "Subsection",
  CellTags->"LC"],

Cell[CellGroupData[{
Cell[1478515, 47523, 36, 0, 48, "Subsubsection"],
Cell[1478554, 47525, 223, 4, 50, "Text"],
Cell[1478780, 47531, 277, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1479094, 47549, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1479152, 47553, 132, 3, 27, "Input"],
Cell[1479287, 47558, 212, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1479536, 47568, 55, 1, 27, "Input"],
Cell[1479594, 47571, 616, 19, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1480247, 47595, 64, 1, 27, "Input"],
Cell[1480314, 47598, 307, 6, 56, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1480658, 47609, 154, 4, 27, "Input"],
Cell[1480815, 47615, 200, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1481052, 47625, 94, 2, 27, "Input"],
Cell[1481149, 47629, 349, 8, 56, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1481535, 47642, 407, 11, 27, "Input"],
Cell[1481945, 47655, 532, 16, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1482538, 47678, 44, 1, 56, "Subsection",
  CellTags->"LCD"],

Cell[CellGroupData[{
Cell[1482607, 47683, 36, 0, 48, "Subsubsection"],
Cell[1482646, 47685, 216, 4, 50, "Text"],
Cell[1482865, 47691, 138, 3, 50, "Text"],
Cell[1483006, 47696, 274, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1483317, 47714, 33, 0, 48, "Subsubsection"],
Cell[1483353, 47716, 164, 4, 27, "Input"],

Cell[CellGroupData[{
Cell[1483542, 47724, 133, 3, 27, "Input"],
Cell[1483678, 47729, 212, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1483927, 47739, 55, 1, 27, "Input"],
Cell[1483985, 47742, 616, 19, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1484638, 47766, 64, 1, 27, "Input"],
Cell[1484705, 47769, 431, 10, 56, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1485173, 47784, 155, 4, 27, "Input"],
Cell[1485331, 47790, 200, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1485568, 47800, 94, 2, 27, "Input"],
Cell[1485665, 47804, 411, 10, 56, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1486113, 47819, 425, 11, 27, "Input"],
Cell[1486541, 47832, 456, 13, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1487034, 47850, 138, 3, 27, "Input"],
Cell[1487175, 47855, 132, 4, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1487368, 47866, 62, 1, 56, "Subsection",
  CellTags->"LeftPartialD"],

Cell[CellGroupData[{
Cell[1487455, 47871, 36, 0, 48, "Subsubsection"],
Cell[1487494, 47873, 330, 11, 35, "Text"],
Cell[1487827, 47886, 584, 23, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1488448, 47914, 42, 0, 47, "Subsubsection"],

Cell[CellGroupData[{
Cell[1488515, 47918, 222, 5, 27, "Input"],
Cell[1488740, 47925, 308, 9, 34, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1489085, 47939, 70, 1, 27, "Input"],
Cell[1489158, 47942, 308, 9, 34, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1489503, 47956, 68, 1, 27, "Input"],
Cell[1489574, 47959, 268, 6, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1489879, 47970, 115, 2, 27, "Input"],
Cell[1489997, 47974, 116, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1490150, 47981, 356, 8, 43, "Input"],
Cell[1490509, 47991, 351, 10, 34, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1490897, 48006, 70, 1, 27, "Input"],
Cell[1490970, 48009, 374, 11, 34, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1491381, 48025, 68, 1, 27, "Input"],
Cell[1491452, 48028, 431, 10, 56, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1491944, 48045, 72, 1, 56, "Subsection",
  CellTags->"LeftRightPartialD"],

Cell[CellGroupData[{
Cell[1492041, 48050, 36, 0, 48, "Subsubsection"],
Cell[1492080, 48052, 626, 20, 53, "Text"],
Cell[1492709, 48074, 854, 32, 50, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1493600, 48111, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1493658, 48115, 77, 1, 27, "Input"],
Cell[1493738, 48118, 242, 7, 34, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1494017, 48130, 72, 1, 27, "Input"],
Cell[1494092, 48133, 602, 17, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1494731, 48155, 227, 5, 27, "Input"],
Cell[1494961, 48162, 313, 9, 34, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1495311, 48176, 70, 1, 27, "Input"],
Cell[1495384, 48179, 785, 23, 44, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1496206, 48207, 361, 8, 43, "Input"],
Cell[1496570, 48217, 356, 10, 34, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1496963, 48232, 70, 1, 27, "Input"],
Cell[1497036, 48235, 976, 28, 43, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1498073, 48270, 74, 1, 56, "Subsection",
  CellTags->"LeftRightPartialD2"],

Cell[CellGroupData[{
Cell[1498172, 48275, 36, 0, 48, "Subsubsection"],
Cell[1498211, 48277, 624, 20, 53, "Text"],
Cell[1498838, 48299, 712, 28, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1499587, 48332, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1499645, 48336, 78, 1, 27, "Input"],
Cell[1499726, 48339, 242, 7, 34, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1500005, 48351, 72, 1, 27, "Input"],
Cell[1500080, 48354, 467, 14, 34, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1500584, 48373, 228, 5, 27, "Input"],
Cell[1500815, 48380, 313, 9, 34, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1501165, 48394, 70, 1, 27, "Input"],
Cell[1501238, 48397, 697, 21, 34, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1501972, 48423, 362, 8, 43, "Input"],
Cell[1502337, 48433, 356, 10, 34, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1502730, 48448, 70, 1, 27, "Input"],
Cell[1502803, 48451, 853, 25, 35, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1503717, 48483, 59, 1, 56, "Subsection",
  CellTags->"LeviCivita"],

Cell[CellGroupData[{
Cell[1503801, 48488, 36, 0, 48, "Subsubsection"],
Cell[1503840, 48490, 1409, 45, 104, "Text"],

Cell[CellGroupData[{
Cell[1505274, 48539, 72, 1, 27, "Input"],
Cell[1505349, 48542, 132, 4, 28, "Output"]
}, Open  ]],
Cell[1505496, 48549, 253, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1505786, 48567, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1505844, 48571, 147, 3, 27, "Input"],
Cell[1505994, 48576, 623, 19, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1506654, 48600, 169, 4, 27, "Input"],
Cell[1506826, 48606, 508, 15, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1507371, 48626, 140, 3, 27, "Input"],
Cell[1507514, 48631, 376, 11, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1507927, 48647, 146, 3, 27, "Input"],
Cell[1508076, 48652, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1508172, 48658, 217, 6, 27, "Input"],
Cell[1508392, 48666, 294, 6, 56, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1508723, 48677, 412, 11, 27, "Input"],
Cell[1509138, 48690, 1282, 38, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1510457, 48733, 65, 1, 27, "Input"],
Cell[1510525, 48736, 82, 2, 28, "Output"]
}, Open  ]],
Cell[1510622, 48741, 298, 7, 43, "Input"],

Cell[CellGroupData[{
Cell[1510945, 48752, 430, 11, 27, "Input"],
Cell[1511378, 48765, 289, 9, 28, "Output"]
}, Open  ]],
Cell[1511682, 48777, 298, 7, 43, "Input"],

Cell[CellGroupData[{
Cell[1512005, 48788, 139, 3, 27, "Input"],
Cell[1512147, 48793, 219, 5, 29, "Output"]
}, Open  ]],
Cell[1512381, 48801, 62, 1, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1512492, 48808, 45, 1, 56, "Subsection",
  CellTags->"Li2"],

Cell[CellGroupData[{
Cell[1512562, 48813, 36, 0, 48, "Subsubsection"],
Cell[1512601, 48815, 101, 3, 32, "Text"],
Cell[1512705, 48820, 145, 7, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1512887, 48832, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1512945, 48836, 59, 1, 27, "Input"],
Cell[1513007, 48839, 120, 3, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1513164, 48847, 66, 1, 27, "Input"],
Cell[1513233, 48850, 116, 3, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1513386, 48858, 248, 7, 43, "Input"],
Cell[1513637, 48867, 120, 3, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1513818, 48877, 45, 1, 56, "Subsection",
  CellTags->"Li3"],

Cell[CellGroupData[{
Cell[1513888, 48882, 36, 0, 48, "Subsubsection"],
Cell[1513927, 48884, 102, 3, 32, "Text"],
Cell[1514032, 48889, 145, 7, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1514214, 48901, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1514272, 48905, 59, 1, 27, "Input"],
Cell[1514334, 48908, 120, 3, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1514491, 48916, 66, 1, 27, "Input"],
Cell[1514560, 48919, 116, 3, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1514713, 48927, 120, 3, 27, "Input"],
Cell[1514836, 48932, 152, 5, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1515025, 48942, 183, 5, 43, "Input"],
Cell[1515211, 48949, 120, 3, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1515392, 48959, 62, 1, 56, "Subsection",
  CellTags->"LorentzIndex"],

Cell[CellGroupData[{
Cell[1515479, 48964, 36, 0, 48, "Subsubsection"],
Cell[1515518, 48966, 216, 4, 50, "Text"],
Cell[1515737, 48972, 307, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1516081, 48990, 33, 0, 48, "Subsubsection"],
Cell[1516117, 48992, 62, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[1516204, 48996, 75, 1, 27, "Input"],
Cell[1516282, 48999, 66, 1, 28, "Output"]
}, Open  ]],
Cell[1516363, 49003, 100, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[1516488, 49010, 102, 2, 27, "Input"],
Cell[1516593, 49014, 66, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1516696, 49020, 63, 0, 27, "Input"],
Cell[1516762, 49022, 94, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1516893, 49029, 119, 2, 27, "Input"],
Cell[1517015, 49033, 91, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1517143, 49040, 166, 3, 27, "Input"],
Cell[1517312, 49045, 65, 1, 28, "Output"]
}, Open  ]],
Cell[1517392, 49049, 114, 3, 32, "Text"],
Cell[1517509, 49054, 91, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[1517625, 49060, 334, 8, 27, "Input"],
Cell[1517962, 49070, 429, 14, 28, "Output"]
}, Open  ]],
Cell[1518406, 49087, 102, 2, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1518557, 49095, 46, 1, 56, "Subsection",
  CellTags->"Map2"],

Cell[CellGroupData[{
Cell[1518628, 49100, 36, 0, 48, "Subsubsection"],
Cell[1518667, 49102, 116, 3, 32, "Text"],
Cell[1518786, 49107, 161, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1518984, 49120, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1519042, 49124, 116, 3, 27, "Input"],
Cell[1519161, 49129, 180, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1519378, 49139, 87, 2, 27, "Input"],
Cell[1519468, 49143, 91, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1519596, 49150, 157, 4, 27, "Input"],
Cell[1519756, 49156, 164, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1519957, 49166, 87, 2, 27, "Input"],
Cell[1520047, 49170, 91, 2, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1520199, 49179, 62, 1, 56, "Subsection",
  CellTags->"MetricTensor"],

Cell[CellGroupData[{
Cell[1520286, 49184, 36, 0, 48, "Subsubsection"],
Cell[1520325, 49186, 199, 6, 32, "Text"],
Cell[1520527, 49194, 604, 28, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1521168, 49227, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1521226, 49231, 108, 2, 27, "Input"],
Cell[1521337, 49235, 141, 4, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1521515, 49244, 91, 2, 27, "Input"],
Cell[1521609, 49248, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1521705, 49254, 161, 3, 27, "Input"],
Cell[1521869, 49259, 350, 11, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1522256, 49275, 91, 2, 27, "Input"],
Cell[1522350, 49279, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1522446, 49285, 140, 3, 27, "Input"],
Cell[1522589, 49290, 178, 4, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1522804, 49299, 195, 4, 27, "Input"],
Cell[1523002, 49305, 240, 6, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1523279, 49316, 178, 4, 27, "Input"],
Cell[1523460, 49322, 86, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1523583, 49329, 238, 6, 27, "Input"],
Cell[1523824, 49337, 87, 2, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1523972, 49346, 68, 1, 56, "Subsection",
  CellTags->"MomentumCombine"],

Cell[CellGroupData[{
Cell[1524065, 49351, 36, 0, 48, "Subsubsection"],
Cell[1524104, 49353, 153, 3, 50, "Text"],
Cell[1524260, 49358, 449, 17, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1524746, 49380, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1524804, 49384, 310, 8, 27, "Input"],
Cell[1525117, 49394, 121, 3, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1525275, 49402, 297, 8, 27, "Input"],
Cell[1525575, 49412, 185, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1525797, 49422, 72, 1, 27, "Input"],
Cell[1525872, 49425, 337, 11, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1526246, 49441, 114, 3, 27, "Input"],
Cell[1526363, 49446, 242, 6, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1526642, 49457, 125, 3, 27, "Input"],
Cell[1526770, 49462, 511, 17, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1527318, 49484, 68, 1, 27, "Input"],
Cell[1527389, 49487, 404, 10, 56, "Output"]
}, Open  ]],
Cell[1527808, 49500, 62, 1, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1527919, 49507, 66, 1, 56, "Subsection",
  CellTags->"MomentumExpand"],

Cell[CellGroupData[{
Cell[1528010, 49512, 36, 0, 48, "Subsubsection"],
Cell[1528049, 49514, 120, 3, 50, "Text"],
Cell[1528172, 49519, 340, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1528549, 49537, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1528607, 49541, 184, 5, 27, "Input"],
Cell[1528794, 49548, 131, 3, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1528962, 49556, 125, 3, 27, "Input"],
Cell[1529090, 49561, 297, 8, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1529424, 49574, 64, 1, 27, "Input"],
Cell[1529491, 49577, 201, 5, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1529729, 49587, 174, 4, 27, "Input"],
Cell[1529906, 49593, 297, 8, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1530240, 49606, 64, 1, 27, "Input"],
Cell[1530307, 49609, 244, 6, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1530588, 49620, 205, 5, 27, "Input"],
Cell[1530796, 49627, 366, 9, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1531199, 49641, 64, 1, 27, "Input"],
Cell[1531266, 49644, 346, 9, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1531649, 49658, 60, 1, 27, "Input"],
Cell[1531712, 49661, 727, 21, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1532500, 49689, 54, 1, 56, "Subsection",
  CellTags->"Momentum"],

Cell[CellGroupData[{
Cell[1532579, 49694, 36, 0, 48, "Subsubsection"],
Cell[1532618, 49696, 238, 5, 68, "Text"],
Cell[1532859, 49703, 545, 23, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1533441, 49731, 33, 0, 48, "Subsubsection"],
Cell[1533477, 49733, 52, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[1533554, 49737, 64, 1, 27, "Input"],
Cell[1533621, 49740, 59, 1, 28, "Output"]
}, Open  ]],
Cell[1533695, 49744, 115, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[1533835, 49751, 91, 2, 27, "Input"],
Cell[1533929, 49755, 59, 1, 28, "Output"]
}, Open  ]],
Cell[1534003, 49759, 63, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[1534091, 49763, 132, 3, 27, "Input"],
Cell[1534226, 49768, 92, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1534355, 49775, 116, 3, 27, "Input"],
Cell[1534474, 49780, 81, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1534592, 49787, 65, 1, 27, "Input"],
Cell[1534660, 49790, 87, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1534784, 49797, 239, 6, 27, "Input"],
Cell[1535026, 49805, 175, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1535238, 49815, 65, 1, 27, "Input"],
Cell[1535306, 49818, 189, 5, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1535532, 49828, 106, 2, 27, "Input"],
Cell[1535641, 49832, 131, 3, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1535809, 49840, 110, 3, 27, "Input"],
Cell[1535922, 49845, 92, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1536051, 49852, 189, 5, 27, "Input"],
Cell[1536243, 49859, 92, 2, 40, "Output"]
}, Open  ]],
Cell[1536350, 49864, 90, 2, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1536489, 49872, 42, 1, 56, "Subsection",
  CellTags->"MT"],

Cell[CellGroupData[{
Cell[1536556, 49877, 36, 0, 48, "Subsubsection"],
Cell[1536595, 49879, 177, 6, 32, "Text"],
Cell[1536775, 49887, 634, 28, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1537446, 49920, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1537504, 49924, 98, 2, 27, "Input"],
Cell[1537605, 49928, 141, 4, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1537783, 49937, 249, 6, 27, "Input"],
Cell[1538035, 49945, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1538131, 49951, 126, 3, 27, "Input"],
Cell[1538260, 49956, 86, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1538383, 49963, 167, 5, 27, "Input"],
Cell[1538553, 49970, 178, 4, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1538768, 49979, 211, 6, 27, "Input"],
Cell[1538982, 49987, 86, 2, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1539129, 49996, 44, 1, 56, "Subsection",
  CellTags->"MTD"],

Cell[CellGroupData[{
Cell[1539198, 50001, 36, 0, 48, "Subsubsection"],
Cell[1539237, 50003, 178, 6, 32, "Text"],
Cell[1539418, 50011, 631, 28, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1540086, 50044, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1540144, 50048, 99, 2, 27, "Input"],
Cell[1540246, 50052, 141, 4, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1540424, 50061, 251, 6, 27, "Input"],
Cell[1540678, 50069, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1540774, 50075, 143, 4, 27, "Input"],
Cell[1540920, 50081, 100, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1541057, 50088, 185, 5, 27, "Input"],
Cell[1541245, 50095, 253, 6, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1541535, 50106, 220, 6, 27, "Input"],
Cell[1541758, 50114, 95, 2, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1541914, 50123, 52, 1, 56, "Subsection",
  CellTags->"Nielsen"],

Cell[CellGroupData[{
Cell[1541991, 50128, 36, 0, 48, "Subsubsection"],
Cell[1542030, 50130, 64, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[1542119, 50134, 69, 1, 27, "Input"],
Cell[1542191, 50137, 134, 4, 28, "Output"]
}, Open  ]],
Cell[1542340, 50144, 188, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1542565, 50157, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1542623, 50161, 100, 2, 27, "Input"],
Cell[1542726, 50165, 272, 9, 28, "Output"]
}, Open  ]],
Cell[1543013, 50177, 157, 4, 50, "Text"],

Cell[CellGroupData[{
Cell[1543195, 50185, 136, 3, 27, "Input"],
Cell[1543334, 50190, 80, 1, 28, "Output"]
}, Open  ]],
Cell[1543429, 50194, 49, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[1543503, 50198, 484, 12, 43, "Input"],
Cell[1543990, 50212, 711, 21, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1544738, 50238, 154, 3, 27, "Input"],
Cell[1544895, 50243, 770, 22, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1545702, 50270, 154, 3, 27, "Input"],
Cell[1545859, 50275, 992, 28, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1546888, 50308, 154, 3, 27, "Input"],
Cell[1547045, 50313, 120, 3, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1547226, 50323, 50, 1, 56, "Subsection",
  CellTags->"NTerms"],

Cell[CellGroupData[{
Cell[1547301, 50328, 36, 0, 48, "Subsubsection"],
Cell[1547340, 50330, 128, 3, 50, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1547505, 50338, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1547563, 50342, 89, 2, 27, "Input"],
Cell[1547655, 50346, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1547751, 50352, 99, 2, 27, "Input"],
Cell[1547853, 50356, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1547949, 50362, 62, 1, 27, "Input"],
Cell[1548014, 50365, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1548110, 50371, 62, 1, 27, "Input"],
Cell[1548175, 50374, 59, 1, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1548295, 50382, 52, 1, 56, "Subsection",
  CellTags->"OneLoop"],
Cell[1548350, 50385, 649, 16, 86, "Text"],
Cell[1549002, 50403, 52, 0, 32, "Text"],
Cell[1549057, 50405, 1050, 26, 68, "Text"],

Cell[CellGroupData[{
Cell[1550132, 50435, 36, 0, 48, "Subsubsection"],
Cell[1550171, 50437, 260, 5, 68, "Text"],
Cell[1550434, 50444, 332, 6, 86, "Text"],

Cell[CellGroupData[{
Cell[1550791, 50454, 69, 1, 27, "Input"],
Cell[1550863, 50457, 1342, 27, 118, "Output"]
}, Open  ]],
Cell[1552220, 50487, 702, 33, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1552959, 50525, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1553017, 50529, 269, 8, 27, "Input"],
Cell[1553289, 50539, 336, 10, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1553662, 50554, 90, 2, 27, "Input"],
Cell[1553755, 50558, 333, 9, 29, "Output"]
}, Open  ]],
Cell[1554103, 50570, 367, 9, 43, "Input"],
Cell[1554473, 50581, 714, 19, 32, "Text"],

Cell[CellGroupData[{
Cell[1555212, 50604, 1073, 28, 83, "Input"],
Cell[1556288, 50634, 2446, 63, 51, "Output"]
}, Open  ]],
Cell[1558749, 50700, 197, 4, 50, "Text"],

Cell[CellGroupData[{
Cell[1558971, 50708, 90, 2, 27, "Input"],
Cell[1559064, 50712, 2664, 67, 53, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1561765, 50784, 68, 1, 27, "Input"],
Cell[1561836, 50787, 2036, 51, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1563909, 50843, 339, 10, 27, "Input"],
Cell[1564251, 50855, 522, 16, 47, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1564810, 50876, 90, 2, 27, "Input"],
Cell[1564903, 50880, 2689, 70, 48, "Output"]
}, Open  ]],
Cell[1567607, 50953, 61, 1, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1567717, 50960, 68, 1, 56, "Subsection",
  CellTags->"OneLoopSimplify"],

Cell[CellGroupData[{
Cell[1567810, 50965, 36, 0, 48, "Subsubsection"],
Cell[1567849, 50967, 142, 3, 50, "Text"],

Cell[CellGroupData[{
Cell[1568016, 50974, 77, 1, 27, "Input"],
Cell[1568096, 50977, 771, 16, 64, "Output"]
}, Open  ]],
Cell[1568882, 50996, 375, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1569294, 51019, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1569352, 51023, 339, 10, 27, "Input"],
Cell[1569694, 51035, 522, 16, 47, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1570253, 51056, 98, 2, 27, "Input"],
Cell[1570354, 51060, 633, 18, 47, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1571024, 51083, 165, 4, 27, "Input"],
Cell[1571192, 51089, 615, 16, 84, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1571844, 51110, 404, 10, 27, "Input"],
Cell[1572251, 51122, 663, 21, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1572951, 51148, 103, 2, 27, "Input"],
Cell[1573057, 51152, 771, 22, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1573865, 51179, 231, 7, 27, "Input"],
Cell[1574099, 51188, 771, 22, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1574907, 51215, 452, 13, 27, "Input"],
Cell[1575362, 51230, 2374, 63, 87, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1577797, 51300, 54, 1, 56, "Subsection",
  CellTags->"OPEDelta"],

Cell[CellGroupData[{
Cell[1577876, 51305, 36, 0, 48, "Subsubsection"],
Cell[1577915, 51307, 117, 3, 32, "Text"],
Cell[1578035, 51312, 200, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1578272, 51325, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1578330, 51329, 104, 2, 27, "Input"],
Cell[1578437, 51333, 150, 4, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1578624, 51342, 91, 2, 27, "Input"],
Cell[1578718, 51346, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1578814, 51352, 99, 2, 27, "Input"],
Cell[1578916, 51356, 59, 1, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1579036, 51364, 82, 1, 56, "Subsection",
  CellTags->"OPEi"],

Cell[CellGroupData[{
Cell[1579143, 51369, 36, 0, 48, "Subsubsection"],
Cell[1579182, 51371, 152, 3, 50, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1579371, 51379, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1579429, 51383, 30, 0, 27, "Input"],
Cell[1579462, 51385, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1579558, 51391, 234, 5, 43, "Input"],
Cell[1579795, 51398, 206, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1580038, 51408, 1295, 38, 43, "Input"],
Cell[1581336, 51448, 183, 5, 28, "Output"]
}, Open  ]],
Cell[1581534, 51456, 36, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[1581595, 51460, 562, 15, 43, "Input"],
Cell[1582160, 51477, 169, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1582366, 51486, 592, 17, 27, "Input"],
Cell[1582961, 51505, 137, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1583135, 51514, 562, 15, 43, "Input"],
Cell[1583700, 51531, 169, 4, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1583930, 51542, 66, 1, 56, "Subsection",
  CellTags->"OPESumExplicit"],

Cell[CellGroupData[{
Cell[1584021, 51547, 36, 0, 48, "Subsubsection"],
Cell[1584060, 51549, 56, 0, 32, "Text"],
Cell[1584119, 51551, 298, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1584454, 51569, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1584512, 51573, 402, 11, 27, "Input"],
Cell[1584917, 51586, 404, 12, 53, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1585358, 51603, 71, 1, 27, "Input"],
Cell[1585432, 51606, 442, 14, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1585911, 51625, 606, 16, 27, "Input"],
Cell[1586520, 51643, 826, 24, 59, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1587383, 51672, 71, 1, 27, "Input"],
Cell[1587457, 51675, 2762, 81, 88, "Output"]
}, Open  ]],
Cell[1590234, 51759, 114, 3, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1590397, 51768, 50, 1, 56, "Subsection",
  CellTags->"OPESum"],

Cell[CellGroupData[{
Cell[1590472, 51773, 36, 0, 48, "Subsubsection"],
Cell[1590511, 51775, 106, 3, 32, "Text"],
Cell[1590620, 51780, 322, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1590979, 51798, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1591037, 51802, 499, 13, 27, "Input"],
Cell[1591539, 51817, 1032, 28, 53, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1592608, 51850, 71, 1, 27, "Input"],
Cell[1592682, 51853, 2074, 58, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1594793, 51916, 606, 16, 27, "Input"],
Cell[1595402, 51934, 826, 24, 59, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1596265, 51963, 71, 1, 27, "Input"],
Cell[1596339, 51966, 2762, 81, 88, "Output"]
}, Open  ]],
Cell[1599116, 52050, 114, 3, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1599279, 52059, 66, 1, 56, "Subsection",
  CellTags->"OPESumSimplify"],

Cell[CellGroupData[{
Cell[1599370, 52064, 36, 0, 48, "Subsubsection"],
Cell[1599409, 52066, 63, 0, 32, "Text"],
Cell[1599475, 52068, 298, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1599810, 52086, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1599868, 52090, 115, 4, 27, "Input"],
Cell[1599986, 52096, 1334, 35, 51, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1601357, 52136, 59, 1, 27, "Input"],
Cell[1601419, 52139, 1571, 38, 51, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1603027, 52182, 248, 6, 27, "Input"],
Cell[1603278, 52190, 280, 9, 51, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1603595, 52204, 455, 12, 27, "Input"],
Cell[1604053, 52218, 668, 20, 57, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1604758, 52243, 64, 1, 27, "Input"],
Cell[1604825, 52246, 396, 11, 42, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1605282, 52264, 46, 1, 56, "Subsection",
  CellTags->"Pair"],

Cell[CellGroupData[{
Cell[1605353, 52269, 36, 0, 48, "Subsubsection"],
Cell[1605392, 52271, 492, 10, 86, "Text"],
Cell[1605887, 52283, 1053, 48, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1606977, 52336, 33, 0, 48, "Subsubsection"],
Cell[1607013, 52338, 65, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[1607103, 52342, 190, 4, 27, "Input"],
Cell[1607296, 52348, 141, 4, 29, "Output"]
}, Open  ]],
Cell[1607452, 52355, 54, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[1607531, 52359, 252, 6, 27, "Input"],
Cell[1607786, 52367, 350, 11, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1608173, 52383, 180, 4, 27, "Input"],
Cell[1608356, 52389, 239, 8, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1608632, 52402, 169, 4, 27, "Input"],
Cell[1608804, 52408, 219, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1609060, 52419, 169, 4, 27, "Input"],
Cell[1609232, 52425, 134, 4, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1609403, 52434, 200, 5, 27, "Input"],
Cell[1609606, 52441, 297, 8, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1609940, 52454, 202, 5, 27, "Input"],
Cell[1610145, 52461, 134, 4, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1610316, 52470, 202, 5, 27, "Input"],
Cell[1610521, 52477, 134, 4, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1610692, 52486, 306, 8, 27, "Input"],
Cell[1611001, 52496, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1611097, 52502, 258, 6, 27, "Input"],
Cell[1611358, 52510, 337, 10, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1611732, 52525, 390, 10, 27, "Input"],
Cell[1612125, 52537, 59, 1, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1612245, 52545, 54, 1, 56, "Subsection",
  CellTags->"PartialD"],

Cell[CellGroupData[{
Cell[1612324, 52550, 36, 0, 48, "Subsubsection"],
Cell[1612363, 52552, 893, 30, 50, "Text"],
Cell[1613259, 52584, 465, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1613761, 52607, 42, 0, 47, "Subsubsection"],

Cell[CellGroupData[{
Cell[1613828, 52611, 206, 5, 27, "Input"],
Cell[1614037, 52618, 308, 9, 34, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1614382, 52632, 70, 1, 27, "Input"],
Cell[1614455, 52635, 308, 9, 34, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1614800, 52649, 68, 1, 27, "Input"],
Cell[1614871, 52652, 268, 6, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1615200, 52665, 46, 1, 56, "Subsection",
  CellTags->"PaVe"],

Cell[CellGroupData[{
Cell[1615271, 52670, 36, 0, 48, "Subsubsection"],
Cell[1615310, 52672, 411, 7, 86, "Text"],
Cell[1615724, 52681, 258, 7, 50, "Text"],
Cell[1615985, 52690, 173, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1616195, 52703, 33, 0, 48, "Subsubsection"],
Cell[1616231, 52705, 59, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[1616315, 52709, 295, 8, 27, "Input"],
Cell[1616613, 52719, 635, 12, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1617285, 52736, 263, 7, 27, "Input"],
Cell[1617551, 52745, 2503, 59, 123, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1620115, 52811, 56, 1, 56, "Subsection",
  CellTags->"PaVeOrder"],

Cell[CellGroupData[{
Cell[1620196, 52816, 36, 0, 48, "Subsubsection"],
Cell[1620235, 52818, 331, 6, 86, "Text"],
Cell[1620569, 52826, 173, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1620779, 52839, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1620837, 52843, 416, 10, 43, "Input"],
Cell[1621256, 52855, 728, 15, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1622021, 52875, 416, 10, 43, "Input"],
Cell[1622440, 52887, 728, 15, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1623205, 52907, 776, 18, 59, "Input"],
Cell[1623984, 52927, 1385, 27, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1625430, 52961, 58, 1, 56, "Subsection",
  CellTags->"PaVeReduce"],

Cell[CellGroupData[{
Cell[1625513, 52966, 36, 0, 48, "Subsubsection"],
Cell[1625552, 52968, 142, 3, 50, "Text"],

Cell[CellGroupData[{
Cell[1625719, 52975, 72, 1, 27, "Input"],
Cell[1625794, 52978, 455, 11, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1626286, 52994, 58, 1, 27, "Input"],
Cell[1626347, 52997, 217, 4, 42, "Print"]
}, Open  ]],
Cell[1626579, 53004, 274, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1626890, 53022, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1626948, 53026, 567, 14, 43, "Input"],
Cell[1627518, 53042, 4008, 105, 132, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1631563, 53152, 58, 1, 27, "Input"],
Cell[1631624, 53155, 92, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1631753, 53162, 59, 1, 27, "Input"],
Cell[1631815, 53165, 92, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1631944, 53172, 531, 13, 59, "Input"],
Cell[1632478, 53187, 460, 12, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1632975, 53204, 411, 10, 59, "Input"],
Cell[1633389, 53216, 541, 13, 75, "Output"]
}, Open  ]],
Cell[1633945, 53232, 179, 5, 27, "Input"],
Cell[1634127, 53239, 123, 3, 27, "Input"],

Cell[CellGroupData[{
Cell[1634275, 53246, 718, 17, 59, "Input"],
Cell[1634996, 53265, 11753, 271, 443, "Output"]
}, Open  ]],
Cell[1646764, 53539, 241, 6, 27, "Input"],

Cell[CellGroupData[{
Cell[1647030, 53549, 383, 9, 59, "Input"],
Cell[1647416, 53560, 4340, 90, 525, "Output"]
}, Open  ]],
Cell[1651771, 53653, 223, 5, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1652043, 53664, 70, 1, 56, "Subsection",
  CellTags->"PlusDistribution"],

Cell[CellGroupData[{
Cell[1652138, 53669, 36, 0, 48, "Subsubsection"],
Cell[1652177, 53671, 116, 3, 32, "Text"],
Cell[1652296, 53676, 173, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1652506, 53689, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1652564, 53693, 159, 4, 27, "Input"],
Cell[1652726, 53699, 176, 5, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1652939, 53709, 226, 6, 27, "Input"],
Cell[1653168, 53717, 251, 7, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1653456, 53729, 316, 8, 27, "Input"],
Cell[1653775, 53739, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1653871, 53745, 391, 10, 27, "Input"],
Cell[1654265, 53757, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1654361, 53763, 428, 11, 27, "Input"],
Cell[1654792, 53776, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1654888, 53782, 294, 8, 27, "Input"],
Cell[1655185, 53792, 389, 12, 46, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1655635, 53811, 62, 1, 56, "Subsection",
  CellTags->"Polarization"],

Cell[CellGroupData[{
Cell[1655722, 53816, 36, 0, 48, "Subsubsection"],
Cell[1655761, 53818, 742, 11, 158, "Text"],
Cell[1656506, 53831, 197, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1656740, 53844, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1656798, 53848, 68, 1, 27, "Input"],
Cell[1656869, 53851, 163, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1657069, 53860, 110, 3, 27, "Input"],
Cell[1657182, 53865, 96, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1657315, 53872, 114, 3, 27, "Input"],
Cell[1657432, 53877, 362, 9, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1657831, 53891, 159, 4, 27, "Input"],
Cell[1657993, 53897, 120, 3, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1658150, 53905, 109, 2, 27, "Input"],
Cell[1658262, 53909, 350, 9, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1658649, 53923, 157, 4, 27, "Input"],
Cell[1658809, 53929, 182, 4, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1659028, 53938, 282, 7, 27, "Input"],
Cell[1659313, 53947, 59, 1, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1659433, 53955, 68, 1, 56, "Subsection",
  CellTags->"PolarizationSum"],

Cell[CellGroupData[{
Cell[1659526, 53960, 36, 0, 48, "Subsubsection"],
Cell[1659565, 53962, 249, 5, 68, "Text"],

Cell[CellGroupData[{
Cell[1659839, 53971, 77, 1, 27, "Input"],
Cell[1659919, 53974, 132, 4, 28, "Output"]
}, Open  ]],
Cell[1660066, 53981, 179, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1660282, 53994, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1660340, 53998, 106, 2, 27, "Input"],
Cell[1660449, 54002, 162, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1660648, 54012, 116, 2, 27, "Input"],
Cell[1660767, 54016, 750, 24, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1661554, 54045, 126, 2, 27, "Input"],
Cell[1661683, 54049, 2403, 72, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1664123, 54126, 215, 5, 27, "Input"],
Cell[1664341, 54133, 6040, 171, 90, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1670442, 54311, 74, 1, 56, "Subsection",
  CellTags->"PolarizationVector"],

Cell[CellGroupData[{
Cell[1670541, 54316, 36, 0, 48, "Subsubsection"],
Cell[1670580, 54318, 70, 0, 32, "Text"],
Cell[1670653, 54320, 404, 17, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1671094, 54342, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1671152, 54346, 105, 2, 27, "Input"],
Cell[1671260, 54350, 144, 4, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1671441, 54359, 147, 3, 27, "Input"],
Cell[1671591, 54364, 154, 4, 30, "Output"]
}, Open  ]],
Cell[1671760, 54371, 231, 8, 32, "Text"],

Cell[CellGroupData[{
Cell[1672016, 54383, 143, 3, 27, "Input"],
Cell[1672162, 54388, 255, 6, 40, "Output"]
}, Open  ]],
Cell[1672432, 54397, 513, 17, 32, "Text"],

Cell[CellGroupData[{
Cell[1672970, 54418, 254, 6, 27, "Input"],
Cell[1673227, 54426, 207, 6, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1673471, 54437, 65, 1, 27, "Input"],
Cell[1673539, 54440, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1673635, 54446, 287, 7, 27, "Input"],
Cell[1673925, 54455, 284, 8, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1674246, 54468, 65, 1, 27, "Input"],
Cell[1674314, 54471, 448, 11, 28, "Output"]
}, Open  ]],
Cell[1674777, 54485, 1953, 52, 47, "Text"],

Cell[CellGroupData[{
Cell[1676755, 54541, 287, 7, 27, "Input"],
Cell[1677045, 54550, 284, 8, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1677366, 54563, 65, 1, 27, "Input"],
Cell[1677434, 54566, 448, 11, 28, "Output"]
}, Open  ]],
Cell[1677897, 54580, 101, 2, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1678047, 54588, 64, 1, 56, "Subsection",
  CellTags->"PowerSimplify"],

Cell[CellGroupData[{
Cell[1678136, 54593, 36, 0, 48, "Subsubsection"],
Cell[1678175, 54595, 380, 6, 86, "Text"],
Cell[1678558, 54603, 185, 10, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1678780, 54618, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1678838, 54622, 209, 6, 27, "Input"],
Cell[1679050, 54630, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1679146, 54636, 214, 6, 27, "Input"],
Cell[1679363, 54644, 142, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1679542, 54653, 214, 6, 27, "Input"],
Cell[1679759, 54661, 142, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1679938, 54670, 154, 4, 27, "Input"],
Cell[1680095, 54676, 142, 4, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1680298, 54687, 96, 1, 56, "Subsection",
  CellTags->"PropagatorDenominatorExplicit"],

Cell[CellGroupData[{
Cell[1680419, 54692, 36, 0, 48, "Subsubsection"],
Cell[1680458, 54694, 199, 4, 50, "Text"],
Cell[1680660, 54700, 355, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1681052, 54718, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1681110, 54722, 288, 9, 27, "Input"],
Cell[1681401, 54733, 352, 11, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1681790, 54749, 122, 3, 27, "Input"],
Cell[1681915, 54754, 1118, 32, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1683070, 54791, 64, 1, 27, "Input"],
Cell[1683137, 54794, 609, 17, 59, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1683807, 54818, 80, 1, 56, "Subsection",
  CellTags->"PropagatorDenominator"],

Cell[CellGroupData[{
Cell[1683912, 54823, 36, 0, 48, "Subsubsection"],
Cell[1683951, 54825, 532, 14, 68, "Text"],
Cell[1684486, 54841, 379, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1684902, 54859, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1684960, 54863, 128, 3, 27, "Input"],
Cell[1685091, 54868, 177, 5, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1685305, 54878, 77, 1, 27, "Input"],
Cell[1685385, 54881, 114, 3, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1685536, 54889, 160, 4, 27, "Input"],
Cell[1685699, 54895, 177, 5, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1685913, 54905, 103, 2, 27, "Input"],
Cell[1686019, 54909, 177, 4, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1686233, 54918, 144, 3, 27, "Input"],
Cell[1686380, 54923, 177, 4, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1686594, 54932, 86, 1, 27, "Input"],
Cell[1686683, 54935, 232, 7, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1686952, 54947, 68, 1, 27, "Input"],
Cell[1687023, 54950, 292, 8, 59, "Output"]
}, Open  ]],
Cell[1687330, 54961, 62, 1, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1687441, 54968, 62, 1, 56, "Subsection",
  CellTags->"QuantumField"],

Cell[CellGroupData[{
Cell[1687528, 54973, 36, 0, 48, "Subsubsection"],
Cell[1687567, 54975, 329, 6, 86, "Text"],
Cell[1687899, 54983, 423, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1688359, 55006, 33, 0, 48, "Subsubsection"],
Cell[1688395, 55008, 44, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[1688464, 55012, 68, 1, 27, "Input"],
Cell[1688535, 55015, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1688631, 55021, 81, 1, 27, "Input"],
Cell[1688715, 55024, 91, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1688843, 55031, 77, 1, 27, "Input"],
Cell[1688923, 55034, 64, 1, 28, "Output"]
}, Open  ]],
Cell[1689002, 55038, 53, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[1689080, 55042, 128, 3, 27, "Input"],
Cell[1689211, 55047, 89, 2, 29, "Output"]
}, Open  ]],
Cell[1689315, 55052, 67, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[1689407, 55056, 176, 4, 27, "Input"],
Cell[1689586, 55062, 342, 12, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1689965, 55079, 64, 1, 27, "Input"],
Cell[1690032, 55082, 205, 4, 40, "Output"]
}, Open  ]],
Cell[1690252, 55089, 381, 13, 32, "Text"],

Cell[CellGroupData[{
Cell[1690658, 55106, 170, 4, 27, "Input"],
Cell[1690831, 55112, 254, 8, 28, "Output"]
}, Open  ]],
Cell[1691100, 55123, 160, 4, 50, "Text"],

Cell[CellGroupData[{
Cell[1691285, 55131, 215, 5, 27, "Input"],
Cell[1691503, 55138, 195, 4, 40, "Output"]
}, Open  ]],
Cell[1691713, 55145, 61, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[1691799, 55149, 183, 4, 27, "Input"],
Cell[1691985, 55155, 383, 12, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1692405, 55172, 146, 4, 27, "Input"],
Cell[1692554, 55178, 237, 7, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1692828, 55190, 228, 5, 27, "Input"],
Cell[1693059, 55197, 446, 14, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1693542, 55216, 271, 7, 27, "Input"],
Cell[1693816, 55225, 454, 14, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1694307, 55244, 195, 4, 27, "Input"],
Cell[1694505, 55250, 62, 1, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1694628, 55258, 58, 1, 56, "Subsection",
  CellTags->"QuarkField"],

Cell[CellGroupData[{
Cell[1694711, 55263, 36, 0, 48, "Subsubsection"],
Cell[1694750, 55265, 167, 4, 50, "Text"],
Cell[1694920, 55271, 316, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1695273, 55289, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1695331, 55293, 36, 0, 27, "Input"],
Cell[1695370, 55295, 64, 1, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1695495, 55303, 70, 1, 56, "Subsection",
  CellTags->"QuarkGluonVertex"],

Cell[CellGroupData[{
Cell[1695590, 55308, 36, 0, 48, "Subsubsection"],
Cell[1695629, 55310, 201, 6, 32, "Text"],

Cell[CellGroupData[{
Cell[1695855, 55320, 78, 1, 27, "Input"],
Cell[1695936, 55323, 435, 10, 46, "Output"]
}, Open  ]],
Cell[1696386, 55336, 176, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1696599, 55349, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1696657, 55353, 103, 2, 27, "Input"],
Cell[1696763, 55357, 466, 14, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1697266, 55376, 163, 3, 27, "Input"],
Cell[1697432, 55381, 812, 23, 51, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1698281, 55409, 163, 3, 27, "Input"],
Cell[1698447, 55414, 675, 19, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1699159, 55438, 163, 3, 27, "Input"],
Cell[1699325, 55443, 777, 22, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1700139, 55470, 291, 7, 27, "Input"],
Cell[1700433, 55479, 2627, 68, 53, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1703097, 55552, 292, 7, 27, "Input"],
Cell[1703392, 55561, 466, 14, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1703919, 55582, 68, 1, 56, "Subsection",
  CellTags->"QuarkPropagator"],

Cell[CellGroupData[{
Cell[1704012, 55587, 36, 0, 48, "Subsubsection"],
Cell[1704051, 55589, 144, 3, 50, "Text"],

Cell[CellGroupData[{
Cell[1704220, 55596, 77, 1, 27, "Input"],
Cell[1704300, 55599, 485, 11, 46, "Output"]
}, Open  ]],
Cell[1704800, 55613, 331, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1705168, 55631, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1705226, 55635, 71, 1, 27, "Input"],
Cell[1705300, 55638, 363, 10, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1705700, 55653, 127, 3, 27, "Input"],
Cell[1705830, 55658, 514, 14, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1706381, 55677, 108, 2, 27, "Input"],
Cell[1706492, 55681, 363, 10, 45, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1706916, 55698, 64, 1, 56, "Subsection",
  CellTags->"RightPartialD"],

Cell[CellGroupData[{
Cell[1707005, 55703, 36, 0, 48, "Subsubsection"],
Cell[1707044, 55705, 246, 9, 32, "Text"],
Cell[1707293, 55716, 435, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1707765, 55739, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1707823, 55743, 73, 1, 27, "Input"],
Cell[1707899, 55746, 238, 7, 34, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1708174, 55758, 223, 5, 27, "Input"],
Cell[1708400, 55765, 309, 9, 34, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1708746, 55779, 70, 1, 27, "Input"],
Cell[1708819, 55782, 383, 12, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1709239, 55799, 64, 1, 27, "Input"],
Cell[1709306, 55802, 247, 5, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1709590, 55812, 115, 3, 27, "Input"],
Cell[1709708, 55817, 117, 2, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1709886, 55826, 62, 1, 56, "Subsection",
  CellTags->"RussianTrick"],

Cell[CellGroupData[{
Cell[1709973, 55831, 36, 0, 48, "Subsubsection"],
Cell[1710012, 55833, 291, 5, 68, "Text"],
Cell[1710306, 55840, 63, 0, 32, "Text"],
Cell[1710372, 55842, 319, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1710728, 55860, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1710786, 55864, 395, 11, 27, "Input"],
Cell[1711184, 55877, 569, 19, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1711790, 55901, 58, 1, 27, "Input"],
Cell[1711851, 55904, 1435, 43, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1713323, 55952, 131, 3, 27, "Input"],
Cell[1713457, 55957, 8478, 239, 208, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1721972, 56201, 62, 1, 27, "Input"],
Cell[1722037, 56204, 3550, 109, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1725624, 56318, 89, 2, 27, "Input"],
Cell[1725716, 56322, 4081, 110, 45, "Output"]
}, Open  ]],
Cell[1729812, 56435, 61, 1, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1729922, 56442, 64, 1, 56, "Subsection",
  CellTags->"ScalarProduct"],

Cell[CellGroupData[{
Cell[1730011, 56447, 36, 0, 48, "Subsubsection"],
Cell[1730050, 56449, 846, 20, 122, "Text"],
Cell[1730899, 56471, 764, 32, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1731700, 56508, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1731758, 56512, 96, 2, 27, "Input"],
Cell[1731857, 56516, 219, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1732113, 56527, 149, 4, 27, "Input"],
Cell[1732265, 56533, 383, 11, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1732685, 56549, 96, 2, 27, "Input"],
Cell[1732784, 56553, 134, 4, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1732955, 56562, 69, 1, 27, "Input"],
Cell[1733027, 56565, 134, 4, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1733198, 56574, 137, 3, 27, "Input"],
Cell[1733338, 56579, 170, 4, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1733545, 56588, 199, 5, 27, "Input"],
Cell[1733747, 56595, 232, 6, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1734016, 56606, 222, 6, 27, "Input"],
Cell[1734241, 56614, 84, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1734362, 56621, 279, 7, 27, "Input"],
Cell[1734644, 56630, 707, 20, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1735388, 56655, 264, 7, 27, "Input"],
Cell[1735655, 56664, 707, 20, 40, "Output"]
}, Open  ]],
Cell[1736377, 56687, 45, 0, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1736471, 56693, 76, 1, 56, "Subsection",
  CellTags->"ScalarProductCancel"],

Cell[CellGroupData[{
Cell[1736572, 56698, 36, 0, 48, "Subsubsection"],
Cell[1736611, 56700, 152, 3, 50, "Text"],

Cell[CellGroupData[{
Cell[1736788, 56707, 29, 0, 27, "Input"],
Cell[1736820, 56709, 77, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1736934, 56715, 77, 1, 27, "Input"],
Cell[1737014, 56718, 372, 8, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1737423, 56731, 49, 1, 27, "Input"],
Cell[1737475, 56734, 124, 3, 24, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1737648, 56743, 33, 0, 48, "Subsubsection"],
Cell[1737684, 56745, 221, 8, 32, "Text"],

Cell[CellGroupData[{
Cell[1737930, 56757, 456, 13, 27, "Input"],
Cell[1738389, 56772, 522, 16, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1738948, 56793, 103, 2, 27, "Input"],
Cell[1739054, 56797, 1037, 31, 58, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1740128, 56833, 109, 2, 27, "Input"],
Cell[1740240, 56837, 827, 25, 58, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1741104, 56867, 132, 3, 27, "Input"],
Cell[1741239, 56872, 827, 25, 58, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1742103, 56902, 627, 16, 59, "Input"],
Cell[1742733, 56920, 1886, 57, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1744656, 56982, 149, 3, 27, "Input"],
Cell[1744808, 56987, 6277, 177, 153, "Output"]
}, Open  ]],
Cell[1751100, 57167, 114, 3, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1751263, 57176, 42, 1, 56, "Subsection",
  CellTags->"SD"],

Cell[CellGroupData[{
Cell[1751330, 57181, 36, 0, 48, "Subsubsection"],
Cell[1751369, 57183, 200, 4, 50, "Text"],
Cell[1751572, 57189, 167, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1751776, 57202, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1751834, 57206, 85, 2, 27, "Input"],
Cell[1751922, 57210, 130, 3, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1752089, 57218, 94, 2, 27, "Input"],
Cell[1752186, 57222, 174, 4, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1752397, 57231, 94, 2, 27, "Input"],
Cell[1752494, 57235, 86, 2, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1752641, 57244, 58, 1, 56, "Subsection",
  CellTags->"SelectFree"],

Cell[CellGroupData[{
Cell[1752724, 57249, 36, 0, 48, "Subsubsection"],
Cell[1752763, 57251, 230, 4, 50, "Text"],
Cell[1752996, 57257, 52, 0, 32, "Text"],
Cell[1753051, 57259, 295, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1753383, 57277, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1753441, 57281, 181, 5, 27, "Input"],
Cell[1753625, 57288, 86, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1753748, 57295, 127, 3, 27, "Input"],
Cell[1753878, 57300, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1753974, 57306, 253, 6, 27, "Input"],
Cell[1754230, 57314, 86, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1754353, 57321, 93, 2, 27, "Input"],
Cell[1754449, 57325, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1754545, 57331, 93, 2, 27, "Input"],
Cell[1754641, 57335, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1754737, 57341, 93, 2, 27, "Input"],
Cell[1754833, 57345, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1754929, 57351, 127, 3, 27, "Input"],
Cell[1755059, 57356, 59, 1, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1755179, 57364, 64, 1, 56, "Subsection",
  CellTags->"SelectNotFree"],

Cell[CellGroupData[{
Cell[1755268, 57369, 36, 0, 48, "Subsubsection"],
Cell[1755307, 57371, 115, 3, 32, "Text"],
Cell[1755425, 57376, 244, 5, 68, "Text"],
Cell[1755672, 57383, 55, 0, 32, "Text"],
Cell[1755730, 57385, 286, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1756053, 57403, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1756111, 57407, 171, 4, 27, "Input"],
Cell[1756285, 57413, 120, 3, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1756442, 57421, 251, 6, 27, "Input"],
Cell[1756696, 57429, 130, 3, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1756863, 57437, 96, 2, 27, "Input"],
Cell[1756962, 57441, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1757058, 57447, 125, 3, 27, "Input"],
Cell[1757186, 57452, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1757282, 57458, 96, 2, 27, "Input"],
Cell[1757381, 57462, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1757477, 57468, 96, 2, 27, "Input"],
Cell[1757576, 57472, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1757672, 57478, 130, 3, 27, "Input"],
Cell[1757805, 57483, 91, 2, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1757957, 57492, 52, 1, 56, "Subsection",
  CellTags->"Series2"],

Cell[CellGroupData[{
Cell[1758034, 57497, 36, 0, 48, "Subsubsection"],
Cell[1758073, 57499, 272, 5, 68, "Text"],

Cell[CellGroupData[{
Cell[1758370, 57508, 69, 1, 27, "Input"],
Cell[1758442, 57511, 471, 12, 28, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1758962, 57529, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1759020, 57533, 352, 10, 27, "Input"],
Cell[1759375, 57545, 313, 9, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1759725, 57559, 141, 4, 27, "Input"],
Cell[1759869, 57565, 210, 6, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1760116, 57576, 197, 5, 27, "Input"],
Cell[1760316, 57583, 334, 9, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1760687, 57597, 141, 4, 27, "Input"],
Cell[1760831, 57603, 513, 15, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1761381, 57623, 273, 6, 27, "Input"],
Cell[1761657, 57631, 1321, 35, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1763015, 57671, 296, 8, 27, "Input"],
Cell[1763314, 57681, 1321, 35, 45, "Output"]
}, Open  ]],
Cell[1764650, 57719, 93, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[1764768, 57726, 599, 16, 59, "Input"],
Cell[1765370, 57744, 648, 19, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1766055, 57768, 720, 18, 59, "Input"],
Cell[1766778, 57788, 1354, 39, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1768169, 57832, 212, 4, 27, "Input"],
Cell[1768384, 57838, 806, 24, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1769227, 57867, 166, 4, 27, "Input"],
Cell[1769396, 57873, 2743, 70, 143, "Output"]
}, Open  ]],
Cell[1772154, 57946, 332, 9, 50, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1772535, 57961, 80, 1, 56, "Subsection",
  CellTags->"SimplifyDeltaFunction"],

Cell[CellGroupData[{
Cell[1772640, 57966, 36, 0, 48, "Subsubsection"],
Cell[1772679, 57968, 258, 5, 68, "Text"],
Cell[1772940, 57975, 331, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1773308, 57993, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1773366, 57997, 157, 4, 27, "Input"],
Cell[1773526, 58003, 245, 7, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1773808, 58015, 109, 2, 27, "Input"],
Cell[1773920, 58019, 400, 12, 37, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1774357, 58036, 157, 4, 27, "Input"],
Cell[1774517, 58042, 284, 8, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1774838, 58055, 109, 2, 27, "Input"],
Cell[1774950, 58059, 938, 26, 37, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1775925, 58090, 174, 4, 27, "Input"],
Cell[1776102, 58096, 296, 8, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1776435, 58109, 109, 2, 27, "Input"],
Cell[1776547, 58113, 176, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1776760, 58123, 228, 6, 27, "Input"],
Cell[1776991, 58131, 346, 10, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1777374, 58146, 109, 2, 27, "Input"],
Cell[1777486, 58150, 206, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1777729, 58161, 267, 7, 27, "Input"],
Cell[1777999, 58170, 392, 11, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1778428, 58186, 109, 2, 27, "Input"],
Cell[1778540, 58190, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1778636, 58196, 228, 6, 27, "Input"],
Cell[1778867, 58204, 346, 10, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1779250, 58219, 109, 2, 27, "Input"],
Cell[1779362, 58223, 206, 6, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1779629, 58236, 68, 1, 56, "Subsection",
  CellTags->"SimplifyPolyLog"],

Cell[CellGroupData[{
Cell[1779722, 58241, 36, 0, 48, "Subsubsection"],
Cell[1779761, 58243, 171, 4, 50, "Text"],
Cell[1779935, 58249, 164, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1780136, 58262, 33, 0, 48, "Subsubsection"],
Cell[1780172, 58264, 184, 4, 27, "Input"],

Cell[CellGroupData[{
Cell[1780381, 58272, 157, 4, 27, "Input"],
Cell[1780541, 58278, 795, 21, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1781373, 58304, 126, 3, 27, "Input"],
Cell[1781502, 58309, 481, 13, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1782020, 58327, 190, 5, 27, "Input"],
Cell[1782213, 58334, 765, 21, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1783015, 58360, 157, 4, 27, "Input"],
Cell[1783175, 58366, 737, 20, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1783949, 58391, 257, 7, 27, "Input"],
Cell[1784209, 58400, 786, 22, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1785032, 58427, 225, 6, 27, "Input"],
Cell[1785260, 58435, 756, 21, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1786053, 58461, 267, 7, 27, "Input"],
Cell[1786323, 58470, 903, 28, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1787263, 58503, 185, 5, 27, "Input"],
Cell[1787451, 58510, 565, 16, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1788053, 58531, 157, 4, 27, "Input"],
Cell[1788213, 58537, 62, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1788312, 58543, 157, 4, 27, "Input"],
Cell[1788472, 58549, 1509, 42, 47, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1790018, 58596, 257, 7, 27, "Input"],
Cell[1790278, 58605, 1826, 50, 63, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1792141, 58660, 190, 5, 27, "Input"],
Cell[1792334, 58667, 1745, 49, 67, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1794116, 58721, 257, 7, 27, "Input"],
Cell[1794376, 58730, 3692, 101, 117, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1798105, 58836, 153, 4, 27, "Input"],
Cell[1798261, 58842, 321, 10, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1798619, 58857, 122, 3, 27, "Input"],
Cell[1798744, 58862, 214, 6, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1798995, 58873, 97, 2, 27, "Input"],
Cell[1799095, 58877, 418, 13, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1799550, 58895, 97, 2, 27, "Input"],
Cell[1799650, 58899, 353, 11, 38, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1800040, 58915, 97, 2, 27, "Input"],
Cell[1800140, 58919, 353, 11, 38, "Output"]
}, Open  ]],
Cell[1800508, 58933, 63, 1, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1800620, 58940, 44, 1, 56, "Subsection",
  CellTags->"SOD"],

Cell[CellGroupData[{
Cell[1800689, 58945, 36, 0, 48, "Subsubsection"],
Cell[1800728, 58947, 173, 4, 50, "Text"],
Cell[1800904, 58953, 512, 23, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1801453, 58981, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1801511, 58985, 59, 1, 27, "Input"],
Cell[1801573, 58988, 268, 8, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1801878, 59001, 86, 2, 27, "Input"],
Cell[1801967, 59005, 419, 13, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1802423, 59023, 137, 4, 27, "Input"],
Cell[1802563, 59029, 239, 6, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1802863, 59042, 50, 1, 56, "Subsection",
  CellTags->"Solve2"],

Cell[CellGroupData[{
Cell[1802938, 59047, 36, 0, 48, "Subsubsection"],
Cell[1802977, 59049, 324, 6, 68, "Text"],

Cell[CellGroupData[{
Cell[1803326, 59059, 68, 1, 27, "Input"],
Cell[1803397, 59062, 248, 6, 28, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1803694, 59074, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1803752, 59078, 419, 12, 27, "Input"],
Cell[1804174, 59092, 399, 11, 43, "Output"]
}, Open  ]],
Cell[1804588, 59106, 92, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[1804705, 59113, 118, 3, 27, "Input"],
Cell[1804826, 59118, 147, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1805010, 59127, 250, 6, 27, "Input"],
Cell[1805263, 59135, 147, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1805447, 59144, 477, 13, 27, "Input"],
Cell[1805927, 59159, 321, 9, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1806285, 59173, 418, 12, 27, "Input"],
Cell[1806706, 59187, 449, 13, 43, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1807216, 59207, 42, 1, 56, "Subsection",
  CellTags->"SO"],

Cell[CellGroupData[{
Cell[1807283, 59212, 36, 0, 48, "Subsubsection"],
Cell[1807322, 59214, 156, 3, 50, "Text"],
Cell[1807481, 59219, 609, 27, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1808127, 59251, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1808185, 59255, 58, 1, 27, "Input"],
Cell[1808246, 59258, 268, 8, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1808551, 59271, 85, 2, 27, "Input"],
Cell[1808639, 59275, 419, 13, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1809095, 59293, 136, 4, 27, "Input"],
Cell[1809234, 59299, 177, 4, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1809472, 59310, 42, 1, 56, "Subsection",
  CellTags->"SP"],

Cell[CellGroupData[{
Cell[1809539, 59315, 36, 0, 48, "Subsubsection"],
Cell[1809578, 59317, 335, 10, 50, "Text"],
Cell[1809916, 59329, 441, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1810394, 59352, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1810452, 59356, 157, 4, 27, "Input"],
Cell[1810612, 59362, 460, 15, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1811109, 59382, 169, 5, 27, "Input"],
Cell[1811281, 59389, 399, 10, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1811717, 59404, 222, 6, 27, "Input"],
Cell[1811942, 59412, 463, 14, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1812442, 59431, 137, 3, 27, "Input"],
Cell[1812582, 59436, 480, 15, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1813099, 59456, 126, 3, 27, "Input"],
Cell[1813228, 59461, 86, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1813351, 59468, 167, 5, 27, "Input"],
Cell[1813521, 59475, 170, 4, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1813728, 59484, 203, 6, 27, "Input"],
Cell[1813934, 59492, 86, 2, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1814081, 59501, 44, 1, 56, "Subsection",
  CellTags->"SPD"],

Cell[CellGroupData[{
Cell[1814150, 59506, 36, 0, 48, "Subsubsection"],
Cell[1814189, 59508, 353, 10, 50, "Text"],
Cell[1814545, 59520, 441, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1815023, 59543, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1815081, 59547, 159, 4, 27, "Input"],
Cell[1815243, 59553, 460, 15, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1815740, 59573, 170, 5, 27, "Input"],
Cell[1815913, 59580, 399, 10, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1816349, 59595, 223, 6, 27, "Input"],
Cell[1816575, 59603, 463, 14, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1817075, 59622, 138, 3, 27, "Input"],
Cell[1817216, 59627, 480, 15, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1817733, 59647, 127, 3, 27, "Input"],
Cell[1817863, 59652, 87, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1817987, 59659, 168, 5, 27, "Input"],
Cell[1818158, 59666, 232, 6, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1818427, 59677, 204, 6, 27, "Input"],
Cell[1818634, 59685, 87, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1818758, 59692, 263, 7, 27, "Input"],
Cell[1819024, 59701, 87, 2, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1819172, 59710, 50, 1, 56, "Subsection",
  CellTags->"Spinor"],

Cell[CellGroupData[{
Cell[1819247, 59715, 36, 0, 48, "Subsubsection"],
Cell[1819286, 59717, 666, 19, 86, "Text"],
Cell[1819955, 59738, 664, 28, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1820656, 59771, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1820714, 59775, 62, 1, 27, "Input"],
Cell[1820779, 59778, 190, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1821006, 59787, 89, 2, 27, "Input"],
Cell[1821098, 59791, 237, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1821372, 59801, 237, 6, 27, "Input"],
Cell[1821612, 59809, 324, 8, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1821973, 59822, 101, 2, 27, "Input"],
Cell[1822077, 59826, 144, 4, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1822258, 59835, 187, 5, 27, "Input"],
Cell[1822448, 59842, 144, 4, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1822629, 59851, 130, 3, 27, "Input"],
Cell[1822762, 59856, 144, 4, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1822943, 59865, 185, 5, 27, "Input"],
Cell[1823131, 59872, 144, 4, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1823336, 59883, 58, 1, 56, "Subsection",
  CellTags->"SpinorUBar"],

Cell[CellGroupData[{
Cell[1823419, 59888, 36, 0, 48, "Subsubsection"],
Cell[1823458, 59890, 159, 6, 32, "Text"],
Cell[1823620, 59898, 518, 23, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1824175, 59926, 33, 0, 48, "Subsubsection"],
Cell[1824211, 59928, 60, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[1824296, 59932, 66, 1, 27, "Input"],
Cell[1824365, 59935, 228, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1824630, 59946, 93, 2, 27, "Input"],
Cell[1824726, 59950, 273, 7, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1825036, 59962, 134, 3, 27, "Input"],
Cell[1825173, 59967, 94, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1825304, 59974, 175, 5, 27, "Input"],
Cell[1825482, 59981, 144, 4, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1825663, 59990, 211, 6, 27, "Input"],
Cell[1825877, 59998, 144, 4, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1826082, 60009, 52, 1, 56, "Subsection",
  CellTags->"SpinorU"],

Cell[CellGroupData[{
Cell[1826159, 60014, 36, 0, 48, "Subsubsection"],
Cell[1826198, 60016, 57, 0, 32, "Text"],
Cell[1826258, 60018, 527, 23, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1826822, 60046, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1826880, 60050, 63, 1, 27, "Input"],
Cell[1826946, 60053, 140, 3, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1827123, 60061, 90, 2, 27, "Input"],
Cell[1827216, 60065, 187, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1827440, 60074, 131, 3, 27, "Input"],
Cell[1827574, 60079, 91, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1827702, 60086, 172, 5, 27, "Input"],
Cell[1827877, 60093, 144, 4, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1828058, 60102, 208, 6, 27, "Input"],
Cell[1828269, 60110, 144, 4, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1828474, 60121, 58, 1, 56, "Subsection",
  CellTags->"SpinorVBar"],

Cell[CellGroupData[{
Cell[1828557, 60126, 36, 0, 48, "Subsubsection"],
Cell[1828596, 60128, 164, 6, 32, "Text"],
Cell[1828763, 60136, 518, 23, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1829318, 60164, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1829376, 60168, 66, 1, 27, "Input"],
Cell[1829445, 60171, 228, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1829710, 60182, 93, 2, 27, "Input"],
Cell[1829806, 60186, 273, 7, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1830116, 60198, 134, 3, 27, "Input"],
Cell[1830253, 60203, 94, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1830384, 60210, 175, 5, 27, "Input"],
Cell[1830562, 60217, 174, 5, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1830773, 60227, 211, 6, 27, "Input"],
Cell[1830987, 60235, 174, 5, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1831222, 60247, 52, 1, 56, "Subsection",
  CellTags->"SpinorV"],

Cell[CellGroupData[{
Cell[1831299, 60252, 36, 0, 48, "Subsubsection"],
Cell[1831338, 60254, 57, 0, 32, "Text"],
Cell[1831398, 60256, 527, 23, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1831962, 60284, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1832020, 60288, 63, 1, 27, "Input"],
Cell[1832086, 60291, 140, 3, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1832263, 60299, 90, 2, 27, "Input"],
Cell[1832356, 60303, 187, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1832580, 60312, 131, 3, 27, "Input"],
Cell[1832714, 60317, 91, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1832842, 60324, 172, 5, 27, "Input"],
Cell[1833017, 60331, 174, 5, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1833228, 60341, 208, 6, 27, "Input"],
Cell[1833439, 60349, 174, 5, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1833674, 60361, 72, 1, 56, "Subsection",
  CellTags->"SplittingFunction"],

Cell[CellGroupData[{
Cell[1833771, 60366, 36, 0, 48, "Subsubsection"],
Cell[1833810, 60368, 198, 6, 32, "Text"],

Cell[CellGroupData[{
Cell[1834033, 60378, 79, 1, 27, "Input"],
Cell[1834115, 60381, 135, 4, 28, "Output"]
}, Open  ]],
Cell[1834265, 60388, 197, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1834499, 60401, 33, 0, 48, "Subsubsection"],
Cell[1834535, 60403, 33, 0, 32, "Text"],
Cell[1834571, 60405, 181, 4, 27, "Input"],
Cell[1834755, 60411, 138, 3, 50, "Text"],

Cell[CellGroupData[{
Cell[1834918, 60418, 75, 1, 27, "Input"],
Cell[1834996, 60421, 632, 18, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1835665, 60444, 75, 1, 27, "Input"],
Cell[1835743, 60447, 291, 9, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1836071, 60461, 75, 1, 27, "Input"],
Cell[1836149, 60464, 248, 7, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1836434, 60476, 75, 1, 27, "Input"],
Cell[1836512, 60479, 1009, 27, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1837558, 60511, 75, 1, 27, "Input"],
Cell[1837636, 60514, 1406, 37, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1839079, 60556, 75, 1, 27, "Input"],
Cell[1839157, 60559, 687, 19, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1839881, 60583, 75, 1, 27, "Input"],
Cell[1839959, 60586, 617, 17, 30, "Output"]
}, Open  ]],
Cell[1840591, 60606, 75, 1, 27, "Input"],
Cell[1840669, 60609, 31, 0, 32, "Text"],
Cell[1840703, 60611, 181, 4, 27, "Input"],

Cell[CellGroupData[{
Cell[1840909, 60619, 75, 1, 27, "Input"],
Cell[1840987, 60622, 632, 18, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1841656, 60645, 75, 1, 27, "Input"],
Cell[1841734, 60648, 211, 7, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1841982, 60660, 75, 1, 27, "Input"],
Cell[1842060, 60663, 207, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1842304, 60674, 75, 1, 27, "Input"],
Cell[1842382, 60677, 1001, 28, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1843420, 60710, 75, 1, 27, "Input"],
Cell[1843498, 60713, 1541, 41, 85, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1845076, 60759, 75, 1, 27, "Input"],
Cell[1845154, 60762, 563, 16, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1845754, 60783, 76, 1, 27, "Input"],
Cell[1845833, 60786, 490, 14, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1846360, 60805, 75, 1, 27, "Input"],
Cell[1846438, 60808, 490, 14, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1846965, 60827, 76, 1, 27, "Input"],
Cell[1847044, 60830, 490, 14, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1847571, 60849, 75, 1, 27, "Input"],
Cell[1847649, 60852, 1732, 45, 82, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1849418, 60902, 76, 1, 27, "Input"],
Cell[1849497, 60905, 1732, 45, 82, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1851266, 60955, 76, 1, 27, "Input"],
Cell[1851345, 60958, 749, 21, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1852131, 60984, 77, 1, 27, "Input"],
Cell[1852211, 60987, 7231, 176, 305, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1859479, 61168, 75, 1, 27, "Input"],
Cell[1859557, 61171, 3699, 93, 85, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1863293, 61269, 75, 1, 27, "Input"],
Cell[1863371, 61272, 4595, 117, 193, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1868003, 61394, 75, 1, 27, "Input"],
Cell[1868081, 61397, 5613, 140, 221, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1873755, 61544, 46, 1, 56, "Subsection",
  CellTags->"SumP"],

Cell[CellGroupData[{
Cell[1873826, 61549, 36, 0, 48, "Subsubsection"],
Cell[1873865, 61551, 621, 19, 35, "Text"],
Cell[1874489, 61572, 262, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1874788, 61590, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1874846, 61594, 116, 3, 27, "Input"],
Cell[1874965, 61599, 249, 6, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1875251, 61610, 116, 3, 27, "Input"],
Cell[1875370, 61615, 249, 6, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1875656, 61626, 87, 2, 27, "Input"],
Cell[1875746, 61630, 218, 5, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1876001, 61640, 87, 2, 27, "Input"],
Cell[1876091, 61644, 86, 2, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1876214, 61651, 314, 10, 52, "Input"],
Cell[1876531, 61663, 86, 2, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1876654, 61670, 159, 4, 27, "Input"],
Cell[1876816, 61676, 913, 27, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1877766, 61708, 86, 2, 27, "Input"],
Cell[1877855, 61712, 86, 2, 43, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1878002, 61721, 46, 1, 56, "Subsection",
  CellTags->"SumS"],

Cell[CellGroupData[{
Cell[1878073, 61726, 36, 0, 48, "Subsubsection"],
Cell[1878112, 61728, 1247, 38, 71, "Text"],

Cell[CellGroupData[{
Cell[1879384, 61770, 66, 1, 27, "Input"],
Cell[1879453, 61773, 133, 4, 28, "Output"]
}, Open  ]],
Cell[1879601, 61780, 262, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1879900, 61798, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1879958, 61802, 116, 3, 27, "Input"],
Cell[1880077, 61807, 197, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1880311, 61818, 116, 3, 27, "Input"],
Cell[1880430, 61823, 197, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1880664, 61834, 111, 3, 27, "Input"],
Cell[1880778, 61839, 145, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1880960, 61848, 140, 3, 27, "Input"],
Cell[1881103, 61853, 261, 8, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1881401, 61866, 169, 4, 27, "Input"],
Cell[1881573, 61872, 364, 12, 46, "Output"]
}, Open  ]],
Cell[1881952, 61887, 165, 4, 27, "Input"],

Cell[CellGroupData[{
Cell[1882142, 61895, 116, 3, 27, "Input"],
Cell[1882261, 61900, 570, 18, 46, "Output"]
}, Open  ]],
Cell[1882846, 61921, 166, 4, 27, "Input"],

Cell[CellGroupData[{
Cell[1883037, 61929, 87, 2, 27, "Input"],
Cell[1883127, 61933, 86, 2, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1883250, 61940, 145, 4, 52, "Input"],
Cell[1883398, 61946, 86, 2, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1883521, 61953, 126, 3, 27, "Input"],
Cell[1883650, 61958, 198, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1883885, 61969, 108, 2, 27, "Input"],
Cell[1883996, 61973, 112, 3, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1884145, 61981, 111, 3, 27, "Input"],
Cell[1884259, 61986, 109, 3, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1884405, 61994, 87, 2, 27, "Input"],
Cell[1884495, 61998, 109, 3, 43, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1884665, 62008, 46, 1, 56, "Subsection",
  CellTags->"SumT"],

Cell[CellGroupData[{
Cell[1884736, 62013, 36, 0, 48, "Subsubsection"],
Cell[1884775, 62015, 516, 14, 50, "Text"],
Cell[1885294, 62031, 262, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1885593, 62049, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1885651, 62053, 116, 3, 27, "Input"],
Cell[1885770, 62058, 228, 7, 31, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1886035, 62070, 116, 3, 27, "Input"],
Cell[1886154, 62075, 228, 7, 31, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1886419, 62087, 87, 2, 27, "Input"],
Cell[1886509, 62091, 153, 5, 31, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1886699, 62101, 140, 3, 27, "Input"],
Cell[1886842, 62106, 392, 13, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1887271, 62124, 87, 2, 27, "Input"],
Cell[1887361, 62128, 109, 3, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1887507, 62136, 233, 7, 52, "Input"],
Cell[1887743, 62145, 109, 3, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1887889, 62153, 126, 3, 27, "Input"],
Cell[1888018, 62158, 229, 7, 31, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1888284, 62170, 98, 2, 27, "Input"],
Cell[1888385, 62174, 230, 5, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1888652, 62184, 87, 2, 27, "Input"],
Cell[1888742, 62188, 109, 3, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1888888, 62196, 111, 3, 27, "Input"],
Cell[1889002, 62201, 109, 3, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1889148, 62209, 262, 8, 52, "Input"],
Cell[1889413, 62219, 847, 24, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1890297, 62248, 98, 2, 27, "Input"],
Cell[1890398, 62252, 137, 4, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1890572, 62261, 122, 3, 27, "Input"],
Cell[1890697, 62266, 230, 5, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1890964, 62276, 91, 2, 27, "Input"],
Cell[1891058, 62280, 499, 15, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1891594, 62300, 234, 7, 27, "Input"],
Cell[1891831, 62309, 499, 15, 43, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1892391, 62331, 54, 1, 56, "Subsection",
  CellTags->"SUNDelta"],

Cell[CellGroupData[{
Cell[1892470, 62336, 36, 0, 48, "Subsubsection"],
Cell[1892509, 62338, 101, 3, 32, "Text"],
Cell[1892613, 62343, 749, 32, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1893399, 62380, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1893457, 62384, 91, 2, 27, "Input"],
Cell[1893551, 62388, 130, 3, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1893718, 62396, 242, 6, 27, "Input"],
Cell[1893963, 62404, 342, 11, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1894342, 62420, 173, 4, 27, "Input"],
Cell[1894518, 62426, 342, 11, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1894897, 62442, 132, 3, 27, "Input"],
Cell[1895032, 62447, 92, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1895161, 62454, 173, 5, 27, "Input"],
Cell[1895337, 62461, 174, 4, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1895548, 62470, 209, 6, 27, "Input"],
Cell[1895760, 62478, 86, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1895883, 62485, 167, 5, 27, "Input"],
Cell[1896053, 62492, 174, 4, 40, "Output"]
}, Open  ]],
Cell[1896242, 62499, 236, 5, 68, "Text"],

Cell[CellGroupData[{
Cell[1896503, 62508, 306, 8, 27, "Input"],
Cell[1896812, 62518, 574, 18, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1897423, 62541, 64, 1, 27, "Input"],
Cell[1897490, 62544, 377, 9, 56, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1897904, 62558, 173, 5, 27, "Input"],
Cell[1898080, 62565, 190, 4, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1898331, 62576, 46, 1, 56, "Subsection",
  CellTags->"SUND"],

Cell[CellGroupData[{
Cell[1898402, 62581, 36, 0, 48, "Subsubsection"],
Cell[1898441, 62583, 179, 6, 32, "Text"],
Cell[1898623, 62591, 402, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1899062, 62614, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1899120, 62618, 97, 2, 27, "Input"],
Cell[1899220, 62622, 149, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1899406, 62631, 179, 4, 27, "Input"],
Cell[1899588, 62637, 1549, 43, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1901174, 62685, 97, 2, 27, "Input"],
Cell[1901274, 62689, 149, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1901460, 62698, 97, 2, 27, "Input"],
Cell[1901560, 62702, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1901656, 62708, 250, 6, 27, "Input"],
Cell[1901909, 62716, 250, 7, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1902196, 62728, 381, 10, 43, "Input"],
Cell[1902580, 62740, 316, 10, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1902933, 62755, 381, 10, 43, "Input"],
Cell[1903317, 62767, 588, 17, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1903942, 62789, 141, 4, 27, "Input"],
Cell[1904086, 62795, 98, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1904221, 62802, 179, 5, 27, "Input"],
Cell[1904403, 62809, 221, 5, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1904661, 62819, 217, 6, 27, "Input"],
Cell[1904881, 62827, 98, 2, 40, "Output"]
}, Open  ]],
Cell[1904994, 62832, 62, 1, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1905105, 62839, 46, 1, 56, "Subsection",
  CellTags->"SUNF"],

Cell[CellGroupData[{
Cell[1905176, 62844, 36, 0, 48, "Subsubsection"],
Cell[1905215, 62846, 123, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[1905363, 62853, 66, 1, 27, "Input"],
Cell[1905432, 62856, 135, 4, 28, "Output"]
}, Open  ]],
Cell[1905582, 62863, 738, 33, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1906357, 62901, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1906415, 62905, 292, 8, 27, "Input"],
Cell[1906710, 62915, 333, 9, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1907080, 62929, 61, 1, 27, "Input"],
Cell[1907144, 62932, 1058, 32, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1908239, 62969, 68, 1, 27, "Input"],
Cell[1908310, 62972, 575, 18, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1908922, 62995, 97, 2, 27, "Input"],
Cell[1909022, 62999, 149, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1909208, 63008, 130, 3, 27, "Input"],
Cell[1909341, 63013, 59, 1, 28, "Output"]
}, Open  ]],
Cell[1909415, 63017, 355, 10, 32, "Text"],

Cell[CellGroupData[{
Cell[1909795, 63031, 152, 3, 27, "Input"],
Cell[1909950, 63036, 1653, 43, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1911640, 63084, 250, 6, 27, "Input"],
Cell[1911893, 63092, 417, 13, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1912347, 63110, 217, 5, 27, "Input"],
Cell[1912567, 63117, 1677, 44, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1914281, 63166, 141, 4, 27, "Input"],
Cell[1914425, 63172, 98, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1914560, 63179, 179, 5, 27, "Input"],
Cell[1914742, 63186, 221, 5, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1915000, 63196, 217, 6, 27, "Input"],
Cell[1915220, 63204, 98, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1915355, 63211, 97, 2, 27, "Input"],
Cell[1915455, 63215, 149, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1915641, 63224, 129, 3, 27, "Input"],
Cell[1915773, 63229, 510, 16, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1916344, 63252, 54, 1, 56, "Subsection",
  CellTags->"SUNIndex"],

Cell[CellGroupData[{
Cell[1916423, 63257, 36, 0, 48, "Subsubsection"],
Cell[1916462, 63259, 53, 0, 32, "Text"],
Cell[1916518, 63261, 410, 17, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1916965, 63283, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1917023, 63287, 64, 1, 27, "Input"],
Cell[1917090, 63290, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1917186, 63296, 64, 1, 27, "Input"],
Cell[1917253, 63299, 65, 1, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1917355, 63305, 173, 5, 27, "Input"],
Cell[1917531, 63312, 174, 4, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1917766, 63323, 60, 1, 56, "Subsection",
  CellTags->"SUNSimplify"],

Cell[CellGroupData[{
Cell[1917851, 63328, 36, 0, 48, "Subsubsection"],
Cell[1917890, 63330, 350, 6, 86, "Text"],

Cell[CellGroupData[{
Cell[1918265, 63340, 73, 1, 27, "Input"],
Cell[1918341, 63343, 517, 11, 46, "Output"]
}, Open  ]],
Cell[1918873, 63357, 158, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1919068, 63370, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1919126, 63374, 224, 6, 27, "Input"],
Cell[1919353, 63382, 236, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1919626, 63393, 68, 1, 27, "Input"],
Cell[1919697, 63396, 342, 11, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1920076, 63412, 154, 4, 27, "Input"],
Cell[1920233, 63418, 244, 8, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1920514, 63431, 68, 1, 27, "Input"],
Cell[1920585, 63434, 85, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1920707, 63441, 138, 3, 27, "Input"],
Cell[1920848, 63446, 174, 5, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1921059, 63456, 231, 6, 27, "Input"],
Cell[1921293, 63464, 267, 7, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1921597, 63476, 68, 1, 27, "Input"],
Cell[1921668, 63479, 417, 13, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1922122, 63497, 246, 6, 27, "Input"],
Cell[1922371, 63505, 267, 7, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1922675, 63517, 68, 1, 27, "Input"],
Cell[1922746, 63520, 162, 4, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1922945, 63529, 236, 6, 27, "Input"],
Cell[1923184, 63537, 267, 7, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1923488, 63549, 68, 1, 27, "Input"],
Cell[1923559, 63552, 417, 13, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1924013, 63570, 236, 6, 27, "Input"],
Cell[1924252, 63578, 267, 7, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1924556, 63590, 135, 3, 27, "Input"],
Cell[1924694, 63595, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1924790, 63601, 390, 10, 43, "Input"],
Cell[1925183, 63613, 316, 10, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1925536, 63628, 395, 10, 43, "Input"],
Cell[1925934, 63640, 536, 16, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1926507, 63661, 217, 5, 27, "Input"],
Cell[1926727, 63668, 1677, 44, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1928441, 63717, 217, 5, 27, "Input"],
Cell[1928661, 63724, 1631, 43, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1930329, 63772, 250, 6, 27, "Input"],
Cell[1930582, 63780, 227, 6, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1930846, 63791, 456, 12, 27, "Input"],
Cell[1931305, 63805, 515, 13, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1931857, 63823, 141, 3, 27, "Input"],
Cell[1932001, 63828, 954, 30, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1932992, 63863, 203, 4, 27, "Input"],
Cell[1933198, 63869, 1972, 60, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1935207, 63934, 282, 7, 27, "Input"],
Cell[1935492, 63943, 600, 17, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1936129, 63965, 68, 1, 27, "Input"],
Cell[1936200, 63968, 899, 27, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1937136, 64000, 141, 3, 27, "Input"],
Cell[1937280, 64005, 837, 26, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1938154, 64036, 191, 5, 27, "Input"],
Cell[1938348, 64043, 488, 12, 72, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1938873, 64060, 168, 4, 27, "Input"],
Cell[1939044, 64066, 391, 12, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1939472, 64083, 64, 1, 27, "Input"],
Cell[1939539, 64086, 209, 5, 40, "Output"]
}, Open  ]],
Cell[1939763, 64094, 172, 4, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1939984, 64104, 46, 1, 56, "Subsection",
  CellTags->"SUNT"],

Cell[CellGroupData[{
Cell[1940055, 64109, 36, 0, 48, "Subsubsection"],
Cell[1940094, 64111, 187, 6, 32, "Text"],
Cell[1940284, 64119, 711, 33, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1941032, 64157, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1941090, 64161, 60, 1, 27, "Input"],
Cell[1941153, 64164, 132, 4, 28, "Output"]
}, Open  ]],
Cell[1941300, 64171, 196, 6, 32, "Text"],

Cell[CellGroupData[{
Cell[1941521, 64181, 177, 4, 27, "Input"],
Cell[1941701, 64187, 333, 11, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1942071, 64203, 107, 2, 27, "Input"],
Cell[1942181, 64207, 407, 14, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1942625, 64226, 220, 5, 27, "Input"],
Cell[1942848, 64233, 283, 9, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1943168, 64247, 151, 3, 27, "Input"],
Cell[1943322, 64252, 95, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1943454, 64259, 141, 3, 27, "Input"],
Cell[1943598, 64264, 442, 14, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1944077, 64283, 220, 5, 27, "Input"],
Cell[1944300, 64290, 283, 9, 43, "Output"]
}, Open  ]],
Cell[1944598, 64302, 471, 14, 32, "Text"],

Cell[CellGroupData[{
Cell[1945094, 64320, 128, 3, 27, "Input"],
Cell[1945225, 64325, 387, 12, 42, "Output"]
}, Open  ]],
Cell[1945627, 64340, 190, 6, 32, "Text"],

Cell[CellGroupData[{
Cell[1945842, 64350, 170, 4, 27, "Input"],
Cell[1946015, 64356, 417, 13, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1946469, 64374, 174, 5, 27, "Input"],
Cell[1946646, 64381, 232, 6, 56, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1946915, 64392, 138, 4, 27, "Input"],
Cell[1947056, 64398, 100, 2, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1947193, 64405, 172, 5, 27, "Input"],
Cell[1947368, 64412, 61, 1, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1947490, 64420, 54, 1, 56, "Subsection",
  CellTags->"SUNTrace"],

Cell[CellGroupData[{
Cell[1947569, 64425, 36, 0, 48, "Subsubsection"],
Cell[1947608, 64427, 58, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[1947691, 64431, 70, 1, 27, "Input"],
Cell[1947764, 64434, 135, 4, 28, "Output"]
}, Open  ]],
Cell[1947914, 64441, 277, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1948228, 64459, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1948286, 64463, 128, 3, 27, "Input"],
Cell[1948417, 64468, 387, 12, 42, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1948841, 64485, 138, 3, 27, "Input"],
Cell[1948982, 64490, 684, 20, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1949703, 64515, 214, 5, 27, "Input"],
Cell[1949920, 64522, 1138, 34, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1951095, 64561, 148, 3, 27, "Input"],
Cell[1951246, 64566, 848, 25, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1952131, 64596, 260, 6, 27, "Input"],
Cell[1952394, 64604, 6212, 182, 79, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1958643, 64791, 140, 3, 27, "Input"],
Cell[1958786, 64796, 848, 25, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1959671, 64826, 281, 7, 27, "Input"],
Cell[1959955, 64835, 17454, 439, 202, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1977446, 65279, 140, 3, 27, "Input"],
Cell[1977589, 65284, 1012, 30, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1978638, 65319, 250, 6, 27, "Input"],
Cell[1978891, 65327, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1978987, 65333, 250, 6, 27, "Input"],
Cell[1979240, 65341, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1979336, 65347, 250, 6, 27, "Input"],
Cell[1979589, 65355, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1979685, 65361, 250, 6, 27, "Input"],
Cell[1979938, 65369, 690, 21, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1980689, 65397, 58, 1, 56, "Subsection",
  CellTags->"Symmetrize"],

Cell[CellGroupData[{
Cell[1980772, 65402, 36, 0, 48, "Subsubsection"],
Cell[1980811, 65404, 120, 3, 32, "Text"],
Cell[1980934, 65409, 178, 7, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1981149, 65421, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[1981207, 65425, 218, 6, 27, "Input"],
Cell[1981428, 65433, 322, 10, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1981787, 65448, 238, 6, 27, "Input"],
Cell[1982028, 65456, 718, 18, 43, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1982807, 65481, 46, 1, 56, "Subsection",
  CellTags->"Tdec"],

Cell[CellGroupData[{
Cell[1982878, 65486, 36, 0, 48, "Subsubsection"],
Cell[1982917, 65488, 239, 5, 68, "Text"],

Cell[CellGroupData[{
Cell[1983181, 65497, 66, 1, 27, "Input"],
Cell[1983250, 65500, 474, 10, 46, "Output"]
}, Open  ]],
Cell[1983739, 65513, 399, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1984175, 65536, 33, 0, 48, "Subsubsection"],
Cell[1984211, 65538, 736, 22, 40, "Text"],

Cell[CellGroupData[{
Cell[1984972, 65564, 180, 5, 27, "Input"],
Cell[1985155, 65571, 852, 24, 44, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1986044, 65600, 175, 5, 27, "Input"],
Cell[1986222, 65607, 539, 17, 47, "Output"]
}, Open  ]],
Cell[1986776, 65627, 903, 27, 32, "Text"],
Cell[1987682, 65656, 627, 17, 27, "Input"],

Cell[CellGroupData[{
Cell[1988334, 65677, 175, 5, 27, "Input"],
Cell[1988512, 65684, 14113, 356, 176, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2002662, 66045, 395, 11, 27, "Input"],
Cell[2003060, 66058, 702, 20, 28, "Output"]
}, Open  ]],
Cell[2003777, 66081, 83, 2, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2003909, 66089, 66, 1, 56, "Subsection",
  CellTags->"Tensorfunction"],

Cell[CellGroupData[{
Cell[2004000, 66094, 36, 0, 48, "Subsubsection"],
Cell[2004039, 66096, 331, 6, 86, "Text"],
Cell[2004373, 66104, 173, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2004583, 66117, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[2004641, 66121, 133, 3, 27, "Input"],
Cell[2004777, 66126, 296, 9, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2005110, 66140, 64, 1, 27, "Input"],
Cell[2005177, 66143, 243, 5, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2005457, 66153, 161, 4, 27, "Input"],
Cell[2005621, 66159, 241, 7, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2005899, 66171, 64, 1, 27, "Input"],
Cell[2005966, 66174, 235, 5, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2006238, 66184, 197, 5, 27, "Input"],
Cell[2006438, 66191, 235, 7, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2006710, 66203, 197, 5, 27, "Input"],
Cell[2006910, 66210, 235, 7, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2007182, 66222, 64, 1, 27, "Input"],
Cell[2007249, 66225, 188, 4, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2007474, 66234, 66, 1, 27, "Input"],
Cell[2007543, 66237, 94, 2, 28, "Output"]
}, Open  ]],
Cell[2007652, 66242, 106, 2, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2007807, 66250, 44, 1, 56, "Subsection",
  CellTags->"TFi"],

Cell[CellGroupData[{
Cell[2007876, 66255, 36, 0, 48, "Subsubsection"],
Cell[2007915, 66257, 560, 9, 122, "Text"],
Cell[2008478, 66268, 231, 7, 32, "Text"],
Cell[2008712, 66277, 169, 4, 50, "Text"],
Cell[2008884, 66283, 267, 12, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2009188, 66300, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[2009246, 66304, 560, 15, 27, "Input"],
Cell[2009809, 66321, 1096, 33, 31, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2010942, 66359, 296, 8, 27, "Input"],
Cell[2011241, 66369, 1086, 33, 31, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2012388, 66409, 46, 1, 56, "Subsection",
  CellTags->"TIDL"],

Cell[CellGroupData[{
Cell[2012459, 66414, 36, 0, 48, "Subsubsection"],
Cell[2012498, 66416, 68, 0, 32, "Text"],
Cell[2012569, 66418, 145, 7, 32, "Text"]
}, Open  ]],
Cell[2012729, 66428, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[2012787, 66432, 147, 4, 47, "Subsubsection"],
Cell[2012937, 66438, 487, 13, 32, "Text"],

Cell[CellGroupData[{
Cell[2013449, 66455, 180, 5, 27, "Input"],
Cell[2013632, 66462, 592, 19, 47, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2014273, 66487, 152, 4, 47, "Subsubsection"],

Cell[CellGroupData[{
Cell[2014450, 66495, 320, 9, 27, "Input"],
Cell[2014773, 66506, 2917, 81, 48, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2017739, 66593, 147, 4, 47, "Subsubsection"],

Cell[CellGroupData[{
Cell[2017911, 66601, 275, 8, 27, "Input"],
Cell[2018189, 66611, 4253, 117, 50, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2022491, 66734, 152, 4, 47, "Subsubsection"],

Cell[CellGroupData[{
Cell[2022668, 66742, 415, 12, 27, "Input"],
Cell[2023086, 66756, 41424, 949, 356, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2064559, 67711, 162, 4, 47, "Subsubsection"],
Cell[2064724, 67717, 478, 13, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2065239, 67735, 170, 4, 47, "Subsubsection"],
Cell[2065412, 67741, 567, 15, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2066016, 67761, 147, 4, 47, "Subsubsection"],

Cell[CellGroupData[{
Cell[2066188, 67769, 317, 9, 27, "Input"],
Cell[2066508, 67780, 25216, 598, 173, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2091773, 68384, 152, 4, 47, "Subsubsection"],
Cell[2091928, 68390, 401, 11, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2092366, 68406, 162, 4, 47, "Subsubsection"],
Cell[2092531, 68412, 488, 13, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2093056, 68430, 170, 4, 47, "Subsubsection"],
Cell[2093229, 68436, 594, 16, 27, "Input"],

Cell[CellGroupData[{
Cell[2093848, 68456, 217, 5, 27, "Input"],
Cell[2094068, 68463, 131, 4, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2094260, 68474, 44, 1, 56, "Subsection",
  CellTags->"TID"],

Cell[CellGroupData[{
Cell[2094329, 68479, 36, 0, 48, "Subsubsection"],
Cell[2094368, 68481, 62, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2094455, 68485, 65, 1, 27, "Input"],
Cell[2094523, 68488, 680, 13, 64, "Output"]
}, Open  ]],
Cell[2095218, 68504, 295, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2095550, 68522, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[2095608, 68526, 389, 10, 27, "Input"],
Cell[2096000, 68538, 621, 20, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2096658, 68563, 155, 4, 27, "Input"],
Cell[2096816, 68569, 15419, 370, 216, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2112272, 68944, 263, 7, 27, "Input"],
Cell[2112538, 68953, 439, 15, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2113014, 68973, 86, 2, 27, "Input"],
Cell[2113103, 68977, 1412, 44, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2114552, 69026, 163, 4, 27, "Input"],
Cell[2114718, 69032, 478, 16, 48, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2115257, 69055, 66, 1, 56, "Subsection",
  CellTags->"ToDistribution"],

Cell[CellGroupData[{
Cell[2115348, 69060, 36, 0, 48, "Subsubsection"],
Cell[2115387, 69062, 398, 7, 86, "Text"],

Cell[CellGroupData[{
Cell[2115810, 69073, 76, 1, 27, "Input"],
Cell[2115889, 69076, 150, 4, 28, "Output"]
}, Open  ]],
Cell[2116054, 69083, 191, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2116282, 69096, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[2116340, 69100, 178, 4, 27, "Input"],
Cell[2116521, 69106, 158, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2116716, 69115, 263, 8, 27, "Input"],
Cell[2116982, 69125, 1614, 46, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2118633, 69176, 163, 3, 27, "Input"],
Cell[2118799, 69181, 150, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2118986, 69190, 263, 8, 27, "Input"],
Cell[2119252, 69200, 1628, 48, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2120917, 69253, 483, 12, 68, "Input"],
Cell[2121403, 69267, 378, 11, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2121818, 69283, 66, 1, 27, "Input"],
Cell[2121887, 69286, 323, 8, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2122247, 69299, 433, 12, 27, "Input"],
Cell[2122683, 69313, 378, 11, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2123122, 69331, 70, 1, 56, "Subsection",
  CellTags->"ToHypergeometric"],

Cell[CellGroupData[{
Cell[2123217, 69336, 36, 0, 48, "Subsubsection"],
Cell[2123256, 69338, 1406, 41, 68, "Text"],
Cell[2124665, 69381, 477, 18, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2125179, 69404, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[2125237, 69408, 410, 12, 31, "Input"],
Cell[2125650, 69422, 1121, 30, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2126808, 69457, 521, 14, 31, "Input"],
Cell[2127332, 69473, 897, 24, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2128266, 69502, 410, 12, 31, "Input"],
Cell[2128679, 69516, 1202, 32, 50, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2129918, 69553, 521, 14, 31, "Input"],
Cell[2130442, 69569, 1004, 27, 50, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2131507, 69603, 52, 1, 56, "Subsection",
  CellTags->"ToLarin"],

Cell[CellGroupData[{
Cell[2131584, 69608, 36, 0, 48, "Subsubsection"],
Cell[2131623, 69610, 670, 19, 32, "Text"],
Cell[2132296, 69631, 152, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2132485, 69644, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[2132543, 69648, 196, 4, 27, "Input"],
Cell[2132742, 69654, 427, 13, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2133206, 69672, 63, 1, 27, "Input"],
Cell[2133272, 69675, 1425, 43, 43, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2134758, 69725, 48, 1, 56, "Subsection",
  CellTags->"ToTFi"],

Cell[CellGroupData[{
Cell[2134831, 69730, 36, 0, 48, "Subsubsection"],
Cell[2134870, 69732, 254, 5, 68, "Text"],
Cell[2135127, 69739, 261, 12, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2135425, 69756, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[2135483, 69760, 333, 9, 27, "Input"],
Cell[2135819, 69771, 3046, 76, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2138902, 69852, 110, 2, 27, "Input"],
Cell[2139015, 69856, 1084, 33, 31, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2140136, 69894, 64, 1, 27, "Input"],
Cell[2140203, 69897, 594, 16, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2140834, 69918, 503, 13, 27, "Input"],
Cell[2141340, 69933, 1922, 57, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2143299, 69995, 61, 1, 27, "Input"],
Cell[2143363, 69998, 1127, 34, 31, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2144527, 70037, 64, 1, 27, "Input"],
Cell[2144594, 70040, 710, 19, 56, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2145365, 70066, 42, 1, 56, "Subsection",
  CellTags->"Tr"],

Cell[CellGroupData[{
Cell[2145432, 70071, 36, 0, 48, "Subsubsection"],
Cell[2145471, 70073, 246, 5, 68, "Text"],

Cell[CellGroupData[{
Cell[2145742, 70082, 64, 1, 27, "Input"],
Cell[2145809, 70085, 692, 15, 64, "Output"]
}, Open  ]],
Cell[2146516, 70103, 556, 22, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2147109, 70130, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[2147167, 70134, 93, 2, 27, "Input"],
Cell[2147263, 70138, 168, 4, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2147468, 70147, 58, 1, 27, "Input"],
Cell[2147529, 70150, 167, 5, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2147733, 70160, 385, 10, 27, "Input"],
Cell[2148121, 70172, 2268, 65, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2150426, 70242, 188, 5, 27, "Input"],
Cell[2150617, 70249, 725, 21, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2151379, 70275, 220, 5, 27, "Input"],
Cell[2151602, 70282, 17124, 440, 65, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2168763, 70727, 31, 0, 27, "Input"],
Cell[2168797, 70729, 62, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2168896, 70735, 51, 1, 27, "Input"],
Cell[2168950, 70738, 334, 5, 78, "Print"]
}, Open  ]],
Cell[2169299, 70746, 149, 3, 50, "Text"],

Cell[CellGroupData[{
Cell[2169473, 70753, 60, 1, 27, "Input"],
Cell[2169536, 70756, 63, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2169636, 70762, 443, 11, 27, "Input"],
Cell[2170082, 70775, 8439, 216, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2178558, 70996, 140, 3, 27, "Input"],
Cell[2178701, 71001, 1560, 44, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2180298, 71050, 472, 12, 27, "Input"],
Cell[2180773, 71064, 420, 12, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2181230, 71081, 204, 5, 27, "Input"],
Cell[2181437, 71088, 176, 5, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2181650, 71098, 64, 1, 27, "Input"],
Cell[2181717, 71101, 128, 3, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2181882, 71109, 345, 8, 27, "Input"],
Cell[2182230, 71119, 186, 5, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2182453, 71129, 64, 1, 27, "Input"],
Cell[2182520, 71132, 138, 3, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2182719, 71142, 68, 1, 56, "Subsection",
  CellTags->"TrickMandelstam"],

Cell[CellGroupData[{
Cell[2182812, 71147, 36, 0, 48, "Subsubsection"],
Cell[2182851, 71149, 1071, 36, 68, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2183959, 71190, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[2184017, 71194, 583, 16, 31, "Input"],
Cell[2184603, 71212, 225, 7, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2184865, 71224, 640, 16, 43, "Input"],
Cell[2185508, 71242, 223, 7, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2185792, 71256, 48, 1, 56, "Subsection",
  CellTags->"Trick"],

Cell[CellGroupData[{
Cell[2185865, 71261, 36, 0, 48, "Subsubsection"],
Cell[2185904, 71263, 157, 4, 50, "Text"],
Cell[2186064, 71269, 652, 28, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2186753, 71302, 42, 0, 47, "Subsubsection"],
Cell[2186798, 71304, 399, 14, 32, "Text"],

Cell[CellGroupData[{
Cell[2187222, 71322, 357, 9, 27, "Input"],
Cell[2187582, 71333, 275, 9, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2187894, 71347, 326, 9, 27, "Input"],
Cell[2188223, 71358, 565, 17, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2188825, 71380, 61, 1, 27, "Input"],
Cell[2188889, 71383, 727, 21, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2189653, 71409, 198, 4, 27, "Input"],
Cell[2189854, 71415, 441, 13, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2190332, 71433, 94, 2, 27, "Input"],
Cell[2190429, 71437, 170, 4, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2190660, 71448, 76, 1, 56, "Subsection",
  CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{
Cell[2190761, 71453, 36, 0, 48, "Subsubsection"],
Cell[2190800, 71455, 930, 14, 194, "Text"],
Cell[2191733, 71471, 200, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2191970, 71484, 65, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[2192060, 71488, 299, 6, 43, "Input"],
Cell[2192362, 71496, 437, 10, 46, "Output"]
}, Open  ]],
Cell[2192814, 71509, 28, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2192867, 71513, 240, 6, 27, "Input"],
Cell[2193110, 71521, 1124, 33, 31, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2194271, 71559, 290, 7, 27, "Input"],
Cell[2194564, 71568, 1506, 44, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2196107, 71617, 178, 4, 27, "Input"],
Cell[2196288, 71623, 341, 10, 31, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2196666, 71638, 228, 5, 27, "Input"],
Cell[2196897, 71645, 939, 27, 29, "Output"]
}, Open  ]],
Cell[2197851, 71675, 28, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2197904, 71679, 335, 8, 27, "Input"],
Cell[2198242, 71689, 1556, 46, 31, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2199835, 71740, 305, 8, 27, "Input"],
Cell[2200143, 71750, 540, 16, 31, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2200720, 71771, 355, 9, 27, "Input"],
Cell[2201078, 71782, 16785, 418, 188, "Output"]
}, Open  ]],
Cell[2217878, 72203, 28, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2217931, 72207, 422, 10, 27, "Input"],
Cell[2218356, 72219, 7175, 183, 51, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2225568, 72407, 382, 10, 27, "Input"],
Cell[2225953, 72419, 684, 20, 31, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2226674, 72444, 432, 11, 27, "Input"],
Cell[2227109, 72457, 36926, 899, 437, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2264084, 73362, 66, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[2264175, 73366, 228, 5, 43, "Input"],
Cell[2264406, 73373, 437, 10, 46, "Output"]
}, Open  ]],
Cell[2264858, 73386, 28, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2264911, 73390, 240, 6, 27, "Input"],
Cell[2265154, 73398, 1139, 34, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2266330, 73437, 290, 7, 27, "Input"],
Cell[2266623, 73446, 4098, 111, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2270758, 73562, 178, 4, 27, "Input"],
Cell[2270939, 73568, 341, 10, 31, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2271317, 73583, 228, 5, 27, "Input"],
Cell[2271548, 73590, 3516, 93, 29, "Output"]
}, Open  ]],
Cell[2275079, 73686, 28, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2275132, 73690, 335, 8, 27, "Input"],
Cell[2275470, 73700, 1641, 48, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2277148, 73753, 305, 8, 27, "Input"],
Cell[2277456, 73763, 540, 16, 31, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2278033, 73784, 355, 9, 27, "Input"],
Cell[2278391, 73795, 27235, 680, 206, "Output"]
}, Open  ]],
Cell[2305641, 74478, 28, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2305694, 74482, 422, 10, 27, "Input"],
Cell[2306119, 74494, 7190, 184, 66, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2313346, 74683, 382, 10, 27, "Input"],
Cell[2313731, 74695, 684, 20, 31, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2314452, 74720, 432, 11, 27, "Input"],
Cell[2314887, 74733, 52438, 1323, 563, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2367374, 76062, 42, 0, 48, "Subsubsection"],
Cell[2367419, 76064, 63, 0, 32, "Text"],
Cell[2367485, 76066, 249, 5, 43, "Input"],
Cell[2367737, 76073, 374, 10, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2368148, 76088, 68, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[2368241, 76092, 231, 4, 43, "Input"],
Cell[2368475, 76098, 436, 10, 46, "Output"]
}, Open  ]],
Cell[2368926, 76111, 28, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2368979, 76115, 300, 8, 27, "Input"],
Cell[2369282, 76125, 5447, 130, 43, "Output"]
}, Open  ]],
Cell[2374744, 76258, 28, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2374797, 76262, 335, 8, 27, "Input"],
Cell[2375135, 76272, 91884, 1865, 1271, "Output"]
}, Open  ]],
Cell[2467034, 78140, 28, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2467087, 78144, 478, 11, 43, "Input"],
Cell[2467568, 78157, 7175, 183, 51, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2474792, 78346, 70, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[2474887, 78350, 231, 4, 43, "Input"],
Cell[2475121, 78356, 436, 10, 46, "Output"]
}, Open  ]],
Cell[2475572, 78369, 28, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2475625, 78373, 300, 8, 27, "Input"],
Cell[2475928, 78383, 4742, 127, 63, "Output"]
}, Open  ]],
Cell[2480685, 78513, 28, 0, 32, "Text"],
Cell[2480716, 78515, 374, 10, 27, "Input"],

Cell[CellGroupData[{
Cell[2481115, 78529, 305, 8, 27, "Input"],
Cell[2481423, 78539, 4743, 127, 64, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2486203, 78671, 355, 9, 27, "Input"],
Cell[2486561, 78682, 4743, 127, 64, "Output"]
}, Open  ]],
Cell[2491319, 78812, 28, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2491372, 78816, 478, 11, 43, "Input"],
Cell[2491853, 78829, 7190, 184, 66, "Output"]
}, Open  ]],
Cell[2499058, 79016, 44, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2499127, 79020, 540, 14, 59, "Input"],
Cell[2499670, 79036, 11716, 283, 114, "Output"]
}, Open  ]],
Cell[2511401, 79322, 62, 0, 32, "Text"],
Cell[2511466, 79324, 249, 5, 43, "Input"],

Cell[CellGroupData[{
Cell[2511740, 79333, 437, 11, 27, "Input"],
Cell[2512180, 79346, 8694, 197, 124, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2520935, 79550, 76, 1, 56, "Subsection",
  CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{
Cell[2521036, 79555, 36, 0, 48, "Subsubsection"],
Cell[2521075, 79557, 946, 14, 194, "Text"],
Cell[2522024, 79573, 200, 8, 32, "Text"]
}, Open  ]],
Cell[2522239, 79584, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[2522297, 79588, 64, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[2522386, 79592, 228, 5, 43, "Input"],
Cell[2522617, 79599, 436, 10, 46, "Output"]
}, Open  ]],
Cell[2523068, 79612, 41, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2523134, 79616, 103, 2, 27, "Input"],
Cell[2523240, 79620, 772, 21, 29, "Output"]
}, Open  ]],
Cell[2524027, 79644, 47, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2524099, 79648, 282, 7, 27, "Input"],
Cell[2524384, 79657, 2112, 56, 53, "Output"]
}, Open  ]],
Cell[2526511, 79716, 53, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2526589, 79720, 370, 9, 27, "Input"],
Cell[2526962, 79731, 7392, 166, 135, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2534403, 79903, 66, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[2534494, 79907, 220, 4, 43, "Input"],
Cell[2534717, 79913, 436, 10, 46, "Output"]
}, Open  ]],
Cell[2535168, 79926, 41, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2535234, 79930, 103, 2, 27, "Input"],
Cell[2535340, 79934, 596, 16, 29, "Output"]
}, Open  ]],
Cell[2535951, 79953, 47, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2536023, 79957, 282, 7, 27, "Input"],
Cell[2536308, 79966, 2037, 54, 53, "Output"]
}, Open  ]],
Cell[2538360, 80023, 53, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2538438, 80027, 370, 9, 27, "Input"],
Cell[2538811, 80038, 7267, 163, 132, "Output"]
}, Open  ]],
Cell[2546093, 80204, 59, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2546177, 80208, 153, 4, 27, "Input"],
Cell[2546333, 80214, 216, 6, 42, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2546586, 80225, 354, 8, 43, "Input"],
Cell[2546943, 80235, 2866, 77, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2549846, 80317, 438, 10, 43, "Input"],
Cell[2550287, 80329, 15722, 329, 213, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2566058, 80664, 68, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[2566151, 80668, 229, 5, 43, "Input"],
Cell[2566383, 80675, 437, 10, 46, "Output"]
}, Open  ]],
Cell[2566835, 80688, 147, 2, 50, "Text"],
Cell[2566985, 80692, 243, 5, 68, "Text"],

Cell[CellGroupData[{
Cell[2567253, 80701, 163, 4, 27, "Input"],
Cell[2567419, 80707, 1077, 29, 29, "Output"]
}, Open  ]],
Cell[2568511, 80739, 66, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2568602, 80743, 282, 7, 27, "Input"],
Cell[2568887, 80752, 3433, 82, 53, "Output"]
}, Open  ]],
Cell[2572335, 80837, 53, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2572413, 80841, 103, 2, 27, "Input"],
Cell[2572519, 80845, 1252, 32, 76, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2573820, 80883, 70, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[2573915, 80887, 229, 5, 43, "Input"],
Cell[2574147, 80894, 437, 10, 46, "Output"]
}, Open  ]],
Cell[2574599, 80907, 41, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2574665, 80911, 163, 4, 27, "Input"],
Cell[2574831, 80917, 1077, 29, 29, "Output"]
}, Open  ]],
Cell[2575923, 80949, 47, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2575995, 80953, 282, 7, 27, "Input"],
Cell[2576280, 80962, 3433, 82, 53, "Output"]
}, Open  ]],
Cell[2579728, 81047, 174, 5, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2579951, 81058, 58, 1, 56, "Subsection",
  CellTags->"Uncontract"],

Cell[CellGroupData[{
Cell[2580034, 81063, 36, 0, 48, "Subsubsection"],
Cell[2580073, 81065, 221, 4, 50, "Text"],

Cell[CellGroupData[{
Cell[2580319, 81073, 72, 1, 27, "Input"],
Cell[2580394, 81076, 356, 9, 28, "Output"]
}, Open  ]],
Cell[2580765, 81088, 167, 8, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2580969, 81101, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[2581027, 81105, 206, 5, 27, "Input"],
Cell[2581236, 81112, 494, 15, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2581767, 81132, 99, 2, 27, "Input"],
Cell[2581869, 81136, 891, 27, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2582797, 81168, 94, 2, 27, "Input"],
Cell[2582894, 81172, 234, 6, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2583165, 81183, 99, 2, 27, "Input"],
Cell[2583267, 81187, 510, 16, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2583814, 81208, 109, 2, 27, "Input"],
Cell[2583926, 81212, 1243, 38, 30, "Output"]
}, Open  ]],
Cell[2585184, 81253, 64, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2585273, 81257, 175, 4, 27, "Input"],
Cell[2585451, 81263, 219, 6, 28, "Output"]
}, Open  ]],
Cell[2585685, 81272, 130, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[2585840, 81278, 220, 5, 27, "Input"],
Cell[2586063, 81285, 554, 18, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2586654, 81308, 253, 6, 27, "Input"],
Cell[2586910, 81316, 1042, 34, 30, "Output"]
}, Open  ]],
Cell[2587967, 81353, 90, 2, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2588106, 81361, 84, 1, 56, "Subsection",
  CellTags->"UnDeclareNonCommutative"],

Cell[CellGroupData[{
Cell[2588215, 81366, 36, 0, 48, "Subsubsection"],
Cell[2588254, 81368, 162, 4, 50, "Text"],
Cell[2588419, 81374, 325, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2588781, 81392, 33, 0, 48, "Subsubsection"],
Cell[2588817, 81394, 77, 1, 27, "Input"],
Cell[2588897, 81397, 115, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[2589037, 81404, 43, 0, 27, "Input"],
Cell[2589083, 81406, 62, 1, 28, "Output"]
}, Open  ]],
Cell[2589160, 81410, 65, 0, 32, "Text"],
Cell[2589228, 81412, 79, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[2589332, 81417, 104, 2, 27, "Input"],
Cell[2589439, 81421, 63, 1, 28, "Output"]
}, Open  ]],
Cell[2589517, 81425, 104, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[2589646, 81431, 114, 2, 27, "Input"],
Cell[2589763, 81435, 121, 3, 28, "Output"]
}, Open  ]],
Cell[2589899, 81441, 106, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[2590030, 81447, 114, 2, 27, "Input"],
Cell[2590147, 81451, 123, 3, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2590331, 81461, 50, 1, 56, "Subsection",
  CellTags->"Write2"],

Cell[CellGroupData[{
Cell[2590406, 81466, 36, 0, 48, "Subsubsection"],
Cell[2590445, 81468, 276, 5, 68, "Text"],
Cell[2590724, 81475, 132, 3, 50, "Text"],

Cell[CellGroupData[{
Cell[2590881, 81482, 92, 1, 27, "Input"],
Cell[2590976, 81485, 197, 6, 41, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2591210, 81496, 68, 1, 27, "Input"],
Cell[2591281, 81499, 587, 12, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2591905, 81516, 71, 1, 27, "Input"],
Cell[2591979, 81519, 137, 4, 28, "Output"]
}, Open  ]],
Cell[2592131, 81526, 289, 13, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2592457, 81544, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[2592515, 81548, 481, 13, 32, "Input"],
Cell[2592999, 81563, 982, 28, 30, "Output"]
}, Open  ]],
Cell[2593996, 81594, 187, 5, 27, "Input"],
Cell[2594186, 81601, 57, 0, 32, "Text"],
Cell[2594246, 81603, 155, 4, 27, "Input"],
Cell[2594404, 81609, 52, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2594481, 81613, 376, 9, 59, "Input"],
Cell[2594860, 81624, 601, 16, 75, "Output"]
}, Open  ]],
Cell[2595476, 81643, 278, 7, 27, "Input"],

Cell[CellGroupData[{
Cell[2595779, 81654, 211, 5, 27, "Input"],
Cell[2595993, 81661, 840, 26, 29, "Output"]
}, Open  ]],
Cell[2596848, 81690, 156, 4, 27, "Input"],

Cell[CellGroupData[{
Cell[2597029, 81698, 376, 9, 59, "Input"],
Cell[2597408, 81709, 1035, 25, 201, "Output"]
}, Open  ]],
Cell[2598458, 81737, 278, 7, 27, "Input"],
Cell[2598739, 81746, 88, 3, 32, "Text"],
Cell[2598830, 81751, 220, 6, 27, "Input"],

Cell[CellGroupData[{
Cell[2599075, 81761, 376, 9, 59, "Input"],
Cell[2599454, 81772, 840, 20, 129, "Output"]
}, Open  ]],
Cell[2600309, 81795, 409, 10, 59, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2600767, 81811, 48, 1, 56, "Subsection",
  CellTags->"Zeta2"],

Cell[CellGroupData[{
Cell[2600840, 81816, 36, 0, 48, "Subsubsection"],
Cell[2600879, 81818, 74, 0, 32, "Text"],
Cell[2600956, 81820, 181, 7, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2601174, 81832, 33, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[2601232, 81836, 31, 0, 27, "Input"],
Cell[2601266, 81838, 97, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2601400, 81845, 61, 1, 27, "Input"],
Cell[2601464, 81848, 78, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2601579, 81854, 99, 2, 27, "Input"],
Cell[2601681, 81858, 126, 3, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2601868, 81868, 46, 1, 56, "Subsection",
  CellTags->"$FCT"],

Cell[CellGroupData[{
Cell[2601939, 81873, 36, 0, 48, "Subsubsection"],
Cell[2601978, 81875, 161, 4, 50, "Text"],
Cell[2602142, 81881, 90, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[2602257, 81887, 136, 3, 27, "Input"],
Cell[2602396, 81892, 168, 4, 28, "Output"]
}, Open  ]],
Cell[2602579, 81899, 91, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[2602695, 81905, 136, 3, 27, "Input"],
Cell[2602834, 81910, 168, 4, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2603063, 81921, 68, 1, 56, "Subsection",
  CellTags->"$LorentzIndices"],

Cell[CellGroupData[{
Cell[2603156, 81926, 36, 0, 48, "Subsubsection"],
Cell[2603195, 81928, 134, 3, 50, "Text"],
Cell[2603332, 81933, 91, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[2603448, 81939, 310, 7, 27, "Input"],
Cell[2603761, 81948, 844, 26, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2604642, 81979, 64, 1, 27, "Input"],
Cell[2604709, 81982, 447, 11, 56, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2605193, 81998, 280, 7, 27, "Input"],
Cell[2605476, 82007, 1139, 36, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2606652, 82048, 64, 1, 27, "Input"],
Cell[2606719, 82051, 680, 17, 72, "Output"]
}, Open  ]],
Cell[2607414, 82071, 92, 2, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2607555, 82079, 64, 1, 56, "Subsection",
  CellTags->"$PairBrackets"],

Cell[CellGroupData[{
Cell[2607644, 82084, 36, 0, 48, "Subsubsection"],
Cell[2607683, 82086, 127, 3, 50, "Text"],

Cell[CellGroupData[{
Cell[2607835, 82093, 39, 0, 27, "Input"],
Cell[2607877, 82095, 63, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2607977, 82101, 171, 5, 27, "Input"],
Cell[2608151, 82108, 566, 18, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2608754, 82131, 67, 1, 27, "Input"],
Cell[2608824, 82134, 62, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2608923, 82140, 171, 5, 27, "Input"],
Cell[2609097, 82147, 642, 18, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2609776, 82170, 68, 1, 27, "Input"],
Cell[2609847, 82173, 63, 1, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2609971, 82181, 62, 1, 56, "Subsection",
  CellTags->"$VeryVerbose"],

Cell[CellGroupData[{
Cell[2610058, 82186, 36, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[2610119, 82190, 58, 1, 27, "Input"],
Cell[2610180, 82193, 241, 4, 42, "Print"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2610458, 82202, 38, 0, 27, "Input"],
Cell[2610499, 82204, 59, 1, 28, "Output"]
}, Open  ]],
Cell[2610573, 82208, 95, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[2610693, 82214, 248, 7, 27, "Input"],
Cell[2610944, 82223, 96, 3, 38, "Print"],
Cell[2611043, 82228, 266, 7, 24, "Print"],
Cell[2611312, 82237, 284, 7, 24, "Print"],
Cell[2611599, 82246, 1053, 30, 29, "Output"]
}, Open  ]],
Cell[2612667, 82279, 85, 2, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2612801, 82287, 48, 1, 56, "Subsection",
  CellTags->"$West"],

Cell[CellGroupData[{
Cell[2612874, 82292, 36, 0, 48, "Subsubsection"],

Cell[CellGroupData[{
Cell[2612935, 82296, 51, 1, 27, "Input"],
Cell[2612989, 82299, 334, 5, 78, "Print"]
}, Open  ]],
Cell[2613338, 82307, 148, 7, 32, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2613535, 82320, 102, 1, 56, "Subsection",
  CellTags->"Evaluation time and memory usage"],

Cell[CellGroupData[{
Cell[2613662, 82325, 106, 2, 27, "Input"],
Cell[2613771, 82329, 68, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2613876, 82335, 62, 1, 27, "Input"],
Cell[2613941, 82338, 66, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2614044, 82344, 64, 1, 27, "Input"],
Cell[2614111, 82347, 66, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2614214, 82353, 59, 1, 27, "Input"],
Cell[2614276, 82356, 78, 1, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

