(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    605944,      16530]*)
(*NotebookOutlinePosition[    629682,      17278]*)
(*  CellTagsIndexPosition[    623906,      17093]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["TARCER 1.95", "Title",
  TextAlignment->Center,
  CellTags->"T1.1"],

Cell[TextData[{
  "A ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " program for the reduction of two-loop propagator integrals "
}], "Text",
  TextAlignment->Center],

Cell[TextData[{
  "Rolf Mertig\n",
  ButtonBox["Mertig Research & Consulting\n",
    ButtonData:>{
      URL[ "http://www.mertig.com"], None},
    ButtonStyle->"Hyperlink"],
  "and\nRainer Scharf\n",
  ButtonBox["Universit\[ADoubleDot]t Leipzig",
    ButtonData:>{
      URL[ "http://www.physik.uni-leipzig.de/~scharf/"], None},
    ButtonStyle->"Hyperlink"]
}], "Text",
  Editable->False,
  TextAlignment->Center,
  CellTags->"T1.2"],

Cell["April 23, 1999", "Text",
  TextAlignment->Center],

Cell[TextData[{
  "COMMENTS (RM 25.4.): ",
  ButtonBox["TFISimplify",
    ButtonData:>"T1.7.1.1",
    ButtonStyle->"Hyperlink"],
  " (like TFIRecurse, but without trule[9])."
}], "Text"],

Cell[TextData[{
  "COMMENTS (RM 23.4.): bug fixed in ",
  ButtonBox["tlrule[17]",
    ButtonData:>"tlrule17",
    ButtonStyle->"Hyperlink"],
  "."
}], "Text"],

Cell[TextData[{
  "COMMENTS (RM 6.4.):  ",
  ButtonBox["special TVR relations",
    ButtonData:>"specialTVR",
    ButtonStyle->"Hyperlink"],
  " added to TVR."
}], "Text"],

Cell[TextData[{
  "COMMENTS (RM 22.3.):  ",
  ButtonBox["TVI Symmetry",
    ButtonData:>"Tsymmetry",
    ButtonStyle->"Hyperlink"],
  " added to TVR."
}], "Text"],

Cell[TextData[{
  "COMMENTS (RM 18.3.): More rules added to ",
  ButtonBox["$TTable",
    ButtonData:>"$TTable",
    ButtonStyle->"Hyperlink"],
  ". "
}], "Text"],

Cell[TextData[{
  "COMMENTS (RM 18.2.): More rules added to ",
  ButtonBox["$TTable",
    ButtonData:>"$TTable",
    ButtonStyle->"Hyperlink"],
  ". "
}], "Text"],

Cell[TextData[{
  "COMMENTS (RM 21.2.): Included ",
  ButtonBox["$TTable",
    ButtonData:>"$TTable",
    ButtonStyle->"Hyperlink"],
  ". added tlrule[25]"
}], "Text"],

Cell["\<\
COMMENTS (RM 20.2.): $RankLimit = {0,6} ;  Changed TarcerRecurse \
such that special TJR and TVR (reducing D -> D-2) are used first.\
\>", "Text"],

Cell["\<\
COMMENTS (RM 15.2.): $RankLimit = {0,8} ;  
                                              The following test now works:
                                              
                                                 Print[\"test 1 :  TFI[D, pp, \
{0, 0, 0, 1, 3}, {{1, MW}, {1, MZ}, {0, 0}, {1, MZ}, {1,MW}}] = \",
                                                            Timing[t1 \
=TarcerRecurse[TFI[D, pp, {0, 0, 0, 1, 3}, {{1, MW}, {1, MZ}, {0, 0}, {1, \
MZ}, {1,MW}}]]
                                                                       ] // \
InputForm
                                                         ]

                                                  Print[\"test 2 :  TFI[d, \
pp, 0, {0, 0, 0, 0, 4},{{1, m1}, {1, m2}, 0, 0, 0}] = \",
                                                           Timing[t2 \
=TarcerRecurse[TFI[D, pp,  0, {0, 0, 0, 0, 4},{{1, m1}, {1, m2}, 0, 0, 0}]];
                                                                       ] // \
InputForm
                                                          ]
\
\>", "Text"],

Cell["\<\
COMMENTS (RM 13.2.): tlrule[11] and trule[12] made more \
restrictive: inhibiting thus a recursion if {n5,m5} equals {0,0}.\
\>", "Text"],

Cell["\<\
COMMENTS (RM 12.2.): tlrule[15] made safer (in order to not get   0 \
:> 0 );
                                             TFIRecurse made safer (to not \
get into a recursion);
                                                  There is still something \
wrong with the tlrule's ; since 
                                         TFIRecurse[TFI[D, pp, {0, 0, 0, 1, \
3}, {{1, MW}, {1, MZ}, {0, 0}, {1, MZ}, {1,   MW}}]]
                                         doesn't work ..\
\>", "Text"],

Cell["\<\
COMMENTS (RM 5.2.): tlrule[10] slightly changed to cover more \
cases.\
\>", "Text"],

Cell["\<\
COMMENTS (RM 2.2.):  FunctionExpand added in tlrule[9]. Option \
TimeConstraint added to TarcerRecurse (because TimeConstrained is buggy under \
Linux).\
\>", "Text"],

Cell[TextData[{
  "COMMENTS (RM 1.2.):  Changed TarcerRecurse. New TLR ",
  ButtonBox["definition",
    ButtonData:>"TLRdef",
    ButtonStyle->"Hyperlink"],
  ". \n                          default $RankLimit={0,6};"
}], "Text"],

Cell[BoxData[
    \(\($Remember = False; \)\)], "Input"],

Cell["\<\
COMMENTS (RM 19.1.):   changes are:  tlrule[22] and tlrule[23], \
minor efficiency changes (Expand3 = Identity) and other small programming \
optimizations.\
\>", "Text"],

Cell["\<\
COMMENTS (RM 22.1.):   changes are: bug in tlrule[23] fixed, new \
tlrule[24], new option Factor to TarcerReduce (doesn't change anything by \
default, but allows for improved Factor-ing).\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  ButtonBox["hep-ph/9801383",
    ButtonData:>{
      URL[ "http://xxx.lanl.gov/abs/hep-ph/9801383"], None},
    ButtonStyle->"Hyperlink"],
  ", published in Comput. Phys. Commun. 111 (1998) 265-273"
}], "Text",
  Editable->False],

Cell[CellGroupData[{

Cell["Table of contents", "Section",
  CellDingbat->"\[FilledSmallCircle]"],

Cell[TextData[ButtonBox["Preliminaries",
  ButtonData:>"T1.1.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Usages",
  ButtonData:>"T1.2.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Creating the tarcerij.mx file",
  ButtonData:>"T1.2.1.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Prologue (setting of $RankLimit)",
  ButtonData:>"T1.3.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Typesetting rules",
  ButtonData:>"T1.3.1.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["The Palettes",
  ButtonData:>"T1.3.2.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["OperatorApply",
  ButtonData:>"T1.3.3.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MassDerivative",
  ButtonData:>"T1.3.4.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["TarasovT",
  ButtonData:>"T1.3.5.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["ApplyTarasovT",
  ButtonData:>"T1.3.6.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Misc",
  ButtonData:>"T1.3.7.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Colors",
  ButtonData:>"T1.3.7.1.1",
  ButtonStyle->"Hyperlink"]], "SmallText"],

Cell[TextData[ButtonBox["PQ",
  ButtonData:>"T1.3.7.2.1",
  ButtonStyle->"Hyperlink"]], "SmallText"],

Cell[TextData[ButtonBox["PNQ",
  ButtonData:>"T1.3.7.3.1",
  ButtonStyle->"Hyperlink"]], "SmallText"],

Cell[TextData[ButtonBox["FactorC",
  ButtonData:>"T1.3.7.4.1",
  ButtonStyle->"Hyperlink"]], "SmallText"],

Cell[TextData[ButtonBox["TFI",
  ButtonData:>"T1.3.7.5.1",
  ButtonStyle->"Hyperlink"]], "SmallText"],

Cell[TextData[ButtonBox["TJI",
  ButtonData:>"T1.3.7.6.1",
  ButtonStyle->"Hyperlink"]], "SmallText"],

Cell[TextData[ButtonBox["TBI",
  ButtonData:>"T1.3.7.7.1",
  ButtonStyle->"Hyperlink"]], "SmallText"],

Cell[TextData[ButtonBox["TAI",
  ButtonData:>"T1.3.7.8.1",
  ButtonStyle->"Hyperlink"]], "SmallText"],

Cell[TextData[ButtonBox["Definitions and notation",
  ButtonData:>"T1.4.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[StyleBox[ButtonBox["ToArgs",
  ButtonData:>"T1.4.1.1",
  ButtonStyle->"Hyperlink"],
  FontColor->RGBColor[0.2, 0.6, 1]]], "Text"],

Cell[TextData[StyleBox[ButtonBox["Cayley",
  ButtonData:>"T1.4.2.1",
  ButtonStyle->"Hyperlink"],
  FontColor->RGBColor[1, 0, 0.4]]], "Text"],

Cell[TextData[Cell[BoxData[
    FormBox[
      ButtonBox[
        StyleBox[
          RowBox[{
            RowBox[{
              StyleBox["Cayley",
                FontColor->RGBColor[1, 0, 0.4]], "[", "i", "]"}], 
            "=", \(\[CapitalDelta]\_i\)}],
          SingleLetterItalics->False],
        ButtonFunction:>(FrontEndExecute[ {
            FrontEnd`NotebookLocate[ #2]}]&),
        ButtonData:>"T1.4.3.1",
        Active->True,
        Background->GrayLevel[1],
        ButtonNote->ButtonData], TextForm]]]], "Text"],

Cell[TextData[Cell[BoxData[
    FormBox[
      ButtonBox[
        StyleBox[
          RowBox[{
            RowBox[{
              StyleBox["CayleyD",
                FontColor->RGBColor[1, 0.2, 1]], "[", "i", "]"}], 
            "=", \(\[CapitalDelta]\_i/\[CapitalDelta]\)}],
          SingleLetterItalics->False],
        ButtonFunction:>(FrontEndExecute[ {
            FrontEnd`NotebookLocate[ #2]}]&),
        ButtonData:>"T1.4.4.1",
        Active->True,
        Background->GrayLevel[1],
        ButtonNote->ButtonData], TextForm]]]], "Text"],

Cell[TextData[ButtonBox["Utility Functions",
  ButtonData:>"T1.5.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun and friends",
  ButtonData:>"T1.5.1.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Interchange",
  ButtonData:>"T1.5.1.1.1",
  ButtonStyle->"Hyperlink"]], "SmallText"],

Cell[TextData[{
  ButtonBox["\[Nu]",
    ButtonData:>"T1.5.2.1",
    ButtonStyle->"Hyperlink"],
  ButtonBox["explicit",
    ButtonData:>"T1.5.2.1",
    ButtonStyle->"Hyperlink"]
}], "Text"],

Cell[TextData[ButtonBox["TA",
  ButtonData:>"T1.5.2.1.1",
  ButtonStyle->"Hyperlink"]], "SmallText"],

Cell[TextData[ButtonBox["CheckRecursion",
  ButtonData:>"T1.5.3.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["CheckTVIRecursion",
  ButtonData:>"T1.5.3.1.1",
  ButtonStyle->"Hyperlink"]], "SmallText"],

Cell[TextData[ButtonBox["CheckTJIRecursion",
  ButtonData:>"T1.5.3.2.1",
  ButtonStyle->"Hyperlink"]], "SmallText"],

Cell[TextData[ButtonBox["Recurrence Relations Input and Recursion Function \
Generation",
  ButtonData:>"T1.6.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (35)",
  ButtonData:>"T1.6.1.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (35)",
  ButtonData:>"T1.6.2.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[Cell[BoxData[
    FormBox[
      ButtonBox[
        StyleBox[
          RowBox[{
            FormBox[\(1\^\(\(+\)\(\ \)\)\),
              "TraditionalForm"], \(\(\ \)(\(35  _\) 1)\)}],
          SingleLetterItalics->False],
        ButtonFunction:>(FrontEndExecute[ {
            FrontEnd`NotebookLocate[ #2]}]&),
        ButtonData:>"T1.6.2.1.1",
        Active->True,
        Background->GrayLevel[1],
        ButtonNote->ButtonData], TraditionalForm]]]], "SmallText"],

Cell[TextData[Cell[BoxData[
    FormBox[
      ButtonBox[
        StyleBox[
          RowBox[{
            FormBox[\(2\^\(\(+\)\(\ \)\)\),
              "TraditionalForm"], \(\(\ \)(\(35  _\) 2)\)}],
          SingleLetterItalics->False],
        ButtonFunction:>(FrontEndExecute[ {
            FrontEnd`NotebookLocate[ #2]}]&),
        ButtonData:>"T1.6.2.2.1",
        Active->True,
        Background->GrayLevel[1],
        ButtonNote->ButtonData], TraditionalForm]]]], "SmallText"],

Cell[TextData[Cell[BoxData[
    FormBox[
      ButtonBox[
        StyleBox[
          RowBox[{
            FormBox[\(3\^\(\(+\)\(\ \)\)\),
              "TraditionalForm"], \(\(\ \)(\(35  _\) 3)\)}],
          SingleLetterItalics->False],
        ButtonFunction:>(FrontEndExecute[ {
            FrontEnd`NotebookLocate[ #2]}]&),
        ButtonData:>"T1.6.2.3.1",
        Active->True,
        Background->GrayLevel[1],
        ButtonNote->ButtonData], TraditionalForm]]]], "SmallText"],

Cell[TextData[Cell[BoxData[
    FormBox[
      ButtonBox[
        StyleBox[
          RowBox[{
            FormBox[\(4\^\(\(+\)\(\ \)\)\),
              "TraditionalForm"], \(\(\ \)(\(35  _\) 4)\)}],
          SingleLetterItalics->False],
        ButtonFunction:>(FrontEndExecute[ {
            FrontEnd`NotebookLocate[ #2]}]&),
        ButtonData:>"T1.6.2.4.1",
        Active->True,
        Background->GrayLevel[1],
        ButtonNote->ButtonData], TraditionalForm]]]], "SmallText"],

Cell[TextData[ButtonBox["Input (3501)",
  ButtonData:>"T1.6.3.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[Cell[BoxData[
    FormBox[
      ButtonBox[
        StyleBox[
          
          RowBox[{\(\(MakeFun\ \((3501)\)\ \[CapitalDelta]\)\(=\)\(0\)\(\ \
\)\), 
            FormBox[\((p\^2 = m\^2)\),
              "TraditionalForm"]}],
          SingleLetterItalics->False],
        ButtonFunction:>(FrontEndExecute[ {
            FrontEnd`NotebookLocate[ #2]}]&),
        ButtonData:>"T1.6.4.1",
        Active->True,
        Background->GrayLevel[1],
        ButtonNote->ButtonData], TraditionalForm]]]], "Text"],

Cell[TextData[ButtonBox["Input (3502)",
  ButtonData:>"T1.6.5.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[Cell[BoxData[
    FormBox[
      ButtonBox[
        StyleBox[
          
          RowBox[{\(\(MakeFun\ \((3502)\)\ \[CapitalDelta]\)\(=\)\(0\)\(\ \
\)\), 
            FormBox[\((p\^2 = m\^2)\),
              "TraditionalForm"]}],
          SingleLetterItalics->False],
        ButtonFunction:>(FrontEndExecute[ {
            FrontEnd`NotebookLocate[ #2]}]&),
        ButtonData:>"T1.6.6.1",
        Active->True,
        Background->GrayLevel[1],
        ButtonNote->ButtonData], TraditionalForm]]]], "Text"],

Cell[TextData[Cell[BoxData[
    FormBox[
      ButtonBox[
        StyleBox[
          
          RowBox[{\(\(MakeFun\ \((3503)\)\ \[CapitalDelta]\)\(=\)\(0\)\(\ \
\)\), 
            FormBox[\((p\^2 = m\^2)\),
              "TraditionalForm"]}],
          SingleLetterItalics->False],
        ButtonFunction:>(FrontEndExecute[ {
            FrontEnd`NotebookLocate[ #2]}]&),
        ButtonData:>"T1.6.7.1",
        Active->True,
        Background->GrayLevel[1],
        ButtonNote->ButtonData], TraditionalForm]]]], "Text"],

Cell[TextData[Cell[BoxData[
    FormBox[
      ButtonBox[
        StyleBox[
          
          RowBox[{\(\(MakeFun\ \((3504)\)\ \[CapitalDelta]\)\(=\)\(0\)\(\ \
\)\), 
            FormBox[\((p\^2 = m\^2)\),
              "TraditionalForm"]}],
          SingleLetterItalics->False],
        ButtonFunction:>(FrontEndExecute[ {
            FrontEnd`NotebookLocate[ #2]}]&),
        ButtonData:>"T1.6.8.1",
        Active->True,
        Background->GrayLevel[1],
        ButtonNote->ButtonData], TraditionalForm]]]], "Text"],

Cell[TextData[ButtonBox["Input (41)",
  ButtonData:>"T1.6.9.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (41)",
  ButtonData:>"T1.6.10.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (43)",
  ButtonData:>"T1.6.11.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (44)",
  ButtonData:>"T1.6.12.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (45)",
  ButtonData:>"T1.6.13.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (43) and (44)",
  ButtonData:>"T1.6.14.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (50)",
  ButtonData:>"T1.6.15.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (51)",
  ButtonData:>"T1.6.16.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (51)",
  ButtonData:>"T1.6.17.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (52)",
  ButtonData:>"T1.6.18.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (52)",
  ButtonData:>"T1.6.19.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (53)",
  ButtonData:>"T1.6.20.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (54)",
  ButtonData:>"T1.6.21.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (54)",
  ButtonData:>"T1.6.22.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (55)",
  ButtonData:>"T1.6.23.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (55)",
  ButtonData:>"T1.6.24.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (56)",
  ButtonData:>"T1.6.25.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (56)",
  ButtonData:>"T1.6.26.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (59)",
  ButtonData:>"T1.6.27.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (59)",
  ButtonData:>"T1.6.28.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (60)",
  ButtonData:>"T1.6.29.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (60)",
  ButtonData:>"T1.6.30.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (61)",
  ButtonData:>"T1.6.31.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (61)",
  ButtonData:>"T1.6.32.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (6153)",
  ButtonData:>"T1.6.33.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (62)",
  ButtonData:>"T1.6.34.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (62)",
  ButtonData:>"T1.6.35.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (63)",
  ButtonData:>"T1.6.36.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (63)",
  ButtonData:>"T1.6.37.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (6552)",
  ButtonData:>"T1.6.38.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (6552)",
  ButtonData:>"T1.6.39.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (6553)",
  ButtonData:>"T1.6.40.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (66)",
  ButtonData:>"T1.6.41.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (66)",
  ButtonData:>"T1.6.42.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (alternative67)",
  ButtonData:>"T1.6.43.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (67)",
  ButtonData:>"T1.6.44.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (67)",
  ButtonData:>"T1.6.45.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (6713)",
  ButtonData:>"T1.6.46.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (68)",
  ButtonData:>"T1.6.47.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (68)",
  ButtonData:>"T1.6.48.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (6813)",
  ButtonData:>"T1.6.49.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (70)",
  ButtonData:>"T1.6.50.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (70)",
  ButtonData:>"T1.6.51.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (71)",
  ButtonData:>"T1.6.52.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (71)",
  ButtonData:>"T1.6.53.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (7123)",
  ButtonData:>"T1.6.54.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (7113)",
  ButtonData:>"T1.6.55.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (78)",
  ButtonData:>"T1.6.56.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (78)",
  ButtonData:>"T1.6.57.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (80)",
  ButtonData:>"T1.6.58.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (80)",
  ButtonData:>"T1.6.59.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (81)",
  ButtonData:>"T1.6.60.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (81)",
  ButtonData:>"T1.6.61.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (8123)",
  ButtonData:>"T1.6.62.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (8113)",
  ButtonData:>"T1.6.63.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (82)",
  ButtonData:>"T1.6.64.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (82)",
  ButtonData:>"T1.6.65.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (8213)",
  ButtonData:>"T1.6.66.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (8212)",
  ButtonData:>"T1.6.67.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (83)",
  ButtonData:>"T1.6.68.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (83)",
  ButtonData:>"T1.6.69.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (87)",
  ButtonData:>"T1.6.70.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["(88)",
  ButtonData:>"T1.6.70.1.1",
  ButtonStyle->"Hyperlink"]], "SmallText"],

Cell[TextData[ButtonBox["(89)",
  ButtonData:>"T1.6.70.2.1",
  ButtonStyle->"Hyperlink"]], "SmallText"],

Cell[TextData[ButtonBox["MakeFun (87)",
  ButtonData:>"T1.6.71.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (90)",
  ButtonData:>"T1.6.72.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (90)",
  ButtonData:>"T1.6.73.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (91)",
  ButtonData:>"T1.6.74.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (911)",
  ButtonData:>"T1.6.75.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (912)",
  ButtonData:>"T1.6.76.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (913)",
  ButtonData:>"T1.6.77.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (914)",
  ButtonData:>"T1.6.78.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (92)",
  ButtonData:>"T1.6.79.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (93)",
  ButtonData:>"T1.6.80.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (93)",
  ButtonData:>"T1.6.81.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (94)",
  ButtonData:>"T1.6.82.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (94)",
  ButtonData:>"T1.6.83.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (9401)",
  ButtonData:>"T1.6.84.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (9402)",
  ButtonData:>"T1.6.85.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (95)",
  ButtonData:>"T1.6.86.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (95)",
  ButtonData:>"T1.6.87.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (96)",
  ButtonData:>"T1.6.88.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (96)",
  ButtonData:>"T1.6.89.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (97)",
  ButtonData:>"T1.6.90.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (97)",
  ButtonData:>"T1.6.91.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (98)",
  ButtonData:>"T1.6.92.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (98)",
  ButtonData:>"T1.6.93.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (99)",
  ButtonData:>"T1.6.94.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (99)",
  ButtonData:>"T1.6.95.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (9912)",
  ButtonData:>"T1.6.96.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (9913)",
  ButtonData:>"T1.6.97.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (100)",
  ButtonData:>"T1.6.98.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (100)",
  ButtonData:>"T1.6.99.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (1002)",
  ButtonData:>"T1.6.100.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (1003)",
  ButtonData:>"T1.6.101.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (1004)",
  ButtonData:>"T1.6.102.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (1005)",
  ButtonData:>"T1.6.103.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (1006)",
  ButtonData:>"T1.6.104.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (150)",
  ButtonData:>"T1.6.105.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (150)",
  ButtonData:>"T1.6.106.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (151)",
  ButtonData:>"T1.6.107.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (151)",
  ButtonData:>"T1.6.108.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (152)",
  ButtonData:>"T1.6.109.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (152)",
  ButtonData:>"T1.6.110.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (153)",
  ButtonData:>"T1.6.111.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (153)",
  ButtonData:>"T1.6.112.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (200)",
  ButtonData:>"T1.6.113.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (200)",
  ButtonData:>"T1.6.114.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (201)",
  ButtonData:>"T1.6.115.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (202)",
  ButtonData:>"T1.6.116.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (203)",
  ButtonData:>"T1.6.117.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (204)",
  ButtonData:>"T1.6.118.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (205)",
  ButtonData:>"T1.6.119.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (1xxxx)",
  ButtonData:>"T1.6.120.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["five -> four",
  ButtonData:>"T1.6.121.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["four -> three",
  ButtonData:>"T1.6.122.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["three -> two",
  ButtonData:>"T1.6.123.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["two -> one",
  ButtonData:>"T1.6.124.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["TarcerRecurse and TFIRecurse",
  ButtonData:>"T1.7.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["TFIRecurse",
  ButtonData:>"T1.7.1.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["T*IS",
  ButtonData:>"T1.7.2.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["TComment",
  ButtonData:>"T1.7.3.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["TarcerRecurse",
  ButtonData:>"T1.7.4.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["TarcerExpand",
  ButtonData:>"T1.8.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Construct T's",
  ButtonData:>"T1.9.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["$BasisIntegrals",
  ButtonData:>"T1.10.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Save",
  ButtonData:>"T1.9.1",
  ButtonStyle->"Hyperlink"]], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Preliminaries", "Section",
  CellDingbat->"\[FilledSmallCircle]",
  CellTags->"T1.1.1"],

Cell[TextData[{
  "This is free software.\nYou may use it, distribute it, change it, improve \
it, etc..\nIf you have ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " questions, ask them to support@wolfram.com or to your local ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " reseller."
}], "SmallText",
  Editable->False],

Cell["\<\
 We consider the following type of integrals, where a,b,u,v,r,s,t \
are non-negative integers.\
\>", "Text"],

Cell[TextData[{
  Cell[BoxData[
      \(TraditionalForm\`\(\(TFI\)\([\)\(d, 
        p\^2, \[CapitalDelta]p, {a, b}, {u, v, r, s, t}\)\)\)]],
  ", ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            RowBox[{
              RowBox[{
                RowBox[{
                  RowBox[{
                    RowBox[{
                      FormBox[\({{\[Nu]\_1, m\_1}, {\(\[Nu]\_2\)\(,\)\),
                        "TraditionalForm"], \(m\_2\)}], "}"}], 
                  ",", \({\[Nu]\_3, m\_3}\), ",", \({\[Nu]\_4, m\_4}\), 
                  ",", \({\[Nu]\_5, m\_5}\)}], "}"}], "]"}], " ", "="}], 
          " "}], TraditionalForm]]],
  "\n\n",
  Cell[BoxData[
      \(TraditionalForm\`1\/\[Pi]\^d\)]],
  "\[Integral] \[Integral] ",
  Cell[BoxData[
      FormBox[
        FractionBox[
          RowBox[{
            
            FormBox[\(\(d\^d\) k\_1\ \(d\^d\) 
                k\_2\ \(\((\[CapitalDelta]\[CenterDot]
                        k\_1)\)\^a\) \((\[CapitalDelta]\[CenterDot]k\_1)\)\^b\
\),
              
              "TraditionalForm"], \(\((k\_1\%2)\)\^u\), \(\((k\_2\%2)\)\^v\), \
\(\((p\[CenterDot]k\_1)\)\^r\), \(\((p\[CenterDot]
                    k\_2)\)\^s\), \(\((k\_1\[CenterDot]
                    k\_2)\)\^t\)}], \(\(\(\(\(\(\([\)\(k\_1\%2 - 
                                        m\_1\%2\)\(]\)\)\^\[Nu]\_1\)[
                                  k\_2\%2 - 
                                    m\_2\%2]\^\[Nu]\_2\)[\((k\_1 - p)\)\^2 - 
                              m\_3\%2]\^\[Nu]\_3\)[\((k\_2 - p)\)\^2 - 
                        m\_4\%2]\^\[Nu]\_4\)[\((k\_1 - k\_2)\)\^2 - 
                  m\_5\%2]\^\[Nu]\_5\)], TraditionalForm]]]
}], "Text"],

Cell["\<\
Putting the        $Post = (#;)&;       instruction after   
Unset[$Post]   suppresses all output cells.
Putting Unset[$Post] behind  $Post=(#:)&;
will produce a lot of typsetted reduction formulae.\
\>", "Text"],

Cell[BoxData[
    \(\(Unset[$Post]; \)\)], "Input"],

Cell[BoxData[
    \(\($Post\  = \ \((#; )\)&; \)\)], "Input"],

Cell[BoxData[
    \(\(If[$Notebooks =!= True, $Post\  = \ \((#; )\)&]; \)\)], "Input"],

Cell[BoxData[
    \(\($HistoryLength = 0; \)\)], "Input"],

Cell["\<\
This determines the directory into which the resulting file \
tarcer.mx is written.\
\>", "Text"],

Cell[BoxData[
    \(If[$Notebooks, 
      SetDirectory[
        ToFileName[
          First["\<FileName\>" /. NotebookInformation[InputNotebook[]]]]]]
      \)], "Input"],

Cell[TextData[{
  "Somehow ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " crashes on my (R.M.) good old NeXTStation if I don't open all cells first \
...; this opens all cells automatically:"
}], "SmallText"],

Cell[BoxData[
    \(If[$System === "\<NeXT\>", 
      SelectionMove[EvaluationNotebook[], All, Notebook]; 
      FrontEndExecute[{
          FrontEnd`FrontEndToken[EvaluationNotebook[], 
            "\<SelectionOpenAllGroups\>"]}]]\)], "Input"],

Cell[BoxData[
    \(\(Off[General::spell]; \)\)], "Input"],

Cell[BoxData[
    \(\(Off[General::spell1]; \)\)], "Input"],

Cell[CellGroupData[{

Cell["Creating the tarcerij.mx file", "Subsubsection",
  CellTags->"T1.2.1.1"],

Cell[TextData[{
  "Running this notebook will produce a tarcerij.mx file where ij will be \
replaced by \nthe two number set by the variable $RankLimit (default 33). \n\
In the same directory as this notebook which can be loaded  into ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " by    <<tarcer33.mx,\nor, when running in Terminal mode, on Unix systems, \
already when starting ",
  StyleBox["Mathematica, ",
    FontSlant->"Italic"],
  "as \nmyshellprompt>  math -initfile tarcer33.mx"
}], "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Usages", "Section",
  CellDingbat->"\[FilledSmallCircle]",
  CellTags->"T1.2.1"],

Cell[BoxData[
    \(\(BeginPackage["\<HighEnergyPhysics`Tarcer`\>"]; \)\)], "Input"],

Cell[BoxData[
    \(\(TarcerRecurse::usage = 
      "\<TarcerRecurse[expr] applies recurrence relations for all TFI's (and \
TVI, TJI, TBI, TAI) in expr. For evaluation and Laurent expansion of the \
resulting basis integrals use TarcerExpand.\>"; \)\)], "Input"],

Cell[BoxData[
    \(\(TarcerExpand::usage = 
      "\<TarcerExpand[expr, d -> (4+e), order] or TarcerExpand[epxr, d -> \
(4-e), order] or TarcerExpand[expr, d -> (4 - 2 om)substitutes integrals as \
specified by the option TarcerRules, and does a Laurent expansion in the \
variable around 0 at the right hand side of the second argument (i.e., you \
can use whatever convention you like ...). The same effect as \
TarcerExpand[TarcerRecurse[expr], ...] can be achieved by setting the option \
TarcerRecurse of TarcerExpand to True. TarcerExpand[expr, d -> (4+e)] is \
equivalent to TarcerExpand[expr, d -> (4+e), 0].\>"; \)\)], "Input"],

Cell[BoxData[
    \(\(TarcerReduce::usage = 
      "\<TarcerReduce is an option to TarcerExpand. It should be set to a \
list of basic integrals.\>"; \)\)], "Input"],

Cell[BoxData[
    \(\(Cayley::usage = 
      "\<Cayley[m1,m2,m3,m4,m5,Sqrt[pp]] is the Cayley determinant.\>"; 
    \)\)], "Input"],

Cell[BoxData[
    \(\(Cayleyu::usage = 
      "\<Cayleyu are the sub-Cayley-determinants as described in Tarasov's \
paper.\>"; \)\)], "Input"],

Cell[BoxData[
    \(\(CayleyD::usage = 
      "\<CayleyD are the sub-Cayley-determinants as described in Tarasov's \
paper.\>"; \)\)], "Input"],

Cell[BoxData[
    \(\(PreFactor::usage = 
      "\<PreFactor[expr] may be used to prohibit TarcerExpand from \
Taylor-expanding expr.\>"; \)\)], "Input"],

Cell[BoxData[
    \(\(SEpsilon::usage = 
      "\<Sepsilon[d] is an abbreviation for Exp[(d-4)/2 EulerGamma].\>"; 
    \)\)], "Input"],

Cell[BoxData[
    \(\(TFI::usage = 
      "\<TFI[d, pp, dp, {a,b},{u,v,r,s,t},{ \
{n1,m1},{n2,m2},{n3,m3},{n4,m4},{n5,m5} }] is the massive 2-loop selfenergy \
type integral with five propagators in d dimensions. \n\t\tSee the TARCER \
paper for a detailed descriptioin of the notation used.\>"; \)\)], "Input"],

Cell[BoxData[
    \(\(TFIRecurse::usage = 
      "\<TFIRecurse[expr] applies the recursion algorithm implemented in \
TARCER.nb (following Tarasov) to TFI's (and TVI's, TJI's, TBI's, TAI's in \
expr.\>"; \)\)], "Input"],

Cell[BoxData[
    \(\(TFISimplify::usage = 
      "\<TFISimplify[expr] applies the recursion algorithm implemented in \
TARCER.nb (following Tarasov) to TFI's (and TVI's, TJI's, TBI's, TAI's in \
expr, except k1.k2.\>"; \)\)], "Input"],

Cell[BoxData[
    \(\(TVI::usage = 
      "\<TVI[d,pp,{ {n1,m1},{n2,m2},{n3,m3},{n4,m4}}] is the massive 2-loop \
integral with four propagators in d dimensions. \>"; \)\)], "Input"],

Cell[BoxData[
    \(\(TJI::usage = 
      "\<TJI[d, pp, { {n1,m1},{n2,m2},{n3,m3} }] is the massive 2-loop \
integral with three propagators in d dimensions. \>"; \)\)], "Input"],

Cell[BoxData[
    \(\(TKI::usage = 
      "\<TKI[d, 0, { {n1,m1},{n2,m2},{n3,m3} }] is the bubble massive 2-loop \
selfenergy type integral with three propagators in d dimensions. \>"; \)\)], 
  "Input"],

Cell[BoxData[
    \(\(TBI::usage = 
      "\<TBI[d, pp, { {n1,m1},{n2,m2} }] is the massive 1-loop selfenergy \
type integral with two propagators in d dimensions.\>"; \)\)], "Input"],

Cell[BoxData[
    \(\(TAI::usage = 
      "\<TAI[d, pp, { {n1,m1} }] is the massive 1-loop selfenergy type \
integral with one propagator in d dimensions.\>"; \)\)], "Input"],

Cell[BoxData[
    \(\(TFR::usage = 
      "\<TFR is similar in syntax to TFI but applies once the recurrence \
relations derived for TFI.\>"; \)\)], "Input"],

Cell[BoxData[
    \(\(TVR::usage = 
      "\<TVR is similar in syntax to TVI but applies once the recurrence \
relations derived for TVI.\>"; \)\)], "Input"],

Cell[BoxData[
    \(\(TJR::usage = 
      "\<TJR is similar in syntax to TJI but applies once the recurrence \
relations derived for TJI.\>"; \)\)], "Input"],

Cell[BoxData[
    \(\(TBR::usage = 
      "\<TBR is similar in syntax to TBI but applies once the recurrence \
relations derived for TBI.\>"; \)\)], "Input"],

Cell[BoxData[
    \(\(TAR::usage = 
      "\<TAR is similar in syntax to TAI but applies once the recurrence \
relations derived for TAI.\>"; \)\)], "Input"],

Cell[BoxData[
    \(\($RankLimit::usage = 
      "\<$RankLimit is the list of integers specifying the highest possible \
sums of exponents of scalar products; i.e., {a+b, r+s} means that at most \
(Delta.k1)^a * (Delta.k2)^2 * (p.k1)^r * (p.k2)^s in the numerator will be \
calculated (a,b,r,s have to be non-negative integares). $RankLimit has to be \
set in the TARCER.nb notebook. In the tarcer.mx file this is not possible \
anymore.\>"; \)\)], "Input"],

Cell[BoxData[
    \(\($TarcerRecursed::usage = 
      "\<If $Comment is set to True all integrals\n\t\twhich are reduced are \
collected in the global variable $TarcerRecured.\>"; \)\)], "Input"],

Cell[BoxData[
    \(\($BasisIntegrals::usage = 
      "\<$BasisIntegrals is a list of some basis integrals. It is the default \
setting of TarcerReduce of TarcerExpand.\>"; \)\)], "Input"],

Cell[BoxData[
    \(\($Comment::usage = 
      "\<$Comment can be set to True or False. If set to False no comments \
are generated.\>"; \)\)], "Input"],

Cell[BoxData[
    \(\($CommentNotebook::usage = 
      "\<$Comment can be set to True or False. Setting it to True will \
generate a separate notebook (if and only if $Comment is True) where the used \
recursion relations are displayed.\>"; \)\)], "Input"],

Cell[BoxData[
    \(\($TTable::usage = 
      "\<$TTable is a list of precomputed reductions. $TTable is the default \
setting of the option Table of TarcerRecurse. \>"; \)\)], "Input"],

Cell[BoxData[
    \(\(Begin["\<`Private`\>"]; \)\)], "Input"],

Cell[BoxData[
    \(\(Expand3 = Identity; \)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Prologue (setting of $RankLimit)", "Section",
  CellDingbat->"\[FilledSmallCircle]",
  CellTags->"T1.3.1"],

Cell[TextData[{
  "This sets the limit up to which the ",
  Cell[BoxData[
      \(TraditionalForm\`T\)]],
  "-operators are constructed automatically, i.e., $RankLimit = {a+b, r+s} , \
concerning: ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\((\[CapitalDelta]\[CenterDot]
              k\_1)\)\^a\)\(\ \)\)\)]],
  Cell[BoxData[
      \(TraditionalForm\`\((\[CapitalDelta]\[CenterDot]k\_2)\)\^b\)]],
  " ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          RowBox[{"(", 
            FormBox[\(p\[CenterDot]k\_1\),
              "TraditionalForm"], ")"}], "r"], TraditionalForm]]],
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          RowBox[{"(", 
            FormBox[\(p\[CenterDot]k\_2\),
              "TraditionalForm"], ")"}], "s"], TraditionalForm]]]
}], "Text"],

Cell[BoxData[
    \(\($RankLimit = {0, 6}; \)\)], "Input"],

Cell["Check recursion relations by changing False to True here:", "Text"],

Cell[BoxData[
    \(\($CheckRecursion = False; \)\)], "Input"],

Cell["\<\
Change False to True for getting the input palettes for entering \
new recursion relations.\
\>", "Text"],

Cell[BoxData[
    \(\($PutPalettes = False; \)\)], "Input"],

Cell[TextData[{
  "Notice:  Instead of using shift operators",
  Cell[BoxData[
      FormBox[
        RowBox[{
          FormBox[
            RowBox[{"  ", 
              StyleBox[\(1\^+\),
                FontWeight->"Bold"]}],
            "TraditionalForm"], ",", 
          StyleBox[\(2\^+\),
            FontWeight->"Bold"]}], TraditionalForm]]],
  " etc. the \[Nu]-indices are colored blue; instead of ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          FormBox[
            RowBox[{"  ", 
              StyleBox[\(1\^-\),
                FontWeight->"Bold"]}],
            "TraditionalForm"], ",", 
          StyleBox[\(2\^-\),
            FontWeight->"Bold"]}], TraditionalForm]]],
  " etc. the \[Nu]-indices are colored red."
}], "Text"],

Cell[CellGroupData[{

Cell["Typesetting rules", "Subsection",
  CellTags->"T1.3.1.1"],

Cell[BoxData[
    \(\(atime = AbsoluteTime[]; \)\)], "Input"],

Cell[BoxData[
    \(SEpsilon /: MakeBoxes[SEpsilon[d_], fmt_] := 
      InterpretationBox@@{
          StyleBox[SubscriptBox["\<S\>", First[Variables[d]]], 
            FontWeight \[Rule] "\<Bold\>"], SEpsilon[d], 
          Editable \[Rule] False}\)], "Input"],

Cell[BoxData[
    \(MakeBoxes[SEpsilon[d_]\^n_, fmt_] := 
      InterpretationBox@@{
          StyleBox[SubsuperscriptBox["\<S\>", First[Variables[d]], n], 
            FontWeight \[Rule] "\<Bold\>"], SEpsilon[d], 
          Editable \[Rule] False}\)], "Input"],

Cell[BoxData[
    \(PreFactor /: MakeBoxes[PreFactor[d_], fmt_] := 
      InterpretationBox@@{MakeBoxes[d, fmt], PreFactor[d], Editable -> False}
          \)], "Input"],

Cell["\<\
Don't mess with the InterpretationBox's !! (important for the \
palette and OperatorApply).\
\>", "Text"],

Cell[BoxData[
    \(MakeBoxes[Schiebe[i_, pm_String]\ , fmt_] := 
      InterpretationBox[
        StyleBox[SuperscriptBox[i, pm], FontWeight -> "\<Bold\>"], 
        Schiebe[i, pm], Editable -> True]\)], "Input"],

Cell[BoxData[
    \(\(CayleyD /: MakeBoxes[\(CayleyD[i_Integer]\)[args_], fmt_] := 
      FractionBox[SubscriptBox["\<\[CapitalDelta]\>", \ i], 
        "\<\[CapitalDelta]\>"]; \)\)], "Input"],

Cell[BoxData[
    \(\(CayleyD /: 
      MakeBoxes[\(CayleyD[i_Integer, j_Integer]\)[args_], fmt_] := 
      FractionBox[SubscriptBox["\<\[CapitalDelta]\>", 10\ i + j], 
        "\<\[CapitalDelta]\>"]; \)\)], "Input"],

Cell[BoxData[
    \(\(CayleyD /: 
      MakeBoxes[\(CayleyD[i_Integer, j_Integer, k_Integer]\)[args_], fmt_] := 
      FractionBox[SubscriptBox["\<\[CapitalDelta]\>", 100\ \ i + 10  j + k], 
        "\<\[CapitalDelta]\>"]; \)\)], "Input"],

Cell[BoxData[
    \(\(Cayley /: MakeBoxes[\(Cayley[i_Integer]\)[args_], fmt_] := 
      SubscriptBox["\<\[CapitalDelta]\>", \ i]; \)\)], "Input"],

Cell[BoxData[
    \(\(Cayley /: MakeBoxes[\(Cayley[i_Integer, j_Integer]\)[args_], fmt_] := 
      SubscriptBox["\<\[CapitalDelta]\>", 10\ i + j]; \)\)], "Input"],

Cell[BoxData[
    \(\(Cayley /: 
      MakeBoxes[\(Cayley[i_Integer, j_Integer, k_Integer]\)[args_], fmt_] := 
      SubscriptBox["\<\[CapitalDelta]\>", 100\ \ i + 10  j + k]; \)\)], 
  "Input"],

Cell[BoxData[
    \(\(Cayleyu /: 
      MakeBoxes[\(Cayleyu[i_Integer, j_Integer, k_Integer]\)[args_], fmt_] := 
      SubscriptBox["\<u\>", 100\ \ i + 10  j + k]; \)\)], "Input"],

Cell["\<\
MakeBoxes[PP, fm_] := InterpretationBox[SuperscriptBox[p, 2], PP]; 
MakeBoxes[m1^(p_), _] := InterpretationBox[SubsuperscriptBox[m, 1, p], m1^p]; \

MakeBoxes[m2^(p_), _] := InterpretationBox[SubsuperscriptBox[m, 2, p], m2^p]; \

MakeBoxes[m3^(p_), _] := InterpretationBox[SubsuperscriptBox[m, 3, p], m3^p]; \

MakeBoxes[m4^(p_), _] := InterpretationBox[SubsuperscriptBox[m, 4, p], m4^p]; \

MakeBoxes[m5^(p_), _] := InterpretationBox[SubsuperscriptBox[m, 5, p], m5^p]; \

m6 = Sqrt[PP]; \
\>", "Input"],

Cell[BoxData[
    \(\(MakeBoxes[m1, _] := InterpretationBox[SubscriptBox[m, 1], m1]; \)\)], 
  "Input"],

Cell[BoxData[
    \(\(MakeBoxes[m2, _] := InterpretationBox[SubscriptBox[m, 2], m2]; \)\)], 
  "Input"],

Cell[BoxData[
    \(\(MakeBoxes[m3, _] := InterpretationBox[SubscriptBox[m, 3], m3]; \)\)], 
  "Input"],

Cell[BoxData[
    \(\(MakeBoxes[m4, _] := InterpretationBox[SubscriptBox[m, 4], m4]; \)\)], 
  "Input"],

Cell[BoxData[
    \(\(MakeBoxes[m5, _] := InterpretationBox[SubscriptBox[m, 5], m5]; \)\)], 
  "Input"],

Cell[BoxData[
    \(\(mbt[z_] := ToBoxes[z, TraditionalForm]; \)\)], "Input"],

Cell[BoxData[
    \(\(redblue[z_ /; Head[z] =!= Plus] := mbt[z]; \)\)], "Input"],

Cell[BoxData[
    \(\(redblue[z_ - 1] := 
      StyleBox[mbt[z], FontColor \[Rule] RGBColor[1, 0, 0]]; \)\)], "Input"],

Cell[BoxData[
    \(\(redblue[z_Subscript - 2] := 
      StyleBox[SubscriptBox[
          OverscriptBox[z\[LeftDoubleBracket]1\[RightDoubleBracket], 
            "\<_\>"], z\[LeftDoubleBracket]2\[RightDoubleBracket]], 
        FontColor \[Rule] RGBColor[1, 0, 0]]; \)\)], "Input"],

Cell[BoxData[
    \(\(redblue[z_ - 2] := 
      StyleBox[OverscriptBox[mbt[z], "\<_\>"], 
        FontColor \[Rule] RGBColor[1, 0, 0]]; \)\)], "Input"],

Cell[BoxData[
    \(\(redblue[z_ - 3] := 
      StyleBox[UnderscriptBox[OverscriptBox[mbt[z], "\<_\>"], "\<_\>"], 
        FontColor \[Rule] RGBColor[1, 0, 0]]; \)\)], "Input"],

Cell[BoxData[
    \(\(redblue[z_ + 1] := 
      StyleBox[mbt[z], FontColor \[Rule] RGBColor[0, 0, 1]]; \)\)], "Input"],

Cell[BoxData[
    \(\(redblue[z_ + 2] := 
      StyleBox[OverscriptBox[mbt[z], "\<_\>"], 
        FontColor \[Rule] RGBColor[0, 0, 1]]; \)\)], "Input"],

Cell[BoxData[
    \(\(redblue[z_Subscript + 2] := 
      StyleBox[SubscriptBox[
          OverscriptBox[z\[LeftDoubleBracket]1\[RightDoubleBracket], 
            "\<_\>"], z\[LeftDoubleBracket]2\[RightDoubleBracket]], 
        FontColor \[Rule] RGBColor[0, 0, 1]]; \)\)], "Input"],

Cell[BoxData[
    \(\(TFI /: MakeBoxes[TFI[d_, pp_, {ur__}, {den__}], fmt_] := \n\t
      \((InterpretationBox@@{
              SubsuperscriptBox[
                StyleBox["\<F\>", SingleLetterItalics -> False, 
                  FontWeight -> "\<Bold\>"], RowBox@@{\((redblue/@{den})\)}, 
                RowBox[{"\<(\>", ToBoxes[d, TraditionalForm], "\<)\>", 
                    "\< \>", ur}]], \n\t\tTFI[d, pp, {ur}, {den}], 
              Editable -> True})\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]; \)\)], "Input"],

Cell[BoxData[
    \(\(TFI /: MakeBoxes[TFI[d_, pp_, dp_, any__, {den__}], fmt_] := \n\t
      \(\((InterpretationBox@@{
                SubsuperscriptBox[
                  StyleBox["\<F\>", SingleLetterItalics -> False, 
                    FontWeight -> "\<Bold\>"], 
                  RowBox@@{\((redblue/@{den})\)}, 
                  RowBox[{"\<(\>", ToBoxes[d, TraditionalForm], "\<)\>", 
                      "\< \>", Apply[Sequence, Flatten[{any}]]}]], \n\t\t
                TFI[d, pp, dp, any, {den}], Editable -> True})\) /; 
          MemberQ[{StandardForm, TraditionalForm}, fmt]\) /; 
        Head[dp] =!= List; \)\)], "Input"],

Cell[BoxData[
    \(\(TFI /: MakeBoxes[TFI[dpp__, {den__}], fmt_] := \n\t
      \((InterpretationBox@@{
              SubsuperscriptBox[
                StyleBox["\<F\>", SingleLetterItalics -> False, 
                  FontWeight -> "\<Bold\>"], RowBox@@{\((redblue/@{den})\)}, 
                RowBox[{"\<(\>", ToBoxes[First[{dpp}], TraditionalForm], 
                    "\<)\>"}]], \n\t\tTFI[dpp, {den}], Editable -> True})
          \) /; MemberQ[{StandardForm, TraditionalForm}, fmt] && 
          Head[Last[{dpp}]] =!= List; \)\)], "Input"],

Cell[BoxData[
    \(\(TVI /: MakeBoxes[TVI[dpp__, {den__}], fmt_] := \n
      \((\t\((InterpretationBox@@{
                  SubsuperscriptBox[
                    StyleBox["\<V\>", SingleLetterItalics -> False, 
                      FontWeight -> "\<Bold\>"], 
                    RowBox@@{\((redblue/@{den})\)}, 
                    RowBox[{"\<(\>", 
                        ToBoxes[{dpp}[\([1]\)], TraditionalForm], "\<)\>"}]], 
                  \n\t\(Hold[TVI]\)[dpp, {den}], Editable -> True})\) /. 
            Hold[TVI] -> TVI)\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt] && 
          Head[Last[{dpp}]] =!= List; \)\)], "Input"],

Cell[BoxData[
    \({TVI[d, {\[Alpha], \[Beta], \[Gamma], \[Delta]}], 
      TVI[d, {\[Alpha] + 1, \[Beta], \[Gamma], \[Delta]}], 
      TVI[d, {\[Alpha] - 1, \[Beta], \[Gamma], \[Delta]}]}\)], "Input"],

Cell[BoxData[
    \(TJI /: MakeBoxes[TJI[d_, 0, {den__}], fmt_] := \t
      \((\n\((\((InterpretationBox@@{
                      SubsuperscriptBox[
                        StyleBox["\<K\>", SingleLetterItalics -> False, 
                          FontWeight -> "\<Bold\>"], 
                        RowBox@@{\((redblue/@{den})\)}, 
                        RowBox[{"\<(\>", ToBoxes[d, TraditionalForm], 
                            "\<)\>"}]], \n\t\t\(Hold[TJI]\)[d, 0, {den}], 
                      Editable -> True})\) /. Hold[TJI] -> TJI)\) /; 
            MemberQ[{StandardForm, TraditionalForm}, fmt]\n\t)\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]\)], "Input"],

Cell[BoxData[
    \(TJI /: MakeBoxes[TJI[dpp__, {den__}], fmt_] := \t
      \((\((InterpretationBox@@{
                  SubsuperscriptBox[
                    StyleBox["\<J\>", SingleLetterItalics -> False, 
                      FontWeight -> "\<Bold\>"], 
                    RowBox@@{\((redblue/@{den})\)}, 
                    RowBox[{"\<(\>", 
                        ToBoxes[{dpp}[\([1]\)], TraditionalForm], "\<)\>"}]], 
                  \n\t\t\(Hold[TJI]\)[dpp, {den}], Editable -> True})\) /. 
            Hold[TJI] -> TJI)\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]\)], "Input"],

Cell[BoxData[
    \(\(TKI /: MakeBoxes[TKI[d_, pp___, {den__}], fmt_] := \n\t
      \((SubsuperscriptBox[
            StyleBox["\<K\>", SingleLetterItalics -> False, 
              FontWeight -> "\<Bold\>"], RowBox@@{\((redblue/@{den})\)}, 
            RowBox[{"\<(\>", ToBoxes[d, TraditionalForm], "\<)\>"}]])\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]; \)\)], "Input"],

Cell[BoxData[
    \(TKI /: MakeBoxes[TKI[dpp__, {den__}], fmt_] := \t
      \((\((InterpretationBox@@{
                  SubsuperscriptBox[
                    StyleBox["\<K\>", SingleLetterItalics -> False, 
                      FontWeight -> "\<Bold\>"], 
                    RowBox@@{\((redblue/@{den})\)}, 
                    RowBox[{"\<(\>", 
                        ToBoxes[{dpp}[\([1]\)], TraditionalForm], "\<)\>"}]], 
                  \n\t\t\(Hold[TKI]\)[dpp, {den}], Editable -> True})\) /. 
            Hold[TKI] -> TKI)\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]\)], "Input"],

Cell[BoxData[
    \({TKI[d, {\[Alpha], \[Beta], \[Gamma]}], 
      TKI[d, {\[Alpha] + 1, \[Beta], \[Gamma]}], 
      TKI[d, {\[Alpha] - 1, \[Beta], \[Gamma]}]}\)], "Input"],

Cell[BoxData[
    \(\(TBI /: MakeBoxes[TBI[d_, pp___, {den__}], fmt_] := \n\t
      \((SubsuperscriptBox[
            StyleBox["\<B\>", SingleLetterItalics -> False, 
              FontWeight -> "\<Bold\>"], RowBox@@{\((redblue/@{den})\)}, 
            RowBox[{"\<(\>", ToBoxes[d, TraditionalForm], "\<)\>"}]])\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]; \)\)], "Input"],

Cell[BoxData[
    \(\(TBI /: MakeBoxes[TBI[d_, pp___, {den__}], fmt_] := \n\t
      \((InterpretationBox@@{
              SubsuperscriptBox[
                StyleBox["\<B\>", SingleLetterItalics -> False, 
                  FontWeight -> "\<Bold\>"], RowBox@@{\((redblue/@{den})\)}, 
                RowBox[{"\<(\>", ToBoxes[d, TraditionalForm], "\<)\>"}]], \n
              \t\tTBI[d, pp, {den}], Editable -> True})\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]; \)\)], "Input"],

Cell[BoxData[
    \({TBI[d, {\[Alpha], \[Beta]}], TBI[d, {\[Alpha] + 1, \[Beta]}], 
      TBI[d, {\[Alpha] - 1, \[Beta]}]}\)], "Input"],

Cell[BoxData[
    \(\(TAI /: MakeBoxes[TAI[d_, pp___, {den__}], fmt_] := \n\t
      \((InterpretationBox@@{
              SubsuperscriptBox[
                StyleBox["\<A\>", SingleLetterItalics -> False, 
                  FontWeight -> "\<Bold\>"], RowBox@@{\((redblue/@{den})\)}, 
                RowBox[{"\<(\>", ToBoxes[d, TraditionalForm], "\<)\>"}]], \n
              \t\tTAI[d, {den}], Editable -> True})\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]; \)\)], "Input"],

Cell[BoxData[
    \({TAI[d, {\[Alpha]}], TAI[d, {\[Alpha] + 1}], TAI[d, {\[Alpha] - 1}]}
      \)], "Input"],

Cell["\<\
T*IC are just like T*I , but used only for commentary in the 
recursion (TComment).\
\>", "Text"],

Cell[BoxData[
    \(\(TFIC /: MakeBoxes[TFIC[d_, pp_, {0, 0, i_, 0, 0}, {den__}], fmt_] := 
      \n\t\((InterpretationBox@@{
              SubsuperscriptBox[
                StyleBox["\<F\>", SingleLetterItalics -> False, 
                  FontWeight -> "\<Bold\>"], RowBox@@{\((redblue/@{den})\)}, 
                RowBox[{"\<(\>", ToBoxes[d, TraditionalForm], "\<)\>", 
                    "\< \>", i, "\<0\>"}]], \n\t\t
              TFIC[d, pp, {0, 0, i, 0, 0}, {den}], Editable -> True})\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]; \)\)], "Input"],

Cell[BoxData[
    \(\(TFIC /: MakeBoxes[TFIC[d_, pp_, {0, 0, 0, i_, 0}, {den__}], fmt_] := 
      \n\t\((InterpretationBox@@{
              SubsuperscriptBox[
                StyleBox["\<F\>", SingleLetterItalics -> False, 
                  FontWeight -> "\<Bold\>"], RowBox@@{\((redblue/@{den})\)}, 
                RowBox[{"\<(\>", ToBoxes[d, TraditionalForm], "\<)\>", 
                    "\< \>", "\<0\>", i}]], \n\t\t
              TFIC[d, pp, {0, 0, 0, i, 0}, {den}], Editable -> True})\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]; \)\)], "Input"],

Cell[BoxData[
    \(\(TFIC /: MakeBoxes[TFIC[dpp__, {den__}], fmt_] := \n\t
      \((InterpretationBox@@{
              SubsuperscriptBox[
                StyleBox["\<F\>", SingleLetterItalics -> False, 
                  FontWeight -> "\<Bold\>"], RowBox@@{\((redblue/@{den})\)}, 
                RowBox[{"\<(\>", ToBoxes[First[{dpp}], TraditionalForm], 
                    "\<)\>"}]], \n\t\tTFIC[dpp, {den}], Editable -> True})
          \) /; MemberQ[{StandardForm, TraditionalForm}, fmt] && 
          Head[Last[{dpp}]] =!= List; \)\)], "Input"],

Cell[BoxData[
    \(\(TVIC /: MakeBoxes[TVIC[dpp__, {den__}], fmt_] := \n
      \((\t\((InterpretationBox@@{
                  SubsuperscriptBox[
                    StyleBox["\<V\>", SingleLetterItalics -> False, 
                      FontWeight -> "\<Bold\>"], 
                    RowBox@@{\((redblue/@{den})\)}, 
                    RowBox[{"\<(\>", 
                        ToBoxes[{dpp}[\([1]\)], TraditionalForm], "\<)\>"}]], 
                  \n\t\(Hold[TVIC]\)[dpp, {den}], Editable -> True})\) /. 
            Hold[TVIC] -> TVIC)\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt] && 
          Head[Last[{dpp}]] =!= List; \)\)], "Input"],

Cell[BoxData[
    \(TJIC /: MakeBoxes[TJIC[d_, 0, {den__}], fmt_] := \t
      \((\n\((\((InterpretationBox@@{
                      SubsuperscriptBox[
                        StyleBox["\<K\>", SingleLetterItalics -> False, 
                          FontWeight -> "\<Bold\>"], 
                        RowBox@@{\((redblue/@{den})\)}, 
                        RowBox[{"\<(\>", ToBoxes[d, TraditionalForm], 
                            "\<)\>"}]], \n\t\t\(Hold[TJIC]\)[d, 0, {den}], 
                      Editable -> True})\) /. Hold[TJIC] -> TJIC)\) /; 
            MemberQ[{StandardForm, TraditionalForm}, fmt]\n\t)\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]\)], "Input"],

Cell[BoxData[
    \(TJIC /: MakeBoxes[TJIC[dpp__, {den__}], fmt_] := \t
      \((\((InterpretationBox@@{
                  SubsuperscriptBox[
                    StyleBox["\<J\>", SingleLetterItalics -> False, 
                      FontWeight -> "\<Bold\>"], 
                    RowBox@@{\((redblue/@{den})\)}, 
                    RowBox[{"\<(\>", 
                        ToBoxes[{dpp}[\([1]\)], TraditionalForm], "\<)\>"}]], 
                  \n\t\t\(Hold[TJIC]\)[dpp, {den}], Editable -> True})\) /. 
            Hold[TJIC] -> TJIC)\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]\)], "Input"],

Cell[BoxData[
    \(\(TBIC /: MakeBoxes[TBIC[d_, pp___, {den__}], fmt_] := \n\t
      \((SubsuperscriptBox[
            StyleBox["\<B\>", SingleLetterItalics -> False, 
              FontWeight -> "\<Bold\>"], RowBox@@{\((redblue/@{den})\)}, 
            RowBox[{"\<(\>", ToBoxes[d, TraditionalForm], "\<)\>"}]])\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]; \)\)], "Input"],

Cell[BoxData[
    \(\(TBIC /: MakeBoxes[TBIC[d_, pp___, {den__}], fmt_] := \n\t
      \((InterpretationBox@@{
              SubsuperscriptBox[
                StyleBox["\<B\>", SingleLetterItalics -> False, 
                  FontWeight -> "\<Bold\>"], RowBox@@{\((redblue/@{den})\)}, 
                RowBox[{"\<(\>", ToBoxes[d, TraditionalForm], "\<)\>"}]], \n
              \t\tTBIC[d, pp, {den}], Editable -> True})\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]; \)\)], "Input"],

Cell[BoxData[
    \(\(TAIC /: MakeBoxes[TAIC[d_, pp___, {den__}], fmt_] := \n\t
      \((InterpretationBox@@{
              SubsuperscriptBox[
                StyleBox["\<A\>", SingleLetterItalics -> False, 
                  FontWeight -> "\<Bold\>"], RowBox@@{\((redblue/@{den})\)}, 
                RowBox[{"\<(\>", ToBoxes[d, TraditionalForm], "\<)\>"}]], \n
              \t\tTAIC[d, {den}], Editable -> True})\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]; \)\)], "Input"],

Cell[BoxData[
    \(\(TFR /: MakeBoxes[TFR[d_, pp_, {0, 0, i_, 0, 0}, {den__}], fmt_] := \n
      \t\((InterpretationBox@@{
              SubsuperscriptBox[
                StyleBox["\<F\>", SingleLetterItalics -> False, 
                  FontWeight -> "\<Bold\>"], RowBox@@{\((redblue/@{den})\)}, 
                RowBox[{"\<(\>", ToBoxes[d, TraditionalForm], "\<)\>", 
                    "\< \>", i, "\<0\>"}]], \n\t\t
              TFR[d, pp, {0, 0, i, 0, 0}, {den}], Editable -> True})\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]; \)\)], "Input"],

Cell[BoxData[
    \(\(TFR /: MakeBoxes[TFR[d_, pp_, {0, 0, 0, i_, 0}, {den__}], fmt_] := \n
      \t\((InterpretationBox@@{
              SubsuperscriptBox[
                StyleBox["\<F\>", SingleLetterItalics -> False, 
                  FontWeight -> "\<Bold\>"], RowBox@@{\((redblue/@{den})\)}, 
                RowBox[{"\<(\>", ToBoxes[d, TraditionalForm], "\<)\>", 
                    "\< \>", "\<0\>", i}]], \n\t\t
              TFR[d, pp, {0, 0, 0, i, 0}, {den}], Editable -> True})\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]; \)\)], "Input"],

Cell[BoxData[
    \(\(TFR /: MakeBoxes[TFR[dpp__, {den__}], fmt_] := \n\t
      \((InterpretationBox@@{
              SubsuperscriptBox[
                StyleBox["\<F\>", SingleLetterItalics -> False, 
                  FontWeight -> "\<Bold\>"], RowBox@@{\((redblue/@{den})\)}, 
                RowBox[{"\<(\>", ToBoxes[First[{dpp}], TraditionalForm], 
                    "\<)\>"}]], \n\t\tTFR[dpp, {den}], Editable -> True})
          \) /; MemberQ[{StandardForm, TraditionalForm}, fmt] && 
          Head[Last[{dpp}]] =!= List; \)\)], "Input"],

Cell[BoxData[
    \(\(TVR /: MakeBoxes[TVR[dpp__, {den__}], fmt_] := \n
      \((\t\((InterpretationBox@@{
                  SubsuperscriptBox[
                    StyleBox["\<V\>", SingleLetterItalics -> False, 
                      FontWeight -> "\<Bold\>"], 
                    RowBox@@{\((redblue/@{den})\)}, 
                    RowBox[{"\<(\>", 
                        ToBoxes[{dpp}[\([1]\)], TraditionalForm], "\<)\>"}]], 
                  \n\t\(Hold[TVR]\)[dpp, {den}], Editable -> True})\) /. 
            Hold[TVR] -> TVR)\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt] && 
          Head[Last[{dpp}]] =!= List; \)\)], "Input"],

Cell[BoxData[
    \(TJR /: MakeBoxes[TJR[d_, 0, {den__}], fmt_] := \t
      \((\n\((\((InterpretationBox@@{
                      SubsuperscriptBox[
                        StyleBox["\<K\>", SingleLetterItalics -> False, 
                          FontWeight -> "\<Bold\>"], 
                        RowBox@@{\((redblue/@{den})\)}, 
                        RowBox[{"\<(\>", ToBoxes[d, TraditionalForm], 
                            "\<)\>"}]], \n\t\t\(Hold[TJR]\)[d, 0, {den}], 
                      Editable -> True})\) /. Hold[TJR] -> TJR)\) /; 
            MemberQ[{StandardForm, TraditionalForm}, fmt]\n\t)\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]\)], "Input"],

Cell[BoxData[
    \(TJR /: MakeBoxes[TJR[dpp__, {den__}], fmt_] := \t
      \((\((InterpretationBox@@{
                  SubsuperscriptBox[
                    StyleBox["\<J\>", SingleLetterItalics -> False, 
                      FontWeight -> "\<Bold\>"], 
                    RowBox@@{\((redblue/@{den})\)}, 
                    RowBox[{"\<(\>", 
                        ToBoxes[{dpp}[\([1]\)], TraditionalForm], "\<)\>"}]], 
                  \n\t\t\(Hold[TJR]\)[dpp, {den}], Editable -> True})\) /. 
            Hold[TJR] -> TJR)\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]\)], "Input"],

Cell[BoxData[
    \(\(TBR /: MakeBoxes[TBR[d_, pp___, {den__}], fmt_] := \n\t
      \((SubsuperscriptBox[
            StyleBox["\<B\>", SingleLetterItalics -> False, 
              FontWeight -> "\<Bold\>"], RowBox@@{\((redblue/@{den})\)}, 
            RowBox[{"\<(\>", ToBoxes[d, TraditionalForm], "\<)\>"}]])\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]; \)\)], "Input"],

Cell[BoxData[
    \(\(TBR /: MakeBoxes[TBR[d_, pp___, {den__}], fmt_] := \n\t
      \((InterpretationBox@@{
              SubsuperscriptBox[
                StyleBox["\<B\>", SingleLetterItalics -> False, 
                  FontWeight -> "\<Bold\>"], RowBox@@{\((redblue/@{den})\)}, 
                RowBox[{"\<(\>", ToBoxes[d, TraditionalForm], "\<)\>"}]], \n
              \t\tTBR[d, pp, {den}], Editable -> True})\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]; \)\)], "Input"],

Cell[BoxData[
    \(\(TAR /: MakeBoxes[TAR[d_, pp___, {den__}], fmt_] := \n\t
      \((InterpretationBox@@{
              SubsuperscriptBox[
                StyleBox["\<A\>", SingleLetterItalics -> False, 
                  FontWeight -> "\<Bold\>"], RowBox@@{\((redblue/@{den})\)}, 
                RowBox[{"\<(\>", ToBoxes[d, TraditionalForm], "\<)\>"}]], \n
              \t\tTAR[d, {den}], Editable -> True})\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]; \)\)], "Input"],

Cell[BoxData[
    \(\(n1 /: MakeBoxes[n1, TraditionalForm] := SubscriptBox["\<\[Nu]\>", 1]; 
    \)\)], "Input"],

Cell[BoxData[
    \(\(n2 /: MakeBoxes[n2, TraditionalForm] := SubscriptBox["\<\[Nu]\>", 2]; 
    \)\)], "Input"],

Cell[BoxData[
    \(\(n3 /: MakeBoxes[n3, TraditionalForm] := SubscriptBox["\<\[Nu]\>", 3]; 
    \)\)], "Input"],

Cell[BoxData[
    \(\(n4 /: MakeBoxes[n4, TraditionalForm] := SubscriptBox["\<\[Nu]\>", 4]; 
    \)\)], "Input"],

Cell[BoxData[
    \(\(n5 /: MakeBoxes[n5, TraditionalForm] := SubscriptBox["\<\[Nu]\>", 5]; 
    \)\)], "Input"],

Cell[BoxData[
    \(\(MakeBoxes[ParD[i_]\^j_, TraditionalForm] := 
      SubsuperscriptBox["\<\[PartialD]\>", i, j]; \)\)], "Input"],

Cell[BoxData[
    \(\(MakeBoxes[ParD[i_], TraditionalForm] ^= 
      SubscriptBox["\<\[PartialD]\>", i]; \)\)], "Input"],

Cell[BoxData[
    \(\(MakeBoxes[DPlus, TraditionalForm] := 
      StyleBox[SuperscriptBox["\<d\>", "\<+\>"], 
        FontWeight \[Rule] "\<Bold\>", SingleLetterItalics \[Rule] False]; 
    \)\)], "Input"],

Cell[BoxData[
    \(MakeBoxes[TpD[_, b_], TraditionalForm] := 
      MakeBoxes[b, TraditionalForm]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Palettes", "Subsection",
  CellTags->"T1.3.2.1"],

Cell[BoxData[
    \(\(If[$PutPalettes === True, \n\t
      \(tobut[a_]\  := \ 
        ButtonBox[ToBoxes[a, StandardForm], \ ButtonStyle -> "\<Paste\>"]; \n
      \ If[Head[pal1] === NotebookObject, \n\tNotebookClose[pal1]]; \n\t
      pal1 = NotebookPut[
          Notebook[{
              Cell[BoxData[
                  GridBox[\n\t\t\t\t\t\t\t
                    Join[Transpose@{\n\t\t\t\t\t\t\t\t\t
                          Table[tobut@\(m\_j\^2\), {j, 5}], \n
                          \t\t\t\t\t\t\t\t\tTable[tobut@\[Nu]\_j, {j, 5}], \n
                          \t\t\t\t\t\t\t\t\t
                          Table[tobut@Schiebe[i, "\<+\>"], {i, 5}], \n
                          \t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ 
                          Table[tobut@Schiebe[i, "\<-\>"], {i, 5}]}, \n\t\t
                      \t\t\t\t\t
                      \({\ttobut/@{PP, \[CapitalDelta]\_\[Placeholder], 
                            u\_\[Placeholder], \[ScriptCapitalC]\ }}\n\)\t
                      \t\t\t\t\t\t], RowSpacings -> 0, \n\ \ \ \ \ \ 
                    ColumnSpacings -> 0]], NotebookDefault]}, \n\t
            Editable -> True, \nWindowToolbars -> {}, \n
            WindowSize -> {Fit, \ Fit}, \n
            WindowMargins -> {{0, \ Automatic}, \ {0, Automatic}}, \n
            WindowFrame -> "\<Palette\>", \nWindowElements -> {}, \n
            WindowFrameElements -> "\<CloseBox\>", \n
            WindowClickSelect -> False, \n
            ScrollingOptions -> {"\<PagewiseScrolling\>" -> True}, \n
            ShowCellBracket -> False, \n
            CellMargins -> {{0, \ 0}, \ {0, \ 0}}, \nActive -> True, \n
            CellOpen -> True, \nShowCellLabel -> False, \n
            ShowCellTags -> False, \nImageMargins -> {{0, \ 0}, \ {0, \ 0}}, 
            \nMagnification -> 2, Visible -> True]]; \n
      SetOptions[pal1, WindowMargins -> {{0, Automatic}, {Automatic, 0}}]; \n
      SetOptions[pal1, Visible -> True, WindowSize -> {Fit, Fit}]; \n\ 
      If[Head[pal2] === NotebookObject, \n\tNotebookClose[pal2]]; \n\t
      pal2 = NotebookPut[
          Notebook[{
              Cell[BoxData[
                  GridBox[\n\t\t\t\t\t\t\t
                    JJJ = Join[{
                          tobut/@{
                              TFI[d + 2, {\[Nu]\_1, \[Nu]\_2, \[Nu]\_3, 
                                  \[Nu]\_4, \[Nu]\_5}], 
                              TVI[d + 2, {\[Nu]\_1, \[Nu]\_2, \[Nu]\_3, 
                                  \[Nu]\_4}], 
                              TJI[d + 2, {\[Nu]\_1, \[Nu]\_2, \[Nu]\_3}], 
                              TKI[d, {\[Nu]\_1, \[Nu]\_2, \[Nu]\_3}], 
                              TBI[d, {\[Nu]\_1, \[Nu]\_2}], 
                              TAI[d, {\[Nu]\_1}]}}, \n\t\t\t\t\t\t\t\t
                        \({tobut/@{
                              TFI[d, {\[Nu]\_1, \[Nu]\_2, \[Nu]\_3, \[Nu]\_4, 
                                  \[Nu]\_5}], 
                              TVI[d, {\[Nu]\_1, \[Nu]\_2, \[Nu]\_3, 
                                  \[Nu]\_4}], 
                              TJI[d, {\[Nu]\_1, \[Nu]\_2, \[Nu]\_3}], 
                              TKI[d - 2, {\[Nu]\_1, \[Nu]\_2, \[Nu]\_3}], 
                              TBI[d - 2, {\[Nu]\_1, \[Nu]\_2}], 
                              TAI[d - 2, {\[Nu]\_1}]}}\n\)\t\t\t\t\t\t\t], 
                    RowSpacings -> 0, \n\ \ \ \ \ \ ColumnSpacings -> 0]], 
                NotebookDefault]}, \n\tEditable -> True, \n
            WindowToolbars -> {}, \nWindowSize -> {FitAll, \ FitAll}, \n
            WindowMargins -> {{0, Automatic}, \ {0, Automatic}}, \n
            WindowFrame -> "\<Palette\>", \nWindowElements -> {}, \n
            WindowFrameElements -> "\<CloseBox\>", \n
            WindowClickSelect -> False, \n
            ScrollingOptions -> {"\<PagewiseScrolling\>" -> True}, \n
            ShowCellBracket -> False, \nCellMargins -> {{0, 0}, \ {0, 0}}, \n
            Active -> True, \nCellOpen -> True, \nShowCellLabel -> False, \n
            ShowCellTags -> False, \nImageMargins -> {{0, 0}, \ {0, \ 0}}, \n
            Magnification -> 2, Visible -> True]]; \n
      SetOptions[pal2, WindowMargins -> {{Automatic, 15}, {Automatic, 4}}]; \n
      SetOptions[pal2, Visible -> True, WindowSize -> {Fit, Fit}]; \n\t
      SetOptions[pal2, WindowSize -> {Fit, Fit}]\n\)\t]; \)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["OperatorApply", "Subsection",
  CellTags->"T1.3.3.1"],

Cell[BoxData[
    \(\(OperatorApplyF[x_ == y_] := 
      Block[{sch}, \n\t\t
        sch\  = \ {
            Schiebe[i_, "\<+\>"]\^ni_. \ TFI[d_, {n15__}] :> \n\t\t\t\t\t
              TFI[d, ReplacePart[{n15}, {n15}[\([i]\)] + ni, i]], \n\t\t\t\t
            \(Schiebe[i_, "\<-\>"]\^ni_. \ TFI[d_, {n15__}] :> \n\t\t\t\t\t
              TFI[d, ReplacePart[{n15}, {n15}[\([i]\)] - ni, i]]\n\)\t\t\t}; 
        Map[Collect[Expand[#] //. sch, TFI[__], Factor]&, \n\t\t\tx == y]]; 
    \)\)], "Input"],

Cell[BoxData[
    \(\(OperatorApplyV[x_ == y_] := 
      Block[{sch}, \n\t\t
        sch\  = \ {
            Schiebe[i_, "\<+\>"]\^ni_. \ TVI[d_, {n15__}] :> \n\t\t\t\t\t
              TVI[d, ReplacePart[{n15}, {n15}[\([i]\)] + ni, i]], \n\t\t\t\t
            \(Schiebe[i_, "\<-\>"]\^ni_. \ TVI[d_, {n15__}] :> \n\t\t\t\t\t
              TVI[d, ReplacePart[{n15}, {n15}[\([i]\)] - ni, i]]\n\)\t\t\t}; 
        Map[Collect[Expand[#] //. sch, TVI[__], Factor]&, \n\t\t\tx == y]]; 
    \)\)], "Input"],

Cell[BoxData[
    \(\(OperatorApplyJ[x_ == y_] := 
      Block[{sch}, \n\t\t
        sch\  = \ {
            Schiebe[i_, "\<+\>"]\^ni_. \ TJI[d_, {n15__}] :> \n\t\t\t\t\t
              TJI[d, ReplacePart[{n15}, {n15}[\([i]\)] + ni, i]], \n\t\t\t\t
            \(Schiebe[i_, "\<-\>"]\^ni_. \ TJI[d_, {n15__}] :> \n\t\t\t\t\t
              TJI[d, ReplacePart[{n15}, {n15}[\([i]\)] - ni, i]]\n\)\t\t\t}; 
        Map[Collect[Expand[#] //. sch, TJI[__], Factor]&, \n\t\t\tx == y]]; 
    \)\)], "Input"],

Cell[BoxData[
    \(\(OperatorApplyK[x_ == y_] := 
      Block[{sch}, \n\t\t
        sch\  = \ {
            Schiebe[i_, "\<+\>"]\^ni_. \ TKI[d_, {n15__}] :> \n\t\t\t\t\t
              TKI[d, ReplacePart[{n15}, {n15}[\([i]\)] + ni, i]], \n\t\t\t\t
            \(Schiebe[i_, "\<-\>"]\^ni_. \ TKI[d_, {n15__}] :> \n\t\t\t\t\t
              TKI[d, ReplacePart[{n15}, {n15}[\([i]\)] - ni, i]]\n\)\t\t\t}; 
        Map[Collect[Expand[#] //. sch, TKI[__], Factor]&, \n\t\t\tx == y]]; 
    \)\)], "Input"],

Cell[BoxData[
    \(\(OperatorApplyK[x_ == y_] := 
      Block[{sch}, \n\t\t
        sch\  = \ {
            Schiebe[i_, "\<+\>"]\^ni_. \ TKI[d_, {n15__}] :> \n\t\t\t\t\t
              TKI[d, ReplacePart[{n15}, {n15}[\([i]\)] + ni, i]], \n\t\t\t\t
            \(Schiebe[i_, "\<-\>"]\^ni_. \ TKI[d_, {n15__}] :> \n\t\t\t\t\t
              TKI[d, ReplacePart[{n15}, {n15}[\([i]\)] - ni, i]]\n\)\t\t\t}; 
        Map[Collect[Expand[#] //. sch, TKI[__], Factor]&, \n\t\t\tx == y]]; 
    \)\)], "Input"],

Cell[BoxData[
    \(\(OperatorApplyB[x_ == y_] := 
      Block[{sch}, \n\t\t
        sch\  = \ {
            Schiebe[i_, "\<+\>"]\^ni_. \ TBI[d_, {n15__}] :> \n\t\t\t\t\t
              TBI[d, ReplacePart[{n15}, {n15}[\([i]\)] + ni, i]], \n\t\t\t\t
            \(Schiebe[i_, "\<-\>"]\^ni_. \ TBI[d_, {n15__}] :> \n\t\t\t\t\t
              TBI[d, ReplacePart[{n15}, {n15}[\([i]\)] - ni, i]]\n\)\t\t\t}; 
        Map[Collect[Expand[#] //. sch, TBI[__], Factor]&, \n\t\t\tx == y]]; 
    \)\)], "Input"],

Cell[BoxData[
    \(\(OperatorApplyA[x_ == y_] := 
      Block[{sch}, \n\t\t
        sch\  = \ {
            Schiebe[i_, "\<+\>"]\^ni_. \ TAI[d_, {n15__}] :> \n\t\t\t\t\t
              TAI[d, ReplacePart[{n15}, {n15}[\([i]\)] + ni, i]], \n\t\t\t\t
            \(Schiebe[i_, "\<-\>"]\^ni_. \ TAI[d_, {n15__}] :> \n\t\t\t\t\t
              TAI[d, ReplacePart[{n15}, {n15}[\([i]\)] - ni, i]]\n\)\t\t\t}; 
        Map[Collect[Expand[#] //. sch, TAI[__], Factor]&, \n\t\t\tx == y]]; 
    \)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MassDerivative", "Subsection",
  CellTags->"T1.3.4.1"],

Cell[BoxData[
    RowBox[{
      RowBox[{
      \(MassDerivative[
          STLI[any__, {\n
              \ \ \ \ \ \ \ {al_, m1_}, {be_, m2_}, {ga_, m3_}, {de_, m4_}, {
                ep_, m5_}}], {{0, d1_}, {0, d2_}, \ {0, d3_}, \ {0, d4_}, 
            \ {0, d5_}}]\), 
        StyleBox[" ",
          FontWeight->"Plain"], 
        StyleBox[":=",
          FontWeight->"Plain"], "\n", 
        \(\((\ \(Pochhammer[al, d1]\ Pochhammer[be, d2]\ Pochhammer[ga, d3]\)*
              \n\ \ Pochhammer[de, d4]\ Pochhammer[ep, d5]\ )\)\ *\n\ 
          STLI[any, {{al + d1, m1}, \ {be + d2, m2}, \ {ga + d3, m3}, 
              \ {de + d4, m4}, \ {ep + d5, m5}}]\)}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["TarasovT", "Subsection",
  CellTags->"T1.3.5.1"],

Cell["\<\
TarasovT[abrs__, qq_ /; Head[qq] =!= List] := 
  TarasovT[abrs, qq, {\"g\", \"g\", \"g\", \"g\", \"g\"}]\
\>", "Input"],

Cell[BoxData[
    \(\(TarasovT[r_, \ s_, \ qq_, \ {alp1_, alp2_, \ alp3_, alp4_, alp5_}]
        \  := \nBlock[{\ al1, al2, al3, al4, al5, alrul, new, Q1, Q11, Q12, 
          Q2, Q22, dum1, dum2}, \n\ \(If[alp1\  === \ 0, \ al1\  = \ 0]; \ 
        If[alp2\  === \ 0, \ al2\  = \ 0]; \n\ 
        If[alp3\  === \ 0, \ al3\  = \ 0]; \n
        If[alp4\  === \ 0, \ al4\  = \ 0]; \ 
        If[alp5\  === \ 0, \ al5\  = \ 0]; \n\ 
        alrul\  = \ 
          Select[{al1 :> I\ ParD[1], \ al2 :> I\ ParD[2], \ 
              al3 :> I\ ParD[3], \n\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ 
              al4 :> I\ ParD[4], \ al5 :> I\ ParD[5]}, \ 
            \(! MatchQ[#, \ 0 :> _]\)&]; \n\ 
        Q1\  = \ al3\ al5\  + \ al4\ al5\  + \ al2\ al3\  + \ al3\ al4; \n\ 
        Q2\  = \ al4\ al5\  + \ al3\ al5\  + \ al1\ al4\  + \ al3\ al4; \n\ 
        Q11\  = 1/\((\(-4\))\)\ \((\ al2\  + \ al4\  + \ al5\ )\); \n\ 
        Q22\  = 1/\((\(-4\))\)\ \((\ al1\  + \ al3\  + \ al5\ )\); \n\ 
        Q12\  = 1/\((\(-2\))\)\ al5; \n\t\t
        new = \n\ 
          I^\((\(-r\) - s)\) 
              \((\ D[\ Exp[\ 
                    I\ qq\ \((\n\ \ \ \ 
                        be1\ Q1\  + \ be2\ Q2\  + be1^2\ Q11\  + 
                          be2^2\ Q22\  + \ be1\ be2\ Q12\n
                        \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ \ \ \ )\)\ rho\ ], {be1, r}, {be2, s}\n\t\t\t\t]\ )\) /. \n\ 
            \((\ Join[alrul, 
                \ {be1\  :> \ 0, be2\  :> \ 0, \ 
                  rho\  :> \ \(-Pi^2\)/Pi^2\ DPlus\ }])\); 
        Map[\((Select[#, \((FreeQ[#, ParD] && FreeQ[#, DPlus])\)&]\ 
                  TpD[qq, 
                    Select[#, 
                      \((\(! \((FreeQ[#, ParD] && FreeQ[#, DPlus])\)\))\)&]])
                \)&, \(\((Expand[new, ParD] + \ dum1\ dum2)\)\n\)\t\t] /. {
            dum1 :> 0, dum2 :> 0}\n\)\t\t]; \)\)], "Input"],

Cell[BoxData[
    \(\(\nTarasovT[r_, \ s_, \ pp_, \ dp_ /; Head[dp] =!= List]\  := \ 
      TarasovT[r, s, pp, dp, {"\<g\>", "\<g\>", "\<g\>", "\<g\>", "\<g\>"}]
        \)\)], "Input"],

Cell[TextData[{
  Cell[BoxData[
      \(TraditionalForm\`\((\[CapitalDelta]\ q\_1)\)\^a\)]],
  Cell[BoxData[
      \(TraditionalForm\`\((\[CapitalDelta]\ q\_2)\)\^b\)]],
  Cell[BoxData[
      \(TraditionalForm\`\((p\ q\_1)\)\^r\)]],
  Cell[BoxData[
      \(TraditionalForm\`\((p\ q\_2)\)\^s\)]]
}], "Text"],

Cell[BoxData[
    \(\(\n\(TarasovT[a_, b_, r_, s_, pp_, 
        dp_, {alp1_, alp2_, alp3_, alp4_, alp5_}] := 
      Block[{al1, al2, al3, al4, al5, alrul, new, Q1, Q11, Q12, Q2, Q22, 
          dum1, dum2}, If[alp1 === 0, al1 = 0]; If[alp2 === 0, al2 = 0]; 
        If[alp3 === 0, al3 = 0]; If[alp4 === 0, al4 = 0]; 
        If[alp5 === 0, al5 = 0]; 
        alrul = Select[{al1 \[RuleDelayed] I\ ParD[1], 
              al2 \[RuleDelayed] I\ ParD[2], al3 \[RuleDelayed] I\ ParD[3], 
              al4 \[RuleDelayed] I\ ParD[4], al5 \[RuleDelayed] I\ ParD[5]}, 
            \(\[InvisibleSpace]\(! 
                \((MatchQ[#1, 0 \[RuleDelayed] _])\)\)\)&]; 
        Q1 = al3\ al5 + al4\ al5 + al2\ al3 + al3\ al4; 
        Q2 = al4\ al5 + al3\ al5 + al1\ al4 + al3\ al4; 
        Q11 = \(-\(1\/4\)\)\ \((al2 + al4 + al5)\); 
        Q22 = \(-\(1\/4\)\)\ \((al1 + al3 + al5)\); Q12 = \(-\(al5\/2\)\); 
        new = I\^\(\(-r\) - s - a - b\)\ 
              \[PartialD]\_\({be1, r}, {be2, s}, {ga1, a}, {ga2, b}\)Exp[
                  I\ \((\((be1\ pp + ga1\ dp)\)\ Q1 + 
                        \((be2\ pp + ga2\ dp)\)\ Q2 + 
                        be1\ \((be1\ pp + 2\ ga1\ dp)\)\ Q11 + 
                        be2\ \((be2\ pp + 2\ ga2\ dp)\)\ Q22 + 
                        \((be1\ be2\ pp + \((be1\ ga2 + be2\ ga1)\)\ dp)\)\ 
                          Q12)\)\ rho] /. 
            Join[alrul, {be1 \[RuleDelayed] 0, be2 \[RuleDelayed] 0, 
                ga1 :> 0, ga2 :> 0, 
                rho \[RuleDelayed] \(-\(\(\[Pi]\^2\ DPlus\)\/\[Pi]\^2\)\)}]; 
        \((Select[#1, FreeQ[#1, ParD] && FreeQ[#1, DPlus]&]\ 
                  TpD[pp, 
                    Select[#1, 
                      \(\[InvisibleSpace]\(! 
                          \((FreeQ[#1, ParD] && FreeQ[#1, DPlus])\)\)\)&]]&)
              \)/@\((Expand[new] + dum1\ dum2)\) /. {dum1 \[RuleDelayed] 0, 
            dum2 \[RuleDelayed] 0}]; \)\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["ApplyTarasovT", "Subsection",
  CellTags->"T1.3.6.1"],

Cell[BoxData[
    \(ApplyTarasovT[tij_, \ exp_] := 
      Block[{dump}, \n\t\tdump = Product[ParD[j]^dummy, {j, 5}]; \n\t\t
        Expand[tij\ exp] /. \n
          \((STLI[de_, pp_, pe__List]\ TpD[tpp_, DPlus^i_. \ t_])\) :> \n\t
            MassDerivative[STLI[\ de /. de :> \((de\  + \ 2\ i)\), pp, pe], \n
              \t\t\t\t\t\t\t
              \(\(t\ dump\  /. Times -> List\) /. Power[_, n_] :> {0, n}\) /. 
                dummy -> 0]]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Misc", "Subsection",
  CellTags->"T1.3.7.1"],

Cell[BoxData[
    \(PreFactor /: \t\tPreFactor[a_]^n_ := PreFactor[a^n]\)], "Input"],

Cell[CellGroupData[{

Cell["Colors", "Subsubsection",
  CellTags->"T1.3.7.1.1"],

Cell[TextData[{
  StyleBox["TFI",
    FontColor->RGBColor[1, 0.6, 0]],
  " ",
  StyleBox["TVI ",
    FontColor->RGBColor[0.2, 0.4, 0.8]],
  StyleBox["TFR ",
    FontColor->RGBColor[0, 0.6, 0]],
  StyleBox["TVR ",
    FontColor->RGBColor[0.2, 0.8, 1]],
  StyleBox["MakeFun ",
    FontColor->RGBColor[1, 0.6, 0.6]],
  StyleBox["Cayley ",
    FontColor->RGBColor[1, 0, 0.4]],
  StyleBox["CayleyD ",
    FontColor->RGBColor[1, 0.2, 1]],
  StyleBox["Cayleyu",
    FontColor->RGBColor[0.8, 0.8, 0.4]],
  StyleBox[" ",
    FontColor->RGBColor[1, 0.2, 1]],
  StyleBox["IFF ",
    FontColor->RGBColor[0.6, 0.4, 1]],
  StyleBox["ToArgs ",
    FontColor->RGBColor[0.2, 0.6, 1]],
  StyleBox["Block",
    FontSize->16,
    FontWeight->"Bold",
    FontColor->RGBColor[1, 0.2, 0.2]]
}], "Text"],

Cell[BoxData[
    \(\(MakeBoxes[Block, StandardForm] := 
      StyleBox["\<Block\>", FontSize \[Rule] 16, 
        FontWeight \[Rule] "\<Bold\>", 
        FontColor \[Rule] RGBColor[1, 0.2, 0.2]]; \)\)], "Input"],

Cell[BoxData[
    \(\(MakeBoxes[Cayley, StandardForm] := 
      StyleBox["\<Cayley\>", FontColor \[Rule] RGBColor[1, 0, 0.4]]; \)\)], 
  "Input"],

Cell[BoxData[
    \(\(MakeBoxes[CayleyD, StandardForm] := 
      StyleBox["\<CayleyD\>", FontColor \[Rule] RGBColor[1, 0.2, 1]]; \)\)], 
  "Input"],

Cell[BoxData[
    \(\(MakeBoxes[Cayleyu, StandardForm] := 
      StyleBox["\<Cayleyu\>", FontColor \[Rule] RGBColor[0.8, 0.8, 0.4]]; 
    \)\)], "Input"],

Cell[BoxData[
    \(\(MakeBoxes[IFF, StandardForm] := 
      StyleBox["\<IFF\>", FontColor \[Rule] RGBColor[0.6, 0.4, 1]]; \)\)], 
  "Input"],

Cell[BoxData[
    \(\(MakeBoxes[MakeFun, StandardForm] := 
      StyleBox["\<MakeFun\>", FontColor \[Rule] RGBColor[1, 0.6, 0.6]]; 
    \)\)], "Input"],

Cell[BoxData[
    \(\(MakeBoxes[TFI, StandardForm] := 
      StyleBox["\<TFI\>", FontColor \[Rule] RGBColor[1, 0.6, 0]]; \)\)], 
  "Input"],

Cell[BoxData[
    \(\(MakeBoxes[TFR, StandardForm] := 
      StyleBox["\<TFR\>", FontColor \[Rule] RGBColor[0, 0.6, 0]]; \)\)], 
  "Input"],

Cell[BoxData[
    \(\(MakeBoxes[ToArgs, StandardForm] := 
      StyleBox["\<ToArgs\>", FontColor \[Rule] RGBColor[0.2, 0.6, 1]]; \)\)], 
  "Input"],

Cell[BoxData[
    \(\(MakeBoxes[TVI, StandardForm] := 
      StyleBox["\<TVI\>", FontColor \[Rule] RGBColor[0.2, 0.4, 0.8]]; \)\)], 
  "Input"],

Cell[BoxData[
    \(MakeBoxes[TVR, StandardForm] := 
      StyleBox["\<TVR\>", FontColor \[Rule] RGBColor[0.2, 0.8, 1]]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["PQ", "Subsubsection",
  CellTags->"T1.3.7.2.1"],

Cell[BoxData[
    \(\(PQ[_Integer?Positive]\  := \ True; \)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["PNQ", "Subsubsection",
  CellTags->"T1.3.7.3.1"],

Cell[BoxData[
    \(\(PNQ[0] = True; \)\)], "Input"],

Cell[BoxData[
    \(\(PNQ[_Integer?Positive]\  := \ True; \)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["FactorC", "Subsubsection",
  CellTags->"T1.3.7.4.1"],

Cell[BoxData[
    \(FactorC[z_] := \(FactorC[z] = Factor[z]\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["TFI", "Subsubsection",
  CellTags->"T1.3.7.5.1"],

Cell[BoxData[
    \(\(TFI[depp__, {a___, b_Integer, c___}] := TFI[depp, {a, {b, 0}, c}]; 
    \)\)], "Input"],

Cell[BoxData[
    \(\(TFI[depp__, {a___, {0, m_ /; m =!= 0}, b___}] := 
      TFI[depp, {a, {0, 0}, b}]; \)\)], "Input"],

Cell[BoxData[
    \(\(TFI[a__, {0, 0}, b__List] := TFI[a, b]; \)\)], "Input"],

Cell[BoxData[
    \(\(TFI[d_, pp_, dp_, {0, 0}, {x1_, x2_, x3_, x4_, x5_}, list_List] := 
      TFI[d, pp, {x1, x2, x3, x4, x5}, list]; \)\)], "Input"],

Cell[BoxData[
    \(\(TFI[d_, pp_, dp_, {a_, b_}, {0, 0, 0, 0, 0}, list_List] := 
      TFI[d, pp, dp, {a, b}, list]; \)\)], "Input"],

Cell[BoxData[
    \(\(TFI[d_, pp_, dp_ /; Head[dp] =!= List, {0, 0, 0, 0, 0}, list_List] := 
      TFI[d, pp, list]; \)\)], "Input"],

Cell[BoxData[
    \(\(TFI[d_, pp_, dp_ /; Head[dp] =!= List, list_List] := 
      TFI[d, pp, list]; \)\)], "Input"],

Cell[BoxData[
    \(\(TFI[d_, pp_, dp_ /; Head[dp] =!= List, {x1_, x2_, x3_, x4_, x5_}, 
        list_List] := TFI[d, pp, {x1, x2, x3, x4, x5}, list]; \)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{_, _}, {0, 0}, {_, _}, {0, 0}, {_, 0}}] := 0; \)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{0, 0}, {_, _}, {0, 0}, {_, _}, {_, 0}}] := 0; \)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{0, 0}, {_, _}, {_, 0}, {_, _}, {0, 0}}] := 0; \)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{_, 0}, {_, _}, {0, 0}, {_, _}, {0, 0}}] := 0; \)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{_, _}, {_, 0}, {_, _}, {0, 0}, {0, 0}}] := 0; \)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{_, _}, {0, 0}, {_, _}, {_, 0}, {0, 0}}] := 0; \)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{0, 0}, {0, 0}, {0, 0}, {_, 0}, {_, _}}] := 0; \)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{0, 0}, {0, 0}, {_, 0}, {0, 0}, {_, _}}] := 0; \)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{0, 0}, {_, 0}, {0, 0}, {0, 0}, {_, _}}] := 0; \)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{_, 0}, {0, 0}, {0, 0}, {0, 0}, {_, _}}] := 0; \)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{_, _}, {_, 0}, {0, 0}, {0, 0}, {0, 0}}] := 0; \)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{_, 0}, {_, _}, {0, 0}, {0, 0}, {0, 0}}] := 0; \)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{_, _}, {0, 0}, {0, 0}, {_, 0}, {0, 0}}] := 0; \)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{_, 0}, {0, 0}, {0, 0}, {_, _}, {0, 0}}] := 0; \)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{0, 0}, {_, 0}, {_, _}, {0, 0}, {0, 0}}] := 0; \)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{0, 0}, {_, _}, {_, 0}, {0, 0}, {0, 0}}] := 0; \)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{0, 0}, {0, 0}, {_, _}, {_, 0}, {0, 0}}] := 0; \)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{0, 0}, {0, 0}, {_, 0}, {_, _}, {0, 0}}] := 0; \)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["TJI", "Subsubsection",
  CellTags->"T1.3.7.6.1"],

Cell[BoxData[
    \(TJI[de_, 
        pp_, {{n1_Integer, m1_}, {n2_Integer, m2_}, {n3_Integer, m3_}}] := 
      \((TJI[de, pp, {{n1, m1}, {n2, m2}, {n3, m3}}] = 
            TJI[de, pp, Reverse[Sort[{{n1, m1}, {n2, m2}, {n3, m3}}]]])\) /; 
        \(\[InvisibleSpace]\(! 
          \((OrderedQ[{{n3, m3}, {n2, m2}, {n1, m1}}])\)\)\)\)], "Input"],

Cell[BoxData[
    \(TJI[d, pp, {{1, m1}, {1, m2}, {1, m3}}]\)], "Input"],

Cell[BoxData[
    \(TJI[d_, 0, {{_, 0}, {_, 0}, {_, 0}}] := 0\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["TBI", "Subsubsection",
  CellTags->"T1.3.7.7.1"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        StyleBox["TBI",
          FontColor->RGBColor[1, 0.6, 0]], "[", \(de_, 
        pp_, {{n1_Integer, m1_}, {n2_Integer, m2_}}\), "]"}], ":=", 
      RowBox[{
        RowBox[{"(", 
          RowBox[{\(TJI[de, pp, {{n1, m1}, {n2, m2}}]\), "=", 
            RowBox[{
              StyleBox["TBI",
                FontColor->RGBColor[1, 0.6, 0]], "[", \(de, pp, 
              Reverse@\(Sort@{{n1, m1}, {n2, m2}}\)\), "]"}]}], ")"}], "/;", 
        "\[InvisibleSpace]", \(! \((OrderedQ[{{n2, m2}, {n1, m1}}])\)\)}]}]], 
  "Input"],

Cell[BoxData[
    \(TBI[d_, 0, {{_, 0}, {_, 0}}] := 0\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["TAI", "Subsubsection",
  CellTags->"T1.3.7.8.1"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          StyleBox["TAI",
            FontColor->RGBColor[1, 0.6, 0]], "[", \(de_, 
          pp_, {{n1_Integer, 0}}\), "]"}], ":=", "0"}], ";"}]], "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Definitions and notation", "Section",
  CellDingbat->"\[FilledSmallCircle]",
  CellTags->"T1.4.1"],

Cell[BoxData[
    RowBox[{
      RowBox[{"\[CapitalDelta]\[ScriptCapitalC]", "=", 
        RowBox[{\(-\(1\/2\)\), 
          RowBox[{"Det", "[", " ", 
            RowBox[{"(", GridBox[{
                  {"0", "1", "1", "1", "1"},
                  {"1", "0", \(\[Mu]\_6\), \(\[Mu]\_4\), \(\[Mu]\_3\)},
                  {"1", \(\[Mu]\_6\), "0", \(\[Mu]\_2\), \(\[Mu]\_1\)},
                  {"1", \(\[Mu]\_4\), \(\[Mu]\_2\), "0", \(\[Mu]\_5\)},
                  {"1", \(\[Mu]\_3\), \(\[Mu]\_1\), \(\[Mu]\_5\), "0"}
                  }], ")"}], "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(\(checkli\  = 
      \ {\n\t\t\[ScriptCapitalC] -> \[CapitalDelta]\[ScriptCapitalC], \n
        \[CapitalDelta]\_i_ :> 
          \[PartialD]\_\(\[Mu]\_i\)\[CapitalDelta]\[ScriptCapitalC], \n\t\t
        \[Mu]\_6 :> PP, \[Mu]\_i_ :> ToExpression["\<m\>"<>ToString[i]]^2, \n
        \t\tm\_1 :> \@m1\^2, m\_2 :> \@m2\^2, m\_3 :> \@m3\^2, 
        m\_4 :> \@m4\^2, m\_5 :> \@m5\^2, \n
        \[CapitalDelta]\_\(i_, j_, k_\) :> 
          \[Mu]\_i\%2 + \[Mu]\_j\%2 + \[Mu]\_k\%2 - 
            2 \((\(\[Mu]\_i\) \[Mu]\_j + \(\[Mu]\_j\) \[Mu]\_k + 
                  \(\[Mu]\_k\) \[Mu]\_i)\), \n
        u\_\(i_, j_, k_\) :> \[Mu]\_i - \[Mu]\_j - \[Mu]\_k, \n
        \[CapitalSigma]\_i_ :> 
          3\ d\  - 2\ \((\[Sum]\_\(j = 1\)\%i \[Nu]\_j)\) - 2, \n\t
        \[Nu]\_1 \[RuleDelayed] n1, \[Nu]\_2 \[RuleDelayed] n2, 
        \[Nu]\_3 \[RuleDelayed] n3, \[Nu]\_4 \[RuleDelayed] n4, 
        \[Nu]\_5 \[RuleDelayed] n5}; \)\)], "Input"],

Cell[TextData[{
  "Define symmetry property for ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta]\_\(i, j, k\)\)]]
}], "Text"],

Cell[BoxData[
    \(\[CapitalDelta]\_\(i_, j_, k_\) := 
      \[CapitalDelta]\_\(Sequence@@Sort[{i, j, k}]\) /; 
        \(! OrderedQ[{i, j, k}]\)\)], "Input"],

Cell[BoxData[
    \(\[CapitalDelta]\_\(3, 1, 2\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    StyleBox["ToArgs",
      FontColor->RGBColor[0.2, 0.6, 1]]], "Subsection",
  CellTags->"T1.4.1.1"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          StyleBox["ToArgs",
            FontColor->RGBColor[0.2, 0.6, 1]], "[", "z_", "]"}], ":=", 
        \(Expand[z //. checkli]\)}], ";"}]], "Input"],

Cell[BoxData[
    \(ToArgs[\[CapitalSigma]\_4] // InputForm\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    StyleBox["Cayley",
      FontColor->RGBColor[1, 0, 0.4]]], "Subsection",
  CellTags->"T1.4.2.1"],

Cell[TextData[{
  "This is (40).  ( p = ",
  Cell[BoxData[
      \(TraditionalForm\`\@p\^2\)]],
  ")"
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"setd", "[", 
        RowBox[{
          RowBox[{"hCayley", "[", 
            RowBox[{
              RowBox[{
                InterpretationBox[\(m\_1\),
                  m1], ":", "_"}], ",", 
              RowBox[{
                InterpretationBox[\(m\_2\),
                  m2], ":", "_"}], ",", 
              RowBox[{
                InterpretationBox[\(m\_3\),
                  m3], ":", "_"}], ",", 
              RowBox[{
                InterpretationBox[\(m\_4\),
                  m4], ":", "_"}], ",", 
              RowBox[{
                InterpretationBox[\(m\_5\),
                  m5], ":", "_"}], ",", "p_"}], "]"}], ",", 
          RowBox[{"set", "[", 
            RowBox[{
              RowBox[{"hCayley", "[", 
                RowBox[{
                  InterpretationBox[\(m\_1\),
                    m1], ",", 
                  InterpretationBox[\(m\_2\),
                    m2], ",", 
                  InterpretationBox[\(m\_3\),
                    m3], ",", 
                  InterpretationBox[\(m\_4\),
                    m4], ",", 
                  InterpretationBox[\(m\_5\),
                    m5], ",", "p"}], "]"}], ",", 
              RowBox[{"factor", "[", 
                RowBox[{
                  RowBox[{
                    StyleBox["ToArgs",
                      FontColor->RGBColor[0.2, 0.6, 1]], "[", 
                    "\[ScriptCapitalC]", "]"}], "/.", 
                  RowBox[{
                    InterpretationBox[\(p\^2\),
                      PP], "\[Rule]", \(p\^2\)}]}], "]"}]}], "]"}]}], "]"}], 
      "/.", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{"hCayley", "\[RuleDelayed]", 
            StyleBox["Cayley",
              FontColor->RGBColor[1, 0, 0.4]]}], ",", 
          \(setd \[RuleDelayed] SetDelayed\), ",", 
          \(set \[RuleDelayed] Set\), ",", 
          \(factor \[RuleDelayed] Factor\)}], "}"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        StyleBox["Cayley",
          FontColor->RGBColor[1, 0, 0.4]], "[", "i", "]"}], 
      "=", \(\[CapitalDelta]\_i\)}]], "Subsection",
  CellTags->"T1.4.3.1"],

Cell[BoxData[
    RowBox[{"Do", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"setd", "[", 
            RowBox[{
              RowBox[{\(hCayley[i]\), "[", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      InterpretationBox[\(m\_1\),
                        m1], ":", "_"}], ",", 
                    RowBox[{
                      InterpretationBox[\(m\_2\),
                        m2], ":", "_"}], ",", 
                    RowBox[{
                      InterpretationBox[\(m\_3\),
                        m3], ":", "_"}], ",", 
                    RowBox[{
                      InterpretationBox[\(m\_4\),
                        m4], ":", "_"}], ",", 
                    RowBox[{
                      InterpretationBox[\(m\_5\),
                        m5], ":", "_"}], ",", "p_"}], "}"}], "]"}], ",", 
              RowBox[{"set", "[", 
                RowBox[{
                  RowBox[{\(hCayley[i]\), "[", 
                    RowBox[{"{", 
                      RowBox[{
                        InterpretationBox[\(m\_1\),
                          m1], ",", 
                        InterpretationBox[\(m\_2\),
                          m2], ",", 
                        InterpretationBox[\(m\_3\),
                          m3], ",", 
                        InterpretationBox[\(m\_4\),
                          m4], ",", 
                        InterpretationBox[\(m\_5\),
                          m5], ",", "p"}], "}"}], "]"}], ",", 
                  RowBox[{"factor", "[", 
                    RowBox[{
                      RowBox[{
                        StyleBox["ToArgs",
                          FontColor->RGBColor[0.2, 0.6, 1]], "[", 
                        \(\[CapitalDelta]\_i\), "]"}], "/.", 
                      RowBox[{
                        InterpretationBox[\(p\^2\),
                          PP], "\[Rule]", \(p\^2\)}]}], "]"}]}], "]"}]}], 
            "]"}], "/.", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"hCayley", "\[RuleDelayed]", 
                StyleBox["Cayley",
                  FontColor->RGBColor[1, 0, 0.4]]}], ",", 
              \(setd \[RuleDelayed] SetDelayed\), ",", 
              \(set \[RuleDelayed] Set\), ",", 
              \(factor \[RuleDelayed] Factor\)}], "}"}]}], ",", \({i, 6}\)}], 
      "]"}]], "Input"],

Cell[BoxData[
    RowBox[{
      StyleBox["ToArgs",
        FontColor->RGBColor[0.2, 0.6, 1]], "[", 
      \(\[CapitalDelta]\_\(1, 2, 6\)\), "]"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{
            StyleBox["Cayley",
              FontColor->RGBColor[1, 0, 0.4]], "[", \(i_, j_, k_\), "]"}], 
          "[", "m_List", "]"}], ":=", 
        RowBox[{
          RowBox[{
            RowBox[{
              StyleBox["Cayley",
                FontColor->RGBColor[1, 0, 0.4]], "[", \(i, j, k\), "]"}], 
            "[", "m", "]"}], "=", 
          \(Block[{mm = m}, \n\t\t\t
            While[Length[mm] < 7, AppendTo[mm, Last[mm]]]; \n\t\t\t
            FactorC[mm\[LeftDoubleBracket]i\[RightDoubleBracket]\^4 - 
                2\ mm\[LeftDoubleBracket]j\[RightDoubleBracket]\^2\ 
                  mm\[LeftDoubleBracket]i\[RightDoubleBracket]\^2 - 
                2\ mm\[LeftDoubleBracket]k\[RightDoubleBracket]\^2\ 
                  mm\[LeftDoubleBracket]i\[RightDoubleBracket]\^2 + 
                mm\[LeftDoubleBracket]j\[RightDoubleBracket]\^4 + 
                mm\[LeftDoubleBracket]k\[RightDoubleBracket]\^4 - 
                2\ mm\[LeftDoubleBracket]j\[RightDoubleBracket]\^2\ 
                  mm\[LeftDoubleBracket]k\[RightDoubleBracket]\^2]]\)}]}], 
      ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{
            StyleBox["Cayleyu",
              FontColor->RGBColor[0.8, 0.8, 0.4]], "[", \(i_, j_, k_\), 
            "]"}], "[", "m_List", "]"}], ":=", 
        RowBox[{
          RowBox[{
            RowBox[{
              StyleBox["Cayleyu",
                FontColor->RGBColor[0.8, 0.8, 0.4]], "[", \(i, j, k\), "]"}], 
            "[", "mm", "]"}], "=", 
          \(Block[{mm = m}, \n\t\t\t\t\t
            While[Length[mm] < 7, AppendTo[mm, Last[mm]]]; 
            FactorC[mm\[LeftDoubleBracket]i\[RightDoubleBracket]\^2 - 
                mm\[LeftDoubleBracket]j\[RightDoubleBracket]\^2 - 
                mm\[LeftDoubleBracket]k\[RightDoubleBracket]\^2]]\)}]}], 
      ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        StyleBox["CayleyD",
          FontColor->RGBColor[1, 0.2, 1]], "[", "i", "]"}], 
      "=", \(\[CapitalDelta]\_i\/\[CapitalDelta]\)}]], "Subsection",
  CellTags->"T1.4.4.1"],

Cell[BoxData[
    RowBox[{"Do", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"setd", "[", 
            RowBox[{
              RowBox[{\(hCayleyD[i]\), "[", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      InterpretationBox[\(m\_1\),
                        m1], ":", "_"}], ",", 
                    RowBox[{
                      InterpretationBox[\(m\_2\),
                        m2], ":", "_"}], ",", 
                    RowBox[{
                      InterpretationBox[\(m\_3\),
                        m3], ":", "_"}], ",", 
                    RowBox[{
                      InterpretationBox[\(m\_4\),
                        m4], ":", "_"}], ",", 
                    RowBox[{
                      InterpretationBox[\(m\_5\),
                        m5], ":", "_"}], ",", "p_"}], "}"}], "]"}], ",", 
              RowBox[{"set", "[", 
                RowBox[{
                  RowBox[{\(hCayleyD[i]\), "[", 
                    RowBox[{"{", 
                      RowBox[{
                        InterpretationBox[\(m\_1\),
                          m1], ",", 
                        InterpretationBox[\(m\_2\),
                          m2], ",", 
                        InterpretationBox[\(m\_3\),
                          m3], ",", 
                        InterpretationBox[\(m\_4\),
                          m4], ",", 
                        InterpretationBox[\(m\_5\),
                          m5], ",", "p"}], "}"}], "]"}], ",", 
                  RowBox[{"factor", "[", 
                    FractionBox[
                      RowBox[{
                        RowBox[{
                          StyleBox["ToArgs",
                            FontColor->RGBColor[0.2, 0.6, 1]], "[", 
                          \(\[CapitalDelta]\_i\), "]"}], "/.", 
                        RowBox[{
                          InterpretationBox[\(p\^2\),
                            PP], "\[Rule]", \(p\^2\)}]}], 
                      RowBox[{"\[ScriptCapitalC]", "[", 
                        RowBox[{
                          InterpretationBox[\(m\_1\),
                            m1], ",", 
                          InterpretationBox[\(m\_2\),
                            m2], ",", 
                          InterpretationBox[\(m\_3\),
                            m3], ",", 
                          InterpretationBox[\(m\_4\),
                            m4], ",", 
                          InterpretationBox[\(m\_5\),
                            m5], ",", "p"}], "]"}]], "]"}]}], "]"}]}], "]"}], 
          "/.", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"hCayleyD", "\[RuleDelayed]", 
                StyleBox["CayleyD",
                  FontColor->RGBColor[1, 0.2, 1]]}], ",", 
              \(setd \[RuleDelayed] SetDelayed\), ",", 
              \(set \[RuleDelayed] Set\), ",", 
              \(factor \[RuleDelayed] Factor\), ",", 
              RowBox[{"\[ScriptCapitalC]", "\[RuleDelayed]", 
                StyleBox["Cayley",
                  FontColor->RGBColor[1, 0, 0.4]]}]}], "}"}]}], ",", 
        \({i, 6}\)}], "]"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"setd", "[", 
        RowBox[{
          RowBox[{\(hCayleyD[1, 2, 5]\), "[", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{
                  InterpretationBox[\(m\_1\),
                    m1], ":", "_"}], ",", 
                RowBox[{
                  InterpretationBox[\(m\_2\),
                    m2], ":", "_"}], ",", 
                RowBox[{
                  InterpretationBox[\(m\_3\),
                    m3], ":", "_"}], ",", 
                RowBox[{
                  InterpretationBox[\(m\_4\),
                    m4], ":", "_"}], ",", 
                RowBox[{
                  InterpretationBox[\(m\_5\),
                    m5], ":", "_"}], ",", "p_"}], "}"}], "]"}], ",", 
          RowBox[{"set", "[", 
            RowBox[{
              RowBox[{\(hCayleyD[1, 2, 5]\), "[", 
                RowBox[{"{", 
                  RowBox[{
                    InterpretationBox[\(m\_1\),
                      m1], ",", 
                    InterpretationBox[\(m\_2\),
                      m2], ",", 
                    InterpretationBox[\(m\_3\),
                      m3], ",", 
                    InterpretationBox[\(m\_4\),
                      m4], ",", 
                    InterpretationBox[\(m\_5\),
                      m5], ",", "p"}], "}"}], "]"}], ",", 
              RowBox[{"factor", "[", 
                FractionBox[
                  RowBox[{
                    RowBox[{
                      StyleBox["ToArgs",
                        FontColor->RGBColor[0.2, 0.6, 1]], "[", 
                      \(\[CapitalDelta]\_\(1, 2, 5\)\), "]"}], "/.", 
                    RowBox[{
                      InterpretationBox[\(p\^2\),
                        PP], "\[Rule]", \(p\^2\)}]}], 
                  RowBox[{"\[ScriptCapitalC]", "[", 
                    RowBox[{
                      InterpretationBox[\(m\_1\),
                        m1], ",", 
                      InterpretationBox[\(m\_2\),
                        m2], ",", 
                      InterpretationBox[\(m\_3\),
                        m3], ",", 
                      InterpretationBox[\(m\_4\),
                        m4], ",", 
                      InterpretationBox[\(m\_5\),
                        m5], ",", "p"}], "]"}]], "]"}]}], "]"}]}], "]"}], 
      "/.", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{"hCayleyD", "\[RuleDelayed]", 
            StyleBox["CayleyD",
              FontColor->RGBColor[1, 0.2, 1]]}], ",", 
          \(setd \[RuleDelayed] SetDelayed\), ",", 
          \(set \[RuleDelayed] Set\), ",", \(factor \[RuleDelayed] Factor\), 
          ",", 
          RowBox[{"\[ScriptCapitalC]", "\[RuleDelayed]", 
            StyleBox["Cayley",
              FontColor->RGBColor[1, 0, 0.4]]}]}], "}"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"setd", "[", 
        RowBox[{
          RowBox[{\(hCayleyD[1, 3, 6]\), "[", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{
                  InterpretationBox[\(m\_1\),
                    m1], ":", "_"}], ",", 
                RowBox[{
                  InterpretationBox[\(m\_2\),
                    m2], ":", "_"}], ",", 
                RowBox[{
                  InterpretationBox[\(m\_3\),
                    m3], ":", "_"}], ",", 
                RowBox[{
                  InterpretationBox[\(m\_4\),
                    m4], ":", "_"}], ",", 
                RowBox[{
                  InterpretationBox[\(m\_5\),
                    m5], ":", "_"}], ",", "p_"}], "}"}], "]"}], ",", 
          RowBox[{"set", "[", 
            RowBox[{
              RowBox[{\(hCayleyD[1, 3, 6]\), "[", 
                RowBox[{"{", 
                  RowBox[{
                    InterpretationBox[\(m\_1\),
                      m1], ",", 
                    InterpretationBox[\(m\_2\),
                      m2], ",", 
                    InterpretationBox[\(m\_3\),
                      m3], ",", 
                    InterpretationBox[\(m\_4\),
                      m4], ",", 
                    InterpretationBox[\(m\_5\),
                      m5], ",", "p"}], "}"}], "]"}], ",", 
              RowBox[{"factor", "[", 
                FractionBox[
                  RowBox[{
                    RowBox[{
                      StyleBox["ToArgs",
                        FontColor->RGBColor[0.2, 0.6, 1]], "[", 
                      \(\[CapitalDelta]\_\(1, 3, 6\)\), "]"}], "/.", 
                    RowBox[{
                      InterpretationBox[\(p\^2\),
                        PP], "\[Rule]", \(p\^2\)}]}], 
                  RowBox[{"\[ScriptCapitalC]", "[", 
                    RowBox[{
                      InterpretationBox[\(m\_1\),
                        m1], ",", 
                      InterpretationBox[\(m\_2\),
                        m2], ",", 
                      InterpretationBox[\(m\_3\),
                        m3], ",", 
                      InterpretationBox[\(m\_4\),
                        m4], ",", 
                      InterpretationBox[\(m\_5\),
                        m5], ",", "p"}], "]"}]], "]"}]}], "]"}]}], "]"}], 
      "/.", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{"hCayleyD", "\[RuleDelayed]", 
            StyleBox["CayleyD",
              FontColor->RGBColor[1, 0.2, 1]]}], ",", 
          \(setd \[RuleDelayed] SetDelayed\), ",", 
          \(set \[RuleDelayed] Set\), ",", \(factor \[RuleDelayed] Factor\), 
          ",", 
          RowBox[{"\[ScriptCapitalC]", "\[RuleDelayed]", 
            StyleBox["Cayley",
              FontColor->RGBColor[1, 0, 0.4]]}]}], "}"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"setd", "[", 
        RowBox[{
          RowBox[{\(hCayleyD[3, 4, 5]\), "[", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{
                  InterpretationBox[\(m\_1\),
                    m1], ":", "_"}], ",", 
                RowBox[{
                  InterpretationBox[\(m\_2\),
                    m2], ":", "_"}], ",", 
                RowBox[{
                  InterpretationBox[\(m\_3\),
                    m3], ":", "_"}], ",", 
                RowBox[{
                  InterpretationBox[\(m\_4\),
                    m4], ":", "_"}], ",", 
                RowBox[{
                  InterpretationBox[\(m\_5\),
                    m5], ":", "_"}], ",", "p_"}], "}"}], "]"}], ",", 
          RowBox[{"set", "[", 
            RowBox[{
              RowBox[{\(hCayleyD[3, 4, 5]\), "[", 
                RowBox[{"{", 
                  RowBox[{
                    InterpretationBox[\(m\_1\),
                      m1], ",", 
                    InterpretationBox[\(m\_2\),
                      m2], ",", 
                    InterpretationBox[\(m\_3\),
                      m3], ",", 
                    InterpretationBox[\(m\_4\),
                      m4], ",", 
                    InterpretationBox[\(m\_5\),
                      m5], ",", "p"}], "}"}], "]"}], ",", 
              RowBox[{"factor", "[", 
                FractionBox[
                  RowBox[{
                    RowBox[{
                      StyleBox["ToArgs",
                        FontColor->RGBColor[0.2, 0.6, 1]], "[", 
                      \(\[CapitalDelta]\_\(3, 4, 5\)\), "]"}], "/.", 
                    RowBox[{
                      InterpretationBox[\(p\^2\),
                        PP], "\[Rule]", \(p\^2\)}]}], 
                  RowBox[{"\[ScriptCapitalC]", "[", 
                    RowBox[{
                      InterpretationBox[\(m\_1\),
                        m1], ",", 
                      InterpretationBox[\(m\_2\),
                        m2], ",", 
                      InterpretationBox[\(m\_3\),
                        m3], ",", 
                      InterpretationBox[\(m\_4\),
                        m4], ",", 
                      InterpretationBox[\(m\_5\),
                        m5], ",", "p"}], "]"}]], "]"}]}], "]"}]}], "]"}], 
      "/.", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{"hCayleyD", "\[RuleDelayed]", 
            StyleBox["CayleyD",
              FontColor->RGBColor[1, 0.2, 1]]}], ",", 
          \(setd \[RuleDelayed] SetDelayed\), ",", 
          \(set \[RuleDelayed] Set\), ",", \(factor \[RuleDelayed] Factor\), 
          ",", 
          RowBox[{"\[ScriptCapitalC]", "\[RuleDelayed]", 
            StyleBox["Cayley",
              FontColor->RGBColor[1, 0, 0.4]]}]}], "}"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      StyleBox["ToArgs",
        FontColor->RGBColor[0.2, 0.6, 1]], "[", 
      \(\[CapitalDelta]\_\(i, j, k\)\), "]"}]], "Input"],

Cell[BoxData[
    RowBox[{
      StyleBox["ToArgs",
        FontColor->RGBColor[0.2, 0.6, 1]], "[", 
      \(\[CapitalDelta]\_\(1, 2, 6\)\), "]"}]], "Input"],

Cell[BoxData[
    RowBox[{
      StyleBox["ToArgs",
        FontColor->RGBColor[0.2, 0.6, 1]], "[", \(u\_\(i, j, k\)\), "]"}]], 
  "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Utility Functions", "Section",
  CellDingbat->"\[FilledSmallCircle]",
  CellTags->"T1.5.1"],

Cell[CellGroupData[{

Cell["MakeFun and friends", "Subsection",
  CellTags->"T1.5.1.1"],

Cell["\<\
This function generates the recursion function definitions for 
TARCER starting from the entered relations.\
\>", "Text"],

Cell["The idea is to recurse TFR into TFI's step by step.", "Text"],

Cell[TextData[{
  "Define: S cD[ijk__] := ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[ScriptCapitalC]\^\(-1\)\) \
\[CapitalDelta]\_ijk\)]]
}], "Text"],

Cell[BoxData[
    \(\(nutonand\[CapitalDelta]u[w_, 5] := 
      \(w /. {\[Nu]\_1 \[RuleDelayed] n1, \[Nu]\_2 \[RuleDelayed] n2, 
            \[Nu]\_3 \[RuleDelayed] n3, \[Nu]\_4 \[RuleDelayed] n4, 
            \[Nu]\_5 \[RuleDelayed] n5, 
            \[CapitalDelta]\_ijk__ \[RuleDelayed] \[ScriptCapitalC]\ cD[ijk], 
            u\_ijk__ \[RuleDelayed] cu[ijk], \n\t\tD\_ijk__ :> TA[D, ijk], 
            \[Rho]\_ijk__ :> TA[\[Rho], ijk], 
            \[Phi]\_\(ijk__\ \) :> TA[\[Phi], ijk], \n\t\t\t
            \[Sigma]\_\(i_, j_, k_\) :> 
              TA[\[Sigma], d, i, j, k, \[Nu]\_i, \[Nu]\_j, \[Nu]\_k], 
            h\_\(i_, j_, k_\) :> 
              TA[h, d, i, j, k, \[Nu]\_i, \[Nu]\_j, \[Nu]\_k], 
            S\_\(i_, j_, k_\) :> 
              TA[S, d, i, j, k, \[Nu]\_i, \[Nu]\_j, \[Nu]\_k], \n\t\t\t
            \(\[CapitalSigma]\_i_ :> ToArgs[\[CapitalSigma]\_i]\n\)\t\t}
            \) /. {\t\[Nu]\_1 \[RuleDelayed] n1, \[Nu]\_2 \[RuleDelayed] n2, 
          \[Nu]\_3 \[RuleDelayed] n3, \[Nu]\_4 \[RuleDelayed] n4, 
          \[Nu]\_5 \[RuleDelayed] n5, m\_1 :> m1, m\_2 :> m2, m\_3 :> m3, 
          m\_4 :> m4, m\_5 :> m5}; \)\)], "Input"],

Cell[BoxData[
    \(\(nutonand\[CapitalDelta]u[w_] := 
      \(w /. {\[Nu]\_1 \[RuleDelayed] n1, \[Nu]\_2 \[RuleDelayed] n2, 
            \[Nu]\_3 \[RuleDelayed] n3, \[Nu]\_4 \[RuleDelayed] n4, 
            \[Nu]\_5 \[RuleDelayed] n5, 
            \[CapitalDelta]\_ijk__ \[RuleDelayed] cD[ijk], 
            u\_ijk__ \[RuleDelayed] cu[ijk], 
            D\_ijk__ \[RuleDelayed] TA[D, ijk], 
            \[Rho]\_ijk__ \[RuleDelayed] TA[\[Rho], ijk], 
            \[Phi]\_ijk__ \[RuleDelayed] TA[\[Phi], ijk], 
            \[Sigma]\_\(i_, j_, k_\) \[RuleDelayed] 
              TA[\[Sigma], d, i, j, k, \[Nu]\_i, \[Nu]\_j, \[Nu]\_k], 
            h\_\(i_, j_, k_\) \[RuleDelayed] 
              TA[h, d, i, j, k, \[Nu]\_i, \[Nu]\_j, \[Nu]\_k], 
            S\_\(i_, j_, k_\) \[RuleDelayed] 
              TA[S, d, i, j, k, \[Nu]\_i, \[Nu]\_j, \[Nu]\_k], 
            \[CapitalSigma]\_i_ \[RuleDelayed] ToArgs[\[CapitalSigma]\_i]}
            \) /. {\[Nu]\_1 \[RuleDelayed] n1, \[Nu]\_2 \[RuleDelayed] n2, 
          \[Nu]\_3 \[RuleDelayed] n3, \[Nu]\_4 \[RuleDelayed] n4, 
          \[Nu]\_5 \[RuleDelayed] n5, m\_1 \[RuleDelayed] m1, 
          m\_2 \[RuleDelayed] m2, m\_3 \[RuleDelayed] m3, 
          m\_4 \[RuleDelayed] m4, m\_5 \[RuleDelayed] m5}; \)\)], "Input"],

Cell[BoxData[
    \(\(addPeP[TFR[d_, pp_, li1_, rest_]] := TFR[d, pp, li1, rest]; \)\)], 
  "Input"],

Cell[BoxData[
    \(\(addPeP[TFI[d_, pp_, li1_List]] := TFI[d, pp, li1]; \)\)], "Input"],

Cell[BoxData[
    \(\(addPeP[TFR[d_, pp_, dp_Symbol, li1_, rest_]] := 
      TFR[d, pp, dp, li1, rest]; \)\)], "Input"],

Cell[BoxData[
    \(\(addPeP[TFI[d_, pp_, dp_Symbol, li1_List]] := TFI[d, pp, dp, li1]; 
    \)\)], "Input"],

Cell[BoxData[
    \(\(addPeP[TVI[d_, pp_, li1_List]] := TVI[d, pp, li1]; \)\)], "Input"],

Cell[BoxData[
    \(nnnnnaddPeP[any_[d_, pp_, li1_List, rest__List]] := 
      any[d, pp, li1, rest] /; any =!= List\)], "Input"],

Cell[BoxData[
    \(\(addPeP[z_List] := Append[z, \@PP]; \)\)], "Input"],

Cell["addPeP[TFI[d_, z_List]] := TFI[d, PP, z]; ", "Input"],

Cell[BoxData[
    \(\(addPeP[TVI[d_, z_List]] := TVI[d, PP, z]; \)\)], "Input"],

Cell[BoxData[
    \(\(addPeP[TJI[d_, z_List]] := TJI[d, PP, z]; \)\)], "Input"],

Cell[BoxData[
    \(\(addPeP[TFR[d_, z_List]] := TFR[d, PP_, z]; \)\)], "Input"],

Cell[BoxData[
    \(\(addPeP[TFR[d_, pl_List, z_List]] := TFR[d, PP_, pl, z]; \)\)], "Input"],

Cell[BoxData[
    \(\(addPeP[TVR[d_, z__List]] := TVR[d, PP_, z]; \)\)], "Input"],

Cell[BoxData[
    \(\(addPeP[TJR[d_, z__List]] := TJR[d, PP_, z]; \)\)], "Input"],

Cell[BoxData[
    \(\(nutomass[w_] := 
      \(\(w /. {\[Nu]\_1 + i1_.  \[RuleDelayed] {n1 + i1, m1}, 
              \[Nu]\_2 + i2_.  \[RuleDelayed] {n2 + i2, m2}, 
              \[Nu]\_3 + i3_.  \[RuleDelayed] {n3 + i3, m3}, 
              \[Nu]\_4 + i4_.  \[RuleDelayed] {n4 + i4, m4}, 
              \[Nu]\_5 + i5_.  \[RuleDelayed] {n5 + i5, m5}}\) /. {
              in1_Integer, in2_Integer, in3_Integer, in4_Integer} 
            \[RuleDelayed] {{in1, m1}, {in2, m2}, {in3, m3}, {in4, m4}}\) /. {
            in1_Integer, in2_Integer, in3_Integer} 
          \[RuleDelayed] {{in1, m1}, {in2, m2}, {in3, m3}}; \)\)], "Input"],

Cell[BoxData[
    \(\(Clear[maybeF]; \)\)], "Input"],

Cell[BoxData[
    \(\(maybeF[z_Times] := 
      FactorC[z] /; 
        FreeQ[z, TFI] && FreeQ[z, TVI] && FreeQ[z, TJI] && 
          Length[Cases[z, cD[__], \(-1\)]] > 1; \)\)], "Input"],

Cell[BoxData[
    \(maybeF[z_] := z; SetAttributes[top, Listable]; \)], "Input"],

Cell["This is the function for the information of side conditions.", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetAttributes", "[", 
        RowBox[{
          StyleBox["IFF",
            FontColor->RGBColor[0.6, 0.4, 1]], ",", "HoldAll"}], "]"}], 
      ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", 
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "]"}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          StyleBox["MakeFun",
            FontColor->RGBColor[1, 0.6, 0.6]], "[", 
          RowBox[{
            RowBox[{
              RowBox[{"c_.", " ", 
                RowBox[{
                  StyleBox["TFI",
                    FontColor->RGBColor[1, 0.6, 0]], "[", "t__", "]"}]}], 
              "==", "rhs_"}], ",", "eqn_String", ",", 
            RowBox[{
              StyleBox["IFF",
                FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds_", "]"}], ",", 
            \(mrel_ : {}\)}], "]"}], ":=", 
        RowBox[{
          StyleBox["Block",
            CellFrame->True,
            FontSize->16,
            FontWeight->"Bold",
            FontColor->RGBColor[1, 0.2, 0.2]], "[", 
          RowBox[{\({nm}\), ",", 
            RowBox[{
            \(nmrel = Select[Flatten[{mrel}], FreeQ[#1, Factor]&]\), ";", 
              RowBox[{
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        RowBox[{
                          RowBox[{
                            RowBox[{
                              RowBox[{
                                RowBox[{
                                  RowBox[{
                                    RowBox[{"HoldForm", "@@", 
                                      RowBox[{"{", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"setdel", "[", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"addPeP", "[", 
                                        RowBox[{
                                        StyleBox["TFR",
                                        FontColor->RGBColor[0, 0.6, 0]], 
                                        "@@", \(top/@nutomass[{t}]\)}], 
                                        "]"}], "/.", "nmrel"}], ",", 
                                        RowBox[{\(Hold[Condition]\), "[", 
                                        RowBox[{
                                        RowBox[{
                                        \(Hold[CompoundExpression]\), "[", 
                                        "\t", 
                                        RowBox[{
                                        \(\(Hold[TComment]\)[eqn, 
                                        \(\(\(addPeP[nutomass[TFR[t]]] /. 
                                        TFR -> TFIC\) /. {Pattern :> pat, 
                                        Blank :> blank}\) /. pat[PP, 
                                        blank[]] :> PP\) /. nmrel]\), ",", 
                                        RowBox[{
                                        RowBox[{"Collect", "[", 
                                        RowBox[{
                                        RowBox[{"TT", "=", 
                                        RowBox[{"Expand3", "[", 
                                        RowBox[{
                                        "nutonand\[CapitalDelta]u", "[", 
                                        RowBox[{
                                        RowBox[{\(1\/c\), 
                                        RowBox[{"(", 
                                        RowBox[{"rhs", "/.", 
                                        RowBox[{
                                        RowBox[{
                                        StyleBox["TFI",
                                        FontColor->RGBColor[1, 0.6, 0]], "[", 
                                        "te__", "]"}], "\[RuleDelayed]", 
                                        RowBox[{"addPeP", "[", 
                                        RowBox[{"nutomass", "[", 
                                        RowBox[{
                                        StyleBox["TFI",
                                        FontColor->RGBColor[1, 0.6, 0]], "[", 
                                        "te", "]"}], "]"}], "]"}]}]}], 
                                        ")"}]}], ",", "5"}], "]"}], "]"}]}], 
                                        ",", 
                                        RowBox[{
                                        StyleBox["TFI",
                                        FontColor->RGBColor[1, 0.6, 0]], "[", 
                                        "__", "]"}], ",", 
                                        \(maybeF[\((Collect[Numerator[#1], 
                                        cD[__], 
                                        Factor]/Collect[Denominator[#1], 
                                        cD[__], Factor]&)\)/@Factor[#1]]&
                                        \)}], "]"}], "/.", "nmrel"}]}], 
                                        "]"}], ",", 
                                        RowBox[{
                                        "nutonand\[CapitalDelta]u", "[", 
                                        RowBox[{
                                        RowBox[{
                                        StyleBox["IFF",
                                        FontColor->RGBColor[0.6, 0.4, 1]], 
                                        "[", "conds", "]"}], ",", "5"}], 
                                        "]"}]}], "]"}]}], "]"}], "/.", 
                                        \(top[i_Integer] \[Rule] i\)}], "/.", 
                                        \(top \[RuleDelayed] 
                                        \((patternTest[pattern[#1, blank[]], 
                                        PQ]&)\)\)}], "/.", 
                                        RowBox[{
                                        RowBox[{"patternTest", "[", 
                                        RowBox[{
                                        RowBox[{"pattern", "[", 
                                        RowBox[{
                                        RowBox[{"dm_", "/;", 
                                        RowBox[{"MemberQ", "[", 
                                        RowBox[{
                                        RowBox[{"{", 
                                        RowBox[{"d", ",", 
                                        InterpretationBox[\(p\^2\),
                                        PP], ",", "DP", ",", 
                                        InterpretationBox[\(m\_1\),
                                        m1], ",", 
                                        InterpretationBox[\(m\_2\),
                                        m2], ",", 
                                        InterpretationBox[\(m\_3\),
                                        m3], ",", 
                                        InterpretationBox[\(m\_4\),
                                        m4], ",", 
                                        InterpretationBox[\(m\_5\),
                                        m5]}], "}"}], ",", "dm"}], "]"}]}], 
                                        ",", \(blank[]\)}], "]"}], ",", 
                                        "_"}], "]"}], "\[RuleDelayed]", 
                                        \(pattern[dm, blank[]]\)}]}], "/.", 
                                        "\n", "\t\t\t\t\t", 
                                        \({blank \[RuleDelayed] Blank, 
                                        pattern \[RuleDelayed] Pattern, 
                                        patternTest \[RuleDelayed] 
                                        PatternTest}\)}], "}"}]}], "/.", 
                                    RowBox[{\(cD[ijk__]\), "\[Rule]", 
                                      RowBox[{
                                        RowBox[{"Hold", "[", 
                                        RowBox[{
                                        StyleBox["CayleyD",
                                        FontColor->RGBColor[1, 0.2, 1]], "[", 
                                        "ijk", "]"}], "]"}], "[", 
                                        \(addPeP[
                                        BLA = 
                                        \(nutomass[Last[{t}]] /. nmrel
                                        \) /. {nuu_, ma_} \[RuleDelayed] ma]
                                        \), "]"}]}]}], "/.", 
                                  \(addPeP -> Identity\)}], "/.", 
                                \(Hold[CompoundExpression] \[Rule] 
                                  CompoundExpression\)}], "/.", 
                              RowBox[{
                                RowBox[{\(Hold[Condition]\), "[", 
                                  RowBox[{"a_", ",", 
                                    RowBox[{
                                      StyleBox["IFF",
                                        FontColor->RGBColor[0.6, 0.4, 1]], 
                                      "[", "b_", "]"}]}], "]"}], "\[Rule]", 
                                \(a /; b\)}]}], "/.", 
                            \(Hold[Set] \[Rule] Set\)}], "/.", 
                          \(Hold[TComment] -> TComment\)}], "/.", 
                        RowBox[{
                          RowBox[{"Hold", "[", 
                            RowBox[{
                              StyleBox["CayleyD",
                                FontColor->RGBColor[1, 0.2, 1]], "[", 
                              "ijk__", "]"}], "]"}], "\[Rule]", 
                          RowBox[{
                            StyleBox["CayleyD",
                              FontColor->RGBColor[1, 0.2, 1]], "[", "ijk", 
                            "]"}]}]}], "/.", 
                      \({setdel \[RuleDelayed] SetDelayed}\)}], ")"}], "/.", 
                  \(Identity -> identit\)}], "/.", 
                \(identit[a_] -> a\)}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          StyleBox["MakeFun",
            FontColor->RGBColor[1, 0.6, 0.6]], "[", 
          RowBox[{
            RowBox[{
              RowBox[{"c_.", " ", 
                RowBox[{
                  StyleBox["TFI",
                    FontColor->RGBColor[1, 0.6, 0]], "[", "t__", "]"}]}], 
              "==", "rhs_"}], ",", "eqn_String", ",", \(mrel_ : {}\)}], 
          "]"}], ":=", 
        RowBox[{
          RowBox[{
            StyleBox["Block",
              CellFrame->True,
              FontSize->16,
              FontWeight->"Bold",
              FontColor->RGBColor[1, 0.2, 0.2]], "[", 
            RowBox[{\({nm}\), ",", 
              RowBox[{
              \(nmrel = Select[Flatten[{mrel}], FreeQ[#1, Factor]&]\), ";", 
                RowBox[{
                  RowBox[{
                    RowBox[{
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{
                            RowBox[{
                              RowBox[{
                                RowBox[{
                                  RowBox[{
                                    RowBox[{
                                      RowBox[{"HoldForm", "@@", 
                                        RowBox[{"{", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"setdel", "[", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"addPeP", "[", 
                                        RowBox[{
                                        StyleBox["TFR",
                                        FontColor->RGBColor[0, 0.6, 0]], 
                                        "@@", \(top/@nutomass[{t}]\)}], 
                                        "]"}], "/.", "nmrel"}], ",", 
                                        RowBox[{
                                        \(Hold[CompoundExpression]\), "[", 
                                        RowBox[{
                                        \(\(Hold[TComment]\)[eqn, 
                                        \(\(\(addPeP[nutomass[TFR[t]]] /. 
                                        TFR -> TFIC\) /. {Pattern :> pat, 
                                        Blank :> blank}\) /. pat[PP, 
                                        blank[]] :> PP\) /. nmrel]\), ",", 
                                        RowBox[{
                                        RowBox[{"Collect", "[", 
                                        RowBox[{
                                        RowBox[{"TT", "=", 
                                        RowBox[{"Expand3", "[", 
                                        RowBox[{
                                        "nutonand\[CapitalDelta]u", "[", 
                                        RowBox[{\(1\/c\), 
                                        RowBox[{"(", 
                                        RowBox[{"rhs", "/.", 
                                        RowBox[{
                                        RowBox[{
                                        StyleBox["TFI",
                                        FontColor->RGBColor[1, 0.6, 0]], "[", 
                                        "te__", "]"}], "\[RuleDelayed]", 
                                        RowBox[{"addPeP", "[", 
                                        RowBox[{"nutomass", "[", 
                                        RowBox[{
                                        StyleBox["TFI",
                                        FontColor->RGBColor[1, 0.6, 0]], "[", 
                                        "te", "]"}], "]"}], "]"}]}]}], 
                                        ")"}]}], "]"}], "]"}]}], ",", 
                                        RowBox[{
                                        StyleBox["TFI",
                                        FontColor->RGBColor[1, 0.6, 0]], "[", 
                                        "__", "]"}], ",", 
                                        \(maybeF[\((Collect[Numerator[#1], 
                                        cD[__], 
                                        Factor]/Collect[Denominator[#1], 
                                        cD[__], Factor]&)\)/@Factor[#1]]&
                                        \)}], "]"}], "/.", "nmrel"}]}], 
                                        "]"}]}], "]"}], "/.", 
                                        \(top[i_Integer] \[Rule] i\)}], "/.", 
                                        \(top \[RuleDelayed] 
                                        \((patternTest[pattern[#1, blank[]], 
                                        PQ]&)\)\)}], "/.", 
                                        RowBox[{
                                        RowBox[{"patternTest", "[", 
                                        RowBox[{
                                        RowBox[{"pattern", "[", 
                                        RowBox[{
                                        RowBox[{"dm_", "/;", 
                                        RowBox[{"MemberQ", "[", 
                                        RowBox[{
                                        RowBox[{"{", 
                                        RowBox[{"d", ",", 
                                        InterpretationBox[\(p\^2\),
                                        PP], ",", "DP", ",", 
                                        InterpretationBox[\(m\_1\),
                                        m1], ",", 
                                        InterpretationBox[\(m\_2\),
                                        m2], ",", 
                                        InterpretationBox[\(m\_3\),
                                        m3], ",", 
                                        InterpretationBox[\(m\_4\),
                                        m4], ",", 
                                        InterpretationBox[\(m\_5\),
                                        m5]}], "}"}], ",", "dm"}], "]"}]}], 
                                        ",", \(blank[]\)}], "]"}], ",", 
                                        "_"}], "]"}], "\[RuleDelayed]", 
                                        \(pattern[dm, blank[]]\)}]}], "/.", 
                                        "\t", 
                                        \({blank \[RuleDelayed] Blank, 
                                        pattern \[RuleDelayed] Pattern, 
                                        patternTest \[RuleDelayed] 
                                        PatternTest}\)}], "}"}]}], "/.", 
                                      RowBox[{\(cD[ijk__]\), "\[Rule]", 
                                        RowBox[{
                                        RowBox[{"Hold", "[", 
                                        RowBox[{
                                        StyleBox["CayleyD",
                                        FontColor->RGBColor[1, 0.2, 1]], "[", 
                                        "ijk", "]"}], "]"}], "[", 
                                        \(addPeP[
                                        \(\(nutomass[Last[{t}]] /. 
                                        addPeP -> Identity\) /. nmrel
                                        \) /. {nuu_, ma_} \[RuleDelayed] ma]
                                        \), "]"}]}]}], "/.", 
                                    \(Hold[CompoundExpression] \[Rule] 
                                      CompoundExpression\)}], "/.", 
                                  RowBox[{
                                    RowBox[{\(Hold[Condition]\), "[", 
                                      RowBox[{"a_", ",", 
                                        RowBox[{
                                        StyleBox["IFF",
                                        FontColor->RGBColor[0.6, 0.4, 1]], 
                                        "[", "b_", "]"}]}], "]"}], "\[Rule]", 
                                    \(a /; b\)}]}], "/.", 
                                \(Hold[Set] \[Rule] Set\)}], "/.", 
                              \(Hold[TComment] -> TComment\)}], "/.", 
                            RowBox[{
                              RowBox[{"Hold", "[", 
                                RowBox[{
                                  StyleBox["CayleyD",
                                    FontColor->RGBColor[1, 0.2, 1]], "[", 
                                  "ijk__", "]"}], "]"}], "\[Rule]", 
                              RowBox[{
                                StyleBox["CayleyD",
                                  FontColor->RGBColor[1, 0.2, 1]], "[", 
                                "ijk", "]"}]}]}], "/.", 
                          \({setdel \[RuleDelayed] SetDelayed}\)}], ")"}], 
                      "/.", \(addPeP -> Identity\)}], "/.", 
                    \(Identity -> identit\)}], "/.", 
                  \(identit[a_] -> a\)}]}]}], "]"}], "/;", 
          RowBox[{\(Head[mrel]\), "=!=", 
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          StyleBox["MakeFun",
            FontColor->RGBColor[1, 0.6, 0.6]], "[", 
          RowBox[{
            RowBox[{
              RowBox[{"c_.", " ", 
                RowBox[{
                  StyleBox["TVI",
                    FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "t__", "]"}]}], 
              "==", "rhs_"}], ",", "eqn_String", ",", 
            RowBox[{
              StyleBox["IFF",
                FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds_", "]"}], ",", 
            \(mrel_ : {}\)}], "]"}], ":=", 
        RowBox[{
          StyleBox["Block",
            FontSize->16,
            FontWeight->"Bold",
            FontColor->RGBColor[1, 0.2, 0.2]], "[", 
          RowBox[{\({nm}\), ",", 
            RowBox[{
            \(nmrel = Select[Flatten[{mrel}], FreeQ[#1, Factor]&]\), ";", 
              RowBox[{"nmrel", "=", 
                RowBox[{"Join", "[", 
                  RowBox[{"nmrel", ",", 
                    RowBox[{"{", 
                      RowBox[{
                        RowBox[{\(m\_1\), "\[RuleDelayed]", 
                          InterpretationBox[\(m\_1\),
                            m1]}], ",", 
                        RowBox[{\(m\_2\), "\[RuleDelayed]", 
                          InterpretationBox[\(m\_2\),
                            m2]}], ",", 
                        RowBox[{\(m\_3\), "\[RuleDelayed]", 
                          InterpretationBox[\(m\_3\),
                            m3]}], ",", 
                        RowBox[{\(m\_4\), "\[RuleDelayed]", 
                          InterpretationBox[\(m\_4\),
                            m4]}], ",", 
                        \(\[CapitalSigma]\_i_ :> 
                          ToArgs[\[CapitalSigma]\_i]\)}], "}"}]}], "]"}]}], 
              ";", 
              RowBox[{
                RowBox[{
                  RowBox[{
                    RowBox[{
                      RowBox[{
                        RowBox[{
                          RowBox[{
                            RowBox[{
                              RowBox[{
                                RowBox[{
                                  RowBox[{
                                    RowBox[{
                                      RowBox[{
                                        RowBox[{
                                        RowBox[{"HoldForm", "@@", 
                                        RowBox[{"{", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"setdel", "[", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"addPeP", "[", 
                                        RowBox[{
                                        StyleBox["TVR",
                                        FontColor->RGBColor[0.2, 0.8, 1]], 
                                        "@@", \(top/@nutomass[{t}]\)}], 
                                        "]"}], "/.", "nmrel"}], ",", 
                                        RowBox[{\(Hold[Condition]\), "[", 
                                        RowBox[{
                                        RowBox[{
                                        \(Hold[CompoundExpression]\), "[", 
                                        RowBox[{
                                        \(\(Hold[TComment]\)[eqn, 
                                        \(\(\(addPeP[nutomass[TVR[t]]] /. 
                                        TVR -> TVIC\) /. {Pattern :> pat, 
                                        Blank :> blank}\) /. pat[PP, 
                                        blank[]] :> PP\) /. nmrel]\), ",", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"Collect", "[", 
                                        RowBox[{
                                        RowBox[{"TT", "=", 
                                        RowBox[{"Expand3", "[", 
                                        RowBox[{"nutonand\[CapitalDelta]u", 
                                        "[", 
                                        RowBox[{\(1\/c\), 
                                        RowBox[{"(", 
                                        RowBox[{"rhs", "/.", 
                                        RowBox[{
                                        RowBox[{
                                        StyleBox["TVI",
                                        FontColor->RGBColor[0.2, 0.4, 0.8]], 
                                        "[", "te__", "]"}], "\[RuleDelayed]", 
                                        
                                        RowBox[{"addPeP", "[", 
                                        RowBox[{"nutomass", "[", 
                                        RowBox[{
                                        StyleBox["TVI",
                                        FontColor->RGBColor[0.2, 0.4, 0.8]], 
                                        "[", "te", "]"}], "]"}], "]"}]}]}], 
                                        ")"}]}], "]"}], "]"}]}], ",", 
                                        RowBox[{
                                        StyleBox["TVI",
                                        FontColor->RGBColor[0.2, 0.4, 0.8]], 
                                        "[", "__", "]"}], ",", 
                                        \(maybeF[\((Collect[Numerator[#1], 
                                        cu[__], 
                                        Factor]/Collect[Denominator[#1], 
                                        cu[__], 
                                        Factor]&)\)/@Factor[#1]]&\)}], "]"}], 
                                        "/.", \(\[ScriptCapitalC] -> 1\)}], 
                                        "/.", "nmrel"}]}], "]"}], ",", 
                                        RowBox[{
                                        "nutonand\[CapitalDelta]u", "[", 
                                        RowBox[{
                                        StyleBox["IFF",
                                        FontColor->RGBColor[0.6, 0.4, 1]], 
                                        "[", "conds", "]"}], "]"}]}], 
                                        "]"}]}], "]"}], "/.", 
                                        \(top[i_Integer] \[Rule] i\)}], "/.", 
                                        \(top \[RuleDelayed] 
                                        \((patternTest[pattern[#1, blank[]], 
                                        PQ]&)\)\)}], "/.", 
                                        RowBox[{
                                        RowBox[{"patternTest", "[", 
                                        RowBox[{
                                        RowBox[{"pattern", "[", 
                                        RowBox[{
                                        RowBox[{"dm_", "/;", 
                                        RowBox[{"MemberQ", "[", 
                                        RowBox[{
                                        RowBox[{"{", 
                                        RowBox[{"d", ",", 
                                        InterpretationBox[\(p\^2\),
                                        PP], ",", 
                                        InterpretationBox[\(m\_1\),
                                        m1], ",", 
                                        InterpretationBox[\(m\_2\),
                                        m2], ",", 
                                        InterpretationBox[\(m\_3\),
                                        m3], ",", 
                                        InterpretationBox[\(m\_4\),
                                        m4]}], "}"}], ",", "dm"}], "]"}]}], 
                                        ",", \(blank[]\)}], "]"}], ",", 
                                        "_"}], "]"}], "\[RuleDelayed]", 
                                        \(pattern[dm, blank[]]\)}]}], "/.", 
                                        "\t", \({blank \[RuleDelayed] Blank, 
                                        pattern \[RuleDelayed] Pattern, 
                                        patternTest \[RuleDelayed] 
                                        PatternTest}\)}], "}"}]}], "/.", 
                                        "\n", "\t\t\t", 
                                        \(HoldPattern[TA[aijk__]] \[Rule] 
                                        \(Hold[TA[aijk]]\)[
                                        addPeP[\(nutomass[Last[{t}]] /. nmrel
                                        \) /. {nuu_, ma_} \[RuleDelayed] ma]]
                                        \)}], "/.", "\n", "\t\t\t", 
                                        \(cD[ijk__] \[Rule] 
                                        \(Hold[Cayley[ijk]]\)[
                                        addPeP[
                                        \(nutomass[Last[{t}]] /. nmrel
                                        \) /. {nuu_, ma_} \[RuleDelayed] ma]]
                                        \)}], "/.", 
                                      RowBox[{\(cu[ijk__]\), "\[Rule]", 
                                        RowBox[{
                                        RowBox[{"Hold", "[", 
                                        RowBox[{
                                        StyleBox["Cayleyu",
                                        FontColor->RGBColor[0.8, 0.8, 0.4]], 
                                        "[", "ijk", "]"}], "]"}], "[", 
                                        \(addPeP[
                                        \(\(nutomass[Last[{t}]] /. 
                                        addPeP -> Identity\) /. nmrel
                                        \) /. {nuu_, ma_} \[RuleDelayed] ma]
                                        \), "]"}]}]}], "/.", 
                                    \(Hold[CompoundExpression] \[Rule] 
                                      CompoundExpression\)}], "/.", 
                                  RowBox[{
                                    RowBox[{\(Hold[Condition]\), "[", 
                                      RowBox[{"a_", ",", 
                                        RowBox[{
                                        StyleBox["IFF",
                                        FontColor->RGBColor[0.6, 0.4, 1]], 
                                        "[", "b_", "]"}]}], "]"}], "\[Rule]", 
                                    \(a /; b\)}]}], "/.", 
                                \(Hold[Set] \[Rule] Set\)}], "/.", 
                              \(Hold[TComment] -> TComment\)}], "/.", 
                            \(addPeP -> Identity\)}], "/.", 
                          RowBox[{
                            RowBox[{"Hold", "[", 
                              RowBox[{
                                StyleBox["Cayleyu",
                                  FontColor->RGBColor[0.8, 0.8, 0.4]], "[", 
                                "ijk__", "]"}], "]"}], "\[Rule]", 
                            RowBox[{
                              StyleBox["Cayleyu",
                                FontColor->RGBColor[0.8, 0.8, 0.4]], "[", 
                              "ijk", "]"}]}]}], "/.", 
                        RowBox[{
                          RowBox[{"Hold", "[", 
                            RowBox[{
                              StyleBox["Cayley",
                                FontColor->RGBColor[1, 0.2, 1]], "[", 
                              "ijk__", "]"}], "]"}], "\[Rule]", 
                          RowBox[{
                            StyleBox["Cayley",
                              FontColor->RGBColor[1, 0.2, 1]], "[", "ijk", 
                            "]"}]}]}], "/.", "\n", "\t\t\t", 
                      \(Hold[TA] -> TA\)}], "/.", 
                    \({setdel \[RuleDelayed] SetDelayed}\)}], "/.", 
                  \(Identity -> identit\)}], "/.", 
                \(identit[a_] -> a\)}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          StyleBox["MakeFun",
            FontColor->RGBColor[1, 0.6, 0.6]], "[", 
          RowBox[{
            RowBox[{
              RowBox[{"c_.", " ", 
                RowBox[{
                  StyleBox["TVI",
                    FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "t__", "]"}]}], 
              "==", "rhs_"}], ",", "eqn_String", ",", \(mrel_ : {}\)}], 
          "]"}], ":=", 
        RowBox[{
          RowBox[{
            StyleBox["Block",
              FontSize->16,
              FontWeight->"Bold",
              FontColor->RGBColor[1, 0.2, 0.2]], "[", 
            RowBox[{\({nm}\), ",", 
              RowBox[{
              \(nmrel = Select[Flatten[{mrel}], FreeQ[#1, Factor]&]\), ";", 
                RowBox[{"nmrel", "=", 
                  RowBox[{"Join", "[", 
                    RowBox[{"nmrel", ",", 
                      RowBox[{"{", 
                        RowBox[{
                          RowBox[{\(m\_1\), "\[RuleDelayed]", 
                            InterpretationBox[\(m\_1\),
                              m1]}], ",", 
                          RowBox[{\(m\_2\), "\[RuleDelayed]", 
                            InterpretationBox[\(m\_2\),
                              m2]}], ",", 
                          RowBox[{\(m\_3\), "\[RuleDelayed]", 
                            InterpretationBox[\(m\_3\),
                              m3]}], ",", 
                          RowBox[{\(m\_4\), "\[RuleDelayed]", 
                            InterpretationBox[\(m\_4\),
                              m4]}], ",", 
                          RowBox[{\(\[CapitalSigma]\_i_\), "\[RuleDelayed]", 
                            RowBox[{
                              StyleBox["ToArgs",
                                FontColor->RGBColor[0.2, 0.6, 1]], "[", 
                              \(\[CapitalSigma]\_i\), "]"}]}]}], "}"}]}], 
                    "]"}]}], ";", 
                RowBox[{
                  RowBox[{
                    RowBox[{
                      RowBox[{
                        RowBox[{
                          RowBox[{
                            RowBox[{
                              RowBox[{
                                RowBox[{
                                  RowBox[{
                                    RowBox[{
                                      RowBox[{"HoldForm", "@@", 
                                        RowBox[{"{", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"setdel", "[", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"addPeP", "[", 
                                        RowBox[{
                                        StyleBox["TVR",
                                        FontColor->RGBColor[0.2, 0.8, 1]], 
                                        "@@", \(top/@nutomass[{t}]\)}], 
                                        "]"}], "/.", "nmrel"}], ",", 
                                        RowBox[{
                                        \(Hold[CompoundExpression]\), "[", 
                                        RowBox[{
                                        \(\(Hold[TComment]\)[eqn, 
                                        \(\(\(addPeP[nutomass[TVR[t]]] /. 
                                        TVR -> TVIC\) /. {Pattern :> pat, 
                                        Blank :> blank}\) /. pat[PP, 
                                        blank[]] :> PP\) /. nmrel]\), "\n", 
                                        "\t\t\t\t\t\t", ",", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"Collect", "[", 
                                        RowBox[{
                                        RowBox[{"TT", "=", 
                                        RowBox[{"Expand3", "[", 
                                        RowBox[{
                                        "nutonand\[CapitalDelta]u", "[", 
                                        RowBox[{\(1\/c\), 
                                        RowBox[{"(", 
                                        RowBox[{"rhs", "/.", 
                                        RowBox[{
                                        RowBox[{
                                        StyleBox["TVI",
                                        FontColor->RGBColor[0.2, 0.4, 0.8]], 
                                        "[", "te__", "]"}], "\[RuleDelayed]", 
                                        
                                        RowBox[{"addPeP", "[", 
                                        RowBox[{"nutomass", "[", 
                                        RowBox[{
                                        StyleBox["TVI",
                                        FontColor->RGBColor[0.2, 0.4, 0.8]], 
                                        "[", "te", "]"}], "]"}], "]"}]}]}], 
                                        ")"}]}], "]"}], "]"}]}], ",", 
                                        RowBox[{
                                        StyleBox["TVI",
                                        FontColor->RGBColor[0.2, 0.4, 0.8]], 
                                        "[", "__", "]"}], ",", \(maybeF[
                                        \((Collect[Numerator[#1], cu[__], 
                                        Factor]/Collect[Denominator[#1], 
                                        cu[__], Factor]&)\)/@Factor[#1]]&
                                        \)}], "]"}], "/.", "nmrel"}], "/.", 
                                        \(\[ScriptCapitalC] \[Rule] 1\)}]}], 
                                        "]"}]}], "]"}], "/.", 
                                        \(top[i_Integer] \[Rule] i\)}], "/.", 
                                        \(top \[RuleDelayed] 
                                        \((patternTest[pattern[#1, blank[]], 
                                        PQ]&)\)\)}], "/.", 
                                        RowBox[{
                                        RowBox[{"patternTest", "[", 
                                        RowBox[{
                                        RowBox[{"pattern", "[", 
                                        RowBox[{
                                        RowBox[{"dm_", "/;", 
                                        RowBox[{"MemberQ", "[", 
                                        RowBox[{
                                        RowBox[{"{", 
                                        RowBox[{"d", ",", 
                                        InterpretationBox[\(p\^2\),
                                        PP], ",", 
                                        InterpretationBox[\(m\_1\),
                                        m1], ",", 
                                        InterpretationBox[\(m\_2\),
                                        m2], ",", 
                                        InterpretationBox[\(m\_3\),
                                        m3], ",", 
                                        InterpretationBox[\(m\_4\),
                                        m4]}], "}"}], ",", "dm"}], "]"}]}], 
                                        ",", \(blank[]\)}], "]"}], ",", 
                                        "_"}], "]"}], "\[RuleDelayed]", 
                                        \(pattern[dm, blank[]]\)}]}], "/.", 
                                        \({blank \[RuleDelayed] Blank, 
                                        pattern \[RuleDelayed] Pattern, 
                                        patternTest \[RuleDelayed] 
                                        PatternTest}\)}], "}"}]}], "/.", 
                                      RowBox[{\(cD[ijk__]\), "\[Rule]", 
                                        RowBox[{
                                        RowBox[{"Hold", "[", 
                                        RowBox[{
                                        StyleBox["Cayley",
                                        FontColor->RGBColor[1, 0, 0.4]], "[", 
                                        "ijk", "]"}], "]"}], "[", 
                                        \(addPeP[
                                        \(nutomass[Last[{t}]] /. nmrel
                                        \) /. {nuu_, ma_Symbol} 
                                        \[RuleDelayed] ma]\), "]"}]}]}], 
                                    "/.", 
                                    RowBox[{\(cu[ijk__]\), "\[Rule]", 
                                      RowBox[{
                                        RowBox[{"Hold", "[", 
                                        RowBox[{
                                        StyleBox["Cayleyu",
                                        FontColor->RGBColor[0.8, 0.8, 0.4]], 
                                        "[", "ijk", "]"}], "]"}], "[", 
                                        \(addPeP[
                                        \(nutomass[Last[{t}]] /. nmrel
                                        \) /. {nuu_, ma_Symbol} 
                                        \[RuleDelayed] ma]\), "]"}]}]}], 
                                  "/.", 
                                  \(Hold[CompoundExpression] \[Rule] 
                                    CompoundExpression\)}], "/.", 
                                \(Hold[Set] \[Rule] Set\)}], "/.", 
                              \(Hold[TComment] -> TComment\)}], "/.", 
                            \(addPeP -> Identity\)}], "/.", 
                          RowBox[{
                            RowBox[{"Hold", "[", 
                              RowBox[{
                                StyleBox["Cayleyu",
                                  FontColor->RGBColor[0.8, 0.8, 0.4]], "[", 
                                "ijk__", "]"}], "]"}], "\[Rule]", 
                            RowBox[{
                              StyleBox["Cayleyu",
                                FontColor->RGBColor[0.8, 0.8, 0.4]], "[", 
                              "ijk", "]"}]}]}], "/.", 
                        RowBox[{
                          RowBox[{"Hold", "[", 
                            RowBox[{
                              StyleBox["Cayley",
                                FontColor->RGBColor[1, 0, 0.4]], "[", 
                              "ijk__", "]"}], "]"}], "\[Rule]", 
                          RowBox[{
                            StyleBox["Cayley",
                              FontColor->RGBColor[1, 0, 0.4]], "[", "ijk", 
                            "]"}]}]}], "/.", 
                      \({setdel \[RuleDelayed] SetDelayed}\)}], "/.", 
                    \(Identity -> identit\)}], "/.", 
                  \(identit[a_] -> a\)}]}]}], "]"}], "/;", 
          RowBox[{\(Head[mrel]\), "=!=", 
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          StyleBox["MakeFun",
            FontColor->RGBColor[1, 0.6, 0.6]], "[", 
          RowBox[{\(c_. \ TJI[t__] == rhs_\), ",", "eqn_String", ",", 
            RowBox[{
              StyleBox["IFF",
                FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds_", "]"}], ",", 
            \(mrel_ : {}\)}], "]"}], ":=", 
        RowBox[{
          StyleBox["Block",
            FontSize->16,
            FontWeight->"Bold",
            FontColor->RGBColor[1, 0.2, 0.2]], "[", 
          RowBox[{\({nm, nmrel, \ result}\), ",", 
            RowBox[{
            \(\[ScriptCapitalC] = 1\), ";", 
              \(nmrel = Select[Flatten[{mrel}], FreeQ[#1, Factor]&]\), ";", 
              RowBox[{"nmrel", "=", 
                RowBox[{"Join", "[", 
                  RowBox[{"nmrel", ",", 
                    RowBox[{"{", 
                      RowBox[{
                        RowBox[{\(m\_1\), "\[RuleDelayed]", 
                          InterpretationBox[\(m\_1\),
                            m1]}], ",", 
                        RowBox[{\(m\_2\), "\[RuleDelayed]", 
                          InterpretationBox[\(m\_2\),
                            m2]}], ",", 
                        RowBox[{\(m\_3\), "\[RuleDelayed]", 
                          InterpretationBox[\(m\_3\),
                            m3]}], ",", 
                        RowBox[{\(m\_4\), "\[RuleDelayed]", 
                          InterpretationBox[\(m\_4\),
                            m4]}], ",", 
                        RowBox[{\(\[CapitalSigma]\_i_\), "\[RuleDelayed]", 
                          RowBox[{
                            StyleBox["ToArgs",
                              FontColor->RGBColor[0.2, 0.6, 1]], "[", 
                            \(\[CapitalSigma]\_i\), "]"}]}]}], "}"}]}], 
                  "]"}]}], ";", "\n", "\t\t", 
              RowBox[{"result", " ", "=", " ", 
                RowBox[{
                  RowBox[{
                    RowBox[{
                      RowBox[{
                        RowBox[{
                          RowBox[{
                            RowBox[{
                              RowBox[{
                                RowBox[{
                                  RowBox[{
                                    RowBox[{
                                      RowBox[{"HoldForm", "@@", 
                                        RowBox[{"{", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"setdel", "[", 
                                        RowBox[{
                                        \(addPeP[TJR@@\(top/@nutomass[{
                                        tT = t}]\)] /. nmrel\), ",", 
                                        RowBox[{\(Hold[Condition]\), "[", 
                                        RowBox[{
                                        \(\(Hold[CompoundExpression]\)[
                                        \(Hold[TComment]\)[eqn, 
                                        \(\(\(addPeP[nutomass[TJR[t]]] /. 
                                        TJR -> TJIC\) /. {Pattern :> pat, 
                                        Blank :> blank}\) /. pat[PP, 
                                        blank[]] :> PP\) /. nmrel]\t, 
                                        TJ = \(Collect[TT = 
                                        Expand3[nutonand\[CapitalDelta]u[
                                        \(1\/c\) \((rhs /. TJI[te__] 
                                        \[RuleDelayed] 
                                        addPeP[nutomass[TJI[te]]])\)]], 
                                        TJI[__], 
                                        maybeF[\((Collect[Numerator[#1], 
                                        cu[__], 
                                        Factor]/Collect[Denominator[#1], 
                                        cu[__], Factor]&)\)/@Factor[#1]]&] /. 
                                        \[ScriptCapitalC] \[Rule] 1\) /. 
                                        nmrel]\), ",", 
                                        RowBox[{
                                        "nutonand\[CapitalDelta]u", "[", 
                                        RowBox[{
                                        StyleBox["IFF",
                                        FontColor->RGBColor[0.6, 0.4, 1]], 
                                        "[", "conds", "]"}], "]"}]}], 
                                        "]"}]}], "]"}], "/.", 
                                        \(top[i_Integer] \[Rule] i\)}], "/.", 
                                        \(top \[RuleDelayed] 
                                        \((patternTest[pattern[#1, blank[]], 
                                        PQ]&)\)\)}], "/.", 
                                        RowBox[{
                                        RowBox[{"patternTest", "[", 
                                        RowBox[{
                                        RowBox[{"pattern", "[", 
                                        RowBox[{
                                        RowBox[{"dm_", "/;", 
                                        RowBox[{"MemberQ", "[", 
                                        RowBox[{
                                        RowBox[{"{", 
                                        RowBox[{"d", ",", 
                                        InterpretationBox[\(p\^2\),
                                        PP], ",", 
                                        InterpretationBox[\(m\_1\),
                                        m1], ",", 
                                        InterpretationBox[\(m\_2\),
                                        m2], ",", 
                                        InterpretationBox[\(m\_3\),
                                        m3], ",", 
                                        InterpretationBox[\(m\_4\),
                                        m4]}], "}"}], ",", "dm"}], "]"}]}], 
                                        ",", \(blank[]\)}], "]"}], ",", 
                                        "_"}], "]"}], "\[RuleDelayed]", 
                                        \(pattern[dm, blank[]]\)}]}], "/.", 
                                        \({blank \[RuleDelayed] Blank, 
                                        pattern \[RuleDelayed] Pattern, 
                                        patternTest \[RuleDelayed] 
                                        PatternTest}\)}], "}"}]}], "/.", 
                                      \(HoldPattern[TA[aijk__]] \[Rule] 
                                        \(\(Hold[TA]\)[aijk]\)[
                                        addPeP[
                                        \(nutomass[Last[{t}]] /. nmrel
                                        \) /. {nuu_, ma_} \[RuleDelayed] ma]]
                                        \)}], "/.", 
                                    RowBox[{\(cD[ijk__]\), "\[Rule]", 
                                      RowBox[{
                                        RowBox[{"Hold", "[", 
                                        RowBox[{
                                        StyleBox["Cayley",
                                        FontColor->RGBColor[1, 0, 0.4]], "[", 
                                        "ijk", "]"}], "]"}], "[", 
                                        \(addPeP[
                                        \(nutomass[Last[{t}]] /. nmrel
                                        \) /. {nuu_, ma_Symbol} 
                                        \[RuleDelayed] ma]\), "]"}]}]}], 
                                  "/.", 
                                  RowBox[{\(cu[ijk__]\), "\[Rule]", 
                                    RowBox[{
                                      RowBox[{"Hold", "[", 
                                        RowBox[{
                                        StyleBox["Cayleyu",
                                        FontColor->RGBColor[0.8, 0.8, 0.4]], 
                                        "[", "ijk", "]"}], "]"}], "[", 
                                      \(addPeP[
                                        \(nutomass[Last[{t}]] /. nmrel
                                        \) /. {nuu_, ma_Symbol} 
                                        \[RuleDelayed] ma]\), "]"}]}]}], 
                                "/.", 
                                \(Hold[CompoundExpression] \[Rule] 
                                  CompoundExpression\)}], "/.", 
                              RowBox[{
                                RowBox[{\(Hold[Condition]\), "[", 
                                  RowBox[{"a_", ",", 
                                    RowBox[{
                                      StyleBox["IFF",
                                        FontColor->RGBColor[0.6, 0.4, 1]], 
                                      "[", "b_", "]"}]}], "]"}], "\[Rule]", 
                                \(a /; b\)}]}], "/.", 
                            \(Hold[Set] \[Rule] Set\)}], "/.", 
                          \(Hold[TComment] -> TComment\)}], "/.", 
                        RowBox[{
                          RowBox[{"Hold", "[", 
                            RowBox[{
                              StyleBox["Cayleyu",
                                FontColor->RGBColor[0.8, 0.8, 0.4]], "[", 
                              "ijk__", "]"}], "]"}], "\[Rule]", 
                          RowBox[{
                            StyleBox["Cayleyu",
                              FontColor->RGBColor[0.8, 0.8, 0.4]], "[", 
                            "ijk", "]"}]}]}], "/.", 
                      RowBox[{
                        RowBox[{"Hold", "[", 
                          RowBox[{
                            StyleBox["Cayley",
                              FontColor->RGBColor[1, 0, 0.4]], "[", "ijk__", 
                            "]"}], "]"}], "\[Rule]", 
                        RowBox[{
                          StyleBox["Cayley",
                            FontColor->RGBColor[1, 0, 0.4]], "[", "ijk", 
                          "]"}]}]}], "/.", \(Hold[TA] \[Rule] TA\)}], "/.", 
                  \({setdel \[RuleDelayed] SetDelayed}\)}]}], ";", 
              \(\[ScriptCapitalC] =  . \), ";", " ", "result"}]}], "]"}]}], 
      ";"}]], "Input"],

Cell[BoxData[
    \(\(SetAttributes[paTT, HoldFirst]; \)\)], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{
          RowBox[{
            RowBox[{"c_.", " ", 
              RowBox[{
                StyleBox["TKI",
                  FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "t__", "]"}]}], 
            "==", "rhs_"}], ",", "eqn_String", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds_", "]"}], ",", 
          \(mrel_ : {}\)}], "]"}], ":=", " ", 
      \(\(\(\(MakeFun[c\ TJI[t] == rhs /. TKI -> TJI, eqn, IFF[conds], 
                mrel] /. {PP :> 0, Sqrt[PP] :> 0, Pattern :> paTT}\) /. 
            paTT[0, _] -> 0\) /. paTT -> Pattern\) /. 
        TJIC[d_, li_List] :> TJIC[d, 0, li]\)}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          StyleBox["MakeFun",
            FontColor->RGBColor[1, 0.6, 0.6]], "[", 
          RowBox[{
            RowBox[{
              RowBox[{"c_.", " ", 
                RowBox[{
                  StyleBox["TBI",
                    FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "t__", "]"}]}], 
              "==", "rhs_"}], ",", "eqn_String", ",", 
            RowBox[{
              StyleBox["IFF",
                FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds_", "]"}], ",", 
            \(mrel_ : {}\)}], "]"}], ":=", " ", 
        \(MakeFun[c\ TJI[t] == rhs /. TBI -> TJI, eqn, IFF[conds], mrel] /. {
            \n\t\t\tTJI :> TBI, TJR :> TBR, TJIC :> TBIC}\)}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          StyleBox["MakeFun",
            FontColor->RGBColor[1, 0.6, 0.6]], "[", 
          RowBox[{
            RowBox[{
              RowBox[{"c_.", " ", 
                RowBox[{
                  StyleBox["TAI",
                    FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "t__", "]"}]}], 
              "==", "rhs_"}], ",", "eqn_String", ",", 
            RowBox[{
              StyleBox["IFF",
                FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds_", "]"}], ",", 
            \(mrel_ : {}\)}], "]"}], ":=", " ", 
        \(MakeFun[c\ TJI[t] == rhs /. TAI -> TJI, eqn, IFF[conds], mrel] /. {
            \n\t\t\tTJI :> TAI, TJR :> TAR, TJIC :> TAIC}\)}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell["Interchange", "Subsubsection",
  CellTags->"T1.5.1.1.1"],

Cell[BoxData[
    RowBox[{\(Interchange[z_, i_ \[LeftRightArrow] j_]\), ":=", 
      RowBox[{
      \(\[Nu]explicit[z] /. {\[Nu]\_i :> \[Nu]\_j, \[Nu]\_j :> \[Nu]\_i, 
            m\_i :> m\_j, m\_j :> m\_i, 
            \[CapitalDelta]\_ij__ :> 
              \((\[CapitalDelta]\_ij /. {i :> j, j :> i})\), 
            u\_ij__ :> \((u\_ij /. {i :> j, j :> i})\), \n\t\t
            h\_ij__ :> \((h\_ij /. {i :> j, j :> i})\), 
            \[Phi]\_ij__ :> \((\[Phi]\_ij /. {i :> j, j :> i})\), 
            \[Rho]\_ij__ :> \((\[Rho]\_ij /. {i :> j, j :> i})\), 
            \[Sigma]\_ij__ :> \((\[Sigma]\_ij /. {i :> j, j :> i})\), 
            D\_ij__ :> \((D\_ij /. {i :> j, j :> i})\)}\), "/.", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{
              RowBox[{
                StyleBox["TFI",
                  FontColor->RGBColor[1, 0.6, 0]], "[", \(depp__, args_List
                \), "]"}], ":>", 
              RowBox[{
                RowBox[{
                  StyleBox["TFI",
                    FontColor->RGBColor[1, 0.6, 0]], "[", \(depp, 
                  Sort[args]\), "]"}], "/;", "\[InvisibleSpace]", 
                \(! \((OrderedQ[args])\)\)}]}], ",", "\n", "\t", 
            RowBox[{
              RowBox[{
                StyleBox["TVI",
                  FontColor->RGBColor[1, 0.6, 0]], "[", \(depp__, args_List
                \), "]"}], ":>", 
              RowBox[{
                RowBox[{
                  StyleBox["TVI",
                    FontColor->RGBColor[1, 0.6, 0]], "[", \(depp, 
                  Sort[args]\), "]"}], "/;", "\[InvisibleSpace]", 
                \(! \((OrderedQ[args])\)\)}]}], ",", "\n", "\t", 
            RowBox[{
              RowBox[{
                StyleBox["TJI",
                  FontColor->RGBColor[1, 0.6, 0]], "[", \(depp__, args_List
                \), "]"}], ":>", 
              RowBox[{
                RowBox[{
                  StyleBox["TJI",
                    FontColor->RGBColor[1, 0.6, 0]], "[", \(depp, 
                  Sort[args]\), "]"}], "/;", "\[InvisibleSpace]", 
                \(! \((OrderedQ[args])\)\)}]}], ",", "\n", "\t\t\t", 
            RowBox[{
              RowBox[{
                StyleBox["TBI",
                  FontColor->RGBColor[1, 0.6, 0]], "[", \(depp__, args_List
                \), "]"}], ":>", 
              RowBox[{
                RowBox[{
                  StyleBox["TBI",
                    FontColor->RGBColor[1, 0.6, 0]], "[", \(depp, 
                  Sort[args]\), "]"}], "/;", "\[InvisibleSpace]", 
                \(! \((OrderedQ[args])\)\)}]}]}], "}"}]}]}]], "Input"],

Cell[BoxData[
    \(\(Interchange[z_, i_ \[LeftRightArrow] j_, vw__] := 
      Interchange[Interchange[z, i \[LeftRightArrow] j], vw]; \)\)], "Input"],

Cell[BoxData[
    \(\(Interchange[z_, {i_ \[LeftRightArrow] j_}] := 
      Interchange[Interchange[z, i \[LeftRightArrow] j]]; \)\)], "Input"],

Cell[BoxData[
    \(\(Interchange[z_, {i_ \[LeftRightArrow] j_, vw__}] := 
      Interchange[Interchange[z, i \[LeftRightArrow] j], vw]; \)\)], "Input"],

Cell[BoxData[
    \(Interchange[
      test[\[CapitalDelta]\_\(x, b, y\), \[Nu]\_x, \[Nu]\_y, \[Nu]\_a, 
        \[Nu]\_b, m\_x], \ x \[LeftRightArrow] y, a \[LeftRightArrow] b]\)], 
  "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\[Nu]explicit", "Subsection",
  CellTags->"T1.5.2.1"],

Cell[BoxData[
    \(\[Nu]explicit[z_] := 
      \(z /. \[CapitalSigma]\_i_ :> 
            \((Expand[3\ d\  - 2\  - 2\ \(\[Sum]\_\(j = 1\)\%i \[Nu]\_j\)])
              \)\) /. \n
        \t{\t\[Sigma]\_\(i_, j_, k_\) :> 
            \(-\(1\/4\)\) \((d - \[Nu]\_i - 2  \[Nu]\_j)\) 
                \[Phi]\_\(i, j, k\) - 
              \(1\/4\) \((d - 2  \[Nu]\_i - \[Nu]\_j)\) \[Phi]\_\(j, i, k\) - 
              \n\t\t\t\t\t
              \(1\/4\) \((2  d - 2  \[Nu]\_i - 2  \[Nu]\_j - \[Nu]\_k - 1)\) 
                \[Phi]\_\(k, i, j\), \n\t\t\t
          h\_\(i_, j_, k_\ \) :> \n\t\t\t\t
            \(-\(1\/2\)\) \((d - 2  \[Nu]\_j - \[Nu]\_k)\) \(m\_k\^2\) 
                \[Phi]\_\(i, j, k\) - 
              \(1\/2\) \((2  d - \[Nu]\_i - 2  \[Nu]\_j - 2  \[Nu]\_k - 1)\) 
                \(m\_i\^2\) \[Phi]\_\(k, i, j\) + \n\t\t\t\t\t
              \((d - \[Nu]\_j - 2\ \[Nu]\_k)\) \[Rho]\_\(i, j, k\), \n
          S\_\(i_, j_, k_\ \) :> \n\t\t\t\t
            \(-\((d - 2  \[Nu]\_j - \[Nu]\_k)\)\) \(m\_k\^2\) 
                \[Phi]\_\(j, i, k\)\  - 
              \((d - \[Nu]\_j - 2  \[Nu]\_k)\) \(m\_j\^2\) 
                \[Phi]\_\(k, i, j\) + \n\t\t\t\t\t
              2 \((2  d - \[Nu]\_i - 2  \[Nu]\_j - 2  \[Nu]\_k - 1)\) 
                \[Rho]\_\(i, j, k\)}\)], "Input"],

Cell[CellGroupData[{

Cell["TA", "Subsubsection",
  CellTags->"T1.5.2.1.1"],

Cell[BoxData[
    \(MakeBoxes[\(TA[a_, d___Symbol, ijk___Integer]\)[__], fmt_] := 
      SubscriptBox[a, RowBox[{ijk}]]\)], "Input"],

Cell[BoxData[
    \(MakeBoxes[\(TA[a_, d___Symbol, ijk___Integer, ___Symbol]\)[__], 
        fmt_] := SubscriptBox[a, RowBox[{ijk}]]\)], "Input"],

Cell[BoxData[
    \(\(\(TA[args__]\)[mar_List] := 
      \((\(TA[args]\)[mar] = \(ta[args]\)[mar])\); \)\)], "Input"],

Cell[TextData[{
  "Assume that the last argument of {em__} is always p =",
  Cell[BoxData[
      FormBox[
        SqrtBox[
          RowBox[{" ", 
            FormBox[\(p\^2\),
              "TraditionalForm"]}]], TraditionalForm]]],
  "."
}], "Text"],

Cell["(72)", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(\(ta[D, i_Integer, j_Integer, k_Integer]\)[{em__}]\), ":=", 
        RowBox[{"With", "[", 
          RowBox[{
          \({q = Last[{em}], 
              mi = {em}\[LeftDoubleBracket]i\[RightDoubleBracket], 
              mj = {em}\[LeftDoubleBracket]j\[RightDoubleBracket], 
              mk = {em}\[LeftDoubleBracket]k\[RightDoubleBracket]}\), ",", 
            RowBox[{"Factor", "[", 
              RowBox[{
              \(q\^8\), "-", \(4\ q\^6\ \((mi\^2 + mj\^2 + mk\^2)\)\), "+", 
                \(q\^4\ \((
                    6\ \((mi\^4 + mj\^4 + mk\^4)\) + 
                      4\ \((mi\^2\ mj\^2 + mk\^2\ mi\^2 + mk\^2\ mj\^2)\))
                    \)\), "-", 
                \(4\ q\^2\ 
                  \((mi\^6 + mj\^6 + mk\^6 - mi\^2\ \((mj\^4 + mk\^4)\) - 
                      mj\^2\ \((mi\^4 + mk\^4)\) - 
                      mk\^2\ \((mi\^4 + mj\^4)\) + 10\ mi\^2\ mj\^2\ mk\^2)
                    \)\), "+", 
                SuperscriptBox[
                  RowBox[{
                    RowBox[{
                      StyleBox["Cayley",
                        FontColor->RGBColor[1, 0, 0.4]], "[", \(i, j, k\), 
                      "]"}], "[", \({em}\), "]"}], "2"]}], "]"}]}], "]"}]}], 
      ";"}]], "Input"],

Cell["(73)", "Text"],

Cell[BoxData[
    \(\(\(ta[\[Rho], i_Integer, j_Integer, k_Integer]\)[{em__}] := 
      With[{q = Last[{em}], 
          mi = {em}\[LeftDoubleBracket]i\[RightDoubleBracket], 
          mj = {em}\[LeftDoubleBracket]j\[RightDoubleBracket], 
          mk = {em}\[LeftDoubleBracket]k\[RightDoubleBracket]}, 
        Factor[\(-q\^6\) + 3\ q\^4\ \((mi\^2 + mj\^2 + mk\^2)\) - 
            q\^2\ \((
                3\ \((mi\^4 + mj\^4 + mk\^4)\) + 
                  2\ \((mi\^2\ mj\^2 + mk\^2\ mi\^2 + mk\^2\ mj\^2)\))\) + \ 
            \((mi\^6 + mj\^6 + mk\^6 - mi\^2\ \((mj\^4 + mk\^4)\) - 
                mj\^2\ \((mi\^4 + mk\^4)\) - mk\^2\ \((mi\^4 + mj\^4)\) + 
                10\ mi\^2\ mj\^2\ mk\^2)\)]]; \)\)], "Input"],

Cell["(74)", "Text"],

Cell[BoxData[
    \(\(\(ta[\[Phi], i_Integer, j_Integer, k_Integer]\)[{em__}] := 
      With[{q = Last[{em}], 
          mi = {em}\[LeftDoubleBracket]i\[RightDoubleBracket], 
          mj = {em}\[LeftDoubleBracket]j\[RightDoubleBracket], 
          mk = {em}\[LeftDoubleBracket]k\[RightDoubleBracket]}, 
        4  Factor[
            q\^4\  + 2 \( q\^2\) \((mi\^2 - mj\^2 - mk\^2)\) + 
              \((mj\^2 - mk\^2)\)\^2 + 
              \(mi\^2\) \((2  mj\^2 + 2  mk\^2 - 3  mi\^2)\)]]; \)\)], "Input"],

Cell["(75)", "Text"],

Cell[BoxData[
    \(\(\(ta[\[Sigma], d_, i_Integer, j_Integer, k_Integer, \[Nu]i_Integer, 
          \[Nu]j_Integer, \[Nu]k_Integer]\)[{em__}] := 
      Factor[\(\(-\(1\/4\)\) \((d - \[Nu]i - 2  \[Nu]j)\) 
              \(TA[\[Phi], i, j, k]\)[{em}]\)\ 
            \(-\(\(1\/4\) \((d - 2  \[Nu]i - \[Nu]j)\) 
                \(TA[\[Phi], j, i, k]\)[{em}]\)\) - 
          \(\(1\/4\) \((2  d - 2  \[Nu]i - 2  \[Nu]j - \[Nu]k - 1)\) 
            \(TA[\[Phi], k, i, j]\)[{em}]\ \)\ \ ]; \)\)], "Input"],

Cell["(76)", "Text"],

Cell[BoxData[
    \(\(\(ta[h, d_, i_Integer, j_Integer, k_Integer, \[Nu]i_Integer, 
          \[Nu]j_Integer, \[Nu]k_Integer]\)[{em__}] := \n\t
      With[{q = Last[{em}], 
          mi = {em}\[LeftDoubleBracket]i\[RightDoubleBracket], 
          mj = {em}\[LeftDoubleBracket]j\[RightDoubleBracket], 
          mk = {em}\[LeftDoubleBracket]k\[RightDoubleBracket]}, 
        Factor[\(-\(1\/2\)\) \((d - 2  \[Nu]j - \[Nu]k)\) \(mk\^2\) 
              \(TA[\[Phi], i, j, k]\)[{em}] - 
            \(1\/2\) \((2  d - \[Nu]i - 2  \[Nu]j - 2  \[Nu]k - 1)\) 
              \(mi\^2\) \(TA[\[Phi], k, i, j]\)[{em}] + 
            \(\((d - \[Nu]j - 2\ \[Nu]k)\) \(TA[\[Rho], i, j, k]\)[{em}]\ \)\ 
          \ ]]; \)\)], "Input"],

Cell["(79)", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{
      \(\(ta[S, d_, i_Integer, j_Integer, k_Integer, \[Nu]i_Integer, 
            \[Nu]j_Integer, \[Nu]k_Integer]\)[{em__}]\), ":=", "\n", "\t\t", 
        RowBox[{"With", "[", 
          RowBox[{
          \({q = Last[{em}], 
              mi = {em}\[LeftDoubleBracket]i\[RightDoubleBracket], 
              mj = {em}\[LeftDoubleBracket]j\[RightDoubleBracket], 
              mk = {em}\[LeftDoubleBracket]k\[RightDoubleBracket]}\), ",", 
            RowBox[{"Factor", "[", 
              RowBox[{
              \(\(-\((d - 2  \[Nu]j - \[Nu]k)\)\) \(mk\^2\) 
                  \(TA[\[Phi], j, i, k]\)[{em}]\), " ", "-", 
                RowBox[{
                \((d - \[Nu]j - 2  \[Nu]k)\), \(mj\^2\), 
                  \(\(TA[\[Phi], k, i, j]\)[{em}]\), "test", "\n", 
                  FormBox["test",
                    "TraditionalForm"], "2", 
                  \((2  d - \[Nu]i - 2  \[Nu]j - 2  \[Nu]k - 1)\), 
                  \(\(TA[\[Rho], i, j, k]\)[{em}]\)}]}], "   ", "]"}]}], 
          "]"}]}], ";"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["CheckRecursion", "Subsection",
  CellTags->"T1.5.3.1"],

Cell["\<\
The following functions check the recurrence relations for J and V \
(using the final form as generated by MakeFun) via their Mellin-Integral \
Kernels.\
\>", "Text"],

Cell["\<\
This test is applicable in the general case  and if some of the 
masses vanish, but not for other special cases !!!\
\>", "Text"],

Cell[CellGroupData[{

Cell["CheckTVIRecursion", "Subsubsection",
  CellTags->"T1.5.3.1.1"],

Cell[BoxData[
    \(\(TVIKernel[d_, PP, \ lis_] := \n
      \(\((\(-PP\))\)\^\(d - n1 - n2 - n3 - n4\)\) 
        \(\((m1\^2\/\(-PP\))\)\^s1\) \(\((m2\^2\/\(-PP\))\)\^s2\) 
        \(\((m3\^2\/\(-PP\))\)\^s3\) \((m4\^2\/\(-PP\))\)\^s4*
        TVIpsi\ [d, \ \((lis /. {m1 -> s1, m2 -> s2, m3 -> s3, m4 -> s4})\)]; 
    \)\)], "Input"],

Cell[BoxData[
    \(\(TVIPsi[d_, \n
        \t{{n1_, \ s1_}, \ {n2_, s2_}, {n3_, s3_}, {n4_, s4_}}] := 
      \(\((\(-1\))\)\^\(n1 + n2 + n3 + n4 + 1\)\) 
        \(\(gamma[\(-s1\)] gamma[d/2 - n1 - s1]\)\/gamma[n1]\) 
        \(\(gamma[\(-s2\)] gamma[d/2 - n2 - s2]\)\/gamma[n2]\) 
        \(\(gamma[\(-s3\)] gamma[d/2 - n3 - s3]\)\/gamma[n3]\) 
        \(\(gamma[\(-s4\)] gamma[n4 + s4]\)\/gamma[n4]\) 
        gamma[n1 + n2 + n3 + n4 + s1 + s2 + s3 + s4 - d]/
          gamma[3  d/2 - n1 - n2 - n3 - n4 - s1 - s2 - s3 - s4] 
        \(gamma[n1 + n3 + s1 + s3 - d/2]\/gamma[
              n1 + n3 + n4 + s1 + s3 + s4 - d/2]\) 
        gamma[d - n1 - n3 - n4 - s1 - s3 - s4]\/gamma[d - n1 - n3 - s1 - s3]; 
    \)\)], "Input"],

Cell[BoxData[
    \(CheckTVIRecursion[expr_] := \ 
      If[\ $CheckRecursion === True, \n\t
        Block[{rel, kern, \ nkern}, \n
          rel = \((
              \((Extract[expr /. TVIC -> TVIkernel, {1, 2, 1, 1, 2}])\) - 
                Extract[expr /. TVI -> TVIkernel, {1, 2, 1, 2}])\); \n
          If[\ {TVIkernel} === 
              Union[Head/@
                  Cases[rel, ff_[__] /; Context[ff] =!= "\<System`\>", 4]], \t
            \nrel = Collect[Numerator[Together[rel]], \ TVI[__], Factor]; \n
            kern = rel /. TVIkernel -> TVIKernel; \n
            kern = kern /. {PP -> \(-1\), m1 -> Sqrt[z1], m2 -> Sqrt[z2], 
                  m3 -> Sqrt[z3], m4 -> Sqrt[z4]}; \n
            kern = PowerExpand[kern]; \nkern = Expand[kern]; \n
            kern = kern /. {
                  \(z1\^\(s1 + w1_. \)\) \(z2\^\(s2 + w2_. \)\) 
                      \(z3\^\(s3 + w3_. \)\) \(z4\^\(s4 + w4_. \)\) fac_ :> 
                    \((fac /. {s1 -> s1 - w1, s2 -> s2 - w2, s3 -> s3 - w3, 
                          s4 -> s4 - w4})\)}\ ; \n
            kern = \(\(kern /. TVIpsi -> TVIPsi\) /. {
                    gamma[zz_] :> gamma[Expand[zz]]}\) /. {
                  gamma[zz_ + nn_Integer] :> gamma[zz] Pochhammer[zz, nn]}; \n
            \t\t\t\tKERN = kern; \nkern = kern /. {gamma[zz_] :> 1}; \n
            kern = \(kern /. {
                    \((\(-1\))\)\^\(nn_ + ww_Integer\) :> \((\(-1\))\)\^ww}
                    \) /. {\((\(-1\))\)\^nn_ :> 1}; \n
            nkern = Table[\n\t\t\t
                kern /. 
                  Thread[{d, \ n1, \ n2, \ n3, n4, s1, s2, \ s3, s4} -> 
                      Table[Random[], {9}]], {3}]; \nnkern\ \ , \n\t\t
            Print["\<Matching Error\>"]\ ]\ ], \n\t\t
        If[$Notebooks, Print["\<Off\>"]]]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["CheckTJIRecursion", "Subsubsection",
  CellTags->"T1.5.3.2.1"],

Cell[BoxData[
    \(\(TJIKernel[d_, PP, \ lis_] := \n
      \(\((\(-PP\))\)\^\(d - n1 - n2 - n3\)\) \(\((m1\^2\/\(-PP\))\)\^s1\) 
        \(\((m2\^2\/\(-PP\))\)\^s2\) \((m3\^2\/\(-PP\))\)\^s3*
        TJIpsi\ [d, \ \((lis /. {m1 -> s1, m2 -> s2, m3 -> s3})\)]; \)\)], 
  "Input"],

Cell[BoxData[
    \(TJIPsi\ [d_, {{n1_, \ s1_}, \ {n2_, s2_}, {n3_, s3_}}] := 
      Evaluate[TVIPsi[d, \n\t{{n1, \ s1}, \ {n2, s2}, {n3, s3}, {0, 0}}]/
          gamma[0]]\)], "Input"],

Cell[BoxData[
    \(\(Clear[CheckTJIRecursion]; \)\)], "Input"],

Cell[BoxData[
    \(CheckTJIRecursion[expr_] := \ 
      If[\ $CheckRecursion === True, \n\t
        Block[{rel, kern, nkern}, \n
          rel = \((
              \((Extract[expr /. TJIC -> TJIkernel, {1, 2, 1, 1, 2}])\) - 
                Extract[expr /. TJI -> TJIkernel, {1, 2, 1, 2}])\); \n\t\t\t
          REL = rel; \t\t\t\n
          If[{TJIkernel} === 
              Union[Head/@
                  Cases[rel, ff_[__] /; Context[ff] =!= "\<System`\>", 4]]\ , 
            \t\t\t\nrel = 
              Collect[Numerator[Together[rel]], \ TJI[__], Factor]; \n
            kern = rel /. TJIkernel -> TJIKernel; \n
            kern = kern /. {PP -> \(-1\), m1 -> Sqrt[z1], m2 -> Sqrt[z2], 
                  m3 -> Sqrt[z3]}; \nkern = PowerExpand[kern]; \n
            kern = Expand[kern]; \n
            kern = kern /. {
                  \(z1\^\(s1 + w1_. \)\) \(z2\^\(s2 + w2_. \)\) 
                      \(z3\^\(s3 + w3_. \)\) fac_ :> 
                    \((fac /. {s1 -> s1 - w1, s2 -> s2 - w2, s3 -> s3 - w3})
                      \)}; \ \n
            kern = \(\(kern /. TJIpsi -> TJIPsi\) /. {
                    gamma[zz_] :> gamma[Expand[zz]]}\) /. {
                  gamma[zz_ + nn_Integer] :> gamma[zz] Pochhammer[zz, nn]}; \n
            \ \ \ \ \ \ KERN = kern; \nkern = kern /. {gamma[zz_] :> 1}; \n
            kern = \(kern /. {
                    \((\(-1\))\)\^\(nn_ + ww_Integer\) :> \((\(-1\))\)\^ww}
                    \) /. {\((\(-1\))\)\^nn_ :> 1}; \n
            nkern = Table[\n\t\t\t
                kern /. 
                  Thread[{d, \ n1, \ n2, \ n3, s1, s2, \ s3} -> 
                      Table[Random[], {7}]], {3}]; \nnkern\ \ , \n\t\t
            Print["\<Matching Error\>"]\ ]\ ], \n\t\t
        If[$Notebooks, Print["\<Off\>"]]]\)], "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Recurrence Relations Input and Recursion Function Generation", "Section",
  CellDingbat->"\[FilledSmallCircle]",
  CellTags->"T1.6.1"],

Cell[TextData[{
  "Most ",
  Cell[BoxData[
      \(TraditionalForm\`e\_\(\(...\)\(\ \)\)\)]],
  "equations below were entered by the palettes which show up when \
$PutPalettes is set to True.\nExamples of alternative (hand) input are  (50) \
, (201) -- (205)."
}], "Text"],

Cell[CellGroupData[{

Cell["Input (35)", "Subsection",
  CellTags->"T1.6.1.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_35\), "=", "\n", 
        RowBox[{
          RowBox[{"2", \(\[Nu]\_1\), "\[ScriptCapitalC]", " ", 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["F",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_5\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TFI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4], 
                Subscript[ \[Nu], 5]}],
              Editable->True]}], "\[Equal]", " ", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
              \(\((d - 2  \[Nu]\_1 - \[Nu]\_3 - \[Nu]\_5)\) 
                  \[CapitalDelta]\_1\), "+", 
                RowBox[{\(\[CapitalDelta]\_\(3, 4, 5\)\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\(\[Nu]\_5\), 
                        InterpretationBox[
                          StyleBox[\(5\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 5, "+"],
                          Editable->True], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(2\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 2, "-"],
                              Editable->True], "-", 
                            InterpretationBox[
                              StyleBox[\(1\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 1, "-"],
                              Editable->True]}], ")"}]}], "-", 
                      RowBox[{\(\[Nu]\_3\), 
                        InterpretationBox[
                          StyleBox[\(3\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "+"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(1\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "-"],
                          Editable->True]}]}], ")"}]}], "+", 
                RowBox[{\(\[CapitalDelta]\_2\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\(\[Nu]\_1\), 
                        InterpretationBox[
                          StyleBox[\(1\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "+"],
                          Editable->True], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(5\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 5, "-"],
                              Editable->True], "-", 
                            InterpretationBox[
                              StyleBox[\(2\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 2, "-"],
                              Editable->True]}], ")"}]}], "+", 
                      RowBox[{\(\[Nu]\_3\), 
                        InterpretationBox[
                          StyleBox[\(3\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "+"],
                          Editable->True], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(5\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 5, "-"]], "-", 
                            InterpretationBox[
                              StyleBox[\(4\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 4, "-"],
                              Editable->True]}], ")"}]}], "+", \(\[Nu]\_5\), 
                      "-", \(\[Nu]\_1\)}], ")"}]}], "+", "\n", "\t\t\t\t", 
                RowBox[{\(\[CapitalDelta]\_6\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\(\[Nu]\_1\), 
                        InterpretationBox[
                          StyleBox[\(1\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "+"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(3\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "-"],
                          Editable->True]}], "+", 
                      RowBox[{\(\[Nu]\_5\), 
                        InterpretationBox[
                          StyleBox[\(5\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 5, "+"],
                          Editable->True], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(3\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 3, "-"],
                              Editable->True], "-", 
                            InterpretationBox[
                              StyleBox[\(4\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 4, "-"],
                              Editable->True]}], ")"}]}], "+", \(\[Nu]\_3\), 
                      "-", \(\[Nu]\_1\)}], ")"}]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["F",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_5\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TFI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4], 
                Subscript[ \[Nu], 5]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_35 = OperatorApplyF[e\_35]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (35)", "Subsection",
  CellTags->"T1.6.2.1"],

Cell[BoxData[
    \(h\_351 = s\_35 /. \[Nu]\_1 -> \((\[Nu]\_1 - 1)\)\)], "Input"],

Cell[CellGroupData[{

Cell[TextData[{
  Cell[BoxData[
      \(TraditionalForm\`1\^\(\(+\)\(\ \)\)\)]],
  " (35_1)"
}], "Subsubsection",
  CellTags->"T1.6.2.1.1"],

Cell[BoxData[
    RowBox[{\(f\_351\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_351\), ",", "\"\<(35_1)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            RowBox[{\(\[Nu]\_1 > 1\), "&&", 
              RowBox[{
                RowBox[{
                  StyleBox["Cayley",
                    FontColor->RGBColor[1, 0, 0.4]], "[", 
                  RowBox[{
                    InterpretationBox[\(m\_1\),
                      m1], ",", 
                    InterpretationBox[\(m\_2\),
                      m2], ",", 
                    InterpretationBox[\(m\_3\),
                      m3], ",", 
                    InterpretationBox[\(m\_4\),
                      m4], ",", 
                    InterpretationBox[\(m\_5\),
                      m5], ",", 
                    SqrtBox[
                      InterpretationBox[\(p\^2\),
                        PP]]}], "]"}], "=!=", "0"}]}], "]"}]}], "]"}]}]], 
  "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  Cell[BoxData[
      \(TraditionalForm\`2\^\(\(+\)\(\ \)\)\)]],
  " (35_2)"
}], "Subsubsection",
  CellTags->"T1.6.2.2.1"],

Cell[BoxData[
    RowBox[{\(f\_352\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{
        \(Interchange[h\_351, 1 \[LeftRightArrow] 2, 3 \[LeftRightArrow] 4]
            \), ",", "\"\<(35_2)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            RowBox[{\(\[Nu]\_2 > 1\), "&&", 
              RowBox[{
                RowBox[{
                  StyleBox["Cayley",
                    FontColor->RGBColor[1, 0, 0.4]], "[", 
                  RowBox[{
                    InterpretationBox[\(m\_1\),
                      m1], ",", 
                    InterpretationBox[\(m\_2\),
                      m2], ",", 
                    InterpretationBox[\(m\_3\),
                      m3], ",", 
                    InterpretationBox[\(m\_4\),
                      m4], ",", 
                    InterpretationBox[\(m\_5\),
                      m5], ",", 
                    SqrtBox[
                      InterpretationBox[\(p\^2\),
                        PP]]}], "]"}], "=!=", "0"}]}], "]"}]}], "]"}]}]], 
  "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  Cell[BoxData[
      \(TraditionalForm\`3\^\(\(+\)\(\ \)\)\)]],
  " (35_3)"
}], "Subsubsection",
  CellTags->"T1.6.2.3.1"],

Cell[BoxData[
    RowBox[{\(f\_353\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{
        \(Interchange[h\_351, 1 \[LeftRightArrow] 3, 2 \[LeftRightArrow] 4]
            \), ",", "\"\<(35_3)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            RowBox[{\(\[Nu]\_3 > 1\), "&&", 
              RowBox[{
                RowBox[{
                  StyleBox["Cayley",
                    FontColor->RGBColor[1, 0, 0.4]], "[", 
                  RowBox[{
                    InterpretationBox[\(m\_1\),
                      m1], ",", 
                    InterpretationBox[\(m\_2\),
                      m2], ",", 
                    InterpretationBox[\(m\_3\),
                      m3], ",", 
                    InterpretationBox[\(m\_4\),
                      m4], ",", 
                    InterpretationBox[\(m\_5\),
                      m5], ",", 
                    SqrtBox[
                      InterpretationBox[\(p\^2\),
                        PP]]}], "]"}], "=!=", "0"}]}], "]"}]}], "]"}]}]], 
  "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  Cell[BoxData[
      \(TraditionalForm\`4\^\(\(+\)\(\ \)\)\)]],
  " (35_4)"
}], "Subsubsection",
  CellTags->"T1.6.2.4.1"],

Cell[BoxData[
    RowBox[{\(f\_354\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{
        \(Interchange[h\_351, 1 \[LeftRightArrow] 4, 2 \[LeftRightArrow] 3]
            \), ",", "\"\<(35_4)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            RowBox[{\(\[Nu]\_4 > 1\), "&&", 
              RowBox[{
                RowBox[{
                  StyleBox["Cayley",
                    FontColor->RGBColor[1, 0, 0.4]], "[", 
                  RowBox[{
                    InterpretationBox[\(m\_1\),
                      m1], ",", 
                    InterpretationBox[\(m\_2\),
                      m2], ",", 
                    InterpretationBox[\(m\_3\),
                      m3], ",", 
                    InterpretationBox[\(m\_4\),
                      m4], ",", 
                    InterpretationBox[\(m\_5\),
                      m5], ",", 
                    SqrtBox[
                      InterpretationBox[\(p\^2\),
                        PP]]}], "]"}], "=!=", "0"}]}], "]"}]}], "]"}]}]], 
  "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (3501)", "Subsection",
  CellTags->"T1.6.3.1"],

Cell[BoxData[
    \(Cayley[m1, m5, m5, m4, m5, m5] // Factor\)], "Input"],

Cell[TextData[Cell[BoxData[
    \(TraditionalForm\`m\_1 = 0\ \ ; \ \ m\_2 = \(m\_3 = m\_5\); \ \ \ 
    p\^2 = m\_5\%2\)]]], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_3501\), "=", 
        RowBox[{
          RowBox[{\((d - 2  \[Nu]\_1 - \[Nu]\_3 - \[Nu]\_5)\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["F",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_5\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TFI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4], 
                Subscript[ \[Nu], 5]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", " ", 
              RowBox[{
                RowBox[{\(\[Nu]\_5\), 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(1\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 1, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(2\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 2, "-"],
                        Editable->True]}], ")"}], 
                  InterpretationBox[
                    StyleBox[\(5\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 5, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(1\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "-"],
                    Editable->True], 
                  InterpretationBox[
                    RowBox[{
                      StyleBox[\(3\^+\),
                        FontWeight->"Bold"], " "}],
                    Schiebe[ 3, "+"],
                    Editable->True]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["F",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_5\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TFI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4], 
                Subscript[ \[Nu], 5]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_3501 = OperatorApplyF[e\_3501]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "MakeFun (3501) \[CapitalDelta]=0 ",
  Cell[BoxData[
      \(TraditionalForm\`\((p\^2 = m\^2)\)\)]]
}], "Subsection",
  CellTags->"T1.6.4.1"],

Cell[BoxData[
    \(h\_3501 = s\_3501\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_3501\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(s\_3501\), ",", "\"\<(3501)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", \(PP === m5^2\), "]"}], 
          ",", \({m1 :> 0, m2 :> m5, m3 :> m5}\)}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (3502)", "Subsection",
  CellTags->"T1.6.5.1"],

Cell[BoxData[
    \(Cayley[m1, m5, m5, m4, m5, m5] // Factor\)], "Input"],

Cell[TextData[Cell[BoxData[
    \(TraditionalForm\`m\_4 = 0\ \ ; \ \ m\_2 = \(m\_3 = m\_5\); \ \ \ 
    p\^2 = m\_5\%2\)]]], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_3502\), "=", 
        RowBox[{
          RowBox[{\((d - 2  \[Nu]\_1 - \[Nu]\_3 - \[Nu]\_5)\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["F",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_5\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TFI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4], 
                Subscript[ \[Nu], 5]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", " ", 
              RowBox[{
                RowBox[{\(\[Nu]\_5\), 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(1\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 1, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(2\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 2, "-"],
                        Editable->True]}], ")"}], 
                  InterpretationBox[
                    StyleBox[\(5\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 5, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(1\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "-"],
                    Editable->True], 
                  InterpretationBox[
                    RowBox[{
                      StyleBox[\(3\^+\),
                        FontWeight->"Bold"], " "}],
                    Schiebe[ 3, "+"],
                    Editable->True]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["F",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_5\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TFI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4], 
                Subscript[ \[Nu], 5]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_3502 = OperatorApplyF[e\_3502]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "MakeFun (3502) \[CapitalDelta]=0 ",
  Cell[BoxData[
      \(TraditionalForm\`\((p\^2 = m\^2)\)\)]]
}], "Subsection",
  CellTags->"T1.6.6.1"],

Cell[BoxData[
    \(h\_3502 = s\_3502\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_3502\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_3502\), ",", "\"\<(3502)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", \(PP === m5^2\), "]"}], 
          ",", \({m4 :> 0, m2 :> m5, m3 :> m5}\)}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "MakeFun (3503) \[CapitalDelta]=0 ",
  Cell[BoxData[
      \(TraditionalForm\`\((p\^2 = m\^2)\)\)]]
}], "Subsection",
  CellTags->"T1.6.7.1"],

Cell[BoxData[
    \(Cayley[m5, 0, m3, m5, m5, m5]\)], "Input"],

Cell[BoxData[
    \(h\_3503 = 
      Interchange[s\_3502, 1 \[LeftRightArrow] 2, \ 3 \[LeftRightArrow] 4]
        \)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_3503\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_3503\), ",", "\"\<(3503)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", \(PP === m5^2\), "]"}], 
          ",", \({m2 :> 0, m1 :> m5, m4 :> m5}\)}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "MakeFun (3504) \[CapitalDelta]=0 ",
  Cell[BoxData[
      \(TraditionalForm\`\((p\^2 = m\^2)\)\)]]
}], "Subsection",
  CellTags->"T1.6.8.1"],

Cell[BoxData[
    \(Cayley[m5, m2, 0, m5, m5, m5]\)], "Input"],

Cell[BoxData[
    \(h\_3504 = 
      Interchange[s\_3502, 1 \[LeftRightArrow] 3, \ 2 \[LeftRightArrow] 4]
        \)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_3504\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_3504\), ",", "\"\<(3504)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", \(PP === m5^2\), "]"}], 
          ",", \({m3 :> 0, m1 :> m5, m4 :> m5}\)}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (41)", "Subsection",
  CellTags->"T1.6.9.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_41\), "=", "\n", 
        RowBox[{
          RowBox[{"2", \(\[Nu]\_5\), " ", "\[ScriptCapitalC]", " ", 
            InterpretationBox[
              StyleBox[\(5\^+\),
                FontWeight->"Bold"],
              Schiebe[ 5, "+"]], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["F",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_5\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TFI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4], 
                Subscript[ \[Nu], 5]}]]}], " ", "\[Equal]", " ", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(\[CapitalDelta]\_\(1, 3, 6\)\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\(\[Nu]\_3\), 
                        InterpretationBox[
                          StyleBox[\(3\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "+"]], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(4\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 4, "-"]], "-", 
                            InterpretationBox[
                              StyleBox[\(5\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 5, "-"]]}], ")"}]}], "+", 
                      RowBox[{\(\[Nu]\_1\), 
                        InterpretationBox[
                          StyleBox[\(1\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "+"]], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(2\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 2, "-"]], "-", 
                            InterpretationBox[
                              StyleBox[\(5\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 5, "-"]]}], ")"}]}]}], ")"}]}], "+", 
                \(\((d - \[Nu]\_1 - \[Nu]\_3 - 2  \[Nu]\_5)\) 
                  \[CapitalDelta]\_5\), "+", 
                RowBox[{\(\[CapitalDelta]\_2\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\(\[Nu]\_5\), 
                        InterpretationBox[
                          StyleBox[\(5\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 5, "+"]], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(1\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 1, "-"]], "-", 
                            InterpretationBox[
                              StyleBox[\(2\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 2, "-"]]}], ")"}]}], "+", 
                      RowBox[{\(\[Nu]\_3\), 
                        InterpretationBox[
                          StyleBox[\(3\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "+"]], 
                        InterpretationBox[
                          StyleBox[\(1\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "-"]]}], "+", \(\[Nu]\_1\), "-", 
                      \(\[Nu]\_5\)}], ")"}]}], "+", 
                RowBox[{\(\[CapitalDelta]\_4\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\(\[Nu]\_1\), 
                        InterpretationBox[
                          StyleBox[\(1\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "+"]], 
                        InterpretationBox[
                          StyleBox[\(3\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "-"]]}], "+", 
                      RowBox[{\(\[Nu]\_5\), 
                        InterpretationBox[
                          StyleBox[\(5\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 5, "+"]], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(3\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 3, "-"]], "-", 
                            InterpretationBox[
                              StyleBox[\(4\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 4, "-"]]}], ")"}]}], "+", 
                      \(\[Nu]\_3\), "-", \(\[Nu]\_5\)}], ")"}]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["F",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_5\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TFI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4], 
                Subscript[ \[Nu], 5]}]]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_41 = OperatorApplyF[e\_41]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (41)", "Subsection",
  CellTags->"T1.6.10.1"],

Cell[BoxData[
    \(h\_41 = s\_41 /. \[Nu]\_5 \[Rule] \[Nu]\_5 - 1\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_41\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_41\), ",", "\"\<(41)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            RowBox[{\(\[Nu]\_5 > 1\), "&&", 
              RowBox[{
                RowBox[{
                  StyleBox["Cayley",
                    FontColor->RGBColor[1, 0, 0.4]], "[", 
                  RowBox[{
                    InterpretationBox[\(m\_1\),
                      m1], ",", 
                    InterpretationBox[\(m\_2\),
                      m2], ",", 
                    InterpretationBox[\(m\_3\),
                      m3], ",", 
                    InterpretationBox[\(m\_4\),
                      m4], ",", 
                    InterpretationBox[\(m\_5\),
                      m5], ",", 
                    SqrtBox[
                      InterpretationBox[\(p\^2\),
                        PP]]}], "]"}], "=!=", "0"}]}], "]"}]}], "]"}]}]], 
  "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (43)", "Subsection",
  CellTags->"T1.6.11.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(p\^2\)]],
  " and ( ",
  Cell[BoxData[
      \(TraditionalForm\`m\_5\%2 == 0\)]],
  "  &&  ",
  Cell[BoxData[
      \(TraditionalForm\`m\_1\%2\)]],
  "==",
  Cell[BoxData[
      \(TraditionalForm\`m\_2\%2\)]],
  " && ",
  Cell[BoxData[
      \(TraditionalForm\`m\_3\%2\)]],
  "== ",
  Cell[BoxData[
      \(TraditionalForm\`m\_4\%2\)]],
  " ) \[DoubleLongRightArrow] \[CapitalDelta] == 0"
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_43\), "=", 
        RowBox[{
          RowBox[{\((d - \[Nu]\_1 - \[Nu]\_3 - 2  \[Nu]\_5)\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["F",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_5\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TFI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4], 
                Subscript[ \[Nu], 5]}],
              Editable->True]}], "\[Equal]", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(5\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 5, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(2\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 2, "-"],
                        Editable->True]}], ")"}]}], "+", 
                RowBox[{\(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(5\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 5, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(4\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 4, "-"],
                        Editable->True]}], ")"}]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["F",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_5\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TFI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4], 
                Subscript[ \[Nu], 5]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_43 = OperatorApplyF[e\_43]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (44)", "Subsection",
  CellTags->"T1.6.12.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(p\^2\)]],
  " and ( ",
  Cell[BoxData[
      \(TraditionalForm\`m\_5\%2 == 0\)]],
  "  &&  ",
  Cell[BoxData[
      \(TraditionalForm\`m\_1\%2\)]],
  "==",
  Cell[BoxData[
      \(TraditionalForm\`m\_2\%2\)]],
  " && ",
  Cell[BoxData[
      \(TraditionalForm\`m\_3\%2\)]],
  "== ",
  Cell[BoxData[
      \(TraditionalForm\`m\_4\%2\)]],
  " ) \[DoubleLongRightArrow] \[CapitalDelta] == 0"
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_44\), "=", 
        RowBox[{
          RowBox[{\((d - \[Nu]\_2 - \[Nu]\_4 - 2  \[Nu]\_5)\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["F",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_5\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TFI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4], 
                Subscript[ \[Nu], 5]}],
              Editable->True]}], "\[Equal]", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(5\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 5, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(1\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 1, "-"],
                        Editable->True]}], ")"}]}], "+", 
                RowBox[{\(\[Nu]\_4\), 
                  InterpretationBox[
                    StyleBox[\(4\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(5\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 5, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(3\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 3, "-"],
                        Editable->True]}], ")"}]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["F",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_5\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TFI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4], 
                Subscript[ \[Nu], 5]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_44 = OperatorApplyF[e\_44]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (45)", "Subsection",
  CellTags->"T1.6.13.1"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_45\), "=", " ", 
        RowBox[{
          RowBox[{\(\[CapitalDelta]\_\(1, 3, 6\)\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["F",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_5\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TFI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4], 
                Subscript[ \[Nu], 5]}],
              Editable->True]}], "\[Equal]", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(u\_\(6, 1, 3\)\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\(\[Nu]\_1\), 
                        InterpretationBox[
                          StyleBox[\(1\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "+"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(3\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "-"],
                          Editable->True]}], "+", 
                      RowBox[{\(\[Nu]\_5\), 
                        InterpretationBox[
                          StyleBox[\(5\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 5, "+"],
                          Editable->True], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(3\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 3, "-"],
                              Editable->True], "-", 
                            InterpretationBox[
                              StyleBox[\(4\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 4, "-"],
                              Editable->True]}], ")"}]}]}], ")"}]}], "-", 
                RowBox[{"2", \(u\_\(1, 3, 6\)\), 
                  RowBox[{"(", " ", 
                    RowBox[{
                      RowBox[{\(\[Nu]\_2\), 
                        InterpretationBox[
                          StyleBox[\(2\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 2, "+"],
                          Editable->True], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(5\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 5, "-"],
                              Editable->True], "-", 
                            InterpretationBox[
                              StyleBox[\(1\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 1, "-"],
                              Editable->True]}], ")"}]}], "+", " ", 
                      RowBox[{\(\[Nu]\_4\), 
                        InterpretationBox[
                          StyleBox[\(4\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 4, "+"],
                          Editable->True], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(5\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 5, "-"],
                              Editable->True], "-", 
                            InterpretationBox[
                              StyleBox[\(3\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 3, "-"],
                              Editable->True]}], ")"}]}], "+", 
                      \(-\[CapitalSigma]\_5\), "-", \(\[Nu]\_1\), "+", 
                      \(3  \[Nu]\_3\)}], ")"}]}], "+", 
                RowBox[{"2", \(m\_3\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\(\[Nu]\_5\), 
                        InterpretationBox[
                          StyleBox[\(5\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 5, "+"],
                          Editable->True], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(1\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 1, "-"],
                              Editable->True], "-", 
                            InterpretationBox[
                              StyleBox[\(2\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 2, "-"],
                              Editable->True]}], ")"}]}], "+", 
                      RowBox[{\(\[Nu]\_3\), 
                        InterpretationBox[
                          StyleBox[\(3\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "+"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(1\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "-"],
                          Editable->True]}], "+", \(\[Nu]\_1\), "-", 
                      \(\[Nu]\_3\)}], ")"}]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["F",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_5\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TFI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4], 
                Subscript[ \[Nu], 5]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_45 = OperatorApplyF[e\_45]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (43) and (44)", "Subsection",
  CellTags->"T1.6.14.1"],

Cell[BoxData[
    \(f\_43 = 
      MakeFun[s\_43, "\<(43)\>", 
        IFF[\[Nu]\_1 + \[Nu]\_3 + 2  \[Nu]\_5 - \((d /. dd_Symbol :> 4)\) =!= 
            0], {m5 :> 0, m4 :> m3, m2 :> m1}]\)], "Input"],

Cell[BoxData[
    \(f\_44 = MakeFun[s\_44, "\<(44)\>", {m5 :> 0, m4 :> m3, m2 :> m1}]\)], 
  "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (50)", "Subsection",
  CellTags->"T1.6.15.1"],

Cell["\<\
Subscript[f, 50] = 
   HoldForm[TFR[d_, PP_, 
      {{(n1_)?PQ, m1_}, {(n2_)?PQ, m2_}, {(n3_)?PQ, m3_}, 
       {(n4_)?PQ, m4_}, {(n5_)?PQ, m5_}}] := 
     (TComment[\"(50)\", TFIC[d, PP, 
          {{n1, m1}, {n2, m2}, {n3, m3}, {n4, m4}, 
           {n5, m5}}]]; 
        Block[{r1 = 3 - d, r2 = 4 - d, 
          r3 = 
           -(m1^2*m2^2*m3^2) + m2^4*m3^2 + m2^2*m3^4 + 
            m1^4*m4^2 - m1^2*m2^2*m4^2 - 
            m1^2*m3^2*m4^2 - m2^2*m3^2*m4^2 + 
            m1^2*m4^4 + m1^2*m2^2*m5^2 - 
            m2^2*m3^2*m5^2 - m1^2*m4^2*m5^2 + 
            m3^2*m4^2*m5^2 + m1^2*m3^2*PP - 
            m2^2*m3^2*PP - m1^2*m4^2*PP + m2^2*m4^2*PP - 
            m1^2*m5^2*PP - m2^2*m5^2*PP - m3^2*m5^2*PP - 
            m4^2*m5^2*PP + m5^4*PP + m5^2*PP^2, 
          r4 = -4 + d, r5 = -3 + d, r6 = -7 + 2*d, 
          r7 = -(m2^2*r4) + m3^2*r5 - m5^2*r5 + m4^2*r6, 
          r8 = -10 + 3*d, r9 = -m3^2 + m4^2, 
          r10 = -11 + 3*d, 
          r11 = -(m4^2*r10) - 3*m5^2*r5 + m3^2*r6, 
          r12 = 
           m2^2*r11 + m5^4*r5 + m2^4*r8 + m4^2*r4*r9 + 
            m5^2*r5*r9, r13 = -6 + d, 
          r14 = -(m4^2*r13) + m5^2*r5, 
          r15 = -3*m3^2 - m4^2*r10 + m5^2*r5, 
          r16 = 
           m3^2*r14 + m2^2*r15 + m2^4*r4 - 2*m3^4*r5 - 
            3*m4^2*m5^2*r5 + m5^4*r5 + m4^4*r8, 
          r17 = m3^2 + 2*m4^2, r18 = -1 + d, 
          r19 = -(m3^2*r13) - 2*m4^2*r18 + 2*m5^2*r5, 
          r20 = 
           2*m2^4 - 3*m3^4 + d*m3^4 - 3*m3^2*m4^2 + 
            2*m4^4 + m2^2*r19 - 2*m5^4*r5 + m5^2*r17*r5, 
          r21 = 2 + d, r22 = 
           2*m5^2 - m4^2*r21 + m2^2*r4 - m3^2*r4, 
          r23 = -7 + d, r24 = 6*m4^2 - 4*m5^2, 
          r25 = 3*m3^2 + m4^2*r18 + m5^2*r23, 
          r26 = 
           -6*m4^4 - 2*m4^2*m5^2*r23 + m3^2*r24 + 
            2*m2^2*r25 - 2*m2^4*r4 - m3^4*r4, 
          r27 = m3^4 + m3^2*m4^2 - 2*m4^4, 
          r28 = m3^2 + m4^2*r23, 
          r29 = -2*m4^2*r18 + m3^2*r21 + 2*m5^2*r23, 
          r30 = -6*m2^4 + 2*m5^2*r28 - m2^2*r29 + r27*r4, 
          r31 = m1^4*r22 + m1^2*r26 + m3^2*r30 + m1^6*r4, 
          r32 = m3 - m4, r33 = m3 + m4, 
          r34 = 2*m3^2 + m4^2, r35 = -17 + 3*d, 
          r36 = m3^2*r23 + m4^2*r35, r37 = -9 + d, 
          r38 = 17 - 3*d, 
          r39 = 5*m3^2 + m4^2*r37 + m5^2*r38, 
          r40 = 1 + d, r41 = 
           5*m4^2 - m5^2*r23 + m2^2*r4 - m3^2*r40, 
          r42 = 
           m5^4*r35 - m5^2*r36 + m2^2*r39 - 2*m1^4*r4 + 
            m2^4*r4 - r32*r33*r34*r4 + m1^2*r41, 
          r43 = m1^2 - m2^2 + m3^2 - m4^2, 
          r44 = 2*m5^2*r13 + r4*r43, 
          r45 = 
           m3^4*r12 + m1^4*r16 + m1^2*m3^2*r20 + 
            PP*r31 + PP^2*r42 + PP^3*r44 + m1^6*r7, 
          r46 = m2^2 + m4^2, r47 = 13 - 3*d, 
          r48 = m4^2*r47 - m5^2*r5 + m3^2*r6, 
          r49 = m5^4*r5 + m5^2*r32*r33*r5 + m4^2*r4*r9, 
          r50 = -18 + 4*d, r51 = -(m4^2*r13) - 3*m5^2*r5, 
          r52 = 
           2*m3^4*r5 + m4^2*m5^2*r5 + m5^4*r5 + 
            m4^4*r50 + m3^2*r51, 
          r53 = -3*m4^2 + 2*m5^2*r5, 
          r54 = 
           m4^4*r47 + 2*m3^4*r5 + m4^2*m5^2*r5 - 
            2*m5^4*r5 + m3^2*r53, 
          r55 = m4^2*r13 + 6*m3^2*r5 - 2*m5^2*r5, 
          r56 = 3*m4^2 + m3^2*r5 - m5^2*r5, 
          r57 = 7 - 2*d, 
          r58 = m3^2*r5 + 3*m5^2*r5 + m4^2*r57, 
          r59 = 
           m2^6*r4 + m2^2*m4^2*r55 + m2^4*r56 + m4^4*r58, 
          r60 = m4^2 - m3^2*r5, r61 = 10 - 3*d, 
          r62 = 2*m5^2 + 3*m4^2*r4 + m3^2*r61, 
          r63 = m4^2 + m5^2*r5, 
          r64 = 
           -4*m4^2*m5^2 + 3*m4^4*r4 - 2*m3^4*r5 + 
            2*m3^2*r63, r65 = m4^2 + m3^2*r5 - m5^2*r5, 
          r66 = 2*m3^2*r5 + 2*m5^2*r5 + m4^2*r61, 
          r67 = 3*m2^4*r4 + 2*m2^2*r65 + m4^2*r66, 
          r68 = 
           -3*m2^6*r4 - 2*m1^4*m4^2*r5 + 
            2*m4^2*m5^2*r60 + m2^4*r62 + m2^2*r64 + 
            m1^2*r67 - 3*m4^4*r4*r9, r69 = -13 + 3*d, 
          r70 = m3^2*r5 + m4^2*r69, 
          r71 = m4^2 - 3*m2^2*r4 - m3^2*r5 + m5^2*r5, 
          r72 = -5 + d, r73 = m3^2 + m5^2*r69 + m4^2*r72, 
          r74 = 
           3*m2^4*r4 - m5^4*r5 + m5^2*r70 + m1^2*r71 + 
            m2^2*r73 + 3*m4^2*r4*r9, 
          r75 = m1^2 - m2^2 + m3^2 - m4^2 - 2*m5^2, 
          r76 = 
           m2^8*r4 + m2^6*r48 + m4^4*r49 + 
            2*m1^4*m4^2*r46*r5 + m2^4*r52 + 
            m2^2*m4^2*r54 - m1^2*r59 + PP*r68 + 
            PP^2*r74 + PP^3*r4*r75, 
          r77 = m3^2 - m4^2 + m5^2, 
          r78 = -2*m2^2 + m4^2 + m5^2, 
          r79 = -2*m1^2 - m3^2 - m4^2 + m5^2, 
          r80 = 
           m1^4 + PP^2 - m1^2*r77 + m3^2*r78 + PP*r79, 
          r81 = m3^2 + m4^2, r82 = m3^2 + m4^2 + m5^2, 
          r83 = m1^2 + m3^2 - m5^2, 
          r84 = 
           m1^4 + 2*m3^2*m4^2 - m4^4 + PP^2 - m2^2*r77 + 
            m5^2*r81 - m1^2*r82 - 2*PP*r83, 
          r85 = m1 - m3, r86 = -PP + r85^2, 
          r87 = m1 + m3, r88 = -PP + r87^2, 
          r89 = m2 - m4, r90 = m2 + m4, 
          r91 = -m1^2 - m2^2 - m3^2 - m4^2 + 2*m5^2, 
          r92 = PP^2 + r85*r87*r89*r90 + PP*r91, 
          r93 = m2^2 - 3*m4^2, r94 = m4 - m5, 
          r95 = m4 + m5, r96 = 2*m3^2 - m4^2 + m5^2, 
          r97 = -m1^2 - m2^2 - 2*m3^2 + 3*m5^2, 
          r98 = 
           PP^2 + m1^2*r93 + m2^2*r94*r95 + m4^2*r96 + 
            PP*r97, r99 = -2 + d, r100 = -m1^2 + m2^2, 
          r101 = m2^2 + m1^2*r72, 
          r102 = m3^2*r101 - 2*m3^4*r4 + m2^2*r100*r4, 
          r103 = 9 - 2*d, r104 = m2^2*r23 + m3^2*r72, 
          r105 = 
           2*m1^4 + m2^4*r103 + m1^2*r104 + m2^2*m3^2*r5, 
          r106 = -9 + 2*d, r107 = m1 - m2, 
          r108 = m1 + m2, 
          r109 = -(r106*r107*r108) + m3^2*r4, 
          r110 = 
           m2^2*r102 + m4^2*r105 + m4^4*r109 - m4^6*r4, 
          r111 = m1^2 + m2^2, 
          r112 = m2^2*r4 - 2*m1^2*r72, r113 = -19 + 5*d, 
          r114 = m3^2*r112 + m3^4*r113 + m2^2*r111*r4, 
          r115 = m3^2*r5 + m2^2*r6, 
          r116 = 
           2*m1^4 + 2*m1^2*r115 + m2^4*r13 + m3^4*r72 - 
            2*m2^2*m3^2*r99, 
          r117 = m2^2*r106 + m3^2*r4 - m1^2*r99, 
          r118 = 
           m2^2*r114 - m4^2*r116 + m4^4*r117 - m4^6*r6, 
          r119 = -(m2^2*r5) + 2*m3^2*r57 + m1^2*r72, 
          r120 = m1^2*r18 + 2*m3^2*r4 - m2^2*r5, 
          r121 = m2^2*r119 + m4^2*r120 + 2*m4^4*r5, 
          r122 = m3^2*r5 + m4^2*r72, 
          r123 = -2*m2^2*r4 - m4^2*r69 + m3^2*r72, 
          r124 = 
           m2^2*r122 + m1^2*r123 + 2*m2^4*r4 + 
            2*m4^2*r32*r33*r4, 
          r125 = -3*m4^2 + m2^2*r4 - m3^2*r72, 
          r126 = m3^2 - m4^2*r72, 
          r127 = 
           -2*m3^2*m4^2 - 2*m1^2*r125 - 2*m2^2*r126 + 
            m4^4*r18 - 4*m2^4*r4 + m3^4*r72, 
          r128 = 
           -(m2^2*r18) + 2*m4^2*r5 + m1^2*r72 + 
            2*m3^2*r72, 
          r129 = 
           m5^2*r127 - m5^4*r128 - r124*r32*r33 + 
            m5^6*r72, r130 = -m1^2 + m2^2 + m3^2 - m4^2, 
          r131 = -14 + 3*d, 
          r132 = 
           -(m3^2*r13) + m4^2*r131 + m1^2*r4 + 5*m2^2*r4, 
          r133 = -2*m5^4 + m5^2*r132 + r130*r32*r33*r4, 
          r134 = 
           m5^2*r118 + m5^4*r121 + PP*r129 + PP^2*r133 + 
            r110*r32*r33 - 2*m5^2*PP^3*r4 + 
            m5^6*r5*r89*r90, r135 = m3 - m4 - m5, 
          r136 = m3 + m4 - m5, r137 = m3 - m4 + m5, 
          r138 = m3 + m4 + m5, r139 = -10 + d, 
          r140 = 13 - 2*d, r141 = d*m4^2 + 2*m3^2*r5, 
          r142 = -8 + d, 
          r143 = m4^2*r142 - m3^2*r99 + m5^2*r99, 
          r144 = 
           m3^2*m4^2*r139 + m4^4*r140 - m5^2*r141 + 
            m2^2*r143 + m3^4*r5 + m5^4*r5, 
          r145 = 3*m3^4 - 12*m3^2*m4^2 + 7*m4^4, 
          r146 = 3*m3^2 + 4*m4^2, r147 = m3^2 - 11*m4^2, 
          r148 = 
           7*m3^4 + 10*m3^2*m4^2 - 17*m4^4 - 5*m5^4 - 
            2*m5^2*r147, r149 = m2^2 + 2*m3^2 + m4^2, 
          r150 = 
           -m2^4 + m2^2*m3^2 + m3^4 + 3*m2^2*m4^2 + 
            m3^2*m4^2 - m4^4 + m5^4 - m5^2*r149, 
          r151 = 
           3*m3^6 + 6*m3^4*m4^2 - 17*m3^2*m4^4 + 
            8*m4^6 + 3*m5^6 - m5^2*r145 - m5^4*r146 + 
            m2^2*r148 - d*r150*r77 + 6*m2^4*r94*r95, 
          r152 = -(m3^2*m4) + m4^3, r153 = 11 - 2*d, 
          r154 = -(d*m3^2*m4^2) + m4^4*r153 + m3^4*r5, 
          r155 = m4^2 + m3^2*r5, 
          r156 = r142*r32*r33 + m5^2*r99, 
          r157 = 11 - 3*d, r158 = -8 + 3*d, 
          r159 = m5^2*r158 + m4^2*r21, 
          r160 = 
           -3*m3^4 + 2*m4^2*m5^2*r106 + m5^4*r157 + 
            m3^2*r159 - m4^4*r18, 
          r161 = 
           -2*m2^6 + m5^2*r154 - 2*m5^4*r155 + 
            m2^4*r156 + m2^2*r160 + r152^2*r4 + m5^6*r5, 
          r162 = m4^2*r23 - 3*m3^2*r72, 
          r163 = -4*m2^2 + m4^2*r18 + m3^2*r23, 
          r164 = -m3^4 + m4^4, 
          r165 = 
           -(m2^4*r13) + m2^2*r162 + m1^2*r163 + 
            r164*r4 + m1^4*r99, r166 = -19 + 4*d, 
          r167 = m4^2 + m3^2*r72, r168 = m3^2 + m4^2*r69, 
          r169 = -2*r168 + 4*m2^2*r4, 
          r170 = 
           -m3^4 + m2^4*r13 - m4^4*r166 + 2*m2^2*r167 + 
            m1^2*r169 + 2*m3^2*m4^2*r72 + m1^4*r99, 
          r171 = m1^2 - m2^2 + 4*m4^2, 
          r172 = 
           -(m5^2*r170) + r165*r32*r33 + m5^6*r72 - 
            m5^4*r171*r72, r173 = m2^2 + m3^2, 
          r174 = -(m4^2*r13) + r131*r173 + m1^2*r8, 
          r175 = m5^2*r174 - 2*m5^4*r4 + r130*r32*r33*r4, 
          r176 = 
           2*m1^6*m4^2 + m1^4*r144 + m1^2*r151 + 
            m3^2*r161 + PP*r172 + PP^2*r175 - 
            2*m5^2*PP^3*r4, r177 = m3^2 + m4^2 - m5^2, 
          r178 = -2*m1^2 + m3^2 - m4^2 + m5^2, 
          r179 = -m3^2 + m4^2 + m5^2, 
          r180 = m2^2*r177 + m4^2*r178 + PP*r179, 
          r181 = -2*m2^2 - m3^2 + m4^2 + m5^2, 
          r182 = m1^2*r177 + m3^2*r181 + PP*r77, 
          r183 = -(m2^2*r13) - 2*m3^2*r5, 
          r184 = 
           m1^2*r183 - 2*m3^4*r4 + m2^4*r57 + m1^4*r72 - 
            m2^2*m3^2*r99, r185 = 3*m2^2*r4 + m3^2*r72, 
          r186 = 
           m2^4*r106 + 2*m1^2*r185 + m1^4*r40 - 
            2*m2^2*m3^2*r6, r187 = r111*r13 - m3^2*r4, 
          r188 = 
           m2^2*r184 + m4^2*r186 - m4^4*r187 + m4^6*r4, 
          r189 = 2*m1^2 + m3^2*r10 + 2*m2^2*r5, 
          r190 = m2^2*r5 + m3^2*r72 + 2*m1^2*r99, 
          r191 = m2^2*r189 - m4^2*r190 - m4^4*r5, 
          r192 = -(m3^2*r5) - m4^2*r72, 
          r193 = 2*m2^2*r4 + m4^2*r69 - m3^2*r72, 
          r194 = 
           m2^2*r192 + m1^2*r193 - 2*m2^4*r4 + 
            2*m4^2*r4*r9, r195 = m2^2 + m4^2 + m3^2*r72, 
          r196 = -(m3^2*r6) - m4^2*r72, 
          r197 = 
           m2^4*r157 + 2*m1^2*r195 + 2*m2^2*r196 + 
            2*m4^2*r17*r4 + m1^4*r72, 
          r198 = 
           m4^2*r10 - 2*m2^2*r5 + 2*m1^2*r72 + m3^2*r72, 
          r199 = 
           r107*r108*r194 + m5^2*r197 - m5^4*r198 + 
            m5^6*r72, r200 = m1^2 - m2^2 - m3^2 + m4^2, 
          r201 = m1^2 + m2^2 + m3^2 + 5*m4^2, 
          r202 = 2*m5^4 - r107*r108*r200 - m5^2*r201, 
          r203 = 
           r107*r108*r110 - m5^2*r188 - m5^4*r191 + 
            PP*r199 + 2*m5^2*PP^3*r4 + PP^2*r202*r4 + 
            m5^6*r5*r89*r90, r204 = m4^2 + m5^2, 
          r205 = -2*m5^2*r5 + m4^2*r99, r206 = 18 - 5*d, 
          r207 = m4^2*r206 + m5^2*r8 - 2*m3^2*r99, 
          r208 = 
           m2^2*r207 + 2*m2^4*r4 - m3^4*r5 - 
            m3^2*r204*r5 + r205*r94*r95, 
          r209 = 2*m3^2*m4^2 + m4^4*r142 + 2*m3^4*r5, 
          r210 = -(m4^2*r18) + m3^2*r5, r211 = 3 - 2*d, 
          r212 = m5^2 + m4^2*r157 + m3^2*r211, 
          r213 = m4^2*r106 + m3^2*r4, 
          r214 = 
           -(d*m3^4) + 10*m3^2*m4^2 - 2*d*m3^2*m4^2 - 
            2*m4^4 + d*m4^4 + 2*m5^2*r213 - 2*m5^4*r4, 
          r215 = 
           d*m3^2*m4^4 - 2*m4^6 - m5^2*r209 + 
            m5^4*r210 + m2^4*r212 + m2^2*r214 + 
            m2^6*r4 + m5^6*r5 - m3^4*m4^2*r99, 
          r216 = m4^2*r13 + m3^2*r99, 
          r217 = m3^2*r5 - m4^2*r72, r218 = 13 - 4*d, 
          r219 = 3*m3^2 + m5^2*r218 + m4^2*r23, 
          r220 = m4^2*r142 + m5^2*r61, 
          r221 = 
           2*m3^4 + 2*m4^2*m5^2 - m4^4*r13 + m3^2*r220 + 
            2*m5^4*r6, r222 = 
           m4^2*m5^2*r216 + m5^4*r217 + m2^4*r219 + 
            m2^2*r221 - m5^6*r5 + m2^6*r99, 
          r223 = -(m3^2*r10) + m4^2*r5, 
          r224 = 
           4*m3^2*m4^2 + m1^2*r122 + m4^4*r13 + 
            m2^2*r223 + m1^4*r4 - m2^4*r4 - m3^4*r99, 
          r225 = -m3^2 + m2^2*r72 - m4^2*r72, 
          r226 = -2*m4^2*r106 + 2*m3^2*r69, 
          r227 = 
           m1^4*r106 + m2^4*r106 - m4^4*r13 - 
            2*m1^2*r225 + m2^2*r226 - 4*m3^2*m4^2*r4 - 
            m3^4*r99, r228 = m3^2 + 3*m4^2, 
          r229 = 
           r107*r108*r224 - m5^2*r227 + m5^6*r72 - 
            m5^4*r228*r72, 
          r230 = m4^2*r131 + m1^2*r4 + m2^2*r4 + m3^2*r8, 
          r231 = 
           2*m5^4*r106 - m5^2*r230 - r107*r108*r200*r4, 
          r232 = 
           m1^4*r208 - m1^2*r215 - m3^2*r222 + PP*r229 + 
            PP^2*r231 + 2*m5^2*PP^3*r4 + m1^6*r7, 
          r233 = -2*m3^2 + m4^2, 
          r234 = -m2^2 + m4^2 - 2*m5^2, 
          r235 = -m1^2 + m2^2 + m5^2, 
          r236 = 
           m1^4 + m5^4 + m2^2*r233 + m1^2*r234 + 
            PP*r235 - m5^2*r46, r237 = -m2^2 + m5^2, 
          r238 = m2^2 + m3^2 - 2*m4^2 + m5^2, 
          r239 = -m1^2 + m2^2 - m5^2, 
          r240 = m1^4 + m3^2*r237 - m1^2*r238 + PP*r239, 
          r241 = m3^2 + 3*m4^2 - m5^2, 
          r242 = -3*m3^2 - m4^2 + m5^2, 
          r243 = 
           m5^4 - m5^2*r228 + m1^2*r241 + m2^2*r242 + 
            2*PP*r32*r33 + 2*m4^2*r9, 
          r244 = 2*m1^2 + m2^2 + m4^2, 
          r245 = -2*m1^2 - m3^2 - m4^2 + 3*m5^2, 
          r246 = 
           m1^4 - 3*m2^2*m3^2 + m5^4 + PP^2 + 
            m4^2*r173 - m5^2*r244 + PP*r245, 
          r247 = -2*m3^2 + m4^2 + m5^2, 
          r248 = 
           -m2^4 - m4^2*m5^2 + PP*r235 + m2^2*r247 + 
            m1^2*r46, r249 = m2^2*r153 + m1^2*r5, 
          r250 = 2*m1^2 + m2^2*r4, 
          r251 = m2^2*r13 + m1^2*r139, 
          r252 = r107*r108*r250 + m3^2*r251 - m3^4*r4, 
          r253 = m1^2*r140 + 2*m3^2*r4 + m2^2*r72, 
          r254 = 
           m3^4*r249 + m4^2*r252 + m4^4*r253 - m4^6*r4 + 
            m2^2*m3^2*r100*r99, r255 = 2*m3^2 + m4^2*r13, 
          r256 = -2*m3^2*r5 + m2^2*r99 - m4^2*r99, 
          r257 = -11 + 2*d, 
          r258 = 
           -(m4^4*r106) + 2*m3^2*m4^2*r23 - m3^4*r257, 
          r259 = 4*m4^2 + m3^2*r13, 
          r260 = 
           -2*m2^4 + 6*m3^2*m4^2 + 2*m2^2*r259 + 
            m4^4*r37 + m3^4*r5, 
          r261 = 
           -5*m3^4*m4^2 + m2^4*r255 + m1^4*r256 + 
            m2^2*r258 + m1^2*r260 - m2^6*r4 + m3^6*r5 + 
            m4^6*r6 - 3*m3^2*m4^4*r72, 
          r262 = m3^4 + m3^2*m4^2 + m4^4, 
          r263 = 
           m1^4 + m2^4 - 2*r262 + m1^2*r81 + m2^2*r81, 
          r264 = m1^2 + m2^2 - m3^2 - m4^2, 
          r265 = -(r13*r32*r33) + m2^2*r99, 
          r266 = 
           -2*m1^2*r265 - r32^2*r33^2*r4 + 
            2*m2^2*r13*r9 + m1^4*r99 + m2^4*r99, 
          r267 = m3^2*r13 + m4^2*r4, r268 = -16 + 3*d, 
          r269 = m3^2*r13 - m4^2*r268 + m2^2*r4, 
          r270 = 
           2*m2^2*r267 + 2*m1^2*r269 + 
            r13*r228*r32*r33 + m2^4*r61 + m1^4*r99, 
          r271 = 2*m2^2 + m4^2*r23 - m3^2*r72, 
          r272 = 
           -(m5^2*r270) - 2*m5^4*r271 + r266*r32*r33, 
          r273 = 2*m3^2 - 2*m4^2 - m5^2, 
          r274 = m1^2*r5 - m2^2*r5 + r273*r72, 
          r275 = 
           r200*r254 + m5^2*r261 + PP*r272 + 
            2*m5^2*PP^2*r274 + m5^4*r263*r5 - 
            m5^6*r264*r5, 
          r276 = 
           -2*m2^2*m3^2 - 2*m3^4 + 3*m3^2*m4^2 - m4^4 + 
            3*m3^2*m5^2 + 2*m4^2*m5^2 - m5^4 + 
            m1^2*r177 + PP*r77, 
          r277 = -(m4^2*r106) + m2^2*r4 - m3^2*r5, 
          r278 = 
           -2*m3^4 - m3^2*m4^2*r13 + m2^2*r196 + 
            m2^4*r4 + m4^4*r4, 
          r279 = m4^2*r131 + m3^2*r8, 
          r280 = 
           m2^2*r279 - 2*m2^4*r4 + m4^2*r32*r33*r99, 
          r281 = m1^4*r277 + m2^2*r278 + m1^2*r280, 
          r282 = 11*m3^2 - 3*d*m3^2 + 9*m4^2 - 2*d*m4^2, 
          r283 = 15 - 4*d, 
          r284 = m4^2*r106 + m2^2*r283 + m3^2*r5, 
          r285 = m4^4*r13 + 4*m3^2*m4^2*r5 - m3^4*r99, 
          r286 = -m4^2 + m3^2*r5, 
          r287 = -(m4^2*r5) + m3^2*r6, 
          r288 = 2*m2^2*r287 + m2^4*r72 - 2*r286*r81, 
          r289 = 
           -2*m3^2*m4^4 + m2^4*r282 + m1^4*r284 + 
            m2^2*r285 + m1^2*r288 - m4^6*r4 + m1^6*r5 + 
            m2^6*r6 + m3^4*m4^2*r99, 
          r290 = 2*m2^2 - m3^2 - m4^2, 
          r291 = 
           2*m1^4 + 2*m2^4 - m3^4 - m4^4 + m1^2*r290 - 
            m2^2*r81, r292 = m2^2 - m3^2 + m4^2, 
          r293 = 
           m1^4*r4 + m2^4*r4 - 2*m1^2*r292*r4 + 
            2*m2^2*r4*r9 - r32^2*r33^2*r99, 
          r294 = m3^2*r13 - m4^2*r131, 
          r295 = m2^2*r13 - r4*r81, 
          r296 = 
           m1^4*r131 + 2*m2^2*r294 - 2*m1^2*r295 - 
            2*m3^2*m4^2*r4 + m4^4*r8 - m2^4*r99 - 
            m3^4*r99, r297 = 2*m4^2 - m2^2*r5 + m1^2*r72, 
          r298 = 
           r107*r108*r293 - m5^2*r296 + 2*m5^4*r297, 
          r299 = -2*m5^2*r32*r33*r5 + 2*m5^4*r72, 
          r300 = 
           r130*r281 - m5^2*r289 + PP*r298 + PP^2*r299 - 
            m5^6*r264*r5 + m5^4*r291*r5, 
          r301 = 
           -2*m2^2*m3^2 - m3^2*m4^2 + m4^4 + m5^4 - 
            m5^2*r17 + m1^2*r177 + PP*r77, 
          r302 = m2^2 + 2*m3^2 - 3*m4^2, 
          r303 = -m1^2 + m2^2 - 3*m5^2, 
          r304 = 
           m1^4 - m2^2*m4^2 - m5^4 + m5^2*r149 - 
            m1^2*r302 + PP*r303, 
          r305 = 
           -((r45*TBI[d, PP, {{1, m4}, {1, m2}}]*
                 TBI[d, PP, {{2, m1}, {1, m3}}])/(4*r3)) 
+ (r76*TBI[d, PP, {{1, m3}, {1, m1}}]*
               TBI[d, PP, {{2, m2}, {1, m4}}])/(4*r3) - 
            (r80*TBI[d, PP, {{2, m1}, {1, m3}}]*
               TBI[d, PP, {{2, m2}, {1, m4}}])/2 - 
            (r45*TBI[d, PP, {{1, m4}, {1, m2}}]*
               TBI[d, PP, {{2, m3}, {1, m1}}])/(4*r3) - 
            (r84*TBI[d, PP, {{2, m2}, {1, m4}}]*
               TBI[d, PP, {{2, m3}, {1, m1}}])/2 - 
            r86*r88*TBI[d, PP, {{1, m4}, {1, m2}}]*
             TBI[d, PP, {{2, m3}, {2, m1}}] + 
            (r76*TBI[d, PP, {{1, m3}, {1, m1}}]*
               TBI[d, PP, {{2, m4}, {1, m2}}])/(4*r3) - 
            (r92*TBI[d, PP, {{2, m1}, {1, m3}}]*
               TBI[d, PP, {{2, m4}, {1, m2}}])/2 - 
            (r98*TBI[d, PP, {{2, m3}, {1, m1}}]*
               TBI[d, PP, {{2, m4}, {1, m2}}])/2 - 
            r86*r88*TBI[d, PP, {{1, m4}, {1, m2}}]*
             TBI[d, PP, {{3, m1}, {1, m3}}] - 
            r86*r88*TBI[d, PP, {{1, m4}, {1, m2}}]*
             TBI[d, PP, {{3, m3}, {1, m1}}] - 
            r3*TFI[r99, PP, 
              {{1, m1}, {1, m2}, {1, m3}, {1, m4}, 
               {1, m5}}] - 
            (r134*TVI[d, PP, 
                {{1, m5}, {1, m1}, {2, m4}, {1, m3}}])/
             (4*r3) - (r135*r136*r137*r138*
               TVI[d, PP, 
                {{1, m5}, {1, m1}, {2, m4}, {2, m3}}])/2 
- (r176*TVI[d, PP, {{1, m5}, {1, m2}, {2, m3}, 
                 {1, m4}}])/(4*r3) + 
            (r180*TVI[d, PP, 
                {{1, m5}, {1, m2}, {2, m3}, {2, m4}}])/2 
- r182*TVI[d, PP, {{1, m5}, {1, m2}, {3, m3}, 
               {1, m4}}] + 
            (r203*TVI[d, PP, 
                {{1, m5}, {1, m3}, {2, m2}, {1, m1}}])/
             (4*r3) + (r232*
               TVI[d, PP, 
                {{1, m5}, {1, m4}, {2, m1}, {1, m2}}])/
             (4*r3) + (r236*
               TVI[d, PP, 
                {{1, m5}, {1, m4}, {2, m1}, {2, m2}}])/2 
+ r240*TVI[d, PP, {{1, m5}, {1, m4}, {3, m1}, 
               {1, m2}}] + 
            (r45*TVI[d, PP, 
                {{1, m5}, {2, m1}, {1, m4}, {1, m3}}])/
             (4*r3) + (r86*r88*
               TVI[d, PP, 
                {{1, m5}, {2, m1}, {1, m4}, {2, m3}}])/2 
+ (r92*TVI[d, PP, {{1, m5}, {2, m1}, {2, m4}, 
                 {1, m3}}])/2 - 
            (r76*TVI[d, PP, 
                {{1, m5}, {2, m2}, {1, m3}, {1, m4}}])/
             (4*r3) - (r243*
               TVI[d, PP, 
                {{1, m5}, {2, m2}, {2, m3}, {1, m4}}])/2 
+ (r45*TVI[d, PP, {{1, m5}, {2, m3}, {1, m2}, 
                 {1, m1}}])/(4*r3) + 
            (r86*r88*TVI[d, PP, 
                {{1, m5}, {2, m3}, {1, m2}, {2, m1}}])/2 
+ (r246*TVI[d, PP, {{1, m5}, {2, m3}, {2, m2}, 
                 {1, m1}}])/2 - 
            (r76*TVI[d, PP, 
                {{1, m5}, {2, m4}, {1, m1}, {1, m2}}])/
             (4*r3) + (r248*
               TVI[d, PP, 
                {{1, m5}, {2, m4}, {2, m1}, {1, m2}}])/2 
+ r86*r88*TVI[d, PP, {{1, m5}, {3, m1}, {1, m4}, 
               {1, m3}}] + 
            r86*r88*TVI[d, PP, 
              {{1, m5}, {3, m3}, {1, m2}, {1, m1}}] + 
            (r275*TVI[d, PP, 
                {{2, m5}, {1, m1}, {1, m4}, {1, m3}}])/
             (4*r3) + (r276*
               TVI[d, PP, 
                {{2, m5}, {1, m1}, {1, m4}, {2, m3}}])/2 
- (r180*TVI[d, PP, {{2, m5}, {1, m1}, {2, m4}, 
                 {1, m3}}])/2 - 
            (r275*TVI[d, PP, 
                {{2, m5}, {1, m2}, {1, m3}, {1, m4}}])/
             (4*r3) + (r180*
               TVI[d, PP, 
                {{2, m5}, {1, m2}, {1, m3}, {2, m4}}])/2 
- (r276*TVI[d, PP, {{2, m5}, {1, m2}, {2, m3}, 
                 {1, m4}}])/2 - 
            (r300*TVI[d, PP, 
                {{2, m5}, {1, m3}, {1, m2}, {1, m1}}])/
             (4*r3) - (r240*
               TVI[d, PP, 
                {{2, m5}, {1, m3}, {1, m2}, {2, m1}}])/2 
- (r236*TVI[d, PP, {{2, m5}, {1, m3}, {2, m2}, 
                 {1, m1}}])/2 + 
            (r300*TVI[d, PP, 
                {{2, m5}, {1, m4}, {1, m1}, {1, m2}}])/
             (4*r3) + (r236*
               TVI[d, PP, 
                {{2, m5}, {1, m4}, {1, m1}, {2, m2}}])/2 
+ (r240*TVI[d, PP, {{2, m5}, {1, m4}, {2, m1}, 
                 {1, m2}}])/2 + 
            (r182*TVI[d, PP, 
                {{2, m5}, {2, m1}, {1, m4}, {1, m3}}])/2 
- (r301*TVI[d, PP, {{2, m5}, {2, m2}, {1, m3}, 
                 {1, m4}}])/2 - 
            (r304*TVI[d, PP, 
                {{2, m5}, {2, m3}, {1, m2}, {1, m1}}])/2 
+ (r248*TVI[d, PP, {{2, m5}, {2, m4}, {1, m1}, 
                 {1, m2}}])/2, r306}, 
         r306 = Factor /@ r305/(PP*r1*r2); r306]) /; 
         n1 === 1 && n2 === 1 && n3 === 1 && n4 === 1 && n5 === 1 &&  
       MatchQ[d, _Symbol + _Integer?Positive] && PP =!= 0 &&
       Cayley[m1, m2, m3, m4, m5, Sqrt[PP]] =!= 0]; \
\>", "Input",
  Editable->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (51)", "Subsection",
  CellTags->"T1.6.16.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_51\), "=", 
        RowBox[{
          RowBox[{\(\[CapitalDelta]\_\(1, 3, 4\)\), \(\[Nu]\_1\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{"2", \(m\_3\%2\), \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(1\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 1, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(4\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 4, "-"],
                        Editable->True]}], ")"}]}], "+", 
                RowBox[{\(u\_\(4, 1, 3\)\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(3\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 3, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(4\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 4, "-"],
                        Editable->True]}], ")"}]}], "+", 
                \(\(u\_\(1, 3, 4\)\) \((d - \[Nu]\_1 - 2  \[Nu]\_3)\)\), "+", 
                \(2 \( m\_3\%2\) \((\[Nu]\_1 - \[Nu]\_3)\)\)}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_51 = OperatorApplyV[e\_51]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (51)", "Subsection",
  CellTags->"T1.6.17.1"],

Cell[BoxData[
    \(s\_51\)], "Input"],

Cell[BoxData[
    \(h\_51 = s\_51 /. \[Nu]\_1 \[Rule] \[Nu]\_1 - 1\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_51\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_51\), ",", "\"\<(51)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            \(\[Nu]\_1 > 1 && \[CapitalDelta]\_\(1, 3, 4\) =!= 0\), "]"}]}], 
        "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTVIRecursion[f\_51]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (52)", "Subsection",
  CellTags->"T1.6.18.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_52\), "=", 
        RowBox[{
          RowBox[{\(\[CapitalDelta]\_\(2, 4, 6\)\), \(\[Nu]\_2\), 
            InterpretationBox[
              StyleBox[\(2\^+\),
                FontWeight->"Bold"],
              Schiebe[ 2, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{"2", \(m\_1\%2\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(4\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 4, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(2\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 2, "-"],
                        Editable->True]}], ")"}]}], "+", 
                RowBox[{"2", \(m\_3\%2\), \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(4\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 4, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(2\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 2, "-"],
                        Editable->True]}], ")"}]}], "-", 
                RowBox[{"2", \(m\_4\%2\), \(\[Nu]\_4\), 
                  InterpretationBox[
                    StyleBox[\(4\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(2\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "-"],
                    Editable->True]}], "+", 
                RowBox[{\((2  m\_2\%2 - u\_\(4, 2, 6\))\), \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(4\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "-"],
                    Editable->True]}], "-", 
                RowBox[{\((\[CapitalSigma]\_4 + 2)\), 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(4\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 4, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(2\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 2, "-"],
                        Editable->True]}], ")"}]}], "+", 
                \(\(u\_\(2, 4, 6\)\) \((d - 3  \[Nu]\_2)\)\), "+", 
                \(2 \( m\_4\%2\) \((\[Nu]\_4 - \[Nu]\_2)\)\)}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_52 = OperatorApplyV[e\_52]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (52)", "Subsection",
  CellTags->"T1.6.19.1"],

Cell[BoxData[
    \(h\_52 = \[Nu]explicit[s\_52] /. \[Nu]\_2 \[Rule] \[Nu]\_2 - 1\)], 
  "Input"],

Cell[BoxData[
    RowBox[{\(f\_52\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_52\), ",", "\"\<(52)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            \(\[Nu]\_2 > 1 && \[CapitalDelta]\_\(2, 4, 6\) =!= 0\), "]"}]}], 
        "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTVIRecursion[f\_52]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (53)", "Subsection",
  CellTags->"T1.6.20.1"],

Cell[BoxData[
    \(h\_53 = 
      Interchange[s\_51, 1 \[LeftRightArrow] 3] /. 
        \[Nu]\_3 \[Rule] \[Nu]\_3 - 1\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_53\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_53\), ",", "\"\<(53)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            \(\[Nu]\_3 > 1 && \[CapitalDelta]\_\(3, 1, 4\) =!= 0\), "]"}]}], 
        "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTVIRecursion[f\_53]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (54)", "Subsection",
  CellTags->"T1.6.21.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_54\), "=", 
        RowBox[{
          RowBox[{"2", \(m\_3\%2\), \(\[Nu]\_3\), 
            InterpretationBox[
              StyleBox[\(3\^+\),
                FontWeight->"Bold"],
              Schiebe[ 3, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(u\_\(4, 1, 3\)\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(4\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 4, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(3\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 3, "-"],
                        Editable->True]}], ")"}]}], "+", "\n", "\t\t\t\t", 
                \((d - \[Nu]\_1 - 2  \[Nu]\_3)\)}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_54 = OperatorApplyV[e\_54]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (54)", "Subsection",
  CellTags->"T1.6.22.1"],

Cell[BoxData[
    \(h\_54 = s\_54 /. \[Nu]\_3 \[Rule] \[Nu]\_3 - 1\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_54\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_54\), ",", "\"\<(54)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", " ", 
            \(\[Nu]\_3 > 1 && \ m3 =!= 0\), "]"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTVIRecursion[f\_54]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (55)", "Subsection",
  CellTags->"T1.6.23.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_55\), "=", 
        RowBox[{
          RowBox[{"2", " ", \(m\_4\%2\), \(\[Nu]\_4\), 
            InterpretationBox[
              StyleBox[\(4\^+\),
                FontWeight->"Bold"],
              Schiebe[ 4, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(u\_\(3, 1, 4\)\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(u\_\(6, 2, 4\)\), \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True]}], "+", "\n", "\t\t\t\t", 
                RowBox[{\(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(3\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 3, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(4\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 4, "-"],
                        Editable->True]}], ")"}]}], "-", 
                RowBox[{\(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(4\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "-"],
                    Editable->True]}], "+", 
                \((d - \[Nu]\_1 - \[Nu]\_2 - 2  \[Nu]\_4)\)}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_55 = OperatorApplyV[e\_55]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (55)", "Subsection",
  CellTags->"T1.6.24.1"],

Cell[BoxData[
    \(h\_55 = s\_55 /. \[Nu]\_4 \[Rule] \[Nu]\_4 - 1\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_55\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_55\), ",", "\"\<(55)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            \(\[Nu]\_4 > 1 && \ m4 =!= 0\  && \ \[Nu]\_3 > 1\), "]"}]}], 
        "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTVIRecursion[f\_55]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (56)", "Subsection",
  CellTags->"T1.6.25.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_56\), "=", 
        RowBox[{
          RowBox[{"2", " ", \(m\_4\%2\), \(\[Nu]\_4\), 
            InterpretationBox[
              StyleBox[\(4\^+\),
                FontWeight->"Bold"],
              Schiebe[ 4, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(-2\), \(m\_1\%2\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(u\_\(6, 2, 4\)\), \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True]}], "-", 
                RowBox[{"2", \(m\_3\%2\), \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}], "-", "\n", "\t\t\t\t", 
                RowBox[{\(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(4\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "-"],
                    Editable->True]}], "+", 
                \((2  d - 2  \[Nu]\_1 - \[Nu]\_2 - 2  \[Nu]\_3 - 2  \[Nu]\_4)
                  \)}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_56 = OperatorApplyV[e\_56]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (56)", "Subsection",
  CellTags->"T1.6.26.1"],

Cell[BoxData[
    \(h\_56 = s\_56 /. \[Nu]\_4 \[Rule] \[Nu]\_4 - 1\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_56\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_56\), ",", "\"\<(56)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            \(\[Nu]\_4 > 1 && \ m4 =!= 0\  && \ \[Nu]\_3 === 1\), "]"}]}], 
        "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTVIRecursion[f\_56]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (59)", "Subsection",
  CellTags->"T1.6.27.1"],

Cell[TextData[{
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta]\_\(1, 3, 4\)\)]],
  "== 0 but all  ",
  Cell[BoxData[
      \(m\_i\%2\)]],
  " != 0"
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_59\), "=", 
        RowBox[{
          RowBox[{
          \((2 \( m\_1\%2\) \(u\_\(1, 3, 4\)\) 
                  \((d - \[Nu]\_1 - 2  \[Nu]\_3)\) + 
                \((\[Nu]\_3 - 1)\) u\_\(4, 1, 3\)\%2 + \n\t
                4 \( m\_1\%2\) \(m\_3\%2\) \((\[Nu]\_1 - \[Nu]\_3)\))\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(u\_\(4, 1, 3\)\), \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(4\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "-"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(4\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 4, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(1\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 1, "-"],
                        Editable->True]}], ")"}]}], "-", "\n", "\t", 
                RowBox[{"4", \(m\_1\%2\), \(m\_3\%2\), \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(1\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "-"],
                    Editable->True]}], "+", 
                RowBox[{
                \((4 \( m\_1\%2\) m\_3\%2 + u\_\(4, 1, 3\)\%2)\), 
                  \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(4\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "-"],
                    Editable->True]}], "+", 
                RowBox[{\(u\_\(4, 1, 3\)\), \((\[Nu]\_3 - 1)\), 
                  InterpretationBox[
                    StyleBox[\(1\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "-"],
                    Editable->True]}], "-", "\n", "\t\t", 
                RowBox[{
                \((d - 2  \[Nu]\_1 - \[Nu]\_3 + 1)\), \(u\_\(4, 1, 3\)\), 
                  InterpretationBox[
                    StyleBox[\(3\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "-"],
                    Editable->True]}], "+", 
                RowBox[{
                \((d - 2  \[Nu]\_1 - 2  \[Nu]\_3 + 1)\), \(u\_\(4, 1, 3\)\), 
                  InterpretationBox[
                    StyleBox[\(4\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "-"],
                    Editable->True]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_59 = OperatorApplyV[e\_59]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (59)", "Subsection",
  CellTags->"T1.6.28.1"],

Cell[BoxData[
    \(h\_59 = s\_59\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_59\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_59\), ",", "\"\<(59)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            \(\[CapitalDelta]\_\(1, 3, 4\) == 0 && 
              \((4\ m\_3\%2\ m\_1\%2 - 4\ m\_3\%2\ \[Nu]\_3\ m\_1\%2 + 
                    2\ d\ u\_\(1, 3, 4\)\ m\_1\%2 - 
                    4\ \[Nu]\_3\ u\_\(1, 3, 4\)\ m\_1\%2 - 
                    2\ u\_\(1, 3, 4\)\ m\_1\%2 + 
                    \[Nu]\_3\ u\_\(4, 1, 3\)\%2 - u\_\(4, 1, 3\)\%2)
                  \)\ \  =!= 0 && n1\  === \ 1 && \ n2\  === \ 1 && \ 
              n4\  === \ 1 && MatchQ[d, _Integer]\), "]"}]}], "]"}]}]], 
  "Input"],

Cell["CheckTVIRecursion not applicable .", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (60) ", "Subsection",
  CellTags->"T1.6.29.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_60\), "=", 
        RowBox[{
          RowBox[{"2", \(m\_1\%2\), \(\[Nu]\_1\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(u\_\(4, 1, 3\)\), \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(4\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 4, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(1\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 1, "-"],
                        Editable->True]}], ")"}]}], "+", "\n", "\t\t\t\t", 
                \((d - \[Nu]\_3 - 2  \[Nu]\_1)\)}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_60 = OperatorApplyV[e\_60]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (60)", "Subsection",
  CellTags->"T1.6.30.1"],

Cell[BoxData[
    \(h\_60 = s\_60 /. \[Nu]\_1 \[Rule] \[Nu]\_1 - 1\)], "Input"],

Cell[BoxData[
    RowBox[{\(notnecessaryf\_60\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_60\), ",", "\"\<(60)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            \(\[Nu]\_1 > 1 && \ m1 =!= 0\), "]"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTVIRecursion[notnecessaryf\_60]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (61) ", "Subsection",
  CellTags->"T1.6.31.1"],

Cell[TextData[{
  " ( ",
  Cell[BoxData[
      \(m\_1\%2\)]],
  "== 0 && ",
  Cell[BoxData[
      \(m\_4\%2\)]],
  "== ",
  Cell[BoxData[
      \(m\_3\%2\)]],
  " )  \[DoubleLongRightArrow] ( ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta]\_\(1, 3, 4\)\)]],
  "== 0 )"
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_61\), "=", 
        RowBox[{
          RowBox[{\((d - 2  \[Nu]\_1 - \[Nu]\_3)\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}], "==", 
          RowBox[{\(\[Nu]\_3\), 
            InterpretationBox[
              StyleBox[\(3\^+\),
                FontWeight->"Bold"],
              Schiebe[ 3, "+"],
              Editable->True], 
            RowBox[{"(", 
              RowBox[{
                InterpretationBox[
                  StyleBox[\(1\^-\),
                    FontWeight->"Bold"],
                  Schiebe[ 1, "-"],
                  Editable->True], "-", 
                InterpretationBox[
                  StyleBox[\(4\^-\),
                    FontWeight->"Bold"],
                  Schiebe[ 4, "-"],
                  Editable->True]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_61 = OperatorApplyV[e\_61]\)], "Input"],

Cell[TextData[{
  "For  ( ",
  Cell[BoxData[
      \(m\_3\%2\)]],
  "== 0 && ",
  Cell[BoxData[
      \(m\_4\%2\)]],
  "== ",
  Cell[BoxData[
      \(m\_1\%2\)]],
  " )  \[DoubleLongRightArrow] ( ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta]\_\(1, 3, 4\)\)]],
  "== 0 )\nuse ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              InterpretationBox[
                SubsuperscriptBox[
                  StyleBox["V",
                    SingleLetterItalics->False,
                    FontWeight->"Bold"], 
                  RowBox[{
                    FormBox[\(\[Nu]\_1\),
                      "TraditionalForm"], 
                    FormBox[\(\[Nu]\_2\),
                      "TraditionalForm"], 
                    FormBox[\(\[Nu]\_3\),
                      "TraditionalForm"], 
                    FormBox[\(\[Nu]\_4\),
                      "TraditionalForm"]}], 
                  RowBox[{"(", 
                    FormBox["d",
                      "TraditionalForm"], ")"}]],
                TVI[ d, {
                  Subscript[ \[Nu], 1], 
                  Subscript[ \[Nu], 2], 
                  Subscript[ \[Nu], 3], 
                  Subscript[ \[Nu], 4]}],
                Editable->True], "==", 
              InterpretationBox[
                SubsuperscriptBox[
                  StyleBox["V",
                    SingleLetterItalics->False,
                    FontWeight->"Bold"], 
                  RowBox[{
                    FormBox[\(\[Nu]\_3\),
                      "TraditionalForm"], 
                    FormBox[\(\[Nu]\_2\),
                      "TraditionalForm"], 
                    FormBox[\(\[Nu]\_1\),
                      "TraditionalForm"], 
                    FormBox[\(\[Nu]\_4\),
                      "TraditionalForm"]}], 
                  RowBox[{"(", 
                    FormBox["d",
                      "TraditionalForm"], ")"}]],
                TVI[ d, {
                  Subscript[ \[Nu], 1], 
                  Subscript[ \[Nu], 2], 
                  Subscript[ \[Nu], 3], 
                  Subscript[ \[Nu], 4]}],
                Editable->True]}], ")"}], " "}], TextForm]]],
  " and switch the masses accordingly."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (61)", "Subsection",
  CellTags->"T1.6.32.1"],

Cell[TextData[{
  "( ",
  Cell[BoxData[
      \(m\_1\%2\)]],
  "== 0 && ",
  Cell[BoxData[
      \(m\_4\%2\)]],
  "== ",
  Cell[BoxData[
      \(m\_3\%2\)]],
  " )  \[DoubleLongRightArrow] ( ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta]\_\(1, 3, 4\)\)]],
  "== 0 )"
}], "Text"],

Cell[BoxData[
    \(h\_61 = s\_61\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_61\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_61\), ",", "\"\<(61)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            RowBox[{
              RowBox[{
                InterpretationBox[\(m\_1\),
                  m1], "===", "0"}], "&&", 
              RowBox[{
                InterpretationBox[\(m\_3\),
                  m3], "===", 
                InterpretationBox[\(m\_4\),
                  m4]}]}], "]"}], ",", \({n1 :> 1, n4 :> 1}\)}], "]"}]}]], 
  "Input"],

Cell["CheckTVIRecursion not applicable .", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (6153)", "Subsection",
  CellTags->"T1.6.33.1"],

Cell[TextData[{
  " ( ",
  Cell[BoxData[
      \(m\_3\%2\)]],
  "== 0 && ",
  Cell[BoxData[
      \(m\_4\%2\)]],
  "== ",
  Cell[BoxData[
      \(m\_1\%2\)]],
  ")  \[DoubleLongRightArrow] ( ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta]\_\(1, 3, 4\)\)]],
  "== 0 )"
}], "Text"],

Cell[BoxData[
    \(h\_6153 = Interchange[s\_61, 1 \[LeftRightArrow] 3]\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_6153\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_6153\), ",", "\"\<(6153)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            RowBox[{
              RowBox[{
                InterpretationBox[\(m\_3\),
                  m3], "===", "0"}], "&&", 
              RowBox[{
                InterpretationBox[\(m\_1\),
                  m1], "===", 
                InterpretationBox[\(m\_4\),
                  m4]}]}], "]"}], ",", \({n3 :> 1, n4 :> 1}\)}], "]"}]}]], 
  "Input"],

Cell["CheckTVIRecursion not applicable .", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (62) ", "Subsection",
  CellTags->"T1.6.34.1"],

Cell[TextData[{
  "( ",
  Cell[BoxData[
      \(m\_4\%2\)]],
  "== 0 && ",
  Cell[BoxData[
      \(m\_1\%2\)]],
  "== ",
  Cell[BoxData[
      \(m\_3\%2\)]],
  " ) \[DoubleLongRightArrow] (",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\ \)\(\[CapitalDelta]\_\(1, 3, 4\)\)\)\)]],
  "== 0 )\nlower ",
  Cell[BoxData[
      \(TraditionalForm\`\[Nu]\_2\)]],
  " via (52) to one first"
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_62\), "=", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                InterpretationBox[\(p\^2\),
                  PP], "-", \(m\_2\%2\)}], ")"}], 
            \((\[Nu]\_1 - 2  \[Nu]\_2 + 2  \[Nu]\_4 + 2)\), 
            InterpretationBox[
              StyleBox[\(4\^+\),
                FontWeight->"Bold"],
              Schiebe[ 4, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}], "==", "\n", "\t\t", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\((m\_1\%2 - u\_\(6, 1, 2\))\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True]}], "+", 
                RowBox[{"4", \(m\_2\%2\), \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True]}], "+", 
                RowBox[{"2", \(m\_1\%2\), \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}], "+", "\n", "\t\t\t\t", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{\(\[CapitalSigma]\_4\), "-", 
                      RowBox[{"2", \(m\_1\%2\), \(\[Nu]\_1\), 
                        InterpretationBox[
                          StyleBox[\(1\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "+"],
                          Editable->True]}], "-", 
                      RowBox[{"2", \(m\_1\%2\), \(\[Nu]\_3\), 
                        InterpretationBox[
                          StyleBox[\(3\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "+"],
                          Editable->True]}]}], ")"}], 
                  InterpretationBox[
                    StyleBox[\(2\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "-"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(4\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "+"],
                    Editable->True]}], "+", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[\(p\^2\),
                        PP], "-", \(m\_2\%2\)}], ")"}], \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(4\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(3\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "-"],
                    Editable->True]}], "-", "\n", "\t\t\t\t", 
                \(\[CapitalSigma]\_4\)}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_62 = OperatorApplyV[e\_62]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (62)", "Subsection",
  CellTags->"T1.6.35.1"],

Cell[BoxData[
    \(h\_62 = \[Nu]explicit[s\_62] /. \[Nu]\_4 \[Rule] \[Nu]\_4 - 1\)], 
  "Input"],

Cell[BoxData[
    RowBox[{\(f\_62\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_62\), ",", "\"\<(62)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            RowBox[{
              RowBox[{
                InterpretationBox[\(p\^2\),
                  PP], "=!=", 
                InterpretationBox[\(m\_2\%2\),
                  Power[ m2, 2]]}], "&&", 
              \(\[Nu]\_1 - 2\ \[Nu]\_2 + 2\ \[Nu]\_4 =!= 0\)}], "]"}], ",", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{
                InterpretationBox[\(m\_4\),
                  m4], "\[RuleDelayed]", "0"}], ",", 
              RowBox[{
                InterpretationBox[\(m\_3\),
                  m3], "\[RuleDelayed]", 
                InterpretationBox[\(m\_1\),
                  m1]}], ",", \(n2 \[RuleDelayed] 1\)}], "}"}]}], "]"}]}]], 
  "Input"],

Cell["CheckTVIRecursion not applicable .", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (63) ", "Subsection",
  CellTags->"T1.6.36.1"],

Cell[TextData[{
  Cell[BoxData[
      \(m\_4\%2\)]],
  "== 0  && ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\ \)\(\[CapitalDelta]\_\(1, 3, 4\)\)\)\)]],
  " != 0\nreplace (55) by the following"
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_63\), "=", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                InterpretationBox[\(p\^2\),
                  PP], "-", \(m\_2\%2\)}], ")"}], 
            \((\((m\_3\%2 - m\_1\%2)\) 
                  \((d - 2  \[Nu]\_2 - 2  \[Nu]\_3 + 2  \[Nu]\_4)\) + 
                2 \( m\_3\%2\) \((\[Nu]\_3 - \[Nu]\_1)\))\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}], "==", "\n", "\t\t", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\((m\_1\%2 - m\_3\%2)\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{"2", \(m\_1\%2\), \(\[Nu]\_1\), 
                              InterpretationBox[
                                StyleBox[\(1\^+\),
                                  FontWeight->"Bold"],
                                Schiebe[ 1, "+"],
                                Editable->True]}], "+", 
                            RowBox[{"2", \(m\_3\%2\), \(\[Nu]\_3\), 
                              InterpretationBox[
                                StyleBox[\(3\^+\),
                                  FontWeight->"Bold"],
                                Schiebe[ 3, "+"],
                                Editable->True]}], "-", 
                            \(\[CapitalSigma]\_4\), "-", "2"}], ")"}], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(2\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 2, "-"],
                              Editable->True], "-", 
                            InterpretationBox[
                              StyleBox[\(4\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 4, "-"],
                              Editable->True]}], ")"}]}], "-", 
                      RowBox[{"4", \(m\_2\%2\), \(\[Nu]\_2\), 
                        InterpretationBox[
                          StyleBox[\(2\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 2, "+"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(4\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 4, "-"],
                          Editable->True]}]}], ")"}]}], "+", "\n", 
                "\t\t\t\t", 
                RowBox[{"2", 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[\(p\^2\),
                        PP], "-", \(m\_2\%2\)}], ")"}], 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\(m\_3\%2\), \(\[Nu]\_3\), 
                        InterpretationBox[
                          StyleBox[\(3\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "+"],
                          Editable->True], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(1\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 1, "-"],
                              Editable->True], "-", 
                            InterpretationBox[
                              StyleBox[\(4\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 4, "-"],
                              Editable->True]}], ")"}]}], "+", 
                      RowBox[{\(m\_1\%2\), \(\[Nu]\_1\), 
                        InterpretationBox[
                          StyleBox[\(1\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "+"],
                          Editable->True], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(4\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 4, "-"],
                              Editable->True], "-", 
                            InterpretationBox[
                              StyleBox[\(3\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 3, "-"],
                              Editable->True]}], ")"}]}]}], ")"}]}]}], 
              ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_63 = OperatorApplyV[e\_63]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (63)", "Subsection",
  CellTags->"T1.6.37.1"],

Cell[TextData[{
  Cell[BoxData[
      \(m\_4\%2\)]],
  "== 0  && ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\ \)\(\[CapitalDelta]\_\(1, 3, 4\)\)\)\)]],
  " != 0"
}], "Text"],

Cell[BoxData[
    \(h\_63 = \[Nu]explicit[s\_63]\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_63\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_63\), ",", "\"\<(63)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            \(PP\  =!= \ m2^2 && 
              \(Cayley[1, 3, 4]\)[{m1, m2, m3, m4}] =!= 0\), "]"}], ",", 
          RowBox[{"{", 
            RowBox[{
              InterpretationBox[\(m\_4\),
                m4], "\[RuleDelayed]", "0"}], "}"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTVIRecursion[f\_63]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]],

Cell[BoxData[
    RowBox[{\(f\_63\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_63\), ",", "\"\<(63)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            \(PP\  =!= \ m2^2 && 
              \(Cayley[1, 3, 4]\)[{m1, m2, m3, m4}] =!= 0\), "]"}], ",", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{
                InterpretationBox[\(m\_4\),
                  m4], "\[RuleDelayed]", "0"}], ",", \(n1 :> 1\), ",", 
              \(n3 :> 1\)}], "}"}]}], "]"}]}]], "Input"],

Cell["\<\

CheckTVIRecursion not applicable for special indices since A's and 
B's appear .
\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (6552)", "Subsection",
  CellTags->"T1.6.38.1"],

Cell[BoxData[
    \(r\_6552 = \[Nu]explicit[e\_52] /. \[CapitalDelta]\_\(2, 4, 6\) -> 0\)], 
  "Input"],

Cell[BoxData[
    RowBox[{"formal", "=", 
      RowBox[{
        RowBox[{
          RowBox[{"Solve", "[", 
            RowBox[{\(e\_55 /. TVI[__] -> 1\), ",", 
              InterpretationBox[
                StyleBox[\(4\^+\),
                  FontWeight->"Bold"],
                Schiebe[ 4, "+"],
                Editable->True]}], "]"}], "[", \([1, 1]\), "]"}], "/.", 
        \(\[Nu]\_2 -> \[Nu]\_2 - 1\)}]}]], "Input"],

Cell[BoxData[
    \(r\_6552 /. formal\)], "Input"],

Cell[BoxData[
    \(n\_6552 = OperatorApplyV[r\_6552 /. formal]\)], "Input"],

Cell[BoxData[
    \(n\_6552[\([2, 4]\)]\)], "Input"],

Cell[BoxData[
    \(s\_6552 = 
      \(-n\_6552[\([2, 4]\)]\) == 
        \((n\_6552[\([2]\)] - n\_6552[\([2, 4]\)])\)\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_6552\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(s\_6552\), ",", "\"\<(6552)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            RowBox[{\(\[CapitalDelta]\_\(2, 4, 6\) === 0\), "&&", 
              RowBox[{
                RowBox[{"Expand", "[", 
                  RowBox[{
                    RowBox[{"2", " ", \((\[Nu]\_2 - \[Nu]\_4)\), " ", 
                      InterpretationBox[\(m\_4\%2\),
                        Power[ m4, 2]]}], "+", 
                    \(\((3\ \[Nu]\_2 - d)\)\ u\_\(2, 4, 6\)\), "+", 
                    \(\[Nu]\_2\ u\_\(6, 2, 4\)\)}], "]"}], "=!=", "0"}]}], 
            "]"}]}], "]"}]}]], "Input"],

Cell["CheckTVIRecursion not applicable .", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (6552)", "Subsection",
  CellTags->"T1.6.39.1"],

Cell[BoxData[
    \(r\_6552 = \[Nu]explicit[e\_52] /. \[CapitalDelta]\_\(2, 4, 6\) -> 0\)], 
  "Input"],

Cell[BoxData[
    \(n\_6552 = OperatorApplyV[r\_6552]\)], "Input"],

Cell[BoxData[
    \(n\_6552[\([2, 5]\)]\)], "Input"],

Cell[BoxData[
    \(s\_6552 = 
      \(-n\_6552[\([2, 5]\)]\) == 
        \((n\_6552[\([2]\)] - n\_6552[\([2, 5]\)])\)\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_6552\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(s\_6552\), ",", "\"\<(6552)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            \(\[CapitalDelta]\_\(2, 4, 6\) === 0 && 
              Expand[\((
                    \(-2\)\ \[Nu]\_2\ m\_4\%2 + 2\ \[Nu]\_4\ m\_4\%2 + 
                      d\ u\_\(2, 4, 6\) - 3\ \[Nu]\_2\ u\_\(2, 4, 6\))\)] =!= 
                0\), "]"}]}], "]"}]}]], "Input"],

Cell["CheckTVIRecursion not applicable .", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (6553)", "Subsection",
  CellTags->"T1.6.40.1"],

Cell[BoxData[
    \(r\_6553 = 
      \(\(\[Nu]explicit[s\_55] /. u\_\(3, 1, 4\) -> 0\) /. 
          u\_\(6, 2, 4\) -> 0\) /. m\_4 -> 0\)], "Input"],

Cell[BoxData[
    \(h\_6553 = 
      \(-r\_6553[\([2, 1]\)]\) == 
        \((r\_6553[\([2]\)] - r\_6553[\([2, 1]\)])\)\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_6553\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_6553\), ",", "\"\<(6553)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            \(u\_\(6, 2, 4\) === 0 && u\_\(1, 3, 4\) === 0\), "]"}], ",", 
          \(m4 :> 0\)}], "]"}]}]], "Input"],

Cell["CheckTVIRecursion not applicable .", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (66) ", "Subsection",
  CellTags->"T1.6.41.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_66\), "=", 
        RowBox[{
          RowBox[{"4", 
            InterpretationBox[\(p\^2\),
              PP], \(\[Nu]\_1\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], \(\[Nu]\_2\), 
            InterpretationBox[
              StyleBox[\(2\^+\),
                FontWeight->"Bold"],
              Schiebe[ 2, "+"],
              Editable->True], \(\[Nu]\_3\), 
            InterpretationBox[
              StyleBox[\(3\^+\),
                FontWeight->"Bold"],
              Schiebe[ 3, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox[\(d + 2\),
                    "TraditionalForm"], ")"}]],
              TVI[ 
                Plus[ 2, d], {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"-", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                    \(6  d\), "-", \(6  \[Nu]\_1\), "-", \(5  \[Nu]\_2\), 
                      "-", \(6  \[Nu]\_3\), "-", \(4  \[Nu]\_4\), "-", 
                      RowBox[{"6", \(m\_1\%2\), \(\[Nu]\_1\), 
                        InterpretationBox[
                          StyleBox[\(1\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "+"],
                          Editable->True]}]}], ")"}], "+", 
                  RowBox[{\((u\_\(6, 2, 4\) - 4  m\_2\%2)\), \(\[Nu]\_2\), 
                    InterpretationBox[
                      StyleBox[\(2\^+\),
                        FontWeight->"Bold"],
                      Schiebe[ 2, "+"],
                      Editable->True]}], "-", 
                  RowBox[{"6", \(m\_3\%2\), \(\[Nu]\_3\), 
                    InterpretationBox[
                      StyleBox[\(3\^+\),
                        FontWeight->"Bold"],
                      Schiebe[ 3, "+"],
                      Editable->True]}], "+", 
                  RowBox[{"2", \((u\_\(6, 2, 4\) - m\_4\%2)\), \(\[Nu]\_4\), 
                    InterpretationBox[
                      StyleBox[\(4\^+\),
                        FontWeight->"Bold"],
                      Schiebe[ 4, "+"],
                      Editable->True]}], "-", 
                  RowBox[{\(\[Nu]\_2\), 
                    InterpretationBox[
                      StyleBox[\(2\^+\),
                        FontWeight->"Bold"],
                      Schiebe[ 2, "+"],
                      Editable->True], 
                    InterpretationBox[
                      StyleBox[\(4\^-\),
                        FontWeight->"Bold"],
                      Schiebe[ 4, "-"],
                      Editable->True]}], "-", 
                  RowBox[{"2", \(\[Nu]\_4\), 
                    InterpretationBox[
                      StyleBox[\(4\^+\),
                        FontWeight->"Bold"],
                      Schiebe[ 4, "+"],
                      Editable->True], 
                    InterpretationBox[
                      StyleBox[\(2\^-\),
                        FontWeight->"Bold"],
                      Schiebe[ 2, "-"],
                      Editable->True]}]}], ")"}]}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_66 = OperatorApplyV[e\_66]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (66)", "Subsection",
  CellTags->"T1.6.42.1"],

Cell[BoxData[
    \(h\_66 = 
      s\_66 /. {\[Nu]\_1 :> \[Nu]\_1 - 1, \[Nu]\_2 :> \[Nu]\_2 - 1, 
          \[Nu]\_3 :> \[Nu]\_3 - 1, d :> d - 2}\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_66\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_66\), ",", "\"\<(66)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            \(PP =!= 0 && \[Nu]\_1 > 1 && \[Nu]\_2 > 1 && \[Nu]\_3 > 1 && 
              MatchQ[d, _Symbol + _Integer?Positive]\), "]"}]}], "]"}]}]], 
  "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTVIRecursion[f\_66]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (alternative67)", "Subsection",
  CellTags->"T1.6.43.1"],

Cell["\<\
we dereived an alternative formula for Tarasov's (67), but do not \
save it by default.\
\>", "Text"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_67\), "=", 
        RowBox[{
          RowBox[{"4", 
            InterpretationBox[\(p\^2\),
              PP], \(\[Nu]\_1\), \((\[Nu]\_1 + 1)\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], \(\[Nu]\_2\), 
            InterpretationBox[
              StyleBox[\(2\^+\),
                FontWeight->"Bold"],
              Schiebe[ 2, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox[\(d + 2\),
                    "TraditionalForm"], ")"}]],
              TVI[ 
                Plus[ 2, d], {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}], " ", "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
              \((6  d - 6  \[Nu]\_1 - 5  \[Nu]\_2 - 6  \[Nu]\_3 - 4  \[Nu]\_4)
                  \), "-", 
                RowBox[{
                "2", \((u\_\(2, 4, 6\) + 3  m\_1\%2)\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True]}], "+", 
                RowBox[{\((u\_\(6, 2, 4\) - 4  m\_2\%2)\), \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True]}], "-", 
                RowBox[{"6", \(m\_3\%2\), \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}], "+", 
                RowBox[{"2", \((u\_\(6, 2, 4\) - m\_4\%2)\), \(\[Nu]\_4\), 
                  InterpretationBox[
                    StyleBox[\(4\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "+"],
                    Editable->True]}], "+", 
                RowBox[{"2", \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(4\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 4, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(2\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 2, "-"],
                        Editable->True]}], ")"}]}], "-", 
                RowBox[{\(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(4\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "-"],
                    Editable->True]}], "-", 
                RowBox[{"2", \(\[Nu]\_4\), 
                  InterpretationBox[
                    StyleBox[\(4\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(2\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "-"],
                    Editable->True]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_67 = OperatorApplyV[e\_67]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (67)", "Subsection",
  CellTags->"T1.6.44.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_67\), "=", 
        RowBox[{
          RowBox[{"4", 
            InterpretationBox[\(p\^2\),
              PP], \(\[Nu]\_1\), \((\[Nu]\_1 + 1)\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], \(\[Nu]\_2\), 
            InterpretationBox[
              StyleBox[\(2\^+\),
                FontWeight->"Bold"],
              Schiebe[ 2, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox[\(d + 2\),
                    "TraditionalForm"], ")"}]],
              TVI[ 
                Plus[ 2, d], {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}], " ", "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
              \(2 \((2  d - 2  \[Nu]\_1 - 2  \[Nu]\_2 - 2  \[Nu]\_3 - 
                      \[Nu]\_4)\)\), "-", 
                RowBox[{
                "2", \((u\_\(2, 4, 6\) + 2  m\_1\%2)\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(-4\), \(m\_2\%2\), \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True]}], "-", 
                RowBox[{"4", \(m\_3\%2\), \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}], "+", 
                RowBox[{"2", \(u\_\(6, 2, 4\)\), \(\[Nu]\_4\), 
                  InterpretationBox[
                    StyleBox[\(4\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "+"],
                    Editable->True]}], "+", 
                RowBox[{"2", \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(4\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 4, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(2\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 2, "-"],
                        Editable->True]}], ")"}]}], "-", 
                RowBox[{"2", \(\[Nu]\_4\), 
                  InterpretationBox[
                    StyleBox[\(4\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(2\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "-"],
                    Editable->True]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{\(-4\), " ", \(\[Nu]\_2\), " ", 
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["V",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                FormBox[\(\[Nu]\_1\),
                  "TraditionalForm"], 
                StyleBox[
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"],
                  FontColor->RGBColor[0, 0, 1]], 
                FormBox[\(\[Nu]\_3\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_4\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox["d",
                  "TraditionalForm"], ")"}]],
            TVI[ d, {
              Subscript[ \[Nu], 1], 
              Plus[ 1, 
                Subscript[ \[Nu], 2]], 
              Subscript[ \[Nu], 3], 
              Subscript[ \[Nu], 4]}],
            Editable->True], " ", \(m\_2\%2\)}], "-", 
        RowBox[{"2", " ", \(\[Nu]\_4\), " ", 
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["V",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                FormBox[\(\[Nu]\_1\),
                  "TraditionalForm"], 
                StyleBox[
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"],
                  FontColor->RGBColor[1, 0, 0]], 
                FormBox[\(\[Nu]\_3\),
                  "TraditionalForm"], 
                StyleBox[
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"],
                  FontColor->RGBColor[0, 0, 1]]}], 
              RowBox[{"(", 
                FormBox["d",
                  "TraditionalForm"], ")"}]],
            TVI[ d, {
              Subscript[ \[Nu], 1], 
              Plus[ -1, 
                Subscript[ \[Nu], 2]], 
              Subscript[ \[Nu], 3], 
              Plus[ 1, 
                Subscript[ \[Nu], 4]]}],
            Editable->True]}], "+", 
        RowBox[{
        "2", " ", \((2\ d - 2\ \[Nu]\_1 - 2\ \[Nu]\_2 - 
              2\ \[Nu]\_3 - \[Nu]\_4)\), " ", 
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["V",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                FormBox[\(\[Nu]\_1\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_2\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_3\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_4\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox["d",
                  "TraditionalForm"], ")"}]],
            TVI[ d, {
              Subscript[ \[Nu], 1], 
              Subscript[ \[Nu], 2], 
              Subscript[ \[Nu], 3], 
              Subscript[ \[Nu], 4]}],
            Editable->True]}], "-", 
        RowBox[{
        "2", " ", \((m\_2\%2 + m\_4\%2 - p\^2)\), " ", \(\[Nu]\_4\), " ", 
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["V",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                FormBox[\(\[Nu]\_1\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_2\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_3\),
                  "TraditionalForm"], 
                StyleBox[
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"],
                  FontColor->RGBColor[0, 0, 1]]}], 
              RowBox[{"(", 
                FormBox["d",
                  "TraditionalForm"], ")"}]],
            TVI[ d, {
              Subscript[ \[Nu], 1], 
              Subscript[ \[Nu], 2], 
              Subscript[ \[Nu], 3], 
              Plus[ 1, 
                Subscript[ \[Nu], 4]]}],
            Editable->True]}], "-", 
        RowBox[{"4", " ", \(m\_3\%2\), " ", \(\[Nu]\_3\), " ", 
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["V",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                FormBox[\(\[Nu]\_1\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_2\),
                  "TraditionalForm"], 
                StyleBox[
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"],
                  FontColor->RGBColor[0, 0, 1]], 
                FormBox[\(\[Nu]\_4\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox["d",
                  "TraditionalForm"], ")"}]],
            TVI[ d, {
              Subscript[ \[Nu], 1], 
              Subscript[ \[Nu], 2], 
              Plus[ 1, 
                Subscript[ \[Nu], 3]], 
              Subscript[ \[Nu], 4]}],
            Editable->True]}], "-", 
        RowBox[{"2", " ", \(\[Nu]\_1\), " ", 
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["V",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                StyleBox[
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"],
                  FontColor->RGBColor[0, 0, 1]], 
                StyleBox[
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"],
                  FontColor->RGBColor[1, 0, 0]], 
                FormBox[\(\[Nu]\_3\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_4\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox["d",
                  "TraditionalForm"], ")"}]],
            TVI[ d, {
              Plus[ 1, 
                Subscript[ \[Nu], 1]], 
              Plus[ -1, 
                Subscript[ \[Nu], 2]], 
              Subscript[ \[Nu], 3], 
              Subscript[ \[Nu], 4]}],
            Editable->True]}], "+", 
        RowBox[{"2", " ", \(\[Nu]\_1\), " ", 
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["V",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                StyleBox[
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"],
                  FontColor->RGBColor[0, 0, 1]], 
                FormBox[\(\[Nu]\_2\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_3\),
                  "TraditionalForm"], 
                StyleBox[
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"],
                  FontColor->RGBColor[1, 0, 0]]}], 
              RowBox[{"(", 
                FormBox["d",
                  "TraditionalForm"], ")"}]],
            TVI[ d, {
              Plus[ 1, 
                Subscript[ \[Nu], 1]], 
              Subscript[ \[Nu], 2], 
              Subscript[ \[Nu], 3], 
              Plus[ -1, 
                Subscript[ \[Nu], 4]]}],
            Editable->True]}], "-", 
        RowBox[{
        "2", " ", \((2\ m\_1\%2 + m\_2\%2 - m\_4\%2 - p\^2)\), 
          " ", \(\[Nu]\_1\), " ", 
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["V",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                StyleBox[
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"],
                  FontColor->RGBColor[0, 0, 1]], 
                FormBox[\(\[Nu]\_2\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_3\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_4\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox["d",
                  "TraditionalForm"], ")"}]],
            TVI[ d, {
              Plus[ 1, 
                Subscript[ \[Nu], 1]], 
              Subscript[ \[Nu], 2], 
              Subscript[ \[Nu], 3], 
              Subscript[ \[Nu], 4]}],
            Editable->True]}], "-", 
        RowBox[{
        "4", " ", \(p\^2\), " ", \(\[Nu]\_1\), " ", \((\[Nu]\_1 + 1)\), 
          " ", \(\[Nu]\_2\), " ", 
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["V",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                StyleBox[
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"],
                  FontWeight->"Bold",
                  FontColor->RGBColor[0, 0, 1]], 
                StyleBox[
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"],
                  FontColor->RGBColor[0, 0, 1]], 
                FormBox[\(\[Nu]\_3\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_4\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox[\(d + 2\),
                  "TraditionalForm"], ")"}]],
            TVI[ 
              Plus[ 2, d], {
              Plus[ 2, 
                Subscript[ \[Nu], 1]], 
              Plus[ 1, 
                Subscript[ \[Nu], 2]], 
              Subscript[ \[Nu], 3], 
              Subscript[ \[Nu], 4]}],
            Editable->True]}]}], TraditionalForm]]]], "Text"],

Cell[BoxData[
    \(s\_67 = OperatorApplyV[e\_67]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (67)", "Subsection",
  CellTags->"T1.6.45.1"],

Cell[BoxData[
    \(h\_67 = 
      s\_67 /. {\[Nu]\_1 :> \[Nu]\_1 - 2, \[Nu]\_2 :> \[Nu]\_2 - 1, 
          d :> d - 2}\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_67\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_67\), ",", "\"\<(67)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            \(PP =!= 0 && \[Nu]\_1 > 2 && \[Nu]\_2 > 1 && 
              MatchQ[d, _Symbol + _Integer?Positive]\), "]"}]}], "]"}]}]], 
  "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTVIRecursion[f\_67]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (6713)", "Subsection",
  CellTags->"T1.6.46.1"],

Cell[BoxData[
    \(h\_6713 = 
      Interchange[s\_67, 1 \[LeftRightArrow] 3] /. {\[Nu]\_3 :> \[Nu]\_3 - 2, 
          \[Nu]\_2 :> \[Nu]\_2 - 1, d :> d - 2}\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_6713\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_6713\), ",", "\"\<(6713)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            \(PP =!= 0 && \[Nu]\_3 > 2 && \[Nu]\_2 > 1 && 
              MatchQ[d, _Symbol + _Integer?Positive]\), "]"}]}], "]"}]}]], 
  "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTVIRecursion[f\_6713]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (68)", "Subsection",
  CellTags->"T1.6.47.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_68\), "=", 
        RowBox[{
          RowBox[{"4", 
            InterpretationBox[\(p\^2\),
              PP], \(m\_4\%2\), \(\[Nu]\_1\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], \(\[Nu]\_2\), \((\[Nu]\_2 + 1)\), 
            InterpretationBox[
              StyleBox[\(2\^+\),
                FontWeight->"Bold"],
              Schiebe[ 2, "+"],
              Editable->True], 
            InterpretationBox[
              StyleBox[\(2\^+\),
                FontWeight->"Bold"],
              Schiebe[ 2, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox[\(d + 2\),
                    "TraditionalForm"], ")"}]],
              TVI[ 
                Plus[ 2, d], {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"-", 
              RowBox[{"(", 
                RowBox[{
                \((\((m\_1\%2 - m\_3\%2)\) 
                        \((2  d - 2  \[Nu]\_1 - \[Nu]\_2 - 2  \[Nu]\_3 - 
                            2  \[Nu]\_4)\) + 
                      \(m\_4\%2\) \((2  \[Nu]\_1 + \[Nu]\_2 - 2  \[Nu]\_3)\))
                    \), "-", 
                  RowBox[{\(u\_\(1, 3, 4\)\), 
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"2", \(m\_1\%2\), \(\[Nu]\_1\), 
                          InterpretationBox[
                            StyleBox[\(1\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 1, "+"],
                            Editable->True]}], "+", 
                        RowBox[{\(u\_\(2, 4, 6\)\), \(\[Nu]\_2\), 
                          InterpretationBox[
                            StyleBox[\(2\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 2, "+"],
                            Editable->True]}]}], ")"}]}], "+", 
                  RowBox[{\(u\_\(3, 1, 4\)\), 
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"2", \(m\_3\%2\), \(\[Nu]\_3\), 
                          InterpretationBox[
                            StyleBox[\(3\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 3, "+"],
                            Editable->True]}], "+", 
                        RowBox[{\(\[Nu]\_2\), 
                          InterpretationBox[
                            StyleBox[\(2\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 2, "+"],
                            Editable->True], 
                          InterpretationBox[
                            StyleBox[\(4\^-\),
                              FontWeight->"Bold"],
                            Schiebe[ 4, "-"],
                            Editable->True]}]}], ")"}]}], "-", 
                  RowBox[{"2", \(m\_4\%2\), 
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{\(\[Nu]\_2\), 
                          InterpretationBox[
                            StyleBox[\(2\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 2, "+"],
                            Editable->True]}], "+", 
                        RowBox[{\(\[Nu]\_4\), 
                          InterpretationBox[
                            StyleBox[\(4\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 4, "+"],
                            Editable->True]}]}], ")"}], 
                    RowBox[{"(", 
                      RowBox[{
                        InterpretationBox[
                          StyleBox[\(3\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "-"],
                          Editable->True], "-", 
                        InterpretationBox[
                          StyleBox[\(1\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "-"],
                          Editable->True]}], ")"}]}]}], ")"}]}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_68 = OperatorApplyV[e\_68]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (68)", "Subsection",
  CellTags->"T1.6.48.1"],

Cell[BoxData[
    \(h\_68 = 
      s\_68 /. {\[Nu]\_1 :> \[Nu]\_1 - 1, \[Nu]\_2 :> \[Nu]\_2 - 2, 
          d :> d - 2}\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_68\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_68\), ",", "\"\<(68)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            \(PP =!= 0 && m4 =!= 0 && \[Nu]\_1 > 1 && \[Nu]\_2 > 2 && 
              MatchQ[d, _Symbol + _Integer?Positive]\), "]"}]}], "]"}]}]], 
  "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTVIRecursion[f\_68]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (6813)", "Subsection",
  CellTags->"T1.6.49.1"],

Cell[BoxData[
    \(h\_6813 = 
      Interchange[s\_68, 1 \[LeftRightArrow] 3] /. {\[Nu]\_2 :> \[Nu]\_2 - 2, 
          \[Nu]\_3 :> \[Nu]\_3 - 1, d :> d - 2}\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_6813\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_6813\), ",", "\"\<(6813)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            \(\(PP =!= 0 && m4 =!= 0\) && \[Nu]\_2 > 2 && \[Nu]\_3 > 1 && 
              MatchQ[d, _Symbol + _Integer?Positive]\), "]"}]}], "]"}]}]], 
  "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTVIRecursion[f\_6813]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (70)", "Subsection",
  CellTags->"T1.6.50.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_70\), "=", 
        RowBox[{
          RowBox[{\((3  d - 10)\), \(\((d - 3)\)\^2\), \(m\_4\%2\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox["1",
                    "TraditionalForm"], 
                  FormBox["1",
                    "TraditionalForm"], 
                  FormBox["1",
                    "TraditionalForm"], 
                  FormBox["1",
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {1, 1, 1, 1}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{
              FractionBox["1", 
                RowBox[{"4", 
                  InterpretationBox[\(p\^2\),
                    PP]}]], 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{\((3  d - 10)\), \(\[CapitalDelta]\_\(1, 3, 4\)\), 
                    RowBox[{"(", 
                      RowBox[{\(\[CapitalDelta]\_\(2, 4, 6\)\), "+", 
                        RowBox[{\(u\_\(2, 4, 6\)\), 
                          RowBox[{"(", 
                            RowBox[{
                              InterpretationBox[
                                StyleBox[\(2\^-\),
                                  FontWeight->"Bold"],
                                Schiebe[ 2, "-"],
                                Editable->True], "-", 
                              InterpretationBox[
                                StyleBox[\(4\^-\),
                                  FontWeight->"Bold"],
                                Schiebe[ 4, "-"],
                                Editable->True]}], ")"}]}]}], ")"}]}], "-", 
                  RowBox[{"8", \(\((d - 4)\)\^2\), 
                    InterpretationBox[\(p\^2\),
                      PP], \(m\_3\%2\), \(m\_4\%2\), 
                    InterpretationBox[
                      StyleBox[\(4\^-\),
                        FontWeight->"Bold"],
                      Schiebe[ 4, "-"],
                      Editable->True]}]}], ")"}], 
              InterpretationBox[
                SubsuperscriptBox[
                  StyleBox["V",
                    SingleLetterItalics->False,
                    FontWeight->"Bold"], 
                  RowBox[{
                    FormBox["1",
                      "TraditionalForm"], 
                    FormBox["1",
                      "TraditionalForm"], 
                    FormBox["1",
                      "TraditionalForm"], 
                    FormBox["1",
                      "TraditionalForm"]}], 
                  RowBox[{"(", 
                    FormBox[\(d - 2\),
                      "TraditionalForm"], ")"}]],
                TVI[ 
                  Plus[ -2, d], {1, 1, 1, 1}],
                Editable->True]}], "+", "\n", "\t\t\t", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{\((d - 3)\), \(m\_4\%2\), 
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"4", \(m\_1\%2\), \((d - 4)\), 
                          InterpretationBox[
                            StyleBox[\(1\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 1, "+"],
                            Editable->True], 
                          InterpretationBox[
                            StyleBox[\(1\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 1, "+"],
                            Editable->True]}], "-", 
                        RowBox[{"8", \((d - 3)\), \(m\_2\%2\), 
                          InterpretationBox[
                            StyleBox[\(2\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 2, "+"],
                            Editable->True], 
                          InterpretationBox[
                            StyleBox[\(2\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 2, "+"],
                            Editable->True]}], "+", 
                        RowBox[{\((3  d - 10)\), \((d - 3)\), 
                          InterpretationBox[
                            StyleBox[\(2\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 2, "+"],
                            Editable->True]}]}], ")"}]}], "+", 
                  RowBox[{\(u\_\(6, 2, 4\)\), 
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{
                        \(1\/2\), \((3  d - 10)\), \(u\_\(1, 3, 4\)\), 
                          RowBox[{"(", 
                            RowBox[{
                              InterpretationBox[
                                StyleBox[\(1\^+\),
                                  FontWeight->"Bold"],
                                Schiebe[ 1, "+"],
                                Editable->True], "-", 
                              InterpretationBox[
                                StyleBox[\(3\^+\),
                                  FontWeight->"Bold"],
                                Schiebe[ 3, "+"],
                                Editable->True]}], ")"}]}], "+", 
                        RowBox[{\((d - 2)\), \((d - 3)\), \(m\_4\%2\), 
                          InterpretationBox[
                            StyleBox[\(1\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 1, "+"],
                            Editable->True]}]}], ")"}], 
                    InterpretationBox[
                      StyleBox[\(2\^+\),
                        FontWeight->"Bold"],
                      Schiebe[ 2, "+"],
                      Editable->True]}], "-", 
                  RowBox[{\(m\_4\%2\), 
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{\(\((d - 4)\)\^2\), \(u\_\(1, 3, 4\)\), 
                          InterpretationBox[
                            StyleBox[\(2\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 2, "+"],
                            Editable->True]}], "+", 
                        RowBox[{\((3  d - 10)\), \(u\_\(4, 1, 3\)\), 
                          InterpretationBox[
                            StyleBox[\(3\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 3, "+"],
                            Editable->True]}]}], ")"}], 
                    InterpretationBox[
                      StyleBox[\(1\^+\),
                        FontWeight->"Bold"],
                      Schiebe[ 1, "+"],
                      Editable->True]}]}], ")"}], 
              InterpretationBox[
                StyleBox[\(4\^-\),
                  FontWeight->"Bold"],
                Schiebe[ 4, "-"],
                Editable->True], 
              InterpretationBox[
                SubsuperscriptBox[
                  StyleBox["V",
                    SingleLetterItalics->False,
                    FontWeight->"Bold"], 
                  RowBox[{
                    FormBox["1",
                      "TraditionalForm"], 
                    FormBox["1",
                      "TraditionalForm"], 
                    FormBox["1",
                      "TraditionalForm"], 
                    FormBox["1",
                      "TraditionalForm"]}], 
                  RowBox[{"(", 
                    FormBox["d",
                      "TraditionalForm"], ")"}]],
                TVI[ d, {1, 1, 1, 1}],
                Editable->True]}], "-", 
            RowBox[{\((d - 3)\), \(m\_4\%2\), 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"4", \((d - 4)\), \(m\_1\%2\), 
                    InterpretationBox[
                      StyleBox[\(1\^+\),
                        FontWeight->"Bold"],
                      Schiebe[ 1, "+"],
                      Editable->True]}], "+", 
                  RowBox[{\((d - 2)\), 
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{\(u\_\(6, 2, 4\)\), 
                          InterpretationBox[
                            StyleBox[\(2\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 2, "+"],
                            Editable->True]}], "-", 
                        RowBox[{"2", \(m\_4\%2\), 
                          InterpretationBox[
                            StyleBox[\(4\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 4, "+"],
                            Editable->True]}]}], ")"}]}]}], ")"}], 
              InterpretationBox[
                StyleBox[\(1\^+\),
                  FontWeight->"Bold"],
                Schiebe[ 1, "+"],
                Editable->True], 
              InterpretationBox[
                StyleBox[\(3\^-\),
                  FontWeight->"Bold"],
                Schiebe[ 3, "-"],
                Editable->True], 
              InterpretationBox[
                SubsuperscriptBox[
                  StyleBox["V",
                    SingleLetterItalics->False,
                    FontWeight->"Bold"], 
                  RowBox[{
                    FormBox["1",
                      "TraditionalForm"], 
                    FormBox["1",
                      "TraditionalForm"], 
                    FormBox["1",
                      "TraditionalForm"], 
                    FormBox["1",
                      "TraditionalForm"]}], 
                  RowBox[{"(", 
                    FormBox["d",
                      "TraditionalForm"], ")"}]],
                TVI[ d, {1, 1, 1, 1}],
                Editable->True]}], "+", 
            RowBox[{\(1\/2\), \((3  d - 10)\), \(u\_\(1, 3, 4\)\), 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{\(u\_\(6, 2, 4\)\), 
                    InterpretationBox[
                      StyleBox[\(2\^+\),
                        FontWeight->"Bold"],
                      Schiebe[ 2, "+"],
                      Editable->True]}], "-", 
                  RowBox[{"2", \(m\_4\%2\), 
                    InterpretationBox[
                      StyleBox[\(4\^+\),
                        FontWeight->"Bold"],
                      Schiebe[ 4, "+"],
                      Editable->True]}]}], ")"}], 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    InterpretationBox[
                      StyleBox[\(3\^+\),
                        FontWeight->"Bold"],
                      Schiebe[ 3, "+"],
                      Editable->True], 
                    InterpretationBox[
                      StyleBox[\(1\^-\),
                        FontWeight->"Bold"],
                      Schiebe[ 1, "-"],
                      Editable->True]}], "-", 
                  RowBox[{
                    InterpretationBox[
                      StyleBox[\(1\^+\),
                        FontWeight->"Bold"],
                      Schiebe[ 1, "+"],
                      Editable->True], 
                    InterpretationBox[
                      StyleBox[\(3\^-\),
                        FontWeight->"Bold"],
                      Schiebe[ 3, "-"],
                      Editable->True]}]}], ")"}], 
              InterpretationBox[
                SubsuperscriptBox[
                  StyleBox["V",
                    SingleLetterItalics->False,
                    FontWeight->"Bold"], 
                  RowBox[{
                    FormBox["1",
                      "TraditionalForm"], 
                    FormBox["1",
                      "TraditionalForm"], 
                    FormBox["1",
                      "TraditionalForm"], 
                    FormBox["1",
                      "TraditionalForm"]}], 
                  RowBox[{"(", 
                    FormBox["d",
                      "TraditionalForm"], ")"}]],
                TVI[ d, {1, 1, 1, 1}],
                Editable->True]}]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_70 = OperatorApplyV[e\_70]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (70)", "Subsection",
  CellTags->"T1.6.51.1"],

Cell[BoxData[
    \(h\_70 = s\_70\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_70\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_70\), ",", "\"\<(70)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            \(\(m4 =!= 0\  && \ PP =!= 0\) && 
              MatchQ[d, _Symbol + _Integer?Positive]\), "]"}]}], "]"}]}]], 
  "Input"],

Cell["\<\
CheckTVIRecursion not applicable for special indices since A's and 
B's appear .\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (71)", "Subsection",
  CellTags->"T1.6.52.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`\[Nu]\_1, \[Nu]\_2 > 0\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_71\), "=", 
        RowBox[{
          RowBox[{"2", \(D\_\(1, 2, 3\)\), \(\[Nu]\_1\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], \(\[Nu]\_2\), 
            InterpretationBox[
              StyleBox[\(2\^+\),
                FontWeight->"Bold"],
              Schiebe[ 2, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{"2", \(h\_\(1, 2, 3\)\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True]}], "+", 
                RowBox[{"2", \(h\_\(2, 1, 3\)\), \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True]}], "+", 
                RowBox[{
                "4", \(m\_3\%2\), \(\[Sigma]\_\(1, 2, 3\)\), \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(m\_3\%2\), \(\[Phi]\_\(2, 1, 3\)\), \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True], \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(1\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "-"],
                    Editable->True]}], "+", 
                RowBox[{\(m\_3\%2\), \(\[Phi]\_\(1, 2, 3\)\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(2\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "-"],
                    Editable->True]}], "-", 
                RowBox[{"2", \(\[Rho]\_\(1, 2, 3\)\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(3\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "-"],
                    Editable->True]}], "+", 
                \(\(1\/2\) \(\[CapitalSigma]\_3\) 
                  \((d - \[Nu]\_1 - \[Nu]\_2 - \[Nu]\_3)\) 
                  \[Phi]\_\(3, 2, 1\)\)}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_71 = OperatorApplyJ[e\_71]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (71)", "Subsection",
  CellTags->"T1.6.53.1"],

Cell[BoxData[
    \(h\_71 = 
      \(\[Nu]explicit[s\_71] /. \[Nu]\_1 -> \[Nu]\_1 - 1\) /. 
        \[Nu]\_2 -> \[Nu]\_2 - 1\)], "Input"],

Cell[BoxData[
    \(f\_71 = 
      MakeFun[h\_71, "\<(71)\>", 
        IFF[\[Nu]\_1 > 1 && \[Nu]\_2 > 1 && D\_\(1, 2, 3\) =!= 0]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_71]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (7123)", "Subsection",
  CellTags->"T1.6.54.1"],

Cell[BoxData[
    \(h\_7123 = 
      \(Interchange[s\_71, 2 \[LeftRightArrow] 3] /. 
          \[Nu]\_1 -> \[Nu]\_1 - 1\) /. \[Nu]\_3 -> \[Nu]\_3 - 1\)], "Input"],

Cell[BoxData[
    \(f\_7123 = 
      MakeFun[h\_7123, "\<(7123)\>", 
        IFF[\[Nu]\_1 > 1 && \[Nu]\_3 > 1 && D\_\(1, 3, 2\) =!= 0]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_7123]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (7113)", "Subsection",
  CellTags->"T1.6.55.1"],

Cell[BoxData[
    \(h\_7113 = 
      \(Interchange[s\_71, 1 \[LeftRightArrow] 3] /. 
          \[Nu]\_2 -> \[Nu]\_2 - 1\) /. \[Nu]\_3 -> \[Nu]\_3 - 1\)], "Input"],

Cell[BoxData[
    \(f\_7113 = 
      MakeFun[h\_7113, "\<(7113)\>", 
        IFF[\[Nu]\_2 > 1 && \[Nu]\_3 > 1 && D\_\(3, 2, 1\) =!= 0]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_7113]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (78)", "Subsection",
  CellTags->"T1.6.56.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`\[Nu]\_1 > 0\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_78\), "=", 
        RowBox[{
          RowBox[{
          "2", \(m\_1\%2\), " ", \(D\_\(1, 2, 3\)\), \(\[Nu]\_1\), 
            \((\[Nu]\_1 + 1)\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
              \(\(-\[CapitalSigma]\_3\) 
                  \((d - \[Nu]\_1 - \[Nu]\_2 - \[Nu]\_3)\) 
                  \[Rho]\_\(1, 2, 3\)\), "+", 
                RowBox[{
                \(m\_2\%2\), \(m\_3\%2\), \(\[Phi]\_\(1, 2, 3\)\), 
                  \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True], \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(1\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "-"],
                    Editable->True]}], "+", 
                RowBox[{
                \(m\_1\%2\), \(m\_3\%2\), \(\[Phi]\_\(2, 1, 3\)\), 
                  \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(2\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "-"],
                    Editable->True]}], "+", 
                RowBox[{
                \(m\_1\%2\), \(m\_2\%2\), \(\[Phi]\_\(3, 1, 2\)\), 
                  \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(3\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "-"],
                    Editable->True]}], "+", 
                RowBox[{
                \((d - 2 - 2  \[Nu]\_1)\), \(D\_\(1, 2, 3\)\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(m\_1\%2\), \(S\_\(1, 2, 3\)\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(m\_2\%2\), \(S\_\(2, 1, 3\)\), \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(m\_3\%2\), \(S\_\(3, 1, 2\)\), \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_78 = OperatorApplyJ[e\_78]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (78)", "Subsection",
  CellTags->"T1.6.57.1"],

Cell[BoxData[
    \(h\_78 = \[Nu]explicit[s\_78] /. \[Nu]\_1 -> \[Nu]\_1 - 2\)], "Input"],

Cell[BoxData[
    \(f\_78 = 
      MakeFun[h\_78, "\<(78)\>", 
        IFF[\(\[Nu]\_1 > 2 && m1 =!= 0\) && D\_\(1, 2, 3\) =!= 0]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_78]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (80)", "Subsection",
  CellTags->"T1.6.58.1"],

Cell[TextData[{
  Cell[BoxData[
      \(m\_1\%2 == 0\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`\[Nu]\_1 > 0\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_80\), "=", 
        RowBox[{
          RowBox[{
          \(\[Nu]\_1\), " ", \((d - 2\ \[Nu]\_1 - 2)\), 
            \(\[CapitalDelta]\_\(2, 3, 6\)\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(-4\), \(m\_2\%2\), \(m\_3\%2\), \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True], \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(1\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "-"],
                    Editable->True]}], "+", "\n", "\t\t\t\t", 
                RowBox[{"2", \(m\_2\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[\(p\^2\),
                              PP], "-", \(m\_2\%2\)}], ")"}], 
                        \((2  d - 2  \[Nu]\_1 - \[Nu]\_2 - 2  \[Nu]\_3 - 1)
                          \)}], "-", 
                      \(\(m\_3\%2\) \((2  \[Nu]\_1 - \[Nu]\_2 - 1)\)\)}], 
                    ")"}], \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True]}], "+", "\n", "\t\t\t  ", 
                RowBox[{"2", " ", \(m\_3\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[\(p\^2\),
                              PP], "-", \(m\_3\%2\)}], ")"}], 
                        \((2  d - 2  \[Nu]\_1 - 2  \[Nu]\_2 - \[Nu]\_3 - 1)
                          \)}], "-", 
                      \(\(m\_2\%2\) \((2  \[Nu]\_1 - \[Nu]\_3 - 1)\)\)}], 
                    ")"}], \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}], "-", "\n", "\t\t\t\t", 
                \(\(\[CapitalSigma]\_3\) 
                  \((d - \[Nu]\_1 - \[Nu]\_2 - \[Nu]\_3)\) 
                  u\_\(6, 2, 3\)\)}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_80 = OperatorApplyJ[e\_80]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (80)", "Subsection",
  CellTags->"T1.6.59.1"],

Cell[BoxData[
    \(h\_80 = \[Nu]explicit[s\_80] /. \[Nu]\_1 \[Rule] \[Nu]\_1 - 1\)], 
  "Input"],

Cell[BoxData[
    \(f\_80 = 
      MakeFun[h\_80, "\<(80)\>", 
        IFF[\[Nu]\_1 > 1 && \[CapitalDelta]\_\(2, 3, 6\) =!= 0], m1 :> 0]\)], 
  "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_80]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (81)", "Subsection",
  CellTags->"T1.6.60.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]],
  "and ",
  Cell[BoxData[
      \(TraditionalForm\`\[Nu]\_1, \[Nu]\_2 > 0\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_81\), "=", 
        RowBox[{
          RowBox[{
            InterpretationBox[\(p\^2\),
              PP], \((\[CapitalSigma]\_3 + 2)\), \(\[Nu]\_1\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], \(\[Nu]\_2\), 
            InterpretationBox[
              StyleBox[\(2\^+\),
                FontWeight->"Bold"],
              Schiebe[ 2, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox[\(d + 2\),
                    "TraditionalForm"], ")"}]],
              TJI[ 
                Plus[ 2, d], {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  InterpretationBox[\(p\^2\),
                    PP], \((d - 2  \[Nu]\_3)\)}], "+", 
                \(\(m\_1\%2\) \((d - \[Nu]\_1 - 2  \[Nu]\_2)\)\), "+", 
                \(\(m\_2\%2\) \((d - 2  \[Nu]\_1 - \[Nu]\_2)\)\), "-", 
                \(2 \( m\_3\%2\) \((d - \[Nu]\_1 - \[Nu]\_2 - \[Nu]\_3)\)\), 
                "-", 
                RowBox[{"2", \(m\_3\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[\(p\^2\),
                        PP], "-", \(m\_3\%2\)}], ")"}], \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(m\_1\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[\(p\^2\),
                        PP], "-", \(m\_1\%2\), "-", \(3  m\_2\%2\), "+", 
                      \(3  m\_3\%2\), "-", 
                      InterpretationBox[
                        StyleBox[\(2\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 2, "-"],
                        Editable->True], "+", 
                      InterpretationBox[
                        StyleBox[\(3\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 3, "-"],
                        Editable->True]}], ")"}], \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(m\_2\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[\(p\^2\),
                        PP], "-", \(3  m\_1\%2\), "-", \(m\_2\%2\), "+", 
                      \(3  m\_3\%2\), "-", 
                      InterpretationBox[
                        StyleBox[\(1\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 1, "-"],
                        Editable->True], "+", 
                      InterpretationBox[
                        StyleBox[\(3\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 3, "-"],
                        Editable->True]}], ")"}], \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_81 = OperatorApplyJ[e\_81]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (81)", "Subsection",
  CellTags->"T1.6.61.1"],

Cell[BoxData[
    \(h\_81 = 
      \(\(\[Nu]explicit[s\_81] /. \[Nu]\_1 -> \[Nu]\_1 - 1\) /. 
          \[Nu]\_2 -> \[Nu]\_2 - 1\) /. d -> d - 2\)], "Input"],

Cell[BoxData[
    \(f\_81 = 
      MakeFun[h\_81, "\<(81)\>", 
        IFF[PP =!= 0 && \[Nu]\_1 > 1 && \[Nu]\_2 > 1 && 
            MatchQ[d, _Symbol + _Integer?Positive]]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_81]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (8123)", "Subsection",
  CellTags->"T1.6.62.1"],

Cell[BoxData[
    \(h\_8123 = 
      \(\(Interchange[s\_81, 2 \[LeftRightArrow] 3] /. 
            \[Nu]\_1 -> \[Nu]\_1 - 1\) /. \[Nu]\_3 -> \[Nu]\_3 - 1\) /. 
        d -> d - 2\)], "Input"],

Cell[BoxData[
    \(f\_8123 = 
      MakeFun[h\_8123, "\<(8123)\>", 
        IFF[PP =!= 0 && \[Nu]\_1 > 1 && \[Nu]\_3 > 1 && 
            MatchQ[d, _Symbol + _Integer?Positive]]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_8123]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (8113)", "Subsection",
  CellTags->"T1.6.63.1"],

Cell[BoxData[
    \(h\_8113 = 
      \(\(Interchange[s\_81, 1 \[LeftRightArrow] 3] /. 
            \[Nu]\_2 \[Rule] \[Nu]\_2 - 1\) /. 
          \[Nu]\_3 \[Rule] \[Nu]\_3 - 1\) /. d \[Rule] d - 2\)], "Input"],

Cell[BoxData[
    \(f\_8113 = 
      MakeFun[h\_8113, "\<(8113)\>", 
        IFF[PP =!= 0 && \[Nu]\_2 > 1 && \[Nu]\_3 > 1 && 
            MatchQ[d, _Symbol + _Integer?Positive]]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_8113]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (82)", "Subsection",
  CellTags->"T1.6.64.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]],
  "and ",
  Cell[BoxData[
      \(TraditionalForm\`\[Nu]\_1 > 0\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_82\), "=", 
        RowBox[{
          RowBox[{
            RowBox[{
              RowBox[{"-", 
                InterpretationBox[\(p\^2\),
                  PP]}], \(m\_1\%2\), \((\[CapitalSigma]\_3 + 2)\), 
              \(\[Nu]\_1\), \((\[Nu]\_1 + 1)\), 
              InterpretationBox[
                StyleBox[\(1\^+\),
                  FontWeight->"Bold"],
                Schiebe[ 1, "+"],
                Editable->True], 
              InterpretationBox[
                StyleBox[\(1\^+\),
                  FontWeight->"Bold"],
                Schiebe[ 1, "+"],
                Editable->True], 
              InterpretationBox[
                SubsuperscriptBox[
                  StyleBox["J",
                    SingleLetterItalics->False,
                    FontWeight->"Bold"], 
                  RowBox[{
                    FormBox[\(\[Nu]\_1\),
                      "TraditionalForm"], 
                    FormBox[\(\[Nu]\_2\),
                      "TraditionalForm"], 
                    FormBox[\(\[Nu]\_3\),
                      "TraditionalForm"]}], 
                  RowBox[{"(", 
                    FormBox[\(d + 2\),
                      "TraditionalForm"], ")"}]],
                TJI[ 
                  Plus[ 2, d], {
                  Subscript[ \[Nu], 1], 
                  Subscript[ \[Nu], 2], 
                  Subscript[ \[Nu], 3]}],
                Editable->True]}], "+", 
            RowBox[{\(1\/2\), 
              InterpretationBox[\(p\^2\),
                PP], \((\[CapitalSigma]\_3 + 2)\), 
              \((\[CapitalSigma]\_3 + 4)\), \(\[Nu]\_1\), 
              InterpretationBox[
                StyleBox[\(1\^+\),
                  FontWeight->"Bold"],
                Schiebe[ 1, "+"],
                Editable->True], 
              InterpretationBox[
                SubsuperscriptBox[
                  StyleBox["J",
                    SingleLetterItalics->False,
                    FontWeight->"Bold"], 
                  RowBox[{
                    FormBox[\(\[Nu]\_1\),
                      "TraditionalForm"], 
                    FormBox[\(\[Nu]\_2\),
                      "TraditionalForm"], 
                    FormBox[\(\[Nu]\_3\),
                      "TraditionalForm"]}], 
                  RowBox[{"(", 
                    FormBox[\(d + 2\),
                      "TraditionalForm"], ")"}]],
                TJI[ 
                  Plus[ 2, d], {
                  Subscript[ \[Nu], 1], 
                  Subscript[ \[Nu], 2], 
                  Subscript[ \[Nu], 3]}],
                Editable->True]}]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(-\(1\/2\)\), 
                  SuperscriptBox[
                    RowBox[{"(", 
                      InterpretationBox[\(p\^2\),
                        PP], ")"}], "2"], 
                  \((d + 2  \[Nu]\_1 - 2  \[Nu]\_2 - 2  \[Nu]\_3)\)}], "+", 
                RowBox[{\(1\/2\), 
                  InterpretationBox[\(p\^2\),
                    PP], 
                  \((\(m\_1\%2\) 
                        \((6  \[Nu]\_1 + 6  \[Nu]\_2 + 6  \[Nu]\_3 - 7  d)
                          \) + \(m\_2\%2\) 
                        \((7  d - 2  \[Nu]\_1 - 4  \[Nu]\_2 - 10  \[Nu]\_3)
                          \) + \(m\_3\%2\) 
                        \((7  d - 2  \[Nu]\_1 - 10  \[Nu]\_2 - 4  \[Nu]\_3)\))
                    \)}], "+", 
                \(\(\((m\_2\%2)\)\^2\) \((d - 2  \[Nu]\_1 - \[Nu]\_2)\)\), 
                "+", \(\(\((m\_3\%2)\)\^2\) 
                  \((d - 2  \[Nu]\_1 - \[Nu]\_3)\)\), "+", 
                \(\(m\_1\%2\) \(m\_2\%2\) \((d - \[Nu]\_1 - 2  \[Nu]\_2)\)\), 
                "+", \(\(m\_1\%2\) \(m\_3\%2\) 
                  \((d - \[Nu]\_1 - 2  \[Nu]\_3)\)\), "-", 
                \(4 \( m\_2\%2\) \(m\_3\%2\) 
                  \((d - \[Nu]\_1 - \[Nu]\_2 - \[Nu]\_3)\)\), "-", 
                RowBox[{\(m\_1\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"2", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            InterpretationBox[\(p\^2\),
                              PP], ")"}], "2"]}], "-", 
                      RowBox[{
                        InterpretationBox[\(p\^2\),
                          PP], \((2  m\_1\%2 + m\_2\%2 + m\_3\%2)\)}], "+", 
                      \(\(m\_1\%2\) \((m\_2\%2 + m\_3\%2)\)\), "+", 
                      \(3 \((m\_2\%2 - m\_3\%2)\)\^2\)}], ")"}], 
                  \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(m\_2\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          InterpretationBox[\(p\^2\),
                            PP], ")"}], "2"], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[\(p\^2\),
                              PP], "-", \(m\_2\%2\)}], ")"}], 
                        RowBox[{"(", 
                          RowBox[{\(3  m\_1\%2\), "-", \(5  m\_3\%2\), "+", 
                            InterpretationBox[
                              StyleBox[\(1\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 1, "-"],
                              Editable->True], "-", 
                            InterpretationBox[
                              StyleBox[\(3\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 3, "-"],
                              Editable->True]}], ")"}]}], "-", 
                      \(\((m\_2\%2)\)\^2\)}], ")"}], \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(m\_3\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          InterpretationBox[\(p\^2\),
                            PP], ")"}], "2"], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[\(p\^2\),
                              PP], "-", \(m\_3\%2\)}], ")"}], 
                        RowBox[{"(", 
                          RowBox[{\(3  m\_1\%2\), "-", \(5  m\_2\%2\), "+", 
                            InterpretationBox[
                              StyleBox[\(1\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 1, "-"],
                              Editable->True], "-", 
                            InterpretationBox[
                              StyleBox[\(2\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 2, "-"],
                              Editable->True]}], ")"}]}], "-", 
                      \(\((m\_3\%2)\)\^2\)}], ")"}], \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}], "-", 
                RowBox[{\(1\/2\), 
                  InterpretationBox[\(p\^2\),
                    PP], \((\[CapitalSigma]\_3 + 2)\), 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(1\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 1, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(2\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 2, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(3\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 3, "-"],
                        Editable->True]}], ")"}]}], "-", 
                RowBox[{\(m\_1\%2\), \((m\_2\%2 - m\_3\%2)\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(2\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 2, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(3\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 3, "-"],
                        Editable->True]}], ")"}]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(r\_82 = OperatorApplyJ[e\_82]\)], "Input"],

Cell[BoxData[
    \(r\_82[\([1, 1]\)]\)], "Input"],

Cell[BoxData[
    \(\(s\_82 = 
      \((r\_82[\([1, 2]\)] == \(-r\_82[\([1, 1]\)]\) + r\_82[\([2]\)])\); 
    \)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (82)", "Subsection",
  CellTags->"T1.6.65.1"],

Cell[BoxData[
    \(h\_82 = 
      \(\[Nu]explicit[s\_82] /. \[Nu]\_1 -> \[Nu]\_1 - 2\) /. d -> d - 2\)], 
  "Input"],

Cell[BoxData[
    \(f\_82 = 
      MakeFun[h\_82, "\<(82)\>", 
        IFF[\(PP =!= 0 && \[Nu]\_1 > 2 && m1 =!= 0\) && 
            MatchQ[d, _Symbol + _Integer?Positive]]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_82]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (8213)", "Subsection",
  CellTags->"T1.6.66.1"],

Cell[BoxData[
    \(h\_8213 = 
      \(Interchange[s\_82, 1 \[LeftRightArrow] 3] /. 
          \[Nu]\_3 -> \[Nu]\_3 - 2\) /. d -> d - 2\)], "Input"],

Cell[BoxData[
    \(f\_8213 = 
      MakeFun[h\_8213, "\<(8213)\>", 
        IFF[\(PP =!= 0 && m3 =!= 0\) && \[Nu]\_3 > 2 && 
            MatchQ[d, _Symbol + _Integer?Positive]]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_8213]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (8212)", "Subsection",
  CellTags->"T1.6.67.1"],

Cell[BoxData[
    \(h\_8212 = 
      \(Interchange[s\_82, 1 \[LeftRightArrow] 2] /. 
          \[Nu]\_2 -> \[Nu]\_2 - 2\) /. d -> d - 2\)], "Input"],

Cell[BoxData[
    \(f\_8212 = 
      MakeFun[h\_8212, "\<(8212)\>", 
        IFF[\(PP =!= 0 && \[Nu]\_2 > 2 && m2 =!= 0\) && 
            MatchQ[d, _Symbol + _Integer?Positive]]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_8212]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (83)", "Subsection",
  CellTags->"T1.6.68.1"],

Cell[TextData[{
  Cell[BoxData[
      \(m\_1\%2 == 0\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`\[Nu]\_1 > 0\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_83\), "=", 
        RowBox[{
          RowBox[{
            InterpretationBox[\(p\^2\),
              PP], \((\[CapitalSigma]\_3 + 2)\), 
            \((\[CapitalSigma]\_3 + 4)\), \(\[Nu]\_1\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox[\(d + 2\),
                    "TraditionalForm"], ")"}]],
              TJI[ 
                Plus[ 2, d], {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"-", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        InterpretationBox[\(p\^2\),
                          PP], ")"}], "2"]}], 
                  \((d + 2  \[Nu]\_1 - 2  \[Nu]\_2 - 2  \[Nu]\_3)\)}], "+", 
                RowBox[{
                  InterpretationBox[\(p\^2\),
                    PP], 
                  \((\(m\_2\%2\) 
                        \((7  d - 2  \[Nu]\_1 - 4  \[Nu]\_2 - 10  \[Nu]\_3)
                          \) + \(m\_3\%2\) 
                        \((7  d - 2  \[Nu]\_1 - 10  \[Nu]\_2 - 4  \[Nu]\_3)\))
                    \)}], "+", 
                \(2 \(\((m\_2\%2)\)\^2\) \((d - 2  \[Nu]\_1 - \[Nu]\_2)\)\), 
                "+", \(2 \(\((m\_3\%2)\)\^2\) 
                  \((d - 2  \[Nu]\_1 - \[Nu]\_3)\)\), "-", 
                \(8 \( m\_2\%2\) \(m\_3\%2\) 
                  \((d - \[Nu]\_1 - \[Nu]\_2 - \[Nu]\_3)\)\), "+", 
                RowBox[{"2", \(m\_2\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          InterpretationBox[\(p\^2\),
                            PP], ")"}], "2"], "-", 
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[\(p\^2\),
                              PP], "-", \(m\_2\%2\)}], ")"}], 
                        RowBox[{"(", 
                          RowBox[{\(5  m\_3\%2\), "-", 
                            InterpretationBox[
                              StyleBox[\(1\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 1, "-"],
                              Editable->True], "+", 
                            InterpretationBox[
                              StyleBox[\(3\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 3, "-"],
                              Editable->True]}], ")"}]}], "-", 
                      \(\((m\_2\%2)\)\^2\)}], ")"}], \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True]}], "+", 
                RowBox[{"2", \(m\_3\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          InterpretationBox[\(p\^2\),
                            PP], ")"}], "2"], "-", 
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[\(p\^2\),
                              PP], "-", \(m\_3\%2\)}], ")"}], 
                        RowBox[{"(", 
                          RowBox[{\(5  m\_2\%2\), "-", 
                            InterpretationBox[
                              StyleBox[\(1\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 1, "-"],
                              Editable->True], "+", 
                            InterpretationBox[
                              StyleBox[\(2\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 2, "-"],
                              Editable->True]}], ")"}]}], "-", 
                      \(\((m\_3\%2)\)\^2\)}], ")"}], \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}], "-", 
                RowBox[{
                  InterpretationBox[\(p\^2\),
                    PP], \((\[CapitalSigma]\_3 + 2)\), 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(1\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 1, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(2\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 2, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(3\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 3, "-"],
                        Editable->True]}], ")"}]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_83 = OperatorApplyJ[e\_83]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (83)", "Subsection",
  CellTags->"T1.6.69.1"],

Cell[BoxData[
    \(h\_83 = 
      \(\[Nu]explicit[s\_83] /. \[Nu]\_1 -> \[Nu]\_1 - 1\) /. d -> d - 2\)], 
  "Input"],

Cell[BoxData[
    \(f\_83 = 
      MakeFun[h\_83, "\<(83)\>", 
        IFF[PP =!= 0 && \[Nu]\_1 > 1 && 
            MatchQ[d, _Symbol + _Integer?Positive]], m1 :> 0]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_83]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (87)", "Subsection",
  CellTags->"T1.6.70.1"],

Cell[CellGroupData[{

Cell["(88)", "Subsubsection",
  CellTags->"T1.6.70.1.1"],

Cell[BoxData[
    \(\(t\_1 = m\_1\%2 + m\_2\%2 + m\_3\%2; \)\)], "Input"],

Cell[BoxData[
    \(\(t\_2 = 
      3 \((m\_1\%4 + m\_2\%4 + m\_3\%4)\) + 
        2 \((\(m\_1\%2\) m\_2\%2 + \(m\_1\%2\) m\_3\%2 + \(m\_2\%2\) m\_3\%2)
            \); \)\)], "Input"],

Cell[BoxData[
    \(\(t\_3 = 
      \(m\_1\%2\) \((m\_1\%4 - m\_2\%4 - m\_3\%4)\) + 
        \(m\_2\%2\) \((m\_2\%4 - m\_1\%4 - m\_3\%4)\) + 
        \(m\_3\%2\) \((m\_3\%4 - m\_1\%4 - m\_2\%4)\) + 
        10 \( m\_1\%2\) \(m\_2\%2\) m\_3\%2; \)\)], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(f[\[Mu]1_, \[Mu]2_, \ \[Mu]3_]\), ":=", 
        RowBox[{"\[Mu]1", 
          RowBox[{"(", 
            RowBox[{
              InterpretationBox[\(p\^2\),
                PP], "-", "\[Mu]1"}], ")"}], 
          RowBox[{"(", 
            RowBox[{
              RowBox[{\(-2\), \((d - 4)\), 
                SuperscriptBox[
                  RowBox[{"(", 
                    InterpretationBox[\(p\^2\),
                      PP], ")"}], "2"]}], "+", 
              RowBox[{
                InterpretationBox[\(p\^2\),
                  PP], 
                \((4 \( t\_1\) \((5  d - 18)\) - 24  \[Mu]1 \((2  d - 7)\))
                  \)}], "-", \(2 \((4  d - 13)\) t\_1\^2\), "+", 
              \(2 \((9  d - 31)\) t\_2\), "-", 
              \(24  \[Mu]2\ \[Mu]3 \((4  d - 13)\)\), "-", 
              \(24\ \(\[Mu]1\^2\) \((2  d - 7)\)\)}], ")"}]}]}], ";"}]], 
  "Input"],

Cell[BoxData[
    \(\(f\_\(1, 2, 3\) = f[m\_1\%2, m\_2\%2, m\_3\%2]; \)\)], "Input"],

Cell[BoxData[
    \(\(f\_\(2, 1, 3\) = f[m\_2\%2, m\_1\%2, m\_3\%2]; \)\)], "Input"],

Cell[BoxData[
    \(\(f\_\(3, 2, 1\) = f[m\_3\%2, m\_2\%2, m\_1\%2]; \)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["(89)", "Subsubsection",
  CellTags->"T1.6.70.2.1"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(g[\[Mu]1_, \[Mu]2_, \ \[Mu]3_]\), ":=", 
        RowBox[{\(\(\[Mu]1\ \[Mu]2\)\/\(d - 4\)\), 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"4", \((d - 4)\), 
                SuperscriptBox[
                  RowBox[{"(", 
                    InterpretationBox[\(p\^2\),
                      PP], ")"}], "2"]}], "-", 
              RowBox[{"4", \((7  d - 24)\), 
                InterpretationBox[\(p\^2\),
                  PP], \((3  \[Mu]3 - 2  t\_1)\)}], "-", 
              \(\(t\_1\^2\) \((23  d - 80)\)\), "+", 
              \(\(t\_2\) \((9  d - 32)\)\), "-", 
              \(12 \( \[Mu]3\^2\) \((d - 4)\)\), "+", 
              \(12  \[Mu]1\ \[Mu]2\ \((7  d - 24)\)\)}], ")"}]}]}], ";"}]], 
  "Input"],

Cell[BoxData[
    \(\(g\_\(1, 2, 3\) = g[m\_1\%2, m\_2\%2, m\_3\%2]; \)\)], "Input"],

Cell[BoxData[
    \(\(g\_\(1, 3, 2\) = g[m\_1\%2, m\_3\%2, m\_2\%2]; \)\)], "Input"],

Cell[BoxData[
    \(\(g\_\(2, 3, 1\) = g[m\_2\%2, m\_3\%2, m\_1\%2]; \)\)], "Input"],

Cell[BoxData[
    \(g\_\(1, 2, 3\)\)], "Input"],

Cell[TextData[{
  "general  ",
  Cell[BoxData[
      \(m\_i\%2\)]],
  " "
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_87\), "=", 
        RowBox[{
          RowBox[{"3", 
            InterpretationBox[\(p\^2\),
              PP], \((d - 3)\), \((d - 4)\), \((3  d - 8)\), 
            \((3  d - 10)\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox["1",
                    "TraditionalForm"], 
                  FormBox["1",
                    "TraditionalForm"], 
                  FormBox["1",
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {1, 1, 1}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(\((d - 4)\)\^2\), 
                  SuperscriptBox[
                    RowBox[{"(", 
                      InterpretationBox[\(p\^2\),
                        PP], ")"}], "3"]}], "-", 
                RowBox[{"2", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      InterpretationBox[\(p\^2\),
                        PP], ")"}], "2"], \(t\_1\), \((d - 4)\), 
                  \((6  d - 23)\)}], "+", 
                RowBox[{
                  InterpretationBox[\(p\^2\),
                    PP], 
                  \((5 \( t\_1\^2\) \((15  d\^2 - 117  d + 224)\) - 
                      \(t\_2\) \((42  d\^2 - 331  d + 640)\))\)}], "-", 
                \(\(1\/4\) \((d - 5)\) 
                  \((\(t\_3\) \((27  d - 90)\) - 
                      \(t\_1\) \(t\_2\) \((3  d - 2)\) - 
                      2 \( t\_1\^3\) \((5  d - 26)\))\)\), "+", 
                RowBox[{\(f\_\(1, 2, 3\)\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(f\_\(2, 1, 3\)\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(f\_\(3, 2, 1\)\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(g\_\(1, 2, 3\)\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(3\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "-"],
                    Editable->True]}], "+", 
                RowBox[{\(g\_\(1, 3, 2\)\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(2\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "-"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(g\_\(2, 3, 1\)\), 
                  InterpretationBox[
                    StyleBox[\(1\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "-"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox["1",
                    "TraditionalForm"], 
                  FormBox["1",
                    "TraditionalForm"], 
                  FormBox["1",
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox[\(d - 2\),
                    "TraditionalForm"], ")"}]],
              TJI[ 
                Plus[ -2, d], {1, 1, 1}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_87 = OperatorApplyJ[e\_87]\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (87)", "Subsection",
  CellTags->"T1.6.71.1"],

Cell[BoxData[
    \(h\_87 = s\_87\)], "Input"],

Cell[BoxData[
    \(\(f\_87 = 
      MakeFun[h\_87, "\<(87)\>", 
        IFF[PP =!= 0 && MatchQ[d, _Symbol + _Integer?Positive]]]; \)\)], 
  "Input"],

Cell["CheckTJIRecursion not applicable.", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (90)", "Subsection",
  CellTags->"T1.6.72.1"],

Cell[TextData[{
  "general  ",
  Cell[BoxData[
      \(m\_i\%2\)]],
  " "
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_90\), "=", 
        RowBox[{
          RowBox[{"3", 
            InterpretationBox[\(p\^2\),
              PP], \((d - 3)\), \((d - 4)\), \((3  d - 10)\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox["1",
                    "TraditionalForm"], 
                  FormBox["1",
                    "TraditionalForm"], 
                  FormBox["1",
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {1, 1, 1}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(-m\_1\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        SuperscriptBox[
                          RowBox[{"(", 
                            InterpretationBox[\(p\^2\),
                              PP], ")"}], "2"], \((7  d - 24)\)}], "-", 
                      RowBox[{"2", 
                        InterpretationBox[\(p\^2\),
                          PP], 
                        \((\((4  d - 15)\) t\_1 - \((d - 5)\) m\_1\%2)\)}], 
                      "+", \(\(3\/2\) \((d - 5)\) t\_1\^2\), "+", 
                      \(\(5\/2\) \((d - 3)\) t\_2\), "-", 
                      \(2 \((5  d - 17)\) \((m\_1\%2)\)\^2\), "-", 
                      \(2 \((13  d - 45)\) \(m\_2\%2\) m\_3\%2\)}], ")"}], 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True]}], "+", "\n", "\t\t\t\t", 
                RowBox[{"2", \(m\_2\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[\(p\^2\),
                        PP], "-", \(m\_2\%2\)}], ")"}], 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\((d - 3)\), 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[\(p\^2\),
                              PP], "+", \(m\_2\%2\), "-", \(5  m\_3\%2\)}], 
                          ")"}]}], "+", \(\((7  d - 25)\) m\_1\%2\)}], 
                    ")"}], 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True]}], "+", "\n", "\t\t\t\t", 
                RowBox[{"2", \(m\_3\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[\(p\^2\),
                        PP], "-", \(m\_3\%2\)}], ")"}], 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\((d - 3)\), 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[\(p\^2\),
                              PP], "-", \(5  m\_2\%2\), "+", \(m\_3\%2\)}], 
                          ")"}]}], "+", \(\((7  d - 25)\) m\_1\%2\)}], 
                    ")"}], 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}], "-", "\n", "\t\t\t\t", 
                RowBox[{\((d - 3)\), \((d - 4)\), 
                  SuperscriptBox[
                    RowBox[{"(", 
                      InterpretationBox[\(p\^2\),
                        PP], ")"}], "2"]}], "+", 
                RowBox[{
                  InterpretationBox[\(p\^2\),
                    PP], 
                  \((\((7  d - 30)\) \((d - 3)\) t\_1 - 
                      \((7  d - 31)\) \((3  d - 10)\) m\_1\%2)\)}], "+", 
                \(\(1\/4\) \((d - 5)\) 
                  \((\((17  d - 66)\) t\_1\^2 - \((3  d - 14)\) t\_2 - 
                      4 \((3  d - 10)\) 
                        \((\((m\_1\%2)\)\^2 + 5 \( m\_2\%2\) m\_3\%2)\))\)\), 
                "+", "\t", 
                RowBox[{\(2\/\(d - 4\)\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              RowBox[{"(", 
                                RowBox[{
                                  InterpretationBox[\(p\^2\),
                                    PP], "+", \(m\_2\%2\)}], ")"}], 
                              \((7  d - 24)\)}], "+", 
                            \(\((d - 4)\) m\_1\%2\), "-", 
                            \(\((5  d - 18)\) m\_3\%2\)}], ")"}], 
                        \(m\_1\%2\), \(m\_2\%2\), 
                        InterpretationBox[
                          StyleBox[\(1\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "+"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(2\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 2, "+"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(3\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "-"],
                          Editable->True]}], "+", "\n", "\t\t\t\t\t\t", 
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              RowBox[{"(", 
                                RowBox[{
                                  InterpretationBox[\(p\^2\),
                                    PP], "+", \(m\_3\%2\)}], ")"}], 
                              \((7  d - 24)\)}], "+", 
                            \(\((d - 4)\) m\_1\%2\), "-", 
                            \(\((5  d - 18)\) m\_2\%2\)}], ")"}], 
                        \(m\_1\%2\), \(m\_3\%2\), 
                        InterpretationBox[
                          StyleBox[\(1\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "+"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(2\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 2, "-"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(3\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "+"],
                          Editable->True]}], "-", "\n", "\t\t\t\t\t\t", 
                      RowBox[{"2", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              RowBox[{"(", 
                                RowBox[{
                                  InterpretationBox[\(p\^2\),
                                    PP], "+", \(m\_2\%2\), "+", 
                                  \(m\_3\%2\)}], ")"}], \((d - 3)\)}], "+", 
                            \(\((d - 4)\) m\_1\%2\)}], ")"}], \(m\_2\%2\), 
                        \(m\_3\%2\), 
                        InterpretationBox[
                          StyleBox[\(1\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "-"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(2\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 2, "+"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(3\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "+"],
                          Editable->True]}]}], ")"}]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox["1",
                    "TraditionalForm"], 
                  FormBox["1",
                    "TraditionalForm"], 
                  FormBox["1",
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox[\(d - 2\),
                    "TraditionalForm"], ")"}]],
              TJI[ 
                Plus[ -2, d], {1, 1, 1}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_90 = OperatorApplyJ[e\_90]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (90)", "Subsection",
  CellTags->"T1.6.73.1"],

Cell[BoxData[
    \(\(h\_90 = s\_90; \)\)], "Input"],

Cell[BoxData[
    \(f\_90 = 
      MakeFun[h\_90, "\<(90)\>", 
        IFF[PP =!= 0 && MatchQ[d, _Symbol + _Integer?Positive]]]\)], "Input"],

Cell["CheckTJIRecursion not applicable.", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (91)", "Subsection",
  CellTags->"T1.6.74.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_91.1\), "=", " ", 
        RowBox[{
          RowBox[{\((d - 2)\), \(\[Nu]\_1\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["K",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TKI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{\(-u\_\(1, 2, 3\)\), "-", 
                InterpretationBox[
                  StyleBox[\(1\^-\),
                    FontWeight->"Bold"],
                  Schiebe[ 1, "-"],
                  Editable->True], "+", 
                InterpretationBox[
                  StyleBox[\(2\^-\),
                    FontWeight->"Bold"],
                  Schiebe[ 2, "-"],
                  Editable->True], "+", 
                InterpretationBox[
                  StyleBox[\(3\^-\),
                    FontWeight->"Bold"],
                  Schiebe[ 3, "-"],
                  Editable->True]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["K",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox[\(d - 2\),
                    "TraditionalForm"], ")"}]],
              TKI[ 
                Plus[ -2, d], {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_91.1 = OperatorApplyK[e\_91.1]\)], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_91.2\), "=", 
        RowBox[{
          RowBox[{\((d - 2)\), \(\[Nu]\_2\), 
            InterpretationBox[
              StyleBox[\(2\^+\),
                FontWeight->"Bold"],
              Schiebe[ 2, "+"],
              Editable->True], \(\[Nu]\_3\), 
            InterpretationBox[
              StyleBox[\(3\^+\),
                FontWeight->"Bold"],
              Schiebe[ 3, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["K",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TKI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(-2\), \(m\_1\%2\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True]}], "+", \((d - 2 - 2  \[Nu]\_1)\)}], 
              ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["K",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox[\(d - 2\),
                    "TraditionalForm"], ")"}]],
              TKI[ 
                Plus[ -2, d], {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_91.2 = OperatorApplyK[e\_91.2]\)], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_91.3\), "=", 
        RowBox[{
          RowBox[{\((d - 2)\), \((d - \[Nu]\_1 - \[Nu]\_2 - \[Nu]\_3)\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["K",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TKI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"-", 
              RowBox[{"(", 
                RowBox[{\(\[CapitalDelta]\_\(1, 2, 3\)\), "+", 
                  RowBox[{\(u\_\(1, 2, 3\)\), 
                    InterpretationBox[
                      StyleBox[\(1\^-\),
                        FontWeight->"Bold"],
                      Schiebe[ 1, "-"],
                      Editable->True]}], "+", 
                  RowBox[{\(u\_\(2, 1, 3\)\), 
                    InterpretationBox[
                      StyleBox[\(2\^-\),
                        FontWeight->"Bold"],
                      Schiebe[ 2, "-"],
                      Editable->True]}], "+", 
                  RowBox[{\(u\_\(3, 1, 2\)\), 
                    InterpretationBox[
                      StyleBox[\(3\^-\),
                        FontWeight->"Bold"],
                      Schiebe[ 3, "-"],
                      Editable->True]}]}], ")"}]}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["K",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox[\(d - 2\),
                    "TraditionalForm"], ")"}]],
              TKI[ 
                Plus[ -2, d], {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_91.3 = OperatorApplyK[e\_91.3]\)], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_91.4\), "=", 
        RowBox[{
          RowBox[{\(\[CapitalDelta]\_\(1, 2, 3\)\), \(\[Nu]\_1\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["K",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TKI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
              \(\(u\_\(1, 2, 3\)\) \((d - \[Nu]\_1 - 2  \[Nu]\_2)\)\), "+", 
                \(2 \( m\_2\%2\) \((\[Nu]\_1 - \[Nu]\_2)\)\), "+", 
                RowBox[{\(u\_\(3, 1, 2\)\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(2\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 2, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(3\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 3, "-"],
                        Editable->True]}], ")"}]}], "+", 
                RowBox[{"2", \(m\_2\%2\), \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(1\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 1, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(3\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 3, "-"],
                        Editable->True]}], ")"}]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["K",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TKI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_91.4 = OperatorApplyK[e\_91.4]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (911)", "Subsection",
  CellTags->"T1.6.75.1"],

Cell[BoxData[
    \(h\_911 = s\_91.1 /. \[Nu]\_1 -> \[Nu]\_1 - 1\)], "Input"],

Cell[BoxData[
    \(f\_911 = 
      MakeFun[h\_911, "\<(911)\>", 
        IFF[\[Nu]\_1 > 1 && MatchQ[d, _Symbol + _Integer?Positive]]]\)], 
  "Input"],

Cell["CheckTJIRecursion not applicable.", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (912)", "Subsection",
  CellTags->"T1.6.76.1"],

Cell[BoxData[
    \(h\_912 = 
      \(s\_91.2 /. \[Nu]\_2 -> \[Nu]\_2 - 1\) /. \[Nu]\_3 -> \[Nu]\_3 - 1\)], 
  "Input"],

Cell[BoxData[
    \(f\_912 = 
      MakeFun[h\_912, "\<(912)\>", 
        IFF[\[Nu]\_2 > 1 && \[Nu]\_3 > 1 && 
            MatchQ[d, _Symbol + _Integer?Positive]]]\)], "Input"],

Cell["CheckTJIRecursion not applicable.", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (913)", "Subsection",
  CellTags->"T1.6.77.1"],

Cell[BoxData[
    \(h\_913 = s\_91.3\)], "Input"],

Cell[BoxData[
    \(f\_913 = 
      MakeFun[h\_913, "\<(913)\>", 
        IFF[\((n2 === 1\  || \ n3 === 1)\) && 
            MatchQ[d, _Symbol + _Integer?Positive]], n1 :> 1]\)], "Input"],

Cell["CheckTJIRecursion not applicable.", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (914)", "Subsection",
  CellTags->"T1.6.78.1"],

Cell[BoxData[
    \(h\_914 = s\_91.4 /. \[Nu]\_1 -> \[Nu]\_1 - 1\)], "Input"],

Cell[BoxData[
    \(f\_914 = 
      MakeFun[h\_914, "\<(914)\>", 
        IFF[\[CapitalDelta]\_\(1, 2, 3\) =!= 0 && \((\[Nu]\_1\  > \ 1\ )\) && 
            MatchQ[d, _Symbol]]]\)], "Input"],

Cell["CheckTJIRecursion not applicable.", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (92)", "Subsection",
  CellTags->"T1.6.79.1"],

Cell[TextData[{
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta]\_\(1, 2, 3\)\)]],
  " = Cayley[1,2,3][{m1,m2,m3}]"
}], "Text"],

Cell[BoxData[
    \(\(Cayley[1, 2, 3]\)[{m1, m2, m3}]\)], "Input"],

Cell[TextData[Cell[BoxData[
    \(TraditionalForm\`\((\ 
        m\_1\%2 == \((m\_2 + m\_3)\)\^2\ )\)\ \[DoubleLongRightArrow]\ \((\ \
\[CapitalDelta]\_\(1, 2, 3\) == 0\ )\)\)]]], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_92\), "=", 
        RowBox[{
          RowBox[{
          "2", \(m\_2\), \(m\_3\), \((m\_2 + m\_3)\), 
            \((d - 2  \[Nu]\_1 - 2  \[Nu]\_2 - 2  \[Nu]\_3 - 1)\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["K",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TKI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(m\_2\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                      \((d - \[Nu]\_1 - \[Nu]\_2 - 2  \[Nu]\_3 - 1)\), 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              InterpretationBox[
                                StyleBox[\(1\^+\),
                                  FontWeight->"Bold"],
                                Schiebe[ 1, "+"],
                                Editable->True], 
                              InterpretationBox[
                                StyleBox[\(2\^-\),
                                  FontWeight->"Bold"],
                                Schiebe[ 2, "-"],
                                Editable->True]}], "-", "1"}], ")"}]}], "+", 
                      RowBox[{\((\[Nu]\_1 - \[Nu]\_2 + 1)\), 
                        InterpretationBox[
                          StyleBox[\(1\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "+"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(3\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "-"],
                          Editable->True]}]}], ")"}]}], "+", "\n", "\t\t\t", 
                RowBox[{\(m\_3\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                      \((d - \[Nu]\_1 - 2  \[Nu]\_2 - \[Nu]\_3 - 1)\), 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              InterpretationBox[
                                StyleBox[\(1\^+\),
                                  FontWeight->"Bold"],
                                Schiebe[ 1, "+"],
                                Editable->True], 
                              InterpretationBox[
                                StyleBox[\(3\^-\),
                                  FontWeight->"Bold"],
                                Schiebe[ 3, "-"],
                                Editable->True]}], "-", "1"}], ")"}]}], "+", 
                      RowBox[{\((\[Nu]\_1 - \[Nu]\_3 + 1)\), 
                        InterpretationBox[
                          StyleBox[\(1\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "+"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(2\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 2, "-"],
                          Editable->True]}]}], ")"}]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["K",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TKI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_92 = OperatorApplyK[e\_92]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (93)", "Subsection",
  CellTags->"T1.6.80.1"],

Cell[TextData[{
  "( ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\((m\_1 == 0)\)\  && \ \((m\_2 == 
                m\_3)\)\)\(\ \)\(\ \)\()\)\)\ \[DoubleLongRightArrow]\ \((\ \
\[CapitalDelta]\_\(1, 2, 3\) == 0\ )\)\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_93\), "=", 
        RowBox[{
          RowBox[{
          "2", \(m\_2\%2\), \((d - 2  \[Nu]\_1 - 2)\), 
            \((d - 2  \[Nu]\_1 - \[Nu]\_2 - \[Nu]\_3 - 1)\), 
            \((d - 2  \[Nu]\_1 - \[Nu]\_2 - \[Nu]\_3)\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["K",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TKI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}], "==", 
          RowBox[{
          \((d - 2  \[Nu]\_1 - 2  \[Nu]\_3)\), 
            \((d - 2  \[Nu]\_1 - 2  \[Nu]\_2)\), 
            \((d - \[Nu]\_1 - \[Nu]\_2 - \[Nu]\_3)\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["K",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TKI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_93 = OperatorApplyK[e\_93]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (93)", "Subsection",
  CellTags->"T1.6.81.1"],

Cell[BoxData[
    \(h\_93 = s\_93 /. \[Nu]\_1 -> \[Nu]\_1 - 1\)], "Input"],

Cell[BoxData[
    \(f\_93 = 
      MakeFun[h\_93, "\<(93)\>", 
        IFF[\[Nu]\_1 > 1 && m\_2 =!= 0], {m1 :> 0, m3 :> m2}]\)], "Input"],

Cell["CheckTJIRecursion not applicable.", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (94)", "Subsection",
  CellTags->"T1.6.82.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_94\), "=", 
        RowBox[{
          RowBox[{\(\[CapitalDelta]\_\(1, 2, 6\)\), \(\[Nu]\_1\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["B",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TBI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(u\_\(6, 1, 2\)\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\(\[Nu]\_1\), 
                        InterpretationBox[
                          StyleBox[\(1\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "+"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(2\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 2, "-"],
                          Editable->True]}], "-", "d", "+", \(\[Nu]\_1\), 
                      "+", \(2  \[Nu]\_2\)}], ")"}]}], "+", 
                RowBox[{"2", \(m\_2\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\(\[Nu]\_2\), 
                        InterpretationBox[
                          StyleBox[\(2\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 2, "+"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(1\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "-"],
                          Editable->True]}], "-", "d", "+", \(2  \[Nu]\_1\), 
                      "+", \(\[Nu]\_2\)}], ")"}]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["B",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TBI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_94 = OperatorApplyB[e\_94]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (94)", "Subsection",
  CellTags->"T1.6.83.1"],

Cell[BoxData[
    \(h\_94 = s\_94 /. \[Nu]\_1 -> \[Nu]\_1 - 1\)], "Input"],

Cell[BoxData[
    \(f\_94 = 
      MakeFun[h\_94, "\<(94)\>", 
        IFF[\[CapitalDelta]\_\(1, 2, 6\) =!= 0 && \[Nu]\_1 > 1]]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (9401)", "Subsection",
  CellTags->"T1.6.84.1"],

Cell[BoxData[
    \(n\_9401 = 
      \(s\_94 /. \[CapitalDelta]\_\(1, 2, 6\) -> 0\) /. 
        u\_\(6, 1, 2\) -> 0\)], "Input"],

Cell[BoxData[
    \(h\_9401 = \(-n\_9401[\([2, 2]\)]\) == n\_9401[\([2, 1]\)]\)], "Input"],

Cell[BoxData[
    \(f\_9401 = 
      MakeFun[h\_9401, "\<(9401)\>", 
        IFF[\[CapitalDelta]\_\(1, 2, 6\) === 0 && u\_\(6, 1, 2\) === 0 && 
            m2 =!= 0]]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (9402)", "Subsection",
  CellTags->"T1.6.85.1"],

Cell[BoxData[
    \(n\_9402 = 
      Interchange[
        \(s\_94 /. \[CapitalDelta]\_\(1, 2, 6\) -> 0\) /. 
          u\_\(6, 1, 2\) -> 0, 1 \[LeftRightArrow] 2]\)], "Input"],

Cell[BoxData[
    \(h\_9402 = \(-n\_9402[\([2, 1]\)]\) == n\_9402[\([2, 2]\)]\)], "Input"],

Cell[BoxData[
    \(f\_9402 = 
      MakeFun[h\_9402, "\<(9402)\>", 
        IFF[\[CapitalDelta]\_\(1, 2, 6\) === 0 && u\_\(6, 1, 2\) === 0 && 
            m1 =!= 0]]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (95)", "Subsection",
  CellTags->"T1.6.86.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_95\), "=", 
        RowBox[{
          RowBox[{"2", 
            InterpretationBox[\(p\^2\),
              PP], \(\[Nu]\_1\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["B",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TBI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{\(u\_\(1, 2, 6\)\), "+", 
                InterpretationBox[
                  StyleBox[\(1\^-\),
                    FontWeight->"Bold"],
                  Schiebe[ 1, "-"],
                  Editable->True], "-", 
                InterpretationBox[
                  StyleBox[\(2\^-\),
                    FontWeight->"Bold"],
                  Schiebe[ 2, "-"],
                  Editable->True]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["B",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox[\(d - 2\),
                    "TraditionalForm"], ")"}]],
              TBI[ 
                Plus[ -2, d], {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_95 = OperatorApplyB[e\_95]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (95)", "Subsection",
  CellTags->"T1.6.87.1"],

Cell[BoxData[
    \(h\_95 = s\_95 /. \[Nu]\_1 -> \[Nu]\_1 - 1\)], "Input"],

Cell[BoxData[
    \(f\_95 = 
      MakeFun[h\_95, "\<(95)\>", 
        IFF[\[Nu]\_1 > 1\  && \ MatchQ[d, _Symbol + _Integer?Positive]]]\)], 
  "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (96)", "Subsection",
  CellTags->"T1.6.88.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_96\), "=", 
        RowBox[{
          RowBox[{"2", 
            InterpretationBox[\(p\^2\),
              PP], \((d - \[Nu]\_1 - \[Nu]\_2 - 1)\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["B",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TBI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{\(\[CapitalDelta]\_\(1, 2, 6\)\), "+", 
                RowBox[{\(u\_\(1, 2, 6\)\), 
                  InterpretationBox[
                    StyleBox[\(1\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "-"],
                    Editable->True]}], "+", 
                RowBox[{\(u\_\(2, 1, 6\)\), 
                  InterpretationBox[
                    StyleBox[\(2\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "-"],
                    Editable->True]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["B",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox[\(d - 2\),
                    "TraditionalForm"], ")"}]],
              TBI[ 
                Plus[ -2, d], {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_96 = OperatorApplyB[e\_96]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (96)", "Subsection",
  CellTags->"T1.6.89.1"],

Cell[BoxData[
    \(h\_96 = s\_96\)], "Input"],

Cell[BoxData[
    \(f\_96 = 
      MakeFun[h\_96, "\<(96)\>", 
        IFF[\ MatchQ[d, _Symbol + _Integer\ ?Positive]], n1 :> 1]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (97)", "Subsection",
  CellTags->"T1.6.90.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_97\), "=", 
        RowBox[{
          RowBox[{"2", \(m\_1\%2\), \(\[Nu]\_1\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["A",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                FormBox[\(\[Nu]\_1\),
                  "TraditionalForm"], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TAI[ d, {
                Subscript[ \[Nu], 1]}],
              Editable->True]}], "==", 
          RowBox[{\((d - 2  \[Nu]\_1)\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["A",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                FormBox[\(\[Nu]\_1\),
                  "TraditionalForm"], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TAI[ d, {
                Subscript[ \[Nu], 1]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_97 = OperatorApplyA[e\_97]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (97)", "Subsection",
  CellTags->"T1.6.91.1"],

Cell[BoxData[
    \(h\_97 = s\_97 /. \[Nu]\_1 -> \[Nu]\_1 - 1\)], "Input"],

Cell[BoxData[
    \(f\_97 = 
      MakeFun[h\_97, "\<(97)\>", IFF[\[Nu]\_1 > 1 && \ MatchQ[d, _Symbol\ ]]]
        \)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (98)", "Subsection",
  CellTags->"T1.6.92.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_98\), "=", 
        RowBox[{
          RowBox[{\((d - 2  \[Nu]\_1)\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["A",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                FormBox[\(\[Nu]\_1\),
                  "TraditionalForm"], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TAI[ d, {
                Subscript[ \[Nu], 1]}],
              Editable->True]}], "==", 
          RowBox[{\(-2\), \(m\_1\%2\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["A",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                FormBox[\(\[Nu]\_1\),
                  "TraditionalForm"], 
                RowBox[{"(", 
                  FormBox[\(d - 2\),
                    "TraditionalForm"], ")"}]],
              TAI[ 
                Plus[ -2, d], {
                Subscript[ \[Nu], 1]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_98 = OperatorApplyA[e\_98]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (98)", "Subsection",
  CellTags->"T1.6.93.1"],

Cell[BoxData[
    \(h\_98 = s\_98\)], "Input"],

Cell[BoxData[
    \(f\_98 = 
      MakeFun[h\_98, "\<(98)\>", 
        IFF[MatchQ[d, _Symbol\  + _Integer?Positive]]]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (99)", "Subsection",
  CellTags->"T1.6.94.1"],

Cell[TextData[Cell[BoxData[
    \(TraditionalForm\`m\_1 = M, \ p\^2 = M\^2, 
    m\_2 = \(m\_3 = 0\)\)]]], "Text"],

Cell[TextData[{
  "Herleitung in three.nb ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          
          RowBox[{\(-\[Nu]\_1\), " ", \((d - 2\ \[Nu]\_2 - 2\ \[Nu]\_3)\), 
            " ", \((d - \[Nu]\_2 - \[Nu]\_3 - 1)\), " ", 
            SubsuperscriptBox[
              StyleBox["J",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                StyleBox[
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"],
                  FontColor->RGBColor[0, 0, 1]], 
                FormBox[\(\[Nu]\_2\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_3\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox["d",
                  "TraditionalForm"], ")"}]]}], "==", 
          
          RowBox[{\(\[Nu]\_2\), " ", \((\(-d\) + 2\ \[Nu]\_2 + 2)\), 
            " ", \((\(-2\)\ d + \[Nu]\_1 + 2\ \[Nu]\_2 + 2\ \[Nu]\_3 + 2)\), 
            " ", 
            SubsuperscriptBox[
              StyleBox["J",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                FormBox[\(\[Nu]\_1\),
                  "TraditionalForm"], 
                StyleBox[
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"],
                  FontColor->RGBColor[0, 0, 1]], 
                FormBox[\(\[Nu]\_3\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox["d",
                  "TraditionalForm"], ")"}]]}]}], TraditionalForm]]]
}], "Text"],

Cell[BoxData[
    RowBox[{\(e\_99\), "=", 
      RowBox[{
        RowBox[{
        \(\[Nu]\_1\), " ", \((d - 2\ \[Nu]\_2 - 2\ \[Nu]\_3)\), " ", 
          \((d - \[Nu]\_2 - \[Nu]\_3 - 1)\), " ", 
          InterpretationBox[
            StyleBox[\(1\^+\),
              FontWeight->"Bold"],
            Schiebe[ 1, "+"],
            Editable->True], 
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["J",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                FormBox[\(\[Nu]\_1\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_2\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_3\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox["d",
                  "TraditionalForm"], ")"}]],
            TJI[ d, {
              Subscript[ \[Nu], 1], 
              Subscript[ \[Nu], 2], 
              Subscript[ \[Nu], 3]}],
            Editable->True]}], "==", 
        RowBox[{
        \(-\[Nu]\_2\), " ", \((\(-d\) + 2\ \[Nu]\_2 + 2)\), " ", 
          \((\(-2\)\ d + \[Nu]\_1 + 2\ \[Nu]\_2 + 2\ \[Nu]\_3 + 2)\), 
          InterpretationBox[
            StyleBox[\(2\^+\),
              FontWeight->"Bold"],
            Schiebe[ 2, "+"],
            Editable->True], 
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["J",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                FormBox[\(\[Nu]\_1\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_2\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_3\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox["d",
                  "TraditionalForm"], ")"}]],
            TJI[ d, {
              Subscript[ \[Nu], 1], 
              Subscript[ \[Nu], 2], 
              Subscript[ \[Nu], 3]}],
            Editable->True]}]}]}]], "Input"],

Cell[BoxData[
    \(s\_99 = OperatorApplyJ[e\_99]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (99)", "Subsection",
  CellTags->"T1.6.95.1"],

Cell[BoxData[
    \(h\_99 = s\_99 /. \[Nu]\_1 -> \[Nu]\_1 - 1\)], "Input"],

Cell[BoxData[
    \(f\_99 = 
      MakeFun[h\_99, "\<(99)\>", 
        IFF[m1^2 === PP && \[Nu]\_1 > 1], {m2 :> 0, m3 :> 0}]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (9912)", "Subsection",
  CellTags->"T1.6.96.1"],

Cell[BoxData[
    \(h\_9912 = 
      Interchange[s\_99 /. \[Nu]\_1 -> \[Nu]\_1 - 1, 1 \[LeftRightArrow] 2]
        \)], "Input"],

Cell[BoxData[
    \(f\_9912 = 
      MakeFun[h\_9912, "\<(9912)\>", 
        IFF[m2^2 === PP && \[Nu]\_2 > 1], {m1 :> 0, m3 :> 0}]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (9913)", "Subsection",
  CellTags->"T1.6.97.1"],

Cell[BoxData[
    \(h\_9913 = 
      Interchange[s\_99 /. \[Nu]\_1 -> \[Nu]\_1 - 1, 1 \[LeftRightArrow] 3]
        \)], "Input"],

Cell[BoxData[
    \(f\_9913 = 
      MakeFun[h\_9913, "\<(9913)\>", 
        IFF[m3^2 === PP && \[Nu]\_3 > 1], {m1 :> 0, m2 :> 0}]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (100)", "Subsection",
  CellTags->"T1.6.98.1"],

Cell["Herleitung in three.nb ", "Text"],

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
        "2", " ", \(m\_2\%2\), " ", \((1 - \[Nu]\_2)\), " ", \(\[Nu]\_2\), 
          " ", 
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["J",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                FormBox[\(\[Nu]\_1\),
                  "TraditionalForm"], 
                StyleBox[
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"],
                  FontColor->RGBColor[0, 0, 1]], 
                FormBox[\(\[Nu]\_3\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox["d",
                  "TraditionalForm"], ")"}]],
            TJI[ d, {
              Subscript[ \[Nu], 1], 
              Plus[ 1, 
                Subscript[ \[Nu], 2]], 
              Subscript[ \[Nu], 3]}],
            Editable->True]}], "==", 
        RowBox[{
          RowBox[{\((d - 2\ \[Nu]\_2)\), " ", \((1 - \[Nu]\_2)\), " ", 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}], "-", 
          RowBox[{\(\[Nu]\_1\), " ", \((\(-d\) + 2\ \[Nu]\_1 + 2)\), " ", 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  StyleBox[
                    FormBox[\(\[Nu]\_1\),
                      "TraditionalForm"],
                    FontColor->RGBColor[0, 0, 1]], 
                  StyleBox[
                    FormBox[\(\[Nu]\_2\),
                      "TraditionalForm"],
                    FontColor->RGBColor[1, 0, 0]], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {
                Plus[ 1, 
                  Subscript[ \[Nu], 1]], 
                Plus[ -1, 
                  Subscript[ \[Nu], 2]], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}]}]}], TraditionalForm]], "Input"]], "Text"],

Cell[TextData[Cell[BoxData[
    \(TraditionalForm\`m\_1 = 0\)]]], "Text"],

Cell[BoxData[
    RowBox[{\(e\_100\), "=", 
      RowBox[{
        RowBox[{
        "2", " ", \(m\_2\%2\), " ", \((1 - \[Nu]\_2)\), " ", \(\[Nu]\_2\), 
          " ", 
          InterpretationBox[
            StyleBox[\(2\^+\),
              FontWeight->"Bold"],
            Schiebe[ 2, "+"],
            Editable->True], 
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["J",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                FormBox[\(\[Nu]\_1\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_2\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_3\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox["d",
                  "TraditionalForm"], ")"}]],
            TJI[ d, {
              Subscript[ \[Nu], 1], 
              Subscript[ \[Nu], 2], 
              Subscript[ \[Nu], 3]}],
            Editable->True]}], "==", 
        RowBox[{
          RowBox[{\((d - 2\ \[Nu]\_2)\), " ", \((1 - \[Nu]\_2)\), " ", 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}], "-", 
          RowBox[{\(\[Nu]\_1\), " ", \((\(-d\) + 2\ \[Nu]\_1 + 2)\), " ", 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              StyleBox[\(2\^-\),
                FontWeight->"Bold"],
              Schiebe[ 2, "-"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}]}]}]}]], "Input"],

Cell[BoxData[
    \(s\_100 = OperatorApplyJ[e\_100]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (100)", "Subsection",
  CellTags->"T1.6.99.1"],

Cell[BoxData[
    \(h\_100 = s\_100\)], "Input"],

Cell[BoxData[
    \(f\_100 = 
      MakeFun[h\_100 /. \[Nu]\_2 -> \[Nu]\_2 - 1, "\<(100)\>", 
        IFF[\[Nu]\_2 > 2 && m2 =!= 0], m1 :> 0]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_100]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (1002)", "Subsection",
  CellTags->"T1.6.100.1"],

Cell[BoxData[
    \(h\_1002 = Interchange[s\_100, 1 \[LeftRightArrow] 3]\)], "Input"],

Cell[BoxData[
    \(f\_1002 = 
      MakeFun[h\_1002 /. \[Nu]\_2 -> \[Nu]\_2 - 1, "\<(1002)\>", 
        IFF[\[Nu]\_2 > 2 && m2 =!= 0], m3 :> 0]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_1002]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (1003)", "Subsection",
  CellTags->"T1.6.101.1"],

Cell[BoxData[
    \(h\_1003 = 
      Interchange[s\_100, 1 \[LeftRightArrow] 3, 1 \[LeftRightArrow] 2]\)], 
  "Input"],

Cell[BoxData[
    \(f\_1003 = 
      MakeFun[h\_1003 /. \[Nu]\_1 -> \[Nu]\_1 - 1, "\<(1003)\>", 
        IFF[\[Nu]\_1 > 2 && m1 =!= 0], m3 :> 0]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_1003]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (1004)", "Subsection",
  CellTags->"T1.6.102.1"],

Cell[BoxData[
    \(h\_1004 = Interchange[s\_100, 1 \[LeftRightArrow] 2]\)], "Input"],

Cell[BoxData[
    \(f\_1004 = 
      MakeFun[h\_1004 /. \[Nu]\_1 -> \[Nu]\_1 - 1, "\<(1004)\>", 
        IFF[\[Nu]\_1 > 2 && m1 =!= 0], m2 :> 0]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_1004]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (1005)", "Subsection",
  CellTags->"T1.6.103.1"],

Cell[BoxData[
    \(h\_1005 = Interchange[s\_100, 2 \[LeftRightArrow] 3]\)], "Input"],

Cell[BoxData[
    \(f\_1005 = 
      MakeFun[h\_1005 /. \[Nu]\_3 -> \[Nu]\_3 - 1, "\<(1005)\>", 
        IFF[\[Nu]\_3 > 2 && m3 =!= 0], m1 :> 0]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_1005]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (1006)", "Subsection",
  CellTags->"T1.6.104.1"],

Cell[BoxData[
    \(h\_1006 = 
      Interchange[s\_100, 2 \[LeftRightArrow] 3, 1 \[LeftRightArrow] 2]\)], 
  "Input"],

Cell[BoxData[
    \(f\_1006 = 
      MakeFun[h\_1006 /. \[Nu]\_3 -> \[Nu]\_3 - 1, "\<(1006)\>", 
        IFF[\[Nu]\_3 > 2 && m3 =!= 0], m2 :> 0]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_1006]\)], "Input"],

Cell[BoxData[
    \("Off"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (150)", "Subsection",
  CellTags->"T1.6.105.1"],

Cell[BoxData[
    \(TextForm\`\[Nu]\_4 = \(-1\)\)], "Text"],

Cell[BoxData[
    \(TextForm\`\((\(q\_2\)\(-\)\(p\)\(\ \))\)\^2 - m\_4\%2 = \ q\_2\%2 - 
      m\_2\%2 - 2  p\ q\_2 + p\^2 + m\_2\%2 - m\_4\%2\)], "Text"],

Cell[BoxData[
    \(s\_150 = 
      TFI[d, PP, {{n1, m1}, {n2, m2}, {n3, m3}, {\(-1\), m4}, {n5, m5}}] == 
        TFI[d, PP, {{n1, m1}, {n2 - 1, m2}, {n3, m3}, {0, 0}, {n5, m5}}] - 
          2  TFI[d, 
              PP, {0, 0, 0, 1, 
                0}, {{n1, m1}, {n2, m2}, {n3, m3}, {0, 0}, {n5, m5}}] + 
          \((PP + m2^2 - m4^2)\) 
            TFI[d, PP, {{n1, m1}, {n2, m2}, {n3, m3}, {0, 0}, {n5, m5}}]\)], 
  "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (150)", "Subsection",
  CellTags->"T1.6.106.1"],

Cell[BoxData[
    \(h\_150 = s\_150\)], "Input"],

Cell[BoxData[
    \(f\_150 = MakeFun[h\_150, "\<(150)\>"]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (151)", "Subsection",
  CellTags->"T1.6.107.1"],

Cell[BoxData[
    \(TextForm\`\[Nu]\_4 = \(-1\)\)], "Text"],

Cell[BoxData[
    \(TextForm\`\((\(q\_2\)\(-\)\(p\)\(\ \))\)\^2 - m\_4\%2 = \ q\_2\%2 - 
      m\_2\%2 - 2  p\ q\_2 + p\^2 + m\_2\%2 - m\_4\%2\)], "Text"],

Cell[BoxData[
    \(s\_151 = 
      TFI[d, PP, 
          DP, {a, b}, {{n1, m1}, {n2, m2}, {n3, m3}, {\(-1\), m4}, {n5, 
              m5}}] == 
        TFI[d, PP, 
            DP, {a, b}, {{n1, m1}, {n2 - 1, m2}, {n3, m3}, {0, 0}, {n5, 
                m5}}] - 
          2  TFI[d, PP, 
              DP, {a, b}, {0, 0, 0, 1, 
                0}, {{n1, m1}, {n2, m2}, {n3, m3}, {0, 0}, {n5, m5}}] + 
          \((PP + m2^2 - m4^2)\) 
            TFI[d, PP, 
              DP, {a, b}, {{n1, m1}, {n2, m2}, {n3, m3}, {0, 0}, {n5, m5}}]
              \)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (151)", "Subsection",
  CellTags->"T1.6.108.1"],

Cell[BoxData[
    \(h\_151 = s\_151\)], "Input"],

Cell[BoxData[
    \(f\_151 = 
      ReplacePart[MakeFun[h\_151, "\<(151)\>"], {a_, b_}, {1, 1, 4}]\)], 
  "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (152)", "Subsection",
  CellTags->"T1.6.109.1"],

Cell[BoxData[
    \(TextForm\`\[Nu]\_4 = \(-1\)\)], "Text"],

Cell[BoxData[
    \(TextForm\`\((\(q\_2\)\(-\)\(p\)\(\ \))\)\^2 - m\_4\%2 = \ q\_2\%2 - 
      m\_2\%2 - 2  p\ q\_2 + p\^2 + m\_2\%2 - m\_4\%2\)], "Text"],

Cell[BoxData[
    \(s\_152 = 
      TVI[d, PP, {{n1, m1}, {n2, m2}, {n3, m3}, {\(-1\), m4}}] == 
        TVI[d, PP, {{n1, m1}, {n2 - 1, m2}, {n3, m3}, {0, 0}}] - 
          2  TFI[d, 
              PP, {0, 0, 0, 1, 
                0}, {{0, 0}, {n2, m2}, {n3, m3}, {0, 0}, {n1, m1}}] + 
          \((PP + m2^2 - m4^2)\) 
            TVI[d, PP, {{n1, m1}, {n2, m2}, {n3, m3}, {0, 0}}]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (152)", "Subsection",
  CellTags->"T1.6.110.1"],

Cell[BoxData[
    \(h\_152 = s\_152\)], "Input"],

Cell[BoxData[
    \(f\_152 = MakeFun[h\_152, "\<(152)\>"]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (153)", "Subsection",
  CellTags->"T1.6.111.1"],

Cell[BoxData[
    \(TextForm\`\[Nu]\_2 = \(-1\)\)], "Text"],

Cell[BoxData[
    \(s\_153 = 
      TVI[d, PP, {{n1, m1}, {\(-1\), m2}, {n3, m3}, {n4, m4}}] == \n\t\t
        TFI[d, \ PP, {0, 1, 0, 0, 0}, 
            \ {{0, 0}, {0, 0}, {n3, m3}, {n4, m4}, {n1, m1}}] + 
          m2^2\ TVI[d, \ PP, \ {{n1, m1}, {0, 0}, {n3, m3}, {n4, m4}}]\)], 
  "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (153)", "Subsection",
  CellTags->"T1.6.112.1"],

Cell[BoxData[
    \(h\_153 = s\_153\)], "Input"],

Cell[BoxData[
    \(f\_153 = MakeFun[h\_153, "\<(153)\>"]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (200)", "Subsection",
  CellTags->"T1.6.113.1"],

Cell["Derivation in TJRspecial.nb", "Text"],

Cell[TextData[Cell[BoxData[
    \(TraditionalForm\`m\_3 = \(m\_2 = m\_1\), \ p\^2 = m\_1\%2\)]]], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(s\_200\), "=", 
        RowBox[{
          RowBox[{
          "16", " ", \(m\_1\%2\), " ", \(\[Nu]\_1\), " ", 
            \((d - \[Nu]\_1 - \[Nu]\_2 - \[Nu]\_3)\), " ", 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  StyleBox[
                    FormBox[\(\[Nu]\_1\),
                      "TraditionalForm"],
                    FontColor->RGBColor[0, 0, 1]], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {
                Plus[ 1, 
                  Subscript[ \[Nu], 1]], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{
            \(-\((1 + 3\ d - 3\ \[Nu]\_1 - 4\ \[Nu]\_2)\)\), " ", 
              \(\[Nu]\_3\), " ", 
              InterpretationBox[
                SubsuperscriptBox[
                  StyleBox["J",
                    SingleLetterItalics->False,
                    FontWeight->"Bold"], 
                  RowBox[{
                    StyleBox[
                      FormBox[\(\[Nu]\_1\),
                        "TraditionalForm"],
                      FontColor->RGBColor[1, 0, 0]], 
                    FormBox[\(\[Nu]\_2\),
                      "TraditionalForm"], 
                    StyleBox[
                      FormBox[\(\[Nu]\_3\),
                        "TraditionalForm"],
                      FontColor->RGBColor[0, 0, 1]]}], 
                  RowBox[{"(", 
                    FormBox["d",
                      "TraditionalForm"], ")"}]],
                TJI[ d, {
                  Plus[ -1, 
                    Subscript[ \[Nu], 1]], 
                  Subscript[ \[Nu], 2], 
                  Plus[ 1, 
                    Subscript[ \[Nu], 3]]}],
                Editable->True]}], "-", 
            RowBox[{
            \(\[Nu]\_2\), " ", \((1 + 3\ d - 3\ \[Nu]\_1 - 4\ \[Nu]\_3)\), 
              " ", 
              InterpretationBox[
                SubsuperscriptBox[
                  StyleBox["J",
                    SingleLetterItalics->False,
                    FontWeight->"Bold"], 
                  RowBox[{
                    StyleBox[
                      FormBox[\(\[Nu]\_1\),
                        "TraditionalForm"],
                      FontColor->RGBColor[1, 0, 0]], 
                    StyleBox[
                      FormBox[\(\[Nu]\_2\),
                        "TraditionalForm"],
                      FontColor->RGBColor[0, 0, 1]], 
                    FormBox[\(\[Nu]\_3\),
                      "TraditionalForm"]}], 
                  RowBox[{"(", 
                    FormBox["d",
                      "TraditionalForm"], ")"}]],
                TJI[ d, {
                  Plus[ -1, 
                    Subscript[ \[Nu], 1]], 
                  Plus[ 1, 
                    Subscript[ \[Nu], 2]], 
                  Subscript[ \[Nu], 3]}],
                Editable->True]}], "+", 
            RowBox[{
            \((\(-1\) + 2\ d - \[Nu]\_1 - 2\ \[Nu]\_2)\), " ", \(\[Nu]\_3\), 
              " ", 
              InterpretationBox[
                SubsuperscriptBox[
                  StyleBox["J",
                    SingleLetterItalics->False,
                    FontWeight->"Bold"], 
                  RowBox[{
                    FormBox[\(\[Nu]\_1\),
                      "TraditionalForm"], 
                    StyleBox[
                      FormBox[\(\[Nu]\_2\),
                        "TraditionalForm"],
                      FontColor->RGBColor[1, 0, 0]], 
                    StyleBox[
                      FormBox[\(\[Nu]\_3\),
                        "TraditionalForm"],
                      FontColor->RGBColor[0, 0, 1]]}], 
                  RowBox[{"(", 
                    FormBox["d",
                      "TraditionalForm"], ")"}]],
                TJI[ d, {
                  Subscript[ \[Nu], 1], 
                  Plus[ -1, 
                    Subscript[ \[Nu], 2]], 
                  Plus[ 1, 
                    Subscript[ \[Nu], 3]]}],
                Editable->True]}], "-", 
            RowBox[{
            \((\(-6\)\ d\^2 + 22\ d\ \[Nu]\_1 - 16\ \[Nu]\_1\%2 - \[Nu]\_2 + 
                  6\ d\ \[Nu]\_2 - 13\ \[Nu]\_1\ \[Nu]\_2 - \[Nu]\_3 + 
                  6\ d\ \[Nu]\_3 - 13\ \[Nu]\_1\ \[Nu]\_3 - 
                  4\ \[Nu]\_2\ \[Nu]\_3)\), " ", 
              InterpretationBox[
                SubsuperscriptBox[
                  StyleBox["J",
                    SingleLetterItalics->False,
                    FontWeight->"Bold"], 
                  RowBox[{
                    FormBox[\(\[Nu]\_1\),
                      "TraditionalForm"], 
                    FormBox[\(\[Nu]\_2\),
                      "TraditionalForm"], 
                    FormBox[\(\[Nu]\_3\),
                      "TraditionalForm"]}], 
                  RowBox[{"(", 
                    FormBox["d",
                      "TraditionalForm"], ")"}]],
                TJI[ d, {
                  Subscript[ \[Nu], 1], 
                  Subscript[ \[Nu], 2], 
                  Subscript[ \[Nu], 3]}],
                Editable->True]}], "+", 
            RowBox[{
            \(\[Nu]\_2\), " ", \((\(-1\) + 2\ d - \[Nu]\_1 - 2\ \[Nu]\_3)\), 
              " ", 
              InterpretationBox[
                SubsuperscriptBox[
                  StyleBox["J",
                    SingleLetterItalics->False,
                    FontWeight->"Bold"], 
                  RowBox[{
                    FormBox[\(\[Nu]\_1\),
                      "TraditionalForm"], 
                    StyleBox[
                      FormBox[\(\[Nu]\_2\),
                        "TraditionalForm"],
                      FontColor->RGBColor[0, 0, 1]], 
                    StyleBox[
                      FormBox[\(\[Nu]\_3\),
                        "TraditionalForm"],
                      FontColor->RGBColor[1, 0, 0]]}], 
                  RowBox[{"(", 
                    FormBox["d",
                      "TraditionalForm"], ")"}]],
                TJI[ d, {
                  Subscript[ \[Nu], 1], 
                  Plus[ 1, 
                    Subscript[ \[Nu], 2]], 
                  Plus[ -1, 
                    Subscript[ \[Nu], 3]]}],
                Editable->True]}]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(TextForm\`Due\ to\ sorting\ \ \[Nu]\_1 \[GreaterSlantEqual] \[Nu]\_2 \
\[GreaterSlantEqual] \[Nu]\_3\ \ by\ TJI\ only\ this\ rule\ is\ \(\(necessary\
\)\(.\)\)\)], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (200)", "Subsection",
  CellTags->"T1.6.114.1"],

Cell[BoxData[
    \(h\_200 = s\_200 /. \[Nu]\_1 -> \[Nu]\_1 - 1\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_200\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_200\), ",", "\"\<(200)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            \(\[Nu]\_1 > 1 && m1 =!= 0 && PP === m1^2\), "]"}], ",", 
          \({m2 :> m1, m3 :> m1}\)}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (201)", "Subsection",
  CellTags->"T1.6.115.1"],

Cell["\<\
Subscript[f, 201] = 
  HoldForm[TJR[d_, PP_, 
     {{(n1_)?PQ, 0}, 
      {(n2_)?PQ, 0}, 
      {(n3_)?PQ, m3_}}] := 
    (TComment[\"(201)\", 
        TJIC[d, 
         {{n1, 0}, {n2, 0}, 
          {n3, m3}}]]; 
       -(((d - n1 - n2)*
          (-2 - d + 
         2*n1 + 2*n2)*
          (3*d - 2*n1 - 
         2*n2 - 2*n3)*
          (-1 - d + n1 + 
          n2 + n3)*
          TJI[d, PP, 
          {{-1 + n1, 0}, 
          {n2, 0}, {n3, m3}}]
)/(2*m3^2*(d - 2*n1)*
          (-1 + n1)*
          (2*d - 2*n1 - 
          2*n2 - n3)*
          (1 + 2*d - 2*n1 - 
          2*n2 - n3)))) /; 
     n1 > 1 && m3 =!= 0 && 
      PP === m3^2]\
\>", "Input",
  Editable->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["MakeFun (202)", "Subsection",
  CellTags->"T1.6.116.1"],

Cell["\<\
Subscript[f, 202] = 
  HoldForm[TJR[d_, PP_, 
     {{(n1_)?PQ, 0}, 
      {(n3_)?PQ, m3_}, 
      {(n2_)?PQ, 0}}] := 
    (TComment[\"(202)\", 
        TJIC[d, 
         {{n1, 0}, {n2, 0}, 
          {n3, m3}}]]; 
       -(((d - n1 - n2)*
          (-2 - d + 2*n1 + 
          2*n2)*
          (3*d - 2*n1 - 
          2*n2 - 2*n3)*
          (-1 - d + n1 + 
          n2 + n3)*
          TJI[d, m3^2, 
          {{-1 + n1, 0}, 
          {n2, 0}, {n3, m3}}
])/(2*m3^2*(d - 2*n1)*
          (-1 + n1)*
          (2*d - 2*n1 - 
          2*n2 - n3)*
          (1 + 2*d - 2*n1 - 
          2*n2 - n3)))) /; 
     n1 > 1 && m3 =!= 0 && 
      PP === m3^2]\
\>", "Input",
  Editable->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[" MakeFun (203)", "Subsection",
  CellTags->"T1.6.117.1"],

Cell[BoxData[
    \(Subscript[f, \ 203]\  = 
      HoldForm[\n\t\t\ 
        TJR[d_, \ 0, 
            \ {{\((n1_)\)?PQ, \ 0}, \ {\((n2_)\)?PQ, \ m2_}, \ \n
              \ \ \ \ {\((n3_)\)?PQ, \ m2_}}]\  := 
          \((\n\t\t\t\t\ 
            TComment["\<(203)\>", \ \n\ \ \ \ \ \ \ \ 
              TJIC[d, 0, \ {{n1, \ 0}, \ {n2, \ m2}, \ {n3, \ m2}}]]; \ \n
            \t\t\t\t\((
                \((d\  - \ 2*\((\(-1\)\  + \ n1\  + \ n2)\))\)*\n\ \ \ \ \ 
                  \((1\  + \ d\  - \ n1\  - \ n2\  - \ n3)\)*\n\ \ \ \ \ 
                  \((d\  - \ 2*\((\(-1\)\  + \ n1\  + \ n3)\))\)*\n\ \ \ \ \ 
                  TJR[d, \ 0, 
                    \ {{\(-1\)\  + \ n1, \ 0}, \ {n2, \ m2}, \ \n
                      \ \ \ \ \ \ \ {n3, \ m2}}])\)/\n\ \ \ 
              \((2*m2^2*\((d\  - \ 2*n1)\)*\n\ \ \ \ \ 
                  \((1\  + \ d\  - \ 2*n1\  - \ n2\  - \ n3)\)*\n\ \ \ \ \ 
                  \((2\  + \ d\  - \ 2*n1\  - \ n2\  - \ n3)\))\)\n\t\t\t\n
            \t\t\t)\)]\)], "Input",
  Editable->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[" MakeFun (204)", "Subsection",
  CellTags->"T1.6.118.1"],

Cell[BoxData[
    \(Subscript[f, \ 204]\  = 
      HoldForm[\n\t\t\ 
        TJR[d_, \ 0, 
            \ {\ {\((n2_)\)?PQ, \ m2_}, \ {\((n1_)\)?PQ, \ 0}, \n
              \ \ \ \ {\((n3_)\)?PQ, \ m2_}}]\  := 
          \((\n\t\ TComment["\<(204)\>", \ \n\ \ \ \ \ \ \ \ 
              TJIC[d, 0, \ {\ {n2, \ m2}, \ {n1, \ 0}, {n3, \ m2}}]]; \ \n\t
            \((\((d\  - \ 2*\((\(-1\)\  + \ n1\  + \ n2)\))\)*\n\ \ \ \ \ 
                  \((1\  + \ d\  - \ n1\  - \ n2\  - \ n3)\)*\n\ \ \ \ \ 
                  \((d\  - \ 2*\((\(-1\)\  + \ n1\  + \ n3)\))\)*\n\ \ \ \ \ 
                  TJR[d, \ 0, 
                    \ {{\(-1\)\  + \ n1, \ 0}, \ {n2, \ m2}, \ \n
                      \ \ \ \ \ \ \ {n3, \ m2}}])\)/\n\ \ \ 
              \((2*m2^2*\((d\  - \ 2*n1)\)*\n\ \ \ \ \ 
                  \((1\  + \ d\  - \ 2*n1\  - \ n2\  - \ n3)\)*\n\ \ \ \ \ 
                  \((2\  + \ d\  - \ 2*n1\  - \ n2\  - \ n3)\))\)\n\t\t\t)\)]
        \)], "Input",
  Editable->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[" MakeFun (205)", "Subsection",
  CellTags->"T1.6.119.1"],

Cell[BoxData[
    \(Subscript[f, \ 205]\  = 
      HoldForm[\n\t\t\ 
        TJR[d_, \ 0, 
            \ {\ {\((n2_)\)?PQ, \ m2_}, \n\ \ \ \ {\((n3_)\)?PQ, \ m2_}, 
              \ {\((n1_)\)?PQ, \ 0}}]\  := 
          \((\n\t\ TComment["\<(205)\>", \ \n\ \ \ \ \ \ \ \ 
              TJIC[d, 0, \ {\ {n2, \ m2}, {n3, \ m2}, \ {n1, \ 0}}]]; \ \n
            \((\((d\  - \ 2*\((\(-1\)\  + \ n1\  + \ n2)\))\)*\n\ \ \ \ \ 
                  \((1\  + \ d\  - \ n1\  - \ n2\  - \ n3)\)*\n\ \ \ \ \ 
                  \((d\  - \ 2*\((\(-1\)\  + \ n1\  + \ n3)\))\)*\n\ \ \ \ \ 
                  TJR[d, \ 0, 
                    \ {{\(-1\)\  + \ n1, \ 0}, \ {n2, \ m2}, \ \n
                      \ \ \ \ \ \ \ {n3, \ m2}}])\)/\n\ \ \ 
              \((2*m2^2*\((d\  - \ 2*n1)\)*\n\ \ \ \ \ 
                  \((1\  + \ d\  - \ 2*n1\  - \ n2\  - \ n3)\)*\n\ \ \ \ \ 
                  \((2\  + \ d\  - \ 2*n1\  - \ n2\  - \ n3)\))\)\n\t\t\t)\)]
        \)], "Input",
  Editable->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input (1xxxx)", "Subsection",
  CellTags->"T1.6.120.1"],

Cell["\<\
If[MatchQ[$RankLimit, {_Integer?NonNegative, 
     _Integer?NonNegative}], 
   $TarasovTdeltaplimit = $RankLimit[[1]]; 
    ($TarasovTplimit = $RankLimit[[2]]; ), 
   $TarasovTdeltaplimit = 2; $TarasovTplimit = 2]; \
\>", "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"tarti", "=", 
      RowBox[{"Timing", "[", 
        RowBox[{"Do", "[", 
          RowBox[{
            RowBox[{"If", "[", 
              RowBox[{\(0 < a + b + r + es\), ",", 
                RowBox[{\(WriteString["\<stdout\>", {a, b, r, es}]\), ";", 
                  RowBox[{"Set", "@@", 
                    RowBox[{"{", 
                      RowBox[{
                      \(s\_\(10\^4 + 1000\ a + 100\ b + 10\ r + es\)\), ",", 
                        RowBox[{
                          RowBox[{
                            RowBox[{
                              StyleBox["TFI",
                                FontColor->RGBColor[1, 0.6, 0]], "[", 
                              RowBox[{"d", ",", 
                                InterpretationBox[\(p\^2\),
                                  PP], ",", "DP", ",", \({a, b}\), ",", 
                                \({0, 0, r, es, 0}\), ",", 
                                RowBox[{"{", 
                                  RowBox[{
                                    RowBox[{"{", 
                                      RowBox[{"n1", ",", 
                                        InterpretationBox[\(m\_1\),
                                        m1]}], "}"}], ",", 
                                    RowBox[{"{", 
                                      RowBox[{"n2", ",", 
                                        InterpretationBox[\(m\_2\),
                                        m2]}], "}"}], ",", 
                                    RowBox[{"{", 
                                      RowBox[{"n3", ",", 
                                        InterpretationBox[\(m\_3\),
                                        m3]}], "}"}], ",", 
                                    RowBox[{"{", 
                                      RowBox[{"n4", ",", 
                                        InterpretationBox[\(m\_4\),
                                        m4]}], "}"}], ",", 
                                    RowBox[{"{", 
                                      RowBox[{"n5", ",", 
                                        InterpretationBox[\(m\_5\),
                                        m5]}], "}"}]}], "}"}]}], "]"}], 
                            "==", 
                            RowBox[{"ApplyTarasovT", "[", 
                              RowBox[{
                                RowBox[{"TarasovT", "[", 
                                  RowBox[{
                                  "a", ",", "b", ",", "r", ",", "es", ",", 
                                    InterpretationBox[\(p\^2\),
                                      PP], ",", "DP"}], "]"}], ",", 
                                RowBox[{"STLI", "[", 
                                  RowBox[{"d", ",", 
                                    InterpretationBox[\(p\^2\),
                                      PP], ",", 
                                    RowBox[{"{", 
                                      RowBox[{
                                        RowBox[{"{", 
                                        RowBox[{"n1", ",", 
                                        InterpretationBox[\(m\_1\),
                                        m1]}], "}"}], ",", 
                                        RowBox[{"{", 
                                        RowBox[{"n2", ",", 
                                        InterpretationBox[\(m\_2\),
                                        m2]}], "}"}], ",", 
                                        RowBox[{"{", 
                                        RowBox[{"n3", ",", 
                                        InterpretationBox[\(m\_3\),
                                        m3]}], "}"}], ",", 
                                        RowBox[{"{", 
                                        RowBox[{"n4", ",", 
                                        InterpretationBox[\(m\_4\),
                                        m4]}], "}"}], ",", 
                                        RowBox[{"{", 
                                        RowBox[{"n5", ",", 
                                        InterpretationBox[\(m\_5\),
                                        m5]}], "}"}]}], "}"}]}], "]"}]}], 
                              "]"}]}], "/.", 
                          RowBox[{"STLI", "\[Rule]", 
                            StyleBox["TFI",
                              FontColor->RGBColor[1, 0.6, 0]]}]}]}], 
                      "}"}]}]}]}], "]"}], ",", 
            \({a, 0, $TarasovTdeltaplimit}\), ",", 
            \({b, 0, $TarasovTdeltaplimit - a}\), ",", 
            \({r, 0, $TarasovTplimit}\), ",", 
            \({es, 0, $TarasovTplimit - r}\)}], "]"}], "]"}]}]], "Input"],

Cell["\<\
{0, 0, 0, 1}{0, 0, 0, 2}{0, 0, 0, 3}{0, 0, 0, 4}{0, 0, 0, 5}{0, 0, \
0, 6}{0, 0, 1, 0}{0, 0, 1, 1}{0, 0, 1, 2}{0, 0, 1, 3}{0, 0, 1, 4}{0, 0, 1, \
5}{0, 0, 2, 0}{0, 0, 2, 1}{0, 0, 2, 2}{0, 0, 2, 3}{0, 0, 2, 4}{0, 0, 3, 0}{0, \
0, 3, 1}{0, 0, 3, 2}{0, 0, 3, 3}{0, 0, 4, 0}{0, 0, 4, 1}{0, 0, 4, 2}{0, 0, 5, \
0}{0, 0, 5, 1}{0, 0, 6, 0}\
\>", "Print"]
}, Open  ]],

Cell[BoxData[
    \(Share[]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(matarti = 
      Timing[Do[
          If[0 < ia + ib + ir + es, 
            WriteString["\<stdout\>", {ia, ib, ir, es}]; 
            Set[f\_\(10\^4 + 1000\ ia + 100\ ib + 10\ ir + es\), 
              MakeFun[s\_\(10\^4 + 1000\ ia + 100\ ib + 10\ ir + es\), 
                  "\<(1\>"<>ToString[ia]<>ToString[ib]<>ToString[ir]<>
                    ToString[es]<>"\<)\>"] /. PQ :> PNQ]], {ia, 0, 
            \ $TarasovTdeltaplimit}, {ib, 0, $TarasovTdeltaplimit - ia}, {ir, 
            0, \ \ $TarasovTplimit}, {es, 0, $TarasovTplimit - ir}]]\)], 
  "Input"],

Cell["\<\
{0, 0, 0, 1}{0, 0, 0, 2}{0, 0, 0, 3}{0, 0, 0, 4}{0, 0, 0, 5}{0, 0, \
0, 6}{0, 0, 1, 0}{0, 0, 1, 1}{0, 0, 1, 2}{0, 0, 1, 3}{0, 0, 1, 4}{0, 0, 1, \
5}{0, 0, 2, 0}{0, 0, 2, 1}{0, 0, 2, 2}{0, 0, 2, 3}{0, 0, 2, 4}{0, 0, 3, 0}{0, \
0, 3, 1}{0, 0, 3, 2}{0, 0, 3, 3}{0, 0, 4, 0}{0, 0, 4, 1}{0, 0, 4, 2}{0, 0, 5, \
0}{0, 0, 5, 1}{0, 0, 6, 0}\
\>", "Print"]
}, Open  ]],

Cell[BoxData[
    \(Share[]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["symmetry", "Subsection",
  CellTags->"Tsymmetry"],

Cell["\<\
TVR[d_, pp_, {a_, b_, c_, z_}] := 
  TVR[d, pp, {c, b, a, z}] /;  !OrderedQ[{a, c}]\
\>", "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["special TVR reductions", "Subsection",
  CellTags->"specialTVR"],

Cell["\<\
TVR[2 + d_Symbol, pp_, {{1, m1_}, {1, m1_}, {2, 0}, {1, m1_}}] := 
   ((-6*m1^2 + 2*d*m1^2 + 3*pp)*TAI[d, 0, {{1, m1}}]^2)/
      (12*(-3 + d)*(-1 + d)*m1^2*pp) + 
     ((4*m1^2 - pp)*TAI[d, 0, {{1, m1}}]*TBI[d, pp, {{1, m1}, {1, m1}}])/
      (4*(-3 + d)*(-1 + d)*m1^2) - 
     ((6*m1^2 - 2*d*m1^2 - 7*pp + 2*d*pp)*
        TJI[d, pp, {{1, m1}, {1, m1}, {1, 0}}])/(6*(-2 + d)*(-1 + d)*pp) + 
     ((-4*m1^2 + pp)*(-6*m1^2 + 2*d*m1^2 + 3*pp)*
        TJI[d, pp, {{2, m1}, {1, m1}, {1, 0}}])/
      (6*(-3 + d)*(-2 + d)*(-1 + d)*pp) /; (m1=!=0)&&(pp=!=0);\
\>", "Input"],

Cell["\<\
TVR[4 + d_Symbol, pp_, {{1, m1_}, {1, m1_}, {2, 0}, {1, m1_}}] := 
   -((-96*d*m1^6 + 352*d^2*m1^6 - 384*d^3*m1^6 + 128*d^4*m1^6 - 
           192*m1^4*pp + 1008*d*m1^4*pp - 1632*d^2*m1^4*pp + 
           912*d^3*m1^4*pp - 144*d^4*m1^4*pp + 288*m1^2*pp^2 - 
           1536*d*m1^2*pp^2 + 2316*d^2*m1^2*pp^2 - 1086*d^3*m1^2*pp^2 + 
           66*d^4*m1^2*pp^2 + 72*d*pp^3 - 166*d^2*pp^3 + 81*d^3*pp^3 + 
           4*d^4*pp^3)*TAI[d, 0, {{1, m1}}]^2)/
      (36*(-1 + d)^2*d^2*(1 + d)*(-4 + 3*d)*(-2 + 3*d)*pp^2) + 
     ((4*m1^2 - pp)^2*TAI[d, 0, {{1, m1}}]*TBI[d, pp, {{1, m1}, {1, m1}}])/
      (4*(-1 + d)^2*d*(1 + d)) - 
     ((288*m1^8 - 1152*d*m1^8 + 1504*d^2*m1^8 - 768*d^3*m1^8 + 
          128*d^4*m1^8 - 816*m1^6*pp + 2480*d*m1^6*pp - 
          2224*d^2*m1^6*pp + 736*d^3*m1^6*pp - 80*d^4*m1^6*pp + 
          864*m1^4*pp^2 - 2196*d*m1^4*pp^2 + 1446*d^2*m1^4*pp^2 - 
          252*d^3*m1^4*pp^2 - 6*d^4*m1^4*pp^2 - 432*m1^2*pp^3 + 
          1086*d*m1^2*pp^3 - 895*d^2*m1^2*pp^3 + 315*d^3*m1^2*pp^3 - 
          44*d^4*m1^2*pp^3 + 4*d*pp^4 - 2*d^2*pp^4 - 4*d^3*pp^4 + 
          2*d^4*pp^4)*TJI[d, pp, {{1, m1}, {1, m1}, {1, 0}}])/
      (18*(-2 + d)*(-1 + d)^2*d*(1 + d)*(-4 + 3*d)*(-2 + 3*d)*pp^2) + 
     (m1^2*(4*m1^2 - pp)*(-96*m1^6 + 352*d*m1^6 - 384*d^2*m1^6 + 
          128*d^3*m1^6 + 240*m1^4*pp - 624*d*m1^4*pp + 384*d^2*m1^4*pp - 
          48*d^3*m1^4*pp - 192*m1^2*pp^2 + 420*d*m1^2*pp^2 - 
          162*d^2*m1^2*pp^2 - 18*d^3*m1^2*pp^2 + 72*pp^3 - 154*d*pp^3 + 
          81*d^2*pp^3 - 8*d^3*pp^3)*TJI[d, pp, {{2, m1}, {1, m1}, {1, 0}}])/
      (18*(-2 + d)*(-1 + d)^2*d*(1 + d)*(-4 + 3*d)*(-2 + 3*d)*pp^2) /; \
(m1=!=0)&&(pp=!=0);\
\>", "Input"],

Cell["\<\
TVR[4 + d_Symbol, pp_, {{1, m1_}, {1, m1_}, {3, 0}, {1, m1_}}] := 
   ((-72*d^2*m1^6 + 144*d^3*m1^6 - 88*d^4*m1^6 + 16*d^5*m1^6 - 
          240*d*m1^4*pp + 572*d^2*m1^4*pp - 398*d^3*m1^4*pp + 
          84*d^4*m1^4*pp - 2*d^5*m1^4*pp - 96*m1^2*pp^2 + 
          480*d*m1^2*pp^2 - 744*d^2*m1^2*pp^2 + 390*d^3*m1^2*pp^2 - 
          50*d^4*m1^2*pp^2 + 4*d^5*m1^2*pp^2 - 24*d*pp^3 + 54*d^2*pp^3 - 
          27*d^3*pp^3)*TAI[d, 0, {{1, m1}}]^2)/
      (24*(-3 + d)*(-1 + d)^2*d*(1 + d)*(-4 + 3*d)*(-2 + 3*d)*m1^2*pp^2) + 
     ((4*m1^2 - pp)^2*TAI[d, 0, {{1, m1}}]*TBI[d, pp, {{1, m1}, {1, m1}}])/
      (8*(-3 + d)*(-1 + d)^2*(1 + d)*m1^2) + 
     ((-72*d*m1^6 + 144*d^2*m1^6 - 88*d^3*m1^6 + 16*d^4*m1^6 - 
          96*m1^4*pp + 260*d*m1^4*pp - 166*d^2*m1^4*pp + 
          12*d^3*m1^4*pp + 6*d^4*m1^4*pp + 160*m1^2*pp^2 - 
          300*d*m1^2*pp^2 + 50*d^2*m1^2*pp^2 + 90*d^3*m1^2*pp^2 - 
          24*d^4*m1^2*pp^2 - 48*pp^3 + 114*d*pp^3 - 67*d^2*pp^3 + 
          4*d^3*pp^3 + 2*d^4*pp^3)*TJI[d, pp, {{1, m1}, {1, m1}, {1, 0}}])/
      (12*(-2 + d)*(-1 + d)^2*(1 + d)*(-4 + 3*d)*(-2 + 3*d)*pp^2) - 
     ((-4*m1^2 + pp)*(72*d*m1^6 - 144*d^2*m1^6 + 88*d^3*m1^6 - 
          16*d^4*m1^6 + 96*m1^4*pp - 236*d*m1^4*pp + 122*d^2*m1^4*pp + 
          12*d^3*m1^4*pp - 10*d^4*m1^4*pp - 96*m1^2*pp^2 + 
          180*d*m1^2*pp^2 - 30*d^2*m1^2*pp^2 - 46*d^3*m1^2*pp^2 + 
          8*d^4*m1^2*pp^2 + 24*pp^3 - 54*d*pp^3 + 27*d^2*pp^3)*
        TJI[d, pp, {{2, m1}, {1, m1}, {1, 0}}])/
(12*(-3 + d)*(-2 + d)*(-1 + d)^2*(1 + d)*(-4 + 3*d)*(-2 + 3*d)*pp^2) /; \
(m1=!=0)&&(pp=!=0);\
\>", "Input"],

Cell["\<\
TVR[6 + d_Symbol, pp_, 
    {{1, m1_}, {1, m1_}, {4, 0}, {1, m1_}}] := 
   ((2304*d*m1^10 - 4608*d^2*m1^10 - 3520*d^3*m1^10 + 6976*d^4*m1^10 + 
          1472*d^5*m1^10 - 2624*d^6*m1^10 - 256*d^7*m1^10 + 
          256*d^8*m1^10 + 4608*m1^8*pp - 9600*d*m1^8*pp - 
          7680*d^2*m1^8*pp + 9312*d^3*m1^8*pp + 11680*d^4*m1^8*pp - 
          6560*d^5*m1^8*pp - 2880*d^6*m1^8*pp + 1088*d^7*m1^8*pp + 
          32*d^8*m1^8*pp + 6912*m1^6*pp^2 - 35712*d*m1^6*pp^2 + 
          1056*d^2*m1^6*pp^2 + 85296*d^3*m1^6*pp^2 - 
          37032*d^4*m1^6*pp^2 - 20484*d^5*m1^6*pp^2 + 
          8700*d^6*m1^6*pp^2 - 12*d^7*m1^6*pp^2 - 84*d^8*m1^6*pp^2 - 
          20736*m1^4*pp^3 + 58752*d*m1^4*pp^3 + 40560*d^2*m1^4*pp^3 - 
          150536*d^3*m1^4*pp^3 + 18980*d^4*m1^4*pp^3 + 
          48946*d^5*m1^4*pp^3 - 6748*d^6*m1^4*pp^3 - 146*d^7*m1^4*pp^3 + 
          128*d^8*m1^4*pp^3 + 3456*m1^2*pp^4 - 13824*d*m1^2*pp^4 - 
          4176*d^2*m1^2*pp^4 + 35784*d^3*m1^2*pp^4 - 
          8332*d^4*m1^2*pp^4 - 10162*d^5*m1^2*pp^4 + 
          1122*d^6*m1^2*pp^4 - 80*d^7*m1^2*pp^4 - 8*d^8*m1^2*pp^4 + 
          864*d*pp^5 - 216*d^2*pp^5 - 2268*d^3*pp^5 + 486*d^4*pp^5 + 
          729*d^5*pp^5)*TAI[d, 0, {{1, m1}}]^2)/
      (432*(-3 + d)*(-1 + d)^2*d^2*(1 + d)*(2 + d)*(3 + d)*(-4 + 3*d)*
        (-2 + 3*d)*(2 + 3*d)*m1^2*pp^3) + 
     ((4*m1^2 - pp)^3*TAI[d, 0, {{1, m1}}]*TBI[d, pp, {{1, m1}, {1, m1}}])/
      (16*(-3 + d)*(-1 + d)^2*d*(1 + d)*(3 + d)*m1^2) - 
     ((-1152*m1^10 + 2880*d*m1^10 + 320*d^2*m1^10 - 3648*d^3*m1^10 + 
          1088*d^4*m1^10 + 768*d^5*m1^10 - 256*d^6*m1^10 - 192*m1^8*pp + 
          1312*d*m1^8*pp + 2080*d^2*m1^8*pp - 6880*d^3*m1^8*pp + 
          4032*d^4*m1^8*pp - 192*d^5*m1^8*pp - 160*d^6*m1^8*pp + 
          6336*m1^6*pp^2 - 6960*d*m1^6*pp^2 - 16960*d^2*m1^6*pp^2 + 
          17292*d^3*m1^6*pp^2 - 1828*d^4*m1^6*pp^2 - 828*d^5*m1^6*pp^2 + 
          68*d^6*m1^6*pp^2 - 9504*m1^4*pp^3 + 3624*d*m1^4*pp^3 + 
          26104*d^2*m1^4*pp^3 - 10938*d^3*m1^4*pp^3 - 
          4976*d^4*m1^4*pp^3 + 330*d^5*m1^4*pp^3 + 400*d^6*m1^4*pp^3 + 
          6336*m1^2*pp^4 - 3640*d*m1^2*pp^4 - 14584*d^2*m1^2*pp^4 + 
          9190*d^3*m1^2*pp^4 + 1374*d^4*m1^2*pp^4 - 600*d^5*m1^2*pp^4 - 
          56*d^6*m1^2*pp^4 - 864*pp^5 + 732*d*pp^5 + 1864*d^2*pp^5 - 
          1695*d^3*pp^5 + 148*d^4*pp^5 + 36*d^5*pp^5 + 4*d^6*pp^5)*
        TJI[d, pp, {{1, m1}, {1, m1}, {1, 0}}])/
      (216*(-2 + d)*(-1 + d)^2*d*(1 + d)*(3 + d)*(-4 + 3*d)*(-2 + 3*d)*
        (2 + 3*d)*pp^3) + ((-4*m1^2 + pp)*
        (1152*m1^10 - 2880*d*m1^10 - 320*d^2*m1^10 + 3648*d^3*m1^10 - 
          1088*d^4*m1^10 - 768*d^5*m1^10 + 256*d^6*m1^10 + 576*m1^8*pp - 
          2208*d*m1^8*pp - 2304*d^2*m1^8*pp + 8000*d^3*m1^8*pp - 
          4256*d^4*m1^8*pp - 32*d^5*m1^8*pp + 224*d^6*m1^8*pp - 
          6336*m1^6*pp^2 + 6288*d*m1^6*pp^2 + 16320*d^2*m1^6*pp^2 - 
          14868*d^3*m1^6*pp^2 + 684*d^4*m1^6*pp^2 + 804*d^5*m1^6*pp^2 - 
          12*d^6*m1^6*pp^2 + 6912*m1^4*pp^3 - 3192*d*m1^4*pp^3 - 
          18160*d^2*m1^4*pp^3 + 7950*d^3*m1^4*pp^3 + 3128*d^4*m1^4*pp^3 - 
          78*d^5*m1^4*pp^3 - 160*d^6*m1^4*pp^3 - 3456*m1^2*pp^4 + 
          1800*d*m1^2*pp^4 + 8136*d^2*m1^2*pp^4 - 4322*d^3*m1^2*pp^4 - 
          1042*d^4*m1^2*pp^4 + 128*d^5*m1^2*pp^4 + 16*d^6*m1^2*pp^4 + 
          432*pp^5 - 324*d*pp^5 - 972*d^2*pp^5 + 729*d^3*pp^5)*
        TJI[d, pp, {{2, m1}, {1, m1}, {1, 0}}])/
      (216*(-3 + d)*(-2 + d)*(-1 + d)^2*d*(1 + d)*(3 + d)*(-4 + 3*d)*
        (-2 + 3*d)*(2 + 3*d)*pp^3) /; (m1=!=0)&&(pp=!=0);\
\>", "Input"],

Cell["\<\
 TVR[6 + d_Symbol, pp_, {{2, m1_}, {1, m1_}, {3, 0}, {1, m1_}}] := 
   -((-1536*d*m1^10 + 2560*d^2*m1^10 + 3200*d^3*m1^10 - 3584*d^4*m1^10 - 
           2176*d^5*m1^10 + 1024*d^6*m1^10 + 512*d^7*m1^10 - 
           3072*m1^8*pp + 6528*d*m1^8*pp + 5376*d^2*m1^8*pp - 
           7520*d^3*m1^8*pp - 5568*d^4*m1^8*pp + 3232*d^5*m1^8*pp + 
           1344*d^6*m1^8*pp - 320*d^7*m1^8*pp - 2304*m1^6*pp^2 + 
           13824*d*m1^6*pp^2 + 192*d^2*m1^6*pp^2 - 31584*d^3*m1^6*pp^2 + 
           11232*d^4*m1^6*pp^2 + 8088*d^5*m1^6*pp^2 - 
           2304*d^6*m1^6*pp^2 - 24*d^7*m1^6*pp^2 + 6912*m1^4*pp^3 - 
           19584*d*m1^4*pp^3 - 12688*d^2*m1^4*pp^3 + 
           50584*d^3*m1^4*pp^3 - 7516*d^4*m1^4*pp^3 - 
           16982*d^5*m1^4*pp^3 + 2564*d^6*m1^4*pp^3 + 310*d^7*m1^4*pp^3 - 
           1152*m1^2*pp^4 + 4608*d*m1^2*pp^4 + 1392*d^2*m1^2*pp^4 - 
           11896*d^3*m1^2*pp^4 + 2820*d^4*m1^2*pp^4 + 
           3374*d^5*m1^2*pp^4 - 414*d^6*m1^2*pp^4 + 8*d^7*m1^2*pp^4 - 
           288*d*pp^5 + 72*d^2*pp^5 + 756*d^3*pp^5 - 162*d^4*pp^5 - 
           243*d^5*pp^5)*TAI[d, 0, {{1, m1}}]^2)/
      (144*(-1 + d)^2*d^2*(1 + d)*(2 + d)*(3 + d)*(-4 + 3*d)*(-2 + 3*d)*
        (2 + 3*d)*m1^2*pp^3) + 
     ((4*m1^2 - pp)^3*TAI[d, 0, {{1, m1}}]*TBI[d, pp, {{1, m1}, {1, m1}}])/
      (16*(-1 + d)^2*d*(1 + d)*(3 + d)*m1^2) - 
     ((2304*m1^10 - 5760*d*m1^10 - 640*d^2*m1^10 + 7296*d^3*m1^10 - 
          2176*d^4*m1^10 - 1536*d^5*m1^10 + 512*d^6*m1^10 - 
          1344*m1^8*pp + 1120*d*m1^8*pp - 1376*d^2*m1^8*pp + 
          5280*d^3*m1^8*pp - 4896*d^4*m1^8*pp + 1280*d^5*m1^8*pp - 
          64*d^6*m1^8*pp - 5184*m1^6*pp^2 + 7584*d*m1^6*pp^2 + 
          14800*d^2*m1^6*pp^2 - 21496*d^3*m1^6*pp^2 + 
          6120*d^4*m1^6*pp^2 + 280*d^5*m1^6*pp^2 - 184*d^6*m1^6*pp^2 + 
          9504*m1^4*pp^3 - 3720*d*m1^4*pp^3 - 27296*d^2*m1^4*pp^3 + 
          15378*d^3*m1^4*pp^3 + 4114*d^4*m1^4*pp^3 - 666*d^5*m1^4*pp^3 - 
          674*d^6*m1^4*pp^3 - 6336*m1^2*pp^4 + 5720*d*m1^2*pp^4 + 
          13712*d^2*m1^2*pp^4 - 13926*d^3*m1^2*pp^4 + 
          1284*d^4*m1^2*pp^4 + 946*d^5*m1^2*pp^4 - 80*d^6*m1^2*pp^4 + 
          864*pp^5 - 1020*d*pp^5 - 1636*d^2*pp^5 + 2311*d^3*pp^5 - 
          693*d^4*pp^5 + 20*d^5*pp^5 + 4*d^6*pp^5)*
        TJI[d, pp, {{1, m1}, {1, m1}, {1, 0}}])/
      (72*(-2 + d)*(-1 + d)^2*d*(1 + d)*(3 + d)*(-4 + 3*d)*(-2 + 3*d)*
        (2 + 3*d)*pp^3) + ((4*m1^2 - pp)*
        (-768*m1^10 + 1664*d*m1^10 + 768*d^2*m1^10 - 2176*d^3*m1^10 + 
          512*d^5*m1^10 + 192*m1^8*pp + 288*d*m1^8*pp + 704*d^2*m1^8*pp - 
          2272*d^3*m1^8*pp + 1024*d^4*m1^8*pp + 64*d^5*m1^8*pp + 
          1920*m1^6*pp^2 - 1824*d*m1^6*pp^2 - 5472*d^2*m1^6*pp^2 + 
          4584*d^3*m1^6*pp^2 - 168*d^5*m1^6*pp^2 - 2304*m1^4*pp^3 + 
          616*d*m1^4*pp^3 + 6192*d^2*m1^4*pp^3 - 1946*d^3*m1^4*pp^3 - 
          1128*d^4*m1^4*pp^3 - 230*d^5*m1^4*pp^3 + 1152*m1^2*pp^4 - 
          600*d*m1^2*pp^4 - 2744*d^2*m1^2*pp^4 + 1414*d^3*m1^2*pp^4 + 
          374*d^4*m1^2*pp^4 - 16*d^5*m1^2*pp^4 - 144*pp^5 + 108*d*pp^5 + 
          324*d^2*pp^5 - 243*d^3*pp^5)*
        TJI[d, pp, {{2, m1}, {1, m1}, {1, 0}}])/
      (72*(-2 + d)*(-1 + d)^2*d*(1 + d)*(3 + d)*(-4 + 3*d)*(-2 + 3*d)*
        (2 + 3*d)*pp^3) /; (m1=!=0)&&(pp=!=0);\
\>", "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["five -> four", "Subsection",
  CellTags->"T1.6.121.1"],

Cell[BoxData[
    \(TFI[d_, \ 
        PP_ /; Head[PP] =!= List, {\ {0, \ m1_}, \ {n2_, \ m2_}, \ \n
          \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ {n3_, 
            \ m3_}, \ {n4_, \ m4_}, \ \n
          \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ {n5_, 
            \ m5_}\ }]\  := \ \n
      TVI[d, PP, \ {{n5, m5}, \ {n2, m2}, \ {n3, m3}, \ {n4, m4}}]\)], "Input"],

Cell[BoxData[
    \(TFI[d_, \ \ 
        PP_ /; Head[PP] =!= List, {\ {\((n2_)\), \ m2_}, {0, \ m1_}, \n
          \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ {
            \((n4_)\), \ m4_}, \ {\((n3_)\), \ m3_}, \ \n
          \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ {
            \((n5_)\), \ m5_}\ }]\  := \ \n
      TVI[d, PP, \ {{n5, m5}, \ {n2, m2}, \ {n3, m3}, \ {n4, m4}}]\)], "Input"],

Cell[BoxData[
    \(TFI[d_, \ \ \ 
        PP_ /; Head[PP] =!= List, {\ {\((n3_)\), \ m3_}, 
          \ {\((n4_)\), \ m4_}, \n
          \t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ {0, \ 
            m1_}, \ {\((n2_)\), \ m2_}, \ \n
          \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ {
            \((n5_)\), \ m5_}\ }]\  := \ \n
      TVI[d, PP, \ {{n5, m5}, \ {n2, m2}, \ {n3, m3}, \ {n4, m4}}]\)], "Input"],

Cell[BoxData[
    \(TFI[d_, \ \ \ 
        PP_ /; Head[PP] =!= List, {\ {\((n4_)\), \ m4_}, {\((n3_)\), \ m3_}, 
          \n\t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ {
            \((n2_)\), \ m2_}, \ {0, \ m1_}, \ \n
          \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ {
            \((n5_)\), \ m5_}\ }]\  := \ \n
      TVI[d, PP, \ {{n5, m5}, \ {n2, m2}, \ {n3, m3}, \ {n4, m4}}]\)], "Input"],

Cell["\<\

TFI[d_,  PP_/;Head[PP]=!=List,{{(n1_), m1_}, 

    {(n2_), m2_}, 

    {(n3_), m3_}, 

    {(n4_), m4_}, {0, m5_}}] := 

  TBI[d, PP, {{n1, m1}, {n3, m3}}]*

   TBI[d, PP, {{n2, m2}, {n4, m4}}]
\
\>", "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["four -> three", "Subsection",
  CellTags->"T1.6.122.1"],

Cell[BoxData[
    \(TVI[d_, \ 
        PP_, {\ {0, \ m1_}, \ {n2_, \ m2_}, \ \n
          \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ {n3_, 
            \ m3_}, \ {n4_, \ m4_}\ }]\  := \ \n
      TAI[d, 0, {{n3, m3}}]\ TBI[d, PP, {{n2, m2}, {n4, m4}}]\)], "Input"],

Cell[BoxData[
    \(TVI[d_, \ 
        PP_, {\ {n1_, \ m1_}, \ {n2_, \ m2_}, \ \n
          \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ {0, \ 
            m3_}, \ {n4_, \ m4_}\ }]\  := \ \n
      TAI[d, 0, {{n1, m1}}]\ TBI[d, PP, {{n2, m2}, {n4, m4}}]\)], "Input"],

Cell["\<\
TVI[d_, PP_, {{n1_, m1_}, {0, m2_}, {n3_, m3_}, {n4_, m4_}}] := 
  TJI[d, 0, {{n1, m1}, {n3, m3}, {n4, m4}}]\
\>", "Input"],

Cell[BoxData[
    \(TVI[d_, \ 
        PP_, {\ {n1_, \ m1_}, \ {n2_, \ m2_}, \ \n
          \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ {n3_, 
            \ m3_}, \ {0, \ m4_}\ }]\  := \ \n
      TJI[d, PP, \ {{n2, m2}, {n1, m1}, {n3, m3}\ }]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["three -> two", "Subsection",
  CellTags->"T1.6.123.1"],

Cell[BoxData[
    \(TJI[d_, \ 
        PP_, {\ {0, \ m1_}, \ {\((n2_)\), \ m2_}, \ {\((n3_)\), \ m3_}\ }]
        \  := \ \nTAI[d, 0, {{n2, m2}}]\ \ TAI[d, 0, {{n3, m3}}]\)], "Input"],

Cell[BoxData[
    \(TJI[d_, \ 
        PP_, {\ {\((n1_)\), \ m1_}, \ {0, \ m2_}, \ {\((n3_)\), \ m3_}\ }]
        \  := \ \nTAI[d, 0, {{n1, m1}}]\ \ TAI[d, 0, {{n3, m3}}]\)], "Input"],

Cell[BoxData[
    \(TJI[d_, \ 
        PP_, {\ {\((n1_)\), \ m1_}, \ {\((n2_)\), \ m2_}, \ {0, \ m3_}\ }]
        \  := \ \nTAI[d, 0, {{n1, m1}}]\ \ TAI[d, 0, {{n2, m2}}]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["two -> one", "Subsection",
  CellTags->"T1.6.124.1"],

Cell[BoxData[
    \(\(\(TBI[d_, \ PP_, {\ {0, \ m1_}, \ {\((n2_)\), \ m2_}\ }]\ \) := \ \n
      TAI[d, 0, {{n2, m2}}]\ \)\)], "Input"],

Cell[BoxData[
    \(\(\(TBI[d_, \ PP_, {\ {\((n1_)\), \ m1_}, \ {0, \ m2_}\ }]\ \) := \ \n
      TAI[d, 0, {{n1, m1}}]\ \)\)], "Input"],

Cell[BoxData[
    \(\(TAI[d_, {{n_Integer, m_}}] := TAI[d, 0, {{n, m}}]; \)\)], "Input"],

Cell[BoxData[
    \(\(TAI[d_, {n_Integer, m_}] := TAI[d, 0, {{n, m}}]; \)\)], "Input"],

Cell[BoxData[
    \(\(TAI[d_, 0, {n_Integer, m_}] := TAI[d, 0, {{n, m}}]; \)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["TarcerRecurse and TFIRecurse", "Section",
  CellDingbat->"\[FilledSmallCircle]",
  CellTags->"T1.7.1"],

Cell[CellGroupData[{

Cell["TFIRecurse & TFISimplify", "Subsection",
  CellTags->"T1.7.1.1"],

Cell["This cancels scalar products.", "Text"],

Cell[BoxData[
    \(\($TLRComment = False; \)\)], "Input"],

Cell[BoxData[
    \(\(TLRComment[s_String] := 
      WriteString["\<stdout\>", s, "\< \>"] /; $TLRComment === True; \)\)], 
  "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(applytlrules[y_, fun_]\), " ", ":=", 
        RowBox[{"fun", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"y", " ", "/.", 
                RowBox[{
                  StyleBox["TFI",
                    FontColor->RGBColor[1, 0.6, 0]], "->", "TLR"}]}], "/.", 
              "TLRules"}], "/.", 
            RowBox[{"TLR", "->", 
              StyleBox["TFI",
                FontColor->RGBColor[1, 0.6, 0]]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
    \(\(TFIRecurse[z_, f_ : Identity]\  := \ 
      FixedPoint[\ applytlrules[#, f]&, z, 1000]; \)\)], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(applytlrules2[y_, fun_]\), " ", ":=", 
        RowBox[{"fun", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"y", " ", "/.", 
                RowBox[{
                  StyleBox["TFI",
                    FontColor->RGBColor[1, 0.6, 0]], "->", "TLR"}]}], "/.", 
              "TLRules2"}], "/.", 
            RowBox[{"TLR", "->", 
              StyleBox["TFI",
                FontColor->RGBColor[1, 0.6, 0]]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
    \(\(TFISimplify[z_, f_ : Identity]\  := \ 
      FixedPoint[\ applytlrules2[#, f]&, z, 1000]; \)\)], "Input"],

Cell["\<\
TFIRecurse[z_]:=z/.TFI\[Rule]TLR//.TLRules/.TLR\[Rule]TFI;\
\>", \
"Text"],

Cell[BoxData[
    \(\(ExpandMaybe = Identity; \)\)], "Input"],

Cell[BoxData[
    \(\(fN[li_List] := \(fN[li] = li /. {0, _} \[RuleDelayed] {0, 0}\); 
    \)\)], "Input"],

Cell["\<\
RM: new Feb. 1999  
need TLR[ ...]:= rule;
(copy from TFI definition) (in order to get e.g. 
TarcerRecurse[TFI[D,pp,{1,1,1,1,1},{{1,0},{1,M},{0,0},{1,m},{1,M}}]]
to work)\
\>", "Text",
  CellTags->"TLRdef"],

Cell[BoxData[
    \(\(TLR[depp__, {a___, b_Integer, c___}] := TLR[depp, {a, {b, 0}, c}]; 
    \)\)], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(tlrule[1]\), "=", 
        RowBox[{\(TLR[d_, pp_, {0, 0, 0, 0, 0}, pr_]\), "\[RuleDelayed]", 
          RowBox[{"(", 
            RowBox[{\(NoTLRComment["\<tlrule1\>"]\), ";", 
              RowBox[{
                StyleBox["TFI",
                  FontColor->RGBColor[1, 0.6, 0]], "[", \(d, pp, 
                pr /. {0, _} \[RuleDelayed] {0, 0}\), "]"}]}], ")"}]}]}], 
      ";"}]], "Input"],

Cell["(* q1^2 *)", "Text"],

Cell[BoxData[
    \(\(tlrule[2] = 
      TLR[dpp__, {v_?PQ, w_, x_, y_, z_}, {{n1_?PQ, m1_}, nm2_, nm3_, nm4_, 
            nm5_}] \[RuleDelayed] 
        \((TLRComment["\<tlrule2\>"]; 
          ExpandMaybe[
            If[v \[LessEqual] n1, 
              If[m1 === 0, 
                TLR[dpp, {0, w, x, y, z}, {{n1 - v, 0}, nm2, nm3, nm4, nm5}], 
                \[Sum]\+\(i = 0\)\%v Binomial[v, i]\ 
                    TLR[dpp, 
                      \ {0, w, x, y, z}, {fN[{n1 - i, m1}], nm2, nm3, nm4, 
                        nm5}]\ \((m1\^2)\)\^\(v - i\)], 
              If[m1 === 0, 
                TLR[dpp, {v - n1, w, x, y, z}, {{0, 0}, nm2, nm3, nm4, nm5}], 
                \[Sum]\+\(i = 0\)\%n1 Binomial[n1, i]\ 
                    TLR[dpp, {v - n1, w, x, y, z}, {fN[{n1 - i, m1}], nm2, 
                        nm3, nm4, nm5}]\ \((m1\^2)\)\^\(n1 - i\)]]])\); 
    \)\)], "Input"],

Cell[TextData[Cell[BoxData[
    \(TraditionalForm\`q\_2\%2\)]]], "Text"],

Cell[BoxData[
    \(\(tlrule[3] = 
      TLR[dpp__, {v_, w_?PQ, x_, y_, z_}, {nm1_, {n2_?PQ, m2_}, nm3_, nm4_, 
            nm5_}] \[RuleDelayed] 
        \((TLRComment["\<tlrule3\>"]; 
          ExpandMaybe[
            If[w \[LessEqual] n2, 
              If[m2 === 0, 
                TLR[dpp, {v, 0, x, y, z}, {nm1, {n2 - w, 0}, nm3, nm4, nm5}], 
                \[Sum]\+\(i = 0\)\%w Binomial[w, i]\ 
                    TLR[dpp, {v, 0, x, y, z}, {nm1, fN[{n2 - i, m2}], nm3, 
                        nm4, nm5}]\ \((m2\^2)\)\^\(w - i\)], 
              If[m2 === 0, 
                TLR[dpp, {v, w - n2, x, y, z}, {nm1, {0, 0}, nm3, nm4, nm5}], 
                \[Sum]\+\(i = 0\)\%n2 Binomial[n2, i]\ 
                    TLR[dpp, {v, w - n2, x, y, z}, {nm1, fN[{n2 - i, m2}], 
                        nm3, nm4, nm5}]\ \((m2\^2)\)\^\(n2 - i\)]]])\); 
    \)\)], "Input"],

Cell[TextData[Cell[BoxData[
    \(TraditionalForm\`p\[CenterDot]q\_1\)]]], "Text"],

Cell[BoxData[
    \(\(tlrule[4] = 
      TLR[dpp__, {v_, w_, x_?PQ, y_, z_}, {{n1_?PQ, m1_}, 
            nm2_, {n3_?PQ, m3_}, nm4_, nm5_}] \[RuleDelayed] 
        \((TLRComment["\<tlrule4\>"]; 
          ExpandMaybe[
            1\/2\ TLR[
                  dpp, {v + 1, w, x - 1, y, z}, {{n1, m1}, nm2, {n3, m3}, 
                    nm4, nm5}] - 
              1\/2\ TLR[
                  dpp, {v, w, x - 1, y, z}, {{n1, m1}, nm2, fN[{n3 - 1, m3}], 
                    nm4, nm5}] + 
              1\/2\ \(({dpp}\[LeftDoubleBracket]2\[RightDoubleBracket] - 
                    m3\^2)\)\ 
                TLR[dpp, {v, w, x - 1, y, z}, {{n1, m1}, nm2, {n3, m3}, nm4, 
                    nm5}]])\); \)\)], "Input"],

Cell[TextData[Cell[BoxData[
    \(TraditionalForm\`p\[CenterDot]q\_2\)]]], "Text"],

Cell[BoxData[
    \(\(tlrule[5] = 
      TLR[dpp__, {v_, w_, x_, y_?PQ, z_}, {{n1_, m1_}, {n2_?PQ, m2_}, 
            nm3_, {n4_?PQ, m4_}, nm5_}] \[RuleDelayed] 
        \((TLRComment["\<tlrule5\>"]; 
          ExpandMaybe[
            1\/2\ TLR[
                  dpp, {v, w + 1, x, y - 1, z}, {{n1, m1}, {n2, m2}, 
                    nm3, {n4, m4}, nm5}] - 
              1\/2\ TLR[
                  dpp, {v, w, x, y - 1, z}, {{n1, m1}, {n2, m2}, nm3, 
                    fN[{n4 - 1, m4}], nm5}] + 
              1\/2\ \(({dpp}\[LeftDoubleBracket]2\[RightDoubleBracket] - 
                    m4\^2)\)\ 
                TLR[dpp, {v, w, x, y - 1, z}, {{n1, m1}, {n2, m2}, 
                    nm3, {n4, m4}, nm5}]])\); \)\)], "Input"],

Cell[TextData[Cell[BoxData[
    \(TraditionalForm\`q\_1\[CenterDot]q\_2\)]]], "Text"],

Cell[BoxData[
    \(\(tlrule[6] = 
      TLR[dpp__, {v_, w_, x_, y_, z_?PQ}, {{n1_, m1_}, {n2_, m2_}, nm3_, 
            nm4_, {n5_?PQ, m5_}}] \[RuleDelayed] 
        \((TLRComment["\<tlrule6\>"]; 
          ExpandMaybe[
            1\/2\ TLR[
                  dpp, {v + 1, w, x, y, z - 1}, {{n1, m1}, {n2, m2}, nm3, 
                    nm4, {n5, m5}}] - 
              1\/2\ TLR[
                  dpp, {v, w, x, y, z - 1}, {{n1, m1}, {n2, m2}, nm3, nm4, 
                    fN[{n5 - 1, m5}]}] + 
              1\/2\ TLR[
                  dpp, {v, w + 1, x, y, z - 1}, {{n1, m1}, {n2, m2}, nm3, 
                    nm4, {n5, m5}}] - 
              1\/2\ m5\^2\ 
                TLR[dpp, {v, w, x, y, z - 1}, {{n1, m1}, {n2, m2}, nm3, 
                    nm4, {n5, m5}}]])\); \)\)], "Input"],

Cell["(* q1.q1  --> q1.p *)", "Text"],

Cell[BoxData[
    \(\(tlrule[7] = 
      TLR[d_, pp_, 
          dp___, {v_?PQ, w_, x_, y_, z_}, {{0, _}, nm2_, {n3_?PQ, m3_}, nm4_, 
            nm5_}] \[RuleDelayed] 
        \((TLRComment["\<tlrule7\>"]; 
          ExpandMaybe[
            TLR[d, pp, 
                dp, {v - 1, w, x, y, z}, {{0, 0}, nm2, fN[{n3 - 1, m3}], nm4, 
                  nm5}] + 
              2\ TLR[d, pp, 
                  dp, {v - 1, w, x + 1, y, z}, {{0, 0}, nm2, {n3, m3}, nm4, 
                    nm5}] - 
              \((pp - m3\^2)\)\ 
                TLR[d, pp, 
                  dp, {v - 1, w, x, y, z}, {{0, 0}, nm2, {n3, m3}, nm4, 
                    nm5}]])\); \)\)], "Input"],

Cell["(* q2.q2  --> q2.p *)", "Text"],

Cell[BoxData[
    \(\(tlrule[8] = 
      TLR[d_, pp_, 
          dp___, {v_, w_?PQ, x_, y_, z_}, {nm1_, {0, _}, nm3_, {n4_?PQ, m4_}, 
            nm5_}] \[RuleDelayed] 
        \((TLRComment["\<tlrule8\>"]; 
          ExpandMaybe[
            TLR[d, pp, 
                dp, {v, w - 1, x, y, z}, {nm1, {0, 0}, nm3, fN[{n4 - 1, m4}], 
                  nm5}] + 
              2\ TLR[d, pp, 
                  dp, {v, w - 1, x, y + 1, z}, {nm1, {0, 0}, nm3, {n4, m4}, 
                    nm5}] - 
              \((pp - m4\^2)\)\ 
                TLR[d, pp, 
                  dp, {v, w - 1, x, y, z}, {nm1, {0, 0}, nm3, {n4, m4}, 
                    nm5}]])\); \)\)], "Input"],

Cell["(* eq. (3) - (5) *)", "Text"],

Cell[BoxData[
    \(\(tlrule[9] = 
      TLR[d_, pp_, {v_, w_, x_, y_, z_?PQ}, {nm1_, nm2_, nm3_, nm4_, {0, _}}] 
        \[RuleDelayed] 
        \((TLRComment["\<tlrule9\>"]; 
          FunctionExpand[
            ExpandMaybe[
              \[Sum]\+\(k = 0\)\%\(Floor[z\/2]\)\(\[Sum]\+\(i = 0\)\%k
                    \(\[Sum]\+\(j = 0\)\%k
                      \((Gamma[k + 1\/2]\ Gamma[d\/2 - 1\/2]\ 
                          Binomial[z, 2\ k]\ Binomial[k, i]\ Binomial[k, j]\ 
                          \((\(-1\))\)\^\(i + j\)\ pp\^\(i + j - z\)\ 
                          TLR[d, 
                            pp, {v + i, w + j, x + z - 2\ i, y + z - 2\ j, 
                              0}, {nm1, nm2, nm3, nm4, {0, 0}}])\)/
                      \((Gamma[1\/2]\ Gamma[k + d\/2 - 1\/2])\)\)\)]])\); 
    \)\)], "Input"],

Cell[" OLD trule[10]:", "Text"],

Cell["\<\
tlrule[10] = 
(* (9) and (10) *)
TLR[d_, pp_, {v_,w_,x_?PQ,y_?PQ,z_}, {{n1_?PQ,m1_},{n2_?PQ,m2_}, 
                                {0,_},{0,_},nm5_}] :>  
( TLRComment[\"tlrule10\"]; 0 /; OddQ[x+y] );\
\>", "Text"],

Cell["New tlrule[10]:", "Text"],

Cell["\<\
tlrule[10] = 
   TLR[d_, pp_, {v_, w_, (x_)?PNQ, (y_)?PNQ, z_}, 
     {{(n1_)?PQ, m1_}, {(n2_)?PQ, m2_}, {0, _}, {0, _}, nm5_}] :> \
(TLRComment[\"tlrule10\"]; 0 /; OddQ[x + y]); \
\>", "Input"],

Cell["\<\
Bug in  tlrule[11] and tlrule[12] fixed on Feb. 26th 1998 by \
R.Scharf 
with thanks to York Schroeder for locating the bug.\
\>", "Text"],

Cell["\<\

tlrule[11] = 
TLR[d_, pp_, {v_,w_,x_?PQ,y_?PQ,z_}, {nm1_,nm2_,{0,_},{0,_},{(n5_)?PQ, m5_}}] \
:> 
(
TLRComment[\"tlrule11\"];
 ExpandMaybe[
 pp/(d+x+y-2)( (x-1) TLR[d, pp, {v+1,w,x-2,y,z}, 
                         {{n1,m1},{n2,m2},{0,0},{0,0},{n5,m5}}] +
                   y TLR[d, pp, {v,w,x-1,y-1,z+1}, 
                         {nm1,nm2,{0,0},{0,0},{n5,m5}}]
             )
           ] /; EvenQ[x+y] && x > 1
);\
\>", "Input"],

Cell["\<\

tlrule[12] = 
TLR[d_, pp_, {v_,w_,x_?PQ,y_?PQ,z_}, {nm1_,nm2_,{0,_},{0,_},{(n5_)?PQ, m5_}}] \
:> 
(
TLRComment[\"tlrule12\"];
 ExpandMaybe[
 pp/(d+x+y-2)( (y-1) TLR[d, pp, {v,w+1,x,y-2,z}, 
                         {{n1,m1},{n2,m2},{0,0},{0,0},{n5,m5}}] +
                   x TLR[d, pp, {v,w,x-1,y-1,z+1}, 
                         {nm1,nm2,{0,0},{0,0},{n5,m5}}]
             )
           ] /; EvenQ[x+y] && y > 1
);\
\>", "Input"],

Cell["(*symm.   1 <-> 3  &&  2 <-> 4 *)", "Text"],

Cell[BoxData[
    \(\(tlrule[13] = 
      TLR[d_, pp_, {0, 0, 0, y_?PQ, 0}, {nm1_, {0, 0}, nm3_, {n4_?PQ, m4_}, 
            nm5_}] \[RuleDelayed] 
        \((TLRComment["\<tlrule13\>"]; 
          \[Sum]\+\(i = 0\)\%y Binomial[y, i]\ pp\^\(y - i\)\ 
              \((\(-1\))\)\^i\ 
              TLR[d, pp, {0, 0, 0, i, 0}, {nm3, {n4, m4}, nm1, {0, 0}, nm5}])
          \); \)\)], "Input"],

Cell["(*symm.   1 <-> 3  &&  2 <-> 4 *)", "Text"],

Cell[BoxData[
    \(\(tlrule[14] = 
      TLR[d_, pp_, {0, 0, x_?PQ, 0, 0}, {{0, 0}, nm2_, {n3_?PQ, m3_}, nm4_, 
            nm5_}] \[RuleDelayed] 
        \((TLRComment["\<tlrule14\>"]; 
          \[Sum]\+\(i = 0\)\%x Binomial[x, i]\ pp\^\(x - i\)\ 
              \((\(-1\))\)\^i\ 
              TLR[d, pp, {0, 0, i, 0, 0}, {{n3, m3}, nm4, {0, 0}, nm2, nm5}])
          \); \)\)], "Input"],

Cell["(* 0's *)", "Text"],

Cell[BoxData[
    \(\(tlrule[15] = 
      HoldPattern[
          TLR[d_, pp_, {v_, w_, x_, y_, 
              z_}, {{n1_, 0}, {n2_, 0}, {0, _}, {0, _}, {0, _}}]] 
        \[RuleDelayed] \((TLRComment["\<tlrule15\>"]; 0)\); \)\)], "Input"],

Cell[BoxData[
    \(\(tlrule[16] = 
      TLR[d_, pp_, {0, 0, x_?PQ, y_?PQ, 
            0}, {{n1_, _}, {n2_, _}, {0, _}, {0, _}, {0, _}}] \[RuleDelayed] 
        \((TLRComment["\<tlrule16\>"]; 0 /; OddQ[x] || OddQ[y])\); \)\)], 
  "Input"],

Cell["bug in tlrule[17] fixedApril 23rd 1999", "Text",
  CellTags->"tlrule17"],

Cell[BoxData[
    \(\(tlrule[17] = 
      TLR[d_, pp_, {a_?PQ, b_, 0, de_, 
            0}, {{0, _}, {n2_, m2_}, {0, _}, {n4_, m4_}, {n5_, m5_}}] 
        \[RuleDelayed] 
        \((TLRComment["\<tlrule17\>"]; 
          \[Sum]\+\(i = 0\)\%a
              \(\[Sum]\+\(j = 0\)\%i Binomial[a, i]\ Binomial[i, j]\ 
                \((1 + \((\(-1\))\)\^j)\)\ 2\^\(j - 1\)\ 
                TLR[d, pp, {a - i, b + i - j, 0, de, 
                    j}, {{n5, m5}, {n2, m2}, {0, 0}, {n4, m4}, {0, 0}}]\))\); 
    \)\)], "Input"],

Cell[BoxData[
    \(\(tlrule[18] = 
      TLR[d_, pp_, 
          dp_, {a_?PNQ, b_?PNQ}, {1, 0, 0, 0, 
            0}, {{0, 0}, {n2_, m2_}, {n3_, m3_}, {n4_, m4_}, {n5_, m5_}}] 
        \[RuleDelayed] 
        \((TLRComment["\<tlrule18\>"]; 
          \[Sum]\+\(i = 0\)\%a
              \(\[Sum]\+\(j = 0\)\%b Binomial[a, i]\ Binomial[b, j]\ 
                \((\((pp + m3\^2)\)\ 
                      TLR[d, pp, 
                        dp, {i, 
                          j}, {{n3, m3}, {n4, m4}, {0, 0}, {n2, m2}, {n5, 
                            m5}}] + 
                    TLR[d, pp, 
                      dp, {i, 
                        j}, {{n3 - 1, m3}, {n4, m4}, {0, 0}, {n2, m2}, {n5, 
                          m5}}] - 
                    2\ TLR[d, pp, 
                        dp, {i, j}, {0, 0, 1, 0, 
                          0}, {{n3, m3}, {n4, m4}, {0, 0}, {n2, m2}, {n5, 
                            m5}}])\)\ \((\(-1\))\)\^\(i + j\)\ 
                dp\^\(a + b - i - j\)\))\); \)\)], "Input"],

Cell[BoxData[
    \(\(tlrule[19] = 
      TLR[d_, pp_, 
          dp_, {a_?PNQ, b_?PNQ}, {0, 1, 0, 0, 
            0}, {{n1_, m1_}, {0, 0}, {n3_, m3_}, {n4_, m4_}, {n5_, m5_}}] 
        \[RuleDelayed] 
        \((TLRComment["\<tlrule18\>"]; 
          \[Sum]\+\(i = 0\)\%a
              \(\[Sum]\+\(j = 0\)\%b Binomial[a, i]\ Binomial[b, j]\ 
                \((\((pp + m4\^2)\)\ 
                      TLR[d, pp, 
                        dp, {i, 
                          j}, {{n3, m3}, {n4, m4}, {n1, m1}, {0, 0}, {n5, 
                            m5}}] + 
                    TLR[d, pp, 
                      dp, {i, 
                        j}, {{n3, m3}, {n4 - 1, m4}, {n1, m1}, {0, 0}, {n5, 
                          m5}}] - 
                    2\ TLR[d, pp, 
                        dp, {i, j}, {0, 0, 0, 1, 
                          0}, {{n3, m3}, {n4, m4}, {n1, m1}, {0, 0}, {n5, 
                            m5}}])\)\ \((\(-1\))\)\^\(i + j\)\ 
                dp\^\(a + b - i - j\)\))\); \)\)], "Input"],

Cell["(* q1.q1  --> q1.p *)", "Text"],

Cell[BoxData[
    \(\(tlrule[20] = 
      TLR[d_, pp_, {v_?PQ, w_, x_, y_, 0}, {{0, _}, nm2_, {0, _}, 
            nm4_, {n5_?PQ, m5_}}] \[RuleDelayed] 
        \((TLRComment["\<tlrule20\>"]; 
          \[Sum]\+\(j = 0\)\%x
              \(\[Sum]\+\(i = 0\)\%v
                \(\[Sum]\+\(r = 0\)\%i Binomial[i, r]\ Binomial[v, i]\ 
                  Binomial[x, j]\ 2\^r\ 
                  TLR[d, pp, {v - i, \(-r\) + i + w, j, x - j + y, 
                      r}, {{n5, m5}, nm2, 0, nm4, 0}]\)\))\); \)\)], "Input"],

Cell[BoxData[
    \(\(tlrule[21] = 
      TLR[d_, pp_, {v_, w_?PQ, x_, y_, 0}, {nm1_, {0, _}, nm3_, {0, _}, 
            nm5_}] \[RuleDelayed] 
        \((TLRComment["\<tlrule21\>"]; 
          TLR[d, pp, {w, v, y, x, 0}, {{0, 0}, nm1, {0, 0}, nm3, nm5}])\); 
    \)\)], "Input"],

Cell[BoxData[
    \(\(tlrule[22] = 
      TLR[d_, pp_, 
          dp_, {0, b_?PNQ}, {0, w_?PNQ, 0, y_?PNQ, 1}, {nm1_, nm2_, nm3_, 
            nm4_, {0, 0}}] \[RuleDelayed] 
        \((TLRComment["\<tlrule22\>"]; 
          \(1\/pp\) 
            \((TLR[d, pp, 
                dp, {0, b}, {0, w, 1, y + 1, 0}, {nm1, nm2, nm3, 
                  nm4, {0, 0}}])\))\); \)\)], "Input"],

Cell[TextData[{
  "Using  FeynCalc : \n\n",
  Cell[BoxData[
      \(TIDL[{{k1, \[Mu]}, {k1, \[Nu]}}, {p}]\)], "Input"],
  "\n",
  Cell[BoxData[
      FormBox[
        RowBox[{
          FractionBox[
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      FormBox[
                        RowBox[{
                          FormBox["k1",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}],
                        "TraditionalForm"], ")"}], "2"], "-", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["k1",
                        "TraditionalForm"], "2"], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"]}]}], ")"}]}], 
            RowBox[{\((1 - D)\), " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"]}]], "-", 
          FractionBox[
            RowBox[{
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"D", " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["k1",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["p",
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"]}], "-", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["k1",
                        "TraditionalForm"], "2"], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"]}]}], ")"}]}], 
            RowBox[{\((1 - D)\), " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "4"]}]]}], TraditionalForm]], 
    "Output"],
  "\n",
  Cell[BoxData[
      \(Collect2[Calc[\ FVD[OPEDelta, \[Mu]] FVD[k2, \[Nu]] %], {k1, k2}]\)], 
    "Input"],
  "\n",
  Cell[BoxData[
      FormBox[
        RowBox[{
          FractionBox[
            RowBox[{
              RowBox[{
                FormBox["k2",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"]}], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  FormBox[
                    RowBox[{
                      FormBox["k1",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}],
                    "TraditionalForm"], ")"}], "2"]}], 
            RowBox[{\((1 - D)\), " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"]}]], "-", 
          FractionBox[
            RowBox[{"D", " ", 
              RowBox[{
                FormBox["k2",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  FormBox[
                    RowBox[{
                      FormBox["k1",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}],
                    "TraditionalForm"], ")"}], "2"]}], 
            RowBox[{\((1 - D)\), " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "4"]}]], "-", 
          FractionBox[
            RowBox[{
              SuperscriptBox[
                FormBox["k1",
                  "TraditionalForm"], "2"], " ", 
              RowBox[{
                FormBox["k2",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"]}]}], \(1 - D\)], "+", 
          FractionBox[
            RowBox[{
              SuperscriptBox[
                FormBox["k1",
                  "TraditionalForm"], "2"], " ", 
              RowBox[{
                FormBox["k2",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}]}], 
            RowBox[{\((1 - D)\), " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"]}]]}], TraditionalForm]], 
    "Output"],
  "\n"
}], "Text"],

Cell[BoxData[
    \(\(tlrule[23] = 
      TLR[d_, pp_, 
          dp_, {1, b_?PNQ}, {0, w_?PNQ, 0, y_?PNQ, 1}, {nm1_, nm2_, nm3_, 
            nm4_, {0, 0}}] \[RuleDelayed] 
        \((TLRComment["\<tlrule23\>"]; 
          \(1\/\(\((1 - d)\) pp\)\) 
              \((TLR[d, pp, 
                  dp, {0, b + 1}, {0, w, 2, y, 0}, {nm1, nm2, nm3, 
                    nm4, {0, 0}}])\) - 
            \(\(d\ \ dp\)\/\(\((1 - d)\) pp\^2\)\) 
              \((TLR[d, pp, 
                  dp, {0, b}, {0, w, 2, y + 1, 0}, {nm1, nm2, nm3, 
                    nm4, {0, 0}}])\) - 
            \(1\/\(1 - d\)\) 
              TLR[d, pp, 
                dp, {0, b + 1}, {1, w, 0, y, 0}, {nm1, nm2, nm3, 
                  nm4, {0, 0}}] + 
            \(dp\/\(\((1 - d)\) pp\)\) 
              TLR[d, pp, 
                dp, {0, b}, {1, w, 0, y + 1, 0}, {nm1, nm2, nm3, 
                  nm4, {0, 0}}])\); \)\)], "Input"],

Cell[TextData[{
  Cell[BoxData[
      \(TIDL[{{k1, \[Mu]}, {k1, \[Nu]}, {k1, \[Rho]}}, {p}]\)], "Input"],
  "\n",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"-", 
            FractionBox[
              RowBox[{
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["k1",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{\((D + 2)\), " ", 
                      SuperscriptBox[
                        RowBox[{"(", 
                          FormBox[
                            RowBox[{
                              FormBox["k1",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}],
                            "TraditionalForm"], ")"}], "2"]}], "-", 
                    RowBox[{"3", " ", 
                      SuperscriptBox[
                        FormBox["k1",
                          "TraditionalForm"], "2"], " ", 
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], "2"]}]}], ")"}]}], 
              RowBox[{\((1 - D)\), " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "6"]}]]}], "+", 
          FractionBox[
            RowBox[{
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{
                FormBox["k1",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], " ", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      FormBox[
                        RowBox[{
                          FormBox["k1",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}],
                        "TraditionalForm"], ")"}], "2"], "-", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["k1",
                        "TraditionalForm"], "2"], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"]}]}], ")"}]}], 
            RowBox[{\((1 - D)\), " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "4"]}]], "+", 
          FractionBox[
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{
                FormBox["k1",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], " ", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      FormBox[
                        RowBox[{
                          FormBox["k1",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}],
                        "TraditionalForm"], ")"}], "2"], "-", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["k1",
                        "TraditionalForm"], "2"], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"]}]}], ")"}]}], 
            RowBox[{\((1 - D)\), " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "4"]}]], "+", 
          FractionBox[
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{
                FormBox["k1",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], " ", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      FormBox[
                        RowBox[{
                          FormBox["k1",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}],
                        "TraditionalForm"], ")"}], "2"], "-", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["k1",
                        "TraditionalForm"], "2"], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"]}]}], ")"}]}], 
            RowBox[{\((1 - D)\), " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "4"]}]]}], TraditionalForm]], 
    "Output"],
  "\n",
  Cell[BoxData[
      \(Collect2[
        Calc[\ FVD[OPEDelta, \[Mu]] FVD[OPEDelta, \[Nu]] 
            FVD[k2, \[Rho]] %], {k1, k2}]\)], "Input"],
  "\n",
  Cell[BoxData[
      FormBox[
        RowBox[{
          FractionBox[
            RowBox[{"2", " ", 
              RowBox[{
                FormBox["k2",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  FormBox[
                    RowBox[{
                      FormBox["k1",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}],
                    "TraditionalForm"], ")"}], "3"]}], 
            RowBox[{\((1 - D)\), " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "4"]}]], "-", 
          FractionBox[
            RowBox[{\((D + 2)\), " ", 
              RowBox[{
                FormBox["k2",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  FormBox[
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}],
                    "TraditionalForm"], ")"}], "2"], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  FormBox[
                    RowBox[{
                      FormBox["k1",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}],
                    "TraditionalForm"], ")"}], "3"]}], 
            RowBox[{\((1 - D)\), " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "6"]}]], "-", 
          FractionBox[
            RowBox[{"2", " ", 
              SuperscriptBox[
                FormBox["k1",
                  "TraditionalForm"], "2"], " ", 
              RowBox[{
                FormBox["k2",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox["k1",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}]}], 
            RowBox[{\((1 - D)\), " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"]}]], "+", 
          FractionBox[
            RowBox[{"3", " ", 
              SuperscriptBox[
                FormBox["k1",
                  "TraditionalForm"], "2"], " ", 
              RowBox[{
                FormBox["k2",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  FormBox[
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}],
                    "TraditionalForm"], ")"}], "2"], " ", 
              RowBox[{
                FormBox["k1",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}]}], 
            RowBox[{\((1 - D)\), " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "4"]}]]}], TraditionalForm]], "Output"]
}], "Text"],

Cell[BoxData[
    \(\(tlrule[24] = 
      TLR[d_, pp_, 
          dp_, {2, b_?PNQ}, {0, w_?PNQ, 0, y_?PNQ, 1}, {nm1_, nm2_, nm3_, 
            nm4_, {0, 0}}] \[RuleDelayed] 
        \((TLRComment["\<tlrule24\>"]; 
          2  dp \(\( 1\/\(\((1 - d)\) pp\^2\)\) 
                \((TLR[d, pp, 
                    dp, {0, b + 1}, {0, w, 3, y, 0}, {nm1, nm2, nm3, 
                      nm4, {0, 0}}])\)\) - 
            \(\(\((d + 2)\)\ \ dp\^2\)\/\(\((1 - d)\) pp\^3\)\) 
              \((TLR[d, pp, 
                  dp, {0, b}, {0, w, 3, y + 1, 0}, {nm1, nm2, nm3, 
                    nm4, {0, 0}}])\) - 
            \(\(2  dp\)\/\(\((1 - d)\) pp\)\) 
              TLR[d, pp, 
                dp, {0, b + 1}, {1, w, 1, y, 0}, {nm1, nm2, nm3, 
                  nm4, {0, 0}}] + 
            \(\(3  dp\^2\)\/\(\((1 - d)\) pp\^2\)\) 
              TLR[d, pp, 
                dp, {0, b}, {1, w, 1, y + 1, 0}, {nm1, nm2, nm3, 
                  nm4, {0, 0}}])\); \)\)], "Input"],

Cell["Symmetry 1 <--> 2; added February 21 1999 by R.M.", "Text"],

Cell[BoxData[
    \(\(\(tlrule[25] = 
      TLR[dim_, pp_, \ {a_, \ b_, \ \((c_)\)?PNQ, \ \((d_)\)?PQ, \ 0}, \ 
          \({nm1_, \ nm2_, \ nm3_, \ {0, \ _}, \ nm5_}\n\)\ \ \ ]\  :> 
        \((TLRComment["\<tlrule25\>"]; \ 
            TLR[dim, \ pp, \ {b, a, \ d, \ c, \ 0}, 
              \ {nm2, \ nm1, \ {0, 0}, \ nm3, \ nm5}])\)\  /; \ d\  > \ c; 
    \)\n\)\)], "Input"],

Cell[BoxData[
    \(\(TLRules = Array[tlrule, 25]; \)\)], "Input"],

Cell[BoxData[
    \(\(TLRules2 = Drop[TLRules, {9, 9}]; \)\)], "Input"],

Cell[BoxData[
    \(\(TLR[__, {{_, _}, {0, _}, {_, _}, {0, _}, {_, 0}}] := 0; \)\)], "Input"],

Cell[BoxData[
    \(\(TLR[__, {{0, _}, {_, _}, {0, _}, {_, _}, {_, 0}}] := 0; \)\)], "Input"],

Cell[BoxData[
    \(\(TLR[__, {{0, _}, {_, _}, {_, 0}, {_, _}, {0, _}}] := 0; \)\)], "Input"],

Cell[BoxData[
    \(\(TLR[__, {{_, 0}, {_, _}, {0, _}, {_, _}, {0, _}}] := 0; \)\)], "Input"],

Cell[BoxData[
    \(\(TLR[__, {{_, _}, {_, 0}, {_, _}, {0, _}, {0, _}}] := 0; \)\)], "Input"],

Cell[BoxData[
    \(\(TLR[__, {{_, _}, {0, _}, {_, _}, {_, 0}, {0, _}}] := 0; \)\)], "Input"],

Cell[BoxData[
    \(\(SetAttributes[seeetDelayed, HoldAll]; \)\)], "Input"],

Cell[BoxData[
    \(seeetDelayed[TLR[a__], b_] := SetDelayed@@{TLR[a], b}\)], "Input"],

Cell[BoxData[
    \(\(lis1 = Permutations[{{0, _}, {0, _}, {0, _}, {_, _}, {_, 0}}]; \)\)], 
  "Input"],

Cell[BoxData[
    \(\(lis2 = 
      Table[TLR[__, lis1\[LeftDoubleBracket]i\[RightDoubleBracket]] 
            \[RuleDelayed] 0, {i, Length[lis1]}] /. 
        RuleDelayed \[Rule] seeetDelayed; \)\)], "Input"],

Cell[BoxData[
    \(\(Clear[lis1, lis2]; \)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["T*IS", "Subsection",
  CellTags->"T1.7.2.1"],

Cell["\<\
These functions below are never used.
They can serve as tests.\
\>", "Text"],

Cell[BoxData[
    \(TJIS[d_, Power[M_, \ 2], 
        \ {{\[Alpha]_, 0}, {\[Beta]_, 0}, {\[Gamma]_, 0}}] := 
      \(\(SMu\^2\) 
        \((\(\((\(-1\))\)\^\(\[Alpha] + \[Beta] + \[Gamma] + 1\)\/\((
                    \(-\ M\^2\))\)\^\(\[Alpha] + \[Beta] + \[Gamma] - 4\)\) 
            Exp[\(-I\)\ Pi\ \((d - 4)\)]\ 
            \(Gamma[\[Alpha] + \[Beta] + \[Gamma] - d]\/Gamma[
                  \(3\/2\) d - \[Alpha] - \[Beta] - \[Gamma]]\) 
            \(Gamma[d\/2 - \[Alpha]]\/Gamma[\[Alpha]]\) 
            \(Gamma[d\/2 - \[Beta]]\/Gamma[\[Beta]]\) 
            Gamma[d\/2 - \[Gamma]]\/Gamma[\[Gamma]])\)\n\)\)], "Input"],

Cell[BoxData[
    \(TJIS[d_, Power[M_, \ 2], 
        \ {{\[Beta]_, 0}, {\[Gamma]_, 0}, {\[Alpha]_, M_}}] := \n\t
      TJIS[d, M^2, {{\[Alpha], M}, {\[Beta], 0}, {\[Gamma], 0}}]\)], "Input"],

Cell[BoxData[
    \(TJIS[d_, Power[M_, \ 2], 
        \ {{\[Beta]_, 0}, {\[Alpha]_, M_}, {\[Gamma]_, 0}}] := \n\t
      TJIS[d, M^2, {{\[Alpha], M}, {\[Beta], 0}, {\[Gamma], 0}}]\)], "Input"],

Cell[BoxData[
    \(TJIS[d_, Power[M_, \ 2], 
        \ {{\[Alpha]_, M_}, {\[Beta]_, 0}, {\[Gamma]_, 0}}] := \n
      \(SMu\^2\/\((M\^2)\)\^\(\[Alpha] + \[Beta] + \[Gamma] - 4\)\) 
        \(\((\(-1\))\)\^\(\[Alpha] + \[Beta] + \[Gamma] + 1\)\) 
        \(Gamma[\[Alpha] + \[Beta] + \[Gamma] - d]\/\(Gamma[\[Alpha]] 
              Gamma[\[Beta]] Gamma[\[Gamma]]\)\) 
        \(\(Gamma[d\/2 - \[Gamma]] Gamma[d\/2 - \[Beta]]\)\/Gamma[
              d - \[Beta] - \[Gamma]]\) 
        \(Gamma[\[Beta] + \[Gamma] - d\/2] 
            Gamma[2\ d - \[Alpha] - 2  \[Beta] - 2  \[Gamma]]\)\/Gamma[
            \(3  d\)\/2 - \[Alpha] - \[Beta] - \[Gamma]]\)], "Input"],

Cell[BoxData[
    \(TJIS[d_, 0, \ {{\[Beta]_, 0}, {\[Alpha]_, M_}, {\[Gamma]_, 0}}] := 
      TJIS[d, 0, {{\[Alpha], M}, {\[Beta], 0}, {\[Gamma], 0}}]\)], "Input"],

Cell[BoxData[
    \(TJIS[d_, 0, \ {{\[Beta]_, 0}, {\[Gamma]_, 0}, {\[Alpha]_, M_}}] := 
      TJIS[d, 0, {{\[Alpha], M}, {\[Beta], 0}, {\[Gamma], 0}}]\)], "Input"],

Cell[BoxData[
    \(TJIS[d_, 0, \ {{\[Alpha]_, M_}, {\[Beta]_, 0}, {\[Gamma]_, 0}}] := \n
      \(SMu\^2\/\((M\^2)\)\^\(\[Alpha] + \[Beta] + \[Gamma] - 4\)\) 
        \(\((\(-1\))\)\^\(\[Alpha] + \[Beta] + \[Gamma] + 1\)\) 
        \(Gamma[\[Alpha] + \[Beta] + \[Gamma] - d]\/\(Gamma[\[Alpha]] 
              Gamma[\[Beta]] Gamma[\[Gamma]]\)\) 
        \((Gamma[\[Beta] + \[Gamma] - d/2] Gamma[d/2 - \[Beta]] 
              Gamma[d/2 - \[Gamma]])\)/Gamma[d/2]\)], "Input"],

Cell[BoxData[
    \(TJIS[d_, 0, \ {{\[Beta]_, M_}, {\[Alpha]_, 0}, {\[Gamma]_, M_}}] := 
      TJIS[d, 0, {{\[Alpha], 0}, {\[Beta], M}, {\[Gamma], M}}]\)], "Input"],

Cell[BoxData[
    \(TJIS[d_, 0, \ {{\[Beta]_, M_}, {\[Gamma]_, M_}, {\[Alpha]_, 0}}] := 
      TJIS[d, 0, {{\[Alpha], 0}, {\[Beta], M}, {\[Gamma], M}}]\)], "Input"],

Cell[BoxData[
    \(TJIS[d_, 0, \ {{\[Alpha]_, 0}, {\[Beta]_, M_}, {\[Gamma]_, M_}}] := \n
      \(SMu\^2\/\((M\^2)\)\^\(\[Alpha] + \[Beta] + \[Gamma] - 4\)\) 
        \(\((\(-1\))\)\^\(\[Alpha] + \[Beta] + \[Gamma] + 1\)\) 
        \(Gamma[\[Alpha] + \[Beta] + \[Gamma] - d]\/\(Gamma[\[Beta]] 
              Gamma[\[Gamma]]\)\) \(Gamma[d\/2 - \[Alpha]]\/Gamma[d\/2]\) 
        \(Gamma[\[Alpha] + \[Gamma] - d\/2] 
            Gamma[\[Alpha] + \[Beta] - d\/2]\)\/Gamma[
            2  \[Alpha] + \[Beta] + \[Gamma] - d]\)], "Input"],

Cell[BoxData[
    \(TJIS[d_, 0, {{\[Alpha]_, 0}, {\[Beta]_, M_}, {\[Gamma]_, M_}}] := 
      \((SMu\^2\ \((\(-1\))\)\^\(\[Alpha] + \[Beta] + \[Gamma] + 1\)\ 
            Gamma[\[Alpha] + \[Beta] + \[Gamma] - d]\ Gamma[d\/2 - \[Alpha]]\ 
            \((Gamma[\[Alpha] + \[Gamma] - d\/2]\ 
                Gamma[\[Alpha] + \[Beta] - d\/2])\))\)/
        \((\((M\^2)\)\^\(\[Alpha] + \[Beta] + \[Gamma] - 4\)\ 
            \((Gamma[\[Beta]]\ Gamma[\[Gamma]])\)\ Gamma[d\/2]\ 
            Gamma[2\ \[Alpha] + \[Beta] + \[Gamma] - d])\)\)], "Input"],

Cell["(2.65)", "Text"],

Cell[BoxData[
    \(TAIS[d_, 0, {{1, M_}}] := \ 
      \(-I\)\ \ M\^2\ \ SMu\ Gamma[\((4 - d)\)/2 - 1]\)], "Input"],

Cell[BoxData[
    \(\(TBIS[d_, 0, {{\[Alpha]_, 0}, {\[Beta]_, 0}}] := 0; \)\)], "Input"],

Cell[BoxData[
    \(TBIS[d_, Power[M_, \ 2], {{\[Alpha]_, 0}, {\[Beta]_, 0}}] := 
      \(\(I\ \) SMu\ *
        \((\(\((\(-1\))\)\^\(\[Alpha] + \[Beta]\)\/\((\(-\ M\^2\))
                    \)\^\(\[Alpha] + \[Beta] - 2\)\) 
            Exp[\(-I\)\ Pi\ \((d - 4)\)/2]\ 
            \(Gamma[\[Alpha] + \[Beta] - d/2]\/\(Gamma[\[Alpha]] 
                  Gamma[\[Beta]]\)\) 
            \(Gamma[d/2 - \[Alpha]] Gamma[d/2 - \[Beta]]\)\/Gamma[
                d - \[Alpha] - \[Beta]])\)\n\)\)], "Input"],

Cell[BoxData[
    \(TJIS[d_Symbol, M_\^2, {{1, M_}, {1, M_}, {1, M_}}] := 
      M\^2\ SMu\^2\ 
        \((\(-\(6\/\((\(-4\) + d)\)\^2\)\) + 17\/\(2\ \((\(-4\) + d)\)\) + 
            1\/8\ \((\(-59\) - 2\ \[Pi]\^2)\) + 
            \((\(-4\) + d)\)\^3\ TJI111e + 
            \(1\/1920\) 
              \((\((\(-4\) + d)\)\^2\ 
                  \((16755 - 4750\ \[Pi]\^2 - 14\ \[Pi]\^4 + 
                      3840\ \[Pi]\^2\ Log[2] - 12080\ Zeta[3])\))\) + 
            1\/96\ \((\(-4\) + d)\)\ \((195 + 98\ \[Pi]\^2 - 48\ Zeta[3])\))
          \)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["TComment", "Subsection",
  CellTags->"T1.7.3.1"],

Cell[BoxData[
    \(\($CommentNotebook = True; \)\)], "Input"],

Cell[BoxData[
    \(TComment[s_String, b_] := 
      If[If[\(\[InvisibleSpace]\(! 
              \((ValueQ[$TarcerRecursed])\)\)\), $TarcerRecursed = {}]; 
          \(\[InvisibleSpace]\(! \((MemberQ[$TarcerRecursed, {s, b}])\)\)\), 
          AppendTo[$TarcerRecursed, {s, b}]; 
          If[$Notebooks && $CommentNotebook === True, 
            If[$commentnb === False, $commentnb = True; $cnb = 
                NotebookCreate[
                  StyleDefinitions \[Rule] "\<NaturalColor.nb\>", 
                  Magnification \[Rule] 1.5]; 
              SetOptions[$cnb, 
                WindowMargins \[Rule] {{0, Automatic}, {Automatic, 0}}]; 
              SetOptions[$cnb, WindowSize \[Rule] {500, 300}]; 
              NotebookWrite[$cnb, 
                Cell[BoxData[ToBoxes[b /. TFIC \[Rule] TFI, StandardForm]], 
                  "\<Input\>", CellDingbat \[Rule] s<>"\< \>"]], 
              NotebookWrite[$cnb, 
                Cell[BoxData[ToBoxes[b /. TFIC \[Rule] TFI, StandardForm]], 
                  "\<Input\>", CellDingbat \[Rule] s<>"\< \>"]]], 
            Print[s, 
              b /. {TFIC \[RuleDelayed] TFI, TVIC \[RuleDelayed] TVI, 
                  TJIC \[RuleDelayed] TJI, TBIC \[RuleDelayed] TBI, 
                  TAIC \[RuleDelayed] TAI}]]] /; $Comment === True\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["$TTable", "Subsection",
  CellTags->"$TTable"],

Cell[BoxData[
    \(\($TTable = {}; \)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["TarcerRecurse", "Subsection",
  CellTags->"T1.7.4.1"],

Cell[BoxData[
    \(\(FEPrint[a__String] := Print[a]\  /; \ $Notebooks === False; \)\)], 
  "Input"],

Cell[BoxData[
    \(\(FEPrint[a__String] := 
      If[Head[$cnb] === NotebookObject, \n\t\t
          NotebookWrite[$cnb, Cell[StringJoin@@{a}, "\<SmallText\>"]]; \n\t
          SelectionMove[$cnb, After, Cell]] /; $Notebooks === True; \)\)], 
  "Input"],

Cell[BoxData[
    \(\(Options[TarcerRecurse] = {Factor \[Rule] Factor, 
        TimeConstraint -> Infinity, Table :> $TTable, Together -> Together, 
        Replace -> {}}; \)\)], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(TarcerRecurse[z_, opts___]\), ":=", 
        RowBox[{"Catch", "[", 
          RowBox[{
            StyleBox["Block",
              FontSize->16,
              FontWeight->"Bold",
              FontColor->RGBColor[1, 0.2, 0.2]], "[", 
            RowBox[{
            \({basisdone, factorfun, fertig, dummm1, dummm2, getvars, te, 
                time, cutoff, tog, ttable, facfun, nexp, oldexpr, rec, 
                special, sti, tl, tcheck, tjrd, tvrd, together, 
                vars, $cc = 0, $cv}\), ",", "\n", "\t\t\t", 
              RowBox[{
              \(time\  = \ AbsoluteTime[]\), ";", "\n", "\t\t\t", 
                \(ttable = \(Table /. {opts}\) /. Options[TarcerRecurse]\), 
                ";", "\n", "\t\t\t", 
                \(If[\(! MatchQ[ttable, {__RuleDelayed}]\), ttable = {}]\), 
                ";", "\n", "\t\t\t", 
                \(special = 
                  \(Replace /. {opts}\) /. Options[TarcerRecurse]\), ";", 
                "\n", "\t\t\t", 
                \(cutoff = 
                  \(TimeConstraint /. {opts}\) /. Options[TarcerRecurse]\), 
                ";", "\n", "\t\t\t", 
                \(together = 
                  \(Together /. {opts}\) /. Options[TarcerRecurse]\), ";", 
                \(factorfun = 
                  \(Factor /. {opts}\) /. Options[TarcerRecurse]\), ";", 
                \($commentnb = False\), ";", 
                \(tog = 
                  \((\((If[#1 > 1001, 
                              If[$Comment, 
                                FEPrint[
                                  "\<coefficient # \>"<>
                                    If[ValueQ[$cv], ToString[$cc], "\<\>"]<>
                                    If[ValueQ[$cv], 
                                      "\< (\>"<>ToString[$cv]<>"\<)\>", 
                                      "\<\>"]<>"\< simplified, \>", 
                                  ToString[#1]]]]; #2)\)&)\)[LeafCount[#1], 
                      \($cc++\); together[#1]]&\), ";", 
                \(If[\(\[InvisibleSpace]\(! 
                    \((ValueQ[$TarcerRecursed])\)\)\), $TarcerRecursed = {}]
                  \), ";", \($TarcerRecursed = {}\), ";", 
                RowBox[{"If", "[", "\[InvisibleSpace]", 
                  RowBox[{
                  \(! \((MemberQ[$ContextPath, 
                          "\<HighEnergyPhysics`FeynCalc`ToTFi`\>"])\)\), ",", 
                    \(t0 = z\), ",", 
                    RowBox[{"t0", "=", 
                      RowBox[{
                      \(HighEnergyPhysics`FeynCalc`ToTFi`ToTFi[z]\), "/.", 
                        RowBox[{"{", 
                          RowBox[{
                            RowBox[{
                            "HighEnergyPhysics`FeynCalc`TFi`TFi", 
                              "\[RuleDelayed]", 
                              StyleBox["TFI",
                                FontColor->RGBColor[1, 0.6, 0]]}], ",", 
                            \(HighEnergyPhysics`FeynCalc`ToTFi`ToTFi 
                              \[RuleDelayed] Identity\)}], "}"}]}]}]}], 
                  "]"}], ";", "\n", "\t\t\t", 
                \(If[ttable =!= {}, t0 = t0 /. ttable]\), ";", "\n", 
                "\t\t\t", 
                RowBox[{"getvars", "=", 
                  RowBox[{
                    RowBox[{"Select", "[", 
                      RowBox[{\(Variables[#1]\), ",", 
                        RowBox[{
                          RowBox[{"MatchQ", "[", 
                            RowBox[{"#1", ",", 
                              RowBox[{
                                RowBox[{
                                  StyleBox["TFI",
                                    FontColor->RGBColor[1, 0.6, 0]], "[", 
                                  "__", "]"}], "|", 
                                RowBox[{
                                  StyleBox["TVI",
                                    FontColor->RGBColor[0.2, 0.4, 0.8]], "[", 
                                  "__", "]"}], "|", \(TJI[__]\), "|", 
                                \(TBI[__]\), "|", \(TAI[__]\)}]}], "]"}], 
                          "&"}]}], "]"}], "&"}]}], ";", "\n", "\t\t\t", 
                \(If[\(! FreeQ[t0, TFI]\), 
                  t0 = Collect[t0 = TFIRecurse[t0] /. ttable, getvars[t0], 
                      tog]]\), ";", "\n", "\t\t\t", 
                \(If[\(! FreeQ[t0, TVI[_Plus, __]]\), \n\t\t\t
                  tvrd = Select[DownValues[TVR], 
                        \(! FreeQ[#, TVI[_ - 2, __]]\)&] /. TVR -> TVI; \n
                  \t\t\t\tt0 = 
                    Collect[
                      t0 = \(\(\(t0 /. tvrd\) /. tvrd\) /. tvrd\) /. tvrd, 
                      getvars[t0], tog]; \n\t\t\t\t
                  If[\(! FreeQ[t0, TVI[_Plus, __]]\), \t
                    t0 = Collect[t0 = t0 //. tvrd, getvars[t0], tog]]]\), 
                ";", "\n", "\t\t\t\t", "\n", "\t\t\t", 
                \(tjrd = 
                  Select[DownValues[TJR], \(! FreeQ[#, TJI[_ - 2, __]]\)&] /. 
                    TJR -> TJI\), ";", "\n", "\t\t\t", "\n", "\t\t\t", 
                \(If[\(! FreeQ[t0, TJI[_Plus, __]]\), \n\t\t\t
                  tjrd = Select[DownValues[TJR], 
                        \(! FreeQ[#, TJI[_ - 2, __]]\)&] /. TJR -> TJI; \n
                  \t\t\t\tt0 = 
                    Collect[
                      t0 = \(\(\(t0 /. tjrd\) /. tjrd\) /. tjrd\) /. tjrd, 
                      getvars[t0], tog]; \n\t\t\t\t
                  If[\(! FreeQ[t0, TJI[_Plus, __]]\), \t
                    t0 = Collect[t0 = t0 //. tjrd, getvars[t0], tog]]]\), 
                ";", "\n", "\t\t\t", "\n", "\t\t\t", 
                \(rec[expr_, {ti_, tr_}] := 
                  \((sti = ToString[ti]; 
                    tcheck = 
                      \(\[InvisibleSpace]\(! \((FreeQ[expr, ti])\)\)\); 
                    oldexpr =  . ; nexp = expr; 
                    While[tcheck === True && oldexpr =!= nexp, 
                      oldexpr = nexp; 
                      If[$Comment, 
                        \((If[#1 === 1, 
                                FEPrint[sti<>"\< level, there is 1 \>"<>sti], 
                                FEPrint[sti<>"\< level, there are \>", 
                                  ToString[#1], "\< \>"<>sti<>"\<'s \>"]]&)\)[
                          Count[nexp, ti[__], \(-1\)]]]; \n\t\t\t\t\n
                      \t\t\t\t\t\t
                      nexp = \(nexp /. ti \[Rule] tr\) /. tr \[Rule] ti; \n
                      \t\t\t\t\t\t\t
                      If[\(! FreeQ[nexp, TJI[_Plus, __]]\), 
                        nexp = nexp //. tjrd]; \n\t\t\t\t\t\t\n\t\t\t\t\t\t
                      tvars = getvars[nexp]; $cc = 0; $cv = Length[tvars]; 
                      nexp = Collect[nexp, tvars, tog]; \n\t\t\t\t\t\t
                      If[cutuff =!= Infinity, 
                        If[AbsoluteTime[] - time > cutoff, Throw[$Failed]]]; 
                      \n\t\t\t\t\t\t\n\t\t\t\t\t\t
                      tcheck = 
                        \[InvisibleSpace]\(! \((FreeQ[nexp, ti])\)\); ]; nexp)
                    \)\), ";", \(basisdone = 0\), ";", 
                RowBox[{"tl", "=", 
                  RowBox[{"{", 
                    RowBox[{
                      RowBox[{"{", 
                        RowBox[{
                          StyleBox["TFI",
                            FontColor->RGBColor[1, 0.6, 0]], ",", 
                          StyleBox["TFR",
                            FontColor->RGBColor[0, 0.6, 0]]}], "}"}], ",", 
                      RowBox[{"{", 
                        RowBox[{
                          StyleBox["TVI",
                            FontColor->RGBColor[0.2, 0.4, 0.8]], ",", 
                          StyleBox["TVR",
                            FontColor->RGBColor[0.2, 0.8, 1]]}], "}"}], ",", 
                      \({TJI, TJR}\), ",", \({TBI, TBR}\), ",", 
                      \({TAI, TAR}\)}], "}"}]}], ";", 
                \(Do[t0 = 
                    rec[t0, tl\[LeftDoubleBracket]i\[RightDoubleBracket]]; 
                  If[Head[t0] === Plus, 
                    fertig = 
                      Select[t0, 
                        \(\[InvisibleSpace]\(! 
                          \((FreeQ[#1, 
                              tl\[LeftDoubleBracket]i, 1
                                \[RightDoubleBracket]])\)\)\)]; 
                    basisdone = basisdone + fertig; t0 = t0 - fertig], {i, 
                    Length[tl]}]\), ";", \(t0 = t0 + basisdone\), ";", 
                \($cv =  . \), ";", 
                \(t0 = Collect[
                    t0, {TAI[__], TAI[a1__]\ TAI[a2__], TAI[__], TBI[__], 
                      TJI[__], TVI[__], TFI[__]}, factorfun]\), ";", 
                \(If[$Comment, 
                  FEPrint["\<the result has \>", 
                    ToString[Length[t0 + dummm1 + dummm2] - 2], "\< terms\>"]]
                  \), ";", 
                \(If[$Notebooks, SetSelectedNotebook[EvaluationNotebook[]]]
                  \), ";", "t0"}]}], "]"}], "]"}]}], ";"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["TarcerExpand", "Section",
  CellDingbat->"\[FilledSmallCircle]",
  CellTags->"T1.8.1"],

Cell[BoxData[
    \(\(Options[TarcerExpand] = {\n\t\tTarcerReduce :> $BasisIntegrals, \n\t
        TarcerRecurse -> False, Zeta -> True}; \)\)], "Input"],

Cell[BoxData[
    \(\(TarcerExpand[expr_, d_Symbol -> fe_ /; Length[Variables[fe]] === 1, 
        ru___Rule] := TarcerExpand[expr, d\  -> fe, 0, ru]; \)\)], "Input"],

Cell[BoxData[
    RowBox[{
    \(TarcerExpand[expr_, 
        d_Symbol \[Rule] fe_ /; Length[Variables[fe]] === 1, 
        order_Integer?NonNegative, opts___Rule]\), ":=", 
      RowBox[{
        StyleBox["Block",
          FontSize->16,
          FontWeight->"Bold",
          FontColor->RGBColor[1, 0.2, 0.2]], "[", 
        RowBox[{\({}\), ",", 
          RowBox[{"Catch", "[", 
            RowBox[{
            \(e = First[Variables[fe]]\), ";", 
              \({ints, recurse, 
                  zeta2replace} = {TarcerReduce, TarcerRecurse, Zeta} /. 
                  Flatten[{opts, Options[TarcerExpand]}]\), ";", 
              \(If[zeta2replace =!= True && 
                  MemberQ[$ContextPath, 
                    "\<HighEnergyPhysics`FeynCalc`Zeta2`\>"], z2rule = {}, 
                z2rule = 
                  \[Pi]\^2 \[RuleDelayed] 
                    6\ ToExpression[
                        "\<HighEnergyPhysics`FeynCalc`Zeta2`Zeta2\>"]]\), 
              ";", \(ints = Flatten[ints]\), ";", 
              \(If[recurse =!= True, ta0 = expr, ta0 = TarcerRecurse[expr]]
                \), ";", 
              RowBox[{"t1", "=", 
                RowBox[{\(ta0 /. ints\), "/.", 
                  RowBox[{
                    InterpretationBox["any_",
                      PreFactor[ 
                        Pattern[ FE`Conversion`Private`any, 
                          Blank[ ]]],
                      Editable->False], "\[RuleDelayed]", 
                    InterpretationBox["any",
                      PreFactor[ FE`Conversion`Private`any],
                      Editable->False]}]}]}], ";", 
              \(vars = Select[Variables[t1], Length[#1] > 0&]\), ";", 
              RowBox[{"If", "[", 
                RowBox[{
                  RowBox[{"\[InvisibleSpace]", "!", 
                    RowBox[{"(", 
                      RowBox[{"FreeQ", "[", 
                        RowBox[{"vars", ",", 
                          RowBox[{
                            StyleBox["TFI",
                              FontColor->RGBColor[1, 0.6, 0]], "|", 
                            StyleBox["TVI",
                              FontColor->RGBColor[0.2, 0.4, 0.8]], "|", 
                            "TJI", "|", "TKI", "|", "TBI", "|", "TAI"}]}], 
                        "]"}], ")"}]}], ",", 
                  RowBox[{
                  \(Print["\<Returning the list of remaining integrals.\>"]
                      \), ";", 
                    RowBox[{"Throw", "[", 
                      RowBox[{"Select", "[", 
                        RowBox[{"vars", ",", 
                          RowBox[{
                            RowBox[{"\[InvisibleSpace]", "!", 
                              RowBox[{"(", 
                                RowBox[{"FreeQ", "[", 
                                  RowBox[{"#1", ",", 
                                    RowBox[{
                                      StyleBox["TFI",
                                        FontColor->RGBColor[1, 0.6, 0]], 
                                      "|", 
                                      StyleBox["TVI",
                                        FontColor->RGBColor[0.2, 0.4, 0.8]], 
                                      "|", "TJI", "|", "TKI", "|", "TBI", 
                                      "|", "TAI"}]}], "]"}], ")"}]}], 
                            "&"}]}], "]"}], "]"}]}]}], "]"}], ";", 
              \(t1 = t1 /. d \[Rule] fe\), ";", 
              \(polygafu[2, 1] = \(-2\)\ Zeta[3]\), ";", 
              \(polygafu[2, 2] = 2 - 2\ Zeta[3]\), ";", 
              \(polygafu[x_, y_] := 
                \(polygafu[x, y] = FunctionExpand[PolyGamma[x, y]]\)\), ";", 
              \(t2 = Normal[t1 + O[e]\^\(order + 1\)] /. 
                  PolyGamma \[Rule] polygafu\), ";", 
              RowBox[{"t3", "=", 
                RowBox[{
                  RowBox[{\(Collect[t2, e, Expand[#1] /. z2rule&]\), "//.", 
                    RowBox[{"{", 
                      RowBox[{
                        RowBox[{
                          InterpretationBox["a_",
                            PreFactor[ 
                              Pattern[ FE`Conversion`Private`a, 
                                Blank[ ]]],
                            Editable->False], " ", 
                          InterpretationBox["b_",
                            PreFactor[ 
                              Pattern[ FE`Conversion`Private`b, 
                                Blank[ ]]],
                            Editable->False]}], "\[RuleDelayed]", 
                        InterpretationBox[\(a\ b\),
                          PreFactor[ 
                            Times[ 
                            FE`Conversion`Private`a, 
                              FE`Conversion`Private`b]],
                          Editable->False]}], "}"}]}], "/.", 
                  RowBox[{
                    InterpretationBox["c_",
                      PreFactor[ 
                        Pattern[ FE`Conversion`Private`c, 
                          Blank[ ]]],
                      Editable->False], "\[RuleDelayed]", 
                    InterpretationBox["c",
                      PreFactor[ FE`Conversion`Private`c],
                      Editable->False]}]}]}], ";", 
              RowBox[{"prec", "=", 
                RowBox[{"Union", "[", 
                  RowBox[{"Cases", "[", 
                    RowBox[{"t3", ",", 
                      InterpretationBox["_",
                        PreFactor[ 
                          Blank[ ]],
                        Editable->False], ",", \(-1\)}], "]"}], "]"}]}], 
              ";", 
              RowBox[{"If", "[", 
                RowBox[{\(Length[prec] === 1\), ",", 
                  RowBox[{"If", "[", 
                    RowBox[{\(Head[t3] === Plus\), ",", 
                      RowBox[{
                      \(mass = 
                          Select[
                            Variables[
                              prec\[LeftDoubleBracket]1, 1
                                \[RightDoubleBracket]], FreeQ[#1, SEpsilon]&]
                            \), ";", 
                        \(If[Length[mass] === 1, 
                          If[FreeQ[First[t3], First[mass]], massfactor = 1, 
                            massfactor = 
                              First[mass]\^Exponent[First[t3], First[mass]]], 
                          massfactor = 1]\), ";", \(delpfactor = 1\), ";", 
                        RowBox[{"delp", "=", 
                          RowBox[{"Variables", "[", 
                            RowBox[{\(First[t3]\/massfactor\), "/.", 
                              RowBox[{"{", 
                                RowBox[{
                                  RowBox[{
                                    InterpretationBox["__",
                                      PreFactor[ 
                                        BlankSequence[ ]],
                                      Editable->False], "\[RuleDelayed]", 
                                    "1"}], ",", 
                                  \(ToExpression[
                                      "\<HighEnergyPhysics`Tarcer`Zeta2`Zeta2\
\>"] \[Rule] Zeta[2]\)}], "}"}]}], "]"}]}], ";", 
                        \(If[Length[delp] === 1, delp = First[delp]; 
                          If[Length[t3] === 
                              Length[
                                Select[t3, 
                                  \(\[InvisibleSpace]\(! 
                                      \((FreeQ[#1, delp])\)\)\)&]], 
                            delpfactor = delp\^Exponent[First[t3], delp]]]\), 
                        ";", \(t3 = 
                          \((\(massfactor\ delpfactor\ First[prec] /. 
                                  PreFactor \[Rule] Identity\) /. {
                                  \((m_\^2)\)\^n_\ m_\^j_?EvenQ 
                                    \[RuleDelayed] \((m\^2)\)\^\(n + j\/2\)})
                              \)\ Collect[
                              \((#1\/\(First[prec]\ massfactor\ delpfactor\)&)
                                  \)/@t3, e, Expand]\)}]}], "]"}]}], "]"}], 
              ";", \(t3 /. PreFactor \[Rule] Identity\)}], "]"}]}], "]"}]}]], 
  "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Construct T's", "Section",
  CellDingbat->"\[FilledSmallCircle]",
  CellTags->"T1.9.1"],

Cell[CellGroupData[{

Cell["remember", "Subsection",
  CellTags->"T1.9.1.1"],

Cell["The function remember is not used by default.", "Text"],

Cell[BoxData[
    \(\(tfch = {TFR :> Hold[TFR], TVR :> Hold[TVR], TJR :> Hold[TJR], 
        TBR :> Hold[TBR], TAR :> Hold[TAR], TComment :> Hold[TComment]}; 
    \)\)], "Input"],

Cell[BoxData[
    \(\(rtfch = Map[Reverse, tfch]; \)\)], "Input"],

Cell[BoxData[
    \(\(SetAttributes[condrest, HoldRest]; \)\)], "Input"],

Cell[BoxData[
    \(\(Clear[remember]; \)\)], "Input"],

Cell[BoxData[
    \(\(remember[zi_HoldForm] := 
      Block[{z = zi /. tfch}, 
        If[\(\[InvisibleSpace]\(! \((FreeQ[z, Condition])\)\)\), 
          \(\(\(\(Hold[HoldForm]\)[
                    \(Hold[SetDelayed]\)[
                      z\[LeftDoubleBracket]1, 1\[RightDoubleBracket], 
                      ReplacePart[
                        z\[LeftDoubleBracket]1, 2\[RightDoubleBracket], 
                        Prepend[Extract[z, {1, 2, 1}, Hold], 
                          \(\(\(z\[LeftDoubleBracket]1, 1
                                    \[RightDoubleBracket] /. 
                                  PatternTest \[Rule] pt\) /. 
                                pt[a_, b_] \[RuleDelayed] a\) /. 
                              Pattern \[Rule] pat\) /. 
                            pat[a_, b_] \[RuleDelayed] a], 1]]] /. 
                  Hold[HoldForm] \[Rule] HoldForm\) /. rtfch\) /. 
              Hold[SetDelayed] \[Rule] SetDelayed\) /. Hold \[Rule] Set, 
          \(\(\(\(Hold[HoldForm]\)[
                    \(Hold[SetDelayed]\)[
                      z\[LeftDoubleBracket]1, 1\[RightDoubleBracket], 
                      Prepend[Extract[z, {1, 2}, Hold], 
                        \(\(\(z\[LeftDoubleBracket]1, 1
                                  \[RightDoubleBracket] /. 
                                PatternTest \[Rule] pt\) /. 
                              pt[a_, b_] \[RuleDelayed] a\) /. 
                            Pattern \[Rule] pat\) /. 
                          pat[a_, b_] \[RuleDelayed] a]]] /. 
                  Hold[HoldForm] \[Rule] HoldForm\) /. rtfch\) /. 
              Hold[SetDelayed] \[Rule] SetDelayed\) /. Hold \[Rule] Set]]; 
    \)\)], "Input"],

Cell[BoxData[
    \($TarasovTdeltaplimit\)], "Input"],

Cell[BoxData[
    \(\(aa2\  = \ 
      Array[f\_#&, \ 10000 + \(($TarasovTdeltaplimit + 1)\)\ 1000]; \)\)], 
  "Input"],

Cell[BoxData[
    \(\(bb12 = Select[aa2, Head[#] =!= Subscript&]; \)\)], "Input"],

Cell[BoxData[
    \(\(Clear[aa2]; \)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Print["\<there are \>", Length[bb12], 
      "\< recursion equations for $RankLimit = \>", $RankLimit]\)], "Input"],

Cell[BoxData[
    InterpretationBox[
      \("there are "\[InvisibleSpace]103
        \[InvisibleSpace]" recursion equations for $RankLimit = "\
\[InvisibleSpace]{0, 6}\),
      SequenceForm[ 
      "there are ", 103, " recursion equations for $RankLimit = ", {0, 6}],
      Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
    \(\(If[Global`$Remember\  === \ True, Global`RCHECK = True; \ 
      nb\  = Map[remember, bb12], nb = bb12]; \)\)], "Input"],

Cell[BoxData[
    \(\(Clear[bb12]; \)\)], "Input"],

Cell[BoxData[
    \(\(ReleaseHold[nb]; \)\)], "Input"],

Cell[BoxData[
    \(Unset[$Post]\)], "Input"],

Cell[BoxData[
    \(\($Comment = True; \)\)], "Input"],

Cell[BoxData[
    \(\(End[]; \)\)], "Input"],

Cell[BoxData[
    \(\(EndPackage[]; \)\)], "Input"],

Cell[BoxData[
    \(\(BeginPackage["\<HighEnergyPhysics`FeynCalc`Zeta2`\>"]; \)\)], "Input"],

Cell[BoxData[
    \(\(Zeta2::usage = \n"\<Zeta2 denotes Zeta[2].\>"; \)\)], "Input"],

Cell[BoxData[
    \(\(Begin["\<`Private`\>"]; \)\)], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetAttributes", "[", 
        RowBox[{
          InterpretationBox[\(\[Zeta] \((2)\)\),
            Zeta2,
            Editable->False], ",", "ReadProtected"}], "]"}], ";"}]], "Input"],

Cell[BoxData[
    \(\(Zeta2\  /: \ N[Zeta2]\  = \ N[Zeta[2]]; \)\)], "Input"],

Cell[BoxData[
    \(Zeta2 /: MakeBoxes[Zeta2, fmt_] := 
      InterpretationBox@@{\ \ RowBox[{"\<\[Zeta]\>", "\<(\>", 2, "\<)\>"}], 
          Zeta2, Editable -> False}\)], "Input"],

Cell[BoxData[
    \(\(End[]; \)\)], "Input"],

Cell[BoxData[
    \(\(\ \(EndPackage[]; \)\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["$BasisIntegrals", "Section",
  CellDingbat->"\[EmptySmallCircle]",
  CellTags->"T1.10.1"],

Cell["\<\
This a (incomplete) set of basis integrals. You can add your \
favorite integrals here.\
\>", "Text"],

Cell[CellGroupData[{

Cell["\<\
$BasisIntegrals = 
  {TAI[d_, 0, {{1, M_}}] :> 
    -I*E^(1/2*(4 - d)*EulerGamma)*M^2*
     Gamma[-1 + (4 - d)/2]*
     PreFactor[(M^2)^(1/2*(-4 + d))*
       SEpsilon[d]], 
       
       TBI[d_, pp_, {{1, 0},{1, 0}}] :> 
   (I*E^(((4 - d)*EulerGamma)/2 - 
         I/2*(-4 + d)*Pi)*Gamma[2 - d/2]*
      Gamma[-1 + d/2]^2*
      PreFactor[(pp)^((-4 + d)/2)*
        SEpsilon[d]])/Gamma[-2 + d],
        
        TJI[d_, pp_, {{1,0},{1,0},{1,0}}] :>-((pp*Gamma[3 - d]*Gamma[-1 + \
d/2]^3*
    E^((4 - d)*EulerGamma)*  PreFactor[
        (pp)^(-4 + d)*SEpsilon[d]^2])/
    (E^(I*(-4 + d)*Pi)*Gamma[-3 + (3*d)/2])
),
    
          TJI[d_, M_^2, {{1,M_},{1,0},{1,0}}] :>(M^2*Gamma[3 - d]*Gamma[2 - \
d/2]*
    Gamma[-1 + d/2]^2*Gamma[-5 + 2*d]*E^((4 - d)*EulerGamma)*
    PreFactor[
      (M^2)^(-4 + d)*SEpsilon[d]^2])/
  (Gamma[-2 + d]*Gamma[-3 + (3*d)/2]),
  
  TJI[d_Symbol, (M_)^2, 
  {{1, M_}, {1, M_}, {1, M_}}]:>M^2*PreFactor[
    (M^2)^(-4 + d)*SEpsilon[d]^2]*
  (-6/(-4 + d)^2 + 17/(2*(-4 + d)) + 
    (-59 - 2*Pi^2)/8 + 
    (-4 + d)^3*TJI111e + 
    ((-4 + d)^2*(16755 - 4750*Pi^2 - 
         14*Pi^4 + 3840*Pi^2*Log[2] - 
         12080*Zeta[3]))/1920 + 
    ((-4 + d)*(195 + 98*Pi^2 - 
         48*Zeta[3]))/96),
         
         TJI[d_, 0, {{1, M_}, {1, M_}, {1, 0}}]:>(M^2*Gamma[3 - d]*Gamma[2 - \
d/2]^2*
    Gamma[-1 + d/2]*E^((4 - d)*EulerGamma)*
    PreFactor[
      (M^2)^(-4 + d)*SEpsilon[d]^2])/
  (Gamma[4 - d]*Gamma[d/2]),
  
  TJI[d_, 0, {{1,M_},{1,0},{1,0}}] :>(M^2*Gamma[3 - d]*Gamma[2 - d/2]*
    Gamma[-1 + d/2]^2*E^((4 - d)*EulerGamma)*
    PreFactor[
      (M^2)^(-4 + d)*SEpsilon[d]^2])/
  Gamma[d/2]
        }\
\>", "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["A",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              FormBox[\({1, M_}\),
                "TraditionalForm"], 
              RowBox[{"(", 
                FormBox["d_",
                  "TraditionalForm"], ")"}]],
            TAI[ 
              Pattern[ d, 
                Blank[ ]], 0, {{1, 
              Pattern[ M, 
                Blank[ ]]}}],
            Editable->True], "\[RuleDelayed]", 
          RowBox[{
          \(-I\), " ", \(E\^\(1\/2\ \((4 - d)\)\ EulerGamma\)\), " ", 
            \(M\^2\), " ", \(Gamma[\(-1\) + \(4 - d\)\/2]\), " ", 
            RowBox[{"(", 
              InterpretationBox[
                RowBox[{\(\((M\^2)\)\^\(1\/2\ \((\(-4\) + d)\)\)\), " ", 
                  InterpretationBox[
                    StyleBox[\(S\_d\),
                      FontWeight->"Bold"],
                    SEpsilon[ d],
                    Editable->False]}],
                PreFactor[ 
                  Times[ 
                    Power[ 
                      Power[ M, 2], 
                      Times[ 
                        Rational[ 1, 2], 
                        Plus[ -4, d]]], 
                    SEpsilon[ d]]],
                Editable->False], ")"}]}]}], ",", 
        RowBox[{
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["B",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                FormBox[\({1, 0}\),
                  "TraditionalForm"], 
                FormBox[\({1, 0}\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox["d_",
                  "TraditionalForm"], ")"}]],
            TBI[ 
              Pattern[ d, 
                Blank[ ]], 
              Pattern[ pp, 
                Blank[ ]], {{1, 0}, {1, 0}}],
            Editable->True], "\[RuleDelayed]", 
          RowBox[{\(1\/Gamma[\(-2\) + d]\), 
            RowBox[{"(", 
              RowBox[{
              "I", " ", 
                \(E\^\(1\/2\ \((4 - d)\)\ EulerGamma - 
                      1\/2\ I\ \((\(-4\) + d)\)\ \[Pi]\)\), " ", 
                \(Gamma[2 - d\/2]\), " ", \(Gamma[\(-1\) + d\/2]\^2\), " ", 
                RowBox[{"(", 
                  InterpretationBox[
                    RowBox[{\(pp\^\(1\/2\ \((\(-4\) + d)\)\)\), " ", 
                      InterpretationBox[
                        StyleBox[\(S\_d\),
                          FontWeight->"Bold"],
                        SEpsilon[ d],
                        Editable->False]}],
                    PreFactor[ 
                      Times[ 
                        Power[ pp, 
                          Times[ 
                            Rational[ 1, 2], 
                            Plus[ -4, d]]], 
                        SEpsilon[ d]]],
                    Editable->False], ")"}]}], ")"}]}]}], ",", 
        RowBox[{
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["J",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                FormBox[\({1, 0}\),
                  "TraditionalForm"], 
                FormBox[\({1, 0}\),
                  "TraditionalForm"], 
                FormBox[\({1, 0}\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox["d_",
                  "TraditionalForm"], ")"}]],
            TJI[ 
              Pattern[ d, 
                Blank[ ]], 
              Pattern[ pp, 
                Blank[ ]], {{1, 0}, {1, 0}, {1, 0}}],
            Editable->True], "\[RuleDelayed]", 
          RowBox[{"-", 
            FractionBox[
              RowBox[{
              "pp", " ", \(Gamma[3 - d]\), " ", \(Gamma[\(-1\) + d\/2]\^3\), 
                " ", \(E\^\(\((4 - d)\)\ EulerGamma\)\), " ", 
                RowBox[{"(", 
                  InterpretationBox[
                    RowBox[{\(pp\^\(\(-4\) + d\)\), " ", 
                      InterpretationBox[
                        StyleBox[\(S\_d\%2\),
                          FontWeight->"Bold"],
                        SEpsilon[ d],
                        Editable->False]}],
                    PreFactor[ 
                      Times[ 
                        Power[ pp, 
                          Plus[ -4, d]], 
                        Power[ 
                          SEpsilon[ d], 2]]],
                    Editable->False], ")"}]}], 
              \(E\^\(I\ \((\(-4\) + d)\)\ \[Pi]\)\ 
                Gamma[\(-3\) + \(3\ d\)\/2]\)]}]}], ",", 
        RowBox[{
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["J",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                FormBox[\({1, M_}\),
                  "TraditionalForm"], 
                FormBox[\({1, 0}\),
                  "TraditionalForm"], 
                FormBox[\({1, 0}\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox["d_",
                  "TraditionalForm"], ")"}]],
            TJI[ 
              Pattern[ d, 
                Blank[ ]], 
              Power[ 
                Pattern[ M, 
                  Blank[ ]], 2], {{1, 
              Pattern[ M, 
                Blank[ ]]}, {1, 0}, {1, 0}}],
            Editable->True], "\[RuleDelayed]", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
              \(M\^2\), " ", \(Gamma[3 - d]\), " ", \(Gamma[2 - d\/2]\), " ", 
                \(Gamma[\(-1\) + d\/2]\^2\), " ", \(Gamma[\(-5\) + 2\ d]\), 
                " ", \(E\^\(\((4 - d)\)\ EulerGamma\)\), " ", 
                RowBox[{"(", 
                  InterpretationBox[
                    RowBox[{\(\((M\^2)\)\^\(\(-4\) + d\)\), " ", 
                      InterpretationBox[
                        StyleBox[\(S\_d\%2\),
                          FontWeight->"Bold"],
                        SEpsilon[ d],
                        Editable->False]}],
                    PreFactor[ 
                      Times[ 
                        Power[ 
                          Power[ M, 2], 
                          Plus[ -4, d]], 
                        Power[ 
                          SEpsilon[ d], 2]]],
                    Editable->False], ")"}]}], ")"}], "/", 
            \((Gamma[\(-2\) + d]\ Gamma[\(-3\) + \(3\ d\)\/2])\)}]}], ",", 
        RowBox[{
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["J",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                FormBox[\({1, M_}\),
                  "TraditionalForm"], 
                FormBox[\({1, M_}\),
                  "TraditionalForm"], 
                FormBox[\({1, M_}\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox["d_Symbol",
                  "TraditionalForm"], ")"}]],
            TJI[ 
              Pattern[ d, 
                Blank[ Symbol]], 
              Power[ 
                Pattern[ M, 
                  Blank[ ]], 2], {{1, 
              Pattern[ M, 
                Blank[ ]]}, {1, 
              Pattern[ M, 
                Blank[ ]]}, {1, 
              Pattern[ M, 
                Blank[ ]]}}],
            Editable->True], "\[RuleDelayed]", 
          RowBox[{\(M\^2\), " ", 
            RowBox[{"(", 
              InterpretationBox[
                RowBox[{\(\((M\^2)\)\^\(\(-4\) + d\)\), " ", 
                  InterpretationBox[
                    StyleBox[\(S\_d\%2\),
                      FontWeight->"Bold"],
                    SEpsilon[ d],
                    Editable->False]}],
                PreFactor[ 
                  Times[ 
                    Power[ 
                      Power[ M, 2], 
                      Plus[ -4, d]], 
                    Power[ 
                      SEpsilon[ d], 2]]],
                Editable->False], ")"}], " ", 
            \((\(-\(6\/\((\(-4\) + d)\)\^2\)\) + 
                17\/\(2\ \((\(-4\) + d)\)\) + 
                1\/8\ \((\(-59\) - 2\ \[Pi]\^2)\) + 
                \((\(-4\) + d)\)\^3\ TJI111e + 
                \(\((\(-4\) + d)\)\^2\ 
                    \((16755 - 4750\ \[Pi]\^2 - 14\ \[Pi]\^4 + 
                        3840\ \[Pi]\^2\ Log[2] - 12080\ Zeta[3])\)\)\/1920 + 
                1\/96\ \((\(-4\) + d)\)\ 
                  \((195 + 98\ \[Pi]\^2 - 48\ Zeta[3])\))\)}]}], ",", 
        RowBox[{
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["K",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                FormBox[\({1, M_}\),
                  "TraditionalForm"], 
                FormBox[\({1, M_}\),
                  "TraditionalForm"], 
                FormBox[\({1, 0}\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox["d_",
                  "TraditionalForm"], ")"}]],
            TJI[ 
              Pattern[ d, 
                Blank[ ]], 0, {{1, 
              Pattern[ M, 
                Blank[ ]]}, {1, 
              Pattern[ M, 
                Blank[ ]]}, {1, 0}}],
            Editable->True], "\[RuleDelayed]", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
              \(M\^2\), " ", \(Gamma[3 - d]\), " ", \(Gamma[2 - d\/2]\^2\), 
                " ", \(Gamma[\(-1\) + d\/2]\), " ", 
                \(E\^\(\((4 - d)\)\ EulerGamma\)\), " ", 
                RowBox[{"(", 
                  InterpretationBox[
                    RowBox[{\(\((M\^2)\)\^\(\(-4\) + d\)\), " ", 
                      InterpretationBox[
                        StyleBox[\(S\_d\%2\),
                          FontWeight->"Bold"],
                        SEpsilon[ d],
                        Editable->False]}],
                    PreFactor[ 
                      Times[ 
                        Power[ 
                          Power[ M, 2], 
                          Plus[ -4, d]], 
                        Power[ 
                          SEpsilon[ d], 2]]],
                    Editable->False], ")"}]}], ")"}], "/", 
            \((Gamma[4 - d]\ Gamma[d\/2])\)}]}], ",", 
        RowBox[{
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["K",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                FormBox[\({1, M_}\),
                  "TraditionalForm"], 
                FormBox[\({1, 0}\),
                  "TraditionalForm"], 
                FormBox[\({1, 0}\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox["d_",
                  "TraditionalForm"], ")"}]],
            TJI[ 
              Pattern[ d, 
                Blank[ ]], 0, {{1, 
              Pattern[ M, 
                Blank[ ]]}, {1, 0}, {1, 0}}],
            Editable->True], "\[RuleDelayed]", 
          RowBox[{\(1\/Gamma[d\/2]\), 
            RowBox[{"(", 
              RowBox[{
              \(M\^2\), " ", \(Gamma[3 - d]\), " ", \(Gamma[2 - d\/2]\), " ", 
                \(Gamma[\(-1\) + d\/2]\^2\), " ", 
                \(E\^\(\((4 - d)\)\ EulerGamma\)\), " ", 
                RowBox[{"(", 
                  InterpretationBox[
                    RowBox[{\(\((M\^2)\)\^\(\(-4\) + d\)\), " ", 
                      InterpretationBox[
                        StyleBox[\(S\_d\%2\),
                          FontWeight->"Bold"],
                        SEpsilon[ d],
                        Editable->False]}],
                    PreFactor[ 
                      Times[ 
                        Power[ 
                          Power[ M, 2], 
                          Plus[ -4, d]], 
                        Power[ 
                          SEpsilon[ d], 2]]],
                    Editable->False], ")"}]}], ")"}]}]}]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Save tarcer*.mx file", "Section",
  CellDingbat->"\[FilledSmallCircle]",
  CellTags->"T1.11.1"],

Cell[BoxData[
    \(\(Share[]; \)\)], "Input"],

Cell["\<\
DumpSave[StringJoin[\"tarcer\", StringJoin @@ 
     ToString /@ $RankLimit, \".mx\"], 
   {$ContextPath, HighEnergyPhysics`Tarcer`Cayley, 
    HighEnergyPhysics`Tarcer`CayleyD, 
    HighEnergyPhysics`Tarcer`Cayleyu, 
    HighEnergyPhysics`Tarcer`PreFactor, 
    HighEnergyPhysics`Tarcer`TarcerRecurse, 
    HighEnergyPhysics`Tarcer`TFIRecurse, 
    HighEnergyPhysics`Tarcer`TFISimplify, 
    HighEnergyPhysics`Tarcer`SEpsilon, 
    HighEnergyPhysics`Tarcer`TarcerExpand, 
    HighEnergyPhysics`Tarcer`TarcerReduce, 
    HighEnergyPhysics`Tarcer`TAI, 
    HighEnergyPhysics`Tarcer`TAR, 
    HighEnergyPhysics`Tarcer`TBI, 
    HighEnergyPhysics`Tarcer`TBR, 
    HighEnergyPhysics`Tarcer`TFI, 
    HighEnergyPhysics`Tarcer`TFR, 
    HighEnergyPhysics`Tarcer`TJI, 
    HighEnergyPhysics`Tarcer`TJR, 
    HighEnergyPhysics`Tarcer`TKI, 
    HighEnergyPhysics`Tarcer`TVI, 
    HighEnergyPhysics`Tarcer`TVR, 
    HighEnergyPhysics`Tarcer`$BasisIntegrals, 
    HighEnergyPhysics`Tarcer`$Comment, 
    HighEnergyPhysics`Tarcer`$CommentNotebook, 
    HighEnergyPhysics`Tarcer`$RankLimit, 
    HighEnergyPhysics`Tarcer`$TTable, 
    HighEnergyPhysics`Tarcer`Private`applytlrules, 
    HighEnergyPhysics`Tarcer`Private`applytlrules2,
    HighEnergyPhysics`Tarcer`Private`ExpandMaybe, 
    HighEnergyPhysics`Tarcer`Private`FactorC, 
    HighEnergyPhysics`Tarcer`Private`FEPrint, 
    HighEnergyPhysics`Tarcer`Private`fN, 
    HighEnergyPhysics`Tarcer`Private`mbt, 
    HighEnergyPhysics`Tarcer`Private`PNQ, 
    HighEnergyPhysics`Tarcer`Private`PQ, 
    HighEnergyPhysics`Tarcer`Private`ta, 
    HighEnergyPhysics`Tarcer`Private`TA, 
    HighEnergyPhysics`Tarcer`Private`TAIC, 
    HighEnergyPhysics`Tarcer`Private`TBIC, 
    HighEnergyPhysics`Tarcer`Private`TComment, 
    HighEnergyPhysics`Tarcer`Private`TLRComment, 
    HighEnergyPhysics`Tarcer`Private`TFIC, 
    HighEnergyPhysics`Tarcer`Private`TJIC, 
    HighEnergyPhysics`Tarcer`Private`TLR, 
    HighEnergyPhysics`Tarcer`Private`tlrule, 
    HighEnergyPhysics`Tarcer`Private`redblue, 
    HighEnergyPhysics`Tarcer`Private`TLRules, 
    HighEnergyPhysics`Tarcer`Private`TLRules2, 
    HighEnergyPhysics`Tarcer`Private`TVIC, 
    HighEnergyPhysics`Tarcer`Private`$TLRComment, 
    HighEnergyPhysics`FeynCalc`Zeta2`Zeta2}]; \
\>", "Input",
  Editable->True],

Cell[BoxData[
    \(\($HistoryLength = \[Infinity]; \)\)], "Input"],

Cell["\<\
This gives the absolute time (in minutes) to run this \
notebook.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Round[AbsoluteTime[] - HighEnergyPhysics`Tarcer`Private`atime]/60. \ min
      \)], "Input"],

Cell[BoxData[
    \(2.98333333333333339`\ min\)], "Output"]
}, Open  ]],

Cell["Display the size of the generated file:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(filename\  = \ 
      "\<tarcer\>"<>\((Apply[StringJoin, ToString/@$RankLimit])\)<>
        "\<.mx\>"\)], "Input"],

Cell[BoxData[
    \("tarcer06.mx"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FileByteCount[filename]\ Byte\)], "Input"],

Cell[BoxData[
    \(1649280\ Byte\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Directory[]\)], "Input"],

Cell[BoxData[
    \("/home/rolfm/tarcer"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(MaxMemoryUsed[]\)], "Input"],

Cell[BoxData[
    \(7366816\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(MemoryInUse[]\)], "Input"],

Cell[BoxData[
    \(3337168\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(TimeUsed[]\)], "Input"],

Cell[BoxData[
    \(130.949999999999988`\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\ \)], "Input"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"X 3.0",
ScreenRectangle->{{0, 1152}, {0, 864}},
WindowToolbars->{},
WindowSize->{845, 605},
WindowMargins->{{107, Automatic}, {Automatic, 75}},
CellLabelAutoDelete->True,
Magnification->1.5
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "T1.1"->{
    Cell[1731, 51, 73, 2, 150, "Title",
      CellTags->"T1.1"]},
  "T1.2"->{
    Cell[1987, 63, 434, 14, 146, "Text",
      CellTags->"T1.2"]},
  "T1.1.1"->{
    Cell[29641, 1053, 93, 2, 77, "Section",
      CellTags->"T1.1.1"]},
  "T1.2.1.1"->{
    Cell[33299, 1173, 78, 1, 60, "Subsubsection",
      CellTags->"T1.2.1.1"]},
  "T1.2.1"->{
    Cell[33941, 1193, 86, 2, 77, "Section",
      CellTags->"T1.2.1"]},
  "T1.3.1"->{
    Cell[40028, 1361, 112, 2, 77, "Section",
      CellTags->"T1.3.1"]},
  "T1.3.1.1"->{
    Cell[42105, 1435, 63, 1, 64, "Subsection",
      CellTags->"T1.3.1.1"]},
  "T1.3.2.1"->{
    Cell[65197, 1975, 58, 1, 64, "Subsection",
      CellTags->"T1.3.2.1"]},
  "T1.3.3.1"->{
    Cell[69647, 2056, 59, 1, 64, "Subsection",
      CellTags->"T1.3.3.1"]},
  "T1.3.4.1"->{
    Cell[73250, 2139, 60, 1, 64, "Subsection",
      CellTags->"T1.3.4.1"]},
  "T1.3.5.1"->{
    Cell[74032, 2162, 54, 1, 64, "Subsection",
      CellTags->"T1.3.5.1"]},
  "T1.3.6.1"->{
    Cell[78596, 2260, 59, 1, 64, "Subsection",
      CellTags->"T1.3.6.1"]},
  "T1.3.7.1"->{
    Cell[79154, 2276, 50, 1, 64, "Subsection",
      CellTags->"T1.3.7.1"]},
  "T1.3.7.1.1"->{
    Cell[79316, 2284, 57, 1, 60, "Subsubsection",
      CellTags->"T1.3.7.1.1"]},
  "T1.3.7.2.1"->{
    Cell[81874, 2375, 53, 1, 60, "Subsubsection",
      CellTags->"T1.3.7.2.1"]},
  "T1.3.7.3.1"->{
    Cell[82039, 2384, 54, 1, 60, "Subsubsection",
      CellTags->"T1.3.7.3.1"]},
  "T1.3.7.4.1"->{
    Cell[82261, 2396, 58, 1, 60, "Subsubsection",
      CellTags->"T1.3.7.4.1"]},
  "T1.3.7.5.1"->{
    Cell[82433, 2405, 54, 1, 60, "Subsubsection",
      CellTags->"T1.3.7.5.1"]},
  "T1.3.7.6.1"->{
    Cell[85279, 2496, 54, 1, 60, "Subsubsection",
      CellTags->"T1.3.7.6.1"]},
  "T1.3.7.7.1"->{
    Cell[85870, 2516, 54, 1, 60, "Subsubsection",
      CellTags->"T1.3.7.7.1"]},
  "T1.3.7.8.1"->{
    Cell[86607, 2541, 54, 1, 60, "Subsubsection",
      CellTags->"T1.3.7.8.1"]},
  "T1.4.1"->{
    Cell[86940, 2557, 104, 2, 77, "Section",
      CellTags->"T1.4.1"]},
  "T1.4.1.1"->{
    Cell[88973, 2609, 116, 3, 69, "Subsection",
      CellTags->"T1.4.1.1"]},
  "T1.4.2.1"->{
    Cell[89417, 2628, 114, 3, 69, "Subsection",
      CellTags->"T1.4.2.1"]},
  "T1.4.3.1"->{
    Cell[91639, 2697, 205, 6, 69, "Subsection",
      CellTags->"T1.4.3.1"]},
  "T1.4.4.1"->{
    Cell[96378, 2821, 223, 6, 91, "Subsection",
      CellTags->"T1.4.4.1"]},
  "T1.5.1"->{
    Cell[108760, 3142, 97, 2, 77, "Section",
      CellTags->"T1.5.1"]},
  "T1.5.1.1"->{
    Cell[108882, 3148, 65, 1, 64, "Subsection",
      CellTags->"T1.5.1.1"]},
  "T1.5.1.1.1"->{
    Cell[169627, 4314, 62, 1, 60, "Subsubsection",
      CellTags->"T1.5.1.1.1"]},
  "T1.5.2.1"->{
    Cell[173011, 4399, 59, 1, 64, "Subsection",
      CellTags->"T1.5.2.1"]},
  "T1.5.2.1.1"->{
    Cell[174395, 4430, 53, 1, 60, "Subsubsection",
      CellTags->"T1.5.2.1.1"]},
  "T1.5.3.1"->{
    Cell[180120, 4574, 60, 1, 64, "Subsection",
      CellTags->"T1.5.3.1"]},
  "T1.5.3.1.1"->{
    Cell[180526, 4590, 68, 1, 60, "Subsubsection",
      CellTags->"T1.5.3.1.1"]},
  "T1.5.3.2.1"->{
    Cell[183499, 4653, 68, 1, 60, "Subsubsection",
      CellTags->"T1.5.3.2.1"]},
  "T1.6.1"->{
    Cell[185956, 4712, 140, 2, 77, "Section",
      CellTags->"T1.6.1"]},
  "T1.6.1.1"->{
    Cell[186396, 4727, 56, 1, 64, "Subsection",
      CellTags->"T1.6.1.1"]},
  "T1.6.2.1"->{
    Cell[193999, 4914, 58, 1, 64, "Subsection",
      CellTags->"T1.6.2.1"]},
  "T1.6.2.1.1"->{
    Cell[194166, 4922, 139, 5, 60, "Subsubsection",
      CellTags->"T1.6.2.1.1"]},
  "T1.6.2.2.1"->{
    Cell[195427, 4962, 139, 5, 60, "Subsubsection",
      CellTags->"T1.6.2.2.1"]},
  "T1.6.2.3.1"->{
    Cell[196769, 5004, 139, 5, 60, "Subsubsection",
      CellTags->"T1.6.2.3.1"]},
  "T1.6.2.4.1"->{
    Cell[198111, 5046, 139, 5, 60, "Subsubsection",
      CellTags->"T1.6.2.4.1"]},
  "T1.6.3.1"->{
    Cell[199465, 5089, 58, 1, 64, "Subsection",
      CellTags->"T1.6.3.1"]},
  "T1.6.4.1"->{
    Cell[203258, 5196, 159, 5, 66, "Subsection",
      CellTags->"T1.6.4.1"]},
  "T1.6.5.1"->{
    Cell[203903, 5220, 58, 1, 64, "Subsection",
      CellTags->"T1.6.5.1"]},
  "T1.6.6.1"->{
    Cell[207696, 5327, 159, 5, 66, "Subsection",
      CellTags->"T1.6.6.1"]},
  "T1.6.7.1"->{
    Cell[208341, 5351, 159, 5, 66, "Subsection",
      CellTags->"T1.6.7.1"]},
  "T1.6.8.1"->{
    Cell[209128, 5380, 159, 5, 66, "Subsection",
      CellTags->"T1.6.8.1"]},
  "T1.6.9.1"->{
    Cell[209915, 5409, 56, 1, 64, "Subsection",
      CellTags->"T1.6.9.1"]},
  "T1.6.10.1"->{
    Cell[216764, 5578, 59, 1, 64, "Subsection",
      CellTags->"T1.6.10.1"]},
  "T1.6.11.1"->{
    Cell[218023, 5617, 57, 1, 64, "Subsection",
      CellTags->"T1.6.11.1"]},
  "T1.6.12.1"->{
    Cell[222326, 5745, 57, 1, 64, "Subsection",
      CellTags->"T1.6.12.1"]},
  "T1.6.13.1"->{
    Cell[226629, 5873, 57, 1, 64, "Subsection",
      CellTags->"T1.6.13.1"]},
  "T1.6.14.1"->{
    Cell[234104, 6054, 68, 1, 64, "Subsection",
      CellTags->"T1.6.14.1"]},
  "T1.6.15.1"->{
    Cell[234517, 6070, 59, 1, 64, "Subsection",
      CellTags->"T1.6.15.1"]},
  "T1.6.16.1"->{
    Cell[257271, 6614, 57, 1, 64, "Subsection",
      CellTags->"T1.6.16.1"]},
  "T1.6.17.1"->{
    Cell[261304, 6727, 59, 1, 64, "Subsection",
      CellTags->"T1.6.17.1"]},
  "T1.6.18.1"->{
    Cell[262058, 6760, 57, 1, 64, "Subsection",
      CellTags->"T1.6.18.1"]},
  "T1.6.19.1"->{
    Cell[267570, 6908, 59, 1, 64, "Subsection",
      CellTags->"T1.6.19.1"]},
  "T1.6.20.1"->{
    Cell[268301, 6939, 59, 1, 64, "Subsection",
      CellTags->"T1.6.20.1"]},
  "T1.6.21.1"->{
    Cell[269066, 6971, 57, 1, 64, "Subsection",
      CellTags->"T1.6.21.1"]},
  "T1.6.22.1"->{
    Cell[272487, 7071, 59, 1, 64, "Subsection",
      CellTags->"T1.6.22.1"]},
  "T1.6.23.1"->{
    Cell[273172, 7100, 57, 1, 64, "Subsection",
      CellTags->"T1.6.23.1"]},
  "T1.6.24.1"->{
    Cell[277306, 7217, 59, 1, 64, "Subsection",
      CellTags->"T1.6.24.1"]},
  "T1.6.25.1"->{
    Cell[278015, 7247, 57, 1, 64, "Subsection",
      CellTags->"T1.6.25.1"]},
  "T1.6.26.1"->{
    Cell[281702, 7353, 59, 1, 64, "Subsection",
      CellTags->"T1.6.26.1"]},
  "T1.6.27.1"->{
    Cell[282413, 7383, 57, 1, 64, "Subsection",
      CellTags->"T1.6.27.1"]},
  "T1.6.28.1"->{
    Cell[287734, 7527, 59, 1, 64, "Subsection",
      CellTags->"T1.6.28.1"]},
  "T1.6.29.1"->{
    Cell[288738, 7557, 58, 1, 64, "Subsection",
      CellTags->"T1.6.29.1"]},
  "T1.6.30.1"->{
    Cell[292160, 7657, 59, 1, 64, "Subsection",
      CellTags->"T1.6.30.1"]},
  "T1.6.31.1"->{
    Cell[292864, 7686, 58, 1, 64, "Subsection",
      CellTags->"T1.6.31.1"]},
  "T1.6.32.1"->{
    Cell[298087, 7849, 59, 1, 64, "Subsection",
      CellTags->"T1.6.32.1"]},
  "T1.6.33.1"->{
    Cell[299235, 7896, 61, 1, 64, "Subsection",
      CellTags->"T1.6.33.1"]},
  "T1.6.34.1"->{
    Cell[300429, 7943, 58, 1, 64, "Subsection",
      CellTags->"T1.6.34.1"]},
  "T1.6.35.1"->{
    Cell[306052, 8100, 59, 1, 64, "Subsection",
      CellTags->"T1.6.35.1"]},
  "T1.6.36.1"->{
    Cell[307289, 8140, 58, 1, 64, "Subsection",
      CellTags->"T1.6.36.1"]},
  "T1.6.37.1"->{
    Cell[314014, 8307, 59, 1, 64, "Subsection",
      CellTags->"T1.6.37.1"]},
  "T1.6.38.1"->{
    Cell[315807, 8375, 61, 1, 64, "Subsection",
      CellTags->"T1.6.38.1"]},
  "T1.6.39.1"->{
    Cell[317628, 8434, 61, 1, 64, "Subsection",
      CellTags->"T1.6.39.1"]},
  "T1.6.40.1"->{
    Cell[318722, 8472, 61, 1, 64, "Subsection",
      CellTags->"T1.6.40.1"]},
  "T1.6.41.1"->{
    Cell[319578, 8502, 58, 1, 64, "Subsection",
      CellTags->"T1.6.41.1"]},
  "T1.6.42.1"->{
    Cell[324671, 8641, 59, 1, 64, "Subsection",
      CellTags->"T1.6.42.1"]},
  "T1.6.43.1"->{
    Cell[325520, 8674, 68, 1, 64, "Subsection",
      CellTags->"T1.6.43.1"]},
  "T1.6.44.1"->{
    Cell[331280, 8834, 57, 1, 64, "Subsection",
      CellTags->"T1.6.44.1"]},
  "T1.6.45.1"->{
    Cell[345936, 9243, 59, 1, 64, "Subsection",
      CellTags->"T1.6.45.1"]},
  "T1.6.46.1"->{
    Cell[346743, 9276, 61, 1, 64, "Subsection",
      CellTags->"T1.6.46.1"]},
  "T1.6.47.1"->{
    Cell[347598, 9309, 57, 1, 64, "Subsection",
      CellTags->"T1.6.47.1"]},
  "T1.6.48.1"->{
    Cell[353620, 9465, 59, 1, 64, "Subsection",
      CellTags->"T1.6.48.1"]},
  "T1.6.49.1"->{
    Cell[354439, 9498, 61, 1, 64, "Subsection",
      CellTags->"T1.6.49.1"]},
  "T1.6.50.1"->{
    Cell[355310, 9531, 57, 1, 64, "Subsection",
      CellTags->"T1.6.50.1"]},
  "T1.6.51.1"->{
    Cell[367857, 9840, 59, 1, 64, "Subsection",
      CellTags->"T1.6.51.1"]},
  "T1.6.52.1"->{
    Cell[368538, 9867, 57, 1, 64, "Subsection",
      CellTags->"T1.6.52.1"]},
  "T1.6.53.1"->{
    Cell[373938, 10014, 59, 1, 64, "Subsection",
      CellTags->"T1.6.53.1"]},
  "T1.6.54.1"->{
    Cell[374454, 10039, 61, 1, 64, "Subsection",
      CellTags->"T1.6.54.1"]},
  "T1.6.55.1"->{
    Cell[375005, 10064, 61, 1, 64, "Subsection",
      CellTags->"T1.6.55.1"]},
  "T1.6.56.1"->{
    Cell[375556, 10089, 57, 1, 64, "Subsection",
      CellTags->"T1.6.56.1"]},
  "T1.6.57.1"->{
    Cell[381431, 10250, 59, 1, 64, "Subsection",
      CellTags->"T1.6.57.1"]},
  "T1.6.58.1"->{
    Cell[381899, 10273, 57, 1, 64, "Subsection",
      CellTags->"T1.6.58.1"]},
  "T1.6.59.1"->{
    Cell[386534, 10399, 59, 1, 64, "Subsection",
      CellTags->"T1.6.59.1"]},
  "T1.6.60.1"->{
    Cell[387020, 10424, 57, 1, 64, "Subsection",
      CellTags->"T1.6.60.1"]},
  "T1.6.61.1"->{
    Cell[392135, 10563, 59, 1, 64, "Subsection",
      CellTags->"T1.6.61.1"]},
  "T1.6.62.1"->{
    Cell[392714, 10589, 61, 1, 64, "Subsection",
      CellTags->"T1.6.62.1"]},
  "T1.6.63.1"->{
    Cell[393337, 10616, 61, 1, 64, "Subsection",
      CellTags->"T1.6.63.1"]},
  "T1.6.64.1"->{
    Cell[393977, 10643, 57, 1, 64, "Subsection",
      CellTags->"T1.6.64.1"]},
  "T1.6.65.1"->{
    Cell[404503, 10905, 59, 1, 64, "Subsection",
      CellTags->"T1.6.65.1"]},
  "T1.6.66.1"->{
    Cell[405042, 10931, 61, 1, 64, "Subsection",
      CellTags->"T1.6.66.1"]},
  "T1.6.67.1"->{
    Cell[405622, 10957, 61, 1, 64, "Subsection",
      CellTags->"T1.6.67.1"]},
  "T1.6.68.1"->{
    Cell[406202, 10983, 57, 1, 64, "Subsection",
      CellTags->"T1.6.68.1"]},
  "T1.6.69.1"->{
    Cell[413104, 11158, 59, 1, 64, "Subsection",
      CellTags->"T1.6.69.1"]},
  "T1.6.70.1"->{
    Cell[413636, 11184, 57, 1, 64, "Subsection",
      CellTags->"T1.6.70.1"]},
  "T1.6.70.1.1"->{
    Cell[413718, 11189, 56, 1, 60, "Subsubsection",
      CellTags->"T1.6.70.1.1"]},
  "T1.6.70.2.1"->{
    Cell[415518, 11245, 56, 1, 60, "Subsubsection",
      CellTags->"T1.6.70.2.1"]},
  "T1.6.71.1"->{
    Cell[422004, 11427, 59, 1, 64, "Subsection",
      CellTags->"T1.6.71.1"]},
  "T1.6.72.1"->{
    Cell[422353, 11444, 57, 1, 64, "Subsection",
      CellTags->"T1.6.72.1"]},
  "T1.6.73.1"->{
    Cell[431651, 11668, 59, 1, 64, "Subsection",
      CellTags->"T1.6.73.1"]},
  "T1.6.74.1"->{
    Cell[431997, 11684, 57, 1, 64, "Subsection",
      CellTags->"T1.6.74.1"]},
  "T1.6.75.1"->{
    Cell[443477, 12006, 60, 1, 64, "Subsection",
      CellTags->"T1.6.75.1"]},
  "T1.6.76.1"->{
    Cell[443859, 12023, 60, 1, 64, "Subsection",
      CellTags->"T1.6.76.1"]},
  "T1.6.77.1"->{
    Cell[444309, 12042, 60, 1, 64, "Subsection",
      CellTags->"T1.6.77.1"]},
  "T1.6.78.1"->{
    Cell[444700, 12059, 60, 1, 64, "Subsection",
      CellTags->"T1.6.78.1"]},
  "T1.6.79.1"->{
    Cell[445122, 12076, 57, 1, 64, "Subsection",
      CellTags->"T1.6.79.1"]},
  "T1.6.80.1"->{
    Cell[450400, 12213, 57, 1, 64, "Subsection",
      CellTags->"T1.6.80.1"]},
  "T1.6.81.1"->{
    Cell[452903, 12288, 59, 1, 64, "Subsection",
      CellTags->"T1.6.81.1"]},
  "T1.6.82.1"->{
    Cell[453268, 12304, 57, 1, 64, "Subsection",
      CellTags->"T1.6.82.1"]},
  "T1.6.83.1"->{
    Cell[456528, 12397, 59, 1, 64, "Subsection",
      CellTags->"T1.6.83.1"]},
  "T1.6.84.1"->{
    Cell[456844, 12411, 61, 1, 64, "Subsection",
      CellTags->"T1.6.84.1"]},
  "T1.6.85.1"->{
    Cell[457348, 12431, 61, 1, 64, "Subsection",
      CellTags->"T1.6.85.1"]},
  "T1.6.86.1"->{
    Cell[457899, 12452, 57, 1, 64, "Subsection",
      CellTags->"T1.6.86.1"]},
  "T1.6.87.1"->{
    Cell[460245, 12528, 59, 1, 64, "Subsection",
      CellTags->"T1.6.87.1"]},
  "T1.6.88.1"->{
    Cell[460572, 12543, 57, 1, 64, "Subsection",
      CellTags->"T1.6.88.1"]},
  "T1.6.89.1"->{
    Cell[462902, 12616, 59, 1, 64, "Subsection",
      CellTags->"T1.6.89.1"]},
  "T1.6.90.1"->{
    Cell[463191, 12630, 57, 1, 64, "Subsection",
      CellTags->"T1.6.90.1"]},
  "T1.6.91.1"->{
    Cell[464719, 12683, 59, 1, 64, "Subsection",
      CellTags->"T1.6.91.1"]},
  "T1.6.92.1"->{
    Cell[465023, 12697, 57, 1, 64, "Subsection",
      CellTags->"T1.6.92.1"]},
  "T1.6.93.1"->{
    Cell[466409, 12746, 59, 1, 64, "Subsection",
      CellTags->"T1.6.93.1"]},
  "T1.6.94.1"->{
    Cell[466687, 12760, 57, 1, 64, "Subsection",
      CellTags->"T1.6.94.1"]},
  "T1.6.95.1"->{
    Cell[470653, 12879, 59, 1, 64, "Subsection",
      CellTags->"T1.6.95.1"]},
  "T1.6.96.1"->{
    Cell[470966, 12893, 61, 1, 64, "Subsection",
      CellTags->"T1.6.96.1"]},
  "T1.6.97.1"->{
    Cell[471341, 12909, 61, 1, 64, "Subsection",
      CellTags->"T1.6.97.1"]},
  "T1.6.98.1"->{
    Cell[471716, 12925, 58, 1, 64, "Subsection",
      CellTags->"T1.6.98.1"]},
  "T1.6.99.1"->{
    Cell[477885, 13103, 60, 1, 64, "Subsection",
      CellTags->"T1.6.99.1"]},
  "T1.6.100.1"->{
    Cell[478326, 13126, 62, 1, 64, "Subsection",
      CellTags->"T1.6.100.1"]},
  "T1.6.101.1"->{
    Cell[478810, 13149, 62, 1, 64, "Subsection",
      CellTags->"T1.6.101.1"]},
  "T1.6.102.1"->{
    Cell[479327, 13174, 62, 1, 64, "Subsection",
      CellTags->"T1.6.102.1"]},
  "T1.6.103.1"->{
    Cell[479811, 13197, 62, 1, 64, "Subsection",
      CellTags->"T1.6.103.1"]},
  "T1.6.104.1"->{
    Cell[480295, 13220, 62, 1, 64, "Subsection",
      CellTags->"T1.6.104.1"]},
  "T1.6.105.1"->{
    Cell[480812, 13245, 59, 1, 64, "Subsection",
      CellTags->"T1.6.105.1"]},
  "T1.6.106.1"->{
    Cell[481561, 13269, 61, 1, 64, "Subsection",
      CellTags->"T1.6.106.1"]},
  "T1.6.107.1"->{
    Cell[481783, 13281, 59, 1, 64, "Subsection",
      CellTags->"T1.6.107.1"]},
  "T1.6.108.1"->{
    Cell[482663, 13310, 61, 1, 64, "Subsection",
      CellTags->"T1.6.108.1"]},
  "T1.6.109.1"->{
    Cell[482929, 13324, 59, 1, 64, "Subsection",
      CellTags->"T1.6.109.1"]},
  "T1.6.110.1"->{
    Cell[483643, 13347, 61, 1, 64, "Subsection",
      CellTags->"T1.6.110.1"]},
  "T1.6.111.1"->{
    Cell[483865, 13359, 59, 1, 64, "Subsection",
      CellTags->"T1.6.111.1"]},
  "T1.6.112.1"->{
    Cell[484319, 13376, 61, 1, 64, "Subsection",
      CellTags->"T1.6.112.1"]},
  "T1.6.113.1"->{
    Cell[484541, 13388, 59, 1, 64, "Subsection",
      CellTags->"T1.6.113.1"]},
  "T1.6.114.1"->{
    Cell[491697, 13577, 61, 1, 64, "Subsection",
      CellTags->"T1.6.114.1"]},
  "T1.6.115.1"->{
    Cell[492299, 13598, 61, 1, 64, "Subsection",
      CellTags->"T1.6.115.1"]},
  "T1.6.116.1"->{
    Cell[493092, 13635, 61, 1, 64, "Subsection",
      CellTags->"T1.6.116.1"]},
  "T1.6.117.1"->{
    Cell[493889, 13672, 62, 1, 64, "Subsection",
      CellTags->"T1.6.117.1"]},
  "T1.6.118.1"->{
    Cell[495028, 13700, 62, 1, 64, "Subsection",
      CellTags->"T1.6.118.1"]},
  "T1.6.119.1"->{
    Cell[496113, 13726, 62, 1, 64, "Subsection",
      CellTags->"T1.6.119.1"]},
  "T1.6.120.1"->{
    Cell[497196, 13752, 61, 1, 64, "Subsection",
      CellTags->"T1.6.120.1"]},
  "Tsymmetry"->{
    Cell[503682, 13894, 55, 1, 64, "Subsection",
      CellTags->"Tsymmetry"]},
  "specialTVR"->{
    Cell[503885, 13905, 70, 1, 64, "Subsection",
      CellTags->"specialTVR"]},
  "T1.6.121.1"->{
    Cell[514674, 14093, 60, 1, 64, "Subsection",
      CellTags->"T1.6.121.1"]},
  "T1.6.122.1"->{
    Cell[516729, 14152, 61, 1, 64, "Subsection",
      CellTags->"T1.6.122.1"]},
  "T1.6.123.1"->{
    Cell[517809, 14184, 60, 1, 64, "Subsection",
      CellTags->"T1.6.123.1"]},
  "T1.6.124.1"->{
    Cell[518464, 14205, 58, 1, 64, "Subsection",
      CellTags->"T1.6.124.1"]},
  "T1.7.1"->{
    Cell[519119, 14229, 108, 2, 77, "Section",
      CellTags->"T1.7.1"]},
  "T1.7.1.1"->{
    Cell[519252, 14235, 70, 1, 64, "Subsection",
      CellTags->"T1.7.1.1"]},
  "TLRdef"->{
    Cell[521099, 14298, 216, 7, 146, "Text",
      CellTags->"TLRdef"]},
  "tlrule17"->{
    Cell[531525, 14588, 78, 1, 46, "Text",
      CellTags->"tlrule17"]},
  "T1.7.2.1"->{
    Cell[558101, 15301, 50, 1, 64, "Subsection",
      CellTags->"T1.7.2.1"]},
  "T1.7.3.1"->{
    Cell[563488, 15426, 54, 1, 64, "Subsection",
      CellTags->"T1.7.3.1"]},
  "$TTable"->{
    Cell[564960, 15460, 52, 1, 64, "Subsection",
      CellTags->"$TTable"]},
  "T1.7.4.1"->{
    Cell[565103, 15469, 59, 1, 64, "Subsection",
      CellTags->"T1.7.4.1"]},
  "T1.8.1"->{
    Cell[574860, 15666, 92, 2, 77, "Section",
      CellTags->"T1.8.1"]},
  "T1.9.1"->{
    Cell[583543, 15848, 93, 2, 77, "Section",
      CellTags->"T1.9.1"]},
  "T1.9.1.1"->{
    Cell[583661, 15854, 54, 1, 64, "Subsection",
      CellTags->"T1.9.1.1"]},
  "T1.10.1"->{
    Cell[588013, 15992, 95, 2, 77, "Section",
      CellTags->"T1.10.1"]},
  "T1.11.1"->{
    Cell[602084, 16385, 101, 2, 77, "Section",
      CellTags->"T1.11.1"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"T1.1", 606592, 16549},
  {"T1.2", 606670, 16552},
  {"T1.1.1", 606751, 16555},
  {"T1.2.1.1", 606839, 16558},
  {"T1.2.1", 606933, 16561},
  {"T1.3.1", 607019, 16564},
  {"T1.3.1.1", 607108, 16567},
  {"T1.3.2.1", 607201, 16570},
  {"T1.3.3.1", 607294, 16573},
  {"T1.3.4.1", 607387, 16576},
  {"T1.3.5.1", 607480, 16579},
  {"T1.3.6.1", 607573, 16582},
  {"T1.3.7.1", 607666, 16585},
  {"T1.3.7.1.1", 607761, 16588},
  {"T1.3.7.2.1", 607861, 16591},
  {"T1.3.7.3.1", 607961, 16594},
  {"T1.3.7.4.1", 608061, 16597},
  {"T1.3.7.5.1", 608161, 16600},
  {"T1.3.7.6.1", 608261, 16603},
  {"T1.3.7.7.1", 608361, 16606},
  {"T1.3.7.8.1", 608461, 16609},
  {"T1.4.1", 608557, 16612},
  {"T1.4.1.1", 608646, 16615},
  {"T1.4.2.1", 608740, 16618},
  {"T1.4.3.1", 608834, 16621},
  {"T1.4.4.1", 608928, 16624},
  {"T1.5.1", 609020, 16627},
  {"T1.5.1.1", 609109, 16630},
  {"T1.5.1.1.1", 609205, 16633},
  {"T1.5.2.1", 609304, 16636},
  {"T1.5.2.1.1", 609400, 16639},
  {"T1.5.3.1", 609499, 16642},
  {"T1.5.3.1.1", 609595, 16645},
  {"T1.5.3.2.1", 609696, 16648},
  {"T1.6.1", 609793, 16651},
  {"T1.6.1.1", 609883, 16654},
  {"T1.6.2.1", 609977, 16657},
  {"T1.6.2.1.1", 610073, 16660},
  {"T1.6.2.2.1", 610175, 16663},
  {"T1.6.2.3.1", 610277, 16666},
  {"T1.6.2.4.1", 610379, 16669},
  {"T1.6.3.1", 610479, 16672},
  {"T1.6.4.1", 610573, 16675},
  {"T1.6.5.1", 610668, 16678},
  {"T1.6.6.1", 610762, 16681},
  {"T1.6.7.1", 610857, 16684},
  {"T1.6.8.1", 610952, 16687},
  {"T1.6.9.1", 611047, 16690},
  {"T1.6.10.1", 611142, 16693},
  {"T1.6.11.1", 611238, 16696},
  {"T1.6.12.1", 611334, 16699},
  {"T1.6.13.1", 611430, 16702},
  {"T1.6.14.1", 611526, 16705},
  {"T1.6.15.1", 611622, 16708},
  {"T1.6.16.1", 611718, 16711},
  {"T1.6.17.1", 611814, 16714},
  {"T1.6.18.1", 611910, 16717},
  {"T1.6.19.1", 612006, 16720},
  {"T1.6.20.1", 612102, 16723},
  {"T1.6.21.1", 612198, 16726},
  {"T1.6.22.1", 612294, 16729},
  {"T1.6.23.1", 612390, 16732},
  {"T1.6.24.1", 612486, 16735},
  {"T1.6.25.1", 612582, 16738},
  {"T1.6.26.1", 612678, 16741},
  {"T1.6.27.1", 612774, 16744},
  {"T1.6.28.1", 612870, 16747},
  {"T1.6.29.1", 612966, 16750},
  {"T1.6.30.1", 613062, 16753},
  {"T1.6.31.1", 613158, 16756},
  {"T1.6.32.1", 613254, 16759},
  {"T1.6.33.1", 613350, 16762},
  {"T1.6.34.1", 613446, 16765},
  {"T1.6.35.1", 613542, 16768},
  {"T1.6.36.1", 613638, 16771},
  {"T1.6.37.1", 613734, 16774},
  {"T1.6.38.1", 613830, 16777},
  {"T1.6.39.1", 613926, 16780},
  {"T1.6.40.1", 614022, 16783},
  {"T1.6.41.1", 614118, 16786},
  {"T1.6.42.1", 614214, 16789},
  {"T1.6.43.1", 614310, 16792},
  {"T1.6.44.1", 614406, 16795},
  {"T1.6.45.1", 614502, 16798},
  {"T1.6.46.1", 614598, 16801},
  {"T1.6.47.1", 614694, 16804},
  {"T1.6.48.1", 614790, 16807},
  {"T1.6.49.1", 614886, 16810},
  {"T1.6.50.1", 614982, 16813},
  {"T1.6.51.1", 615078, 16816},
  {"T1.6.52.1", 615174, 16819},
  {"T1.6.53.1", 615270, 16822},
  {"T1.6.54.1", 615367, 16825},
  {"T1.6.55.1", 615464, 16828},
  {"T1.6.56.1", 615561, 16831},
  {"T1.6.57.1", 615658, 16834},
  {"T1.6.58.1", 615755, 16837},
  {"T1.6.59.1", 615852, 16840},
  {"T1.6.60.1", 615949, 16843},
  {"T1.6.61.1", 616046, 16846},
  {"T1.6.62.1", 616143, 16849},
  {"T1.6.63.1", 616240, 16852},
  {"T1.6.64.1", 616337, 16855},
  {"T1.6.65.1", 616434, 16858},
  {"T1.6.66.1", 616531, 16861},
  {"T1.6.67.1", 616628, 16864},
  {"T1.6.68.1", 616725, 16867},
  {"T1.6.69.1", 616822, 16870},
  {"T1.6.70.1", 616919, 16873},
  {"T1.6.70.1.1", 617018, 16876},
  {"T1.6.70.2.1", 617122, 16879},
  {"T1.6.71.1", 617224, 16882},
  {"T1.6.72.1", 617321, 16885},
  {"T1.6.73.1", 617418, 16888},
  {"T1.6.74.1", 617515, 16891},
  {"T1.6.75.1", 617612, 16894},
  {"T1.6.76.1", 617709, 16897},
  {"T1.6.77.1", 617806, 16900},
  {"T1.6.78.1", 617903, 16903},
  {"T1.6.79.1", 618000, 16906},
  {"T1.6.80.1", 618097, 16909},
  {"T1.6.81.1", 618194, 16912},
  {"T1.6.82.1", 618291, 16915},
  {"T1.6.83.1", 618388, 16918},
  {"T1.6.84.1", 618485, 16921},
  {"T1.6.85.1", 618582, 16924},
  {"T1.6.86.1", 618679, 16927},
  {"T1.6.87.1", 618776, 16930},
  {"T1.6.88.1", 618873, 16933},
  {"T1.6.89.1", 618970, 16936},
  {"T1.6.90.1", 619067, 16939},
  {"T1.6.91.1", 619164, 16942},
  {"T1.6.92.1", 619261, 16945},
  {"T1.6.93.1", 619358, 16948},
  {"T1.6.94.1", 619455, 16951},
  {"T1.6.95.1", 619552, 16954},
  {"T1.6.96.1", 619649, 16957},
  {"T1.6.97.1", 619746, 16960},
  {"T1.6.98.1", 619843, 16963},
  {"T1.6.99.1", 619940, 16966},
  {"T1.6.100.1", 620038, 16969},
  {"T1.6.101.1", 620137, 16972},
  {"T1.6.102.1", 620236, 16975},
  {"T1.6.103.1", 620335, 16978},
  {"T1.6.104.1", 620434, 16981},
  {"T1.6.105.1", 620533, 16984},
  {"T1.6.106.1", 620632, 16987},
  {"T1.6.107.1", 620731, 16990},
  {"T1.6.108.1", 620830, 16993},
  {"T1.6.109.1", 620929, 16996},
  {"T1.6.110.1", 621028, 16999},
  {"T1.6.111.1", 621127, 17002},
  {"T1.6.112.1", 621226, 17005},
  {"T1.6.113.1", 621325, 17008},
  {"T1.6.114.1", 621424, 17011},
  {"T1.6.115.1", 621523, 17014},
  {"T1.6.116.1", 621622, 17017},
  {"T1.6.117.1", 621721, 17020},
  {"T1.6.118.1", 621820, 17023},
  {"T1.6.119.1", 621919, 17026},
  {"T1.6.120.1", 622018, 17029},
  {"Tsymmetry", 622116, 17032},
  {"specialTVR", 622214, 17035},
  {"T1.6.121.1", 622313, 17038},
  {"T1.6.122.1", 622412, 17041},
  {"T1.6.123.1", 622511, 17044},
  {"T1.6.124.1", 622610, 17047},
  {"T1.7.1", 622705, 17050},
  {"T1.7.1.1", 622796, 17053},
  {"TLRdef", 622889, 17056},
  {"tlrule17", 622978, 17059},
  {"T1.7.2.1", 623067, 17062},
  {"T1.7.3.1", 623162, 17065},
  {"$TTable", 623256, 17068},
  {"T1.7.4.1", 623350, 17071},
  {"T1.8.1", 623443, 17074},
  {"T1.9.1", 623531, 17077},
  {"T1.9.1.1", 623621, 17080},
  {"T1.10.1", 623715, 17083},
  {"T1.11.1", 623805, 17086}
  }
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 73, 2, 150, "Title",
  CellTags->"T1.1"],
Cell[1807, 55, 177, 6, 46, "Text"],
Cell[1987, 63, 434, 14, 146, "Text",
  CellTags->"T1.2"],
Cell[2424, 79, 55, 1, 46, "Text"],
Cell[2482, 82, 186, 6, 46, "Text"],
Cell[2671, 90, 158, 6, 46, "Text"],
Cell[2832, 98, 171, 6, 46, "Text"],
Cell[3006, 106, 162, 6, 46, "Text"],
Cell[3171, 114, 162, 6, 46, "Text"],
Cell[3336, 122, 162, 6, 46, "Text"],
Cell[3501, 130, 167, 6, 46, "Text"],
Cell[3671, 138, 156, 3, 71, "Text"],
Cell[3830, 143, 1081, 21, 421, "Text"],
Cell[4914, 166, 147, 3, 71, "Text"],
Cell[5064, 171, 499, 10, 171, "Text"],
Cell[5566, 183, 94, 3, 46, "Text"],
Cell[5663, 188, 176, 4, 71, "Text"],
Cell[5842, 194, 228, 6, 71, "Text"],
Cell[6073, 202, 56, 1, 39, "Input"],
Cell[6132, 205, 180, 4, 71, "Text"],
Cell[6315, 211, 212, 4, 71, "Text"],
Cell[6530, 217, 262, 8, 46, "Text"],

Cell[CellGroupData[{
Cell[6817, 229, 75, 1, 77, "Section"],
Cell[6895, 232, 102, 2, 46, "Text"],
Cell[7000, 236, 95, 2, 46, "Text"],
Cell[7098, 240, 120, 2, 46, "Text"],
Cell[7221, 244, 121, 2, 46, "Text"],
Cell[7345, 248, 108, 2, 46, "Text"],
Cell[7456, 252, 103, 2, 46, "Text"],
Cell[7562, 256, 104, 2, 46, "Text"],
Cell[7669, 260, 105, 2, 46, "Text"],
Cell[7777, 264, 99, 2, 46, "Text"],
Cell[7879, 268, 104, 2, 46, "Text"],
Cell[7986, 272, 95, 2, 46, "Text"],
Cell[8084, 276, 104, 2, 40, "SmallText"],
Cell[8191, 280, 100, 2, 40, "SmallText"],
Cell[8294, 284, 101, 2, 40, "SmallText"],
Cell[8398, 288, 105, 2, 40, "SmallText"],
Cell[8506, 292, 101, 2, 40, "SmallText"],
Cell[8610, 296, 101, 2, 40, "SmallText"],
Cell[8714, 300, 101, 2, 40, "SmallText"],
Cell[8818, 304, 101, 2, 40, "SmallText"],
Cell[8922, 308, 113, 2, 46, "Text"],
Cell[9038, 312, 143, 3, 46, "Text"],
Cell[9184, 317, 141, 3, 46, "Text"],
Cell[9328, 322, 530, 15, 48, "Text"],
Cell[9861, 339, 547, 15, 48, "Text"],
Cell[10411, 356, 106, 2, 46, "Text"],
Cell[10520, 360, 110, 2, 46, "Text"],
Cell[10633, 364, 109, 2, 40, "SmallText"],
Cell[10745, 368, 189, 7, 46, "Text"],
Cell[10937, 377, 100, 2, 40, "SmallText"],
Cell[11040, 381, 105, 2, 46, "Text"],
Cell[11148, 385, 115, 2, 40, "SmallText"],
Cell[11266, 389, 115, 2, 40, "SmallText"],
Cell[11384, 393, 151, 3, 46, "Text"],
Cell[11538, 398, 101, 2, 46, "Text"],
Cell[11642, 402, 103, 2, 46, "Text"],
Cell[11748, 406, 485, 13, 44, "SmallText"],
Cell[12236, 421, 485, 13, 44, "SmallText"],
Cell[12724, 436, 485, 13, 44, "SmallText"],
Cell[13212, 451, 485, 13, 44, "SmallText"],
Cell[13700, 466, 103, 2, 46, "Text"],
Cell[13806, 470, 523, 15, 53, "Text"],
Cell[14332, 487, 103, 2, 46, "Text"],
Cell[14438, 491, 523, 15, 53, "Text"],
Cell[14964, 508, 523, 15, 53, "Text"],
Cell[15490, 525, 523, 15, 53, "Text"],
Cell[16016, 542, 101, 2, 46, "Text"],
Cell[16120, 546, 104, 2, 46, "Text"],
Cell[16227, 550, 102, 2, 46, "Text"],
Cell[16332, 554, 102, 2, 46, "Text"],
Cell[16437, 558, 102, 2, 46, "Text"],
Cell[16542, 562, 113, 2, 46, "Text"],
Cell[16658, 566, 104, 2, 46, "Text"],
Cell[16765, 570, 102, 2, 46, "Text"],
Cell[16870, 574, 104, 2, 46, "Text"],
Cell[16977, 578, 102, 2, 46, "Text"],
Cell[17082, 582, 104, 2, 46, "Text"],
Cell[17189, 586, 104, 2, 46, "Text"],
Cell[17296, 590, 102, 2, 46, "Text"],
Cell[17401, 594, 104, 2, 46, "Text"],
Cell[17508, 598, 102, 2, 46, "Text"],
Cell[17613, 602, 104, 2, 46, "Text"],
Cell[17720, 606, 102, 2, 46, "Text"],
Cell[17825, 610, 104, 2, 46, "Text"],
Cell[17932, 614, 102, 2, 46, "Text"],
Cell[18037, 618, 104, 2, 46, "Text"],
Cell[18144, 622, 102, 2, 46, "Text"],
Cell[18249, 626, 104, 2, 46, "Text"],
Cell[18356, 630, 102, 2, 46, "Text"],
Cell[18461, 634, 104, 2, 46, "Text"],
Cell[18568, 638, 106, 2, 46, "Text"],
Cell[18677, 642, 102, 2, 46, "Text"],
Cell[18782, 646, 104, 2, 46, "Text"],
Cell[18889, 650, 102, 2, 46, "Text"],
Cell[18994, 654, 104, 2, 46, "Text"],
Cell[19101, 658, 106, 2, 46, "Text"],
Cell[19210, 662, 106, 2, 46, "Text"],
Cell[19319, 666, 106, 2, 46, "Text"],
Cell[19428, 670, 102, 2, 46, "Text"],
Cell[19533, 674, 104, 2, 46, "Text"],
Cell[19640, 678, 113, 2, 46, "Text"],
Cell[19756, 682, 102, 2, 46, "Text"],
Cell[19861, 686, 104, 2, 46, "Text"],
Cell[19968, 690, 106, 2, 46, "Text"],
Cell[20077, 694, 102, 2, 46, "Text"],
Cell[20182, 698, 104, 2, 46, "Text"],
Cell[20289, 702, 106, 2, 46, "Text"],
Cell[20398, 706, 102, 2, 46, "Text"],
Cell[20503, 710, 104, 2, 46, "Text"],
Cell[20610, 714, 102, 2, 46, "Text"],
Cell[20715, 718, 104, 2, 46, "Text"],
Cell[20822, 722, 106, 2, 46, "Text"],
Cell[20931, 726, 106, 2, 46, "Text"],
Cell[21040, 730, 102, 2, 46, "Text"],
Cell[21145, 734, 104, 2, 46, "Text"],
Cell[21252, 738, 102, 2, 46, "Text"],
Cell[21357, 742, 104, 2, 46, "Text"],
Cell[21464, 746, 102, 2, 46, "Text"],
Cell[21569, 750, 104, 2, 46, "Text"],
Cell[21676, 754, 106, 2, 46, "Text"],
Cell[21785, 758, 106, 2, 46, "Text"],
Cell[21894, 762, 102, 2, 46, "Text"],
Cell[21999, 766, 104, 2, 46, "Text"],
Cell[22106, 770, 106, 2, 46, "Text"],
Cell[22215, 774, 106, 2, 46, "Text"],
Cell[22324, 778, 102, 2, 46, "Text"],
Cell[22429, 782, 104, 2, 46, "Text"],
Cell[22536, 786, 102, 2, 46, "Text"],
Cell[22641, 790, 103, 2, 40, "SmallText"],
Cell[22747, 794, 103, 2, 40, "SmallText"],
Cell[22853, 798, 104, 2, 46, "Text"],
Cell[22960, 802, 102, 2, 46, "Text"],
Cell[23065, 806, 104, 2, 46, "Text"],
Cell[23172, 810, 102, 2, 46, "Text"],
Cell[23277, 814, 105, 2, 46, "Text"],
Cell[23385, 818, 105, 2, 46, "Text"],
Cell[23493, 822, 105, 2, 46, "Text"],
Cell[23601, 826, 105, 2, 46, "Text"],
Cell[23709, 830, 102, 2, 46, "Text"],
Cell[23814, 834, 102, 2, 46, "Text"],
Cell[23919, 838, 104, 2, 46, "Text"],
Cell[24026, 842, 102, 2, 46, "Text"],
Cell[24131, 846, 104, 2, 46, "Text"],
Cell[24238, 850, 106, 2, 46, "Text"],
Cell[24347, 854, 106, 2, 46, "Text"],
Cell[24456, 858, 102, 2, 46, "Text"],
Cell[24561, 862, 104, 2, 46, "Text"],
Cell[24668, 866, 102, 2, 46, "Text"],
Cell[24773, 870, 104, 2, 46, "Text"],
Cell[24880, 874, 102, 2, 46, "Text"],
Cell[24985, 878, 104, 2, 46, "Text"],
Cell[25092, 882, 102, 2, 46, "Text"],
Cell[25197, 886, 104, 2, 46, "Text"],
Cell[25304, 890, 102, 2, 46, "Text"],
Cell[25409, 894, 104, 2, 46, "Text"],
Cell[25516, 898, 106, 2, 46, "Text"],
Cell[25625, 902, 106, 2, 46, "Text"],
Cell[25734, 906, 103, 2, 46, "Text"],
Cell[25840, 910, 105, 2, 46, "Text"],
Cell[25948, 914, 107, 2, 46, "Text"],
Cell[26058, 918, 107, 2, 46, "Text"],
Cell[26168, 922, 107, 2, 46, "Text"],
Cell[26278, 926, 107, 2, 46, "Text"],
Cell[26388, 930, 107, 2, 46, "Text"],
Cell[26498, 934, 104, 2, 46, "Text"],
Cell[26605, 938, 106, 2, 46, "Text"],
Cell[26714, 942, 104, 2, 46, "Text"],
Cell[26821, 946, 106, 2, 46, "Text"],
Cell[26930, 950, 104, 2, 46, "Text"],
Cell[27037, 954, 106, 2, 46, "Text"],
Cell[27146, 958, 104, 2, 46, "Text"],
Cell[27253, 962, 106, 2, 46, "Text"],
Cell[27362, 966, 104, 2, 46, "Text"],
Cell[27469, 970, 106, 2, 46, "Text"],
Cell[27578, 974, 106, 2, 46, "Text"],
Cell[27687, 978, 106, 2, 46, "Text"],
Cell[27796, 982, 106, 2, 46, "Text"],
Cell[27905, 986, 106, 2, 46, "Text"],
Cell[28014, 990, 106, 2, 46, "Text"],
Cell[28123, 994, 106, 2, 46, "Text"],
Cell[28232, 998, 105, 2, 46, "Text"],
Cell[28340, 1002, 106, 2, 46, "Text"],
Cell[28449, 1006, 105, 2, 46, "Text"],
Cell[28557, 1010, 103, 2, 46, "Text"],
Cell[28663, 1014, 117, 2, 46, "Text"],
Cell[28783, 1018, 101, 2, 46, "Text"],
Cell[28887, 1022, 95, 2, 46, "Text"],
Cell[28985, 1026, 99, 2, 46, "Text"],
Cell[29087, 1030, 104, 2, 46, "Text"],
Cell[29194, 1034, 101, 2, 46, "Text"],
Cell[29298, 1038, 102, 2, 46, "Text"],
Cell[29403, 1042, 105, 2, 46, "Text"],
Cell[29511, 1046, 93, 2, 46, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[29641, 1053, 93, 2, 77, "Section",
  CellTags->"T1.1.1"],
Cell[29737, 1057, 342, 10, 84, "SmallText"],
Cell[30082, 1069, 118, 3, 46, "Text"],
Cell[30203, 1074, 1703, 43, 118, "Text"],
Cell[31909, 1119, 222, 5, 121, "Text"],
Cell[32134, 1126, 51, 1, 39, "Input"],
Cell[32188, 1129, 61, 1, 39, "Input"],
Cell[32252, 1132, 86, 1, 39, "Input"],
Cell[32341, 1135, 57, 1, 39, "Input"],
Cell[32401, 1138, 107, 3, 46, "Text"],
Cell[32511, 1143, 170, 5, 62, "Input"],
Cell[32684, 1150, 220, 6, 62, "SmallText"],
Cell[32907, 1158, 244, 5, 108, "Input"],
Cell[33154, 1165, 58, 1, 39, "Input"],
Cell[33215, 1168, 59, 1, 39, "Input"],

Cell[CellGroupData[{
Cell[33299, 1173, 78, 1, 60, "Subsubsection",
  CellTags->"T1.2.1.1"],
Cell[33380, 1176, 512, 11, 146, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[33941, 1193, 86, 2, 77, "Section",
  CellTags->"T1.2.1"],
Cell[34030, 1197, 84, 1, 39, "Input"],
Cell[34117, 1200, 263, 4, 108, "Input"],
Cell[34383, 1206, 637, 9, 246, "Input"],
Cell[35023, 1217, 165, 3, 62, "Input"],
Cell[35191, 1222, 131, 3, 62, "Input"],
Cell[35325, 1227, 143, 3, 62, "Input"],
Cell[35471, 1232, 143, 3, 62, "Input"],
Cell[35617, 1237, 153, 3, 62, "Input"],
Cell[35773, 1242, 134, 3, 62, "Input"],
Cell[35910, 1247, 311, 5, 131, "Input"],
Cell[36224, 1254, 219, 4, 85, "Input"],
Cell[36446, 1260, 235, 4, 85, "Input"],
Cell[36684, 1266, 182, 3, 85, "Input"],
Cell[36869, 1271, 178, 3, 85, "Input"],
Cell[37050, 1276, 203, 4, 85, "Input"],
Cell[37256, 1282, 183, 3, 85, "Input"],
Cell[37442, 1287, 174, 3, 62, "Input"],
Cell[37619, 1292, 157, 3, 62, "Input"],
Cell[37779, 1297, 157, 3, 62, "Input"],
Cell[37939, 1302, 157, 3, 62, "Input"],
Cell[38099, 1307, 157, 3, 62, "Input"],
Cell[38259, 1312, 157, 3, 62, "Input"],
Cell[38419, 1317, 457, 7, 177, "Input"],
Cell[38879, 1326, 195, 3, 85, "Input"],
Cell[39077, 1331, 188, 3, 85, "Input"],
Cell[39268, 1336, 152, 3, 62, "Input"],
Cell[39423, 1341, 256, 4, 108, "Input"],
Cell[39682, 1347, 185, 3, 85, "Input"],
Cell[39870, 1352, 61, 1, 39, "Input"],
Cell[39934, 1355, 57, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[40028, 1361, 112, 2, 77, "Section",
  CellTags->"T1.3.1"],
Cell[40143, 1365, 798, 24, 71, "Text"],
Cell[40944, 1391, 58, 1, 39, "Input"],
Cell[41005, 1394, 73, 0, 46, "Text"],
Cell[41081, 1396, 62, 1, 39, "Input"],
Cell[41146, 1399, 115, 3, 46, "Text"],
Cell[41264, 1404, 59, 1, 39, "Input"],
Cell[41326, 1407, 754, 24, 71, "Text"],

Cell[CellGroupData[{
Cell[42105, 1435, 63, 1, 64, "Subsection",
  CellTags->"T1.3.1.1"],
Cell[42171, 1438, 61, 1, 39, "Input"],
Cell[42235, 1441, 261, 5, 108, "Input"],
Cell[42499, 1448, 261, 5, 110, "Input"],
Cell[42763, 1455, 169, 3, 62, "Input"],
Cell[42935, 1460, 115, 3, 46, "Text"],
Cell[43053, 1465, 213, 4, 85, "Input"],
Cell[43269, 1471, 192, 3, 62, "Input"],
Cell[43464, 1476, 216, 4, 62, "Input"],
Cell[43683, 1482, 238, 4, 85, "Input"],
Cell[43924, 1488, 145, 2, 62, "Input"],
Cell[44072, 1492, 162, 2, 62, "Input"],
Cell[44237, 1496, 194, 4, 85, "Input"],
Cell[44434, 1502, 179, 3, 85, "Input"],
Cell[44616, 1507, 513, 13, 278, "Input"],
Cell[45132, 1522, 103, 2, 39, "Input"],
Cell[45238, 1526, 103, 2, 39, "Input"],
Cell[45344, 1530, 103, 2, 39, "Input"],
Cell[45450, 1534, 103, 2, 39, "Input"],
Cell[45556, 1538, 103, 2, 39, "Input"],
Cell[45662, 1542, 77, 1, 39, "Input"],
Cell[45742, 1545, 80, 1, 39, "Input"],
Cell[45825, 1548, 118, 2, 39, "Input"],
Cell[45946, 1552, 280, 5, 85, "Input"],
Cell[46229, 1559, 151, 3, 62, "Input"],
Cell[46383, 1564, 176, 3, 85, "Input"],
Cell[46562, 1569, 118, 2, 39, "Input"],
Cell[46683, 1573, 151, 3, 62, "Input"],
Cell[46837, 1578, 280, 5, 85, "Input"],
Cell[47120, 1585, 539, 9, 177, "Input"],
Cell[47662, 1596, 642, 11, 223, "Input"],
Cell[48307, 1609, 547, 9, 200, "Input"],
Cell[48857, 1620, 654, 12, 200, "Input"],
Cell[49514, 1634, 202, 3, 39, "Input"],
Cell[49719, 1639, 684, 11, 200, "Input"],
Cell[50406, 1652, 607, 11, 177, "Input"],
Cell[51016, 1665, 387, 6, 131, "Input"],
Cell[51406, 1673, 607, 11, 177, "Input"],
Cell[52016, 1686, 172, 3, 39, "Input"],
Cell[52191, 1691, 387, 6, 131, "Input"],
Cell[52581, 1699, 493, 8, 177, "Input"],
Cell[53077, 1709, 135, 2, 39, "Input"],
Cell[53215, 1713, 489, 8, 177, "Input"],
Cell[53707, 1723, 108, 2, 39, "Input"],
Cell[53818, 1727, 107, 3, 71, "Text"],
Cell[53928, 1732, 571, 9, 177, "Input"],
Cell[54502, 1743, 571, 9, 177, "Input"],
Cell[55076, 1754, 550, 9, 200, "Input"],
Cell[55629, 1765, 659, 12, 200, "Input"],
Cell[56291, 1779, 689, 11, 223, "Input"],
Cell[56983, 1792, 612, 11, 177, "Input"],
Cell[57598, 1805, 389, 6, 131, "Input"],
Cell[57990, 1813, 496, 8, 177, "Input"],
Cell[58489, 1823, 492, 8, 177, "Input"],
Cell[58984, 1833, 568, 9, 177, "Input"],
Cell[59555, 1844, 568, 9, 177, "Input"],
Cell[60126, 1855, 547, 9, 200, "Input"],
Cell[60676, 1866, 654, 12, 200, "Input"],
Cell[61333, 1880, 684, 11, 200, "Input"],
Cell[62020, 1893, 607, 11, 177, "Input"],
Cell[62630, 1906, 387, 6, 131, "Input"],
Cell[63020, 1914, 493, 8, 177, "Input"],
Cell[63516, 1924, 489, 8, 177, "Input"],
Cell[64008, 1934, 112, 2, 39, "Input"],
Cell[64123, 1938, 112, 2, 39, "Input"],
Cell[64238, 1942, 112, 2, 39, "Input"],
Cell[64353, 1946, 112, 2, 39, "Input"],
Cell[64468, 1950, 112, 2, 39, "Input"],
Cell[64583, 1954, 132, 2, 70, "Input"],
Cell[64718, 1958, 120, 2, 39, "Input"],
Cell[64841, 1962, 205, 4, 85, "Input"],
Cell[65049, 1968, 111, 2, 62, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[65197, 1975, 58, 1, 64, "Subsection",
  CellTags->"T1.3.2.1"],
Cell[65258, 1978, 4352, 73, 1625, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[69647, 2056, 59, 1, 64, "Subsection",
  CellTags->"T1.3.3.1"],
Cell[69709, 2059, 498, 9, 202, "Input"],
Cell[70210, 2070, 498, 9, 202, "Input"],
Cell[70711, 2081, 498, 9, 202, "Input"],
Cell[71212, 2092, 498, 9, 202, "Input"],
Cell[71713, 2103, 498, 9, 202, "Input"],
Cell[72214, 2114, 498, 9, 202, "Input"],
Cell[72715, 2125, 498, 9, 202, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[73250, 2139, 60, 1, 64, "Subsection",
  CellTags->"T1.3.4.1"],
Cell[73313, 2142, 682, 15, 177, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[74032, 2162, 54, 1, 64, "Subsection",
  CellTags->"T1.3.5.1"],
Cell[74089, 2165, 129, 3, 58, "Input"],
Cell[74221, 2170, 1901, 34, 568, "Input"],
Cell[76125, 2206, 185, 3, 85, "Input"],
Cell[76313, 2211, 306, 9, 43, "Text"],
Cell[76622, 2222, 1937, 33, 641, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[78596, 2260, 59, 1, 64, "Subsection",
  CellTags->"T1.3.6.1"],
Cell[78658, 2263, 459, 8, 154, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[79154, 2276, 50, 1, 64, "Subsection",
  CellTags->"T1.3.7.1"],
Cell[79207, 2279, 84, 1, 39, "Input"],

Cell[CellGroupData[{
Cell[79316, 2284, 57, 1, 60, "Subsubsection",
  CellTags->"T1.3.7.1.1"],
Cell[79376, 2287, 779, 28, 53, "Text"],
Cell[80158, 2317, 212, 4, 62, "Input"],
Cell[80373, 2323, 145, 3, 62, "Input"],
Cell[80521, 2328, 147, 3, 62, "Input"],
Cell[80671, 2333, 153, 3, 62, "Input"],
Cell[80827, 2338, 141, 3, 62, "Input"],
Cell[80971, 2343, 151, 3, 62, "Input"],
Cell[81125, 2348, 139, 3, 62, "Input"],
Cell[81267, 2353, 139, 3, 62, "Input"],
Cell[81409, 2358, 147, 3, 62, "Input"],
Cell[81559, 2363, 143, 3, 62, "Input"],
Cell[81705, 2368, 132, 2, 62, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[81874, 2375, 53, 1, 60, "Subsubsection",
  CellTags->"T1.3.7.2.1"],
Cell[81930, 2378, 72, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[82039, 2384, 54, 1, 60, "Subsubsection",
  CellTags->"T1.3.7.3.1"],
Cell[82096, 2387, 52, 1, 39, "Input"],
Cell[82151, 2390, 73, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[82261, 2396, 58, 1, 60, "Subsubsection",
  CellTags->"T1.3.7.4.1"],
Cell[82322, 2399, 74, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[82433, 2405, 54, 1, 60, "Subsubsection",
  CellTags->"T1.3.7.5.1"],
Cell[82490, 2408, 109, 2, 39, "Input"],
Cell[82602, 2412, 120, 2, 62, "Input"],
Cell[82725, 2416, 77, 1, 39, "Input"],
Cell[82805, 2419, 151, 2, 62, "Input"],
Cell[82959, 2423, 133, 2, 62, "Input"],
Cell[83095, 2427, 132, 2, 62, "Input"],
Cell[83230, 2431, 115, 2, 39, "Input"],
Cell[83348, 2435, 166, 2, 85, "Input"],
Cell[83517, 2439, 93, 1, 39, "Input"],
Cell[83613, 2442, 93, 1, 39, "Input"],
Cell[83709, 2445, 93, 1, 39, "Input"],
Cell[83805, 2448, 93, 1, 39, "Input"],
Cell[83901, 2451, 93, 1, 39, "Input"],
Cell[83997, 2454, 93, 1, 39, "Input"],
Cell[84093, 2457, 93, 1, 39, "Input"],
Cell[84189, 2460, 93, 1, 39, "Input"],
Cell[84285, 2463, 93, 1, 39, "Input"],
Cell[84381, 2466, 93, 1, 39, "Input"],
Cell[84477, 2469, 93, 1, 39, "Input"],
Cell[84573, 2472, 93, 1, 39, "Input"],
Cell[84669, 2475, 93, 1, 39, "Input"],
Cell[84765, 2478, 93, 1, 39, "Input"],
Cell[84861, 2481, 93, 1, 39, "Input"],
Cell[84957, 2484, 93, 1, 39, "Input"],
Cell[85053, 2487, 93, 1, 39, "Input"],
Cell[85149, 2490, 93, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[85279, 2496, 54, 1, 60, "Subsubsection",
  CellTags->"T1.3.7.6.1"],
Cell[85336, 2499, 345, 6, 131, "Input"],
Cell[85684, 2507, 72, 1, 39, "Input"],
Cell[85759, 2510, 74, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[85870, 2516, 54, 1, 60, "Subsubsection",
  CellTags->"T1.3.7.7.1"],
Cell[85927, 2519, 574, 14, 108, "Input"],
Cell[86504, 2535, 66, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[86607, 2541, 54, 1, 60, "Subsubsection",
  CellTags->"T1.3.7.8.1"],
Cell[86664, 2544, 215, 6, 39, "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[86940, 2557, 104, 2, 77, "Section",
  CellTags->"T1.4.1"],
Cell[87047, 2561, 574, 11, 137, "Input"],
Cell[87624, 2574, 959, 17, 285, "Input"],
Cell[88586, 2593, 136, 4, 46, "Text"],
Cell[88725, 2599, 159, 3, 42, "Input"],
Cell[88887, 2604, 61, 1, 41, "Input"],

Cell[CellGroupData[{
Cell[88973, 2609, 116, 3, 69, "Subsection",
  CellTags->"T1.4.1.1"],
Cell[89092, 2614, 213, 6, 39, "Input"],
Cell[89308, 2622, 72, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[89417, 2628, 114, 3, 69, "Subsection",
  CellTags->"T1.4.2.1"],
Cell[89534, 2633, 113, 5, 53, "Text"],
Cell[89650, 2640, 1952, 52, 97, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[91639, 2697, 205, 6, 69, "Subsection",
  CellTags->"T1.4.3.1"],
Cell[91847, 2705, 2379, 57, 152, "Input"],
Cell[94229, 2764, 157, 4, 41, "Input"],
Cell[94389, 2770, 1177, 25, 127, "Input"],
Cell[95569, 2797, 772, 19, 96, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[96378, 2821, 223, 6, 91, "Subsection",
  CellTags->"T1.4.4.1"],
Cell[96604, 2829, 3158, 73, 212, "Input"],
Cell[99765, 2904, 2826, 70, 187, "Input"],
Cell[102594, 2976, 2826, 70, 187, "Input"],
Cell[105423, 3048, 2826, 70, 187, "Input"],
Cell[108252, 3120, 157, 4, 42, "Input"],
Cell[108412, 3126, 157, 4, 41, "Input"],
Cell[108572, 3132, 139, 4, 42, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[108760, 3142, 97, 2, 77, "Section",
  CellTags->"T1.5.1"],

Cell[CellGroupData[{
Cell[108882, 3148, 65, 1, 64, "Subsection",
  CellTags->"T1.5.1.1"],
Cell[108950, 3151, 131, 3, 71, "Text"],
Cell[109084, 3156, 67, 0, 46, "Text"],
Cell[109154, 3158, 154, 5, 46, "Text"],
Cell[109311, 3165, 1161, 19, 239, "Input"],
Cell[110475, 3186, 1267, 21, 214, "Input"],
Cell[111745, 3209, 100, 2, 39, "Input"],
Cell[111848, 3213, 88, 1, 39, "Input"],
Cell[111939, 3216, 119, 2, 62, "Input"],
Cell[112061, 3220, 108, 2, 39, "Input"],
Cell[112172, 3224, 88, 1, 39, "Input"],
Cell[112263, 3227, 129, 2, 62, "Input"],
Cell[112395, 3231, 72, 1, 54, "Input"],
Cell[112470, 3234, 59, 0, 38, "Input"],
Cell[112532, 3236, 79, 1, 39, "Input"],
Cell[112614, 3239, 79, 1, 39, "Input"],
Cell[112696, 3242, 80, 1, 39, "Input"],
Cell[112779, 3245, 93, 1, 39, "Input"],
Cell[112875, 3248, 81, 1, 39, "Input"],
Cell[112959, 3251, 81, 1, 39, "Input"],
Cell[113043, 3254, 630, 10, 177, "Input"],
Cell[113676, 3266, 52, 1, 39, "Input"],
Cell[113731, 3269, 186, 4, 62, "Input"],
Cell[113920, 3275, 80, 1, 39, "Input"],
Cell[114003, 3278, 76, 0, 46, "Text"],
Cell[114082, 3280, 202, 6, 39, "Input"],
Cell[114287, 3288, 152, 4, 39, "Input"],
Cell[114442, 3294, 9842, 176, 1036, "Input"],
Cell[124287, 3472, 9225, 164, 975, "Input"],
Cell[133515, 3638, 12143, 217, 1153, "Input"],
Cell[145661, 3857, 11120, 200, 990, "Input"],
Cell[156784, 4059, 10441, 187, 1103, "Input"],
Cell[167228, 4248, 69, 1, 39, "Input"],
Cell[167300, 4251, 804, 19, 131, "Input"],
Cell[168107, 4272, 746, 18, 85, "Input"],
Cell[168856, 4292, 746, 18, 85, "Input"],

Cell[CellGroupData[{
Cell[169627, 4314, 62, 1, 60, "Subsubsection",
  CellTags->"T1.5.1.1.1"],
Cell[169692, 4317, 2621, 58, 352, "Input"],
Cell[172316, 4377, 150, 2, 62, "Input"],
Cell[172469, 4381, 142, 2, 39, "Input"],
Cell[172614, 4385, 152, 2, 62, "Input"],
Cell[172769, 4389, 193, 4, 42, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[173011, 4399, 59, 1, 64, "Subsection",
  CellTags->"T1.5.2.1"],
Cell[173073, 4402, 1297, 24, 363, "Input"],

Cell[CellGroupData[{
Cell[174395, 4430, 53, 1, 60, "Subsubsection",
  CellTags->"T1.5.2.1.1"],
Cell[174451, 4433, 132, 2, 62, "Input"],
Cell[174586, 4437, 145, 2, 85, "Input"],
Cell[174734, 4441, 117, 2, 39, "Input"],
Cell[174854, 4445, 251, 9, 53, "Text"],
Cell[175108, 4456, 20, 0, 46, "Text"],
Cell[175131, 4458, 1288, 27, 221, "Input"],
Cell[176422, 4487, 20, 0, 46, "Text"],
Cell[176445, 4489, 726, 12, 190, "Input"],
Cell[177174, 4503, 20, 0, 46, "Text"],
Cell[177197, 4505, 506, 9, 129, "Input"],
Cell[177706, 4516, 20, 0, 46, "Text"],
Cell[177729, 4518, 500, 8, 230, "Input"],
Cell[178232, 4528, 20, 0, 46, "Text"],
Cell[178255, 4530, 719, 12, 231, "Input"],
Cell[178977, 4544, 20, 0, 46, "Text"],
Cell[179000, 4546, 1071, 22, 176, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[180120, 4574, 60, 1, 64, "Subsection",
  CellTags->"T1.5.3.1"],
Cell[180183, 4577, 176, 4, 71, "Text"],
Cell[180362, 4583, 139, 3, 71, "Text"],

Cell[CellGroupData[{
Cell[180526, 4590, 68, 1, 60, "Subsubsection",
  CellTags->"T1.5.3.1.1"],
Cell[180597, 4593, 334, 6, 132, "Input"],
Cell[180934, 4601, 727, 13, 380, "Input"],
Cell[181664, 4616, 1798, 32, 655, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[183499, 4653, 68, 1, 60, "Subsubsection",
  CellTags->"T1.5.3.2.1"],
Cell[183570, 4656, 279, 5, 132, "Input"],
Cell[183852, 4663, 185, 3, 85, "Input"],
Cell[184040, 4668, 63, 1, 39, "Input"],
Cell[184106, 4671, 1789, 34, 678, "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[185956, 4712, 140, 2, 77, "Section",
  CellTags->"T1.6.1"],
Cell[186099, 4716, 272, 7, 96, "Text"],

Cell[CellGroupData[{
Cell[186396, 4727, 56, 1, 64, "Subsection",
  CellTags->"T1.6.1.1"],
Cell[186455, 4730, 77, 4, 46, "Text"],
Cell[186535, 4736, 7362, 170, 153, "Input"],
Cell[193900, 4908, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[193999, 4914, 58, 1, 64, "Subsection",
  CellTags->"T1.6.2.1"],
Cell[194060, 4917, 81, 1, 39, "Input"],

Cell[CellGroupData[{
Cell[194166, 4922, 139, 5, 60, "Subsubsection",
  CellTags->"T1.6.2.1.1"],
Cell[194308, 4929, 1082, 28, 84, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[195427, 4962, 139, 5, 60, "Subsubsection",
  CellTags->"T1.6.2.2.1"],
Cell[195569, 4969, 1163, 30, 84, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[196769, 5004, 139, 5, 60, "Subsubsection",
  CellTags->"T1.6.2.3.1"],
Cell[196911, 5011, 1163, 30, 84, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[198111, 5046, 139, 5, 60, "Subsubsection",
  CellTags->"T1.6.2.4.1"],
Cell[198253, 5053, 1163, 30, 84, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[199465, 5089, 58, 1, 64, "Subsection",
  CellTags->"T1.6.3.1"],
Cell[199526, 5092, 73, 1, 39, "Input"],
Cell[199602, 5095, 132, 2, 44, "Text"],
Cell[199737, 5099, 3415, 89, 81, "Input"],
Cell[203155, 5190, 66, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[203258, 5196, 159, 5, 66, "Subsection",
  CellTags->"T1.6.4.1"],
Cell[203420, 5203, 50, 1, 39, "Input"],
Cell[203473, 5206, 393, 9, 62, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[203903, 5220, 58, 1, 64, "Subsection",
  CellTags->"T1.6.5.1"],
Cell[203964, 5223, 73, 1, 39, "Input"],
Cell[204040, 5226, 132, 2, 44, "Text"],
Cell[204175, 5230, 3415, 89, 81, "Input"],
Cell[207593, 5321, 66, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[207696, 5327, 159, 5, 66, "Subsection",
  CellTags->"T1.6.6.1"],
Cell[207858, 5334, 50, 1, 39, "Input"],
Cell[207911, 5337, 393, 9, 62, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[208341, 5351, 159, 5, 66, "Subsection",
  CellTags->"T1.6.7.1"],
Cell[208503, 5358, 62, 1, 39, "Input"],
Cell[208568, 5361, 127, 3, 39, "Input"],
Cell[208698, 5366, 393, 9, 62, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[209128, 5380, 159, 5, 66, "Subsection",
  CellTags->"T1.6.8.1"],
Cell[209290, 5387, 62, 1, 39, "Input"],
Cell[209355, 5390, 127, 3, 39, "Input"],
Cell[209485, 5395, 393, 9, 62, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[209915, 5409, 56, 1, 64, "Subsection",
  CellTags->"T1.6.9.1"],
Cell[209974, 5412, 77, 4, 46, "Text"],
Cell[210054, 5418, 6608, 152, 151, "Input"],
Cell[216665, 5572, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[216764, 5578, 59, 1, 64, "Subsection",
  CellTags->"T1.6.10.1"],
Cell[216826, 5581, 79, 1, 39, "Input"],
Cell[216908, 5584, 1078, 28, 84, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[218023, 5617, 57, 1, 64, "Subsection",
  CellTags->"T1.6.11.1"],
Cell[218083, 5620, 454, 20, 46, "Text"],
Cell[218540, 5642, 3684, 95, 81, "Input"],
Cell[222227, 5739, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[222326, 5745, 57, 1, 64, "Subsection",
  CellTags->"T1.6.12.1"],
Cell[222386, 5748, 454, 20, 46, "Text"],
Cell[222843, 5770, 3684, 95, 81, "Input"],
Cell[226530, 5867, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[226629, 5873, 57, 1, 64, "Subsection",
  CellTags->"T1.6.13.1"],
Cell[226689, 5876, 7313, 170, 138, "Input"],
Cell[234005, 6048, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[234104, 6054, 68, 1, 64, "Subsection",
  CellTags->"T1.6.14.1"],
Cell[234175, 6057, 201, 4, 85, "Input"],
Cell[234379, 6063, 101, 2, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[234517, 6070, 59, 1, 64, "Subsection",
  CellTags->"T1.6.15.1"],
Cell[234579, 6073, 22655, 536, 10718, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[257271, 6614, 57, 1, 64, "Subsection",
  CellTags->"T1.6.16.1"],
Cell[257331, 6617, 77, 4, 46, "Text"],
Cell[257411, 6623, 3791, 96, 114, "Input"],
Cell[261205, 6721, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[261304, 6727, 59, 1, 64, "Subsection",
  CellTags->"T1.6.17.1"],
Cell[261366, 6730, 38, 1, 39, "Input"],
Cell[261407, 6733, 79, 1, 39, "Input"],
Cell[261489, 6736, 397, 10, 41, "Input"],

Cell[CellGroupData[{
Cell[261911, 6750, 57, 1, 39, "Input"],
Cell[261971, 6753, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[262058, 6760, 57, 1, 64, "Subsection",
  CellTags->"T1.6.18.1"],
Cell[262118, 6763, 77, 4, 46, "Text"],
Cell[262198, 6769, 5270, 131, 145, "Input"],
Cell[267471, 6902, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[267570, 6908, 59, 1, 64, "Subsection",
  CellTags->"T1.6.19.1"],
Cell[267632, 6911, 97, 2, 39, "Input"],
Cell[267732, 6915, 397, 10, 41, "Input"],

Cell[CellGroupData[{
Cell[268154, 6929, 57, 1, 39, "Input"],
Cell[268214, 6932, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[268301, 6939, 59, 1, 64, "Subsection",
  CellTags->"T1.6.20.1"],
Cell[268363, 6942, 131, 3, 39, "Input"],
Cell[268497, 6947, 397, 10, 41, "Input"],

Cell[CellGroupData[{
Cell[268919, 6961, 57, 1, 39, "Input"],
Cell[268979, 6964, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[269066, 6971, 57, 1, 64, "Subsection",
  CellTags->"T1.6.21.1"],
Cell[269126, 6974, 77, 4, 46, "Text"],
Cell[269206, 6980, 3179, 83, 83, "Input"],
Cell[272388, 7065, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[272487, 7071, 59, 1, 64, "Subsection",
  CellTags->"T1.6.22.1"],
Cell[272549, 7074, 79, 1, 39, "Input"],
Cell[272631, 7077, 369, 9, 39, "Input"],

Cell[CellGroupData[{
Cell[273025, 7090, 57, 1, 39, "Input"],
Cell[273085, 7093, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[273172, 7100, 57, 1, 64, "Subsection",
  CellTags->"T1.6.23.1"],
Cell[273232, 7103, 77, 4, 46, "Text"],
Cell[273312, 7109, 3892, 100, 83, "Input"],
Cell[277207, 7211, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[277306, 7217, 59, 1, 64, "Subsection",
  CellTags->"T1.6.24.1"],
Cell[277368, 7220, 79, 1, 39, "Input"],
Cell[277450, 7223, 393, 10, 39, "Input"],

Cell[CellGroupData[{
Cell[277868, 7237, 57, 1, 39, "Input"],
Cell[277928, 7240, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[278015, 7247, 57, 1, 64, "Subsection",
  CellTags->"T1.6.25.1"],
Cell[278075, 7250, 77, 4, 46, "Text"],
Cell[278155, 7256, 3445, 89, 83, "Input"],
Cell[281603, 7347, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[281702, 7353, 59, 1, 64, "Subsection",
  CellTags->"T1.6.26.1"],
Cell[281764, 7356, 79, 1, 39, "Input"],
Cell[281846, 7359, 395, 10, 39, "Input"],

Cell[CellGroupData[{
Cell[282266, 7373, 57, 1, 39, "Input"],
Cell[282326, 7376, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[282413, 7383, 57, 1, 64, "Subsection",
  CellTags->"T1.6.27.1"],
Cell[282473, 7386, 169, 7, 46, "Text"],
Cell[282645, 7395, 4987, 124, 146, "Input"],
Cell[287635, 7521, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[287734, 7527, 59, 1, 64, "Subsection",
  CellTags->"T1.6.28.1"],
Cell[287796, 7530, 46, 1, 39, "Input"],
Cell[287845, 7533, 803, 17, 156, "Input"],
Cell[288651, 7552, 50, 0, 46, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[288738, 7557, 58, 1, 64, "Subsection",
  CellTags->"T1.6.29.1"],
Cell[288799, 7560, 77, 4, 46, "Text"],
Cell[288879, 7566, 3179, 83, 83, "Input"],
Cell[292061, 7651, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[292160, 7657, 59, 1, 64, "Subsection",
  CellTags->"T1.6.30.1"],
Cell[292222, 7660, 79, 1, 39, "Input"],
Cell[292304, 7663, 376, 9, 41, "Input"],

Cell[CellGroupData[{
Cell[292705, 7676, 69, 1, 41, "Input"],
Cell[292777, 7679, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[292864, 7686, 58, 1, 64, "Subsection",
  CellTags->"T1.6.31.1"],
Cell[292925, 7689, 292, 14, 46, "Text"],
Cell[293220, 7705, 2494, 68, 48, "Input"],
Cell[295717, 7775, 62, 1, 39, "Input"],
Cell[295782, 7778, 2268, 66, 74, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[298087, 7849, 59, 1, 64, "Subsection",
  CellTags->"T1.6.32.1"],
Cell[298149, 7852, 291, 14, 46, "Text"],
Cell[298443, 7868, 46, 1, 39, "Input"],
Cell[298492, 7871, 653, 18, 39, "Input"],
Cell[299148, 7891, 50, 0, 46, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[299235, 7896, 61, 1, 64, "Subsection",
  CellTags->"T1.6.33.1"],
Cell[299299, 7899, 291, 14, 46, "Text"],
Cell[299593, 7915, 84, 1, 39, "Input"],
Cell[299680, 7918, 659, 18, 62, "Input"],
Cell[300342, 7938, 50, 0, 46, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[300429, 7943, 58, 1, 64, "Subsection",
  CellTags->"T1.6.34.1"],
Cell[300490, 7946, 394, 17, 71, "Text"],
Cell[300887, 7965, 5063, 127, 145, "Input"],
Cell[305953, 8094, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[306052, 8100, 59, 1, 64, "Subsection",
  CellTags->"T1.6.35.1"],
Cell[306114, 8103, 97, 2, 39, "Input"],
Cell[306214, 8107, 985, 26, 75, "Input"],
Cell[307202, 8135, 50, 0, 46, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[307289, 8140, 58, 1, 64, "Subsection",
  CellTags->"T1.6.36.1"],
Cell[307350, 8143, 209, 7, 71, "Text"],
Cell[307562, 8152, 6350, 147, 139, "Input"],
Cell[313915, 8301, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[314014, 8307, 59, 1, 64, "Subsection",
  CellTags->"T1.6.37.1"],
Cell[314076, 8310, 178, 7, 46, "Text"],
Cell[314257, 8319, 61, 1, 39, "Input"],
Cell[314321, 8322, 561, 14, 85, "Input"],

Cell[CellGroupData[{
Cell[314907, 8340, 57, 1, 39, "Input"],
Cell[314967, 8343, 38, 1, 33, "Print"]
}, Open  ]],
Cell[315020, 8347, 641, 16, 85, "Input"],
Cell[315664, 8365, 106, 5, 121, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[315807, 8375, 61, 1, 64, "Subsection",
  CellTags->"T1.6.38.1"],
Cell[315871, 8378, 103, 2, 41, "Input"],
Cell[315977, 8382, 425, 11, 40, "Input"],
Cell[316405, 8395, 50, 1, 39, "Input"],
Cell[316458, 8398, 76, 1, 39, "Input"],
Cell[316537, 8401, 52, 1, 39, "Input"],
Cell[316592, 8404, 131, 3, 39, "Input"],
Cell[316726, 8409, 812, 18, 74, "Input"],
Cell[317541, 8429, 50, 0, 46, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[317628, 8434, 61, 1, 64, "Subsection",
  CellTags->"T1.6.39.1"],
Cell[317692, 8437, 103, 2, 41, "Input"],
Cell[317798, 8441, 66, 1, 39, "Input"],
Cell[317867, 8444, 52, 1, 39, "Input"],
Cell[317922, 8447, 131, 3, 39, "Input"],
Cell[318056, 8452, 576, 13, 74, "Input"],
Cell[318635, 8467, 50, 0, 46, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[318722, 8472, 61, 1, 64, "Subsection",
  CellTags->"T1.6.40.1"],
Cell[318786, 8475, 148, 3, 41, "Input"],
Cell[318937, 8480, 131, 3, 39, "Input"],
Cell[319071, 8485, 417, 10, 64, "Input"],
Cell[319491, 8497, 50, 0, 46, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[319578, 8502, 58, 1, 64, "Subsection",
  CellTags->"T1.6.41.1"],
Cell[319639, 8505, 77, 4, 46, "Text"],
Cell[319719, 8511, 4850, 122, 145, "Input"],
Cell[324572, 8635, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[324671, 8641, 59, 1, 64, "Subsection",
  CellTags->"T1.6.42.1"],
Cell[324733, 8644, 158, 3, 39, "Input"],
Cell[324894, 8649, 454, 11, 62, "Input"],

Cell[CellGroupData[{
Cell[325373, 8664, 57, 1, 39, "Input"],
Cell[325433, 8667, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[325520, 8674, 68, 1, 64, "Subsection",
  CellTags->"T1.6.43.1"],
Cell[325591, 8677, 111, 3, 46, "Text"],
Cell[325705, 8682, 77, 4, 46, "Text"],
Cell[325785, 8688, 5393, 138, 169, "Input"],
Cell[331181, 8828, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[331280, 8834, 57, 1, 64, "Subsection",
  CellTags->"T1.6.44.1"],
Cell[331340, 8837, 77, 4, 46, "Text"],
Cell[331420, 8843, 4938, 127, 145, "Input"],
Cell[336361, 8972, 9473, 263, 116, "Text"],
Cell[345837, 9237, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[345936, 9243, 59, 1, 64, "Subsection",
  CellTags->"T1.6.45.1"],
Cell[345998, 9246, 132, 3, 39, "Input"],
Cell[346133, 9251, 438, 11, 62, "Input"],

Cell[CellGroupData[{
Cell[346596, 9266, 57, 1, 39, "Input"],
Cell[346656, 9269, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[346743, 9276, 61, 1, 64, "Subsection",
  CellTags->"T1.6.46.1"],
Cell[346807, 9279, 170, 3, 39, "Input"],
Cell[346980, 9284, 444, 11, 62, "Input"],

Cell[CellGroupData[{
Cell[347449, 9299, 59, 1, 39, "Input"],
Cell[347511, 9302, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[347598, 9309, 57, 1, 64, "Subsection",
  CellTags->"T1.6.47.1"],
Cell[347658, 9312, 77, 4, 46, "Text"],
Cell[347738, 9318, 5780, 139, 176, "Input"],
Cell[353521, 9459, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[353620, 9465, 59, 1, 64, "Subsection",
  CellTags->"T1.6.48.1"],
Cell[353682, 9468, 132, 3, 39, "Input"],
Cell[353817, 9473, 450, 11, 62, "Input"],

Cell[CellGroupData[{
Cell[354292, 9488, 57, 1, 39, "Input"],
Cell[354352, 9491, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[354439, 9498, 61, 1, 64, "Subsection",
  CellTags->"T1.6.49.1"],
Cell[354503, 9501, 170, 3, 39, "Input"],
Cell[354676, 9506, 460, 11, 62, "Input"],

Cell[CellGroupData[{
Cell[355161, 9521, 59, 1, 39, "Input"],
Cell[355223, 9524, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[355310, 9531, 57, 1, 64, "Subsection",
  CellTags->"T1.6.50.1"],
Cell[355370, 9534, 77, 4, 46, "Text"],
Cell[355450, 9540, 12305, 292, 354, "Input"],
Cell[367758, 9834, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[367857, 9840, 59, 1, 64, "Subsection",
  CellTags->"T1.6.51.1"],
Cell[367919, 9843, 46, 1, 39, "Input"],
Cell[367968, 9846, 426, 11, 62, "Input"],
Cell[368397, 9859, 104, 3, 71, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[368538, 9867, 57, 1, 64, "Subsection",
  CellTags->"T1.6.52.1"],
Cell[368598, 9870, 157, 7, 46, "Text"],
Cell[368758, 9879, 5078, 127, 183, "Input"],
Cell[373839, 10008, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[373938, 10014, 59, 1, 64, "Subsection",
  CellTags->"T1.6.53.1"],
Cell[374000, 10017, 137, 3, 39, "Input"],
Cell[374140, 10022, 142, 3, 41, "Input"],

Cell[CellGroupData[{
Cell[374307, 10029, 57, 1, 39, "Input"],
Cell[374367, 10032, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[374454, 10039, 61, 1, 64, "Subsection",
  CellTags->"T1.6.54.1"],
Cell[374518, 10042, 162, 3, 39, "Input"],
Cell[374683, 10047, 148, 3, 41, "Input"],

Cell[CellGroupData[{
Cell[374856, 10054, 59, 1, 39, "Input"],
Cell[374918, 10057, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[375005, 10064, 61, 1, 64, "Subsection",
  CellTags->"T1.6.55.1"],
Cell[375069, 10067, 162, 3, 39, "Input"],
Cell[375234, 10072, 148, 3, 41, "Input"],

Cell[CellGroupData[{
Cell[375407, 10079, 59, 1, 39, "Input"],
Cell[375469, 10082, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[375556, 10089, 57, 1, 64, "Subsection",
  CellTags->"T1.6.56.1"],
Cell[375616, 10092, 147, 7, 46, "Text"],
Cell[375766, 10101, 5563, 141, 173, "Input"],
Cell[381332, 10244, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[381431, 10250, 59, 1, 64, "Subsection",
  CellTags->"T1.6.57.1"],
Cell[381493, 10253, 89, 1, 39, "Input"],
Cell[381585, 10256, 142, 3, 41, "Input"],

Cell[CellGroupData[{
Cell[381752, 10263, 57, 1, 39, "Input"],
Cell[381812, 10266, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[381899, 10273, 57, 1, 64, "Subsection",
  CellTags->"T1.6.58.1"],
Cell[381959, 10276, 138, 6, 46, "Text"],
Cell[382100, 10284, 4332, 107, 145, "Input"],
Cell[386435, 10393, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[386534, 10399, 59, 1, 64, "Subsection",
  CellTags->"T1.6.59.1"],
Cell[386596, 10402, 97, 2, 39, "Input"],
Cell[386696, 10406, 152, 4, 41, "Input"],

Cell[CellGroupData[{
Cell[386873, 10414, 57, 1, 39, "Input"],
Cell[386933, 10417, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[387020, 10424, 57, 1, 64, "Subsection",
  CellTags->"T1.6.60.1"],
Cell[387080, 10427, 156, 7, 46, "Text"],
Cell[387239, 10436, 4794, 119, 174, "Input"],
Cell[392036, 10557, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[392135, 10563, 59, 1, 64, "Subsection",
  CellTags->"T1.6.61.1"],
Cell[392197, 10566, 157, 3, 39, "Input"],
Cell[392357, 10571, 185, 4, 62, "Input"],

Cell[CellGroupData[{
Cell[392567, 10579, 57, 1, 39, "Input"],
Cell[392627, 10582, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[392714, 10589, 61, 1, 64, "Subsection",
  CellTags->"T1.6.62.1"],
Cell[392778, 10592, 191, 4, 39, "Input"],
Cell[392972, 10598, 191, 4, 62, "Input"],

Cell[CellGroupData[{
Cell[393188, 10606, 59, 1, 39, "Input"],
Cell[393250, 10609, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[393337, 10616, 61, 1, 64, "Subsection",
  CellTags->"T1.6.63.1"],
Cell[393401, 10619, 208, 4, 39, "Input"],
Cell[393612, 10625, 191, 4, 62, "Input"],

Cell[CellGroupData[{
Cell[393828, 10633, 59, 1, 39, "Input"],
Cell[393890, 10636, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[393977, 10643, 57, 1, 64, "Subsection",
  CellTags->"T1.6.64.1"],
Cell[394037, 10646, 146, 7, 46, "Text"],
Cell[394186, 10655, 10034, 234, 442, "Input"],
Cell[404223, 10891, 62, 1, 39, "Input"],
Cell[404288, 10894, 50, 1, 39, "Input"],
Cell[404341, 10897, 125, 3, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[404503, 10905, 59, 1, 64, "Subsection",
  CellTags->"T1.6.65.1"],
Cell[404565, 10908, 117, 3, 39, "Input"],
Cell[404685, 10913, 185, 4, 62, "Input"],

Cell[CellGroupData[{
Cell[404895, 10921, 57, 1, 39, "Input"],
Cell[404955, 10924, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[405042, 10931, 61, 1, 64, "Subsection",
  CellTags->"T1.6.66.1"],
Cell[405106, 10934, 148, 3, 39, "Input"],
Cell[405257, 10939, 191, 4, 62, "Input"],

Cell[CellGroupData[{
Cell[405473, 10947, 59, 1, 39, "Input"],
Cell[405535, 10950, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[405622, 10957, 61, 1, 64, "Subsection",
  CellTags->"T1.6.67.1"],
Cell[405686, 10960, 148, 3, 39, "Input"],
Cell[405837, 10965, 191, 4, 62, "Input"],

Cell[CellGroupData[{
Cell[406053, 10973, 59, 1, 39, "Input"],
Cell[406115, 10976, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[406202, 10983, 57, 1, 64, "Subsection",
  CellTags->"T1.6.68.1"],
Cell[406262, 10986, 138, 6, 46, "Text"],
Cell[406403, 10994, 6599, 156, 273, "Input"],
Cell[413005, 11152, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[413104, 11158, 59, 1, 64, "Subsection",
  CellTags->"T1.6.69.1"],
Cell[413166, 11161, 117, 3, 39, "Input"],
Cell[413286, 11166, 178, 4, 85, "Input"],

Cell[CellGroupData[{
Cell[413489, 11174, 57, 1, 39, "Input"],
Cell[413549, 11177, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[413636, 11184, 57, 1, 64, "Subsection",
  CellTags->"T1.6.70.1"],

Cell[CellGroupData[{
Cell[413718, 11189, 56, 1, 60, "Subsubsection",
  CellTags->"T1.6.70.1.1"],
Cell[413777, 11192, 73, 1, 45, "Input"],
Cell[413853, 11195, 184, 4, 47, "Input"],
Cell[414040, 11201, 259, 5, 47, "Input"],
Cell[414302, 11208, 918, 23, 102, "Input"],
Cell[415223, 11233, 84, 1, 47, "Input"],
Cell[415310, 11236, 84, 1, 47, "Input"],
Cell[415397, 11239, 84, 1, 47, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[415518, 11245, 56, 1, 60, "Subsubsection",
  CellTags->"T1.6.70.2.1"],
Cell[415577, 11248, 775, 18, 101, "Input"],
Cell[416355, 11268, 84, 1, 47, "Input"],
Cell[416442, 11271, 84, 1, 47, "Input"],
Cell[416529, 11274, 84, 1, 47, "Input"],
Cell[416616, 11277, 47, 1, 41, "Input"],
Cell[416666, 11280, 85, 5, 46, "Text"],
Cell[416754, 11287, 5136, 131, 292, "Input"],
Cell[421893, 11420, 62, 1, 39, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[422004, 11427, 59, 1, 64, "Subsection",
  CellTags->"T1.6.71.1"],
Cell[422066, 11430, 46, 1, 39, "Input"],
Cell[422115, 11433, 149, 4, 62, "Input"],
Cell[422267, 11439, 49, 0, 46, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[422353, 11444, 57, 1, 64, "Subsection",
  CellTags->"T1.6.72.1"],
Cell[422413, 11447, 85, 5, 46, "Text"],
Cell[422501, 11454, 9048, 206, 482, "Input"],
Cell[431552, 11662, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[431651, 11668, 59, 1, 64, "Subsection",
  CellTags->"T1.6.73.1"],
Cell[431713, 11671, 52, 1, 39, "Input"],
Cell[431768, 11674, 140, 3, 62, "Input"],
Cell[431911, 11679, 49, 0, 46, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[431997, 11684, 57, 1, 64, "Subsection",
  CellTags->"T1.6.74.1"],
Cell[432057, 11687, 77, 4, 46, "Text"],
Cell[432137, 11693, 2490, 68, 49, "Input"],
Cell[434630, 11763, 66, 1, 39, "Input"],
Cell[434699, 11766, 2377, 65, 49, "Input"],
Cell[437079, 11833, 66, 1, 39, "Input"],
Cell[437148, 11836, 2600, 67, 83, "Input"],
Cell[439751, 11905, 66, 1, 39, "Input"],
Cell[439820, 11908, 3551, 90, 113, "Input"],
Cell[443374, 12000, 66, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[443477, 12006, 60, 1, 64, "Subsection",
  CellTags->"T1.6.75.1"],
Cell[443540, 12009, 77, 1, 39, "Input"],
Cell[443620, 12012, 150, 4, 62, "Input"],
Cell[443773, 12018, 49, 0, 46, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[443859, 12023, 60, 1, 64, "Subsection",
  CellTags->"T1.6.76.1"],
Cell[443922, 12026, 119, 3, 39, "Input"],
Cell[444044, 12031, 176, 4, 62, "Input"],
Cell[444223, 12037, 49, 0, 46, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[444309, 12042, 60, 1, 64, "Subsection",
  CellTags->"T1.6.77.1"],
Cell[444372, 12045, 49, 1, 39, "Input"],
Cell[444424, 12048, 187, 4, 85, "Input"],
Cell[444614, 12054, 49, 0, 46, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[444700, 12059, 60, 1, 64, "Subsection",
  CellTags->"T1.6.78.1"],
Cell[444763, 12062, 77, 1, 39, "Input"],
Cell[444843, 12065, 190, 4, 64, "Input"],
Cell[445036, 12071, 49, 0, 46, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[445122, 12076, 57, 1, 64, "Subsection",
  CellTags->"T1.6.79.1"],
Cell[445182, 12079, 135, 4, 46, "Text"],
Cell[445320, 12085, 66, 1, 39, "Input"],
Cell[445389, 12088, 187, 3, 44, "Text"],
Cell[445579, 12093, 4719, 112, 104, "Input"],
Cell[450301, 12207, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[450400, 12213, 57, 1, 64, "Subsection",
  CellTags->"T1.6.80.1"],
Cell[450460, 12216, 236, 6, 46, "Text"],
Cell[450699, 12224, 2102, 56, 107, "Input"],
Cell[452804, 12282, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[452903, 12288, 59, 1, 64, "Subsection",
  CellTags->"T1.6.81.1"],
Cell[452965, 12291, 74, 1, 39, "Input"],
Cell[453042, 12294, 137, 3, 39, "Input"],
Cell[453182, 12299, 49, 0, 46, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[453268, 12304, 57, 1, 64, "Subsection",
  CellTags->"T1.6.82.1"],
Cell[453328, 12307, 77, 4, 46, "Text"],
Cell[453408, 12313, 3018, 76, 83, "Input"],
Cell[456429, 12391, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[456528, 12397, 59, 1, 64, "Subsection",
  CellTags->"T1.6.83.1"],
Cell[456590, 12400, 74, 1, 39, "Input"],
Cell[456667, 12403, 140, 3, 41, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[456844, 12411, 61, 1, 64, "Subsection",
  CellTags->"T1.6.84.1"],
Cell[456908, 12414, 128, 3, 41, "Input"],
Cell[457039, 12419, 90, 1, 39, "Input"],
Cell[457132, 12422, 179, 4, 64, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[457348, 12431, 61, 1, 64, "Subsection",
  CellTags->"T1.6.85.1"],
Cell[457412, 12434, 175, 4, 41, "Input"],
Cell[457590, 12440, 90, 1, 39, "Input"],
Cell[457683, 12443, 179, 4, 64, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[457899, 12452, 57, 1, 64, "Subsection",
  CellTags->"T1.6.86.1"],
Cell[457959, 12455, 77, 4, 46, "Text"],
Cell[458039, 12461, 2104, 59, 49, "Input"],
Cell[460146, 12522, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[460245, 12528, 59, 1, 64, "Subsection",
  CellTags->"T1.6.87.1"],
Cell[460307, 12531, 74, 1, 39, "Input"],
Cell[460384, 12534, 151, 4, 62, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[460572, 12543, 57, 1, 64, "Subsection",
  CellTags->"T1.6.88.1"],
Cell[460632, 12546, 77, 4, 46, "Text"],
Cell[460712, 12552, 2088, 56, 49, "Input"],
Cell[462803, 12610, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[462902, 12616, 59, 1, 64, "Subsection",
  CellTags->"T1.6.89.1"],
Cell[462964, 12619, 46, 1, 39, "Input"],
Cell[463013, 12622, 141, 3, 62, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[463191, 12630, 57, 1, 64, "Subsection",
  CellTags->"T1.6.90.1"],
Cell[463251, 12633, 77, 4, 46, "Text"],
Cell[463331, 12639, 1286, 36, 49, "Input"],
Cell[464620, 12677, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[464719, 12683, 59, 1, 64, "Subsection",
  CellTags->"T1.6.91.1"],
Cell[464781, 12686, 74, 1, 39, "Input"],
Cell[464858, 12689, 128, 3, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[465023, 12697, 57, 1, 64, "Subsection",
  CellTags->"T1.6.92.1"],
Cell[465083, 12700, 77, 4, 46, "Text"],
Cell[465163, 12706, 1144, 32, 49, "Input"],
Cell[466310, 12740, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[466409, 12746, 59, 1, 64, "Subsection",
  CellTags->"T1.6.93.1"],
Cell[466471, 12749, 46, 1, 39, "Input"],
Cell[466520, 12752, 130, 3, 62, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[466687, 12760, 57, 1, 64, "Subsection",
  CellTags->"T1.6.94.1"],
Cell[466747, 12763, 114, 2, 43, "Text"],
Cell[466864, 12767, 1616, 44, 76, "Text"],
Cell[468483, 12813, 2068, 58, 81, "Input"],
Cell[470554, 12873, 62, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[470653, 12879, 59, 1, 64, "Subsection",
  CellTags->"T1.6.95.1"],
Cell[470715, 12882, 74, 1, 39, "Input"],
Cell[470792, 12885, 137, 3, 62, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[470966, 12893, 61, 1, 64, "Subsection",
  CellTags->"T1.6.96.1"],
Cell[471030, 12896, 128, 3, 39, "Input"],
Cell[471161, 12901, 143, 3, 62, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[471341, 12909, 61, 1, 64, "Subsection",
  CellTags->"T1.6.97.1"],
Cell[471405, 12912, 128, 3, 39, "Input"],
Cell[471536, 12917, 143, 3, 62, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[471716, 12925, 58, 1, 64, "Subsection",
  CellTags->"T1.6.98.1"],
Cell[471777, 12928, 39, 0, 46, "Text"],
Cell[471819, 12930, 2876, 77, 48, "Text"],
Cell[474698, 13009, 73, 1, 40, "Text"],
Cell[474774, 13012, 3007, 83, 83, "Input"],
Cell[477784, 13097, 64, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[477885, 13103, 60, 1, 64, "Subsection",
  CellTags->"T1.6.99.1"],
Cell[477948, 13106, 48, 1, 39, "Input"],
Cell[477999, 13109, 154, 3, 62, "Input"],

Cell[CellGroupData[{
Cell[478178, 13116, 58, 1, 39, "Input"],
Cell[478239, 13119, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[478326, 13126, 62, 1, 64, "Subsection",
  CellTags->"T1.6.100.1"],
Cell[478391, 13129, 85, 1, 39, "Input"],
Cell[478479, 13132, 157, 3, 62, "Input"],

Cell[CellGroupData[{
Cell[478661, 13139, 59, 1, 39, "Input"],
Cell[478723, 13142, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[478810, 13149, 62, 1, 64, "Subsection",
  CellTags->"T1.6.101.1"],
Cell[478875, 13152, 118, 3, 39, "Input"],
Cell[478996, 13157, 157, 3, 62, "Input"],

Cell[CellGroupData[{
Cell[479178, 13164, 59, 1, 39, "Input"],
Cell[479240, 13167, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[479327, 13174, 62, 1, 64, "Subsection",
  CellTags->"T1.6.102.1"],
Cell[479392, 13177, 85, 1, 39, "Input"],
Cell[479480, 13180, 157, 3, 62, "Input"],

Cell[CellGroupData[{
Cell[479662, 13187, 59, 1, 39, "Input"],
Cell[479724, 13190, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[479811, 13197, 62, 1, 64, "Subsection",
  CellTags->"T1.6.103.1"],
Cell[479876, 13200, 85, 1, 39, "Input"],
Cell[479964, 13203, 157, 3, 62, "Input"],

Cell[CellGroupData[{
Cell[480146, 13210, 59, 1, 39, "Input"],
Cell[480208, 13213, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[480295, 13220, 62, 1, 64, "Subsection",
  CellTags->"T1.6.104.1"],
Cell[480360, 13223, 118, 3, 39, "Input"],
Cell[480481, 13228, 157, 3, 62, "Input"],

Cell[CellGroupData[{
Cell[480663, 13235, 59, 1, 39, "Input"],
Cell[480725, 13238, 38, 1, 33, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[480812, 13245, 59, 1, 64, "Subsection",
  CellTags->"T1.6.105.1"],
Cell[480874, 13248, 59, 1, 43, "Text"],
Cell[480936, 13251, 154, 2, 48, "Text"],
Cell[481093, 13255, 431, 9, 177, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[481561, 13269, 61, 1, 64, "Subsection",
  CellTags->"T1.6.106.1"],
Cell[481625, 13272, 48, 1, 39, "Input"],
Cell[481676, 13275, 70, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[481783, 13281, 59, 1, 64, "Subsection",
  CellTags->"T1.6.107.1"],
Cell[481845, 13284, 59, 1, 43, "Text"],
Cell[481907, 13287, 154, 2, 48, "Text"],
Cell[482064, 13291, 562, 14, 177, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[482663, 13310, 61, 1, 64, "Subsection",
  CellTags->"T1.6.108.1"],
Cell[482727, 13313, 48, 1, 39, "Input"],
Cell[482778, 13316, 114, 3, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[482929, 13324, 59, 1, 64, "Subsection",
  CellTags->"T1.6.109.1"],
Cell[482991, 13327, 59, 1, 43, "Text"],
Cell[483053, 13330, 154, 2, 48, "Text"],
Cell[483210, 13334, 396, 8, 131, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[483643, 13347, 61, 1, 64, "Subsection",
  CellTags->"T1.6.110.1"],
Cell[483707, 13350, 48, 1, 39, "Input"],
Cell[483758, 13353, 70, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[483865, 13359, 59, 1, 64, "Subsection",
  CellTags->"T1.6.111.1"],
Cell[483927, 13362, 59, 1, 43, "Text"],
Cell[483989, 13365, 293, 6, 108, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[484319, 13376, 61, 1, 64, "Subsection",
  CellTags->"T1.6.112.1"],
Cell[484383, 13379, 48, 1, 39, "Input"],
Cell[484434, 13382, 70, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[484541, 13388, 59, 1, 64, "Subsection",
  CellTags->"T1.6.113.1"],
Cell[484603, 13391, 43, 0, 46, "Text"],
Cell[484649, 13393, 105, 1, 44, "Text"],
Cell[484757, 13396, 6709, 171, 185, "Input"],
Cell[491469, 13569, 191, 3, 43, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[491697, 13577, 61, 1, 64, "Subsection",
  CellTags->"T1.6.114.1"],
Cell[491761, 13580, 76, 1, 39, "Input"],
Cell[491840, 13583, 422, 10, 62, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[492299, 13598, 61, 1, 64, "Subsection",
  CellTags->"T1.6.115.1"],
Cell[492363, 13601, 692, 29, 558, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[493092, 13635, 61, 1, 64, "Subsection",
  CellTags->"T1.6.116.1"],
Cell[493156, 13638, 696, 29, 558, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[493889, 13672, 62, 1, 64, "Subsection",
  CellTags->"T1.6.117.1"],
Cell[493954, 13675, 1037, 20, 361, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[495028, 13700, 62, 1, 64, "Subsection",
  CellTags->"T1.6.118.1"],
Cell[495093, 13703, 983, 18, 338, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[496113, 13726, 62, 1, 64, "Subsection",
  CellTags->"T1.6.119.1"],
Cell[496178, 13729, 981, 18, 338, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[497196, 13752, 61, 1, 64, "Subsection",
  CellTags->"T1.6.120.1"],
Cell[497260, 13755, 240, 6, 118, "Input"],

Cell[CellGroupData[{
Cell[497525, 13765, 4682, 85, 268, "Input"],
Cell[502210, 13852, 357, 6, 112, "Print"]
}, Open  ]],
Cell[502582, 13861, 40, 1, 39, "Input"],

Cell[CellGroupData[{
Cell[502647, 13866, 583, 11, 229, "Input"],
Cell[503233, 13879, 357, 6, 112, "Print"]
}, Open  ]],
Cell[503605, 13888, 40, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[503682, 13894, 55, 1, 64, "Subsection",
  CellTags->"Tsymmetry"],
Cell[503740, 13897, 108, 3, 58, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[503885, 13905, 70, 1, 64, "Subsection",
  CellTags->"specialTVR"],
Cell[503958, 13908, 579, 11, 258, "Input"],
Cell[504540, 13921, 1683, 27, 698, "Input"],
Cell[506226, 13950, 1597, 26, 658, "Input"],
Cell[507826, 13978, 3547, 56, 1698, "Input"],
Cell[511376, 14036, 3261, 52, 1598, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[514674, 14093, 60, 1, 64, "Subsection",
  CellTags->"T1.6.121.1"],
Cell[514737, 14096, 412, 7, 108, "Input"],
Cell[515152, 14105, 434, 7, 108, "Input"],
Cell[515589, 14114, 445, 8, 108, "Input"],
Cell[516037, 14124, 432, 7, 108, "Input"],
Cell[516472, 14133, 220, 14, 278, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[516729, 14152, 61, 1, 64, "Subsection",
  CellTags->"T1.6.122.1"],
Cell[516793, 14155, 282, 5, 85, "Input"],
Cell[517078, 14162, 282, 5, 85, "Input"],
Cell[517363, 14169, 133, 3, 58, "Input"],
Cell[517499, 14174, 273, 5, 85, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[517809, 14184, 60, 1, 64, "Subsection",
  CellTags->"T1.6.123.1"],
Cell[517872, 14187, 183, 3, 62, "Input"],
Cell[518058, 14192, 183, 3, 62, "Input"],
Cell[518244, 14197, 183, 3, 62, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[518464, 14205, 58, 1, 64, "Subsection",
  CellTags->"T1.6.124.1"],
Cell[518525, 14208, 135, 2, 62, "Input"],
Cell[518663, 14212, 135, 2, 62, "Input"],
Cell[518801, 14216, 88, 1, 39, "Input"],
Cell[518892, 14219, 86, 1, 39, "Input"],
Cell[518981, 14222, 89, 1, 39, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[519119, 14229, 108, 2, 77, "Section",
  CellTags->"T1.7.1"],

Cell[CellGroupData[{
Cell[519252, 14235, 70, 1, 64, "Subsection",
  CellTags->"T1.7.1.1"],
Cell[519325, 14238, 45, 0, 46, "Text"],
Cell[519373, 14240, 58, 1, 39, "Input"],
Cell[519434, 14243, 134, 3, 62, "Input"],
Cell[519571, 14248, 501, 13, 62, "Input"],
Cell[520075, 14263, 125, 2, 62, "Input"],
Cell[520203, 14267, 503, 13, 62, "Input"],
Cell[520709, 14282, 127, 2, 62, "Input"],
Cell[520839, 14286, 84, 3, 46, "Text"],
Cell[520926, 14291, 61, 1, 39, "Input"],
Cell[520990, 14294, 106, 2, 39, "Input"],
Cell[521099, 14298, 216, 7, 146, "Text",
  CellTags->"TLRdef"],
Cell[521318, 14307, 109, 2, 39, "Input"],
Cell[521430, 14311, 438, 10, 62, "Input"],
Cell[521871, 14323, 26, 0, 46, "Text"],
Cell[521900, 14325, 899, 18, 438, "Input"],
Cell[522802, 14345, 72, 1, 44, "Text"],
Cell[522877, 14348, 874, 17, 438, "Input"],
Cell[523754, 14367, 82, 1, 40, "Text"],
Cell[523839, 14370, 717, 15, 325, "Input"],
Cell[524559, 14387, 82, 1, 40, "Text"],
Cell[524644, 14390, 738, 15, 376, "Input"],
Cell[525385, 14407, 85, 1, 40, "Text"],
Cell[525473, 14410, 797, 17, 455, "Input"],
Cell[526273, 14429, 37, 0, 46, "Text"],
Cell[526313, 14431, 676, 16, 221, "Input"],
Cell[526992, 14449, 37, 0, 46, "Text"],
Cell[527032, 14451, 676, 16, 221, "Input"],
Cell[527711, 14469, 35, 0, 46, "Text"],
Cell[527749, 14471, 826, 16, 469, "Input"],
Cell[528578, 14489, 31, 0, 46, "Text"],
Cell[528612, 14491, 224, 6, 146, "Text"],
Cell[528839, 14499, 31, 0, 46, "Text"],
Cell[528873, 14501, 204, 5, 98, "Input"],
Cell[529080, 14508, 148, 4, 71, "Text"],
Cell[529231, 14514, 443, 15, 298, "Input"],
Cell[529677, 14531, 443, 15, 298, "Input"],
Cell[530123, 14548, 49, 0, 46, "Text"],
Cell[530175, 14550, 390, 8, 194, "Input"],
Cell[530568, 14560, 49, 0, 46, "Text"],
Cell[530620, 14562, 390, 8, 194, "Input"],
Cell[531013, 14572, 25, 0, 46, "Text"],
Cell[531041, 14574, 238, 5, 85, "Input"],
Cell[531282, 14581, 240, 5, 85, "Input"],
Cell[531525, 14588, 78, 1, 46, "Text",
  CellTags->"tlrule17"],
Cell[531606, 14591, 522, 11, 252, "Input"],
Cell[532131, 14604, 1018, 22, 420, "Input"],
Cell[533152, 14628, 1018, 22, 420, "Input"],
Cell[534173, 14652, 37, 0, 46, "Text"],
Cell[534213, 14654, 519, 10, 258, "Input"],
Cell[534735, 14666, 279, 6, 85, "Input"],
Cell[535017, 14674, 382, 9, 186, "Input"],
Cell[535402, 14685, 6408, 176, 212, "Text"],
Cell[541813, 14863, 917, 21, 435, "Input"],
Cell[542733, 14886, 12636, 333, 188, "Text"],
Cell[555372, 15221, 980, 21, 456, "Input"],
Cell[556355, 15244, 65, 0, 46, "Text"],
Cell[556423, 15246, 378, 7, 131, "Input"],
Cell[556804, 15255, 66, 1, 39, "Input"],
Cell[556873, 15258, 71, 1, 39, "Input"],
Cell[556947, 15261, 93, 1, 39, "Input"],
Cell[557043, 15264, 93, 1, 39, "Input"],
Cell[557139, 15267, 93, 1, 39, "Input"],
Cell[557235, 15270, 93, 1, 39, "Input"],
Cell[557331, 15273, 93, 1, 39, "Input"],
Cell[557427, 15276, 93, 1, 39, "Input"],
Cell[557523, 15279, 75, 1, 39, "Input"],
Cell[557601, 15282, 86, 1, 39, "Input"],
Cell[557690, 15285, 103, 2, 39, "Input"],
Cell[557796, 15289, 209, 4, 62, "Input"],
Cell[558008, 15295, 56, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[558101, 15301, 50, 1, 64, "Subsection",
  CellTags->"T1.7.2.1"],
Cell[558154, 15304, 86, 3, 71, "Text"],
Cell[558243, 15309, 630, 11, 247, "Input"],
Cell[558876, 15322, 191, 3, 62, "Input"],
Cell[559070, 15327, 191, 3, 62, "Input"],
Cell[559264, 15332, 660, 11, 196, "Input"],
Cell[559927, 15345, 163, 2, 62, "Input"],
Cell[560093, 15349, 163, 2, 62, "Input"],
Cell[560259, 15353, 472, 7, 130, "Input"],
Cell[560734, 15362, 164, 2, 62, "Input"],
Cell[560901, 15366, 164, 2, 62, "Input"],
Cell[561068, 15370, 532, 8, 209, "Input"],
Cell[561603, 15380, 543, 8, 201, "Input"],
Cell[562149, 15390, 22, 0, 46, "Text"],
Cell[562174, 15392, 115, 2, 44, "Input"],
Cell[562292, 15396, 88, 1, 39, "Input"],
Cell[562383, 15399, 501, 9, 194, "Input"],
Cell[562887, 15410, 564, 11, 249, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[563488, 15426, 54, 1, 64, "Subsection",
  CellTags->"T1.7.3.1"],
Cell[563545, 15429, 62, 1, 39, "Input"],
Cell[563610, 15432, 1313, 23, 407, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[564960, 15460, 52, 1, 64, "Subsection",
  CellTags->"$TTable"],
Cell[565015, 15463, 51, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[565103, 15469, 59, 1, 64, "Subsection",
  CellTags->"T1.7.4.1"],
Cell[565165, 15472, 100, 2, 39, "Input"],
Cell[565268, 15476, 254, 5, 85, "Input"],
Cell[565525, 15483, 188, 3, 85, "Input"],
Cell[565716, 15488, 9095, 172, 1743, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[574860, 15666, 92, 2, 77, "Section",
  CellTags->"T1.8.1"],
Cell[574955, 15670, 153, 2, 85, "Input"],
Cell[575111, 15674, 166, 2, 85, "Input"],
Cell[575280, 15678, 8226, 165, 1062, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[583543, 15848, 93, 2, 77, "Section",
  CellTags->"T1.9.1"],

Cell[CellGroupData[{
Cell[583661, 15854, 54, 1, 64, "Subsection",
  CellTags->"T1.9.1.1"],
Cell[583718, 15857, 61, 0, 46, "Text"],
Cell[583782, 15859, 178, 3, 62, "Input"],
Cell[583963, 15864, 65, 1, 39, "Input"],
Cell[584031, 15867, 72, 1, 39, "Input"],
Cell[584106, 15870, 54, 1, 39, "Input"],
Cell[584163, 15873, 1703, 30, 361, "Input"],
Cell[585869, 15905, 53, 1, 39, "Input"],
Cell[585925, 15908, 119, 3, 39, "Input"],
Cell[586047, 15913, 81, 1, 39, "Input"],
Cell[586131, 15916, 49, 1, 39, "Input"],

Cell[CellGroupData[{
Cell[586205, 15921, 135, 2, 62, "Input"],
Cell[586343, 15925, 302, 7, 33, "Print"]
}, Open  ]],
Cell[586660, 15935, 142, 2, 62, "Input"],
Cell[586805, 15939, 50, 1, 39, "Input"],
Cell[586858, 15942, 54, 1, 39, "Input"],
Cell[586915, 15945, 45, 1, 39, "Input"],
Cell[586963, 15948, 54, 1, 39, "Input"],
Cell[587020, 15951, 44, 1, 39, "Input"],
Cell[587067, 15954, 51, 1, 39, "Input"],
Cell[587121, 15957, 92, 1, 39, "Input"],
Cell[587216, 15960, 84, 1, 62, "Input"],
Cell[587303, 15963, 61, 1, 39, "Input"],
Cell[587367, 15966, 226, 6, 39, "Input"],
Cell[587596, 15974, 77, 1, 39, "Input"],
Cell[587676, 15977, 181, 3, 62, "Input"],
Cell[587860, 15982, 44, 1, 39, "Input"],
Cell[587907, 15985, 57, 1, 39, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[588013, 15992, 95, 2, 77, "Section",
  CellTags->"T1.10.1"],
Cell[588111, 15996, 111, 3, 46, "Text"],

Cell[CellGroupData[{
Cell[588247, 16003, 1669, 54, 1098, "Input"],
Cell[589919, 16059, 12116, 320, 897, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[602084, 16385, 101, 2, 77, "Section",
  CellTags->"T1.11.1"],
Cell[602188, 16389, 46, 1, 39, "Input"],
Cell[602237, 16392, 2309, 55, 1078, "Input"],
Cell[604549, 16449, 67, 1, 39, "Input"],
Cell[604619, 16452, 89, 3, 46, "Text"],

Cell[CellGroupData[{
Cell[604733, 16459, 112, 2, 62, "Input"],
Cell[604848, 16463, 59, 1, 39, "Output"]
}, Open  ]],
Cell[604922, 16467, 55, 0, 46, "Text"],

Cell[CellGroupData[{
Cell[605002, 16471, 134, 3, 62, "Input"],
Cell[605139, 16476, 47, 1, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[605223, 16482, 62, 1, 39, "Input"],
Cell[605288, 16485, 47, 1, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[605372, 16491, 44, 1, 39, "Input"],
Cell[605419, 16494, 54, 1, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[605510, 16500, 48, 1, 39, "Input"],
Cell[605561, 16503, 41, 1, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[605639, 16509, 46, 1, 39, "Input"],
Cell[605688, 16512, 41, 1, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[605766, 16518, 43, 1, 39, "Input"],
Cell[605812, 16521, 54, 1, 39, "Output"]
}, Open  ]],
Cell[605881, 16525, 35, 1, 39, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

